import{r as we,j as se,s as Et,a as dt,F as io,T as Fn,S as Vo,R as Zy,b as Q0,B as Qy,c as Jy,d as J0,e as ED}from"./index.2165e0a4.js";var de=typeof globalThis!="undefined"?globalThis:typeof window!="undefined"?window:typeof global!="undefined"?global:typeof self!="undefined"?self:{};function RD(i){if(i.__esModule)return i;var t=Object.defineProperty({},"__esModule",{value:!0});return Object.keys(i).forEach(function(e){var r=Object.getOwnPropertyDescriptor(i,e);Object.defineProperty(t,e,r.get?r:{enumerable:!0,get:function(){return i[e]}})}),t}const eT=we.exports.createContext({height:0,width:0});var tT=(i=>(i[i.BP1=520]="BP1",i[i.BP2=900]="BP2",i))(tT||{});const PD=({children:i})=>{const[t,e]=we.exports.useState(window.innerWidth),[r,n]=we.exports.useState(window.innerHeight),o=()=>{e(window.innerWidth),n(window.innerHeight)};return we.exports.useEffect(()=>(window.addEventListener("resize",o),()=>window.removeEventListener("resize",o)),[]),se(eT.Provider,{value:{width:t,height:r},children:i})};/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */function rT(i,t,e,r){function n(o){return o instanceof e?o:new e(function(a){a(o)})}return new(e||(e=Promise))(function(o,a){function s(c){try{u(r.next(c))}catch(p){a(p)}}function l(c){try{u(r.throw(c))}catch(p){a(p)}}function u(c){c.done?o(c.value):n(c.value).then(s,l)}u((r=r.apply(i,t||[])).next())})}function nT(i,t){var e={label:0,sent:function(){if(o[0]&1)throw o[1];return o[1]},trys:[],ops:[]},r,n,o,a;return a={next:s(0),throw:s(1),return:s(2)},typeof Symbol=="function"&&(a[Symbol.iterator]=function(){return this}),a;function s(u){return function(c){return l([u,c])}}function l(u){if(r)throw new TypeError("Generator is already executing.");for(;e;)try{if(r=1,n&&(o=u[0]&2?n.return:u[0]?n.throw||((o=n.return)&&o.call(n),0):n.next)&&!(o=o.call(n,u[1])).done)return o;switch(n=0,o&&(u=[u[0]&2,o.value]),u[0]){case 0:case 1:o=u;break;case 4:return e.label++,{value:u[1],done:!1};case 5:e.label++,n=u[1],u=[0];continue;case 7:u=e.ops.pop(),e.trys.pop();continue;default:if(o=e.trys,!(o=o.length>0&&o[o.length-1])&&(u[0]===6||u[0]===2)){e=0;continue}if(u[0]===3&&(!o||u[1]>o[0]&&u[1]<o[3])){e.label=u[1];break}if(u[0]===6&&e.label<o[1]){e.label=o[1],o=u;break}if(o&&e.label<o[2]){e.label=o[2],e.ops.push(u);break}o[2]&&e.ops.pop(),e.trys.pop();continue}u=t.call(i,e)}catch(c){u=[6,c],n=0}finally{r=o=0}if(u[0]&5)throw u[1];return{value:u[0]?u[1]:void 0,done:!0}}}var Ti=function(){},yr=Ti(),rc=Object,Or=function(i){return i===yr},jo=function(i){return typeof i=="function"},oo=function(i,t){return rc.assign({},i,t)},eC="undefined",tC=function(){return typeof window!=eC},AD=function(){return typeof document!=eC},DD=function(){return tC()&&typeof window.requestAnimationFrame!=eC},cp=new WeakMap,MD=0,wc=function(i){var t=typeof i,e=i&&i.constructor,r=e==Date,n,o;if(rc(i)===i&&!r&&e!=RegExp){if(n=cp.get(i),n)return n;if(n=++MD+"~",cp.set(i,n),e==Array){for(n="@",o=0;o<i.length;o++)n+=wc(i[o])+",";cp.set(i,n)}if(e==rc){n="#";for(var a=rc.keys(i).sort();!Or(o=a.pop());)Or(i[o])||(n+=o+":"+wc(i[o])+",");cp.set(i,n)}}else n=r?i.toJSON():t=="symbol"?i.toString():t=="string"?JSON.stringify(i):""+i;return n},ym=!0,FD=function(){return ym},iT=tC(),rC=AD(),Cm=iT&&window.addEventListener?window.addEventListener.bind(window):Ti,ID=rC?document.addEventListener.bind(document):Ti,wm=iT&&window.removeEventListener?window.removeEventListener.bind(window):Ti,LD=rC?document.removeEventListener.bind(document):Ti,ND=function(){var i=rC&&document.visibilityState;return Or(i)||i!=="hidden"},GD=function(i){return ID("visibilitychange",i),Cm("focus",i),function(){LD("visibilitychange",i),wm("focus",i)}},kD=function(i){var t=function(){ym=!0,i()},e=function(){ym=!1};return Cm("online",t),Cm("offline",e),function(){wm("online",t),wm("offline",e)}},WD={isOnline:FD,isVisible:ND},VD={initFocus:GD,initReconnect:kD},zd=!tC()||"Deno"in window,BD=function(i){return DD()?window.requestAnimationFrame(i):setTimeout(i,1)},nc=zd?we.exports.useEffect:we.exports.useLayoutEffect,Jf=typeof navigator!="undefined"&&navigator.connection,Cb=!zd&&Jf&&(["slow-2g","2g"].includes(Jf.effectiveType)||Jf.saveData),oT=function(i){if(jo(i))try{i=i()}catch{i=""}var t=[].concat(i);i=typeof i=="string"?i:(Array.isArray(i)?i.length:i)?wc(i):"";var e=i?"$swr$"+i:"";return[i,t,e]},za=new WeakMap,aT=0,sT=1,lT=2,ic=function(i,t,e,r,n,o,a){a===void 0&&(a=!0);var s=za.get(i),l=s[0],u=s[1],c=s[3],p=l[t],h=u[t];if(a&&h)for(var d=0;d<h.length;++d)h[d](e,r,n);return o&&(delete c[t],p&&p[0])?p[0](lT).then(function(){return i.get(t)}):i.get(t)},HD=0,bm=function(){return++HD},uT=function(){for(var i=[],t=0;t<arguments.length;t++)i[t]=arguments[t];return rT(void 0,void 0,void 0,function(){var e,r,n,o,a,s,l,u,c,p,h,d,f,g,v,m,C,w,b,S,T;return nT(this,function(E){switch(E.label){case 0:if(e=i[0],r=i[1],n=i[2],o=i[3],a=typeof o=="boolean"?{revalidate:o}:o||{},s=Or(a.populateCache)?!0:a.populateCache,l=a.revalidate!==!1,u=a.rollbackOnError!==!1,c=a.optimisticData,p=oT(r),h=p[0],d=p[2],!h)return[2];if(f=za.get(e),g=f[2],i.length<3)return[2,ic(e,h,e.get(h),yr,yr,l,!0)];if(v=n,C=bm(),g[h]=[C,0],w=!Or(c),b=e.get(h),w&&(S=jo(c)?c(b):c,e.set(h,S),ic(e,h,S)),jo(v))try{v=v(e.get(h))}catch(P){m=P}return v&&jo(v.then)?[4,v.catch(function(P){m=P})]:[3,2];case 1:if(v=E.sent(),C!==g[h][0]){if(m)throw m;return[2,v]}else m&&w&&u&&(s=!0,v=b,e.set(h,b));E.label=2;case 2:return s&&(m||(jo(s)&&(v=s(v,b)),e.set(h,v)),e.set(d,oo(e.get(d),{error:m}))),g[h][1]=bm(),[4,ic(e,h,v,m,yr,l,!!s)];case 3:if(T=E.sent(),m)throw m;return[2,s?T:v]}})})},wb=function(i,t){for(var e in i)i[e][0]&&i[e][0](t)},cT=function(i,t){if(!za.has(i)){var e=oo(VD,t),r={},n=uT.bind(yr,i),o=Ti;if(za.set(i,[r,{},{},{},n]),!zd){var a=e.initFocus(setTimeout.bind(yr,wb.bind(yr,r,aT))),s=e.initReconnect(setTimeout.bind(yr,wb.bind(yr,r,sT)));o=function(){a&&a(),s&&s(),za.delete(i)}}return[i,n,o]}return[i,za.get(i)[4]]},jD=function(i,t,e,r,n){var o=e.errorRetryCount,a=n.retryCount,s=~~((Math.random()+.5)*(1<<(a<8?a:8)))*e.errorRetryInterval;!Or(o)&&a>o||setTimeout(r,s,n)},pT=cT(new Map),hT=pT[0],UD=pT[1],dT=oo({onLoadingSlow:Ti,onSuccess:Ti,onError:Ti,onErrorRetry:jD,onDiscarded:Ti,revalidateOnFocus:!0,revalidateOnReconnect:!0,revalidateIfStale:!0,shouldRetryOnError:!0,errorRetryInterval:Cb?1e4:5e3,focusThrottleInterval:5*1e3,dedupingInterval:2*1e3,loadingTimeout:Cb?5e3:3e3,compare:function(i,t){return wc(i)==wc(t)},isPaused:function(){return!1},cache:hT,mutate:UD,fallback:{}},WD),fT=function(i,t){var e=oo(i,t);if(t){var r=i.use,n=i.fallback,o=t.use,a=t.fallback;r&&o&&(e.use=r.concat(o)),n&&a&&(e.fallback=oo(n,a))}return e},_m=we.exports.createContext({}),zD=function(i){var t=i.value,e=fT(we.exports.useContext(_m),t),r=t&&t.provider,n=we.exports.useState(function(){return r?cT(r(e.cache||hT),t):yr})[0];return n&&(e.cache=n[0],e.mutate=n[1]),nc(function(){return n?n[2]:yr},[]),we.exports.createElement(_m.Provider,oo(i,{value:e}))},$D=function(i,t){var e=we.exports.useState({})[1],r=we.exports.useRef(i),n=we.exports.useRef({data:!1,error:!1,isValidating:!1}),o=we.exports.useCallback(function(a){var s=!1,l=r.current;for(var u in a){var c=u;l[c]!==a[c]&&(l[c]=a[c],n.current[c]&&(s=!0))}s&&!t.current&&e({})},[]);return nc(function(){r.current=i}),[r,n.current,o]},KD=function(i){return jo(i[1])?[i[0],i[1],i[2]||{}]:[i[0],null,(i[1]===null?i[2]:i[1])||{}]},YD=function(){return oo(dT,we.exports.useContext(_m))},XD=function(i){return function(){for(var e=[],r=0;r<arguments.length;r++)e[r]=arguments[r];var n=YD(),o=KD(e),a=o[0],s=o[1],l=o[2],u=fT(n,l),c=i,p=u.use;if(p)for(var h=p.length;h-- >0;)c=p[h](c);return c(a,s||u.fetcher,u)}},bb=function(i,t,e){var r=t[i]||(t[i]=[]);return r.push(e),function(){var n=r.indexOf(e);n>=0&&(r[n]=r[r.length-1],r.pop())}},eg={dedupe:!0},qD=function(i,t,e){var r=e.cache,n=e.compare,o=e.fallbackData,a=e.suspense,s=e.revalidateOnMount,l=e.refreshInterval,u=e.refreshWhenHidden,c=e.refreshWhenOffline,p=za.get(r),h=p[0],d=p[1],f=p[2],g=p[3],v=oT(i),m=v[0],C=v[1],w=v[2],b=we.exports.useRef(!1),S=we.exports.useRef(!1),T=we.exports.useRef(m),E=we.exports.useRef(t),P=we.exports.useRef(e),R=function(){return P.current},M=function(){return R().isVisible()&&R().isOnline()},N=function(_e){return r.set(w,oo(r.get(w),_e))},W=r.get(m),G=Or(o)?e.fallback[m]:o,k=Or(W)?G:W,V=r.get(w)||{},U=V.error,J=!b.current,Z=function(){return J&&!Or(s)?s:R().isPaused()?!1:a?Or(k)?!1:e.revalidateIfStale:Or(k)||e.revalidateIfStale},le=function(){return!m||!t?!1:V.isValidating?!0:J&&Z()},Y=le(),ae=$D({data:k,error:U,isValidating:Y},S),z=ae[0],X=ae[1],ue=ae[2],pe=we.exports.useCallback(function(_e){return rT(void 0,void 0,void 0,function(){var Te,Pe,Ee,Le,Ze,Re,Ge,Xe,ie,He,Me,ne,ke;return nT(this,function(qe){switch(qe.label){case 0:if(Te=E.current,!m||!Te||S.current||R().isPaused())return[2,!1];Le=!0,Ze=_e||{},Re=!g[m]||!Ze.dedupe,Ge=function(){return!S.current&&m===T.current&&b.current},Xe=function(){var it=g[m];it&&it[1]===Ee&&delete g[m]},ie={isValidating:!1},He=function(){N({isValidating:!1}),Ge()&&ue(ie)},N({isValidating:!0}),ue({isValidating:!0}),qe.label=1;case 1:return qe.trys.push([1,3,,4]),Re&&(ic(r,m,z.current.data,z.current.error,!0),e.loadingTimeout&&!r.get(m)&&setTimeout(function(){Le&&Ge()&&R().onLoadingSlow(m,e)},e.loadingTimeout),g[m]=[Te.apply(void 0,C),bm()]),ke=g[m],Pe=ke[0],Ee=ke[1],[4,Pe];case 2:return Pe=qe.sent(),Re&&setTimeout(Xe,e.dedupingInterval),!g[m]||g[m][1]!==Ee?(Re&&Ge()&&R().onDiscarded(m),[2,!1]):(N({error:yr}),ie.error=yr,Me=f[m],!Or(Me)&&(Ee<=Me[0]||Ee<=Me[1]||Me[1]===0)?(He(),Re&&Ge()&&R().onDiscarded(m),[2,!1]):(n(z.current.data,Pe)?ie.data=z.current.data:ie.data=Pe,n(r.get(m),Pe)||r.set(m,Pe),Re&&Ge()&&R().onSuccess(Pe,m,e),[3,4]));case 3:return ne=qe.sent(),Xe(),R().isPaused()||(N({error:ne}),ie.error=ne,Re&&Ge()&&(R().onError(ne,m,e),(typeof e.shouldRetryOnError=="boolean"&&e.shouldRetryOnError||jo(e.shouldRetryOnError)&&e.shouldRetryOnError(ne))&&M()&&R().onErrorRetry(ne,m,e,pe,{retryCount:(Ze.retryCount||0)+1,dedupe:!0}))),[3,4];case 4:return Le=!1,He(),Ge()&&Re&&ic(r,m,ie.data,ie.error,!1),[2,!0]}})})},[m]),be=we.exports.useCallback(uT.bind(yr,r,function(){return T.current}),[]);if(nc(function(){E.current=t,P.current=e}),nc(function(){if(!!m){var _e=m!==T.current,Te=pe.bind(yr,eg),Pe=function(Ge,Xe,ie){ue(oo({error:Xe,isValidating:ie},n(z.current.data,Ge)?yr:{data:Ge}))},Ee=0,Le=function(Ge){if(Ge==aT){var Xe=Date.now();R().revalidateOnFocus&&Xe>Ee&&M()&&(Ee=Xe+R().focusThrottleInterval,Te())}else if(Ge==sT)R().revalidateOnReconnect&&M()&&Te();else if(Ge==lT)return pe()},Ze=bb(m,d,Pe),Re=bb(m,h,Le);return S.current=!1,T.current=m,b.current=!0,_e&&ue({data:k,error:U,isValidating:Y}),Z()&&(Or(k)||zd?Te():BD(Te)),function(){S.current=!0,Ze(),Re()}}},[m,pe]),nc(function(){var _e;function Te(){var Ee=jo(l)?l(k):l;Ee&&_e!==-1&&(_e=setTimeout(Pe,Ee))}function Pe(){!z.current.error&&(u||R().isVisible())&&(c||R().isOnline())?pe(eg).then(Te):Te()}return Te(),function(){_e&&(clearTimeout(_e),_e=-1)}},[l,u,c,pe]),we.exports.useDebugValue(k),a&&Or(k)&&m)throw E.current=t,P.current=e,S.current=!1,Or(U)?pe(eg):U;return{mutate:be,get data(){return X.data=!0,k},get error(){return X.error=!0,U},get isValidating(){return X.isValidating=!0,Y}}};rc.defineProperty(zD,"default",{value:dT});var gT=XD(qD);const ZD=Et("div",{display:"flex",flexFlow:"column",padding:"$2",variants:{width:{full:{width:"100%"},"one-half":{width:"50%"},"one-third":{width:"33%"},"one-fourth":{width:"25%"}},align:{center:{margin:"0 auto"}}},defaultVariants:{width:"full"}}),_b=Et("h2",{color:"$primaryDark",variants:{size:{small:{fontSize:"$2",fontWeight:"$big"},medium:{fontSize:"$3",fontWeight:"$big"},large:{fontSize:"$5",lineHeight:"24px",fontWeight:"$big"}}},defaultVariants:{size:"medium"}}),oc=Et("h1",{fontWeight:"$semiBold",fontSize:"$4",variants:{color:{dark:{color:"$primaryDark"},light:{color:"$white"}},margin:{none:{margin:0}}},defaultVariants:{color:"dark"}}),QD=Et("h2",{fontWeight:"$semiBold",fontSize:"$3",variants:{color:{dark:{color:"$primaryDark"},light:{color:"$white"}},margin:{none:{margin:0}}},defaultVariants:{color:"dark"}}),bc=Et("strong",{fontSize:"inherit",color:"inherit",fontWeight:"$semiBold"}),Gh=Et("a",{textDecoration:"none",fontSize:"inherit",fontWeight:"inherit",variants:{color:{white:{color:"$white","&:hover":{color:"$primaryPurple",transition:"color 0.1s ease"}},blue:{color:"$linkBlue"},purple:{color:"$primaryPurple"}},weight:{normal:{fontWeight:"$normal"},semiBold:{fontWeight:"$semiBold"}}}}),Sb=Et("div",{width:"fit-content",backgroundColor:"$lightGray",padding:"$2",whiteSpace:"pre-wrap",fontFamily:"monospace",fontSize:"$3"}),JD=`import databutton as db
import streamlit as st

@db.apps.streamlit(route="/app", name="My App")
def app():
    st.title("My app")

@db.jobs.repeat_every(seconds=60, name="My Job")
def job():
    # Something amazing here
    print("Success!")
`,eM="$ databutton deploy",tM=({mode:i,editButton:t,deleteButton:e})=>i==="local"?dt(io,{children:[se(_b,{size:"large",children:"Oh, this project seems a bit empty"}),se(Fn,{children:"To add components to your project add one of the following decorators to any of your .py-files and they will magically appear here"}),se(Vo,{height:"small"}),se(Sb,{children:JD}),se(Vo,{height:"small"}),dt(Fn,{children:["To learn more about Databutton head over to our"," ",se(Gh,{href:"https://docs.databutton.com",target:"_blank",rel:"noreferrer",color:"blue",children:"documentation"})]})]}):dt(io,{children:[se(_b,{size:"large",children:"Oh, this project seems not to have been deployed yet."}),se(Fn,{children:"To deploy your project, navigate to your project folder and enter the following command."}),se(Vo,{height:"small"}),se(Sb,{children:eM}),se(Vo,{height:"small"}),dt(Fn,{children:["To learn more about Databutton,"," ",se(Gh,{color:"blue",href:"https://docs.databutton.com",target:"_blank",rel:"noreferrer",children:"head over to our documentation"}),"."]}),se(Vo,{}),dt(Zy,{children:[t,e]})]});/*!
 * Font Awesome Free 6.1.1 by @fontawesome - https://fontawesome.com
 * License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License)
 * Copyright 2022 Fonticons, Inc.
 */var rM={prefix:"fas",iconName:"rocket",icon:[512,512,[],"f135","M156.6 384.9L125.7 353.1C117.2 345.5 114.2 333.1 117.1 321.8C120.1 312.9 124.1 301.3 129.8 288H24C15.38 288 7.414 283.4 3.146 275.9C-1.123 268.4-1.042 259.2 3.357 251.8L55.83 163.3C68.79 141.4 92.33 127.1 117.8 127.1H200C202.4 124 204.8 120.3 207.2 116.7C289.1-4.07 411.1-8.142 483.9 5.275C495.6 7.414 504.6 16.43 506.7 28.06C520.1 100.9 516.1 222.9 395.3 304.8C391.8 307.2 387.1 309.6 384 311.1V394.2C384 419.7 370.6 443.2 348.7 456.2L260.2 508.6C252.8 513 243.6 513.1 236.1 508.9C228.6 504.6 224 496.6 224 488V380.8C209.9 385.6 197.6 389.7 188.3 392.7C177.1 396.3 164.9 393.2 156.6 384.9V384.9zM384 167.1C406.1 167.1 424 150.1 424 127.1C424 105.9 406.1 87.1 384 87.1C361.9 87.1 344 105.9 344 127.1C344 150.1 361.9 167.1 384 167.1z"]},nM={prefix:"fas",iconName:"spinner",icon:[512,512,[],"f110","M304 48C304 74.51 282.5 96 256 96C229.5 96 208 74.51 208 48C208 21.49 229.5 0 256 0C282.5 0 304 21.49 304 48zM304 464C304 490.5 282.5 512 256 512C229.5 512 208 490.5 208 464C208 437.5 229.5 416 256 416C282.5 416 304 437.5 304 464zM0 256C0 229.5 21.49 208 48 208C74.51 208 96 229.5 96 256C96 282.5 74.51 304 48 304C21.49 304 0 282.5 0 256zM512 256C512 282.5 490.5 304 464 304C437.5 304 416 282.5 416 256C416 229.5 437.5 208 464 208C490.5 208 512 229.5 512 256zM74.98 437C56.23 418.3 56.23 387.9 74.98 369.1C93.73 350.4 124.1 350.4 142.9 369.1C161.6 387.9 161.6 418.3 142.9 437C124.1 455.8 93.73 455.8 74.98 437V437zM142.9 142.9C124.1 161.6 93.73 161.6 74.98 142.9C56.24 124.1 56.24 93.73 74.98 74.98C93.73 56.23 124.1 56.23 142.9 74.98C161.6 93.73 161.6 124.1 142.9 142.9zM369.1 369.1C387.9 350.4 418.3 350.4 437 369.1C455.8 387.9 455.8 418.3 437 437C418.3 455.8 387.9 455.8 369.1 437C350.4 418.3 350.4 387.9 369.1 369.1V369.1z"]},iM={prefix:"fas",iconName:"stopwatch",icon:[448,512,[9201],"f2f2","M272 0C289.7 0 304 14.33 304 32C304 49.67 289.7 64 272 64H256V98.45C293.5 104.2 327.7 120 355.7 143L377.4 121.4C389.9 108.9 410.1 108.9 422.6 121.4C435.1 133.9 435.1 154.1 422.6 166.6L398.5 190.8C419.7 223.3 432 262.2 432 304C432 418.9 338.9 512 224 512C109.1 512 16 418.9 16 304C16 200 92.32 113.8 192 98.45V64H176C158.3 64 144 49.67 144 32C144 14.33 158.3 0 176 0L272 0zM248 192C248 178.7 237.3 168 224 168C210.7 168 200 178.7 200 192V320C200 333.3 210.7 344 224 344C237.3 344 248 333.3 248 320V192z"]},oM={prefix:"fas",iconName:"table",icon:[512,512,[],"f0ce","M448 32C483.3 32 512 60.65 512 96V416C512 451.3 483.3 480 448 480H64C28.65 480 0 451.3 0 416V96C0 60.65 28.65 32 64 32H448zM224 256V160H64V256H224zM64 320V416H224V320H64zM288 416H448V320H288V416zM448 256V160H288V256H448z"]},vT={prefix:"fas",iconName:"xmark",icon:[320,512,[128473,10005,10006,10060,215,"close","multiply","remove","times"],"f00d","M310.6 361.4c12.5 12.5 12.5 32.75 0 45.25C304.4 412.9 296.2 416 288 416s-16.38-3.125-22.62-9.375L160 301.3L54.63 406.6C48.38 412.9 40.19 416 32 416S15.63 412.9 9.375 406.6c-12.5-12.5-12.5-32.75 0-45.25l105.4-105.4L9.375 150.6c-12.5-12.5-12.5-32.75 0-45.25s32.75-12.5 45.25 0L160 210.8l105.4-105.4c12.5-12.5 32.75-12.5 45.25 0s12.5 32.75 0 45.25l-105.4 105.4L310.6 361.4z"]};/*!
 * Font Awesome Free 6.1.1 by @fontawesome - https://fontawesome.com
 * License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License)
 * Copyright 2022 Fonticons, Inc.
 */function xb(i,t){var e=Object.keys(i);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(i);t&&(r=r.filter(function(n){return Object.getOwnPropertyDescriptor(i,n).enumerable})),e.push.apply(e,r)}return e}function fe(i){for(var t=1;t<arguments.length;t++){var e=arguments[t]!=null?arguments[t]:{};t%2?xb(Object(e),!0).forEach(function(r){lM(i,r,e[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(i,Object.getOwnPropertyDescriptors(e)):xb(Object(e)).forEach(function(r){Object.defineProperty(i,r,Object.getOwnPropertyDescriptor(e,r))})}return i}function kh(i){return kh=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},kh(i)}function aM(i,t){if(!(i instanceof t))throw new TypeError("Cannot call a class as a function")}function Tb(i,t){for(var e=0;e<t.length;e++){var r=t[e];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(i,r.key,r)}}function sM(i,t,e){return t&&Tb(i.prototype,t),e&&Tb(i,e),Object.defineProperty(i,"prototype",{writable:!1}),i}function lM(i,t,e){return t in i?Object.defineProperty(i,t,{value:e,enumerable:!0,configurable:!0,writable:!0}):i[t]=e,i}function nC(i,t){return cM(i)||hM(i,t)||mT(i,t)||fM()}function $d(i){return uM(i)||pM(i)||mT(i)||dM()}function uM(i){if(Array.isArray(i))return Sm(i)}function cM(i){if(Array.isArray(i))return i}function pM(i){if(typeof Symbol!="undefined"&&i[Symbol.iterator]!=null||i["@@iterator"]!=null)return Array.from(i)}function hM(i,t){var e=i==null?null:typeof Symbol!="undefined"&&i[Symbol.iterator]||i["@@iterator"];if(e!=null){var r=[],n=!0,o=!1,a,s;try{for(e=e.call(i);!(n=(a=e.next()).done)&&(r.push(a.value),!(t&&r.length===t));n=!0);}catch(l){o=!0,s=l}finally{try{!n&&e.return!=null&&e.return()}finally{if(o)throw s}}return r}}function mT(i,t){if(!!i){if(typeof i=="string")return Sm(i,t);var e=Object.prototype.toString.call(i).slice(8,-1);if(e==="Object"&&i.constructor&&(e=i.constructor.name),e==="Map"||e==="Set")return Array.from(i);if(e==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(e))return Sm(i,t)}}function Sm(i,t){(t==null||t>i.length)&&(t=i.length);for(var e=0,r=new Array(t);e<t;e++)r[e]=i[e];return r}function dM(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function fM(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}var Ob=function(){},iC={},yT={},CT=null,wT={mark:Ob,measure:Ob};try{typeof window!="undefined"&&(iC=window),typeof document!="undefined"&&(yT=document),typeof MutationObserver!="undefined"&&(CT=MutationObserver),typeof performance!="undefined"&&(wT=performance)}catch{}var gM=iC.navigator||{},Eb=gM.userAgent,Rb=Eb===void 0?"":Eb,aa=iC,_t=yT,Pb=CT,pp=wT;aa.document;var vo=!!_t.documentElement&&!!_t.head&&typeof _t.addEventListener=="function"&&typeof _t.createElement=="function",bT=~Rb.indexOf("MSIE")||~Rb.indexOf("Trident/"),ao="___FONT_AWESOME___",xm=16,_T="fa",ST="svg-inline--fa",os="data-fa-i2svg",Tm="data-fa-pseudo-element",vM="data-fa-pseudo-element-pending",oC="data-prefix",aC="data-icon",Ab="fontawesome-i2svg",mM="async",yM=["HTML","HEAD","STYLE","SCRIPT"],xT=function(){try{return!0}catch{return!1}}(),sC={fas:"solid","fa-solid":"solid",far:"regular","fa-regular":"regular",fal:"light","fa-light":"light",fat:"thin","fa-thin":"thin",fad:"duotone","fa-duotone":"duotone",fab:"brands","fa-brands":"brands",fak:"kit","fa-kit":"kit",fa:"solid"},Wh={solid:"fas",regular:"far",light:"fal",thin:"fat",duotone:"fad",brands:"fab",kit:"fak"},TT={fab:"fa-brands",fad:"fa-duotone",fak:"fa-kit",fal:"fa-light",far:"fa-regular",fas:"fa-solid",fat:"fa-thin"},CM={"fa-brands":"fab","fa-duotone":"fad","fa-kit":"fak","fa-light":"fal","fa-regular":"far","fa-solid":"fas","fa-thin":"fat"},wM=/fa[srltdbk\-\ ]/,OT="fa-layers-text",bM=/Font ?Awesome ?([56 ]*)(Solid|Regular|Light|Thin|Duotone|Brands|Free|Pro|Kit)?.*/i,_M={"900":"fas","400":"far",normal:"far","300":"fal","100":"fat"},ET=[1,2,3,4,5,6,7,8,9,10],SM=ET.concat([11,12,13,14,15,16,17,18,19,20]),xM=["class","data-prefix","data-icon","data-fa-transform","data-fa-mask"],$a={GROUP:"duotone-group",SWAP_OPACITY:"swap-opacity",PRIMARY:"primary",SECONDARY:"secondary"},TM=[].concat($d(Object.keys(Wh)),["2xs","xs","sm","lg","xl","2xl","beat","border","fade","beat-fade","bounce","flip-both","flip-horizontal","flip-vertical","flip","fw","inverse","layers-counter","layers-text","layers","li","pull-left","pull-right","pulse","rotate-180","rotate-270","rotate-90","rotate-by","shake","spin-pulse","spin-reverse","spin","stack-1x","stack-2x","stack","ul",$a.GROUP,$a.SWAP_OPACITY,$a.PRIMARY,$a.SECONDARY]).concat(ET.map(function(i){return"".concat(i,"x")})).concat(SM.map(function(i){return"w-".concat(i)})),RT=aa.FontAwesomeConfig||{};function OM(i){var t=_t.querySelector("script["+i+"]");if(t)return t.getAttribute(i)}function EM(i){return i===""?!0:i==="false"?!1:i==="true"?!0:i}if(_t&&typeof _t.querySelector=="function"){var RM=[["data-family-prefix","familyPrefix"],["data-style-default","styleDefault"],["data-replacement-class","replacementClass"],["data-auto-replace-svg","autoReplaceSvg"],["data-auto-add-css","autoAddCss"],["data-auto-a11y","autoA11y"],["data-search-pseudo-elements","searchPseudoElements"],["data-observe-mutations","observeMutations"],["data-mutate-approach","mutateApproach"],["data-keep-original-source","keepOriginalSource"],["data-measure-performance","measurePerformance"],["data-show-missing-icons","showMissingIcons"]];RM.forEach(function(i){var t=nC(i,2),e=t[0],r=t[1],n=EM(OM(e));n!=null&&(RT[r]=n)})}var PM={familyPrefix:_T,styleDefault:"solid",replacementClass:ST,autoReplaceSvg:!0,autoAddCss:!0,autoA11y:!0,searchPseudoElements:!1,observeMutations:!0,mutateApproach:"async",keepOriginalSource:!0,measurePerformance:!1,showMissingIcons:!0},ac=fe(fe({},PM),RT);ac.autoReplaceSvg||(ac.observeMutations=!1);var xe={};Object.keys(ac).forEach(function(i){Object.defineProperty(xe,i,{enumerable:!0,set:function(e){ac[i]=e,mh.forEach(function(r){return r(xe)})},get:function(){return ac[i]}})});aa.FontAwesomeConfig=xe;var mh=[];function AM(i){return mh.push(i),function(){mh.splice(mh.indexOf(i),1)}}var _o=xm,Oi={size:16,x:0,y:0,rotate:0,flipX:!1,flipY:!1};function DM(i){if(!(!i||!vo)){var t=_t.createElement("style");t.setAttribute("type","text/css"),t.innerHTML=i;for(var e=_t.head.childNodes,r=null,n=e.length-1;n>-1;n--){var o=e[n],a=(o.tagName||"").toUpperCase();["STYLE","LINK"].indexOf(a)>-1&&(r=o)}return _t.head.insertBefore(t,r),i}}var MM="0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";function _c(){for(var i=12,t="";i-- >0;)t+=MM[Math.random()*62|0];return t}function Vl(i){for(var t=[],e=(i||[]).length>>>0;e--;)t[e]=i[e];return t}function lC(i){return i.classList?Vl(i.classList):(i.getAttribute("class")||"").split(" ").filter(function(t){return t})}function PT(i){return"".concat(i).replace(/&/g,"&amp;").replace(/"/g,"&quot;").replace(/'/g,"&#39;").replace(/</g,"&lt;").replace(/>/g,"&gt;")}function FM(i){return Object.keys(i||{}).reduce(function(t,e){return t+"".concat(e,'="').concat(PT(i[e]),'" ')},"").trim()}function Kd(i){return Object.keys(i||{}).reduce(function(t,e){return t+"".concat(e,": ").concat(i[e].trim(),";")},"")}function uC(i){return i.size!==Oi.size||i.x!==Oi.x||i.y!==Oi.y||i.rotate!==Oi.rotate||i.flipX||i.flipY}function IM(i){var t=i.transform,e=i.containerWidth,r=i.iconWidth,n={transform:"translate(".concat(e/2," 256)")},o="translate(".concat(t.x*32,", ").concat(t.y*32,") "),a="scale(".concat(t.size/16*(t.flipX?-1:1),", ").concat(t.size/16*(t.flipY?-1:1),") "),s="rotate(".concat(t.rotate," 0 0)"),l={transform:"".concat(o," ").concat(a," ").concat(s)},u={transform:"translate(".concat(r/2*-1," -256)")};return{outer:n,inner:l,path:u}}function LM(i){var t=i.transform,e=i.width,r=e===void 0?xm:e,n=i.height,o=n===void 0?xm:n,a=i.startCentered,s=a===void 0?!1:a,l="";return s&&bT?l+="translate(".concat(t.x/_o-r/2,"em, ").concat(t.y/_o-o/2,"em) "):s?l+="translate(calc(-50% + ".concat(t.x/_o,"em), calc(-50% + ").concat(t.y/_o,"em)) "):l+="translate(".concat(t.x/_o,"em, ").concat(t.y/_o,"em) "),l+="scale(".concat(t.size/_o*(t.flipX?-1:1),", ").concat(t.size/_o*(t.flipY?-1:1),") "),l+="rotate(".concat(t.rotate,"deg) "),l}var NM=`:root, :host {
  --fa-font-solid: normal 900 1em/1 "Font Awesome 6 Solid";
  --fa-font-regular: normal 400 1em/1 "Font Awesome 6 Regular";
  --fa-font-light: normal 300 1em/1 "Font Awesome 6 Light";
  --fa-font-thin: normal 100 1em/1 "Font Awesome 6 Thin";
  --fa-font-duotone: normal 900 1em/1 "Font Awesome 6 Duotone";
  --fa-font-brands: normal 400 1em/1 "Font Awesome 6 Brands";
}

svg:not(:root).svg-inline--fa, svg:not(:host).svg-inline--fa {
  overflow: visible;
  box-sizing: content-box;
}

.svg-inline--fa {
  display: var(--fa-display, inline-block);
  height: 1em;
  overflow: visible;
  vertical-align: -0.125em;
}
.svg-inline--fa.fa-2xs {
  vertical-align: 0.1em;
}
.svg-inline--fa.fa-xs {
  vertical-align: 0em;
}
.svg-inline--fa.fa-sm {
  vertical-align: -0.0714285705em;
}
.svg-inline--fa.fa-lg {
  vertical-align: -0.2em;
}
.svg-inline--fa.fa-xl {
  vertical-align: -0.25em;
}
.svg-inline--fa.fa-2xl {
  vertical-align: -0.3125em;
}
.svg-inline--fa.fa-pull-left {
  margin-right: var(--fa-pull-margin, 0.3em);
  width: auto;
}
.svg-inline--fa.fa-pull-right {
  margin-left: var(--fa-pull-margin, 0.3em);
  width: auto;
}
.svg-inline--fa.fa-li {
  width: var(--fa-li-width, 2em);
  top: 0.25em;
}
.svg-inline--fa.fa-fw {
  width: var(--fa-fw-width, 1.25em);
}

.fa-layers svg.svg-inline--fa {
  bottom: 0;
  left: 0;
  margin: auto;
  position: absolute;
  right: 0;
  top: 0;
}

.fa-layers-counter, .fa-layers-text {
  display: inline-block;
  position: absolute;
  text-align: center;
}

.fa-layers {
  display: inline-block;
  height: 1em;
  position: relative;
  text-align: center;
  vertical-align: -0.125em;
  width: 1em;
}
.fa-layers svg.svg-inline--fa {
  -webkit-transform-origin: center center;
          transform-origin: center center;
}

.fa-layers-text {
  left: 50%;
  top: 50%;
  -webkit-transform: translate(-50%, -50%);
          transform: translate(-50%, -50%);
  -webkit-transform-origin: center center;
          transform-origin: center center;
}

.fa-layers-counter {
  background-color: var(--fa-counter-background-color, #ff253a);
  border-radius: var(--fa-counter-border-radius, 1em);
  box-sizing: border-box;
  color: var(--fa-inverse, #fff);
  line-height: var(--fa-counter-line-height, 1);
  max-width: var(--fa-counter-max-width, 5em);
  min-width: var(--fa-counter-min-width, 1.5em);
  overflow: hidden;
  padding: var(--fa-counter-padding, 0.25em 0.5em);
  right: var(--fa-right, 0);
  text-overflow: ellipsis;
  top: var(--fa-top, 0);
  -webkit-transform: scale(var(--fa-counter-scale, 0.25));
          transform: scale(var(--fa-counter-scale, 0.25));
  -webkit-transform-origin: top right;
          transform-origin: top right;
}

.fa-layers-bottom-right {
  bottom: var(--fa-bottom, 0);
  right: var(--fa-right, 0);
  top: auto;
  -webkit-transform: scale(var(--fa-layers-scale, 0.25));
          transform: scale(var(--fa-layers-scale, 0.25));
  -webkit-transform-origin: bottom right;
          transform-origin: bottom right;
}

.fa-layers-bottom-left {
  bottom: var(--fa-bottom, 0);
  left: var(--fa-left, 0);
  right: auto;
  top: auto;
  -webkit-transform: scale(var(--fa-layers-scale, 0.25));
          transform: scale(var(--fa-layers-scale, 0.25));
  -webkit-transform-origin: bottom left;
          transform-origin: bottom left;
}

.fa-layers-top-right {
  top: var(--fa-top, 0);
  right: var(--fa-right, 0);
  -webkit-transform: scale(var(--fa-layers-scale, 0.25));
          transform: scale(var(--fa-layers-scale, 0.25));
  -webkit-transform-origin: top right;
          transform-origin: top right;
}

.fa-layers-top-left {
  left: var(--fa-left, 0);
  right: auto;
  top: var(--fa-top, 0);
  -webkit-transform: scale(var(--fa-layers-scale, 0.25));
          transform: scale(var(--fa-layers-scale, 0.25));
  -webkit-transform-origin: top left;
          transform-origin: top left;
}

.fa-1x {
  font-size: 1em;
}

.fa-2x {
  font-size: 2em;
}

.fa-3x {
  font-size: 3em;
}

.fa-4x {
  font-size: 4em;
}

.fa-5x {
  font-size: 5em;
}

.fa-6x {
  font-size: 6em;
}

.fa-7x {
  font-size: 7em;
}

.fa-8x {
  font-size: 8em;
}

.fa-9x {
  font-size: 9em;
}

.fa-10x {
  font-size: 10em;
}

.fa-2xs {
  font-size: 0.625em;
  line-height: 0.1em;
  vertical-align: 0.225em;
}

.fa-xs {
  font-size: 0.75em;
  line-height: 0.0833333337em;
  vertical-align: 0.125em;
}

.fa-sm {
  font-size: 0.875em;
  line-height: 0.0714285718em;
  vertical-align: 0.0535714295em;
}

.fa-lg {
  font-size: 1.25em;
  line-height: 0.05em;
  vertical-align: -0.075em;
}

.fa-xl {
  font-size: 1.5em;
  line-height: 0.0416666682em;
  vertical-align: -0.125em;
}

.fa-2xl {
  font-size: 2em;
  line-height: 0.03125em;
  vertical-align: -0.1875em;
}

.fa-fw {
  text-align: center;
  width: 1.25em;
}

.fa-ul {
  list-style-type: none;
  margin-left: var(--fa-li-margin, 2.5em);
  padding-left: 0;
}
.fa-ul > li {
  position: relative;
}

.fa-li {
  left: calc(var(--fa-li-width, 2em) * -1);
  position: absolute;
  text-align: center;
  width: var(--fa-li-width, 2em);
  line-height: inherit;
}

.fa-border {
  border-color: var(--fa-border-color, #eee);
  border-radius: var(--fa-border-radius, 0.1em);
  border-style: var(--fa-border-style, solid);
  border-width: var(--fa-border-width, 0.08em);
  padding: var(--fa-border-padding, 0.2em 0.25em 0.15em);
}

.fa-pull-left {
  float: left;
  margin-right: var(--fa-pull-margin, 0.3em);
}

.fa-pull-right {
  float: right;
  margin-left: var(--fa-pull-margin, 0.3em);
}

.fa-beat {
  -webkit-animation-name: fa-beat;
          animation-name: fa-beat;
  -webkit-animation-delay: var(--fa-animation-delay, 0);
          animation-delay: var(--fa-animation-delay, 0);
  -webkit-animation-direction: var(--fa-animation-direction, normal);
          animation-direction: var(--fa-animation-direction, normal);
  -webkit-animation-duration: var(--fa-animation-duration, 1s);
          animation-duration: var(--fa-animation-duration, 1s);
  -webkit-animation-iteration-count: var(--fa-animation-iteration-count, infinite);
          animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  -webkit-animation-timing-function: var(--fa-animation-timing, ease-in-out);
          animation-timing-function: var(--fa-animation-timing, ease-in-out);
}

.fa-bounce {
  -webkit-animation-name: fa-bounce;
          animation-name: fa-bounce;
  -webkit-animation-delay: var(--fa-animation-delay, 0);
          animation-delay: var(--fa-animation-delay, 0);
  -webkit-animation-direction: var(--fa-animation-direction, normal);
          animation-direction: var(--fa-animation-direction, normal);
  -webkit-animation-duration: var(--fa-animation-duration, 1s);
          animation-duration: var(--fa-animation-duration, 1s);
  -webkit-animation-iteration-count: var(--fa-animation-iteration-count, infinite);
          animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  -webkit-animation-timing-function: var(--fa-animation-timing, cubic-bezier(0.28, 0.84, 0.42, 1));
          animation-timing-function: var(--fa-animation-timing, cubic-bezier(0.28, 0.84, 0.42, 1));
}

.fa-fade {
  -webkit-animation-name: fa-fade;
          animation-name: fa-fade;
  -webkit-animation-delay: var(--fa-animation-delay, 0);
          animation-delay: var(--fa-animation-delay, 0);
  -webkit-animation-direction: var(--fa-animation-direction, normal);
          animation-direction: var(--fa-animation-direction, normal);
  -webkit-animation-duration: var(--fa-animation-duration, 1s);
          animation-duration: var(--fa-animation-duration, 1s);
  -webkit-animation-iteration-count: var(--fa-animation-iteration-count, infinite);
          animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  -webkit-animation-timing-function: var(--fa-animation-timing, cubic-bezier(0.4, 0, 0.6, 1));
          animation-timing-function: var(--fa-animation-timing, cubic-bezier(0.4, 0, 0.6, 1));
}

.fa-beat-fade {
  -webkit-animation-name: fa-beat-fade;
          animation-name: fa-beat-fade;
  -webkit-animation-delay: var(--fa-animation-delay, 0);
          animation-delay: var(--fa-animation-delay, 0);
  -webkit-animation-direction: var(--fa-animation-direction, normal);
          animation-direction: var(--fa-animation-direction, normal);
  -webkit-animation-duration: var(--fa-animation-duration, 1s);
          animation-duration: var(--fa-animation-duration, 1s);
  -webkit-animation-iteration-count: var(--fa-animation-iteration-count, infinite);
          animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  -webkit-animation-timing-function: var(--fa-animation-timing, cubic-bezier(0.4, 0, 0.6, 1));
          animation-timing-function: var(--fa-animation-timing, cubic-bezier(0.4, 0, 0.6, 1));
}

.fa-flip {
  -webkit-animation-name: fa-flip;
          animation-name: fa-flip;
  -webkit-animation-delay: var(--fa-animation-delay, 0);
          animation-delay: var(--fa-animation-delay, 0);
  -webkit-animation-direction: var(--fa-animation-direction, normal);
          animation-direction: var(--fa-animation-direction, normal);
  -webkit-animation-duration: var(--fa-animation-duration, 1s);
          animation-duration: var(--fa-animation-duration, 1s);
  -webkit-animation-iteration-count: var(--fa-animation-iteration-count, infinite);
          animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  -webkit-animation-timing-function: var(--fa-animation-timing, ease-in-out);
          animation-timing-function: var(--fa-animation-timing, ease-in-out);
}

.fa-shake {
  -webkit-animation-name: fa-shake;
          animation-name: fa-shake;
  -webkit-animation-delay: var(--fa-animation-delay, 0);
          animation-delay: var(--fa-animation-delay, 0);
  -webkit-animation-direction: var(--fa-animation-direction, normal);
          animation-direction: var(--fa-animation-direction, normal);
  -webkit-animation-duration: var(--fa-animation-duration, 1s);
          animation-duration: var(--fa-animation-duration, 1s);
  -webkit-animation-iteration-count: var(--fa-animation-iteration-count, infinite);
          animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  -webkit-animation-timing-function: var(--fa-animation-timing, linear);
          animation-timing-function: var(--fa-animation-timing, linear);
}

.fa-spin {
  -webkit-animation-name: fa-spin;
          animation-name: fa-spin;
  -webkit-animation-delay: var(--fa-animation-delay, 0);
          animation-delay: var(--fa-animation-delay, 0);
  -webkit-animation-direction: var(--fa-animation-direction, normal);
          animation-direction: var(--fa-animation-direction, normal);
  -webkit-animation-duration: var(--fa-animation-duration, 2s);
          animation-duration: var(--fa-animation-duration, 2s);
  -webkit-animation-iteration-count: var(--fa-animation-iteration-count, infinite);
          animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  -webkit-animation-timing-function: var(--fa-animation-timing, linear);
          animation-timing-function: var(--fa-animation-timing, linear);
}

.fa-spin-reverse {
  --fa-animation-direction: reverse;
}

.fa-pulse,
.fa-spin-pulse {
  -webkit-animation-name: fa-spin;
          animation-name: fa-spin;
  -webkit-animation-direction: var(--fa-animation-direction, normal);
          animation-direction: var(--fa-animation-direction, normal);
  -webkit-animation-duration: var(--fa-animation-duration, 1s);
          animation-duration: var(--fa-animation-duration, 1s);
  -webkit-animation-iteration-count: var(--fa-animation-iteration-count, infinite);
          animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  -webkit-animation-timing-function: var(--fa-animation-timing, steps(8));
          animation-timing-function: var(--fa-animation-timing, steps(8));
}

@media (prefers-reduced-motion: reduce) {
  .fa-beat,
.fa-bounce,
.fa-fade,
.fa-beat-fade,
.fa-flip,
.fa-pulse,
.fa-shake,
.fa-spin,
.fa-spin-pulse {
    -webkit-animation-delay: -1ms;
            animation-delay: -1ms;
    -webkit-animation-duration: 1ms;
            animation-duration: 1ms;
    -webkit-animation-iteration-count: 1;
            animation-iteration-count: 1;
    transition-delay: 0s;
    transition-duration: 0s;
  }
}
@-webkit-keyframes fa-beat {
  0%, 90% {
    -webkit-transform: scale(1);
            transform: scale(1);
  }
  45% {
    -webkit-transform: scale(var(--fa-beat-scale, 1.25));
            transform: scale(var(--fa-beat-scale, 1.25));
  }
}
@keyframes fa-beat {
  0%, 90% {
    -webkit-transform: scale(1);
            transform: scale(1);
  }
  45% {
    -webkit-transform: scale(var(--fa-beat-scale, 1.25));
            transform: scale(var(--fa-beat-scale, 1.25));
  }
}
@-webkit-keyframes fa-bounce {
  0% {
    -webkit-transform: scale(1, 1) translateY(0);
            transform: scale(1, 1) translateY(0);
  }
  10% {
    -webkit-transform: scale(var(--fa-bounce-start-scale-x, 1.1), var(--fa-bounce-start-scale-y, 0.9)) translateY(0);
            transform: scale(var(--fa-bounce-start-scale-x, 1.1), var(--fa-bounce-start-scale-y, 0.9)) translateY(0);
  }
  30% {
    -webkit-transform: scale(var(--fa-bounce-jump-scale-x, 0.9), var(--fa-bounce-jump-scale-y, 1.1)) translateY(var(--fa-bounce-height, -0.5em));
            transform: scale(var(--fa-bounce-jump-scale-x, 0.9), var(--fa-bounce-jump-scale-y, 1.1)) translateY(var(--fa-bounce-height, -0.5em));
  }
  50% {
    -webkit-transform: scale(var(--fa-bounce-land-scale-x, 1.05), var(--fa-bounce-land-scale-y, 0.95)) translateY(0);
            transform: scale(var(--fa-bounce-land-scale-x, 1.05), var(--fa-bounce-land-scale-y, 0.95)) translateY(0);
  }
  57% {
    -webkit-transform: scale(1, 1) translateY(var(--fa-bounce-rebound, -0.125em));
            transform: scale(1, 1) translateY(var(--fa-bounce-rebound, -0.125em));
  }
  64% {
    -webkit-transform: scale(1, 1) translateY(0);
            transform: scale(1, 1) translateY(0);
  }
  100% {
    -webkit-transform: scale(1, 1) translateY(0);
            transform: scale(1, 1) translateY(0);
  }
}
@keyframes fa-bounce {
  0% {
    -webkit-transform: scale(1, 1) translateY(0);
            transform: scale(1, 1) translateY(0);
  }
  10% {
    -webkit-transform: scale(var(--fa-bounce-start-scale-x, 1.1), var(--fa-bounce-start-scale-y, 0.9)) translateY(0);
            transform: scale(var(--fa-bounce-start-scale-x, 1.1), var(--fa-bounce-start-scale-y, 0.9)) translateY(0);
  }
  30% {
    -webkit-transform: scale(var(--fa-bounce-jump-scale-x, 0.9), var(--fa-bounce-jump-scale-y, 1.1)) translateY(var(--fa-bounce-height, -0.5em));
            transform: scale(var(--fa-bounce-jump-scale-x, 0.9), var(--fa-bounce-jump-scale-y, 1.1)) translateY(var(--fa-bounce-height, -0.5em));
  }
  50% {
    -webkit-transform: scale(var(--fa-bounce-land-scale-x, 1.05), var(--fa-bounce-land-scale-y, 0.95)) translateY(0);
            transform: scale(var(--fa-bounce-land-scale-x, 1.05), var(--fa-bounce-land-scale-y, 0.95)) translateY(0);
  }
  57% {
    -webkit-transform: scale(1, 1) translateY(var(--fa-bounce-rebound, -0.125em));
            transform: scale(1, 1) translateY(var(--fa-bounce-rebound, -0.125em));
  }
  64% {
    -webkit-transform: scale(1, 1) translateY(0);
            transform: scale(1, 1) translateY(0);
  }
  100% {
    -webkit-transform: scale(1, 1) translateY(0);
            transform: scale(1, 1) translateY(0);
  }
}
@-webkit-keyframes fa-fade {
  50% {
    opacity: var(--fa-fade-opacity, 0.4);
  }
}
@keyframes fa-fade {
  50% {
    opacity: var(--fa-fade-opacity, 0.4);
  }
}
@-webkit-keyframes fa-beat-fade {
  0%, 100% {
    opacity: var(--fa-beat-fade-opacity, 0.4);
    -webkit-transform: scale(1);
            transform: scale(1);
  }
  50% {
    opacity: 1;
    -webkit-transform: scale(var(--fa-beat-fade-scale, 1.125));
            transform: scale(var(--fa-beat-fade-scale, 1.125));
  }
}
@keyframes fa-beat-fade {
  0%, 100% {
    opacity: var(--fa-beat-fade-opacity, 0.4);
    -webkit-transform: scale(1);
            transform: scale(1);
  }
  50% {
    opacity: 1;
    -webkit-transform: scale(var(--fa-beat-fade-scale, 1.125));
            transform: scale(var(--fa-beat-fade-scale, 1.125));
  }
}
@-webkit-keyframes fa-flip {
  50% {
    -webkit-transform: rotate3d(var(--fa-flip-x, 0), var(--fa-flip-y, 1), var(--fa-flip-z, 0), var(--fa-flip-angle, -180deg));
            transform: rotate3d(var(--fa-flip-x, 0), var(--fa-flip-y, 1), var(--fa-flip-z, 0), var(--fa-flip-angle, -180deg));
  }
}
@keyframes fa-flip {
  50% {
    -webkit-transform: rotate3d(var(--fa-flip-x, 0), var(--fa-flip-y, 1), var(--fa-flip-z, 0), var(--fa-flip-angle, -180deg));
            transform: rotate3d(var(--fa-flip-x, 0), var(--fa-flip-y, 1), var(--fa-flip-z, 0), var(--fa-flip-angle, -180deg));
  }
}
@-webkit-keyframes fa-shake {
  0% {
    -webkit-transform: rotate(-15deg);
            transform: rotate(-15deg);
  }
  4% {
    -webkit-transform: rotate(15deg);
            transform: rotate(15deg);
  }
  8%, 24% {
    -webkit-transform: rotate(-18deg);
            transform: rotate(-18deg);
  }
  12%, 28% {
    -webkit-transform: rotate(18deg);
            transform: rotate(18deg);
  }
  16% {
    -webkit-transform: rotate(-22deg);
            transform: rotate(-22deg);
  }
  20% {
    -webkit-transform: rotate(22deg);
            transform: rotate(22deg);
  }
  32% {
    -webkit-transform: rotate(-12deg);
            transform: rotate(-12deg);
  }
  36% {
    -webkit-transform: rotate(12deg);
            transform: rotate(12deg);
  }
  40%, 100% {
    -webkit-transform: rotate(0deg);
            transform: rotate(0deg);
  }
}
@keyframes fa-shake {
  0% {
    -webkit-transform: rotate(-15deg);
            transform: rotate(-15deg);
  }
  4% {
    -webkit-transform: rotate(15deg);
            transform: rotate(15deg);
  }
  8%, 24% {
    -webkit-transform: rotate(-18deg);
            transform: rotate(-18deg);
  }
  12%, 28% {
    -webkit-transform: rotate(18deg);
            transform: rotate(18deg);
  }
  16% {
    -webkit-transform: rotate(-22deg);
            transform: rotate(-22deg);
  }
  20% {
    -webkit-transform: rotate(22deg);
            transform: rotate(22deg);
  }
  32% {
    -webkit-transform: rotate(-12deg);
            transform: rotate(-12deg);
  }
  36% {
    -webkit-transform: rotate(12deg);
            transform: rotate(12deg);
  }
  40%, 100% {
    -webkit-transform: rotate(0deg);
            transform: rotate(0deg);
  }
}
@-webkit-keyframes fa-spin {
  0% {
    -webkit-transform: rotate(0deg);
            transform: rotate(0deg);
  }
  100% {
    -webkit-transform: rotate(360deg);
            transform: rotate(360deg);
  }
}
@keyframes fa-spin {
  0% {
    -webkit-transform: rotate(0deg);
            transform: rotate(0deg);
  }
  100% {
    -webkit-transform: rotate(360deg);
            transform: rotate(360deg);
  }
}
.fa-rotate-90 {
  -webkit-transform: rotate(90deg);
          transform: rotate(90deg);
}

.fa-rotate-180 {
  -webkit-transform: rotate(180deg);
          transform: rotate(180deg);
}

.fa-rotate-270 {
  -webkit-transform: rotate(270deg);
          transform: rotate(270deg);
}

.fa-flip-horizontal {
  -webkit-transform: scale(-1, 1);
          transform: scale(-1, 1);
}

.fa-flip-vertical {
  -webkit-transform: scale(1, -1);
          transform: scale(1, -1);
}

.fa-flip-both,
.fa-flip-horizontal.fa-flip-vertical {
  -webkit-transform: scale(-1, -1);
          transform: scale(-1, -1);
}

.fa-rotate-by {
  -webkit-transform: rotate(var(--fa-rotate-angle, none));
          transform: rotate(var(--fa-rotate-angle, none));
}

.fa-stack {
  display: inline-block;
  vertical-align: middle;
  height: 2em;
  position: relative;
  width: 2.5em;
}

.fa-stack-1x,
.fa-stack-2x {
  bottom: 0;
  left: 0;
  margin: auto;
  position: absolute;
  right: 0;
  top: 0;
  z-index: var(--fa-stack-z-index, auto);
}

.svg-inline--fa.fa-stack-1x {
  height: 1em;
  width: 1.25em;
}
.svg-inline--fa.fa-stack-2x {
  height: 2em;
  width: 2.5em;
}

.fa-inverse {
  color: var(--fa-inverse, #fff);
}

.sr-only,
.fa-sr-only {
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  margin: -1px;
  overflow: hidden;
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border-width: 0;
}

.sr-only-focusable:not(:focus),
.fa-sr-only-focusable:not(:focus) {
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  margin: -1px;
  overflow: hidden;
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border-width: 0;
}

.svg-inline--fa .fa-primary {
  fill: var(--fa-primary-color, currentColor);
  opacity: var(--fa-primary-opacity, 1);
}

.svg-inline--fa .fa-secondary {
  fill: var(--fa-secondary-color, currentColor);
  opacity: var(--fa-secondary-opacity, 0.4);
}

.svg-inline--fa.fa-swap-opacity .fa-primary {
  opacity: var(--fa-secondary-opacity, 0.4);
}

.svg-inline--fa.fa-swap-opacity .fa-secondary {
  opacity: var(--fa-primary-opacity, 1);
}

.svg-inline--fa mask .fa-primary,
.svg-inline--fa mask .fa-secondary {
  fill: black;
}

.fad.fa-inverse,
.fa-duotone.fa-inverse {
  color: var(--fa-inverse, #fff);
}`;function AT(){var i=_T,t=ST,e=xe.familyPrefix,r=xe.replacementClass,n=NM;if(e!==i||r!==t){var o=new RegExp("\\.".concat(i,"\\-"),"g"),a=new RegExp("\\--".concat(i,"\\-"),"g"),s=new RegExp("\\.".concat(t),"g");n=n.replace(o,".".concat(e,"-")).replace(a,"--".concat(e,"-")).replace(s,".".concat(r))}return n}var Db=!1;function tg(){xe.autoAddCss&&!Db&&(DM(AT()),Db=!0)}var GM={mixout:function(){return{dom:{css:AT,insertCss:tg}}},hooks:function(){return{beforeDOMElementCreation:function(){tg()},beforeI2svg:function(){tg()}}}},so=aa||{};so[ao]||(so[ao]={});so[ao].styles||(so[ao].styles={});so[ao].hooks||(so[ao].hooks={});so[ao].shims||(so[ao].shims=[]);var Zn=so[ao],DT=[],kM=function i(){_t.removeEventListener("DOMContentLoaded",i),Vh=1,DT.map(function(t){return t()})},Vh=!1;vo&&(Vh=(_t.documentElement.doScroll?/^loaded|^c/:/^loaded|^i|^c/).test(_t.readyState),Vh||_t.addEventListener("DOMContentLoaded",kM));function WM(i){!vo||(Vh?setTimeout(i,0):DT.push(i))}function Bc(i){var t=i.tag,e=i.attributes,r=e===void 0?{}:e,n=i.children,o=n===void 0?[]:n;return typeof i=="string"?PT(i):"<".concat(t," ").concat(FM(r),">").concat(o.map(Bc).join(""),"</").concat(t,">")}function Mb(i,t,e){if(i&&i[t]&&i[t][e])return{prefix:t,iconName:e,icon:i[t][e]}}var VM=function(t,e){return function(r,n,o,a){return t.call(e,r,n,o,a)}},rg=function(t,e,r,n){var o=Object.keys(t),a=o.length,s=n!==void 0?VM(e,n):e,l,u,c;for(r===void 0?(l=1,c=t[o[0]]):(l=0,c=r);l<a;l++)u=o[l],c=s(c,t[u],u,t);return c};function BM(i){for(var t=[],e=0,r=i.length;e<r;){var n=i.charCodeAt(e++);if(n>=55296&&n<=56319&&e<r){var o=i.charCodeAt(e++);(o&64512)==56320?t.push(((n&1023)<<10)+(o&1023)+65536):(t.push(n),e--)}else t.push(n)}return t}function Om(i){var t=BM(i);return t.length===1?t[0].toString(16):null}function HM(i,t){var e=i.length,r=i.charCodeAt(t),n;return r>=55296&&r<=56319&&e>t+1&&(n=i.charCodeAt(t+1),n>=56320&&n<=57343)?(r-55296)*1024+n-56320+65536:r}function Fb(i){return Object.keys(i).reduce(function(t,e){var r=i[e],n=!!r.icon;return n?t[r.iconName]=r.icon:t[e]=r,t},{})}function Em(i,t){var e=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},r=e.skipHooks,n=r===void 0?!1:r,o=Fb(t);typeof Zn.hooks.addPack=="function"&&!n?Zn.hooks.addPack(i,Fb(t)):Zn.styles[i]=fe(fe({},Zn.styles[i]||{}),o),i==="fas"&&Em("fa",t)}var sc=Zn.styles,jM=Zn.shims,UM=Object.values(TT),cC=null,MT={},FT={},IT={},LT={},NT={},zM=Object.keys(sC);function $M(i){return~TM.indexOf(i)}function KM(i,t){var e=t.split("-"),r=e[0],n=e.slice(1).join("-");return r===i&&n!==""&&!$M(n)?n:null}var GT=function(){var t=function(o){return rg(sc,function(a,s,l){return a[l]=rg(s,o,{}),a},{})};MT=t(function(n,o,a){if(o[3]&&(n[o[3]]=a),o[2]){var s=o[2].filter(function(l){return typeof l=="number"});s.forEach(function(l){n[l.toString(16)]=a})}return n}),FT=t(function(n,o,a){if(n[a]=a,o[2]){var s=o[2].filter(function(l){return typeof l=="string"});s.forEach(function(l){n[l]=a})}return n}),NT=t(function(n,o,a){var s=o[2];return n[a]=a,s.forEach(function(l){n[l]=a}),n});var e="far"in sc||xe.autoFetchSvg,r=rg(jM,function(n,o){var a=o[0],s=o[1],l=o[2];return s==="far"&&!e&&(s="fas"),typeof a=="string"&&(n.names[a]={prefix:s,iconName:l}),typeof a=="number"&&(n.unicodes[a.toString(16)]={prefix:s,iconName:l}),n},{names:{},unicodes:{}});IT=r.names,LT=r.unicodes,cC=Yd(xe.styleDefault)};AM(function(i){cC=Yd(i.styleDefault)});GT();function pC(i,t){return(MT[i]||{})[t]}function YM(i,t){return(FT[i]||{})[t]}function cl(i,t){return(NT[i]||{})[t]}function kT(i){return IT[i]||{prefix:null,iconName:null}}function XM(i){var t=LT[i],e=pC("fas",i);return t||(e?{prefix:"fas",iconName:e}:null)||{prefix:null,iconName:null}}function sa(){return cC}var hC=function(){return{prefix:null,iconName:null,rest:[]}};function Yd(i){var t=sC[i],e=Wh[i]||Wh[t],r=i in Zn.styles?i:null;return e||r||null}function Xd(i){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},e=t.skipLookups,r=e===void 0?!1:e,n=null,o=i.reduce(function(a,s){var l=KM(xe.familyPrefix,s);if(sc[s]?(s=UM.includes(s)?CM[s]:s,n=s,a.prefix=s):zM.indexOf(s)>-1?(n=s,a.prefix=Yd(s)):l?a.iconName=l:s!==xe.replacementClass&&a.rest.push(s),!r&&a.prefix&&a.iconName){var u=n==="fa"?kT(a.iconName):{},c=cl(a.prefix,a.iconName);u.prefix&&(n=null),a.iconName=u.iconName||c||a.iconName,a.prefix=u.prefix||a.prefix,a.prefix==="far"&&!sc.far&&sc.fas&&!xe.autoFetchSvg&&(a.prefix="fas")}return a},hC());return(o.prefix==="fa"||n==="fa")&&(o.prefix=sa()||"fas"),o}var qM=function(){function i(){aM(this,i),this.definitions={}}return sM(i,[{key:"add",value:function(){for(var e=this,r=arguments.length,n=new Array(r),o=0;o<r;o++)n[o]=arguments[o];var a=n.reduce(this._pullDefinitions,{});Object.keys(a).forEach(function(s){e.definitions[s]=fe(fe({},e.definitions[s]||{}),a[s]),Em(s,a[s]);var l=TT[s];l&&Em(l,a[s]),GT()})}},{key:"reset",value:function(){this.definitions={}}},{key:"_pullDefinitions",value:function(e,r){var n=r.prefix&&r.iconName&&r.icon?{0:r}:r;return Object.keys(n).map(function(o){var a=n[o],s=a.prefix,l=a.iconName,u=a.icon,c=u[2];e[s]||(e[s]={}),c.length>0&&c.forEach(function(p){typeof p=="string"&&(e[s][p]=u)}),e[s][l]=u}),e}}]),i}(),Ib=[],pl={},_l={},ZM=Object.keys(_l);function QM(i,t){var e=t.mixoutsTo;return Ib=i,pl={},Object.keys(_l).forEach(function(r){ZM.indexOf(r)===-1&&delete _l[r]}),Ib.forEach(function(r){var n=r.mixout?r.mixout():{};if(Object.keys(n).forEach(function(a){typeof n[a]=="function"&&(e[a]=n[a]),kh(n[a])==="object"&&Object.keys(n[a]).forEach(function(s){e[a]||(e[a]={}),e[a][s]=n[a][s]})}),r.hooks){var o=r.hooks();Object.keys(o).forEach(function(a){pl[a]||(pl[a]=[]),pl[a].push(o[a])})}r.provides&&r.provides(_l)}),e}function Rm(i,t){for(var e=arguments.length,r=new Array(e>2?e-2:0),n=2;n<e;n++)r[n-2]=arguments[n];var o=pl[i]||[];return o.forEach(function(a){t=a.apply(null,[t].concat(r))}),t}function as(i){for(var t=arguments.length,e=new Array(t>1?t-1:0),r=1;r<t;r++)e[r-1]=arguments[r];var n=pl[i]||[];n.forEach(function(o){o.apply(null,e)})}function lo(){var i=arguments[0],t=Array.prototype.slice.call(arguments,1);return _l[i]?_l[i].apply(null,t):void 0}function Pm(i){i.prefix==="fa"&&(i.prefix="fas");var t=i.iconName,e=i.prefix||sa();if(!!t)return t=cl(e,t)||t,Mb(WT.definitions,e,t)||Mb(Zn.styles,e,t)}var WT=new qM,JM=function(){xe.autoReplaceSvg=!1,xe.observeMutations=!1,as("noAuto")},e1={i2svg:function(){var t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};return vo?(as("beforeI2svg",t),lo("pseudoElements2svg",t),lo("i2svg",t)):Promise.reject("Operation requires a DOM of some kind.")},watch:function(){var t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},e=t.autoReplaceSvgRoot;xe.autoReplaceSvg===!1&&(xe.autoReplaceSvg=!0),xe.observeMutations=!0,WM(function(){r1({autoReplaceSvgRoot:e}),as("watch",t)})}},t1={icon:function(t){if(t===null)return null;if(kh(t)==="object"&&t.prefix&&t.iconName)return{prefix:t.prefix,iconName:cl(t.prefix,t.iconName)||t.iconName};if(Array.isArray(t)&&t.length===2){var e=t[1].indexOf("fa-")===0?t[1].slice(3):t[1],r=Yd(t[0]);return{prefix:r,iconName:cl(r,e)||e}}if(typeof t=="string"&&(t.indexOf("".concat(xe.familyPrefix,"-"))>-1||t.match(wM))){var n=Xd(t.split(" "),{skipLookups:!0});return{prefix:n.prefix||sa(),iconName:cl(n.prefix,n.iconName)||n.iconName}}if(typeof t=="string"){var o=sa();return{prefix:o,iconName:cl(o,t)||t}}}},vn={noAuto:JM,config:xe,dom:e1,parse:t1,library:WT,findIconDefinition:Pm,toHtml:Bc},r1=function(){var t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},e=t.autoReplaceSvgRoot,r=e===void 0?_t:e;(Object.keys(Zn.styles).length>0||xe.autoFetchSvg)&&vo&&xe.autoReplaceSvg&&vn.dom.i2svg({node:r})};function qd(i,t){return Object.defineProperty(i,"abstract",{get:t}),Object.defineProperty(i,"html",{get:function(){return i.abstract.map(function(r){return Bc(r)})}}),Object.defineProperty(i,"node",{get:function(){if(!!vo){var r=_t.createElement("div");return r.innerHTML=i.html,r.children}}}),i}function n1(i){var t=i.children,e=i.main,r=i.mask,n=i.attributes,o=i.styles,a=i.transform;if(uC(a)&&e.found&&!r.found){var s=e.width,l=e.height,u={x:s/l/2,y:.5};n.style=Kd(fe(fe({},o),{},{"transform-origin":"".concat(u.x+a.x/16,"em ").concat(u.y+a.y/16,"em")}))}return[{tag:"svg",attributes:n,children:t}]}function i1(i){var t=i.prefix,e=i.iconName,r=i.children,n=i.attributes,o=i.symbol,a=o===!0?"".concat(t,"-").concat(xe.familyPrefix,"-").concat(e):o;return[{tag:"svg",attributes:{style:"display: none;"},children:[{tag:"symbol",attributes:fe(fe({},n),{},{id:a}),children:r}]}]}function dC(i){var t=i.icons,e=t.main,r=t.mask,n=i.prefix,o=i.iconName,a=i.transform,s=i.symbol,l=i.title,u=i.maskId,c=i.titleId,p=i.extra,h=i.watchable,d=h===void 0?!1:h,f=r.found?r:e,g=f.width,v=f.height,m=n==="fak",C=[xe.replacementClass,o?"".concat(xe.familyPrefix,"-").concat(o):""].filter(function(R){return p.classes.indexOf(R)===-1}).filter(function(R){return R!==""||!!R}).concat(p.classes).join(" "),w={children:[],attributes:fe(fe({},p.attributes),{},{"data-prefix":n,"data-icon":o,class:C,role:p.attributes.role||"img",xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 ".concat(g," ").concat(v)})},b=m&&!~p.classes.indexOf("fa-fw")?{width:"".concat(g/v*16*.0625,"em")}:{};d&&(w.attributes[os]=""),l&&(w.children.push({tag:"title",attributes:{id:w.attributes["aria-labelledby"]||"title-".concat(c||_c())},children:[l]}),delete w.attributes.title);var S=fe(fe({},w),{},{prefix:n,iconName:o,main:e,mask:r,maskId:u,transform:a,symbol:s,styles:fe(fe({},b),p.styles)}),T=r.found&&e.found?lo("generateAbstractMask",S)||{children:[],attributes:{}}:lo("generateAbstractIcon",S)||{children:[],attributes:{}},E=T.children,P=T.attributes;return S.children=E,S.attributes=P,s?i1(S):n1(S)}function Lb(i){var t=i.content,e=i.width,r=i.height,n=i.transform,o=i.title,a=i.extra,s=i.watchable,l=s===void 0?!1:s,u=fe(fe(fe({},a.attributes),o?{title:o}:{}),{},{class:a.classes.join(" ")});l&&(u[os]="");var c=fe({},a.styles);uC(n)&&(c.transform=LM({transform:n,startCentered:!0,width:e,height:r}),c["-webkit-transform"]=c.transform);var p=Kd(c);p.length>0&&(u.style=p);var h=[];return h.push({tag:"span",attributes:u,children:[t]}),o&&h.push({tag:"span",attributes:{class:"sr-only"},children:[o]}),h}function o1(i){var t=i.content,e=i.title,r=i.extra,n=fe(fe(fe({},r.attributes),e?{title:e}:{}),{},{class:r.classes.join(" ")}),o=Kd(r.styles);o.length>0&&(n.style=o);var a=[];return a.push({tag:"span",attributes:n,children:[t]}),e&&a.push({tag:"span",attributes:{class:"sr-only"},children:[e]}),a}var ng=Zn.styles;function Am(i){var t=i[0],e=i[1],r=i.slice(4),n=nC(r,1),o=n[0],a=null;return Array.isArray(o)?a={tag:"g",attributes:{class:"".concat(xe.familyPrefix,"-").concat($a.GROUP)},children:[{tag:"path",attributes:{class:"".concat(xe.familyPrefix,"-").concat($a.SECONDARY),fill:"currentColor",d:o[0]}},{tag:"path",attributes:{class:"".concat(xe.familyPrefix,"-").concat($a.PRIMARY),fill:"currentColor",d:o[1]}}]}:a={tag:"path",attributes:{fill:"currentColor",d:o}},{found:!0,width:t,height:e,icon:a}}var a1={found:!1,width:512,height:512};function s1(i,t){!xT&&!xe.showMissingIcons&&i&&console.error('Icon with name "'.concat(i,'" and prefix "').concat(t,'" is missing.'))}function Dm(i,t){var e=t;return t==="fa"&&xe.styleDefault!==null&&(t=sa()),new Promise(function(r,n){if(lo("missingIconAbstract"),e==="fa"){var o=kT(i)||{};i=o.iconName||i,t=o.prefix||t}if(i&&t&&ng[t]&&ng[t][i]){var a=ng[t][i];return r(Am(a))}s1(i,t),r(fe(fe({},a1),{},{icon:xe.showMissingIcons&&i?lo("missingIconAbstract")||{}:{}}))})}var Nb=function(){},Mm=xe.measurePerformance&&pp&&pp.mark&&pp.measure?pp:{mark:Nb,measure:Nb},$u='FA "6.1.1"',l1=function(t){return Mm.mark("".concat($u," ").concat(t," begins")),function(){return VT(t)}},VT=function(t){Mm.mark("".concat($u," ").concat(t," ends")),Mm.measure("".concat($u," ").concat(t),"".concat($u," ").concat(t," begins"),"".concat($u," ").concat(t," ends"))},fC={begin:l1,end:VT},yh=function(){};function Gb(i){var t=i.getAttribute?i.getAttribute(os):null;return typeof t=="string"}function u1(i){var t=i.getAttribute?i.getAttribute(oC):null,e=i.getAttribute?i.getAttribute(aC):null;return t&&e}function c1(i){return i&&i.classList&&i.classList.contains&&i.classList.contains(xe.replacementClass)}function p1(){if(xe.autoReplaceSvg===!0)return Ch.replace;var i=Ch[xe.autoReplaceSvg];return i||Ch.replace}function h1(i){return _t.createElementNS("http://www.w3.org/2000/svg",i)}function d1(i){return _t.createElement(i)}function BT(i){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},e=t.ceFn,r=e===void 0?i.tag==="svg"?h1:d1:e;if(typeof i=="string")return _t.createTextNode(i);var n=r(i.tag);Object.keys(i.attributes||[]).forEach(function(a){n.setAttribute(a,i.attributes[a])});var o=i.children||[];return o.forEach(function(a){n.appendChild(BT(a,{ceFn:r}))}),n}function f1(i){var t=" ".concat(i.outerHTML," ");return t="".concat(t,"Font Awesome fontawesome.com "),t}var Ch={replace:function(t){var e=t[0];if(e.parentNode)if(t[1].forEach(function(n){e.parentNode.insertBefore(BT(n),e)}),e.getAttribute(os)===null&&xe.keepOriginalSource){var r=_t.createComment(f1(e));e.parentNode.replaceChild(r,e)}else e.remove()},nest:function(t){var e=t[0],r=t[1];if(~lC(e).indexOf(xe.replacementClass))return Ch.replace(t);var n=new RegExp("".concat(xe.familyPrefix,"-.*"));if(delete r[0].attributes.id,r[0].attributes.class){var o=r[0].attributes.class.split(" ").reduce(function(s,l){return l===xe.replacementClass||l.match(n)?s.toSvg.push(l):s.toNode.push(l),s},{toNode:[],toSvg:[]});r[0].attributes.class=o.toSvg.join(" "),o.toNode.length===0?e.removeAttribute("class"):e.setAttribute("class",o.toNode.join(" "))}var a=r.map(function(s){return Bc(s)}).join(`
`);e.setAttribute(os,""),e.innerHTML=a}};function kb(i){i()}function HT(i,t){var e=typeof t=="function"?t:yh;if(i.length===0)e();else{var r=kb;xe.mutateApproach===mM&&(r=aa.requestAnimationFrame||kb),r(function(){var n=p1(),o=fC.begin("mutate");i.map(n),o(),e()})}}var gC=!1;function jT(){gC=!0}function Fm(){gC=!1}var Bh=null;function Wb(i){if(!!Pb&&!!xe.observeMutations){var t=i.treeCallback,e=t===void 0?yh:t,r=i.nodeCallback,n=r===void 0?yh:r,o=i.pseudoElementsCallback,a=o===void 0?yh:o,s=i.observeMutationsRoot,l=s===void 0?_t:s;Bh=new Pb(function(u){if(!gC){var c=sa();Vl(u).forEach(function(p){if(p.type==="childList"&&p.addedNodes.length>0&&!Gb(p.addedNodes[0])&&(xe.searchPseudoElements&&a(p.target),e(p.target)),p.type==="attributes"&&p.target.parentNode&&xe.searchPseudoElements&&a(p.target.parentNode),p.type==="attributes"&&Gb(p.target)&&~xM.indexOf(p.attributeName))if(p.attributeName==="class"&&u1(p.target)){var h=Xd(lC(p.target)),d=h.prefix,f=h.iconName;p.target.setAttribute(oC,d||c),f&&p.target.setAttribute(aC,f)}else c1(p.target)&&n(p.target)})}}),vo&&Bh.observe(l,{childList:!0,attributes:!0,characterData:!0,subtree:!0})}}function g1(){!Bh||Bh.disconnect()}function v1(i){var t=i.getAttribute("style"),e=[];return t&&(e=t.split(";").reduce(function(r,n){var o=n.split(":"),a=o[0],s=o.slice(1);return a&&s.length>0&&(r[a]=s.join(":").trim()),r},{})),e}function m1(i){var t=i.getAttribute("data-prefix"),e=i.getAttribute("data-icon"),r=i.innerText!==void 0?i.innerText.trim():"",n=Xd(lC(i));return n.prefix||(n.prefix=sa()),t&&e&&(n.prefix=t,n.iconName=e),n.iconName&&n.prefix||n.prefix&&r.length>0&&(n.iconName=YM(n.prefix,i.innerText)||pC(n.prefix,Om(i.innerText))),n}function y1(i){var t=Vl(i.attributes).reduce(function(n,o){return n.name!=="class"&&n.name!=="style"&&(n[o.name]=o.value),n},{}),e=i.getAttribute("title"),r=i.getAttribute("data-fa-title-id");return xe.autoA11y&&(e?t["aria-labelledby"]="".concat(xe.replacementClass,"-title-").concat(r||_c()):(t["aria-hidden"]="true",t.focusable="false")),t}function C1(){return{iconName:null,title:null,titleId:null,prefix:null,transform:Oi,symbol:!1,mask:{iconName:null,prefix:null,rest:[]},maskId:null,extra:{classes:[],styles:{},attributes:{}}}}function Vb(i){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{styleParser:!0},e=m1(i),r=e.iconName,n=e.prefix,o=e.rest,a=y1(i),s=Rm("parseNodeAttributes",{},i),l=t.styleParser?v1(i):[];return fe({iconName:r,title:i.getAttribute("title"),titleId:i.getAttribute("data-fa-title-id"),prefix:n,transform:Oi,mask:{iconName:null,prefix:null,rest:[]},maskId:null,symbol:!1,extra:{classes:o,styles:l,attributes:a}},s)}var w1=Zn.styles;function UT(i){var t=xe.autoReplaceSvg==="nest"?Vb(i,{styleParser:!1}):Vb(i);return~t.extra.classes.indexOf(OT)?lo("generateLayersText",i,t):lo("generateSvgReplacementMutation",i,t)}function Bb(i){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:null;if(!vo)return Promise.resolve();var e=_t.documentElement.classList,r=function(p){return e.add("".concat(Ab,"-").concat(p))},n=function(p){return e.remove("".concat(Ab,"-").concat(p))},o=xe.autoFetchSvg?Object.keys(sC):Object.keys(w1),a=[".".concat(OT,":not([").concat(os,"])")].concat(o.map(function(c){return".".concat(c,":not([").concat(os,"])")})).join(", ");if(a.length===0)return Promise.resolve();var s=[];try{s=Vl(i.querySelectorAll(a))}catch{}if(s.length>0)r("pending"),n("complete");else return Promise.resolve();var l=fC.begin("onTree"),u=s.reduce(function(c,p){try{var h=UT(p);h&&c.push(h)}catch(d){xT||d.name==="MissingIcon"&&console.error(d)}return c},[]);return new Promise(function(c,p){Promise.all(u).then(function(h){HT(h,function(){r("active"),r("complete"),n("pending"),typeof t=="function"&&t(),l(),c()})}).catch(function(h){l(),p(h)})})}function b1(i){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:null;UT(i).then(function(e){e&&HT([e],t)})}function _1(i){return function(t){var e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},r=(t||{}).icon?t:Pm(t||{}),n=e.mask;return n&&(n=(n||{}).icon?n:Pm(n||{})),i(r,fe(fe({},e),{},{mask:n}))}}var S1=function(t){var e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},r=e.transform,n=r===void 0?Oi:r,o=e.symbol,a=o===void 0?!1:o,s=e.mask,l=s===void 0?null:s,u=e.maskId,c=u===void 0?null:u,p=e.title,h=p===void 0?null:p,d=e.titleId,f=d===void 0?null:d,g=e.classes,v=g===void 0?[]:g,m=e.attributes,C=m===void 0?{}:m,w=e.styles,b=w===void 0?{}:w;if(!!t){var S=t.prefix,T=t.iconName,E=t.icon;return qd(fe({type:"icon"},t),function(){return as("beforeDOMElementCreation",{iconDefinition:t,params:e}),xe.autoA11y&&(h?C["aria-labelledby"]="".concat(xe.replacementClass,"-title-").concat(f||_c()):(C["aria-hidden"]="true",C.focusable="false")),dC({icons:{main:Am(E),mask:l?Am(l.icon):{found:!1,width:null,height:null,icon:{}}},prefix:S,iconName:T,transform:fe(fe({},Oi),n),symbol:a,title:h,maskId:c,titleId:f,extra:{attributes:C,styles:b,classes:v}})})}},x1={mixout:function(){return{icon:_1(S1)}},hooks:function(){return{mutationObserverCallbacks:function(e){return e.treeCallback=Bb,e.nodeCallback=b1,e}}},provides:function(t){t.i2svg=function(e){var r=e.node,n=r===void 0?_t:r,o=e.callback,a=o===void 0?function(){}:o;return Bb(n,a)},t.generateSvgReplacementMutation=function(e,r){var n=r.iconName,o=r.title,a=r.titleId,s=r.prefix,l=r.transform,u=r.symbol,c=r.mask,p=r.maskId,h=r.extra;return new Promise(function(d,f){Promise.all([Dm(n,s),c.iconName?Dm(c.iconName,c.prefix):Promise.resolve({found:!1,width:512,height:512,icon:{}})]).then(function(g){var v=nC(g,2),m=v[0],C=v[1];d([e,dC({icons:{main:m,mask:C},prefix:s,iconName:n,transform:l,symbol:u,maskId:p,title:o,titleId:a,extra:h,watchable:!0})])}).catch(f)})},t.generateAbstractIcon=function(e){var r=e.children,n=e.attributes,o=e.main,a=e.transform,s=e.styles,l=Kd(s);l.length>0&&(n.style=l);var u;return uC(a)&&(u=lo("generateAbstractTransformGrouping",{main:o,transform:a,containerWidth:o.width,iconWidth:o.width})),r.push(u||o.icon),{children:r,attributes:n}}}},T1={mixout:function(){return{layer:function(e){var r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=r.classes,o=n===void 0?[]:n;return qd({type:"layer"},function(){as("beforeDOMElementCreation",{assembler:e,params:r});var a=[];return e(function(s){Array.isArray(s)?s.map(function(l){a=a.concat(l.abstract)}):a=a.concat(s.abstract)}),[{tag:"span",attributes:{class:["".concat(xe.familyPrefix,"-layers")].concat($d(o)).join(" ")},children:a}]})}}}},O1={mixout:function(){return{counter:function(e){var r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=r.title,o=n===void 0?null:n,a=r.classes,s=a===void 0?[]:a,l=r.attributes,u=l===void 0?{}:l,c=r.styles,p=c===void 0?{}:c;return qd({type:"counter",content:e},function(){return as("beforeDOMElementCreation",{content:e,params:r}),o1({content:e.toString(),title:o,extra:{attributes:u,styles:p,classes:["".concat(xe.familyPrefix,"-layers-counter")].concat($d(s))}})})}}}},E1={mixout:function(){return{text:function(e){var r=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=r.transform,o=n===void 0?Oi:n,a=r.title,s=a===void 0?null:a,l=r.classes,u=l===void 0?[]:l,c=r.attributes,p=c===void 0?{}:c,h=r.styles,d=h===void 0?{}:h;return qd({type:"text",content:e},function(){return as("beforeDOMElementCreation",{content:e,params:r}),Lb({content:e,transform:fe(fe({},Oi),o),title:s,extra:{attributes:p,styles:d,classes:["".concat(xe.familyPrefix,"-layers-text")].concat($d(u))}})})}}},provides:function(t){t.generateLayersText=function(e,r){var n=r.title,o=r.transform,a=r.extra,s=null,l=null;if(bT){var u=parseInt(getComputedStyle(e).fontSize,10),c=e.getBoundingClientRect();s=c.width/u,l=c.height/u}return xe.autoA11y&&!n&&(a.attributes["aria-hidden"]="true"),Promise.resolve([e,Lb({content:e.innerHTML,width:s,height:l,transform:o,title:n,extra:a,watchable:!0})])}}},R1=new RegExp('"',"ug"),Hb=[1105920,1112319];function P1(i){var t=i.replace(R1,""),e=HM(t,0),r=e>=Hb[0]&&e<=Hb[1],n=t.length===2?t[0]===t[1]:!1;return{value:Om(n?t[0]:t),isSecondary:r||n}}function jb(i,t){var e="".concat(vM).concat(t.replace(":","-"));return new Promise(function(r,n){if(i.getAttribute(e)!==null)return r();var o=Vl(i.children),a=o.filter(function(T){return T.getAttribute(Tm)===t})[0],s=aa.getComputedStyle(i,t),l=s.getPropertyValue("font-family").match(bM),u=s.getPropertyValue("font-weight"),c=s.getPropertyValue("content");if(a&&!l)return i.removeChild(a),r();if(l&&c!=="none"&&c!==""){var p=s.getPropertyValue("content"),h=~["Solid","Regular","Light","Thin","Duotone","Brands","Kit"].indexOf(l[2])?Wh[l[2].toLowerCase()]:_M[u],d=P1(p),f=d.value,g=d.isSecondary,v=l[0].startsWith("FontAwesome"),m=pC(h,f),C=m;if(v){var w=XM(f);w.iconName&&w.prefix&&(m=w.iconName,h=w.prefix)}if(m&&!g&&(!a||a.getAttribute(oC)!==h||a.getAttribute(aC)!==C)){i.setAttribute(e,C),a&&i.removeChild(a);var b=C1(),S=b.extra;S.attributes[Tm]=t,Dm(m,h).then(function(T){var E=dC(fe(fe({},b),{},{icons:{main:T,mask:hC()},prefix:h,iconName:C,extra:S,watchable:!0})),P=_t.createElement("svg");t==="::before"?i.insertBefore(P,i.firstChild):i.appendChild(P),P.outerHTML=E.map(function(R){return Bc(R)}).join(`
`),i.removeAttribute(e),r()}).catch(n)}else r()}else r()})}function A1(i){return Promise.all([jb(i,"::before"),jb(i,"::after")])}function D1(i){return i.parentNode!==document.head&&!~yM.indexOf(i.tagName.toUpperCase())&&!i.getAttribute(Tm)&&(!i.parentNode||i.parentNode.tagName!=="svg")}function Ub(i){if(!!vo)return new Promise(function(t,e){var r=Vl(i.querySelectorAll("*")).filter(D1).map(A1),n=fC.begin("searchPseudoElements");jT(),Promise.all(r).then(function(){n(),Fm(),t()}).catch(function(){n(),Fm(),e()})})}var M1={hooks:function(){return{mutationObserverCallbacks:function(e){return e.pseudoElementsCallback=Ub,e}}},provides:function(t){t.pseudoElements2svg=function(e){var r=e.node,n=r===void 0?_t:r;xe.searchPseudoElements&&Ub(n)}}},zb=!1,F1={mixout:function(){return{dom:{unwatch:function(){jT(),zb=!0}}}},hooks:function(){return{bootstrap:function(){Wb(Rm("mutationObserverCallbacks",{}))},noAuto:function(){g1()},watch:function(e){var r=e.observeMutationsRoot;zb?Fm():Wb(Rm("mutationObserverCallbacks",{observeMutationsRoot:r}))}}}},$b=function(t){var e={size:16,x:0,y:0,flipX:!1,flipY:!1,rotate:0};return t.toLowerCase().split(" ").reduce(function(r,n){var o=n.toLowerCase().split("-"),a=o[0],s=o.slice(1).join("-");if(a&&s==="h")return r.flipX=!0,r;if(a&&s==="v")return r.flipY=!0,r;if(s=parseFloat(s),isNaN(s))return r;switch(a){case"grow":r.size=r.size+s;break;case"shrink":r.size=r.size-s;break;case"left":r.x=r.x-s;break;case"right":r.x=r.x+s;break;case"up":r.y=r.y-s;break;case"down":r.y=r.y+s;break;case"rotate":r.rotate=r.rotate+s;break}return r},e)},I1={mixout:function(){return{parse:{transform:function(e){return $b(e)}}}},hooks:function(){return{parseNodeAttributes:function(e,r){var n=r.getAttribute("data-fa-transform");return n&&(e.transform=$b(n)),e}}},provides:function(t){t.generateAbstractTransformGrouping=function(e){var r=e.main,n=e.transform,o=e.containerWidth,a=e.iconWidth,s={transform:"translate(".concat(o/2," 256)")},l="translate(".concat(n.x*32,", ").concat(n.y*32,") "),u="scale(".concat(n.size/16*(n.flipX?-1:1),", ").concat(n.size/16*(n.flipY?-1:1),") "),c="rotate(".concat(n.rotate," 0 0)"),p={transform:"".concat(l," ").concat(u," ").concat(c)},h={transform:"translate(".concat(a/2*-1," -256)")},d={outer:s,inner:p,path:h};return{tag:"g",attributes:fe({},d.outer),children:[{tag:"g",attributes:fe({},d.inner),children:[{tag:r.icon.tag,children:r.icon.children,attributes:fe(fe({},r.icon.attributes),d.path)}]}]}}}},ig={x:0,y:0,width:"100%",height:"100%"};function Kb(i){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0;return i.attributes&&(i.attributes.fill||t)&&(i.attributes.fill="black"),i}function L1(i){return i.tag==="g"?i.children:[i]}var N1={hooks:function(){return{parseNodeAttributes:function(e,r){var n=r.getAttribute("data-fa-mask"),o=n?Xd(n.split(" ").map(function(a){return a.trim()})):hC();return o.prefix||(o.prefix=sa()),e.mask=o,e.maskId=r.getAttribute("data-fa-mask-id"),e}}},provides:function(t){t.generateAbstractMask=function(e){var r=e.children,n=e.attributes,o=e.main,a=e.mask,s=e.maskId,l=e.transform,u=o.width,c=o.icon,p=a.width,h=a.icon,d=IM({transform:l,containerWidth:p,iconWidth:u}),f={tag:"rect",attributes:fe(fe({},ig),{},{fill:"white"})},g=c.children?{children:c.children.map(Kb)}:{},v={tag:"g",attributes:fe({},d.inner),children:[Kb(fe({tag:c.tag,attributes:fe(fe({},c.attributes),d.path)},g))]},m={tag:"g",attributes:fe({},d.outer),children:[v]},C="mask-".concat(s||_c()),w="clip-".concat(s||_c()),b={tag:"mask",attributes:fe(fe({},ig),{},{id:C,maskUnits:"userSpaceOnUse",maskContentUnits:"userSpaceOnUse"}),children:[f,m]},S={tag:"defs",children:[{tag:"clipPath",attributes:{id:w},children:L1(h)},b]};return r.push(S,{tag:"rect",attributes:fe({fill:"currentColor","clip-path":"url(#".concat(w,")"),mask:"url(#".concat(C,")")},ig)}),{children:r,attributes:n}}}},G1={provides:function(t){var e=!1;aa.matchMedia&&(e=aa.matchMedia("(prefers-reduced-motion: reduce)").matches),t.missingIconAbstract=function(){var r=[],n={fill:"currentColor"},o={attributeType:"XML",repeatCount:"indefinite",dur:"2s"};r.push({tag:"path",attributes:fe(fe({},n),{},{d:"M156.5,447.7l-12.6,29.5c-18.7-9.5-35.9-21.2-51.5-34.9l22.7-22.7C127.6,430.5,141.5,440,156.5,447.7z M40.6,272H8.5 c1.4,21.2,5.4,41.7,11.7,61.1L50,321.2C45.1,305.5,41.8,289,40.6,272z M40.6,240c1.4-18.8,5.2-37,11.1-54.1l-29.5-12.6 C14.7,194.3,10,216.7,8.5,240H40.6z M64.3,156.5c7.8-14.9,17.2-28.8,28.1-41.5L69.7,92.3c-13.7,15.6-25.5,32.8-34.9,51.5 L64.3,156.5z M397,419.6c-13.9,12-29.4,22.3-46.1,30.4l11.9,29.8c20.7-9.9,39.8-22.6,56.9-37.6L397,419.6z M115,92.4 c13.9-12,29.4-22.3,46.1-30.4l-11.9-29.8c-20.7,9.9-39.8,22.6-56.8,37.6L115,92.4z M447.7,355.5c-7.8,14.9-17.2,28.8-28.1,41.5 l22.7,22.7c13.7-15.6,25.5-32.9,34.9-51.5L447.7,355.5z M471.4,272c-1.4,18.8-5.2,37-11.1,54.1l29.5,12.6 c7.5-21.1,12.2-43.5,13.6-66.8H471.4z M321.2,462c-15.7,5-32.2,8.2-49.2,9.4v32.1c21.2-1.4,41.7-5.4,61.1-11.7L321.2,462z M240,471.4c-18.8-1.4-37-5.2-54.1-11.1l-12.6,29.5c21.1,7.5,43.5,12.2,66.8,13.6V471.4z M462,190.8c5,15.7,8.2,32.2,9.4,49.2h32.1 c-1.4-21.2-5.4-41.7-11.7-61.1L462,190.8z M92.4,397c-12-13.9-22.3-29.4-30.4-46.1l-29.8,11.9c9.9,20.7,22.6,39.8,37.6,56.9 L92.4,397z M272,40.6c18.8,1.4,36.9,5.2,54.1,11.1l12.6-29.5C317.7,14.7,295.3,10,272,8.5V40.6z M190.8,50 c15.7-5,32.2-8.2,49.2-9.4V8.5c-21.2,1.4-41.7,5.4-61.1,11.7L190.8,50z M442.3,92.3L419.6,115c12,13.9,22.3,29.4,30.5,46.1 l29.8-11.9C470,128.5,457.3,109.4,442.3,92.3z M397,92.4l22.7-22.7c-15.6-13.7-32.8-25.5-51.5-34.9l-12.6,29.5 C370.4,72.1,384.4,81.5,397,92.4z"})});var a=fe(fe({},o),{},{attributeName:"opacity"}),s={tag:"circle",attributes:fe(fe({},n),{},{cx:"256",cy:"364",r:"28"}),children:[]};return e||s.children.push({tag:"animate",attributes:fe(fe({},o),{},{attributeName:"r",values:"28;14;28;28;14;28;"})},{tag:"animate",attributes:fe(fe({},a),{},{values:"1;0;1;1;0;1;"})}),r.push(s),r.push({tag:"path",attributes:fe(fe({},n),{},{opacity:"1",d:"M263.7,312h-16c-6.6,0-12-5.4-12-12c0-71,77.4-63.9,77.4-107.8c0-20-17.8-40.2-57.4-40.2c-29.1,0-44.3,9.6-59.2,28.7 c-3.9,5-11.1,6-16.2,2.4l-13.1-9.2c-5.6-3.9-6.9-11.8-2.6-17.2c21.2-27.2,46.4-44.7,91.2-44.7c52.3,0,97.4,29.8,97.4,80.2 c0,67.6-77.4,63.5-77.4,107.8C275.7,306.6,270.3,312,263.7,312z"}),children:e?[]:[{tag:"animate",attributes:fe(fe({},a),{},{values:"1;0;0;0;0;1;"})}]}),e||r.push({tag:"path",attributes:fe(fe({},n),{},{opacity:"0",d:"M232.5,134.5l7,168c0.3,6.4,5.6,11.5,12,11.5h9c6.4,0,11.7-5.1,12-11.5l7-168c0.3-6.8-5.2-12.5-12-12.5h-23 C237.7,122,232.2,127.7,232.5,134.5z"}),children:[{tag:"animate",attributes:fe(fe({},a),{},{values:"0;0;1;1;0;0;"})}]}),{tag:"g",attributes:{class:"missing"},children:r}}}},k1={hooks:function(){return{parseNodeAttributes:function(e,r){var n=r.getAttribute("data-fa-symbol"),o=n===null?!1:n===""?!0:n;return e.symbol=o,e}}}},W1=[GM,x1,T1,O1,E1,M1,F1,I1,N1,G1,k1];QM(W1,{mixoutsTo:vn});vn.noAuto;vn.config;vn.library;vn.dom;var Im=vn.parse;vn.findIconDefinition;vn.toHtml;var V1=vn.icon;vn.layer;vn.text;vn.counter;var Zd={exports:{}},B1="SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED",H1=B1,j1=H1;function zT(){}function $T(){}$T.resetWarningCache=zT;var U1=function(){function i(r,n,o,a,s,l){if(l!==j1){var u=new Error("Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types");throw u.name="Invariant Violation",u}}i.isRequired=i;function t(){return i}var e={array:i,bigint:i,bool:i,func:i,number:i,object:i,string:i,symbol:i,any:i,arrayOf:t,element:i,elementType:i,instanceOf:t,node:i,objectOf:t,oneOf:t,oneOfType:t,shape:t,exact:t,checkPropTypes:$T,resetWarningCache:zT};return e.PropTypes=e,e};Zd.exports=U1();var je=Zd.exports;function Yb(i,t){var e=Object.keys(i);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(i);t&&(r=r.filter(function(n){return Object.getOwnPropertyDescriptor(i,n).enumerable})),e.push.apply(e,r)}return e}function Uo(i){for(var t=1;t<arguments.length;t++){var e=arguments[t]!=null?arguments[t]:{};t%2?Yb(Object(e),!0).forEach(function(r){hl(i,r,e[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(i,Object.getOwnPropertyDescriptors(e)):Yb(Object(e)).forEach(function(r){Object.defineProperty(i,r,Object.getOwnPropertyDescriptor(e,r))})}return i}function Hh(i){return Hh=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},Hh(i)}function hl(i,t,e){return t in i?Object.defineProperty(i,t,{value:e,enumerable:!0,configurable:!0,writable:!0}):i[t]=e,i}function z1(i,t){if(i==null)return{};var e={},r=Object.keys(i),n,o;for(o=0;o<r.length;o++)n=r[o],!(t.indexOf(n)>=0)&&(e[n]=i[n]);return e}function KT(i,t){if(i==null)return{};var e=z1(i,t),r,n;if(Object.getOwnPropertySymbols){var o=Object.getOwnPropertySymbols(i);for(n=0;n<o.length;n++)r=o[n],!(t.indexOf(r)>=0)&&(!Object.prototype.propertyIsEnumerable.call(i,r)||(e[r]=i[r]))}return e}function Lm(i){return $1(i)||K1(i)||Y1(i)||X1()}function $1(i){if(Array.isArray(i))return Nm(i)}function K1(i){if(typeof Symbol!="undefined"&&i[Symbol.iterator]!=null||i["@@iterator"]!=null)return Array.from(i)}function Y1(i,t){if(!!i){if(typeof i=="string")return Nm(i,t);var e=Object.prototype.toString.call(i).slice(8,-1);if(e==="Object"&&i.constructor&&(e=i.constructor.name),e==="Map"||e==="Set")return Array.from(i);if(e==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(e))return Nm(i,t)}}function Nm(i,t){(t==null||t>i.length)&&(t=i.length);for(var e=0,r=new Array(t);e<t;e++)r[e]=i[e];return r}function X1(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function q1(i){var t,e=i.beat,r=i.fade,n=i.beatFade,o=i.bounce,a=i.shake,s=i.flash,l=i.spin,u=i.spinPulse,c=i.spinReverse,p=i.pulse,h=i.fixedWidth,d=i.inverse,f=i.border,g=i.listItem,v=i.flip,m=i.size,C=i.rotation,w=i.pull,b=(t={"fa-beat":e,"fa-fade":r,"fa-beat-fade":n,"fa-bounce":o,"fa-shake":a,"fa-flash":s,"fa-spin":l,"fa-spin-reverse":c,"fa-spin-pulse":u,"fa-pulse":p,"fa-fw":h,"fa-inverse":d,"fa-border":f,"fa-li":g,"fa-flip-horizontal":v==="horizontal"||v==="both","fa-flip-vertical":v==="vertical"||v==="both"},hl(t,"fa-".concat(m),typeof m!="undefined"&&m!==null),hl(t,"fa-rotate-".concat(C),typeof C!="undefined"&&C!==null&&C!==0),hl(t,"fa-pull-".concat(w),typeof w!="undefined"&&w!==null),hl(t,"fa-swap-opacity",i.swapOpacity),t);return Object.keys(b).map(function(S){return b[S]?S:null}).filter(function(S){return S})}function Z1(i){return i=i-0,i===i}function YT(i){return Z1(i)?i:(i=i.replace(/[\-_\s]+(.)?/g,function(t,e){return e?e.toUpperCase():""}),i.substr(0,1).toLowerCase()+i.substr(1))}var Q1=["style"];function J1(i){return i.charAt(0).toUpperCase()+i.slice(1)}function eF(i){return i.split(";").map(function(t){return t.trim()}).filter(function(t){return t}).reduce(function(t,e){var r=e.indexOf(":"),n=YT(e.slice(0,r)),o=e.slice(r+1).trim();return n.startsWith("webkit")?t[J1(n)]=o:t[n]=o,t},{})}function XT(i,t){var e=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{};if(typeof t=="string")return t;var r=(t.children||[]).map(function(l){return XT(i,l)}),n=Object.keys(t.attributes||{}).reduce(function(l,u){var c=t.attributes[u];switch(u){case"class":l.attrs.className=c,delete t.attributes.class;break;case"style":l.attrs.style=eF(c);break;default:u.indexOf("aria-")===0||u.indexOf("data-")===0?l.attrs[u.toLowerCase()]=c:l.attrs[YT(u)]=c}return l},{attrs:{}}),o=e.style,a=o===void 0?{}:o,s=KT(e,Q1);return n.attrs.style=Uo(Uo({},n.attrs.style),a),i.apply(void 0,[t.tag,Uo(Uo({},n.attrs),s)].concat(Lm(r)))}var qT=!1;try{qT=!0}catch{}function tF(){if(!qT&&console&&typeof console.error=="function"){var i;(i=console).error.apply(i,arguments)}}function Xb(i){if(i&&Hh(i)==="object"&&i.prefix&&i.iconName&&i.icon)return i;if(Im.icon)return Im.icon(i);if(i===null)return null;if(i&&Hh(i)==="object"&&i.prefix&&i.iconName)return i;if(Array.isArray(i)&&i.length===2)return{prefix:i[0],iconName:i[1]};if(typeof i=="string")return{prefix:"fas",iconName:i}}function og(i,t){return Array.isArray(t)&&t.length>0||!Array.isArray(t)&&t?hl({},i,t):{}}var rF=["forwardedRef"];function Fi(i){var t=i.forwardedRef,e=KT(i,rF),r=e.icon,n=e.mask,o=e.symbol,a=e.className,s=e.title,l=e.titleId,u=e.maskId,c=Xb(r),p=og("classes",[].concat(Lm(q1(e)),Lm(a.split(" ")))),h=og("transform",typeof e.transform=="string"?Im.transform(e.transform):e.transform),d=og("mask",Xb(n)),f=V1(c,Uo(Uo(Uo(Uo({},p),h),d),{},{symbol:o,title:s,titleId:l,maskId:u}));if(!f)return tF("Could not find icon",c),null;var g=f.abstract,v={ref:t};return Object.keys(e).forEach(function(m){Fi.defaultProps.hasOwnProperty(m)||(v[m]=e[m])}),nF(g[0],v)}Fi.displayName="FontAwesomeIcon";Fi.propTypes={beat:je.bool,border:je.bool,bounce:je.bool,className:je.string,fade:je.bool,flash:je.bool,mask:je.oneOfType([je.object,je.array,je.string]),maskId:je.string,fixedWidth:je.bool,inverse:je.bool,flip:je.oneOf(["horizontal","vertical","both"]),icon:je.oneOfType([je.object,je.array,je.string]),listItem:je.bool,pull:je.oneOf(["right","left"]),pulse:je.bool,rotation:je.oneOf([0,90,180,270]),shake:je.bool,size:je.oneOf(["2xs","xs","sm","lg","xl","2xl","1x","2x","3x","4x","5x","6x","7x","8x","9x","10x"]),spin:je.bool,spinPulse:je.bool,spinReverse:je.bool,symbol:je.oneOfType([je.bool,je.string]),title:je.string,titleId:je.string,transform:je.oneOfType([je.string,je.object]),swapOpacity:je.bool};Fi.defaultProps={border:!1,className:"",mask:null,maskId:null,fixedWidth:!1,inverse:!1,flip:null,icon:null,listItem:!1,pull:null,pulse:!1,rotation:null,size:null,spin:!1,beat:!1,fade:!1,beatFade:!1,bounce:!1,shake:!1,symbol:!1,title:"",titleId:null,transform:null,swapOpacity:!1};var nF=XT.bind(null,Q0.createElement);const ZT=Et(Qy,{borderRadius:"5px",backgroundColor:"purple",padding:"$2",boxShadow:"0px 4px 6px rgba(0, 0, 0, 0.1)",width:"100%",variants:{color:{purple:{background:"$darkPurpleGradient",color:"$white"},black:{background:"$blackGradient",color:"$white"},gray:{boxShadow:"none",background:"$lightGray",color:"$primaryDark"}},state:{focused:{outline:"1px solid $purpleStroke"},none:{}}},defaultVariants:{color:"purple"}}),iF=Et("div",{borderRadius:"4px",filter:"drop-shadow(0px 2px 4px rgba(0, 0, 0, 0.16))",background:"$lightGray",cursor:"pointer",variants:{state:{focused:{outline:"2px solid $purpleStroke"},none:{}},clickable:{yes:{cursor:"pointer"},no:{cursor:"not-allowed"}}}}),oF=Et("div",{$$borderRadius:"4px",display:"flex",alignItems:"center",gap:"$1",borderTopLeftRadius:"$$borderRadius",borderTopRightRadius:"$$borderRadius",fontSize:"$2",color:"$white",padding:"$1 $2",variants:{component:{app:{background:"$appCardColor"},job:{background:"$jobCardColor"},data:{background:"$dataCardColor"}}}}),aF=Et("div",{$$borderRadius:"4px",background:"$gray7CardBackground",padding:"$3 $2",borderBottomRightRadius:"$$borderRadius",borderBottomLeftRadius:"$$borderRadius"}),vC=Et("div",{variants:{width:{mobile:{width:"100%"},tablet:{width:"45%"},desktop:{width:"320px"}}}}),ag=({message:i})=>se(vC,{width:{"@initial":"mobile","@bp1":"tablet","@bp2":"desktop"},children:se(ZT,{color:"gray",css:{textAlign:"center"},children:se(Fn,{weight:"big",margin:"none",children:i})})}),mC=({type:i,metadata:t,title:e,isSelected:r,onClick:n})=>se(vC,{width:{"@initial":"mobile","@bp1":"tablet","@bp2":"desktop"},children:dt(iF,{state:r?"focused":"none",role:n&&"button",onClick:n,clickable:n?"yes":"no",children:[se(oF,{component:i,children:t}),se(aF,{children:se(QD,{margin:"none",children:e})})]})}),qb=40,sF=({app:i,onClick:t,isSelected:e})=>{var n;const r=o=>o.length>qb?`${o.slice(0,qb)}...`:o;return se(mC,{type:"app",isSelected:e,metadata:dt(io,{children:[se(Fi,{icon:rM}),se(Fn,{margin:"none",color:"white",children:se(bc,{children:i.memory})}),dt(Fn,{margin:"none",color:"white",children:[se(bc,{children:i.cpu})," CPU"]})]}),title:r((n=i.name)!=null?n:i.filename),onClick:()=>t(i)})},lF=({dataframe:i,isSelected:t,onClick:e})=>se(mC,{isSelected:t,type:"data",title:i.key,metadata:dt(io,{children:[se(Fi,{icon:oM}),dt(Fn,{color:"white",margin:"none",children:[se(bc,{children:i.numberOfProperties})," properties"]}),dt(Fn,{color:"white",margin:"none",children:[se(bc,{children:i.numberOfRows})," rows"]})]}),onClick:()=>{e(i)}}),QT=Et("button",{cursor:"pointer",color:"$primaryDark",background:"none",border:"none",display:"flex",margin:0,padding:"$1",fontSize:"$3",fontWeight:"900",borderRadius:"4px","&:hover":{backgroundColor:"#D9D9D9"},"&:active":{backgroundColor:"$gray5"}}),Zb=Et("div"),uF=({projectUrl:i,app:t,onClose:e})=>dt(io,{children:[dt(Zy,{css:{justifyContent:"space-between"},children:[se(Zb,{css:{display:"flex",alignItems:"center",gap:"$1"},children:se(oc,{children:t.name})}),se(Zb,{css:{display:"flex",gap:"$1",alignItems:"flex-start"},children:se(QT,{onClick:e,children:se(Fi,{icon:vT})})})]}),se("iframe",{src:`${i}${t.route}`,title:t.name})]}),Qb=Et("div",{borderRadius:"4px",padding:"$1 $2",width:"fit-content",border:"2px solid transparent",variants:{intent:{info:{background:"$gray2",color:"$white"},danger:{background:`
            $transparentRedError padding-box,
            $redGradient border-box
          `,color:"#8d282e"},success:{background:"$cardCircleGreen",color:"$white"}}},defaultVariants:{intent:"info"}});var yC={},CC={};/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function Jo(i){return i==null||i===""?null:i}function q(i,t){return t===void 0&&(t=!1),i!=null&&(i!==""||t)}function me(i){return!q(i)}function pn(i){return i==null||i.length===0}function wC(i){return i!=null&&typeof i.toString=="function"?i.toString():null}function In(i){if(i!==void 0){if(i===null||i==="")return null;if(typeof i=="number")return isNaN(i)?void 0:i;var t=parseInt(i,10);return isNaN(t)?void 0:t}}function $i(i){if(i!==void 0)return i===null||i===""?!1:typeof i=="boolean"?i:/true/i.test(i)}function cF(i){if(!(i==null||i===""))return i}function pF(i,t){return i==null&&t==null?!0:i==null&&t!=null||i!=null&&t==null?!1:i===t}function hF(i,t){var e=i?JSON.stringify(i):null,r=t?JSON.stringify(t):null;return e===r}function Gm(i,t,e){e===void 0&&(e=!1);var r=i==null,n=t==null;if(i&&i.toNumber&&(i=i.toNumber()),t&&t.toNumber&&(t=t.toNumber()),r&&n)return 0;if(r)return-1;if(n)return 1;function o(a,s){return a>s?1:a<s?-1:0}if(typeof i!="string"||!e)return o(i,t);try{return i.localeCompare(t)}catch{return o(i,t)}}function ms(i){if(i instanceof Set||i instanceof Map){var t=[];return i.forEach(function(e){return t.push(e)}),t}return Object.values(i)}var dF=Object.freeze(Object.defineProperty({__proto__:null,makeNull:Jo,exists:q,missing:me,missingOrEmpty:pn,toStringOrNull:wC,attrToNumber:In,attrToBoolean:$i,attrToString:cF,referenceCompare:pF,jsonEquals:hF,defaultComparator:Gm,values:ms},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var JT=function(){function i(){this.existingKeys={}}return i.prototype.addExistingKeys=function(t){for(var e=0;e<t.length;e++)this.existingKeys[t[e]]=!0},i.prototype.getUniqueKey=function(t,e){t=wC(t);for(var r=0;;){var n=void 0;if(t?(n=t,r!==0&&(n+="_"+r)):e?(n=e,r!==0&&(n+="_"+r)):n=""+r,!this.existingKeys[n])return this.existingKeys[n]=!0,n;r++}},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function Ke(i,t){i!=null&&(Array.isArray(i)?i.forEach(function(e,r){return t(""+r,e)}):Object.keys(i).forEach(function(e){return t(e,i[e])}))}function bC(i){for(var t={},e=Object.keys(i),r=0;r<e.length;r++){var n=e[r],o=i[n];t[n]=o}return t}function fF(i){return JSON.parse(JSON.stringify(i))}function jh(i,t){if(!!i){var e=i,r={};return Object.keys(e).forEach(function(n){if(!(t&&t.indexOf(n)>=0)){var o=e[n],a=Rl(o)&&o.constructor===Object;a?r[n]=jh(o):r[n]=o}}),r}}function eO(i,t){return i[t]}function tO(i,t,e){i[t]=e}function gF(i,t){for(var e=[],r=2;r<arguments.length;r++)e[r-2]=arguments[r];e.forEach(function(n){return rO(i,t,n)})}function rO(i,t,e,r){var n=eO(i,e);n!==void 0&&tO(t,e,r?r(n):n)}function nO(i){var t={};return i.filter(function(e){return e!=null}).forEach(function(e){Object.keys(e).forEach(function(r){return t[r]=null})}),Object.keys(t)}function Jn(i){if(!i)return[];var t=Object;if(typeof t.values=="function")return t.values(i);var e=[];for(var r in i)i.hasOwnProperty(r)&&i.propertyIsEnumerable(r)&&e.push(i[r]);return e}function Ln(i,t,e,r){e===void 0&&(e=!0),r===void 0&&(r=!1),q(t)&&Ke(t,function(n,o){var a=i[n];if(a!==o){if(r){var s=a==null&&o!=null;if(s){var l=typeof o=="object"&&o.constructor===Object,u=l;u&&(a={},i[n]=a)}}Rl(o)&&Rl(a)&&!Array.isArray(a)?Ln(a,o,e,r):(e||o!==void 0)&&(i[n]=o)}})}function vF(i){return me(i)||Object.keys(i).length===0}function mF(i,t,e){if(i==null)return e;for(var r=t.split("."),n=i;r.length>1;)if(n=n[r.shift()],n==null)return e;var o=n[r[0]];return o!=null?o:e}function yF(i,t,e){if(i!=null){for(var r=t.split("."),n=i;r.length>1;)if(n=n[r.shift()],n==null)return;n[r[0]]=e}}function iO(i){return Object.freeze(i),ms(i).forEach(function(t){(Rl(t)||typeof t=="function")&&iO(t)}),i}function Uh(i,t,e){if(!(!t||!i)){if(!e)return i[t];for(var r=t.split("."),n=i,o=0;o<r.length;o++){if(n==null)return;n=n[r[o]]}return n}}function oO(i,t){Object.keys(i).forEach(function(n){var o=i[n];typeof o=="object"&&(i[n]=void 0)});var e=Object.getPrototypeOf(i),r={};Object.keys(e).forEach(function(n){var o=e[n];if(typeof o=="function"){var a=function(){console.warn("AG Grid: "+t+" function "+n+`() cannot be called as the grid has been destroyed.
                     Please don't call grid API functions on destroyed grids - as a matter of fact you shouldn't
                     be keeping the API reference, your application has a memory leak! Remove the API reference
                     when the grid is destroyed.`)};r[n]={value:a,writable:!0}}}),Object.defineProperties(i,r)}function Rl(i){return typeof i=="object"&&i!==null}var CF=Object.freeze(Object.defineProperty({__proto__:null,iterateObject:Ke,cloneObject:bC,deepCloneObject:fF,deepCloneDefinition:jh,getProperty:eO,setProperty:tO,copyPropertiesIfPresent:gF,copyPropertyIfPresent:rO,getAllKeysInObjects:nO,getAllValuesInObject:Jn,mergeDeep:Ln,missingOrEmptyObject:vF,get:mF,set:yF,deepFreeze:iO,getValueUsingField:Uh,removeAllReferences:oO,isNonNullObject:Rl},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var wF=/((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg,bF=/([^\s,]+)/g,Jb={};function Ye(i,t){Jb[t]||(i(),Jb[t]=!0)}function Qd(i){if(i.name)return i.name;var t=/function\s+([^\(]+)/.exec(i.toString());return t&&t.length===2?t[1].trim():null}function _F(i){var t=i.toString().replace(wF,"");return t.slice(t.indexOf("(")+1,t.indexOf(")")).match(bF)||[]}function _C(i){return!!(i&&i.constructor&&i.call&&i.apply)}function aO(i){sO(i,400)}var sg=[],lg=!1;function km(i){sg.push(i),!lg&&(lg=!0,window.setTimeout(function(){var t=sg.slice();sg.length=0,lg=!1,t.forEach(function(e){return e()})},0))}function sO(i,t){t===void 0&&(t=0),i.length>0&&window.setTimeout(function(){return i.forEach(function(e){return e()})},t)}function uo(i,t,e){e===void 0&&(e=!1);var r;return function(){for(var n=[],o=0;o<arguments.length;o++)n[o]=arguments[o];var a=this,s=e&&!r;window.clearTimeout(r),r=window.setTimeout(function(){r=null,e||i.apply(a,n)},t),s&&i.apply(a,n)}}function Wm(i,t){var e=0;return function(){for(var r=[],n=0;n<arguments.length;n++)r[n]=arguments[n];var o=this,a=new Date().getTime();a-e<t||(e=a,i.apply(o,r))}}function lO(i,t,e,r){e===void 0&&(e=100);var n=new Date().getTime(),o=null,a=!1,s=function(){var l=new Date().getTime()-n>e;(i()||l)&&(t(),a=!0,o!=null&&(window.clearInterval(o),o=null),l&&r&&console.warn(r))};s(),a||(o=window.setInterval(s,10))}function SF(){for(var i=[],t=0;t<arguments.length;t++)i[t]=arguments[t];return function(e){return i.reduce(function(r,n){return n(r)},e)}}function uO(i){i&&i()}var xF=Object.freeze(Object.defineProperty({__proto__:null,doOnce:Ye,getFunctionName:Qd,getFunctionParameters:_F,isFunction:_C,executeInAWhile:aO,executeNextVMTurn:km,executeAfter:sO,debounce:uo,throttle:Wm,waitUntil:lO,compose:SF,callIfPresent:uO},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var cO=function(){function i(t,e){if(this.beanWrappers={},this.destroyed=!1,!(!t||!t.beanClasses)){this.contextParams=t,this.logger=e,this.logger.log(">> creating ag-Application Context"),this.createBeans();var r=this.getBeanInstances();this.wireBeans(r),this.logger.log(">> ag-Application Context ready - component is alive")}}return i.prototype.getBeanInstances=function(){return ms(this.beanWrappers).map(function(t){return t.beanInstance})},i.prototype.createBean=function(t,e){if(!t)throw Error("Can't wire to bean since it is null");return this.wireBeans([t],e),t},i.prototype.wireBeans=function(t,e){this.autoWireBeans(t),this.methodWireBeans(t),this.callLifeCycleMethods(t,"preConstructMethods"),q(e)&&t.forEach(e),this.callLifeCycleMethods(t,"postConstructMethods")},i.prototype.createBeans=function(){var t=this;this.contextParams.beanClasses.forEach(this.createBeanWrapper.bind(this)),Ke(this.beanWrappers,function(r,n){var o;n.bean.__agBeanMetaData&&n.bean.__agBeanMetaData.autowireMethods&&n.bean.__agBeanMetaData.autowireMethods.agConstructor&&(o=n.bean.__agBeanMetaData.autowireMethods.agConstructor);var a=t.getBeansForParameters(o,n.bean.name),s=TF(n.bean,a);n.beanInstance=s});var e=Object.keys(this.beanWrappers).join(", ");this.logger.log("created beans: "+e)},i.prototype.createBeanWrapper=function(t){var e=t.__agBeanMetaData;if(!e){var r=void 0;t.prototype.constructor?r=Qd(t.prototype.constructor):r=""+t,console.error("Context item "+r+" is not a bean");return}var n={bean:t,beanInstance:null,beanName:e.beanName};this.beanWrappers[e.beanName]=n},i.prototype.autoWireBeans=function(t){var e=this;t.forEach(function(r){e.forEachMetaDataInHierarchy(r,function(n,o){var a=n.agClassAttributes;!a||a.forEach(function(s){var l=e.lookupBeanInstance(o,s.beanName,s.optional);r[s.attributeName]=l})})})},i.prototype.methodWireBeans=function(t){var e=this;t.forEach(function(r){e.forEachMetaDataInHierarchy(r,function(n,o){Ke(n.autowireMethods,function(a,s){if(a!=="agConstructor"){var l=e.getBeansForParameters(s,o);r[a].apply(r,l)}})})})},i.prototype.forEachMetaDataInHierarchy=function(t,e){for(var r=Object.getPrototypeOf(t);r!=null;){var n=r.constructor;if(n.hasOwnProperty("__agBeanMetaData")){var o=n.__agBeanMetaData,a=this.getBeanName(n);e(o,a)}r=Object.getPrototypeOf(r)}},i.prototype.getBeanName=function(t){if(t.__agBeanMetaData&&t.__agBeanMetaData.beanName)return t.__agBeanMetaData.beanName;var e=t.toString(),r=e.substring(9,e.indexOf("("));return r},i.prototype.getBeansForParameters=function(t,e){var r=this,n=[];return t&&Ke(t,function(o,a){var s=r.lookupBeanInstance(e,a);n[Number(o)]=s}),n},i.prototype.lookupBeanInstance=function(t,e,r){if(r===void 0&&(r=!1),e==="context")return this;if(this.contextParams.providedBeanInstances&&this.contextParams.providedBeanInstances.hasOwnProperty(e))return this.contextParams.providedBeanInstances[e];var n=this.beanWrappers[e];return n?n.beanInstance:(r||console.error("AG Grid: unable to find bean reference "+e+" while initialising "+t),null)},i.prototype.callLifeCycleMethods=function(t,e){var r=this;t.forEach(function(n){return r.callLifeCycleMethodsOnBean(n,e)})},i.prototype.callLifeCycleMethodsOnBean=function(t,e,r){var n={};this.forEachMetaDataInHierarchy(t,function(a){var s=a[e];s&&s.forEach(function(l){l!=r&&(n[l]=!0)})});var o=Object.keys(n);o.forEach(function(a){return t[a]()})},i.prototype.getBean=function(t){return this.lookupBeanInstance("getBean",t,!0)},i.prototype.destroy=function(){if(!this.destroyed){this.logger.log(">> Shutting down ag-Application Context");var t=this.getBeanInstances();this.destroyBeans(t),this.contextParams.providedBeanInstances=null,this.destroyed=!0,this.logger.log(">> ag-Application Context shut down - component is dead")}},i.prototype.destroyBean=function(t){!t||this.destroyBeans([t])},i.prototype.destroyBeans=function(t){var e=this;return t?(t.forEach(function(r){e.callLifeCycleMethodsOnBean(r,"preDestroyMethods","destroy");var n=r;typeof n.destroy=="function"&&n.destroy()}),[]):[]},i}();function TF(i,t){var e=[null].concat(t),r=i.bind.apply(i,e);return new r}function ss(i,t,e){var r=ls(i.constructor);r.preConstructMethods||(r.preConstructMethods=[]),r.preConstructMethods.push(t)}function B(i,t,e){var r=ls(i.constructor);r.postConstructMethods||(r.postConstructMethods=[]),r.postConstructMethods.push(t)}function nt(i,t,e){var r=ls(i.constructor);r.preDestroyMethods||(r.preDestroyMethods=[]),r.preDestroyMethods.push(t)}function Q(i){return function(t){var e=ls(t);e.beanName=i}}function y(i){return function(t,e,r){pO(t,i,!1,t,e,null)}}function De(i){return function(t,e,r){pO(t,i,!0,t,e,null)}}function pO(i,t,e,r,n,o){if(t===null){console.error("AG Grid: Autowired name should not be null");return}if(typeof o=="number"){console.error("AG Grid: Autowired should be on an attribute");return}var a=ls(i.constructor);a.agClassAttributes||(a.agClassAttributes=[]),a.agClassAttributes.push({attributeName:n,beanName:t,optional:e})}function Yt(i){return function(t,e,r){var n=typeof t=="function"?t:t.constructor,o;if(typeof r=="number"){var a=void 0;e?(o=ls(n),a=e):(o=ls(n),a="agConstructor"),o.autowireMethods||(o.autowireMethods={}),o.autowireMethods[a]||(o.autowireMethods[a]={}),o.autowireMethods[a][r]=i}}}function ls(i){return i.hasOwnProperty("__agBeanMetaData")||(i.__agBeanMetaData={}),i.__agBeanMetaData}/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var e_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},hp=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},oi=function(){function i(){this.allSyncListeners=new Map,this.allAsyncListeners=new Map,this.globalSyncListeners=new Set,this.globalAsyncListeners=new Set,this.asyncFunctionsQueue=[],this.scheduled=!1,this.firedEvents={}}return i.prototype.setBeans=function(t,e,r,n){if(n===void 0&&(n=null),this.frameworkOverrides=r,n){var o=e.useAsyncEvents();this.addGlobalListener(n,o)}},i.prototype.getListeners=function(t,e,r){var n=e?this.allAsyncListeners:this.allSyncListeners,o=n.get(t);return!o&&r&&(o=new Set,n.set(t,o)),o},i.prototype.noRegisteredListenersExist=function(){return this.allSyncListeners.size===0&&this.allAsyncListeners.size===0&&this.globalSyncListeners.size===0&&this.globalAsyncListeners.size===0},i.prototype.addEventListener=function(t,e,r){r===void 0&&(r=!1),this.getListeners(t,r,!0).add(e)},i.prototype.removeEventListener=function(t,e,r){r===void 0&&(r=!1);var n=this.getListeners(t,r,!1);if(!!n&&(n.delete(e),n.size===0)){var o=r?this.allAsyncListeners:this.allSyncListeners;o.delete(t)}},i.prototype.addGlobalListener=function(t,e){e===void 0&&(e=!1),(e?this.globalAsyncListeners:this.globalSyncListeners).add(t)},i.prototype.removeGlobalListener=function(t,e){e===void 0&&(e=!1),(e?this.globalAsyncListeners:this.globalSyncListeners).delete(t)},i.prototype.dispatchEvent=function(t){this.dispatchToListeners(t,!0),this.dispatchToListeners(t,!1),this.firedEvents[t.type]=!0},i.prototype.dispatchEventOnce=function(t){this.firedEvents[t.type]||this.dispatchEvent(t)},i.prototype.dispatchToListeners=function(t,e){var r=this,n=t.type,o=function(l){return l.forEach(function(u){e?r.dispatchAsync(function(){return u(t)}):u(t)})},a=this.getListeners(n,e,!1);a&&o(a);var s=e?this.globalAsyncListeners:this.globalSyncListeners;s.forEach(function(l){e?r.dispatchAsync(function(){return r.frameworkOverrides.dispatchEvent(n,function(){return l(n,t)},!0)}):r.frameworkOverrides.dispatchEvent(n,function(){return l(n,t)},!0)})},i.prototype.dispatchAsync=function(t){this.asyncFunctionsQueue.push(t),this.scheduled||(window.setTimeout(this.flushAsyncQueue.bind(this),0),this.scheduled=!0)},i.prototype.flushAsyncQueue=function(){this.scheduled=!1;var t=this.asyncFunctionsQueue.slice();this.asyncFunctionsQueue=[],t.forEach(function(e){return e()})},e_([hp(0,Yt("loggerFactory")),hp(1,Yt("gridOptionsWrapper")),hp(2,Yt("frameworkOverrides")),hp(3,Yt("globalEventListener"))],i.prototype,"setBeans",null),i=e_([Q("eventService")],i),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var L=function(){function i(){}return i.ROW_BUFFER_SIZE=10,i.LAYOUT_INTERVAL=500,i.BATCH_WAIT_MILLIS=50,i.EXPORT_TYPE_DRAG_COPY="dragCopy",i.EXPORT_TYPE_CLIPBOARD="clipboard",i.EXPORT_TYPE_EXCEL="excel",i.EXPORT_TYPE_CSV="csv",i.ROW_MODEL_TYPE_INFINITE="infinite",i.ROW_MODEL_TYPE_VIEWPORT="viewport",i.ROW_MODEL_TYPE_CLIENT_SIDE="clientSide",i.ROW_MODEL_TYPE_SERVER_SIDE="serverSide",i.ALWAYS="always",i.ONLY_WHEN_GROUPING="onlyWhenGrouping",i.PINNED_TOP="top",i.PINNED_BOTTOM="bottom",i.DOM_LAYOUT_NORMAL="normal",i.DOM_LAYOUT_PRINT="print",i.DOM_LAYOUT_AUTO_HEIGHT="autoHeight",i.GROUP_AUTO_COLUMN_ID="ag-Grid-AutoColumn",i.SOURCE_PASTE="paste",i.PINNED_RIGHT="right",i.PINNED_LEFT="left",i.SORT_ASC="asc",i.SORT_DESC="desc",i.INPUT_SELECTOR="input, select, button, textarea",i.FOCUSABLE_SELECTOR="[tabindex], input, select, button, textarea",i.FOCUSABLE_EXCLUDE=".ag-hidden, .ag-hidden *, [disabled], .ag-disabled, .ag-disabled *",i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var oe;(function(i){i.CommunityCoreModule="@ag-grid-community/core",i.CommunityAllModules="@ag-grid-community/all",i.InfiniteRowModelModule="@ag-grid-community/infinite-row-model",i.ClientSideRowModelModule="@ag-grid-community/client-side-row-model",i.CsvExportModule="@ag-grid-community/csv-export",i.EnterpriseCoreModule="@ag-grid-enterprise/core",i.EnterpriseAllModules="@ag-grid-enterprise/all",i.RowGroupingModule="@ag-grid-enterprise/row-grouping",i.ColumnToolPanelModule="@ag-grid-enterprise/column-tool-panel",i.FiltersToolPanelModule="@ag-grid-enterprise/filter-tool-panel",i.MenuModule="@ag-grid-enterprise/menu",i.SetFilterModule="@ag-grid-enterprise/set-filter",i.MultiFilterModule="@ag-grid-enterprise/multi-filter",i.StatusBarModule="@ag-grid-enterprise/status-bar",i.SideBarModule="@ag-grid-enterprise/side-bar",i.RangeSelectionModule="@ag-grid-enterprise/range-selection",i.MasterDetailModule="@ag-grid-enterprise/master-detail",i.RichSelectModule="@ag-grid-enterprise/rich-select",i.GridChartsModule="@ag-grid-enterprise/charts",i.ViewportRowModelModule="@ag-grid-enterprise/viewport-row-model",i.ServerSideRowModelModule="@ag-grid-enterprise/server-side-row-model",i.ExcelExportModule="@ag-grid-enterprise/excel-export",i.ClipboardModule="@ag-grid-enterprise/clipboard",i.SparklinesModule="@ag-grid-enterprise/sparklines",i.AngularModule="@ag-grid-community/angular",i.ReactModule="@ag-grid-community/react",i.VueModule="@ag-grid-community/vue",i.PolymerModule="@ag-grid-community/polymer"})(oe||(oe={}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ge=function(){function i(){}return i.register=function(t,e){e===void 0&&(e=!0),i.modulesMap[t.moduleName]=t,i.moduleBased===void 0?i.moduleBased=e:i.moduleBased!==e&&Ye(function(){console.warn("AG Grid: You are mixing modules (i.e. @ag-grid-community/core) and packages (ag-grid-community) - you can only use one or the other of these mechanisms."),console.warn("Please see https://www.ag-grid.com/javascript-grid/packages-modules/ for more information.")},"ModulePackageCheck")},i.registerModules=function(t,e){e===void 0&&(e=!0),t&&t.forEach(function(r){return i.register(r,e)})},i.assertRegistered=function(t,e){if(this.isRegistered(t))return!0;var r=e+t,n;return i.moduleBased?n="AG Grid: unable to use "+e+" as module "+t+" is not present. Please see: https://www.ag-grid.com/javascript-grid/modules/":n="AG Grid: unable to use "+e+" as package 'ag-grid-enterprise' is not present. Please see: https://www.ag-grid.com/javascript-grid/packages/",Ye(function(){console.warn(n)},r),!1},i.isRegistered=function(t){return!!i.modulesMap[t]},i.getRegisteredModules=function(){return ms(i.modulesMap)},i.isPackageBased=function(){return!i.moduleBased},i.modulesMap={},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var As=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},OF=0,Se=function(){function i(t,e,r,n){this.instanceId=OF++,this.moving=!1,this.menuVisible=!1,this.filterActive=!1,this.eventService=new oi,this.rowGroupActive=!1,this.pivotActive=!1,this.aggregationActive=!1,this.colDef=t,this.userProvidedColDef=e,this.colId=r,this.primary=n,this.setState(t)}return i.prototype.getInstanceId=function(){return this.instanceId},i.prototype.setState=function(t){t.sort!==void 0?(t.sort===L.SORT_ASC||t.sort===L.SORT_DESC)&&(this.sort=t.sort):(t.initialSort===L.SORT_ASC||t.initialSort===L.SORT_DESC)&&(this.sort=t.initialSort);var e=In(t.sortIndex),r=In(t.initialSortIndex);e!==void 0?e!==null&&(this.sortIndex=e):r!==null&&(this.sortIndex=r);var n=$i(t.hide),o=$i(t.initialHide);n!==void 0?this.visible=!n:this.visible=!o,t.pinned!==void 0?this.setPinned(t.pinned):this.setPinned(t.initialPinned);var a=In(t.flex),s=In(t.initialFlex);a!==void 0?this.flex=a:s!==void 0&&(this.flex=s)},i.prototype.setColDef=function(t,e){this.colDef=t,this.userProvidedColDef=e,this.initMinAndMaxWidths(),this.initDotNotation()},i.prototype.getUserProvidedColDef=function(){return this.userProvidedColDef},i.prototype.setParent=function(t){this.parent=t},i.prototype.getParent=function(){return this.parent},i.prototype.setOriginalParent=function(t){this.originalParent=t},i.prototype.getOriginalParent=function(){return this.originalParent},i.prototype.initialise=function(){this.initMinAndMaxWidths(),this.resetActualWidth("gridInitializing"),this.initDotNotation(),this.validate()},i.prototype.initDotNotation=function(){var t=this.gridOptionsWrapper.isSuppressFieldDotNotation();this.fieldContainsDots=q(this.colDef.field)&&this.colDef.field.indexOf(".")>=0&&!t,this.tooltipFieldContainsDots=q(this.colDef.tooltipField)&&this.colDef.tooltipField.indexOf(".")>=0&&!t},i.prototype.initMinAndMaxWidths=function(){var t=this.colDef;this.minWidth=this.columnUtils.calculateColMinWidth(t),this.maxWidth=this.columnUtils.calculateColMaxWidth(t)},i.prototype.resetActualWidth=function(t){t===void 0&&(t="api");var e=this.columnUtils.calculateColInitialWidth(this.colDef);this.setActualWidth(e,t,!0)},i.prototype.isEmptyGroup=function(){return!1},i.prototype.isRowGroupDisplayed=function(t){if(me(this.colDef)||me(this.colDef.showRowGroup))return!1;var e=this.colDef.showRowGroup===!0,r=this.colDef.showRowGroup===t;return e||r},i.prototype.isPrimary=function(){return this.primary},i.prototype.isFilterAllowed=function(){var t=!!this.colDef.filter||!!this.colDef.filterFramework;return t},i.prototype.isFieldContainsDots=function(){return this.fieldContainsDots},i.prototype.isTooltipFieldContainsDots=function(){return this.tooltipFieldContainsDots},i.prototype.validate=function(){var t=this.colDef;function e(a,s,l){Ye(function(){l?console.warn(a,l):Ye(function(){return console.warn(a)},s)},s)}var r=this.gridOptionsWrapper.isRowModelDefault();if(r&&!ge.isRegistered(oe.RowGroupingModule)){var n=["enableRowGroup","rowGroup","rowGroupIndex","enablePivot","enableValue","pivot","pivotIndex","aggFunc"];n.forEach(function(a){q(t[a])&&(ge.isPackageBased()?e("AG Grid: "+a+" is only valid in ag-grid-enterprise, your column definition should not have "+a,"ColumnRowGroupingMissing"+a):e("AG Grid: "+a+" is only valid with AG Grid Enterprise Module "+oe.RowGroupingModule+" - your column definition should not have "+a,"ColumnRowGroupingMissing"+a))})}if(ge.isRegistered(oe.RichSelectModule)||this.colDef.cellEditor==="agRichSelect"&&(ge.isPackageBased()?e("AG Grid: "+this.colDef.cellEditor+" can only be used with ag-grid-enterprise","ColumnRichSelectMissing"):e("AG Grid: "+this.colDef.cellEditor+" can only be used with AG Grid Enterprise Module "+oe.RichSelectModule,"ColumnRichSelectMissing")),this.gridOptionsWrapper.isTreeData()){var o=["rowGroup","rowGroupIndex","pivot","pivotIndex"];o.forEach(function(a){q(t[a])&&e("AG Grid: "+a+" is not possible when doing tree data, your column definition should not have "+a,"TreeDataCannotRowGroup")})}q(this.colDef.width)&&typeof this.colDef.width!="number"&&e("AG Grid: colDef.width should be a number, not "+typeof this.colDef.width,"ColumnCheck_asdfawef"),t.pinnedRowCellRenderer&&e("AG Grid: pinnedRowCellRenderer no longer exists, use cellRendererSelector if you want a different Cell Renderer for pinned rows. Check params.node.rowPinned. This was an unfortunate (but necessary) change we had to do to allow future plans we have of re-skinng the data grid in frameworks such as React, Angular and Vue. See https://www.ag-grid.com/javascript-grid/cell-rendering/#many-renderers-one-column","colDef.pinnedRowCellRenderer-deprecated"),t.pinnedRowCellRendererParams&&e("AG Grid: pinnedRowCellRenderer no longer exists, use cellRendererSelector if you want a different Cell Renderer for pinned rows. Check params.node.rowPinned. This was an unfortunate (but necessary) change we had to do to allow future plans we have of re-skinng the data grid in frameworks such as React, Angular and Vue. See https://www.ag-grid.com/javascript-grid/cell-rendering/#many-renderers-one-column","colDef.pinnedRowCellRenderer-deprecated"),t.pinnedRowCellRendererFramework&&e("AG Grid: pinnedRowCellRenderer no longer exists, use cellRendererSelector if you want a different Cell Renderer for pinned rows. Check params.node.rowPinned. This was an unfortunate (but necessary) change we had to do to allow future plans we have of re-skinng the data grid in frameworks such as React, Angular and Vue. See https://www.ag-grid.com/javascript-grid/cell-rendering/#many-renderers-one-column","colDef.pinnedRowCellRenderer-deprecated"),t.pinnedRowValueGetter&&e("AG Grid: pinnedRowCellRenderer is deprecated, use cellRendererSelector if you want a different Cell Renderer for pinned rows. Check params.node.rowPinned. This was an unfortunate (but necessary) change we had to do to allow future plans we have of re-skinng the data grid in frameworks such as React, Angular and Vue.","colDef.pinnedRowCellRenderer-deprecated")},i.prototype.addEventListener=function(t,e){this.eventService.addEventListener(t,e)},i.prototype.removeEventListener=function(t,e){this.eventService.removeEventListener(t,e)},i.prototype.createColumnFunctionCallbackParams=function(t){return{node:t,data:t.data,column:this,colDef:this.colDef,context:this.gridOptionsWrapper.getContext(),api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi()}},i.prototype.isSuppressNavigable=function(t){if(typeof this.colDef.suppressNavigable=="boolean")return this.colDef.suppressNavigable;if(typeof this.colDef.suppressNavigable=="function"){var e=this.createColumnFunctionCallbackParams(t),r=this.colDef.suppressNavigable;return r(e)}return!1},i.prototype.isCellEditable=function(t){return t.group&&!this.gridOptionsWrapper.isEnableGroupEdit()?!1:this.isColumnFunc(t,this.colDef.editable)},i.prototype.isSuppressFillHandle=function(){return!!$i(this.colDef.suppressFillHandle)},i.prototype.isAutoHeight=function(){return!!$i(this.colDef.autoHeight)},i.prototype.isRowDrag=function(t){return this.isColumnFunc(t,this.colDef.rowDrag)},i.prototype.isDndSource=function(t){return this.isColumnFunc(t,this.colDef.dndSource)},i.prototype.isCellCheckboxSelection=function(t){return this.isColumnFunc(t,this.colDef.checkboxSelection)},i.prototype.isSuppressPaste=function(t){return this.isColumnFunc(t,this.colDef?this.colDef.suppressPaste:null)},i.prototype.isResizable=function(){return!!$i(this.colDef.resizable)},i.prototype.isColumnFunc=function(t,e){if(typeof e=="boolean")return e;if(typeof e=="function"){var r=this.createColumnFunctionCallbackParams(t),n=e;return n(r)}return!1},i.prototype.setMoving=function(t,e){e===void 0&&(e="api"),this.moving=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_MOVING_CHANGED,e))},i.prototype.createColumnEvent=function(t,e){return{api:this.gridApi,columnApi:this.columnApi,type:t,column:this,columns:[this],source:e}},i.prototype.isMoving=function(){return this.moving},i.prototype.getSort=function(){return this.sort},i.prototype.setSort=function(t,e){e===void 0&&(e="api"),this.sort!==t&&(this.sort=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_SORT_CHANGED,e)))},i.prototype.setMenuVisible=function(t,e){e===void 0&&(e="api"),this.menuVisible!==t&&(this.menuVisible=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_MENU_VISIBLE_CHANGED,e)))},i.prototype.isMenuVisible=function(){return this.menuVisible},i.prototype.isSortAscending=function(){return this.sort===L.SORT_ASC},i.prototype.isSortDescending=function(){return this.sort===L.SORT_DESC},i.prototype.isSortNone=function(){return me(this.sort)},i.prototype.isSorting=function(){return q(this.sort)},i.prototype.getSortIndex=function(){return this.sortIndex},i.prototype.setSortIndex=function(t){this.sortIndex=t},i.prototype.setAggFunc=function(t){this.aggFunc=t},i.prototype.getAggFunc=function(){return this.aggFunc},i.prototype.getLeft=function(){return this.left},i.prototype.getOldLeft=function(){return this.oldLeft},i.prototype.getRight=function(){return this.left+this.actualWidth},i.prototype.setLeft=function(t,e){e===void 0&&(e="api"),this.oldLeft=this.left,this.left!==t&&(this.left=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_LEFT_CHANGED,e)))},i.prototype.isFilterActive=function(){return this.filterActive},i.prototype.setFilterActive=function(t,e,r){e===void 0&&(e="api"),this.filterActive!==t&&(this.filterActive=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_FILTER_ACTIVE_CHANGED,e)));var n=this.createColumnEvent(i.EVENT_FILTER_CHANGED,e);r&&Ln(n,r),this.eventService.dispatchEvent(n)},i.prototype.setPinned=function(t){t===!0||t===L.PINNED_LEFT?this.pinned=L.PINNED_LEFT:t===L.PINNED_RIGHT?this.pinned=L.PINNED_RIGHT:this.pinned=null},i.prototype.setFirstRightPinned=function(t,e){e===void 0&&(e="api"),this.firstRightPinned!==t&&(this.firstRightPinned=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_FIRST_RIGHT_PINNED_CHANGED,e)))},i.prototype.setLastLeftPinned=function(t,e){e===void 0&&(e="api"),this.lastLeftPinned!==t&&(this.lastLeftPinned=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_LAST_LEFT_PINNED_CHANGED,e)))},i.prototype.isFirstRightPinned=function(){return this.firstRightPinned},i.prototype.isLastLeftPinned=function(){return this.lastLeftPinned},i.prototype.isPinned=function(){return this.pinned===L.PINNED_LEFT||this.pinned===L.PINNED_RIGHT},i.prototype.isPinnedLeft=function(){return this.pinned===L.PINNED_LEFT},i.prototype.isPinnedRight=function(){return this.pinned===L.PINNED_RIGHT},i.prototype.getPinned=function(){return this.pinned},i.prototype.setVisible=function(t,e){e===void 0&&(e="api");var r=t===!0;this.visible!==r&&(this.visible=r,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_VISIBLE_CHANGED,e)))},i.prototype.isVisible=function(){return this.visible},i.prototype.getColDef=function(){return this.colDef},i.prototype.getColumnGroupShow=function(){return this.colDef.columnGroupShow},i.prototype.getColId=function(){return this.colId},i.prototype.getId=function(){return this.getColId()},i.prototype.getUniqueId=function(){return this.getId()},i.prototype.getDefinition=function(){return this.colDef},i.prototype.getActualWidth=function(){return this.actualWidth},i.prototype.createBaseColDefParams=function(t){var e={node:t,data:t.data,colDef:this.colDef,column:this,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()};return e},i.prototype.getColSpan=function(t){if(me(this.colDef.colSpan))return 1;var e=this.createBaseColDefParams(t),r=this.colDef.colSpan(e);return Math.max(r,1)},i.prototype.getRowSpan=function(t){if(me(this.colDef.rowSpan))return 1;var e=this.createBaseColDefParams(t),r=this.colDef.rowSpan(e);return Math.max(r,1)},i.prototype.setActualWidth=function(t,e,r){e===void 0&&(e="api"),r===void 0&&(r=!1),this.minWidth!=null&&(t=Math.max(t,this.minWidth)),this.maxWidth!=null&&(t=Math.min(t,this.maxWidth)),this.actualWidth!==t&&(this.actualWidth=t,this.flex&&e!=="flex"&&e!=="gridInitializing"&&(this.flex=null),r||this.fireColumnWidthChangedEvent(e))},i.prototype.fireColumnWidthChangedEvent=function(t){this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_WIDTH_CHANGED,t))},i.prototype.isGreaterThanMax=function(t){return this.maxWidth!=null?t>this.maxWidth:!1},i.prototype.getMinWidth=function(){return this.minWidth},i.prototype.getMaxWidth=function(){return this.maxWidth},i.prototype.getFlex=function(){return this.flex||0},i.prototype.setFlex=function(t){this.flex!==t&&(this.flex=t)},i.prototype.setMinimum=function(t){t===void 0&&(t="api"),q(this.minWidth)&&this.setActualWidth(this.minWidth,t)},i.prototype.setRowGroupActive=function(t,e){e===void 0&&(e="api"),this.rowGroupActive!==t&&(this.rowGroupActive=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_ROW_GROUP_CHANGED,e)))},i.prototype.isRowGroupActive=function(){return this.rowGroupActive},i.prototype.setPivotActive=function(t,e){e===void 0&&(e="api"),this.pivotActive!==t&&(this.pivotActive=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_PIVOT_CHANGED,e)))},i.prototype.isPivotActive=function(){return this.pivotActive},i.prototype.isAnyFunctionActive=function(){return this.isPivotActive()||this.isRowGroupActive()||this.isValueActive()},i.prototype.isAnyFunctionAllowed=function(){return this.isAllowPivot()||this.isAllowRowGroup()||this.isAllowValue()},i.prototype.setValueActive=function(t,e){e===void 0&&(e="api"),this.aggregationActive!==t&&(this.aggregationActive=t,this.eventService.dispatchEvent(this.createColumnEvent(i.EVENT_VALUE_CHANGED,e)))},i.prototype.isValueActive=function(){return this.aggregationActive},i.prototype.isAllowPivot=function(){return this.colDef.enablePivot===!0},i.prototype.isAllowValue=function(){return this.colDef.enableValue===!0},i.prototype.isAllowRowGroup=function(){return this.colDef.enableRowGroup===!0},i.prototype.getMenuTabs=function(t){var e=this.getColDef().menuTabs;return e==null&&(e=t),e},i.prototype.isLockPosition=function(){return console.warn("AG Grid: since v21, col.isLockPosition() should not be used, please use col.getColDef().lockPosition instead."),this.colDef?!!this.colDef.lockPosition:!1},i.prototype.isLockVisible=function(){return console.warn("AG Grid: since v21, col.isLockVisible() should not be used, please use col.getColDef().lockVisible instead."),this.colDef?!!this.colDef.lockVisible:!1},i.prototype.isLockPinned=function(){return console.warn("AG Grid: since v21, col.isLockPinned() should not be used, please use col.getColDef().lockPinned instead."),this.colDef?!!this.colDef.lockPinned:!1},i.EVENT_MOVING_CHANGED="movingChanged",i.EVENT_LEFT_CHANGED="leftChanged",i.EVENT_WIDTH_CHANGED="widthChanged",i.EVENT_LAST_LEFT_PINNED_CHANGED="lastLeftPinnedChanged",i.EVENT_FIRST_RIGHT_PINNED_CHANGED="firstRightPinnedChanged",i.EVENT_VISIBLE_CHANGED="visibleChanged",i.EVENT_FILTER_CHANGED="filterChanged",i.EVENT_FILTER_ACTIVE_CHANGED="filterActiveChanged",i.EVENT_SORT_CHANGED="sortChanged",i.EVENT_MENU_VISIBLE_CHANGED="menuVisibleChanged",i.EVENT_ROW_GROUP_CHANGED="columnRowGroupChanged",i.EVENT_PIVOT_CHANGED="columnPivotChanged",i.EVENT_VALUE_CHANGED="columnValueChanged",As([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),As([y("columnUtils")],i.prototype,"columnUtils",void 0),As([y("columnApi")],i.prototype,"columnApi",void 0),As([y("gridApi")],i.prototype,"gridApi",void 0),As([y("context")],i.prototype,"context",void 0),As([B],i.prototype,"initialise",null),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function hO(){for(var i=[],t=0;t<arguments.length;t++)i[t]=arguments[t];for(var e=0;e<i.length;e++){var r=i[e];if(q(r))return r}return null}function EF(i){return i!=null&&i.length>0}function rt(i){if(!(!i||!i.length))return i[i.length-1]}function es(i,t,e){return i==null&&t==null?!0:i!=null&&t!=null&&i.length===t.length&&i.every(function(r,n){return e?e(r,t[n]):t[n]===r})}function RF(i,t){return es(i,t)}function PF(i,t){return es(i,t)}function dO(i){return i.sort(function(t,e){return t-e})}function AF(i,t){if(!!i)for(var e=i.length-2;e>=0;e--){var r=i[e]===t,n=i[e+1]===t;r&&n&&i.splice(e+1,1)}}function gr(i,t){var e=i.indexOf(t);e>=0&&i.splice(e,1)}function SC(i,t){t.forEach(function(e){return gr(i,e)})}function Sc(i,t,e){i.splice(e,0,t)}function DF(i,t,e){if(!(i==null||t==null))for(var r=t.length-1;r>=0;r--){var n=t[r];Sc(i,n,e)}}function Vm(i,t,e){SC(i,t),t.slice().reverse().forEach(function(r){return Sc(i,r,e)})}function fn(i,t){return i.indexOf(t)>-1}function MF(i){return[].concat.apply([],i)}function Bm(i,t){t==null||i==null||t.forEach(function(e){return i.push(e)})}function FF(i){return i.map(wC)}function IF(i,t){if(i!=null)for(var e=i.length-1;e>=0;e--)t(i[e],e)}var LF=Object.freeze(Object.defineProperty({__proto__:null,firstExistingValue:hO,existsAndNotEmpty:EF,last:rt,areEqual:es,compareArrays:RF,shallowCompare:PF,sortNumerically:dO,removeRepeatsFromArray:AF,removeFromArray:gr,removeAllFromArray:SC,insertIntoArray:Sc,insertArrayIntoArray:DF,moveInArray:Vm,includes:fn,flatten:MF,pushAll:Bm,toStrings:FF,forEachReverse:IF},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var NF=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},sr=function(){function i(t,e,r,n){this.displayedChildren=[],this.localEventService=new oi,this.groupId=e,this.instanceId=r,this.providedColumnGroup=t,this.pinned=n}return i.createUniqueId=function(t,e){return t+"_"+e},i.prototype.reset=function(){this.parent=null,this.children=null,this.displayedChildren=null},i.prototype.getParent=function(){return this.parent},i.prototype.setParent=function(t){this.parent=t},i.prototype.getUniqueId=function(){return i.createUniqueId(this.groupId,this.instanceId)},i.prototype.isEmptyGroup=function(){return this.displayedChildren.length===0},i.prototype.isMoving=function(){var t=this.getProvidedColumnGroup().getLeafColumns();return!t||t.length===0?!1:t.every(function(e){return e.isMoving()})},i.prototype.checkLeft=function(){if(this.displayedChildren.forEach(function(n){n instanceof i&&n.checkLeft()}),this.displayedChildren.length>0)if(this.gridOptionsWrapper.isEnableRtl()){var t=rt(this.displayedChildren),e=t.getLeft();this.setLeft(e)}else{var r=this.displayedChildren[0].getLeft();this.setLeft(r)}else this.setLeft(null)},i.prototype.getLeft=function(){return this.left},i.prototype.getOldLeft=function(){return this.oldLeft},i.prototype.setLeft=function(t){this.oldLeft=t,this.left!==t&&(this.left=t,this.localEventService.dispatchEvent(this.createAgEvent(i.EVENT_LEFT_CHANGED)))},i.prototype.getPinned=function(){return this.pinned},i.prototype.createAgEvent=function(t){return{type:t}},i.prototype.addEventListener=function(t,e){this.localEventService.addEventListener(t,e)},i.prototype.removeEventListener=function(t,e){this.localEventService.removeEventListener(t,e)},i.prototype.getGroupId=function(){return this.groupId},i.prototype.getInstanceId=function(){return this.instanceId},i.prototype.isChildInThisGroupDeepSearch=function(t){var e=!1;return this.children.forEach(function(r){t===r&&(e=!0),r instanceof i&&r.isChildInThisGroupDeepSearch(t)&&(e=!0)}),e},i.prototype.getActualWidth=function(){var t=0;return this.displayedChildren&&this.displayedChildren.forEach(function(e){t+=e.getActualWidth()}),t},i.prototype.isResizable=function(){if(!this.displayedChildren)return!1;var t=!1;return this.displayedChildren.forEach(function(e){e.isResizable()&&(t=!0)}),t},i.prototype.getMinWidth=function(){var t=0;return this.displayedChildren.forEach(function(e){t+=e.getMinWidth()||0}),t},i.prototype.addChild=function(t){this.children||(this.children=[]),this.children.push(t)},i.prototype.getDisplayedChildren=function(){return this.displayedChildren},i.prototype.getLeafColumns=function(){var t=[];return this.addLeafColumns(t),t},i.prototype.getDisplayedLeafColumns=function(){var t=[];return this.addDisplayedLeafColumns(t),t},i.prototype.getDefinition=function(){return this.providedColumnGroup.getColGroupDef()},i.prototype.getColGroupDef=function(){return this.providedColumnGroup.getColGroupDef()},i.prototype.isPadding=function(){return this.providedColumnGroup.isPadding()},i.prototype.isExpandable=function(){return this.providedColumnGroup.isExpandable()},i.prototype.isExpanded=function(){return this.providedColumnGroup.isExpanded()},i.prototype.setExpanded=function(t){this.providedColumnGroup.setExpanded(t)},i.prototype.addDisplayedLeafColumns=function(t){this.displayedChildren.forEach(function(e){e instanceof Se?t.push(e):e instanceof i&&e.addDisplayedLeafColumns(t)})},i.prototype.addLeafColumns=function(t){this.children.forEach(function(e){e instanceof Se?t.push(e):e instanceof i&&e.addLeafColumns(t)})},i.prototype.getChildren=function(){return this.children},i.prototype.getColumnGroupShow=function(){return this.providedColumnGroup.getColumnGroupShow()},i.prototype.getProvidedColumnGroup=function(){return this.providedColumnGroup},i.prototype.getOriginalColumnGroup=function(){return console.warn("AG Grid: columnGroup.getOriginalColumnGroup() is deprecated due to a method rename, use columnGroup.getProvidedColumnGroup() instead"),this.getProvidedColumnGroup()},i.prototype.getPaddingLevel=function(){var t=this.getParent();return!this.isPadding()||!t||!t.isPadding()?0:1+t.getPaddingLevel()},i.prototype.calculateDisplayedColumns=function(){var t=this;this.displayedChildren=[];for(var e=this;e!=null&&e.isPadding();)e=e.getParent();var r=e?e.providedColumnGroup.isExpandable():!1;if(!r){this.displayedChildren=this.children,this.localEventService.dispatchEvent(this.createAgEvent(i.EVENT_DISPLAYED_CHILDREN_CHANGED));return}this.children.forEach(function(n){var o=n instanceof i&&(!n.displayedChildren||!n.displayedChildren.length);if(!o){var a=n.getColumnGroupShow();switch(a){case i.HEADER_GROUP_SHOW_OPEN:e.providedColumnGroup.isExpanded()&&t.displayedChildren.push(n);break;case i.HEADER_GROUP_SHOW_CLOSED:e.providedColumnGroup.isExpanded()||t.displayedChildren.push(n);break;default:t.displayedChildren.push(n);break}}}),this.localEventService.dispatchEvent(this.createAgEvent(i.EVENT_DISPLAYED_CHILDREN_CHANGED))},i.HEADER_GROUP_SHOW_OPEN="open",i.HEADER_GROUP_SHOW_CLOSED="closed",i.EVENT_LEFT_CHANGED="leftChanged",i.EVENT_DISPLAYED_CHILDREN_CHANGED="displayedChildrenChanged",NF([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var $e=function(){function i(t,e,r,n){this.localEventService=new oi,this.expandable=!1,this.colGroupDef=t,this.groupId=e,this.expanded=!!t&&!!t.openByDefault,this.padding=r,this.level=n}return i.prototype.setOriginalParent=function(t){this.originalParent=t},i.prototype.getOriginalParent=function(){return this.originalParent},i.prototype.getLevel=function(){return this.level},i.prototype.isVisible=function(){return this.children?this.children.some(function(t){return t.isVisible()}):!1},i.prototype.isPadding=function(){return this.padding},i.prototype.setExpanded=function(t){this.expanded=t===void 0?!1:t;var e={type:i.EVENT_EXPANDED_CHANGED};this.localEventService.dispatchEvent(e)},i.prototype.isExpandable=function(){return this.expandable},i.prototype.isExpanded=function(){return this.expanded},i.prototype.getGroupId=function(){return this.groupId},i.prototype.getId=function(){return this.getGroupId()},i.prototype.setChildren=function(t){this.children=t},i.prototype.getChildren=function(){return this.children},i.prototype.getColGroupDef=function(){return this.colGroupDef},i.prototype.getLeafColumns=function(){var t=[];return this.addLeafColumns(t),t},i.prototype.addLeafColumns=function(t){!this.children||this.children.forEach(function(e){e instanceof Se?t.push(e):e instanceof i&&e.addLeafColumns(t)})},i.prototype.getColumnGroupShow=function(){var t=this.colGroupDef;if(!!t)return t.columnGroupShow},i.prototype.setupExpandable=function(){var t=this;this.setExpandable(),this.getLeafColumns().forEach(function(e){return e.addEventListener(Se.EVENT_VISIBLE_CHANGED,t.onColumnVisibilityChanged.bind(t))})},i.prototype.setExpandable=function(){if(!this.isPadding()){for(var t=!1,e=!1,r=!1,n=this.findChildrenRemovingPadding(),o=0,a=n.length;o<a;o++){var s=n[o];if(!!s.isVisible()){var l=s.getColumnGroupShow();l===sr.HEADER_GROUP_SHOW_OPEN?(t=!0,r=!0):l===sr.HEADER_GROUP_SHOW_CLOSED?(e=!0,r=!0):(t=!0,e=!0)}}var u=t&&e&&r;if(this.expandable!==u){this.expandable=u;var c={type:i.EVENT_EXPANDABLE_CHANGED};this.localEventService.dispatchEvent(c)}}},i.prototype.findChildrenRemovingPadding=function(){var t=[],e=function(r){r.forEach(function(n){var o=n instanceof i&&n.isPadding();o?e(n.children):t.push(n)})};return e(this.children),t},i.prototype.onColumnVisibilityChanged=function(){this.setExpandable()},i.prototype.addEventListener=function(t,e){this.localEventService.addEventListener(t,e)},i.prototype.removeEventListener=function(t,e){this.localEventService.removeEventListener(t,e)},i.EVENT_EXPANDED_CHANGED="expandedChanged",i.EVENT_EXPANDABLE_CHANGED="expandableChanged",i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var GF={numericColumn:{headerClass:"ag-right-aligned-header",cellClass:"ag-right-aligned-cell"},rightAligned:{headerClass:"ag-right-aligned-header",cellClass:"ag-right-aligned-cell"}};/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var fO="__ag_Grid_Stop_Propagation",kF=["touchstart","touchend","touchmove","touchcancel"],ug={};function Pl(i){i[fO]=!0}function Ai(i){return i[fO]===!0}var xC=function(){var i={select:"input",change:"input",submit:"form",reset:"form",error:"img",load:"img",abort:"img"},t=function(e){if(typeof ug[e]=="boolean")return ug[e];var r=document.createElement(i[e]||"div");return e="on"+e,ug[e]=e in r};return t}();function zh(i,t,e){for(var r=t.target;r;){var n=i.getDomData(r,e);if(n)return n;r=r.parentElement}return null}function WF(i,t){i.addEventListener("changed",t),i.addEventListener("paste",t),i.addEventListener("input",t)}function $h(i,t){return!t||!i?!1:TC(t).indexOf(i)>=0}function gO(i){for(var t=[],e=i.target;e;)t.push(e),e=e.parentElement;return t}function VF(i){i.__agGridEventPath=TC(i)}function TC(i){var t=i;return t.path?t.path:t.composedPath?t.composedPath():t.__agGridEventPath?t.__agGridEventPath:gO(i)}function vO(i,t,e,r){var n=fn(kF,e),o=n?{passive:!0}:void 0;i&&i.addEventListener&&i.addEventListener(t,e,r,o)}var BF=Object.freeze(Object.defineProperty({__proto__:null,stopPropagationForAgGrid:Pl,isStopPropagationForAgGrid:Ai,isEventSupported:xC,getCtrlForEvent:zh,addChangeListener:WF,isElementInEventPath:$h,createEventPath:gO,addAgGridEventPath:VF,getEventPath:TC,addSafePassiveEventListener:vO},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var nu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},H=function(){function i(){var t=this;this.destroyFunctions=[],this.destroyed=!1,this.__v_skip=!0,this.isAlive=function(){return!t.destroyed}}return i.prototype.getFrameworkOverrides=function(){return this.frameworkOverrides},i.prototype.getContext=function(){return this.context},i.prototype.destroy=function(){this.destroyFunctions.forEach(function(t){return t()}),this.destroyFunctions.length=0,this.destroyed=!0,this.dispatchEvent({type:i.EVENT_DESTROYED})},i.prototype.addEventListener=function(t,e){this.localEventService||(this.localEventService=new oi),this.localEventService.addEventListener(t,e)},i.prototype.removeEventListener=function(t,e){this.localEventService&&this.localEventService.removeEventListener(t,e)},i.prototype.dispatchEventAsync=function(t){var e=this;window.setTimeout(function(){return e.dispatchEvent(t)},0)},i.prototype.dispatchEvent=function(t){this.localEventService&&this.localEventService.dispatchEvent(t)},i.prototype.addManagedListener=function(t,e,r){var n=this;if(!this.destroyed){t instanceof HTMLElement?vO(this.getFrameworkOverrides(),t,e,r):t.addEventListener(e,r);var o=function(){return t.removeEventListener(e,r),n.destroyFunctions=n.destroyFunctions.filter(function(a){return a!==o}),null};return this.destroyFunctions.push(o),o}},i.prototype.addDestroyFunc=function(t){this.isAlive()?this.destroyFunctions.push(t):t()},i.prototype.createManagedBean=function(t,e){var r=this.createBean(t,e);return this.addDestroyFunc(this.destroyBean.bind(this,t,e)),r},i.prototype.createBean=function(t,e,r){return(e||this.getContext()).createBean(t,r)},i.prototype.destroyBean=function(t,e){return(e||this.getContext()).destroyBean(t)},i.prototype.destroyBeans=function(t,e){var r=this;return t&&t.forEach(function(n){return r.destroyBean(n,e)}),[]},i.EVENT_DESTROYED="destroyed",nu([y("frameworkOverrides")],i.prototype,"frameworkOverrides",void 0),nu([y("context")],i.prototype,"context",void 0),nu([y("eventService")],i.prototype,"eventService",void 0),nu([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),nu([nt],i.prototype,"destroy",null),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var HF=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),cg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},jF=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},mO=function(i){HF(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.setBeans=function(e){this.logger=e.create("ColumnFactory")},t.prototype.createColumnTree=function(e,r,n){var o=new JT,a=this.extractExistingTreeData(n),s=a.existingCols,l=a.existingGroups,u=a.existingColKeys;o.addExistingKeys(u);var c=this.recursivelyCreateColumns(e,0,r,s,o,l),p=this.findMaxDept(c,0);this.logger.log("Number of levels for grouped columns is "+p);var h=this.balanceColumnTree(c,0,p,o),d=function(f,g){f instanceof $e&&f.setupExpandable(),f.setOriginalParent(g)};return this.columnUtils.depthFirstOriginalTreeSearch(null,h,d),{columnTree:h,treeDept:p}},t.prototype.extractExistingTreeData=function(e){var r=[],n=[],o=[];return e&&this.columnUtils.depthFirstOriginalTreeSearch(null,e,function(a){if(a instanceof $e){var s=a;n.push(s)}else{var l=a;o.push(l.getId()),r.push(l)}}),{existingCols:r,existingGroups:n,existingColKeys:o}},t.prototype.createForAutoGroups=function(e,r){var n=this;return e.map(function(o){return n.createAutoGroupTreeItem(r,o)})},t.prototype.createAutoGroupTreeItem=function(e,r){for(var n=this.findDepth(e),o=r,a=n-1;a>=0;a--){var s=new $e(null,"FAKE_PATH_"+r.getId()+"}_"+a,!0,a);this.context.createBean(s),s.setChildren([o]),o.setOriginalParent(s),o=s}return o},t.prototype.findDepth=function(e){for(var r=0,n=e;n&&n[0]&&n[0]instanceof $e;)r++,n=n[0].getChildren();return r},t.prototype.balanceColumnTree=function(e,r,n,o){for(var a=[],s=0;s<e.length;s++){var l=e[s];if(l instanceof $e){var u=l,c=this.balanceColumnTree(u.getChildren(),r+1,n,o);u.setChildren(c),a.push(u)}else{for(var p=void 0,h=void 0,d=n-1;d>=r;d--){var f=o.getUniqueKey(null,null),g=this.createMergedColGroupDef(null),v=new $e(g,f,!0,r);this.context.createBean(v),h&&h.setChildren([v]),h=v,p||(p=h)}if(p&&h){a.push(p);var m=e.some(function(C){return C instanceof $e});if(m){h.setChildren([l]);continue}else{h.setChildren(e);break}}a.push(l)}}return a},t.prototype.findMaxDept=function(e,r){for(var n=r,o=0;o<e.length;o++){var a=e[o];if(a instanceof $e){var s=a,l=this.findMaxDept(s.getChildren(),r+1);n<l&&(n=l)}}return n},t.prototype.recursivelyCreateColumns=function(e,r,n,o,a,s){var l=this;return(e||[]).map(function(u){return l.isColumnGroup(u)?l.createColumnGroup(n,u,r,o,a,s):l.createColumn(n,u,o,a)})},t.prototype.createColumnGroup=function(e,r,n,o,a,s){var l=this.createMergedColGroupDef(r),u=a.getUniqueKey(l.groupId||null,null),c=new $e(l,u,!1,n);this.context.createBean(c);var p=this.findExistingGroup(r,s);p&&gr(s,p),p&&p.isExpanded()&&c.setExpanded(!0);var h=this.recursivelyCreateColumns(l.children,n+1,e,o,a,s);return c.setChildren(h),c},t.prototype.createMergedColGroupDef=function(e){var r={};return Object.assign(r,this.gridOptionsWrapper.getDefaultColGroupDef()),Object.assign(r,e),this.checkForDeprecatedItems(r),r},t.prototype.createColumn=function(e,r,n,o){var a=this.mergeColDefs(r);this.checkForDeprecatedItems(a);var s=this.findExistingColumn(r,n);if(n&&s&&gr(n,s),s)s.setColDef(a,r),this.applyColumnState(s,a);else{var l=o.getUniqueKey(a.colId,a.field);s=new Se(a,r,l,e),this.context.createBean(s)}return s},t.prototype.applyColumnState=function(e,r){var n=In(r.flex);n!==void 0&&e.setFlex(n);var o=e.getFlex()<=0;if(o){var a=In(r.width);if(a!=null)e.setActualWidth(a);else{var s=e.getActualWidth();e.setActualWidth(s)}}r.sort!==void 0&&(r.sort==L.SORT_ASC||r.sort==L.SORT_DESC?e.setSort(r.sort):e.setSort(void 0));var l=In(r.sortIndex);l!==void 0&&e.setSortIndex(l);var u=$i(r.hide);u!==void 0&&e.setVisible(!u),r.pinned!==void 0&&e.setPinned(r.pinned)},t.prototype.findExistingColumn=function(e,r){return(r||[]).find(function(n){var o=n.getUserProvidedColDef();if(!o)return!1;var a=e.colId!=null,s=e.field!=null;return a?n.getId()===e.colId:s?o.field===e.field:o===e})},t.prototype.findExistingGroup=function(e,r){return r.find(function(n){var o=n.getColGroupDef();if(!o)return!1;var a=e.groupId!=null;return a?n.getId()===e.groupId:!1})},t.prototype.mergeColDefs=function(e){var r={},n=this.gridOptionsWrapper.getDefaultColDef();Ln(r,n,!1,!0);var o=e.type;return o||(o=n&&n.type),o&&this.assignColumnTypes(o,r),Ln(r,e,!1,!0),r},t.prototype.assignColumnTypes=function(e,r){var n=[];if(e instanceof Array){var o=e.some(function(l){return typeof l!="string"});o?console.warn("AG Grid: if colDef.type is supplied an array it should be of type 'string[]'"):n=e}else if(typeof e=="string")n=e.split(",");else{console.warn("AG Grid: colDef.type should be of type 'string' | 'string[]'");return}var a=Object.assign({},GF),s=this.gridOptionsWrapper.getColumnTypes()||{};Ke(s,function(l,u){l in a?console.warn("AG Grid: the column type '"+l+"' is a default column type and cannot be overridden."):a[l]=u}),n.forEach(function(l){var u=a[l.trim()];u?Ln(r,u,!1,!0):console.warn("AG Grid: colDef.type '"+l+"' does not correspond to defined gridOptions.columnTypes")})},t.prototype.checkForDeprecatedItems=function(e){if(e){var r=e;r.group!==void 0&&console.warn("AG Grid: colDef.group is invalid, please check documentation on how to do grouping as it changed in version 3"),r.headerGroup!==void 0&&console.warn("AG Grid: colDef.headerGroup is invalid, please check documentation on how to do grouping as it changed in version 3"),r.headerGroupShow!==void 0&&console.warn("AG Grid: colDef.headerGroupShow is invalid, should be columnGroupShow, please check documentation on how to do grouping as it changed in version 3"),r.suppressRowGroup!==void 0&&console.warn("AG Grid: colDef.suppressRowGroup is deprecated, please use colDef.type instead"),r.suppressAggregation!==void 0&&console.warn("AG Grid: colDef.suppressAggregation is deprecated, please use colDef.type instead"),(r.suppressRowGroup||r.suppressAggregation)&&console.warn("AG Grid: colDef.suppressAggregation and colDef.suppressRowGroup are deprecated, use allowRowGroup, allowPivot and allowValue instead"),r.displayName&&(console.warn("AG Grid: Found displayName "+r.displayName+", please use headerName instead, displayName is deprecated."),r.headerName=r.displayName)}},t.prototype.isColumnGroup=function(e){return e.children!==void 0},cg([y("columnUtils")],t.prototype,"columnUtils",void 0),cg([jF(0,Yt("loggerFactory"))],t.prototype,"setBeans",null),t=cg([Q("columnFactory")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var x=function(){function i(){}return i.EVENT_COLUMN_EVERYTHING_CHANGED="columnEverythingChanged",i.EVENT_NEW_COLUMNS_LOADED="newColumnsLoaded",i.EVENT_COLUMN_PIVOT_MODE_CHANGED="columnPivotModeChanged",i.EVENT_COLUMN_ROW_GROUP_CHANGED="columnRowGroupChanged",i.EVENT_EXPAND_COLLAPSE_ALL="expandOrCollapseAll",i.EVENT_COLUMN_PIVOT_CHANGED="columnPivotChanged",i.EVENT_GRID_COLUMNS_CHANGED="gridColumnsChanged",i.EVENT_COLUMN_VALUE_CHANGED="columnValueChanged",i.EVENT_COLUMN_MOVED="columnMoved",i.EVENT_COLUMN_VISIBLE="columnVisible",i.EVENT_COLUMN_PINNED="columnPinned",i.EVENT_COLUMN_GROUP_OPENED="columnGroupOpened",i.EVENT_COLUMN_RESIZED="columnResized",i.EVENT_DISPLAYED_COLUMNS_CHANGED="displayedColumnsChanged",i.EVENT_VIRTUAL_COLUMNS_CHANGED="virtualColumnsChanged",i.EVENT_ASYNC_TRANSACTIONS_FLUSHED="asyncTransactionsFlushed",i.EVENT_ROW_GROUP_OPENED="rowGroupOpened",i.EVENT_ROW_DATA_CHANGED="rowDataChanged",i.EVENT_ROW_DATA_UPDATED="rowDataUpdated",i.EVENT_PINNED_ROW_DATA_CHANGED="pinnedRowDataChanged",i.EVENT_RANGE_SELECTION_CHANGED="rangeSelectionChanged",i.EVENT_CHART_CREATED="chartCreated",i.EVENT_CHART_RANGE_SELECTION_CHANGED="chartRangeSelectionChanged",i.EVENT_CHART_OPTIONS_CHANGED="chartOptionsChanged",i.EVENT_CHART_DESTROYED="chartDestroyed",i.EVENT_TOOL_PANEL_VISIBLE_CHANGED="toolPanelVisibleChanged",i.EVENT_COLUMN_PANEL_ITEM_DRAG_START="columnPanelItemDragStart",i.EVENT_COLUMN_PANEL_ITEM_DRAG_END="columnPanelItemDragEnd",i.EVENT_MODEL_UPDATED="modelUpdated",i.EVENT_PASTE_START="pasteStart",i.EVENT_PASTE_END="pasteEnd",i.EVENT_FILL_START="fillStart",i.EVENT_FILL_END="fillEnd",i.EVENT_CELL_CLICKED="cellClicked",i.EVENT_CELL_DOUBLE_CLICKED="cellDoubleClicked",i.EVENT_CELL_MOUSE_DOWN="cellMouseDown",i.EVENT_CELL_CONTEXT_MENU="cellContextMenu",i.EVENT_CELL_VALUE_CHANGED="cellValueChanged",i.EVENT_CELL_EDIT_REQUEST="cellEditRequest",i.EVENT_ROW_VALUE_CHANGED="rowValueChanged",i.EVENT_CELL_FOCUSED="cellFocused",i.EVENT_FULL_WIDTH_ROW_FOCUSED="fullWidthRowFocused",i.EVENT_ROW_SELECTED="rowSelected",i.EVENT_SELECTION_CHANGED="selectionChanged",i.EVENT_CELL_KEY_DOWN="cellKeyDown",i.EVENT_CELL_KEY_PRESS="cellKeyPress",i.EVENT_CELL_MOUSE_OVER="cellMouseOver",i.EVENT_CELL_MOUSE_OUT="cellMouseOut",i.EVENT_FILTER_CHANGED="filterChanged",i.EVENT_FILTER_MODIFIED="filterModified",i.EVENT_FILTER_OPENED="filterOpened",i.EVENT_SORT_CHANGED="sortChanged",i.EVENT_VIRTUAL_ROW_REMOVED="virtualRowRemoved",i.EVENT_ROW_CLICKED="rowClicked",i.EVENT_ROW_DOUBLE_CLICKED="rowDoubleClicked",i.EVENT_GRID_READY="gridReady",i.EVENT_GRID_SIZE_CHANGED="gridSizeChanged",i.EVENT_VIEWPORT_CHANGED="viewportChanged",i.EVENT_SCROLLBAR_WIDTH_CHANGED="scrollbarWidthChanged",i.EVENT_FIRST_DATA_RENDERED="firstDataRendered",i.EVENT_DRAG_STARTED="dragStarted",i.EVENT_DRAG_STOPPED="dragStopped",i.EVENT_CHECKBOX_CHANGED="checkboxChanged",i.EVENT_ROW_EDITING_STARTED="rowEditingStarted",i.EVENT_ROW_EDITING_STOPPED="rowEditingStopped",i.EVENT_CELL_EDITING_STARTED="cellEditingStarted",i.EVENT_CELL_EDITING_STOPPED="cellEditingStopped",i.EVENT_BODY_SCROLL="bodyScroll",i.EVENT_BODY_SCROLL_END="bodyScrollEnd",i.EVENT_HEIGHT_SCALE_CHANGED="heightScaleChanged",i.EVENT_PAGINATION_CHANGED="paginationChanged",i.EVENT_COMPONENT_STATE_CHANGED="componentStateChanged",i.EVENT_BODY_HEIGHT_CHANGED="bodyHeightChanged",i.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED="displayedColumnsWidthChanged",i.EVENT_SCROLL_VISIBILITY_CHANGED="scrollVisibilityChanged",i.EVENT_COLUMN_HOVER_CHANGED="columnHoverChanged",i.EVENT_FLASH_CELLS="flashCells",i.EVENT_PAGINATION_PIXEL_OFFSET_CHANGED="paginationPixelOffsetChanged",i.EVENT_DISPLAYED_ROWS_CHANGED="displayedRowsChanged",i.EVENT_LEFT_PINNED_WIDTH_CHANGED="leftPinnedWidthChanged",i.EVENT_RIGHT_PINNED_WIDTH_CHANGED="rightPinnedWidthChanged",i.EVENT_ROW_CONTAINER_HEIGHT_CHANGED="rowContainerHeightChanged",i.EVENT_ROW_DRAG_ENTER="rowDragEnter",i.EVENT_ROW_DRAG_MOVE="rowDragMove",i.EVENT_ROW_DRAG_LEAVE="rowDragLeave",i.EVENT_ROW_DRAG_END="rowDragEnd",i.EVENT_POPUP_TO_FRONT="popupToFront",i.EVENT_COLUMN_ROW_GROUP_CHANGE_REQUEST="columnRowGroupChangeRequest",i.EVENT_COLUMN_PIVOT_CHANGE_REQUEST="columnPivotChangeRequest",i.EVENT_COLUMN_VALUE_CHANGE_REQUEST="columnValueChangeRequest",i.EVENT_COLUMN_AGG_FUNC_CHANGE_REQUEST="columnAggFuncChangeRequest",i.EVENT_KEYBOARD_FOCUS="keyboardFocus",i.EVENT_MOUSE_FOCUS="mouseFocus",i.EVENT_STORE_UPDATED="storeUpdated",i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var OC=function(){function i(){this.existingIds={}}return i.prototype.getInstanceIdForKey=function(t){var e=this.existingIds[t],r;return typeof e!="number"?r=0:r=e+1,this.existingIds[t]=r,r},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var UF=/[&<>"']/g,zF={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"};function $F(i){var t=String.fromCharCode;function e(p){var h=[];if(!p)return[];for(var d=p.length,f=0,g,v;f<d;)g=p.charCodeAt(f++),g>=55296&&g<=56319&&f<d?(v=p.charCodeAt(f++),(v&64512)==56320?h.push(((g&1023)<<10)+(v&1023)+65536):(h.push(g),f--)):h.push(g);return h}function r(p){if(p>=55296&&p<=57343)throw Error("Lone surrogate U+"+p.toString(16).toUpperCase()+" is not a scalar value")}function n(p,h){return t(p>>h&63|128)}function o(p){if(p>=0&&p<=31&&p!==10){var h=p.toString(16).toUpperCase(),d=h.padStart(4,"0");return"_x"+d+"_"}if((p&4294967168)==0)return t(p);var f="";return(p&4294965248)==0?f=t(p>>6&31|192):(p&4294901760)==0?(r(p),f=t(p>>12&15|224),f+=n(p,6)):(p&4292870144)==0&&(f=t(p>>18&7|240),f+=n(p,12),f+=n(p,6)),f+=t(p&63|128),f}for(var a=e(i),s=a.length,l=-1,u,c="";++l<s;)u=a[l],c+=o(u);return c}function yO(i){return i==null?null:i.replace(/([A-Z])/g,function(t){return"-"+t[0].toLowerCase()})}function CO(i){return i==null?null:i.replace(/-([a-z])/g,function(t){return t[1].toUpperCase()})}function Hm(i){return i[0].toUpperCase()+i.substr(1).toLowerCase()}function Ii(i){return i==null?null:i.toString().toString().replace(UF,function(t){return zF[t]})}function EC(i){if(!i||i==null)return null;var t=/([A-Z])([A-Z])([a-z])|([a-z])([A-Z])/g,e=i.replace(t,"$1$4 $2$3$5").replace("."," ").split(" ");return e.map(function(r){return r.substring(0,1).toUpperCase()+(r.length>1?r.substring(1,r.length):"")}).join(" ")}var KF=Object.freeze(Object.defineProperty({__proto__:null,utf8_encode:$F,camelCaseToHyphen:yO,hyphenToCamelCase:CO,capitalise:Hm,escapeString:Ii,camelCaseToHumanText:EC},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function zo(i){var t=new Map;return i.forEach(function(e){return t.set(e[0],e[1])}),t}function YF(i,t){var e=new Map;return i.forEach(function(r){return e.set(t(r),r)}),e}function XF(i){var t=[];return i.forEach(function(e,r){return t.push(r)}),t}var qF=Object.freeze(Object.defineProperty({__proto__:null,convertToMap:zo,mapById:YF,keys:XF},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ZF=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),kt=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},QF=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},t_=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},JF=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},iu=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(JF(arguments[t]));return i},wO=function(i){ZF(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.primaryHeaderRowCount=0,e.secondaryHeaderRowCount=0,e.gridHeaderRowCount=0,e.displayedColumnsLeft=[],e.displayedColumnsRight=[],e.displayedColumnsCenter=[],e.displayedColumns=[],e.displayedColumnsAndGroupsMap={},e.viewportColumns=[],e.viewportColumnsCenter=[],e.autoHeightActiveAtLeastOnce=!1,e.rowGroupColumns=[],e.valueColumns=[],e.pivotColumns=[],e.ready=!1,e.autoGroupsNeedBuilding=!1,e.forceRecreateAutoGroups=!1,e.pivotMode=!1,e.bodyWidth=0,e.leftWidth=0,e.rightWidth=0,e.bodyWidthDirty=!0,e.colDefVersion=0,e.flexColsCalculatedAtLestOnce=!1,e}return t.prototype.init=function(){var e=this;this.suppressColumnVirtualisation=this.gridOptionsWrapper.isSuppressColumnVirtualisation();var r=this.gridOptionsWrapper.isPivotMode();this.isPivotSettingAllowed(r)&&(this.pivotMode=r),this.usingTreeData=this.gridOptionsWrapper.isTreeData(),this.addManagedListener(this.gridOptionsWrapper,"autoGroupColumnDef",function(){return e.onAutoGroupColumnDefChanged()}),this.addManagedListener(this.gridOptionsWrapper,"defaultColDef",function(){return e.onDefaultColDefChanged()})},t.prototype.onAutoGroupColumnDefChanged=function(){this.autoGroupsNeedBuilding=!0,this.forceRecreateAutoGroups=!0,this.updateGridColumns(),this.updateDisplayedColumns("gridOptionsChanged")},t.prototype.onDefaultColDefChanged=function(){this.colDefVersion++,this.forceRecreateAutoGroups=!0,this.createColumnsFromColumnDefs(!0)},t.prototype.getColDefVersion=function(){return this.colDefVersion},t.prototype.setColumnDefs=function(e,r){r===void 0&&(r="api");var n=!!this.columnDefs;this.colDefVersion++,this.columnDefs=e,this.createColumnsFromColumnDefs(n,r)},t.prototype.createColumnsFromColumnDefs=function(e,r){var n=this;r===void 0&&(r="api");var o=e?this.compareColumnStatesAndRaiseEvents(r):void 0;this.valueCache.expire(),this.autoGroupsNeedBuilding=!0;var a=this.primaryColumns,s=this.primaryColumnTree,l=this.columnFactory.createColumnTree(this.columnDefs,!0,s);this.primaryColumnTree=l.columnTree,this.primaryHeaderRowCount=l.treeDept+1,this.primaryColumns=this.getColumnsFromTree(this.primaryColumnTree),this.primaryColumnsMap={},this.primaryColumns.forEach(function(p){return n.primaryColumnsMap[p.getId()]=p}),this.extractRowGroupColumns(r,a),this.extractPivotColumns(r,a),this.extractValueColumns(r,a),this.ready=!0;var u=this.gridColsArePrimary===void 0,c=this.gridColsArePrimary||u;c&&(this.updateGridColumns(),e&&!this.gridOptionsWrapper.isMaintainColumnOrder()&&this.orderGridColumnsLikePrimary(),this.updateDisplayedColumns(r),this.checkViewportColumns()),this.dispatchEverythingChanged(r),o&&o(),this.dispatchNewColumnsLoaded()},t.prototype.dispatchNewColumnsLoaded=function(){var e={type:x.EVENT_NEW_COLUMNS_LOADED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(e)},t.prototype.dispatchEverythingChanged=function(e){e===void 0&&(e="api");var r={type:x.EVENT_COLUMN_EVERYTHING_CHANGED,api:this.gridApi,columnApi:this.columnApi,source:e};this.eventService.dispatchEvent(r)},t.prototype.orderGridColumnsLikePrimary=function(){var e=this,r=this.primaryColumns;!r||this.gridColumns.sort(function(n,o){var a=r.indexOf(n),s=r.indexOf(o),l=a>=0,u=s>=0;if(l&&u)return a-s;if(l)return 1;if(u)return-1;var c=e.gridColumns.indexOf(n),p=e.gridColumns.indexOf(o);return c-p})},t.prototype.getAllDisplayedAutoHeightCols=function(){return this.displayedAutoHeightCols},t.prototype.setViewport=function(){this.gridOptionsWrapper.isEnableRtl()?(this.viewportLeft=this.bodyWidth-this.scrollPosition-this.scrollWidth,this.viewportRight=this.bodyWidth-this.scrollPosition):(this.viewportLeft=this.scrollPosition,this.viewportRight=this.scrollWidth+this.scrollPosition)},t.prototype.getDisplayedColumnsStartingAt=function(e){for(var r=e,n=[];r!=null;)n.push(r),r=this.getDisplayedColAfter(r);return n},t.prototype.checkViewportColumns=function(){if(this.displayedColumnsCenter!=null){var e=this.viewportColumns.map(function(o){return o.getId()}).join("#");this.extractViewport();var r=this.viewportColumns.map(function(o){return o.getId()}).join("#");if(e!==r){var n={type:x.EVENT_VIRTUAL_COLUMNS_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(n)}}},t.prototype.setViewportPosition=function(e,r){(e!==this.scrollWidth||r!==this.scrollPosition||this.bodyWidthDirty)&&(this.scrollWidth=e,this.scrollPosition=r,this.bodyWidthDirty=!0,this.setViewport(),this.ready&&this.checkViewportColumns())},t.prototype.isPivotMode=function(){return this.pivotMode},t.prototype.isPivotSettingAllowed=function(e){return e&&this.gridOptionsWrapper.isTreeData()?(console.warn("AG Grid: Pivot mode not available in conjunction Tree Data i.e. 'gridOptions.treeData: true'"),!1):!0},t.prototype.setPivotMode=function(e,r){if(r===void 0&&(r="api"),!(e===this.pivotMode||!this.isPivotSettingAllowed(this.pivotMode))){this.pivotMode=e,this.autoGroupsNeedBuilding=!0,this.updateGridColumns(),this.updateDisplayedColumns(r);var n={type:x.EVENT_COLUMN_PIVOT_MODE_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(n)}},t.prototype.getSecondaryPivotColumn=function(e,r){if(me(this.secondaryColumns))return null;var n=this.getPrimaryColumn(r),o=null;return this.secondaryColumns.forEach(function(a){var s=a.getColDef().pivotKeys,l=a.getColDef().pivotValueColumn,u=es(s,e),c=l===n;u&&c&&(o=a)}),o},t.prototype.setBeans=function(e){this.logger=e.create("columnModel")},t.prototype.setFirstRightAndLastLeftPinned=function(e){var r,n;this.gridOptionsWrapper.isEnableRtl()?(r=this.displayedColumnsLeft?this.displayedColumnsLeft[0]:null,n=this.displayedColumnsRight?rt(this.displayedColumnsRight):null):(r=this.displayedColumnsLeft?rt(this.displayedColumnsLeft):null,n=this.displayedColumnsRight?this.displayedColumnsRight[0]:null),this.gridColumns.forEach(function(o){o.setLastLeftPinned(o===r,e),o.setFirstRightPinned(o===n,e)})},t.prototype.autoSizeColumns=function(e){var r=this,n=e.columns,o=e.skipHeader,a=e.skipHeaderGroups,s=e.stopAtGroup,l=e.source,u=l===void 0?"api":l;this.animationFrameService.flushAllFrames();for(var c=[],p=-1,h=o!=null?o:this.gridOptionsWrapper.isSkipHeaderOnAutoSize(),d=a!=null?a:h;p!==0;)p=0,this.actionOnGridColumns(n,function(f){if(c.indexOf(f)>=0)return!1;var g=r.autoWidthCalculator.getPreferredWidthForColumn(f,h);if(g>0){var v=r.normaliseColumnWidth(f,g);f.setActualWidth(v,u),c.push(f),p++}return!0},u);d||this.autoSizeColumnGroupsByColumns(n,s),this.fireColumnResizedEvent(c,!0,"autosizeColumns")},t.prototype.fireColumnResizedEvent=function(e,r,n,o){if(o===void 0&&(o=null),e&&e.length){var a={type:x.EVENT_COLUMN_RESIZED,columns:e,column:e.length===1?e[0]:null,flexColumns:o,finished:r,api:this.gridApi,columnApi:this.columnApi,source:n};this.eventService.dispatchEvent(a)}},t.prototype.autoSizeColumn=function(e,r,n){n===void 0&&(n="api"),e&&this.autoSizeColumns({columns:[e],skipHeader:r,skipHeaderGroups:!0,source:n})},t.prototype.autoSizeColumnGroupsByColumns=function(e,r){var n,o,a,s,l=new Set,u=this.getGridColumns(e);u.forEach(function(C){for(var w=C.getParent();w&&w!=r;)w.isPadding()||l.add(w),w=w.getParent()});var c,p=[];try{for(var h=t_(l),d=h.next();!d.done;d=h.next()){var f=d.value;try{for(var g=(a=void 0,t_(this.ctrlsService.getHeaderRowContainerCtrls())),v=g.next();!v.done;v=g.next()){var m=v.value;if(c=m.getHeaderCtrlForColumn(f),c)break}}catch(C){a={error:C}}finally{try{v&&!v.done&&(s=g.return)&&s.call(g)}finally{if(a)throw a.error}}c&&c.resizeLeafColumnsToFit()}}catch(C){n={error:C}}finally{try{d&&!d.done&&(o=h.return)&&o.call(h)}finally{if(n)throw n.error}}return p},t.prototype.autoSizeAllColumns=function(e,r){r===void 0&&(r="api");var n=this.getAllDisplayedColumns();this.autoSizeColumns({columns:n,skipHeader:e,source:r})},t.prototype.getColumnsFromTree=function(e){var r=[],n=function(o){for(var a=0;a<o.length;a++){var s=o[a];s instanceof Se?r.push(s):s instanceof $e&&n(s.getChildren())}};return n(e),r},t.prototype.getAllDisplayedTrees=function(){return this.displayedTreeLeft&&this.displayedTreeRight&&this.displayedTreeCentre?this.displayedTreeLeft.concat(this.displayedTreeCentre).concat(this.displayedTreeRight):null},t.prototype.getPrimaryColumnTree=function(){return this.primaryColumnTree},t.prototype.getHeaderRowCount=function(){return this.gridHeaderRowCount},t.prototype.getDisplayedTreeLeft=function(){return this.displayedTreeLeft},t.prototype.getDisplayedTreeRight=function(){return this.displayedTreeRight},t.prototype.getDisplayedTreeCentre=function(){return this.displayedTreeCentre},t.prototype.isColumnDisplayed=function(e){return this.getAllDisplayedColumns().indexOf(e)>=0},t.prototype.getAllDisplayedColumns=function(){return this.displayedColumns},t.prototype.getViewportColumns=function(){return this.viewportColumns},t.prototype.getDisplayedLeftColumnsForRow=function(e){return this.colSpanActive?this.getDisplayedColumnsForRow(e,this.displayedColumnsLeft):this.displayedColumnsLeft},t.prototype.getDisplayedRightColumnsForRow=function(e){return this.colSpanActive?this.getDisplayedColumnsForRow(e,this.displayedColumnsRight):this.displayedColumnsRight},t.prototype.getDisplayedColumnsForRow=function(e,r,n,o){for(var a=[],s=null,l=function(p){var h=r[p],d=r.length-p,f=Math.min(h.getColSpan(e),d),g=[h];if(f>1){for(var v=f-1,m=1;m<=v;m++)g.push(r[p+m]);p+=v}var C;if(n?(C=!1,g.forEach(function(b){n(b)&&(C=!0)})):C=!0,C){if(a.length===0&&s){var w=o?o(h):!1;w&&a.push(s)}a.push(h)}s=h,u=p},u,c=0;c<r.length;c++)l(c),c=u;return a},t.prototype.getViewportCenterColumnsForRow=function(e){var r=this;if(!this.colSpanActive)return this.viewportColumnsCenter;var n=function(a){var s=a.getLeft();return q(s)&&s>r.viewportLeft},o=this.suppressColumnVirtualisation?null:this.isColumnInViewport.bind(this);return this.getDisplayedColumnsForRow(e,this.displayedColumnsCenter,o,n)},t.prototype.getAriaColumnIndex=function(e){return this.getAllGridColumns().indexOf(e)+1},t.prototype.isColumnInViewport=function(e){if(e.isAutoHeight())return!0;var r=e.getLeft()||0,n=r+e.getActualWidth(),o=this.viewportLeft-200,a=this.viewportRight+200,s=r<o&&n<o,l=r>a&&n>a;return!s&&!l},t.prototype.getDisplayedColumnsLeftWidth=function(){return this.getWidthOfColsInList(this.displayedColumnsLeft)},t.prototype.getDisplayedColumnsRightWidth=function(){return this.getWidthOfColsInList(this.displayedColumnsRight)},t.prototype.updatePrimaryColumnList=function(e,r,n,o,a,s){var l=this;if(s===void 0&&(s="api"),!(!e||pn(e))){var u=!1;if(e.forEach(function(p){var h=l.getPrimaryColumn(p);if(!!h){if(n){if(r.indexOf(h)>=0)return;r.push(h)}else{if(r.indexOf(h)<0)return;gr(r,h)}o(h),u=!0}}),!!u){this.autoGroupsNeedBuilding&&this.updateGridColumns(),this.updateDisplayedColumns(s);var c={type:a,columns:r,column:r.length===1?r[0]:null,api:this.gridApi,columnApi:this.columnApi,source:s};this.eventService.dispatchEvent(c)}}},t.prototype.setRowGroupColumns=function(e,r){r===void 0&&(r="api"),this.autoGroupsNeedBuilding=!0,this.setPrimaryColumnList(e,this.rowGroupColumns,x.EVENT_COLUMN_ROW_GROUP_CHANGED,this.setRowGroupActive.bind(this),r)},t.prototype.setRowGroupActive=function(e,r,n){e!==r.isRowGroupActive()&&(r.setRowGroupActive(e,n),!e&&!this.gridOptionsWrapper.isSuppressMakeColumnVisibleAfterUnGroup()&&r.setVisible(!0,n))},t.prototype.addRowGroupColumn=function(e,r){r===void 0&&(r="api"),e&&this.addRowGroupColumns([e],r)},t.prototype.addRowGroupColumns=function(e,r){r===void 0&&(r="api"),this.autoGroupsNeedBuilding=!0,this.updatePrimaryColumnList(e,this.rowGroupColumns,!0,this.setRowGroupActive.bind(this,!0),x.EVENT_COLUMN_ROW_GROUP_CHANGED,r)},t.prototype.removeRowGroupColumns=function(e,r){r===void 0&&(r="api"),this.autoGroupsNeedBuilding=!0,this.updatePrimaryColumnList(e,this.rowGroupColumns,!1,this.setRowGroupActive.bind(this,!1),x.EVENT_COLUMN_ROW_GROUP_CHANGED,r)},t.prototype.removeRowGroupColumn=function(e,r){r===void 0&&(r="api"),e&&this.removeRowGroupColumns([e],r)},t.prototype.addPivotColumns=function(e,r){r===void 0&&(r="api"),this.updatePrimaryColumnList(e,this.pivotColumns,!0,function(n){return n.setPivotActive(!0,r)},x.EVENT_COLUMN_PIVOT_CHANGED,r)},t.prototype.setPivotColumns=function(e,r){r===void 0&&(r="api"),this.setPrimaryColumnList(e,this.pivotColumns,x.EVENT_COLUMN_PIVOT_CHANGED,function(n,o){o.setPivotActive(n,r)},r)},t.prototype.addPivotColumn=function(e,r){r===void 0&&(r="api"),this.addPivotColumns([e],r)},t.prototype.removePivotColumns=function(e,r){r===void 0&&(r="api"),this.updatePrimaryColumnList(e,this.pivotColumns,!1,function(n){return n.setPivotActive(!1,r)},x.EVENT_COLUMN_PIVOT_CHANGED,r)},t.prototype.removePivotColumn=function(e,r){r===void 0&&(r="api"),this.removePivotColumns([e],r)},t.prototype.setPrimaryColumnList=function(e,r,n,o,a){var s=this;r.length=0,q(e)&&e.forEach(function(l){var u=s.getPrimaryColumn(l);u&&r.push(u)}),(this.primaryColumns||[]).forEach(function(l){var u=r.indexOf(l)>=0;o(u,l)}),this.autoGroupsNeedBuilding&&this.updateGridColumns(),this.updateDisplayedColumns(a),this.fireColumnEvent(n,r,a)},t.prototype.setValueColumns=function(e,r){r===void 0&&(r="api"),this.setPrimaryColumnList(e,this.valueColumns,x.EVENT_COLUMN_VALUE_CHANGED,this.setValueActive.bind(this),r)},t.prototype.setValueActive=function(e,r,n){if(e!==r.isValueActive()&&(r.setValueActive(e,n),e&&!r.getAggFunc())){var o=this.aggFuncService.getDefaultAggFunc(r);r.setAggFunc(o)}},t.prototype.addValueColumns=function(e,r){r===void 0&&(r="api"),this.updatePrimaryColumnList(e,this.valueColumns,!0,this.setValueActive.bind(this,!0),x.EVENT_COLUMN_VALUE_CHANGED,r)},t.prototype.addValueColumn=function(e,r){r===void 0&&(r="api"),e&&this.addValueColumns([e],r)},t.prototype.removeValueColumn=function(e,r){r===void 0&&(r="api"),this.removeValueColumns([e],r)},t.prototype.removeValueColumns=function(e,r){r===void 0&&(r="api"),this.updatePrimaryColumnList(e,this.valueColumns,!1,this.setValueActive.bind(this,!1),x.EVENT_COLUMN_VALUE_CHANGED,r)},t.prototype.normaliseColumnWidth=function(e,r){var n=e.getMinWidth();q(n)&&r<n&&(r=n);var o=e.getMaxWidth();return q(o)&&e.isGreaterThanMax(r)&&(r=o),r},t.prototype.getPrimaryOrGridColumn=function(e){var r=this.getPrimaryColumn(e);return r||this.getGridColumn(e)},t.prototype.setColumnWidths=function(e,r,n,o){var a=this;o===void 0&&(o="api");var s=[];e.forEach(function(l){var u=a.getPrimaryOrGridColumn(l.key);if(!!u){s.push({width:l.newWidth,ratios:[1],columns:[u]});var c=a.gridOptionsWrapper.getColResizeDefault()==="shift";if(c&&(r=!r),r){var p=a.getDisplayedColAfter(u);if(!p)return;var h=u.getActualWidth()-l.newWidth,d=p.getActualWidth()+h;s.push({width:d,ratios:[1],columns:[p]})}}}),s.length!==0&&this.resizeColumnSets({resizeSets:s,finished:n,source:o})},t.prototype.checkMinAndMaxWidthsForSet=function(e){var r=e.columns,n=e.width,o=0,a=0,s=!0;r.forEach(function(c){var p=c.getMinWidth();o+=p||0;var h=c.getMaxWidth();q(h)&&h>0?a+=h:s=!1});var l=n>=o,u=!s||n<=a;return l&&u},t.prototype.resizeColumnSets=function(e){var r=this,n=e.resizeSets,o=e.finished,a=e.source,s=!n||n.every(function(f){return r.checkMinAndMaxWidthsForSet(f)});if(!s){if(o){var l=n&&n.length>0?n[0].columns:null;this.fireColumnResizedEvent(l,o,a)}return}var u=[],c=[];n.forEach(function(f){var g=f.width,v=f.columns,m=f.ratios,C={},w={};v.forEach(function(P){return c.push(P)});for(var b=!0,S=0,T=function(){if(S++,S>1e3)return console.error("AG Grid: infinite loop in resizeColumnSets"),"break";b=!1;var P=[],R=0,M=g;v.forEach(function(W,G){var k=w[W.getId()];if(k)M-=C[W.getId()];else{P.push(W);var V=m[G];R+=V}});var N=1/R;P.forEach(function(W,G){var k=G===P.length-1,V;k?V=M:(V=Math.round(m[G]*g*N),M-=V);var U=W.getMinWidth(),J=W.getMaxWidth();q(U)&&V<U?(V=U,w[W.getId()]=!0,b=!0):q(J)&&J>0&&V>J&&(V=J,w[W.getId()]=!0,b=!0),C[W.getId()]=V})};b;){var E=T();if(E==="break")break}v.forEach(function(P){var R=C[P.getId()],M=P.getActualWidth();M!==R&&(P.setActualWidth(R,a),u.push(P))})});var p=u.length>0,h=[];p&&(h=this.refreshFlexedColumns({resizingCols:c,skipSetLeft:!0}),this.setLeftValues(a),this.updateBodyWidths(),this.checkViewportColumns());var d=c.concat(h);(p||o)&&this.fireColumnResizedEvent(d,o,a,h)},t.prototype.setColumnAggFunc=function(e,r,n){if(n===void 0&&(n="api"),!!e){var o=this.getPrimaryColumn(e);!o||(o.setAggFunc(r),this.fireColumnEvent(x.EVENT_COLUMN_VALUE_CHANGED,[o],n))}},t.prototype.fireColumnEvent=function(e,r,n){var o={type:e,columns:r,column:r&&r.length==1?r[0]:null,api:this.gridApi,columnApi:this.columnApi,source:n};this.eventService.dispatchEvent(o)},t.prototype.moveRowGroupColumn=function(e,r,n){n===void 0&&(n="api");var o=this.rowGroupColumns[e];this.rowGroupColumns.splice(e,1),this.rowGroupColumns.splice(r,0,o);var a={type:x.EVENT_COLUMN_ROW_GROUP_CHANGED,columns:this.rowGroupColumns,column:this.rowGroupColumns.length===1?this.rowGroupColumns[0]:null,api:this.gridApi,columnApi:this.columnApi,source:n};this.eventService.dispatchEvent(a)},t.prototype.moveColumns=function(e,r,n){if(n===void 0&&(n="api"),this.columnAnimationService.start(),r>this.gridColumns.length-e.length){console.warn("AG Grid: tried to insert columns in invalid location, toIndex = "+r),console.warn("AG Grid: remember that you should not count the moving columns when calculating the new index");return}var o=this.getGridColumns(e),a=!this.doesMovePassRules(o,r);if(!a){Vm(this.gridColumns,o,r),this.updateDisplayedColumns(n);var s={type:x.EVENT_COLUMN_MOVED,columns:o,column:o.length===1?o[0]:null,toIndex:r,api:this.gridApi,columnApi:this.columnApi,source:n};this.eventService.dispatchEvent(s),this.columnAnimationService.finish()}},t.prototype.doesMovePassRules=function(e,r){var n=this.gridColumns.slice();return Vm(n,e,r),!(!this.doesMovePassMarryChildren(n)||!this.doesMovePassLockedPositions(n))},t.prototype.sortColumnsLikeGridColumns=function(e){var r=this;if(!(!e||e.length<=1)){var n=e.filter(function(o){return r.gridColumns.indexOf(o)<0}).length>0;n||e.sort(function(o,a){var s=r.gridColumns.indexOf(o),l=r.gridColumns.indexOf(a);return s-l})}},t.prototype.doesMovePassLockedPositions=function(e){var r=0,n=!0,o=function(a){return a?a===!0||a==="left"?0:2:1};return e.forEach(function(a){var s=o(a.getColDef().lockPosition);s<r&&(n=!1),r=s}),n},t.prototype.doesMovePassMarryChildren=function(e){var r=!0;return this.columnUtils.depthFirstOriginalTreeSearch(null,this.gridBalancedTree,function(n){if(n instanceof $e){var o=n,a=o.getColGroupDef(),s=a&&a.marryChildren;if(!!s){var l=[];o.getLeafColumns().forEach(function(d){var f=e.indexOf(d);l.push(f)});var u=Math.max.apply(Math,l),c=Math.min.apply(Math,l),p=u-c,h=o.getLeafColumns().length-1;p>h&&(r=!1)}}}),r},t.prototype.moveColumn=function(e,r,n){n===void 0&&(n="api"),this.moveColumns([e],r,n)},t.prototype.moveColumnByIndex=function(e,r,n){n===void 0&&(n="api");var o=this.gridColumns[e];this.moveColumn(o,r,n)},t.prototype.getColumnDefs=function(){var e=this;if(!!this.primaryColumns){var r=this.primaryColumns.slice();return this.gridColsArePrimary?r.sort(function(n,o){return e.gridColumns.indexOf(n)-e.gridColumns.indexOf(o)}):this.lastPrimaryOrder&&r.sort(function(n,o){return e.lastPrimaryOrder.indexOf(n)-e.lastPrimaryOrder.indexOf(o)}),this.columnDefFactory.buildColumnDefs(r,this.rowGroupColumns,this.pivotColumns)}},t.prototype.getBodyContainerWidth=function(){return this.bodyWidth},t.prototype.getContainerWidth=function(e){switch(e){case L.PINNED_LEFT:return this.leftWidth;case L.PINNED_RIGHT:return this.rightWidth;default:return this.bodyWidth}},t.prototype.updateBodyWidths=function(){var e=this.getWidthOfColsInList(this.displayedColumnsCenter),r=this.getWidthOfColsInList(this.displayedColumnsLeft),n=this.getWidthOfColsInList(this.displayedColumnsRight);this.bodyWidthDirty=this.bodyWidth!==e;var o=this.bodyWidth!==e||this.leftWidth!==r||this.rightWidth!==n;if(o){this.bodyWidth=e,this.leftWidth=r,this.rightWidth=n;var a={type:x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(a)}},t.prototype.getValueColumns=function(){return this.valueColumns?this.valueColumns:[]},t.prototype.getPivotColumns=function(){return this.pivotColumns?this.pivotColumns:[]},t.prototype.isPivotActive=function(){return this.pivotColumns&&this.pivotColumns.length>0&&this.pivotMode},t.prototype.getRowGroupColumns=function(){return this.rowGroupColumns?this.rowGroupColumns:[]},t.prototype.getDisplayedCenterColumns=function(){return this.displayedColumnsCenter},t.prototype.getDisplayedLeftColumns=function(){return this.displayedColumnsLeft},t.prototype.getDisplayedRightColumns=function(){return this.displayedColumnsRight},t.prototype.getDisplayedColumns=function(e){switch(e){case L.PINNED_LEFT:return this.getDisplayedLeftColumns();case L.PINNED_RIGHT:return this.getDisplayedRightColumns();default:return this.getDisplayedCenterColumns()}},t.prototype.getAllPrimaryColumns=function(){return this.primaryColumns?this.primaryColumns.slice():null},t.prototype.getSecondaryColumns=function(){return this.secondaryColumns?this.secondaryColumns.slice():null},t.prototype.getAllColumnsForQuickFilter=function(){return this.columnsForQuickFilter},t.prototype.getAllGridColumns=function(){return this.gridColumns},t.prototype.isEmpty=function(){return pn(this.gridColumns)},t.prototype.isRowGroupEmpty=function(){return pn(this.rowGroupColumns)},t.prototype.setColumnVisible=function(e,r,n){n===void 0&&(n="api"),this.setColumnsVisible([e],r,n)},t.prototype.setColumnsVisible=function(e,r,n){var o=this;r===void 0&&(r=!1),n===void 0&&(n="api"),this.columnAnimationService.start(),this.actionOnGridColumns(e,function(a){return a.isVisible()!==r?(a.setVisible(r,n),!0):!1},n,function(){var a={type:x.EVENT_COLUMN_VISIBLE,visible:r,column:null,columns:null,api:o.gridApi,columnApi:o.columnApi,source:n};return a}),this.columnAnimationService.finish()},t.prototype.setColumnPinned=function(e,r,n){n===void 0&&(n="api"),e&&this.setColumnsPinned([e],r,n)},t.prototype.setColumnsPinned=function(e,r,n){var o=this;if(n===void 0&&(n="api"),this.gridOptionsWrapper.getDomLayout()==="print"){console.warn("Changing the column pinning status is not allowed with domLayout='print'");return}this.columnAnimationService.start();var a;r===!0||r===L.PINNED_LEFT?a=L.PINNED_LEFT:r===L.PINNED_RIGHT?a=L.PINNED_RIGHT:a=null,this.actionOnGridColumns(e,function(s){return s.getPinned()!==a?(s.setPinned(a),!0):!1},n,function(){var s={type:x.EVENT_COLUMN_PINNED,pinned:a,column:null,columns:null,api:o.gridApi,columnApi:o.columnApi,source:n};return s}),this.columnAnimationService.finish()},t.prototype.actionOnGridColumns=function(e,r,n,o){var a=this;if(!pn(e)){var s=[];if(e.forEach(function(u){var c=a.getGridColumn(u);if(!!c){var p=r(c);p!==!1&&s.push(c)}}),!!s.length&&(this.updateDisplayedColumns(n),q(o)&&o)){var l=o();l.columns=s,l.column=s.length===1?s[0]:null,this.eventService.dispatchEvent(l)}}},t.prototype.getDisplayedColBefore=function(e){var r=this.getAllDisplayedColumns(),n=r.indexOf(e);return n>0?r[n-1]:null},t.prototype.getDisplayedColAfter=function(e){var r=this.getAllDisplayedColumns(),n=r.indexOf(e);return n<r.length-1?r[n+1]:null},t.prototype.getDisplayedGroupAfter=function(e){return this.getDisplayedGroupAtDirection(e,"After")},t.prototype.getDisplayedGroupBefore=function(e){return this.getDisplayedGroupAtDirection(e,"Before")},t.prototype.getDisplayedGroupAtDirection=function(e,r){for(var n=e.getProvidedColumnGroup().getLevel()+e.getPaddingLevel(),o=e.getDisplayedLeafColumns(),a=r==="After"?rt(o):o[0],s="getDisplayedCol"+r;;){var l=this[s](a);if(!l)return null;var u=this.getColumnGroupAtLevel(l,n);if(u!==e)return u}},t.prototype.getColumnGroupAtLevel=function(e,r){for(var n=e.getParent(),o,a;;){var s=n.getProvidedColumnGroup();if(o=s.getLevel(),a=n.getPaddingLevel(),o+a<=r)break;n=n.getParent()}return n},t.prototype.isPinningLeft=function(){return this.displayedColumnsLeft.length>0},t.prototype.isPinningRight=function(){return this.displayedColumnsRight.length>0},t.prototype.getPrimaryAndSecondaryAndAutoColumns=function(){var e;return(e=[]).concat.apply(e,iu([this.primaryColumns||[],this.groupAutoColumns||[],this.secondaryColumns||[]]))},t.prototype.getPrimaryAndAutoGroupCols=function(){var e;return(e=[]).concat.apply(e,iu([this.primaryColumns||[],this.groupAutoColumns||[]]))},t.prototype.getPrimaryAndSecondaryColumns=function(){var e;return(e=[]).concat.apply(e,iu([this.primaryColumns||[],this.secondaryColumns||[]]))},t.prototype.createStateItemFromColumn=function(e){var r=e.isRowGroupActive()?this.rowGroupColumns.indexOf(e):null,n=e.isPivotActive()?this.pivotColumns.indexOf(e):null,o=e.isValueActive()?e.getAggFunc():null,a=e.getSort()!=null?e.getSort():null,s=e.getSortIndex()!=null?e.getSortIndex():null,l=e.getFlex()!=null&&e.getFlex()>0?e.getFlex():null,u={colId:e.getColId(),width:e.getActualWidth(),hide:!e.isVisible(),pinned:e.getPinned(),sort:a,sortIndex:s,aggFunc:o,rowGroup:e.isRowGroupActive(),rowGroupIndex:r,pivot:e.isPivotActive(),pivotIndex:n,flex:l};return u},t.prototype.getColumnState=function(){if(me(this.primaryColumns)||!this.isAlive())return[];var e=this.getPrimaryAndSecondaryAndAutoColumns(),r=e.map(this.createStateItemFromColumn.bind(this));return this.orderColumnStateList(r),r},t.prototype.orderColumnStateList=function(e){var r=zo(this.gridColumns.map(function(n,o){return[n.getColId(),o]}));e.sort(function(n,o){var a=r.has(n.colId)?r.get(n.colId):-1,s=r.has(o.colId)?r.get(o.colId):-1;return a-s})},t.prototype.resetColumnState=function(e){e===void 0&&(e="api");var r=this.getColumnsFromTree(this.primaryColumnTree),n=[],o=1e3,a=1e3,s=[];this.groupAutoColumns&&(s=s.concat(this.groupAutoColumns)),r&&(s=s.concat(r)),s.forEach(function(l){var u=function(E,P){return E!=null?E:P!=null?P:null},c=l.getColDef(),p=u(c.sort,c.initialSort),h=u(c.sortIndex,c.initialSortIndex),d=u(c.hide,c.initialHide),f=u(c.pinned,c.initialPinned),g=u(c.width,c.initialWidth),v=u(c.flex,c.initialFlex),m=u(c.rowGroupIndex,c.initialRowGroupIndex),C=u(c.rowGroup,c.initialRowGroup);m==null&&(C==null||C==!1)&&(m=null,C=null);var w=u(c.pivotIndex,c.initialPivotIndex),b=u(c.pivot,c.initialPivot);w==null&&(b==null||b==!1)&&(w=null,b=null);var S=u(c.aggFunc,c.initialAggFunc),T={colId:l.getColId(),sort:p,sortIndex:h,hide:d,pinned:f,width:g,flex:v,rowGroup:C,rowGroupIndex:m,pivot:b,pivotIndex:w,aggFunc:S};me(m)&&C&&(T.rowGroupIndex=o++),me(w)&&b&&(T.pivotIndex=a++),n.push(T)}),this.applyColumnState({state:n,applyOrder:!0},e)},t.prototype.applyColumnState=function(e,r){var n=this;if(r===void 0&&(r="api"),pn(this.primaryColumns))return!1;if(e&&e.state&&!e.state.forEach)return console.warn("AG Grid: applyColumnState() - the state attribute should be an array, however an array was not found. Please provide an array of items (one for each col you want to change) for state."),!1;var o=function(u,c,p){var h=n.compareColumnStatesAndRaiseEvents(r);n.autoGroupsNeedBuilding=!0;var d=c.slice(),f={},g={},v=[],m=[],C=0,w=n.rowGroupColumns.slice(),b=n.pivotColumns.slice();u.forEach(function(P){var R=P.colId||"",M=R.startsWith(L.GROUP_AUTO_COLUMN_ID);if(M){v.push(P),m.push(P);return}var N=p(R);N?(n.syncColumnWithStateItem(N,P,e.defaultState,f,g,!1,r),gr(d,N)):(m.push(P),C+=1)});var S=function(P){return n.syncColumnWithStateItem(P,null,e.defaultState,f,g,!1,r)};d.forEach(S);var T=function(P,R,M,N){var W=P[M.getId()],G=P[N.getId()],k=W!=null,V=G!=null;if(k&&V)return W-G;if(k)return-1;if(V)return 1;var U=R.indexOf(M),J=R.indexOf(N),Z=U>=0,le=J>=0;return Z&&le?U-J:Z?-1:1};n.rowGroupColumns.sort(T.bind(n,f,w)),n.pivotColumns.sort(T.bind(n,g,b)),n.updateGridColumns();var E=n.groupAutoColumns?n.groupAutoColumns.slice():[];return v.forEach(function(P){var R=n.getAutoColumn(P.colId);gr(E,R),n.syncColumnWithStateItem(R,P,e.defaultState,null,null,!0,r)}),E.forEach(S),n.applyOrderAfterApplyState(e),n.updateDisplayedColumns(r),n.dispatchEverythingChanged(r),h(),{unmatchedAndAutoStates:m,unmatchedCount:C}};this.columnAnimationService.start();var a=o(e.state||[],this.primaryColumns||[],function(u){return n.getPrimaryColumn(u)}),s=a.unmatchedAndAutoStates,l=a.unmatchedCount;return(s.length>0||q(e.defaultState))&&(l=o(s,this.secondaryColumns||[],function(u){return n.getSecondaryColumn(u)}).unmatchedCount),this.columnAnimationService.finish(),l===0},t.prototype.applyOrderAfterApplyState=function(e){var r=this;if(!(!e.applyOrder||!e.state)){var n=[],o={};e.state.forEach(function(s){if(!(!s.colId||o[s.colId])){var l=r.gridColumnsMap[s.colId];l&&(n.push(l),o[s.colId]=!0)}});var a=0;if(this.gridColumns.forEach(function(s){var l=s.getColId(),u=o[l]!=null;if(!u){var c=l.startsWith(L.GROUP_AUTO_COLUMN_ID);c?Sc(n,s,a++):n.push(s)}}),n=this.placeLockedColumns(n),!this.doesMovePassMarryChildren(n)){console.warn("AG Grid: Applying column order broke a group where columns should be married together. Applying new order has been discarded.");return}this.gridColumns=n}},t.prototype.compareColumnStatesAndRaiseEvents=function(e){var r=this,n={rowGroupColumns:this.rowGroupColumns.slice(),pivotColumns:this.pivotColumns.slice(),valueColumns:this.valueColumns.slice()},o=this.getColumnState(),a={};return o.forEach(function(s){a[s.colId]=s}),function(){if(!r.gridOptionsWrapper.isSuppressColumnStateEvents()){var s=r.getPrimaryAndSecondaryAndAutoColumns(),l=function(m,C,w,b){var S=C.map(b),T=w.map(b),E=es(S,T);if(!E){var P={type:m,columns:w,column:w.length===1?w[0]:null,api:r.gridApi,columnApi:r.columnApi,source:e};r.eventService.dispatchEvent(P)}},u=function(m){var C=[];return s.forEach(function(w){var b=a[w.getColId()];b&&m(b,w)&&C.push(w)}),C},c=function(m){return m.getColId()};l(x.EVENT_COLUMN_ROW_GROUP_CHANGED,n.rowGroupColumns,r.rowGroupColumns,c),l(x.EVENT_COLUMN_PIVOT_CHANGED,n.pivotColumns,r.pivotColumns,c);var p=function(m,C){var w=m.aggFunc!=null,b=w!=C.isValueActive(),S=w&&m.aggFunc!=C.getAggFunc();return b||S},h=u(p);h.length>0&&r.fireColumnEvent(x.EVENT_COLUMN_VALUE_CHANGED,r.valueColumns,e);var d=function(m,C){return m.width!=C.getActualWidth()};r.fireColumnResizedEvent(u(d),!0,e);var f=function(m,C){return m.pinned!=C.getPinned()};r.raiseColumnPinnedEvent(u(f),e);var g=function(m,C){return m.hide==C.isVisible()};r.raiseColumnVisibleEvent(u(g),e);var v=function(m,C){return m.sort!=C.getSort()||m.sortIndex!=C.getSortIndex()};u(v).length>0&&r.sortController.dispatchSortChangedEvents(e),r.raiseColumnMovedEvent(o,e)}}},t.prototype.raiseColumnPinnedEvent=function(e,r){if(!!e.length){var n=e.length===1?e[0]:null,o=this.getCommonValue(e,function(s){return s.getPinned()}),a={type:x.EVENT_COLUMN_PINNED,pinned:o!=null?o:null,columns:e,column:n,api:this.gridApi,columnApi:this.columnApi,source:r};this.eventService.dispatchEvent(a)}},t.prototype.getCommonValue=function(e,r){if(!(!e||e.length==0)){for(var n=r(e[0]),o=1;o<e.length;o++)if(n!==r(e[o]))return;return n}},t.prototype.raiseColumnVisibleEvent=function(e,r){if(!!e.length){var n=e.length===1?e[0]:null,o=this.getCommonValue(e,function(s){return s.isVisible()}),a={type:x.EVENT_COLUMN_VISIBLE,visible:o,columns:e,column:n,api:this.gridApi,columnApi:this.columnApi,source:r};this.eventService.dispatchEvent(a)}},t.prototype.raiseColumnMovedEvent=function(e,r){var n=this,o=this.getColumnState(),a={};o.forEach(function(h){return a[h.colId]=h});var s={};e.forEach(function(h){a[h.colId]&&(s[h.colId]=!0)});var l=e.filter(function(h){return s[h.colId]}),u=o.filter(function(h){return s[h.colId]}),c=[];if(u.forEach(function(h,d){var f=l&&l[d];if(f&&f.colId!==h.colId){var g=n.getGridColumn(f.colId);g&&c.push(g)}}),!!c.length){var p={type:x.EVENT_COLUMN_MOVED,columns:c,column:null,api:this.gridApi,columnApi:this.columnApi,source:r};this.eventService.dispatchEvent(p)}},t.prototype.syncColumnWithStateItem=function(e,r,n,o,a,s,l){if(!!e){var u=function(R,M){var N={value1:void 0,value2:void 0},W=!1;return r&&(r[R]!==void 0&&(N.value1=r[R],W=!0),q(M)&&r[M]!==void 0&&(N.value2=r[M],W=!0)),!W&&n&&(n[R]!==void 0&&(N.value1=n[R]),q(M)&&n[M]!==void 0&&(N.value2=n[M])),N},c=u("hide").value1;c!==void 0&&e.setVisible(!c,l);var p=u("pinned").value1;p!==void 0&&e.setPinned(p);var h=this.columnUtils.calculateColMinWidth(e.getColDef()),d=u("flex").value1;d!==void 0&&e.setFlex(d);var f=e.getFlex()<=0;if(f){var g=u("width").value1;g!=null&&h!=null&&g>=h&&e.setActualWidth(g,l)}var v=u("sort").value1;v!==void 0&&(v===L.SORT_DESC||v===L.SORT_ASC?e.setSort(v,l):e.setSort(void 0,l));var m=u("sortIndex").value1;if(m!==void 0&&e.setSortIndex(m),!(s||!e.isPrimary())){var C=u("aggFunc").value1;C!==void 0&&(typeof C=="string"?(e.setAggFunc(C),e.isValueActive()||(e.setValueActive(!0,l),this.valueColumns.push(e))):(q(C)&&console.warn("AG Grid: stateItem.aggFunc must be a string. if using your own aggregation functions, register the functions first before using them in get/set state. This is because it is intended for the column state to be stored and retrieved as simple JSON."),e.isValueActive()&&(e.setValueActive(!1,l),gr(this.valueColumns,e))));var w=u("rowGroup","rowGroupIndex"),b=w.value1,S=w.value2;(b!==void 0||S!==void 0)&&(typeof S=="number"||b?(e.isRowGroupActive()||(e.setRowGroupActive(!0,l),this.rowGroupColumns.push(e)),o&&typeof S=="number"&&(o[e.getId()]=S)):e.isRowGroupActive()&&(e.setRowGroupActive(!1,l),gr(this.rowGroupColumns,e)));var T=u("pivot","pivotIndex"),E=T.value1,P=T.value2;(E!==void 0||P!==void 0)&&(typeof P=="number"||E?(e.isPivotActive()||(e.setPivotActive(!0,l),this.pivotColumns.push(e)),a&&typeof P=="number"&&(a[e.getId()]=P)):e.isPivotActive()&&(e.setPivotActive(!1,l),gr(this.pivotColumns,e)))}}},t.prototype.getGridColumns=function(e){return this.getColumns(e,this.getGridColumn.bind(this))},t.prototype.getColumns=function(e,r){var n=[];return e&&e.forEach(function(o){var a=r(o);a&&n.push(a)}),n},t.prototype.getColumnWithValidation=function(e){if(e==null)return null;var r=this.getGridColumn(e);return r||console.warn("AG Grid: could not find column "+e),r},t.prototype.getPrimaryColumn=function(e){return this.primaryColumns?this.getColumn(e,this.primaryColumns,this.primaryColumnsMap):null},t.prototype.getGridColumn=function(e){return this.getColumn(e,this.gridColumns,this.gridColumnsMap)},t.prototype.getSecondaryColumn=function(e){return this.secondaryColumns?this.getColumn(e,this.secondaryColumns,this.secondaryColumnsMap):null},t.prototype.getColumn=function(e,r,n){if(!e)return null;if(typeof e=="string"&&n[e])return n[e];for(var o=0;o<r.length;o++)if(this.columnsMatch(r[o],e))return r[o];return this.getAutoColumn(e)},t.prototype.getAutoColumn=function(e){var r=this;return!this.groupAutoColumns||!q(this.groupAutoColumns)||me(this.groupAutoColumns)?null:this.groupAutoColumns.find(function(n){return r.columnsMatch(n,e)})||null},t.prototype.columnsMatch=function(e,r){var n=e===r,o=e.getColDef()===r,a=e.getColId()==r;return n||o||a},t.prototype.getDisplayNameForColumn=function(e,r,n){if(n===void 0&&(n=!1),!e)return null;var o=this.getHeaderName(e.getColDef(),e,null,null,r);return n?this.wrapHeaderNameWithAggFunc(e,o):o},t.prototype.getDisplayNameForProvidedColumnGroup=function(e,r,n){var o=r?r.getColGroupDef():null;return o?this.getHeaderName(o,null,e,r,n):null},t.prototype.getDisplayNameForColumnGroup=function(e,r){return this.getDisplayNameForProvidedColumnGroup(e,e.getProvidedColumnGroup(),r)},t.prototype.getHeaderName=function(e,r,n,o,a){var s=e.headerValueGetter;if(s){var l={colDef:e,column:r,columnGroup:n,providedColumnGroup:o,location:a,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()};return typeof s=="function"?s(l):typeof s=="string"?this.expressionService.evaluate(s,l):(console.warn("ag-grid: headerValueGetter must be a function or a string"),"")}else{if(e.headerName!=null)return e.headerName;if(e.field)return EC(e.field)}return""},t.prototype.wrapHeaderNameWithAggFunc=function(e,r){if(this.gridOptionsWrapper.isSuppressAggFuncInHeader())return r;var n=e.getColDef().pivotValueColumn,o=q(n),a=null,s;if(o){var l=this.gridOptionsWrapper.isRemovePivotHeaderRowWhenSingleValueColumn()&&this.valueColumns.length===1,u=e.getColDef().pivotTotalColumnIds!==void 0;if(l&&!u)return r;a=n?n.getAggFunc():null,s=!0}else{var c=e.isValueActive(),p=this.pivotMode||!this.isRowGroupEmpty();c&&p?(a=e.getAggFunc(),s=!0):s=!1}if(s){var h=typeof a=="string"?a:"func",d=this.gridOptionsWrapper.getLocaleTextFunc(),f=d(h,h);return f+"("+r+")"}return r},t.prototype.getColumnGroup=function(e,r){if(!e)return null;if(e instanceof sr)return e;var n=this.getAllDisplayedTrees(),o=typeof r=="number",a=null;return this.columnUtils.depthFirstAllColumnTreeSearch(n,function(s){if(s instanceof sr){var l=s,u=void 0;o?u=e===l.getGroupId()&&r===l.getInstanceId():u=e===l.getGroupId(),u&&(a=l)}}),a},t.prototype.isReady=function(){return this.ready},t.prototype.extractValueColumns=function(e,r){this.valueColumns=this.extractColumns(r,this.valueColumns,function(n,o){return n.setValueActive(o,e)},function(){},function(){},function(n){var o=n.aggFunc;if(o===null||o==="")return null;if(o!==void 0)return!!o},function(n){return n.initialAggFunc!=null&&n.initialAggFunc!=""}),this.valueColumns.forEach(function(n){var o=n.getColDef();o.aggFunc!=null&&o.aggFunc!=""?n.setAggFunc(o.aggFunc):n.getAggFunc()||n.setAggFunc(o.initialAggFunc)})},t.prototype.extractRowGroupColumns=function(e,r){this.rowGroupColumns=this.extractColumns(r,this.rowGroupColumns,function(n,o){return n.setRowGroupActive(o,e)},function(n){return n.rowGroupIndex},function(n){return n.initialRowGroupIndex},function(n){return n.rowGroup},function(n){return n.initialRowGroup})},t.prototype.extractColumns=function(e,r,n,o,a,s,l){e===void 0&&(e=[]),r===void 0&&(r=[]);var u=[],c=[];(this.primaryColumns||[]).forEach(function(d){var f=e.indexOf(d)<0,g=d.getColDef(),v=$i(s(g)),m=$i(l(g)),C=In(o(g)),w=In(a(g)),b,S=v!==void 0,T=C!==void 0,E=m!==void 0,P=w!==void 0;if(S?b=v:T?C===null?b=!1:b=C>=0:f?E?b=m:P?b=w!=null&&w>=0:b=!1:b=r.indexOf(d)>=0,b){var R=f?C!=null||w!=null:C!=null;R?u.push(d):c.push(d)}});var p=function(d){var f=o(d.getColDef()),g=a(d.getColDef());return f!=null?f:g};u.sort(function(d,f){var g=p(d),v=p(f);return g===v?0:g<v?-1:1});var h=[].concat(u);return r.forEach(function(d){c.indexOf(d)>=0&&h.push(d)}),c.forEach(function(d){h.indexOf(d)<0&&h.push(d)}),r.forEach(function(d){h.indexOf(d)<0&&n(d,!1)}),h.forEach(function(d){r.indexOf(d)<0&&n(d,!0)}),h},t.prototype.extractPivotColumns=function(e,r){this.pivotColumns=this.extractColumns(r,this.pivotColumns,function(n,o){return n.setPivotActive(o,e)},function(n){return n.pivotIndex},function(n){return n.initialPivotIndex},function(n){return n.pivot},function(n){return n.initialPivot})},t.prototype.resetColumnGroupState=function(e){e===void 0&&(e="api");var r=[];this.columnUtils.depthFirstOriginalTreeSearch(null,this.primaryColumnTree,function(n){if(n instanceof $e){var o=n.getColGroupDef(),a={groupId:n.getGroupId(),open:o?o.openByDefault:void 0};r.push(a)}}),this.setColumnGroupState(r,e)},t.prototype.getColumnGroupState=function(){var e=[];return this.columnUtils.depthFirstOriginalTreeSearch(null,this.gridBalancedTree,function(r){r instanceof $e&&e.push({groupId:r.getGroupId(),open:r.isExpanded()})}),e},t.prototype.setColumnGroupState=function(e,r){var n=this;r===void 0&&(r="api"),this.columnAnimationService.start();var o=[];e.forEach(function(a){var s=a.groupId,l=a.open,u=n.getProvidedColumnGroup(s);!u||u.isExpanded()!==l&&(n.logger.log("columnGroupOpened("+u.getGroupId()+","+l+")"),u.setExpanded(l),o.push(u))}),this.updateGroupsAndDisplayedColumns(r),this.setFirstRightAndLastLeftPinned(r),o.forEach(function(a){var s={type:x.EVENT_COLUMN_GROUP_OPENED,columnGroup:a,api:n.gridApi,columnApi:n.columnApi};n.eventService.dispatchEvent(s)}),this.columnAnimationService.finish()},t.prototype.setColumnGroupOpened=function(e,r,n){n===void 0&&(n="api");var o;e instanceof $e?o=e.getId():o=e||"",this.setColumnGroupState([{groupId:o,open:r}],n)},t.prototype.getProvidedColumnGroup=function(e){typeof e!="string"&&console.error("AG Grid: group key must be a string");var r=null;return this.columnUtils.depthFirstOriginalTreeSearch(null,this.gridBalancedTree,function(n){n instanceof $e&&n.getId()===e&&(r=n)}),r},t.prototype.calculateColumnsForDisplay=function(){var e=this,r;return this.pivotMode&&me(this.secondaryColumns)?r=this.gridColumns.filter(function(n){var o=e.groupAutoColumns&&fn(e.groupAutoColumns,n),a=e.valueColumns&&fn(e.valueColumns,n);return o||a}):r=this.gridColumns.filter(function(n){var o=e.groupAutoColumns&&fn(e.groupAutoColumns,n);return o||n.isVisible()}),r},t.prototype.checkColSpanActiveInCols=function(e){var r=!1;return e.forEach(function(n){q(n.getColDef().colSpan)&&(r=!0)}),r},t.prototype.calculateColumnsForGroupDisplay=function(){var e=this;this.groupDisplayColumns=[];var r=function(n){var o=n.getColDef();o&&q(o.showRowGroup)&&e.groupDisplayColumns.push(n)};this.gridColumns.forEach(r),this.groupAutoColumns&&this.groupAutoColumns.forEach(r)},t.prototype.getGroupDisplayColumns=function(){return this.groupDisplayColumns},t.prototype.updateDisplayedColumns=function(e){var r=this.calculateColumnsForDisplay();this.buildDisplayedTrees(r),this.calculateColumnsForGroupDisplay(),this.updateGroupsAndDisplayedColumns(e),this.setFirstRightAndLastLeftPinned(e)},t.prototype.isSecondaryColumnsPresent=function(){return q(this.secondaryColumns)},t.prototype.setSecondaryColumns=function(e,r){var n=this;r===void 0&&(r="api");var o=e&&e.length>0;if(!(!o&&me(this.secondaryColumns))){if(o){this.processSecondaryColumnDefinitions(e);var a=this.columnFactory.createColumnTree(e,!1,this.secondaryBalancedTree||this.previousSecondaryColumns||void 0);this.secondaryBalancedTree=a.columnTree,this.secondaryHeaderRowCount=a.treeDept+1,this.secondaryColumns=this.getColumnsFromTree(this.secondaryBalancedTree),this.secondaryColumnsMap={},this.secondaryColumns.forEach(function(s){return n.secondaryColumnsMap[s.getId()]=s}),this.previousSecondaryColumns=null}else this.previousSecondaryColumns=this.secondaryBalancedTree,this.secondaryBalancedTree=null,this.secondaryHeaderRowCount=-1,this.secondaryColumns=null,this.secondaryColumnsMap={};this.updateGridColumns(),this.updateDisplayedColumns(r)}},t.prototype.processSecondaryColumnDefinitions=function(e){var r=this.gridOptionsWrapper.getProcessSecondaryColDefFunc(),n=this.gridOptionsWrapper.getProcessSecondaryColGroupDefFunc();if(!(!r&&!n)){var o=function(a){a.forEach(function(s){var l=q(s.children);if(l){var u=s;n&&n(u),o(u.children)}else{var c=s;r&&r(c)}})};e&&o(e)}},t.prototype.updateGridColumns=function(){var e=this;if(this.gridColsArePrimary?this.lastPrimaryOrder=this.gridColumns:this.lastSecondaryOrder=this.gridColumns,this.secondaryColumns&&this.secondaryBalancedTree){var r=this.secondaryColumns.every(function(o){return e.gridColumnsMap[o.getColId()]!==void 0});this.gridBalancedTree=this.secondaryBalancedTree.slice(),this.gridHeaderRowCount=this.secondaryHeaderRowCount,this.gridColumns=this.secondaryColumns.slice(),this.gridColsArePrimary=!1,r&&this.orderGridColsLike(this.lastSecondaryOrder)}else this.primaryColumns&&(this.gridBalancedTree=this.primaryColumnTree.slice(),this.gridHeaderRowCount=this.primaryHeaderRowCount,this.gridColumns=this.primaryColumns.slice(),this.gridColsArePrimary=!0,this.orderGridColsLike(this.lastPrimaryOrder));this.addAutoGroupToGridColumns(),this.gridColumns=this.placeLockedColumns(this.gridColumns),this.setupQuickFilterColumns(),this.clearDisplayedAndViewportColumns(),this.colSpanActive=this.checkColSpanActiveInCols(this.gridColumns),this.gridColumnsMap={},this.gridColumns.forEach(function(o){return e.gridColumnsMap[o.getId()]=o}),this.setAutoHeightActive();var n={type:x.EVENT_GRID_COLUMNS_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(n)},t.prototype.setAutoHeightActive=function(){if(this.autoHeightActive=this.gridColumns.filter(function(o){return o.isAutoHeight()}).length>0,this.autoHeightActive){this.autoHeightActiveAtLeastOnce=!0;var e=this.rowModel.getType(),r=e===L.ROW_MODEL_TYPE_CLIENT_SIDE||e===L.ROW_MODEL_TYPE_SERVER_SIDE;if(!r){var n="AG Grid - autoHeight columns only work with Client Side Row Model and Server Side Row Model.";Ye(function(){return console.warn(n)},"autoHeightActive.wrongRowModel")}}},t.prototype.orderGridColsLike=function(e){if(!me(e)){var r=zo(e.map(function(c,p){return[c,p]})),n=!0;if(this.gridColumns.forEach(function(c){r.has(c)&&(n=!1)}),!n){var o=zo(this.gridColumns.map(function(c){return[c,!0]})),a=e.filter(function(c){return o.has(c)}),s=zo(a.map(function(c){return[c,!0]})),l=this.gridColumns.filter(function(c){return!s.has(c)}),u=a.slice();l.forEach(function(c){var p=c.getOriginalParent();if(!p){u.push(c);return}for(var h=[];!h.length&&p;){var d=p.getLeafColumns();d.forEach(function(v){var m=u.indexOf(v)>=0,C=h.indexOf(v)<0;m&&C&&h.push(v)}),p=p.getOriginalParent()}if(!h.length){u.push(c);return}var f=h.map(function(v){return u.indexOf(v)}),g=Math.max.apply(Math,iu(f));Sc(u,c,g+1)}),this.gridColumns=u}}},t.prototype.isPrimaryColumnGroupsPresent=function(){return this.primaryHeaderRowCount>1},t.prototype.setupQuickFilterColumns=function(){this.groupAutoColumns?this.columnsForQuickFilter=(this.primaryColumns||[]).concat(this.groupAutoColumns):this.primaryColumns&&(this.columnsForQuickFilter=this.primaryColumns)},t.prototype.placeLockedColumns=function(e){var r=[],n=[],o=[];return e.forEach(function(a){var s=a.getColDef().lockPosition;s==="right"?o.push(a):s==="left"||s===!0?r.push(a):n.push(a)}),iu(r,n,o)},t.prototype.addAutoGroupToGridColumns=function(){if(this.createGroupAutoColumnsIfNeeded(),!me(this.groupAutoColumns)){this.gridColumns=this.groupAutoColumns?this.groupAutoColumns.concat(this.gridColumns):this.gridColumns;var e=this.columnFactory.createForAutoGroups(this.groupAutoColumns,this.gridBalancedTree);this.gridBalancedTree=e.concat(this.gridBalancedTree)}},t.prototype.clearDisplayedAndViewportColumns=function(){this.displayedTreeLeft=[],this.displayedTreeRight=[],this.displayedTreeCentre=[],this.viewportRowLeft={},this.viewportRowRight={},this.viewportRowCenter={},this.displayedColumnsLeft=[],this.displayedColumnsRight=[],this.displayedColumnsCenter=[],this.displayedColumns=[],this.viewportColumns=[]},t.prototype.updateGroupsAndDisplayedColumns=function(e){this.updateOpenClosedVisibilityInColumnGroups(),this.deriveDisplayedColumns(e),this.refreshFlexedColumns(),this.extractViewport(),this.updateBodyWidths();var r={type:x.EVENT_DISPLAYED_COLUMNS_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(r)},t.prototype.deriveDisplayedColumns=function(e){this.derivedDisplayedColumnsFromDisplayedTree(this.displayedTreeLeft,this.displayedColumnsLeft),this.derivedDisplayedColumnsFromDisplayedTree(this.displayedTreeCentre,this.displayedColumnsCenter),this.derivedDisplayedColumnsFromDisplayedTree(this.displayedTreeRight,this.displayedColumnsRight),this.joinDisplayedColumns(),this.setLeftValues(e),this.displayedAutoHeightCols=this.displayedColumns.filter(function(r){return r.isAutoHeight()})},t.prototype.isAutoRowHeightActive=function(){return this.autoHeightActive},t.prototype.wasAutoRowHeightEverActive=function(){return this.autoHeightActiveAtLeastOnce},t.prototype.joinDisplayedColumns=function(){this.gridOptionsWrapper.isEnableRtl()?this.displayedColumns=this.displayedColumnsRight.concat(this.displayedColumnsCenter).concat(this.displayedColumnsLeft):this.displayedColumns=this.displayedColumnsLeft.concat(this.displayedColumnsCenter).concat(this.displayedColumnsRight)},t.prototype.setLeftValues=function(e){this.setLeftValuesOfColumns(e),this.setLeftValuesOfGroups()},t.prototype.setLeftValuesOfColumns=function(e){var r=this;if(!!this.primaryColumns){var n=this.primaryColumns.slice(0),o=this.gridOptionsWrapper.isEnableRtl();[this.displayedColumnsLeft,this.displayedColumnsRight,this.displayedColumnsCenter].forEach(function(a){if(o){var s=r.getWidthOfColsInList(a);a.forEach(function(u){s-=u.getActualWidth(),u.setLeft(s,e)})}else{var l=0;a.forEach(function(u){u.setLeft(l,e),l+=u.getActualWidth()})}SC(n,a)}),n.forEach(function(a){a.setLeft(null,e)})}},t.prototype.setLeftValuesOfGroups=function(){[this.displayedTreeLeft,this.displayedTreeRight,this.displayedTreeCentre].forEach(function(e){e.forEach(function(r){if(r instanceof sr){var n=r;n.checkLeft()}})})},t.prototype.derivedDisplayedColumnsFromDisplayedTree=function(e,r){r.length=0,this.columnUtils.depthFirstDisplayedColumnTreeSearch(e,function(n){n instanceof Se&&r.push(n)})},t.prototype.extractViewportColumns=function(){this.suppressColumnVirtualisation?this.viewportColumnsCenter=this.displayedColumnsCenter:this.viewportColumnsCenter=this.filterOutColumnsWithinViewport(),this.viewportColumns=this.viewportColumnsCenter.concat(this.displayedColumnsLeft).concat(this.displayedColumnsRight)},t.prototype.getVirtualHeaderGroupRow=function(e,r){var n;switch(e){case L.PINNED_LEFT:n=this.viewportRowLeft[r];break;case L.PINNED_RIGHT:n=this.viewportRowRight[r];break;default:n=this.viewportRowCenter[r];break}return me(n)&&(n=[]),n},t.prototype.extractViewportRows=function(){this.viewportRowLeft={},this.viewportRowRight={},this.viewportRowCenter={};var e={};this.viewportColumns.forEach(function(n){return e[n.getId()]=!0});var r=function(n,o,a){for(var s=!1,l=0;l<n.length;l++){var u=n[l],c=!1;if(u instanceof Se)c=e[u.getId()]===!0;else{var p=u,h=p.getDisplayedChildren();h&&(c=r(h,o,a+1))}c&&(s=!0,o[a]||(o[a]=[]),o[a].push(u))}return s};r(this.displayedTreeLeft,this.viewportRowLeft,0),r(this.displayedTreeRight,this.viewportRowRight,0),r(this.displayedTreeCentre,this.viewportRowCenter,0)},t.prototype.extractViewport=function(){this.extractViewportColumns(),this.extractViewportRows()},t.prototype.filterOutColumnsWithinViewport=function(){return this.displayedColumnsCenter.filter(this.isColumnInViewport.bind(this))},t.prototype.refreshFlexedColumns=function(e){var r=this;e===void 0&&(e={});var n=e.source?e.source:"flex";if(e.viewportWidth!=null&&(this.flexViewportWidth=e.viewportWidth),!this.flexViewportWidth)return[];var o=-1;e.resizingCols&&e.resizingCols.forEach(function(b){var S=r.displayedColumnsCenter.indexOf(b);o<S&&(o=S)});var a=function(b){var S=r.displayedColumnsCenter.indexOf(b)>o;return b.getFlex()&&S},s=this.displayedColumnsCenter.filter(function(b){return!a(b)}),l=this.displayedColumnsCenter.filter(function(b){return a(b)}),u=[];if(!l.length)return[];var c=[],p;e:for(;;){var h=l.reduce(function(b,S){return b+S.getFlex()},0);p=this.flexViewportWidth-this.getWidthOfColsInList(s);for(var d=0;d<l.length;d++){var f=l[d],g=p*f.getFlex()/h,v=0,m=f.getMinWidth(),C=f.getMaxWidth();if(q(m)&&g<m?v=m:q(C)&&g>C&&(v=C),v){f.setActualWidth(v,n),gr(l,f),u.push(f),s.push(f);continue e}c[d]=Math.round(g)}break}var w=p;return l.forEach(function(b,S){b.setActualWidth(Math.min(c[S],w),n),u.push(b),w-=c[S]}),e.skipSetLeft||this.setLeftValues(n),e.updateBodyWidths&&this.updateBodyWidths(),e.fireResizedEvent&&this.fireColumnResizedEvent(u,!0,n,l),this.flexColsCalculatedAtLestOnce||(this.gridOptionsWrapper.isRowModelDefault()&&this.rowModel.resetRowHeights(),this.flexColsCalculatedAtLestOnce=!0),l},t.prototype.sizeColumnsToFit=function(e,r,n){r===void 0&&(r="sizeColumnsToFit");var o=this.getAllDisplayedColumns();if(!(e<=0||!o.length)){var a=[],s=[];o.forEach(function(w){w.getColDef().suppressSizeToFit===!0?s.push(w):a.push(w)});var l=a.slice(0),u=!1,c=function(w){gr(a,w),s.push(w)};for(a.forEach(function(w){return w.resetActualWidth(r)});!u;){u=!0;var p=e-this.getWidthOfColsInList(s);if(p<=0)a.forEach(function(w){w.setMinimum(r)});else for(var h=p/this.getWidthOfColsInList(a),d=p,f=a.length-1;f>=0;f--){var g=a[f],v=g.getMinWidth(),m=g.getMaxWidth(),C=Math.round(g.getActualWidth()*h);q(v)&&C<v?(C=v,c(g),u=!1):q(m)&&g.isGreaterThanMax(C)?(C=m,c(g),u=!1):f===0&&(C=d),g.setActualWidth(C,r,!0),d-=C}}l.forEach(function(w){w.fireColumnWidthChangedEvent(r)}),this.setLeftValues(r),this.updateBodyWidths(),!n&&this.fireColumnResizedEvent(l,!0,r)}},t.prototype.buildDisplayedTrees=function(e){var r=[],n=[],o=[];e.forEach(function(s){switch(s.getPinned()){case"left":r.push(s);break;case"right":n.push(s);break;default:o.push(s);break}});var a=new OC;this.displayedTreeLeft=this.displayedGroupCreator.createDisplayedGroups(r,this.gridBalancedTree,a,L.PINNED_LEFT,this.displayedTreeLeft),this.displayedTreeRight=this.displayedGroupCreator.createDisplayedGroups(n,this.gridBalancedTree,a,L.PINNED_RIGHT,this.displayedTreeRight),this.displayedTreeCentre=this.displayedGroupCreator.createDisplayedGroups(o,this.gridBalancedTree,a,null,this.displayedTreeCentre),this.updateDisplayedMap()},t.prototype.updateDisplayedMap=function(){var e=this;this.displayedColumnsAndGroupsMap={};var r=function(n){e.displayedColumnsAndGroupsMap[n.getUniqueId()]=n};this.columnUtils.depthFirstAllColumnTreeSearch(this.displayedTreeCentre,r),this.columnUtils.depthFirstAllColumnTreeSearch(this.displayedTreeLeft,r),this.columnUtils.depthFirstAllColumnTreeSearch(this.displayedTreeRight,r)},t.prototype.isDisplayed=function(e){var r=this.displayedColumnsAndGroupsMap[e.getUniqueId()];return r===e},t.prototype.updateOpenClosedVisibilityInColumnGroups=function(){var e=this.getAllDisplayedTrees();this.columnUtils.depthFirstAllColumnTreeSearch(e,function(r){if(r instanceof sr){var n=r;n.calculateDisplayedColumns()}})},t.prototype.getGroupAutoColumns=function(){return this.groupAutoColumns},t.prototype.createGroupAutoColumnsIfNeeded=function(){if(!!this.autoGroupsNeedBuilding){this.autoGroupsNeedBuilding=!1;var e=this.gridOptionsWrapper.isGroupUseEntireRow(this.pivotMode),r=this.pivotMode?this.gridOptionsWrapper.isPivotSuppressAutoColumn():this.gridOptionsWrapper.isGroupSuppressAutoColumn(),n=this.rowGroupColumns.length>0||this.usingTreeData,o=n&&!r&&!e;if(o){var a=this.groupAutoColumns||[],s=this.autoGroupColService.createAutoGroupColumns(a,this.rowGroupColumns),l=!this.autoColsEqual(s,this.groupAutoColumns);(l||this.forceRecreateAutoGroups)&&(this.groupAutoColumns=s)}else this.groupAutoColumns=null}},t.prototype.autoColsEqual=function(e,r){return es(e,r,function(n,o){return n.getColId()===o.getColId()})},t.prototype.getWidthOfColsInList=function(e){return e.reduce(function(r,n){return r+n.getActualWidth()},0)},t.prototype.getGridBalancedTree=function(){return this.gridBalancedTree},t.prototype.hasFloatingFilters=function(){if(!this.gridColumns)return!1;var e=this.gridColumns.some(function(r){return r.getColDef().floatingFilter});return e},t.prototype.getFirstDisplayedColumn=function(){var e=this.gridOptionsWrapper.isEnableRtl(),r=["getDisplayedLeftColumns","getDisplayedCenterColumns","getDisplayedRightColumns"];e&&r.reverse();for(var n=0;n<r.length;n++){var o=this[r[n]]();if(o.length)return e?rt(o):o[0]}return null},kt([y("expressionService")],t.prototype,"expressionService",void 0),kt([y("columnFactory")],t.prototype,"columnFactory",void 0),kt([y("displayedGroupCreator")],t.prototype,"displayedGroupCreator",void 0),kt([y("ctrlsService")],t.prototype,"ctrlsService",void 0),kt([y("autoWidthCalculator")],t.prototype,"autoWidthCalculator",void 0),kt([y("columnUtils")],t.prototype,"columnUtils",void 0),kt([y("columnAnimationService")],t.prototype,"columnAnimationService",void 0),kt([y("autoGroupColService")],t.prototype,"autoGroupColService",void 0),kt([De("aggFuncService")],t.prototype,"aggFuncService",void 0),kt([De("valueCache")],t.prototype,"valueCache",void 0),kt([De("animationFrameService")],t.prototype,"animationFrameService",void 0),kt([y("rowModel")],t.prototype,"rowModel",void 0),kt([y("columnApi")],t.prototype,"columnApi",void 0),kt([y("gridApi")],t.prototype,"gridApi",void 0),kt([y("sortController")],t.prototype,"sortController",void 0),kt([y("columnDefFactory")],t.prototype,"columnDefFactory",void 0),kt([B],t.prototype,"init",null),kt([QF(0,Yt("loggerFactory"))],t.prototype,"setBeans",null),t=kt([Q("columnModel")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var eI=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),tI=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},bO=function(i){eI(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.calculateColMinWidth=function(e){return e.minWidth!=null?e.minWidth:this.gridOptionsWrapper.getMinColWidth()},t.prototype.calculateColMaxWidth=function(e){return e.maxWidth!=null?e.maxWidth:this.gridOptionsWrapper.getMaxColWidth()||Number.MAX_SAFE_INTEGER},t.prototype.calculateColInitialWidth=function(e){var r=this.calculateColMinWidth(e),n=this.calculateColMaxWidth(e),o,a=In(e.width),s=In(e.initialWidth);return a!=null?o=a:s!=null?o=s:o=this.gridOptionsWrapper.getColWidth(),Math.max(Math.min(o,n),r)},t.prototype.getOriginalPathForColumn=function(e,r){var n=[],o=!1,a=function(s,l){for(var u=0;u<s.length;u++){if(o)return;var c=s[u];if(c instanceof $e){var p=c;a(p.getChildren(),l+1),n[l]=c}else c===e&&(o=!0)}};return a(r,0),o?n:null},t.prototype.depthFirstOriginalTreeSearch=function(e,r,n){var o=this;!r||r.forEach(function(a){a instanceof $e&&o.depthFirstOriginalTreeSearch(a,a.getChildren(),n),n(a,e)})},t.prototype.depthFirstAllColumnTreeSearch=function(e,r){var n=this;!e||e.forEach(function(o){o instanceof sr&&n.depthFirstAllColumnTreeSearch(o.getChildren(),r),r(o)})},t.prototype.depthFirstDisplayedColumnTreeSearch=function(e,r){var n=this;!e||e.forEach(function(o){o instanceof sr&&n.depthFirstDisplayedColumnTreeSearch(o.getDisplayedChildren(),r),r(o)})},t=tI([Q("columnUtils")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var rI=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),nI=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},_O=function(i){rI(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.createDisplayedGroups=function(e,r,n,o,a){var s=this,l=[],u,c,p=this.mapOldGroupsById(a);return e.forEach(function(h){for(var d=s.getOriginalPathForColumn(r,h),f=[],g=!c,v=0;v<d.length;v++)if(g||d[v]!==c[v]){var m=s.createColumnGroup(d[v],n,p,o);f[v]=m,v==0?l.push(m):f[v-1].addChild(m)}else f[v]=u[v];var C=f.length===0;if(C)l.push(h);else{var w=rt(f);w.addChild(h)}u=f,c=d}),this.setupParentsIntoColumns(l,null),l},t.prototype.createColumnGroup=function(e,r,n,o){var a=e.getGroupId(),s=r.getInstanceIdForKey(a),l=sr.createUniqueId(a,s),u=n[l];return u&&u.getProvidedColumnGroup()!==e&&(u=null),q(u)?u.reset():(u=new sr(e,a,s,o),this.context.createBean(u)),u},t.prototype.mapOldGroupsById=function(e){var r={},n=function(o){o.forEach(function(a){if(a instanceof sr){var s=a;r[a.getUniqueId()]=s,n(s.getChildren())}})};return e&&n(e),r},t.prototype.setupParentsIntoColumns=function(e,r){var n=this;e.forEach(function(o){if(o.setParent(r),o instanceof sr){var a=o;n.setupParentsIntoColumns(a.getChildren(),a)}})},t.prototype.getOriginalPathForColumn=function(e,r){var n=[],o=!1,a=function(s,l){for(var u=0;u<s.length;u++){if(o)return;var c=s[u];c instanceof $e?(a(c.getChildren(),l+1),n[l]=c):c===r&&(o=!0)}};return a(e,0),o?n:(console.warn("AG Grid: could not get path"),null)},t=nI([Q("displayedGroupCreator")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var iI=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},oI=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(iI(arguments[t]));return i},bi=function(){function i(){}return i.STRING_PROPERTIES=["sortingOrder","rowClass","rowSelection","overlayLoadingTemplate","overlayNoRowsTemplate","quickFilterText","rowModelType","editType","domLayout","clipboardDelimiter","rowGroupPanelShow","multiSortKey","pivotColumnGroupTotals","pivotRowTotals","pivotPanelShow","fillHandleDirection","serverSideStoreType","groupDisplayType","treeDataDisplayType"],i.OBJECT_PROPERTIES=["components","frameworkComponents","rowStyle","context","autoGroupColumnDef","localeText","icons","datasource","serverSideDatasource","viewportDatasource","groupRowRendererParams","aggFuncs","fullWidthCellRendererParams","defaultColGroupDef","defaultColDef","defaultExportParams","defaultCsvExportParams","defaultExcelExportParams","columnTypes","rowClassRules","detailCellRendererParams","loadingCellRendererParams","loadingOverlayComponentParams","noRowsOverlayComponentParams","popupParent","colResizeDefault","statusBar","sideBar","chartThemeOverrides","customChartThemes"],i.ARRAY_PROPERTIES=["alignedGrids","rowData","columnDefs","excelStyles","pinnedTopRowData","pinnedBottomRowData","chartThemes"],i.NUMBER_PROPERTIES=["rowHeight","detailRowHeight","rowBuffer","colWidth","headerHeight","groupHeaderHeight","floatingFiltersHeight","pivotHeaderHeight","pivotGroupHeaderHeight","groupDefaultExpanded","minColWidth","maxColWidth","viewportRowModelPageSize","viewportRowModelBufferSize","autoSizePadding","maxBlocksInCache","maxConcurrentDatasourceRequests","tooltipShowDelay","tooltipHideDelay","cacheOverflowSize","paginationPageSize","cacheBlockSize","infiniteInitialRowCount","scrollbarWidth","batchUpdateWaitMillis","asyncTransactionWaitMillis","blockLoadDebounceMillis","keepDetailRowsCount","undoRedoCellEditingLimit","cellFlashDelay","cellFadeDelay","tabIndex"],i.BOOLEAN_PROPERTIES=["suppressMakeColumnVisibleAfterUnGroup","suppressRowClickSelection","suppressCellSelection","suppressCellFocus","suppressHorizontalScroll","alwaysShowHorizontalScroll","alwaysShowVerticalScroll","debug","enableBrowserTooltips","enableCellExpressions","angularCompileRows","angularCompileFilters","groupSuppressAutoColumn","groupSelectsChildren","groupIncludeFooter","groupIncludeTotalFooter","groupUseEntireRow","groupSuppressBlankHeader","suppressMenuHide","suppressRowDeselection","unSortIcon","suppressMultiSort","alwaysMultiSort","singleClickEdit","suppressLoadingOverlay","suppressNoRowsOverlay","suppressAutoSize","skipHeaderOnAutoSize","suppressParentsInRowNodes","suppressColumnMoveAnimation","suppressMovableColumns","suppressFieldDotNotation","enableRangeSelection","enableRangeHandle","enableFillHandle","suppressClearOnFillReduction","deltaSort","suppressTouch","suppressAsyncEvents","allowContextMenuWithControlKey","suppressContextMenu","rememberGroupStateWhenNewData","enableCellChangeFlash","suppressDragLeaveHidesColumns","suppressMiddleClickScrolls","suppressPreventDefaultOnMouseWheel","suppressCopyRowsToClipboard","copyHeadersToClipboard","copyGroupHeadersToClipboard","pivotMode","suppressAggFuncInHeader","suppressColumnVirtualisation","suppressAggAtRootLevel","suppressFocusAfterRefresh","functionsPassive","functionsReadOnly","animateRows","groupSelectsFiltered","groupRemoveSingleChildren","groupRemoveLowestSingleChildren","enableRtl","suppressClickEdit","rowDragEntireRow","rowDragManaged","suppressRowDrag","suppressMoveWhenRowDragging","rowDragMultiRow","enableGroupEdit","embedFullWidthRows","deprecatedEmbedFullWidthRows","suppressPaginationPanel","groupHideOpenParents","groupMultiAutoColumn","pagination","stopEditingWhenGridLosesFocus","paginationAutoPageSize","suppressScrollOnNewData","suppressScrollWhenPopupsAreOpen","purgeClosedRowNodes","cacheQuickFilter","deltaRowDataMode","ensureDomOrder","accentedSort","suppressChangeDetection","valueCache","valueCacheNeverExpires","aggregateOnlyChangedColumns","suppressAnimationFrame","suppressExcelExport","suppressCsvExport","treeData","masterDetail","suppressMultiRangeSelection","enterMovesDownAfterEdit","enterMovesDown","suppressPropertyNamesCheck","rowMultiSelectWithClick","suppressEnterpriseResetOnNewColumns","suppressRowHoverHighlight","suppressRowTransform","suppressClipboardPaste","suppressLastEmptyLineOnPaste","serverSideSortingAlwaysResets","suppressSetColumnStateEvents","suppressColumnStateEvents","enableCharts","deltaColumnMode","suppressMaintainUnsortedOrder","enableCellTextSelection","suppressBrowserResizeObserver","suppressMaxRenderedRowRestriction","excludeChildrenWhenTreeDataFiltering","tooltipMouseTrack","keepDetailRows","paginateChildRows","preventDefaultOnContextMenu","undoRedoCellEditing","allowDragFromColumnsToolPanel","immutableData","immutableColumns","pivotSuppressAutoColumn","suppressExpandablePivotGroups","applyColumnDefOrder","debounceVerticalScrollbar","detailRowAutoHeight","serverSideFilteringAlwaysResets","suppressAggFilteredOnly","showOpenedGroup","suppressClipboardApi","suppressModelUpdateAfterUpdateTransaction","stopEditingWhenCellsLoseFocus","maintainColumnOrder","groupMaintainOrder","columnHoverHighlight","reactUi","suppressReactUi","readOnlyEdit","suppressRowVirtualisation","resetRowDataOnUpdate","removePivotHeaderRowWhenSingleValueColumn","suppressCopySingleCellRanges"],i.FUNCTION_PROPERTIES=["localeTextFunc","getLocaleText","groupRowInnerRenderer","groupRowInnerRendererFramework","groupRowRenderer","groupRowRendererFramework","isExternalFilterPresent","getRowHeight","doesExternalFilterPass","getRowClass","getRowStyle","getContextMenuItems","getMainMenuItems","processRowPostCreate","processCellForClipboard","groupRowAggNodes","getGroupRowAgg","getRowNodeId","isFullWidthCell","isFullWidthRow","fullWidthCellRenderer","fullWidthCellRendererFramework","processSecondaryColDef","processSecondaryColGroupDef","getBusinessKeyForNode","sendToClipboard","navigateToNextHeader","tabToNextHeader","navigateToNextCell","tabToNextCell","processCellFromClipboard","getDocument","postProcessPopup","getChildCount","getDataPath","loadingCellRenderer","loadingCellRendererFramework","loadingOverlayComponent","loadingOverlayComponentFramework","noRowsOverlayComponent","noRowsOverlayComponentFramework","detailCellRenderer","detailCellRendererFramework","isRowMaster","isRowSelectable","postSort","postSortRows","processHeaderForClipboard","processGroupHeaderForClipboard","paginationNumberFormatter","processDataFromClipboard","getServerSideGroupKey","isServerSideGroup","suppressKeyboardEvent","createChartContainer","getChartToolbarItems","fillOperation","isApplyServerSideTransaction","getServerSideStoreParams","isServerSideGroupOpenByDefault","isGroupOpenByDefault","defaultGroupSortComparator","defaultGroupOrderComparator","initialGroupOrderComparator","loadingCellRendererSelector","getRowId","groupAggFiltering"],i.ALL_PROPERTIES=oI(i.ARRAY_PROPERTIES,i.OBJECT_PROPERTIES,i.STRING_PROPERTIES,i.NUMBER_PROPERTIES,i.FUNCTION_PROPERTIES,i.BOOLEAN_PROPERTIES),i.FRAMEWORK_PROPERTIES=["__ob__","__v_skip","__metadata__","mappedColumnProperties","hasChildColumns","toColDef","createColDefFromGridColumn"],i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var jm=globalThis&&globalThis.__assign||function(){return jm=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},jm.apply(this,arguments)},aI=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},r_=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(aI(arguments[t]));return i},ea=function(){function i(){}return i.getEventCallbacks=function(){return i.EVENT_CALLBACKS||(i.EVENT_CALLBACKS=i.EVENTS.map(function(t){return i.getCallbackForEvent(t)})),i.EVENT_CALLBACKS},i.copyAttributesToGridOptions=function(t,e,r){typeof t!="object"&&(t={});var n=t,o=function(a){return typeof e[a]!="undefined"};return o("groupAggFiltering")&&typeof e.groupAggFiltering!="function"&&(n.groupAggFiltering=i.toBoolean(e.groupAggFiltering),delete e.groupAggFiltering),r_(i.ARRAY_PROPERTIES,i.STRING_PROPERTIES,i.OBJECT_PROPERTIES,i.FUNCTION_PROPERTIES,i.getEventCallbacks()).filter(o).forEach(function(a){return n[a]=e[a]}),i.BOOLEAN_PROPERTIES.filter(o).forEach(function(a){return n[a]=i.toBoolean(e[a])}),i.NUMBER_PROPERTIES.filter(o).forEach(function(a){return n[a]=i.toNumber(e[a])}),t},i.getCallbackForEvent=function(t){return!t||t.length<2?t:"on"+t[0].toUpperCase()+t.substr(1)},i.processOnChange=function(t,e,r,n){if(!!t){var o=jm({},t),a=e,s=function(c){return o[c]};s("groupAggFiltering")&&(typeof o.groupAggFiltering=="function"?a.groupAggFiltering=o.groupAggFiltering:a.groupAggFiltering=i.toBoolean(o.groupAggFiltering),delete o.groupAggFiltering),o.rowClass&&(r.setRowClass(o.rowClass.currentValue),delete o.rowClass),r_(i.ARRAY_PROPERTIES,i.OBJECT_PROPERTIES,i.STRING_PROPERTIES,i.getEventCallbacks()).filter(s).forEach(function(c){return a[c]=o[c].currentValue}),i.BOOLEAN_PROPERTIES.filter(s).forEach(function(c){return a[c]=i.toBoolean(o[c].currentValue)}),i.NUMBER_PROPERTIES.filter(s).forEach(function(c){return a[c]=i.toNumber(o[c].currentValue)}),o.enableCellTextSelection&&(r.setEnableCellTextSelection(i.toBoolean(o.enableCellTextSelection.currentValue)),delete o.enableCellTextSelection),o.quickFilterText&&(r.setQuickFilter(o.quickFilterText.currentValue),delete o.quickFilterText),o.autoGroupColumnDef&&(r.setAutoGroupColumnDef(o.autoGroupColumnDef.currentValue,"gridOptionsChanged"),delete o.autoGroupColumnDef),o.columnDefs&&(r.setColumnDefs(o.columnDefs.currentValue,"gridOptionsChanged"),delete o.columnDefs),o.defaultColDef&&(r.setDefaultColDef(o.defaultColDef.currentValue,"gridOptionsChanged"),delete o.defaultColDef),o.paginationPageSize&&(r.paginationSetPageSize(i.toNumber(o.paginationPageSize.currentValue)),delete o.paginationPageSize),o.pivotMode&&(n.setPivotMode(i.toBoolean(o.pivotMode.currentValue)),delete o.pivotMode),o.groupRemoveSingleChildren&&(r.setGroupRemoveSingleChildren(i.toBoolean(o.groupRemoveSingleChildren.currentValue)),delete o.groupRemoveSingleChildren),o.suppressRowDrag&&(r.setSuppressRowDrag(i.toBoolean(o.suppressRowDrag.currentValue)),delete o.suppressRowDrag),o.suppressMoveWhenRowDragging&&(r.setSuppressMoveWhenRowDragging(i.toBoolean(o.suppressMoveWhenRowDragging.currentValue)),delete o.suppressMoveWhenRowDragging),o.suppressRowClickSelection&&(r.setSuppressRowClickSelection(i.toBoolean(o.suppressRowClickSelection.currentValue)),delete o.suppressRowClickSelection),o.suppressClipboardPaste&&(r.setSuppressClipboardPaste(i.toBoolean(o.suppressClipboardPaste.currentValue)),delete o.suppressClipboardPaste),o.headerHeight&&(r.setHeaderHeight(i.toNumber(o.headerHeight.currentValue)),delete o.headerHeight);var l=r;Object.keys(o).forEach(function(c){var p="set"+c.charAt(0).toUpperCase()+c.substring(1);l[p]&&l[p](t[c].currentValue)});var u={type:x.EVENT_COMPONENT_STATE_CHANGED,api:e.api,columnApi:e.columnApi};Ke(t,function(c,p){u[c]=p}),r.dispatchEvent(u)}},i.toBoolean=function(t){return typeof t=="boolean"?t:typeof t=="string"?t.toUpperCase()==="TRUE"||t=="":!1},i.toNumber=function(t){if(typeof t=="number")return t;if(typeof t=="string")return Number(t)},i.EVENTS=[],i.PUBLIC_EVENTS=[],i.EXCLUDED_INTERNAL_EVENTS=[],i.STRING_PROPERTIES=bi.STRING_PROPERTIES,i.OBJECT_PROPERTIES=bi.OBJECT_PROPERTIES,i.ARRAY_PROPERTIES=bi.ARRAY_PROPERTIES,i.NUMBER_PROPERTIES=bi.NUMBER_PROPERTIES,i.BOOLEAN_PROPERTIES=bi.BOOLEAN_PROPERTIES,i.FUNCTION_PROPERTIES=bi.FUNCTION_PROPERTIES,i.ALL_PROPERTIES=bi.ALL_PROPERTIES,i}();ea.EVENTS=ms(x);ea.EXCLUDED_INTERNAL_EVENTS=[x.EVENT_SCROLLBAR_WIDTH_CHANGED,x.EVENT_CHECKBOX_CHANGED,x.EVENT_HEIGHT_SCALE_CHANGED,x.EVENT_BODY_HEIGHT_CHANGED,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,x.EVENT_SCROLL_VISIBILITY_CHANGED,x.EVENT_COLUMN_HOVER_CHANGED,x.EVENT_FLASH_CELLS,x.EVENT_PAGINATION_PIXEL_OFFSET_CHANGED,x.EVENT_DISPLAYED_ROWS_CHANGED,x.EVENT_LEFT_PINNED_WIDTH_CHANGED,x.EVENT_RIGHT_PINNED_WIDTH_CHANGED,x.EVENT_ROW_CONTAINER_HEIGHT_CHANGED,x.EVENT_POPUP_TO_FRONT,x.EVENT_KEYBOARD_FOCUS,x.EVENT_MOUSE_FOCUS,x.EVENT_STORE_UPDATED,x.EVENT_COLUMN_PANEL_ITEM_DRAG_START,x.EVENT_COLUMN_PANEL_ITEM_DRAG_END,x.EVENT_FILL_START,x.EVENT_FILL_END,x.EVENT_FULL_WIDTH_ROW_FOCUSED];ea.PUBLIC_EVENTS=ea.EVENTS.filter(function(i){return!fn(ea.EXCLUDED_INTERNAL_EVENTS,i)});/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var sI=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),lI=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},SO=function(i){sI(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.componentsMappedByName={},e}return t.prototype.setupComponents=function(e){var r=this;e&&e.forEach(function(n){return r.addComponent(n)})},t.prototype.addComponent=function(e){var r=e.componentName.replace(/([a-z])([A-Z])/g,"$1-$2").toLowerCase(),n=r.toUpperCase();this.componentsMappedByName[n]=e.componentClass},t.prototype.getComponentClass=function(e){return this.componentsMappedByName[e]},t=lI([Q("agStackComponentsRegistry")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var uI=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},cI=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(uI(arguments[t]));return i},Um=function(){function i(){}return i.STRING_PROPERTIES=["headerName","columnGroupShow","headerClass","toolPanelClass","headerValueGetter","pivotKeys","groupId","colId","sort","initialSort","field","type","tooltipComponent","tooltipField","headerTooltip","cellClass","showRowGroup","filter","initialAggFunc","defaultAggFunc","aggFunc","pinned","initialPinned","chartDataType","cellEditorPopupPosition"],i.OBJECT_PROPERTIES=["headerGroupComponent","headerGroupComponentFramework","headerGroupComponentParams","cellStyle","cellRenderer","cellRendererParams","cellRendererFramework","cellEditor","cellEditorFramework","cellEditorParams","pinnedRowCellRendererFramework","pinnedRowCellRendererParams","filterFramework","filterParams","pivotValueColumn","headerComponent","headerComponentFramework","headerComponentParams","floatingFilterComponent","floatingFilterComponentParams","floatingFilterComponentFramework","floatingFilterFramework","tooltipComponent","tooltipComponentParams","tooltipComponentFramework","refData","columnsMenuParams"],i.ARRAY_PROPERTIES=["children","sortingOrder","allowedAggFuncs","menuTabs","pivotTotalColumnIds","cellClassRules","icons"],i.NUMBER_PROPERTIES=["sortedAt","sortIndex","initialSortIndex","flex","initialFlex","width","initialWidth","minWidth","maxWidth","rowGroupIndex","initialRowGroupIndex","pivotIndex","initialPivotIndex"],i.BOOLEAN_PROPERTIES=["suppressCellFlash","suppressColumnsToolPanel","suppressFiltersToolPanel","openByDefault","marryChildren","hide","initialHide","rowGroup","initialRowGroup","pivot","initialPivot","checkboxSelection","headerCheckboxSelection","headerCheckboxSelectionFilteredOnly","suppressMenu","suppressMovable","lockPosition","lockVisible","lockPinned","unSortIcon","suppressSizeToFit","suppressAutoSize","enableRowGroup","enablePivot","enableValue","editable","suppressPaste","suppressNavigable","enableCellChangeFlash","rowDrag","dndSource","autoHeight","wrapText","sortable","resizable","singleClickEdit","floatingFilter","cellEditorPopup","suppressFillHandle"],i.FUNCTION_PROPERTIES=["dndSourceOnRowDrag","valueGetter","valueSetter","filterValueGetter","keyCreator","pinnedRowCellRenderer","valueFormatter","pinnedRowValueFormatter","valueParser","comparator","equals","pivotComparator","suppressKeyboardEvent","suppressHeaderKeyboardEvent","colSpan","rowSpan","getQuickFilterText","newValueHandler","onCellValueChanged","onCellClicked","onCellDoubleClicked","onCellContextMenu","rowDragText","tooltipValueGetter","tooltipComponent","tooltipComponentFramework","cellRendererSelector","cellEditorSelector"],i.ALL_PROPERTIES=cI(i.ARRAY_PROPERTIES,i.OBJECT_PROPERTIES,i.STRING_PROPERTIES,i.NUMBER_PROPERTIES,i.FUNCTION_PROPERTIES,i.BOOLEAN_PROPERTIES),i.FRAMEWORK_PROPERTIES=["__ob__","__v_skip","__metadata__","mappedColumnProperties","hasChildColumns","toColDef","createColDefFromGridColumn"],i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var n_=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Bo=function(){function i(t,e,r,n){n===void 0&&(n=1),this.r=Math.min(1,Math.max(0,t||0)),this.g=Math.min(1,Math.max(0,e||0)),this.b=Math.min(1,Math.max(0,r||0)),this.a=Math.min(1,Math.max(0,n||0))}return i.fromString=function(t){if(t.indexOf("#")>=0)return i.fromHexString(t);var e=i.nameToHex[t];if(e)return i.fromHexString(e);if(t.indexOf("rgb")>=0)return i.fromRgbaString(t);throw new Error("Invalid color string: '"+t+"'")},i.parseHex=function(t){t=t.replace(/ /g,"").slice(1);var e;switch(t.length){case 6:case 8:e=[];for(var r=0;r<t.length;r+=2)e.push(parseInt(""+t[r]+t[r+1],16));break;case 3:case 4:e=t.split("").map(function(n){return parseInt(n,16)}).map(function(n){return n+n*16});break}if(e.length>=3&&e.every(function(n){return n>=0}))return e.length===3&&e.push(255),e},i.fromHexString=function(t){var e=i.parseHex(t);if(e){var r=n_(e,4),n=r[0],o=r[1],a=r[2],s=r[3];return new i(n/255,o/255,a/255,s/255)}throw new Error("Malformed hexadecimal color string: '"+t+"'")},i.stringToRgba=function(t){for(var e=n_([NaN,NaN],2),r=e[0],n=e[1],o=0;o<t.length;o++){var a=t[o];if(!r&&a==="(")r=o;else if(a===")"){n=o;break}}var s=r&&n&&t.substring(r+1,n);if(!!s){for(var l=s.split(","),u=[],o=0;o<l.length;o++){var c=l[o],p=parseFloat(c);if(isNaN(p))return;c.indexOf("%")>=0?(p=Math.max(0,Math.min(100,p)),p/=100):o===3?p=Math.max(0,Math.min(1,p)):(p=Math.max(0,Math.min(255,p)),p/=255),u.push(p)}return u}},i.fromRgbaString=function(t){var e=i.stringToRgba(t);if(e){if(e.length===3)return new i(e[0],e[1],e[2]);if(e.length===4)return new i(e[0],e[1],e[2],e[3])}throw new Error("Malformed rgb/rgba color string: '"+t+"'")},i.fromArray=function(t){if(t.length===4)return new i(t[0],t[1],t[2],t[3]);if(t.length===3)return new i(t[0],t[1],t[2]);throw new Error("The given array should contain 3 or 4 color components (numbers).")},i.fromHSB=function(t,e,r,n){n===void 0&&(n=1);var o=i.HSBtoRGB(t,e,r);return new i(o[0],o[1],o[2],n)},i.padHex=function(t){return t.length===1?"0"+t:t},i.prototype.toHexString=function(){var t="#"+i.padHex(Math.round(this.r*255).toString(16))+i.padHex(Math.round(this.g*255).toString(16))+i.padHex(Math.round(this.b*255).toString(16));return this.a<1&&(t+=i.padHex(Math.round(this.a*255).toString(16))),t},i.prototype.toRgbaString=function(t){t===void 0&&(t=3);var e=[Math.round(this.r*255),Math.round(this.g*255),Math.round(this.b*255)],r=Math.pow(10,t);return this.a!==1?(e.push(Math.round(this.a*r)/r),"rgba("+e.join(", ")+")"):"rgb("+e.join(", ")+")"},i.prototype.toString=function(){return this.a===1?this.toHexString():this.toRgbaString()},i.prototype.toHSB=function(){return i.RGBtoHSB(this.r,this.g,this.b)},i.RGBtoHSB=function(t,e,r){var n=Math.min(t,e,r),o=Math.max(t,e,r),a=o!==0?(o-n)/o:0,s=NaN;if(n!==o){var l=o-n,u=(o-t)/l,c=(o-e)/l,p=(o-r)/l;t===o?s=p-c:e===o?s=2+u-p:s=4+c-u,s/=6,s<0&&(s=s+1)}return[s*360,a,o]},i.HSBtoRGB=function(t,e,r){isNaN(t)&&(t=0),t=(t%360+360)%360/360;var n=0,o=0,a=0;if(e===0)n=o=a=r;else{var s=(t-Math.floor(t))*6,l=s-Math.floor(s),u=r*(1-e),c=r*(1-e*l),p=r*(1-e*(1-l));switch(s>>0){case 0:n=r,o=p,a=u;break;case 1:n=c,o=r,a=u;break;case 2:n=u,o=r,a=p;break;case 3:n=u,o=c,a=r;break;case 4:n=p,o=u,a=r;break;case 5:n=r,o=u,a=c;break}}return[n,o,a]},i.prototype.derive=function(t,e,r,n){var o=i.RGBtoHSB(this.r,this.g,this.b),a=o[2];a==0&&r>1&&(a=.05);var s=((o[0]+t)%360+360)%360,l=Math.max(Math.min(o[1]*e,1),0);a=Math.max(Math.min(a*r,1),0);var u=Math.max(Math.min(this.a*n,1),0),c=i.HSBtoRGB(s,l,a);return c.push(u),i.fromArray(c)},i.prototype.brighter=function(){return this.derive(0,1,1/.7,1)},i.prototype.darker=function(){return this.derive(0,1,.7,1)},i.nameToHex=Object.freeze({aliceblue:"#F0F8FF",antiquewhite:"#FAEBD7",aqua:"#00FFFF",aquamarine:"#7FFFD4",azure:"#F0FFFF",beige:"#F5F5DC",bisque:"#FFE4C4",black:"#000000",blanchedalmond:"#FFEBCD",blue:"#0000FF",blueviolet:"#8A2BE2",brown:"#A52A2A",burlywood:"#DEB887",cadetblue:"#5F9EA0",chartreuse:"#7FFF00",chocolate:"#D2691E",coral:"#FF7F50",cornflowerblue:"#6495ED",cornsilk:"#FFF8DC",crimson:"#DC143C",cyan:"#00FFFF",darkblue:"#00008B",darkcyan:"#008B8B",darkgoldenrod:"#B8860B",darkgray:"#A9A9A9",darkgreen:"#006400",darkgrey:"#A9A9A9",darkkhaki:"#BDB76B",darkmagenta:"#8B008B",darkolivegreen:"#556B2F",darkorange:"#FF8C00",darkorchid:"#9932CC",darkred:"#8B0000",darksalmon:"#E9967A",darkseagreen:"#8FBC8F",darkslateblue:"#483D8B",darkslategray:"#2F4F4F",darkslategrey:"#2F4F4F",darkturquoise:"#00CED1",darkviolet:"#9400D3",deeppink:"#FF1493",deepskyblue:"#00BFFF",dimgray:"#696969",dimgrey:"#696969",dodgerblue:"#1E90FF",firebrick:"#B22222",floralwhite:"#FFFAF0",forestgreen:"#228B22",fuchsia:"#FF00FF",gainsboro:"#DCDCDC",ghostwhite:"#F8F8FF",gold:"#FFD700",goldenrod:"#DAA520",gray:"#808080",green:"#008000",greenyellow:"#ADFF2F",grey:"#808080",honeydew:"#F0FFF0",hotpink:"#FF69B4",indianred:"#CD5C5C",indigo:"#4B0082",ivory:"#FFFFF0",khaki:"#F0E68C",lavender:"#E6E6FA",lavenderblush:"#FFF0F5",lawngreen:"#7CFC00",lemonchiffon:"#FFFACD",lightblue:"#ADD8E6",lightcoral:"#F08080",lightcyan:"#E0FFFF",lightgoldenrodyellow:"#FAFAD2",lightgray:"#D3D3D3",lightgreen:"#90EE90",lightgrey:"#D3D3D3",lightpink:"#FFB6C1",lightsalmon:"#FFA07A",lightseagreen:"#20B2AA",lightskyblue:"#87CEFA",lightslategray:"#778899",lightslategrey:"#778899",lightsteelblue:"#B0C4DE",lightyellow:"#FFFFE0",lime:"#00FF00",limegreen:"#32CD32",linen:"#FAF0E6",magenta:"#FF00FF",maroon:"#800000",mediumaquamarine:"#66CDAA",mediumblue:"#0000CD",mediumorchid:"#BA55D3",mediumpurple:"#9370DB",mediumseagreen:"#3CB371",mediumslateblue:"#7B68EE",mediumspringgreen:"#00FA9A",mediumturquoise:"#48D1CC",mediumvioletred:"#C71585",midnightblue:"#191970",mintcream:"#F5FFFA",mistyrose:"#FFE4E1",moccasin:"#FFE4B5",navajowhite:"#FFDEAD",navy:"#000080",oldlace:"#FDF5E6",olive:"#808000",olivedrab:"#6B8E23",orange:"#FFA500",orangered:"#FF4500",orchid:"#DA70D6",palegoldenrod:"#EEE8AA",palegreen:"#98FB98",paleturquoise:"#AFEEEE",palevioletred:"#DB7093",papayawhip:"#FFEFD5",peachpuff:"#FFDAB9",peru:"#CD853F",pink:"#FFC0CB",plum:"#DDA0DD",powderblue:"#B0E0E6",purple:"#800080",rebeccapurple:"#663399",red:"#FF0000",rosybrown:"#BC8F8F",royalblue:"#4169E1",saddlebrown:"#8B4513",salmon:"#FA8072",sandybrown:"#F4A460",seagreen:"#2E8B57",seashell:"#FFF5EE",sienna:"#A0522D",silver:"#C0C0C0",skyblue:"#87CEEB",slateblue:"#6A5ACD",slategray:"#708090",slategrey:"#708090",snow:"#FFFAFA",springgreen:"#00FF7F",steelblue:"#4682B4",tan:"#D2B48C",teal:"#008080",thistle:"#D8BFD8",tomato:"#FF6347",turquoise:"#40E0D0",violet:"#EE82EE",wheat:"#F5DEB3",white:"#FFFFFF",whitesmoke:"#F5F5F5",yellow:"#FFFF00",yellowgreen:"#9ACD32"}),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function xO(i,t){t===void 0&&(t=",");var e=[],r=function(h){return h==="\r"||h===`
`},n=!1;if(i==="")return[[""]];for(var o=function(h,d,f){var g=i[f-1],v=i[f],m=i[f+1],C=function(){e[h]||(e[h]=[]),e[h][d]||(e[h][d]="")};if(C(),v==='"'){if(n)return m==='"'?(e[h][d]+='"',f++):n=!1,a=h,s=d,l=f,"continue";if(g===void 0||g===t||r(g))return n=!0,a=h,s=d,l=f,"continue"}if(!n){if(v===t)return d++,C(),a=h,s=d,l=f,"continue";if(r(v))return d=0,h++,C(),v==="\r"&&m===`
`&&f++,a=h,s=d,l=f,"continue"}e[h][d]+=v,a=h,s=d,l=f},a,s,l,u=0,c=0,p=0;p<i.length;p++)o(u,c,p),u=a,c=s,p=l;return e}var pI=Object.freeze(Object.defineProperty({__proto__:null,stringToArray:xO},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var hI=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},pg,hg,dg,fg,gg,vg,zm,$m;function dI(){return hg===void 0&&(hg=!!document.documentMode),hg}function Jd(){return dg===void 0&&(dg=!dI()&&!!window.StyleMedia),dg}function Hc(){return pg===void 0&&(pg=/^((?!chrome|android).)*safari/i.test(navigator.userAgent)),pg}function jc(){if(fg===void 0){var i=window;fg=!!i.chrome&&(!!i.chrome.webstore||!!i.chrome.runtime)||/Chrome/.test(navigator.userAgent)&&/Google Inc/.test(navigator.vendor)}return fg}function RC(){if(gg===void 0){var i=window;gg=typeof i.InstallTrigger!="undefined"}return gg}function Bl(){return vg===void 0&&(vg=(/iPad|iPhone|iPod/.test(navigator.platform)||navigator.platform==="MacIntel"&&navigator.maxTouchPoints>1)&&!window.MSStream),vg}function Kh(i){if(!i)return null;var t=i.tabIndex,e=i.getAttribute("tabIndex");return t===-1&&(e===null||e===""&&!RC())?null:t.toString()}function TO(){if(!document.body)return-1;var i=1e6,t=navigator.userAgent.toLowerCase().match(/firefox/)?6e6:1e9,e=document.createElement("div");for(document.body.appendChild(e);;){var r=i*2;if(e.style.height=r+"px",r>t||e.clientHeight!==r)break;i=r}return document.body.removeChild(e),i}function OO(){return $m==null&&EO(),$m}function EO(){var i=document.body,t=document.createElement("div");t.style.width=t.style.height="100px",t.style.opacity="0",t.style.overflow="scroll",t.style.msOverflowStyle="scrollbar",t.style.position="absolute",i.appendChild(t);var e=t.offsetWidth-t.clientWidth;e===0&&t.clientWidth===0&&(e=null),t.parentNode&&t.parentNode.removeChild(t),e!=null&&($m=e,zm=e===0)}function RO(){return zm==null&&EO(),zm}function fI(){var i,t,e=["webkit","moz","o","ms"],r=document.createElement("div"),n=document.getElementsByTagName("body")[0],o=!1,a;n.appendChild(r),r.setAttribute("style",e.map(function(c){return"-"+c+"-overflow-scrolling: touch"}).concat("overflow-scrolling: touch").join(";"));var s=window.getComputedStyle(r);if(s.overflowScrolling==="touch"&&(o=!0),!o)try{for(var l=hI(e),u=l.next();!u.done;u=l.next())if(a=u.value,s[a+"OverflowScrolling"]==="touch"){o=!0;break}}catch(c){i={error:c}}finally{try{u&&!u.done&&(t=l.return)&&t.call(l)}finally{if(i)throw i.error}}return r.parentNode&&r.parentNode.removeChild(r),o}function PO(){return document.body?document.body.clientWidth:window.innerHeight?window.innerWidth:document.documentElement&&document.documentElement.clientWidth?document.documentElement.clientWidth:-1}function AO(){return document.body?document.body.clientHeight:window.innerHeight?window.innerHeight:document.documentElement&&document.documentElement.clientHeight?document.documentElement.clientHeight:-1}var gI=Object.freeze(Object.defineProperty({__proto__:null,isBrowserEdge:Jd,isBrowserSafari:Hc,isBrowserChrome:jc,isBrowserFirefox:RC,isIOSUserAgent:Bl,getTabIndex:Kh,getMaxDivHeight:TO,getScrollbarWidth:OO,isInvisibleScrollbar:RO,hasOverflowScrolling:fI,getBodyWidth:PO,getBodyHeight:AO},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var dp;function vI(i,t,e){for(var r=i.parentElement,n=r&&r.firstChild;n;)t&&n.classList.toggle(t,n===i),e&&n.classList.toggle(e,n!==i),n=n.nextSibling}function DO(i){var t=Element.prototype.matches||Element.prototype.msMatchesSelector,e=t.call(i,L.INPUT_SELECTOR),r=t.call(i,L.FOCUSABLE_EXCLUDE),n=jl(i),o=e&&!r&&n;return o}function pt(i,t){i.classList.toggle("ag-hidden",!t)}function MO(i,t){i.classList.toggle("ag-invisible",!t)}function Al(i,t){var e="disabled",r=t?function(n){return n.setAttribute(e,"")}:function(n){return n.removeAttribute(e)};r(i),IC(i.querySelectorAll("input"),function(n){return r(n)})}function ef(i,t,e){for(var r=0;i;){if(i.classList.contains(t))return!0;if(i=i.parentElement,e&&++r>e)break}return!1}function Hl(i){var t=window.getComputedStyle(i),e=t.height,r=t.width,n=t.paddingTop,o=t.paddingRight,a=t.paddingBottom,s=t.paddingLeft,l=t.marginTop,u=t.marginRight,c=t.marginBottom,p=t.marginLeft,h=t.boxSizing;return{height:parseFloat(e),width:parseFloat(r),paddingTop:parseFloat(n),paddingRight:parseFloat(o),paddingBottom:parseFloat(a),paddingLeft:parseFloat(s),marginTop:parseFloat(l),marginRight:parseFloat(u),marginBottom:parseFloat(c),marginLeft:parseFloat(p),boxSizing:h}}function Uc(i){var t=Hl(i);return t.boxSizing==="border-box"?t.height-t.paddingTop-t.paddingBottom:t.height}function tf(i){var t=Hl(i);return t.boxSizing==="border-box"?t.width-t.paddingLeft-t.paddingRight:t.width}function PC(i){var t=Hl(i),e=t.marginBottom+t.marginTop;return Math.ceil(i.offsetHeight+e)}function rf(i){var t=Hl(i),e=t.marginLeft+t.marginRight;return Math.ceil(i.offsetWidth+e)}function xc(){if(typeof dp=="boolean")return dp;var i=document.createElement("div");return i.style.direction="rtl",i.style.width="1px",i.style.height="1px",i.style.position="fixed",i.style.top="0px",i.style.overflow="hidden",i.dir="rtl",i.innerHTML=`<div style="width: 2px">
            <span style="display: inline-block; width: 1px"></span>
            <span style="display: inline-block; width: 1px"></span>
        </div>`,document.body.appendChild(i),i.scrollLeft=1,dp=Math.floor(i.scrollLeft)===0,document.body.removeChild(i),dp}function Tc(i,t){var e=i.scrollLeft;return t&&(e=Math.abs(e),jc()&&!xc()&&(e=i.scrollWidth-i.clientWidth-e)),e}function Yh(i,t,e){e&&(xc()?t*=-1:(Hc()||jc())&&(t=i.scrollWidth-i.clientWidth-t)),i.scrollLeft=t}function gn(i){for(;i&&i.firstChild;)i.removeChild(i.firstChild)}function mI(i,t){qn(i.querySelector(t))}function qn(i){i&&i.parentNode&&i.parentNode.removeChild(i)}function jl(i){return i.offsetParent!==null}function Kt(i){if(DOMParser!==null){var t=new DOMParser,e=t.parseFromString(i,"text/html");return e.body.firstChild}var r=document.createElement("div");return r.innerHTML=(i||"").trim(),r.firstChild}function yI(i,t){i.lastChild?i.insertAdjacentHTML("afterbegin",t):i.innerHTML=t}function CI(i,t){if(i.attributes&&i.attributes[t]){var e=i.attributes[t];return e.value}return null}function Km(i){return i&&i.clientHeight?i.clientHeight:0}function Ym(i){return i&&i.clientWidth?i.clientWidth:0}function AC(i,t,e){e&&e.nextSibling===t||(e?e.nextSibling?i.insertBefore(t,e.nextSibling):i.appendChild(t):i.firstChild&&i.firstChild!==t&&i.insertAdjacentElement("afterbegin",t))}function DC(i,t){for(var e=0;e<t.length;e++){var r=t[e],n=i.children[e];n!==r&&i.insertBefore(r,n)}}function FO(i,t,e){e?e.insertAdjacentElement("afterend",t):i.firstChild?i.insertAdjacentElement("afterbegin",t):i.appendChild(t)}function wI(i,t){q(i.firstChild)?i.insertBefore(t,i.firstChild):i.appendChild(t)}function MC(i,t){!t||Object.keys(t).forEach(function(e){var r=CO(e);r&&(i.style[r]=t[e])})}function Xm(i){return i.clientWidth<i.scrollWidth}function IO(i){return i.clientHeight<i.scrollHeight}function zc(i,t){t==="flex"?(i.style.removeProperty("width"),i.style.removeProperty("minWidth"),i.style.removeProperty("maxWidth"),i.style.flex="1 1 auto"):Li(i,t)}function Li(i,t){t=FC(t),i.style.width=t.toString(),i.style.maxWidth=t.toString(),i.style.minWidth=t.toString()}function bI(i,t){t==="flex"?(i.style.removeProperty("height"),i.style.removeProperty("minHeight"),i.style.removeProperty("maxHeight"),i.style.flex="1 1 auto"):ts(i,t)}function ts(i,t){t=FC(t),i.style.height=t.toString(),i.style.maxHeight=t.toString(),i.style.minHeight=t.toString()}function FC(i){return typeof i=="number"?i+"px":i}function LO(i){return typeof Node=="function"?i instanceof Node:i&&typeof i=="object"&&typeof i.nodeType=="number"&&typeof i.nodeName=="string"}function NO(i){return typeof HTMLElement=="function"?i instanceof HTMLElement:i&&Rl(i)&&i.nodeType===1&&typeof i.nodeName=="string"}function $c(i){return LO(i)||NO(i)}function GO(i){if(i==null)return[];var t=[];return IC(i,function(e){return t.push(e)}),t}function kO(i,t){if(!!i)for(var e=0;e<i.length;e++){var r=i[e];t(r.name,r.value)}}function _I(i,t){typeof t=="boolean"?(i.checked=t,i.indeterminate=!1):i.indeterminate=!0}function lc(i,t,e){e==null?i.removeAttribute(t):i.setAttribute(t,e.toString())}function IC(i,t){if(i!=null)for(var e=0;e<i.length;e++)t(i[e])}var SI=Object.freeze(Object.defineProperty({__proto__:null,radioCssClass:vI,isFocusableFormField:DO,setDisplayed:pt,setVisible:MO,setDisabled:Al,isElementChildOfClass:ef,getElementSize:Hl,getInnerHeight:Uc,getInnerWidth:tf,getAbsoluteHeight:PC,getAbsoluteWidth:rf,isRtlNegativeScroll:xc,getScrollLeft:Tc,setScrollLeft:Yh,clearElement:gn,removeElement:mI,removeFromParent:qn,isVisible:jl,loadTemplate:Kt,appendHtml:yI,getElementAttribute:CI,offsetHeight:Km,offsetWidth:Ym,ensureDomOrder:AC,setDomChildOrder:DC,insertWithDomOrder:FO,prependDC:wI,addStylesToElement:MC,isHorizontalScrollShowing:Xm,isVerticalScrollShowing:IO,setElementWidth:zc,setFixedWidth:Li,setElementHeight:bI,setFixedHeight:ts,formatSize:FC,isNode:LO,isElement:NO,isNodeOrElement:$c,copyNodeList:GO,iterateNamedNodeMap:kO,setCheckboxState:_I,addOrRemoveAttribute:lc,nodeListForEach:IC},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function xI(i){var t=/function (.{1,})\(/,e=i.toString(),r=t.exec(e);return r&&r.length>1?r[1]:""}function TI(i){var t=i.length,e=0;if(t<=1)return i;for(var r=0;r<i.length;r++){var n=i[r],o=n.toString().split("e-");if(o.length>1){e=Math.max(e,parseInt(o[1],10));continue}Math.floor(n)!==n&&(e=Math.max(e,n.toString().split(".")[1].length))}for(var a=0,s=0,l=0,u=0,c=0,p=0;p<t;p++)c=i[p],a+=p,s+=c,u+=p*p,l+=p*c;for(var h=(t*l-a*s)/(t*u-a*a),d=s/t-h*a/t,f=[],p=0;p<=t;p++)f.push(parseFloat((p*h+d).toFixed(e)));return f}function OI(i){if(!i)return"";var t=[];return Ke(i,function(e,r){var n=yO(e);t.push(n+": "+r+";")}),t.join(" ")}function EI(i){var t=document.createElement("div"),e=document.querySelector("#__ag__message");if(t.innerHTML=i,!e){var r='<div id="__ag__message" style="display: inline-block; position: absolute; top: 0px; left: 0px; color: white; background-color: black; z-index: 20; padding: 2px; border: 1px solid darkred; height: 200px; overflow-y: auto;"></div>';e=Kt(r),document.body&&document.body.appendChild(e)}e.insertBefore(t,e.children[0])}function RI(i,t){i.then(function(e){var r=e.getGui();r!=null&&(typeof r=="object"?t.appendChild(r):t.innerHTML=r)})}var PI=Object.freeze(Object.defineProperty({__proto__:null,getNameOfClass:xI,findLineByLeastSquares:TI,cssStyleObjectToMarkup:OI,message:EI,bindCellRendererToHtmlElement:RI},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function Xt(i,t,e){i.setAttribute(WO(t),e.toString())}function ys(i,t){i.removeAttribute(WO(t))}function WO(i){return"aria-"+i}function _r(i,t){t?i.setAttribute("role",t):i.removeAttribute("role")}function VO(i){var t;return i.isSortAscending()?t="ascending":i.isSortDescending()?t="descending":t="none",t}function BO(i){return parseInt(i.getAttribute("aria-level"),10)}function HO(i){return parseInt(i.getAttribute("aria-posinset"),10)}function AI(i){return i.getAttribute("aria-describedby")||""}function Cs(i,t){var e="label";t?Xt(i,e,t):ys(i,e)}function LC(i,t){var e="labelledby";t?Xt(i,e,t):ys(i,e)}function jO(i,t){var e="description";t?Xt(i,e,t):ys(i,e)}function NC(i,t){var e="describedby";t?Xt(i,e,t):ys(i,e)}function UO(i,t){Xt(i,"level",t)}function GC(i,t){Xt(i,"disabled",t)}function dn(i,t){Xt(i,"expanded",t)}function zO(i){ys(i,"expanded")}function kC(i,t){Xt(i,"setsize",t)}function WC(i,t){Xt(i,"posinset",t)}function $O(i,t){Xt(i,"multiselectable",t)}function KO(i,t){Xt(i,"rowcount",t)}function VC(i,t){Xt(i,"rowindex",t)}function YO(i,t){Xt(i,"colcount",t)}function BC(i,t){Xt(i,"colindex",t)}function XO(i,t){Xt(i,"colspan",t)}function qO(i,t){Xt(i,"sort",t)}function ZO(i){ys(i,"sort")}function Dl(i,t){var e="selected";t?Xt(i,e,t):ys(i,e)}function QO(i,t){Xt(i,"checked",t===void 0?"mixed":t)}var DI=Object.freeze(Object.defineProperty({__proto__:null,setAriaRole:_r,getAriaSortState:VO,getAriaLevel:BO,getAriaPosInSet:HO,getAriaDescribedBy:AI,setAriaLabel:Cs,setAriaLabelledBy:LC,setAriaDescription:jO,setAriaDescribedBy:NC,setAriaLevel:UO,setAriaDisabled:GC,setAriaExpanded:dn,removeAriaExpanded:zO,setAriaSetSize:kC,setAriaPosInSet:WC,setAriaMultiSelectable:$O,setAriaRowCount:KO,setAriaRowIndex:VC,setAriaColCount:YO,setAriaColIndex:BC,setAriaColSpan:XO,setAriaSort:qO,removeAriaSort:ZO,setAriaSelected:Dl,setAriaChecked:QO},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function qm(i,t){return i.toString().padStart(t,"0")}function JO(i,t){for(var e=[],r=i;r<=t;r++)e.push(r);return e}function Xh(i){return i!==""&&!isNaN(parseFloat(i))&&isFinite(i)}function MI(i){return typeof i=="string"&&(i=parseInt(i,10)),typeof i=="number"?Math.floor(i):null}function FI(i,t){for(var e="",r=0;r<t;r++)e+=String.fromCharCode(i&255),i>>>=8;return e}function II(i,t,e){return typeof i!="number"?"":HC(Math.round(i*100)/100,t,e)}function HC(i,t,e){return typeof i!="number"?"":i.toString().replace(".",e).replace(/(\d)(?=(\d{3})+(?!\d))/g,"$1"+t)}function LI(i){return i==null?null:i.reduce(function(t,e){return t+e},0)}var NI=Object.freeze(Object.defineProperty({__proto__:null,padStartWidthZeros:qm,createArrayOfNumbers:JO,isNumeric:Xh,cleanNumber:MI,decToHex:FI,formatNumberTwoDecimalPlacesAndCommas:II,formatNumberCommas:HC,sum:LI},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var mg=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o};function $o(i,t,e){if(t===void 0&&(t=!0),e===void 0&&(e="-"),!i)return null;var r=[i.getFullYear(),i.getMonth()+1,i.getDate()].map(function(n){return qm(n,2)}).join(e);return t&&(r+=" "+[i.getHours(),i.getMinutes(),i.getSeconds()].map(function(n){return qm(n,2)}).join(":")),r}function Ko(i){if(!i)return null;var t=mg(i.split(" "),2),e=t[0],r=t[1];if(!e)return null;var n=e.split("-").map(function(f){return parseInt(f,10)});if(n.filter(function(f){return!isNaN(f)}).length!==3)return null;var o=mg(n,3),a=o[0],s=o[1],l=o[2],u=new Date(a,s-1,l);if(u.getFullYear()!==a||u.getMonth()!==s-1||u.getDate()!==l)return null;if(!r||r==="00:00:00")return u;var c=mg(r.split(":").map(function(f){return parseInt(f,10)}),3),p=c[0],h=c[1],d=c[2];return p>=0&&p<24&&u.setHours(p),h>=0&&h<60&&u.setMinutes(h),d>=0&&d<60&&u.setSeconds(d),u}var GI=Object.freeze(Object.defineProperty({__proto__:null,serialiseDate:$o,parseDateTimeFromString:Ko},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function eE(i,t,e){var r={},n=i.filter(function(o){return!t.some(function(a){return a===o})});return n.length>0&&n.forEach(function(o){return r[o]=tE(o,e)}),r}function tE(i,t,e,r){var n=r?nE:rE,o=t.map(function(a){return{value:a,relevance:n(i.toLowerCase(),a.toLocaleLowerCase())}});return o.sort(function(a,s){return s.relevance-a.relevance}),e&&(o=o.filter(function(a){return a.relevance!==0})),o.map(function(a){return a.value})}function Zm(i){var t=i.toLowerCase(),e=new Array(t.length-1),r,n,o;for(r=n=0,o=e.length;n<=o;r=n+=1)e[r]=t.slice(r,r+2);return e}function rE(i,t){if(i.length===0&&t.length===0)return 0;var e=Zm(i),r=Zm(t),n=e.length+r.length,o=0,a,s;for(a=0,s=e.length;a<s;a++){var l=e[a],u=void 0,c=void 0;for(u=0,c=r.length;u<c;u++){var p=r[u];l===p&&o++}}return o>0?2*o/n:0}function nE(i,t){for(var e=i.replace(/\s/g,""),r=t.replace(/\s/g,""),n=0,o=0,a=0;a<e.length;a++){var s=r.indexOf(e[a],o);s!==-1&&(o=s,n+=100-o*100/1e4*100)}return n}var kI=Object.freeze(Object.defineProperty({__proto__:null,fuzzyCheckStrings:eE,fuzzySuggestions:tE,get_bigrams:Zm,string_distances:rE,string_weighted_distances:nE},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var iE={columnGroupOpened:"expanded",columnGroupClosed:"contracted",columnSelectClosed:"tree-closed",columnSelectOpen:"tree-open",columnSelectIndeterminate:"tree-indeterminate",columnMovePin:"pin",columnMoveHide:"eye-slash",columnMoveMove:"arrows",columnMoveLeft:"left",columnMoveRight:"right",columnMoveGroup:"group",columnMoveValue:"aggregation",columnMovePivot:"pivot",dropNotAllowed:"not-allowed",groupContracted:"tree-closed",groupExpanded:"tree-open",chart:"chart",close:"cross",cancel:"cancel",check:"tick",first:"first",previous:"previous",next:"next",last:"last",linked:"linked",unlinked:"unlinked",colorPicker:"color-picker",groupLoading:"loading",menu:"menu",filter:"filter",columns:"columns",maximize:"maximize",minimize:"minimize",menuPin:"pin",menuValue:"aggregation",menuAddRowGroup:"group",menuRemoveRowGroup:"group",clipboardCopy:"copy",clipboardPaste:"paste",pivotPanel:"pivot",rowGroupPanel:"group",valuePanel:"aggregation",columnDrag:"grip",rowDrag:"grip",save:"save",csvExport:"csv",excelExport:"excel",smallDown:"small-down",smallLeft:"small-left",smallRight:"small-right",smallUp:"small-up",sortAscending:"asc",sortDescending:"desc",sortUnSort:"none"};function Tn(i,t,e){var r=Ft(i,t,e);if(r&&r.className.indexOf("ag-icon")>-1)return r;var n=document.createElement("span");return n.appendChild(r),n}function Ft(i,t,e,r){var n=null,o=e&&e.getColDef().icons;if(o&&(n=o[i]),t&&!n){var a=t.getIcons();a&&(n=a[i])}if(n){var s=void 0;if(typeof n=="function")s=n();else if(typeof n=="string")s=n;else throw new Error("icon from grid options needs to be a string or a function");if(typeof s=="string")return Kt(s);if($c(s))return s;console.warn("AG Grid: iconRenderer should return back a string or a dom object")}else{var l=document.createElement("span"),u=iE[i];return u||(r?u=i:(console.warn("AG Grid: Did not find icon "+i),u="")),l.setAttribute("class","ag-icon ag-icon-"+u),l.setAttribute("unselectable","on"),_r(l,"presentation"),l}}var WI=Object.freeze(Object.defineProperty({__proto__:null,iconNameClassMap:iE,createIcon:Tn,createIconNoSpan:Ft},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var VI="Del",BI=46;function oE(i){if(i.altKey||i.ctrlKey||i.metaKey)return!1;var t=i.key.length===1,e=HI(i);return t||e}function qh(i,t,e,r,n){var o=i.getSuppressKeyboardEventFunc(),a=r?r.getColDef().suppressKeyboardEvent:void 0;if(!o&&!a)return!1;var s={event:t,editing:n,column:r,api:i.getApi(),node:e,data:e.data,colDef:r.getColDef(),context:i.getContext(),columnApi:i.getColumnApi()};if(a){var l=a(s);if(l)return!0}return o?o(s):!1}function aE(i,t,e,r){var n=r.getDefinition(),o=n&&n.suppressHeaderKeyboardEvent;if(!q(o))return!1;var a={api:i.getApi(),columnApi:i.getColumnApi(),context:i.getContext(),colDef:n,column:r,headerRowIndex:e,event:t};return!!o(a)}function HI(i){return Jd()&&i.key===VI&&i.charCode===BI}var jI=Object.freeze(Object.defineProperty({__proto__:null,isEventFromPrintableCharacter:oE,isUserSuppressingKeyboardEvent:qh,isUserSuppressingHeaderKeyboardEvent:aE},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function jC(i,t,e){if(e===0)return!1;var r=Math.abs(i.clientX-t.clientX),n=Math.abs(i.clientY-t.clientY);return Math.max(r,n)<=e}var UI=Object.freeze(Object.defineProperty({__proto__:null,areEventsNear:jC},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function zI(i,t){if(!!i){for(var e=function(s,l){var u=t[s.id],c=t[l.id],p=u!==void 0,h=c!==void 0,d=p&&h,f=!p&&!h;return d?u-c:f?s.__objectId-l.__objectId:p?1:-1},r,n,o=!1,a=0;a<i.length-1;a++)if(r=i[a],n=i[a+1],e(r,n)>0){o=!0;break}o&&i.sort(e)}}function $I(i,t){var e=[];r(i);function r(n){!n||n.forEach(function(o){if(o.group||o.hasChildren()){e.push(o.key);var a=e.join("|");t(o,a),r(o.childrenAfterGroup),e.pop()}})}}var KI=Object.freeze(Object.defineProperty({__proto__:null,sortRowNodesByOrder:zI,traverseNodesWithKey:$I},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function UC(i){var t=new Set;return i.forEach(function(e){return t.add(e)}),t}var YI=Object.freeze(Object.defineProperty({__proto__:null,convertToSet:UC},Symbol.toStringTag,{value:"Module"}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Tt=globalThis&&globalThis.__assign||function(){return Tt=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Tt.apply(this,arguments)},XI=Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt(Tt({},PI),DI),LF),gI),pI),GI),SI),BF),xF),kI),dF),WI),jI),qF),UI),NI),CF),KI),YI),KF),_=XI;/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Bt=function(){function i(t,e){t===void 0&&(t=0),e===void 0&&(e=1),this.nextValue=t,this.step=e}return i.prototype.next=function(){var t=this.nextValue;return this.nextValue+=this.step,t},i.prototype.peek=function(){return this.nextValue},i.prototype.skip=function(t){this.nextValue+=t},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Ka;(function(i){i[i.IN_PROGRESS=0]="IN_PROGRESS",i[i.RESOLVED=1]="RESOLVED"})(Ka||(Ka={}));var ct=function(){function i(t){var e=this;this.status=Ka.IN_PROGRESS,this.resolution=null,this.waiters=[],t(function(r){return e.onDone(r)},function(r){return e.onReject(r)})}return i.all=function(t){return new i(function(e){var r=t.length,n=new Array(r);t.forEach(function(o,a){o.then(function(s){n[a]=s,r--,r===0&&e(n)})})})},i.resolve=function(t){return t===void 0&&(t=null),new i(function(e){return e(t)})},i.prototype.then=function(t){var e=this;return new i(function(r){e.status===Ka.RESOLVED?r(t(e.resolution)):e.waiters.push(function(n){return r(t(n))})})},i.prototype.resolveNow=function(t,e){return this.status===Ka.RESOLVED?e(this.resolution):t},i.prototype.onDone=function(t){this.status=Ka.RESOLVED,this.resolution=t,this.waiters.forEach(function(e){return e(t)})},i.prototype.onReject=function(t){console.warn("TBI")},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var qI=function(){function i(){this.timestamp=new Date().getTime()}return i.prototype.print=function(t){var e=new Date().getTime()-this.timestamp;console.info(t+" = "+e),this.timestamp=new Date().getTime()},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ZI=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Qm=globalThis&&globalThis.__assign||function(){return Qm=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Qm.apply(this,arguments)},ou=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},zn;(function(i){i[i.NOTHING=0]="NOTHING",i[i.WAITING_TO_SHOW=1]="WAITING_TO_SHOW",i[i.SHOWING=2]="SHOWING"})(zn||(zn={}));var nf=function(i){ZI(t,i);function t(e){var r=i.call(this)||this;return r.DEFAULT_SHOW_TOOLTIP_DELAY=2e3,r.DEFAULT_HIDE_TOOLTIP_DELAY=1e4,r.SHOW_QUICK_TOOLTIP_DIFF=1e3,r.FADE_OUT_TOOLTIP_TIMEOUT=1e3,r.state=zn.NOTHING,r.tooltipInstanceCount=0,r.tooltipMouseTrack=!1,r.parentComp=e,r}return t.prototype.postConstruct=function(){this.tooltipShowDelay=this.gridOptionsWrapper.getTooltipDelay("show")||this.DEFAULT_SHOW_TOOLTIP_DELAY,this.tooltipHideDelay=this.gridOptionsWrapper.getTooltipDelay("hide")||this.DEFAULT_HIDE_TOOLTIP_DELAY,this.tooltipMouseTrack=this.gridOptionsWrapper.isTooltipMouseTrack();var e=this.parentComp.getGui();this.addManagedListener(e,"mouseenter",this.onMouseEnter.bind(this)),this.addManagedListener(e,"mouseleave",this.onMouseLeave.bind(this)),this.addManagedListener(e,"mousemove",this.onMouseMove.bind(this)),this.addManagedListener(e,"mousedown",this.onMouseDown.bind(this)),this.addManagedListener(e,"keydown",this.onKeyDown.bind(this))},t.prototype.destroy=function(){this.setToDoNothing(),i.prototype.destroy.call(this)},t.prototype.onMouseEnter=function(e){if(!Bl()&&this.state==zn.NOTHING){var r=this.isLastTooltipHiddenRecently()?200:this.tooltipShowDelay;this.showTooltipTimeoutId=window.setTimeout(this.showTooltip.bind(this),r),this.lastMouseEvent=e,this.state=zn.WAITING_TO_SHOW}},t.prototype.onMouseLeave=function(){this.setToDoNothing()},t.prototype.onKeyDown=function(){this.setToDoNothing()},t.prototype.setToDoNothing=function(){this.state===zn.SHOWING&&this.hideTooltip(),this.clearTimeouts(),this.state=zn.NOTHING},t.prototype.onMouseMove=function(e){this.lastMouseEvent=e,this.tooltipMouseTrack&&this.state===zn.SHOWING&&this.tooltipComp&&this.positionTooltipUnderLastMouseEvent()},t.prototype.onMouseDown=function(){this.setToDoNothing()},t.prototype.hideTooltip=function(){this.tooltipComp&&(this.destroyTooltipComp(),t.lastTooltipHideTime=new Date().getTime()),this.state=zn.NOTHING},t.prototype.destroyTooltipComp=function(){var e=this;this.tooltipComp.getGui().classList.add("ag-tooltip-hiding");var r=this.tooltipPopupDestroyFunc,n=this.tooltipComp;window.setTimeout(function(){r(),e.getContext().destroyBean(n)},this.FADE_OUT_TOOLTIP_TIMEOUT),this.tooltipPopupDestroyFunc=void 0,this.tooltipComp=void 0},t.prototype.isLastTooltipHiddenRecently=function(){var e=new Date().getTime(),r=t.lastTooltipHideTime;return e-r<this.SHOW_QUICK_TOOLTIP_DIFF},t.prototype.showTooltip=function(){var e=Qm({},this.parentComp.getTooltipParams());if(!q(e.value)){this.setToDoNothing();return}this.state=zn.SHOWING,this.tooltipInstanceCount++;var r=this.newTooltipComponentCallback.bind(this,this.tooltipInstanceCount),n=this.userComponentFactory.getTooltipCompDetails(e);n.newAgStackInstance().then(r)},t.prototype.newTooltipComponentCallback=function(e,r){var n=this.state!==zn.SHOWING||this.tooltipInstanceCount!==e;if(n){this.getContext().destroyBean(r);return}var o=r.getGui();this.tooltipComp=r,o.classList.contains("ag-tooltip")||o.classList.add("ag-tooltip-custom");var a=this.gridOptionsWrapper.getLocaleTextFunc(),s=this.popupService.addPopup({eChild:o,ariaLabel:a("ariaLabelTooltip","Tooltip")});s&&(this.tooltipPopupDestroyFunc=s.hideFunc),this.positionTooltipUnderLastMouseEvent(),this.hideTooltipTimeoutId=window.setTimeout(this.hideTooltip.bind(this),this.tooltipHideDelay)},t.prototype.positionTooltipUnderLastMouseEvent=function(){this.popupService.positionPopupUnderMouseEvent({type:"tooltip",mouseEvent:this.lastMouseEvent,ePopup:this.tooltipComp.getGui(),nudgeY:18})},t.prototype.clearTimeouts=function(){this.showTooltipTimeoutId&&(window.clearTimeout(this.showTooltipTimeoutId),this.showTooltipTimeoutId=void 0),this.hideTooltipTimeoutId&&(window.clearTimeout(this.hideTooltipTimeoutId),this.hideTooltipTimeoutId=void 0)},ou([y("popupService")],t.prototype,"popupService",void 0),ou([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),ou([y("columnApi")],t.prototype,"columnApi",void 0),ou([y("gridApi")],t.prototype,"gridApi",void 0),ou([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var QI=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),yg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},JI=new Bt,sE=function(){function i(t){this.cssClassStates={},this.getGui=t}return i.prototype.addCssClass=function(t){var e=this,r=(t||"").split(" ");if(r.length>1){r.forEach(function(o){return e.addCssClass(o)});return}var n=this.cssClassStates[t]!==!0;n&&t.length&&(this.getGui().classList.add(t),this.cssClassStates[t]=!0)},i.prototype.removeCssClass=function(t){var e=this,r=(t||"").split(" ");if(r.length>1){r.forEach(function(o){return e.removeCssClass(o)});return}var n=this.cssClassStates[t]!==!1;n&&t.length&&(this.getGui().classList.remove(t),this.cssClassStates[t]=!1)},i.prototype.containsCssClass=function(t){return this.getGui().classList.contains(t)},i.prototype.addOrRemoveCssClass=function(t,e){var r=this,n=(t||"").split(" ");if(n.length>1){n.forEach(function(a){return r.addOrRemoveCssClass(a,e)});return}var o=this.cssClassStates[t]!==e;o&&t.length&&(this.getGui().classList.toggle(t,e),this.cssClassStates[t]=e)},i}(),ee=function(i){QI(t,i);function t(e){var r=i.call(this)||this;return r.displayed=!0,r.visible=!0,r.compId=JI.next(),r.cssClassManager=new sE(function(){return r.eGui}),e&&r.setTemplate(e),r}return t.prototype.preConstructOnComponent=function(){this.usingBrowserTooltips=this.gridOptionsWrapper.isEnableBrowserTooltips()},t.prototype.getCompId=function(){return this.compId},t.prototype.getTooltipParams=function(){return{value:this.tooltipText,location:"UNKNOWN"}},t.prototype.setTooltip=function(e){var r=this,n=function(){r.usingBrowserTooltips?r.getGui().removeAttribute("title"):r.tooltipFeature=r.destroyBean(r.tooltipFeature)},o=function(){r.usingBrowserTooltips?r.getGui().setAttribute("title",r.tooltipText):r.tooltipFeature=r.createBean(new nf(r))};this.tooltipText!=e&&(this.tooltipText&&n(),e!=null&&(this.tooltipText=e,this.tooltipText&&o()))},t.prototype.createChildComponentsFromTags=function(e,r){var n=this,o=GO(e.childNodes);o.forEach(function(a){if(a instanceof HTMLElement){var s=n.createComponentFromElement(a,function(u){var c=u.getGui();c&&n.copyAttributesFromNode(a,u.getGui())},r);if(s){if(s.addItems&&a.children.length){n.createChildComponentsFromTags(a,r);var l=Array.prototype.slice.call(a.children);s.addItems(l)}n.swapComponentForNode(s,e,a)}else a.childNodes&&n.createChildComponentsFromTags(a,r)}})},t.prototype.createComponentFromElement=function(e,r,n){var o=e.nodeName,a=n?n[e.getAttribute("ref")]:void 0,s=this.agStackComponentsRegistry.getComponentClass(o);if(s){t.elementGettingCreated=e;var l=new s(a);return l.setParentComponent(this),this.createBean(l,null,r),l}return null},t.prototype.copyAttributesFromNode=function(e,r){kO(e.attributes,function(n,o){return r.setAttribute(n,o)})},t.prototype.swapComponentForNode=function(e,r,n){var o=e.getGui();r.replaceChild(o,n),r.insertBefore(document.createComment(n.nodeName),o),this.addDestroyFunc(this.destroyBean.bind(this,e)),this.swapInComponentForQuerySelectors(e,n)},t.prototype.swapInComponentForQuerySelectors=function(e,r){var n=this;this.iterateOverQuerySelectors(function(o){n[o.attributeName]===r&&(n[o.attributeName]=e)})},t.prototype.iterateOverQuerySelectors=function(e){for(var r=Object.getPrototypeOf(this);r!=null;){var n=r.__agComponentMetaData,o=Qd(r.constructor);n&&n[o]&&n[o].querySelectors&&n[o].querySelectors.forEach(function(a){return e(a)}),r=Object.getPrototypeOf(r)}},t.prototype.setTemplate=function(e,r){var n=Kt(e);this.setTemplateFromElement(n,r)},t.prototype.setTemplateFromElement=function(e,r){this.eGui=e,this.eGui.__agComponent=this,this.wireQuerySelectors(),this.getContext()&&this.createChildComponentsFromTags(this.getGui(),r)},t.prototype.createChildComponentsPreConstruct=function(){this.getGui()&&this.createChildComponentsFromTags(this.getGui())},t.prototype.wireQuerySelectors=function(){var e=this;if(!!this.eGui){var r=this;this.iterateOverQuerySelectors(function(n){var o=function(l){return r[n.attributeName]=l},a=n.refSelector&&e.eGui.getAttribute("ref")===n.refSelector;if(a)o(e.eGui);else{var s=e.eGui.querySelector(n.querySelector);s&&o(s.__agComponent||s)}})}},t.prototype.getGui=function(){return this.eGui},t.prototype.getFocusableElement=function(){return this.eGui},t.prototype.setParentComponent=function(e){this.parentComponent=e},t.prototype.getParentComponent=function(){return this.parentComponent},t.prototype.setGui=function(e){this.eGui=e},t.prototype.queryForHtmlElement=function(e){return this.eGui.querySelector(e)},t.prototype.queryForHtmlInputElement=function(e){return this.eGui.querySelector(e)},t.prototype.appendChild=function(e,r){if(r||(r=this.eGui),e!=null)if($c(e))r.appendChild(e);else{var n=e;r.appendChild(n.getGui()),this.addDestroyFunc(this.destroyBean.bind(this,n))}},t.prototype.isDisplayed=function(){return this.displayed},t.prototype.setVisible=function(e){e!==this.visible&&(this.visible=e,MO(this.eGui,e))},t.prototype.setDisplayed=function(e){if(e!==this.displayed){this.displayed=e,pt(this.eGui,e);var r={type:t.EVENT_DISPLAYED_CHANGED,visible:this.displayed};this.dispatchEvent(r)}},t.prototype.destroy=function(){this.tooltipFeature&&(this.tooltipFeature=this.destroyBean(this.tooltipFeature)),i.prototype.destroy.call(this)},t.prototype.addGuiEventListener=function(e,r){var n=this;this.eGui.addEventListener(e,r),this.addDestroyFunc(function(){return n.eGui.removeEventListener(e,r)})},t.prototype.addCssClass=function(e){this.cssClassManager.addCssClass(e)},t.prototype.removeCssClass=function(e){this.cssClassManager.removeCssClass(e)},t.prototype.containsCssClass=function(e){return this.cssClassManager.containsCssClass(e)},t.prototype.addOrRemoveCssClass=function(e,r){this.cssClassManager.addOrRemoveCssClass(e,r)},t.prototype.getAttribute=function(e){var r=this.eGui;return r?r.getAttribute(e):null},t.prototype.getRefElement=function(e){return this.queryForHtmlElement('[ref="'+e+'"]')},t.EVENT_DISPLAYED_CHANGED="displayedChanged",yg([y("agStackComponentsRegistry")],t.prototype,"agStackComponentsRegistry",void 0),yg([ss],t.prototype,"preConstructOnComponent",null),yg([ss],t.prototype,"createChildComponentsPreConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function eL(i){return lE.bind(this,i,void 0)}function I(i){return lE.bind(this,"[ref="+i+"]",i)}function lE(i,t,e,r,n){if(i===null){console.error("AG Grid: QuerySelector selector should not be null");return}if(typeof n=="number"){console.error("AG Grid: QuerySelector should be on an attribute");return}tL(e,"querySelectors",{attributeName:r,querySelector:i,refSelector:t})}function tL(i,t,e){var r=rL(i,Qd(i.constructor));r[t]||(r[t]=[]),r[t].push(e)}function rL(i,t){return i.__agComponentMetaData||(i.__agComponentMetaData={}),i.__agComponentMetaData[t]||(i.__agComponentMetaData[t]={}),i.__agComponentMetaData[t]}/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var nL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),i_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},iL=function(i){nL(t,i);function t(){return i.call(this,`
            <div class="ag-floating-filter-input" role="presentation">
                <ag-input-text-field ref="eFloatingFilterText"></ag-input-text-field>
            </div>`)||this}return t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.init=function(e){this.params=e;var r=this.columnModel.getDisplayNameForColumn(e.column,"header",!0),n=this.gridOptionsWrapper.getLocaleTextFunc();this.eFloatingFilterText.setDisabled(!0).setInputAriaLabel(r+" "+n("ariaFilterInput","Filter Input"))},t.prototype.onParentModelChanged=function(e){var r=this;if(!e){this.eFloatingFilterText.setValue("");return}this.params.parentFilterInstance(function(n){if(n.getModelAsString){var o=n.getModelAsString(e);r.eFloatingFilterText.setValue(o)}})},i_([I("eFloatingFilterText")],t.prototype,"eFloatingFilterText",void 0),i_([y("columnModel")],t.prototype,"columnModel",void 0),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var uE=function(){function i(t,e,r,n){var o=this;this.alive=!0,this.context=t,this.eParent=n;var a=e.getDateCompDetails(r),s=a.newAgStackInstance();s.then(function(l){if(!o.alive){t.destroyBean(l);return}o.dateComp=l,l&&(n.appendChild(l.getGui()),l.afterGuiAttached&&l.afterGuiAttached(),o.tempValue&&l.setDate(o.tempValue),o.disabled!=null&&o.setDateCompDisabled(o.disabled))})}return i.prototype.destroy=function(){this.alive=!1,this.dateComp=this.context.destroyBean(this.dateComp)},i.prototype.getDate=function(){return this.dateComp?this.dateComp.getDate():this.tempValue},i.prototype.setDate=function(t){this.dateComp?this.dateComp.setDate(t):this.tempValue=t},i.prototype.setDisabled=function(t){this.dateComp?this.setDateCompDisabled(t):this.disabled=t},i.prototype.setDisplayed=function(t){pt(this.eParent,t)},i.prototype.setInputPlaceholder=function(t){this.dateComp&&this.dateComp.setInputPlaceholder&&this.dateComp.setInputPlaceholder(t)},i.prototype.setInputAriaLabel=function(t){this.dateComp&&this.dateComp.setInputAriaLabel&&this.dateComp.setInputAriaLabel(t)},i.prototype.afterGuiAttached=function(t){this.dateComp&&typeof this.dateComp.afterGuiAttached=="function"&&this.dateComp.afterGuiAttached(t)},i.prototype.setDateCompDisabled=function(t){this.dateComp!=null&&this.dateComp.setDisabled!=null&&this.dateComp.setDisabled(t)},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Jm=globalThis&&globalThis.__assign||function(){return Jm=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Jm.apply(this,arguments)},cE=function(){function i(){this.customFilterOptions={}}return i.prototype.init=function(t,e){this.filterOptions=t.filterOptions||e,this.mapCustomOptions(),this.selectDefaultItem(t),this.checkForDeprecatedParams()},i.prototype.checkForDeprecatedParams=function(){this.filterOptions.some(function(t){return typeof t!="string"&&t.test!=null})&&console.warn("AG Grid: [IFilterOptionDef] since v26.2.0, test() has been replaced with predicate()."),this.filterOptions.some(function(t){return typeof t!="string"&&t.hideFilterInput!=null})&&console.warn("AG Grid: [IFilterOptionDef] since v26.2.0, useOfHideFilterInput has been replaced with numberOfInputs.")},i.prototype.getFilterOptions=function(){return this.filterOptions},i.prototype.mapCustomOptions=function(){var t=this;!this.filterOptions||this.filterOptions.forEach(function(e){if(typeof e!="string"){var r=[["displayKey"],["displayName"],["predicate","test"]],n=function(s){return s.some(function(l){return e[l]!=null})?!0:(console.warn("AG Grid: ignoring FilterOptionDef as it doesn't contain one of '"+s+"'"),!1)};if(!r.every(n)){t.filterOptions=t.filterOptions.filter(function(s){return s===e})||[];return}var o=e.test,a=Jm({},e);o!=null&&e.predicate==null&&(a.predicate=function(s,l){return o(s[0],l)},delete a.test),a.hideFilterInput&&a.numberOfInputs==null&&(a.numberOfInputs=0,delete a.hideFilterInput),t.customFilterOptions[e.displayKey]=a}})},i.prototype.selectDefaultItem=function(t){if(t.defaultOption)this.defaultOption=t.defaultOption;else if(this.filterOptions.length>=1){var e=this.filterOptions[0];typeof e=="string"?this.defaultOption=e:e.displayKey?this.defaultOption=e.displayKey:console.warn("AG Grid: invalid FilterOptionDef supplied as it doesn't contain a 'displayKey'")}else console.warn("AG Grid: no filter options for filter")},i.prototype.getDefaultOption=function(){return this.defaultOption},i.prototype.getCustomOption=function(t){return this.customFilterOptions[t]},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var oL={applyFilter:"Apply",clearFilter:"Clear",resetFilter:"Reset",cancelFilter:"Cancel",textFilter:"Text Filter",numberFilter:"Number Filter",dateFilter:"Date Filter",setFilter:"Set Filter",filterOoo:"Filter...",empty:"Choose One",equals:"Equals",notEqual:"Not equal",lessThan:"Less than",greaterThan:"Greater than",inRange:"In range",inRangeStart:"From",inRangeEnd:"To",lessThanOrEqual:"Less than or equals",greaterThanOrEqual:"Greater than or equals",contains:"Contains",notContains:"Not contains",startsWith:"Starts with",endsWith:"Ends with",blank:"Blank",notBlank:"Not blank",andCondition:"AND",orCondition:"OR",dateFormatOoo:"yyyy-mm-dd"};/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var j=function(){function i(){}return i.BACKSPACE="Backspace",i.TAB="Tab",i.ENTER="Enter",i.ESCAPE="Escape",i.SPACE=" ",i.LEFT="ArrowLeft",i.UP="ArrowUp",i.RIGHT="ArrowRight",i.DOWN="ArrowDown",i.DELETE="Delete",i.F2="F2",i.PAGE_UP="PageUp",i.PAGE_DOWN="PageDown",i.PAGE_HOME="Home",i.PAGE_END="End",i.A="KeyA",i.C="KeyC",i.V="KeyV",i.D="KeyD",i.Z="KeyZ",i.Y="KeyY",i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var aL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ey=globalThis&&globalThis.__assign||function(){return ey=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},ey.apply(this,arguments)},o_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ai=function(i){aL(t,i);function t(e,r){r===void 0&&(r={});var n=i.call(this)||this;return n.eFocusableElement=e,n.callbacks=r,n.callbacks=ey({shouldStopEventPropagation:function(){return!1},onTabKeyDown:function(o){if(!o.defaultPrevented){var a=n.focusService.findNextFocusableElement(n.eFocusableElement,!1,o.shiftKey);!a||(a.focus(),o.preventDefault())}}},r),n}return t.prototype.postConstruct=function(){this.eFocusableElement.classList.add(t.FOCUS_MANAGED_CLASS),this.addKeyDownListeners(this.eFocusableElement),this.callbacks.onFocusIn&&this.addManagedListener(this.eFocusableElement,"focusin",this.callbacks.onFocusIn),this.callbacks.onFocusOut&&this.addManagedListener(this.eFocusableElement,"focusout",this.callbacks.onFocusOut)},t.prototype.addKeyDownListeners=function(e){var r=this;this.addManagedListener(e,"keydown",function(n){if(!(n.defaultPrevented||Ai(n))){if(r.callbacks.shouldStopEventPropagation(n)){Pl(n);return}n.key===j.TAB?r.callbacks.onTabKeyDown(n):r.callbacks.handleKeyDown&&r.callbacks.handleKeyDown(n)}})},t.FOCUS_MANAGED_CLASS="ag-focus-managed",o_([y("focusService")],t.prototype,"focusService",void 0),o_([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var sL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Cg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},us=function(i){sL(t,i);function t(e){var r=i.call(this)||this;return r.filterNameKey=e,r.applyActive=!1,r.hidePopup=null,r.appliedModel=null,r}return t.prototype.postConstruct=function(){this.resetTemplate(),this.createManagedBean(new ai(this.getFocusableElement(),{handleKeyDown:this.handleKeyDown.bind(this)}))},t.prototype.handleKeyDown=function(e){},t.prototype.getFilterTitle=function(){return this.translate(this.filterNameKey)},t.prototype.isFilterActive=function(){return!!this.appliedModel},t.prototype.resetTemplate=function(e){var r=`
            <div class="ag-filter-wrapper">
                <div class="ag-filter-body-wrapper ag-`+this.getCssIdentifier()+`-body-wrapper">
                    `+this.createBodyTemplate()+`
                </div>
            </div>`;this.setTemplate(r,e)},t.prototype.isReadOnly=function(){return!!this.providedFilterParams.readOnly},t.prototype.init=function(e){var r=this;this.setParams(e),this.resetUiToDefaults(!0).then(function(){r.updateUiVisibility(),r.setupOnBtApplyDebounce()})},t.prototype.setParams=function(e){this.providedFilterParams=e,this.applyActive=t.isUseApplyButton(e),this.createButtonPanel()},t.prototype.createButtonPanel=function(){var e=this,r=this.providedFilterParams.buttons;if(!(!r||r.length<1||this.isReadOnly())){var n=document.createElement("div");n.classList.add("ag-filter-apply-panel");var o=function(a){var s,l;switch(a){case"apply":s=e.translate("applyFilter"),l=function(c){return e.onBtApply(!1,!1,c)};break;case"clear":s=e.translate("clearFilter"),l=function(){return e.onBtClear()};break;case"reset":s=e.translate("resetFilter"),l=function(){return e.onBtReset()};break;case"cancel":s=e.translate("cancelFilter"),l=function(c){e.onBtCancel(c)};break;default:console.warn("AG Grid: Unknown button type specified");return}var u=Kt(`<button
                    type="button"
                    ref="`+a+`FilterButton"
                    class="ag-standard-button ag-filter-apply-panel-button"
                >`+s+`
                </button>`);n.appendChild(u),e.addManagedListener(u,"click",l)};UC(r).forEach(function(a){return o(a)}),this.getGui().appendChild(n)}},t.prototype.getDefaultDebounceMs=function(){return 0},t.prototype.setupOnBtApplyDebounce=function(){var e=t.getDebounceMs(this.providedFilterParams,this.getDefaultDebounceMs());this.onBtApplyDebounce=uo(this.onBtApply.bind(this),e)},t.prototype.getModel=function(){return this.appliedModel?this.appliedModel:null},t.prototype.setModel=function(e){var r=this,n=e!=null?this.setModelIntoUi(e):this.resetUiToDefaults();return n.then(function(){r.updateUiVisibility(),r.applyModel()})},t.prototype.onBtCancel=function(e){var r=this,n=this.getModel(),o=function(){r.onUiChanged(!1,"prevent"),r.providedFilterParams.closeOnApply&&r.close(e)};n!=null?this.setModelIntoUi(n).then(o):this.resetUiToDefaults().then(o)},t.prototype.onBtClear=function(){var e=this;this.resetUiToDefaults().then(function(){return e.onUiChanged()})},t.prototype.onBtReset=function(){this.onBtClear(),this.onBtApply()},t.prototype.applyModel=function(){var e=this.getModelFromUi();if(!this.isModelValid(e))return!1;var r=this.appliedModel;return this.appliedModel=e,!this.areModelsEqual(r,e)},t.prototype.isModelValid=function(e){return!0},t.prototype.onBtApply=function(e,r,n){e===void 0&&(e=!1),r===void 0&&(r=!1),this.applyModel()&&this.providedFilterParams.filterChangedCallback({afterFloatingFilter:e,afterDataChange:r});var o=this.providedFilterParams.closeOnApply;o&&this.applyActive&&!e&&!r&&this.close(n)},t.prototype.onNewRowsLoaded=function(){},t.prototype.close=function(e){if(!!this.hidePopup){var r=e,n=r&&r.key,o;(n==="Enter"||n==="Space")&&(o={keyboardEvent:r}),this.hidePopup(o),this.hidePopup=null}},t.prototype.onUiChanged=function(e,r){if(e===void 0&&(e=!1),this.updateUiVisibility(),this.providedFilterParams.filterModifiedCallback(),this.applyActive&&!this.isReadOnly){var n=this.isModelValid(this.getModelFromUi());Al(this.getRefElement("applyFilterButton"),!n)}e&&!r||r==="immediately"?this.onBtApply(e):(!this.applyActive&&!r||r==="debounce")&&this.onBtApplyDebounce()},t.prototype.afterGuiAttached=function(e){e!=null&&(this.hidePopup=e.hidePopup)},t.getDebounceMs=function(e,r){return t.isUseApplyButton(e)?(e.debounceMs!=null&&console.warn("AG Grid: debounceMs is ignored when apply button is present"),0):e.debounceMs!=null?e.debounceMs:r},t.isUseApplyButton=function(e){return!!e.buttons&&e.buttons.indexOf("apply")>=0},t.prototype.destroy=function(){this.hidePopup=null,i.prototype.destroy.call(this)},t.prototype.translate=function(e){var r=this.gridOptionsWrapper.getLocaleTextFunc();return r(e,oL[e])},t.prototype.getCellValue=function(e){var r=this.providedFilterParams,n=r.api,o=r.colDef,a=r.column,s=r.columnApi,l=r.context;return this.providedFilterParams.valueGetter({api:n,colDef:o,column:a,columnApi:s,context:l,data:e.data,getValue:function(u){return e.data[u]},node:e})},Cg([y("rowModel")],t.prototype,"rowModel",void 0),Cg([y("valueService")],t.prototype,"valueService",void 0),Cg([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var lL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),uL=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},zC=function(i){lL(t,i);function t(e,r){var n=i.call(this,r)||this;return n.labelSeparator="",n.labelAlignment="left",n.label="",n.config=e||{},n}return t.prototype.postConstruct=function(){this.addCssClass("ag-labeled"),this.eLabel.classList.add("ag-label");var e=this.config,r=e.labelSeparator,n=e.label,o=e.labelWidth,a=e.labelAlignment;r!=null&&this.setLabelSeparator(r),n!=null&&this.setLabel(n),o!=null&&this.setLabelWidth(o),this.setLabelAlignment(a||this.labelAlignment),this.refreshLabel()},t.prototype.refreshLabel=function(){gn(this.eLabel),typeof this.label=="string"?this.eLabel.innerText=this.label+this.labelSeparator:this.label&&this.eLabel.appendChild(this.label),this.label===""?(this.eLabel.classList.add("ag-hidden"),_r(this.eLabel,"presentation")):(this.eLabel.classList.remove("ag-hidden"),_r(this.eLabel,null))},t.prototype.setLabelSeparator=function(e){return this.labelSeparator===e?this:(this.labelSeparator=e,this.label!=null&&this.refreshLabel(),this)},t.prototype.getLabelId=function(){return this.eLabel.id=this.eLabel.id||"ag-"+this.getCompId()+"-label",this.eLabel.id},t.prototype.getLabel=function(){return this.label},t.prototype.setLabel=function(e){return this.label===e?this:(this.label=e,this.refreshLabel(),this)},t.prototype.setLabelAlignment=function(e){var r=this.getGui(),n=r.classList;return n.toggle("ag-label-align-left",e==="left"),n.toggle("ag-label-align-right",e==="right"),n.toggle("ag-label-align-top",e==="top"),this},t.prototype.setLabelWidth=function(e){return this.label==null?this:(zc(this.eLabel,e),this)},uL([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var cL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ei=function(i){cL(t,i);function t(e,r,n){var o=i.call(this,e,r)||this;return o.className=n,o.disabled=!1,o}return t.prototype.postConstruct=function(){i.prototype.postConstruct.call(this),this.className&&this.addCssClass(this.className)},t.prototype.onValueChange=function(e){var r=this;return this.addManagedListener(this,t.EVENT_CHANGED,function(){return e(r.getValue())}),this},t.prototype.getWidth=function(){return this.getGui().clientWidth},t.prototype.setWidth=function(e){return Li(this.getGui(),e),this},t.prototype.getPreviousValue=function(){return this.previousValue},t.prototype.getValue=function(){return this.value},t.prototype.setValue=function(e,r){return this.value===e?this:(this.previousValue=this.value,this.value=e,r||this.dispatchEvent({type:t.EVENT_CHANGED}),this)},t.prototype.setDisabled=function(e){e=!!e;var r=this.getGui();return Al(r,e),r.classList.toggle("ag-disabled",e),this.disabled=e,this},t.prototype.isDisabled=function(){return!!this.disabled},t.EVENT_CHANGED="valueChange",t}(zC);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var pL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),fp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},pE=function(i){pL(t,i);function t(e,r,n,o){var a=i.call(this,e,`<div class="ag-picker-field" role="presentation">
                <div ref="eLabel"></div>
                <div ref="eWrapper"
                    class="ag-wrapper ag-picker-field-wrapper"
                    tabIndex="-1"
                    aria-expanded="false"
                    `+(o?'role="'+o+'"':"")+`
                >
                    <div ref="eDisplayField" class="ag-picker-field-display"></div>
                    <div ref="eIcon" class="ag-picker-field-icon" aria-hidden="true"></div>
                </div>
            </div>`,r)||this;return a.pickerIcon=n,a.isPickerDisplayed=!1,a.isDestroyingPicker=!1,a.skipClick=!1,a}return t.prototype.postConstruct=function(){var e=this;i.prototype.postConstruct.call(this);var r=this.getCompId()+"-display";this.eDisplayField.setAttribute("id",r),NC(this.eWrapper,r);var n=function(){if(e.skipClick){e.skipClick=!1;return}e.isDisabled()||(e.pickerComponent=e.showPicker())},o=this.getGui();if(this.addManagedListener(o,"mousedown",function(s){!e.skipClick&&e.pickerComponent&&e.pickerComponent.isAlive()&&jl(e.pickerComponent.getGui())&&o.contains(s.target)&&(e.skipClick=!0)}),this.addManagedListener(o,"keydown",function(s){switch(s.key){case j.UP:case j.DOWN:case j.ENTER:case j.SPACE:n();case j.ESCAPE:e.isPickerDisplayed&&s.preventDefault();break}}),this.addManagedListener(this.eWrapper,"click",n),this.addManagedListener(this.eLabel,"click",n),this.pickerIcon){var a=Ft(this.pickerIcon,this.gridOptionsWrapper);a&&this.eIcon.appendChild(a)}},t.prototype.refreshLabel=function(){q(this.getLabel())?LC(this.eWrapper,this.getLabelId()):this.eWrapper.removeAttribute("aria-labelledby"),i.prototype.refreshLabel.call(this)},t.prototype.setAriaLabel=function(e){return Cs(this.eWrapper,e),this},t.prototype.setInputWidth=function(e){return zc(this.eWrapper,e),this},t.prototype.getFocusableElement=function(){return this.eWrapper},fp([I("eLabel")],t.prototype,"eLabel",void 0),fp([I("eWrapper")],t.prototype,"eWrapper",void 0),fp([I("eDisplayField")],t.prototype,"eDisplayField",void 0),fp([I("eIcon")],t.prototype,"eIcon",void 0),t}(ei);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var hL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),dL=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},a_=function(i){hL(t,i);function t(e){e===void 0&&(e="default");var r=i.call(this,'<div class="ag-list ag-'+e+'-list" role="listbox"></div>')||this;return r.cssIdentifier=e,r.options=[],r.itemEls=[],r}return t.prototype.init=function(){this.addManagedListener(this.getGui(),"keydown",this.handleKeyDown.bind(this))},t.prototype.handleKeyDown=function(e){var r=e.key;switch(r){case j.ENTER:if(!this.highlightedEl)this.setValue(this.getValue());else{var n=this.itemEls.indexOf(this.highlightedEl);this.setValueByIndex(n)}break;case j.DOWN:case j.UP:var o=r===j.DOWN,a=void 0;if(e.preventDefault(),!this.highlightedEl)a=this.itemEls[o?0:this.itemEls.length-1];else{var s=this.itemEls.indexOf(this.highlightedEl),l=s+(o?1:-1);l=Math.min(Math.max(l,0),this.itemEls.length-1),a=this.itemEls[l]}this.highlightItem(a);break}},t.prototype.addOptions=function(e){var r=this;return e.forEach(function(n){return r.addOption(n)}),this},t.prototype.addOption=function(e){var r=e.value,n=e.text,o=Ii(n||r);return this.options.push({value:r,text:o}),this.renderOption(r,o),this.updateIndices(),this},t.prototype.updateIndices=function(){var e=this.getGui().querySelectorAll(".ag-list-item");e.forEach(function(r,n){WC(r,n+1),kC(r,e.length)})},t.prototype.renderOption=function(e,r){var n=this,o=document.createElement("div");_r(o,"option"),o.classList.add("ag-list-item","ag-"+this.cssIdentifier+"-list-item"),o.innerHTML="<span>"+r+"</span>",o.tabIndex=-1,this.itemEls.push(o),this.addManagedListener(o,"mouseover",function(){return n.highlightItem(o)}),this.addManagedListener(o,"mouseleave",function(){return n.clearHighlighted()}),this.addManagedListener(o,"click",function(){return n.setValue(e)}),this.getGui().appendChild(o)},t.prototype.setValue=function(e,r){if(this.value===e)return this.fireItemSelected(),this;if(e==null)return this.reset(),this;var n=this.options.findIndex(function(a){return a.value===e});if(n!==-1){var o=this.options[n];this.value=o.value,this.displayValue=o.text!=null?o.text:o.value,this.highlightItem(this.itemEls[n]),r||this.fireChangeEvent()}return this},t.prototype.setValueByIndex=function(e){return this.setValue(this.options[e].value)},t.prototype.getValue=function(){return this.value},t.prototype.getDisplayValue=function(){return this.displayValue},t.prototype.refreshHighlighted=function(){var e=this;this.clearHighlighted();var r=this.options.findIndex(function(n){return n.value===e.value});r!==-1&&this.highlightItem(this.itemEls[r])},t.prototype.reset=function(){this.value=null,this.displayValue=null,this.clearHighlighted(),this.fireChangeEvent()},t.prototype.highlightItem=function(e){!e.offsetParent||(this.clearHighlighted(),this.highlightedEl=e,this.highlightedEl.classList.add(t.ACTIVE_CLASS),Dl(this.highlightedEl,!0),this.highlightedEl.focus())},t.prototype.clearHighlighted=function(){!this.highlightedEl||!this.highlightedEl.offsetParent||(this.highlightedEl.classList.remove(t.ACTIVE_CLASS),Dl(this.highlightedEl,!1),this.highlightedEl=null)},t.prototype.fireChangeEvent=function(){this.dispatchEvent({type:ei.EVENT_CHANGED}),this.fireItemSelected()},t.prototype.fireItemSelected=function(){this.dispatchEvent({type:t.EVENT_ITEM_SELECTED})},t.EVENT_ITEM_SELECTED="selectedItem",t.ACTIVE_CLASS="ag-active-item",dL([B],t.prototype,"init",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var fL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),s_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Kc=function(i){fL(t,i);function t(e){return i.call(this,e,"ag-select","smallDown","listbox")||this}return t.prototype.init=function(){var e=this;this.listComponent=this.createBean(new a_("select")),this.listComponent.setParentComponent(this),this.eWrapper.tabIndex=0,this.listComponent.addManagedListener(this.listComponent,a_.EVENT_ITEM_SELECTED,function(){e.hideList&&e.hideList()}),this.listComponent.addManagedListener(this.listComponent,ei.EVENT_CHANGED,function(){e.setValue(e.listComponent.getValue(),!1,!0),e.hideList&&e.hideList()})},t.prototype.showPicker=function(){var e=this,r=this.listComponent.getGui(),n=this.gridOptionsWrapper.getDocument(),o=this.addManagedListener(n.body,"wheel",function(u){!r.contains(u.target)&&e.hideList&&e.hideList()}),a=this.addManagedListener(r,"focusout",function(u){!r.contains(u.relatedTarget)&&e.hideList&&e.hideList()}),s=this.gridOptionsWrapper.getLocaleTextFunc(),l=this.popupService.addPopup({modal:!0,eChild:r,closeOnEsc:!0,closedCallback:function(){e.hideList=null,e.isPickerDisplayed=!1,a(),o(),e.isAlive()&&(dn(e.eWrapper,!1),e.getFocusableElement().focus())},ariaLabel:s("ariaLabelSelectField","Select Field")});return l&&(this.hideList=l.hideFunc),this.isPickerDisplayed=!0,zc(r,rf(this.eWrapper)),dn(this.eWrapper,!0),r.style.maxHeight=Uc(this.popupService.getPopupParent())+"px",r.style.position="absolute",this.popupService.positionPopupUnderComponent({type:"ag-list",eventSource:this.eWrapper,ePopup:r,keepWithinBounds:!0}),this.listComponent.refreshHighlighted(),this.listComponent},t.prototype.addOptions=function(e){var r=this;return e.forEach(function(n){return r.addOption(n)}),this},t.prototype.addOption=function(e){return this.listComponent.addOption(e),this},t.prototype.setValue=function(e,r,n){if(this.value===e)return this;n||this.listComponent.setValue(e,!0);var o=this.listComponent.getValue();return o===this.getValue()?this:(this.eDisplayField.innerHTML=this.listComponent.getDisplayValue(),i.prototype.setValue.call(this,e,r))},t.prototype.destroy=function(){this.hideList&&this.hideList(),this.destroyBean(this.listComponent),i.prototype.destroy.call(this)},s_([y("popupService")],t.prototype,"popupService",void 0),s_([B],t.prototype,"init",null),t}(pE);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var gL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),wg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Ki=function(i){gL(t,i);function t(e,r,n,o){n===void 0&&(n="text"),o===void 0&&(o="input");var a=i.call(this,e,`
            <div role="presentation">
                <div ref="eLabel" class="ag-input-field-label"></div>
                <div ref="eWrapper" class="ag-wrapper ag-input-wrapper" role="presentation">
                    <`+o+' ref="eInput" class="ag-input-field-input"></'+o+`>
                </div>
            </div>`,r)||this;return a.inputType=n,a.displayFieldTag=o,a}return t.prototype.postConstruct=function(){i.prototype.postConstruct.call(this),this.setInputType(),this.eLabel.classList.add(this.className+"-label"),this.eWrapper.classList.add(this.className+"-input-wrapper"),this.eInput.classList.add(this.className+"-input"),this.addCssClass("ag-input-field"),this.eInput.id=this.eInput.id||"ag-"+this.getCompId()+"-input";var e=this.config,r=e.width,n=e.value;r!=null&&this.setWidth(r),n!=null&&this.setValue(n),this.addInputListeners()},t.prototype.refreshLabel=function(){q(this.getLabel())?LC(this.eInput,this.getLabelId()):this.eInput.removeAttribute("aria-labelledby"),i.prototype.refreshLabel.call(this)},t.prototype.addInputListeners=function(){var e=this;this.addManagedListener(this.eInput,"input",function(r){return e.setValue(r.target.value)})},t.prototype.setInputType=function(){this.displayFieldTag==="input"&&this.eInput.setAttribute("type",this.inputType)},t.prototype.getInputElement=function(){return this.eInput},t.prototype.setInputWidth=function(e){return zc(this.eWrapper,e),this},t.prototype.setInputName=function(e){return this.getInputElement().setAttribute("name",e),this},t.prototype.getFocusableElement=function(){return this.eInput},t.prototype.setMaxLength=function(e){var r=this.eInput;return r.maxLength=e,this},t.prototype.setInputPlaceholder=function(e){return lc(this.eInput,"placeholder",e),this},t.prototype.setInputAriaLabel=function(e){return Cs(this.eInput,e),this},t.prototype.setDisabled=function(e){return Al(this.eInput,e),i.prototype.setDisabled.call(this,e)},wg([I("eLabel")],t.prototype,"eLabel",void 0),wg([I("eWrapper")],t.prototype,"eWrapper",void 0),wg([I("eInput")],t.prototype,"eInput",void 0),t}(ei);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var vL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ma=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},fr;(function(i){i[i.One=0]="One",i[i.Two=1]="Two"})(fr||(fr={}));var Ot=function(i){vL(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getNumberOfInputs=function(e){var r=this.optionsFactory.getCustomOption(e);if(r){var n=r.numberOfInputs;return n!=null?n:1}var o=[t.EMPTY,t.NOT_BLANK,t.BLANK];return e&&o.indexOf(e)>=0?0:e===t.IN_RANGE?2:1},t.prototype.onFloatingFilterChanged=function(e,r){this.setTypeFromFloatingFilter(e),this.setValueFromFloatingFilter(r),this.onUiChanged(!0)},t.prototype.setTypeFromFloatingFilter=function(e){this.eType1.setValue(e),this.eType2.setValue(this.optionsFactory.getDefaultOption()),(this.isDefaultOperator("AND")?this.eJoinOperatorAnd:this.eJoinOperatorOr).setValue(!0)},t.prototype.getModelFromUi=function(){return this.isConditionUiComplete(fr.One)?this.isAllowTwoConditions()&&this.isConditionUiComplete(fr.Two)?{filterType:this.getFilterType(),operator:this.getJoinOperator(),condition1:this.createCondition(fr.One),condition2:this.createCondition(fr.Two)}:this.createCondition(fr.One):null},t.prototype.getConditionTypes=function(){return[this.eType1.getValue(),this.eType2.getValue()]},t.prototype.getJoinOperator=function(){return this.eJoinOperatorOr.getValue()===!0?"OR":"AND"},t.prototype.areModelsEqual=function(e,r){if(!e&&!r)return!0;if(!e&&r||e&&!r)return!1;var n=!e.operator,o=!r.operator,a=!n&&o||n&&!o;if(a)return!1;var s;if(n){var l=e,u=r;s=this.areSimpleModelsEqual(l,u)}else{var c=e,p=r;s=c.operator===p.operator&&this.areSimpleModelsEqual(c.condition1,p.condition1)&&this.areSimpleModelsEqual(c.condition2,p.condition2)}return s},t.prototype.setModelIntoUi=function(e){var r=e.operator;if(r){var n=e,o=n.operator==="OR";this.eJoinOperatorAnd.setValue(!o),this.eJoinOperatorOr.setValue(o),this.eType1.setValue(n.condition1.type),this.eType2.setValue(n.condition2.type),this.setConditionIntoUi(n.condition1,fr.One),this.setConditionIntoUi(n.condition2,fr.Two)}else{var a=e;this.eJoinOperatorAnd.setValue(this.isDefaultOperator("AND")),this.eJoinOperatorOr.setValue(this.isDefaultOperator("OR")),this.eType1.setValue(a.type),this.eType2.setValue(this.optionsFactory.getDefaultOption()),this.setConditionIntoUi(a,fr.One),this.setConditionIntoUi(null,fr.Two)}return ct.resolve()},t.prototype.doesFilterPass=function(e){var r=this,n=this.getModel();if(n==null)return!0;var o=n.operator,a=[];if(o){var s=n;a.push(s.condition1,s.condition2)}else a.push(n);var l=o&&o==="OR"?"some":"every";return a[l](function(u){return r.individualConditionPasses(e,u)})},t.prototype.setParams=function(e){i.prototype.setParams.call(this,e),this.optionsFactory=new cE,this.optionsFactory.init(e,this.getDefaultFilterOptions()),this.allowTwoConditions=!e.suppressAndOrCondition,this.alwaysShowBothConditions=!!e.alwaysShowBothConditions,this.defaultJoinOperator=this.getDefaultJoinOperator(e.defaultJoinOperator),this.putOptionsIntoDropdown(),this.addChangedListeners()},t.prototype.getDefaultJoinOperator=function(e){return fn(["AND","OR"],e)?e:"AND"},t.prototype.putOptionsIntoDropdown=function(){var e=this,r=this.optionsFactory.getFilterOptions(),n=[this.eType1,this.eType2];r.forEach(function(o){var a=typeof o=="string"?e.createBoilerplateListOption(o):e.createCustomListOption(o);n.forEach(function(s){return s.addOption(a)})}),n.forEach(function(o){return o.setDisabled(r.length<=1)})},t.prototype.createBoilerplateListOption=function(e){return{value:e,text:this.translate(e)}},t.prototype.createCustomListOption=function(e){var r=e.displayKey,n=this.optionsFactory.getCustomOption(e.displayKey);return{value:r,text:n?this.gridOptionsWrapper.getLocaleTextFunc()(n.displayKey,n.displayName):this.translate(r)}},t.prototype.isAllowTwoConditions=function(){return this.allowTwoConditions},t.prototype.createBodyTemplate=function(){return`
            <ag-select class="ag-filter-select" ref="eOptions1"></ag-select>
            `+this.createValueTemplate(fr.One)+`
            <div class="ag-filter-condition" ref="eJoinOperatorPanel">
               <ag-radio-button ref="eJoinOperatorAnd" class="ag-filter-condition-operator ag-filter-condition-operator-and"></ag-radio-button>
               <ag-radio-button ref="eJoinOperatorOr" class="ag-filter-condition-operator ag-filter-condition-operator-or"></ag-radio-button>
            </div>
            <ag-select class="ag-filter-select" ref="eOptions2"></ag-select>
            `+this.createValueTemplate(fr.Two)},t.prototype.getCssIdentifier=function(){return"simple-filter"},t.prototype.updateUiVisibility=function(){var e=this,r=[[this.eType1],[this.eType2,this.eJoinOperatorPanel,this.eJoinOperatorAnd,this.eJoinOperatorOr]],n=[this.eCondition1Body,this.eCondition2Body];r.forEach(function(o,a){var s=e.isConditionVisible(a),l=e.isConditionDisabled(a);o.forEach(function(u){u instanceof Ki||u instanceof Kc?(u.setDisabled(l),u.setDisplayed(s)):(Al(u,l),pt(u,s))})}),n.forEach(function(o,a){pt(o,e.isConditionBodyVisible(a))}),this.forEachInput(function(o,a,s,l){e.setElementDisplayed(o,a<l),e.setElementDisabled(o,e.isConditionDisabled(s))}),this.resetPlaceholder()},t.prototype.afterGuiAttached=function(e){if(i.prototype.afterGuiAttached.call(this,e),this.resetPlaceholder(),!e||!e.suppressFocus&&!this.isReadOnly()){var r=this.getInputs()[0][0];if(!r)return;r instanceof Ki&&r.getInputElement().focus()}},t.prototype.resetPlaceholder=function(){var e=this,r=this.gridOptionsWrapper.getLocaleTextFunc();this.forEachInput(function(n,o,a,s){if(n instanceof Ki){var l=o===0&&s>1?"inRangeStart":o===0?"filterOoo":"inRangeEnd",u=o===0&&s>1?r("ariaFilterFromValue","Filter from value"):o===0?r("ariaFilterValue","Filter Value"):r("ariaFilterToValue","Filter to Value");n.setInputPlaceholder(e.translate(l)),n.setInputAriaLabel(u)}})},t.prototype.setElementValue=function(e,r,n){e instanceof Ki&&e.setValue(r!=null?String(r):null,n)},t.prototype.setElementDisplayed=function(e,r){e instanceof ee&&pt(e.getGui(),r)},t.prototype.setElementDisabled=function(e,r){e instanceof ee&&Al(e.getGui(),r)},t.prototype.attachElementOnChange=function(e,r){e instanceof Ki&&e.onValueChange(r)},t.prototype.forEachInput=function(e){var r=this,n=this.getInputs();this.getConditionTypes().forEach(function(o,a){for(var s=r.getNumberOfInputs(o),l=0;l<n[a].length;l++){var u=n[a][l];u!=null&&e(u,l,a,s)}})},t.prototype.isConditionVisible=function(e){return e===0?!0:this.allowTwoConditions?this.isReadOnly()?this.isConditionUiComplete(e):this.alwaysShowBothConditions?!0:this.isConditionUiComplete(e-1):!1},t.prototype.isConditionDisabled=function(e){return this.isReadOnly()||!this.isConditionVisible(e)?!0:e===0?!1:!this.isConditionUiComplete(e-1)},t.prototype.isConditionBodyVisible=function(e){if(!this.isConditionVisible(e))return!1;var r=this.getConditionTypes()[e],n=this.getNumberOfInputs(r);return n>0},t.prototype.isConditionUiComplete=function(e){var r=this.getConditionTypes()[e];return!(r===t.EMPTY||this.getValues(e).some(function(n){return n==null}))},t.prototype.resetUiToDefaults=function(e){var r=this,n=this.gridOptionsWrapper.getLocaleTextFunc(),o=n("ariaFilteringOperator","Filtering operator"),a="ag-simple-filter-and-or-"+this.getCompId(),s=this.optionsFactory.getDefaultOption();return this.eType1.setValue(s,e).setAriaLabel(o).setDisabled(this.isReadOnly()),this.eType2.setValue(this.optionsFactory.getDefaultOption(),e).setAriaLabel(o).setDisabled(this.isReadOnly()),this.eJoinOperatorAnd.setValue(this.isDefaultOperator("AND"),e).setName(a).setLabel(this.translate("andCondition")).setDisabled(this.isReadOnly()),this.eJoinOperatorOr.setValue(this.isDefaultOperator("OR"),e).setName(a).setLabel(this.translate("orCondition")).setDisabled(this.isReadOnly()),this.forEachInput(function(l){r.setElementValue(l,null,e),r.setElementDisabled(l,r.isReadOnly())}),this.resetPlaceholder(),ct.resolve()},t.prototype.setConditionIntoUi=function(e,r){var n=this,o=this.mapValuesFromModel(e);this.forEachInput(function(a,s,l,u){l===r&&n.setElementValue(a,o[s]!=null?o[s]:null)})},t.prototype.setValueFromFloatingFilter=function(e){var r=this;this.forEachInput(function(n,o,a,s){r.setElementValue(n,o===0&&a===0?e:null)})},t.prototype.isDefaultOperator=function(e){return e===this.defaultJoinOperator},t.prototype.addChangedListeners=function(){var e=this;if(!this.isReadOnly()){var r=function(){return e.onUiChanged()};this.eType1.onValueChange(r),this.eType2.onValueChange(r),this.eJoinOperatorOr.onValueChange(r),this.eJoinOperatorAnd.onValueChange(r),this.forEachInput(function(n){e.attachElementOnChange(n,r)})}},t.prototype.individualConditionPasses=function(e,r){var n=this.getCellValue(e.node),o=this.mapValuesFromModel(r),a=this.optionsFactory.getCustomOption(r.type),s=this.evaluateCustomFilter(a,o,n);return s!=null?s:n==null?this.evaluateNullValue(r.type):this.evaluateNonNullValue(o,n,r,e)},t.prototype.evaluateCustomFilter=function(e,r,n){if(e!=null){var o=e.predicate;if(o!=null&&!r.some(function(a){return a==null}))return o(r,n)}},t.prototype.isBlank=function(e){return e==null||typeof e=="string"&&e.trim().length===0},t.EMPTY="empty",t.BLANK="blank",t.NOT_BLANK="notBlank",t.EQUALS="equals",t.NOT_EQUAL="notEqual",t.LESS_THAN="lessThan",t.LESS_THAN_OR_EQUAL="lessThanOrEqual",t.GREATER_THAN="greaterThan",t.GREATER_THAN_OR_EQUAL="greaterThanOrEqual",t.IN_RANGE="inRange",t.CONTAINS="contains",t.NOT_CONTAINS="notContains",t.STARTS_WITH="startsWith",t.ENDS_WITH="endsWith",ma([I("eOptions1")],t.prototype,"eType1",void 0),ma([I("eOptions2")],t.prototype,"eType2",void 0),ma([I("eJoinOperatorPanel")],t.prototype,"eJoinOperatorPanel",void 0),ma([I("eJoinOperatorAnd")],t.prototype,"eJoinOperatorAnd",void 0),ma([I("eJoinOperatorOr")],t.prototype,"eJoinOperatorOr",void 0),ma([I("eCondition1Body")],t.prototype,"eCondition1Body",void 0),ma([I("eCondition2Body")],t.prototype,"eCondition2Body",void 0),t}(us);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var mL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Vt=function(i){mL(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.setParams=function(e){i.prototype.setParams.call(this,e),this.scalarFilterParams=e},t.prototype.evaluateNullValue=function(e){switch(e){case t.EQUALS:case t.NOT_EQUAL:if(this.scalarFilterParams.includeBlanksInEquals)return!0;break;case t.GREATER_THAN:case t.GREATER_THAN_OR_EQUAL:if(this.scalarFilterParams.includeBlanksInGreaterThan)return!0;break;case t.LESS_THAN:case t.LESS_THAN_OR_EQUAL:if(this.scalarFilterParams.includeBlanksInLessThan)return!0;break;case t.IN_RANGE:if(this.scalarFilterParams.includeBlanksInRange)return!0;break;case t.BLANK:return!0;case t.NOT_BLANK:return!1}return!1},t.prototype.evaluateNonNullValue=function(e,r,n){var o=this.comparator(),a=e[0]!=null?o(e[0],r):0;switch(n.type){case t.EQUALS:return a===0;case t.NOT_EQUAL:return a!==0;case t.GREATER_THAN:return a>0;case t.GREATER_THAN_OR_EQUAL:return a>=0;case t.LESS_THAN:return a<0;case t.LESS_THAN_OR_EQUAL:return a<=0;case t.IN_RANGE:{var s=o(e[1],r);return this.scalarFilterParams.inRangeInclusive?a>=0&&s<=0:a>0&&s<0}case t.BLANK:return this.isBlank(r);case t.NOT_BLANK:return!this.isBlank(r);default:return console.warn('AG Grid: Unexpected type of filter "'+n.type+'", it looks like the filter was configured with incorrect Filter Options'),!0}},t}(Ot);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var yL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ty=globalThis&&globalThis.__assign||function(){return ty=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},ty.apply(this,arguments)},au=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},l_=1e3,u_=1/0,$C=function(i){yL(t,i);function t(){var e=i.call(this,"dateFilter")||this;return e.minValidYear=l_,e.maxValidYear=u_,e}return t.prototype.afterGuiAttached=function(e){i.prototype.afterGuiAttached.call(this,e),this.dateCondition1FromComp.afterGuiAttached(e)},t.prototype.mapValuesFromModel=function(e){var r=e||{},n=r.dateFrom,o=r.dateTo,a=r.type;return[n&&Ko(n)||null,o&&Ko(o)||null].slice(0,this.getNumberOfInputs(a))},t.prototype.comparator=function(){return this.dateFilterParams.comparator?this.dateFilterParams.comparator:this.defaultComparator.bind(this)},t.prototype.defaultComparator=function(e,r){var n=r;return r==null||n<e?-1:n>e?1:0},t.prototype.setParams=function(e){i.prototype.setParams.call(this,e),this.dateFilterParams=e;var r=function(n,o){if(e[n]!=null)if(isNaN(e[n]))console.warn("AG Grid: DateFilter "+n+" is not a number");else return e[n]==null?o:Number(e[n]);return o};this.minValidYear=r("minValidYear",l_),this.maxValidYear=r("maxValidYear",u_),this.minValidYear>this.maxValidYear&&console.warn("AG Grid: DateFilter minValidYear should be <= maxValidYear"),this.createDateComponents()},t.prototype.createDateComponents=function(){var e=this,r=function(n){return new uE(e.getContext(),e.userComponentFactory,{onDateChanged:function(){return e.onUiChanged()},filterParams:e.dateFilterParams},n)};this.dateCondition1FromComp=r(this.eCondition1PanelFrom),this.dateCondition1ToComp=r(this.eCondition1PanelTo),this.dateCondition2FromComp=r(this.eCondition2PanelFrom),this.dateCondition2ToComp=r(this.eCondition2PanelTo),this.addDestroyFunc(function(){e.forEachInput(function(n){return n.destroy()})})},t.prototype.setElementValue=function(e,r,n){e.setDate(r)},t.prototype.setElementDisplayed=function(e,r){e.setDisplayed(r)},t.prototype.setElementDisabled=function(e,r){e.setDisabled(r)},t.prototype.getDefaultFilterOptions=function(){return t.DEFAULT_FILTER_OPTIONS},t.prototype.createValueTemplate=function(e){var r=e===fr.One?"1":"2";return`
            <div class="ag-filter-body" ref="eCondition`+r+`Body">
                <div class="ag-filter-from ag-filter-date-from" ref="eCondition`+r+`PanelFrom"></div>
                <div class="ag-filter-to ag-filter-date-to" ref="eCondition`+r+`PanelTo"></div>
            </div>`},t.prototype.isConditionUiComplete=function(e){var r=this;if(!i.prototype.isConditionUiComplete.call(this,e))return!1;var n=function(a){return a!=null&&a.getUTCFullYear()>=r.minValidYear&&a.getUTCFullYear()<=r.maxValidYear},o=!0;return this.forEachInput(function(a,s,l,u){l!==e||!o||s>=u||(o=o&&n(a.getDate()))}),o},t.prototype.areSimpleModelsEqual=function(e,r){return e.dateFrom===r.dateFrom&&e.dateTo===r.dateTo&&e.type===r.type},t.prototype.getFilterType=function(){return"date"},t.prototype.createCondition=function(e){var r=this.getConditionTypes()[e],n={},o=this.getValues(e);return o.length>0&&(n.dateFrom=$o(o[0])),o.length>1&&(n.dateTo=$o(o[1])),ty({dateFrom:null,dateTo:null,filterType:this.getFilterType(),type:r},n)},t.prototype.resetPlaceholder=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=this.translate("dateFormatOoo"),n=e("ariaFilterValue","Filter Value");this.forEachInput(function(o){o.setInputPlaceholder(r),o.setInputAriaLabel(n)})},t.prototype.getInputs=function(){return[[this.dateCondition1FromComp,this.dateCondition1ToComp],[this.dateCondition2FromComp,this.dateCondition2ToComp]]},t.prototype.getValues=function(e){var r=[];return this.forEachInput(function(n,o,a,s){e===a&&o<s&&r.push(n.getDate())}),r},t.DEFAULT_FILTER_OPTIONS=[Vt.EQUALS,Vt.GREATER_THAN,Vt.LESS_THAN,Vt.NOT_EQUAL,Vt.IN_RANGE,Vt.BLANK,Vt.NOT_BLANK],au([I("eCondition1PanelFrom")],t.prototype,"eCondition1PanelFrom",void 0),au([I("eCondition1PanelTo")],t.prototype,"eCondition1PanelTo",void 0),au([I("eCondition2PanelFrom")],t.prototype,"eCondition2PanelFrom",void 0),au([I("eCondition2PanelTo")],t.prototype,"eCondition2PanelTo",void 0),au([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),t}(Vt);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var CL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),hE=function(i){CL(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getDefaultDebounceMs=function(){return 0},t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.getTextFromModel=function(e){if(!e)return null;var r=e.operator!=null;if(r){var n=e,o=n||{},a=o.condition1,s=o.condition2,l=this.optionsFactory.getCustomOption(a.type),u=this.optionsFactory.getCustomOption(s.type);return[this.conditionToString(a,l),n.operator,this.conditionToString(s,u)].join(" ")}else{var c=e,p=this.optionsFactory.getCustomOption(c.type),h=p||{},d=h.displayKey,f=h.displayName,g=h.numberOfInputs;return d&&f&&g===0?(this.gridOptionsWrapper.getLocaleTextFunc()(d,f),f):this.conditionToString(c,p)}},t.prototype.isEventFromFloatingFilter=function(e){return e&&e.afterFloatingFilter},t.prototype.getLastType=function(){return this.lastType},t.prototype.isReadOnly=function(){return this.readOnly},t.prototype.setLastTypeFromModel=function(e){if(!e){this.lastType=this.optionsFactory.getDefaultOption();return}var r=e.operator,n;if(r){var o=e;n=o.condition1}else n=e;this.lastType=n.type},t.prototype.canWeEditAfterModelFromParentFilter=function(e){if(!e)return this.isTypeEditable(this.lastType);var r=e.operator;if(r)return!1;var n=e;return this.isTypeEditable(n.type)},t.prototype.init=function(e){this.optionsFactory=new cE,this.optionsFactory.init(e.filterParams,this.getDefaultFilterOptions()),this.lastType=this.optionsFactory.getDefaultOption(),this.readOnly=!!e.filterParams.readOnly;var r=this.isTypeEditable(this.lastType);this.setEditable(r)},t.prototype.doesFilterHaveSingleInput=function(e){var r=this.optionsFactory.getCustomOption(e),n=(r||{}).numberOfInputs;return n==null||n==1},t.prototype.isTypeEditable=function(e){var r=[Ot.IN_RANGE,Ot.EMPTY,Ot.BLANK,Ot.NOT_BLANK];return!!e&&!this.isReadOnly()&&this.doesFilterHaveSingleInput(e)&&r.indexOf(e)<0},t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var wL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),bg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},bL=function(i){wL(t,i);function t(){return i.call(this,`
            <div class="ag-floating-filter-input" role="presentation">
                <ag-input-text-field ref="eReadOnlyText"></ag-input-text-field>
                <div ref="eDateWrapper" style="display: flex;"></div>
            </div>`)||this}return t.prototype.getDefaultFilterOptions=function(){return $C.DEFAULT_FILTER_OPTIONS},t.prototype.conditionToString=function(e,r){var n=e.type,o=(r||{}).numberOfInputs,a=n==Ot.IN_RANGE||o===2,s=Ko(e.dateFrom),l=Ko(e.dateTo);return a?$o(s,!1)+"-"+$o(l,!1):s!=null?""+$o(s,!1):""+n},t.prototype.init=function(e){i.prototype.init.call(this,e),this.params=e,this.createDateComponent();var r=this.gridOptionsWrapper.getLocaleTextFunc();this.eReadOnlyText.setDisabled(!0).setInputAriaLabel(r("ariaDateFilterInput","Date Filter Input"))},t.prototype.setEditable=function(e){pt(this.eDateWrapper,e),pt(this.eReadOnlyText.getGui(),!e)},t.prototype.onParentModelChanged=function(e,r){if(!this.isEventFromFloatingFilter(r)){i.prototype.setLastTypeFromModel.call(this,e);var n=!this.isReadOnly()&&this.canWeEditAfterModelFromParentFilter(e);if(this.setEditable(n),n){if(e){var o=e;this.dateComp.setDate(Ko(o.dateFrom))}else this.dateComp.setDate(null);this.eReadOnlyText.setValue("")}else this.eReadOnlyText.setValue(this.getTextFromModel(e)),this.dateComp.setDate(null)}},t.prototype.onDateChanged=function(){var e=this,r=this.dateComp.getDate(),n=$o(r);this.params.parentFilterInstance(function(o){if(o){var a=Ko(n);o.onFloatingFilterChanged(e.getLastType()||null,a)}})},t.prototype.createDateComponent=function(){var e=this,r=us.getDebounceMs(this.params.filterParams,this.getDefaultDebounceMs()),n={onDateChanged:uo(this.onDateChanged.bind(this),r),filterParams:this.params.column.getColDef().filterParams};this.dateComp=new uE(this.getContext(),this.userComponentFactory,n,this.eDateWrapper),this.addDestroyFunc(function(){return e.dateComp.destroy()})},bg([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),bg([I("eReadOnlyText")],t.prototype,"eReadOnlyText",void 0),bg([I("eDateWrapper")],t.prototype,"eDateWrapper",void 0),t}(hE);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var _L=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),SL=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},xL=function(i){_L(t,i);function t(){return i.call(this,`
            <div class="ag-filter-filter">
                <ag-input-text-field class="ag-date-filter" ref="eDateInput"></ag-input-text-field>
            </div>`)||this}return t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.init=function(e){var r=this,n=this.gridOptionsWrapper.getDocument(),o=this.eDateInput.getInputElement();this.shouldUseBrowserDatePicker(e)&&(o.type="date"),this.addManagedListener(o,"mousedown",function(){r.eDateInput.isDisabled()||o.focus()}),this.addManagedListener(o,"input",function(u){u.target===n.activeElement&&(r.eDateInput.isDisabled()||e.onDateChanged())});var a=e.filterParams||{},s=a.minValidYear,l=a.maxValidYear;s&&(o.min=s+"-01-01"),l&&(o.max=l+"-12-31")},t.prototype.getDate=function(){return Ko(this.eDateInput.getValue())},t.prototype.setDate=function(e){this.eDateInput.setValue($o(e,!1))},t.prototype.setInputPlaceholder=function(e){this.eDateInput.setInputPlaceholder(e)},t.prototype.setDisabled=function(e){this.eDateInput.setDisabled(e)},t.prototype.afterGuiAttached=function(e){(!e||!e.suppressFocus)&&this.eDateInput.getInputElement().focus()},t.prototype.shouldUseBrowserDatePicker=function(e){return e.filterParams&&e.filterParams.browserDatePicker!=null?e.filterParams.browserDatePicker:jc()||RC()},SL([I("eDateInput")],t.prototype,"eDateInput",void 0),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var TL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),gp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},KC=function(i){TL(t,i);function t(){return i.call(this,"numberFilter")||this}return t.prototype.mapValuesFromModel=function(e){var r=e||{},n=r.filter,o=r.filterTo,a=r.type;return[n==null?null:n,o==null?null:o].slice(0,this.getNumberOfInputs(a))},t.prototype.getDefaultDebounceMs=function(){return 500},t.prototype.comparator=function(){return function(e,r){return e===r?0:e<r?1:-1}},t.prototype.setParams=function(e){this.numberFilterParams=e;var r=this.getAllowedCharPattern();if(r){var n={allowedCharPattern:r};this.resetTemplate({"eValue-index0-1":n,"eValue-index1-1":n,"eValue-index0-2":n,"eValue-index1-2":n})}i.prototype.setParams.call(this,e)},t.prototype.getDefaultFilterOptions=function(){return t.DEFAULT_FILTER_OPTIONS},t.prototype.createValueTemplate=function(e){var r=e===fr.One?"1":"2",n=this.getAllowedCharPattern(),o=n?"ag-input-text-field":"ag-input-number-field";return`
            <div class="ag-filter-body" ref="eCondition`+r+`Body" role="presentation">
                <`+o+' class="ag-filter-from ag-filter-filter" ref="eValue-index0-'+r+'"></'+o+`>
                <`+o+' class="ag-filter-to ag-filter-filter" ref="eValue-index1-'+r+'"></'+o+`>
            </div>`},t.prototype.getValues=function(e){var r=this,n=[];return this.forEachInput(function(o,a,s,l){e===s&&a<l&&n.push(r.stringToFloat(o.getValue()))}),n},t.prototype.areSimpleModelsEqual=function(e,r){return e.filter===r.filter&&e.filterTo===r.filterTo&&e.type===r.type},t.prototype.getFilterType=function(){return"number"},t.prototype.stringToFloat=function(e){if(typeof e=="number")return e;var r=Jo(e);return r!=null&&r.trim()===""&&(r=null),this.numberFilterParams.numberParser?this.numberFilterParams.numberParser(r):r==null||r.trim()==="-"?null:parseFloat(r)},t.prototype.createCondition=function(e){var r=this.getConditionTypes()[e],n={filterType:this.getFilterType(),type:r},o=this.getValues(e);return o.length>0&&(n.filter=o[0]),o.length>1&&(n.filterTo=o[1]),n},t.prototype.getInputs=function(){return[[this.eValueFrom1,this.eValueTo1],[this.eValueFrom2,this.eValueTo2]]},t.prototype.getAllowedCharPattern=function(){var e=(this.numberFilterParams||{}).allowedCharPattern;return e||(!jc()&&!Jd()?"\\d\\-\\.":null)},t.DEFAULT_FILTER_OPTIONS=[Vt.EQUALS,Vt.NOT_EQUAL,Vt.LESS_THAN,Vt.LESS_THAN_OR_EQUAL,Vt.GREATER_THAN,Vt.GREATER_THAN_OR_EQUAL,Vt.IN_RANGE,Vt.BLANK,Vt.NOT_BLANK],gp([I("eValue-index0-1")],t.prototype,"eValueFrom1",void 0),gp([I("eValue-index1-1")],t.prototype,"eValueTo1",void 0),gp([I("eValue-index0-2")],t.prototype,"eValueFrom2",void 0),gp([I("eValue-index1-2")],t.prototype,"eValueTo2",void 0),t}(Vt);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var OL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Zh=globalThis&&globalThis.__assign||function(){return Zh=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Zh.apply(this,arguments)},vp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Yc=function(i){OL(t,i);function t(){return i.call(this,"textFilter")||this}return t.trimInput=function(e){var r=e&&e.trim();return r===""?e:r},t.prototype.getDefaultDebounceMs=function(){return 500},t.prototype.setParams=function(e){i.prototype.setParams.call(this,e),this.textFilterParams=e,this.matcher=this.getTextMatcher(),this.formatter=this.textFilterParams.textFormatter||(this.textFilterParams.caseSensitive?t.DEFAULT_FORMATTER:t.DEFAULT_LOWERCASE_FORMATTER)},t.prototype.getTextMatcher=function(){var e=this.textFilterParams.textCustomComparator;return e?(_.doOnce(function(){return console.warn("AG Grid - textCustomComparator is deprecated, use textMatcher instead.")},"textCustomComparator.deprecated"),function(r){var n=r.filterOption,o=r.value,a=r.filterText;return e(n,o,a)}):this.textFilterParams.textMatcher||t.DEFAULT_MATCHER},t.prototype.createCondition=function(e){var r=this.getConditionTypes()[e],n={filterType:this.getFilterType(),type:r},o=this.getValues(e);return o.length>0&&(n.filter=o[0]),o.length>1&&(n.filterTo=o[1]),n},t.prototype.getFilterType=function(){return"text"},t.prototype.areSimpleModelsEqual=function(e,r){return e.filter===r.filter&&e.filterTo===r.filterTo&&e.type===r.type},t.prototype.getInputs=function(){return[[this.eValueFrom1,this.eValueTo1],[this.eValueFrom2,this.eValueTo2]]},t.prototype.getValues=function(e){var r=this,n=[];return this.forEachInput(function(o,a,s,l){if(e===s&&a<l){var u=Jo(o.getValue()),c=(r.textFilterParams.trimInput?t.trimInput(u):u)||null;n.push(c),o.setValue(c,!0)}}),n},t.prototype.getDefaultFilterOptions=function(){return t.DEFAULT_FILTER_OPTIONS},t.prototype.createValueTemplate=function(e){var r=e===fr.One?"1":"2";return`
            <div class="ag-filter-body" ref="eCondition`+r+`Body" role="presentation">
                <ag-input-text-field class=".ag-filter-from ag-filter-filter" ref="eValue-index0-`+r+`"></ag-input-text-field>
                <ag-input-text-field class="ag-filter-to ag-filter-filter" ref="eValue-index1-`+r+`"></ag-input-text-field>
            </div>`},t.prototype.mapValuesFromModel=function(e){var r=e||{},n=r.filter,o=r.filterTo,a=r.type;return[n||null,o||null].slice(0,this.getNumberOfInputs(a))},t.prototype.evaluateNullValue=function(e){var r=[Ot.NOT_EQUAL,Ot.NOT_CONTAINS,Ot.BLANK];return e?r.indexOf(e)>=0:!1},t.prototype.evaluateNonNullValue=function(e,r,n,o){var a=this,s=e.map(function(m){return a.formatter(m)})||[],l=this.formatter(r),u=this.textFilterParams,c=u.api,p=u.colDef,h=u.column,d=u.columnApi,f=u.context,g=u.textFormatter;if(n.type===Ot.BLANK)return this.isBlank(r);if(n.type===Ot.NOT_BLANK)return!this.isBlank(r);var v={api:c,colDef:p,column:h,columnApi:d,context:f,node:o.node,data:o.data,filterOption:n.type,value:l,textFormatter:g};return s.some(function(m){return a.matcher(Zh(Zh({},v),{filterText:m}))})},t.DEFAULT_FILTER_OPTIONS=[Ot.CONTAINS,Ot.NOT_CONTAINS,Ot.EQUALS,Ot.NOT_EQUAL,Ot.STARTS_WITH,Ot.ENDS_WITH,Ot.BLANK,Ot.NOT_BLANK],t.DEFAULT_FORMATTER=function(e){return e},t.DEFAULT_LOWERCASE_FORMATTER=function(e){return e==null?null:e.toString().toLowerCase()},t.DEFAULT_MATCHER=function(e){var r=e.filterOption,n=e.value,o=e.filterText;if(o==null)return!1;switch(r){case t.CONTAINS:return n.indexOf(o)>=0;case t.NOT_CONTAINS:return n.indexOf(o)<0;case t.EQUALS:return n===o;case t.NOT_EQUAL:return n!=o;case t.STARTS_WITH:return n.indexOf(o)===0;case t.ENDS_WITH:var a=n.lastIndexOf(o);return a>=0&&a===n.length-o.length;default:return!1}},vp([I("eValue-index0-1")],t.prototype,"eValueFrom1",void 0),vp([I("eValue-index1-1")],t.prototype,"eValueTo1",void 0),vp([I("eValue-index0-2")],t.prototype,"eValueFrom2",void 0),vp([I("eValue-index1-2")],t.prototype,"eValueTo2",void 0),t}(Ot);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var EL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),_g=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},dE=function(i){EL(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){this.resetTemplate()},t.prototype.resetTemplate=function(e){this.setTemplate(`
            <div class="ag-floating-filter-input" role="presentation">
                <ag-input-text-field ref="eFloatingFilterInput"></ag-input-text-field>
            </div>
        `,e)},t.prototype.getDefaultDebounceMs=function(){return 500},t.prototype.onParentModelChanged=function(e,r){this.isEventFromFloatingFilter(r)||(this.setLastTypeFromModel(e),this.eFloatingFilterInput.setValue(this.getTextFromModel(e)),this.setEditable(this.canWeEditAfterModelFromParentFilter(e)))},t.prototype.init=function(e){i.prototype.init.call(this,e),this.params=e,this.applyActive=us.isUseApplyButton(this.params.filterParams);var r=this.params.filterParams.allowedCharPattern;if(r!=null&&this.resetTemplate({eFloatingFilterInput:{allowedCharPattern:r}}),!this.isReadOnly()){var n=us.getDebounceMs(this.params.filterParams,this.getDefaultDebounceMs()),o=uo(this.syncUpWithParentFilter.bind(this),n),a=this.eFloatingFilterInput.getGui();this.addManagedListener(a,"input",o),this.addManagedListener(a,"keypress",o),this.addManagedListener(a,"keydown",o)}var s=e.column.getDefinition();(this.isReadOnly()||s.filterParams&&s.filterParams.filterOptions&&s.filterParams.filterOptions.length===1&&s.filterParams.filterOptions[0]==="inRange")&&this.eFloatingFilterInput.setDisabled(!0);var l=this.columnModel.getDisplayNameForColumn(e.column,"header",!0),u=this.gridOptionsWrapper.getLocaleTextFunc();this.eFloatingFilterInput.setInputAriaLabel(l+" "+u("ariaFilterInput","Filter Input"))},t.prototype.syncUpWithParentFilter=function(e){var r=this,n=e.key===j.ENTER;if(!(this.applyActive&&!n)){var o=this.eFloatingFilterInput.getValue();this.params.filterParams.trimInput&&(o=Yc.trimInput(o),this.eFloatingFilterInput.setValue(o,!0)),this.params.parentFilterInstance(function(a){a&&a.onFloatingFilterChanged(r.getLastType()||null,o||null)})}},t.prototype.conditionToString=function(e,r){var n=(r||{}).numberOfInputs,o=e.type==Ot.IN_RANGE||n===2;return o?e.filter+"-"+e.filterTo:e.filter!=null?""+e.filter:""+e.type},t.prototype.setEditable=function(e){this.eFloatingFilterInput.setDisabled(!e)},_g([y("columnModel")],t.prototype,"columnModel",void 0),_g([I("eFloatingFilterInput")],t.prototype,"eFloatingFilterInput",void 0),_g([B],t.prototype,"postConstruct",null),t}(hE);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var RL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),PL=function(i){RL(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getDefaultFilterOptions=function(){return KC.DEFAULT_FILTER_OPTIONS},t}(dE);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var AL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),fE=function(i){AL(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getDefaultFilterOptions=function(){return Yc.DEFAULT_FILTER_OPTIONS},t}(dE);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Cr=function(){function i(t,e){var r=this;e===void 0&&(e=!1),this.destroyFuncs=[],this.touching=!1,this.eventService=new oi,this.eElement=t,this.preventMouseClick=e;var n=this.onTouchStart.bind(this),o=this.onTouchMove.bind(this),a=this.onTouchEnd.bind(this);this.eElement.addEventListener("touchstart",n,{passive:!0}),this.eElement.addEventListener("touchmove",o,{passive:!0}),this.eElement.addEventListener("touchend",a,{passive:!1}),this.destroyFuncs.push(function(){r.eElement.removeEventListener("touchstart",n,{passive:!0}),r.eElement.removeEventListener("touchmove",o,{passive:!0}),r.eElement.removeEventListener("touchend",a,{passive:!1})})}return i.prototype.getActiveTouch=function(t){for(var e=0;e<t.length;e++){var r=t[e].identifier===this.touchStart.identifier;if(r)return t[e]}return null},i.prototype.addEventListener=function(t,e){this.eventService.addEventListener(t,e)},i.prototype.removeEventListener=function(t,e){this.eventService.removeEventListener(t,e)},i.prototype.onTouchStart=function(t){var e=this;if(!this.touching){this.touchStart=t.touches[0],this.touching=!0,this.moved=!1;var r=this.touchStart;window.setTimeout(function(){var n=e.touchStart===r;if(e.touching&&n&&!e.moved){e.moved=!0;var o={type:i.EVENT_LONG_TAP,touchStart:e.touchStart,touchEvent:t};e.eventService.dispatchEvent(o)}},500)}},i.prototype.onTouchMove=function(t){if(!!this.touching){var e=this.getActiveTouch(t.touches);if(!!e){var r=!jC(e,this.touchStart,4);r&&(this.moved=!0)}}},i.prototype.onTouchEnd=function(t){if(!!this.touching){if(!this.moved){var e={type:i.EVENT_TAP,touchStart:this.touchStart};this.eventService.dispatchEvent(e),this.checkForDoubleTap()}this.preventMouseClick&&t.preventDefault(),this.touching=!1}},i.prototype.checkForDoubleTap=function(){var t=new Date().getTime();if(this.lastTapTime&&this.lastTapTime>0){var e=t-this.lastTapTime;if(e>i.DOUBLE_TAP_MILLIS){var r={type:i.EVENT_DOUBLE_TAP,touchStart:this.touchStart};this.eventService.dispatchEvent(r),this.lastTapTime=null}else this.lastTapTime=t}else this.lastTapTime=t},i.prototype.destroy=function(){this.destroyFuncs.forEach(function(t){return t()})},i.EVENT_TAP="tap",i.EVENT_DOUBLE_TAP="doubleTap",i.EVENT_LONG_TAP="longTap",i.DOUBLE_TAP_MILLIS=500,i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var DL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),li=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ML=function(i){DL(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.lastMovingChanged=0,e}return t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.refresh=function(e){return this.params=e,this.workOutTemplate()!=this.currentTemplate||this.workOutShowMenu()!=this.currentShowMenu||this.workOutSort()!=this.currentSort?!1:(this.setDisplayName(e),!0)},t.prototype.workOutTemplate=function(){var e=hO(this.params.template,t.TEMPLATE);return e=e&&e.trim?e.trim():e,e},t.prototype.init=function(e){this.params=e,this.currentTemplate=this.workOutTemplate(),this.setTemplate(this.currentTemplate),this.setupTap(),this.setupIcons(e.column),this.setMenu(),this.setupSort(),this.setupFilterIcon(),this.setDisplayName(e)},t.prototype.setDisplayName=function(e){if(this.currentDisplayName!=e.displayName){this.currentDisplayName=e.displayName;var r=Ii(this.currentDisplayName);this.eText&&(this.eText.innerHTML=r)}},t.prototype.setupIcons=function(e){this.addInIcon("sortAscending",this.eSortAsc,e),this.addInIcon("sortDescending",this.eSortDesc,e),this.addInIcon("sortUnSort",this.eSortNone,e),this.addInIcon("menu",this.eMenu,e),this.addInIcon("filter",this.eFilter,e)},t.prototype.addInIcon=function(e,r,n){if(r!=null){var o=Ft(e,this.gridOptionsWrapper,n);o&&r.appendChild(o)}},t.prototype.setupTap=function(){var e=this,r=this.gridOptionsWrapper;if(!r.isSuppressTouch()){var n=new Cr(this.getGui(),!0),o=r.isSuppressMenuHide(),a=o&&q(this.eMenu),s=a?new Cr(this.eMenu,!0):n;if(this.params.enableMenu){var l=a?"EVENT_TAP":"EVENT_LONG_TAP",u=function(p){r.getApi().showColumnMenuAfterMouseClick(e.params.column,p.touchStart)};this.addManagedListener(s,Cr[l],u)}if(this.params.enableSorting){var c=function(p){var h=p.touchStart.target;o&&e.eMenu.contains(h)||e.sortController.progressSort(e.params.column,!1,"uiColumnSorted")};this.addManagedListener(n,Cr.EVENT_TAP,c)}this.addDestroyFunc(function(){return n.destroy()}),a&&this.addDestroyFunc(function(){return s.destroy()})}},t.prototype.workOutShowMenu=function(){var e=!this.gridOptionsWrapper.isSuppressMenuHide(),r=Bl()&&e,n=this.params.enableMenu&&!r;return n},t.prototype.setMenu=function(){var e=this;if(!!this.eMenu){if(this.currentShowMenu=this.workOutShowMenu(),!this.currentShowMenu){qn(this.eMenu);return}var r=this.gridOptionsWrapper.isSuppressMenuHide();this.addManagedListener(this.eMenu,"click",function(){return e.showMenu(e.eMenu)}),this.eMenu.classList.toggle("ag-header-menu-always-show",r)}},t.prototype.showMenu=function(e){e||(e=this.eMenu),this.menuFactory.showMenuAfterButtonClick(this.params.column,e,"columnMenu")},t.prototype.removeSortIcons=function(){qn(this.eSortAsc),qn(this.eSortDesc),qn(this.eSortNone),qn(this.eSortOrder)},t.prototype.workOutSort=function(){return this.params.enableSorting},t.prototype.setupSort=function(){var e=this;if(this.currentSort=this.params.enableSorting,!this.currentSort){this.removeSortIcons();return}var r=this.gridOptionsWrapper.isMultiSortKeyCtrl();this.addManagedListener(this.params.column,Se.EVENT_MOVING_CHANGED,function(){e.lastMovingChanged=new Date().getTime()}),this.eLabel&&this.addManagedListener(this.eLabel,"click",function(n){var o=e.params.column.isMoving(),a=new Date().getTime(),s=a-e.lastMovingChanged<50,l=o||s;if(!l){var u=r?n.ctrlKey||n.metaKey:n.shiftKey;e.params.progressSort(u)}}),this.addManagedListener(this.params.column,Se.EVENT_SORT_CHANGED,this.onSortChanged.bind(this)),this.onSortChanged(),this.addManagedListener(this.eventService,x.EVENT_SORT_CHANGED,this.setMultiSortOrder.bind(this)),this.setMultiSortOrder()},t.prototype.onSortChanged=function(){if(this.addOrRemoveCssClass("ag-header-cell-sorted-asc",this.params.column.isSortAscending()),this.addOrRemoveCssClass("ag-header-cell-sorted-desc",this.params.column.isSortDescending()),this.addOrRemoveCssClass("ag-header-cell-sorted-none",this.params.column.isSortNone()),this.eSortAsc&&this.eSortAsc.classList.toggle("ag-hidden",!this.params.column.isSortAscending()),this.eSortDesc&&this.eSortDesc.classList.toggle("ag-hidden",!this.params.column.isSortDescending()),this.eSortNone){var e=!this.params.column.getColDef().unSortIcon&&!this.gridOptionsWrapper.isUnSortIcon();this.eSortNone.classList.toggle("ag-hidden",e||!this.params.column.isSortNone())}},t.prototype.setMultiSortOrder=function(){if(!!this.eSortOrder){var e=this.params.column,r=this.sortController.getColumnsWithSortingOrdered(),n=r.indexOf(e),o=r.length>1,a=e.isSorting()&&o;pt(this.eSortOrder,a),n>=0?this.eSortOrder.innerHTML=(n+1).toString():gn(this.eSortOrder)}},t.prototype.setupFilterIcon=function(){!this.eFilter||(this.addManagedListener(this.params.column,Se.EVENT_FILTER_CHANGED,this.onFilterChanged.bind(this)),this.onFilterChanged())},t.prototype.onFilterChanged=function(){var e=this.params.column.isFilterActive();this.eFilter.classList.toggle("ag-hidden",!e)},t.TEMPLATE=`<div class="ag-cell-label-container" role="presentation">
            <span ref="eMenu" class="ag-header-icon ag-header-cell-menu-button" aria-hidden="true"></span>
            <div ref="eLabel" class="ag-header-cell-label" role="presentation">
                <span ref="eText" class="ag-header-cell-text"></span>
                <span ref="eFilter" class="ag-header-icon ag-header-label-icon ag-filter-icon" aria-hidden="true"></span>
                <span ref="eSortOrder" class="ag-header-icon ag-header-label-icon ag-sort-order" aria-hidden="true"></span>
                <span ref="eSortAsc" class="ag-header-icon ag-header-label-icon ag-sort-ascending-icon" aria-hidden="true"></span>
                <span ref="eSortDesc" class="ag-header-icon ag-header-label-icon ag-sort-descending-icon" aria-hidden="true"></span>
                <span ref="eSortNone" class="ag-header-icon ag-header-label-icon ag-sort-none-icon" aria-hidden="true"></span>
            </div>
        </div>`,li([y("sortController")],t.prototype,"sortController",void 0),li([y("menuFactory")],t.prototype,"menuFactory",void 0),li([I("eFilter")],t.prototype,"eFilter",void 0),li([I("eSortAsc")],t.prototype,"eSortAsc",void 0),li([I("eSortDesc")],t.prototype,"eSortDesc",void 0),li([I("eSortNone")],t.prototype,"eSortNone",void 0),li([I("eSortOrder")],t.prototype,"eSortOrder",void 0),li([I("eMenu")],t.prototype,"eMenu",void 0),li([I("eLabel")],t.prototype,"eLabel",void 0),li([I("eText")],t.prototype,"eText",void 0),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var FL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Sg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},IL=function(i){FL(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.init=function(e){this.params=e,this.checkWarnings(),this.setupLabel(),this.addGroupExpandIcon(),this.setupExpandIcons()},t.prototype.checkWarnings=function(){var e=this.params;if(e.template){var r="AG Grid: A template was provided for Header Group Comp - templates are only supported for Header Comps (not groups)";Ye(function(){return console.warn(r)},"HeaderGroupComp.templateNotSupported")}},t.prototype.setupExpandIcons=function(){var e=this;this.addInIcon("columnGroupOpened","agOpened"),this.addInIcon("columnGroupClosed","agClosed");var r=function(a){if(!Ai(a)){var s=!e.params.columnGroup.isExpanded();e.columnModel.setColumnGroupOpened(e.params.columnGroup.getProvidedColumnGroup(),s,"uiColumnExpanded")}};this.addTouchAndClickListeners(this.eCloseIcon,r),this.addTouchAndClickListeners(this.eOpenIcon,r);var n=function(a){Pl(a)};this.addManagedListener(this.eCloseIcon,"dblclick",n),this.addManagedListener(this.eOpenIcon,"dblclick",n),this.addManagedListener(this.getGui(),"dblclick",r),this.updateIconVisibility();var o=this.params.columnGroup.getProvidedColumnGroup();this.addManagedListener(o,$e.EVENT_EXPANDED_CHANGED,this.updateIconVisibility.bind(this)),this.addManagedListener(o,$e.EVENT_EXPANDABLE_CHANGED,this.updateIconVisibility.bind(this))},t.prototype.addTouchAndClickListeners=function(e,r){var n=new Cr(e,!0);this.addManagedListener(n,Cr.EVENT_TAP,r),this.addDestroyFunc(function(){return n.destroy()}),this.addManagedListener(e,"click",r)},t.prototype.updateIconVisibility=function(){var e=this.params.columnGroup;if(e.isExpandable()){var r=this.params.columnGroup.isExpanded();pt(this.eOpenIcon,r),pt(this.eCloseIcon,!r)}else pt(this.eOpenIcon,!1),pt(this.eCloseIcon,!1)},t.prototype.addInIcon=function(e,r){var n=Ft(e,this.gridOptionsWrapper,null);n&&this.getRefElement(r).appendChild(n)},t.prototype.addGroupExpandIcon=function(){if(!this.params.columnGroup.isExpandable()){pt(this.eOpenIcon,!1),pt(this.eCloseIcon,!1);return}},t.prototype.setupLabel=function(){var e=this.params.displayName;if(q(e)){var r=Ii(e);this.getRefElement("agLabel").innerHTML=r}},t.TEMPLATE=`<div class="ag-header-group-cell-label" ref="agContainer" role="presentation">
            <span ref="agLabel" class="ag-header-group-text" role="presentation"></span>
            <span ref="agOpened" class="ag-header-icon ag-header-expand-icon ag-header-expand-icon-expanded"></span>
            <span ref="agClosed" class="ag-header-icon ag-header-expand-icon ag-header-expand-icon-collapsed"></span>
        </div>`,Sg([y("columnModel")],t.prototype,"columnModel",void 0),Sg([I("agOpened")],t.prototype,"eOpenIcon",void 0),Sg([I("agClosed")],t.prototype,"eCloseIcon",void 0),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var LL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ws=function(i){LL(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.isPopup=function(){return!0},t.prototype.setParentComponent=function(e){e.addCssClass("ag-has-popup"),i.prototype.setParentComponent.call(this,e)},t.prototype.destroy=function(){var e=this.parentComponent,r=e&&e.isAlive();r&&e.getGui().classList.remove("ag-has-popup"),i.prototype.destroy.call(this)},t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var NL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),GL=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},gE=function(i){NL(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.init=function(e){this.params=e,this.focusAfterAttached=e.cellStartedEdit,this.eTextArea.setMaxLength(e.maxLength||200).setCols(e.cols||60).setRows(e.rows||10),q(e.value)&&this.eTextArea.setValue(e.value.toString(),!0),this.addGuiEventListener("keydown",this.onKeyDown.bind(this))},t.prototype.onKeyDown=function(e){var r=e.key;(r===j.LEFT||r===j.UP||r===j.RIGHT||r===j.DOWN||e.shiftKey&&r===j.ENTER)&&e.stopPropagation()},t.prototype.afterGuiAttached=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc();this.eTextArea.setInputAriaLabel(e("ariaInputEditor","Input Editor")),this.focusAfterAttached&&this.eTextArea.getFocusableElement().focus()},t.prototype.getValue=function(){return this.params.parseValue(this.eTextArea.getValue())},t.TEMPLATE=`<div class="ag-large-text" tabindex="0">
            <ag-input-text-area ref="eTextArea" class="ag-large-text-input"></ag-input-text-area>
        </div>`,GL([I("eTextArea")],t.prototype,"eTextArea",void 0),t}(ws);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var kL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),c_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},YC=function(i){kL(t,i);function t(){var e=i.call(this,'<div class="ag-cell-edit-wrapper"><ag-select class="ag-cell-editor" ref="eSelect"></ag-select></div>')||this;return e.startedByEnter=!1,e}return t.prototype.init=function(e){var r=this;if(this.focusAfterAttached=e.cellStartedEdit,me(e.values)){console.warn("AG Grid: no values found for select cellEditor");return}this.startedByEnter=e.eventKey!=null?e.eventKey===j.ENTER:!1;var n=!1;e.values.forEach(function(o){var a={value:o},s=r.valueFormatterService.formatValue(e.column,null,null,o),l=s!=null;a.text=l?s:o,r.eSelect.addOption(a),n=n||e.value===o}),n?this.eSelect.setValue(e.value,!0):e.values.length&&this.eSelect.setValue(e.values[0],!0),this.gridOptionsWrapper.isFullRowEdit()||this.eSelect.onValueChange(function(){return e.stopEditing()})},t.prototype.afterGuiAttached=function(){this.focusAfterAttached&&this.eSelect.getFocusableElement().focus(),this.startedByEnter&&this.eSelect.showPicker()},t.prototype.focusIn=function(){this.eSelect.getFocusableElement().focus()},t.prototype.getValue=function(){return this.eSelect.getValue()},t.prototype.isPopup=function(){return!1},c_([y("valueFormatterService")],t.prototype,"valueFormatterService",void 0),c_([I("eSelect")],t.prototype,"eSelect",void 0),t}(ws);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var WL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),vE=function(i){WL(t,i);function t(){var e=i.call(this)||this;return Ye(function(){return console.warn('AG Grid: The PopupSelectCellEditor (agPopupSelectCellEditor) is deprecated. Instead use {cellEditor: "agSelectCellEditor", cellEditorPopup: true} ')},"PopupSelectCellEditor.deprecated"),e}return t.prototype.isPopup=function(){return!0},t}(YC);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var VL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),BL=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Qh=function(i){VL(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.init=function(e){this.params=e;var r=this.eInput,n;e.cellStartedEdit?(this.focusAfterAttached=!0,e.eventKey===j.BACKSPACE||e.eventKey===j.DELETE?n="":e.charPress?n=e.charPress:(n=this.getStartValue(e),e.eventKey!==j.F2&&(this.highlightAllOnFocus=!0))):(this.focusAfterAttached=!1,n=this.getStartValue(e)),n!=null&&r.setValue(n,!0),this.addManagedListener(r.getGui(),"keydown",function(o){var a=o.key;(a===j.PAGE_UP||a===j.PAGE_DOWN)&&o.preventDefault()})},t.prototype.afterGuiAttached=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=this.eInput;if(r.setInputAriaLabel(e("ariaInputEditor","Input Editor")),!!this.focusAfterAttached){Hc()||r.getFocusableElement().focus();var n=r.getInputElement();if(this.highlightAllOnFocus)n.select();else{var o=r.getValue(),a=q(o)&&o.length||0;a&&n.setSelectionRange(a,a)}}},t.prototype.focusIn=function(){var e=this.eInput,r=e.getFocusableElement(),n=e.getInputElement();r.focus(),n.select()},t.prototype.getValue=function(){var e=this.eInput;return this.params.parseValue(e.getValue())},t.prototype.getStartValue=function(e){var r=e.useFormatter||e.column.getColDef().refData;return r?e.formatValue(e.value):e.value},t.prototype.isPopup=function(){return!1},t.TEMPLATE='<div class="ag-cell-edit-wrapper"><ag-input-text-field class="ag-cell-editor" ref="eInput"></ag-input-text-field></div>',BL([I("eInput")],t.prototype,"eInput",void 0),t}(ws);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var HL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),mE=function(i){HL(t,i);function t(){var e=i.call(this)||this;return Ye(function(){return console.warn('AG Grid: The PopupTextCellEditor (agPopupTextCellEditor) is deprecated. Instead use {cellEditor: "agTextCellEditor", cellEditorPopup: true} ')},"PopupTextCellEditor.deprecated"),e}return t.prototype.isPopup=function(){return!0},t}(Qh);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var jL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),UL=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},zL="\u2191",$L="\u2193",yE=function(i){jL(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.refreshCount=0,e}return t.prototype.init=function(e){this.eValue=this.queryForHtmlElement(".ag-value-change-value"),this.eDelta=this.queryForHtmlElement(".ag-value-change-delta"),this.refresh(e)},t.prototype.showDelta=function(e,r){var n=Math.abs(r),o=e.formatValue(n),a=q(o)?o:n,s=r>=0;s?this.eDelta.innerHTML=zL+a:this.eDelta.innerHTML=$L+a,this.eDelta.classList.toggle("ag-value-change-delta-up",s),this.eDelta.classList.toggle("ag-value-change-delta-down",!s)},t.prototype.setTimerToRemoveDelta=function(){var e=this;this.refreshCount++;var r=this.refreshCount;window.setTimeout(function(){r===e.refreshCount&&e.hideDeltaValue()},2e3)},t.prototype.hideDeltaValue=function(){this.eValue.classList.remove("ag-value-change-value-highlight"),gn(this.eDelta)},t.prototype.refresh=function(e){var r=e.value;if(r===this.lastValue||(q(e.valueFormatted)?this.eValue.innerHTML=e.valueFormatted:q(e.value)?this.eValue.innerHTML=r:gn(this.eValue),this.filterManager.isSuppressFlashingCellsBecauseFiltering()))return!1;if(typeof r=="number"&&typeof this.lastValue=="number"){var n=r-this.lastValue;this.showDelta(e,n)}return this.lastValue&&this.eValue.classList.add("ag-value-change-value-highlight"),this.setTimerToRemoveDelta(),this.lastValue=r,!0},t.TEMPLATE='<span><span class="ag-value-change-delta"></span><span class="ag-value-change-value"></span></span>',UL([y("filterManager")],t.prototype,"filterManager",void 0),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var KL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),YL=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},CE=function(i){KL(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.refreshCount=0,e.eCurrent=e.queryForHtmlElement(".ag-value-slide-current"),e}return t.prototype.init=function(e){this.refresh(e)},t.prototype.addSlideAnimation=function(){var e=this;this.refreshCount++;var r=this.refreshCount;this.ePrevious&&this.getGui().removeChild(this.ePrevious),this.ePrevious=Kt('<span class="ag-value-slide-previous ag-value-slide-out"></span>'),this.ePrevious.innerHTML=this.eCurrent.innerHTML,this.getGui().insertBefore(this.ePrevious,this.eCurrent),window.setTimeout(function(){r===e.refreshCount&&e.ePrevious.classList.add("ag-value-slide-out-end")},50),window.setTimeout(function(){r===e.refreshCount&&(e.getGui().removeChild(e.ePrevious),e.ePrevious=null)},3e3)},t.prototype.refresh=function(e){var r=e.value;return me(r)&&(r=""),r===this.lastValue||this.filterManager.isSuppressFlashingCellsBecauseFiltering()?!1:(this.addSlideAnimation(),this.lastValue=r,q(e.valueFormatted)?this.eCurrent.innerHTML=e.valueFormatted:q(e.value)?this.eCurrent.innerHTML=r:gn(this.eCurrent),!0)},t.TEMPLATE=`<span>
            <span class="ag-value-slide-current"></span>
        </span>`,YL([y("filterManager")],t.prototype,"filterManager",void 0),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Di;(function(i){i[i.Above=0]="Above",i[i.Below=1]="Below"})(Di||(Di={}));var Oe=function(){function i(t){this.rowIndex=null,this.key=null,this.childrenMapped={},this.displayed=!1,this.rowTop=null,this.oldRowTop=null,this.selectable=!0,this.__objectId=i.OBJECT_ID_SEQUENCE++,this.__autoHeights={},this.alreadyRendered=!1,this.highlighted=null,this.selected=!1,this.onRowHeightChangedDebounced=uo(this.onRowHeightChanged.bind(this),100),this.beans=t}return i.prototype.setData=function(t){this.setDataCommon(t,!1)},i.prototype.updateData=function(t){this.setDataCommon(t,!0)},i.prototype.setDataCommon=function(t,e){var r=this.data;this.data=t,this.beans.valueCache.onDataChanged(),this.updateDataOnDetailNode(),this.checkRowSelectable();var n=this.createDataChangedEvent(t,r,e);this.dispatchLocalEvent(n)},i.prototype.updateDataOnDetailNode=function(){this.detailNode&&(this.detailNode.data=this.data)},i.prototype.createDataChangedEvent=function(t,e,r){return{type:i.EVENT_DATA_CHANGED,node:this,oldData:e,newData:t,update:r}},i.prototype.createLocalRowEvent=function(t){return{type:t,node:this}},i.prototype.getRowIndexString=function(){return this.rowPinned===L.PINNED_TOP?"t-"+this.rowIndex:this.rowPinned===L.PINNED_BOTTOM?"b-"+this.rowIndex:this.rowIndex.toString()},i.prototype.createDaemonNode=function(){var t=new i(this.beans);return t.id=this.id,t.data=this.data,t.daemon=!0,t.selected=this.selected,t.level=this.level,t},i.prototype.setDataAndId=function(t,e){var r=q(this.id)?this.createDaemonNode():null,n=this.data;this.data=t,this.updateDataOnDetailNode(),this.setId(e),this.beans.selectionService.syncInRowNode(this,r),this.checkRowSelectable();var o=this.createDataChangedEvent(t,n,!1);this.dispatchLocalEvent(o)},i.prototype.checkRowSelectable=function(){var t=this.beans.gridOptionsWrapper.getIsRowSelectableFunc();this.setRowSelectable(t?t(this):!0)},i.prototype.setRowSelectable=function(t){this.selectable!==t&&(this.selectable=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_SELECTABLE_CHANGED)))},i.prototype.setId=function(t){var e=this.beans.gridOptionsWrapper.getRowIdFunc();if(e)if(this.data){var r=this.getGroupKeys(!0);this.id=e({data:this.data,parentKeys:r.length>0?r:void 0,level:this.level}),this.id!==null&&typeof this.id=="string"&&this.id.startsWith(i.ID_PREFIX_ROW_GROUP)&&console.error("AG Grid: Row IDs cannot start with "+i.ID_PREFIX_ROW_GROUP+", this is a reserved prefix for AG Grid's row grouping feature."),this.id!==null&&typeof this.id!="string"&&(this.id=""+this.id)}else this.id=void 0;else this.id=t},i.prototype.getGroupKeys=function(t){t===void 0&&(t=!1);var e=[],r=this;for(t&&(r=r.parent);r&&r.level>=0;)e.push(r.key),r=r.parent;return e.reverse(),e},i.prototype.isPixelInRange=function(t){return!q(this.rowTop)||!q(this.rowHeight)?!1:t>=this.rowTop&&t<this.rowTop+this.rowHeight},i.prototype.setFirstChild=function(t){this.firstChild!==t&&(this.firstChild=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_FIRST_CHILD_CHANGED)))},i.prototype.setLastChild=function(t){this.lastChild!==t&&(this.lastChild=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_LAST_CHILD_CHANGED)))},i.prototype.setChildIndex=function(t){this.childIndex!==t&&(this.childIndex=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_CHILD_INDEX_CHANGED)))},i.prototype.setRowTop=function(t){this.oldRowTop=this.rowTop,this.rowTop!==t&&(this.rowTop=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_TOP_CHANGED)),this.setDisplayed(t!==null))},i.prototype.clearRowTopAndRowIndex=function(){this.oldRowTop=null,this.setRowTop(null),this.setRowIndex(null)},i.prototype.setDisplayed=function(t){this.displayed!==t&&(this.displayed=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_DISPLAYED_CHANGED)))},i.prototype.setDragging=function(t){this.dragging!==t&&(this.dragging=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_DRAGGING_CHANGED)))},i.prototype.setHighlighted=function(t){t!==this.highlighted&&(this.highlighted=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_HIGHLIGHT_CHANGED)))},i.prototype.setAllChildrenCount=function(t){this.allChildrenCount!==t&&(this.allChildrenCount=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_ALL_CHILDREN_COUNT_CHANGED)))},i.prototype.setMaster=function(t){this.master!==t&&(this.master&&!t&&(this.expanded=!1),this.master=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_MASTER_CHANGED)))},i.prototype.setGroup=function(t){this.group!==t&&(this.group&&!t&&(this.expanded=!1),this.group=t,this.updateHasChildren(),this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_GROUP_CHANGED)))},i.prototype.setRowHeight=function(t,e){e===void 0&&(e=!1),this.rowHeight=t,this.rowHeightEstimated=e,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_HEIGHT_CHANGED))},i.prototype.setRowAutoHeight=function(t,e){this.__autoHeights||(this.__autoHeights={});var r=this.__autoHeights;r[e.getId()]=t,t!=null&&(this.checkAutoHeightsDebounced==null&&(this.checkAutoHeightsDebounced=uo(this.checkAutoHeights.bind(this),1)),this.checkAutoHeightsDebounced())},i.prototype.checkAutoHeights=function(){var t=!1,e=!0,r=0,n=this.__autoHeights;if(n!=null){var o=this.beans.columnModel.getAllDisplayedAutoHeightCols();o.forEach(function(a){var s=n[a.getId()];if(s==null){t=!0;return}e=!1,s>r&&(r=s)}),!t&&((e||r<10)&&(r=this.beans.gridOptionsWrapper.getRowHeightForNode(this).height),r!=this.rowHeight&&(this.setRowHeight(r),this.onRowHeightChangedDebounced()))}},i.prototype.onRowHeightChanged=function(){var t=this.beans.rowModel;t.onRowHeightChanged&&t.onRowHeightChanged()},i.prototype.setRowIndex=function(t){this.rowIndex!==t&&(this.rowIndex=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_ROW_INDEX_CHANGED)))},i.prototype.setUiLevel=function(t){this.uiLevel!==t&&(this.uiLevel=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_UI_LEVEL_CHANGED)))},i.prototype.setExpanded=function(t,e){if(this.expanded!==t){this.expanded=t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_EXPANDED_CHANGED));var r=Object.assign({},this.createGlobalRowEvent(x.EVENT_ROW_GROUP_OPENED),{expanded:t,event:e||null});this.beans.rowNodeEventThrottle.dispatchExpanded(r),this.beans.gridOptionsWrapper.isGroupIncludeFooter()&&this.beans.rowRenderer.refreshCells({rowNodes:[this]})}},i.prototype.createGlobalRowEvent=function(t){return{type:t,node:this,data:this.data,rowIndex:this.rowIndex,rowPinned:this.rowPinned,context:this.beans.gridOptionsWrapper.getContext(),api:this.beans.gridOptionsWrapper.getApi(),columnApi:this.beans.gridOptionsWrapper.getColumnApi()}},i.prototype.dispatchLocalEvent=function(t){this.eventService&&this.eventService.dispatchEvent(t)},i.prototype.setDataValue=function(t,e,r){var n=this.beans.columnModel.getPrimaryColumn(t),o=this.beans.valueService.getValue(n,this),a=this.beans.valueService.setValue(this,n,e,r);return this.dispatchCellChangedEvent(n,e,o),a},i.prototype.setGroupValue=function(t,e){var r=this.beans.columnModel.getGridColumn(t);me(this.groupData)&&(this.groupData={});var n=r.getColId(),o=this.groupData[n];o!==e&&(this.groupData[n]=e,this.dispatchCellChangedEvent(r,e,o))},i.prototype.setAggData=function(t){var e=this,r=nO([this.aggData,t]),n=this.aggData;this.aggData=t,this.eventService&&r.forEach(function(o){var a=e.beans.columnModel.getGridColumn(o),s=e.aggData?e.aggData[o]:void 0,l=n?n[o]:void 0;e.dispatchCellChangedEvent(a,s,l)})},i.prototype.updateHasChildren=function(){var t=this.group&&!this.footer||this.childrenAfterGroup&&this.childrenAfterGroup.length>0;t!==this.__hasChildren&&(this.__hasChildren=!!t,this.eventService&&this.eventService.dispatchEvent(this.createLocalRowEvent(i.EVENT_HAS_CHILDREN_CHANGED)))},i.prototype.hasChildren=function(){return this.__hasChildren==null&&this.updateHasChildren(),this.__hasChildren},i.prototype.isEmptyRowGroupNode=function(){return this.group&&pn(this.childrenAfterGroup)},i.prototype.dispatchCellChangedEvent=function(t,e,r){var n={type:i.EVENT_CELL_CHANGED,node:this,column:t,newValue:e,oldValue:r};this.dispatchLocalEvent(n)},i.prototype.resetQuickFilterAggregateText=function(){this.quickFilterAggregateText=null},i.prototype.isExpandable=function(){return!!(this.hasChildren()&&!this.footer||this.master)},i.prototype.isSelected=function(){return this.footer?this.sibling.isSelected():this.selected},i.prototype.depthFirstSearch=function(t){this.childrenAfterGroup&&this.childrenAfterGroup.forEach(function(e){return e.depthFirstSearch(t)}),t(this)},i.prototype.calculateSelectedFromChildren=function(){var t,e=!1,r=!1,n=!1,o;if(!!(!((t=this.childrenAfterGroup)===null||t===void 0)&&t.length)){for(var a=0;a<this.childrenAfterGroup.length;a++){var s=this.childrenAfterGroup[a];if(!!s.selectable){var l=s.isSelected();switch(l){case!0:e=!0;break;case!1:r=!0;break;default:n=!0;break}}}n?o=void 0:e&&!r?o=!0:!e&&r?o=!1:o=void 0,this.selectThisNode(o)}},i.prototype.setSelectedInitialValue=function(t){this.selected=t},i.prototype.setSelected=function(t,e,r){e===void 0&&(e=!1),r===void 0&&(r=!1),this.setSelectedParams({newValue:t,clearSelection:e,suppressFinishActions:r,rangeSelect:!1})},i.prototype.isRowPinned=function(){return this.rowPinned===L.PINNED_TOP||this.rowPinned===L.PINNED_BOTTOM},i.prototype.setSelectedParams=function(t){var e,r=this.beans.gridOptionsWrapper.isGroupSelectsChildren(),n=t.newValue===!0,o=t.clearSelection===!0,a=t.suppressFinishActions===!0,s=t.rangeSelect===!0,l=r&&t.groupSelectsFiltered===!0;if(this.id===void 0)return console.warn("AG Grid: cannot select node until id for node is known"),0;if(this.rowPinned)return console.warn("AG Grid: cannot select pinned rows"),0;if(this.footer)return this.sibling.setSelectedParams(t);if(s&&this.beans.selectionService.getLastSelectedNode()){var u=this.beans.selectionService.getLastSelectedNode()!==this,c=this.beans.gridOptionsWrapper.isRowSelectionMulti();if(u&&c){var p=this.doRowRangeSelection(t.newValue);return this.beans.selectionService.setLastSelectedNode(this),p}}var h=0,d=l&&this.group;if(!d){var f=this.selectThisNode(n);f&&h++}if(r&&((e=this.childrenAfterGroup)===null||e===void 0?void 0:e.length)&&(h+=this.selectChildNodes(n,l)),!a){var g=n&&(o||!this.beans.gridOptionsWrapper.isRowSelectionMulti());if(g&&(h+=this.beans.selectionService.clearOtherNodes(this)),h>0){this.beans.selectionService.updateGroupsFromChildrenSelections();var v={type:x.EVENT_SELECTION_CHANGED,api:this.beans.gridApi,columnApi:this.beans.columnApi};this.beans.eventService.dispatchEvent(v)}n&&this.beans.selectionService.setLastSelectedNode(this)}return h},i.prototype.doRowRangeSelection=function(t){var e=this;t===void 0&&(t=!0);var r=this.beans.gridOptionsWrapper.isGroupSelectsChildren(),n=this.beans.selectionService.getLastSelectedNode(),o=this.beans.rowModel.getNodesInRangeForSelection(this,n),a=0;o.forEach(function(l){if(!(l.group&&r||t===!1&&e===l)){var u=l.selectThisNode(t);u&&a++}}),this.beans.selectionService.updateGroupsFromChildrenSelections();var s={type:x.EVENT_SELECTION_CHANGED,api:this.beans.gridApi,columnApi:this.beans.columnApi};return this.beans.eventService.dispatchEvent(s),a},i.prototype.isParentOfNode=function(t){for(var e=this.parent;e;){if(e===t)return!0;e=e.parent}return!1},i.prototype.selectThisNode=function(t){var e=!this.selectable&&t,r=this.selected===t;if(e||r)return!1;this.selected=t,this.eventService&&this.dispatchLocalEvent(this.createLocalRowEvent(i.EVENT_ROW_SELECTED));var n=this.createGlobalRowEvent(x.EVENT_ROW_SELECTED);return this.beans.eventService.dispatchEvent(n),!0},i.prototype.selectChildNodes=function(t,e){var r=e?this.childrenAfterAggFilter:this.childrenAfterGroup;if(me(r))return 0;for(var n=0,o=0;o<r.length;o++)n+=r[o].setSelectedParams({newValue:t,clearSelection:!1,suppressFinishActions:!0,groupSelectsFiltered:e});return n},i.prototype.addEventListener=function(t,e){this.eventService||(this.eventService=new oi),this.eventService.addEventListener(t,e)},i.prototype.removeEventListener=function(t,e){!this.eventService||(this.eventService.removeEventListener(t,e),this.eventService.noRegisteredListenersExist()&&(this.eventService=null))},i.prototype.onMouseEnter=function(){this.dispatchLocalEvent(this.createLocalRowEvent(i.EVENT_MOUSE_ENTER))},i.prototype.onMouseLeave=function(){this.dispatchLocalEvent(this.createLocalRowEvent(i.EVENT_MOUSE_LEAVE))},i.prototype.getFirstChildOfFirstChild=function(t){for(var e=this,r=!0,n=!1,o=null;r&&!n;){var a=e.parent,s=q(a)&&e.firstChild;s?a.rowGroupColumn===t&&(n=!0,o=a):r=!1,e=a}return n?o:null},i.prototype.isFullWidthCell=function(){var t=this.beans.gridOptionsWrapper.getIsFullWidthCellFunc();return t?t({rowNode:this}):!1},i.prototype.getRoute=function(){if(this.key!=null){for(var t=[],e=this;e.key!=null;)t.push(e.key),e=e.parent;return t.reverse()}},i.ID_PREFIX_ROW_GROUP="row-group-",i.ID_PREFIX_TOP_PINNED="t-",i.ID_PREFIX_BOTTOM_PINNED="b-",i.OBJECT_ID_SEQUENCE=0,i.EVENT_ROW_SELECTED="rowSelected",i.EVENT_DATA_CHANGED="dataChanged",i.EVENT_CELL_CHANGED="cellChanged",i.EVENT_ALL_CHILDREN_COUNT_CHANGED="allChildrenCountChanged",i.EVENT_MASTER_CHANGED="masterChanged",i.EVENT_GROUP_CHANGED="groupChanged",i.EVENT_MOUSE_ENTER="mouseEnter",i.EVENT_MOUSE_LEAVE="mouseLeave",i.EVENT_HEIGHT_CHANGED="heightChanged",i.EVENT_TOP_CHANGED="topChanged",i.EVENT_DISPLAYED_CHANGED="displayedChanged",i.EVENT_FIRST_CHILD_CHANGED="firstChildChanged",i.EVENT_LAST_CHILD_CHANGED="lastChildChanged",i.EVENT_CHILD_INDEX_CHANGED="childIndexChanged",i.EVENT_ROW_INDEX_CHANGED="rowIndexChanged",i.EVENT_EXPANDED_CHANGED="expandedChanged",i.EVENT_HAS_CHILDREN_CHANGED="hasChildrenChanged",i.EVENT_SELECTABLE_CHANGED="selectableChanged",i.EVENT_UI_LEVEL_CHANGED="uiLevelChanged",i.EVENT_HIGHLIGHT_CHANGED="rowHighlightChanged",i.EVENT_DRAGGING_CHANGED="draggingChanged",i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var XL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),p_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},XC=function(i){XL(t,i);function t(){return i.call(this,`
            <div class="ag-selection-checkbox" role="presentation">
                <ag-checkbox role="presentation" ref="eCheckbox"></ag-checkbox>
            </div>`)||this}return t.prototype.postConstruct=function(){this.eCheckbox.setPassive(!0)},t.prototype.getCheckboxId=function(){return this.eCheckbox.getInputElement().id},t.prototype.onDataChanged=function(){this.onSelectionChanged()},t.prototype.onSelectableChanged=function(){this.showOrHideSelect()},t.prototype.onSelectionChanged=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=this.rowNode.isSelected(),n=r===void 0?e("ariaIndeterminate","indeterminate"):r===!0?e("ariaChecked","checked"):e("ariaUnchecked","unchecked"),o=e("ariaRowToggleSelection","Press Space to toggle row selection");this.eCheckbox.setValue(r,!0),this.eCheckbox.setInputAriaLabel(o+" ("+n+")")},t.prototype.onCheckedClicked=function(e){var r=this.gridOptionsWrapper.isGroupSelectsFiltered(),n=this.rowNode.setSelectedParams({newValue:!1,rangeSelect:e.shiftKey,groupSelectsFiltered:r});return n},t.prototype.onUncheckedClicked=function(e){var r=this.gridOptionsWrapper.isGroupSelectsFiltered(),n=this.rowNode.setSelectedParams({newValue:!0,rangeSelect:e.shiftKey,groupSelectsFiltered:r});return n},t.prototype.init=function(e){var r=this;this.rowNode=e.rowNode,this.column=e.column,this.onSelectionChanged(),this.addGuiEventListener("click",function(s){return Pl(s)}),this.addGuiEventListener("dblclick",function(s){return Pl(s)}),this.addManagedListener(this.eCheckbox.getInputElement(),"click",function(s){var l=r.eCheckbox.getValue(),u=r.eCheckbox.getPreviousValue();if(u===void 0||l===void 0){var c=r.onUncheckedClicked(s||{});c===0&&r.onCheckedClicked(s)}else l?r.onCheckedClicked(s):r.onUncheckedClicked(s||{})}),this.addManagedListener(this.rowNode,Oe.EVENT_ROW_SELECTED,this.onSelectionChanged.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_DATA_CHANGED,this.onDataChanged.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_SELECTABLE_CHANGED,this.onSelectableChanged.bind(this));var n=this.gridOptionsWrapper.getIsRowSelectableFunc(),o=n||this.checkboxCallbackExists();if(o){var a=this.showOrHideSelect.bind(this);this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,a),this.addManagedListener(this.rowNode,Oe.EVENT_DATA_CHANGED,a),this.addManagedListener(this.rowNode,Oe.EVENT_CELL_CHANGED,a),this.showOrHideSelect()}this.eCheckbox.getInputElement().setAttribute("tabindex","-1")},t.prototype.showOrHideSelect=function(){var e=this.rowNode.selectable;e&&this.checkboxCallbackExists()&&(e=this.column.isCellCheckboxSelection(this.rowNode)),this.setVisible(e)},t.prototype.checkboxCallbackExists=function(){var e=this.column?this.column.getColDef():null;return!!e&&typeof e.checkboxSelection=="function"},p_([I("eCheckbox")],t.prototype,"eCheckbox",void 0),p_([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var qL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ya=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},h_=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},At;(function(i){i[i.ToolPanel=0]="ToolPanel",i[i.HeaderCell=1]="HeaderCell",i[i.RowDrag=2]="RowDrag",i[i.ChartPanel=3]="ChartPanel"})(At||(At={}));var la;(function(i){i[i.Up=0]="Up",i[i.Down=1]="Down"})(la||(la={}));var Pn;(function(i){i[i.Left=0]="Left",i[i.Right=1]="Right"})(Pn||(Pn={}));var ht=function(i){qL(t,i);function t(){var r=i!==null&&i.apply(this,arguments)||this;return r.dragSourceAndParamsList=[],r.dropTargets=[],r}e=t,t.prototype.init=function(){this.ePinnedIcon=Tn("columnMovePin",this.gridOptionsWrapper,null),this.eHideIcon=Tn("columnMoveHide",this.gridOptionsWrapper,null),this.eMoveIcon=Tn("columnMoveMove",this.gridOptionsWrapper,null),this.eLeftIcon=Tn("columnMoveLeft",this.gridOptionsWrapper,null),this.eRightIcon=Tn("columnMoveRight",this.gridOptionsWrapper,null),this.eGroupIcon=Tn("columnMoveGroup",this.gridOptionsWrapper,null),this.eAggregateIcon=Tn("columnMoveValue",this.gridOptionsWrapper,null),this.ePivotIcon=Tn("columnMovePivot",this.gridOptionsWrapper,null),this.eDropNotAllowedIcon=Tn("dropNotAllowed",this.gridOptionsWrapper,null)},t.prototype.addDragSource=function(r,n){n===void 0&&(n=!1);var o={eElement:r.eElement,dragStartPixels:r.dragStartPixels,onDragStart:this.onDragStart.bind(this,r),onDragStop:this.onDragStop.bind(this),onDragging:this.onDragging.bind(this)};this.dragSourceAndParamsList.push({params:o,dragSource:r}),this.dragService.addDragSource(o,n)},t.prototype.removeDragSource=function(r){var n=this.dragSourceAndParamsList.find(function(o){return o.dragSource===r});n&&(this.dragService.removeDragSource(n.params),gr(this.dragSourceAndParamsList,n))},t.prototype.clearDragSourceParamsList=function(){var r=this;this.dragSourceAndParamsList.forEach(function(n){return r.dragService.removeDragSource(n.params)}),this.dragSourceAndParamsList.length=0},t.prototype.nudge=function(){this.dragging&&this.onDragging(this.eventLastTime,!0)},t.prototype.onDragStart=function(r,n){this.dragging=!0,this.dragSource=r,this.eventLastTime=n,this.dragItem=this.dragSource.getDragItem(),this.lastDropTarget=this.dragSource.dragSourceDropTarget,this.dragSource.onDragStarted&&this.dragSource.onDragStarted(),this.createGhost()},t.prototype.onDragStop=function(r){if(this.eventLastTime=null,this.dragging=!1,this.dragSource.onDragStopped&&this.dragSource.onDragStopped(),this.lastDropTarget&&this.lastDropTarget.onDragStop){var n=this.createDropTargetEvent(this.lastDropTarget,r,null,null,!1);this.lastDropTarget.onDragStop(n)}this.lastDropTarget=null,this.dragItem=null,this.removeGhost()},t.prototype.onDragging=function(r,n){var o=this,a=this.getHorizontalDirection(r),s=this.getVerticalDirection(r);this.eventLastTime=r,this.positionGhost(r);var l=this.dropTargets.filter(function(h){return o.isMouseOnDropTarget(r,h)}),u=l.length,c=null;if(u>0&&(c=u===1?l[0]:l.reduce(function(h,d){if(!h)return d;var f=h.getContainer(),g=d.getContainer();return f.contains(g)?d:h})),c!==this.lastDropTarget)this.leaveLastTargetIfExists(r,a,s,n),this.enterDragTargetIfExists(c,r,a,s,n),this.lastDropTarget=c;else if(c&&c.onDragging){var p=this.createDropTargetEvent(c,r,a,s,n);c.onDragging(p)}},t.prototype.enterDragTargetIfExists=function(r,n,o,a,s){if(!!r){if(r.onDragEnter){var l=this.createDropTargetEvent(r,n,o,a,s);r.onDragEnter(l)}this.setGhostIcon(r.getIconName?r.getIconName():null)}},t.prototype.leaveLastTargetIfExists=function(r,n,o,a){if(!!this.lastDropTarget){if(this.lastDropTarget.onDragLeave){var s=this.createDropTargetEvent(this.lastDropTarget,r,n,o,a);this.lastDropTarget.onDragLeave(s)}this.setGhostIcon(null)}},t.prototype.getAllContainersFromDropTarget=function(r){var n=r.getSecondaryContainers?r.getSecondaryContainers():null,o=[[r.getContainer()]];return n?o.concat(n):o},t.prototype.allContainersIntersect=function(r,n){var o,a;try{for(var s=h_(n),l=s.next();!l.done;l=s.next()){var u=l.value,c=u.getBoundingClientRect();if(c.width===0||c.height===0)return!1;var p=r.clientX>=c.left&&r.clientX<c.right,h=r.clientY>=c.top&&r.clientY<c.bottom;if(!p||!h)return!1}}catch(d){o={error:d}}finally{try{l&&!l.done&&(a=s.return)&&a.call(s)}finally{if(o)throw o.error}}return!0},t.prototype.isMouseOnDropTarget=function(r,n){var o,a,s=this.getAllContainersFromDropTarget(n),l=!1;try{for(var u=h_(s),c=u.next();!c.done;c=u.next()){var p=c.value;if(this.allContainersIntersect(r,p)){l=!0;break}}}catch(h){o={error:h}}finally{try{c&&!c.done&&(a=u.return)&&a.call(u)}finally{if(o)throw o.error}}return n.targetContainsSource&&!n.getContainer().contains(this.dragSource.eElement)?!1:l&&n.isInterestedIn(this.dragSource.type,this.dragSource.eElement)},t.prototype.addDropTarget=function(r){this.dropTargets.push(r)},t.prototype.removeDropTarget=function(r){this.dropTargets=this.dropTargets.filter(function(n){return n.getContainer()!==r.getContainer()})},t.prototype.hasExternalDropZones=function(){return this.dropTargets.some(function(r){return r.external})},t.prototype.findExternalZone=function(r){var n=this.dropTargets.filter(function(o){return o.external});return n.find(function(o){return o.getContainer()===r.getContainer()})||null},t.prototype.getHorizontalDirection=function(r){var n=this.eventLastTime&&this.eventLastTime.clientX,o=r.clientX;return n===o?null:n>o?Pn.Left:Pn.Right},t.prototype.getVerticalDirection=function(r){var n=this.eventLastTime&&this.eventLastTime.clientY,o=r.clientY;return n===o?null:n>o?la.Up:la.Down},t.prototype.createDropTargetEvent=function(r,n,o,a,s){var l=r.getContainer(),u=l.getBoundingClientRect(),c=this,p=c.gridApi,h=c.columnApi,d=c.dragItem,f=c.dragSource,g=n.clientX-u.left,v=n.clientY-u.top;return{event:n,x:g,y:v,vDirection:a,hDirection:o,dragSource:f,fromNudge:s,dragItem:d,api:p,columnApi:h,dropZoneTarget:l}},t.prototype.positionGhost=function(r){var n=this.eGhost;if(!!n){var o=n.getBoundingClientRect(),a=o.height,s=PO()-2,l=AO()-2,u=r.pageY-a/2,c=r.pageX-10,p=this.gridOptionsWrapper.getDocument(),h=p.defaultView||window,d=h.pageYOffset||p.documentElement.scrollTop,f=h.pageXOffset||p.documentElement.scrollLeft;s>0&&c+n.clientWidth>s+f&&(c=s+f-n.clientWidth),c<0&&(c=0),l>0&&u+n.clientHeight>l+d&&(u=l+d-n.clientHeight),u<0&&(u=0),n.style.left=c+"px",n.style.top=u+"px"}},t.prototype.removeGhost=function(){this.eGhost&&this.eGhostParent&&this.eGhostParent.removeChild(this.eGhost),this.eGhost=null},t.prototype.createGhost=function(){this.eGhost=Kt(e.GHOST_TEMPLATE);var r=this.environment.getTheme().theme;r&&this.eGhost.classList.add(r),this.eGhostIcon=this.eGhost.querySelector(".ag-dnd-ghost-icon"),this.setGhostIcon(null);var n=this.eGhost.querySelector(".ag-dnd-ghost-label"),o=this.dragSource.dragItemName;_C(o)&&(o=o()),n.innerHTML=Ii(o)||"",this.eGhost.style.height="25px",this.eGhost.style.top="20px",this.eGhost.style.left="20px";var a=this.gridOptionsWrapper.getDocument(),s=null;try{s=a.fullscreenElement}catch{}finally{s||(s=a.querySelector("body"))}this.eGhostParent=s,this.eGhostParent?this.eGhostParent.appendChild(this.eGhost):console.warn("AG Grid: could not find document body, it is needed for dragging columns")},t.prototype.setGhostIcon=function(r,n){n===void 0&&(n=!1),gn(this.eGhostIcon);var o=null;switch(r||(r=this.dragSource.defaultIconName||e.ICON_NOT_ALLOWED),r){case e.ICON_PINNED:o=this.ePinnedIcon;break;case e.ICON_MOVE:o=this.eMoveIcon;break;case e.ICON_LEFT:o=this.eLeftIcon;break;case e.ICON_RIGHT:o=this.eRightIcon;break;case e.ICON_GROUP:o=this.eGroupIcon;break;case e.ICON_AGGREGATE:o=this.eAggregateIcon;break;case e.ICON_PIVOT:o=this.ePivotIcon;break;case e.ICON_NOT_ALLOWED:o=this.eDropNotAllowedIcon;break;case e.ICON_HIDE:o=this.eHideIcon;break}this.eGhostIcon.classList.toggle("ag-shake-left-to-right",n),!(o===this.eHideIcon&&this.gridOptionsWrapper.isSuppressDragLeaveHidesColumns())&&o&&this.eGhostIcon.appendChild(o)};var e;return t.ICON_PINNED="pinned",t.ICON_MOVE="move",t.ICON_LEFT="left",t.ICON_RIGHT="right",t.ICON_GROUP="group",t.ICON_AGGREGATE="aggregate",t.ICON_PIVOT="pivot",t.ICON_NOT_ALLOWED="notAllowed",t.ICON_HIDE="hide",t.GHOST_TEMPLATE=`<div class="ag-dnd-ghost ag-unselectable">
            <span class="ag-dnd-ghost-icon ag-shake-left-to-right"></span>
            <div class="ag-dnd-ghost-label"></div>
        </div>`,ya([y("dragService")],t.prototype,"dragService",void 0),ya([y("environment")],t.prototype,"environment",void 0),ya([y("columnApi")],t.prototype,"columnApi",void 0),ya([y("gridApi")],t.prototype,"gridApi",void 0),ya([B],t.prototype,"init",null),ya([nt],t.prototype,"clearDragSourceParamsList",null),t=e=ya([Q("dragAndDropService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var of=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),uc=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Jh=function(i){of(t,i);function t(e,r,n,o,a,s){var l=i.call(this)||this;return l.cellValueFn=e,l.rowNode=r,l.column=n,l.customGui=o,l.dragStartPixels=a,l.suppressVisibilityChange=s,l.dragSource=null,l}return t.prototype.isCustomGui=function(){return this.customGui!=null},t.prototype.postConstruct=function(){if(this.customGui?this.setDragElement(this.customGui,this.dragStartPixels):(this.setTemplate('<div class="ag-drag-handle ag-row-drag" aria-hidden="true"></div>'),this.getGui().appendChild(Ft("rowDrag",this.beans.gridOptionsWrapper,null)),this.addDragSource()),this.checkCompatibility(),!this.suppressVisibilityChange){var e=this.beans.gridOptionsWrapper.isRowDragManaged()?new QL(this,this.beans,this.rowNode,this.column):new ZL(this,this.beans,this.rowNode,this.column);this.createManagedBean(e,this.beans.context)}},t.prototype.setDragElement=function(e,r){this.setTemplateFromElement(e),this.addDragSource(r)},t.prototype.getSelectedCount=function(){var e=this.beans.gridOptionsWrapper.isRowDragMultiRow();if(!e)return 1;var r=this.beans.selectionService.getSelectedNodes();return r.indexOf(this.rowNode)!==-1?r.length:1},t.prototype.checkCompatibility=function(){var e=this.beans.gridOptionsWrapper.isRowDragManaged(),r=this.beans.gridOptionsWrapper.isTreeData();r&&e&&Ye(function(){return console.warn("AG Grid: If using row drag with tree data, you cannot have rowDragManaged=true")},"RowDragComp.managedAndTreeData")},t.prototype.addDragSource=function(e){var r=this;e===void 0&&(e=4),this.dragSource&&this.removeDragSource();var n={rowNode:this.rowNode,columns:this.column?[this.column]:void 0,defaultTextValue:this.cellValueFn()},o=this.column&&this.column.getColDef().rowDragText,a=this.gridOptionsWrapper.getLocaleTextFunc();this.dragSource={type:At.RowDrag,eElement:this.getGui(),dragItemName:function(){var s=r.getSelectedCount();return o?o(n,s):s===1?r.cellValueFn():s+" "+a("rowDragRows","rows")},getDragItem:function(){return n},dragStartPixels:e,dragSourceDomDataKey:this.beans.gridOptionsWrapper.getDomDataKey()},this.beans.dragAndDropService.addDragSource(this.dragSource,!0)},t.prototype.removeDragSource=function(){this.dragSource&&this.beans.dragAndDropService.removeDragSource(this.dragSource),this.dragSource=null},uc([y("beans")],t.prototype,"beans",void 0),uc([B],t.prototype,"postConstruct",null),uc([nt],t.prototype,"removeDragSource",null),t}(ee),wE=function(i){of(t,i);function t(e,r,n){var o=i.call(this)||this;return o.parent=e,o.rowNode=r,o.column=n,o}return t.prototype.setDisplayedOrVisible=function(e){if(e)this.parent.setDisplayed(!1);else{var r=!0,n=!1;this.column&&(r=this.column.isRowDrag(this.rowNode)||this.parent.isCustomGui(),n=_C(this.column.getColDef().rowDrag)),n?(this.parent.setDisplayed(!0),this.parent.setVisible(r)):(this.parent.setDisplayed(r),this.parent.setVisible(!0))}},t}(H),ZL=function(i){of(t,i);function t(e,r,n,o){var a=i.call(this,e,n,o)||this;return a.beans=r,a}return t.prototype.postConstruct=function(){this.addManagedListener(this.beans.gridOptionsWrapper,"suppressRowDrag",this.onSuppressRowDrag.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_DATA_CHANGED,this.workOutVisibility.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_CELL_CHANGED,this.workOutVisibility.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_CELL_CHANGED,this.workOutVisibility.bind(this)),this.addManagedListener(this.beans.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.workOutVisibility.bind(this)),this.workOutVisibility()},t.prototype.onSuppressRowDrag=function(){this.workOutVisibility()},t.prototype.workOutVisibility=function(){var e=this.beans.gridOptionsWrapper.isSuppressRowDrag();this.setDisplayedOrVisible(e)},uc([B],t.prototype,"postConstruct",null),t}(wE),QL=function(i){of(t,i);function t(e,r,n,o){var a=i.call(this,e,n,o)||this;return a.beans=r,a}return t.prototype.postConstruct=function(){this.addManagedListener(this.beans.eventService,x.EVENT_SORT_CHANGED,this.workOutVisibility.bind(this)),this.addManagedListener(this.beans.eventService,x.EVENT_FILTER_CHANGED,this.workOutVisibility.bind(this)),this.addManagedListener(this.beans.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,this.workOutVisibility.bind(this)),this.addManagedListener(this.beans.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.workOutVisibility.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_DATA_CHANGED,this.workOutVisibility.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_CELL_CHANGED,this.workOutVisibility.bind(this)),this.addManagedListener(this.beans.gridOptionsWrapper,"suppressRowDrag",this.onSuppressRowDrag.bind(this)),this.workOutVisibility()},t.prototype.onSuppressRowDrag=function(){this.workOutVisibility()},t.prototype.workOutVisibility=function(){var e=this.beans.ctrlsService.getGridBodyCtrl(),r=e.getRowDragFeature(),n=r&&r.shouldPreventRowMove(),o=this.beans.gridOptionsWrapper.isSuppressRowDrag(),a=this.beans.dragAndDropService.hasExternalDropZones(),s=n&&!a||o;this.setDisplayedOrVisible(s)},uc([B],t.prototype,"postConstruct",null),t}(wE);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var JL=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ed=globalThis&&globalThis.__assign||function(){return ed=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},ed.apply(this,arguments)},su=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},bE=function(i){JL(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.init=function(e,r,n,o,a,s,l){this.params=l,this.eGui=r,this.eCheckbox=n,this.eExpanded=o,this.eContracted=a,this.comp=e,this.compClass=s;var u=this.isTopLevelFooter(),c=this.isEmbeddedRowMismatch(),p=l.value==null,h=!1;if(this.gridOptionsWrapper.isGroupIncludeFooter()&&this.gridOptionsWrapper.isGroupHideOpenParents()){var d=l.node;if(d.footer){var f=l.colDef&&l.colDef.showRowGroup,g=d.rowGroupColumn&&d.rowGroupColumn.getColId();h=f!==g}}this.cellIsBlank=u?!1:c||p||h,!this.cellIsBlank&&(this.setupShowingValueForOpenedParent(),this.findDisplayedGroupNode(),this.addFullWidthRowDraggerIfNeeded(),this.addExpandAndContract(),this.addCheckboxIfNeeded(),this.addValueElement(),this.setupIndent(),this.refreshAriaExpanded())},t.prototype.destroy=function(){i.prototype.destroy.call(this),this.expandListener=null},t.prototype.refreshAriaExpanded=function(){var e=this.params,r=e.node,n=e.eParentOfValue;if(this.expandListener&&(this.expandListener=this.expandListener()),!this.isExpandable()){zO(n);return}var o=function(){dn(n,!!r.expanded)};this.expandListener=this.addManagedListener(r,Oe.EVENT_EXPANDED_CHANGED,o)||null,o()},t.prototype.isTopLevelFooter=function(){if(!this.gridOptionsWrapper.isGroupIncludeTotalFooter()||this.params.value!=null||this.params.node.level!=-1)return!1;var e=this.params.colDef,r=e==null;if(r||e.showRowGroup===!0)return!0;var n=this.columnModel.getRowGroupColumns();if(!n||n.length===0)return!0;var o=n[0];return o.getId()===e.showRowGroup},t.prototype.isEmbeddedRowMismatch=function(){if(!this.params.fullWidth||!this.gridOptionsWrapper.isEmbedFullWidthRows())return!1;var e=this.params.pinned===L.PINNED_LEFT,r=this.params.pinned===L.PINNED_RIGHT,n=!e&&!r;return this.gridOptionsWrapper.isEnableRtl()?this.columnModel.isPinningLeft()?!r:!n:this.columnModel.isPinningLeft()?!e:!n},t.prototype.findDisplayedGroupNode=function(){var e=this.params.column,r=this.params.node;if(this.showingValueForOpenedParent)for(var n=r.parent;n!=null;){if(n.rowGroupColumn&&e.isRowGroupDisplayed(n.rowGroupColumn.getId())){this.displayedGroupNode=n;break}n=n.parent}me(this.displayedGroupNode)&&(this.displayedGroupNode=r)},t.prototype.setupShowingValueForOpenedParent=function(){var e=this.params.node,r=this.params.column;if(!this.gridOptionsWrapper.isGroupHideOpenParents()){this.showingValueForOpenedParent=!1;return}if(!e.groupData){this.showingValueForOpenedParent=!1;return}var n=e.rowGroupColumn!=null;if(n){var o=e.rowGroupColumn.getId(),a=r.isRowGroupDisplayed(o);if(a){this.showingValueForOpenedParent=!1;return}}var s=e.groupData[r.getId()]!=null;this.showingValueForOpenedParent=s},t.prototype.addValueElement=function(){this.displayedGroupNode.footer?this.addFooterValue():(this.addGroupValue(),this.addChildCount())},t.prototype.addGroupValue=function(){var e=this.adjustParamsWithDetailsFromRelatedColumn(),r=this.getInnerCompDetails(e),n=e.valueFormatted,o=e.value,a=n!=null?n:o;this.comp.setInnerRenderer(r,a)},t.prototype.adjustParamsWithDetailsFromRelatedColumn=function(){var e=this.displayedGroupNode.rowGroupColumn,r=this.params.column;if(!e)return this.params;var n=r!=null;if(n){var o=r.isRowGroupDisplayed(e.getId());if(!o)return this.params}var a=this.params,s=this.params,l=s.value,u=s.node,c=this.valueFormatterService.formatValue(e,u,l),p=ed(ed({},a),{valueFormatted:c});return p},t.prototype.addFooterValue=function(){var e=this.params.footerValueGetter,r="";if(e){var n=bC(this.params);n.value=this.params.value,typeof e=="function"?r=e(n):typeof e=="string"?r=this.expressionService.evaluate(e,n):console.warn("AG Grid: footerValueGetter should be either a function or a string (expression)")}else r="Total "+(this.params.value!=null?this.params.value:"");var o=this.getInnerCompDetails(this.params);this.comp.setInnerRenderer(o,r)},t.prototype.getInnerCompDetails=function(e){var r=this;if(e.fullWidth)return this.userComponentFactory.getFullWidthGroupRowInnerCellRenderer(this.gridOptions.groupRowRendererParams,e);var n=this.userComponentFactory.getInnerRendererDetails(e,e),o=function(c){return c&&c.componentClass==r.compClass};if(n&&!o(n))return n;var a=this.displayedGroupNode.rowGroupColumn,s=a?a.getColDef():void 0;if(!!s){var l=this.userComponentFactory.getCellRendererDetails(s,e);if(l&&!o(l))return l;if(o(l)&&s.cellRendererParams&&s.cellRendererParams.innerRenderer){var u=this.userComponentFactory.getInnerRendererDetails(s.cellRendererParams,e);return u}}},t.prototype.addChildCount=function(){this.params.suppressCount||(this.addManagedListener(this.displayedGroupNode,Oe.EVENT_ALL_CHILDREN_COUNT_CHANGED,this.updateChildCount.bind(this)),this.updateChildCount())},t.prototype.updateChildCount=function(){var e=this.displayedGroupNode.allChildrenCount,r=this.isShowRowGroupForThisRow(),n=r&&e!=null&&e>=0,o=n?"("+e+")":"";this.comp.setChildCount(o)},t.prototype.isShowRowGroupForThisRow=function(){if(this.gridOptionsWrapper.isTreeData())return!0;var e=this.displayedGroupNode.rowGroupColumn;if(!e)return!1;var r=this.params.column,n=r==null||r.isRowGroupDisplayed(e.getId());return n},t.prototype.addExpandAndContract=function(){var e=this.params,r=Ft("groupExpanded",this.gridOptionsWrapper,null),n=Ft("groupContracted",this.gridOptionsWrapper,null);r&&this.eExpanded.appendChild(r),n&&this.eContracted.appendChild(n);var o=e.eGridCell;!this.gridOptionsWrapper.isEnableGroupEdit()&&this.isExpandable()&&!e.suppressDoubleClickExpand&&this.addManagedListener(o,"dblclick",this.onCellDblClicked.bind(this)),this.addManagedListener(this.eExpanded,"click",this.onExpandClicked.bind(this)),this.addManagedListener(this.eContracted,"click",this.onExpandClicked.bind(this)),this.addManagedListener(o,"keydown",this.onKeyDown.bind(this)),this.addManagedListener(e.node,Oe.EVENT_EXPANDED_CHANGED,this.showExpandAndContractIcons.bind(this)),this.showExpandAndContractIcons();var a=this.onRowNodeIsExpandableChanged.bind(this);this.addManagedListener(this.displayedGroupNode,Oe.EVENT_ALL_CHILDREN_COUNT_CHANGED,a),this.addManagedListener(this.displayedGroupNode,Oe.EVENT_MASTER_CHANGED,a),this.addManagedListener(this.displayedGroupNode,Oe.EVENT_GROUP_CHANGED,a),this.addManagedListener(this.displayedGroupNode,Oe.EVENT_HAS_CHILDREN_CHANGED,a)},t.prototype.onExpandClicked=function(e){Ai(e)||(Pl(e),this.onExpandOrContract(e))},t.prototype.onExpandOrContract=function(e){var r=this.displayedGroupNode,n=!r.expanded;r.setExpanded(n,e)},t.prototype.isExpandable=function(){if(this.showingValueForOpenedParent)return!0;var e=this.displayedGroupNode,r=this.columnModel.isPivotMode()&&e.leafGroup,n=e.isExpandable()&&!e.footer&&!r;if(!n)return!1;var o=this.params.column,a=o!=null&&typeof o.getColDef().showRowGroup=="string";if(a){var s=this.isShowRowGroupForThisRow();return s}return!0},t.prototype.showExpandAndContractIcons=function(){var e=this,r=e.params,n=e.displayedGroupNode,o=e.columnModel,a=r.node,s=this.isExpandable();if(s){var l=this.showingValueForOpenedParent?!0:a.expanded;this.comp.setExpandedDisplayed(l),this.comp.setContractedDisplayed(!l)}else this.comp.setExpandedDisplayed(!1),this.comp.setContractedDisplayed(!1);var u=o.isPivotMode(),c=u&&n.leafGroup,p=s&&!c,h=a.footer&&a.level===-1;this.comp.addOrRemoveCssClass("ag-cell-expandable",p),this.comp.addOrRemoveCssClass("ag-row-group",p),u?this.comp.addOrRemoveCssClass("ag-pivot-leaf-group",c):h||this.comp.addOrRemoveCssClass("ag-row-group-leaf-indent",!p)},t.prototype.onRowNodeIsExpandableChanged=function(){this.showExpandAndContractIcons(),this.setIndent(),this.refreshAriaExpanded()},t.prototype.setupIndent=function(){var e=this.params.node,r=this.params.suppressPadding;r||(this.addManagedListener(e,Oe.EVENT_UI_LEVEL_CHANGED,this.setIndent.bind(this)),this.setIndent())},t.prototype.setIndent=function(){if(!this.gridOptionsWrapper.isGroupHideOpenParents()){var e=this.params,r=e.node,n=!!e.colDef,o=this.gridOptionsWrapper.isTreeData(),a=!n||o||e.colDef.showRowGroup===!0,s=a?r.uiLevel:0,l=e.padding>=0;l&&Ye(function(){return console.warn("AG Grid: cellRendererParams.padding no longer works, it was deprecated in since v14.2 and removed in v26, configuring padding for groupCellRenderer should be done with Sass variables and themes. Please see the AG Grid documentation page for Themes, in particular the property $row-group-indent-size.")},"groupCellRenderer->doDeprecatedWay"),this.indentClass&&this.comp.addOrRemoveCssClass(this.indentClass,!1),this.indentClass="ag-row-group-indent-"+s,this.comp.addOrRemoveCssClass(this.indentClass,!0)}},t.prototype.addFullWidthRowDraggerIfNeeded=function(){var e=this;if(!(!this.params.fullWidth||!this.params.rowDrag)){var r=new Jh(function(){return e.params.value},this.params.node);this.createManagedBean(r,this.context),this.eGui.insertAdjacentElement("afterbegin",r.getGui())}},t.prototype.isUserWantsSelected=function(){var e=this.params.checkbox;return typeof e=="function"?e(this.params):e===!0},t.prototype.addCheckboxIfNeeded=function(){var e=this,r=this.displayedGroupNode,n=this.isUserWantsSelected()&&!r.footer&&!r.rowPinned&&!r.detail;if(n){var o=new XC;this.getContext().createBean(o),o.init({rowNode:r,column:this.params.column}),this.eCheckbox.appendChild(o.getGui()),this.addDestroyFunc(function(){return e.getContext().destroyBean(o)})}this.comp.setCheckboxVisible(n)},t.prototype.onKeyDown=function(e){var r=e.key===j.ENTER;if(!(!r||this.params.suppressEnterExpand)){var n=this.params.column&&this.params.column.isCellEditable(this.params.node);n||this.onExpandOrContract(e)}},t.prototype.onCellDblClicked=function(e){if(!Ai(e)){var r=$h(this.eExpanded,e)||$h(this.eContracted,e);r||this.onExpandOrContract(e)}},su([y("expressionService")],t.prototype,"expressionService",void 0),su([y("valueFormatterService")],t.prototype,"valueFormatterService",void 0),su([y("columnModel")],t.prototype,"columnModel",void 0),su([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),su([y("gridOptions")],t.prototype,"gridOptions",void 0),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var eN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),lu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ry=function(i){eN(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.init=function(e){var r=this,n={setInnerRenderer:function(l,u){return r.setRenderDetails(l,u)},setChildCount:function(l){return r.eChildCount.innerHTML=l},addOrRemoveCssClass:function(l,u){return r.addOrRemoveCssClass(l,u)},setContractedDisplayed:function(l){return pt(r.eContracted,l)},setExpandedDisplayed:function(l){return pt(r.eExpanded,l)},setCheckboxVisible:function(l){return r.eCheckbox.classList.toggle("ag-invisible",!l)}},o=this.createManagedBean(new bE),a=!e.colDef,s=this.getGui();o.init(n,s,this.eCheckbox,this.eExpanded,this.eContracted,this.constructor,e),a&&_r(s,"gridcell")},t.prototype.setRenderDetails=function(e,r){var n=this;if(e){var o=e.newAgStackInstance();if(!o)return;o.then(function(a){if(!!a){var s=function(){return n.context.destroyBean(a)};n.isAlive()?(n.eValue.appendChild(a.getGui()),n.addDestroyFunc(s)):s()}})}else this.eValue.innerText=r},t.prototype.destroy=function(){this.getContext().destroyBean(this.innerCellRenderer),i.prototype.destroy.call(this)},t.prototype.refresh=function(){return!1},t.TEMPLATE=`<span class="ag-cell-wrapper">
            <span class="ag-group-expanded" ref="eExpanded"></span>
            <span class="ag-group-contracted" ref="eContracted"></span>
            <span class="ag-group-checkbox ag-invisible" ref="eCheckbox"></span>
            <span class="ag-group-value" ref="eValue"></span>
            <span class="ag-group-child-count" ref="eChildCount"></span>
        </span>`,lu([I("eExpanded")],t.prototype,"eExpanded",void 0),lu([I("eContracted")],t.prototype,"eContracted",void 0),lu([I("eCheckbox")],t.prototype,"eCheckbox",void 0),lu([I("eValue")],t.prototype,"eValue",void 0),lu([I("eChildCount")],t.prototype,"eChildCount",void 0),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var tN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),d_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},rN=function(i){tN(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.init=function(e){e.node.failedLoad?this.setupFailed():this.setupLoading()},t.prototype.setupFailed=function(){this.eLoadingText.innerText="ERR"},t.prototype.setupLoading=function(){var e=Ft("groupLoading",this.gridOptionsWrapper,null);e&&this.eLoadingIcon.appendChild(e);var r=this.gridOptionsWrapper.getLocaleTextFunc();this.eLoadingText.innerText=r("loadingOoo","Loading")},t.prototype.refresh=function(e){return!1},t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.TEMPLATE=`<div class="ag-loading">
            <span class="ag-loading-icon" ref="eLoadingIcon"></span>
            <span class="ag-loading-text" ref="eLoadingText"></span>
        </div>`,d_([I("eLoadingIcon")],t.prototype,"eLoadingIcon",void 0),d_([I("eLoadingText")],t.prototype,"eLoadingText",void 0),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var nN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),iN=function(i){nN(t,i);function t(){return i.call(this)||this}return t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.init=function(e){var r=this.gridOptionsWrapper.getOverlayLoadingTemplate()?this.gridOptionsWrapper.getOverlayLoadingTemplate():t.DEFAULT_LOADING_OVERLAY_TEMPLATE,n=this.gridOptionsWrapper.getLocaleTextFunc(),o=r.replace("[LOADING...]",n("loadingOoo","Loading..."));this.setTemplate(o)},t.DEFAULT_LOADING_OVERLAY_TEMPLATE='<span class="ag-overlay-loading-center">[LOADING...]</span>',t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var oN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),aN=function(i){oN(t,i);function t(){return i.call(this)||this}return t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.init=function(e){var r=this.gridOptionsWrapper.getOverlayNoRowsTemplate()?this.gridOptionsWrapper.getOverlayNoRowsTemplate():t.DEFAULT_NO_ROWS_TEMPLATE,n=this.gridOptionsWrapper.getLocaleTextFunc(),o=r.replace("[NO_ROWS_TO_SHOW]",n("noRowsToShow","No Rows To Show"));this.setTemplate(o)},t.DEFAULT_NO_ROWS_TEMPLATE='<span class="ag-overlay-no-rows-center">[NO_ROWS_TO_SHOW]</span>',t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var sN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),lN=function(i){sN(t,i);function t(){return i.call(this,'<div class="ag-tooltip"></div>')||this}return t.prototype.init=function(e){var r=e.value;this.getGui().innerHTML=Ii(r)},t}(ws);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var uN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),mp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},_E=function(i){uN(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.agGridDefaults={agDateInput:xL,agColumnHeader:ML,agColumnGroupHeader:IL,agTextColumnFloatingFilter:fE,agNumberColumnFloatingFilter:PL,agDateColumnFloatingFilter:bL,agReadOnlyFloatingFilter:iL,agAnimateShowChangeCellRenderer:yE,agAnimateSlideCellRenderer:CE,agGroupCellRenderer:ry,agGroupRowRenderer:ry,agLoadingCellRenderer:rN,agCellEditor:Qh,agTextCellEditor:Qh,agSelectCellEditor:YC,agPopupTextCellEditor:mE,agPopupSelectCellEditor:vE,agLargeTextCellEditor:gE,agTextColumnFilter:Yc,agNumberColumnFilter:KC,agDateColumnFilter:$C,agLoadingOverlay:iN,agNoRowsOverlay:aN,agTooltipComponent:lN},e.agDeprecatedNames={set:{newComponentName:"agSetColumnFilter",propertyHolder:"filter"},text:{newComponentName:"agTextColumnFilter",propertyHolder:"filter"},number:{newComponentName:"agNumberColumnFilter",propertyHolder:"filter"},date:{newComponentName:"agDateColumnFilter",propertyHolder:"filter"},group:{newComponentName:"agGroupCellRenderer",propertyHolder:"cellRenderer"},animateShowChange:{newComponentName:"agAnimateShowChangeCellRenderer",propertyHolder:"cellRenderer"},animateSlide:{newComponentName:"agAnimateSlideCellRenderer",propertyHolder:"cellRenderer"},select:{newComponentName:"agSelectCellEditor",propertyHolder:"cellEditor"},largeText:{newComponentName:"agLargeTextCellEditor",propertyHolder:"cellEditor"},popupSelect:{newComponentName:"agPopupSelectCellEditor",propertyHolder:"cellEditor"},popupText:{newComponentName:"agPopupTextCellEditor",propertyHolder:"cellEditor"},richSelect:{newComponentName:"agRichSelectCellEditor",propertyHolder:"cellEditor"},headerComponent:{newComponentName:"agColumnHeader",propertyHolder:"headerComponent"}},e.jsComps={},e.fwComps={},e}return t.prototype.init=function(){var e=this;this.gridOptions.components!=null&&Ke(this.gridOptions.components,function(r,n){return e.registerJsComponent(r,n)}),this.gridOptions.frameworkComponents!=null&&Ke(this.gridOptions.frameworkComponents,function(r,n){return e.registerFwComponent(r,n)})},t.prototype.registerDefaultComponent=function(e,r){var n=this.translateIfDeprecated(e);if(this.agGridDefaults[n]){console.error("Trying to overwrite a default component. You should call registerComponent");return}this.agGridDefaults[n]=r},t.prototype.registerJsComponent=function(e,r){var n=this.translateIfDeprecated(e);if(this.fwComps[n]){console.error("Trying to register a component that you have already registered for frameworks: "+n);return}this.jsComps[n]=r},t.prototype.registerFwComponent=function(e,r){var n="AG Grid: As of v27, registering components via grid property frameworkComponents is deprecated. Instead register both JavaScript AND Framework Components via the components property.";Ye(function(){return console.warn(n)},"UserComponentRegistry.frameworkComponentsDeprecated");var o=this.translateIfDeprecated(e);this.fwComps[o]=r},t.prototype.retrieve=function(e){var r=this.translateIfDeprecated(e),n=function(c,p){return{componentFromFramework:p,component:c}},o=this.getFrameworkOverrides().frameworkComponent(r);if(o!=null)return n(o,!0);var a=this.fwComps[r];if(a)return n(a,!0);var s=this.jsComps[r];if(s){var l=this.getFrameworkOverrides().isFrameworkComponent(s);return n(s,l)}var u=this.agGridDefaults[r];return u?n(u,!1):(Object.keys(this.agGridDefaults).indexOf(r)<0&&console.warn("AG Grid: Looking for component ["+r+"] but it wasn't found."),null)},t.prototype.translateIfDeprecated=function(e){var r=this.agDeprecatedNames[e];return r!=null?(Ye(function(){console.warn("ag-grid. Since v15.0 component names have been renamed to be namespaced. You should rename "+r.propertyHolder+":"+e+" to "+r.propertyHolder+":"+r.newComponentName)},"DEPRECATE_COMPONENT_"+e),r.newComponentName):e},mp([y("gridOptions")],t.prototype,"gridOptions",void 0),mp([y("agComponentUtils")],t.prototype,"agComponentUtils",void 0),mp([B],t.prototype,"init",null),t=mp([Q("userComponentRegistry")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var cN={propertyName:"dateComponent",cellRenderer:!1},pN={propertyName:"headerComponent",cellRenderer:!1},hN={propertyName:"headerGroupComponent",cellRenderer:!1},f_={propertyName:"cellRenderer",cellRenderer:!0},dN={propertyName:"cellEditor",cellRenderer:!1},g_={propertyName:"innerRenderer",cellRenderer:!0},fN={propertyName:"loadingOverlayComponent",cellRenderer:!1},gN={propertyName:"noRowsOverlayComponent",cellRenderer:!1},vN={propertyName:"tooltipComponent",cellRenderer:!1},ny={propertyName:"filter",cellRenderer:!1},mN={propertyName:"floatingFilterComponent",cellRenderer:!1},yN={propertyName:"toolPanel",cellRenderer:!1},CN={propertyName:"statusPanel",cellRenderer:!1},wN={propertyName:"fullWidthCellRenderer",cellRenderer:!0},bN={propertyName:"loadingCellRenderer",cellRenderer:!0},_N={propertyName:"groupRowRenderer",cellRenderer:!0},SN={propertyName:"detailCellRenderer",cellRenderer:!0};/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var SE=function(){function i(){}return i.getFloatingFilterType=function(t){return this.filterToFloatingFilterMapping[t]},i.filterToFloatingFilterMapping={set:"agSetColumnFloatingFilter",agSetColumnFilter:"agSetColumnFloatingFilter",multi:"agMultiColumnFloatingFilter",agMultiColumnFilter:"agMultiColumnFloatingFilter",number:"agNumberColumnFloatingFilter",agNumberColumnFilter:"agNumberColumnFloatingFilter",date:"agDateColumnFloatingFilter",agDateColumnFilter:"agDateColumnFloatingFilter",text:"agTextColumnFloatingFilter",agTextColumnFilter:"agTextColumnFloatingFilter"},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var xN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ds=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},xE=function(i){xN(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getHeaderCompDetails=function(e,r){return this.getCompDetails(e,pN,"agColumnHeader",r)},t.prototype.getHeaderGroupCompDetails=function(e){var r=e.columnGroup.getColGroupDef();return this.getCompDetails(r,hN,"agColumnGroupHeader",e)},t.prototype.getFullWidthCellRendererDetails=function(e){return this.getCompDetails(this.gridOptions,wN,null,e,!0)},t.prototype.getFullWidthLoadingCellRendererDetails=function(e){return this.getCompDetails(this.gridOptions,bN,"agLoadingCellRenderer",e,!0)},t.prototype.getFullWidthGroupCellRendererDetails=function(e){return this.getCompDetails(this.gridOptions,_N,"agGroupRowRenderer",e,!0)},t.prototype.getFullWidthDetailCellRendererDetails=function(e){return this.getCompDetails(this.gridOptions,SN,"agDetailCellRenderer",e,!0)},t.prototype.getInnerRendererDetails=function(e,r){return this.getCompDetails(e,g_,null,r)},t.prototype.getFullWidthGroupRowInnerCellRenderer=function(e,r){return this.getCompDetails(e,g_,null,r)},t.prototype.getCellRendererDetails=function(e,r){return this.getCompDetails(e,f_,null,r)},t.prototype.getCellEditorDetails=function(e,r){return this.getCompDetails(e,dN,"agCellEditor",r,!0)},t.prototype.getFilterDetails=function(e,r,n){return this.getCompDetails(e,ny,n,r,!0)},t.prototype.getDateCompDetails=function(e){return this.getCompDetails(this.gridOptions,cN,"agDateInput",e,!0)},t.prototype.getLoadingOverlayCompDetails=function(e){return this.getCompDetails(this.gridOptions,fN,"agLoadingOverlay",e,!0)},t.prototype.getNoRowsOverlayCompDetails=function(e){return this.getCompDetails(this.gridOptions,gN,"agNoRowsOverlay",e,!0)},t.prototype.getTooltipCompDetails=function(e){return this.getCompDetails(e.colDef,vN,"agTooltipComponent",e,!0)},t.prototype.getSetFilterCellRendererDetails=function(e,r){return this.getCompDetails(e,f_,null,r)},t.prototype.getFloatingFilterCompDetails=function(e,r,n){return this.getCompDetails(e,mN,n,r)},t.prototype.getToolPanelCompDetails=function(e,r){return this.getCompDetails(e,yN,null,r,!0)},t.prototype.getStatusPanelCompDetails=function(e,r){return this.getCompDetails(e,CN,null,r,!0)},t.prototype.getCompDetails=function(e,r,n,o,a){var s=this;a===void 0&&(a=!1);var l=r.propertyName,u=r.cellRenderer,c=this.getCompKeys(e,r,o),p=c.compName,h=c.jsComp,d=c.fwComp,f=c.paramsFromSelector,g=c.popupFromSelector,v=c.popupPositionFromSelector,m=function(S){var T=s.userComponentRegistry.retrieve(S);T&&(h=T.componentFromFramework?void 0:T.component,d=T.componentFromFramework?T.component:void 0)};if(p!=null&&m(p),h==null&&d==null&&n!=null&&m(n),h&&u&&!this.agComponentUtils.doesImplementIComponent(h)&&(h=this.agComponentUtils.adaptFunction(l,h)),!h&&!d){a&&console.error("Could not find component "+p+", did you forget to configure this component?");return}var C=this.mergeParamsWithApplicationProvidedParams(e,r,o,f),w=h==null,b=h||d;return{componentFromFramework:w,componentClass:b,params:C,type:r,popupFromSelector:g,popupPositionFromSelector:v,newAgStackInstance:function(){return s.newAgStackInstance(b,w,C,r)}}},t.prototype.getCompKeys=function(e,r,n){var o=this,a=r.propertyName,s,l,u,c,p,h;if(e){var d=e,f=d[a+"Selector"],g=f?f(n):null,v=function(C,w){var b=function(){var T="AG Grid: As of v27, the property "+a+"Framework is deprecated. The property "+a+" can now be used for JavaScript AND Framework Components.";Ye(function(){return console.warn(T)},"UserComponentFactory."+a+"FrameworkDeprecated")};if(typeof C=="string")s=C;else if(typeof w=="string")b(),s=w;else if(C!=null&&C!==!0){var S=o.getFrameworkOverrides().isFrameworkComponent(C);S?u=C:l=C}else w!=null&&(b(),u=w)};if(g){if(g.frameworkComponent!=null){var m="AG Grid: As of v27, the return for "+a+"Selector has attributes [component, params] only. The attribute frameworkComponent is deprecated. You should now return back Framework Components using the 'component' attribute and the grid works out if it's a framework component or not.";Ye(function(){return console.warn(m)},"UserComponentFactory."+a+"FrameworkSelectorDeprecated"),v(g.frameworkComponent,void 0)}else v(g.component,void 0);c=g.params,p=g.popup,h=g.popupPosition}else v(d[a],d[a+"Framework"])}return{compName:s,jsComp:l,fwComp:u,paramsFromSelector:c,popupFromSelector:p,popupPositionFromSelector:h}},t.prototype.newAgStackInstance=function(e,r,n,o){var a=o.propertyName,s=!r,l;if(s)l=new e;else{var u=this.componentMetadataProvider.retrieve(a);l=this.frameworkComponentWrapper.wrap(e,u.mandatoryMethodList,u.optionalMethodList,o)}var c=this.initComponent(l,n);return c==null?ct.resolve(l):c.then(function(){return l})},t.prototype.mergeParamsWithApplicationProvidedParams=function(e,r,n,o){o===void 0&&(o=null);var a={context:this.gridOptionsWrapper.getContext(),columnApi:this.gridOptionsWrapper.getColumnApi(),api:this.gridOptionsWrapper.getApi()};Ln(a,n);var s=e,l=s&&s[r.propertyName+"Params"];if(typeof l=="function"){var u=l(n);Ln(a,u)}else typeof l=="object"&&Ln(a,l);return Ln(a,o),a},t.prototype.initComponent=function(e,r){if(this.context.createBean(e),e.init!=null)return e.init(r)},t.prototype.getDefaultFloatingFilterType=function(e){if(e==null)return null;var r=null,n=this.getCompKeys(e,ny),o=n.compName,a=n.jsComp,s=n.fwComp;if(o)r=SE.getFloatingFilterType(o);else{var l=a==null&&s==null&&e.filter===!0;if(l){var u=ge.isRegistered(oe.SetFilterModule);r=u?"agSetColumnFloatingFilter":"agTextColumnFloatingFilter"}}return r},Ds([y("gridOptions")],t.prototype,"gridOptions",void 0),Ds([y("agComponentUtils")],t.prototype,"agComponentUtils",void 0),Ds([y("componentMetadataProvider")],t.prototype,"componentMetadataProvider",void 0),Ds([y("userComponentRegistry")],t.prototype,"userComponentRegistry",void 0),Ds([De("frameworkComponentWrapper")],t.prototype,"frameworkComponentWrapper",void 0),t=Ds([Q("userComponentFactory")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var to;(function(i){i[i.SINGLE_SHEET=0]="SINGLE_SHEET",i[i.MULTI_SHEET=1]="MULTI_SHEET"})(to||(to={}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var TN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ca=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},TE=function(i){TN(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.dragEndFunctions=[],e.dragSources=[],e}return t.prototype.init=function(){this.logger=this.loggerFactory.create("DragService")},t.prototype.removeAllListeners=function(){this.dragSources.forEach(this.removeListener.bind(this)),this.dragSources.length=0},t.prototype.removeListener=function(e){var r=e.dragSource.eElement,n=e.mouseDownListener;if(r.removeEventListener("mousedown",n),e.touchEnabled){var o=e.touchStartListener;r.removeEventListener("touchstart",o,{passive:!0})}},t.prototype.removeDragSource=function(e){var r=this.dragSources.find(function(n){return n.dragSource===e});!r||(this.removeListener(r),gr(this.dragSources,r))},t.prototype.isDragging=function(){return this.dragging},t.prototype.addDragSource=function(e,r){var n=this;r===void 0&&(r=!1);var o=this.onMouseDown.bind(this,e);e.eElement.addEventListener("mousedown",o);var a=null,s=this.gridOptionsWrapper.isSuppressTouch();r&&!s&&(a=function(l){l.cancelable&&l.preventDefault(),n.onTouchStart(e,l)},e.eElement.addEventListener("touchstart",a,{passive:!0})),this.dragSources.push({dragSource:e,mouseDownListener:o,touchStartListener:a,touchEnabled:r})},t.prototype.onTouchStart=function(e,r){var n=this;this.currentDragParams=e,this.dragging=!1;var o=r.touches[0];this.touchLastTime=o,this.touchStart=o;var a=function(p){return n.onTouchMove(p,e.eElement)},s=function(p){return n.onTouchUp(p,e.eElement)},l=function(p){p.cancelable&&p.preventDefault()},u=e.eElement,c=[{target:document,type:"touchmove",listener:l,options:{passive:!1}},{target:u,type:"touchmove",listener:a,options:{passive:!0}},{target:u,type:"touchend",listener:s,options:{passive:!0}},{target:u,type:"touchcancel",listener:s,options:{passive:!0}}];this.addTemporaryEvents(c),e.dragStartPixels===0&&this.onCommonMove(o,this.touchStart,e.eElement)},t.prototype.onMouseDown=function(e,r){var n=this,o=r;if(!(e.skipMouseEvent&&e.skipMouseEvent(r))&&!o._alreadyProcessedByDragService&&(o._alreadyProcessedByDragService=!0,r.button===0)){this.currentDragParams=e,this.dragging=!1,this.mouseStartEvent=r;var a=this.gridOptionsWrapper.getDocument(),s=function(h){return n.onMouseMove(h,e.eElement)},l=function(h){return n.onMouseUp(h,e.eElement)},u=function(h){return h.preventDefault()},c=a,p=[{target:c,type:"mousemove",listener:s},{target:c,type:"mouseup",listener:l},{target:c,type:"contextmenu",listener:u}];this.addTemporaryEvents(p),e.dragStartPixels===0&&this.onMouseMove(r,e.eElement)}},t.prototype.addTemporaryEvents=function(e){e.forEach(function(r){var n=r.target,o=r.type,a=r.listener,s=r.options;n.addEventListener(o,a,s)}),this.dragEndFunctions.push(function(){e.forEach(function(r){var n=r.target,o=r.type,a=r.listener,s=r.options;n.removeEventListener(o,a,s)})})},t.prototype.isEventNearStartEvent=function(e,r){var n=this.currentDragParams.dragStartPixels,o=q(n)?n:4;return jC(e,r,o)},t.prototype.getFirstActiveTouch=function(e){for(var r=0;r<e.length;r++)if(e[r].identifier===this.touchStart.identifier)return e[r];return null},t.prototype.onCommonMove=function(e,r,n){if(!this.dragging){if(!this.dragging&&this.isEventNearStartEvent(e,r))return;this.dragging=!0;var o={type:x.EVENT_DRAG_STARTED,api:this.gridApi,columnApi:this.columnApi,target:n};this.eventService.dispatchEvent(o),this.currentDragParams.onDragStart(r),this.currentDragParams.onDragging(r)}this.currentDragParams.onDragging(e)},t.prototype.onTouchMove=function(e,r){var n=this.getFirstActiveTouch(e.touches);!n||this.onCommonMove(n,this.touchStart,r)},t.prototype.onMouseMove=function(e,r){Hc()&&e.type==="mousemove"&&e.cancelable&&this.mouseEventService.isEventFromThisGrid(e)&&!this.isOverFormFieldElement(e)&&e.preventDefault(),this.onCommonMove(e,this.mouseStartEvent,r)},t.prototype.isOverFormFieldElement=function(e){var r,n,o=e.target,a=(r=o)===null||r===void 0?void 0:r.tagName.toLocaleLowerCase();return!!(!((n=a)===null||n===void 0)&&n.match("^a$|textarea|input|select|button"))},t.prototype.onTouchUp=function(e,r){var n=this.getFirstActiveTouch(e.changedTouches);n||(n=this.touchLastTime),this.onUpCommon(n,r)},t.prototype.onMouseUp=function(e,r){this.onUpCommon(e,r)},t.prototype.onUpCommon=function(e,r){if(this.dragging){this.dragging=!1,this.currentDragParams.onDragStop(e);var n={type:x.EVENT_DRAG_STOPPED,api:this.gridApi,columnApi:this.columnApi,target:r};this.eventService.dispatchEvent(n)}this.mouseStartEvent=null,this.touchStart=null,this.touchLastTime=null,this.currentDragParams=null,this.dragEndFunctions.forEach(function(o){return o()}),this.dragEndFunctions.length=0},Ca([y("loggerFactory")],t.prototype,"loggerFactory",void 0),Ca([y("columnApi")],t.prototype,"columnApi",void 0),Ca([y("gridApi")],t.prototype,"gridApi",void 0),Ca([y("mouseEventService")],t.prototype,"mouseEventService",void 0),Ca([B],t.prototype,"init",null),Ca([nt],t.prototype,"removeAllListeners",null),t=Ca([Q("dragService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ON=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),td=globalThis&&globalThis.__assign||function(){return td=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},td.apply(this,arguments)},ui=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},OE=function(i){ON(t,i);function t(){var r=i!==null&&i.apply(this,arguments)||this;return r.allColumnFilters=new Map,r.activeAggregateFilters=[],r.activeColumnFilters=[],r.quickFilter=null,r.quickFilterParts=null,r.processingFilterChange=!1,r}e=t,t.prototype.init=function(){var r=this;this.addManagedListener(this.eventService,x.EVENT_GRID_COLUMNS_CHANGED,function(){return r.onColumnsChanged()}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VALUE_CHANGED,function(){return r.refreshFiltersForAggregations()}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_CHANGED,function(){return r.refreshFiltersForAggregations()}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,function(){return r.refreshFiltersForAggregations()}),this.quickFilter=this.parseQuickFilter(this.gridOptionsWrapper.getQuickFilterText()),this.setQuickFilterParts(),this.allowShowChangeAfterFilter=this.gridOptionsWrapper.isAllowShowChangeAfterFilter()},t.prototype.setQuickFilterParts=function(){this.quickFilterParts=this.quickFilter?this.quickFilter.split(" "):null},t.prototype.setFilterModel=function(r){var n=this,o=[],a=this.getFilterModel();if(r){var s=UC(Object.keys(r));this.allColumnFilters.forEach(function(l,u){var c=r[u];o.push(n.setModelOnFilterWrapper(l.filterPromise,c)),s.delete(u)}),s.forEach(function(l){var u=n.columnModel.getPrimaryColumn(l)||n.columnModel.getGridColumn(l);if(!u){console.warn("AG Grid: setFilterModel() - no column found for colId: "+l);return}if(!u.isFilterAllowed()){console.warn("AG Grid: setFilterModel() - unable to fully apply model, filtering disabled for colId: "+l);return}var c=n.getOrCreateFilterWrapper(u,"NO_UI");if(!c){console.warn("AG-Grid: setFilterModel() - unable to fully apply model, unable to create filter for colId: "+l);return}o.push(n.setModelOnFilterWrapper(c.filterPromise,r[l]))})}else this.allColumnFilters.forEach(function(l){o.push(n.setModelOnFilterWrapper(l.filterPromise,null))});ct.all(o).then(function(){var l=n.getFilterModel(),u=[];n.allColumnFilters.forEach(function(c,p){var h=a?a[p]:null,d=l?l[p]:null;_.jsonEquals(h,d)||u.push(c.column)}),u.length>0&&n.onFilterChanged({columns:u})})},t.prototype.setModelOnFilterWrapper=function(r,n){return new ct(function(o){r.then(function(a){typeof a.setModel!="function"&&(console.warn("AG Grid: filter missing setModel method, which is needed for setFilterModel"),o()),(a.setModel(n)||ct.resolve()).then(function(){return o()})})})},t.prototype.getFilterModel=function(){var r={};return this.allColumnFilters.forEach(function(n,o){var a=n.filterPromise,s=a.resolveNow(null,function(u){return u});if(s==null)return null;if(typeof s.getModel!="function"){console.warn("AG Grid: filter API missing getModel method, which is needed for getFilterModel");return}var l=s.getModel();q(l)&&(r[o]=l)}),r},t.prototype.isColumnFilterPresent=function(){return this.activeColumnFilters.length>0},t.prototype.isAggregateFilterPresent=function(){return!!this.activeAggregateFilters.length},t.prototype.doAggregateFiltersPass=function(r,n){return this.doColumnFiltersPass(r,n,!0)},t.prototype.updateActiveFilters=function(){var r=this;this.activeColumnFilters.length=0,this.activeAggregateFilters.length=0;var n=function(s){return s?s.isFilterActive?s.isFilterActive():(console.warn("AG Grid: Filter is missing isFilterActive() method"),!1):!1},o=!!this.gridOptionsWrapper.getGroupAggFiltering(),a=function(s){var l=!s.isPrimary();if(l)return!0;var u=!r.columnModel.isPivotActive(),c=s.isValueActive();return!c||!u?!1:r.columnModel.isPivotMode()?!0:o};this.allColumnFilters.forEach(function(s){if(s.filterPromise.resolveNow(!1,n)){var l=s.filterPromise.resolveNow(null,function(u){return u});a(s.column)?r.activeAggregateFilters.push(l):r.activeColumnFilters.push(l)}})},t.prototype.updateFilterFlagInColumns=function(r,n){this.allColumnFilters.forEach(function(o){var a=o.filterPromise.resolveNow(!1,function(s){return s.isFilterActive()});o.column.setFilterActive(a,r,n)})},t.prototype.isAnyFilterPresent=function(){return this.isQuickFilterPresent()||this.isColumnFilterPresent()||this.isAggregateFilterPresent()||this.gridOptionsWrapper.isExternalFilterPresent()},t.prototype.doColumnFiltersPass=function(r,n,o){for(var a=r.data,s=r.aggData,l=o?this.activeAggregateFilters:this.activeColumnFilters,u=o?s:a,c=0;c<l.length;c++){var p=l[c];if(!(p==null||p===n)){if(typeof p.doesFilterPass!="function")throw new Error("Filter is missing method doesFilterPass");if(!p.doesFilterPass({node:r,data:u}))return!1}}return!0},t.prototype.parseQuickFilter=function(r){return q(r)?this.gridOptionsWrapper.isRowModelDefault()?r.toUpperCase():(console.warn("AG Grid - Quick filtering only works with the Client-Side Row Model"),null):null},t.prototype.setQuickFilter=function(r){if(r!=null&&typeof r!="string"){console.warn("AG Grid - setQuickFilter() only supports string inputs, received: "+typeof r);return}var n=this.parseQuickFilter(r);this.quickFilter!==n&&(this.quickFilter=n,this.setQuickFilterParts(),this.onFilterChanged())},t.prototype.refreshFiltersForAggregations=function(){var r=this.gridOptionsWrapper.getGroupAggFiltering();r&&this.onFilterChanged()},t.prototype.callOnFilterChangedOutsideRenderCycle=function(r){var n=this;r===void 0&&(r={});var o=function(){return n.onFilterChanged(r)};this.rowRenderer.isRefreshInProgress()?setTimeout(o,0):o()},t.prototype.onFilterChanged=function(r){r===void 0&&(r={});var n=r.filterInstance,o=r.additionalEventAttributes,a=r.columns;this.updateActiveFilters(),this.updateFilterFlagInColumns("filterChanged",o),this.allColumnFilters.forEach(function(l){!l.filterPromise||l.filterPromise.then(function(u){u&&u!==n&&u.onAnyFilterChanged&&u.onAnyFilterChanged()})});var s={type:x.EVENT_FILTER_CHANGED,api:this.gridApi,columnApi:this.columnApi,columns:a||[]};o&&Ln(s,o),this.processingFilterChange=!0,this.eventService.dispatchEvent(s),this.processingFilterChange=!1},t.prototype.isSuppressFlashingCellsBecauseFiltering=function(){return!this.allowShowChangeAfterFilter&&this.processingFilterChange},t.prototype.isQuickFilterPresent=function(){return this.quickFilter!==null},t.prototype.doesRowPassOtherFilters=function(r,n){return this.doesRowPassFilter({rowNode:n,filterInstanceToSkip:r})},t.prototype.doesRowPassQuickFilterNoCache=function(r,n){var o=this,a=this.columnModel.getAllColumnsForQuickFilter();return a.some(function(s){var l=o.getQuickFilterTextForColumn(s,r);return q(l)&&l.indexOf(n)>=0})},t.prototype.doesRowPassQuickFilterCache=function(r,n){return r.quickFilterAggregateText||this.aggregateRowForQuickFilter(r),r.quickFilterAggregateText.indexOf(n)>=0},t.prototype.doesRowPassQuickFilter=function(r){var n=this,o=this.gridOptionsWrapper.isCacheQuickFilter();return this.quickFilterParts.every(function(a){return o?n.doesRowPassQuickFilterCache(r,a):n.doesRowPassQuickFilterNoCache(r,a)})},t.prototype.doesRowPassAggregateFilters=function(r){return!(this.isAggregateFilterPresent()&&!this.doAggregateFiltersPass(r.rowNode,r.filterInstanceToSkip))},t.prototype.doesRowPassFilter=function(r){return!(this.isQuickFilterPresent()&&!this.doesRowPassQuickFilter(r.rowNode)||this.gridOptionsWrapper.isExternalFilterPresent()&&!this.gridOptionsWrapper.doesExternalFilterPass(r.rowNode)||this.isColumnFilterPresent()&&!this.doColumnFiltersPass(r.rowNode,r.filterInstanceToSkip))},t.prototype.getQuickFilterTextForColumn=function(r,n){var o=this.valueService.getValue(r,n,!0),a=r.getColDef();if(a.getQuickFilterText){var s={value:o,node:n,data:n.data,column:r,colDef:a,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()};o=a.getQuickFilterText(s)}return q(o)?o.toString().toUpperCase():null},t.prototype.aggregateRowForQuickFilter=function(r){var n=this,o=[],a=this.columnModel.getAllColumnsForQuickFilter();a.forEach(function(s){var l=n.getQuickFilterTextForColumn(s,r);q(l)&&o.push(l)}),r.quickFilterAggregateText=o.join(e.QUICK_FILTER_SEPARATOR)},t.prototype.onNewRowsLoaded=function(r){this.allColumnFilters.forEach(function(n){n.filterPromise.then(function(o){o.onNewRowsLoaded&&o.onNewRowsLoaded()})}),this.updateFilterFlagInColumns(r),this.updateActiveFilters()},t.prototype.createValueGetter=function(r){var n=this;return function(o){var a=o.node;return n.valueService.getValue(r,a,!0)}},t.prototype.getFilterComponent=function(r,n,o){o===void 0&&(o=!0);var a;if(o)return((a=this.getOrCreateFilterWrapper(r,n))===null||a===void 0?void 0:a.filterPromise)||null;var s=this.cachedFilter(r);return s?s.filterPromise:null},t.prototype.isFilterActive=function(r){var n=this.cachedFilter(r);return!!n&&n.filterPromise.resolveNow(!1,function(o){return o.isFilterActive()})},t.prototype.getOrCreateFilterWrapper=function(r,n){if(!r.isFilterAllowed())return null;var o=this.cachedFilter(r);return o?n!=="NO_UI"&&this.putIntoGui(o,n):(o=this.createFilterWrapper(r,n),this.allColumnFilters.set(r.getColId(),o)),o},t.prototype.cachedFilter=function(r){return this.allColumnFilters.get(r.getColId())},t.prototype.createFilterInstance=function(r){var n=this,o=ge.isRegistered(oe.SetFilterModule)?"agSetColumnFilter":"agTextColumnFilter",a=r.getColDef(),s,l=td(td({},this.createFilterParams(r,a)),{filterModifiedCallback:function(){var p={type:x.EVENT_FILTER_MODIFIED,api:n.gridApi,columnApi:n.columnApi,column:r,filterInstance:s};n.eventService.dispatchEvent(p)},filterChangedCallback:function(p){var h={filterInstance:s,additionalEventAttributes:p,columns:[r]};n.callOnFilterChangedOutsideRenderCycle(h)},doesRowPassOtherFilter:function(p){return n.doesRowPassOtherFilters(s,p)}}),u=this.userComponentFactory.getFilterDetails(a,l,o);if(!u)return null;var c=u.newAgStackInstance();return c&&c.then(function(p){return s=p}),c},t.prototype.createFilterParams=function(r,n){var o={api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),column:r,colDef:bC(n),rowModel:this.rowModel,filterChangedCallback:function(){},filterModifiedCallback:function(){},valueGetter:this.createValueGetter(r),context:this.gridOptionsWrapper.getContext(),doesRowPassOtherFilter:function(){return!0}};return o},t.prototype.createFilterWrapper=function(r,n){var o={column:r,filterPromise:null,compiledElement:null,guiPromise:ct.resolve(null)};return o.filterPromise=this.createFilterInstance(r),o.filterPromise&&this.putIntoGui(o,n),o},t.prototype.putIntoGui=function(r,n){var o=this,a=document.createElement("div");a.className="ag-filter",r.guiPromise=new ct(function(s){r.filterPromise.then(function(l){var u=l.getGui();q(u)||console.warn("AG Grid: getGui method from filter returned "+u+", it should be a DOM element or an HTML template string."),typeof u=="string"&&(u=Kt(u)),a.appendChild(u),s(a),o.eventService.dispatchEvent({type:x.EVENT_FILTER_OPENED,column:r.column,source:n,eGui:a,api:o.gridApi,columnApi:o.columnApi})})})},t.prototype.onColumnsChanged=function(){var r=this,n=[];this.allColumnFilters.forEach(function(o,a){var s;o.column.isPrimary()?s=r.columnModel.getPrimaryColumn(a):s=r.columnModel.getGridColumn(a),!s&&(n.push(o.column),r.disposeFilterWrapper(o,"filterDestroyed"))}),n.length>0&&this.onFilterChanged({columns:n})},t.prototype.destroyFilter=function(r,n){n===void 0&&(n="api");var o=this.allColumnFilters.get(r.getColId());o&&(this.disposeFilterWrapper(o,n),this.onFilterChanged({columns:[r]}))},t.prototype.disposeFilterWrapper=function(r,n){var o=this;r.filterPromise.then(function(a){(a.setModel(null)||ct.resolve()).then(function(){o.getContext().destroyBean(a),r.column.setFilterActive(!1,n),o.allColumnFilters.delete(r.column.getColId())})})},t.prototype.destroy=function(){var r=this;i.prototype.destroy.call(this),this.allColumnFilters.forEach(function(n){return r.disposeFilterWrapper(n,"filterDestroyed")})};var e;return t.QUICK_FILTER_SEPARATOR=`
`,ui([y("valueService")],t.prototype,"valueService",void 0),ui([y("columnModel")],t.prototype,"columnModel",void 0),ui([y("rowModel")],t.prototype,"rowModel",void 0),ui([y("columnApi")],t.prototype,"columnApi",void 0),ui([y("gridApi")],t.prototype,"gridApi",void 0),ui([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),ui([y("rowRenderer")],t.prototype,"rowRenderer",void 0),ui([B],t.prototype,"init",null),ui([nt],t.prototype,"destroy",null),t=e=ui([Q("filterManager")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var EN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),qC=function(i){EN(t,i);function t(e,r){var n=i.call(this,e)||this;return n.ctrl=r,n}return t.prototype.getCtrl=function(){return this.ctrl},t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var RN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),yp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},EE=function(i){RN(t,i);function t(e){return i.call(this,t.TEMPLATE,e)||this}return t.prototype.postConstruct=function(){var e=this,r=this.getGui(),n={addOrRemoveCssClass:function(o,a){return e.addOrRemoveCssClass(o,a)},addOrRemoveBodyCssClass:function(o,a){return e.eFloatingFilterBody.classList.toggle(o,a)},addOrRemoveButtonWrapperCssClass:function(o,a){return e.eButtonWrapper.classList.toggle(o,a)},setCompDetails:function(o){return e.setCompDetails(o)},getFloatingFilterComp:function(){return e.compPromise},setWidth:function(o){return r.style.width=o},setMenuIcon:function(o){return e.eButtonShowMainFilter.appendChild(o)}};this.ctrl.setComp(n,r,this.eButtonShowMainFilter,this.eFloatingFilterBody)},t.prototype.setCompDetails=function(e){var r=this;this.compPromise=e.newAgStackInstance(),this.compPromise.then(function(n){return r.afterCompCreated(n)})},t.prototype.afterCompCreated=function(e){var r=this;!e||(this.addDestroyFunc(function(){return r.context.destroyBean(e)}),this.isAlive()&&(this.eFloatingFilterBody.appendChild(e.getGui()),e.afterGuiAttached&&e.afterGuiAttached()))},t.TEMPLATE=`<div class="ag-header-cell ag-floating-filter" role="gridcell" tabindex="-1">
            <div ref="eFloatingFilterBody" role="presentation"></div>
            <div class="ag-floating-filter-button ag-hidden" ref="eButtonWrapper" role="presentation">
                <button type="button" aria-label="Open Filter Menu" class="ag-floating-filter-button-button" ref="eButtonShowMainFilter" tabindex="-1"></button>
            </div>
        </div>`,yp([I("eFloatingFilterBody")],t.prototype,"eFloatingFilterBody",void 0),yp([I("eButtonWrapper")],t.prototype,"eButtonWrapper",void 0),yp([I("eButtonShowMainFilter")],t.prototype,"eButtonShowMainFilter",void 0),yp([B],t.prototype,"postConstruct",null),t}(qC);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var RE=function(){function i(){}return i.parse=function(t){if(!t)return null;if(t===!0)return{toolPanels:[i.DEFAULT_COLUMN_COMP,i.DEFAULT_FILTER_COMP],defaultToolPanel:"columns"};if(typeof t=="string")return i.parse([t]);if(Array.isArray(t)){var e=[];return t.forEach(function(n){var o=i.DEFAULT_BY_KEY[n];if(!o){console.warn("AG Grid: the key "+n+" is not a valid key for specifying a tool panel, valid keys are: "+Object.keys(i.DEFAULT_BY_KEY).join(","));return}e.push(o)}),e.length===0?null:{toolPanels:e,defaultToolPanel:e[0].id}}var r={toolPanels:i.parseComponents(t.toolPanels),defaultToolPanel:t.defaultToolPanel,hiddenByDefault:t.hiddenByDefault,position:t.position};return r},i.parseComponents=function(t){var e=[];return t&&t.forEach(function(r){var n=null;if(typeof r=="string"){var o=i.DEFAULT_BY_KEY[r];if(!o){console.warn("AG Grid: the key "+r+" is not a valid key for specifying a tool panel, valid keys are: "+Object.keys(i.DEFAULT_BY_KEY).join(","));return}n=o}else n=r;e.push(n)}),e},i.DEFAULT_COLUMN_COMP={id:"columns",labelDefault:"Columns",labelKey:"columns",iconKey:"columns",toolPanel:"agColumnsToolPanel"},i.DEFAULT_FILTER_COMP={id:"filters",labelDefault:"Filters",labelKey:"filters",iconKey:"filter",toolPanel:"agFiltersToolPanel"},i.DEFAULT_BY_KEY={columns:i.DEFAULT_COLUMN_COMP,filters:i.DEFAULT_FILTER_COMP},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var rd=globalThis&&globalThis.__assign||function(){return rd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},rd.apply(this,arguments)},So=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},v_=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},PN=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},xg=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(PN(arguments[t]));return i},AN=25,DN=300,MN=5,FN=5,IN=10;function K(i){return i===!0||i==="true"}function Tg(i){if(typeof i=="number")return i;if(typeof i=="string")return parseInt(i,10)}function LN(i,t){return i>=0?i:t}function m_(i,t){var e=parseInt(i,10);return Xh(e)&&e>0?e:t}var ce=function(){function i(){this.propertyEventService=new oi,this.domDataKey="__AG_"+Math.random().toString(),this.destroyed=!1}t=i,i.prototype.agWire=function(e,r){this.gridOptions.api=e,this.gridOptions.columnApi=r,this.checkForDeprecated(),this.checkForViolations()},i.prototype.destroy=function(){this.gridOptions.api=null,this.gridOptions.columnApi=null,this.destroyed=!0},i.prototype.init=function(){var e=this;this.gridOptions.suppressPropertyNamesCheck!==!0&&(this.checkGridOptionsProperties(),this.checkColumnDefProperties()),this.gridOptions.sideBar!=null&&(this.gridOptions.sideBar=RE.parse(this.gridOptions.sideBar));var r=this.useAsyncEvents();if(this.eventService.addGlobalListener(this.globalEventHandler.bind(this),r),this.isGroupSelectsChildren()&&this.isSuppressParentsInRowNodes()&&console.warn("AG Grid: 'groupSelectsChildren' does not work with 'suppressParentsInRowNodes', this selection method needs the part in rowNode to work"),this.isGroupSelectsChildren()&&(this.isRowSelectionMulti()||console.warn("AG Grid: rowSelection must be 'multiple' for groupSelectsChildren to make sense"),this.isRowModelServerSide()&&console.warn("AG Grid: group selects children is NOT support for Server Side Row Model. This is because the rows are lazy loaded, so selecting a group is not possible asthe grid has no way of knowing what the children are.")),this.isGroupRemoveSingleChildren()&&this.isGroupHideOpenParents()&&console.warn("AG Grid: groupRemoveSingleChildren and groupHideOpenParents do not work with each other, you need to pick one. And don't ask us how to us these together on our support forum either you will get the same answer!"),this.isRowModelServerSide()){var n=function(a){return"AG Grid: '"+a+"' is not supported on the Server-Side Row Model"};q(this.gridOptions.groupDefaultExpanded)&&console.warn(n("groupDefaultExpanded")),q(this.gridOptions.groupDefaultExpanded)&&console.warn(n("groupIncludeFooter")),q(this.gridOptions.groupDefaultExpanded)&&console.warn(n("groupIncludeTotalFooter"))}this.isEnableRangeSelection()&&ge.assertRegistered(oe.RangeSelectionModule,"enableRangeSelection"),!this.isEnableRangeSelection()&&(this.isEnableRangeHandle()||this.isEnableFillHandle())&&console.warn("AG Grid: 'enableRangeHandle' and 'enableFillHandle' will not work unless 'enableRangeSelection' is set to true");var o=function(a){e.gridOptions.icons&&e.gridOptions.icons[a]&&console.warn("gridOptions.icons."+a+" is no longer supported. For information on how to style checkboxes and radio buttons, see https://www.ag-grid.com/javascript-grid-icons/")};o("radioButtonOff"),o("radioButtonOn"),o("checkboxChecked"),o("checkboxUnchecked"),o("checkboxIndeterminate"),this.getScrollbarWidth()},i.prototype.checkColumnDefProperties=function(){var e=this;this.gridOptions.columnDefs!=null&&this.gridOptions.columnDefs.forEach(function(r){var n=Object.getOwnPropertyNames(r),o=xg(Um.ALL_PROPERTIES,Um.FRAMEWORK_PROPERTIES);e.checkProperties(n,o,o,"colDef","https://www.ag-grid.com/javascript-grid-column-properties/")})},i.prototype.checkGridOptionsProperties=function(){var e=Object.getOwnPropertyNames(this.gridOptions),r=xg(bi.ALL_PROPERTIES,bi.FRAMEWORK_PROPERTIES,ms(x).map(function(o){return ea.getCallbackForEvent(o)})),n=xg(r,["api","columnApi"]);this.checkProperties(e,n,r,"gridOptions","https://www.ag-grid.com/javascript-data-grid/grid-options/")},i.prototype.checkProperties=function(e,r,n,o,a){var s=eE(e,r,n);Ke(s,function(l,u){console.warn("ag-grid: invalid "+o+" property '"+l+"' did you mean any of these: "+u.slice(0,8).join(", "))}),Object.keys(s).length>0&&console.warn("ag-grid: to see all the valid "+o+" properties please check: "+a)},i.prototype.mergeGridCommonParams=function(e){var r=this;if(e){var n=function(o){var a=rd(rd({},o),{api:r.getApi(),columnApi:r.getColumnApi(),context:r.getContext()});return e(a)};return n}return e},i.prototype.getDomDataKey=function(){return this.domDataKey},i.prototype.getDomData=function(e,r){var n=e[this.getDomDataKey()];return n?n[r]:void 0},i.prototype.setDomData=function(e,r,n){var o=this.getDomDataKey(),a=e[o];me(a)&&(a={},e[o]=a),a[r]=n},i.prototype.isRowSelection=function(){return this.gridOptions.rowSelection==="single"||this.gridOptions.rowSelection==="multiple"},i.prototype.isSuppressRowDeselection=function(){return K(this.gridOptions.suppressRowDeselection)},i.prototype.isRowSelectionMulti=function(){return this.gridOptions.rowSelection==="multiple"},i.prototype.isRowMultiSelectWithClick=function(){return K(this.gridOptions.rowMultiSelectWithClick)},i.prototype.getContext=function(){return this.gridOptions.context},i.prototype.isPivotMode=function(){return K(this.gridOptions.pivotMode)},i.prototype.isSuppressExpandablePivotGroups=function(){return K(this.gridOptions.suppressExpandablePivotGroups)},i.prototype.getPivotColumnGroupTotals=function(){return this.gridOptions.pivotColumnGroupTotals},i.prototype.getPivotRowTotals=function(){return this.gridOptions.pivotRowTotals},i.prototype.isRowModelInfinite=function(){return this.gridOptions.rowModelType===L.ROW_MODEL_TYPE_INFINITE},i.prototype.isRowModelViewport=function(){return this.gridOptions.rowModelType===L.ROW_MODEL_TYPE_VIEWPORT},i.prototype.isRowModelServerSide=function(){return this.gridOptions.rowModelType===L.ROW_MODEL_TYPE_SERVER_SIDE},i.prototype.isRowModelDefault=function(){return me(this.gridOptions.rowModelType)||this.gridOptions.rowModelType===L.ROW_MODEL_TYPE_CLIENT_SIDE},i.prototype.isFullRowEdit=function(){return this.gridOptions.editType==="fullRow"},i.prototype.isSuppressFocusAfterRefresh=function(){return K(this.gridOptions.suppressFocusAfterRefresh)},i.prototype.isSuppressBrowserResizeObserver=function(){return K(this.gridOptions.suppressBrowserResizeObserver)},i.prototype.isSuppressMaintainUnsortedOrder=function(){return K(this.gridOptions.suppressMaintainUnsortedOrder)},i.prototype.isSuppressClearOnFillReduction=function(){return K(this.gridOptions.suppressClearOnFillReduction)},i.prototype.isShowToolPanel=function(){return K(this.gridOptions.sideBar&&Array.isArray(this.getSideBar().toolPanels))},i.prototype.getSideBar=function(){return this.gridOptions.sideBar},i.prototype.isSuppressTouch=function(){return K(this.gridOptions.suppressTouch)},i.prototype.isMaintainColumnOrder=function(){return K(this.gridOptions.maintainColumnOrder)},i.prototype.isSuppressRowTransform=function(){return K(this.gridOptions.suppressRowTransform)},i.prototype.isSuppressColumnStateEvents=function(){return K(this.gridOptions.suppressColumnStateEvents)},i.prototype.isAllowDragFromColumnsToolPanel=function(){return K(this.gridOptions.allowDragFromColumnsToolPanel)},i.prototype.useAsyncEvents=function(){return!K(this.gridOptions.suppressAsyncEvents)},i.prototype.isEnableCellChangeFlash=function(){return K(this.gridOptions.enableCellChangeFlash)},i.prototype.getCellFlashDelay=function(){return this.gridOptions.cellFlashDelay||500},i.prototype.getCellFadeDelay=function(){return this.gridOptions.cellFadeDelay||1e3},i.prototype.isGroupSelectsChildren=function(){return K(this.gridOptions.groupSelectsChildren)},i.prototype.isSuppressRowHoverHighlight=function(){return K(this.gridOptions.suppressRowHoverHighlight)},i.prototype.isColumnHoverHighlight=function(){return K(this.gridOptions.columnHoverHighlight)},i.prototype.isGroupSelectsFiltered=function(){return K(this.gridOptions.groupSelectsFiltered)},i.prototype.isGroupHideOpenParents=function(){return K(this.gridOptions.groupHideOpenParents)},i.prototype.isGroupMaintainOrder=function(){return K(this.gridOptions.groupMaintainOrder)},i.prototype.getAutoGroupColumnDef=function(){return this.gridOptions.autoGroupColumnDef},i.prototype.isGroupMultiAutoColumn=function(){return this.gridOptions.groupDisplayType?this.matchesGroupDisplayType("multipleColumns",this.gridOptions.groupDisplayType):K(this.gridOptions.groupHideOpenParents)},i.prototype.isGroupUseEntireRow=function(e){return e?!1:this.gridOptions.groupDisplayType?this.matchesGroupDisplayType("groupRows",this.gridOptions.groupDisplayType):!1},i.prototype.isGroupSuppressAutoColumn=function(){var e=this.gridOptions.groupDisplayType?this.matchesGroupDisplayType("custom",this.gridOptions.groupDisplayType):!1;return e?!0:this.gridOptions.treeDataDisplayType?this.matchesTreeDataDisplayType("custom",this.gridOptions.treeDataDisplayType):!1},i.prototype.isGroupRemoveSingleChildren=function(){return K(this.gridOptions.groupRemoveSingleChildren)},i.prototype.isGroupRemoveLowestSingleChildren=function(){return K(this.gridOptions.groupRemoveLowestSingleChildren)},i.prototype.isGroupIncludeFooter=function(){return K(this.gridOptions.groupIncludeFooter)},i.prototype.isGroupIncludeTotalFooter=function(){return K(this.gridOptions.groupIncludeTotalFooter)},i.prototype.isGroupSuppressBlankHeader=function(){return K(this.gridOptions.groupSuppressBlankHeader)},i.prototype.isSuppressRowClickSelection=function(){return K(this.gridOptions.suppressRowClickSelection)},i.prototype.isSuppressCellFocus=function(){return K(this.gridOptions.suppressCellFocus)},i.prototype.isSuppressMultiSort=function(){return K(this.gridOptions.suppressMultiSort)},i.prototype.isAlwaysMultiSort=function(){return K(this.gridOptions.alwaysMultiSort)},i.prototype.isMultiSortKeyCtrl=function(){return this.gridOptions.multiSortKey==="ctrl"},i.prototype.isPivotSuppressAutoColumn=function(){return K(this.gridOptions.pivotSuppressAutoColumn)},i.prototype.isSuppressDragLeaveHidesColumns=function(){return K(this.gridOptions.suppressDragLeaveHidesColumns)},i.prototype.isSuppressScrollOnNewData=function(){return K(this.gridOptions.suppressScrollOnNewData)},i.prototype.isSuppressScrollWhenPopupsAreOpen=function(){return K(this.gridOptions.suppressScrollWhenPopupsAreOpen)},i.prototype.isRowDragEntireRow=function(){return K(this.gridOptions.rowDragEntireRow)},i.prototype.isSuppressRowDrag=function(){return K(this.gridOptions.suppressRowDrag)},i.prototype.isRowDragManaged=function(){return K(this.gridOptions.rowDragManaged)},i.prototype.isSuppressMoveWhenRowDragging=function(){return K(this.gridOptions.suppressMoveWhenRowDragging)},i.prototype.isRowDragMultiRow=function(){return K(this.gridOptions.rowDragMultiRow)},i.prototype.getDomLayout=function(){var e=this.gridOptions.domLayout||L.DOM_LAYOUT_NORMAL,r=[L.DOM_LAYOUT_PRINT,L.DOM_LAYOUT_AUTO_HEIGHT,L.DOM_LAYOUT_NORMAL];return r.indexOf(e)===-1?(Ye(function(){return console.warn("AG Grid: "+e+" is not valid for DOM Layout, valid values are "+L.DOM_LAYOUT_NORMAL+", "+L.DOM_LAYOUT_AUTO_HEIGHT+" and "+L.DOM_LAYOUT_PRINT)},"warn about dom layout values"),L.DOM_LAYOUT_NORMAL):e},i.prototype.isSuppressHorizontalScroll=function(){return K(this.gridOptions.suppressHorizontalScroll)},i.prototype.isSuppressMaxRenderedRowRestriction=function(){return K(this.gridOptions.suppressMaxRenderedRowRestriction)},i.prototype.isExcludeChildrenWhenTreeDataFiltering=function(){return K(this.gridOptions.excludeChildrenWhenTreeDataFiltering)},i.prototype.isAlwaysShowHorizontalScroll=function(){return K(this.gridOptions.alwaysShowHorizontalScroll)},i.prototype.isAlwaysShowVerticalScroll=function(){return K(this.gridOptions.alwaysShowVerticalScroll)},i.prototype.isDebounceVerticalScrollbar=function(){return K(this.gridOptions.debounceVerticalScrollbar)},i.prototype.isSuppressLoadingOverlay=function(){return K(this.gridOptions.suppressLoadingOverlay)},i.prototype.isSuppressNoRowsOverlay=function(){return K(this.gridOptions.suppressNoRowsOverlay)},i.prototype.isSuppressFieldDotNotation=function(){return K(this.gridOptions.suppressFieldDotNotation)},i.prototype.getPinnedTopRowData=function(){return this.gridOptions.pinnedTopRowData},i.prototype.getPinnedBottomRowData=function(){return this.gridOptions.pinnedBottomRowData},i.prototype.isFunctionsPassive=function(){return K(this.gridOptions.functionsPassive)},i.prototype.isSuppressChangeDetection=function(){return K(this.gridOptions.suppressChangeDetection)},i.prototype.isSuppressAnimationFrame=function(){return K(this.gridOptions.suppressAnimationFrame)},i.prototype.getQuickFilterText=function(){return this.gridOptions.quickFilterText},i.prototype.isCacheQuickFilter=function(){return K(this.gridOptions.cacheQuickFilter)},i.prototype.isUnSortIcon=function(){return K(this.gridOptions.unSortIcon)},i.prototype.isSuppressMenuHide=function(){return K(this.gridOptions.suppressMenuHide)},i.prototype.isEnterMovesDownAfterEdit=function(){return K(this.gridOptions.enterMovesDownAfterEdit)},i.prototype.isEnterMovesDown=function(){return K(this.gridOptions.enterMovesDown)},i.prototype.isUndoRedoCellEditing=function(){return K(this.gridOptions.undoRedoCellEditing)},i.prototype.getUndoRedoCellEditingLimit=function(){return Tg(this.gridOptions.undoRedoCellEditingLimit)},i.prototype.getRowStyle=function(){return this.gridOptions.rowStyle},i.prototype.getRowClass=function(){return this.gridOptions.rowClass},i.prototype.getRowStyleFunc=function(){return this.mergeGridCommonParams(this.gridOptions.getRowStyle)},i.prototype.getRowClassFunc=function(){return this.mergeGridCommonParams(this.gridOptions.getRowClass)},i.prototype.rowClassRules=function(){return this.gridOptions.rowClassRules},i.prototype.getServerSideStoreType=function(){return this.gridOptions.serverSideStoreType},i.prototype.getServerSideStoreParamsFunc=function(){return this.mergeGridCommonParams(this.gridOptions.getServerSideStoreParams)},i.prototype.getCreateChartContainerFunc=function(){return this.mergeGridCommonParams(this.gridOptions.createChartContainer)},i.prototype.getPopupParent=function(){return this.gridOptions.popupParent},i.prototype.getBlockLoadDebounceMillis=function(){return this.gridOptions.blockLoadDebounceMillis},i.prototype.getPostProcessPopupFunc=function(){return this.mergeGridCommonParams(this.gridOptions.postProcessPopup)},i.prototype.getPaginationNumberFormatterFunc=function(){return this.mergeGridCommonParams(this.gridOptions.paginationNumberFormatter)},i.prototype.getChildCountFunc=function(){return this.gridOptions.getChildCount},i.prototype.getIsApplyServerSideTransactionFunc=function(){return this.mergeGridCommonParams(this.gridOptions.isApplyServerSideTransaction)},i.prototype.getInitialGroupOrderComparator=function(){var e=this.gridOptions,r=e.initialGroupOrderComparator,n=e.defaultGroupOrderComparator;if(r)return this.mergeGridCommonParams(r);if(n)return function(o){return n(o.nodeA,o.nodeB)}},i.prototype.getIsFullWidthCellFunc=function(){var e=this.gridOptions,r=e.isFullWidthRow,n=e.isFullWidthCell;if(r)return this.mergeGridCommonParams(r);if(n)return function(o){return n(o.rowNode)}},i.prototype.getFullWidthCellRendererParams=function(){return this.gridOptions.fullWidthCellRendererParams},i.prototype.isEmbedFullWidthRows=function(){return K(this.gridOptions.embedFullWidthRows)||K(this.gridOptions.deprecatedEmbedFullWidthRows)},i.prototype.isDetailRowAutoHeight=function(){return K(this.gridOptions.detailRowAutoHeight)},i.prototype.getSuppressKeyboardEventFunc=function(){return this.gridOptions.suppressKeyboardEvent},i.prototype.getBusinessKeyForNodeFunc=function(){return this.gridOptions.getBusinessKeyForNode},i.prototype.getApi=function(){return this.gridOptions.api},i.prototype.getColumnApi=function(){return this.gridOptions.columnApi},i.prototype.isReadOnlyEdit=function(){return K(this.gridOptions.readOnlyEdit)},i.prototype.isImmutableData=function(){var e=this.gridOptions.getRowId!=null,r=K(this.gridOptions.immutableData),n=K(this.gridOptions.resetRowDataOnUpdate);return n?!1:e||r},i.prototype.isEnsureDomOrder=function(){return K(this.gridOptions.ensureDomOrder)},i.prototype.isEnableCharts=function(){return K(this.gridOptions.enableCharts)?ge.assertRegistered(oe.GridChartsModule,"enableCharts"):!1},i.prototype.getColResizeDefault=function(){return this.gridOptions.colResizeDefault},i.prototype.isSingleClickEdit=function(){return K(this.gridOptions.singleClickEdit)},i.prototype.isSuppressClickEdit=function(){return K(this.gridOptions.suppressClickEdit)},i.prototype.isStopEditingWhenCellsLoseFocus=function(){return K(this.gridOptions.stopEditingWhenCellsLoseFocus)},i.prototype.getGroupDefaultExpanded=function(){return this.gridOptions.groupDefaultExpanded},i.prototype.getMaxConcurrentDatasourceRequests=function(){var e=Tg(this.gridOptions.maxConcurrentDatasourceRequests);if(e==null)return 2;if(!(e<=0))return e},i.prototype.getMaxBlocksInCache=function(){return this.gridOptions.maxBlocksInCache},i.prototype.getCacheOverflowSize=function(){return this.gridOptions.cacheOverflowSize},i.prototype.getPaginationPageSize=function(){return Tg(this.gridOptions.paginationPageSize)},i.prototype.isPaginateChildRows=function(){var e=this.isGroupRemoveSingleChildren()||this.isGroupRemoveLowestSingleChildren();return e?!0:K(this.gridOptions.paginateChildRows)},i.prototype.getCacheBlockSize=function(){return m_(this.gridOptions.cacheBlockSize)},i.prototype.getInfiniteInitialRowCount=function(){return this.gridOptions.infiniteInitialRowCount},i.prototype.isPurgeClosedRowNodes=function(){return K(this.gridOptions.purgeClosedRowNodes)},i.prototype.isSuppressPaginationPanel=function(){return K(this.gridOptions.suppressPaginationPanel)},i.prototype.getRowData=function(){return this.gridOptions.rowData},i.prototype.isEnableRtl=function(){return K(this.gridOptions.enableRtl)},i.prototype.getRowGroupPanelShow=function(){return this.gridOptions.rowGroupPanelShow},i.prototype.getPivotPanelShow=function(){return this.gridOptions.pivotPanelShow},i.prototype.isAngularCompileRows=function(){return K(this.gridOptions.angularCompileRows)},i.prototype.isAngularCompileFilters=function(){return K(this.gridOptions.angularCompileFilters)},i.prototype.isDebug=function(){return K(this.gridOptions.debug)},i.prototype.getColumnDefs=function(){return this.gridOptions.columnDefs},i.prototype.getColumnTypes=function(){return this.gridOptions.columnTypes},i.prototype.getDatasource=function(){return this.gridOptions.datasource},i.prototype.getViewportDatasource=function(){return this.gridOptions.viewportDatasource},i.prototype.getServerSideDatasource=function(){return this.gridOptions.serverSideDatasource},i.prototype.isAccentedSort=function(){return K(this.gridOptions.accentedSort)},i.prototype.isEnableBrowserTooltips=function(){return K(this.gridOptions.enableBrowserTooltips)},i.prototype.isEnableCellExpressions=function(){return K(this.gridOptions.enableCellExpressions)},i.prototype.isEnableGroupEdit=function(){return K(this.gridOptions.enableGroupEdit)},i.prototype.isSuppressMiddleClickScrolls=function(){return K(this.gridOptions.suppressMiddleClickScrolls)},i.prototype.isPreventDefaultOnContextMenu=function(){return K(this.gridOptions.preventDefaultOnContextMenu)},i.prototype.isSuppressPreventDefaultOnMouseWheel=function(){return K(this.gridOptions.suppressPreventDefaultOnMouseWheel)},i.prototype.isSuppressColumnVirtualisation=function(){return K(this.gridOptions.suppressColumnVirtualisation)},i.prototype.isSuppressRowVirtualisation=function(){return K(this.gridOptions.suppressRowVirtualisation)},i.prototype.isSuppressContextMenu=function(){return K(this.gridOptions.suppressContextMenu)},i.prototype.isAllowContextMenuWithControlKey=function(){return K(this.gridOptions.allowContextMenuWithControlKey)},i.prototype.isSuppressCopyRowsToClipboard=function(){return K(this.gridOptions.suppressCopyRowsToClipboard)},i.prototype.isSuppressCopySingleCellRanges=function(){return K(this.gridOptions.suppressCopySingleCellRanges)},i.prototype.isCopyHeadersToClipboard=function(){return K(this.gridOptions.copyHeadersToClipboard)},i.prototype.isCopyGroupHeadersToClipboard=function(){return K(this.gridOptions.copyGroupHeadersToClipboard)},i.prototype.isSuppressClipboardPaste=function(){return K(this.gridOptions.suppressClipboardPaste)},i.prototype.isSuppressLastEmptyLineOnPaste=function(){return K(this.gridOptions.suppressLastEmptyLineOnPaste)},i.prototype.isPagination=function(){return K(this.gridOptions.pagination)},i.prototype.isSuppressEnterpriseResetOnNewColumns=function(){return K(this.gridOptions.suppressEnterpriseResetOnNewColumns)},i.prototype.getProcessDataFromClipboardFunc=function(){return this.mergeGridCommonParams(this.gridOptions.processDataFromClipboard)},i.prototype.getAsyncTransactionWaitMillis=function(){return q(this.gridOptions.asyncTransactionWaitMillis)?this.gridOptions.asyncTransactionWaitMillis:L.BATCH_WAIT_MILLIS},i.prototype.isSuppressMovableColumns=function(){return K(this.gridOptions.suppressMovableColumns)},i.prototype.isAnimateRows=function(){return this.isEnsureDomOrder()?!1:K(this.gridOptions.animateRows)},i.prototype.isSuppressColumnMoveAnimation=function(){return K(this.gridOptions.suppressColumnMoveAnimation)},i.prototype.isSuppressAggFuncInHeader=function(){return K(this.gridOptions.suppressAggFuncInHeader)},i.prototype.isSuppressAggAtRootLevel=function(){return K(this.gridOptions.suppressAggAtRootLevel)},i.prototype.isSuppressAggFilteredOnly=function(){var e=this.getGroupAggFiltering()!==void 0;return e||K(this.gridOptions.suppressAggFilteredOnly)},i.prototype.isRemovePivotHeaderRowWhenSingleValueColumn=function(){return K(this.gridOptions.removePivotHeaderRowWhenSingleValueColumn)},i.prototype.isShowOpenedGroup=function(){return K(this.gridOptions.showOpenedGroup)},i.prototype.isReactUi=function(){return K(this.gridOptions.reactUi)},i.prototype.isSuppressReactUi=function(){return K(this.gridOptions.suppressReactUi)},i.prototype.isEnableRangeSelection=function(){return ge.isRegistered(oe.RangeSelectionModule)&&K(this.gridOptions.enableRangeSelection)},i.prototype.isEnableRangeHandle=function(){return K(this.gridOptions.enableRangeHandle)},i.prototype.isEnableFillHandle=function(){return K(this.gridOptions.enableFillHandle)},i.prototype.getFillHandleDirection=function(){var e=this.gridOptions.fillHandleDirection;return e?e!=="x"&&e!=="y"&&e!=="xy"?(Ye(function(){return console.warn("AG Grid: valid values for fillHandleDirection are 'x', 'y' and 'xy'. Default to 'xy'.")},"warn invalid fill direction"),"xy"):e:"xy"},i.prototype.getFillOperation=function(){return this.mergeGridCommonParams(this.gridOptions.fillOperation)},i.prototype.isSuppressMultiRangeSelection=function(){return K(this.gridOptions.suppressMultiRangeSelection)},i.prototype.isPaginationAutoPageSize=function(){return K(this.gridOptions.paginationAutoPageSize)},i.prototype.isRememberGroupStateWhenNewData=function(){return K(this.gridOptions.rememberGroupStateWhenNewData)},i.prototype.getIcons=function(){return this.gridOptions.icons},i.prototype.getGroupAggFiltering=function(){var e=this.gridOptions.groupAggFiltering;if(typeof e=="function")return this.mergeGridCommonParams(e);if(K(e))return function(){return!0}},i.prototype.getAggFuncs=function(){return this.gridOptions.aggFuncs},i.prototype.getSortingOrder=function(){return this.gridOptions.sortingOrder},i.prototype.getAlignedGrids=function(){return this.gridOptions.alignedGrids},i.prototype.isMasterDetail=function(){var e=K(this.gridOptions.masterDetail);return e?ge.assertRegistered(oe.MasterDetailModule,"masterDetail"):!1},i.prototype.isKeepDetailRows=function(){return K(this.gridOptions.keepDetailRows)},i.prototype.getKeepDetailRowsCount=function(){var e=this.gridOptions.keepDetailRowsCount;return q(e)&&e>0?this.gridOptions.keepDetailRowsCount:IN},i.prototype.getIsRowMasterFunc=function(){return this.gridOptions.isRowMaster},i.prototype.getIsRowSelectableFunc=function(){return this.gridOptions.isRowSelectable},i.prototype.getGroupRowRendererParams=function(){return this.gridOptions.groupRowRendererParams},i.prototype.getOverlayLoadingTemplate=function(){return this.gridOptions.overlayLoadingTemplate},i.prototype.getOverlayNoRowsTemplate=function(){return this.gridOptions.overlayNoRowsTemplate},i.prototype.isSuppressAutoSize=function(){return K(this.gridOptions.suppressAutoSize)},i.prototype.isEnableCellTextSelection=function(){return K(this.gridOptions.enableCellTextSelection)},i.prototype.isSuppressParentsInRowNodes=function(){return K(this.gridOptions.suppressParentsInRowNodes)},i.prototype.isSuppressClipboardApi=function(){return K(this.gridOptions.suppressClipboardApi)},i.prototype.isFunctionsReadOnly=function(){return K(this.gridOptions.functionsReadOnly)},i.prototype.isEnableCellTextSelect=function(){return K(this.gridOptions.enableCellTextSelection)},i.prototype.getDefaultColDef=function(){return this.gridOptions.defaultColDef},i.prototype.getDefaultColGroupDef=function(){return this.gridOptions.defaultColGroupDef},i.prototype.getDefaultExportParams=function(e){if(this.gridOptions.defaultExportParams)return console.warn("AG Grid: Since v25.2 `defaultExportParams`  has been replaced by `default"+Hm(e)+"ExportParams`'"),e==="csv"?this.gridOptions.defaultExportParams:this.gridOptions.defaultExportParams;if(e==="csv"&&this.gridOptions.defaultCsvExportParams)return this.gridOptions.defaultCsvExportParams;if(e==="excel"&&this.gridOptions.defaultExcelExportParams)return this.gridOptions.defaultExcelExportParams},i.prototype.isSuppressCsvExport=function(){return K(this.gridOptions.suppressCsvExport)},i.prototype.isAllowShowChangeAfterFilter=function(){return K(this.gridOptions.allowShowChangeAfterFilter)},i.prototype.isSuppressExcelExport=function(){return K(this.gridOptions.suppressExcelExport)},i.prototype.isSuppressMakeColumnVisibleAfterUnGroup=function(){return K(this.gridOptions.suppressMakeColumnVisibleAfterUnGroup)},i.prototype.getDataPathFunc=function(){return this.gridOptions.getDataPath},i.prototype.getIsServerSideGroupFunc=function(){return this.gridOptions.isServerSideGroup},i.prototype.getIsServerSideGroupOpenByDefaultFunc=function(){return this.mergeGridCommonParams(this.gridOptions.isServerSideGroupOpenByDefault)},i.prototype.getIsGroupOpenByDefaultFunc=function(){return this.mergeGridCommonParams(this.gridOptions.isGroupOpenByDefault)},i.prototype.getServerSideGroupKeyFunc=function(){return this.gridOptions.getServerSideGroupKey},i.prototype.getGroupRowAggFunc=function(){var e=this.gridOptions,r=e.getGroupRowAgg,n=e.groupRowAggNodes;if(r)return this.mergeGridCommonParams(r);if(n)return function(o){return n(o.nodes)}},i.prototype.getContextMenuItemsFunc=function(){return this.mergeGridCommonParams(this.gridOptions.getContextMenuItems)},i.prototype.getMainMenuItemsFunc=function(){return this.mergeGridCommonParams(this.gridOptions.getMainMenuItems)},i.prototype.getRowIdFunc=function(){var e=this.gridOptions,r=e.getRowId,n=e.getRowNodeId;if(r)return this.mergeGridCommonParams(r);if(n)return function(o){return n(o.data)}},i.prototype.getNavigateToNextHeaderFunc=function(){return this.mergeGridCommonParams(this.gridOptions.navigateToNextHeader)},i.prototype.getTabToNextHeaderFunc=function(){return this.mergeGridCommonParams(this.gridOptions.tabToNextHeader)},i.prototype.getNavigateToNextCellFunc=function(){return this.mergeGridCommonParams(this.gridOptions.navigateToNextCell)},i.prototype.getTabToNextCellFunc=function(){return this.mergeGridCommonParams(this.gridOptions.tabToNextCell)},i.prototype.getGridTabIndex=function(){return(this.gridOptions.tabIndex||0).toString()},i.prototype.isTreeData=function(){var e=K(this.gridOptions.treeData);return e?ge.assertRegistered(oe.RowGroupingModule,"Tree Data"):!1},i.prototype.isValueCache=function(){return K(this.gridOptions.valueCache)},i.prototype.isValueCacheNeverExpires=function(){return K(this.gridOptions.valueCacheNeverExpires)},i.prototype.isDeltaSort=function(){return K(this.gridOptions.deltaSort)},i.prototype.isAggregateOnlyChangedColumns=function(){return K(this.gridOptions.aggregateOnlyChangedColumns)},i.prototype.getProcessSecondaryColDefFunc=function(){return this.gridOptions.processSecondaryColDef},i.prototype.getProcessSecondaryColGroupDefFunc=function(){return this.gridOptions.processSecondaryColGroupDef},i.prototype.getSendToClipboardFunc=function(){return this.mergeGridCommonParams(this.gridOptions.sendToClipboard)},i.prototype.getProcessRowPostCreateFunc=function(){return this.mergeGridCommonParams(this.gridOptions.processRowPostCreate)},i.prototype.getProcessCellForClipboardFunc=function(){return this.mergeGridCommonParams(this.gridOptions.processCellForClipboard)},i.prototype.getProcessHeaderForClipboardFunc=function(){return this.mergeGridCommonParams(this.gridOptions.processHeaderForClipboard)},i.prototype.getProcessGroupHeaderForClipboardFunc=function(){return this.mergeGridCommonParams(this.gridOptions.processGroupHeaderForClipboard)},i.prototype.getProcessCellFromClipboardFunc=function(){return this.mergeGridCommonParams(this.gridOptions.processCellFromClipboard)},i.prototype.getViewportRowModelPageSize=function(){return m_(this.gridOptions.viewportRowModelPageSize,MN)},i.prototype.getViewportRowModelBufferSize=function(){return LN(this.gridOptions.viewportRowModelBufferSize,FN)},i.prototype.isServerSideSortingAlwaysResets=function(){return K(this.gridOptions.serverSideSortingAlwaysResets)},i.prototype.isServerSideFilteringAlwaysResets=function(){return K(this.gridOptions.serverSideFilteringAlwaysResets)},i.prototype.getPostSortFunc=function(){var e=this.gridOptions,r=e.postSortRows,n=e.postSort;if(r)return this.mergeGridCommonParams(r);if(n)return function(o){return n(o.nodes)}},i.prototype.getChartToolbarItemsFunc=function(){return this.mergeGridCommonParams(this.gridOptions.getChartToolbarItems)},i.prototype.getChartThemeOverrides=function(){return this.gridOptions.chartThemeOverrides},i.prototype.getCustomChartThemes=function(){return this.gridOptions.customChartThemes},i.prototype.getChartThemes=function(){return this.gridOptions.chartThemes||["ag-default","ag-material","ag-pastel","ag-vivid","ag-solar"]},i.prototype.getClipboardDelimiter=function(){return q(this.gridOptions.clipboardDelimiter)?this.gridOptions.clipboardDelimiter:"	"},i.prototype.setProperty=function(e,r,n){n===void 0&&(n=!1);var o=this.gridOptions,a=o[e];if(n||a!==r){o[e]=r;var s={type:e,currentValue:r,previousValue:a};this.propertyEventService.dispatchEvent(s)}},i.prototype.addEventListener=function(e,r){this.propertyEventService.addEventListener(e,r)},i.prototype.removeEventListener=function(e,r){this.propertyEventService.removeEventListener(e,r)},i.prototype.isSkipHeaderOnAutoSize=function(){return!!this.gridOptions.skipHeaderOnAutoSize},i.prototype.getAutoSizePadding=function(){var e=this.gridOptions.autoSizePadding;return e!=null&&e>=0?e:20},i.prototype.getHeaderHeight=function(){return typeof this.gridOptions.headerHeight=="number"?this.gridOptions.headerHeight:this.getFromTheme(25,"headerHeight")},i.prototype.getFloatingFiltersHeight=function(){return typeof this.gridOptions.floatingFiltersHeight=="number"?this.gridOptions.floatingFiltersHeight:this.getFromTheme(25,"headerHeight")},i.prototype.getGroupHeaderHeight=function(){return typeof this.gridOptions.groupHeaderHeight=="number"?this.gridOptions.groupHeaderHeight:this.getHeaderHeight()},i.prototype.getPivotHeaderHeight=function(){return typeof this.gridOptions.pivotHeaderHeight=="number"?this.gridOptions.pivotHeaderHeight:this.getHeaderHeight()},i.prototype.getPivotGroupHeaderHeight=function(){return typeof this.gridOptions.pivotGroupHeaderHeight=="number"?this.gridOptions.pivotGroupHeaderHeight:this.getGroupHeaderHeight()},i.prototype.isExternalFilterPresent=function(){return typeof this.gridOptions.isExternalFilterPresent=="function"?this.gridOptions.isExternalFilterPresent({api:this.getApi(),columnApi:this.getColumnApi(),context:this.getContext()}):!1},i.prototype.doesExternalFilterPass=function(e){return typeof this.gridOptions.doesExternalFilterPass=="function"?this.gridOptions.doesExternalFilterPass(e):!1},i.prototype.getTooltipDelay=function(e){var r=this.gridOptions,n=r.tooltipShowDelay,o=r.tooltipHideDelay,a=e==="show"?n:o,s=Hm(e);return q(a)?(a<0&&Ye(function(){return console.warn("ag-grid: tooltip"+s+"Delay should not be lower than 0")},"tooltip"+s+"DelayWarn"),Math.max(200,a)):null},i.prototype.isTooltipMouseTrack=function(){return K(this.gridOptions.tooltipMouseTrack)},i.prototype.isSuppressModelUpdateAfterUpdateTransaction=function(){return K(this.gridOptions.suppressModelUpdateAfterUpdateTransaction)},i.prototype.getDocument=function(){var e=null;return this.gridOptions.getDocument&&q(this.gridOptions.getDocument)?e=this.gridOptions.getDocument():this.eGridDiv&&(e=this.eGridDiv.ownerDocument),e&&q(e)?e:document},i.prototype.getMinColWidth=function(){var e=this.gridOptions.minColWidth;if(q(e)&&e>t.MIN_COL_WIDTH)return this.gridOptions.minColWidth;var r=this.getFromTheme(null,"headerCellMinWidth");return q(r)?Math.max(r,t.MIN_COL_WIDTH):t.MIN_COL_WIDTH},i.prototype.getMaxColWidth=function(){return this.gridOptions.maxColWidth&&this.gridOptions.maxColWidth>t.MIN_COL_WIDTH?this.gridOptions.maxColWidth:null},i.prototype.getColWidth=function(){return typeof this.gridOptions.colWidth!="number"||this.gridOptions.colWidth<t.MIN_COL_WIDTH?200:this.gridOptions.colWidth},i.prototype.getRowBuffer=function(){var e=this.gridOptions.rowBuffer;return typeof e=="number"?e<0&&(Ye(function(){return console.warn("AG Grid: rowBuffer should not be negative")},"warn rowBuffer negative"),this.gridOptions.rowBuffer=e=0):e=L.ROW_BUFFER_SIZE,e},i.prototype.getRowBufferInPixels=function(){var e=this.getRowBuffer(),r=this.getRowHeightAsNumber();return e*r},i.prototype.getScrollbarWidth=function(){if(this.scrollbarWidth==null){var e=typeof this.gridOptions.scrollbarWidth=="number"&&this.gridOptions.scrollbarWidth>=0,r=e?this.gridOptions.scrollbarWidth:OO();r!=null&&(this.scrollbarWidth=r,this.eventService.dispatchEvent({type:x.EVENT_SCROLLBAR_WIDTH_CHANGED}))}return this.scrollbarWidth},i.prototype.checkForDeprecated=function(){var e=this.gridOptions;e.deprecatedEmbedFullWidthRows&&console.warn("AG Grid: since v21.2, deprecatedEmbedFullWidthRows has been replaced with embedFullWidthRows."),e.rowDeselection&&console.warn("AG Grid: since v24.x, rowDeselection is deprecated and the behaviour is true by default. Please use `suppressRowDeselection` to prevent rows from being deselected."),e.enableMultiRowDragging&&(e.rowDragMultiRow=!0,delete e.enableMultiRowDragging,console.warn("AG Grid: since v26.1, `enableMultiRowDragging` is deprecated. Please use `rowDragMultiRow`."));var r=function(a,s,l){e[a]!=null&&(console.warn("AG Grid: since version "+l+", '"+a+"' is deprecated / renamed, please use the new property name '"+s+"' instead."),e[s]==null&&(e[s]=e[a]))};if(r("batchUpdateWaitMillis","asyncTransactionWaitMillis","23.1.x"),r("deltaRowDataMode","immutableData","23.1.x"),(e.immutableColumns||e.deltaColumnMode)&&console.warn("AG Grid: since v24.0, immutableColumns and deltaColumnMode properties are gone. The grid now works like this as default. To keep column order maintained, set grid property applyColumnDefOrder=true"),r("suppressSetColumnStateEvents","suppressColumnStateEvents","24.0.x"),e.groupRowInnerRenderer||e.groupRowInnerRendererParams||e.groupRowInnerRendererFramework){console.warn("AG Grid: since v24.0, grid properties groupRowInnerRenderer, groupRowInnerRendererFramework and groupRowInnerRendererParams are no longer used."),console.warn("  Instead use the grid properties groupRowRendererParams.innerRenderer, groupRowRendererParams.innerRendererFramework and groupRowRendererParams.innerRendererParams."),console.warn("  For example instead of this:"),console.warn('    groupRowInnerRenderer: "myRenderer"'),console.warn("    groupRowInnerRendererParams: {x: a}"),console.warn("  Replace with this:"),console.warn("    groupRowRendererParams: {"),console.warn('      innerRenderer: "myRenderer",'),console.warn("      innerRendererParams: {x: a}"),console.warn("    }"),console.warn("  We have copied the properties over for you. However to stop this error message, please change your application code."),e.groupRowRendererParams||(e.groupRowRendererParams={});var n=e.groupRowRendererParams;e.groupRowInnerRenderer&&(n.innerRenderer=e.groupRowInnerRenderer),e.groupRowInnerRendererParams&&(n.innerRendererParams=e.groupRowInnerRendererParams),e.groupRowInnerRendererFramework&&(n.innerRendererFramework=e.groupRowInnerRendererFramework)}if(e.rememberGroupStateWhenNewData&&console.warn("AG Grid: since v24.0, grid property rememberGroupStateWhenNewData is deprecated. This feature was provided before Transaction Updates worked (which keep group state). Now that transaction updates are possible and they keep group state, this feature is no longer needed."),e.detailCellRendererParams&&e.detailCellRendererParams.autoHeight&&(console.warn("AG Grid: since v24.1, grid property detailCellRendererParams.autoHeight is replaced with grid property detailRowAutoHeight. This allows this feature to work when you provide a custom DetailCellRenderer"),e.detailRowAutoHeight=!0),e.suppressKeyboardEvent&&console.warn(`AG Grid: since v24.1 suppressKeyboardEvent in the gridOptions has been deprecated and will be removed in
                 future versions of AG Grid. If you need this to be set for every column use the defaultColDef property.`),e.suppressEnterpriseResetOnNewColumns&&console.warn("AG Grid: since v25, grid property suppressEnterpriseResetOnNewColumns is deprecated. This was a temporary property to allow changing columns in Server Side Row Model without triggering a reload. Now that it is possible to dynamically change columns in the grid, this is no longer needed."),e.suppressColumnStateEvents&&console.warn('AG Grid: since v25, grid property suppressColumnStateEvents no longer works due to a refactor that we did. It should be possible to achieve similar using event.source, which would be "api" if the event was due to setting column state via the API'),e.defaultExportParams&&console.warn("AG Grid: since v25.2, the grid property `defaultExportParams` has been replaced by `defaultCsvExportParams` and `defaultExcelExportParams`."),e.stopEditingWhenGridLosesFocus&&(console.warn("AG Grid: since v25.2.2, the grid property `stopEditingWhenGridLosesFocus` has been replaced by `stopEditingWhenCellsLoseFocus`."),e.stopEditingWhenCellsLoseFocus=!0),e.applyColumnDefOrder&&console.warn("AG Grid: since v26.0, the grid property `applyColumnDefOrder` is no longer needed, as this is the default behaviour. To turn this behaviour off, set maintainColumnOrder=true"),e.groupMultiAutoColumn&&(console.warn("AG Grid: since v26.0, the grid property `groupMultiAutoColumn` has been replaced by `groupDisplayType = 'multipleColumns'`"),e.groupDisplayType="multipleColumns"),e.groupUseEntireRow&&(console.warn("AG Grid: since v26.0, the grid property `groupUseEntireRow` has been replaced by `groupDisplayType = 'groupRows'`"),e.groupDisplayType="groupRows"),e.groupSuppressAutoColumn){var o=e.treeData?"treeDataDisplayType":"groupDisplayType";console.warn("AG Grid: since v26.0, the grid property `groupSuppressAutoColumn` has been replaced by `"+o+" = 'custom'`"),e.groupDisplayType="custom"}e.defaultGroupOrderComparator&&console.warn("AG Grid: since v27.2, the grid property `defaultGroupOrderComparator` is deprecated and has been replaced by `initialGroupOrderComparator` and now receives a single params object."),e.defaultGroupSortComparator&&(console.warn("AG Grid: since v26.0, the grid property `defaultGroupSortComparator` has been replaced by `initialGroupOrderComparator`"),e.defaultGroupOrderComparator=e.defaultGroupSortComparator),e.groupRowAggNodes&&console.warn("AG Grid: since v27.2, the grid property `groupRowAggNodes` is deprecated and has been replaced by `getGroupRowAgg` and now receives a single params object."),e.postSort&&console.warn("AG Grid: since v27.2, the grid property `postSort` is deprecated and has been replaced by `postSortRows` and now receives a single params object."),e.isFullWidthCell&&console.warn("AG Grid: since v27.2, the grid property `isFullWidthCell` is deprecated and has been replaced by `isFullWidthRow` and now receives a single params object."),e.localeTextFunc&&console.warn("AG Grid: since v27.2, the grid property `localeTextFunc` is deprecated and has been replaced by `getLocaleText` and now receives a single params object."),e.colWidth&&console.warn("AG Grid: since v26.1, the grid property `colWidth` is deprecated and should be set via `defaultColDef.width`."),e.minColWidth&&console.warn("AG Grid: since v26.1, the grid property `minColWidth` is deprecated and should be set via `defaultColDef.minWidth`."),e.maxColWidth&&console.warn("AG Grid: since v26.1, the grid property `maxColWidth` is deprecated and should be set via `defaultColDef.maxWidth`."),e.reactUi&&console.warn("AG Grid: since v27.0, React UI is on by default, so no need for reactUi=true. To turn it off, set suppressReactUi=true."),e.suppressReactUi&&console.warn("AG Grid: The legacy React rendering engine is deprecated and will be removed in the next major version of the grid."),e.suppressCellSelection&&(console.warn("AG Grid: since v27.0, `suppressCellSelection` has been replaced by `suppressCellFocus`."),e.suppressCellFocus=e.suppressCellSelection),e.getRowNodeId&&console.warn("AG Grid: since v27.1, `getRowNodeId` is deprecated and has been replaced by `getRowId`. The difference: if getRowId() is implemented then immutable data is enabled by default."),e.immutableData&&(e.getRowId?console.warn("AG Grid: since v27.1, `immutableData` is deprecated. With the `getRowId` callback implemented, immutable data is enabled by default so you can remove `immutableData=true`."):console.warn("AG Grid: since v27.1, `immutableData` is deprecated. To enable immutable data you must implement the `getRowId()` callback.")),e.clipboardDeliminator&&(console.warn("AG Grid: since v27.1, `clipboardDeliminator` has been replaced by `clipboardDelimiter`."),e.clipboardDelimiter=e.clipboardDeliminator)},i.prototype.checkForViolations=function(){this.isTreeData()&&this.treeDataViolations()},i.prototype.treeDataViolations=function(){this.isRowModelDefault()&&me(this.getDataPathFunc())&&console.warn("AG Grid: property usingTreeData=true with rowModel=clientSide, but you did not provide getDataPath function, please provide getDataPath function if using tree data."),this.isRowModelServerSide()&&(me(this.getIsServerSideGroupFunc())&&console.warn("AG Grid: property usingTreeData=true with rowModel=serverSide, but you did not provide isServerSideGroup function, please provide isServerSideGroup function if using tree data."),me(this.getServerSideGroupKeyFunc())&&console.warn("AG Grid: property usingTreeData=true with rowModel=serverSide, but you did not provide getServerSideGroupKey function, please provide getServerSideGroupKey function if using tree data."))},i.prototype.getLocaleTextFunc=function(){var e=this,r=this.gridOptions,n=r.localeText,o=r.getLocaleText,a=r.localeTextFunc;return o?function(s,l,u){var c={key:s,defaultValue:l,variableValues:u,api:e.getApi(),columnApi:e.getColumnApi(),context:e.getContext()};return o(c)}:a||function(s,l,u){var c=n&&n[s];if(c&&u&&u.length)for(var p=0;!(p>=u.length);){var h=c.indexOf("${variable}");if(h===-1)break;c=c.replace("${variable}",u[p++])}return c!=null?c:l}},i.prototype.globalEventHandler=function(e,r){if(!this.destroyed){var n=ea.getCallbackForEvent(e);typeof this.gridOptions[n]=="function"&&this.gridOptions[n](r)}},i.prototype.setRowHeightVariable=function(e){var r=this.eGridDiv.style.getPropertyValue("--ag-line-height").trim(),n=e+"px";r!=n&&this.eGridDiv.style.setProperty("--ag-line-height",n)},i.prototype.getRowHeightAsNumber=function(){if(!this.gridOptions.rowHeight||me(this.gridOptions.rowHeight))return this.getDefaultRowHeight();var e=this.gridOptions.rowHeight;return e&&this.isNumeric(e)?(this.setRowHeightVariable(e),e):(console.warn("AG Grid row height must be a number if not using standard row model"),this.getDefaultRowHeight())},i.prototype.isGetRowHeightFunction=function(){return typeof this.gridOptions.getRowHeight=="function"},i.prototype.getRowHeightForNode=function(e,r,n){if(r===void 0&&(r=!1),n==null&&(n=this.getDefaultRowHeight()),this.isGetRowHeightFunction()){if(r)return{height:n,estimated:!0};var o={node:e,data:e.data},a=this.mergeGridCommonParams(this.gridOptions.getRowHeight)(o);if(this.isNumeric(a))return a===0&&Ye(function(){return console.warn("AG Grid: The return of `getRowHeight` cannot be zero. If the intention is to hide rows, use a filter instead.")},"invalidRowHeight"),{height:Math.max(1,a),estimated:!1}}if(e.detail&&this.isMasterDetail())return this.isDetailRowAutoHeight()?{height:1,estimated:!1}:this.isNumeric(this.gridOptions.detailRowHeight)?{height:this.gridOptions.detailRowHeight,estimated:!1}:{height:DN,estimated:!1};var s=this.gridOptions.rowHeight&&this.isNumeric(this.gridOptions.rowHeight)?this.gridOptions.rowHeight:n;return{height:s,estimated:!1}},i.prototype.isDynamicRowHeight=function(){return typeof this.gridOptions.getRowHeight=="function"},i.prototype.getListItemHeight=function(){return this.getFromTheme(20,"listItemHeight")},i.prototype.chartMenuPanelWidth=function(){return this.environment.chartMenuPanelWidth()},i.prototype.isNumeric=function(e){return!isNaN(e)&&typeof e=="number"&&isFinite(e)},i.prototype.getFromTheme=function(e,r){var n=this.environment.getTheme().theme;return n&&n.indexOf("ag-theme")===0?this.environment.getSassVariable(n,r):e},i.prototype.getDefaultRowHeight=function(){return this.getFromTheme(AN,"rowHeight")},i.prototype.matchesGroupDisplayType=function(e,r){var n=["groupRows","multipleColumns","custom","singleColumn"];return n.indexOf(r)<0?(console.warn("AG Grid: '"+r+"' is not a valid groupDisplayType value - possible values are: '"+n.join("', '")+"'"),!1):r===e},i.prototype.matchesTreeDataDisplayType=function(e,r){var n=["auto","custom"];return n.indexOf(r)<0?(console.warn("AG Grid: '"+r+"' is not a valid treeDataDisplayType value - possible values are: '"+n.join("', '")+"'"),!1):r===e};var t;return i.MIN_COL_WIDTH=10,i.PROP_HEADER_HEIGHT="headerHeight",i.PROP_GROUP_REMOVE_SINGLE_CHILDREN="groupRemoveSingleChildren",i.PROP_GROUP_REMOVE_LOWEST_SINGLE_CHILDREN="groupRemoveLowestSingleChildren",i.PROP_PIVOT_HEADER_HEIGHT="pivotHeaderHeight",i.PROP_SUPPRESS_CLIPBOARD_PASTE="suppressClipboardPaste",i.PROP_GROUP_HEADER_HEIGHT="groupHeaderHeight",i.PROP_PIVOT_GROUP_HEADER_HEIGHT="pivotGroupHeaderHeight",i.PROP_NAVIGATE_TO_NEXT_CELL="navigateToNextCell",i.PROP_TAB_TO_NEXT_CELL="tabToNextCell",i.PROP_NAVIGATE_TO_NEXT_HEADER="navigateToNextHeader",i.PROP_TAB_TO_NEXT_HEADER="tabToNextHeader",i.PROP_IS_EXTERNAL_FILTER_PRESENT="isExternalFilterPresent",i.PROP_DOES_EXTERNAL_FILTER_PASS="doesExternalFilterPass",i.PROP_FLOATING_FILTERS_HEIGHT="floatingFiltersHeight",i.PROP_SUPPRESS_ROW_CLICK_SELECTION="suppressRowClickSelection",i.PROP_SUPPRESS_ROW_DRAG="suppressRowDrag",i.PROP_SUPPRESS_MOVE_WHEN_ROW_DRAG="suppressMoveWhenRowDragging",i.PROP_GET_ROW_CLASS="getRowClass",i.PROP_GET_ROW_STYLE="getRowStyle",i.PROP_GET_ROW_HEIGHT="getRowHeight",i.PROP_POPUP_PARENT="popupParent",i.PROP_DOM_LAYOUT="domLayout",i.PROP_ROW_CLASS="rowClass",i.PROP_FILL_HANDLE_DIRECTION="fillHandleDirection",i.PROP_GROUP_ROW_AGG_NODES="groupRowAggNodes",i.PROP_GET_GROUP_ROW_AGG="getGroupRowAgg",i.PROP_GET_BUSINESS_KEY_FOR_NODE="getBusinessKeyForNode",i.PROP_GET_CHILD_COUNT="getChildCount",i.PROP_PROCESS_ROW_POST_CREATE="processRowPostCreate",i.PROP_GET_ROW_NODE_ID="getRowNodeId",i.PROP_GET_ROW_ID="getRowId",i.PROP_IS_FULL_WIDTH_CELL="isFullWidthCell",i.PROP_IS_FULL_WIDTH_ROW="isFullWidthRow",i.PROP_IS_ROW_SELECTABLE="isRowSelectable",i.PROP_IS_ROW_MASTER="isRowMaster",i.PROP_POST_SORT="postSort",i.PROP_POST_SORT_ROWS="postSortRows",i.PROP_GET_DOCUMENT="getDocument",i.PROP_POST_PROCESS_POPUP="postProcessPopup",i.PROP_DEFAULT_GROUP_ORDER_COMPARATOR="defaultGroupOrderComparator",i.PROP_INITIAL_GROUP_ORDER_COMPARATOR="initialGroupOrderComparator",i.PROP_PAGINATION_NUMBER_FORMATTER="paginationNumberFormatter",i.PROP_GET_CONTEXT_MENU_ITEMS="getContextMenuItems",i.PROP_GET_MAIN_MENU_ITEMS="getMainMenuItems",i.PROP_PROCESS_CELL_FOR_CLIPBOARD="processCellForClipboard",i.PROP_PROCESS_CELL_FROM_CLIPBOARD="processCellFromClipboard",i.PROP_SEND_TO_CLIPBOARD="sendToClipboard",i.PROP_PROCESS_TO_SECONDARY_COLDEF="processSecondaryColDef",i.PROP_PROCESS_SECONDARY_COL_GROUP_DEF="processSecondaryColGroupDef",i.PROP_GET_CHART_TOOLBAR_ITEMS="getChartToolbarItems",i.PROP_GET_SERVER_SIDE_STORE_PARAMS="getServerSideStoreParams",i.PROP_IS_SERVER_SIDE_GROUPS_OPEN_BY_DEFAULT="isServerSideGroupOpenByDefault",i.PROP_IS_APPLY_SERVER_SIDE_TRANSACTION="isApplyServerSideTransaction",i.PROP_IS_SERVER_SIDE_GROUP="isServerSideGroup",i.PROP_GET_SERVER_SIDE_GROUP_KEY="getServerSideGroupKey",So([y("gridOptions")],i.prototype,"gridOptions",void 0),So([y("eventService")],i.prototype,"eventService",void 0),So([y("environment")],i.prototype,"environment",void 0),So([y("eGridDiv")],i.prototype,"eGridDiv",void 0),So([v_(0,Yt("gridApi")),v_(1,Yt("columnApi"))],i.prototype,"agWire",null),So([nt],i.prototype,"destroy",null),So([B],i.prototype,"init",null),i=t=So([Q("gridOptionsWrapper")],i),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var NN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),y_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Rt;(function(i){i.AUTO_HEIGHT="ag-layout-auto-height",i.NORMAL="ag-layout-normal",i.PRINT="ag-layout-print"})(Rt||(Rt={}));var ZC=function(i){NN(t,i);function t(e){var r=i.call(this)||this;return r.view=e,r}return t.prototype.postConstruct=function(){this.addManagedListener(this.gridOptionsWrapper,ce.PROP_DOM_LAYOUT,this.updateLayoutClasses.bind(this)),this.updateLayoutClasses()},t.prototype.updateLayoutClasses=function(){var e=this.gridOptionsWrapper.getDomLayout(),r={autoHeight:e===L.DOM_LAYOUT_AUTO_HEIGHT,normal:e===L.DOM_LAYOUT_NORMAL,print:e===L.DOM_LAYOUT_PRINT},n=r.autoHeight?Rt.AUTO_HEIGHT:r.print?Rt.PRINT:Rt.NORMAL;this.view.updateLayoutClasses(n,r)},y_([y("gridOptionsWrapper")],t.prototype,"gridOptionsWrapper",void 0),y_([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var GN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ci=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},kN=function(i){GN(t,i);function t(e){var r=i.call(this)||this;return r.scrollLeft=-1,r.nextScrollTop=-1,r.scrollTop=-1,r.eBodyViewport=e,r.resetLastHorizontalScrollElementDebounced=uo(r.resetLastHorizontalScrollElement.bind(r),500),r}return t.prototype.postConstruct=function(){var e=this;this.enableRtl=this.gridOptionsWrapper.isEnableRtl(),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,this.onDisplayedColumnsWidthChanged.bind(this)),this.ctrlsService.whenReady(function(r){e.centerRowContainerCtrl=r.centerRowContainerCtrl,e.onDisplayedColumnsWidthChanged(),e.addScrollListener()})},t.prototype.addScrollListener=function(){var e=this.ctrlsService.getFakeHScrollCtrl();this.addManagedListener(this.centerRowContainerCtrl.getViewportElement(),"scroll",this.onCenterViewportScroll.bind(this)),this.addManagedListener(e.getViewport(),"scroll",this.onFakeHorizontalScroll.bind(this));var r=this.gridOptionsWrapper.isDebounceVerticalScrollbar()?uo(this.onVerticalScroll.bind(this),100):this.onVerticalScroll.bind(this);this.addManagedListener(this.eBodyViewport,"scroll",r)},t.prototype.onDisplayedColumnsWidthChanged=function(){this.enableRtl&&this.horizontallyScrollHeaderCenterAndFloatingCenter()},t.prototype.horizontallyScrollHeaderCenterAndFloatingCenter=function(e){var r=this.centerRowContainerCtrl==null;if(!r){e===void 0&&(e=this.centerRowContainerCtrl.getCenterViewportScrollLeft());var n=this.enableRtl?e:-e,o=this.ctrlsService.getTopCenterRowContainerCtrl(),a=this.ctrlsService.getBottomCenterRowContainerCtrl(),s=this.ctrlsService.getFakeHScrollCtrl(),l=this.ctrlsService.getHeaderRowContainerCtrl();l.setHorizontalScroll(n),a.setContainerTranslateX(n),o.setContainerTranslateX(n);var u=this.lastHorizontalScrollElement===this.centerRowContainerCtrl.getViewportElement()?s.getViewport():this.centerRowContainerCtrl.getViewportElement();Yh(u,Math.abs(e),this.enableRtl)}},t.prototype.isControllingScroll=function(e){return this.lastHorizontalScrollElement?e===this.lastHorizontalScrollElement:(this.lastHorizontalScrollElement=e,!0)},t.prototype.onFakeHorizontalScroll=function(){var e=this.ctrlsService.getFakeHScrollCtrl().getViewport();!this.isControllingScroll(e)||this.onBodyHorizontalScroll(e)},t.prototype.onCenterViewportScroll=function(){var e=this.centerRowContainerCtrl.getViewportElement();!this.isControllingScroll(e)||this.onBodyHorizontalScroll(e)},t.prototype.onBodyHorizontalScroll=function(e){var r=this.centerRowContainerCtrl.getViewportElement(),n=r.scrollLeft;this.shouldBlockScrollUpdate("horizontal",n,!0)||(this.doHorizontalScroll(Math.round(Tc(e,this.enableRtl))),this.resetLastHorizontalScrollElementDebounced())},t.prototype.onVerticalScroll=function(){var e=this.eBodyViewport.scrollTop;this.shouldBlockScrollUpdate("vertical",e,!0)||(this.animationFrameService.setScrollTop(e),this.nextScrollTop=e,this.gridOptionsWrapper.isSuppressAnimationFrame()?(this.scrollTop=this.nextScrollTop,this.redrawRowsAfterScroll()):this.animationFrameService.schedule())},t.prototype.resetLastHorizontalScrollElement=function(){this.lastHorizontalScrollElement=null},t.prototype.doHorizontalScroll=function(e){var r=this.ctrlsService.getFakeHScrollCtrl().getViewport(),n=Tc(r,this.enableRtl);this.scrollLeft===e&&e===n||(this.scrollLeft=e,this.fireScrollEvent("horizontal"),this.horizontallyScrollHeaderCenterAndFloatingCenter(e),this.onHorizontalViewportChanged())},t.prototype.fireScrollEvent=function(e){var r=this,n={type:x.EVENT_BODY_SCROLL,api:this.gridApi,columnApi:this.columnApi,direction:e,left:this.scrollLeft,top:this.scrollTop};this.eventService.dispatchEvent(n),window.clearTimeout(this.scrollTimer),this.scrollTimer=void 0,this.scrollTimer=window.setTimeout(function(){var o=Object.assign({},n,{type:x.EVENT_BODY_SCROLL_END});r.eventService.dispatchEvent(o)},100)},t.prototype.shouldBlockScrollUpdate=function(e,r,n){if(n===void 0&&(n=!1),n&&!Bl())return!1;if(e==="vertical"){var o=Uc(this.eBodyViewport),a=this.eBodyViewport.scrollHeight;if(r<0||r+o>a)return!0}if(e==="horizontal"){var s=this.centerRowContainerCtrl.getCenterWidth(),l=this.centerRowContainerCtrl.getViewportElement().scrollWidth;if(this.enableRtl&&xc()){if(r>0)return!0}else if(r<0)return!0;if(Math.abs(r)+s>l)return!0}return!1},t.prototype.redrawRowsAfterScroll=function(){this.fireScrollEvent("vertical")},t.prototype.onHorizontalViewportChanged=function(){this.centerRowContainerCtrl.onHorizontalViewportChanged()},t.prototype.checkScrollLeft=function(){this.scrollLeft!==this.centerRowContainerCtrl.getCenterViewportScrollLeft()&&this.onBodyHorizontalScroll(this.centerRowContainerCtrl.getViewportElement())},t.prototype.executeAnimationFrameScroll=function(){var e=this.scrollTop!=this.nextScrollTop;return e&&(this.scrollTop=this.nextScrollTop,this.redrawRowsAfterScroll()),e},t.prototype.setHorizontalScrollPosition=function(e){var r=0,n=this.centerRowContainerCtrl.getViewportElement().scrollWidth-this.centerRowContainerCtrl.getCenterWidth();this.shouldBlockScrollUpdate("horizontal",e)&&(this.enableRtl&&xc()?e=e>0?0:n:e=Math.min(Math.max(e,r),n)),Yh(this.centerRowContainerCtrl.getViewportElement(),Math.abs(e),this.enableRtl),this.doHorizontalScroll(e)},t.prototype.setVerticalScrollPosition=function(e){this.eBodyViewport.scrollTop=e},t.prototype.getVScrollPosition=function(){var e={top:this.eBodyViewport.scrollTop,bottom:this.eBodyViewport.scrollTop+this.eBodyViewport.offsetHeight};return e},t.prototype.getHScrollPosition=function(){return this.centerRowContainerCtrl.getHScrollPosition()},t.prototype.isHorizontalScrollShowing=function(){return this.centerRowContainerCtrl.isHorizontalScrollShowing()},t.prototype.scrollHorizontally=function(e){var r=this.centerRowContainerCtrl.getViewportElement().scrollLeft;return this.setHorizontalScrollPosition(r+e),this.centerRowContainerCtrl.getViewportElement().scrollLeft-r},t.prototype.scrollToTop=function(){this.eBodyViewport.scrollTop=0},t.prototype.ensureNodeVisible=function(e,r){r===void 0&&(r=null);for(var n=this.rowModel.getRowCount(),o=typeof e=="function",a=-1,s=0;s<n;s++){var l=this.rowModel.getRow(s);if(o){if(e(l)){a=s;break}}else if(e===l||e===l.data){a=s;break}}a>=0&&this.ensureIndexVisible(a,r)},t.prototype.ensureIndexVisible=function(e,r){if(this.gridOptionsWrapper.getDomLayout()!==L.DOM_LAYOUT_PRINT){var n=this.paginationProxy.getRowCount();if(typeof e!="number"||e<0||e>=n){console.warn("invalid row index for ensureIndexVisible: "+e);return}var o=this.gridOptionsWrapper.isPagination(),a=o&&!this.gridOptionsWrapper.isSuppressPaginationPanel();a||this.paginationProxy.goToPageWithIndex(e);var s=this.paginationProxy.getRow(e),l;do{var u=s.rowTop,c=s.rowHeight,p=this.paginationProxy.getPixelOffset(),h=s.rowTop-p,d=h+s.rowHeight,f=this.getVScrollPosition(),g=this.heightScaler.getDivStretchOffset(),v=f.top+g,m=f.bottom+g,C=m-v,w=this.heightScaler.getScrollPositionForPixel(h),b=this.heightScaler.getScrollPositionForPixel(d-C),S=Math.min((w+b)/2,h),T=v>h,E=m<d,P=null;r==="top"?P=w:r==="bottom"?P=b:r==="middle"?P=S:T?P=w:E&&(P=b),P!==null&&(this.eBodyViewport.scrollTop=P,this.rowRenderer.redrawAfterScroll()),l=u!==s.rowTop||c!==s.rowHeight}while(l);this.animationFrameService.flushAllFrames()}},t.prototype.ensureColumnVisible=function(e,r){r===void 0&&(r="auto");var n=this.columnModel.getGridColumn(e);if(!!n&&!n.isPinned()&&!!this.columnModel.isColumnDisplayed(n)){var o=this.getPositionedHorizontalScroll(n,r);o!==null&&this.centerRowContainerCtrl.setCenterViewportScrollLeft(o),this.centerRowContainerCtrl.onHorizontalViewportChanged(),this.animationFrameService.flushAllFrames()}},t.prototype.getPositionedHorizontalScroll=function(e,r){var n=this.isColumnOutsideViewport(e),o=n.columnBeforeStart,a=n.columnAfterEnd,s=this.centerRowContainerCtrl.getCenterWidth()<e.getActualWidth(),l=this.centerRowContainerCtrl.getCenterWidth(),u=this.enableRtl,c=(u?o:a)||s,p=u?a:o;r!=="auto"&&(c=r==="start",p=r==="end");var h=r==="middle";if(c||p||h){var d=this.getColumnBounds(e),f=d.colLeft,g=d.colMiddle,v=d.colRight;return h?g-l/2:c?u?v:f:u?f-l:v-l}return null},t.prototype.isColumnOutsideViewport=function(e){var r=this.getViewportBounds(),n=r.start,o=r.end,a=this.getColumnBounds(e),s=a.colLeft,l=a.colRight,u=this.enableRtl,c=u?n>l:o<l,p=u?o<s:n>s;return{columnBeforeStart:c,columnAfterEnd:p}},t.prototype.getColumnBounds=function(e){var r=this.enableRtl,n=this.columnModel.getBodyContainerWidth(),o=e.getActualWidth(),a=e.getLeft(),s=r?-1:1,l=r?n-a:a,u=l+o*s,c=l+o/2*s;return{colLeft:l,colMiddle:c,colRight:u}},t.prototype.getViewportBounds=function(){var e=this.centerRowContainerCtrl.getCenterWidth(),r=this.centerRowContainerCtrl.getCenterViewportScrollLeft(),n=r,o=e+r;return{start:n,end:o,width:e}},ci([y("ctrlsService")],t.prototype,"ctrlsService",void 0),ci([y("animationFrameService")],t.prototype,"animationFrameService",void 0),ci([y("columnApi")],t.prototype,"columnApi",void 0),ci([y("gridApi")],t.prototype,"gridApi",void 0),ci([y("paginationProxy")],t.prototype,"paginationProxy",void 0),ci([y("rowModel")],t.prototype,"rowModel",void 0),ci([y("rowContainerHeightService")],t.prototype,"heightScaler",void 0),ci([y("rowRenderer")],t.prototype,"rowRenderer",void 0),ci([y("columnModel")],t.prototype,"columnModel",void 0),ci([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Xc=function(){function i(t){this.tickingInterval=null,this.onScrollCallback=null,this.scrollContainer=t.scrollContainer,this.scrollHorizontally=t.scrollAxis.indexOf("x")!==-1,this.scrollVertically=t.scrollAxis.indexOf("y")!==-1,this.scrollByTick=t.scrollByTick!=null?t.scrollByTick:20,t.onScrollCallback&&(this.onScrollCallback=t.onScrollCallback),this.scrollVertically&&(this.getVerticalPosition=t.getVerticalPosition,this.setVerticalPosition=t.setVerticalPosition),this.scrollHorizontally&&(this.getHorizontalPosition=t.getHorizontalPosition,this.setHorizontalPosition=t.setHorizontalPosition),this.shouldSkipVerticalScroll=t.shouldSkipVerticalScroll||function(){return!1},this.shouldSkipHorizontalScroll=t.shouldSkipHorizontalScroll||function(){return!1}}return i.prototype.check=function(t,e){e===void 0&&(e=!1);var r=e||this.shouldSkipVerticalScroll();if(!(r&&this.shouldSkipHorizontalScroll())){var n=this.scrollContainer.getBoundingClientRect(),o=this.scrollByTick;this.tickLeft=t.clientX<n.left+o,this.tickRight=t.clientX>n.right-o,this.tickUp=t.clientY<n.top+o&&!r,this.tickDown=t.clientY>n.bottom-o&&!r,this.tickLeft||this.tickRight||this.tickUp||this.tickDown?this.ensureTickingStarted():this.ensureCleared()}},i.prototype.ensureTickingStarted=function(){this.tickingInterval===null&&(this.tickingInterval=window.setInterval(this.doTick.bind(this),100),this.tickCount=0)},i.prototype.doTick=function(){this.tickCount++;var t;if(t=this.tickCount>20?200:this.tickCount>10?80:40,this.scrollVertically){var e=this.getVerticalPosition();this.tickUp&&this.setVerticalPosition(e-t),this.tickDown&&this.setVerticalPosition(e+t)}if(this.scrollHorizontally){var r=this.getHorizontalPosition();this.tickLeft&&this.setHorizontalPosition(r-t),this.tickRight&&this.setHorizontalPosition(r+t)}this.onScrollCallback&&this.onScrollCallback()},i.prototype.ensureCleared=function(){this.tickingInterval&&(window.clearInterval(this.tickingInterval),this.tickingInterval=null)},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var WN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),iy=globalThis&&globalThis.__assign||function(){return iy=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},iy.apply(this,arguments)},Lr=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},VN=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},BN=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(VN(arguments[t]));return i},HN=function(i){WN(t,i);function t(e){var r=i.call(this)||this;return r.isMultiRowDrag=!1,r.isGridSorted=!1,r.isGridFiltered=!1,r.isRowGroupActive=!1,r.eContainer=e,r}return t.prototype.postConstruct=function(){var e=this;this.gridOptionsWrapper.isRowModelDefault()&&(this.clientSideRowModel=this.rowModel);var r=function(){e.onSortChanged(),e.onFilterChanged(),e.onRowGroupChanged()};this.addManagedListener(this.eventService,x.EVENT_SORT_CHANGED,this.onSortChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_FILTER_CHANGED,this.onFilterChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,this.onRowGroupChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,function(){r()}),r(),this.ctrlsService.whenReady(function(){var n=e.ctrlsService.getGridBodyCtrl();e.autoScrollService=new Xc({scrollContainer:n.getBodyViewportElement(),scrollAxis:"y",getVerticalPosition:function(){return n.getScrollFeature().getVScrollPosition().top},setVerticalPosition:function(o){return n.getScrollFeature().setVerticalScrollPosition(o)},onScrollCallback:function(){e.onDragging(e.lastDraggingEvent)}})})},t.prototype.onSortChanged=function(){this.isGridSorted=this.sortController.isSortActive()},t.prototype.onFilterChanged=function(){this.isGridFiltered=this.filterManager.isAnyFilterPresent()},t.prototype.onRowGroupChanged=function(){var e=this.columnModel.getRowGroupColumns();this.isRowGroupActive=!pn(e)},t.prototype.getContainer=function(){return this.eContainer},t.prototype.isInterestedIn=function(e){return e===At.RowDrag},t.prototype.getIconName=function(){var e=this.gridOptionsWrapper.isRowDragManaged();return e&&this.shouldPreventRowMove()?ht.ICON_NOT_ALLOWED:ht.ICON_MOVE},t.prototype.shouldPreventRowMove=function(){return this.isGridSorted||this.isGridFiltered||this.isRowGroupActive},t.prototype.getRowNodes=function(e){var r=this;if(!this.isFromThisGrid(e))return e.dragItem.rowNodes||[];var n=this.gridOptionsWrapper.isRowDragMultiRow(),o=BN(this.selectionService.getSelectedNodes()).sort(function(s,l){return r.getRowIndexNumber(s)-r.getRowIndexNumber(l)}),a=e.dragItem.rowNode;return n&&o.indexOf(a)!==-1?(this.isMultiRowDrag=!0,o):(this.isMultiRowDrag=!1,[a])},t.prototype.onDragEnter=function(e){e.dragItem.rowNodes=this.getRowNodes(e),this.dispatchGridEvent(x.EVENT_ROW_DRAG_ENTER,e),this.getRowNodes(e).forEach(function(r){r.setDragging(!0)}),this.onEnterOrDragging(e)},t.prototype.onDragging=function(e){this.onEnterOrDragging(e)},t.prototype.isFromThisGrid=function(e){var r=e.dragSource.dragSourceDomDataKey;return r===this.gridOptionsWrapper.getDomDataKey()},t.prototype.isDropZoneWithinThisGrid=function(e){var r=this.ctrlsService.getGridBodyCtrl(),n=r.getGui(),o=e.dropZoneTarget;return!n.contains(o)},t.prototype.onEnterOrDragging=function(e){this.dispatchGridEvent(x.EVENT_ROW_DRAG_MOVE,e),this.lastDraggingEvent=e;var r=this.mouseEventService.getNormalisedPosition(e).y,n=this.gridOptionsWrapper.isRowDragManaged();n&&this.doManagedDrag(e,r),this.autoScrollService.check(e.event)},t.prototype.doManagedDrag=function(e,r){var n=this.isFromThisGrid(e),o=this.gridOptionsWrapper.isRowDragManaged(),a=e.dragItem.rowNodes;o&&this.shouldPreventRowMove()||(this.gridOptionsWrapper.isSuppressMoveWhenRowDragging()||!n?this.isDropZoneWithinThisGrid(e)||this.clientSideRowModel.highlightRowAtPixel(a[0],r):this.moveRows(a,r))},t.prototype.getRowIndexNumber=function(e){return parseInt(rt(e.getRowIndexString().split("-")),10)},t.prototype.moveRowAndClearHighlight=function(e){var r=this,n=this.clientSideRowModel.getLastHighlightedRowNode(),o=n&&n.highlighted===Di.Below,a=this.mouseEventService.getNormalisedPosition(e).y,s=e.dragItem.rowNodes,l=o?1:0;if(this.isFromThisGrid(e))s.forEach(function(p){p.rowTop<a&&(l-=1)}),this.moveRows(s,a,l);else{var u=this.gridOptionsWrapper.getRowIdFunc(),c=this.clientSideRowModel.getRowIndexAtPixel(a)+1;this.clientSideRowModel.getHighlightPosition(a)===Di.Above&&c--,this.clientSideRowModel.updateRowData({add:s.map(function(p){return p.data}).filter(function(p){return!r.clientSideRowModel.getRowNode(u?u({data:p,level:0}):p.id)}),addIndex:c})}this.clearRowHighlight()},t.prototype.clearRowHighlight=function(){this.clientSideRowModel.highlightRowAtPixel(null)},t.prototype.moveRows=function(e,r,n){n===void 0&&(n=0);var o=this.clientSideRowModel.ensureRowsAtPixel(e,r,n);o&&(this.focusService.clearFocusedCell(),this.rangeService&&this.rangeService.removeAllCellRanges())},t.prototype.addRowDropZone=function(e){var r=this;if(!e.getContainer()){Ye(function(){return console.warn("AG Grid: addRowDropZone - A container target needs to be provided")},"add-drop-zone-empty-target");return}if(this.dragAndDropService.findExternalZone(e)){console.warn("AG Grid: addRowDropZone - target already exists in the list of DropZones. Use `removeRowDropZone` before adding it again.");return}var n={getContainer:e.getContainer};e.fromGrid?(e.fromGrid=void 0,n=e):(e.onDragEnter&&(n.onDragEnter=function(o){e.onDragEnter(r.draggingToRowDragEvent(x.EVENT_ROW_DRAG_ENTER,o))}),e.onDragLeave&&(n.onDragLeave=function(o){e.onDragLeave(r.draggingToRowDragEvent(x.EVENT_ROW_DRAG_LEAVE,o))}),e.onDragging&&(n.onDragging=function(o){e.onDragging(r.draggingToRowDragEvent(x.EVENT_ROW_DRAG_MOVE,o))}),e.onDragStop&&(n.onDragStop=function(o){e.onDragStop(r.draggingToRowDragEvent(x.EVENT_ROW_DRAG_END,o))})),this.dragAndDropService.addDropTarget(iy({isInterestedIn:function(o){return o===At.RowDrag},getIconName:function(){return ht.ICON_MOVE},external:!0},n))},t.prototype.getRowDropZone=function(e){var r=this,n=this.getContainer.bind(this),o=this.onDragEnter.bind(this),a=this.onDragLeave.bind(this),s=this.onDragging.bind(this),l=this.onDragStop.bind(this);return e?{getContainer:n,onDragEnter:e.onDragEnter?function(u){o(u),e.onDragEnter(r.draggingToRowDragEvent(x.EVENT_ROW_DRAG_ENTER,u))}:o,onDragLeave:e.onDragLeave?function(u){a(u),e.onDragLeave(r.draggingToRowDragEvent(x.EVENT_ROW_DRAG_LEAVE,u))}:a,onDragging:e.onDragging?function(u){s(u),e.onDragging(r.draggingToRowDragEvent(x.EVENT_ROW_DRAG_MOVE,u))}:s,onDragStop:e.onDragStop?function(u){l(u),e.onDragStop(r.draggingToRowDragEvent(x.EVENT_ROW_DRAG_END,u))}:l,fromGrid:!0}:{getContainer:n,onDragEnter:o,onDragLeave:a,onDragging:s,onDragStop:l,fromGrid:!0}},t.prototype.draggingToRowDragEvent=function(e,r){var n=this.mouseEventService.getNormalisedPosition(r).y,o=n>this.paginationProxy.getCurrentPageHeight(),a=-1,s;o||(a=this.rowModel.getRowIndexAtPixel(n),s=this.rowModel.getRow(a));var l;switch(r.vDirection){case la.Down:l="down";break;case la.Up:l="up";break;default:l=null;break}var u={type:e,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),event:r.event,node:r.dragItem.rowNode,nodes:r.dragItem.rowNodes,overIndex:a,overNode:s,y:n,vDirection:l};return u},t.prototype.dispatchGridEvent=function(e,r){var n=this.draggingToRowDragEvent(e,r);this.eventService.dispatchEvent(n)},t.prototype.onDragLeave=function(e){this.dispatchGridEvent(x.EVENT_ROW_DRAG_LEAVE,e),this.stopDragging(e),this.gridOptionsWrapper.isRowDragManaged()&&this.clearRowHighlight(),this.isFromThisGrid(e)&&(this.isMultiRowDrag=!1)},t.prototype.onDragStop=function(e){this.dispatchGridEvent(x.EVENT_ROW_DRAG_END,e),this.stopDragging(e),this.gridOptionsWrapper.isRowDragManaged()&&(this.gridOptionsWrapper.isSuppressMoveWhenRowDragging()||!this.isFromThisGrid(e))&&!this.isDropZoneWithinThisGrid(e)&&this.moveRowAndClearHighlight(e)},t.prototype.stopDragging=function(e){this.autoScrollService.ensureCleared(),this.getRowNodes(e).forEach(function(r){r.setDragging(!1)})},Lr([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),Lr([y("rowModel")],t.prototype,"rowModel",void 0),Lr([y("paginationProxy")],t.prototype,"paginationProxy",void 0),Lr([y("columnModel")],t.prototype,"columnModel",void 0),Lr([y("focusService")],t.prototype,"focusService",void 0),Lr([y("sortController")],t.prototype,"sortController",void 0),Lr([y("filterManager")],t.prototype,"filterManager",void 0),Lr([y("selectionService")],t.prototype,"selectionService",void 0),Lr([y("mouseEventService")],t.prototype,"mouseEventService",void 0),Lr([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Lr([De("rangeService")],t.prototype,"rangeService",void 0),Lr([y("columnApi")],t.prototype,"columnApi",void 0),Lr([y("gridApi")],t.prototype,"gridApi",void 0),Lr([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var jN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),yn=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},cs;(function(i){i.ANIMATION_ON="ag-row-animation",i.ANIMATION_OFF="ag-row-no-animation"})(cs||(cs={}));var PE="ag-selectable",AE="ag-force-vertical-scroll",DE="ag-column-moving",ME=function(i){jN(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getScrollFeature=function(){return this.bodyScrollFeature},t.prototype.getBodyViewportElement=function(){return this.eBodyViewport},t.prototype.setComp=function(e,r,n,o,a){this.comp=e,this.eGridBody=r,this.eBodyViewport=n,this.eTop=o,this.eBottom=a,this.setCellTextSelection(this.gridOptionsWrapper.isEnableCellTextSelect()),this.createManagedBean(new ZC(this.comp)),this.bodyScrollFeature=this.createManagedBean(new kN(this.eBodyViewport)),this.addRowDragListener(),this.setupRowAnimationCssClass(),this.addEventListeners(),this.addFocusListeners([o,n,a]),this.onGridColumnsChanged(),this.addBodyViewportListener(),this.setFloatingHeights(),this.disableBrowserDragging(),this.addStopEditingWhenGridLosesFocus(),this.ctrlsService.registerGridBodyCtrl(this)},t.prototype.getComp=function(){return this.comp},t.prototype.addEventListeners=function(){this.addManagedListener(this.eventService,x.EVENT_GRID_COLUMNS_CHANGED,this.onGridColumnsChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_SCROLL_VISIBILITY_CHANGED,this.onScrollVisibilityChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_PINNED_ROW_DATA_CHANGED,this.setFloatingHeights.bind(this))},t.prototype.addFocusListeners=function(e){var r=this;e.forEach(function(n){r.addManagedListener(n,"focusin",function(){n.classList.add("ag-has-focus")}),r.addManagedListener(n,"focusout",function(o){n.contains(o.relatedTarget)||n.classList.remove("ag-has-focus")})})},t.prototype.setColumnMovingCss=function(e){this.comp.setColumnMovingCss(e?DE:null,e)},t.prototype.setCellTextSelection=function(e){e===void 0&&(e=!1);var r=e?PE:null;this.comp.setCellSelectableCss(r,e)},t.prototype.onScrollVisibilityChanged=function(){var e=this.scrollVisibleService.isVerticalScrollShowing();this.setVerticalScrollPaddingVisible(e)},t.prototype.onGridColumnsChanged=function(){var e=this.columnModel.getAllGridColumns();this.comp.setColumnCount(e?e.length:0)},t.prototype.disableBrowserDragging=function(){this.addManagedListener(this.eGridBody,"dragstart",function(e){if(e.target instanceof HTMLImageElement)return e.preventDefault(),!1})},t.prototype.addStopEditingWhenGridLosesFocus=function(){var e=this;if(!!this.gridOptionsWrapper.isStopEditingWhenCellsLoseFocus()){var r=function(o){var a=o.relatedTarget;if(Kh(a)===null){e.rowRenderer.stopEditing();return}var s=n.some(function(u){return u.contains(a)})&&e.mouseEventService.isElementInThisGrid(a);if(!s){var l=e.popupService;s=l.getActivePopups().some(function(u){return u.contains(a)})||l.isElementWithinCustomPopup(a)}s||e.rowRenderer.stopEditing()},n=[this.eBodyViewport,this.eBottom,this.eTop];n.forEach(function(o){return e.addManagedListener(o,"focusout",r)})}},t.prototype.updateRowCount=function(){var e=this.headerNavigationService.getHeaderRowCount(),r=this.paginationProxy.getType(),n=-1;r===L.ROW_MODEL_TYPE_CLIENT_SIDE&&(n=0,this.paginationProxy.forEachNode(function(a){a.group||n++}));var o=n===-1?-1:e+n;this.comp.setRowCount(o)},t.prototype.registerBodyViewportResizeListener=function(e){this.comp.registerBodyViewportResizeListener(e)},t.prototype.setVerticalScrollPaddingVisible=function(e){var r=e?"scroll":"hidden";this.comp.setPinnedTopBottomOverflowY(r)},t.prototype.isVerticalScrollShowing=function(){var e=this.gridOptionsWrapper.isAlwaysShowVerticalScroll(),r=e?AE:null;return this.comp.setAlwaysVerticalScrollClass(r,e),e||IO(this.eBodyViewport)},t.prototype.setupRowAnimationCssClass=function(){var e=this,r=function(){var n=e.gridOptionsWrapper.isAnimateRows()&&!e.rowContainerHeightService.isStretching(),o=n?cs.ANIMATION_ON:cs.ANIMATION_OFF;e.comp.setRowAnimationCssOnBodyViewport(o,n)};r(),this.addManagedListener(this.eventService,x.EVENT_HEIGHT_SCALE_CHANGED,r)},t.prototype.getGridBodyElement=function(){return this.eGridBody},t.prototype.addBodyViewportListener=function(){var e=this,r=function(n){e.gridOptionsWrapper.isPreventDefaultOnContextMenu()&&n.preventDefault();var o=n.target;(o===e.eBodyViewport||o===e.ctrlsService.getCenterRowContainerCtrl().getViewportElement())&&e.contextMenuFactory&&e.contextMenuFactory.onContextMenu(n,null,null,null,null,e.eGridBody)};this.addManagedListener(this.eBodyViewport,"contextmenu",r),this.addManagedListener(this.eBodyViewport,"wheel",this.onWheel.bind(this))},t.prototype.onWheel=function(e){!this.gridOptionsWrapper.isSuppressScrollWhenPopupsAreOpen()||this.popupService.hasAnchoredPopup()&&e.preventDefault()},t.prototype.getGui=function(){return this.eGridBody},t.prototype.scrollVertically=function(e){var r=this.eBodyViewport.scrollTop;return this.bodyScrollFeature.setVerticalScrollPosition(r+e),this.eBodyViewport.scrollTop-r},t.prototype.addRowDragListener=function(){this.rowDragFeature=this.createManagedBean(new HN(this.eBodyViewport)),this.dragAndDropService.addDropTarget(this.rowDragFeature)},t.prototype.getRowDragFeature=function(){return this.rowDragFeature},t.prototype.setFloatingHeights=function(){var e=this.pinnedRowModel,r=e.getPinnedTopTotalHeight();r&&(r+=1);var n=e.getPinnedBottomTotalHeight();n&&(n+=1),this.comp.setTopHeight(r),this.comp.setBottomHeight(n),this.comp.setTopDisplay(r?"inherit":"none"),this.comp.setBottomDisplay(n?"inherit":"none")},t.prototype.sizeColumnsToFit=function(e){var r=this,n=this.isVerticalScrollShowing(),o=n?this.gridOptionsWrapper.getScrollbarWidth():0,a=tf(this.eBodyViewport),s=a-o;if(s>0){this.columnModel.sizeColumnsToFit(s,"sizeColumnsToFit");return}e===void 0?window.setTimeout(function(){r.sizeColumnsToFit(100)},0):e===100?window.setTimeout(function(){r.sizeColumnsToFit(500)},100):e===500?window.setTimeout(function(){r.sizeColumnsToFit(-1)},500):console.warn("AG Grid: tried to call sizeColumnsToFit() but the grid is coming back with zero width, maybe the grid is not visible yet on the screen?")},t.prototype.addScrollEventListener=function(e){this.eBodyViewport.addEventListener("scroll",e)},t.prototype.removeScrollEventListener=function(e){this.eBodyViewport.removeEventListener("scroll",e)},yn([y("rowContainerHeightService")],t.prototype,"rowContainerHeightService",void 0),yn([y("ctrlsService")],t.prototype,"ctrlsService",void 0),yn([y("columnModel")],t.prototype,"columnModel",void 0),yn([y("scrollVisibleService")],t.prototype,"scrollVisibleService",void 0),yn([De("contextMenuFactory")],t.prototype,"contextMenuFactory",void 0),yn([y("headerNavigationService")],t.prototype,"headerNavigationService",void 0),yn([y("paginationProxy")],t.prototype,"paginationProxy",void 0),yn([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),yn([y("pinnedRowModel")],t.prototype,"pinnedRowModel",void 0),yn([y("rowRenderer")],t.prototype,"rowRenderer",void 0),yn([y("popupService")],t.prototype,"popupService",void 0),yn([y("mouseEventService")],t.prototype,"mouseEventService",void 0),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var co;(function(i){i[i.FILL=0]="FILL",i[i.RANGE=1]="RANGE"})(co||(co={}));var zr;(function(i){i[i.VALUE=0]="VALUE",i[i.DIMENSION=1]="DIMENSION"})(zr||(zr={}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var uu="ag-cell-range-selected",UN="ag-cell-range-chart",zN="ag-cell-range-single-cell",$N="ag-cell-range-chart-category",KN="ag-cell-range-handle",YN="ag-cell-range-top",XN="ag-cell-range-right",qN="ag-cell-range-bottom",ZN="ag-cell-range-left",QN=function(){function i(t,e){this.beans=t,this.cellCtrl=e}return i.prototype.setComp=function(t,e){this.cellComp=t,this.eGui=e,this.onRangeSelectionChanged()},i.prototype.onRangeSelectionChanged=function(){!this.cellComp||(this.rangeCount=this.beans.rangeService.getCellRangeCount(this.cellCtrl.getCellPosition()),this.hasChartRange=this.getHasChartRange(),this.cellComp.addOrRemoveCssClass(uu,this.rangeCount!==0),this.cellComp.addOrRemoveCssClass(uu+"-1",this.rangeCount===1),this.cellComp.addOrRemoveCssClass(uu+"-2",this.rangeCount===2),this.cellComp.addOrRemoveCssClass(uu+"-3",this.rangeCount===3),this.cellComp.addOrRemoveCssClass(uu+"-4",this.rangeCount>=4),this.cellComp.addOrRemoveCssClass(UN,this.hasChartRange),Dl(this.eGui,this.rangeCount>0?!0:void 0),this.cellComp.addOrRemoveCssClass(zN,this.isSingleCell()),this.updateRangeBorders(),this.refreshHandle())},i.prototype.updateRangeBorders=function(){var t=this.getRangeBorders(),e=this.isSingleCell(),r=!e&&t.top,n=!e&&t.right,o=!e&&t.bottom,a=!e&&t.left;this.cellComp.addOrRemoveCssClass(YN,r),this.cellComp.addOrRemoveCssClass(XN,n),this.cellComp.addOrRemoveCssClass(qN,o),this.cellComp.addOrRemoveCssClass(ZN,a)},i.prototype.isSingleCell=function(){var t=this.beans.rangeService;return this.rangeCount===1&&t&&!t.isMoreThanOneCell()},i.prototype.getHasChartRange=function(){var t=this.beans.rangeService;if(!this.rangeCount||!t)return!1;var e=t.getCellRanges();return e.length>0&&e.every(function(r){return fn([zr.DIMENSION,zr.VALUE],r.type)})},i.prototype.updateRangeBordersIfRangeCount=function(){this.rangeCount>0&&(this.updateRangeBorders(),this.refreshHandle())},i.prototype.getRangeBorders=function(){var t=this,e=this.beans.gridOptionsWrapper.isEnableRtl(),r=!1,n=!1,o=!1,a=!1,s=this.cellCtrl.getCellPosition().column,l=this.beans,u=l.rangeService,c=l.columnModel,p,h;e?(p=c.getDisplayedColAfter(s),h=c.getDisplayedColBefore(s)):(p=c.getDisplayedColBefore(s),h=c.getDisplayedColAfter(s));var d=u.getCellRanges().filter(function(C){return u.isCellInSpecificRange(t.cellCtrl.getCellPosition(),C)});p||(a=!0),h||(n=!0);for(var f=0;f<d.length&&!(r&&n&&o&&a);f++){var g=d[f],v=u.getRangeStartRow(g),m=u.getRangeEndRow(g);!r&&this.beans.rowPositionUtils.sameRow(v,this.cellCtrl.getCellPosition())&&(r=!0),!o&&this.beans.rowPositionUtils.sameRow(m,this.cellCtrl.getCellPosition())&&(o=!0),!a&&p&&g.columns.indexOf(p)<0&&(a=!0),!n&&h&&g.columns.indexOf(h)<0&&(n=!0)}return{top:r,right:n,bottom:o,left:a}},i.prototype.refreshHandle=function(){if(!!this.beans.rangeService){var t=this.shouldHaveSelectionHandle();this.selectionHandle&&!t&&(this.selectionHandle=this.beans.context.destroyBean(this.selectionHandle)),t&&this.addSelectionHandle(),this.cellComp.addOrRemoveCssClass(KN,!!this.selectionHandle)}},i.prototype.shouldHaveSelectionHandle=function(){var t=this.beans,e=t.gridOptionsWrapper,r=t.rangeService,n=r.getCellRanges(),o=n.length;if(this.rangeCount<1||o<1)return!1;var a=rt(n),s=this.cellCtrl.getCellPosition(),l=e.isEnableFillHandle()&&!this.cellCtrl.isSuppressFillHandle(),u=e.isEnableRangeHandle(),c=o===1&&!this.cellCtrl.isEditing()&&(l||u);if(this.hasChartRange){var p=n[0].type===zr.DIMENSION,h=p&&r.isCellInSpecificRange(s,n[0]);this.cellComp.addOrRemoveCssClass($N,h),c=a.type===zr.VALUE}return c&&a.endRow!=null&&r.isContiguousRange(a)&&r.isBottomRightCell(a,s)},i.prototype.addSelectionHandle=function(){var t=this.beans,e=t.gridOptionsWrapper,r=t.rangeService,n=rt(r.getCellRanges()).type,o=e.isEnableFillHandle()&&me(n),a=o?co.FILL:co.RANGE;this.selectionHandle&&this.selectionHandle.getType()!==a&&(this.selectionHandle=this.beans.context.destroyBean(this.selectionHandle)),this.selectionHandle||(this.selectionHandle=this.beans.selectionHandleFactory.createSelectionHandle(a)),this.selectionHandle.refresh(this.cellCtrl)},i.prototype.destroy=function(){this.beans.context.destroyBean(this.selectionHandle)},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var JN=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),eG=function(i){JN(t,i);function t(e,r){var n=i.call(this)||this;return n.cellCtrl=e,n.beans=r,n.column=e.getColumn(),n.rowNode=e.getRowNode(),n.setupColSpan(),n.setupRowSpan(),n}return t.prototype.setupRowSpan=function(){this.rowSpan=this.column.getRowSpan(this.rowNode)},t.prototype.setComp=function(e){this.eGui=e,this.onLeftChanged(),this.onWidthChanged(),this.applyRowSpan()},t.prototype.onDisplayColumnsChanged=function(){var e=this.getColSpanningList();es(this.colsSpanning,e)||(this.colsSpanning=e,this.onWidthChanged(),this.onLeftChanged())},t.prototype.setupColSpan=function(){this.column.getColDef().colSpan!=null&&(this.colsSpanning=this.getColSpanningList(),this.addManagedListener(this.beans.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,this.onDisplayColumnsChanged.bind(this)),this.addManagedListener(this.beans.eventService,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,this.onWidthChanged.bind(this)))},t.prototype.onWidthChanged=function(){if(!!this.eGui){var e=this.getCellWidth();this.eGui.style.width=e+"px"}},t.prototype.getCellWidth=function(){return this.colsSpanning?this.colsSpanning.reduce(function(e,r){return e+r.getActualWidth()},0):this.column.getActualWidth()},t.prototype.getColSpanningList=function(){var e=this.column.getColSpan(this.rowNode),r=[];if(e===1)r.push(this.column);else for(var n=this.column,o=this.column.getPinned(),a=0;n&&a<e&&(r.push(n),n=this.beans.columnModel.getDisplayedColAfter(n),!(!n||me(n)||o!==n.getPinned()));a++);return r},t.prototype.onLeftChanged=function(){if(!!this.eGui){var e=this.modifyLeftForPrintLayout(this.getCellLeft());this.eGui.style.left=e+"px"}},t.prototype.getCellLeft=function(){var e;return this.beans.gridOptionsWrapper.isEnableRtl()&&this.colsSpanning?e=rt(this.colsSpanning):e=this.column,e.getLeft()},t.prototype.modifyLeftForPrintLayout=function(e){if(!this.cellCtrl.isPrintLayout()||this.column.getPinned()===L.PINNED_LEFT)return e;var r=this.beans.columnModel.getDisplayedColumnsLeftWidth();if(this.column.getPinned()===L.PINNED_RIGHT){var n=this.beans.columnModel.getBodyContainerWidth();return r+n+(e||0)}return r+(e||0)},t.prototype.applyRowSpan=function(){if(this.rowSpan!==1){var e=this.beans.gridOptionsWrapper.getRowHeightAsNumber(),r=e*this.rowSpan;this.eGui.style.height=r+"px",this.eGui.style.zIndex="1"}},t.prototype.destroy=function(){i.prototype.destroy.call(this)},t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var tG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),rG=function(i){tG(t,i);function t(e,r){var n=i.call(this)||this;return n.staticClasses=[],n.cellCtrl=e,n.beans=r,n.column=e.getColumn(),n.rowNode=e.getRowNode(),n}return t.prototype.setComp=function(e){this.cellComp=e,this.applyUserStyles(),this.applyCellClassRules(),this.applyClassesFromColDef()},t.prototype.applyCellClassRules=function(){var e=this,r=this.column.getColDef(),n={value:this.cellCtrl.getValue(),data:this.rowNode.data,node:this.rowNode,colDef:r,rowIndex:this.rowNode.rowIndex,api:this.beans.gridOptionsWrapper.getApi(),columnApi:this.beans.gridOptionsWrapper.getColumnApi(),context:this.beans.gridOptionsWrapper.getContext()};this.beans.stylingService.processClassRules(r.cellClassRules,n,function(o){return e.cellComp.addOrRemoveCssClass(o,!0)},function(o){return e.cellComp.addOrRemoveCssClass(o,!1)})},t.prototype.applyUserStyles=function(){var e=this.column.getColDef();if(!!e.cellStyle){var r;if(typeof e.cellStyle=="function"){var n={column:this.column,value:this.cellCtrl.getValue(),colDef:e,data:this.rowNode.data,node:this.rowNode,rowIndex:this.rowNode.rowIndex,api:this.beans.gridOptionsWrapper.getApi(),columnApi:this.beans.gridOptionsWrapper.getColumnApi(),context:this.beans.gridOptionsWrapper.getContext()},o=e.cellStyle;r=o(n)}else r=e.cellStyle;this.cellComp.setUserStyles(r)}},t.prototype.applyClassesFromColDef=function(){var e=this,r=this.column.getColDef(),n={value:this.cellCtrl.getValue(),data:this.rowNode.data,node:this.rowNode,colDef:r,rowIndex:this.rowNode.rowIndex,api:this.beans.gridOptionsWrapper.getApi(),columnApi:this.beans.gridOptionsWrapper.getColumnApi(),context:this.beans.gridOptionsWrapper.getContext()};this.staticClasses.length&&this.staticClasses.forEach(function(o){return e.cellComp.addOrRemoveCssClass(o,!1)}),this.staticClasses=this.beans.stylingService.getStaticCellClasses(r,n),this.staticClasses.length&&this.staticClasses.forEach(function(o){return e.cellComp.addOrRemoveCssClass(o,!0)})},t.prototype.destroy=function(){i.prototype.destroy.call(this)},t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var nG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),QC=function(i){nG(t,i);function t(e,r){var n=i.call(this)||this;return n.ctrl=e,n.beans=r,n}return t.prototype.setComp=function(e){this.comp=e,this.setupTooltip()},t.prototype.setupTooltip=function(){this.browserTooltips=this.beans.gridOptionsWrapper.isEnableBrowserTooltips(),this.updateTooltipText(),this.browserTooltips?this.comp.setTitle(this.tooltip!=null?this.tooltip:void 0):this.createTooltipFeatureIfNeeded()},t.prototype.updateTooltipText=function(){this.tooltip=this.ctrl.getTooltipValue()},t.prototype.createTooltipFeatureIfNeeded=function(){var e=this;if(this.genericTooltipFeature==null){var r={getTooltipParams:function(){return e.getTooltipParams()},getGui:function(){return e.ctrl.getGui()}};this.genericTooltipFeature=this.createManagedBean(new nf(r),this.beans.context)}},t.prototype.refreshToolTip=function(){this.updateTooltipText(),this.browserTooltips&&this.comp.setTitle(this.tooltip!=null?this.tooltip:void 0)},t.prototype.getTooltipParams=function(){var e=this.ctrl,r=e.getColumn?e.getColumn():void 0,n=e.getColDef?e.getColDef():void 0,o=e.getRowNode?e.getRowNode():void 0;return{location:e.getLocation(),colDef:n,column:r,rowIndex:e.getRowIndex?e.getRowIndex():void 0,node:o,data:o?o.data:void 0,value:this.getTooltipText(),valueFormatted:e.getValueFormatted?e.getValueFormatted():void 0}},t.prototype.getTooltipText=function(){return this.tooltip},t.prototype.destroy=function(){i.prototype.destroy.call(this)},t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Fe=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},JC=function(){function i(){}return i.prototype.postConstruct=function(){this.doingMasterDetail=this.gridOptionsWrapper.isMasterDetail(),this.gridOptionsWrapper.isRowModelDefault()&&(this.clientSideRowModel=this.rowModel),this.gridOptionsWrapper.isRowModelServerSide()&&(this.serverSideRowModel=this.rowModel)},Fe([y("resizeObserverService")],i.prototype,"resizeObserverService",void 0),Fe([y("paginationProxy")],i.prototype,"paginationProxy",void 0),Fe([y("context")],i.prototype,"context",void 0),Fe([y("columnApi")],i.prototype,"columnApi",void 0),Fe([y("gridApi")],i.prototype,"gridApi",void 0),Fe([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),Fe([y("expressionService")],i.prototype,"expressionService",void 0),Fe([y("rowRenderer")],i.prototype,"rowRenderer",void 0),Fe([y("templateService")],i.prototype,"templateService",void 0),Fe([y("valueService")],i.prototype,"valueService",void 0),Fe([y("eventService")],i.prototype,"eventService",void 0),Fe([y("columnModel")],i.prototype,"columnModel",void 0),Fe([y("headerNavigationService")],i.prototype,"headerNavigationService",void 0),Fe([y("navigationService")],i.prototype,"navigationService",void 0),Fe([y("columnAnimationService")],i.prototype,"columnAnimationService",void 0),Fe([De("rangeService")],i.prototype,"rangeService",void 0),Fe([y("focusService")],i.prototype,"focusService",void 0),Fe([De("contextMenuFactory")],i.prototype,"contextMenuFactory",void 0),Fe([y("popupService")],i.prototype,"popupService",void 0),Fe([y("valueFormatterService")],i.prototype,"valueFormatterService",void 0),Fe([y("stylingService")],i.prototype,"stylingService",void 0),Fe([y("columnHoverService")],i.prototype,"columnHoverService",void 0),Fe([y("userComponentFactory")],i.prototype,"userComponentFactory",void 0),Fe([y("userComponentRegistry")],i.prototype,"userComponentRegistry",void 0),Fe([y("animationFrameService")],i.prototype,"animationFrameService",void 0),Fe([y("dragAndDropService")],i.prototype,"dragAndDropService",void 0),Fe([y("sortController")],i.prototype,"sortController",void 0),Fe([y("filterManager")],i.prototype,"filterManager",void 0),Fe([y("rowContainerHeightService")],i.prototype,"rowContainerHeightService",void 0),Fe([y("frameworkOverrides")],i.prototype,"frameworkOverrides",void 0),Fe([y("cellPositionUtils")],i.prototype,"cellPositionUtils",void 0),Fe([y("rowPositionUtils")],i.prototype,"rowPositionUtils",void 0),Fe([y("selectionService")],i.prototype,"selectionService",void 0),Fe([De("selectionHandleFactory")],i.prototype,"selectionHandleFactory",void 0),Fe([y("rowCssClassCalculator")],i.prototype,"rowCssClassCalculator",void 0),Fe([y("rowModel")],i.prototype,"rowModel",void 0),Fe([y("ctrlsService")],i.prototype,"ctrlsService",void 0),Fe([y("ctrlsFactory")],i.prototype,"ctrlsFactory",void 0),Fe([y("agStackComponentsRegistry")],i.prototype,"agStackComponentsRegistry",void 0),Fe([y("valueCache")],i.prototype,"valueCache",void 0),Fe([y("rowNodeEventThrottle")],i.prototype,"rowNodeEventThrottle",void 0),Fe([B],i.prototype,"postConstruct",null),i=Fe([Q("beans")],i),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var iG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),oG=function(i){iG(t,i);function t(e,r,n){var o=i.call(this)||this;return o.cellCtrl=e,o.beans=r,o.column=n,o}return t.prototype.onMouseEvent=function(e,r){if(!Ai(r))switch(e){case"click":this.onCellClicked(r);break;case"mousedown":case"touchstart":this.onMouseDown(r);break;case"dblclick":this.onCellDoubleClicked(r);break;case"mouseout":this.onMouseOut(r);break;case"mouseover":this.onMouseOver(r);break}},t.prototype.onCellClicked=function(e){if(this.isDoubleClickOnIPad()){this.onCellDoubleClicked(e),e.preventDefault();return}var r=this.beans,n=r.eventService,o=r.gridOptionsWrapper,a=this.cellCtrl.createEvent(e,x.EVENT_CELL_CLICKED);n.dispatchEvent(a);var s=this.column.getColDef();s.onCellClicked&&window.setTimeout(function(){return s.onCellClicked(a)},0);var l=(o.isSingleClickEdit()||s.singleClickEdit)&&!o.isSuppressClickEdit();l&&this.cellCtrl.startRowOrCellEdit()},t.prototype.isDoubleClickOnIPad=function(){if(!Bl()||xC("dblclick"))return!1;var e=new Date().getTime(),r=e-this.lastIPadMouseClickEvent<200;return this.lastIPadMouseClickEvent=e,r},t.prototype.onCellDoubleClicked=function(e){var r=this.column.getColDef(),n=this.cellCtrl.createEvent(e,x.EVENT_CELL_DOUBLE_CLICKED);this.beans.eventService.dispatchEvent(n),typeof r.onCellDoubleClicked=="function"&&window.setTimeout(function(){return r.onCellDoubleClicked(n)},0);var o=!this.beans.gridOptionsWrapper.isSingleClickEdit()&&!this.beans.gridOptionsWrapper.isSuppressClickEdit();o&&this.cellCtrl.startRowOrCellEdit(null,null,e)},t.prototype.onMouseDown=function(e){var r=e.ctrlKey,n=e.metaKey,o=e.shiftKey,a=e.target,s=this.beans,l=s.eventService,u=s.rangeService;if(!this.isRightClickInExistingRange(e)){var c=u&&u.getCellRanges().length!=0;if(!o||!c){var p=Jd()&&!this.cellCtrl.isEditing()&&!DO(a);this.cellCtrl.focusCell(p)}if(o&&c&&e.preventDefault(),!this.containsWidget(a)){if(u){var h=this.cellCtrl.getCellPosition();if(o)u.extendLatestRangeToCell(h);else{var d=r||n;u.setRangeToCell(h,d)}}l.dispatchEvent(this.cellCtrl.createEvent(e,x.EVENT_CELL_MOUSE_DOWN))}}},t.prototype.isRightClickInExistingRange=function(e){var r=this.beans.rangeService;if(r){var n=r.isCellInAnyRange(this.cellCtrl.getCellPosition());if(n&&e.button===2)return!0}return!1},t.prototype.containsWidget=function(e){return ef(e,"ag-selection-checkbox",3)},t.prototype.onMouseOut=function(e){if(!this.mouseStayingInsideCell(e)){var r=this.cellCtrl.createEvent(e,x.EVENT_CELL_MOUSE_OUT);this.beans.eventService.dispatchEvent(r),this.beans.columnHoverService.clearMouseOver()}},t.prototype.onMouseOver=function(e){if(!this.mouseStayingInsideCell(e)){var r=this.cellCtrl.createEvent(e,x.EVENT_CELL_MOUSE_OVER);this.beans.eventService.dispatchEvent(r),this.beans.columnHoverService.setMouseOver([this.column])}},t.prototype.mouseStayingInsideCell=function(e){if(!e.target||!e.relatedTarget)return!1;var r=this.cellCtrl.getGui(),n=r.contains(e.target),o=r.contains(e.relatedTarget);return n&&o},t.prototype.destroy=function(){},t}(JC);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var aG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),sG=function(i){aG(t,i);function t(e,r,n,o,a){var s=i.call(this)||this;return s.cellCtrl=e,s.beans=r,s.rowNode=o,s.rowCtrl=a,s}return t.prototype.setComp=function(e){this.eGui=e},t.prototype.onKeyDown=function(e){var r=e.key;switch(r){case j.ENTER:this.onEnterKeyDown(e);break;case j.F2:this.onF2KeyDown(e);break;case j.ESCAPE:this.onEscapeKeyDown(e);break;case j.TAB:this.onTabKeyDown(e);break;case j.BACKSPACE:case j.DELETE:this.onBackspaceOrDeleteKeyPressed(r,e);break;case j.DOWN:case j.UP:case j.RIGHT:case j.LEFT:this.onNavigationKeyPressed(e,r);break}},t.prototype.onNavigationKeyPressed=function(e,r){this.cellCtrl.isEditing()||(e.shiftKey&&this.cellCtrl.isRangeSelectionEnabled()?this.onShiftRangeSelect(e):this.beans.navigationService.navigateToNextCell(e,r,this.cellCtrl.getCellPosition(),!0),e.preventDefault())},t.prototype.onShiftRangeSelect=function(e){if(!!this.beans.rangeService){var r=this.beans.rangeService.extendLatestRangeInDirection(e);r&&this.beans.navigationService.ensureCellVisible(r)}},t.prototype.onTabKeyDown=function(e){this.beans.navigationService.onTabKeyDown(this.cellCtrl,e)},t.prototype.onBackspaceOrDeleteKeyPressed=function(e,r){this.cellCtrl.isEditing()||this.cellCtrl.startRowOrCellEdit(e,void 0,r)},t.prototype.onEnterKeyDown=function(e){this.cellCtrl.isEditing()||this.rowCtrl.isEditing()?this.cellCtrl.stopEditingAndFocus():this.beans.gridOptionsWrapper.isEnterMovesDown()?this.beans.navigationService.navigateToNextCell(null,j.DOWN,this.cellCtrl.getCellPosition(),!1):(this.cellCtrl.startRowOrCellEdit(j.ENTER,void 0,e),this.cellCtrl.isEditing()&&e.preventDefault())},t.prototype.onF2KeyDown=function(e){this.cellCtrl.isEditing()||this.cellCtrl.startRowOrCellEdit(j.F2,void 0,e)},t.prototype.onEscapeKeyDown=function(e){this.cellCtrl.isEditing()&&(this.cellCtrl.stopRowOrCellEdit(!0),this.cellCtrl.focusCell(!0))},t.prototype.onKeyPress=function(e){var r=e.target,n=r!==this.eGui;if(!(n||this.cellCtrl.isEditing())){var o=String.fromCharCode(e.charCode);o===" "?this.onSpaceKeyPressed(e):oE(e)&&(this.cellCtrl.startRowOrCellEdit(null,o,e),e.preventDefault())}},t.prototype.onSpaceKeyPressed=function(e){var r=this.beans.gridOptionsWrapper;if(!this.cellCtrl.isEditing()&&r.isRowSelection()){var n=this.rowNode.isSelected(),o=!n;if(o||!r.isSuppressRowDeselection()){var a=this.beans.gridOptionsWrapper.isGroupSelectsFiltered(),s=this.rowNode.setSelectedParams({newValue:o,rangeSelect:e.shiftKey,groupSelectsFiltered:a});n===void 0&&s===0&&this.rowNode.setSelectedParams({newValue:!1,rangeSelect:e.shiftKey,groupSelectsFiltered:a})}}e.preventDefault()},t.prototype.destroy=function(){},t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var lG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),uG=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},cG=function(i){lG(t,i);function t(e,r,n,o){var a=i.call(this,'<div class="ag-drag-handle ag-row-drag" draggable="true"></div>')||this;return a.rowNode=e,a.column=r,a.beans=n,a.eCell=o,a}return t.prototype.postConstruct=function(){var e=this.getGui();e.appendChild(Ft("rowDrag",this.beans.gridOptionsWrapper,null)),this.addGuiEventListener("mousedown",function(r){r.stopPropagation()}),this.addDragSource(),this.checkVisibility()},t.prototype.addDragSource=function(){this.addGuiEventListener("dragstart",this.onDragStart.bind(this))},t.prototype.onDragStart=function(e){var r=this,n=this.column.getColDef().dndSourceOnRowDrag;e.dataTransfer.setDragImage(this.eCell,0,0);var o=function(){try{var s=JSON.stringify(r.rowNode.data);e.dataTransfer.setData("application/json",s),e.dataTransfer.setData("text/plain",s)}catch{}};if(n){var a={rowNode:this.rowNode,dragEvent:e,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()};n(a)}else o()},t.prototype.checkVisibility=function(){var e=this.column.isDndSource(this.rowNode);this.setDisplayed(e)},uG([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var pG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),nd=globalThis&&globalThis.__assign||function(){return nd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},nd.apply(this,arguments)},hG="ag-cell",dG="ag-cell-auto-height",fG="ag-cell-normal-height",gG="ag-cell-focus",vG="ag-cell-first-right-pinned",mG="ag-cell-last-left-pinned",C_="ag-cell-not-inline-editing",yG="ag-cell-inline-editing",CG="ag-cell-popup-editing",wG="ag-column-hover",bG="ag-cell-wrap-text",_G=0,ta=function(i){pG(t,i);function t(e,r,n,o){var a=i.call(this)||this;return a.suppressRefreshCell=!1,a.column=e,a.rowNode=r,a.beans=n,a.rowCtrl=o,a.instanceId=e.getId()+"-"+_G++,a.createCellPosition(),a.addFeatures(),a}return t.prototype.addFeatures=function(){var e=this;this.cellPositionFeature=new eG(this,this.beans),this.addDestroyFunc(function(){return e.cellPositionFeature.destroy()}),this.cellCustomStyleFeature=new rG(this,this.beans),this.addDestroyFunc(function(){return e.cellCustomStyleFeature.destroy()}),this.cellMouseListenerFeature=new oG(this,this.beans,this.column),this.addDestroyFunc(function(){return e.cellMouseListenerFeature.destroy()}),this.cellKeyboardListenerFeature=new sG(this,this.beans,this.column,this.rowNode,this.rowCtrl),this.addDestroyFunc(function(){return e.cellKeyboardListenerFeature.destroy()});var r=this.beans.rangeService&&this.beans.gridOptionsWrapper.isEnableRangeSelection();r&&(this.cellRangeFeature=new QN(this.beans,this),this.addDestroyFunc(function(){return e.cellRangeFeature.destroy()})),this.addTooltipFeature()},t.prototype.addTooltipFeature=function(){var e=this,r=function(){var o=e.column.getColDef(),a=e.rowNode.data;if(o.tooltipField&&q(a))return Uh(a,o.tooltipField,e.column.isTooltipFieldContainsDots());var s=o.tooltipValueGetter;return s?s({location:"cell",api:e.beans.gridOptionsWrapper.getApi(),columnApi:e.beans.gridOptionsWrapper.getColumnApi(),context:e.beans.gridOptionsWrapper.getContext(),colDef:e.column.getColDef(),column:e.column,rowIndex:e.cellPosition.rowIndex,node:e.rowNode,data:e.rowNode.data,value:e.value,valueFormatted:e.valueFormatted}):null},n={getColumn:function(){return e.column},getColDef:function(){return e.column.getColDef()},getRowIndex:function(){return e.cellPosition.rowIndex},getRowNode:function(){return e.rowNode},getGui:function(){return e.getGui()},getLocation:function(){return"cell"},getTooltipValue:r,getValueFormatted:function(){return e.valueFormatted}};this.tooltipFeature=new QC(n,this.beans),this.addDestroyFunc(function(){return e.tooltipFeature.destroy()})},t.prototype.setComp=function(e,r,n,o,a){this.cellComp=e,this.gow=this.beans.gridOptionsWrapper,this.eGui=r,this.eCellWrapper=n,this.printLayout=o,this.updateAndFormatValue(!0),this.addDomData(),this.onCellFocused(),this.applyStaticCssClasses(),this.setWrapText(),this.onFirstRightPinnedChanged(),this.onLastLeftPinnedChanged(),this.onColumnHover(),this.setupControlComps(),this.setupAutoHeight(),this.setAriaColIndex(),this.gow.isSuppressCellFocus()||this.cellComp.setTabIndex(-1);var s=Ii(this.column.getId());this.cellComp.setColId(s),this.cellComp.setRole("gridcell"),this.cellPositionFeature.setComp(r),this.cellCustomStyleFeature.setComp(e),this.tooltipFeature.setComp(e),this.cellKeyboardListenerFeature.setComp(this.eGui),this.cellRangeFeature&&this.cellRangeFeature.setComp(e,r),a&&this.isCellEditable()?this.startEditing():this.showValue()},t.prototype.setupAutoHeight=function(){var e=this;if(!!this.column.isAutoHeight()){var r=this.eCellWrapper,n=r.parentElement,o=this.beans.gridOptionsWrapper.getRowHeightForNode(this.rowNode).height,a=function(u){if(!e.editing&&!!e.isAlive()){var c=Hl(n),p=c.paddingTop,h=c.paddingBottom,d=r.offsetHeight,f=d+p+h;if(u<5){var g=e.beans.gridOptionsWrapper.getDocument(),v=!g||!g.contains(r),m=f==0;if(v||m){e.beans.frameworkOverrides.setTimeout(function(){return a(u+1)},0);return}}var C=Math.max(f,o);e.rowNode.setRowAutoHeight(C,e.column)}},s=function(){return a(0)};s();var l=this.beans.resizeObserverService.observeResize(r,s);this.addDestroyFunc(function(){l(),e.rowNode.setRowAutoHeight(void 0,e.column)})}},t.prototype.getInstanceId=function(){return this.instanceId},t.prototype.showValue=function(e){e===void 0&&(e=!1);var r=this.valueFormatted!=null?this.valueFormatted:this.value,n=this.createCellRendererParams(),o=this.beans.userComponentFactory.getCellRendererDetails(this.column.getColDef(),n);this.cellComp.setRenderDetails(o,r,e),this.refreshHandle()},t.prototype.setupControlComps=function(){var e=this.column.getColDef();this.includeSelection=this.isIncludeControl(e.checkboxSelection),this.includeRowDrag=this.isIncludeControl(e.rowDrag),this.includeDndSource=this.isIncludeControl(e.dndSource),this.cellComp.setIncludeSelection(this.includeSelection),this.cellComp.setIncludeDndSource(this.includeDndSource),this.cellComp.setIncludeRowDrag(this.includeRowDrag)},t.prototype.isForceWrapper=function(){var e=this.beans.gridOptionsWrapper.isEnableCellTextSelection()||this.column.isAutoHeight();return e},t.prototype.isIncludeControl=function(e){var r=this.rowNode.rowPinned!=null,n=typeof e=="function",o=r?!1:n||e===!0;return o},t.prototype.refreshShouldDestroy=function(){var e=this.column.getColDef(),r=this.includeSelection!=this.isIncludeControl(e.checkboxSelection),n=this.includeRowDrag!=this.isIncludeControl(e.rowDrag),o=this.includeDndSource!=this.isIncludeControl(e.dndSource);return r||n||o},t.prototype.startEditing=function(e,r,n,o){e===void 0&&(e=null),r===void 0&&(r=null),n===void 0&&(n=!1),o===void 0&&(o=null);var a,s;if(!(!this.isCellEditable()||this.editing)){var l=this.createCellEditorParams(e,r,n),u=this.column.getColDef(),c=this.beans.userComponentFactory.getCellEditorDetails(u,l),p=((a=c)===null||a===void 0?void 0:a.popupFromSelector)!=null?c.popupFromSelector:!!u.cellEditorPopup,h=((s=c)===null||s===void 0?void 0:s.popupPositionFromSelector)!=null?c.popupPositionFromSelector:u.cellEditorPopupPosition;this.setEditing(!0,p),this.cellComp.setEditDetails(c,p,h);var d=this.createEvent(o,x.EVENT_CELL_EDITING_STARTED);this.beans.eventService.dispatchEvent(d)}},t.prototype.setEditing=function(e,r){r===void 0&&(r=!1),this.editing!==e&&(this.editing=e,this.editingInPopup=r,this.setInlineEditingClass())},t.prototype.stopRowOrCellEdit=function(e){e===void 0&&(e=!1),this.beans.gridOptionsWrapper.isFullRowEdit()?this.rowCtrl.stopRowEditing(e):this.stopEditing(e)},t.prototype.onPopupEditorClosed=function(){!this.isEditing()||this.stopEditingAndFocus()},t.prototype.takeValueFromCellEditor=function(e){var r={newValueExists:!1};if(e)return r;var n=this.cellComp.getCellEditor();if(!n)return r;var o=n.isCancelAfterEnd&&n.isCancelAfterEnd();if(o)return r;var a=n.getValue();return{newValue:a,newValueExists:!0}},t.prototype.saveNewValue=function(e,r){if(r===e)return!1;if(this.beans.gridOptionsWrapper.isReadOnlyEdit())return this.dispatchEventForSaveValueReadOnly(e,r),!1;this.suppressRefreshCell=!0;var n=this.rowNode.setDataValue(this.column,r);return this.suppressRefreshCell=!1,n},t.prototype.dispatchEventForSaveValueReadOnly=function(e,r){var n=this.rowNode,o={type:x.EVENT_CELL_EDIT_REQUEST,event:null,rowIndex:n.rowIndex,rowPinned:n.rowPinned,column:this.column,api:this.beans.gridApi,columnApi:this.beans.columnApi,colDef:this.column.getColDef(),context:this.beans.gridOptionsWrapper.getContext(),data:n.data,node:n,oldValue:e,newValue:r,value:r,source:void 0};this.beans.eventService.dispatchEvent(o)},t.prototype.stopEditing=function(e){if(e===void 0&&(e=!1),!this.editing)return!1;var r=this.takeValueFromCellEditor(e),n=r.newValue,o=r.newValueExists,a=this.getValueFromValueService(),s=!1;return o&&(s=this.saveNewValue(a,n)),this.setEditing(!1),this.cellComp.setEditDetails(),this.updateAndFormatValue(),this.refreshCell({forceRefresh:!0,suppressFlash:!0}),this.dispatchEditingStoppedEvent(a,n),s},t.prototype.dispatchEditingStoppedEvent=function(e,r){var n=nd(nd({},this.createEvent(null,x.EVENT_CELL_EDITING_STOPPED)),{oldValue:e,newValue:r});this.beans.eventService.dispatchEvent(n)},t.prototype.setInlineEditingClass=function(){if(!!this.isAlive()){var e=this.editing&&!this.editingInPopup,r=this.editing&&this.editingInPopup;this.cellComp.addOrRemoveCssClass(yG,e),this.cellComp.addOrRemoveCssClass(C_,!e),this.cellComp.addOrRemoveCssClass(CG,r),this.rowCtrl.setInlineEditingCss(this.editing)}},t.prototype.hackSayEditingInPopup=function(){this.editingInPopup||(this.editingInPopup=!0,this.setInlineEditingClass())},t.prototype.createCellEditorParams=function(e,r,n){var o={value:this.getValueFromValueService(),key:e,eventKey:e,charPress:r,column:this.column,colDef:this.column.getColDef(),rowIndex:this.getCellPosition().rowIndex,node:this.rowNode,data:this.rowNode.data,api:this.beans.gridOptionsWrapper.getApi(),cellStartedEdit:n,columnApi:this.beans.gridOptionsWrapper.getColumnApi(),context:this.beans.gridOptionsWrapper.getContext(),onKeyDown:this.onKeyDown.bind(this),stopEditing:this.stopEditingAndFocus.bind(this),eGridCell:this.getGui(),parseValue:this.parseValue.bind(this),formatValue:this.formatValue.bind(this)};return o},t.prototype.createCellRendererParams=function(){var e=this,r=function(o,a){console.warn("AG Grid: since AG Grid v26, params.addRowCompListener() is deprecated. If you need this functionality, please contact AG Grid support and advise why so that we can revert with an appropriate workaround, as we dont have any valid use cases for it. This method was originally provided as a work around to know when cells were destroyed in AG Grid before custom Cell Renderers could be provided."),e.rowCtrl.addEventListener(o,a)},n={value:this.value,valueFormatted:this.valueFormatted,getValue:this.getValueFromValueService.bind(this),setValue:function(o){return e.beans.valueService.setValue(e.rowNode,e.column,o)},formatValue:this.formatValue.bind(this),data:this.rowNode.data,node:this.rowNode,colDef:this.column.getColDef(),column:this.column,rowIndex:this.getCellPosition().rowIndex,api:this.beans.gridOptionsWrapper.getApi(),columnApi:this.beans.gridOptionsWrapper.getColumnApi(),context:this.beans.gridOptionsWrapper.getContext(),refreshCell:this.refreshCell.bind(this),eGridCell:this.getGui(),eParentOfValue:this.cellComp.getParentOfValue(),registerRowDragger:function(o,a,s,l){return e.registerRowDragger(o,a,l)},addRowCompListener:r};return n},t.prototype.parseValue=function(e){var r=this.column.getColDef(),n={node:this.rowNode,data:this.rowNode.data,oldValue:this.getValue(),newValue:e,colDef:r,column:this.column,api:this.beans.gridOptionsWrapper.getApi(),columnApi:this.beans.gridOptionsWrapper.getColumnApi(),context:this.beans.gridOptionsWrapper.getContext()},o=r.valueParser;return q(o)?this.beans.expressionService.evaluate(o,n):e},t.prototype.setFocusOutOnEditor=function(){if(!!this.editing){var e=this.cellComp.getCellEditor();e&&e.focusOut&&e.focusOut()}},t.prototype.setFocusInOnEditor=function(){if(!!this.editing){var e=this.cellComp.getCellEditor();e&&e.focusIn?e.focusIn():this.focusCell(!0)}},t.prototype.onCellChanged=function(e){if(!!this.cellComp){var r=e.column===this.column;r&&this.refreshCell({})}},t.prototype.refreshCell=function(e){if(!(this.suppressRefreshCell||this.editing)&&!!this.cellComp){var r=this.column.getColDef(),n=e!=null&&!!e.newData,o=e!=null&&!!e.suppressFlash||!!r.suppressCellFlash,a=r.field==null&&r.valueGetter==null&&r.showRowGroup==null,s=e&&e.forceRefresh||a||n,l=this.updateAndFormatValue(),u=s||l;if(u){this.showValue(n);var c=this.beans.filterManager.isSuppressFlashingCellsBecauseFiltering(),p=!o&&!c&&(this.beans.gridOptionsWrapper.isEnableCellChangeFlash()||r.enableCellChangeFlash);p&&this.flashCell(),this.cellCustomStyleFeature.applyUserStyles(),this.cellCustomStyleFeature.applyClassesFromColDef()}this.refreshToolTip(),this.cellCustomStyleFeature.applyCellClassRules()}},t.prototype.stopEditingAndFocus=function(e){e===void 0&&(e=!1),this.stopRowOrCellEdit(),this.focusCell(!0),e||this.navigateAfterEdit()},t.prototype.navigateAfterEdit=function(){var e=this.beans.gridOptionsWrapper.isFullRowEdit();if(!e){var r=this.beans.gridOptionsWrapper.isEnterMovesDownAfterEdit();r&&this.beans.navigationService.navigateToNextCell(null,j.DOWN,this.getCellPosition(),!1)}},t.prototype.flashCell=function(e){var r=e&&e.flashDelay,n=e&&e.fadeDelay;this.animateCell("data-changed",r,n)},t.prototype.animateCell=function(e,r,n){var o=this,a="ag-cell-"+e,s="ag-cell-"+e+"-animation",l=this.beans.gridOptionsWrapper;r||(r=l.getCellFlashDelay()),q(n)||(n=l.getCellFadeDelay()),this.cellComp.addOrRemoveCssClass(a,!0),this.cellComp.addOrRemoveCssClass(s,!1),window.setTimeout(function(){o.cellComp.addOrRemoveCssClass(a,!1),o.cellComp.addOrRemoveCssClass(s,!0),o.eGui.style.transition="background-color "+n+"ms",window.setTimeout(function(){o.cellComp.addOrRemoveCssClass(s,!1),o.eGui.style.transition=""},n)},r)},t.prototype.onFlashCells=function(e){if(!!this.cellComp){var r=this.beans.cellPositionUtils.createId(this.getCellPosition()),n=e.cells[r];n&&this.animateCell("highlight")}},t.prototype.isCellEditable=function(){return this.column.isCellEditable(this.rowNode)},t.prototype.isSuppressFillHandle=function(){return this.column.isSuppressFillHandle()},t.prototype.formatValue=function(e){var r=this.callValueFormatter(e);return r!=null?r:e},t.prototype.callValueFormatter=function(e){return this.beans.valueFormatterService.formatValue(this.column,this.rowNode,e)},t.prototype.updateAndFormatValue=function(e){e===void 0&&(e=!1);var r=this.value,n=this.valueFormatted;this.value=this.getValueFromValueService(),this.valueFormatted=this.callValueFormatter(this.value);var o=e?!0:!this.valuesAreEqual(r,this.value)||this.valueFormatted!=n;return o},t.prototype.valuesAreEqual=function(e,r){var n=this.column.getColDef();return n.equals?n.equals(e,r):e===r},t.prototype.getComp=function(){return this.cellComp},t.prototype.getValueFromValueService=function(){var e=this.rowNode.leafGroup&&this.beans.columnModel.isPivotMode(),r=this.rowNode.group&&this.rowNode.expanded&&!this.rowNode.footer&&!e,n=this.beans.gridOptionsWrapper.isGroupIncludeFooter(),o=this.beans.gridOptionsWrapper.isGroupSuppressBlankHeader(),a=r&&n&&!o,s=this.beans.valueService.getValue(this.column,this.rowNode,!1,a);return s},t.prototype.getValue=function(){return this.value},t.prototype.getValueFormatted=function(){return this.valueFormatted},t.prototype.addDomData=function(){var e=this,r=this.getGui();this.beans.gridOptionsWrapper.setDomData(r,t.DOM_DATA_KEY_CELL_CTRL,this),this.addDestroyFunc(function(){return e.beans.gridOptionsWrapper.setDomData(r,t.DOM_DATA_KEY_CELL_CTRL,null)})},t.prototype.createEvent=function(e,r){var n={type:r,node:this.rowNode,data:this.rowNode.data,value:this.value,column:this.column,colDef:this.column.getColDef(),context:this.beans.gridOptionsWrapper.getContext(),api:this.beans.gridApi,columnApi:this.beans.columnApi,rowPinned:this.rowNode.rowPinned,event:e,rowIndex:this.rowNode.rowIndex};return n},t.prototype.onKeyPress=function(e){this.cellKeyboardListenerFeature.onKeyPress(e)},t.prototype.onKeyDown=function(e){this.cellKeyboardListenerFeature.onKeyDown(e)},t.prototype.onMouseEvent=function(e,r){this.cellMouseListenerFeature.onMouseEvent(e,r)},t.prototype.getGui=function(){return this.eGui},t.prototype.refreshToolTip=function(){this.tooltipFeature.refreshToolTip()},t.prototype.getColSpanningList=function(){return this.cellPositionFeature.getColSpanningList()},t.prototype.onLeftChanged=function(){!this.cellComp||this.cellPositionFeature.onLeftChanged()},t.prototype.onDisplayedColumnsChanged=function(){!this.eGui||this.setAriaColIndex()},t.prototype.setAriaColIndex=function(){var e=this.beans.columnModel.getAriaColumnIndex(this.column);BC(this.getGui(),e)},t.prototype.isSuppressNavigable=function(){return this.column.isSuppressNavigable(this.rowNode)},t.prototype.onWidthChanged=function(){return this.cellPositionFeature.onWidthChanged()},t.prototype.getColumn=function(){return this.column},t.prototype.getRowNode=function(){return this.rowNode},t.prototype.getBeans=function(){return this.beans},t.prototype.isPrintLayout=function(){return this.printLayout},t.prototype.appendChild=function(e){this.eGui.appendChild(e)},t.prototype.refreshHandle=function(){this.editing||this.cellRangeFeature&&this.cellRangeFeature.refreshHandle()},t.prototype.getCellPosition=function(){return this.cellPosition},t.prototype.isEditing=function(){return this.editing},t.prototype.startRowOrCellEdit=function(e,r,n){n===void 0&&(n=null),this.beans.gridOptionsWrapper.isFullRowEdit()?this.rowCtrl.startRowEditing(e,r,this):this.startEditing(e,r,!0,n)},t.prototype.getRowCtrl=function(){return this.rowCtrl},t.prototype.getRowPosition=function(){return{rowIndex:this.cellPosition.rowIndex,rowPinned:this.cellPosition.rowPinned}},t.prototype.updateRangeBordersIfRangeCount=function(){!this.cellComp||this.cellRangeFeature&&this.cellRangeFeature.updateRangeBordersIfRangeCount()},t.prototype.onRangeSelectionChanged=function(){!this.cellComp||this.cellRangeFeature&&this.cellRangeFeature.onRangeSelectionChanged()},t.prototype.isRangeSelectionEnabled=function(){return this.cellRangeFeature!=null},t.prototype.focusCell=function(e){e===void 0&&(e=!1),this.beans.focusService.setFocusedCell(this.getCellPosition().rowIndex,this.column,this.rowNode.rowPinned,e)},t.prototype.onRowIndexChanged=function(){this.createCellPosition(),this.onCellFocused(),this.cellRangeFeature&&this.cellRangeFeature.onRangeSelectionChanged()},t.prototype.onFirstRightPinnedChanged=function(){if(!!this.cellComp){var e=this.column.isFirstRightPinned();this.cellComp.addOrRemoveCssClass(vG,e)}},t.prototype.onLastLeftPinnedChanged=function(){if(!!this.cellComp){var e=this.column.isLastLeftPinned();this.cellComp.addOrRemoveCssClass(mG,e)}},t.prototype.onCellFocused=function(e){if(!(!this.cellComp||this.gow.isSuppressCellFocus())){var r=this.beans.focusService.isCellFocused(this.cellPosition);if(this.cellComp.addOrRemoveCssClass(gG,r),r&&e&&e.forceBrowserFocus){var n=this.cellComp.getFocusableElement();n.focus()}var o=this.beans.gridOptionsWrapper.isFullRowEdit();!r&&!o&&this.editing&&this.stopRowOrCellEdit()}},t.prototype.createCellPosition=function(){this.cellPosition={rowIndex:this.rowNode.rowIndex,rowPinned:Jo(this.rowNode.rowPinned),column:this.column}},t.prototype.applyStaticCssClasses=function(){this.cellComp.addOrRemoveCssClass(hG,!0),this.cellComp.addOrRemoveCssClass(C_,!0);var e=this.column.isAutoHeight()==!0;this.cellComp.addOrRemoveCssClass(dG,e),this.cellComp.addOrRemoveCssClass(fG,!e)},t.prototype.onColumnHover=function(){if(!!this.cellComp&&!!this.beans.gridOptionsWrapper.isColumnHoverHighlight()){var e=this.beans.columnHoverService.isHovered(this.column);this.cellComp.addOrRemoveCssClass(wG,e)}},t.prototype.onNewColumnsLoaded=function(){!this.cellComp||(this.setWrapText(),this.editing||this.refreshCell({forceRefresh:!0,suppressFlash:!0}))},t.prototype.setWrapText=function(){var e=this.column.getColDef().wrapText==!0;this.cellComp.addOrRemoveCssClass(bG,e)},t.prototype.dispatchCellContextMenuEvent=function(e){var r=this.column.getColDef(),n=this.createEvent(e,x.EVENT_CELL_CONTEXT_MENU);this.beans.eventService.dispatchEvent(n),r.onCellContextMenu&&window.setTimeout(function(){return r.onCellContextMenu(n)},0)},t.prototype.getCellRenderer=function(){return this.cellComp?this.cellComp.getCellRenderer():null},t.prototype.getCellEditor=function(){return this.cellComp?this.cellComp.getCellEditor():null},t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.createSelectionCheckbox=function(){var e=new XC;return this.beans.context.createBean(e),e.init({rowNode:this.rowNode,column:this.column}),e},t.prototype.createDndSource=function(){var e=new cG(this.rowNode,this.column,this.beans,this.eGui);return this.beans.context.createBean(e),e},t.prototype.registerRowDragger=function(e,r,n){var o=this;if(this.customRowDragComp){this.customRowDragComp.setDragElement(e,r);return}var a=this.createRowDragComp(e,r,n);a&&(this.customRowDragComp=a,this.addDestroyFunc(function(){return o.beans.context.destroyBean(a)}))},t.prototype.createRowDragComp=function(e,r,n){var o=this,a=this.beans.gridOptionsWrapper.isPagination(),s=this.beans.gridOptionsWrapper.isRowDragManaged(),l=this.beans.gridOptionsWrapper.isRowModelDefault();if(s){if(!l){Ye(function(){return console.warn("AG Grid: managed row dragging is only allowed in the Client Side Row Model")},"CellComp.addRowDragging");return}if(a){Ye(function(){return console.warn("AG Grid: managed row dragging is not possible when doing pagination")},"CellComp.addRowDragging");return}}var u=new Jh(function(){return o.value},this.rowNode,this.column,e,r,n);return this.beans.context.createBean(u),u},t.DOM_DATA_KEY_CELL_CTRL="cellCtrl",t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var SG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),xG=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},TG=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(xG(arguments[t]));return i},Wr;(function(i){i.Normal="Normal",i.FullWidth="FullWidth",i.FullWidthLoading="FullWidthLoading",i.FullWidthGroup="FullWidthGroup",i.FullWidthDetail="FullWidthDetail"})(Wr||(Wr={}));var OG=0,ra=function(i){SG(t,i);function t(e,r,n,o,a){var s=i.call(this)||this;return s.allRowGuis=[],s.active=!0,s.centerCellCtrls={list:[],map:{}},s.leftCellCtrls={list:[],map:{}},s.rightCellCtrls={list:[],map:{}},s.lastMouseDownOnDragger=!1,s.updateColumnListsPending=!1,s.beans=r,s.rowNode=e,s.paginationPage=s.beans.paginationProxy.getCurrentPage(),s.useAnimationFrameForCreate=o,s.printLayout=a,s.instanceId=e.id+"-"+OG++,s.setAnimateFlags(n),s.rowFocused=s.beans.focusService.isRowFocused(s.rowNode.rowIndex,s.rowNode.rowPinned),s.rowLevel=s.beans.rowCssClassCalculator.calculateRowLevel(s.rowNode),s.setRowType(),s.addListeners(),s.setInitialRowTop(),s}return t.prototype.getBeans=function(){return this.beans},t.prototype.getInstanceId=function(){return this.instanceId},t.prototype.setComp=function(e,r,n){var o={rowComp:e,element:r,containerType:n};this.allRowGuis.push(o),n===Hr.LEFT?this.leftGui=o:n===Hr.RIGHT?this.rightGui=o:n===Hr.FULL_WIDTH?this.fullWidthGui=o:this.centerGui=o;var a=this.leftGui!=null&&this.rightGui!=null&&this.centerGui!=null,s=this.fullWidthGui!=null;(a||s)&&this.initialiseRowComps()},t.prototype.isCacheable=function(){return this.rowType===Wr.FullWidthDetail&&this.beans.gridOptionsWrapper.isKeepDetailRows()},t.prototype.setCached=function(e){var r=e?"none":"";this.allRowGuis.forEach(function(n){return n.element.style.display=r})},t.prototype.initialiseRowComps=function(){var e=this,r=this.beans.gridOptionsWrapper;this.onRowHeightChanged(),this.updateRowIndexes(),this.setFocusedClasses(),this.setStylesFromGridOptions(),r.isRowSelection()&&this.rowNode.selectable&&this.onRowSelected(),this.updateColumnLists(!this.useAnimationFrameForCreate),this.slideRowIn&&km(this.onTopChanged.bind(this)),this.fadeRowIn&&km(function(){e.allRowGuis.forEach(function(s){return s.rowComp.addOrRemoveCssClass("ag-opacity-zero",!1)})});var n=this.getRowBusinessKey(),o=Ii(this.rowNode.id),a=Ii(n);this.allRowGuis.forEach(function(s){var l=s.rowComp;l.setRole("row");var u=e.getInitialRowClasses(s.containerType);u.forEach(function(c){return l.addOrRemoveCssClass(c,!0)}),e.rowNode.group&&dn(s.element,e.rowNode.expanded==!0),o!=null&&l.setRowId(o),a!=null&&l.setRowBusinessKey(a),e.isFullWidth()&&!e.beans.gridOptionsWrapper.isSuppressCellFocus()&&l.setTabIndex(-1),r.setDomData(s.element,t.DOM_DATA_KEY_ROW_CTRL,e),e.addDestroyFunc(function(){return r.setDomData(s.element,t.DOM_DATA_KEY_ROW_CTRL,null)}),e.useAnimationFrameForCreate?e.beans.animationFrameService.createTask(e.addHoverFunctionality.bind(e,s.element),e.rowNode.rowIndex,"createTasksP2"):e.addHoverFunctionality(s.element),e.isFullWidth()&&e.setupFullWidth(s),r.isRowDragEntireRow()&&e.addRowDraggerToRow(s),e.useAnimationFrameForCreate&&e.beans.animationFrameService.addDestroyTask(function(){!e.isAlive()||s.rowComp.addOrRemoveCssClass("ag-after-created",!0)})}),this.executeProcessRowPostCreateFunc()},t.prototype.addRowDraggerToRow=function(e){var r=this.beans.gridOptionsWrapper;if(r.isEnableRangeSelection()){Ye(function(){console.warn("AG Grid: Setting `rowDragEntireRow: true` in the gridOptions doesn't work with `enableRangeSelection: true`")},"rowDragAndRangeSelectionEnabled");return}var n=new Jh(function(){return"1 row"},this.rowNode,void 0,e.element,void 0,!0);this.createManagedBean(n,this.beans.context)},t.prototype.setupFullWidth=function(e){var r=this.getPinnedForContainer(e.containerType),n=this.createFullWidthParams(e.element,r),o=ge.isRegistered(oe.MasterDetailModule);if(this.rowType==Wr.FullWidthDetail&&!o){ge.isPackageBased()?console.warn("AG Grid: cell renderer 'agDetailCellRenderer' (for master detail) not found. Can only be used with ag-grid-enterprise package."):console.warn("AG Grid: cell renderer 'agDetailCellRenderer' (for master detail) not found. Can only be used with AG Grid Enterprise Module "+oe.MasterDetailModule);return}var a;switch(this.rowType){case Wr.FullWidthDetail:a=this.beans.userComponentFactory.getFullWidthDetailCellRendererDetails(n);break;case Wr.FullWidthGroup:a=this.beans.userComponentFactory.getFullWidthGroupCellRendererDetails(n);break;case Wr.FullWidthLoading:a=this.beans.userComponentFactory.getFullWidthLoadingCellRendererDetails(n);break;default:a=this.beans.userComponentFactory.getFullWidthCellRendererDetails(n);break}e.rowComp.showFullWidth(a)},t.prototype.isPrintLayout=function(){return this.printLayout},t.prototype.getCellElement=function(e){var r=this.getCellCtrl(e);return r?r.getGui():null},t.prototype.executeProcessRowPostCreateFunc=function(){var e=this.beans.gridOptionsWrapper.getProcessRowPostCreateFunc();if(!!e){var r={eRow:this.centerGui?this.centerGui.element:void 0,ePinnedLeftRow:this.leftGui?this.leftGui.element:void 0,ePinnedRightRow:this.rightGui?this.rightGui.element:void 0,node:this.rowNode,rowIndex:this.rowNode.rowIndex,addRenderedRowListener:this.addEventListener.bind(this)};e(r)}},t.prototype.setRowType=function(){var e=this.rowNode.stub,r=this.rowNode.isFullWidthCell(),n=this.beans.doingMasterDetail&&this.rowNode.detail,o=this.beans.columnModel.isPivotMode(),a=!!this.rowNode.group&&!this.rowNode.footer,s=a&&this.beans.gridOptionsWrapper.isGroupUseEntireRow(o);e?this.rowType=Wr.FullWidthLoading:n?this.rowType=Wr.FullWidthDetail:r?this.rowType=Wr.FullWidth:s?this.rowType=Wr.FullWidthGroup:this.rowType=Wr.Normal},t.prototype.updateColumnLists=function(e){var r=this;if(e===void 0&&(e=!1),!this.isFullWidth()){var n=e||this.beans.gridOptionsWrapper.isSuppressAnimationFrame()||this.printLayout;if(n){this.updateColumnListsImpl();return}this.updateColumnListsPending||(this.beans.animationFrameService.createTask(function(){!r.active||r.updateColumnListsImpl()},this.rowNode.rowIndex,"createTasksP1"),this.updateColumnListsPending=!0)}},t.prototype.createCellCtrls=function(e,r,n){var o=this;n===void 0&&(n=null);var a={list:[],map:{}},s=function(l,u){a.list.push(u),a.map[l]=u};return r.forEach(function(l){var u=l.getInstanceId(),c=e.map[u];c||(c=new ta(l,o.rowNode,o.beans,o)),s(u,c)}),e.list.forEach(function(l){var u=a.map[l.getColumn().getInstanceId()]!=null;if(!u){var c=!o.isCellEligibleToBeRemoved(l,n);if(c){s(l.getColumn().getInstanceId(),l);return}l.destroy()}}),a},t.prototype.updateColumnListsImpl=function(){var e=this;this.updateColumnListsPending=!1;var r=this.beans.columnModel;if(this.printLayout)this.centerCellCtrls=this.createCellCtrls(this.centerCellCtrls,r.getAllDisplayedColumns()),this.leftCellCtrls={list:[],map:{}},this.rightCellCtrls={list:[],map:{}};else{var n=r.getViewportCenterColumnsForRow(this.rowNode);this.centerCellCtrls=this.createCellCtrls(this.centerCellCtrls,n);var o=r.getDisplayedLeftColumnsForRow(this.rowNode);this.leftCellCtrls=this.createCellCtrls(this.leftCellCtrls,o,L.PINNED_LEFT);var a=r.getDisplayedRightColumnsForRow(this.rowNode);this.rightCellCtrls=this.createCellCtrls(this.rightCellCtrls,a,L.PINNED_RIGHT)}this.allRowGuis.forEach(function(s){var l=s.containerType===Hr.LEFT?e.leftCellCtrls:s.containerType===Hr.RIGHT?e.rightCellCtrls:e.centerCellCtrls;s.rowComp.setCellCtrls(l.list)})},t.prototype.isCellEligibleToBeRemoved=function(e,r){var n=!0,o=!1,a=e.getColumn();if(a.getPinned()!=r)return n;var s=e.isEditing(),l=this.beans.focusService.isCellFocused(e.getCellPosition()),u=s||l;if(u){var c=e.getColumn(),p=this.beans.columnModel.getAllDisplayedColumns(),h=p.indexOf(c)>=0;return h?o:n}return n},t.prototype.setAnimateFlags=function(e){if(e){var r=q(this.rowNode.oldRowTop);this.slideRowIn=r,this.fadeRowIn=!r}else this.slideRowIn=!1,this.fadeRowIn=!1},t.prototype.isEditing=function(){return this.editingRow},t.prototype.stopRowEditing=function(e){this.stopEditing(e)},t.prototype.isFullWidth=function(){return this.rowType!==Wr.Normal},t.prototype.getRowType=function(){return this.rowType},t.prototype.refreshFullWidth=function(){var e=this,r=function(u,c){if(!u)return!0;var p=u.rowComp.getFullWidthCellRenderer();if(!p||!p.refresh)return!1;var h=e.createFullWidthParams(u.element,c),d=p.refresh(h);return d},n=r(this.fullWidthGui,null),o=r(this.centerGui,null),a=r(this.leftGui,L.PINNED_LEFT),s=r(this.rightGui,L.PINNED_RIGHT),l=n&&o&&a&&s;return l},t.prototype.addListeners=function(){this.addManagedListener(this.rowNode,Oe.EVENT_HEIGHT_CHANGED,this.onRowHeightChanged.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_ROW_SELECTED,this.onRowSelected.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_ROW_INDEX_CHANGED,this.onRowIndexChanged.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_TOP_CHANGED,this.onTopChanged.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_EXPANDED_CHANGED,this.updateExpandedCss.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_HAS_CHILDREN_CHANGED,this.updateExpandedCss.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_DATA_CHANGED,this.onRowNodeDataChanged.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_CELL_CHANGED,this.onRowNodeCellChanged.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_HIGHLIGHT_CHANGED,this.onRowNodeHighlightChanged.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_DRAGGING_CHANGED,this.onRowNodeDraggingChanged.bind(this)),this.addManagedListener(this.rowNode,Oe.EVENT_UI_LEVEL_CHANGED,this.onUiLevelChanged.bind(this));var e=this.beans.eventService;this.addManagedListener(e,x.EVENT_PAGINATION_PIXEL_OFFSET_CHANGED,this.onPaginationPixelOffsetChanged.bind(this)),this.addManagedListener(e,x.EVENT_HEIGHT_SCALE_CHANGED,this.onTopChanged.bind(this)),this.addManagedListener(e,x.EVENT_DISPLAYED_COLUMNS_CHANGED,this.onDisplayedColumnsChanged.bind(this)),this.addManagedListener(e,x.EVENT_VIRTUAL_COLUMNS_CHANGED,this.onVirtualColumnsChanged.bind(this)),this.addManagedListener(e,x.EVENT_CELL_FOCUSED,this.onCellFocusChanged.bind(this)),this.addManagedListener(e,x.EVENT_PAGINATION_CHANGED,this.onPaginationChanged.bind(this)),this.addManagedListener(e,x.EVENT_MODEL_UPDATED,this.onModelUpdated.bind(this)),this.addManagedListener(e,x.EVENT_COLUMN_MOVED,this.onColumnMoved.bind(this)),this.addListenersForCellComps()},t.prototype.onColumnMoved=function(){this.updateColumnLists()},t.prototype.addListenersForCellComps=function(){var e=this;this.addManagedListener(this.rowNode,Oe.EVENT_ROW_INDEX_CHANGED,function(){e.getAllCellCtrls().forEach(function(r){return r.onRowIndexChanged()})}),this.addManagedListener(this.rowNode,Oe.EVENT_CELL_CHANGED,function(r){e.getAllCellCtrls().forEach(function(n){return n.onCellChanged(r)})})},t.prototype.onRowNodeDataChanged=function(e){this.getAllCellCtrls().forEach(function(r){return r.refreshCell({suppressFlash:!e.update,newData:!e.update})}),this.onRowSelected(),this.postProcessCss()},t.prototype.onRowNodeCellChanged=function(){this.postProcessCss()},t.prototype.postProcessCss=function(){this.setStylesFromGridOptions(),this.postProcessClassesFromGridOptions(),this.postProcessRowClassRules(),this.postProcessRowDragging()},t.prototype.onRowNodeHighlightChanged=function(){var e=this.rowNode.highlighted;this.allRowGuis.forEach(function(r){var n=e===Di.Above,o=e===Di.Below;r.rowComp.addOrRemoveCssClass("ag-row-highlight-above",n),r.rowComp.addOrRemoveCssClass("ag-row-highlight-below",o)})},t.prototype.onRowNodeDraggingChanged=function(){this.postProcessRowDragging()},t.prototype.postProcessRowDragging=function(){var e=this.rowNode.dragging;this.allRowGuis.forEach(function(r){return r.rowComp.addOrRemoveCssClass("ag-row-dragging",e)})},t.prototype.updateExpandedCss=function(){var e=this.rowNode.isExpandable(),r=this.rowNode.expanded==!0;this.allRowGuis.forEach(function(n){n.rowComp.addOrRemoveCssClass("ag-row-group",e),n.rowComp.addOrRemoveCssClass("ag-row-group-expanded",e&&r),n.rowComp.addOrRemoveCssClass("ag-row-group-contracted",e&&!r),dn(n.element,e&&r)})},t.prototype.onDisplayedColumnsChanged=function(){this.updateColumnLists(!0),this.beans.columnModel.wasAutoRowHeightEverActive()&&this.rowNode.checkAutoHeights()},t.prototype.onVirtualColumnsChanged=function(){this.updateColumnLists()},t.prototype.getRowPosition=function(){return{rowPinned:Jo(this.rowNode.rowPinned),rowIndex:this.rowNode.rowIndex}},t.prototype.onKeyboardNavigate=function(e){var r=this.allRowGuis.find(function(u){return u.element.contains(e.target)}),n=r?r.element:null,o=n===e.target;if(!!o){var a=this.rowNode,s=this.beans.focusService.getFocusedCell(),l={rowIndex:a.rowIndex,rowPinned:a.rowPinned,column:s&&s.column};this.beans.navigationService.navigateToNextCell(e,e.key,l,!0),e.preventDefault()}},t.prototype.onTabKeyDown=function(e){if(!(e.defaultPrevented||Ai(e))){var r=this.allRowGuis.find(function(s){return s.element.contains(e.target)}),n=r?r.element:null,o=n===e.target,a=null;o||(a=this.beans.focusService.findNextFocusableElement(n,!1,e.shiftKey)),(this.isFullWidth()&&o||!a)&&this.beans.navigationService.onTabKeyDown(this,e)}},t.prototype.onFullWidthRowFocused=function(e){var r,n=this.rowNode,o=this.isFullWidth()&&e.rowIndex===n.rowIndex&&e.rowPinned==n.rowPinned,a=this.fullWidthGui?this.fullWidthGui.element:(r=this.centerGui)===null||r===void 0?void 0:r.element;!a||(a.classList.toggle("ag-full-width-focus",o),o&&a.focus({preventScroll:!0}))},t.prototype.refreshCell=function(e){this.centerCellCtrls=this.removeCellCtrl(this.centerCellCtrls,e),this.leftCellCtrls=this.removeCellCtrl(this.leftCellCtrls,e),this.rightCellCtrls=this.removeCellCtrl(this.rightCellCtrls,e),this.updateColumnLists()},t.prototype.removeCellCtrl=function(e,r){var n={list:[],map:{}};return e.list.forEach(function(o){o!==r&&(n.list.push(o),n.map[o.getInstanceId()]=o)}),n},t.prototype.onMouseEvent=function(e,r){switch(e){case"dblclick":this.onRowDblClick(r);break;case"click":this.onRowClick(r);break;case"touchstart":case"mousedown":this.onRowMouseDown(r);break}},t.prototype.createRowEvent=function(e,r){return{type:e,node:this.rowNode,data:this.rowNode.data,rowIndex:this.rowNode.rowIndex,rowPinned:this.rowNode.rowPinned,context:this.beans.gridOptionsWrapper.getContext(),api:this.beans.gridOptionsWrapper.getApi(),columnApi:this.beans.gridOptionsWrapper.getColumnApi(),event:r}},t.prototype.createRowEventWithSource=function(e,r){var n=this.createRowEvent(e,r);return n.source=this,n},t.prototype.onRowDblClick=function(e){if(!Ai(e)){var r=this.createRowEventWithSource(x.EVENT_ROW_DOUBLE_CLICKED,e);this.beans.eventService.dispatchEvent(r)}},t.prototype.onRowMouseDown=function(e){if(this.lastMouseDownOnDragger=ef(e.target,"ag-row-drag",3),!!this.isFullWidth()){var r=this.rowNode,n=this.beans.columnModel;this.beans.focusService.setFocusedCell(r.rowIndex,n.getAllDisplayedColumns()[0],r.rowPinned,!0)}},t.prototype.onRowClick=function(e){var r=Ai(e)||this.lastMouseDownOnDragger;if(!r){var n=this.createRowEventWithSource(x.EVENT_ROW_CLICKED,e);this.beans.eventService.dispatchEvent(n);var o=e.ctrlKey||e.metaKey,a=e.shiftKey,s=this.beans.gridOptionsWrapper.isGroupSelectsChildren();if(!(s&&this.rowNode.group||!this.rowNode.selectable||this.rowNode.rowPinned||!this.beans.gridOptionsWrapper.isRowSelection()||this.beans.gridOptionsWrapper.isSuppressRowClickSelection())){var l=this.beans.gridOptionsWrapper.isRowMultiSelectWithClick(),u=!this.beans.gridOptionsWrapper.isSuppressRowDeselection();if(this.rowNode.isSelected())l?this.rowNode.setSelectedParams({newValue:!1}):o?u&&this.rowNode.setSelectedParams({newValue:!1}):this.rowNode.setSelectedParams({newValue:!0,clearSelection:!a,rangeSelect:a});else{var c=l?!1:!o;this.rowNode.setSelectedParams({newValue:!0,clearSelection:c,rangeSelect:a})}}}},t.prototype.setupDetailRowAutoHeight=function(e){var r=this;if(this.rowType===Wr.FullWidthDetail&&!!this.beans.gridOptionsWrapper.isDetailRowAutoHeight()){var n=function(){var a=e.clientHeight;if(a!=null&&a>0){var s=function(){r.rowNode.setRowHeight(a),r.beans.clientSideRowModel?r.beans.clientSideRowModel.onRowHeightChanged():r.beans.serverSideRowModel&&r.beans.serverSideRowModel.onRowHeightChanged()};r.beans.frameworkOverrides.setTimeout(s,0)}},o=this.beans.resizeObserverService.observeResize(e,n);this.addDestroyFunc(o),n()}},t.prototype.createFullWidthParams=function(e,r){var n=this,o={fullWidth:!0,data:this.rowNode.data,node:this.rowNode,value:this.rowNode.key,valueFormatted:this.rowNode.key,rowIndex:this.rowNode.rowIndex,api:this.beans.gridOptionsWrapper.getApi(),columnApi:this.beans.gridOptionsWrapper.getColumnApi(),context:this.beans.gridOptionsWrapper.getContext(),eGridCell:e,eParentOfValue:e,pinned:r,addRenderedRowListener:this.addEventListener.bind(this),registerRowDragger:function(a,s,l,u){return n.addFullWidthRowDragging(a,s,l,u)}};return o},t.prototype.addFullWidthRowDragging=function(e,r,n,o){if(n===void 0&&(n=""),!!this.isFullWidth()){var a=new Jh(function(){return n},this.rowNode,void 0,e,r,o);this.createManagedBean(a,this.beans.context)}},t.prototype.onUiLevelChanged=function(){var e=this.beans.rowCssClassCalculator.calculateRowLevel(this.rowNode);if(this.rowLevel!=e){var r="ag-row-level-"+e,n="ag-row-level-"+this.rowLevel;this.allRowGuis.forEach(function(o){o.rowComp.addOrRemoveCssClass(r,!0),o.rowComp.addOrRemoveCssClass(n,!1)})}this.rowLevel=e},t.prototype.isFirstRowOnPage=function(){return this.rowNode.rowIndex===this.beans.paginationProxy.getPageFirstRow()},t.prototype.isLastRowOnPage=function(){return this.rowNode.rowIndex===this.beans.paginationProxy.getPageLastRow()},t.prototype.onModelUpdated=function(){this.refreshFirstAndLastRowStyles()},t.prototype.refreshFirstAndLastRowStyles=function(){var e=this.isFirstRowOnPage(),r=this.isLastRowOnPage();this.firstRowOnPage!==e&&(this.firstRowOnPage=e,this.allRowGuis.forEach(function(n){return n.rowComp.addOrRemoveCssClass("ag-row-first",e)})),this.lastRowOnPage!==r&&(this.lastRowOnPage=r,this.allRowGuis.forEach(function(n){return n.rowComp.addOrRemoveCssClass("ag-row-last",r)}))},t.prototype.stopEditing=function(e){e===void 0&&(e=!1);var r=this.getAllCellCtrls().map(function(o){return o.stopEditing(e)});if(!!this.editingRow){if(!e&&r.some(function(o){return o})){var n=this.createRowEvent(x.EVENT_ROW_VALUE_CHANGED);this.beans.eventService.dispatchEvent(n)}this.setEditingRow(!1)}},t.prototype.setInlineEditingCss=function(e){this.allRowGuis.forEach(function(r){r.rowComp.addOrRemoveCssClass("ag-row-inline-editing",e),r.rowComp.addOrRemoveCssClass("ag-row-not-inline-editing",!e)})},t.prototype.setEditingRow=function(e){this.editingRow=e,this.allRowGuis.forEach(function(n){return n.rowComp.addOrRemoveCssClass("ag-row-editing",e)});var r=e?this.createRowEvent(x.EVENT_ROW_EDITING_STARTED):this.createRowEvent(x.EVENT_ROW_EDITING_STOPPED);this.beans.eventService.dispatchEvent(r)},t.prototype.startRowEditing=function(e,r,n,o){e===void 0&&(e=null),r===void 0&&(r=null),n===void 0&&(n=null),o===void 0&&(o=null),!this.editingRow&&(this.getAllCellCtrls().forEach(function(a){var s=a===n;s?a.startEditing(e,r,s,o):a.startEditing(null,null,s,o)}),this.setEditingRow(!0))},t.prototype.getAllCellCtrls=function(){var e=TG(this.centerCellCtrls.list,this.leftCellCtrls.list,this.rightCellCtrls.list);return e},t.prototype.postProcessClassesFromGridOptions=function(){var e=this,r=this.beans.rowCssClassCalculator.processClassesFromGridOptions(this.rowNode);!r||!r.length||r.forEach(function(n){e.allRowGuis.forEach(function(o){return o.rowComp.addOrRemoveCssClass(n,!0)})})},t.prototype.postProcessRowClassRules=function(){var e=this;this.beans.rowCssClassCalculator.processRowClassRules(this.rowNode,function(r){e.allRowGuis.forEach(function(n){return n.rowComp.addOrRemoveCssClass(r,!0)})},function(r){e.allRowGuis.forEach(function(n){return n.rowComp.addOrRemoveCssClass(r,!1)})})},t.prototype.setStylesFromGridOptions=function(){var e=this.processStylesFromGridOptions();this.allRowGuis.forEach(function(r){return r.rowComp.setUserStyles(e)})},t.prototype.getRowBusinessKey=function(){var e=this.beans.gridOptionsWrapper.getBusinessKeyForNodeFunc();if(typeof e=="function")return e(this.rowNode)},t.prototype.getPinnedForContainer=function(e){var r=e===Hr.LEFT?L.PINNED_LEFT:e===Hr.RIGHT?L.PINNED_RIGHT:null;return r},t.prototype.getInitialRowClasses=function(e){var r=this.getPinnedForContainer(e),n={rowNode:this.rowNode,rowFocused:this.rowFocused,fadeRowIn:this.fadeRowIn,rowIsEven:this.rowNode.rowIndex%2===0,rowLevel:this.rowLevel,fullWidthRow:this.isFullWidth(),firstRowOnPage:this.isFirstRowOnPage(),lastRowOnPage:this.isLastRowOnPage(),printLayout:this.printLayout,expandable:this.rowNode.isExpandable(),pinned:r};return this.beans.rowCssClassCalculator.getInitialRowClasses(n)},t.prototype.processStylesFromGridOptions=function(){var e=this.beans.gridOptionsWrapper.getRowStyle();if(e&&typeof e=="function"){console.warn("AG Grid: rowStyle should be an object of key/value styles, not be a function, use getRowStyle() instead");return}var r=this.beans.gridOptionsWrapper.getRowStyleFunc(),n;if(r){var o={data:this.rowNode.data,node:this.rowNode,rowIndex:this.rowNode.rowIndex};n=r(o)}return Object.assign({},e,n)},t.prototype.onRowSelected=function(){var e=this,r=!!this.rowNode.isSelected();this.allRowGuis.forEach(function(n){n.rowComp.addOrRemoveCssClass("ag-row-selected",r),Dl(n.element,r?!0:void 0);var o=e.createAriaLabel();Cs(n.element,o==null?"":o)})},t.prototype.createAriaLabel=function(){var e=this.rowNode.isSelected();if(!(e&&this.beans.gridOptionsWrapper.isSuppressRowDeselection())){var r=this.beans.gridOptionsWrapper.getLocaleTextFunc(),n=r(e?"ariaRowDeselect":"ariaRowSelect","Press SPACE to "+(e?"deselect":"select")+" this row.");return n}},t.prototype.isUseAnimationFrameForCreate=function(){return this.useAnimationFrameForCreate},t.prototype.addHoverFunctionality=function(e){var r=this;!this.active||(this.addManagedListener(e,"mouseenter",function(){return r.rowNode.onMouseEnter()}),this.addManagedListener(e,"mouseleave",function(){return r.rowNode.onMouseLeave()}),this.addManagedListener(this.rowNode,Oe.EVENT_MOUSE_ENTER,function(){r.beans.gridOptionsWrapper.isSuppressRowHoverHighlight()||e.classList.add("ag-row-hover")}),this.addManagedListener(this.rowNode,Oe.EVENT_MOUSE_LEAVE,function(){e.classList.remove("ag-row-hover")}))},t.prototype.roundRowTopToBounds=function(e){var r=this.beans.ctrlsService.getGridBodyCtrl(),n=r.getScrollFeature().getVScrollPosition(),o=this.applyPaginationOffset(n.top,!0)-100,a=this.applyPaginationOffset(n.bottom,!0)+100;return Math.min(Math.max(o,e),a)},t.prototype.getFrameworkOverrides=function(){return this.beans.frameworkOverrides},t.prototype.onRowHeightChanged=function(){if(this.rowNode.rowHeight!=null){var e=this.rowNode.rowHeight,r=this.beans.gridOptionsWrapper.getDefaultRowHeight(),n=this.beans.gridOptionsWrapper.isGetRowHeightFunction(),o=n?this.beans.gridOptionsWrapper.getRowHeightForNode(this.rowNode).height:void 0,a=o?Math.min(r,o)-2+"px":void 0;this.allRowGuis.forEach(function(s){s.element.style.height=e+"px",a&&s.element.style.setProperty("--ag-line-height",a)})}},t.prototype.addEventListener=function(e,r){(e==="renderedRowRemoved"||e==="rowRemoved")&&(e=x.EVENT_VIRTUAL_ROW_REMOVED,console.warn("AG Grid: Since version 11, event renderedRowRemoved is now called "+x.EVENT_VIRTUAL_ROW_REMOVED)),i.prototype.addEventListener.call(this,e,r)},t.prototype.removeEventListener=function(e,r){(e==="renderedRowRemoved"||e==="rowRemoved")&&(e=x.EVENT_VIRTUAL_ROW_REMOVED,console.warn("AG Grid: Since version 11, event renderedRowRemoved and rowRemoved is now called "+x.EVENT_VIRTUAL_ROW_REMOVED)),i.prototype.removeEventListener.call(this,e,r)},t.prototype.destroyFirstPass=function(){this.active=!1,this.setupRemoveAnimation();var e=this.createRowEvent(x.EVENT_VIRTUAL_ROW_REMOVED);this.dispatchEvent(e),this.beans.eventService.dispatchEvent(e),i.prototype.destroy.call(this)},t.prototype.setupRemoveAnimation=function(){var e=this.rowNode.rowTop!=null;if(e){var r=this.roundRowTopToBounds(this.rowNode.rowTop);this.setRowTop(r)}else this.allRowGuis.forEach(function(n){return n.rowComp.addOrRemoveCssClass("ag-opacity-zero",!0)})},t.prototype.destroySecondPass=function(){this.allRowGuis.length=0;var e=function(r){return r.list.forEach(function(n){return n.destroy()}),{list:[],map:{}}};this.centerCellCtrls=e(this.centerCellCtrls),this.leftCellCtrls=e(this.leftCellCtrls),this.rightCellCtrls=e(this.rightCellCtrls)},t.prototype.setFocusedClasses=function(){var e=this;this.allRowGuis.forEach(function(r){r.rowComp.addOrRemoveCssClass("ag-row-focus",e.rowFocused),r.rowComp.addOrRemoveCssClass("ag-row-no-focus",!e.rowFocused)})},t.prototype.onCellFocusChanged=function(){var e=this.beans.focusService.isRowFocused(this.rowNode.rowIndex,this.rowNode.rowPinned);e!==this.rowFocused&&(this.rowFocused=e,this.setFocusedClasses()),!e&&this.editingRow&&this.stopEditing(!1)},t.prototype.onPaginationChanged=function(){var e=this.beans.paginationProxy.getCurrentPage();this.paginationPage!==e&&(this.paginationPage=e,this.onTopChanged()),this.refreshFirstAndLastRowStyles()},t.prototype.onTopChanged=function(){this.setRowTop(this.rowNode.rowTop)},t.prototype.onPaginationPixelOffsetChanged=function(){this.onTopChanged()},t.prototype.applyPaginationOffset=function(e,r){if(r===void 0&&(r=!1),this.rowNode.isRowPinned())return e;var n=this.beans.paginationProxy.getPixelOffset(),o=r?1:-1;return e+n*o},t.prototype.setRowTop=function(e){if(!this.printLayout&&q(e)){var r=this.applyPaginationOffset(e),n=this.rowNode.isRowPinned()?r:this.beans.rowContainerHeightService.getRealPixelPosition(r),o=n+"px";this.setRowTopStyle(o)}},t.prototype.getInitialRowTop=function(){return this.initialTop},t.prototype.getInitialTransform=function(){return this.initialTransform},t.prototype.setInitialRowTop=function(){if(this.printLayout)return"";var e=this.slideRowIn?this.roundRowTopToBounds(this.rowNode.oldRowTop):this.rowNode.rowTop,r=this.applyPaginationOffset(e),n=this.rowNode.isRowPinned()?r:this.beans.rowContainerHeightService.getRealPixelPosition(r),o=n+"px",a=this.beans.gridOptionsWrapper.isSuppressRowTransform();a?this.initialTop=o:this.initialTransform="translateY("+o+")"},t.prototype.setRowTopStyle=function(e){var r=this.beans.gridOptionsWrapper.isSuppressRowTransform();this.allRowGuis.forEach(function(n){return r?n.rowComp.setTop(e):n.rowComp.setTransform("translateY("+e+")")})},t.prototype.getRowNode=function(){return this.rowNode},t.prototype.getCellCtrl=function(e){var r=null;return this.getAllCellCtrls().forEach(function(n){n.getColumn()==e&&(r=n)}),r!=null||this.getAllCellCtrls().forEach(function(n){n.getColSpanningList().indexOf(e)>=0&&(r=n)}),r},t.prototype.onRowIndexChanged=function(){this.rowNode.rowIndex!=null&&(this.onCellFocusChanged(),this.updateRowIndexes(),this.postProcessCss())},t.prototype.updateRowIndexes=function(){var e=this.rowNode.getRowIndexString(),r=this.beans.headerNavigationService.getHeaderRowCount(),n=this.rowNode.rowIndex%2===0,o=r+this.rowNode.rowIndex+1;this.allRowGuis.forEach(function(a){a.rowComp.setRowIndex(e),a.rowComp.addOrRemoveCssClass("ag-row-even",n),a.rowComp.addOrRemoveCssClass("ag-row-odd",!n),VC(a.element,o)})},t.prototype.getPinnedLeftRowElement=function(){return this.leftGui?this.leftGui.element:void 0},t.prototype.getPinnedRightRowElement=function(){return this.rightGui?this.rightGui.element:void 0},t.prototype.getBodyRowElement=function(){return this.centerGui?this.centerGui.element:void 0},t.prototype.getFullWidthRowElement=function(){return this.fullWidthGui?this.fullWidthGui.element:void 0},t.DOM_DATA_KEY_ROW_CTRL="renderedRow",t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var EG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Zr=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},RG=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},PG=function(i){EG(t,i);function t(e){var r=i.call(this)||this;return r.element=e,r}return t.prototype.postConstruct=function(){this.addMouseListeners(),this.mockContextMenuForIPad(),this.addKeyboardEvents()},t.prototype.addKeyboardEvents=function(){var e=this,r=["keydown","keypress"];r.forEach(function(n){var o=e.processKeyboardEvent.bind(e,n);e.addManagedListener(e.element,n,o)})},t.prototype.addMouseListeners=function(){var e=this,r=xC("touchstart")?"touchstart":"mousedown",n=["dblclick","contextmenu","mouseover","mouseout","click",r];n.forEach(function(o){var a=e.processMouseEvent.bind(e,o);e.addManagedListener(e.element,o,a)})},t.prototype.processMouseEvent=function(e,r){if(!(!this.mouseEventService.isEventFromThisGrid(r)||Ai(r))){var n=this.getRowForEvent(r),o=this.mouseEventService.getRenderedCellForEvent(r);e==="contextmenu"?this.handleContextMenuMouseEvent(r,null,n,o):(o&&o.onMouseEvent(e,r),n&&n.onMouseEvent(e,r))}},t.prototype.mockContextMenuForIPad=function(){var e=this;if(!!Bl()){var r=new Cr(this.element),n=function(o){var a=e.getRowForEvent(o.touchEvent),s=e.mouseEventService.getRenderedCellForEvent(o.touchEvent);e.handleContextMenuMouseEvent(null,o.touchEvent,a,s)};this.addManagedListener(r,Cr.EVENT_LONG_TAP,n),this.addDestroyFunc(function(){return r.destroy()})}},t.prototype.getRowForEvent=function(e){for(var r=e.target;r;){var n=this.gridOptionsWrapper.getDomData(r,ra.DOM_DATA_KEY_ROW_CTRL);if(n)return n;r=r.parentElement}return null},t.prototype.handleContextMenuMouseEvent=function(e,r,n,o){var a=n?n.getRowNode():null,s=o?o.getColumn():null,l=null;if(s){var u=e||r;o.dispatchCellContextMenuEvent(u),l=this.valueService.getValue(s,a)}var c=this.ctrlsService.getGridBodyCtrl(),p=o?o.getGui():c.getGridBodyElement();this.contextMenuFactory&&this.contextMenuFactory.onContextMenu(e,r,a,s,l,p)},t.prototype.processKeyboardEvent=function(e,r){var n=zh(this.gridOptionsWrapper,r,ta.DOM_DATA_KEY_CELL_CTRL),o=zh(this.gridOptionsWrapper,r,ra.DOM_DATA_KEY_ROW_CTRL);r.defaultPrevented||(n?this.processCellKeyboardEvent(n,e,r):o&&o.isFullWidth()&&this.processFullWidthRowKeyboardEvent(o,e,r))},t.prototype.processCellKeyboardEvent=function(e,r,n){var o=e.getRowNode(),a=e.getColumn(),s=e.isEditing(),l=!qh(this.gridOptionsWrapper,n,o,a,s);if(l)switch(r){case"keydown":var u=!s&&this.navigationService.handlePageScrollingKey(n);u||e.onKeyDown(n),this.doGridOperations(n,e.isEditing());break;case"keypress":e.onKeyPress(n);break}if(r==="keydown"){var c=e.createEvent(n,x.EVENT_CELL_KEY_DOWN);this.eventService.dispatchEvent(c)}if(r==="keypress"){var p=e.createEvent(n,x.EVENT_CELL_KEY_PRESS);this.eventService.dispatchEvent(p)}},t.prototype.processFullWidthRowKeyboardEvent=function(e,r,n){var o=e.getRowNode(),a=this.focusService.getFocusedCell(),s=a&&a.column,l=!qh(this.gridOptionsWrapper,n,o,s,!1);if(l){var u=n.key;if(r==="keydown")switch(u){case j.UP:case j.DOWN:e.onKeyboardNavigate(n);break;case j.TAB:e.onTabKeyDown(n)}}if(r==="keydown"){var c=e.createRowEvent(x.EVENT_CELL_KEY_DOWN,n);this.eventService.dispatchEvent(c)}if(r==="keypress"){var p=e.createRowEvent(x.EVENT_CELL_KEY_PRESS,n);this.eventService.dispatchEvent(p)}},t.prototype.doGridOperations=function(e,r){if(!(!e.ctrlKey&&!e.metaKey)&&!r&&!!this.mouseEventService.isEventFromThisGrid(e))switch(e.code){case j.A:return this.onCtrlAndA(e);case j.C:return this.onCtrlAndC(e);case j.V:return this.onCtrlAndV();case j.D:return this.onCtrlAndD(e);case j.Z:return e.shiftKey?this.undoRedoService.redo():this.undoRedoService.undo();case j.Y:return this.undoRedoService.redo()}},t.prototype.onCtrlAndA=function(e){var r=this,n=r.pinnedRowModel,o=r.paginationProxy,a=r.rangeService,s=L.PINNED_BOTTOM,l=L.PINNED_TOP;if(a&&o.isRowsToRender()){var u=RG([n.isEmpty(l),n.isEmpty(s)],2),c=u[0],p=u[1],h=c?null:l,d=void 0,f=void 0;p?(d=null,f=this.paginationProxy.getRowCount()-1):(d=s,f=n.getPinnedBottomRowData().length-1);var g=this.columnModel.getAllDisplayedColumns();if(pn(g))return;a.setCellRange({rowStartIndex:0,rowStartPinned:h,rowEndIndex:f,rowEndPinned:d,columnStart:g[0],columnEnd:rt(g)})}e.preventDefault()},t.prototype.onCtrlAndC=function(e){!this.clipboardService||this.gridOptionsWrapper.isEnableCellTextSelection()||(this.clipboardService.copyToClipboard(),e.preventDefault())},t.prototype.onCtrlAndV=function(){ge.isRegistered(oe.ClipboardModule)&&!this.gridOptionsWrapper.isSuppressClipboardPaste()&&this.clipboardService.pasteFromClipboard()},t.prototype.onCtrlAndD=function(e){ge.isRegistered(oe.ClipboardModule)&&!this.gridOptionsWrapper.isSuppressClipboardPaste()&&this.clipboardService.copyRangeDown(),e.preventDefault()},Zr([y("mouseEventService")],t.prototype,"mouseEventService",void 0),Zr([y("valueService")],t.prototype,"valueService",void 0),Zr([De("contextMenuFactory")],t.prototype,"contextMenuFactory",void 0),Zr([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Zr([y("navigationService")],t.prototype,"navigationService",void 0),Zr([y("focusService")],t.prototype,"focusService",void 0),Zr([y("undoRedoService")],t.prototype,"undoRedoService",void 0),Zr([y("columnModel")],t.prototype,"columnModel",void 0),Zr([y("paginationProxy")],t.prototype,"paginationProxy",void 0),Zr([y("pinnedRowModel")],t.prototype,"pinnedRowModel",void 0),Zr([De("rangeService")],t.prototype,"rangeService",void 0),Zr([De("clipboardService")],t.prototype,"clipboardService",void 0),Zr([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var AG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ms=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},DG=function(i){AG(t,i);function t(e){var r=i.call(this)||this;return r.centerContainerCtrl=e,r}return t.prototype.postConstruct=function(){var e=this;this.ctrlsService.whenReady(function(){e.gridBodyCtrl=e.ctrlsService.getGridBodyCtrl(),e.listenForResize()}),this.addManagedListener(this.eventService,x.EVENT_SCROLLBAR_WIDTH_CHANGED,this.onScrollbarWidthChanged.bind(this))},t.prototype.listenForResize=function(){var e=this,r=function(){return e.onCenterViewportResized()};this.centerContainerCtrl.registerViewportResizeListener(r),this.gridBodyCtrl.registerBodyViewportResizeListener(r)},t.prototype.onScrollbarWidthChanged=function(){this.checkViewportAndScrolls()},t.prototype.onCenterViewportResized=function(){if(this.centerContainerCtrl.isViewportVisible()){this.checkViewportAndScrolls();var e=this.centerContainerCtrl.getCenterWidth();e!==this.centerWidth&&(this.centerWidth=e,this.columnModel.refreshFlexedColumns({viewportWidth:this.centerWidth,updateBodyWidths:!0,fireResizedEvent:!0}))}else this.bodyHeight=0},t.prototype.checkViewportAndScrolls=function(){this.updateScrollVisibleService(),this.checkBodyHeight(),this.onHorizontalViewportChanged(),this.gridBodyCtrl.getScrollFeature().checkScrollLeft()},t.prototype.getBodyHeight=function(){return this.bodyHeight},t.prototype.checkBodyHeight=function(){var e=this.gridBodyCtrl.getBodyViewportElement(),r=Uc(e);if(this.bodyHeight!==r){this.bodyHeight=r;var n={type:x.EVENT_BODY_HEIGHT_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(n)}},t.prototype.updateScrollVisibleService=function(){this.updateScrollVisibleServiceImpl(),setTimeout(this.updateScrollVisibleServiceImpl.bind(this),500)},t.prototype.updateScrollVisibleServiceImpl=function(){var e={horizontalScrollShowing:this.isHorizontalScrollShowing(),verticalScrollShowing:this.gridBodyCtrl.isVerticalScrollShowing()};this.scrollVisibleService.setScrollsVisible(e),this.gridBodyCtrl.setVerticalScrollPaddingVisible(e.verticalScrollShowing)},t.prototype.isHorizontalScrollShowing=function(){var e=this.gridOptionsWrapper.isAlwaysShowHorizontalScroll();return e||this.centerContainerCtrl.isViewportHScrollShowing()},t.prototype.onHorizontalViewportChanged=function(){var e=this.centerContainerCtrl.getCenterWidth(),r=this.centerContainerCtrl.getViewportScrollLeft();this.columnModel.setViewportPosition(e,r)},Ms([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Ms([y("columnModel")],t.prototype,"columnModel",void 0),Ms([y("scrollVisibleService")],t.prototype,"scrollVisibleService",void 0),Ms([y("columnApi")],t.prototype,"columnApi",void 0),Ms([y("gridApi")],t.prototype,"gridApi",void 0),Ms([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var MG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),w_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},FG=function(i){MG(t,i);function t(e){var r=i.call(this)||this;return r.element=e,r}return t.prototype.postConstruct=function(){this.addManagedListener(this.eventService,x.EVENT_LEFT_PINNED_WIDTH_CHANGED,this.onPinnedLeftWidthChanged.bind(this))},t.prototype.onPinnedLeftWidthChanged=function(){var e=this.pinnedWidthService.getPinnedLeftWidth(),r=e>0;pt(this.element,r),r&&Li(this.element,e)},w_([y("pinnedWidthService")],t.prototype,"pinnedWidthService",void 0),w_([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var IG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),b_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},LG=function(i){IG(t,i);function t(e){var r=i.call(this)||this;return r.element=e,r}return t.prototype.postConstruct=function(){this.addManagedListener(this.eventService,x.EVENT_RIGHT_PINNED_WIDTH_CHANGED,this.onPinnedRightWidthChanged.bind(this))},t.prototype.onPinnedRightWidthChanged=function(){var e=this.pinnedWidthService.getPinnedRightWidth(),r=e>0;pt(this.element,r),r&&Li(this.element,e)},b_([y("pinnedWidthService")],t.prototype,"pinnedWidthService",void 0),b_([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var NG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),__=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},GG=function(i){NG(t,i);function t(e,r){var n=i.call(this)||this;return n.eContainer=e,n.eWrapper=r,n}return t.prototype.postConstruct=function(){this.addManagedListener(this.eventService,x.EVENT_ROW_CONTAINER_HEIGHT_CHANGED,this.onHeightChanged.bind(this))},t.prototype.onHeightChanged=function(){var e=this.maxDivHeightScaler.getUiContainerHeight(),r=e!=null?e+"px":"";this.eContainer.style.height=r,this.eWrapper&&(this.eWrapper.style.height=r)},__([y("rowContainerHeightService")],t.prototype,"maxDivHeightScaler",void 0),__([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var kG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Og=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},WG=function(i){kG(t,i);function t(e){var r=i.call(this)||this;return r.eContainer=e,r}return t.prototype.postConstruct=function(){var e=this;if(!(!this.gridOptionsWrapper.isEnableRangeSelection()||me(this.rangeService))){var r={dragStartPixels:0,eElement:this.eContainer,onDragStart:this.rangeService.onDragStart.bind(this.rangeService),onDragStop:this.rangeService.onDragStop.bind(this.rangeService),onDragging:this.rangeService.onDragging.bind(this.rangeService)};this.dragService.addDragSource(r),this.addDestroyFunc(function(){return e.dragService.removeDragSource(r)})}},Og([De("rangeService")],t.prototype,"rangeService",void 0),Og([y("dragService")],t.prototype,"dragService",void 0),Og([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var VG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),S_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ew=function(i){VG(t,i);function t(e){var r=i.call(this)||this;return r.callback=e,r}return t.prototype.postConstruct=function(){var e=this.setWidth.bind(this);this.addManagedListener(this.gridOptionsWrapper,ce.PROP_DOM_LAYOUT,e),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,e),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,e),this.setWidth()},t.prototype.setWidth=function(){var e=this.columnModel,r=this.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_PRINT,n=e.getBodyContainerWidth(),o=e.getDisplayedColumnsLeftWidth(),a=e.getDisplayedColumnsRightWidth(),s=r?n+o+a:n;this.callback(s)},S_([y("columnModel")],t.prototype,"columnModel",void 0),S_([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var BG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),wa=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},HG=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},jG=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(HG(arguments[t]));return i},re;(function(i){i.LEFT="left",i.RIGHT="right",i.CENTER="center",i.FULL_WIDTH="fullWidth",i.TOP_LEFT="topLeft",i.TOP_RIGHT="topRight",i.TOP_CENTER="topCenter",i.TOP_FULL_WIDTH="topFullWidth",i.BOTTOM_LEFT="bottomLeft",i.BOTTOM_RIGHT="bottomRight",i.BOTTOM_CENTER="bottomCenter",i.BOTTOM_FULL_WIDTH="bottomFullWidth"})(re||(re={}));var Hr;(function(i){i.LEFT="left",i.RIGHT="right",i.CENTER="center",i.FULL_WIDTH="fullWidth"})(Hr||(Hr={}));function FE(i){switch(i){case re.CENTER:case re.TOP_CENTER:case re.BOTTOM_CENTER:return Hr.CENTER;case re.LEFT:case re.TOP_LEFT:case re.BOTTOM_LEFT:return Hr.LEFT;case re.RIGHT:case re.TOP_RIGHT:case re.BOTTOM_RIGHT:return Hr.RIGHT;case re.FULL_WIDTH:case re.TOP_FULL_WIDTH:case re.BOTTOM_FULL_WIDTH:return Hr.FULL_WIDTH;default:throw Error("Invalid Row Container Type")}}var UG=zo([[re.CENTER,"ag-center-cols-container"],[re.LEFT,"ag-pinned-left-cols-container"],[re.RIGHT,"ag-pinned-right-cols-container"],[re.FULL_WIDTH,"ag-full-width-container"],[re.TOP_CENTER,"ag-floating-top-container"],[re.TOP_LEFT,"ag-pinned-left-floating-top"],[re.TOP_RIGHT,"ag-pinned-right-floating-top"],[re.TOP_FULL_WIDTH,"ag-floating-top-full-width-container"],[re.BOTTOM_CENTER,"ag-floating-bottom-container"],[re.BOTTOM_LEFT,"ag-pinned-left-floating-bottom"],[re.BOTTOM_RIGHT,"ag-pinned-right-floating-bottom"],[re.BOTTOM_FULL_WIDTH,"ag-floating-bottom-full-width-container"]]),zG=zo([[re.CENTER,"ag-center-cols-viewport"],[re.TOP_CENTER,"ag-floating-top-viewport"],[re.BOTTOM_CENTER,"ag-floating-bottom-viewport"]]),$G=zo([[re.CENTER,"ag-center-cols-clipper"]]),id=function(i){BG(t,i);function t(e){var r=i.call(this)||this;return r.name=e,r}return t.getRowContainerCssClasses=function(e){var r=UG.get(e),n=zG.get(e),o=$G.get(e);return{container:r,viewport:n,wrapper:o}},t.getPinned=function(e){switch(e){case re.BOTTOM_LEFT:case re.TOP_LEFT:case re.LEFT:return L.PINNED_LEFT;case re.BOTTOM_RIGHT:case re.TOP_RIGHT:case re.RIGHT:return L.PINNED_RIGHT;default:return null}},t.prototype.postConstruct=function(){var e=this;this.enableRtl=this.gridOptionsWrapper.isEnableRtl(),this.embedFullWidthRows=this.gridOptionsWrapper.isEmbedFullWidthRows(),this.forContainers([re.CENTER],function(){return e.viewportSizeFeature=e.createManagedBean(new DG(e))})},t.prototype.registerWithCtrlsService=function(){switch(this.name){case re.CENTER:this.ctrlsService.registerCenterRowContainerCtrl(this);break;case re.LEFT:this.ctrlsService.registerLeftRowContainerCtrl(this);break;case re.RIGHT:this.ctrlsService.registerRightRowContainerCtrl(this);break;case re.TOP_CENTER:this.ctrlsService.registerTopCenterRowContainerCtrl(this);break;case re.TOP_LEFT:this.ctrlsService.registerTopLeftRowContainerCon(this);break;case re.TOP_RIGHT:this.ctrlsService.registerTopRightRowContainerCtrl(this);break;case re.BOTTOM_CENTER:this.ctrlsService.registerBottomCenterRowContainerCtrl(this);break;case re.BOTTOM_LEFT:this.ctrlsService.registerBottomLeftRowContainerCtrl(this);break;case re.BOTTOM_RIGHT:this.ctrlsService.registerBottomRightRowContainerCtrl(this);break}},t.prototype.forContainers=function(e,r){e.indexOf(this.name)>=0&&r()},t.prototype.getContainerElement=function(){return this.eContainer},t.prototype.getViewportSizeFeature=function(){return this.viewportSizeFeature},t.prototype.setComp=function(e,r,n,o){var a=this;this.comp=e,this.eContainer=r,this.eViewport=n,this.eWrapper=o,this.createManagedBean(new PG(this.eContainer)),this.addPreventScrollWhileDragging(),this.listenOnDomOrder(),this.stopHScrollOnPinnedRows();var s=[re.TOP_CENTER,re.TOP_LEFT,re.TOP_RIGHT],l=[re.BOTTOM_CENTER,re.BOTTOM_LEFT,re.BOTTOM_RIGHT],u=[re.CENTER,re.LEFT,re.RIGHT],c=jG(s,l,u),p=[re.CENTER,re.LEFT,re.RIGHT,re.FULL_WIDTH],h=[re.CENTER,re.TOP_CENTER,re.BOTTOM_CENTER],d=[re.LEFT,re.BOTTOM_LEFT,re.TOP_LEFT],f=[re.RIGHT,re.BOTTOM_RIGHT,re.TOP_RIGHT];this.forContainers(d,function(){return a.createManagedBean(new FG(a.eContainer))}),this.forContainers(f,function(){return a.createManagedBean(new LG(a.eContainer))}),this.forContainers(p,function(){return a.createManagedBean(new GG(a.eContainer,a.eWrapper))}),this.forContainers(c,function(){return a.createManagedBean(new WG(a.eContainer))}),this.forContainers(h,function(){return a.createManagedBean(new ew(function(g){return a.comp.setContainerWidth(g+"px")}))}),this.addListeners(),this.registerWithCtrlsService()},t.prototype.addListeners=function(){var e=this;this.addManagedListener(this.eventService,x.EVENT_SCROLL_VISIBILITY_CHANGED,function(){return e.onScrollVisibilityChanged()}),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,function(){return e.onDisplayedColumnsChanged()}),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,function(){return e.onDisplayedColumnsWidthChanged()}),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_ROWS_CHANGED,function(){return e.onDisplayedRowsChanged()}),this.onScrollVisibilityChanged(),this.onDisplayedColumnsChanged(),this.onDisplayedColumnsWidthChanged(),this.onDisplayedRowsChanged()},t.prototype.listenOnDomOrder=function(){var e=this,r=function(){return e.comp.setDomOrder(e.gridOptionsWrapper.isEnsureDomOrder())};this.addManagedListener(this.gridOptionsWrapper,ce.PROP_DOM_LAYOUT,r),r()},t.prototype.stopHScrollOnPinnedRows=function(){var e=this;this.forContainers([re.TOP_CENTER,re.BOTTOM_CENTER],function(){var r=function(){return e.eViewport.scrollLeft=0};e.addManagedListener(e.eViewport,"scroll",r)})},t.prototype.onDisplayedColumnsChanged=function(){var e=this;this.forContainers([re.CENTER],function(){return e.onHorizontalViewportChanged()})},t.prototype.onDisplayedColumnsWidthChanged=function(){var e=this;this.forContainers([re.CENTER],function(){return e.onHorizontalViewportChanged()})},t.prototype.onScrollVisibilityChanged=function(){if(this.name===re.CENTER){var e=this.scrollVisibleService.isHorizontalScrollShowing(),r=e&&this.gridOptionsWrapper.getScrollbarWidth()||0,n=r==0?"100%":"calc(100% + "+r+"px)";this.comp.setViewportHeight(n)}},t.prototype.addPreventScrollWhileDragging=function(){var e=this,r=function(n){e.dragService.isDragging()&&n.cancelable&&n.preventDefault()};this.eContainer.addEventListener("touchmove",r,{passive:!1}),this.addDestroyFunc(function(){return e.eContainer.removeEventListener("touchmove",r)})},t.prototype.onHorizontalViewportChanged=function(){var e=this.getCenterWidth(),r=this.getCenterViewportScrollLeft();this.columnModel.setViewportPosition(e,r)},t.prototype.getCenterWidth=function(){return tf(this.eViewport)},t.prototype.getCenterViewportScrollLeft=function(){return Tc(this.eViewport,this.enableRtl)},t.prototype.registerViewportResizeListener=function(e){var r=this.resizeObserverService.observeResize(this.eViewport,e);this.addDestroyFunc(function(){return r()})},t.prototype.isViewportVisible=function(){return jl(this.eViewport)},t.prototype.isViewportHScrollShowing=function(){return Xm(this.eViewport)},t.prototype.getViewportScrollLeft=function(){return Tc(this.eViewport,this.enableRtl)},t.prototype.isHorizontalScrollShowing=function(){var e=this.gridOptionsWrapper.isAlwaysShowHorizontalScroll();return e||Xm(this.eViewport)},t.prototype.getViewportElement=function(){return this.eViewport},t.prototype.setContainerTranslateX=function(e){this.eContainer.style.transform="translateX("+e+"px)"},t.prototype.getHScrollPosition=function(){var e={left:this.eViewport.scrollLeft,right:this.eViewport.scrollLeft+this.eViewport.offsetWidth};return e},t.prototype.setCenterViewportScrollLeft=function(e){Yh(this.eViewport,e,this.enableRtl)},t.prototype.onDisplayedRowsChanged=function(){var e=this,r=this.name===re.TOP_FULL_WIDTH||this.name===re.BOTTOM_FULL_WIDTH||this.name===re.FULL_WIDTH,n=function(s){var l=s.isFullWidth(),u=e.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_PRINT,c=e.embedFullWidthRows||u,p=r?!c&&l:c||!l;return p},o=this.getRowCtrls(),a=o.filter(n);this.comp.setRowCtrls(a)},t.prototype.getRowCtrls=function(){switch(this.name){case re.TOP_CENTER:case re.TOP_LEFT:case re.TOP_RIGHT:case re.TOP_FULL_WIDTH:return this.rowRenderer.getTopRowCtrls();case re.BOTTOM_CENTER:case re.BOTTOM_LEFT:case re.BOTTOM_RIGHT:case re.BOTTOM_FULL_WIDTH:return this.rowRenderer.getBottomRowCtrls();default:return this.rowRenderer.getRowCtrls()}},wa([y("scrollVisibleService")],t.prototype,"scrollVisibleService",void 0),wa([y("dragService")],t.prototype,"dragService",void 0),wa([y("ctrlsService")],t.prototype,"ctrlsService",void 0),wa([y("columnModel")],t.prototype,"columnModel",void 0),wa([y("resizeObserverService")],t.prototype,"resizeObserverService",void 0),wa([y("rowRenderer")],t.prototype,"rowRenderer",void 0),wa([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var KG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ba=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},YG=`<div class="ag-root ag-unselectable" role="grid">
        <ag-header-root ref="gridHeader"></ag-header-root>
        <div class="ag-floating-top" ref="eTop" role="presentation">
            <ag-row-container ref="topLeftContainer" name="`+re.TOP_LEFT+`"></ag-row-container>
            <ag-row-container ref="topCenterContainer" name="`+re.TOP_CENTER+`"></ag-row-container>
            <ag-row-container ref="topRightContainer" name="`+re.TOP_RIGHT+`"></ag-row-container>
            <ag-row-container ref="topFullWidthContainer" name="`+re.TOP_FULL_WIDTH+`"></ag-row-container>
        </div>
        <div class="ag-body-viewport" ref="eBodyViewport" role="presentation">
            <ag-row-container ref="leftContainer" name="`+re.LEFT+`"></ag-row-container>
            <ag-row-container ref="centerContainer" name="`+re.CENTER+`"></ag-row-container>
            <ag-row-container ref="rightContainer" name="`+re.RIGHT+`"></ag-row-container>
            <ag-row-container ref="fullWidthContainer" name="`+re.FULL_WIDTH+`"></ag-row-container>
        </div>
        <div class="ag-floating-bottom" ref="eBottom" role="presentation">
            <ag-row-container ref="bottomLeftContainer" name="`+re.BOTTOM_LEFT+`"></ag-row-container>
            <ag-row-container ref="bottomCenterContainer" name="`+re.BOTTOM_CENTER+`"></ag-row-container>
            <ag-row-container ref="bottomRightContainer" name="`+re.BOTTOM_RIGHT+`"></ag-row-container>
            <ag-row-container ref="bottomFullWidthContainer" name="`+re.BOTTOM_FULL_WIDTH+`"></ag-row-container>
        </div>
        <ag-fake-horizontal-scroll></ag-fake-horizontal-scroll>
        <ag-overlay-wrapper></ag-overlay-wrapper>
    </div>`,IE=function(i){KG(t,i);function t(){return i.call(this,YG)||this}return t.prototype.init=function(){var e=this,r=function(o,a){var s=o+"px";a.style.minHeight=s,a.style.height=s},n={setRowAnimationCssOnBodyViewport:function(o,a){return e.setRowAnimationCssOnBodyViewport(o,a)},setColumnCount:function(o){return YO(e.getGui(),o)},setRowCount:function(o){return KO(e.getGui(),o)},setTopHeight:function(o){return r(o,e.eTop)},setBottomHeight:function(o){return r(o,e.eBottom)},setTopDisplay:function(o){return e.eTop.style.display=o},setBottomDisplay:function(o){return e.eBottom.style.display=o},setColumnMovingCss:function(o,a){return e.addOrRemoveCssClass(DE,a)},updateLayoutClasses:function(o,a){var s=e.eBodyViewport.classList;s.toggle(Rt.AUTO_HEIGHT,a.autoHeight),s.toggle(Rt.NORMAL,a.normal),s.toggle(Rt.PRINT,a.print),e.addOrRemoveCssClass(Rt.AUTO_HEIGHT,a.autoHeight),e.addOrRemoveCssClass(Rt.NORMAL,a.normal),e.addOrRemoveCssClass(Rt.PRINT,a.print)},setAlwaysVerticalScrollClass:function(o,a){return e.eBodyViewport.classList.toggle(AE,a)},registerBodyViewportResizeListener:function(o){var a=e.resizeObserverService.observeResize(e.eBodyViewport,o);e.addDestroyFunc(function(){return a()})},setPinnedTopBottomOverflowY:function(o){return e.eTop.style.overflowY=e.eBottom.style.overflowY=o},setCellSelectableCss:function(o,a){[e.eTop,e.eBodyViewport,e.eBottom].forEach(function(s){return s.classList.toggle(PE,a)})}};this.ctrl=this.createManagedBean(new ME),this.ctrl.setComp(n,this.getGui(),this.eBodyViewport,this.eTop,this.eBottom),(this.rangeService||this.gridOptionsWrapper.isRowSelectionMulti())&&$O(this.getGui(),!0)},t.prototype.setRowAnimationCssOnBodyViewport=function(e,r){var n=this.eBodyViewport.classList;n.toggle(cs.ANIMATION_ON,r),n.toggle(cs.ANIMATION_OFF,!r)},t.prototype.getFloatingTopBottom=function(){return[this.eTop,this.eBottom]},ba([y("resizeObserverService")],t.prototype,"resizeObserverService",void 0),ba([De("rangeService")],t.prototype,"rangeService",void 0),ba([I("eBodyViewport")],t.prototype,"eBodyViewport",void 0),ba([I("eTop")],t.prototype,"eTop",void 0),ba([I("eBottom")],t.prototype,"eBottom",void 0),ba([I("gridHeader")],t.prototype,"headerRootComp",void 0),ba([B],t.prototype,"init",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var XG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),cu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},LE=function(i){XG(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,this.onDisplayedColumnsChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,this.onDisplayedColumnsWidthChanged.bind(this))},t.prototype.onDisplayedColumnsChanged=function(){this.update()},t.prototype.onDisplayedColumnsWidthChanged=function(){this.update()},t.prototype.update=function(){this.updateImpl(),setTimeout(this.updateImpl.bind(this),500)},t.prototype.updateImpl=function(){var e=this.ctrlsService.getCenterRowContainerCtrl();if(!!e){var r={horizontalScrollShowing:e.isHorizontalScrollShowing(),verticalScrollShowing:this.isVerticalScrollShowing()};this.setScrollsVisible(r)}},t.prototype.setScrollsVisible=function(e){var r=this.horizontalScrollShowing!==e.horizontalScrollShowing||this.verticalScrollShowing!==e.verticalScrollShowing;if(r){this.horizontalScrollShowing=e.horizontalScrollShowing,this.verticalScrollShowing=e.verticalScrollShowing;var n={type:x.EVENT_SCROLL_VISIBILITY_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(n)}},t.prototype.isHorizontalScrollShowing=function(){return this.horizontalScrollShowing},t.prototype.isVerticalScrollShowing=function(){return this.verticalScrollShowing},cu([y("columnApi")],t.prototype,"columnApi",void 0),cu([y("gridApi")],t.prototype,"gridApi",void 0),cu([y("ctrlsService")],t.prototype,"ctrlsService",void 0),cu([B],t.prototype,"postConstruct",null),t=cu([Q("scrollVisibleService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var qG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),x_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},NE=function(i){qG(t,i);function t(){var r=i!==null&&i.apply(this,arguments)||this;return r.gridInstanceId=e.gridInstanceSequence.next(),r}e=t,t.prototype.stampTopLevelGridCompWithGridInstance=function(r){r[e.GRID_DOM_KEY]=this.gridInstanceId},t.prototype.getRenderedCellForEvent=function(r){return zh(this.gridOptionsWrapper,r,ta.DOM_DATA_KEY_CELL_CTRL)},t.prototype.isEventFromThisGrid=function(r){var n=this.isElementInThisGrid(r.target);return n},t.prototype.isElementInThisGrid=function(r){for(var n=r;n;){var o=n[e.GRID_DOM_KEY];if(q(o)){var a=o===this.gridInstanceId;return a}n=n.parentElement}return!1},t.prototype.getCellPositionForEvent=function(r){var n=this.getRenderedCellForEvent(r);return n?n.getCellPosition():null},t.prototype.getNormalisedPosition=function(r){var n=this.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_NORMAL,o=r,a,s;if(o.clientX!=null||o.clientY!=null?(a=o.clientX,s=o.clientY):(a=o.x,s=o.y),n){var l=this.ctrlsService.getGridBodyCtrl(),u=l.getScrollFeature().getVScrollPosition(),c=l.getScrollFeature().getHScrollPosition();a+=c.left,s+=u.top}return{x:a,y:s}};var e;return t.gridInstanceSequence=new Bt,t.GRID_DOM_KEY="__ag_grid_instance",x_([y("ctrlsService")],t.prototype,"ctrlsService",void 0),t=e=x_([Q("mouseEventService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ZG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),od=globalThis&&globalThis.__assign||function(){return od=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},od.apply(this,arguments)},hr=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},GE=function(i){ZG(t,i);function t(){var e=i.call(this)||this;return e.onPageDown=Wm(e.onPageDown,100),e.onPageUp=Wm(e.onPageUp,100),e}return t.prototype.postConstruct=function(){var e=this;this.ctrlsService.whenReady(function(r){e.gridBodyCon=r.gridBodyCtrl})},t.prototype.handlePageScrollingKey=function(e){var r=e.key,n=e.altKey,o=e.ctrlKey||e.metaKey,a=!!this.rangeService&&e.shiftKey,s=this.mouseEventService.getCellPositionForEvent(e);if(!s)return!1;var l=!1;switch(r){case j.PAGE_HOME:case j.PAGE_END:!o&&!n&&(this.onHomeOrEndKey(r),l=!0);break;case j.LEFT:case j.RIGHT:case j.UP:case j.DOWN:o&&!n&&!a&&(this.onCtrlUpDownLeftRight(r,s),l=!0);break;case j.PAGE_DOWN:!o&&!n&&(this.onPageDown(s),l=!0);break;case j.PAGE_UP:!o&&!n&&(this.onPageUp(s),l=!0);break}return l&&e.preventDefault(),l},t.prototype.navigateTo=function(e){var r=e.scrollIndex,n=e.scrollType,o=e.scrollColumn,a=e.focusIndex,s=e.focusColumn;if(q(o)&&!o.isPinned()&&this.gridBodyCon.getScrollFeature().ensureColumnVisible(o),q(r)&&this.gridBodyCon.getScrollFeature().ensureIndexVisible(r,n),this.animationFrameService.flushAllFrames(),this.focusService.setFocusedCell(a,s,null,!0),this.rangeService){var l={rowIndex:a,rowPinned:null,column:s};this.rangeService.setRangeToCell(l)}},t.prototype.onPageDown=function(e){var r=this.ctrlsService.getGridBodyCtrl(),n=r.getScrollFeature().getVScrollPosition(),o=this.getViewportHeight(),a=this.paginationProxy.getPixelOffset(),s=n.top+o,l=this.paginationProxy.getRowIndexAtPixel(s+a);this.columnModel.isAutoRowHeightActive()?this.navigateToNextPageWithAutoHeight(e,l):this.navigateToNextPage(e,l)},t.prototype.onPageUp=function(e){var r=this.ctrlsService.getGridBodyCtrl(),n=r.getScrollFeature().getVScrollPosition(),o=this.paginationProxy.getPixelOffset(),a=n.top,s=this.paginationProxy.getRowIndexAtPixel(a+o);this.columnModel.isAutoRowHeightActive()?this.navigateToNextPageWithAutoHeight(e,s,!0):this.navigateToNextPage(e,s,!0)},t.prototype.navigateToNextPage=function(e,r,n){n===void 0&&(n=!1);var o,a,s=this.getViewportHeight(),l=this.paginationProxy.getPageFirstRow(),u=this.paginationProxy.getPageLastRow(),c=this.paginationProxy.getPixelOffset(),p=this.paginationProxy.getRow(e.rowIndex),h=n?((o=p)===null||o===void 0?void 0:o.rowHeight)-s-c:s-c,d=((a=p)===null||a===void 0?void 0:a.rowTop)+h,f=this.paginationProxy.getRowIndexAtPixel(d+c);if(f===e.rowIndex){var g=n?-1:1;r=f=e.rowIndex+g}var v;n?(v="bottom",f<l&&(f=l),r<l&&(r=l)):(v="top",f>u&&(f=u),r>u&&(r=u)),this.isRowTallerThanView(f)&&(r=f,v="top"),this.navigateTo({scrollIndex:r,scrollType:v,scrollColumn:null,focusIndex:f,focusColumn:e.column})},t.prototype.navigateToNextPageWithAutoHeight=function(e,r,n){var o=this;n===void 0&&(n=!1),this.navigateTo({scrollIndex:r,scrollType:n?"bottom":"top",scrollColumn:null,focusIndex:r,focusColumn:e.column}),setTimeout(function(){var a=o.getNextFocusIndexForAutoHeight(e,n);o.navigateTo({scrollIndex:r,scrollType:n?"bottom":"top",scrollColumn:null,focusIndex:a,focusColumn:e.column})},50)},t.prototype.getNextFocusIndexForAutoHeight=function(e,r){r===void 0&&(r=!1);for(var n,o=r?-1:1,a=this.getViewportHeight(),s=this.paginationProxy.getPageLastRow(),l=0,u=e.rowIndex;u>=0&&u<=s;){var c=this.paginationProxy.getRow(u);if(c){var p=(n=c.rowHeight,n!=null?n:0);if(l+p>a)break;l+=p}u+=o}return Math.max(0,Math.min(u,s))},t.prototype.getViewportHeight=function(){var e=this.ctrlsService.getGridBodyCtrl(),r=e.getScrollFeature().getVScrollPosition(),n=this.gridOptionsWrapper.getScrollbarWidth(),o=r.bottom-r.top;return this.ctrlsService.getCenterRowContainerCtrl().isHorizontalScrollShowing()&&(o-=n),o},t.prototype.isRowTallerThanView=function(e){var r=this.paginationProxy.getRow(e);if(!r)return!1;var n=r.rowHeight;return typeof n!="number"?!1:n>this.getViewportHeight()},t.prototype.onCtrlUpDownLeftRight=function(e,r){var n=this.cellNavigationService.getNextCellToFocus(e,r,!0),o=n.rowIndex,a=n.column;this.navigateTo({scrollIndex:o,scrollType:null,scrollColumn:a,focusIndex:o,focusColumn:a})},t.prototype.onHomeOrEndKey=function(e){var r=e===j.PAGE_HOME,n=this.columnModel.getAllDisplayedColumns(),o=r?n[0]:rt(n),a=r?this.paginationProxy.getPageFirstRow():this.paginationProxy.getPageLastRow();this.navigateTo({scrollIndex:a,scrollType:null,scrollColumn:o,focusIndex:a,focusColumn:o})},t.prototype.onTabKeyDown=function(e,r){var n=r.shiftKey,o=this.tabToNextCellCommon(e,n,r);if(o){r.preventDefault();return}if(n){var a=e.getRowPosition(),s=a.rowIndex,l=a.rowPinned,u=l?s===0:s===this.paginationProxy.getPageFirstRow();u&&(r.preventDefault(),this.focusService.focusLastHeader(r))}else e instanceof ta&&e.focusCell(!0),this.focusService.focusNextGridCoreContainer(n)&&r.preventDefault()},t.prototype.tabToNextCell=function(e,r){var n=this.focusService.getFocusedCell();if(!n)return!1;var o=this.getCellByPosition(n);return!o&&(o=this.rowRenderer.getRowByPosition(n),!o||!o.isFullWidth())?!1:this.tabToNextCellCommon(o,e,r)},t.prototype.tabToNextCellCommon=function(e,r,n){var o=e.isEditing();if(!o&&e instanceof ta){var a=e,s=a.getRowCtrl();s&&(o=s.isEditing())}var l;return o?this.gridOptionsWrapper.isFullRowEdit()?l=this.moveToNextEditingRow(e,r,n):l=this.moveToNextEditingCell(e,r,n):l=this.moveToNextCellNotEditing(e,r),l||!!this.focusService.getFocusedHeader()},t.prototype.moveToNextEditingCell=function(e,r,n){n===void 0&&(n=null);var o=e.getCellPosition();e.stopEditing();var a=this.findNextCellToFocusOn(o,r,!0);return a==null?!1:(a.startEditing(null,null,!0,n),a.focusCell(!1),!0)},t.prototype.moveToNextEditingRow=function(e,r,n){n===void 0&&(n=null);var o=e.getCellPosition(),a=this.findNextCellToFocusOn(o,r,!0);if(a==null)return!1;var s=a.getCellPosition(),l=this.isCellEditable(o),u=this.isCellEditable(s),c=s&&o.rowIndex===s.rowIndex&&o.rowPinned===s.rowPinned;if(l&&e.setFocusOutOnEditor(),!c){var p=e.getRowCtrl();p.stopEditing();var h=a.getRowCtrl();h.startRowEditing(void 0,void 0,void 0,n)}return u?(a.setFocusInOnEditor(),a.focusCell()):a.focusCell(!0),!0},t.prototype.moveToNextCellNotEditing=function(e,r){var n=this.columnModel.getAllDisplayedColumns(),o;e instanceof ra?o=od(od({},e.getRowPosition()),{column:r?n[0]:rt(n)}):o=e.getCellPosition();var a=this.findNextCellToFocusOn(o,r,!1);if(a instanceof ta)a.focusCell(!0);else if(a)return this.tryToFocusFullWidthRow(a.getRowPosition(),r);return q(a)},t.prototype.findNextCellToFocusOn=function(e,r,n){for(var o=e;;){e!==o&&(e=o),r||(o=this.getLastCellOfColSpan(o)),o=this.cellNavigationService.getNextTabbedCell(o,r);var a=this.gridOptionsWrapper.getTabToNextCellFunc();if(q(a)){var s={backwards:r,editing:n,previousCellPosition:e,nextCellPosition:o||null},l=a(s);q(l)?(l.floating&&(Ye(function(){console.warn("AG Grid: tabToNextCellFunc return type should have attributes: rowIndex, rowPinned, column. However you had 'floating', maybe you meant 'rowPinned'?")},"no floating in userCell"),l.rowPinned=l.floating),o={rowIndex:l.rowIndex,column:l.column,rowPinned:l.rowPinned}):o=null}if(!o)return null;if(o.rowIndex<0){var u=this.headerNavigationService.getHeaderRowCount();return this.focusService.focusHeaderPosition({headerPosition:{headerRowIndex:u+o.rowIndex,column:o.column}}),null}var c=this.gridOptionsWrapper.isFullRowEdit();if(n&&!c){var p=this.isCellEditable(o);if(!p)continue}this.ensureCellVisible(o);var h=this.getCellByPosition(o);if(!h){var d=this.rowRenderer.getRowByPosition(o);if(!d||!d.isFullWidth())continue;return d}if(!h.isSuppressNavigable())return this.rangeService&&this.rangeService.setRangeToCell(o),h}},t.prototype.isCellEditable=function(e){var r=this.lookupRowNodeForCell(e);return r?e.column.isCellEditable(r):!1},t.prototype.getCellByPosition=function(e){var r=this.rowRenderer.getRowByPosition(e);return r?r.getCellCtrl(e.column):null},t.prototype.lookupRowNodeForCell=function(e){return e.rowPinned===L.PINNED_TOP?this.pinnedRowModel.getPinnedTopRow(e.rowIndex):e.rowPinned===L.PINNED_BOTTOM?this.pinnedRowModel.getPinnedBottomRow(e.rowIndex):this.paginationProxy.getRow(e.rowIndex)},t.prototype.navigateToNextCell=function(e,r,n,o){for(var a=n,s=!1;a&&(a===n||!this.isValidNavigateCell(a));)this.gridOptionsWrapper.isEnableRtl()?r===j.LEFT&&(a=this.getLastCellOfColSpan(a)):r===j.RIGHT&&(a=this.getLastCellOfColSpan(a)),a=this.cellNavigationService.getNextCellToFocus(r,a),s=me(a);if(s&&e&&e.key===j.UP&&(a={rowIndex:-1,rowPinned:null,column:n.column}),o){var l=this.gridOptionsWrapper.getNavigateToNextCellFunc();if(q(l)){var u={key:r,previousCellPosition:n,nextCellPosition:a||null,event:e},c=l(u);q(c)?(c.floating&&(Ye(function(){console.warn("AG Grid: tabToNextCellFunc return type should have attributes: rowIndex, rowPinned, column. However you had 'floating', maybe you meant 'rowPinned'?")},"no floating in userCell"),c.rowPinned=c.floating),a={rowPinned:c.rowPinned,rowIndex:c.rowIndex,column:c.column}):a=null}}if(!!a){if(a.rowIndex<0){var p=this.headerNavigationService.getHeaderRowCount();this.focusService.focusHeaderPosition({headerPosition:{headerRowIndex:p+a.rowIndex,column:n.column},event:e||void 0});return}var h=this.getNormalisedPosition(a);h?this.focusPosition(h):this.tryToFocusFullWidthRow(a)}},t.prototype.getNormalisedPosition=function(e){this.ensureCellVisible(e);var r=this.getCellByPosition(e);return r?(e=r.getCellPosition(),this.ensureCellVisible(e),e):null},t.prototype.tryToFocusFullWidthRow=function(e,r){r===void 0&&(r=!1);var n=this.columnModel.getAllDisplayedColumns(),o=this.rowRenderer.getRowByPosition(e);if(!o||!o.isFullWidth())return!1;var a=this.focusService.getFocusedCell(),s={rowIndex:e.rowIndex,rowPinned:e.rowPinned,column:e.column||(r?rt(n):n[0])};this.focusPosition(s);var l=a!=null?this.rowPositionUtils.before(s,a):!1,u={type:x.EVENT_FULL_WIDTH_ROW_FOCUSED,api:this.gridApi,columnApi:this.columnApi,rowIndex:s.rowIndex,rowPinned:s.rowPinned,column:s.column,isFullWidthCell:!0,floating:s.rowPinned,fromBelow:l};return this.eventService.dispatchEvent(u),!0},t.prototype.focusPosition=function(e){this.focusService.setFocusedCell(e.rowIndex,e.column,e.rowPinned,!0),this.rangeService&&this.rangeService.setRangeToCell(e)},t.prototype.isValidNavigateCell=function(e){var r=this.rowPositionUtils.getRowNode(e);return!!r},t.prototype.getLastCellOfColSpan=function(e){var r=this.getCellByPosition(e);if(!r)return e;var n=r.getColSpanningList();return n.length===1?e:{rowIndex:e.rowIndex,column:rt(n),rowPinned:e.rowPinned}},t.prototype.ensureCellVisible=function(e){me(e.rowPinned)&&this.gridBodyCon.getScrollFeature().ensureIndexVisible(e.rowIndex),e.column.isPinned()||this.gridBodyCon.getScrollFeature().ensureColumnVisible(e.column),this.gridBodyCon.getScrollFeature().horizontallyScrollHeaderCenterAndFloatingCenter(),this.animationFrameService.flushAllFrames()},hr([y("columnApi")],t.prototype,"columnApi",void 0),hr([y("gridApi")],t.prototype,"gridApi",void 0),hr([y("mouseEventService")],t.prototype,"mouseEventService",void 0),hr([y("paginationProxy")],t.prototype,"paginationProxy",void 0),hr([y("focusService")],t.prototype,"focusService",void 0),hr([y("animationFrameService")],t.prototype,"animationFrameService",void 0),hr([De("rangeService")],t.prototype,"rangeService",void 0),hr([y("columnModel")],t.prototype,"columnModel",void 0),hr([y("ctrlsService")],t.prototype,"ctrlsService",void 0),hr([y("rowRenderer")],t.prototype,"rowRenderer",void 0),hr([y("headerNavigationService")],t.prototype,"headerNavigationService",void 0),hr([y("rowPositionUtils")],t.prototype,"rowPositionUtils",void 0),hr([y("cellNavigationService")],t.prototype,"cellNavigationService",void 0),hr([y("pinnedRowModel")],t.prototype,"pinnedRowModel",void 0),hr([B],t.prototype,"postConstruct",null),t=hr([Q("navigationService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var QG=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),JG=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},kE=function(i){QG(t,i);function t(e){var r=i.call(this,'<div class="ag-popup-editor" tabindex="-1"/>')||this;return r.params=e,r}return t.prototype.postConstruct=function(){this.gridOptionsWrapper.setDomData(this.getGui(),t.DOM_KEY_POPUP_EDITOR_WRAPPER,!0),this.addKeyDownListener()},t.prototype.addKeyDownListener=function(){var e=this,r=this.getGui(),n=this.params,o=function(a){qh(e.gridOptionsWrapper,a,n.node,n.column,!0)||n.onKeyDown(a)};this.addManagedListener(r,"keydown",o)},t.DOM_KEY_POPUP_EDITOR_WRAPPER="popupEditorWrapper",JG([B],t.prototype,"postConstruct",null),t}(ws);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ek=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),WE=function(i){ek(t,i);function t(e,r,n,o,a){var s=i.call(this)||this;s.rendererVersion=0,s.editorVersion=0,s.beans=e,s.column=r.getColumn(),s.rowNode=r.getRowNode(),s.rowCtrl=r.getRowCtrl(),s.eRow=o,s.setTemplate('<div comp-id="'+s.getCompId()+'"/>');var l=s.getGui();s.forceWrapper=r.isForceWrapper(),s.refreshWrapper(!1);var u=function(p,h,d){var f=d||l;h!=null&&h!=""?f.setAttribute(p,h):f.removeAttribute(p)},c={addOrRemoveCssClass:function(p,h){return s.addOrRemoveCssClass(p,h)},setUserStyles:function(p){return MC(l,p)},getFocusableElement:function(){return s.getFocusableElement()},setTabIndex:function(p){return u("tabindex",p.toString())},setRole:function(p){return _r(l,p)},setColId:function(p){return u("col-id",p)},setTitle:function(p){return u("title",p)},setIncludeSelection:function(p){return s.includeSelection=p},setIncludeRowDrag:function(p){return s.includeRowDrag=p},setIncludeDndSource:function(p){return s.includeDndSource=p},setRenderDetails:function(p,h,d){return s.setRenderDetails(p,h,d)},setEditDetails:function(p,h,d){return s.setEditDetails(p,h,d)},getCellEditor:function(){return s.cellEditor||null},getCellRenderer:function(){return s.cellRenderer||null},getParentOfValue:function(){return s.getParentOfValue()}};return s.cellCtrl=r,r.setComp(c,s.getGui(),s.eCellWrapper,n,a),s}return t.prototype.getParentOfValue=function(){return this.eCellValue?this.eCellValue:this.eCellWrapper?this.eCellWrapper:this.getGui()},t.prototype.setRenderDetails=function(e,r,n){var o=this.cellEditor&&!this.cellEditorPopupWrapper;if(!o){this.firstRender=this.firstRender==null;var a=this.refreshWrapper(!1);if(e){var s=n||a,l=s?!1:this.refreshCellRenderer(e);l||(this.destroyRenderer(),this.createCellRendererInstance(e))}else this.destroyRenderer(),this.insertValueWithoutCellRenderer(r)}},t.prototype.setEditDetails=function(e,r,n){e?this.createCellEditorInstance(e,r,n):this.destroyEditor()},t.prototype.removeControls=function(){this.checkboxSelectionComp=this.beans.context.destroyBean(this.checkboxSelectionComp),this.dndSourceComp=this.beans.context.destroyBean(this.dndSourceComp),this.rowDraggingComp=this.beans.context.destroyBean(this.rowDraggingComp)},t.prototype.refreshWrapper=function(e){var r=this.includeRowDrag||this.includeDndSource||this.includeSelection,n=r||this.forceWrapper,o=n&&this.eCellWrapper==null;o&&(this.eCellWrapper=Kt('<div class="ag-cell-wrapper" role="presentation"></div>'),this.getGui().appendChild(this.eCellWrapper));var a=!n&&this.eCellWrapper!=null;a&&(qn(this.eCellWrapper),this.eCellWrapper=void 0),this.addOrRemoveCssClass("ag-cell-value",!n);var s=!e&&n,l=s&&this.eCellValue==null;l&&(this.eCellValue=Kt('<span class="ag-cell-value" role="presentation"></span>'),this.eCellWrapper.appendChild(this.eCellValue));var u=!s&&this.eCellValue!=null;u&&(qn(this.eCellValue),this.eCellValue=void 0);var c=o||a||l||u;return c&&this.removeControls(),!e&&r&&this.addControls(),c},t.prototype.addControls=function(){var e=this.eCellValue.id="cell-"+this.getCompId(),r=[];this.includeRowDrag&&this.rowDraggingComp==null&&(this.rowDraggingComp=this.cellCtrl.createRowDragComp(),this.rowDraggingComp&&this.eCellWrapper.insertBefore(this.rowDraggingComp.getGui(),this.eCellValue)),this.includeDndSource&&this.dndSourceComp==null&&(this.dndSourceComp=this.cellCtrl.createDndSource(),this.eCellWrapper.insertBefore(this.dndSourceComp.getGui(),this.eCellValue)),this.includeSelection&&(this.checkboxSelectionComp==null&&(this.checkboxSelectionComp=this.cellCtrl.createSelectionCheckbox(),this.eCellWrapper.insertBefore(this.checkboxSelectionComp.getGui(),this.eCellValue)),r.push(this.checkboxSelectionComp.getCheckboxId())),r.push(e),NC(this.getGui(),r.join(" "))},t.prototype.createCellEditorInstance=function(e,r,n){var o=this,a=this.editorVersion,s=e.newAgStackInstance();if(!!s){var l=e.params;s.then(function(c){return o.afterCellEditorCreated(a,c,l,r,n)});var u=me(this.cellEditor);u&&l.cellStartedEdit&&this.cellCtrl.focusCell(!0)}},t.prototype.insertValueWithoutCellRenderer=function(e){var r=this.getParentOfValue();gn(r);var n=e!=null?Ii(e):null;n!=null&&(r.innerHTML=n)},t.prototype.destroyEditorAndRenderer=function(){this.destroyRenderer(),this.destroyEditor()},t.prototype.destroyRenderer=function(){var e=this.beans.context;this.cellRenderer=e.destroyBean(this.cellRenderer),qn(this.cellRendererGui),this.cellRendererGui=null,this.rendererVersion++},t.prototype.destroyEditor=function(){var e=this.beans.context;this.hideEditorPopup&&this.hideEditorPopup(),this.hideEditorPopup=void 0,this.cellEditor=e.destroyBean(this.cellEditor),this.cellEditorPopupWrapper=e.destroyBean(this.cellEditorPopupWrapper),qn(this.cellEditorGui),this.cellEditorGui=null,this.editorVersion++},t.prototype.refreshCellRenderer=function(e){if(this.cellRenderer==null||this.cellRenderer.refresh==null||this.cellRendererClass!==e.componentClass)return!1;var r=this.cellRenderer.refresh(e.params);return r===!0||r===void 0},t.prototype.createCellRendererInstance=function(e){var r=this,n=this.beans.gridOptionsWrapper.isAngularCompileRows(),o=this.beans.gridOptionsWrapper.isSuppressAnimationFrame(),a=!n&&!o,s=this.rendererVersion,l=e.componentClass,u=function(){var c=r.rendererVersion!==s||!r.isAlive();if(!c){var p=e.newAgStackInstance(),h=r.afterCellRendererCreated.bind(r,s,l);p&&p.then(h)}};a&&this.firstRender?this.beans.animationFrameService.createTask(u,this.rowNode.rowIndex,"createTasksP2"):u()},t.prototype.getCtrl=function(){return this.cellCtrl},t.prototype.getRowCtrl=function(){return this.rowCtrl},t.prototype.getCellRenderer=function(){return this.cellRenderer},t.prototype.getCellEditor=function(){return this.cellEditor},t.prototype.afterCellRendererCreated=function(e,r,n){var o=!this.isAlive()||e!==this.rendererVersion;if(o){this.beans.context.destroyBean(n);return}if(this.cellRenderer=n,this.cellRendererClass=r,this.cellRendererGui=this.cellRenderer.getGui(),this.cellRendererGui!=null){var a=this.getParentOfValue();gn(a),a.appendChild(this.cellRendererGui)}},t.prototype.afterCellEditorCreated=function(e,r,n,o,a){var s=e!==this.editorVersion;if(s){this.beans.context.destroyBean(r);return}var l=r.isCancelBeforeStart&&r.isCancelBeforeStart();if(l){this.beans.context.destroyBean(r),this.cellCtrl.stopEditing();return}if(!r.getGui){console.warn("AG Grid: cellEditor for column "+this.column.getId()+" is missing getGui() method"),this.beans.context.destroyBean(r);return}this.cellEditor=r,this.cellEditorGui=r.getGui();var u=o||r.isPopup!==void 0&&r.isPopup();u?(o||this.cellCtrl.hackSayEditingInPopup(),this.addPopupCellEditor(n,a)):this.addInCellEditor(),r.afterGuiAttached&&r.afterGuiAttached()},t.prototype.addInCellEditor=function(){var e=this.getGui(),r=this.beans.gridOptionsWrapper.getDocument();if(e.contains(r.activeElement)&&e.focus(),this.destroyRenderer(),this.refreshWrapper(!0),this.clearParentOfValue(),this.cellEditorGui){var n=this.getParentOfValue();n.appendChild(this.cellEditorGui)}},t.prototype.addPopupCellEditor=function(e,r){var n=this;this.beans.gridOptionsWrapper.isFullRowEdit()&&console.warn("AG Grid: popup cellEditor does not work with fullRowEdit - you cannot use them both - either turn off fullRowEdit, or stop using popup editors.");var o=this.cellEditor;this.cellEditorPopupWrapper=this.beans.context.createBean(new kE(e));var a=this.cellEditorPopupWrapper.getGui();this.cellEditorGui&&a.appendChild(this.cellEditorGui);var s=this.beans.popupService,l=this.beans.gridOptionsWrapper.isStopEditingWhenCellsLoseFocus(),u=r!=null?r:o.getPopupPosition?o.getPopupPosition():"over",c={column:this.column,rowNode:this.rowNode,type:"popupCellEditor",eventSource:this.getGui(),ePopup:a,keepWithinBounds:!0},p=u==="under"?s.positionPopupUnderComponent.bind(s,c):s.positionPopupOverComponent.bind(s,c),h=this.beans.gridOptionsWrapper.getLocaleTextFunc(),d=s.addPopup({modal:l,eChild:a,closeOnEsc:!0,closedCallback:function(){n.cellCtrl.onPopupEditorClosed()},anchorToElement:this.getGui(),positionCallback:p,ariaLabel:h("ariaLabelCellEditor","Cell Editor")});d&&(this.hideEditorPopup=d.hideFunc)},t.prototype.detach=function(){this.eRow.removeChild(this.getGui())},t.prototype.destroy=function(){this.cellCtrl.stopEditing(),this.destroyEditorAndRenderer(),this.removeControls(),i.prototype.destroy.call(this)},t.prototype.clearParentOfValue=function(){var e=this.getGui(),r=this.beans.gridOptionsWrapper.getDocument();e.contains(r.activeElement)&&e.focus({preventScroll:!0}),gn(this.getParentOfValue())},t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var tk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),rk=function(i){tk(t,i);function t(e,r,n){var o=i.call(this)||this;o.cellComps={},o.beans=r,o.rowCtrl=e,o.setTemplate('<div comp-id="'+o.getCompId()+'" style="'+o.getInitialStyle()+'"/>');var a=o.getGui(),s=a.style,l={setDomOrder:function(u){return o.domOrder=u},setCellCtrls:function(u){return o.setCellCtrls(u)},showFullWidth:function(u){return o.showFullWidth(u)},getFullWidthCellRenderer:function(){return o.getFullWidthCellRenderer()},addOrRemoveCssClass:function(u,c){return o.addOrRemoveCssClass(u,c)},setUserStyles:function(u){return MC(a,u)},setTop:function(u){return s.top=u},setTransform:function(u){return s.transform=u},setRowIndex:function(u){return a.setAttribute("row-index",u)},setRole:function(u){return _r(a,u)},setRowId:function(u){return a.setAttribute("row-id",u)},setRowBusinessKey:function(u){return a.setAttribute("row-business-key",u)},setTabIndex:function(u){return a.setAttribute("tabindex",u.toString())}};return e.setComp(l,o.getGui(),n),o}return t.prototype.getInitialStyle=function(){var e=this.rowCtrl.getInitialTransform(),r=this.rowCtrl.getInitialRowTop();return e?"transform: "+e:"top: "+r},t.prototype.showFullWidth=function(e){var r=this,n=function(a){if(r.isAlive()){var s=a.getGui();r.getGui().appendChild(s),r.rowCtrl.setupDetailRowAutoHeight(s),r.setFullWidthRowComp(a)}else r.beans.context.destroyBean(a)},o=e.newAgStackInstance();!o||o.then(n)},t.prototype.setCellCtrls=function(e){var r=this,n=Object.assign({},this.cellComps);e.forEach(function(a){var s=a.getInstanceId(),l=r.cellComps[s];l==null?r.newCellComp(a):n[s]=null});var o=Jn(n).filter(function(a){return a!=null});this.destroyCells(o),this.ensureDomOrder(e)},t.prototype.ensureDomOrder=function(e){var r=this;if(!!this.domOrder){var n=[];e.forEach(function(o){var a=r.cellComps[o.getInstanceId()];a&&n.push(a.getGui())}),DC(this.getGui(),n)}},t.prototype.newCellComp=function(e){var r=new WE(this.beans,e,this.rowCtrl.isPrintLayout(),this.getGui(),this.rowCtrl.isEditing());this.cellComps[e.getInstanceId()]=r,this.getGui().appendChild(r.getGui())},t.prototype.destroy=function(){i.prototype.destroy.call(this),this.destroyAllCells()},t.prototype.destroyAllCells=function(){var e=Jn(this.cellComps).filter(function(r){return r!=null});this.destroyCells(e)},t.prototype.setFullWidthRowComp=function(e){var r=this;this.fullWidthCellRenderer&&console.error("AG Grid - should not be setting fullWidthRowComponent twice"),this.fullWidthCellRenderer=e,this.addDestroyFunc(function(){r.fullWidthCellRenderer=r.beans.context.destroyBean(r.fullWidthCellRenderer)})},t.prototype.getFullWidthCellRenderer=function(){return this.fullWidthCellRenderer},t.prototype.destroyCells=function(e){var r=this;e.forEach(function(n){if(!!n){var o=n.getCtrl().getInstanceId();r.cellComps[o]===n&&(n.detach(),n.destroy(),r.cellComps[o]=null)}})},t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var nk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),oy=globalThis&&globalThis.__assign||function(){return oy=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},oy.apply(this,arguments)},Fs=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o};function ik(){var i=ee.elementGettingCreated.getAttribute("name"),t=id.getRowContainerCssClasses(i),e,r=i===re.CENTER,n=i===re.TOP_CENTER||i===re.BOTTOM_CENTER;return r?e='<div class="'+t.wrapper+`" ref="eWrapper" role="presentation">
                <div class="`+t.viewport+`" ref="eViewport" role="presentation">
                    <div class="`+t.container+`" ref="eContainer"></div>
                </div>
            </div>`:n?e='<div class="'+t.viewport+`" ref="eViewport" role="presentation">
                <div class="`+t.container+`" ref="eContainer"></div>
            </div>`:e='<div class="'+t.container+'" ref="eContainer"></div>',e}var VE=function(i){nk(t,i);function t(){var e=i.call(this,ik())||this;return e.rowComps={},e.name=ee.elementGettingCreated.getAttribute("name"),e.type=FE(e.name),e}return t.prototype.postConstruct=function(){var e=this,r={setViewportHeight:function(o){return e.eViewport.style.height=o},setRowCtrls:function(o){return e.setRowCtrls(o)},setDomOrder:function(o){e.domOrder=o},setContainerWidth:function(o){return e.eContainer.style.width=o}},n=this.createManagedBean(new id(this.name));n.setComp(r,this.eContainer,this.eViewport,this.eWrapper)},t.prototype.preDestroy=function(){this.setRowCtrls([])},t.prototype.setRowCtrls=function(e){var r=this,n=oy({},this.rowComps);this.rowComps={},this.lastPlacedElement=null;var o=function(a){var s=a.getInstanceId(),l=n[s];if(l)r.rowComps[s]=l,delete n[s],r.ensureDomOrder(l.getGui());else{var u=r.newRowComp(a);r.rowComps[s]=u,r.appendRow(u.getGui())}};e.forEach(o),Jn(n).forEach(function(a){r.eContainer.removeChild(a.getGui()),a.destroy()}),_r(this.eContainer,e.length?"rowgroup":"presentation")},t.prototype.appendRow=function(e){this.domOrder?FO(this.eContainer,e,this.lastPlacedElement):this.eContainer.appendChild(e),this.lastPlacedElement=e},t.prototype.ensureDomOrder=function(e){this.domOrder&&(AC(this.eContainer,e,this.lastPlacedElement),this.lastPlacedElement=e)},t.prototype.newRowComp=function(e){id.getPinned(this.name);var r=new rk(e,this.beans,this.type);return r},Fs([y("beans")],t.prototype,"beans",void 0),Fs([I("eViewport")],t.prototype,"eViewport",void 0),Fs([I("eContainer")],t.prototype,"eContainer",void 0),Fs([I("eWrapper")],t.prototype,"eWrapper",void 0),Fs([B],t.prototype,"postConstruct",null),Fs([nt],t.prototype,"preDestroy",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var T_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},BE=function(){function i(t){this.columnsToAggregate=[],this.columnsToGroup=[],this.columnsToPivot=[],this.pinned=t}return i.prototype.onDragEnter=function(t){var e=this;if(this.clearColumnsList(),!this.gridOptionsWrapper.isFunctionsReadOnly()){var r=t.dragItem.columns;!r||r.forEach(function(n){!n.isPrimary()||n.isAnyFunctionActive()||(n.isAllowValue()?e.columnsToAggregate.push(n):n.isAllowRowGroup()?e.columnsToGroup.push(n):n.isAllowPivot()&&e.columnsToPivot.push(n))})}},i.prototype.getIconName=function(){var t=this.columnsToAggregate.length+this.columnsToGroup.length+this.columnsToPivot.length;return t>0?this.pinned?ht.ICON_PINNED:ht.ICON_MOVE:null},i.prototype.onDragLeave=function(t){this.clearColumnsList()},i.prototype.clearColumnsList=function(){this.columnsToAggregate.length=0,this.columnsToGroup.length=0,this.columnsToPivot.length=0},i.prototype.onDragging=function(t){},i.prototype.onDragStop=function(t){this.columnsToAggregate.length>0&&this.columnModel.addValueColumns(this.columnsToAggregate,"toolPanelDragAndDrop"),this.columnsToGroup.length>0&&this.columnModel.addRowGroupColumns(this.columnsToGroup,"toolPanelDragAndDrop"),this.columnsToPivot.length>0&&this.columnModel.addPivotColumns(this.columnsToPivot,"toolPanelDragAndDrop")},T_([y("columnModel")],i.prototype,"columnModel",void 0),T_([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var pu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},HE=function(){function i(t,e){this.needToMoveLeft=!1,this.needToMoveRight=!1,this.pinned=t,this.eContainer=e,this.centerContainer=!q(t)}return i.prototype.init=function(){var t=this;this.ctrlsService.whenReady(function(){t.gridBodyCon=t.ctrlsService.getGridBodyCtrl()})},i.prototype.getIconName=function(){return this.pinned?ht.ICON_PINNED:ht.ICON_MOVE},i.prototype.onDragEnter=function(t){var e=t.dragItem.columns,r=t.dragSource.type===At.ToolPanel;if(r)this.setColumnsVisible(e,!0,"uiColumnDragged");else{var n=t.dragItem.visibleState,o=(e||[]).filter(function(a){return n[a.getId()]});this.setColumnsVisible(o,!0,"uiColumnDragged")}this.setColumnsPinned(e,this.pinned,"uiColumnDragged"),this.onDragging(t,!0)},i.prototype.onDragLeave=function(t){var e=!this.gridOptionsWrapper.isSuppressDragLeaveHidesColumns()&&!t.fromNudge;if(e){var r=t.dragSource.getDragItem(),n=r.columns;this.setColumnsVisible(n,!1,"uiColumnDragged")}this.ensureIntervalCleared()},i.prototype.setColumnsVisible=function(t,e,r){if(r===void 0&&(r="api"),t){var n=t.filter(function(o){return!o.getColDef().lockVisible});this.columnModel.setColumnsVisible(n,e,r)}},i.prototype.setColumnsPinned=function(t,e,r){if(r===void 0&&(r="api"),t){var n=t.filter(function(o){return!o.getColDef().lockPinned});this.columnModel.setColumnsPinned(n,e,r)}},i.prototype.onDragStop=function(){this.ensureIntervalCleared()},i.prototype.normaliseX=function(t){if(this.gridOptionsWrapper.isEnableRtl()){var e=this.eContainer.clientWidth;t=e-t}return this.centerContainer&&(t+=this.ctrlsService.getCenterRowContainerCtrl().getCenterViewportScrollLeft()),t},i.prototype.checkCenterForScrolling=function(t){if(this.centerContainer){var e=this.ctrlsService.getCenterRowContainerCtrl().getCenterViewportScrollLeft(),r=e+this.ctrlsService.getCenterRowContainerCtrl().getCenterWidth();this.gridOptionsWrapper.isEnableRtl()?(this.needToMoveRight=t<e+50,this.needToMoveLeft=t>r-50):(this.needToMoveLeft=t<e+50,this.needToMoveRight=t>r-50),this.needToMoveLeft||this.needToMoveRight?this.ensureIntervalStarted():this.ensureIntervalCleared()}},i.prototype.onDragging=function(t,e){var r=this;if(e===void 0&&(e=!1),this.lastDraggingEvent=t,!me(t.hDirection)){var n=this.normaliseX(t.x);e||this.checkCenterForScrolling(n);var o=this.normaliseDirection(t.hDirection),a=t.dragSource.type,s=t.dragSource.getDragItem().columns;s=s.filter(function(l){return l.getColDef().lockPinned?l.getPinned()==r.pinned:!0}),this.attemptMoveColumns(a,s,o,n,e)}},i.prototype.normaliseDirection=function(t){if(this.gridOptionsWrapper.isEnableRtl())switch(t){case Pn.Left:return Pn.Right;case Pn.Right:return Pn.Left;default:console.error("AG Grid: Unknown direction "+t)}else return t},i.prototype.calculateOldIndex=function(t){var e=this.columnModel.getAllGridColumns(),r=dO(t.map(function(l){return e.indexOf(l)})),n=r[0],o=rt(r),a=o-n,s=a!==r.length-1;return s?null:n},i.prototype.attemptMoveColumns=function(t,e,r,n,o){var a=r===Pn.Left,s=r===Pn.Right,l=e.slice();this.columnModel.sortColumnsLikeGridColumns(l);var u=this.calculateValidMoves(l,s,n),c=this.calculateOldIndex(l);if(u.length!==0){var p=u[0],h=c!==null&&!o;if(t==At.HeaderCell&&(h=c!==null),!(h&&(a&&p>=c||s&&p<=c)))for(var d=0;d<u.length;d++){var f=u[d];if(!!this.columnModel.doesMovePassRules(l,f)){this.columnModel.moveColumns(l,f,"uiColumnDragged");return}}}},i.prototype.calculateValidMoves=function(t,e,r){var n=this.gridOptionsWrapper.isSuppressMovableColumns()||t.some(function(E){return E.getColDef().suppressMovable});if(n)return[];var o=this.columnModel.getDisplayedColumns(this.pinned),a=this.columnModel.getAllGridColumns(),s=o.filter(function(E){return fn(t,E)}),l=o.filter(function(E){return!fn(t,E)}),u=a.filter(function(E){return!fn(t,E)}),c=0,p=r;if(e){var h=0;s.forEach(function(E){return h+=E.getActualWidth()}),p-=h}if(p>0){for(var d=0;d<l.length;d++){var f=l[d];if(p-=f.getActualWidth(),p<0)break;c++}e&&c++}var g;if(c>0){var v=l[c-1];g=u.indexOf(v)+1}else g=u.indexOf(l[0]),g===-1&&(g=0);var m=[g],C=function(E,P){return E-P};if(e){for(var w=g+1,b=a.length-1;w<=b;)m.push(w),w++;m.sort(C)}else{for(var w=g,b=a.length-1,S=a[w];w<=b&&this.isColumnHidden(o,S);)w++,m.push(w),S=a[w];w=g-1;for(var T=0;w>=T;)m.push(w),w--;m.sort(C).reverse()}return m},i.prototype.isColumnHidden=function(t,e){return t.indexOf(e)<0},i.prototype.ensureIntervalStarted=function(){this.movingIntervalId||(this.intervalCount=0,this.failedMoveAttempts=0,this.movingIntervalId=window.setInterval(this.moveInterval.bind(this),100),this.needToMoveLeft?this.dragAndDropService.setGhostIcon(ht.ICON_LEFT,!0):this.dragAndDropService.setGhostIcon(ht.ICON_RIGHT,!0))},i.prototype.ensureIntervalCleared=function(){this.movingIntervalId&&(window.clearInterval(this.movingIntervalId),this.movingIntervalId=null,this.dragAndDropService.setGhostIcon(ht.ICON_MOVE))},i.prototype.moveInterval=function(){var t;this.intervalCount++,t=10+this.intervalCount*5,t>100&&(t=100);var e=null,r=this.gridBodyCon.getScrollFeature();if(this.needToMoveLeft?e=r.scrollHorizontally(-t):this.needToMoveRight&&(e=r.scrollHorizontally(t)),e!==0)this.onDragging(this.lastDraggingEvent),this.failedMoveAttempts=0;else{this.failedMoveAttempts++;var n=this.lastDraggingEvent.dragItem.columns,o=n.filter(function(s){return!s.getColDef().lockPinned});if(o.length>0&&(this.dragAndDropService.setGhostIcon(ht.ICON_PINNED),this.failedMoveAttempts>7)){var a=this.needToMoveLeft?L.PINNED_LEFT:L.PINNED_RIGHT;this.setColumnsPinned(o,a,"uiColumnDragged"),this.dragAndDropService.nudge()}}},pu([y("columnModel")],i.prototype,"columnModel",void 0),pu([y("dragAndDropService")],i.prototype,"dragAndDropService",void 0),pu([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),pu([y("ctrlsService")],i.prototype,"ctrlsService",void 0),pu([B],i.prototype,"init",null),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ok=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),hu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},jE=function(i){ok(t,i);function t(e,r){var n=i.call(this)||this;return n.pinned=e,n.eContainer=r,n}return t.prototype.postConstruct=function(){var e=this;this.ctrlsService.whenReady(function(r){switch(e.pinned){case L.PINNED_LEFT:e.eSecondaryContainers=[[r.gridBodyCtrl.getBodyViewportElement(),r.leftRowContainerCtrl.getContainerElement()],[r.bottomLeftRowContainerCtrl.getContainerElement()],[r.topLeftRowContainerCtrl.getContainerElement()]];break;case L.PINNED_RIGHT:e.eSecondaryContainers=[[r.gridBodyCtrl.getBodyViewportElement(),r.rightRowContainerCtrl.getContainerElement()],[r.bottomRightRowContainerCtrl.getContainerElement()],[r.topRightRowContainerCtrl.getContainerElement()]];break;default:e.eSecondaryContainers=[[r.gridBodyCtrl.getBodyViewportElement(),r.centerRowContainerCtrl.getViewportElement()],[r.bottomCenterRowContainerCtrl.getViewportElement()],[r.topCenterRowContainerCtrl.getViewportElement()]];break}})},t.prototype.isInterestedIn=function(e){return e===At.HeaderCell||e===At.ToolPanel&&this.gridOptionsWrapper.isAllowDragFromColumnsToolPanel()},t.prototype.getSecondaryContainers=function(){return this.eSecondaryContainers},t.prototype.getContainer=function(){return this.eContainer},t.prototype.init=function(){this.moveColumnFeature=this.createManagedBean(new HE(this.pinned,this.eContainer)),this.bodyDropPivotTarget=this.createManagedBean(new BE(this.pinned)),this.dragAndDropService.addDropTarget(this)},t.prototype.getIconName=function(){return this.currentDropListener.getIconName()},t.prototype.isDropColumnInPivotMode=function(e){return this.columnModel.isPivotMode()&&e.dragSource.type===At.ToolPanel},t.prototype.onDragEnter=function(e){this.currentDropListener=this.isDropColumnInPivotMode(e)?this.bodyDropPivotTarget:this.moveColumnFeature,this.currentDropListener.onDragEnter(e)},t.prototype.onDragLeave=function(e){this.currentDropListener.onDragLeave(e)},t.prototype.onDragging=function(e){this.currentDropListener.onDragging(e)},t.prototype.onDragStop=function(e){this.currentDropListener.onDragStop(e)},hu([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),hu([y("columnModel")],t.prototype,"columnModel",void 0),hu([y("ctrlsService")],t.prototype,"ctrlsService",void 0),hu([B],t.prototype,"postConstruct",null),hu([B],t.prototype,"init",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ak=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},sk=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(ak(arguments[t]));return i},qc=function(){function i(){}return i.getHeaderClassesFromColDef=function(t,e,r,n){return me(t)?[]:this.getColumnClassesFromCollDef(t.headerClass,t,e,r,n)},i.getToolPanelClassesFromColDef=function(t,e,r,n){return me(t)?[]:this.getColumnClassesFromCollDef(t.toolPanelClass,t,e,r,n)},i.getClassParams=function(t,e,r,n){return{colDef:t,column:r,columnGroup:n,api:e.getApi(),columnApi:e.getColumnApi(),context:e.getContext()}},i.getColumnClassesFromCollDef=function(t,e,r,n,o){if(me(t))return[];var a;if(typeof t=="function"){var s=this.getClassParams(e,r,n,o);a=t(s)}else a=t;return typeof a=="string"?[a]:Array.isArray(a)?sk(a):[]},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var lk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Eg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},uk=function(i){lk(t,i);function t(e){var r=i.call(this,t.TEMPLATE,e)||this;return r.headerCompVersion=0,r.column=e.getColumnGroupChild(),r.pinned=e.getPinned(),r}return t.prototype.postConstruct=function(){var e=this,r=this.getGui(),n=function(s,l,u){var c=u||r;l!=null&&l!=""?c.setAttribute(s,l):c.removeAttribute(s)},o={setWidth:function(s){return r.style.width=s},addOrRemoveCssClass:function(s,l){return e.addOrRemoveCssClass(s,l)},setColId:function(s){return n("col-id",s)},setTitle:function(s){return n("title",s)},setAriaDescription:function(s){return jO(r,s)},setAriaSort:function(s){return s?qO(r,s):ZO(r)},setUserCompDetails:function(s){return e.setUserCompDetails(s)},getUserCompInstance:function(){return e.headerComp}};this.ctrl.setComp(o,this.getGui(),this.eResize);var a=this.ctrl.getSelectAllGui();this.eResize.insertAdjacentElement("afterend",a)},t.prototype.destroyHeaderComp=function(){this.headerComp&&(this.getGui().removeChild(this.headerCompGui),this.headerComp=this.destroyBean(this.headerComp),this.headerCompGui=void 0)},t.prototype.setUserCompDetails=function(e){var r=this;this.headerCompVersion++;var n=this.headerCompVersion;e.newAgStackInstance().then(function(o){return r.afterCompCreated(n,o)})},t.prototype.afterCompCreated=function(e,r){if(e!=this.headerCompVersion||!this.isAlive()){this.destroyBean(r);return}this.destroyHeaderComp(),this.headerComp=r,this.headerCompGui=r.getGui(),this.getGui().appendChild(this.headerCompGui),this.ctrl.setDragSource(this.headerCompGui)},t.TEMPLATE=`<div class="ag-header-cell" role="columnheader" tabindex="-1">
            <div ref="eResize" class="ag-header-cell-resize" role="presentation"></div>
        </div>`,Eg([I("eResize")],t.prototype,"eResize",void 0),Eg([B],t.prototype,"postConstruct",null),Eg([nt],t.prototype,"destroyHeaderComp",null),t}(qC);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ck=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Rg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},pk=function(i){ck(t,i);function t(e){return i.call(this,t.TEMPLATE,e)||this}return t.prototype.postConstruct=function(){var e=this,r=this.getGui(),n=function(a,s){return s!=null?r.setAttribute(a,s):r.removeAttribute(a)},o={addOrRemoveCssClass:function(a,s){return e.addOrRemoveCssClass(a,s)},addOrRemoveResizableCssClass:function(a,s){return e.eResize.classList.toggle(a,s)},setWidth:function(a){return r.style.width=a},setColId:function(a){return r.setAttribute("col-id",a)},setAriaExpanded:function(a){return n("aria-expanded",a)},setTitle:function(a){return n("title",a)},setUserCompDetails:function(a){return e.setUserCompDetails(a)}};this.ctrl.setComp(o,r,this.eResize)},t.prototype.setUserCompDetails=function(e){var r=this;e.newAgStackInstance().then(function(n){return r.afterHeaderCompCreated(n)})},t.prototype.afterHeaderCompCreated=function(e){var r=this,n=function(){return r.destroyBean(e)};if(!this.isAlive()){n();return}this.getGui().appendChild(e.getGui()),this.addDestroyFunc(n),this.ctrl.setDragSource(e.getGui())},t.TEMPLATE=`<div class="ag-header-group-cell" role="columnheader" tabindex="-1">
            <div ref="eResize" class="ag-header-cell-resize" role="presentation"></div>
        </div>`,Rg([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),Rg([I("eResize")],t.prototype,"eResize",void 0),Rg([B],t.prototype,"postConstruct",null),t}(qC);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var hk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),O_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},wr;(function(i){i.COLUMN_GROUP="group",i.COLUMN="column",i.FLOATING_FILTER="filter"})(wr||(wr={}));var UE=function(i){hk(t,i);function t(e){var r=i.call(this)||this;r.headerComps={};var n=e.getType()==wr.COLUMN_GROUP?"ag-header-row-column-group":e.getType()==wr.FLOATING_FILTER?"ag-header-row-column-filter":"ag-header-row-column";return r.setTemplate('<div class="ag-header-row '+n+'" role="row"></div>'),r.ctrl=e,r}return t.prototype.init=function(){var e=this,r={setTransform:function(n){return e.getGui().style.transform=n},setHeight:function(n){return e.getGui().style.height=n},setTop:function(n){return e.getGui().style.top=n},setHeaderCtrls:function(n){return e.setHeaderCtrls(n)},setWidth:function(n){return e.getGui().style.width=n},setAriaRowIndex:function(n){return VC(e.getGui(),n)}};this.ctrl.setComp(r)},t.prototype.destroyHeaderCtrls=function(){this.setHeaderCtrls([])},t.prototype.setHeaderCtrls=function(e){var r=this;if(!!this.isAlive()){var n=this.headerComps;this.headerComps={},e.forEach(function(l){var u=l.getInstanceId(),c=n[u];delete n[u],c==null&&(c=r.createHeaderComp(l),r.getGui().appendChild(c.getGui())),r.headerComps[u]=c}),Ke(n,function(l,u){r.getGui().removeChild(u.getGui()),r.destroyBean(u)});var o=this.gridOptionsWrapper.isEnsureDomOrder();if(o){var a=Jn(this.headerComps);a.sort(function(l,u){var c=l.getCtrl().getColumnGroupChild().getLeft(),p=u.getCtrl().getColumnGroupChild().getLeft();return c-p});var s=a.map(function(l){return l.getGui()});DC(this.getGui(),s)}}},t.prototype.createHeaderComp=function(e){var r;switch(this.ctrl.getType()){case wr.COLUMN_GROUP:r=new pk(e);break;case wr.FLOATING_FILTER:r=new EE(e);break;default:r=new uk(e);break}return this.createBean(r),r.setParentComponent(this),r},O_([B],t.prototype,"init",null),O_([nt],t.prototype,"destroyHeaderCtrls",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var dk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),fk=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},gk=0,Zc=function(i){dk(t,i);function t(e,r){var n=i.call(this)||this;return n.lastFocusEvent=null,n.columnGroupChild=e,n.parentRowCtrl=r,n.instanceId=e.getUniqueId()+"-"+gk++,n}return t.prototype.shouldStopEventPropagation=function(e){var r=this.focusService.getFocusedHeader(),n=r.headerRowIndex,o=r.column;return aE(this.gridOptionsWrapper,e,n,o)},t.prototype.setGui=function(e){this.eGui=e,this.addDomData()},t.prototype.addDomData=function(){var e=this,r=t.DOM_DATA_KEY_HEADER_CTRL;this.gridOptionsWrapper.setDomData(this.eGui,r,this),this.addDestroyFunc(function(){return e.gridOptionsWrapper.setDomData(e.eGui,r,null)})},t.prototype.getGui=function(){return this.eGui},t.prototype.focus=function(e){return this.eGui?(this.lastFocusEvent=e||null,this.eGui.focus(),!0):!1},t.prototype.getRowIndex=function(){return this.parentRowCtrl.getRowIndex()},t.prototype.getParentRowCtrl=function(){return this.parentRowCtrl},t.prototype.getPinned=function(){return this.parentRowCtrl.getPinned()},t.prototype.getInstanceId=function(){return this.instanceId},t.prototype.getColumnGroupChild=function(){return this.columnGroupChild},t.DOM_DATA_KEY_HEADER_CTRL="headerCtrl",fk([y("focusService")],t.prototype,"focusService",void 0),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ze;(function(i){i.EVERYTHING="group",i.FILTER="filter",i.SORT="sort",i.MAP="map",i.AGGREGATE="aggregate",i.FILTER_AGGREGATES="filter_aggregates",i.PIVOT="pivot",i.NOTHING="nothing"})(ze||(ze={}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Ue=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o};function ka(i){var t=i,e=t!=null&&t.getFrameworkComponentInstance!=null;return e?t.getFrameworkComponentInstance():i}var zE=function(){function i(){this.detailGridInfoMap={},this.destroyCalled=!1}return i.prototype.registerOverlayWrapperComp=function(t){this.overlayWrapperComp=t},i.prototype.registerSideBarComp=function(t){this.sideBarComp=t},i.prototype.init=function(){var t=this;switch(this.rowModel.getType()){case L.ROW_MODEL_TYPE_CLIENT_SIDE:this.clientSideRowModel=this.rowModel;break;case L.ROW_MODEL_TYPE_INFINITE:this.infiniteRowModel=this.rowModel;break;case L.ROW_MODEL_TYPE_SERVER_SIDE:this.serverSideRowModel=this.rowModel;break}this.ctrlsService.whenReady(function(){t.gridBodyCtrl=t.ctrlsService.getGridBodyCtrl()})},i.prototype.__getAlignedGridService=function(){return this.alignedGridsService},i.prototype.__getContext=function(){return this.context},i.prototype.addDetailGridInfo=function(t,e){this.detailGridInfoMap[t]=e},i.prototype.removeDetailGridInfo=function(t){this.detailGridInfoMap[t]=void 0},i.prototype.getDetailGridInfo=function(t){return this.detailGridInfoMap[t]},i.prototype.forEachDetailGridInfo=function(t){var e=0;Ke(this.detailGridInfoMap,function(r,n){q(n)&&(t(n,e),e++)})},i.prototype.getDataAsCsv=function(t){if(ge.assertRegistered(oe.CsvExportModule,"api.getDataAsCsv"))return this.csvCreator.getDataAsCsv(t)},i.prototype.exportDataAsCsv=function(t){ge.assertRegistered(oe.CsvExportModule,"api.exportDataAsCSv")&&this.csvCreator.exportDataAsCsv(t)},i.prototype.getExcelExportMode=function(t){var e=this.gridOptionsWrapper.getDefaultExportParams("excel"),r=Object.assign({exportMode:"xlsx"},e,t);return r.exportMode},i.prototype.getDataAsExcel=function(t){if(!!ge.assertRegistered(oe.ExcelExportModule,"api.getDataAsExcel")){var e=this.getExcelExportMode(t);if(this.excelCreator.getFactoryMode(e)===to.MULTI_SHEET){console.warn("AG Grid: The Excel Exporter is currently on Multi Sheet mode. End that operation by calling `api.getMultipleSheetAsExcel()` or `api.exportMultipleSheetsAsExcel()`");return}return this.excelCreator.getDataAsExcel(t)}},i.prototype.exportDataAsExcel=function(t){if(!!ge.assertRegistered(oe.ExcelExportModule,"api.exportDataAsExcel")){var e=this.getExcelExportMode(t);if(this.excelCreator.getFactoryMode(e)===to.MULTI_SHEET){console.warn("AG Grid: The Excel Exporter is currently on Multi Sheet mode. End that operation by calling `api.getMultipleSheetAsExcel()` or `api.exportMultipleSheetsAsExcel()`");return}this.excelCreator.exportDataAsExcel(t)}},i.prototype.getSheetDataForExcel=function(t){if(!!ge.assertRegistered(oe.ExcelExportModule,"api.getSheetDataForExcel")){var e=this.getExcelExportMode(t);return this.excelCreator.setFactoryMode(to.MULTI_SHEET,e),this.excelCreator.getSheetDataForExcel(t)}},i.prototype.getMultipleSheetsAsExcel=function(t){if(ge.assertRegistered(oe.ExcelExportModule,"api.getMultipleSheetsAsExcel"))return this.excelCreator.getMultipleSheetsAsExcel(t)},i.prototype.exportMultipleSheetsAsExcel=function(t){if(ge.assertRegistered(oe.ExcelExportModule,"api.exportMultipleSheetsAsExcel"))return this.excelCreator.exportMultipleSheetsAsExcel(t)},i.prototype.setEnterpriseDatasource=function(t){console.warn("AG Grid: since version 18.x, api.setEnterpriseDatasource() should be replaced with api.setServerSideDatasource()"),this.setServerSideDatasource(t)},i.prototype.setGridAriaProperty=function(t,e){if(!!t){var r=this.ctrlsService.getGridBodyCtrl().getGui(),n="aria-"+t;e===null?r.removeAttribute(n):r.setAttribute(n,e)}},i.prototype.setServerSideDatasource=function(t){this.serverSideRowModel?this.serverSideRowModel.setDatasource(t):console.warn("AG Grid: you can only use an enterprise datasource when gridOptions.rowModelType is '"+L.ROW_MODEL_TYPE_SERVER_SIDE+"'")},i.prototype.setDatasource=function(t){this.gridOptionsWrapper.isRowModelInfinite()?this.rowModel.setDatasource(t):console.warn("AG Grid: you can only use a datasource when gridOptions.rowModelType is '"+L.ROW_MODEL_TYPE_INFINITE+"'")},i.prototype.setViewportDatasource=function(t){this.gridOptionsWrapper.isRowModelViewport()?this.rowModel.setViewportDatasource(t):console.warn("AG Grid: you can only use a viewport datasource when gridOptions.rowModelType is '"+L.ROW_MODEL_TYPE_VIEWPORT+"'")},i.prototype.setRowData=function(t){var e=this.immutableService==null;if(e){console.warn("AG Grid: you can only set rowData when using the Client Side Row Model");return}this.immutableService.isActive()?this.immutableService.setRowData(t):(this.selectionService.reset(),this.clientSideRowModel.setRowData(t))},i.prototype.setFloatingTopRowData=function(t){console.warn("AG Grid: since v12, api.setFloatingTopRowData() is now api.setPinnedTopRowData()"),this.setPinnedTopRowData(t)},i.prototype.setFloatingBottomRowData=function(t){console.warn("AG Grid: since v12, api.setFloatingBottomRowData() is now api.setPinnedBottomRowData()"),this.setPinnedBottomRowData(t)},i.prototype.getFloatingTopRowCount=function(){return console.warn("AG Grid: since v12, api.getFloatingTopRowCount() is now api.getPinnedTopRowCount()"),this.getPinnedTopRowCount()},i.prototype.getFloatingBottomRowCount=function(){return console.warn("AG Grid: since v12, api.getFloatingBottomRowCount() is now api.getPinnedBottomRowCount()"),this.getPinnedBottomRowCount()},i.prototype.getFloatingTopRow=function(t){return console.warn("AG Grid: since v12, api.getFloatingTopRow() is now api.getPinnedTopRow()"),this.getPinnedTopRow(t)},i.prototype.getFloatingBottomRow=function(t){return console.warn("AG Grid: since v12, api.getFloatingBottomRow() is now api.getPinnedBottomRow()"),this.getPinnedBottomRow(t)},i.prototype.setPinnedTopRowData=function(t){this.pinnedRowModel.setPinnedTopRowData(t)},i.prototype.setPinnedBottomRowData=function(t){this.pinnedRowModel.setPinnedBottomRowData(t)},i.prototype.getPinnedTopRowCount=function(){return this.pinnedRowModel.getPinnedTopRowCount()},i.prototype.getPinnedBottomRowCount=function(){return this.pinnedRowModel.getPinnedBottomRowCount()},i.prototype.getPinnedTopRow=function(t){return this.pinnedRowModel.getPinnedTopRow(t)},i.prototype.getPinnedBottomRow=function(t){return this.pinnedRowModel.getPinnedBottomRow(t)},i.prototype.setColumnDefs=function(t,e){e===void 0&&(e="api"),this.columnModel.setColumnDefs(t,e)},i.prototype.setAutoGroupColumnDef=function(t,e){this.gridOptionsWrapper.setProperty("autoGroupColumnDef",t,!0)},i.prototype.setDefaultColDef=function(t,e){this.gridOptionsWrapper.setProperty("defaultColDef",t,!0)},i.prototype.expireValueCache=function(){this.valueCache.expire()},i.prototype.getVerticalPixelRange=function(){return this.gridBodyCtrl.getScrollFeature().getVScrollPosition()},i.prototype.getHorizontalPixelRange=function(){return this.gridBodyCtrl.getScrollFeature().getHScrollPosition()},i.prototype.setAlwaysShowHorizontalScroll=function(t){this.gridOptionsWrapper.setProperty("alwaysShowHorizontalScroll",t)},i.prototype.setAlwaysShowVerticalScroll=function(t){this.gridOptionsWrapper.setProperty("alwaysShowVerticalScroll",t)},i.prototype.refreshToolPanel=function(){!this.sideBarComp||this.sideBarComp.refresh()},i.prototype.refreshCells=function(t){if(t===void 0&&(t={}),Array.isArray(t)){console.warn("since AG Grid v11.1, refreshCells() now takes parameters, please see the documentation.");return}this.rowRenderer.refreshCells(t)},i.prototype.flashCells=function(t){t===void 0&&(t={}),this.rowRenderer.flashCells(t)},i.prototype.redrawRows=function(t){t===void 0&&(t={});var e=t?t.rowNodes:void 0;this.rowRenderer.redrawRows(e)},i.prototype.setFunctionsReadOnly=function(t){this.gridOptionsWrapper.setProperty("functionsReadOnly",t)},i.prototype.refreshHeader=function(){this.ctrlsService.getHeaderRowContainerCtrls().forEach(function(t){return t.refresh()})},i.prototype.isAnyFilterPresent=function(){return this.filterManager.isAnyFilterPresent()},i.prototype.isColumnFilterPresent=function(){return this.filterManager.isColumnFilterPresent()||this.filterManager.isAggregateFilterPresent()},i.prototype.isQuickFilterPresent=function(){return this.filterManager.isQuickFilterPresent()},i.prototype.getModel=function(){return this.rowModel},i.prototype.setRowNodeExpanded=function(t,e){t&&t.setExpanded(e)},i.prototype.onGroupExpandedOrCollapsed=function(t){me(this.clientSideRowModel)&&console.warn("AG Grid: cannot call onGroupExpandedOrCollapsed unless using normal row model"),q(t)&&console.warn("AG Grid: api.onGroupExpandedOrCollapsed - refreshFromIndex parameter is no longer used, the grid will refresh all rows"),this.clientSideRowModel.refreshModel({step:ze.MAP})},i.prototype.refreshInMemoryRowModel=function(t){console.warn("ag-grid: since version 18.x, api.refreshInMemoryRowModel() should be replaced with api.refreshClientSideRowModel()"),this.refreshClientSideRowModel(t)},i.prototype.refreshClientSideRowModel=function(t){me(this.clientSideRowModel)&&console.warn("cannot call refreshClientSideRowModel unless using normal row model");var e=ze.EVERYTHING,r={group:ze.EVERYTHING,filter:ze.FILTER,map:ze.MAP,aggregate:ze.AGGREGATE,sort:ze.SORT,pivot:ze.PIVOT};if(q(t)&&(e=r[t]),me(e)){console.error("AG Grid: invalid step "+t+", available steps are "+Object.keys(r).join(", "));return}var n=!this.gridOptionsWrapper.isSuppressAnimationFrame(),o={step:e,keepRenderedRows:!0,keepEditingRows:!0,animate:n};this.clientSideRowModel.refreshModel(o)},i.prototype.isAnimationFrameQueueEmpty=function(){return this.animationFrameService.isQueueEmpty()},i.prototype.flushAllAnimationFrames=function(){this.animationFrameService.flushAllFrames()},i.prototype.getRowNode=function(t){return this.rowModel.getRowNode(t)},i.prototype.getSizesForCurrentTheme=function(){return{rowHeight:this.gridOptionsWrapper.getRowHeightAsNumber(),headerHeight:this.gridOptionsWrapper.getHeaderHeight()}},i.prototype.expandAll=function(){this.clientSideRowModel?this.clientSideRowModel.expandOrCollapseAll(!0):this.serverSideRowModel?this.serverSideRowModel.expandAll(!0):console.warn("AG Grid: expandAll only works with Client Side Row Model and Server Side Row Model")},i.prototype.collapseAll=function(){this.clientSideRowModel?this.clientSideRowModel.expandOrCollapseAll(!1):this.serverSideRowModel?this.serverSideRowModel.expandAll(!1):console.warn("AG Grid: collapseAll only works with Client Side Row Model and Server Side Row Model")},i.prototype.getToolPanelInstance=function(t){if(!this.sideBarComp){console.warn("AG Grid: toolPanel is only available in AG Grid Enterprise");return}var e=this.sideBarComp.getToolPanelInstance(t);return ka(e)},i.prototype.addVirtualRowListener=function(t,e,r){typeof t!="string"&&console.warn("AG Grid: addVirtualRowListener is deprecated, please use addRenderedRowListener."),this.addRenderedRowListener(t,e,r)},i.prototype.addRenderedRowListener=function(t,e,r){t==="virtualRowSelected"&&console.warn(`AG Grid: event virtualRowSelected is deprecated, to register for individual row
                selection events, add a listener directly to the row node.`),this.rowRenderer.addRenderedRowListener(t,e,r)},i.prototype.setQuickFilter=function(t){this.filterManager.setQuickFilter(t)},i.prototype.selectIndex=function(t,e,r){console.warn("AG Grid: do not use api for selection, call node.setSelected(value) instead"),r&&console.warn("AG Grid: suppressEvents is no longer supported, stop listening for the event if you no longer want it"),this.selectionService.selectIndex(t,e)},i.prototype.deselectIndex=function(t,e){e===void 0&&(e=!1),console.warn("AG Grid: do not use api for selection, call node.setSelected(value) instead"),e&&console.warn("AG Grid: suppressEvents is no longer supported, stop listening for the event if you no longer want it"),this.selectionService.deselectIndex(t)},i.prototype.selectNode=function(t,e,r){e===void 0&&(e=!1),r===void 0&&(r=!1),console.warn("AG Grid: API for selection is deprecated, call node.setSelected(value) instead"),r&&console.warn("AG Grid: suppressEvents is no longer supported, stop listening for the event if you no longer want it"),t.setSelectedParams({newValue:!0,clearSelection:!e})},i.prototype.deselectNode=function(t,e){e===void 0&&(e=!1),console.warn("AG Grid: API for selection is deprecated, call node.setSelected(value) instead"),e&&console.warn("AG Grid: suppressEvents is no longer supported, stop listening for the event if you no longer want it"),t.setSelectedParams({newValue:!1})},i.prototype.selectAll=function(){this.selectionService.selectAllRowNodes()},i.prototype.deselectAll=function(){this.selectionService.deselectAllRowNodes()},i.prototype.selectAllFiltered=function(){this.selectionService.selectAllRowNodes(!0)},i.prototype.deselectAllFiltered=function(){this.selectionService.deselectAllRowNodes(!0)},i.prototype.recomputeAggregates=function(){me(this.clientSideRowModel)&&console.warn("cannot call recomputeAggregates unless using normal row model"),console.warn("recomputeAggregates is deprecated, please call api.refreshClientSideRowModel('aggregate') instead"),this.clientSideRowModel.refreshModel({step:ze.AGGREGATE})},i.prototype.sizeColumnsToFit=function(){this.gridBodyCtrl.sizeColumnsToFit()},i.prototype.showLoadingOverlay=function(){this.overlayWrapperComp.showLoadingOverlay()},i.prototype.showNoRowsOverlay=function(){this.overlayWrapperComp.showNoRowsOverlay()},i.prototype.hideOverlay=function(){this.overlayWrapperComp.hideOverlay()},i.prototype.isNodeSelected=function(t){return console.warn("AG Grid: no need to call api.isNodeSelected(), just call node.isSelected() instead"),t.isSelected()},i.prototype.getSelectedNodesById=function(){return console.error("AG Grid: since version 3.4, getSelectedNodesById no longer exists, use getSelectedNodes() instead"),null},i.prototype.getSelectedNodes=function(){return this.selectionService.getSelectedNodes()},i.prototype.getSelectedRows=function(){return this.selectionService.getSelectedRows()},i.prototype.getBestCostNodeSelection=function(){return this.selectionService.getBestCostNodeSelection()},i.prototype.getRenderedNodes=function(){return this.rowRenderer.getRenderedNodes()},i.prototype.ensureColIndexVisible=function(t){console.warn("AG Grid: ensureColIndexVisible(index) no longer supported, use ensureColumnVisible(colKey) instead.")},i.prototype.ensureColumnVisible=function(t,e){e===void 0&&(e="auto"),this.gridBodyCtrl.getScrollFeature().ensureColumnVisible(t,e)},i.prototype.ensureIndexVisible=function(t,e){this.gridBodyCtrl.getScrollFeature().ensureIndexVisible(t,e)},i.prototype.ensureNodeVisible=function(t,e){e===void 0&&(e=null),this.gridBodyCtrl.getScrollFeature().ensureNodeVisible(t,e)},i.prototype.forEachLeafNode=function(t){me(this.clientSideRowModel)&&console.warn("cannot call forEachNode unless using normal row model"),this.clientSideRowModel.forEachLeafNode(t)},i.prototype.forEachNode=function(t){this.rowModel.forEachNode(t)},i.prototype.forEachNodeAfterFilter=function(t){me(this.clientSideRowModel)&&console.warn("cannot call forEachNodeAfterFilter unless using normal row model"),this.clientSideRowModel.forEachNodeAfterFilter(t)},i.prototype.forEachNodeAfterFilterAndSort=function(t){me(this.clientSideRowModel)&&console.warn("cannot call forEachNodeAfterFilterAndSort unless using normal row model"),this.clientSideRowModel.forEachNodeAfterFilterAndSort(t)},i.prototype.getFilterInstance=function(t,e){var r=this.getFilterInstanceImpl(t,function(o){if(!!e){var a=ka(o);e(a)}}),n=ka(r);return n},i.prototype.getFilterInstanceImpl=function(t,e){var r=this.columnModel.getPrimaryColumn(t);if(!!r){var n=this.filterManager.getFilterComponent(r,"NO_UI"),o=n&&n.resolveNow(null,function(a){return a});return o?setTimeout(e,0,o):n&&n.then(function(a){e(a)}),o}},i.prototype.destroyFilter=function(t){var e=this.columnModel.getPrimaryColumn(t);if(e)return this.filterManager.destroyFilter(e,"filterDestroyed")},i.prototype.getStatusPanel=function(t){if(!!this.statusBarService){var e=this.statusBarService.getStatusPanel(t);return ka(e)}},i.prototype.getColumnDef=function(t){var e=this.columnModel.getPrimaryColumn(t);return e?e.getColDef():null},i.prototype.getColumnDefs=function(){return this.columnModel.getColumnDefs()},i.prototype.onFilterChanged=function(){this.filterManager.onFilterChanged()},i.prototype.onSortChanged=function(){this.sortController.onSortChanged("api")},i.prototype.setFilterModel=function(t){this.filterManager.setFilterModel(t)},i.prototype.getFilterModel=function(){return this.filterManager.getFilterModel()},i.prototype.getFocusedCell=function(){return this.focusService.getFocusedCell()},i.prototype.clearFocusedCell=function(){return this.focusService.clearFocusedCell()},i.prototype.setFocusedCell=function(t,e,r){this.focusService.setFocusedCell(t,e,r,!0)},i.prototype.setSuppressRowDrag=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_SUPPRESS_ROW_DRAG,t)},i.prototype.setSuppressMoveWhenRowDragging=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_SUPPRESS_MOVE_WHEN_ROW_DRAG,t)},i.prototype.setSuppressRowClickSelection=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_SUPPRESS_ROW_CLICK_SELECTION,t)},i.prototype.addRowDropZone=function(t){this.gridBodyCtrl.getRowDragFeature().addRowDropZone(t)},i.prototype.removeRowDropZone=function(t){var e=this.dragAndDropService.findExternalZone(t);e&&this.dragAndDropService.removeDropTarget(e)},i.prototype.getRowDropZoneParams=function(t){return this.gridBodyCtrl.getRowDragFeature().getRowDropZone(t)},i.prototype.setHeaderHeight=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_HEADER_HEIGHT,t)},i.prototype.setDomLayout=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_DOM_LAYOUT,t)},i.prototype.setEnableCellTextSelection=function(t){this.gridBodyCtrl.setCellTextSelection(t)},i.prototype.setFillHandleDirection=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_FILL_HANDLE_DIRECTION,t)},i.prototype.setGroupHeaderHeight=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GROUP_HEADER_HEIGHT,t)},i.prototype.setFloatingFiltersHeight=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_FLOATING_FILTERS_HEIGHT,t)},i.prototype.setPivotHeaderHeight=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_PIVOT_HEADER_HEIGHT,t)},i.prototype.setPivotGroupHeaderHeight=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_PIVOT_GROUP_HEADER_HEIGHT,t)},i.prototype.setIsExternalFilterPresent=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_IS_EXTERNAL_FILTER_PRESENT,t)},i.prototype.setDoesExternalFilterPass=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_DOES_EXTERNAL_FILTER_PASS,t)},i.prototype.setNavigateToNextCell=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_NAVIGATE_TO_NEXT_CELL,t)},i.prototype.setTabToNextCell=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_TAB_TO_NEXT_CELL,t)},i.prototype.setTabToNextHeader=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_TAB_TO_NEXT_HEADER,t)},i.prototype.setNavigateToNextHeader=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_NAVIGATE_TO_NEXT_HEADER,t)},i.prototype.setGroupRowAggNodes=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GROUP_ROW_AGG_NODES,t)},i.prototype.setGetGroupRowAgg=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_GROUP_ROW_AGG,t)},i.prototype.setGetBusinessKeyForNode=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_BUSINESS_KEY_FOR_NODE,t)},i.prototype.setGetChildCount=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_CHILD_COUNT,t)},i.prototype.setProcessRowPostCreate=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_PROCESS_ROW_POST_CREATE,t)},i.prototype.setGetRowNodeId=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_ROW_NODE_ID,t)},i.prototype.setGetRowId=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_ROW_ID,t)},i.prototype.setGetRowClass=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_ROW_CLASS,t)},i.prototype.setIsFullWidthCell=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_IS_FULL_WIDTH_CELL,t)},i.prototype.setIsFullWidthRow=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_IS_FULL_WIDTH_ROW,t)},i.prototype.setIsRowSelectable=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_IS_ROW_SELECTABLE,t)},i.prototype.setIsRowMaster=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_IS_ROW_MASTER,t)},i.prototype.setPostSort=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_POST_SORT,t)},i.prototype.setPostSortRows=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_POST_SORT_ROWS,t)},i.prototype.setGetDocument=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_DOCUMENT,t)},i.prototype.setGetContextMenuItems=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_CONTEXT_MENU_ITEMS,t)},i.prototype.setGetMainMenuItems=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_MAIN_MENU_ITEMS,t)},i.prototype.setProcessCellForClipboard=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_PROCESS_CELL_FOR_CLIPBOARD,t)},i.prototype.setSendToClipboard=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_SEND_TO_CLIPBOARD,t)},i.prototype.setProcessCellFromClipboard=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_PROCESS_CELL_FROM_CLIPBOARD,t)},i.prototype.setProcessSecondaryColDef=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_PROCESS_TO_SECONDARY_COLDEF,t)},i.prototype.setProcessSecondaryColGroupDef=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_PROCESS_SECONDARY_COL_GROUP_DEF,t)},i.prototype.setPostProcessPopup=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_POST_PROCESS_POPUP,t)},i.prototype.setDefaultGroupOrderComparator=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_DEFAULT_GROUP_ORDER_COMPARATOR,t)},i.prototype.setInitialGroupOrderComparator=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_INITIAL_GROUP_ORDER_COMPARATOR,t)},i.prototype.setGetChartToolbarItems=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_CHART_TOOLBAR_ITEMS,t)},i.prototype.setPaginationNumberFormatter=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_PAGINATION_NUMBER_FORMATTER,t)},i.prototype.setGetServerSideStoreParams=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_SERVER_SIDE_STORE_PARAMS,t)},i.prototype.setIsServerSideGroupOpenByDefault=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_IS_SERVER_SIDE_GROUPS_OPEN_BY_DEFAULT,t)},i.prototype.setIsApplyServerSideTransaction=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_IS_APPLY_SERVER_SIDE_TRANSACTION,t)},i.prototype.setIsServerSideGroup=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_IS_SERVER_SIDE_GROUP,t)},i.prototype.setGetServerSideGroupKey=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_SERVER_SIDE_GROUP_KEY,t)},i.prototype.setGetRowStyle=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_ROW_STYLE,t)},i.prototype.setGetRowHeight=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GET_ROW_HEIGHT,t)},i.prototype.isSideBarVisible=function(){return this.sideBarComp?this.sideBarComp.isDisplayed():!1},i.prototype.setSideBarVisible=function(t){if(!this.sideBarComp){t&&console.warn("AG Grid: sideBar is not loaded");return}this.sideBarComp.setDisplayed(t)},i.prototype.setSideBarPosition=function(t){if(!this.sideBarComp){console.warn("AG Grid: sideBar is not loaded");return}this.sideBarComp.setSideBarPosition(t)},i.prototype.openToolPanel=function(t){if(!this.sideBarComp){console.warn("AG Grid: toolPanel is only available in AG Grid Enterprise");return}this.sideBarComp.openToolPanel(t)},i.prototype.closeToolPanel=function(){if(!this.sideBarComp){console.warn("AG Grid: toolPanel is only available in AG Grid Enterprise");return}this.sideBarComp.close()},i.prototype.getOpenedToolPanel=function(){return this.sideBarComp?this.sideBarComp.openedItem():null},i.prototype.getSideBar=function(){return this.gridOptionsWrapper.getSideBar()},i.prototype.setSideBar=function(t){this.gridOptionsWrapper.setProperty("sideBar",RE.parse(t))},i.prototype.setSuppressClipboardPaste=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_SUPPRESS_CLIPBOARD_PASTE,t)},i.prototype.isToolPanelShowing=function(){return this.sideBarComp.isToolPanelShowing()},i.prototype.doLayout=function(){var t="AG Grid - since version 25.1, doLayout was taken out, as it's not needed. The grid responds to grid size changes automatically";Ye(function(){return console.warn(t)},"doLayoutDeprecated")},i.prototype.resetRowHeights=function(){if(q(this.clientSideRowModel)){if(this.columnModel.isAutoRowHeightActive()){console.warn("AG Grid: calling gridApi.resetRowHeights() makes no sense when using Auto Row Height.");return}this.clientSideRowModel.resetRowHeights()}},i.prototype.setGroupRemoveSingleChildren=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GROUP_REMOVE_SINGLE_CHILDREN,t)},i.prototype.setGroupRemoveLowestSingleChildren=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_GROUP_REMOVE_LOWEST_SINGLE_CHILDREN,t)},i.prototype.onRowHeightChanged=function(){this.clientSideRowModel?this.clientSideRowModel.onRowHeightChanged():this.serverSideRowModel&&this.serverSideRowModel.onRowHeightChanged()},i.prototype.getValue=function(t,e){var r=this.columnModel.getPrimaryColumn(t);return me(r)&&(r=this.columnModel.getGridColumn(t)),me(r)?null:this.valueService.getValue(r,e)},i.prototype.addEventListener=function(t,e){var r=this.gridOptionsWrapper.useAsyncEvents();this.eventService.addEventListener(t,e,r)},i.prototype.addGlobalListener=function(t){var e=this.gridOptionsWrapper.useAsyncEvents();this.eventService.addGlobalListener(t,e)},i.prototype.removeEventListener=function(t,e){var r=this.gridOptionsWrapper.useAsyncEvents();this.eventService.removeEventListener(t,e,r)},i.prototype.removeGlobalListener=function(t){var e=this.gridOptionsWrapper.useAsyncEvents();this.eventService.removeGlobalListener(t,e)},i.prototype.dispatchEvent=function(t){this.eventService.dispatchEvent(t)},i.prototype.destroy=function(){if(!this.destroyCalled){this.destroyCalled=!0;var t=this.ctrlsService.getGridCtrl();t&&t.destroyGridUi(),this.context.destroy()}},i.prototype.cleanDownReferencesToAvoidMemoryLeakInCaseApplicationIsKeepingReferenceToDestroyedGrid=function(){setTimeout(oO.bind(window,this,"Grid API"),100)},i.prototype.warnIfDestroyed=function(t){return this.destroyCalled&&console.warn("AG Grid: Grid API method "+t+" was called on a grid that was destroyed."),this.destroyCalled},i.prototype.resetQuickFilter=function(){this.warnIfDestroyed("resetQuickFilter")||this.rowModel.forEachNode(function(t){return t.quickFilterAggregateText=null})},i.prototype.getRangeSelections=function(){return console.warn(`AG Grid: in v20.1.x, api.getRangeSelections() is gone, please use getCellRanges() instead.
        We had to change how cell selections works a small bit to allow charting to integrate. The return type of
        getCellRanges() is a bit different, please check the AG Grid documentation.`),null},i.prototype.getCellRanges=function(){return this.rangeService?this.rangeService.getCellRanges():(console.warn("AG Grid: cell range selection is only available in AG Grid Enterprise"),null)},i.prototype.camelCaseToHumanReadable=function(t){return EC(t)},i.prototype.addRangeSelection=function(t){console.warn("AG Grid: As of version 21.x, range selection changed slightly to allow charting integration. Please call api.addCellRange() instead of api.addRangeSelection()")},i.prototype.addCellRange=function(t){this.rangeService||console.warn("AG Grid: cell range selection is only available in AG Grid Enterprise"),this.rangeService.addCellRange(t)},i.prototype.clearRangeSelection=function(){this.rangeService||console.warn("AG Grid: cell range selection is only available in AG Grid Enterprise"),this.rangeService.removeAllCellRanges()},i.prototype.undoCellEditing=function(){this.undoRedoService.undo()},i.prototype.redoCellEditing=function(){this.undoRedoService.redo()},i.prototype.getCurrentUndoSize=function(){return this.undoRedoService.getCurrentUndoStackSize()},i.prototype.getCurrentRedoSize=function(){return this.undoRedoService.getCurrentRedoStackSize()},i.prototype.getChartModels=function(){if(ge.assertRegistered(oe.RangeSelectionModule,"api.getChartModels")&&ge.assertRegistered(oe.GridChartsModule,"api.getChartModels"))return this.chartService.getChartModels()},i.prototype.getChartRef=function(t){if(ge.assertRegistered(oe.RangeSelectionModule,"api.getChartRef")&&ge.assertRegistered(oe.GridChartsModule,"api.getChartRef"))return this.chartService.getChartRef(t)},i.prototype.getChartImageDataURL=function(t){if(ge.assertRegistered(oe.RangeSelectionModule,"api.getChartImageDataURL")&&ge.assertRegistered(oe.GridChartsModule,"api.getChartImageDataURL"))return this.chartService.getChartImageDataURL(t)},i.prototype.createRangeChart=function(t){if(ge.assertRegistered(oe.RangeSelectionModule,"api.createRangeChart")&&ge.assertRegistered(oe.GridChartsModule,"api.createRangeChart"))return this.chartService.createRangeChart(t)},i.prototype.createCrossFilterChart=function(t){if(ge.assertRegistered(oe.RangeSelectionModule,"api.createCrossFilterChart")&&ge.assertRegistered(oe.GridChartsModule,"api.createCrossFilterChart"))return this.chartService.createCrossFilterChart(t)},i.prototype.restoreChart=function(t,e){if(ge.assertRegistered(oe.RangeSelectionModule,"api.restoreChart")&&ge.assertRegistered(oe.GridChartsModule,"api.restoreChart"))return this.chartService.restoreChart(t,e)},i.prototype.createPivotChart=function(t){if(ge.assertRegistered(oe.RangeSelectionModule,"api.createPivotChart")&&ge.assertRegistered(oe.GridChartsModule,"api.createPivotChart"))return this.chartService.createPivotChart(t)},i.prototype.copySelectedRowsToClipboard=function(t){this.clipboardService||console.warn("AG Grid: clipboard is only available in AG Grid Enterprise"),this.clipboardService.copySelectedRowsToClipboard(t)},i.prototype.copySelectedRangeToClipboard=function(t){this.clipboardService||console.warn("AG Grid: clipboard is only available in AG Grid Enterprise"),this.clipboardService.copySelectedRangeToClipboard(t)},i.prototype.copySelectedRangeDown=function(){this.clipboardService||console.warn("AG Grid: clipboard is only available in AG Grid Enterprise"),this.clipboardService.copyRangeDown()},i.prototype.showColumnMenuAfterButtonClick=function(t,e){var r=this.columnModel.getGridColumn(t);this.menuFactory.showMenuAfterButtonClick(r,e,"columnMenu")},i.prototype.showColumnMenuAfterMouseClick=function(t,e){var r=this.columnModel.getGridColumn(t);if(r||(r=this.columnModel.getPrimaryColumn(t)),!r){console.error("AG Grid: column '"+t+"' not found");return}this.menuFactory.showMenuAfterMouseEvent(r,e)},i.prototype.hidePopupMenu=function(){this.contextMenuFactory&&this.contextMenuFactory.hideActiveMenu(),this.menuFactory.hideActiveMenu()},i.prototype.setPopupParent=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_POPUP_PARENT,t)},i.prototype.tabToNextCell=function(t){return this.navigationService.tabToNextCell(!1,t)},i.prototype.tabToPreviousCell=function(t){return this.navigationService.tabToNextCell(!0,t)},i.prototype.getCellRendererInstances=function(t){t===void 0&&(t={});var e=this.rowRenderer.getCellRendererInstances(t),r=e.map(ka);return r},i.prototype.getCellEditorInstances=function(t){t===void 0&&(t={});var e=this.rowRenderer.getCellEditorInstances(t),r=e.map(ka);return r},i.prototype.getEditingCells=function(){return this.rowRenderer.getEditingCells()},i.prototype.stopEditing=function(t){t===void 0&&(t=!1),this.rowRenderer.stopEditing(t)},i.prototype.startEditingCell=function(t){var e=this.columnModel.getGridColumn(t.colKey);if(!e){console.warn("AG Grid: no column found for "+t.colKey);return}var r={rowIndex:t.rowIndex,rowPinned:t.rowPinned||null,column:e},n=t.rowPinned==null;n&&this.gridBodyCtrl.getScrollFeature().ensureIndexVisible(t.rowIndex);var o=this.navigationService.getCellByPosition(r);!o||o.startRowOrCellEdit(t.key,t.charPress)},i.prototype.addAggFunc=function(t,e){this.aggFuncService&&this.aggFuncService.addAggFunc(t,e)},i.prototype.addAggFuncs=function(t){this.aggFuncService&&this.aggFuncService.addAggFuncs(t)},i.prototype.clearAggFuncs=function(){this.aggFuncService&&this.aggFuncService.clear()},i.prototype.applyServerSideTransaction=function(t){if(!this.serverSideTransactionManager){console.warn("AG Grid: Cannot apply Server Side Transaction if not using the Server Side Row Model.");return}return this.serverSideTransactionManager.applyTransaction(t)},i.prototype.applyServerSideTransactionAsync=function(t,e){if(!this.serverSideTransactionManager){console.warn("AG Grid: Cannot apply Server Side Transaction if not using the Server Side Row Model.");return}return this.serverSideTransactionManager.applyTransactionAsync(t,e)},i.prototype.retryServerSideLoads=function(){if(!this.serverSideRowModel){console.warn("AG Grid: API retryServerSideLoads() can only be used when using Server-Side Row Model.");return}this.serverSideRowModel.retryLoads()},i.prototype.flushServerSideAsyncTransactions=function(){if(!this.serverSideTransactionManager){console.warn("AG Grid: Cannot flush Server Side Transaction if not using the Server Side Row Model.");return}return this.serverSideTransactionManager.flushAsyncTransactions()},i.prototype.applyTransaction=function(t){if(!this.clientSideRowModel){console.error("AG Grid: updateRowData() only works with ClientSideRowModel. Working with InfiniteRowModel was deprecated in v23.1 and removed in v24.1");return}var e=this.clientSideRowModel.updateRowData(t);return this.rowRenderer.refreshFullWidthRows(e.update),this.gridOptionsWrapper.isSuppressChangeDetection()||this.rowRenderer.refreshCells(),e},i.prototype.setDeltaSort=function(t){this.gridOptionsWrapper.setProperty("deltaSort",t)},i.prototype.updateRowData=function(t){var e="AG Grid: as of v23.1, grid API updateRowData(transaction) is now called applyTransaction(transaction). updateRowData is deprecated and will be removed in a future major release.";return Ye(function(){return console.warn(e)},"updateRowData deprecated"),this.applyTransaction(t)},i.prototype.applyTransactionAsync=function(t,e){if(!this.clientSideRowModel){console.error("AG Grid: api.applyTransactionAsync() only works with ClientSideRowModel.");return}this.clientSideRowModel.batchUpdateRowData(t,e)},i.prototype.flushAsyncTransactions=function(){if(!this.clientSideRowModel){console.error("AG Grid: api.applyTransactionAsync() only works with ClientSideRowModel.");return}this.clientSideRowModel.flushAsyncTransactions()},i.prototype.batchUpdateRowData=function(t,e){var r="AG Grid: as of v23.1, grid API batchUpdateRowData(transaction, callback) is now called applyTransactionAsync(transaction, callback). batchUpdateRowData is deprecated and will be removed in a future major release.";Ye(function(){return console.warn(r)},"batchUpdateRowData deprecated"),this.applyTransactionAsync(t,e)},i.prototype.insertItemsAtIndex=function(t,e,r){console.warn("AG Grid: insertItemsAtIndex() is deprecated, use updateRowData(transaction) instead."),this.updateRowData({add:e,addIndex:t,update:null,remove:null})},i.prototype.removeItems=function(t,e){console.warn("AG Grid: removeItems() is deprecated, use updateRowData(transaction) instead.");var r=t.map(function(n){return n.data});this.updateRowData({add:null,addIndex:null,update:null,remove:r})},i.prototype.addItems=function(t,e){console.warn("AG Grid: addItems() is deprecated, use updateRowData(transaction) instead."),this.updateRowData({add:t,addIndex:null,update:null,remove:null})},i.prototype.refreshVirtualPageCache=function(){console.warn("AG Grid: refreshVirtualPageCache() is now called refreshInfiniteCache(), please call refreshInfiniteCache() instead"),this.refreshInfiniteCache()},i.prototype.refreshInfinitePageCache=function(){console.warn("AG Grid: refreshInfinitePageCache() is now called refreshInfiniteCache(), please call refreshInfiniteCache() instead"),this.refreshInfiniteCache()},i.prototype.refreshInfiniteCache=function(){this.infiniteRowModel?this.infiniteRowModel.refreshCache():console.warn("AG Grid: api.refreshInfiniteCache is only available when rowModelType='infinite'.")},i.prototype.purgeVirtualPageCache=function(){console.warn("AG Grid: purgeVirtualPageCache() is now called purgeInfiniteCache(), please call purgeInfiniteCache() instead"),this.purgeInfinitePageCache()},i.prototype.purgeInfinitePageCache=function(){console.warn("AG Grid: purgeInfinitePageCache() is now called purgeInfiniteCache(), please call purgeInfiniteCache() instead"),this.purgeInfiniteCache()},i.prototype.purgeInfiniteCache=function(){this.infiniteRowModel?this.infiniteRowModel.purgeCache():console.warn("AG Grid: api.purgeInfiniteCache is only available when rowModelType='infinite'.")},i.prototype.purgeEnterpriseCache=function(t){console.warn("ag-grid: since version 18.x, api.purgeEnterpriseCache() should be replaced with api.purgeServerSideCache()"),this.purgeServerSideCache(t)},i.prototype.purgeServerSideCache=function(t){t===void 0&&(t=[]),this.serverSideRowModel?(console.warn("AG Grid: since v25.0, api.purgeServerSideCache is deprecated. Please use api.refreshServerSideStore({purge: true}) instead."),this.refreshServerSideStore({route:t,purge:!0})):console.warn("AG Grid: api.purgeServerSideCache is only available when rowModelType='serverSide'.")},i.prototype.refreshServerSideStore=function(t){this.serverSideRowModel?this.serverSideRowModel.refreshStore(t):console.warn("AG Grid: api.refreshServerSideStore is only available when rowModelType='serverSide'.")},i.prototype.getServerSideStoreState=function(){return this.serverSideRowModel?this.serverSideRowModel.getStoreState():(console.warn("AG Grid: api.getServerSideStoreState is only available when rowModelType='serverSide'."),[])},i.prototype.getVirtualRowCount=function(){return console.warn("AG Grid: getVirtualRowCount() is now called getInfiniteRowCount(), please call getInfiniteRowCount() instead"),this.getInfiniteRowCount()},i.prototype.getInfiniteRowCount=function(){if(this.infiniteRowModel)return this.infiniteRowModel.getRowCount();console.warn("AG Grid: api.getVirtualRowCount is only available when rowModelType='virtual'.")},i.prototype.isMaxRowFound=function(){return console.warn("AG Grid: api.isLastRowIndexKnown is deprecated, please use api.isLastRowIndexKnown()"),this.isLastRowIndexKnown()},i.prototype.isLastRowIndexKnown=function(){if(this.infiniteRowModel)return this.infiniteRowModel.isLastRowIndexKnown();console.warn("AG Grid: api.isMaxRowFound is only available when rowModelType='virtual'.")},i.prototype.setVirtualRowCount=function(t,e){console.warn("AG Grid: setVirtualRowCount() is now called setInfiniteRowCount(), please call setInfiniteRowCount() instead"),this.setRowCount(t,e)},i.prototype.setInfiniteRowCount=function(t,e){console.warn("AG Grid: setInfiniteRowCount() is now called setRowCount(), please call setRowCount() instead"),this.setRowCount(t,e)},i.prototype.setRowCount=function(t,e){this.infiniteRowModel?this.infiniteRowModel.setRowCount(t,e):console.warn("AG Grid: api.setRowCount is only available for Infinite Row Model.")},i.prototype.getVirtualPageState=function(){return console.warn("AG Grid: getVirtualPageState() is now called getCacheBlockState(), please call getCacheBlockState() instead"),this.getCacheBlockState()},i.prototype.getInfinitePageState=function(){return console.warn("AG Grid: getInfinitePageState() is now called getCacheBlockState(), please call getCacheBlockState() instead"),this.getCacheBlockState()},i.prototype.getCacheBlockState=function(){return this.rowNodeBlockLoader.getBlockState()},i.prototype.checkGridSize=function(){console.warn("in AG Grid v25.2.0, checkGridSize() was removed, as it was legacy and didn't do anything uesful.")},i.prototype.getFirstRenderedRow=function(){return console.warn("in AG Grid v12, getFirstRenderedRow() was renamed to getFirstDisplayedRow()"),this.getFirstDisplayedRow()},i.prototype.getFirstDisplayedRow=function(){return this.rowRenderer.getFirstVirtualRenderedRow()},i.prototype.getLastRenderedRow=function(){return console.warn("in AG Grid v12, getLastRenderedRow() was renamed to getLastDisplayedRow()"),this.getLastDisplayedRow()},i.prototype.getLastDisplayedRow=function(){return this.rowRenderer.getLastVirtualRenderedRow()},i.prototype.getDisplayedRowAtIndex=function(t){return this.rowModel.getRow(t)},i.prototype.getDisplayedRowCount=function(){return this.rowModel.getRowCount()},i.prototype.paginationIsLastPageFound=function(){return this.paginationProxy.isLastPageFound()},i.prototype.paginationGetPageSize=function(){return this.paginationProxy.getPageSize()},i.prototype.paginationSetPageSize=function(t){this.gridOptionsWrapper.setProperty("paginationPageSize",t)},i.prototype.paginationGetCurrentPage=function(){return this.paginationProxy.getCurrentPage()},i.prototype.paginationGetTotalPages=function(){return this.paginationProxy.getTotalPages()},i.prototype.paginationGetRowCount=function(){return this.paginationProxy.getMasterRowCount()},i.prototype.paginationGoToNextPage=function(){this.paginationProxy.goToNextPage()},i.prototype.paginationGoToPreviousPage=function(){this.paginationProxy.goToPreviousPage()},i.prototype.paginationGoToFirstPage=function(){this.paginationProxy.goToFirstPage()},i.prototype.paginationGoToLastPage=function(){this.paginationProxy.goToLastPage()},i.prototype.paginationGoToPage=function(t){this.paginationProxy.goToPage(t)},i.prototype.setRowClass=function(t){this.gridOptionsWrapper.setProperty(ce.PROP_ROW_CLASS,t)},Ue([De("immutableService")],i.prototype,"immutableService",void 0),Ue([De("csvCreator")],i.prototype,"csvCreator",void 0),Ue([De("excelCreator")],i.prototype,"excelCreator",void 0),Ue([y("rowRenderer")],i.prototype,"rowRenderer",void 0),Ue([y("navigationService")],i.prototype,"navigationService",void 0),Ue([y("filterManager")],i.prototype,"filterManager",void 0),Ue([y("columnModel")],i.prototype,"columnModel",void 0),Ue([y("selectionService")],i.prototype,"selectionService",void 0),Ue([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),Ue([y("valueService")],i.prototype,"valueService",void 0),Ue([y("alignedGridsService")],i.prototype,"alignedGridsService",void 0),Ue([y("eventService")],i.prototype,"eventService",void 0),Ue([y("pinnedRowModel")],i.prototype,"pinnedRowModel",void 0),Ue([y("context")],i.prototype,"context",void 0),Ue([y("rowModel")],i.prototype,"rowModel",void 0),Ue([y("sortController")],i.prototype,"sortController",void 0),Ue([y("paginationProxy")],i.prototype,"paginationProxy",void 0),Ue([y("focusService")],i.prototype,"focusService",void 0),Ue([y("dragAndDropService")],i.prototype,"dragAndDropService",void 0),Ue([De("rangeService")],i.prototype,"rangeService",void 0),Ue([De("clipboardService")],i.prototype,"clipboardService",void 0),Ue([De("aggFuncService")],i.prototype,"aggFuncService",void 0),Ue([y("menuFactory")],i.prototype,"menuFactory",void 0),Ue([De("contextMenuFactory")],i.prototype,"contextMenuFactory",void 0),Ue([y("valueCache")],i.prototype,"valueCache",void 0),Ue([y("animationFrameService")],i.prototype,"animationFrameService",void 0),Ue([De("statusBarService")],i.prototype,"statusBarService",void 0),Ue([De("chartService")],i.prototype,"chartService",void 0),Ue([De("undoRedoService")],i.prototype,"undoRedoService",void 0),Ue([De("rowNodeBlockLoader")],i.prototype,"rowNodeBlockLoader",void 0),Ue([De("ssrmTransactionManager")],i.prototype,"serverSideTransactionManager",void 0),Ue([y("ctrlsService")],i.prototype,"ctrlsService",void 0),Ue([De("frameworkComponentWrapper")],i.prototype,"frameworkComponentWrapper",void 0),Ue([B],i.prototype,"init",null),Ue([nt],i.prototype,"cleanDownReferencesToAvoidMemoryLeakInCaseApplicationIsKeepingReferenceToDestroyedGrid",null),i=Ue([Q("gridApi")],i),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var vk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),mk=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},af=function(i){vk(t,i);function t(e,r,n,o){var a=i.call(this)||this;return a.columnOrGroup=e,a.eCell=r,a.ariaEl=a.eCell.querySelector("[role=columnheader]")||a.eCell,a.colsSpanning=o,a.beans=n,a}return t.prototype.setColsSpanning=function(e){this.colsSpanning=e,this.onLeftChanged()},t.prototype.getColumnOrGroup=function(){return this.beans.gridOptionsWrapper.isEnableRtl()&&this.colsSpanning?rt(this.colsSpanning):this.columnOrGroup},t.prototype.postConstruct=function(){this.addManagedListener(this.columnOrGroup,Se.EVENT_LEFT_CHANGED,this.onLeftChanged.bind(this)),this.setLeftFirstTime(),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,this.onLeftChanged.bind(this)),this.addManagedListener(this.beans.gridOptionsWrapper,ce.PROP_DOM_LAYOUT,this.onLeftChanged.bind(this))},t.prototype.setLeftFirstTime=function(){var e=this.beans.gridOptionsWrapper.isSuppressColumnMoveAnimation(),r=q(this.columnOrGroup.getOldLeft()),n=this.beans.columnAnimationService.isActive()&&r&&!e;n?this.animateInLeft():this.onLeftChanged()},t.prototype.animateInLeft=function(){var e=this,r=this.getColumnOrGroup(),n=r.getLeft(),o=r.getOldLeft(),a=this.modifyLeftForPrintLayout(r,o),s=this.modifyLeftForPrintLayout(r,n);this.setLeft(a),this.actualLeft=s,this.beans.columnAnimationService.executeNextVMTurn(function(){e.actualLeft===s&&e.setLeft(s)})},t.prototype.onLeftChanged=function(){var e=this.getColumnOrGroup(),r=e.getLeft();this.actualLeft=this.modifyLeftForPrintLayout(e,r),this.setLeft(this.actualLeft)},t.prototype.modifyLeftForPrintLayout=function(e,r){var n=this.beans.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_PRINT;if(!n||e.getPinned()===L.PINNED_LEFT)return r;var o=this.beans.columnModel.getDisplayedColumnsLeftWidth();if(e.getPinned()===L.PINNED_RIGHT){var a=this.beans.columnModel.getBodyContainerWidth();return o+a+r}return o+r},t.prototype.setLeft=function(e){q(e)&&(this.eCell.style.left=e+"px");var r;if(this.columnOrGroup instanceof Se)r=this.columnOrGroup;else{var n=this.columnOrGroup,o=n.getLeafColumns();if(!o.length)return;o.length>1&&XO(this.ariaEl,o.length),r=o[0]}var a=this.beans.columnModel.getAriaColumnIndex(r);BC(this.ariaEl,a)},mk([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var yk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),E_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},tw=function(i){yk(t,i);function t(e,r){var n=i.call(this)||this;return n.columns=e,n.element=r,n}return t.prototype.postConstruct=function(){this.gridOptionsWrapper.isColumnHoverHighlight()&&this.addMouseHoverListeners()},t.prototype.addMouseHoverListeners=function(){this.addManagedListener(this.element,"mouseout",this.onMouseOut.bind(this)),this.addManagedListener(this.element,"mouseover",this.onMouseOver.bind(this))},t.prototype.onMouseOut=function(){this.columnHoverService.clearMouseOver()},t.prototype.onMouseOver=function(){this.columnHoverService.setMouseOver(this.columns)},E_([y("columnHoverService")],t.prototype,"columnHoverService",void 0),E_([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Ck=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Is=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},$E=function(i){Ck(t,i);function t(e,r){var n=i.call(this,e,r)||this;return n.column=e,n}return t.prototype.setComp=function(e,r,n,o){i.prototype.setGui.call(this,r),this.comp=e,this.eButtonShowMainFilter=n,this.eFloatingFilterBody=o;var a=this.column.getColDef(),s=!!a.filter||!!a.filterFramework,l=!!a.floatingFilter;this.active=s&&l,this.setupWidth(),this.setupLeft(),this.setupHover(),this.setupFocus(),this.setupUserComp(),this.setupSyncWithFilter(),this.setupUi(),this.addManagedListener(this.eButtonShowMainFilter,"click",this.showParentFilter.bind(this))},t.prototype.setupUi=function(){if(this.comp.addOrRemoveButtonWrapperCssClass("ag-hidden",!this.active||this.suppressFilterButton),!!this.active){this.comp.addOrRemoveBodyCssClass("ag-floating-filter-full-body",this.suppressFilterButton),this.comp.addOrRemoveBodyCssClass("ag-floating-filter-body",!this.suppressFilterButton);var e=Ft("filter",this.gridOptionsWrapper,this.column);e&&this.eButtonShowMainFilter.appendChild(e)}},t.prototype.setupFocus=function(){this.createManagedBean(new ai(this.eGui,{shouldStopEventPropagation:this.shouldStopEventPropagation.bind(this),onTabKeyDown:this.onTabKeyDown.bind(this),handleKeyDown:this.handleKeyDown.bind(this),onFocusIn:this.onFocusIn.bind(this)}))},t.prototype.onTabKeyDown=function(e){var r=this.gridOptionsWrapper.getDocument(),n=r.activeElement,o=n===this.eGui;if(!o){var a=this.focusService.findNextFocusableElement(this.eGui,null,e.shiftKey);if(a){this.beans.headerNavigationService.scrollToColumn(this.column),e.preventDefault(),a.focus();return}var s=this.findNextColumnWithFloatingFilter(e.shiftKey);!s||this.focusService.focusHeaderPosition({headerPosition:{headerRowIndex:this.getParentRowCtrl().getRowIndex(),column:s},event:e})&&e.preventDefault()}},t.prototype.findNextColumnWithFloatingFilter=function(e){var r=this.beans.columnModel,n=this.column;do if(n=e?r.getDisplayedColBefore(n):r.getDisplayedColAfter(n),!n)break;while(!n.getColDef().filter||!n.getColDef().floatingFilter);return n},t.prototype.handleKeyDown=function(e){var r=this.gridOptionsWrapper.getDocument(),n=r.activeElement,o=n===this.eGui;switch(e.key){case j.UP:case j.DOWN:o||e.preventDefault();case j.LEFT:case j.RIGHT:if(o)return;e.stopPropagation();case j.ENTER:o&&this.focusService.focusInto(this.eGui)&&e.preventDefault();break;case j.ESCAPE:o||this.eGui.focus()}},t.prototype.onFocusIn=function(e){var r=this.eGui.contains(e.relatedTarget);if(!r){var n=!!e.relatedTarget&&!e.relatedTarget.classList.contains("ag-floating-filter"),o=!!e.relatedTarget&&ef(e.relatedTarget,"ag-floating-filter");if(n&&o&&e.target===this.eGui){var a=this.lastFocusEvent,s=!!(a&&a.key===j.TAB);if(a&&s){var l=a.shiftKey;this.focusService.focusInto(this.eGui,l)}}var u=this.getRowIndex();this.beans.focusService.setFocusedHeader(u,this.column)}},t.prototype.setupHover=function(){var e=this;this.createManagedBean(new tw([this.column],this.eGui));var r=function(){if(!!e.gridOptionsWrapper.isColumnHoverHighlight()){var n=e.columnHoverService.isHovered(e.column);e.comp.addOrRemoveCssClass("ag-column-hover",n)}};this.addManagedListener(this.eventService,x.EVENT_COLUMN_HOVER_CHANGED,r),r()},t.prototype.setupLeft=function(){var e=new af(this.column,this.eGui,this.beans);this.createManagedBean(e)},t.prototype.setupUserComp=function(){var e=this;if(!!this.active){var r=this.column.getColDef(),n=this.filterManager.createFilterParams(this.column,r),o=this.userComponentFactory.mergeParamsWithApplicationProvidedParams(r,ny,n),a=this.userComponentFactory.getDefaultFloatingFilterType(r);a==null&&(a="agReadOnlyFloatingFilter");var s={column:this.column,filterParams:o,currentParentModel:function(){return e.currentParentModel()},parentFilterInstance:function(u){return e.parentFilterInstance(u)},showParentFilter:function(){return e.showParentFilter()},suppressFilterButton:!1};this.suppressFilterButton=r.floatingFilterComponentParams?!!r.floatingFilterComponentParams.suppressFilterButton:!1;var l=this.userComponentFactory.getFloatingFilterCompDetails(r,s,a);l&&this.comp.setCompDetails(l)}},t.prototype.currentParentModel=function(){var e=this.getFilterComponent(!1);return e?e.resolveNow(null,function(r){return r&&r.getModel()}):null},t.prototype.getFilterComponent=function(e){return e===void 0&&(e=!0),this.filterManager.getFilterComponent(this.column,"NO_UI",e)},t.prototype.parentFilterInstance=function(e){var r=this.getFilterComponent();r!=null&&r.then(function(n){e(ka(n))})},t.prototype.showParentFilter=function(){var e=this.suppressFilterButton?this.eFloatingFilterBody:this.eButtonShowMainFilter;this.menuFactory.showMenuAfterButtonClick(this.column,e,"floatingFilter","filterMenuTab",["filterMenuTab"])},t.prototype.setupSyncWithFilter=function(){var e=this;if(!!this.active){var r=function(n){var o=e.comp.getFloatingFilterComp();if(!!o){var a=e.currentParentModel();o.then(function(s){s&&s.onParentModelChanged(a,n)})}};this.addManagedListener(this.column,Se.EVENT_FILTER_CHANGED,r),this.filterManager.isFilterActive(this.column)&&r(null)}},t.prototype.setupWidth=function(){var e=this,r=function(){var n=e.column.getActualWidth()+"px";e.comp.setWidth(n)};this.addManagedListener(this.column,Se.EVENT_WIDTH_CHANGED,r),r()},Is([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),Is([y("filterManager")],t.prototype,"filterManager",void 0),Is([y("columnHoverService")],t.prototype,"columnHoverService",void 0),Is([y("gridApi")],t.prototype,"gridApi",void 0),Is([y("menuFactory")],t.prototype,"menuFactory",void 0),Is([y("beans")],t.prototype,"beans",void 0),t}(Zc);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var wk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Pg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},bk=function(i){wk(t,i);function t(e,r,n,o,a){var s=i.call(this)||this;return s.pinned=e,s.column=r,s.eResize=n,s.comp=o,s.ctrl=a,s}return t.prototype.postConstruct=function(){var e=this,r=this.column.getColDef(),n=[],o,a,s=function(){if(pt(e.eResize,o),!!o){var c=e.horizontalResizeService.addResizeBar({eResizeBar:e.eResize,onResizeStart:e.onResizeStart.bind(e),onResizing:e.onResizing.bind(e,!1),onResizeEnd:e.onResizing.bind(e,!0)});if(n.push(c),a){var p=e.gridOptionsWrapper.isSkipHeaderOnAutoSize(),h=function(){e.columnModel.autoSizeColumn(e.column,p,"uiColumnResized")};e.eResize.addEventListener("dblclick",h);var d=new Cr(e.eResize);d.addEventListener(Cr.EVENT_DOUBLE_TAP,h),e.addDestroyFunc(function(){e.eResize.removeEventListener("dblclick",h),d.removeEventListener(Cr.EVENT_DOUBLE_TAP,h),d.destroy()})}}},l=function(){n.forEach(function(c){return c()}),n.length=0},u=function(){var c=e.column.isResizable(),p=!e.gridOptionsWrapper.isSuppressAutoSize()&&!r.suppressAutoSize,h=c!==o||p!==a;h&&(o=c,a=p,l(),s())};u(),this.addDestroyFunc(l),this.ctrl.addRefreshFunction(u)},t.prototype.onResizing=function(e,r){var n=this.normaliseResizeAmount(r),o=[{key:this.column,newWidth:this.resizeStartWidth+n}];this.columnModel.setColumnWidths(o,this.resizeWithShiftKey,e,"uiColumnDragged"),e&&this.comp.addOrRemoveCssClass("ag-column-resizing",!1)},t.prototype.onResizeStart=function(e){this.resizeStartWidth=this.column.getActualWidth(),this.resizeWithShiftKey=e,this.comp.addOrRemoveCssClass("ag-column-resizing",!0)},t.prototype.normaliseResizeAmount=function(e){var r=e,n=this.pinned!==L.PINNED_LEFT,o=this.pinned===L.PINNED_RIGHT;return this.gridOptionsWrapper.isEnableRtl()?n&&(r*=-1):o&&(r*=-1),r},Pg([y("horizontalResizeService")],t.prototype,"horizontalResizeService",void 0),Pg([y("columnModel")],t.prototype,"columnModel",void 0),Pg([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var _k=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ii=function(i){_k(t,i);function t(e,r,n){r===void 0&&(r="ag-checkbox"),n===void 0&&(n="checkbox");var o=i.call(this,e,r,n)||this;return o.labelAlignment="right",o.selected=!1,o.readOnly=!1,o.passive=!1,o}return t.prototype.addInputListeners=function(){this.addManagedListener(this.eInput,"click",this.onCheckboxClick.bind(this)),this.addManagedListener(this.eLabel,"click",this.toggle.bind(this))},t.prototype.getNextValue=function(){return this.selected===void 0?!0:!this.selected},t.prototype.setPassive=function(e){this.passive=e},t.prototype.isReadOnly=function(){return this.readOnly},t.prototype.setReadOnly=function(e){this.eWrapper.classList.toggle("ag-disabled",e),this.eInput.disabled=e,this.readOnly=e},t.prototype.setDisabled=function(e){return this.eWrapper.classList.toggle("ag-disabled",e),i.prototype.setDisabled.call(this,e)},t.prototype.toggle=function(){if(!this.eInput.disabled){var e=this.isSelected(),r=this.getNextValue();this.passive?this.dispatchChange(r,e):this.setValue(r)}},t.prototype.getValue=function(){return this.isSelected()},t.prototype.setValue=function(e,r){return this.refreshSelectedClass(e),this.setSelected(e,r),this},t.prototype.setName=function(e){var r=this.getInputElement();return r.name=e,this},t.prototype.isSelected=function(){return this.selected},t.prototype.setSelected=function(e,r){this.isSelected()!==e&&(this.previousValue=this.isSelected(),e=this.selected=typeof e=="boolean"?e:void 0,this.eInput.checked=e,this.eInput.indeterminate=e===void 0,r||this.dispatchChange(this.selected,this.previousValue))},t.prototype.dispatchChange=function(e,r,n){this.dispatchEvent({type:t.EVENT_CHANGED,selected:e,previousValue:r,event:n});var o=this.getInputElement(),a={type:x.EVENT_CHECKBOX_CHANGED,id:o.id,name:o.name,selected:e,previousValue:r};this.eventService.dispatchEvent(a)},t.prototype.onCheckboxClick=function(e){if(!(this.passive||this.eInput.disabled)){var r=this.isSelected(),n=this.selected=e.target.checked;this.refreshSelectedClass(n),this.dispatchChange(n,r,e)}},t.prototype.refreshSelectedClass=function(e){this.eWrapper.classList.toggle("ag-checked",e===!0),this.eWrapper.classList.toggle("ag-indeterminate",e==null)},t}(Ki);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Sk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Cp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},xk=function(i){Sk(t,i);function t(e){var r=i.call(this)||this;r.cbSelectAllVisible=!1,r.processingEventFromCheckbox=!1,r.column=e;var n=e.getColDef();return r.filteredOnly=n?!!n.headerCheckboxSelectionFilteredOnly:!1,r}return t.prototype.onSpaceKeyPressed=function(e){var r=this.cbSelectAll,n=this.gridOptionsWrapper.getDocument();r.isDisplayed()&&!r.getGui().contains(n.activeElement)&&(e.preventDefault(),r.setValue(!r.getValue()))},t.prototype.getCheckboxGui=function(){return this.cbSelectAll.getGui()},t.prototype.setComp=function(e){this.headerCellCtrl=e,this.cbSelectAll=this.createManagedBean(new ii),this.cbSelectAll.addCssClass("ag-header-select-all"),_r(this.cbSelectAll.getGui(),"presentation"),this.showOrHideSelectAll(),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.showOrHideSelectAll.bind(this)),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,this.showOrHideSelectAll.bind(this)),this.addManagedListener(this.eventService,x.EVENT_SELECTION_CHANGED,this.onSelectionChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,this.onModelChanged.bind(this)),this.addManagedListener(this.cbSelectAll,ii.EVENT_CHANGED,this.onCbSelectAll.bind(this)),this.cbSelectAll.getInputElement().setAttribute("tabindex","-1"),this.refreshSelectAllLabel()},t.prototype.showOrHideSelectAll=function(){this.cbSelectAllVisible=this.isCheckboxSelection(),this.cbSelectAll.setDisplayed(this.cbSelectAllVisible),this.cbSelectAllVisible&&(this.checkRightRowModelType(),this.updateStateOfCheckbox()),this.refreshSelectAllLabel()},t.prototype.onModelChanged=function(){!this.cbSelectAllVisible||this.updateStateOfCheckbox()},t.prototype.onSelectionChanged=function(){!this.cbSelectAllVisible||this.updateStateOfCheckbox()},t.prototype.getNextCheckboxState=function(e){return e.selected===0&&e.notSelected===0?!1:e.selected>0&&e.notSelected>0?null:e.selected>0},t.prototype.updateStateOfCheckbox=function(){if(!this.processingEventFromCheckbox){this.processingEventFromCheckbox=!0;var e=this.getSelectionCount(),r=this.getNextCheckboxState(e);this.cbSelectAll.setValue(r),this.refreshSelectAllLabel(),this.processingEventFromCheckbox=!1}},t.prototype.refreshSelectAllLabel=function(){if(!this.cbSelectAllVisible)this.headerCellCtrl.setAriaDescriptionProperty("selectAll",null);else{var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=this.cbSelectAll.getValue(),n=r?e("ariaChecked","checked"):e("ariaUnchecked","unchecked"),o=e("ariaRowSelectAll","Press Space to toggle all rows selection");this.headerCellCtrl.setAriaDescriptionProperty("selectAll",o+" ("+n+")")}this.headerCellCtrl.refreshAriaDescription()},t.prototype.getSelectionCount=function(){var e=this,r=0,n=0,o=function(a){e.gridOptionsWrapper.isGroupSelectsChildren()&&a.group||(a.isSelected()?r++:a.selectable&&n++)};return this.filteredOnly?this.gridApi.forEachNodeAfterFilter(o):this.gridApi.forEachNode(o),{notSelected:n,selected:r}},t.prototype.checkRightRowModelType=function(){var e=this.rowModel.getType(),r=e===L.ROW_MODEL_TYPE_CLIENT_SIDE;r||console.warn("AG Grid: selectAllCheckbox is only available if using normal row model, you are using "+e)},t.prototype.onCbSelectAll=function(){if(!this.processingEventFromCheckbox&&!!this.cbSelectAllVisible){var e=this.cbSelectAll.getValue();e?this.selectionService.selectAllRowNodes(this.filteredOnly):this.selectionService.deselectAllRowNodes(this.filteredOnly)}},t.prototype.isCheckboxSelection=function(){var e=this.column.getColDef().headerCheckboxSelection;if(typeof e=="function"){var r=e,n={column:this.column,colDef:this.column.getColDef(),columnApi:this.columnApi,api:this.gridApi,context:this.gridOptionsWrapper.getContext()};e=r(n)}return e?this.gridOptionsWrapper.isRowModelServerSide()?(console.warn("AG Grid: headerCheckboxSelection is not supported for Server Side Row Model"),!1):this.gridOptionsWrapper.isRowModelInfinite()?(console.warn("AG Grid: headerCheckboxSelection is not supported for Infinite Row Model"),!1):this.gridOptionsWrapper.isRowModelViewport()?(console.warn("AG Grid: headerCheckboxSelection is not supported for Viewport Row Model"),!1):!0:!1},Cp([y("gridApi")],t.prototype,"gridApi",void 0),Cp([y("columnApi")],t.prototype,"columnApi",void 0),Cp([y("rowModel")],t.prototype,"rowModel",void 0),Cp([y("selectionService")],t.prototype,"selectionService",void 0),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Tk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),pi=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},KE=function(i){Tk(t,i);function t(e,r){var n=i.call(this,e,r)||this;return n.refreshFunctions=[],n.userHeaderClasses=new Set,n.ariaDescriptionProperties=new Map,n.column=e,n}return t.prototype.setComp=function(e,r,n){var o=this;i.prototype.setGui.call(this,r),this.comp=e,this.colDefVersion=this.columnModel.getColDefVersion(),this.updateState(),this.setupWidth(),this.setupMovingCss(),this.setupMenuClass(),this.setupSortableClass(),this.addColumnHoverListener(),this.setupFilterCss(),this.setupColId(),this.setupClassesFromColDef(),this.setupTooltip(),this.addActiveHeaderMouseListeners(),this.setupSelectAll(),this.setupUserComp(),this.refreshAria(),this.createManagedBean(new bk(this.getPinned(),this.column,n,e,this)),this.createManagedBean(new tw([this.column],r)),this.createManagedBean(new af(this.column,r,this.beans)),this.createManagedBean(new ai(r,{shouldStopEventPropagation:function(a){return o.shouldStopEventPropagation(a)},onTabKeyDown:function(){return null},handleKeyDown:this.handleKeyDown.bind(this),onFocusIn:this.onFocusIn.bind(this),onFocusOut:this.onFocusOut.bind(this)})),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.onNewColumnsLoaded.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VALUE_CHANGED,this.onColumnValueChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,this.onColumnRowGroupChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_CHANGED,this.onColumnPivotChanged.bind(this))},t.prototype.setupUserComp=function(){var e=this.lookupUserCompDetails();this.setCompDetails(e)},t.prototype.setCompDetails=function(e){this.userCompDetails=e,this.comp.setUserCompDetails(e)},t.prototype.lookupUserCompDetails=function(){var e=this.createParams(),r=this.column.getColDef();return this.userComponentFactory.getHeaderCompDetails(r,e)},t.prototype.createParams=function(){var e=this,r=this.column.getColDef(),n={column:this.column,displayName:this.displayName,enableSorting:r.sortable,enableMenu:this.menuEnabled,showColumnMenu:function(o){e.gridApi.showColumnMenuAfterButtonClick(e.column,o)},progressSort:function(o){e.sortController.progressSort(e.column,!!o,"uiColumnSorted")},setSort:function(o,a){e.sortController.setSortForColumn(e.column,o,!!a,"uiColumnSorted")},api:this.gridApi,columnApi:this.columnApi,context:this.gridOptionsWrapper.getContext(),eGridHeader:this.getGui()};return n},t.prototype.setupSelectAll=function(){this.selectAllFeature=this.createManagedBean(new xk(this.column)),this.selectAllFeature.setComp(this)},t.prototype.getSelectAllGui=function(){return this.selectAllFeature.getCheckboxGui()},t.prototype.handleKeyDown=function(e){e.key===j.SPACE&&this.selectAllFeature.onSpaceKeyPressed(e),e.key===j.ENTER&&this.onEnterKeyPressed(e)},t.prototype.onEnterKeyPressed=function(e){var r=this.comp.getUserCompInstance();if(!!r){if(e.ctrlKey||e.metaKey)this.menuEnabled&&r.showMenu&&(e.preventDefault(),r.showMenu());else if(this.sortable){var n=e.shiftKey;this.sortController.progressSort(this.column,n,"uiColumnSorted")}}},t.prototype.isMenuEnabled=function(){return this.menuEnabled},t.prototype.onFocusIn=function(e){if(!this.getGui().contains(e.relatedTarget)){var r=this.getRowIndex();this.focusService.setFocusedHeader(r,this.column)}this.setActiveHeader(!0)},t.prototype.onFocusOut=function(e){this.getGui().contains(e.relatedTarget)||this.setActiveHeader(!1)},t.prototype.setupTooltip=function(){var e=this,r={getColumn:function(){return e.column},getColDef:function(){return e.column.getColDef()},getGui:function(){return e.eGui},getLocation:function(){return"header"},getTooltipValue:function(){var o=e.column.getColDef().headerTooltip;return o}},n=this.createManagedBean(new QC(r,this.beans));n.setComp(this.comp),this.refreshFunctions.push(function(){return n.refreshToolTip()})},t.prototype.setupClassesFromColDef=function(){var e=this,r=function(){var n=e.column.getColDef(),o=e.gridOptionsWrapper,a=qc.getHeaderClassesFromColDef(n,o,e.column,null),s=e.userHeaderClasses;e.userHeaderClasses=new Set(a),a.forEach(function(l){s.has(l)?s.delete(l):e.comp.addOrRemoveCssClass(l,!0)}),s.forEach(function(l){return e.comp.addOrRemoveCssClass(l,!1)})};this.refreshFunctions.push(r),r()},t.prototype.setDragSource=function(e){var r=this;this.dragSourceElement=e,this.removeDragSource(),e&&(!this.draggable||(this.moveDragSource={type:At.HeaderCell,eElement:e,defaultIconName:ht.ICON_HIDE,getDragItem:function(){return r.createDragItem()},dragItemName:this.displayName,onDragStarted:function(){return r.column.setMoving(!0,"uiColumnMoved")},onDragStopped:function(){return r.column.setMoving(!1,"uiColumnMoved")}},this.dragAndDropService.addDragSource(this.moveDragSource,!0)))},t.prototype.createDragItem=function(){var e={};return e[this.column.getId()]=this.column.isVisible(),{columns:[this.column],visibleState:e}},t.prototype.removeDragSource=function(){this.moveDragSource&&(this.dragAndDropService.removeDragSource(this.moveDragSource),this.moveDragSource=void 0)},t.prototype.onNewColumnsLoaded=function(){var e=this.columnModel.getColDefVersion();e!=this.colDefVersion&&(this.colDefVersion=e,this.refresh())},t.prototype.updateState=function(){var e=this.column.getColDef();this.menuEnabled=this.menuFactory.isMenuEnabled(this.column)&&!e.suppressMenu,this.sortable=e.sortable,this.displayName=this.calculateDisplayName(),this.draggable=this.workOutDraggable()},t.prototype.addRefreshFunction=function(e){this.refreshFunctions.push(e)},t.prototype.refresh=function(){this.updateState(),this.refreshHeaderComp(),this.refreshAria(),this.refreshFunctions.forEach(function(e){return e()})},t.prototype.refreshHeaderComp=function(){var e=this.lookupUserCompDetails(),r=this.comp.getUserCompInstance(),n=r!=null&&this.userCompDetails.componentClass==e.componentClass,o=n?this.attemptHeaderCompRefresh(e.params):!1;o?this.setDragSource(this.dragSourceElement):this.setCompDetails(e)},t.prototype.attemptHeaderCompRefresh=function(e){var r=this.comp.getUserCompInstance();if(!r||!r.refresh)return!1;var n=r.refresh(e);return n},t.prototype.calculateDisplayName=function(){return this.columnModel.getDisplayNameForColumn(this.column,"header",!0)},t.prototype.checkDisplayName=function(){this.displayName!==this.calculateDisplayName()&&this.refresh()},t.prototype.workOutDraggable=function(){var e=this.column.getColDef(),r=this.gridOptionsWrapper.isSuppressMovableColumns(),n=!r&&!e.suppressMovable&&!e.lockPosition;return!!n||!!e.enableRowGroup||!!e.enablePivot},t.prototype.onColumnRowGroupChanged=function(){this.checkDisplayName()},t.prototype.onColumnPivotChanged=function(){this.checkDisplayName()},t.prototype.onColumnValueChanged=function(){this.checkDisplayName()},t.prototype.setupWidth=function(){var e=this,r=function(){e.comp.setWidth(e.column.getActualWidth()+"px")};this.addManagedListener(this.column,Se.EVENT_WIDTH_CHANGED,r),r()},t.prototype.setupMovingCss=function(){var e=this,r=function(){e.comp.addOrRemoveCssClass("ag-header-cell-moving",e.column.isMoving())};this.addManagedListener(this.column,Se.EVENT_MOVING_CHANGED,r),r()},t.prototype.setupMenuClass=function(){var e=this,r=function(){e.comp.addOrRemoveCssClass("ag-column-menu-visible",e.column.isMenuVisible())};this.addManagedListener(this.column,Se.EVENT_MENU_VISIBLE_CHANGED,r),r()},t.prototype.setupSortableClass=function(){var e=this,r=function(){e.comp.addOrRemoveCssClass("ag-header-cell-sortable",!!e.sortable)};r(),this.addRefreshFunction(r),this.addManagedListener(this.column,Se.EVENT_SORT_CHANGED,this.refreshAriaSort.bind(this))},t.prototype.refreshAriaSort=function(){if(this.sortable){var e=this.gridOptionsWrapper.getLocaleTextFunc();this.comp.setAriaSort(VO(this.column)),this.setAriaDescriptionProperty("sort",e("ariaSortableColumn","Press ENTER to sort."))}else this.comp.setAriaSort(),this.setAriaDescriptionProperty("sort",null)},t.prototype.refreshAriaMenu=function(){if(this.menuEnabled){var e=this.gridOptionsWrapper.getLocaleTextFunc();this.setAriaDescriptionProperty("menu",e("ariaMenuColumn","Press CTRL ENTER to open column menu."))}else this.setAriaDescriptionProperty("menu",null)},t.prototype.setAriaDescriptionProperty=function(e,r){r!=null?this.ariaDescriptionProperties.set(e,r):this.ariaDescriptionProperties.delete(e)},t.prototype.refreshAriaDescription=function(){var e=Array.from(this.ariaDescriptionProperties.values());this.comp.setAriaDescription(e.length?e.join(" "):void 0)},t.prototype.refreshAria=function(){this.refreshAriaSort(),this.refreshAriaMenu(),this.refreshAriaDescription()},t.prototype.addColumnHoverListener=function(){var e=this,r=function(){if(!!e.gridOptionsWrapper.isColumnHoverHighlight()){var n=e.columnHoverService.isHovered(e.column);e.comp.addOrRemoveCssClass("ag-column-hover",n)}};this.addManagedListener(this.eventService,x.EVENT_COLUMN_HOVER_CHANGED,r),r()},t.prototype.setupFilterCss=function(){var e=this,r=function(){e.comp.addOrRemoveCssClass("ag-header-cell-filtered",e.column.isFilterActive())};this.addManagedListener(this.column,Se.EVENT_FILTER_ACTIVE_CHANGED,r),r()},t.prototype.setupColId=function(){this.comp.setColId(this.column.getColId())},t.prototype.addActiveHeaderMouseListeners=function(){var e=this,r=function(n){return e.setActiveHeader(n.type==="mouseenter")};this.addManagedListener(this.getGui(),"mouseenter",r),this.addManagedListener(this.getGui(),"mouseleave",r)},t.prototype.setActiveHeader=function(e){this.comp.addOrRemoveCssClass("ag-header-active",e)},pi([y("columnModel")],t.prototype,"columnModel",void 0),pi([y("columnHoverService")],t.prototype,"columnHoverService",void 0),pi([y("beans")],t.prototype,"beans",void 0),pi([y("sortController")],t.prototype,"sortController",void 0),pi([y("menuFactory")],t.prototype,"menuFactory",void 0),pi([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),pi([y("gridApi")],t.prototype,"gridApi",void 0),pi([y("columnApi")],t.prototype,"columnApi",void 0),pi([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),pi([nt],t.prototype,"removeDragSource",null),t}(Zc);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Ok=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),wp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Ek=function(i){Ok(t,i);function t(e,r,n,o){var a=i.call(this)||this;return a.eResize=r,a.comp=e,a.pinned=n,a.columnGroup=o,a}return t.prototype.postConstruct=function(){var e=this;if(!this.columnGroup.isResizable()){this.comp.addOrRemoveResizableCssClass("ag-hidden",!0);return}var r=this.horizontalResizeService.addResizeBar({eResizeBar:this.eResize,onResizeStart:this.onResizeStart.bind(this),onResizing:this.onResizing.bind(this,!1),onResizeEnd:this.onResizing.bind(this,!0)});if(this.addDestroyFunc(r),!this.gridOptionsWrapper.isSuppressAutoSize()){var n=this.gridOptionsWrapper.isSkipHeaderOnAutoSize();this.eResize.addEventListener("dblclick",function(){var o=[],a=e.columnGroup.getDisplayedLeafColumns();a.forEach(function(s){s.getColDef().suppressAutoSize||o.push(s.getColId())}),o.length>0&&e.columnModel.autoSizeColumns({columns:o,skipHeader:n,stopAtGroup:e.columnGroup,source:"uiColumnResized"}),e.resizeLeafColumnsToFit()})}},t.prototype.onResizeStart=function(e){var r=this;this.calculateInitialValues();var n=null;if(e&&(n=this.columnModel.getDisplayedGroupAfter(this.columnGroup)),n){var o=n.getDisplayedLeafColumns();this.resizeTakeFromCols=o.filter(function(a){return a.isResizable()}),this.resizeTakeFromStartWidth=0,this.resizeTakeFromCols.forEach(function(a){return r.resizeTakeFromStartWidth+=a.getActualWidth()}),this.resizeTakeFromRatios=[],this.resizeTakeFromCols.forEach(function(a){return r.resizeTakeFromRatios.push(a.getActualWidth()/r.resizeTakeFromStartWidth)})}else this.resizeTakeFromCols=null,this.resizeTakeFromStartWidth=null,this.resizeTakeFromRatios=null;this.comp.addOrRemoveCssClass("ag-column-resizing",!0)},t.prototype.onResizing=function(e,r){var n=this.normaliseDragChange(r),o=this.resizeStartWidth+n;this.resizeColumns(o,e)},t.prototype.resizeLeafColumnsToFit=function(){var e=this.autoWidthCalculator.getPreferredWidthForColumnGroup(this.columnGroup);this.calculateInitialValues(),e>this.resizeStartWidth&&this.resizeColumns(e,!0)},t.prototype.resizeColumns=function(e,r){r===void 0&&(r=!0);var n=[];if(n.push({columns:this.resizeCols,ratios:this.resizeRatios,width:e}),this.resizeTakeFromCols){var o=e-this.resizeStartWidth;n.push({columns:this.resizeTakeFromCols,ratios:this.resizeTakeFromRatios,width:this.resizeTakeFromStartWidth-o})}this.columnModel.resizeColumnSets({resizeSets:n,finished:r,source:"uiColumnDragged"}),r&&this.comp.addOrRemoveCssClass("ag-column-resizing",!1)},t.prototype.calculateInitialValues=function(){var e=this,r=this.columnGroup.getDisplayedLeafColumns();this.resizeCols=r.filter(function(n){return n.isResizable()}),this.resizeStartWidth=0,this.resizeCols.forEach(function(n){return e.resizeStartWidth+=n.getActualWidth()}),this.resizeRatios=[],this.resizeCols.forEach(function(n){return e.resizeRatios.push(n.getActualWidth()/e.resizeStartWidth)})},t.prototype.normaliseDragChange=function(e){var r=e;return this.gridOptionsWrapper.isEnableRtl()?this.pinned!==L.PINNED_LEFT&&(r*=-1):this.pinned===L.PINNED_RIGHT&&(r*=-1),r},wp([y("horizontalResizeService")],t.prototype,"horizontalResizeService",void 0),wp([y("autoWidthCalculator")],t.prototype,"autoWidthCalculator",void 0),wp([y("columnModel")],t.prototype,"columnModel",void 0),wp([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Rk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Pk=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Ak=function(i){Rk(t,i);function t(e,r){var n=i.call(this)||this;return n.removeChildListenersFuncs=[],n.columnGroup=r,n.comp=e,n}return t.prototype.postConstruct=function(){this.addListenersToChildrenColumns(),this.addManagedListener(this.columnGroup,sr.EVENT_DISPLAYED_CHILDREN_CHANGED,this.onDisplayedChildrenChanged.bind(this)),this.onWidthChanged(),this.addDestroyFunc(this.removeListenersOnChildrenColumns.bind(this))},t.prototype.addListenersToChildrenColumns=function(){var e=this;this.removeListenersOnChildrenColumns();var r=this.onWidthChanged.bind(this);this.columnGroup.getLeafColumns().forEach(function(n){n.addEventListener(Se.EVENT_WIDTH_CHANGED,r),n.addEventListener(Se.EVENT_VISIBLE_CHANGED,r),e.removeChildListenersFuncs.push(function(){n.removeEventListener(Se.EVENT_WIDTH_CHANGED,r),n.removeEventListener(Se.EVENT_VISIBLE_CHANGED,r)})})},t.prototype.removeListenersOnChildrenColumns=function(){this.removeChildListenersFuncs.forEach(function(e){return e()}),this.removeChildListenersFuncs=[]},t.prototype.onDisplayedChildrenChanged=function(){this.addListenersToChildrenColumns(),this.onWidthChanged()},t.prototype.onWidthChanged=function(){this.comp.setWidth(this.columnGroup.getActualWidth()+"px")},Pk([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Dk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ls=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},YE=function(i){Dk(t,i);function t(e,r){var n=i.call(this,e,r)||this;return n.columnGroup=e,n}return t.prototype.setComp=function(e,r,n){i.prototype.setGui.call(this,r),this.comp=e,this.displayName=this.columnModel.getDisplayNameForColumnGroup(this.columnGroup,"header"),this.addClasses(),this.addAttributes(),this.setupMovingCss(),this.setupExpandable(),this.setupTooltip(),this.setupUserComp();var o=this.getParentRowCtrl().getPinned(),a=this.columnGroup.getProvidedColumnGroup().getLeafColumns();this.createManagedBean(new tw(a,r)),this.createManagedBean(new af(this.columnGroup,r,this.beans)),this.createManagedBean(new Ak(e,this.columnGroup)),this.groupResizeFeature=this.createManagedBean(new Ek(e,n,o,this.columnGroup)),this.createManagedBean(new ai(r,{shouldStopEventPropagation:this.shouldStopEventPropagation.bind(this),onTabKeyDown:function(){},handleKeyDown:this.handleKeyDown.bind(this),onFocusIn:this.onFocusIn.bind(this)}))},t.prototype.resizeLeafColumnsToFit=function(){this.groupResizeFeature.onResizeStart(!1),this.groupResizeFeature.resizeLeafColumnsToFit()},t.prototype.setupUserComp=function(){var e=this,r=this.displayName,n={displayName:this.displayName,columnGroup:this.columnGroup,setExpanded:function(u){e.columnModel.setColumnGroupOpened(e.columnGroup.getProvidedColumnGroup(),u,"gridInitializing")},api:this.gridApi,columnApi:this.columnApi,context:this.gridOptionsWrapper.getContext()};if(!r){for(var o=this.columnGroup,a=o.getLeafColumns();o.getParent()&&o.getParent().getLeafColumns().length===a.length;)o=o.getParent();var s=o.getColGroupDef();s&&(r=s.headerName),r||(r=a?this.columnModel.getDisplayNameForColumn(a[0],"header",!0):"")}var l=this.userComponentFactory.getHeaderGroupCompDetails(n);this.comp.setUserCompDetails(l)},t.prototype.setupTooltip=function(){var e=this,r=this.columnGroup.getColGroupDef(),n={getColumn:function(){return e.columnGroup},getGui:function(){return e.eGui},getLocation:function(){return"headerGroup"},getTooltipValue:function(){return r&&r.headerTooltip}};r&&(n.getColDef=function(){return r});var o=this.createManagedBean(new QC(n,this.beans));o.setComp(this.comp)},t.prototype.setupExpandable=function(){var e=this.columnGroup.getProvidedColumnGroup();this.refreshExpanded(),this.addManagedListener(e,$e.EVENT_EXPANDABLE_CHANGED,this.refreshExpanded.bind(this)),this.addManagedListener(e,$e.EVENT_EXPANDED_CHANGED,this.refreshExpanded.bind(this))},t.prototype.refreshExpanded=function(){var e=this.columnGroup;this.expandable=e.isExpandable();var r=e.isExpanded();this.expandable?this.comp.setAriaExpanded(r?"true":"false"):this.comp.setAriaExpanded(void 0)},t.prototype.addAttributes=function(){this.comp.setColId(this.columnGroup.getUniqueId())},t.prototype.addClasses=function(){var e=this,r=this.columnGroup.getColGroupDef(),n=qc.getHeaderClassesFromColDef(r,this.gridOptionsWrapper,null,this.columnGroup);n.push(this.columnGroup.isPadding()?"ag-header-group-cell-no-group":"ag-header-group-cell-with-group"),n.forEach(function(o){return e.comp.addOrRemoveCssClass(o,!0)})},t.prototype.setupMovingCss=function(){var e=this,r=this.columnGroup.getProvidedColumnGroup(),n=r.getLeafColumns(),o=function(){return e.comp.addOrRemoveCssClass("ag-header-cell-moving",e.columnGroup.isMoving())};n.forEach(function(a){e.addManagedListener(a,Se.EVENT_MOVING_CHANGED,o)}),o()},t.prototype.onFocusIn=function(e){if(!this.eGui.contains(e.relatedTarget)){var r=this.getRowIndex();this.beans.focusService.setFocusedHeader(r,this.columnGroup)}},t.prototype.handleKeyDown=function(e){var r=this.gridOptionsWrapper.getDocument(),n=r.activeElement,o=n===this.eGui;if(!(!this.expandable||!o)&&e.key===j.ENTER){var a=this.columnGroup,s=!a.isExpanded();this.columnModel.setColumnGroupOpened(a.getProvidedColumnGroup(),s,"uiColumnExpanded")}},t.prototype.setDragSource=function(e){var r=this;if(!this.isSuppressMoving()){var n=this.columnGroup.getProvidedColumnGroup().getLeafColumns(),o={type:At.HeaderCell,eElement:e,defaultIconName:ht.ICON_HIDE,dragItemName:this.displayName,getDragItem:this.getDragItemForGroup.bind(this),onDragStarted:function(){return n.forEach(function(a){return a.setMoving(!0,"uiColumnDragged")})},onDragStopped:function(){return n.forEach(function(a){return a.setMoving(!1,"uiColumnDragged")})}};this.dragAndDropService.addDragSource(o,!0),this.addDestroyFunc(function(){return r.dragAndDropService.removeDragSource(o)})}},t.prototype.getDragItemForGroup=function(){var e=this.columnGroup.getProvidedColumnGroup().getLeafColumns(),r={};e.forEach(function(o){return r[o.getId()]=o.isVisible()});var n=[];return this.columnModel.getAllDisplayedColumns().forEach(function(o){e.indexOf(o)>=0&&(n.push(o),gr(e,o))}),e.forEach(function(o){return n.push(o)}),{columns:n,visibleState:r}},t.prototype.isSuppressMoving=function(){var e=!1;this.columnGroup.getLeafColumns().forEach(function(n){(n.getColDef().suppressMovable||n.getColDef().lockPosition)&&(e=!0)});var r=e||this.gridOptionsWrapper.isSuppressMovableColumns();return r},Ls([y("beans")],t.prototype,"beans",void 0),Ls([y("columnModel")],t.prototype,"columnModel",void 0),Ls([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),Ls([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),Ls([y("gridApi")],t.prototype,"gridApi",void 0),Ls([y("columnApi")],t.prototype,"columnApi",void 0),t}(Zc);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Mk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ag=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Fk=0,wh=function(i){Mk(t,i);function t(e,r,n){var o=i.call(this)||this;return o.instanceId=Fk++,o.headerCellCtrls={},o.rowIndex=e,o.pinned=r,o.type=n,o}return t.prototype.getInstanceId=function(){return this.instanceId},t.prototype.setComp=function(e){this.comp=e,this.onRowHeightChanged(),this.onVirtualColumnsChanged(),this.setWidth(),this.addEventListeners(),Hc()&&this.comp.setTransform("translateZ(0)"),e.setAriaRowIndex(this.rowIndex+1)},t.prototype.addEventListeners=function(){this.addManagedListener(this.eventService,x.EVENT_COLUMN_RESIZED,this.onColumnResized.bind(this)),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_DOM_LAYOUT,this.onDisplayedColumnsChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,this.onDisplayedColumnsChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_VIRTUAL_COLUMNS_CHANGED,this.onVirtualColumnsChanged.bind(this)),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_HEADER_HEIGHT,this.onRowHeightChanged.bind(this)),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_PIVOT_HEADER_HEIGHT,this.onRowHeightChanged.bind(this)),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_GROUP_HEADER_HEIGHT,this.onRowHeightChanged.bind(this)),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_PIVOT_GROUP_HEADER_HEIGHT,this.onRowHeightChanged.bind(this)),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_FLOATING_FILTERS_HEIGHT,this.onRowHeightChanged.bind(this))},t.prototype.getHeaderCellCtrl=function(e){return ms(this.headerCellCtrls).find(function(r){return r.getColumnGroupChild()===e})},t.prototype.onDisplayedColumnsChanged=function(){this.onVirtualColumnsChanged(),this.setWidth()},t.prototype.getType=function(){return this.type},t.prototype.onColumnResized=function(){this.setWidth()},t.prototype.setWidth=function(){var e=this.getWidthForRow();this.comp.setWidth(e+"px")},t.prototype.getWidthForRow=function(){var e=this.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_PRINT;if(e){var r=this.pinned!=null;return r?0:this.columnModel.getContainerWidth(L.PINNED_RIGHT)+this.columnModel.getContainerWidth(L.PINNED_LEFT)+this.columnModel.getContainerWidth(null)}return this.columnModel.getContainerWidth(this.pinned)},t.prototype.onRowHeightChanged=function(){var e=this.columnModel.getHeaderRowCount(),r=[],n=0,o,a;this.columnModel.hasFloatingFilters()&&(e++,n=1),this.columnModel.isPivotMode()?(o=this.gridOptionsWrapper.getPivotGroupHeaderHeight(),a=this.gridOptionsWrapper.getPivotHeaderHeight()):(o=this.gridOptionsWrapper.getGroupHeaderHeight(),a=this.gridOptionsWrapper.getHeaderHeight());for(var s=1+n,l=e-s,u=0;u<l;u++)r.push(o);r.push(a);for(var u=0;u<n;u++)r.push(this.gridOptionsWrapper.getFloatingFiltersHeight());for(var c=0,u=0;u<this.rowIndex;u++)c+=r[u];this.comp.setTop(c+"px"),this.comp.setHeight(r[this.rowIndex]+"px")},t.prototype.getPinned=function(){return this.pinned},t.prototype.getRowIndex=function(){return this.rowIndex},t.prototype.onVirtualColumnsChanged=function(){var e=this,r=this.headerCellCtrls;this.headerCellCtrls={};var n=this.getColumnsInViewport();n.forEach(function(s){if(!s.isEmptyGroup()){var l=s.getUniqueId(),u=r[l];delete r[l];var c=u&&u.getColumnGroupChild()!=s;if(c&&(e.destroyBean(u),u=void 0),u==null)switch(e.type){case wr.FLOATING_FILTER:u=e.createBean(new $E(s,e));break;case wr.COLUMN_GROUP:u=e.createBean(new YE(s,e));break;default:u=e.createBean(new KE(s,e));break}e.headerCellCtrls[l]=u}});var o=function(s){var l=e.focusService.isHeaderWrapperFocused(s);if(!l)return!1;var u=e.columnModel.isDisplayed(s.getColumnGroupChild());return u};Ke(r,function(s,l){var u=o(l);u?e.headerCellCtrls[s]=l:e.destroyBean(l)});var a=Jn(this.headerCellCtrls);this.comp.setHeaderCtrls(a)},t.prototype.destroyCtrls=function(){var e=this;Ke(this.headerCellCtrls,function(r,n){e.destroyBean(n)}),this.headerCellCtrls={}},t.prototype.getColumnsInViewport=function(){var e=this.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_PRINT;return e?this.getColumnsInViewportPrintLayout():this.getColumnsInViewportNormalLayout()},t.prototype.getColumnsInViewportPrintLayout=function(){var e=this;if(this.pinned!=null)return[];var r=[],n=this.getActualDepth();return[L.PINNED_LEFT,null,L.PINNED_RIGHT].forEach(function(o){var a=e.columnModel.getVirtualHeaderGroupRow(o,n);r=r.concat(a)}),r},t.prototype.getActualDepth=function(){return this.type==wr.FLOATING_FILTER?this.rowIndex-1:this.rowIndex},t.prototype.getColumnsInViewportNormalLayout=function(){return this.columnModel.getVirtualHeaderGroupRow(this.pinned,this.getActualDepth())},t.prototype.focusHeader=function(e,r){var n=Jn(this.headerCellCtrls),o=n.find(function(a){return a.getColumnGroupChild()==e});return o?(o.focus(r),!0):!1},Ag([y("columnModel")],t.prototype,"columnModel",void 0),Ag([y("focusService")],t.prototype,"focusService",void 0),Ag([nt],t.prototype,"destroyCtrls",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Ik=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),du=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Lk=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Nk=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(Lk(arguments[t]));return i},XE=function(i){Ik(t,i);function t(e){var r=i.call(this)||this;return r.groupsRowCtrls=[],r.pinned=e,r}return t.prototype.setComp=function(e,r){this.comp=e,this.setupCenterWidth(),this.setupPinnedWidth(),this.setupDragAndDrop(r),this.addManagedListener(this.eventService,x.EVENT_GRID_COLUMNS_CHANGED,this.onGridColumnsChanged.bind(this)),this.ctrlsService.registerHeaderContainer(this,this.pinned),this.columnModel.isReady()&&this.refresh()},t.prototype.setupDragAndDrop=function(e){var r=new jE(this.pinned,e);this.createManagedBean(r)},t.prototype.refresh=function(e){var r=this;e===void 0&&(e=!1);var n=new Bt,o=this.focusService.getFocusHeaderToUseAfterRefresh(),a=function(){var c=r.columnModel.getHeaderRowCount()-1;r.groupsRowCtrls=r.destroyBeans(r.groupsRowCtrls);for(var p=0;p<c;p++){var h=r.createBean(new wh(n.next(),r.pinned,wr.COLUMN_GROUP));r.groupsRowCtrls.push(h)}},s=function(){var c=n.next(),p=r.columnsRowCtrl==null||!e||r.columnsRowCtrl.getRowIndex()!==c;p&&(r.destroyBean(r.columnsRowCtrl),r.columnsRowCtrl=r.createBean(new wh(c,r.pinned,wr.COLUMN)))},l=function(){var c=r.columnModel.hasFloatingFilters(),p=function(){r.filtersRowCtrl=r.destroyBean(r.filtersRowCtrl)};if(!c){p();return}var h=n.next();if(r.filtersRowCtrl){var d=r.filtersRowCtrl.getRowIndex()!==h;(!e||d)&&p()}r.filtersRowCtrl||(r.filtersRowCtrl=r.createBean(new wh(h,r.pinned,wr.FLOATING_FILTER)))};a(),s(),l();var u=this.getAllCtrls();this.comp.setCtrls(u),this.restoreFocusOnHeader(o)},t.prototype.restoreFocusOnHeader=function(e){e==null||e.column.getPinned()!=this.pinned||this.focusService.focusHeaderPosition({headerPosition:e})},t.prototype.getAllCtrls=function(){var e=Nk(this.groupsRowCtrls,[this.columnsRowCtrl]);return this.filtersRowCtrl&&e.push(this.filtersRowCtrl),e},t.prototype.onGridColumnsChanged=function(){this.refresh(!0)},t.prototype.setupCenterWidth=function(){var e=this;this.pinned==null&&this.createManagedBean(new ew(function(r){return e.comp.setCenterWidth(r+"px")}))},t.prototype.setHorizontalScroll=function(e){this.comp.setContainerTransform("translateX("+e+"px)")},t.prototype.setupPinnedWidth=function(){var e=this;if(this.pinned!=null){var r=this.pinned===L.PINNED_LEFT,n=this.pinned===L.PINNED_RIGHT,o=function(){var a=r?e.pinnedWidthService.getPinnedLeftWidth():e.pinnedWidthService.getPinnedRightWidth();if(a!=null){var s=a==0,l=e.gridOptionsWrapper.isEnableRtl(),u=e.gridOptionsWrapper.getScrollbarWidth(),c=e.scrollVisibleService.isVerticalScrollShowing()&&(l&&r||!l&&n),p=c?a+u:a;e.comp.setPinnedContainerWidth(p+"px"),e.comp.addOrRemoveCssClass("ag-hidden",s)}};this.addManagedListener(this.eventService,x.EVENT_LEFT_PINNED_WIDTH_CHANGED,o),this.addManagedListener(this.eventService,x.EVENT_RIGHT_PINNED_WIDTH_CHANGED,o),this.addManagedListener(this.eventService,x.EVENT_SCROLL_VISIBILITY_CHANGED,o),this.addManagedListener(this.eventService,x.EVENT_SCROLLBAR_WIDTH_CHANGED,o)}},t.prototype.getHeaderCtrlForColumn=function(e){if(e instanceof Se)return this.columnsRowCtrl?this.columnsRowCtrl.getHeaderCellCtrl(e):void 0;if(this.groupsRowCtrls.length!==0)for(var r=0;r<this.groupsRowCtrls.length;r++){var n=this.groupsRowCtrls[r].getHeaderCellCtrl(e);if(n)return n}},t.prototype.getHtmlElementForColumnHeader=function(e){var r=this.getHeaderCtrlForColumn(e);return r?r.getGui():null},t.prototype.getRowType=function(e){var r=this.getAllCtrls(),n=r[e];return n?n.getType():void 0},t.prototype.focusHeader=function(e,r,n){var o=this.getAllCtrls(),a=o[e];return a?a.focusHeader(r,n):!1},t.prototype.getRowCount=function(){return this.getAllCtrls().length},du([y("ctrlsService")],t.prototype,"ctrlsService",void 0),du([y("scrollVisibleService")],t.prototype,"scrollVisibleService",void 0),du([y("pinnedWidthService")],t.prototype,"pinnedWidthService",void 0),du([y("columnModel")],t.prototype,"columnModel",void 0),du([y("focusService")],t.prototype,"focusService",void 0),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Gk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Dg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},bh=function(i){Gk(t,i);function t(e){var r=i.call(this)||this;return r.headerRowComps={},r.rowCompsList=[],r.pinned=e,r}return t.prototype.init=function(){var e=this;this.selectAndSetTemplate();var r={addOrRemoveCssClass:function(o,a){return e.addOrRemoveCssClass(o,a)},setCtrls:function(o){return e.setCtrls(o)},setCenterWidth:function(o){return e.eCenterContainer.style.width=o},setContainerTransform:function(o){return e.eCenterContainer.style.transform=o},setPinnedContainerWidth:function(o){var a=e.getGui();a.style.width=o,a.style.maxWidth=o,a.style.minWidth=o}},n=this.createManagedBean(new XE(this.pinned));n.setComp(r,this.getGui())},t.prototype.selectAndSetTemplate=function(){var e=this.pinned==L.PINNED_LEFT,r=this.pinned==L.PINNED_RIGHT,n=e?t.PINNED_LEFT_TEMPLATE:r?t.PINNED_RIGHT_TEMPLATE:t.CENTER_TEMPLATE;this.setTemplate(n),this.eRowContainer=this.eCenterContainer?this.eCenterContainer:this.getGui()},t.prototype.destroyRowComps=function(){this.setCtrls([])},t.prototype.destroyRowComp=function(e){this.destroyBean(e),this.eRowContainer.removeChild(e.getGui())},t.prototype.setCtrls=function(e){var r=this,n=this.headerRowComps;this.headerRowComps={},this.rowCompsList=[];var o,a=function(s){var l=s.getGui(),u=l.parentElement!=r.eRowContainer;u&&r.eRowContainer.appendChild(l),o&&AC(r.eRowContainer,l,o),o=l};e.forEach(function(s){var l=s.getInstanceId(),u=n[l];delete n[l];var c=u||r.createBean(new UE(s));r.headerRowComps[l]=c,r.rowCompsList.push(c),a(c)}),Jn(n).forEach(function(s){return r.destroyRowComp(s)})},t.PINNED_LEFT_TEMPLATE='<div class="ag-pinned-left-header" role="presentation"/>',t.PINNED_RIGHT_TEMPLATE='<div class="ag-pinned-right-header" role="presentation"/>',t.CENTER_TEMPLATE=`<div class="ag-header-viewport" role="presentation">
            <div class="ag-header-container" ref="eCenterContainer" role="rowgroup"></div>
        </div>`,Dg([I("eCenterContainer")],t.prototype,"eCenterContainer",void 0),Dg([B],t.prototype,"init",null),Dg([nt],t.prototype,"destroyRowComps",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var kk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ns=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Xn;(function(i){i[i.UP=0]="UP",i[i.DOWN=1]="DOWN",i[i.LEFT=2]="LEFT",i[i.RIGHT=3]="RIGHT"})(Xn||(Xn={}));var qE=function(i){kk(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){var e=this;this.ctrlsService.whenReady(function(r){e.gridBodyCon=r.gridBodyCtrl})},t.prototype.getHeaderRowCount=function(){var e=this.ctrlsService.getHeaderRowContainerCtrl();return e?e.getRowCount():0},t.prototype.getHeaderRowType=function(e){var r=this.ctrlsService.getHeaderRowContainerCtrl();if(r)return r.getRowType(e)},t.prototype.navigateVertically=function(e,r,n){if(r||(r=this.focusService.getFocusedHeader()),!r)return!1;var o=r.headerRowIndex,a=r.column,s=this.getHeaderRowCount(),l=e===Xn.UP,u=l?o-1:o+1,c=null,p=!1;u<0&&(u=0,c=a,p=!0),u>=s&&(u=-1);var h=this.getHeaderRowType(o);if(!p){if(h===wr.COLUMN_GROUP){var d=a;c=l?a.getParent():d.getDisplayedChildren()[0]}else if(h===wr.FLOATING_FILTER)c=a;else{var d=a;c=l?d.getParent():d}if(!c)return!1}return this.focusService.focusHeaderPosition({headerPosition:{headerRowIndex:u,column:c},allowUserOverride:!0,event:n})},t.prototype.navigateHorizontally=function(e,r,n){r===void 0&&(r=!1);var o=this.focusService.getFocusedHeader(),a=e===Xn.LEFT,s=this.gridOptionsWrapper.isEnableRtl(),l,u;return a!==s?(u="Before",l=this.headerPositionUtils.findHeader(o,u)):(u="After",l=this.headerPositionUtils.findHeader(o,u)),l?this.focusService.focusHeaderPosition({headerPosition:l,direction:u,fromTab:r,allowUserOverride:!0,event:n}):r?this.focusNextHeaderRow(o,u,n):!0},t.prototype.focusNextHeaderRow=function(e,r,n){var o=e.headerRowIndex,a=null,s;return r==="Before"?o>0&&(s=o-1,a=this.headerPositionUtils.findColAtEdgeForHeaderRow(s,"end")):(s=o+1,a=this.headerPositionUtils.findColAtEdgeForHeaderRow(s,"start")),this.focusService.focusHeaderPosition({headerPosition:a,direction:r,fromTab:!0,allowUserOverride:!0,event:n})},t.prototype.scrollToColumn=function(e,r){if(r===void 0&&(r="After"),!e.getPinned()){var n;if(e instanceof sr){var o=e.getDisplayedLeafColumns();n=r==="Before"?rt(o):o[0]}else n=e;this.gridBodyCon.getScrollFeature().ensureColumnVisible(n),this.gridBodyCon.getScrollFeature().horizontallyScrollHeaderCenterAndFloatingCenter(),this.animationFrameService.flushAllFrames()}},Ns([y("focusService")],t.prototype,"focusService",void 0),Ns([y("headerPositionUtils")],t.prototype,"headerPositionUtils",void 0),Ns([y("animationFrameService")],t.prototype,"animationFrameService",void 0),Ns([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Ns([B],t.prototype,"postConstruct",null),t=Ns([Q("headerNavigationService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Wk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),bp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ZE=function(i){Wk(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.setComp=function(e,r,n){this.comp=e,this.eGui=r,this.createManagedBean(new ai(n,{onTabKeyDown:this.onTabKeyDown.bind(this),handleKeyDown:this.handleKeyDown.bind(this),onFocusOut:this.onFocusOut.bind(this)})),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,this.onPivotModeChanged.bind(this)),this.onPivotModeChanged(),this.setupHeaderHeight(),this.ctrlsService.registerGridHeaderCtrl(this)},t.prototype.setupHeaderHeight=function(){var e=this.setHeaderHeight.bind(this);e(),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_HEADER_HEIGHT,e),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_PIVOT_HEADER_HEIGHT,e),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_GROUP_HEADER_HEIGHT,e),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_PIVOT_GROUP_HEADER_HEIGHT,e),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_FLOATING_FILTERS_HEIGHT,e),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,e)},t.prototype.setHeaderHeight=function(){var e=this,r=e.columnModel,n=e.gridOptionsWrapper,o=0,a=r.getHeaderRowCount(),s,l,u,c=r.hasFloatingFilters();c&&(a++,o=1),r.isPivotMode()?(l=n.getPivotGroupHeaderHeight(),u=n.getPivotHeaderHeight()):(l=n.getGroupHeaderHeight(),u=n.getHeaderHeight());var p=1+o,h=a-p;s=o*n.getFloatingFiltersHeight(),s+=h*l,s+=u;var d=s+1+"px";this.comp.setHeightAndMinHeight(d)},t.prototype.onPivotModeChanged=function(){var e=this.columnModel.isPivotMode();this.comp.addOrRemoveCssClass("ag-pivot-on",e),this.comp.addOrRemoveCssClass("ag-pivot-off",!e)},t.prototype.onTabKeyDown=function(e){var r=this.gridOptionsWrapper.isEnableRtl(),n=e.shiftKey!==r?Xn.LEFT:Xn.RIGHT;(this.headerNavigationService.navigateHorizontally(n,!0,e)||this.focusService.focusNextGridCoreContainer(e.shiftKey))&&e.preventDefault()},t.prototype.handleKeyDown=function(e){var r=null;switch(e.key){case j.LEFT:r=Xn.LEFT;case j.RIGHT:q(r)||(r=Xn.RIGHT),this.headerNavigationService.navigateHorizontally(r,!1,e);break;case j.UP:r=Xn.UP;case j.DOWN:q(r)||(r=Xn.DOWN),this.headerNavigationService.navigateVertically(r,null,e)&&e.preventDefault();break;default:return}},t.prototype.onFocusOut=function(e){var r=this.gridOptionsWrapper.getDocument(),n=e.relatedTarget;!n&&this.eGui.contains(r.activeElement)||this.eGui.contains(n)||this.focusService.clearFocusedHeader()},bp([y("headerNavigationService")],t.prototype,"headerNavigationService",void 0),bp([y("focusService")],t.prototype,"focusService",void 0),bp([y("columnModel")],t.prototype,"columnModel",void 0),bp([y("ctrlsService")],t.prototype,"ctrlsService",void 0),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Vk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Bk=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},QE=function(i){Vk(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.postConstruct=function(){var e=this,r={addOrRemoveCssClass:function(a,s){return e.addOrRemoveCssClass(a,s)},setHeightAndMinHeight:function(a){e.getGui().style.height=a,e.getGui().style.minHeight=a}},n=this.createManagedBean(new ZE);n.setComp(r,this.getGui(),this.getFocusableElement());var o=function(a){e.createManagedBean(a),e.appendChild(a)};o(new bh(L.PINNED_LEFT)),o(new bh(null)),o(new bh(L.PINNED_RIGHT))},t.TEMPLATE='<div class="ag-header" role="presentation"/>',Bk([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Hk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Mg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},JE=function(i){Hk(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.addResizeBar=function(e){var r=this,n={dragStartPixels:e.dragStartPixels||0,eElement:e.eResizeBar,onDragStart:this.onDragStart.bind(this,e),onDragStop:this.onDragStop.bind(this,e),onDragging:this.onDragging.bind(this,e)};this.dragService.addDragSource(n,!0);var o=function(){return r.dragService.removeDragSource(n)};return o},t.prototype.onDragStart=function(e,r){this.dragStartX=r.clientX,this.setResizeIcons();var n=r instanceof MouseEvent&&r.shiftKey===!0;e.onResizeStart(n)},t.prototype.setResizeIcons=function(){var e=this.ctrlsService.getGridCtrl();e.setResizeCursor(!0),e.disableUserSelect(!0)},t.prototype.onDragStop=function(e,r){e.onResizeEnd(this.resizeAmount),this.resetIcons()},t.prototype.resetIcons=function(){var e=this.ctrlsService.getGridCtrl();e.setResizeCursor(!1),e.disableUserSelect(!1)},t.prototype.onDragging=function(e,r){this.resizeAmount=r.clientX-this.dragStartX,e.onResizing(this.resizeAmount)},Mg([y("dragService")],t.prototype,"dragService",void 0),Mg([y("ctrlsService")],t.prototype,"ctrlsService",void 0),t=Mg([Q("horizontalResizeService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var jk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),fu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},eR=function(i){jk(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.hideActiveMenu=function(){this.hidePopup&&this.hidePopup()},t.prototype.showMenuAfterMouseEvent=function(e,r){var n=this;this.showPopup(e,function(o){n.popupService.positionPopupUnderMouseEvent({column:e,type:"columnMenu",mouseEvent:r,ePopup:o})},r.target)},t.prototype.showMenuAfterButtonClick=function(e,r,n){var o=this;this.showPopup(e,function(a){o.popupService.positionPopupUnderComponent({type:n,eventSource:r,ePopup:a,keepWithinBounds:!0,column:e})},r)},t.prototype.showPopup=function(e,r,n){var o=this,a=this.filterManager.getOrCreateFilterWrapper(e,"COLUMN_MENU");if(!a)throw new Error("AG Grid - unable to show popup filter, filter instantiation failed");var s=document.createElement("div");_r(s,"presentation"),s.classList.add("ag-menu"),this.tabListener=this.addManagedListener(s,"keydown",function(d){return o.trapFocusWithin(d,s)}),a.guiPromise.then(function(d){return s.appendChild(d)});var l,u=n||this.ctrlsService.getGridBodyCtrl().getGui(),c=function(d){e.setMenuVisible(!1,"contextMenu");var f=d instanceof KeyboardEvent;if(o.tabListener&&(o.tabListener=o.tabListener()),f&&n&&jl(n)){var g=o.focusService.findTabbableParent(n);g&&g.focus()}},p=this.gridOptionsWrapper.getLocaleTextFunc(),h=this.popupService.addPopup({modal:!0,eChild:s,closeOnEsc:!0,closedCallback:c,positionCallback:function(){return r(s)},anchorToElement:u,ariaLabel:p("ariaLabelColumnMenu","Column Menu")});h&&(this.hidePopup=l=h.hideFunc),a.filterPromise.then(function(d){r(s),d.afterGuiAttached&&d.afterGuiAttached({container:"columnMenu",hidePopup:l})}),e.setMenuVisible(!0,"contextMenu")},t.prototype.trapFocusWithin=function(e,r){e.key!==j.TAB||e.defaultPrevented||this.focusService.findNextFocusableElement(r,!1,e.shiftKey)||(e.preventDefault(),this.focusService.focusInto(r,e.shiftKey))},t.prototype.isMenuEnabled=function(e){return e.isFilterAllowed()},fu([y("filterManager")],t.prototype,"filterManager",void 0),fu([y("popupService")],t.prototype,"popupService",void 0),fu([y("focusService")],t.prototype,"focusService",void 0),fu([y("ctrlsService")],t.prototype,"ctrlsService",void 0),t=fu([Q("menuFactory")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Uk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),_p=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},rw=function(i){Uk(t,i);function t(e){var r=i.call(this,t.getTemplate(e.cssClass))||this;return r.items=[],r.tabbedItemScrollMap=new Map,r.params=e,e.items&&e.items.forEach(function(n){return r.addItem(n)}),r}return t.prototype.postConstruct=function(){this.createManagedBean(new ai(this.getFocusableElement(),{onTabKeyDown:this.onTabKeyDown.bind(this),handleKeyDown:this.handleKeyDown.bind(this)}))},t.getTemplate=function(e){return'<div class="ag-tabs '+e+`">
            <div ref="eHeader" role="tablist" class="ag-tabs-header `+(e?e+"-header":"")+`"></div>
            <div ref="eBody" role="presentation" class="ag-tabs-body `+(e?e+"-body":"")+`"></div>
        </div>`},t.prototype.handleKeyDown=function(e){var r=this.gridOptionsWrapper.getDocument();switch(e.key){case j.RIGHT:case j.LEFT:if(!this.eHeader.contains(r.activeElement))return;var n=this.items.indexOf(this.activeItem),o=e.key===j.RIGHT?Math.min(n+1,this.items.length-1):Math.max(n-1,0);if(n===o)return;e.preventDefault();var a=this.items[o];this.showItemWrapper(a),a.eHeaderButton.focus();break;case j.UP:case j.DOWN:e.stopPropagation();break}},t.prototype.onTabKeyDown=function(e){if(!e.defaultPrevented){var r=this,n=r.focusService,o=r.eHeader,a=r.eBody,s=r.activeItem,l=this.gridOptionsWrapper.getDocument(),u=l.activeElement;if(e.preventDefault(),o.contains(u))n.focusInto(a,e.shiftKey);else if(n.isFocusUnderManagedComponent(a))s.eHeaderButton.focus();else{var c=n.findNextFocusableElement(a,!1,e.shiftKey);c?c.focus():s.eHeaderButton.focus()}}},t.prototype.setAfterAttachedParams=function(e){this.afterAttachedParams=e},t.prototype.showFirstItem=function(){this.items.length>0&&this.showItemWrapper(this.items[0])},t.prototype.addItem=function(e){var r=document.createElement("span");_r(r,"tab"),r.setAttribute("tabIndex","-1"),r.appendChild(e.title),r.classList.add("ag-tab"),this.eHeader.appendChild(r),Cs(r,e.titleLabel);var n={tabbedItem:e,eHeaderButton:r};this.items.push(n),r.addEventListener("click",this.showItemWrapper.bind(this,n))},t.prototype.showItem=function(e){var r=this.items.find(function(n){return n.tabbedItem===e});r&&this.showItemWrapper(r)},t.prototype.showItemWrapper=function(e){var r=this,n=e.tabbedItem,o=e.eHeaderButton;if(this.params.onItemClicked&&this.params.onItemClicked({item:n}),this.activeItem===e){uO(this.params.onActiveItemClicked);return}this.lastScrollListener&&(this.lastScrollListener=this.lastScrollListener()),gn(this.eBody),n.bodyPromise.then(function(a){r.eBody.appendChild(a);var s=!r.focusService.isKeyboardMode();if(r.focusService.focusInto(r.eBody,!1,s),n.afterAttachedCallback&&n.afterAttachedCallback(r.afterAttachedParams),r.params.keepScrollPosition){var l=n.getScrollableContainer&&n.getScrollableContainer()||a;r.lastScrollListener=r.addManagedListener(l,"scroll",function(){r.tabbedItemScrollMap.set(n.name,l.scrollTop)});var u=r.tabbedItemScrollMap.get(n.name);u!==void 0&&setTimeout(function(){l.scrollTop=u},0)}}),this.activeItem&&this.activeItem.eHeaderButton.classList.remove("ag-tab-selected"),o.classList.add("ag-tab-selected"),this.activeItem=e},_p([y("focusService")],t.prototype,"focusService",void 0),_p([I("eHeader")],t.prototype,"eHeader",void 0),_p([I("eBody")],t.prototype,"eBody",void 0),_p([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function zk(i){return new ct(function(t){var e=new XMLHttpRequest;e.open("GET",i.url),e.send(),e.onreadystatechange=function(){e.readyState===4&&e.status===200&&t(JSON.parse(e.responseText))}})}/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var $k=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Kk=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},R_=50,tR=function(i){$k(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.polyfillFunctions=[],e}return t.prototype.observeResize=function(e,r){var n=this,o=this.gridOptionsWrapper.getDocument(),a=o.defaultView||window,s=uo(r,R_),l=function(){var h=new a.ResizeObserver(s);return h.observe(e),function(){return h.disconnect()}},u=function(){var h=Ym(e),d=Km(e),f=!0,g=function(){if(f){var v=Ym(e),m=Km(e),C=v!==h||m!==d;C&&(h=v,d=m,r()),n.doNextPolyfillTurn(g)}};return g(),function(){return f=!1}},c=this.gridOptionsWrapper.isSuppressBrowserResizeObserver(),p=!!a.ResizeObserver;return p&&!c?l():u()},t.prototype.doNextPolyfillTurn=function(e){this.polyfillFunctions.push(e),this.schedulePolyfill()},t.prototype.schedulePolyfill=function(){var e=this;if(!this.polyfillScheduled){var r=function(){var n=e.polyfillFunctions;e.polyfillScheduled=!1,e.polyfillFunctions=[],n.forEach(function(o){return o()})};this.polyfillScheduled=!0,this.getFrameworkOverrides().setTimeout(r,R_)}},t=Kk([Q("resizeObserverService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Yk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Fg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},rR=function(i){Yk(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.createTasksP1={list:[],sorted:!1},e.createTasksP2={list:[],sorted:!1},e.destroyTasks=[],e.ticking=!1,e.scrollGoingDown=!0,e.lastScrollTop=0,e.taskCount=0,e.cancelledTasks=new Set,e}return t.prototype.setScrollTop=function(e){this.scrollGoingDown=e>this.lastScrollTop,this.lastScrollTop=e},t.prototype.init=function(){this.useAnimationFrame=!this.gridOptionsWrapper.isSuppressAnimationFrame()},t.prototype.isOn=function(){return this.useAnimationFrame},t.prototype.verifyAnimationFrameOn=function(e){this.useAnimationFrame===!1&&console.warn("AG Grid: AnimationFrameService."+e+" called but animation frames are off")},t.prototype.createTask=function(e,r,n){this.verifyAnimationFrameOn(n);var o={task:e,index:r,createOrder:++this.taskCount};this.addTaskToList(this[n],o),this.schedule()},t.prototype.cancelTask=function(e){this.cancelledTasks.add(e)},t.prototype.addTaskToList=function(e,r){e.list.push(r),e.sorted=!1},t.prototype.sortTaskList=function(e){if(!e.sorted){var r=this.scrollGoingDown?1:-1;e.list.sort(function(n,o){return n.index!==o.index?r*(o.index-n.index):o.createOrder-n.createOrder}),e.sorted=!0}},t.prototype.addDestroyTask=function(e){this.verifyAnimationFrameOn("createTasksP3"),this.destroyTasks.push(e),this.schedule()},t.prototype.executeFrame=function(e){this.verifyAnimationFrameOn("executeFrame");for(var r=this.createTasksP1,n=r.list,o=this.createTasksP2,a=o.list,s=this.destroyTasks,l=new Date().getTime(),u=new Date().getTime()-l,c=e<=0,p=this.ctrlsService.getGridBodyCtrl();c||u<e;){var h=p.getScrollFeature().executeAnimationFrameScroll();if(!h){var d=void 0;if(n.length)this.sortTaskList(r),d=n.pop().task;else if(a.length)this.sortTaskList(o),d=a.pop().task;else if(s.length)d=s.pop();else{this.cancelledTasks.clear();break}this.cancelledTasks.has(d)||d()}u=new Date().getTime()-l}n.length||a.length||s.length?this.requestFrame():this.stopTicking()},t.prototype.stopTicking=function(){this.ticking=!1},t.prototype.flushAllFrames=function(){!this.useAnimationFrame||this.executeFrame(-1)},t.prototype.schedule=function(){!this.useAnimationFrame||this.ticking||(this.ticking=!0,this.requestFrame())},t.prototype.requestFrame=function(){var e=this.executeFrame.bind(this,60),r=this.gridOptionsWrapper.getDocument(),n=r.defaultView||window;n.requestAnimationFrame?n.requestAnimationFrame(e):n.webkitRequestAnimationFrame?n.webkitRequestAnimationFrame(e):n.setTimeout(e,0)},t.prototype.isQueueEmpty=function(){return!this.ticking},t.prototype.debounce=function(e){var r=this,n=!1;return function(){if(!r.isOn()){r.getFrameworkOverrides().setTimeout(e,0);return}n||(n=!0,r.addDestroyTask(function(){n=!1,e()}))}},Fg([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Fg([B],t.prototype,"init",null),t=Fg([Q("animationFrameService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Xk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),P_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},nR="ag-resizer-wrapper",qk='<div class="'+nR+`">
        <div ref="eTopLeftResizer" class="ag-resizer ag-resizer-topLeft"></div>
        <div ref="eTopResizer" class="ag-resizer ag-resizer-top"></div>
        <div ref="eTopRightResizer" class="ag-resizer ag-resizer-topRight"></div>
        <div ref="eRightResizer" class="ag-resizer ag-resizer-right"></div>
        <div ref="eBottomRightResizer" class="ag-resizer ag-resizer-bottomRight"></div>
        <div ref="eBottomResizer" class="ag-resizer ag-resizer-bottom"></div>
        <div ref="eBottomLeftResizer" class="ag-resizer ag-resizer-bottomLeft"></div>
        <div ref="eLeftResizer" class="ag-resizer ag-resizer-left"></div>
    </div>`,Qc=function(i){Xk(t,i);function t(e,r){var n=i.call(this)||this;return n.element=e,n.dragStartPosition={x:0,y:0},n.position={x:0,y:0},n.lastSize={width:-1,height:-1},n.positioned=!1,n.resizersAdded=!1,n.resizeListeners=[],n.boundaryEl=null,n.isResizing=!1,n.isMoving=!1,n.resizable={},n.movable=!1,n.currentResizer=null,n.config=Object.assign({},{popup:!1},r),n}return t.prototype.center=function(){var e=this.offsetParent,r=e.clientHeight,n=e.clientWidth,o=n/2-this.getWidth()/2,a=r/2-this.getHeight()/2;this.offsetElement(o,a)},t.prototype.initialisePosition=function(){var e=this.config,r=e.centered,n=e.forcePopupParentAsOffsetParent,o=e.minWidth,a=e.width,s=e.minHeight,l=e.height,u=e.x,c=e.y;this.offsetParent||this.setOffsetParent();var p=0,h=0,d=!!this.element.offsetParent;if(d){var f=this.findBoundaryElement(),g=window.getComputedStyle(f);if(g.minWidth!=null){var v=f.offsetWidth-this.element.offsetWidth;h=parseInt(g.minWidth,10)-v}if(g.minHeight!=null){var m=f.offsetHeight-this.element.offsetHeight;p=parseInt(g.minHeight,10)-m}}if(this.minHeight=s||p,this.minWidth=o||h,a&&this.setWidth(a),l&&this.setHeight(l),(!a||!l)&&this.refreshSize(),r)this.center();else if(u||c)this.offsetElement(u,c);else if(d&&n&&this.boundaryEl){var C=parseFloat(this.boundaryEl.style.top),w=parseFloat(this.boundaryEl.style.left);this.offsetElement(isNaN(w)?0:w,isNaN(C)?0:C)}this.positioned=!!this.offsetParent},t.prototype.isPositioned=function(){return this.positioned},t.prototype.getPosition=function(){return this.position},t.prototype.setMovable=function(e,r){if(!(!this.config.popup||e===this.movable)){this.movable=e;var n=this.moveElementDragListener||{eElement:r,onDragStart:this.onMoveStart.bind(this),onDragging:this.onMove.bind(this),onDragStop:this.onMoveEnd.bind(this)};e?(this.dragService.addDragSource(n),this.moveElementDragListener=n):(this.dragService.removeDragSource(n),this.moveElementDragListener=void 0)}},t.prototype.setResizable=function(e){var r=this;if(this.clearResizeListeners(),e?this.addResizers():this.removeResizers(),typeof e=="boolean"){if(e===!1)return;e={topLeft:e,top:e,topRight:e,right:e,bottomRight:e,bottom:e,bottomLeft:e,left:e}}Object.keys(e).forEach(function(n){var o=e,a=!!o[n],s=r.getResizerElement(n),l={dragStartPixels:0,eElement:s,onDragStart:function(u){return r.onResizeStart(u,n)},onDragging:r.onResize.bind(r),onDragStop:function(u){return r.onResizeEnd(u,n)}};(!!r.resizable[n]!==a||!r.isAlive()&&!a)&&(a?(r.dragService.addDragSource(l),r.resizeListeners.push(l),s.style.pointerEvents="all"):s.style.pointerEvents="none")})},t.prototype.removeSizeFromEl=function(){this.element.style.removeProperty("height"),this.element.style.removeProperty("width"),this.element.style.removeProperty("flex")},t.prototype.restoreLastSize=function(){this.element.style.flex="0 0 auto";var e=this.lastSize,r=e.height,n=e.width;n!==-1&&(this.element.style.width=n+"px"),r!==-1&&(this.element.style.height=r+"px")},t.prototype.getHeight=function(){return this.element.offsetHeight},t.prototype.setHeight=function(e){var r=this.config.popup,n=this.element,o=!1;if(typeof e=="string"&&e.indexOf("%")!==-1)ts(n,e),e=PC(n),o=!0;else if(this.positioned){var a=this.element.getBoundingClientRect(),s=this.offsetParent.getBoundingClientRect();e=Math.max(this.minHeight,e);var l=this.offsetParent.clientHeight,u=r?this.position.y:a.top,c=r?0:s.top;l&&e+u>l+c&&(e=l-u)}this.getHeight()!==e&&(o?(n.style.maxHeight="unset",n.style.minHeight="unset"):r?ts(n,e):(n.style.height=e+"px",n.style.flex="0 0 auto",this.lastSize.height=typeof e=="number"?e:parseFloat(e)))},t.prototype.getWidth=function(){return this.element.offsetWidth},t.prototype.setWidth=function(e){var r=this.element,n=this.config.popup,o=!1;if(typeof e=="string"&&e.indexOf("%")!==-1)Li(r,e),e=rf(r),o=!0;else if(this.positioned){e=Math.max(this.minWidth,e);var a=this.offsetParent.clientWidth,s=n?this.position.x:this.element.getBoundingClientRect().left;a&&e+s>a&&(e=a-s)}this.getWidth()!==e&&(o?(r.style.maxWidth="unset",r.style.minWidth="unset"):this.config.popup?Li(r,e):(r.style.width=e+"px",r.style.flex=" unset",this.lastSize.width=typeof e=="number"?e:parseFloat(e)))},t.prototype.offsetElement=function(e,r){e===void 0&&(e=0),r===void 0&&(r=0);var n=this.config.forcePopupParentAsOffsetParent?this.boundaryEl:this.element;this.popupService.positionPopup({ePopup:n,x:e,y:r,keepWithinBounds:!0}),this.setPosition(parseFloat(n.style.left),parseFloat(n.style.top))},t.prototype.setPosition=function(e,r){this.position.x=e,this.position.y=r},t.prototype.updateDragStartPosition=function(e,r){this.dragStartPosition={x:e,y:r}},t.prototype.calculateMouseMovement=function(e){var r=e.e,n=e.isLeft,o=e.isTop,a=e.anywhereWithin,s=e.topBuffer,l=r.clientX-this.dragStartPosition.x,u=r.clientY-this.dragStartPosition.y,c=this.shouldSkipX(r,!!n,!!a,l)?0:l,p=this.shouldSkipY(r,!!o,s,u)?0:u;return{movementX:c,movementY:p}},t.prototype.shouldSkipX=function(e,r,n,o){var a=this.element.getBoundingClientRect(),s=this.offsetParent.getBoundingClientRect(),l=this.boundaryEl.getBoundingClientRect(),u=this.config.popup?this.position.x:a.left,c=u<=0&&s.left>=e.clientX||s.right<=e.clientX&&s.right<=l.right;return c?!0:(r?c=o<0&&e.clientX>u+s.left||o>0&&e.clientX<u+s.left:n?c=o<0&&e.clientX>l.right||o>0&&e.clientX<u+s.left:c=o<0&&e.clientX>l.right||o>0&&e.clientX<l.right,c)},t.prototype.shouldSkipY=function(e,r,n,o){n===void 0&&(n=0);var a=this.element.getBoundingClientRect(),s=this.offsetParent.getBoundingClientRect(),l=this.boundaryEl.getBoundingClientRect(),u=this.config.popup?this.position.y:a.top,c=u<=0&&s.top>=e.clientY||s.bottom<=e.clientY&&s.bottom<=l.bottom;return c?!0:(r?c=o<0&&e.clientY>u+s.top+n||o>0&&e.clientY<u+s.top:c=o<0&&e.clientY>l.bottom||o>0&&e.clientY<l.bottom,c)},t.prototype.createResizeMap=function(){var e=this.element;this.resizerMap={topLeft:{element:e.querySelector("[ref=eTopLeftResizer]")},top:{element:e.querySelector("[ref=eTopResizer]")},topRight:{element:e.querySelector("[ref=eTopRightResizer]")},right:{element:e.querySelector("[ref=eRightResizer]")},bottomRight:{element:e.querySelector("[ref=eBottomRightResizer]")},bottom:{element:e.querySelector("[ref=eBottomResizer]")},bottomLeft:{element:e.querySelector("[ref=eBottomLeftResizer]")},left:{element:e.querySelector("[ref=eLeftResizer]")}}},t.prototype.addResizers=function(){if(!this.resizersAdded){var e=this.element;if(!!e){var r=new DOMParser,n=r.parseFromString(qk,"text/html").body;e.appendChild(n.firstChild),this.createResizeMap(),this.resizersAdded=!0}}},t.prototype.removeResizers=function(){this.resizerMap=void 0;var e=this.element.querySelector("."+nR);e&&this.element.removeChild(e),this.resizersAdded=!1},t.prototype.getResizerElement=function(e){return this.resizerMap[e].element},t.prototype.onResizeStart=function(e,r){this.boundaryEl=this.findBoundaryElement(),this.positioned||this.initialisePosition(),this.currentResizer={isTop:!!r.match(/top/i),isRight:!!r.match(/right/i),isBottom:!!r.match(/bottom/i),isLeft:!!r.match(/left/i)},this.element.classList.add("ag-resizing"),this.resizerMap[r].element.classList.add("ag-active");var n=this.config,o=n.popup,a=n.forcePopupParentAsOffsetParent;!o&&!a&&this.applySizeToSiblings(this.currentResizer.isBottom||this.currentResizer.isTop),this.isResizing=!0,this.updateDragStartPosition(e.clientX,e.clientY)},t.prototype.getSiblings=function(){var e=this.element,r=e.parentElement;return r?Array.prototype.slice.call(r.children).filter(function(n){return!n.classList.contains("ag-hidden")}):null},t.prototype.getMinSizeOfSiblings=function(){for(var e=this.getSiblings()||[],r=0,n=0,o=0;o<e.length;o++){var a=e[o],s=!!a.style.flex&&a.style.flex!=="0 0 auto";if(a!==this.element){var l=this.minHeight||0,u=this.minWidth||0;if(s){var c=window.getComputedStyle(a);c.minHeight&&(l=parseInt(c.minHeight,10)),c.minWidth&&(u=parseInt(c.minWidth,10))}else l=a.offsetHeight,u=a.offsetWidth;r+=l,n+=u}}return{height:r,width:n}},t.prototype.applySizeToSiblings=function(e){var r=null,n=this.getSiblings();if(!!n){for(var o=0;o<n.length;o++){var a=n[o];a!==r&&(e?a.style.height=a.offsetHeight+"px":a.style.width=a.offsetWidth+"px",a.style.flex="0 0 auto",a===this.element&&(r=n[o+1]))}r&&(r.style.removeProperty("height"),r.style.removeProperty("min-height"),r.style.removeProperty("max-height"),r.style.flex="1 1 auto")}},t.prototype.onResize=function(e){if(!(!this.isResizing||!this.currentResizer)){var r=this.config,n=r.popup,o=r.forcePopupParentAsOffsetParent,a=this.currentResizer,s=a.isTop,l=a.isRight,u=a.isBottom,c=a.isLeft,p=l||c,h=u||s,d=this.calculateMouseMovement({e,isLeft:c,isTop:s}),f=d.movementX,g=d.movementY,v=this.position.x,m=this.position.y,C=0,w=0;if(p&&f){var b=c?-1:1,S=this.getWidth(),T=S+f*b,E=!1;c&&(C=S-T,(v+C<=0||T<=this.minWidth)&&(E=!0,C=0)),E||this.setWidth(T)}if(h&&g){var b=s?-1:1,P=this.getHeight(),R=P+g*b,M=!1;s?(w=P-R,(m+w<=0||R<=this.minHeight)&&(M=!0,w=0)):!this.config.popup&&!this.config.forcePopupParentAsOffsetParent&&P<R&&this.getMinSizeOfSiblings().height+R>this.element.parentElement.offsetHeight&&(M=!0),M||this.setHeight(R)}this.updateDragStartPosition(e.clientX,e.clientY),((n||o)&&C||w)&&this.offsetElement(v+C,m+w)}},t.prototype.onResizeEnd=function(e,r){this.isResizing=!1,this.currentResizer=null,this.boundaryEl=null;var n={type:"resize",api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi()};this.element.classList.remove("ag-resizing"),this.resizerMap[r].element.classList.remove("ag-active"),this.dispatchEvent(n)},t.prototype.refreshSize=function(){var e=this.element;this.config.popup&&(this.config.width||this.setWidth(e.offsetWidth),this.config.height||this.setHeight(e.offsetHeight))},t.prototype.onMoveStart=function(e){this.boundaryEl=this.findBoundaryElement(),this.positioned||this.initialisePosition(),this.isMoving=!0,this.element.classList.add("ag-moving"),this.updateDragStartPosition(e.clientX,e.clientY)},t.prototype.onMove=function(e){if(!!this.isMoving){var r=this.position,n=r.x,o=r.y,a;this.config.calculateTopBuffer&&(a=this.config.calculateTopBuffer());var s=this.calculateMouseMovement({e,isTop:!0,anywhereWithin:!0,topBuffer:a}),l=s.movementX,u=s.movementY;this.offsetElement(n+l,o+u),this.updateDragStartPosition(e.clientX,e.clientY)}},t.prototype.onMoveEnd=function(){this.isMoving=!1,this.boundaryEl=null,this.element.classList.remove("ag-moving")},t.prototype.setOffsetParent=function(){this.config.forcePopupParentAsOffsetParent?this.offsetParent=this.popupService.getPopupParent():this.offsetParent=this.element.offsetParent},t.prototype.findBoundaryElement=function(){for(var e=this.element;e;){if(window.getComputedStyle(e).position!=="static")return e;e=e.parentElement}return this.element},t.prototype.clearResizeListeners=function(){for(;this.resizeListeners.length;){var e=this.resizeListeners.pop();this.dragService.removeDragSource(e)}},t.prototype.destroy=function(){i.prototype.destroy.call(this),this.moveElementDragListener&&this.dragService.removeDragSource(this.moveElementDragListener),this.clearResizeListeners(),this.removeResizers()},P_([y("popupService")],t.prototype,"popupService",void 0),P_([y("dragService")],t.prototype,"dragService",void 0),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Zk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),gu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},iR=function(i){Zk(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){var e=this;this.ctrlsService.whenReady(function(r){e.centerRowContainerCtrl=r.centerRowContainerCtrl})},t.prototype.getPreferredWidthForColumn=function(e,r){var n=this.getHeaderCellForColumn(e);if(!n)return-1;var o=this.rowRenderer.getAllCellsForColumn(e);return r||o.push(n),this.addElementsToContainerAndGetWidth(o)},t.prototype.getPreferredWidthForColumnGroup=function(e){var r=this.getHeaderCellForColumn(e);return r?this.addElementsToContainerAndGetWidth([r]):-1},t.prototype.addElementsToContainerAndGetWidth=function(e){var r=this,n=document.createElement("span");n.style.position="fixed";var o=this.centerRowContainerCtrl.getContainerElement();o.appendChild(n),e.forEach(function(l){return r.cloneItemIntoDummy(l,n)});var a=n.offsetWidth;o.removeChild(n);var s=this.gridOptionsWrapper.getAutoSizePadding();return a+s},t.prototype.getHeaderCellForColumn=function(e){var r=null;return this.ctrlsService.getHeaderRowContainerCtrls().forEach(function(n){var o=n.getHtmlElementForColumnHeader(e);o!=null&&(r=o)}),r},t.prototype.cloneItemIntoDummy=function(e,r){var n=e.cloneNode(!0);n.style.width="",n.style.position="static",n.style.left="";var o=document.createElement("div"),a=o.classList,s=["ag-header-cell","ag-header-group-cell"].some(function(h){return n.classList.contains(h)});s?(a.add("ag-header","ag-header-row"),o.style.position="static"):a.add("ag-row");for(var l=e.parentElement;l;){var u=["ag-header-row","ag-row"].some(function(h){return l.classList.contains(h)});if(u){for(var c=0;c<l.classList.length;c++){var p=l.classList[c];p!="ag-row-position-absolute"&&a.add(p)}break}l=l.parentElement}o.appendChild(n),r.appendChild(o)},gu([y("rowRenderer")],t.prototype,"rowRenderer",void 0),gu([y("ctrlsService")],t.prototype,"ctrlsService",void 0),gu([y("rowCssClassCalculator")],t.prototype,"rowCssClassCalculator",void 0),gu([B],t.prototype,"postConstruct",null),t=gu([Q("autoWidthCalculator")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Qk=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Cn=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Jk=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},A_=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(Jk(arguments[t]));return i},oR=function(i){Qk(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.destroyFuncsForColumnListeners=[],e.rowCtrlsByRowIndex={},e.zombieRowCtrls={},e.allRowCtrls=[],e.topRowCtrls=[],e.bottomRowCtrls=[],e.refreshInProgress=!1,e}return t.prototype.postConstruct=function(){var e=this;this.ctrlsService.whenReady(function(){e.gridBodyCtrl=e.ctrlsService.getGridBodyCtrl(),e.initialise()})},t.prototype.initialise=function(){this.addManagedListener(this.eventService,x.EVENT_PAGINATION_CHANGED,this.onPageLoaded.bind(this)),this.addManagedListener(this.eventService,x.EVENT_PINNED_ROW_DATA_CHANGED,this.onPinnedRowDataChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,this.onDisplayedColumnsChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_BODY_SCROLL,this.redrawAfterScroll.bind(this)),this.addManagedListener(this.eventService,x.EVENT_BODY_HEIGHT_CHANGED,this.redrawAfterScroll.bind(this)),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_DOM_LAYOUT,this.onDomLayoutChanged.bind(this)),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_ROW_CLASS,this.redrawRows.bind(this)),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.onNewColumnsLoaded.bind(this)),this.registerCellEventListeners(),this.initialiseCache(),this.printLayout=this.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_PRINT,this.embedFullWidthRows=this.printLayout||this.gridOptionsWrapper.isEmbedFullWidthRows(),this.redrawAfterModelUpdate()},t.prototype.initialiseCache=function(){if(this.gridOptionsWrapper.isKeepDetailRows()){var e=this.gridOptionsWrapper.getKeepDetailRowsCount(),r=e!=null?e:3;this.cachedRowCtrls=new e2(r)}},t.prototype.getRowCtrls=function(){return this.allRowCtrls},t.prototype.updateAllRowCtrls=function(){var e=Jn(this.rowCtrlsByRowIndex);this.gridOptionsWrapper.isEnsureDomOrder()&&e.sort(function(o,a){return o.getRowNode().rowIndex-a.getRowNode.rowIndex});var r=Jn(this.zombieRowCtrls),n=this.cachedRowCtrls?this.cachedRowCtrls.getEntries():[];this.allRowCtrls=A_(e,r,n)},t.prototype.registerCellEventListeners=function(){var e=this;this.addManagedListener(this.eventService,x.EVENT_CELL_FOCUSED,function(n){e.getAllCellCtrls().forEach(function(o){return o.onCellFocused(n)}),e.getFullWidthRowCtrls().forEach(function(o){o.onFullWidthRowFocused(n)})}),this.addManagedListener(this.eventService,x.EVENT_FLASH_CELLS,function(n){e.getAllCellCtrls().forEach(function(o){return o.onFlashCells(n)})}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_HOVER_CHANGED,function(){e.getAllCellCtrls().forEach(function(n){return n.onColumnHover()})}),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,function(){e.getAllCellCtrls().forEach(function(n){return n.onDisplayedColumnsChanged()})}),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,function(){e.printLayout&&e.getAllCellCtrls().forEach(function(n){return n.onLeftChanged()})});var r=this.gridOptionsWrapper.isEnableRangeSelection();r&&(this.addManagedListener(this.eventService,x.EVENT_RANGE_SELECTION_CHANGED,function(){e.getAllCellCtrls().forEach(function(n){return n.onRangeSelectionChanged()})}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_MOVED,function(){e.getAllCellCtrls().forEach(function(n){return n.updateRangeBordersIfRangeCount()})}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PINNED,function(){e.getAllCellCtrls().forEach(function(n){return n.updateRangeBordersIfRangeCount()})}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VISIBLE,function(){e.getAllCellCtrls().forEach(function(n){return n.updateRangeBordersIfRangeCount()})})),this.refreshListenersToColumnsForCellComps(),this.addManagedListener(this.eventService,x.EVENT_GRID_COLUMNS_CHANGED,this.refreshListenersToColumnsForCellComps.bind(this)),this.addDestroyFunc(this.removeGridColumnListeners.bind(this))},t.prototype.removeGridColumnListeners=function(){this.destroyFuncsForColumnListeners.forEach(function(e){return e()}),this.destroyFuncsForColumnListeners.length=0},t.prototype.refreshListenersToColumnsForCellComps=function(){var e=this;this.removeGridColumnListeners();var r=this.columnModel.getAllGridColumns();!r||r.forEach(function(n){var o=function(c){e.getAllCellCtrls().forEach(function(p){p.getColumn()===n&&c(p)})},a=function(){o(function(c){return c.onLeftChanged()})},s=function(){o(function(c){return c.onWidthChanged()})},l=function(){o(function(c){return c.onFirstRightPinnedChanged()})},u=function(){o(function(c){return c.onLastLeftPinnedChanged()})};n.addEventListener(Se.EVENT_LEFT_CHANGED,a),n.addEventListener(Se.EVENT_WIDTH_CHANGED,s),n.addEventListener(Se.EVENT_FIRST_RIGHT_PINNED_CHANGED,l),n.addEventListener(Se.EVENT_LAST_LEFT_PINNED_CHANGED,u),e.destroyFuncsForColumnListeners.push(function(){n.removeEventListener(Se.EVENT_LEFT_CHANGED,a),n.removeEventListener(Se.EVENT_WIDTH_CHANGED,s),n.removeEventListener(Se.EVENT_FIRST_RIGHT_PINNED_CHANGED,l),n.removeEventListener(Se.EVENT_LAST_LEFT_PINNED_CHANGED,u)})})},t.prototype.onDomLayoutChanged=function(){var e=this.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_PRINT,r=e||this.gridOptionsWrapper.isEmbedFullWidthRows(),n=r!==this.embedFullWidthRows||this.printLayout!==e;this.printLayout=e,this.embedFullWidthRows=r,n&&this.redrawAfterModelUpdate()},t.prototype.datasourceChanged=function(){this.firstRenderedRow=0,this.lastRenderedRow=-1;var e=Object.keys(this.rowCtrlsByRowIndex);this.removeRowCtrls(e)},t.prototype.onPageLoaded=function(e){var r={recycleRows:e.keepRenderedRows,animate:e.animate,newData:e.newData,newPage:e.newPage,onlyBody:!0};this.redrawAfterModelUpdate(r)},t.prototype.getAllCellsForColumn=function(e){var r=[];return this.getAllRowCtrls().forEach(function(n){var o=n.getCellElement(e);o&&r.push(o)}),r},t.prototype.refreshFloatingRowComps=function(){this.refreshFloatingRows(this.topRowCtrls,this.pinnedRowModel.getPinnedTopRowData()),this.refreshFloatingRows(this.bottomRowCtrls,this.pinnedRowModel.getPinnedBottomRowData())},t.prototype.getTopRowCtrls=function(){return this.topRowCtrls},t.prototype.getBottomRowCtrls=function(){return this.bottomRowCtrls},t.prototype.refreshFloatingRows=function(e,r){var n=this;e.forEach(function(o){o.destroyFirstPass(),o.destroySecondPass()}),e.length=0,r&&r.forEach(function(o){var a=new ra(o,n.beans,!1,!1,n.printLayout);e.push(a)})},t.prototype.onPinnedRowDataChanged=function(){var e={recycleRows:!0};this.redrawAfterModelUpdate(e)},t.prototype.getRenderedIndexesForRowNodes=function(e){var r=[];return me(e)||Ke(this.rowCtrlsByRowIndex,function(n,o){var a=o.getRowNode();e.indexOf(a)>=0&&r.push(n)}),r},t.prototype.redrawRows=function(e){var r=e!=null&&e.length>0;if(r){var n=this.getRenderedIndexesForRowNodes(e);this.removeRowCtrls(n)}this.redrawAfterModelUpdate({recycleRows:r})},t.prototype.getCellToRestoreFocusToAfterRefresh=function(e){var r=e.suppressKeepFocus?null:this.focusService.getFocusCellToUseAfterRefresh();if(r==null)return null;var n=this.gridOptionsWrapper.getDocument(),o=n.activeElement,a=this.gridOptionsWrapper.getDomData(o,ta.DOM_DATA_KEY_CELL_CTRL),s=this.gridOptionsWrapper.getDomData(o,ra.DOM_DATA_KEY_ROW_CTRL),l=a||s;return l?r:null},t.prototype.redrawAfterModelUpdate=function(e){e===void 0&&(e={}),this.getLockOnRefresh();var r=this.getCellToRestoreFocusToAfterRefresh(e);this.updateContainerHeights(),this.scrollToTopIfNewData(e);var n=!this.printLayout&&!!e.recycleRows,o=e.animate&&this.gridOptionsWrapper.isAnimateRows(),a=n?this.recycleRows():null;n||this.removeAllRowComps();var s=function(){if(r==null||a==null)return!1;var u=!1;return Ke(a,function(c,p){var h=p.getRowNode(),d=h.rowIndex==r.rowIndex,f=h.rowPinned==r.rowPinned;d&&f&&(u=!0)}),u},l=s();this.redraw(a,o),e.onlyBody||this.refreshFloatingRowComps(),this.dispatchDisplayedRowsChanged(),l||this.restoreFocusedCell(r),this.releaseLockOnRefresh()},t.prototype.scrollToTopIfNewData=function(e){var r=e.newData||e.newPage,n=this.gridOptionsWrapper.isSuppressScrollOnNewData();r&&!n&&this.gridBodyCtrl.getScrollFeature().scrollToTop()},t.prototype.updateContainerHeights=function(){if(this.printLayout){this.rowContainerHeightService.setModelHeight(null);return}var e=this.paginationProxy.getCurrentPageHeight();e===0&&(e=1),this.rowContainerHeightService.setModelHeight(e)},t.prototype.getLockOnRefresh=function(){if(this.refreshInProgress)throw new Error("AG Grid: cannot get grid to draw rows when it is in the middle of drawing rows. Your code probably called a grid API method while the grid was in the render stage. To overcome this, put the API call into a timeout, e.g. instead of api.redrawRows(), call setTimeout(function() { api.redrawRows(); }, 0). To see what part of your code that caused the refresh check this stacktrace.");this.refreshInProgress=!0},t.prototype.releaseLockOnRefresh=function(){this.refreshInProgress=!1},t.prototype.isRefreshInProgress=function(){return this.refreshInProgress},t.prototype.restoreFocusedCell=function(e){e&&this.focusService.setFocusedCell(e.rowIndex,e.column,e.rowPinned,!0)},t.prototype.stopEditing=function(e){e===void 0&&(e=!1),this.getAllRowCtrls().forEach(function(r){r.stopEditing(e)})},t.prototype.onNewColumnsLoaded=function(){this.getAllCellCtrls().forEach(function(e){return e.onNewColumnsLoaded()})},t.prototype.getAllCellCtrls=function(){var e=[];return this.getAllRowCtrls().forEach(function(r){return e=e.concat(r.getAllCellCtrls())}),e},t.prototype.getAllRowCtrls=function(){var e=this,r=A_(this.topRowCtrls,this.bottomRowCtrls);return Object.keys(this.rowCtrlsByRowIndex).forEach(function(n){return r.push(e.rowCtrlsByRowIndex[n])}),r},t.prototype.addRenderedRowListener=function(e,r,n){var o=this.rowCtrlsByRowIndex[r];o&&o.addEventListener(e,n)},t.prototype.flashCells=function(e){e===void 0&&(e={});var r=e.flashDelay,n=e.fadeDelay;this.getCellCtrls(e.rowNodes,e.columns).forEach(function(o){return o.flashCell({flashDelay:r,fadeDelay:n})})},t.prototype.refreshCells=function(e){e===void 0&&(e={});var r={forceRefresh:e.force,newData:!1,suppressFlash:e.suppressFlash};this.getCellCtrls(e.rowNodes,e.columns).forEach(function(n){if(n.refreshShouldDestroy()){var o=n.getRowCtrl();o&&o.refreshCell(n)}else n.refreshCell(r)}),this.getFullWidthRowCtrls(e.rowNodes).forEach(function(n){n.refreshFullWidth()})},t.prototype.getCellRendererInstances=function(e){var r=this.getCellCtrls(e.rowNodes,e.columns).map(function(n){return n.getCellRenderer()}).filter(function(n){return n!=null});return r},t.prototype.getCellEditorInstances=function(e){var r=[];return this.getCellCtrls(e.rowNodes,e.columns).forEach(function(n){var o=n.getCellEditor();o&&r.push(o)}),r},t.prototype.getEditingCells=function(){var e=[];return this.getAllCellCtrls().forEach(function(r){if(r.isEditing()){var n=r.getCellPosition();e.push(n)}}),e},t.prototype.mapRowNodes=function(e){if(!!e){var r={top:{},bottom:{},normal:{}};return e.forEach(function(n){var o=n.id;n.rowPinned===L.PINNED_TOP?r.top[o]=n:n.rowPinned===L.PINNED_BOTTOM?r.bottom[o]=n:r.normal[o]=n}),r}},t.prototype.isRowInMap=function(e,r){var n=e.id,o=e.rowPinned;return o===L.PINNED_BOTTOM?r.bottom[n]!=null:o===L.PINNED_TOP?r.top[n]!=null:r.normal[n]!=null},t.prototype.getCellCtrls=function(e,r){var n=this,o=this.mapRowNodes(e),a=[],s;q(r)&&(s={},r.forEach(function(u){var c=n.columnModel.getGridColumn(u);q(c)&&(s[c.getId()]=!0)}));var l=function(u){var c=u.getRowNode();o!=null&&!n.isRowInMap(c,o)||u.getAllCellCtrls().forEach(function(p){var h=p.getColumn().getId(),d=s&&!s[h];d||a.push(p)})};return Ke(this.rowCtrlsByRowIndex,function(u,c){l(c)}),this.topRowCtrls&&this.topRowCtrls.forEach(l),this.bottomRowCtrls&&this.bottomRowCtrls.forEach(l),a},t.prototype.destroy=function(){this.removeAllRowComps(),i.prototype.destroy.call(this)},t.prototype.removeAllRowComps=function(){var e=Object.keys(this.rowCtrlsByRowIndex);this.removeRowCtrls(e)},t.prototype.recycleRows=function(){var e=[];Ke(this.rowCtrlsByRowIndex,function(n,o){var a=o.getRowNode().id==null;a&&e.push(n)}),this.removeRowCtrls(e);var r={};return Ke(this.rowCtrlsByRowIndex,function(n,o){var a=o.getRowNode();r[a.id]=o}),this.rowCtrlsByRowIndex={},r},t.prototype.removeRowCtrls=function(e){var r=this;e.forEach(function(n){var o=r.rowCtrlsByRowIndex[n];o&&(o.destroyFirstPass(),o.destroySecondPass()),delete r.rowCtrlsByRowIndex[n]})},t.prototype.redrawAfterScroll=function(){this.getLockOnRefresh(),this.redraw(null,!1,!0),this.releaseLockOnRefresh(),this.dispatchDisplayedRowsChanged()},t.prototype.removeRowCompsNotToDraw=function(e){var r={};e.forEach(function(a){return r[a]=!0});var n=Object.keys(this.rowCtrlsByRowIndex),o=n.filter(function(a){return!r[a]});this.removeRowCtrls(o)},t.prototype.calculateIndexesToDraw=function(e){var r=this,n=JO(this.firstRenderedRow,this.lastRenderedRow),o=function(a,s){var l=s.getRowNode().rowIndex;l!=null&&(l<r.firstRenderedRow||l>r.lastRenderedRow)&&r.doNotUnVirtualiseRow(s)&&n.push(l)};return Ke(this.rowCtrlsByRowIndex,o),Ke(e,o),n.sort(function(a,s){return a-s}),n},t.prototype.redraw=function(e,r,n){var o=this;r===void 0&&(r=!1),n===void 0&&(n=!1),this.rowContainerHeightService.updateOffset(),this.workOutFirstAndLastRowsToRender();var a=this.calculateIndexesToDraw(e);if(this.removeRowCompsNotToDraw(a),this.printLayout&&(r=!1),a.forEach(function(l){var u=o.createOrUpdateRowCtrl(l,e,r,n);q(u)}),e){var s=n&&!this.gridOptionsWrapper.isSuppressAnimationFrame()&&!this.printLayout;s?this.beans.animationFrameService.addDestroyTask(function(){o.destroyRowCtrls(e,r),o.updateAllRowCtrls(),o.dispatchDisplayedRowsChanged()}):this.destroyRowCtrls(e,r)}this.updateAllRowCtrls(),this.gridBodyCtrl.updateRowCount()},t.prototype.dispatchDisplayedRowsChanged=function(){var e={type:x.EVENT_DISPLAYED_ROWS_CHANGED};this.eventService.dispatchEvent(e)},t.prototype.onDisplayedColumnsChanged=function(){var e=this.columnModel.isPinningLeft(),r=this.columnModel.isPinningRight(),n=this.pinningLeft!==e||r!==this.pinningRight;n&&(this.pinningLeft=e,this.pinningRight=r,this.embedFullWidthRows&&this.redrawFullWidthEmbeddedRows())},t.prototype.redrawFullWidthEmbeddedRows=function(){var e=[];this.getFullWidthRowCtrls().forEach(function(r){var n=r.getRowNode().rowIndex;e.push(n.toString())}),this.refreshFloatingRowComps(),this.removeRowCtrls(e),this.redrawAfterScroll()},t.prototype.getFullWidthRowCtrls=function(e){var r=this,n=this.mapRowNodes(e);return Jn(this.rowCtrlsByRowIndex).filter(function(o){if(!o.isFullWidth())return!1;var a=o.getRowNode();return!(n!=null&&!r.isRowInMap(a,n))})},t.prototype.refreshFullWidthRows=function(e){var r=[],n=!!e,o=n?{}:void 0;n&&o&&e.forEach(function(a){return o[a.id]=!0}),this.getFullWidthRowCtrls().forEach(function(a){var s=a.getRowNode();if(n&&o){var l=s.level>0&&s.parent?s.parent.id:void 0,u=!o[s.id]&&!o[l];if(u)return}var c=a.refreshFullWidth();if(!c){var p=a.getRowNode().rowIndex;r.push(p.toString())}}),this.removeRowCtrls(r),this.redrawAfterScroll()},t.prototype.createOrUpdateRowCtrl=function(e,r,n,o){var a,s=this.rowCtrlsByRowIndex[e];s||(a=this.paginationProxy.getRow(e),q(a)&&q(r)&&r[a.id]&&a.alreadyRendered&&(s=r[a.id],r[a.id]=null));var l=!s;if(l)if(a||(a=this.paginationProxy.getRow(e)),q(a))s=this.createRowCon(a,n,o);else return;return a&&(a.alreadyRendered=!0),this.rowCtrlsByRowIndex[e]=s,s},t.prototype.destroyRowCtrls=function(e,r){var n=this,o=[];Ke(e,function(a,s){if(!!s){if(n.cachedRowCtrls&&s.isCacheable()){n.cachedRowCtrls.addRow(s);return}s.destroyFirstPass(),r?(n.zombieRowCtrls[s.getInstanceId()]=s,o.push(function(){s.destroySecondPass(),delete n.zombieRowCtrls[s.getInstanceId()]})):s.destroySecondPass()}}),r&&(o.push(function(){n.updateAllRowCtrls(),n.dispatchDisplayedRowsChanged()}),aO(o))},t.prototype.workOutFirstAndLastRowsToRender=function(){var e=this,r,n;if(!this.paginationProxy.isRowsToRender())r=0,n=-1;else if(this.printLayout)r=this.paginationProxy.getPageFirstRow(),n=this.paginationProxy.getPageLastRow();else{var o=this.gridOptionsWrapper.getRowBufferInPixels(),a=this.ctrlsService.getGridBodyCtrl(),s=this.gridOptionsWrapper.isSuppressRowVirtualisation(),l=!1,u=void 0,c=void 0;do{var p=this.paginationProxy.getPixelOffset(),h=this.paginationProxy.getCurrentPagePixelRange(),d=h.pageFirstPixel,f=h.pageLastPixel,g=this.rowContainerHeightService.getDivStretchOffset();if(s)u=d+g,c=f+g;else{var v=a.getScrollFeature().getVScrollPosition(),m=v.top,C=v.bottom;u=Math.max(m+p-o,d)+g,c=Math.min(C+p+o,f)+g}l=this.ensureAllRowsInRangeHaveHeightsCalculated(u,c)}while(l);var w=this.paginationProxy.getRowIndexAtPixel(u),b=this.paginationProxy.getRowIndexAtPixel(c),S=this.paginationProxy.getPageFirstRow(),T=this.paginationProxy.getPageLastRow();w<S&&(w=S),b>T&&(b=T),r=w,n=b}var E=this.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_NORMAL,P=this.gridOptionsWrapper.isSuppressMaxRenderedRowRestriction(),R=Math.max(this.gridOptionsWrapper.getRowBuffer(),500);E&&!P&&n-r>R&&(n=r+R);var M=r!==this.firstRenderedRow,N=n!==this.lastRenderedRow;if(M||N){this.firstRenderedRow=r,this.lastRenderedRow=n;var W={type:x.EVENT_VIEWPORT_CHANGED,firstRow:r,lastRow:n,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(W)}if(this.paginationProxy.isRowsToRender()){var G={type:x.EVENT_FIRST_DATA_RENDERED,firstRow:r,lastRow:n,api:this.gridApi,columnApi:this.columnApi};window.setTimeout(function(){return e.eventService.dispatchEventOnce(G)},50)}},t.prototype.ensureAllRowsInRangeHaveHeightsCalculated=function(e,r){var n=this.paginationProxy.ensureRowHeightsValid(e,r,-1,-1);return n&&this.updateContainerHeights(),n},t.prototype.getFirstVirtualRenderedRow=function(){return this.firstRenderedRow},t.prototype.getLastVirtualRenderedRow=function(){return this.lastRenderedRow},t.prototype.doNotUnVirtualiseRow=function(e){var r=!1,n=!0,o=e.getRowNode(),a=this.focusService.isRowNodeFocused(o),s=e.isEditing(),l=o.detail,u=a||s||l;if(!u)return r;var c=this.paginationProxy.isRowPresent(o);return c?n:r},t.prototype.createRowCon=function(e,r,n){var o=this.cachedRowCtrls?this.cachedRowCtrls.getRow(e):null;if(o)return o;var a=this.gridOptionsWrapper.isSuppressAnimationFrame(),s=n&&!a&&!this.printLayout,l=new ra(e,this.beans,r,s,this.printLayout);return l},t.prototype.getRenderedNodes=function(){var e=this.rowCtrlsByRowIndex;return Object.keys(e).map(function(r){return e[r].getRowNode()})},t.prototype.getRowByPosition=function(e){var r;switch(e.rowPinned){case L.PINNED_TOP:r=this.topRowCtrls[e.rowIndex];break;case L.PINNED_BOTTOM:r=this.bottomRowCtrls[e.rowIndex];break;default:r=this.rowCtrlsByRowIndex[e.rowIndex];break}return r},t.prototype.getRowNode=function(e){switch(e.rowPinned){case L.PINNED_TOP:return this.pinnedRowModel.getPinnedTopRowData()[e.rowIndex];case L.PINNED_BOTTOM:return this.pinnedRowModel.getPinnedBottomRowData()[e.rowIndex];default:return this.rowModel.getRow(e.rowIndex)}},t.prototype.isRangeInRenderedViewport=function(e,r){var n=e==null||r==null;if(n)return!1;var o=e>this.lastRenderedRow,a=r<this.firstRenderedRow,s=!a&&!o;return s},Cn([y("paginationProxy")],t.prototype,"paginationProxy",void 0),Cn([y("columnModel")],t.prototype,"columnModel",void 0),Cn([y("pinnedRowModel")],t.prototype,"pinnedRowModel",void 0),Cn([y("rowModel")],t.prototype,"rowModel",void 0),Cn([y("focusService")],t.prototype,"focusService",void 0),Cn([y("columnApi")],t.prototype,"columnApi",void 0),Cn([y("gridApi")],t.prototype,"gridApi",void 0),Cn([y("beans")],t.prototype,"beans",void 0),Cn([y("rowContainerHeightService")],t.prototype,"rowContainerHeightService",void 0),Cn([De("ctrlsService")],t.prototype,"ctrlsService",void 0),Cn([B],t.prototype,"postConstruct",null),t=Cn([Q("rowRenderer")],t),t}(H),e2=function(){function i(t){this.entriesMap={},this.entriesList=[],this.maxCount=t}return i.prototype.addRow=function(t){if(this.entriesMap[t.getRowNode().id]=t,this.entriesList.push(t),t.setCached(!0),this.entriesList.length>this.maxCount){var e=this.entriesList[0];e.destroyFirstPass(),e.destroySecondPass(),this.removeFromCache(e)}},i.prototype.getRow=function(t){if(t==null||t.id==null)return null;var e=this.entriesMap[t.id];if(!e)return null;this.removeFromCache(e),e.setCached(!1);var r=e.getRowNode()!=t;return r?null:e},i.prototype.removeFromCache=function(t){var e=t.getRowNode().id;delete this.entriesMap[e],gr(this.entriesList,t)},i.prototype.getEntries=function(){return this.entriesList},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var t2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),D_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},aR=function(i){t2(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.formatValue=function(e,r,n,o,a){a===void 0&&(a=!0);var s=null,l,u=e.getColDef();if(o?l=o:a&&(l=r&&r.rowPinned&&u.pinnedRowValueFormatter?u.pinnedRowValueFormatter:u.valueFormatter),l){var c={value:n,node:r,data:r?r.data:null,colDef:u,column:e,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()};s=this.expressionService.evaluate(l,c)}else if(u.refData)return u.refData[n]||"";return s==null&&Array.isArray(n)&&(s=n.join(", ")),s},D_([y("expressionService")],t.prototype,"expressionService",void 0),t=D_([Q("valueFormatterService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var r2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),vu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},sR=function(i){r2(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.init=function(){this.setPinnedTopRowData(this.gridOptionsWrapper.getPinnedTopRowData()),this.setPinnedBottomRowData(this.gridOptionsWrapper.getPinnedBottomRowData())},t.prototype.isEmpty=function(e){var r=e===L.PINNED_TOP?this.pinnedTopRows:this.pinnedBottomRows;return pn(r)},t.prototype.isRowsToRender=function(e){return!this.isEmpty(e)},t.prototype.getRowAtPixel=function(e,r){var n=r===L.PINNED_TOP?this.pinnedTopRows:this.pinnedBottomRows;if(pn(n))return 0;for(var o=0;o<n.length;o++){var a=n[o],s=a.rowTop+a.rowHeight-1;if(s>=e)return o}return n.length-1},t.prototype.setPinnedTopRowData=function(e){this.pinnedTopRows=this.createNodesFromData(e,!0);var r={type:x.EVENT_PINNED_ROW_DATA_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(r)},t.prototype.setPinnedBottomRowData=function(e){this.pinnedBottomRows=this.createNodesFromData(e,!1);var r={type:x.EVENT_PINNED_ROW_DATA_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(r)},t.prototype.createNodesFromData=function(e,r){var n=this,o=[];if(e){var a=0;e.forEach(function(s,l){var u=new Oe(n.beans);u.data=s;var c=r?Oe.ID_PREFIX_TOP_PINNED:Oe.ID_PREFIX_BOTTOM_PINNED;u.id=c+l,u.rowPinned=r?L.PINNED_TOP:L.PINNED_BOTTOM,u.setRowTop(a),u.setRowHeight(n.gridOptionsWrapper.getRowHeightForNode(u).height),u.setRowIndex(l),a+=u.rowHeight,o.push(u)})}return o},t.prototype.getPinnedTopRowData=function(){return this.pinnedTopRows},t.prototype.getPinnedBottomRowData=function(){return this.pinnedBottomRows},t.prototype.getPinnedTopTotalHeight=function(){return this.getTotalHeight(this.pinnedTopRows)},t.prototype.getPinnedTopRowCount=function(){return this.pinnedTopRows?this.pinnedTopRows.length:0},t.prototype.getPinnedBottomRowCount=function(){return this.pinnedBottomRows?this.pinnedBottomRows.length:0},t.prototype.getPinnedTopRow=function(e){return this.pinnedTopRows[e]},t.prototype.getPinnedBottomRow=function(e){return this.pinnedBottomRows[e]},t.prototype.forEachPinnedTopRow=function(e){pn(this.pinnedTopRows)||this.pinnedTopRows.forEach(e)},t.prototype.forEachPinnedBottomRow=function(e){pn(this.pinnedBottomRows)||this.pinnedBottomRows.forEach(e)},t.prototype.getPinnedBottomTotalHeight=function(){return this.getTotalHeight(this.pinnedBottomRows)},t.prototype.getTotalHeight=function(e){if(!e||e.length===0)return 0;var r=rt(e);return r.rowTop+r.rowHeight},vu([y("columnApi")],t.prototype,"columnApi",void 0),vu([y("gridApi")],t.prototype,"gridApi",void 0),vu([y("beans")],t.prototype,"beans",void 0),vu([B],t.prototype,"init",null),t=vu([Q("pinnedRowModel")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var hn;(function(i){i.Applied="Applied",i.StoreNotFound="StoreNotFound",i.StoreLoading="StoreLoading",i.StoreWaitingToLoad="StoreWaitingToLoad",i.StoreLoadingFailed="StoreLoadingFailed",i.StoreWrongType="StoreWrongType",i.Cancelled="Cancelled"})(hn||(hn={}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Jc=function(){function i(t,e){this.active=!0,this.nodeIdsToColumns={},this.mapToItems={},this.keepingColumns=t,this.pathRoot={rowNode:e,children:null},this.mapToItems[e.id]=this.pathRoot}return i.prototype.setInactive=function(){this.active=!1},i.prototype.isActive=function(){return this.active},i.prototype.depthFirstSearchChangedPath=function(t,e){if(t.children)for(var r=0;r<t.children.length;r++)this.depthFirstSearchChangedPath(t.children[r],e);e(t.rowNode)},i.prototype.depthFirstSearchEverything=function(t,e,r){if(t.childrenAfterGroup)for(var n=0;n<t.childrenAfterGroup.length;n++){var o=t.childrenAfterGroup[n];o.childrenAfterGroup?this.depthFirstSearchEverything(t.childrenAfterGroup[n],e,r):r&&e(o)}e(t)},i.prototype.forEachChangedNodeDepthFirst=function(t,e){e===void 0&&(e=!1),this.active?this.depthFirstSearchChangedPath(this.pathRoot,t):this.depthFirstSearchEverything(this.pathRoot.rowNode,t,e)},i.prototype.executeFromRootNode=function(t){t(this.pathRoot.rowNode)},i.prototype.createPathItems=function(t){for(var e=t,r=0;!this.mapToItems[e.id];){var n={rowNode:e,children:null};this.mapToItems[e.id]=n,r++,e=e.parent}return r},i.prototype.populateColumnsMap=function(t,e){var r=this;if(!(!this.keepingColumns||!e))for(var n=t;n;)this.nodeIdsToColumns[n.id]||(this.nodeIdsToColumns[n.id]={}),e.forEach(function(o){return r.nodeIdsToColumns[n.id][o.getId()]=!0}),n=n.parent},i.prototype.linkPathItems=function(t,e){for(var r=t,n=0;n<e;n++){var o=this.mapToItems[r.id],a=this.mapToItems[r.parent.id];a.children||(a.children=[]),a.children.push(o),r=r.parent}},i.prototype.addParentNode=function(t,e){if(!(!t||t.isRowPinned())){var r=this.createPathItems(t);this.linkPathItems(t,r),this.populateColumnsMap(t,e)}},i.prototype.canSkip=function(t){return this.active&&!this.mapToItems[t.id]},i.prototype.getValueColumnsForNode=function(t,e){if(!this.keepingColumns)return e;var r=this.nodeIdsToColumns[t.id],n=e.filter(function(o){return r[o.getId()]});return n},i.prototype.getNotValueColumnsForNode=function(t,e){if(!this.keepingColumns)return null;var r=this.nodeIdsToColumns[t.id],n=e.filter(function(o){return!r[o.getId()]});return n},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var n2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),jr=function(i){n2(t,i);function t(e){var r=i.call(this)||this;return r.state=t.STATE_WAITING_TO_LOAD,r.version=0,r.id=e,r}return t.prototype.getId=function(){return this.id},t.prototype.load=function(){this.state=t.STATE_LOADING,this.loadFromDatasource()},t.prototype.getVersion=function(){return this.version},t.prototype.setStateWaitingToLoad=function(){this.version++,this.state=t.STATE_WAITING_TO_LOAD},t.prototype.getState=function(){return this.state},t.prototype.pageLoadFailed=function(e){var r=this.isRequestMostRecentAndLive(e);r&&(this.state=t.STATE_FAILED,this.processServerFail()),this.dispatchLoadCompleted(!1)},t.prototype.success=function(e,r){this.successCommon(e,r)},t.prototype.pageLoaded=function(e,r,n){this.successCommon(e,{rowData:r,rowCount:n})},t.prototype.isRequestMostRecentAndLive=function(e){var r=e===this.version,n=this.isAlive();return r&&n},t.prototype.successCommon=function(e,r){this.dispatchLoadCompleted();var n=this.isRequestMostRecentAndLive(e);n&&(this.state=t.STATE_LOADED,this.processServerResult(r))},t.prototype.dispatchLoadCompleted=function(e){e===void 0&&(e=!0);var r={type:t.EVENT_LOAD_COMPLETE,success:e,block:this};this.dispatchEvent(r)},t.EVENT_LOAD_COMPLETE="loadComplete",t.STATE_WAITING_TO_LOAD="needsLoading",t.STATE_LOADING="loading",t.STATE_LOADED="loaded",t.STATE_FAILED="failed",t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var i2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ig=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},o2=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},lR=function(i){i2(t,i);function t(){var r=i!==null&&i.apply(this,arguments)||this;return r.activeBlockLoadsCount=0,r.blocks=[],r.active=!0,r}e=t,t.prototype.postConstruct=function(){this.maxConcurrentRequests=this.gridOptionsWrapper.getMaxConcurrentDatasourceRequests();var r=this.gridOptionsWrapper.getBlockLoadDebounceMillis();r&&r>0&&(this.checkBlockToLoadDebounce=_.debounce(this.performCheckBlocksToLoad.bind(this),r))},t.prototype.setBeans=function(r){this.logger=r.create("RowNodeBlockLoader")},t.prototype.addBlock=function(r){this.blocks.push(r),r.addEventListener(jr.EVENT_LOAD_COMPLETE,this.loadComplete.bind(this)),this.checkBlockToLoad()},t.prototype.removeBlock=function(r){_.removeFromArray(this.blocks,r)},t.prototype.destroy=function(){i.prototype.destroy.call(this),this.active=!1},t.prototype.loadComplete=function(){this.activeBlockLoadsCount--,this.checkBlockToLoad(),this.activeBlockLoadsCount==0&&this.dispatchEvent({type:e.BLOCK_LOADER_FINISHED_EVENT})},t.prototype.checkBlockToLoad=function(){this.checkBlockToLoadDebounce?this.checkBlockToLoadDebounce():this.performCheckBlocksToLoad()},t.prototype.performCheckBlocksToLoad=function(){if(!!this.active){if(this.printCacheStatus(),this.maxConcurrentRequests!=null&&this.activeBlockLoadsCount>=this.maxConcurrentRequests){this.logger.log("checkBlockToLoad: max loads exceeded");return}var r=null;this.blocks.forEach(function(n){n.getState()===jr.STATE_WAITING_TO_LOAD&&(r=n)}),r&&(r.load(),this.activeBlockLoadsCount++,this.printCacheStatus())}},t.prototype.getBlockState=function(){var r={};return this.blocks.forEach(function(n){var o=n.getBlockStateJson(),a=o.id,s=o.state;r[a]=s}),r},t.prototype.printCacheStatus=function(){this.logger.isLogging()&&this.logger.log("printCacheStatus: activePageLoadsCount = "+this.activeBlockLoadsCount+","+(" blocks = "+JSON.stringify(this.getBlockState())))},t.prototype.isLoading=function(){return this.activeBlockLoadsCount>0};var e;return t.BLOCK_LOADER_FINISHED_EVENT="blockLoaderFinished",Ig([B],t.prototype,"postConstruct",null),Ig([o2(0,Yt("loggerFactory"))],t.prototype,"setBeans",null),t=e=Ig([Q("rowNodeBlockLoader")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var a2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),mu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},uR=function(i){a2(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.currentPage=0,e.topDisplayedRowIndex=0,e.bottomDisplayedRowIndex=0,e.pixelOffset=0,e.masterRowCount=0,e}return t.prototype.postConstruct=function(){this.active=this.gridOptionsWrapper.isPagination(),this.paginateChildRows=this.gridOptionsWrapper.isPaginateChildRows(),this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,this.onModelUpdated.bind(this)),this.addManagedListener(this.gridOptionsWrapper,"paginationPageSize",this.onPaginationPageSizeChanged.bind(this)),this.onModelUpdated()},t.prototype.ensureRowHeightsValid=function(e,r,n,o){var a=this.rowModel.ensureRowHeightsValid(e,r,this.getPageFirstRow(),this.getPageLastRow());return a&&this.calculatePages(),a},t.prototype.onModelUpdated=function(e){this.calculatePages();var r={type:x.EVENT_PAGINATION_CHANGED,animate:e?e.animate:!1,newData:e?e.newData:!1,newPage:e?e.newPage:!1,keepRenderedRows:e?e.keepRenderedRows:!1,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(r)},t.prototype.onPaginationPageSizeChanged=function(){this.calculatePages();var e={type:x.EVENT_PAGINATION_CHANGED,animate:!1,newData:!1,newPage:!1,keepRenderedRows:!0,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(e)},t.prototype.goToPage=function(e){if(!(!this.active||this.currentPage===e)){this.currentPage=e;var r={type:x.EVENT_MODEL_UPDATED,animate:!1,keepRenderedRows:!1,newData:!1,newPage:!0,api:this.gridApi,columnApi:this.columnApi};this.onModelUpdated(r)}},t.prototype.getPixelOffset=function(){return this.pixelOffset},t.prototype.getRow=function(e){return this.rowModel.getRow(e)},t.prototype.getRowNode=function(e){return this.rowModel.getRowNode(e)},t.prototype.getRowIndexAtPixel=function(e){return this.rowModel.getRowIndexAtPixel(e)},t.prototype.getCurrentPageHeight=function(){return me(this.topRowBounds)||me(this.bottomRowBounds)?0:Math.max(this.bottomRowBounds.rowTop+this.bottomRowBounds.rowHeight-this.topRowBounds.rowTop,0)},t.prototype.getCurrentPagePixelRange=function(){var e=this.topRowBounds?this.topRowBounds.rowTop:0,r=this.bottomRowBounds?this.bottomRowBounds.rowTop+this.bottomRowBounds.rowHeight:0;return{pageFirstPixel:e,pageLastPixel:r}},t.prototype.isRowPresent=function(e){if(!this.rowModel.isRowPresent(e))return!1;var r=e.rowIndex>=this.topDisplayedRowIndex&&e.rowIndex<=this.bottomDisplayedRowIndex;return r},t.prototype.isEmpty=function(){return this.rowModel.isEmpty()},t.prototype.isRowsToRender=function(){return this.rowModel.isRowsToRender()},t.prototype.getNodesInRangeForSelection=function(e,r){return this.rowModel.getNodesInRangeForSelection(e,r)},t.prototype.forEachNode=function(e){return this.rowModel.forEachNode(e)},t.prototype.getType=function(){return this.rowModel.getType()},t.prototype.getRowBounds=function(e){var r=this.rowModel.getRowBounds(e);return r.rowIndex=e,r},t.prototype.getPageFirstRow=function(){return this.topRowBounds?this.topRowBounds.rowIndex:-1},t.prototype.getPageLastRow=function(){return this.bottomRowBounds?this.bottomRowBounds.rowIndex:-1},t.prototype.getRowCount=function(){return this.rowModel.getRowCount()},t.prototype.getPageForIndex=function(e){return Math.floor(e/this.pageSize)},t.prototype.goToPageWithIndex=function(e){if(!!this.active){var r=this.getPageForIndex(e);this.goToPage(r)}},t.prototype.isRowInPage=function(e){if(!this.active)return!0;var r=this.getPageForIndex(e.rowIndex);return r===this.currentPage},t.prototype.isLastPageFound=function(){return this.rowModel.isLastRowIndexKnown()},t.prototype.getCurrentPage=function(){return this.currentPage},t.prototype.goToNextPage=function(){this.goToPage(this.currentPage+1)},t.prototype.goToPreviousPage=function(){this.goToPage(this.currentPage-1)},t.prototype.goToFirstPage=function(){this.goToPage(0)},t.prototype.goToLastPage=function(){var e=this.rowModel.getRowCount(),r=Math.floor(e/this.pageSize);this.goToPage(r)},t.prototype.getPageSize=function(){return this.pageSize},t.prototype.getTotalPages=function(){return this.totalPages},t.prototype.setPageSize=function(){this.pageSize=this.gridOptionsWrapper.getPaginationPageSize(),(this.pageSize==null||this.pageSize<1)&&(this.pageSize=100)},t.prototype.calculatePages=function(){this.active?(this.setPageSize(),this.paginateChildRows?this.calculatePagesAllRows():this.calculatePagesMasterRowsOnly()):this.calculatedPagesNotActive(),this.topRowBounds=this.rowModel.getRowBounds(this.topDisplayedRowIndex),this.topRowBounds&&(this.topRowBounds.rowIndex=this.topDisplayedRowIndex),this.bottomRowBounds=this.rowModel.getRowBounds(this.bottomDisplayedRowIndex),this.bottomRowBounds&&(this.bottomRowBounds.rowIndex=this.bottomDisplayedRowIndex),this.setPixelOffset(q(this.topRowBounds)?this.topRowBounds.rowTop:0)},t.prototype.setPixelOffset=function(e){this.pixelOffset!==e&&(this.pixelOffset=e,this.eventService.dispatchEvent({type:x.EVENT_PAGINATION_PIXEL_OFFSET_CHANGED}))},t.prototype.setZeroRows=function(){this.masterRowCount=0,this.topDisplayedRowIndex=0,this.bottomDisplayedRowIndex=-1,this.currentPage=0,this.totalPages=0},t.prototype.calculatePagesMasterRowsOnly=function(){if(this.masterRowCount=this.rowModel.getTopLevelRowCount(),this.masterRowCount<=0){this.setZeroRows();return}var e=this.masterRowCount-1;this.totalPages=Math.floor(e/this.pageSize)+1,this.currentPage>=this.totalPages&&(this.currentPage=this.totalPages-1),(!Xh(this.currentPage)||this.currentPage<0)&&(this.currentPage=0);var r=this.pageSize*this.currentPage,n=this.pageSize*(this.currentPage+1)-1;if(n>e&&(n=e),this.topDisplayedRowIndex=this.rowModel.getTopLevelRowDisplayedIndex(r),n===e)this.bottomDisplayedRowIndex=this.rowModel.getRowCount()-1;else{var o=this.rowModel.getTopLevelRowDisplayedIndex(n+1);this.bottomDisplayedRowIndex=o-1}},t.prototype.getMasterRowCount=function(){return this.masterRowCount},t.prototype.calculatePagesAllRows=function(){if(this.masterRowCount=this.rowModel.getRowCount(),this.masterRowCount===0){this.setZeroRows();return}var e=this.masterRowCount-1;this.totalPages=Math.floor(e/this.pageSize)+1,this.currentPage>=this.totalPages&&(this.currentPage=this.totalPages-1),(!Xh(this.currentPage)||this.currentPage<0)&&(this.currentPage=0),this.topDisplayedRowIndex=this.pageSize*this.currentPage,this.bottomDisplayedRowIndex=this.pageSize*(this.currentPage+1)-1,this.bottomDisplayedRowIndex>e&&(this.bottomDisplayedRowIndex=e)},t.prototype.calculatedPagesNotActive=function(){this.pageSize=this.rowModel.getRowCount(),this.totalPages=1,this.currentPage=0,this.topDisplayedRowIndex=0,this.bottomDisplayedRowIndex=this.rowModel.getRowCount()-1},mu([y("rowModel")],t.prototype,"rowModel",void 0),mu([y("columnApi")],t.prototype,"columnApi",void 0),mu([y("gridApi")],t.prototype,"gridApi",void 0),mu([B],t.prototype,"postConstruct",null),t=mu([Q("paginationProxy")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var s2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),M_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},cR=function(i){s2(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.processAllCellClasses=function(e,r,n,o){this.processClassRules(e.cellClassRules,r,n,o),this.processStaticCellClasses(e,r,n)},t.prototype.processClassRules=function(e,r,n,o){if(e!=null){for(var a=Object.keys(e),s={},l={},u=function(h){var d=a[h],f=e[d],g;typeof f=="string"?g=c.expressionService.evaluate(f,r):typeof f=="function"&&(g=f(r)),d.split(" ").forEach(function(v){v==null||v.trim()==""||(g?s[v]=!0:l[v]=!0)})},c=this,p=0;p<a.length;p++)u(p);o&&Object.keys(l).forEach(o),Object.keys(s).forEach(n)}},t.prototype.getStaticCellClasses=function(e,r){var n=e.cellClass;if(!n)return[];var o;if(typeof n=="function"){var a=n;o=a(r)}else o=n;return typeof o=="string"&&(o=[o]),o||[]},t.prototype.processStaticCellClasses=function(e,r,n){var o=this.getStaticCellClasses(e,r);o.forEach(function(a){n(a)})},M_([y("expressionService")],t.prototype,"expressionService",void 0),t=M_([Q("stylingService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var l2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),nw=function(i){l2(t,i);function t(e){return i.call(this,e,"ag-radio-button","radio")||this}return t.prototype.isSelected=function(){return this.eInput.checked},t.prototype.toggle=function(){this.eInput.disabled||this.isSelected()||this.setValue(!0)},t.prototype.addInputListeners=function(){i.prototype.addInputListeners.call(this),this.addManagedListener(this.eventService,x.EVENT_CHECKBOX_CHANGED,this.onChange.bind(this))},t.prototype.onChange=function(e){e.selected&&e.name&&this.eInput.name&&this.eInput.name===e.name&&e.id&&this.eInput.id!==e.id&&this.setValue(!1,!0)},t}(ii);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var u2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),sf=function(i){u2(t,i);function t(e){return i.call(this,e,"ag-toggle-button")||this}return t.prototype.setValue=function(e,r){return i.prototype.setValue.call(this,e,r),this.addOrRemoveCssClass("ag-selected",this.getValue()),this},t}(ii);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var c2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),iw=function(i){c2(t,i);function t(e,r,n){return r===void 0&&(r="ag-text-field"),n===void 0&&(n="text"),i.call(this,e,r,n)||this}return t.prototype.postConstruct=function(){i.prototype.postConstruct.call(this),this.config.allowedCharPattern&&this.preventDisallowedCharacters()},t.prototype.setValue=function(e,r){var n=i.prototype.setValue.call(this,e,r);return this.eInput.value!==e&&(this.eInput.value=q(e)?e:""),n},t.prototype.preventDisallowedCharacters=function(){var e=new RegExp("["+this.config.allowedCharPattern+"]"),r=function(n){n.key&&!e.test(n.key)&&n.preventDefault()};this.addManagedListener(this.eInput,"keypress",r),this.addManagedListener(this.eInput,"paste",function(n){var o,a=(o=n.clipboardData)===null||o===void 0?void 0:o.getData("text");a&&a.split("").some(function(s){return!e.test(s)})&&n.preventDefault()})},t}(Ki);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var p2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),pR=function(i){p2(t,i);function t(e){return i.call(this,e,"ag-text-area",null,"textarea")||this}return t.prototype.setValue=function(e,r){var n=i.prototype.setValue.call(this,e,r);return this.eInput.value=e,n},t.prototype.setCols=function(e){return this.eInput.cols=e,this},t.prototype.setRows=function(e){return this.eInput.rows=e,this},t}(Ki);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var h2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),hR=function(i){h2(t,i);function t(e){return i.call(this,e,"ag-number-field","number")||this}return t.prototype.postConstruct=function(){var e=this;i.prototype.postConstruct.call(this),this.addManagedListener(this.eInput,"blur",function(){var r=parseFloat(e.eInput.value),n=isNaN(r)?"":e.normalizeValue(r.toString());e.value!==n&&e.setValue(n)}),this.eInput.step="any"},t.prototype.normalizeValue=function(e){if(e==="")return"";this.precision&&(e=this.adjustPrecision(e));var r=parseFloat(e);return this.min!=null&&r<this.min?e=this.min.toString():this.max!=null&&r>this.max&&(e=this.max.toString()),e},t.prototype.adjustPrecision=function(e){if(this.precision){var r=parseFloat(e).toFixed(this.precision);e=parseFloat(r).toString()}return e},t.prototype.setMin=function(e){return this.min===e?this:(this.min=e,lc(this.eInput,"min",e),this)},t.prototype.setMax=function(e){return this.max===e?this:(this.max=e,lc(this.eInput,"max",e),this)},t.prototype.setPrecision=function(e){return this.precision=e,this},t.prototype.setStep=function(e){return this.step===e?this:(this.step=e,lc(this.eInput,"step",e),this)},t.prototype.setValue=function(e,r){e=this.adjustPrecision(e);var n=this.normalizeValue(e);return e!=n?this:i.prototype.setValue.call(this,e,r)},t}(iw);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var d2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),dR=function(i){d2(t,i);function t(e){return i.call(this,e,"ag-range-field","range")||this}return t.prototype.postConstruct=function(){i.prototype.postConstruct.call(this);var e=this.config,r=e.min,n=e.max,o=e.step;r!=null&&this.setMinValue(r),n!=null&&this.setMaxValue(n),this.setStep(o||1)},t.prototype.addInputListeners=function(){var e=this;this.addManagedListener(this.eInput,"input",function(r){var n=r.target.value;e.setValue(n)})},t.prototype.setMinValue=function(e){return this.min=e,this.eInput.setAttribute("min",e.toString()),this},t.prototype.setMaxValue=function(e){return this.max=e,this.eInput.setAttribute("max",e.toString()),this},t.prototype.setStep=function(e){return this.eInput.setAttribute("step",e.toString()),this},t.prototype.setValue=function(e,r){this.min!=null&&(e=Math.max(parseFloat(e),this.min).toString()),this.max!=null&&(e=Math.min(parseFloat(e),this.max).toString());var n=i.prototype.setValue.call(this,e,r);return this.eInput.value=e,n},t}(Ki);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var f2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Sp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},zi=function(i){f2(t,i);function t(e){var r=i.call(this,e,t.TEMPLATE)||this;return r.labelAlignment="top",r}return t.prototype.init=function(){this.eSlider.addCssClass("ag-slider-field")},t.prototype.onValueChange=function(e){var r=this,n=ei.EVENT_CHANGED;return this.addManagedListener(this.eText,n,function(){var o=parseFloat(r.eText.getValue());r.eSlider.setValue(o.toString(),!0),e(o||0)}),this.addManagedListener(this.eSlider,n,function(){var o=r.eSlider.getValue();r.eText.setValue(o,!0),e(parseFloat(o))}),this},t.prototype.setSliderWidth=function(e){return this.eSlider.setWidth(e),this},t.prototype.setTextFieldWidth=function(e){return this.eText.setWidth(e),this},t.prototype.setMinValue=function(e){return this.eSlider.setMinValue(e),this.eText.setMin(e),this},t.prototype.setMaxValue=function(e){return this.eSlider.setMaxValue(e),this.eText.setMax(e),this},t.prototype.getValue=function(){return this.eText.getValue()},t.prototype.setValue=function(e){return this.getValue()===e?this:(this.eText.setValue(e,!0),this.eSlider.setValue(e,!0),this.dispatchEvent({type:ei.EVENT_CHANGED}),this)},t.prototype.setStep=function(e){return this.eSlider.setStep(e),this.eText.setStep(e),this},t.TEMPLATE=`<div class="ag-slider">
            <label ref="eLabel"></label>
            <div class="ag-wrapper ag-slider-wrapper">
                <ag-input-range ref="eSlider"></ag-input-range>
                <ag-input-number-field ref="eText"></ag-input-number-field>
            </div>
        </div>`,Sp([I("eLabel")],t.prototype,"eLabel",void 0),Sp([I("eSlider")],t.prototype,"eSlider",void 0),Sp([I("eText")],t.prototype,"eText",void 0),Sp([B],t.prototype,"init",null),t}(zC);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var g2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),yu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ow=function(i){g2(t,i);function t(e){var r=i.call(this,e,t.TEMPLATE)||this;return r.radius=0,r.offsetX=0,r.offsetY=0,r}return t.prototype.postConstruct=function(){var e=this;i.prototype.postConstruct.call(this),this.dragListener={eElement:this.eParentCircle,dragStartPixels:0,onDragStart:function(r){e.parentCircleRect=e.eParentCircle.getBoundingClientRect()},onDragging:function(r){return e.calculateAngleDrag(r)},onDragStop:function(){}},this.dragService.addDragSource(this.dragListener),this.eAngleValue.setLabel("").setLabelWidth(5).setInputWidth(45).setMin(0).setMax(360).setValue(""+this.degrees).onValueChange(function(r){(r==null||r==="")&&(r="0"),r=e.eAngleValue.normalizeValue(r);var n=parseFloat(r);n>180&&(n=n-360),e.setValue(n)}),this.updateNumberInput(),q(this.getValue())&&this.eAngleValue.setValue(this.normalizeNegativeValue(this.getValue()).toString()),this.addManagedListener(this,ei.EVENT_CHANGED,function(){var r=e.gridOptionsWrapper.getDocument();e.eAngleValue.getInputElement().contains(r.activeElement)||e.updateNumberInput()})},t.prototype.updateNumberInput=function(){var e=this.normalizeNegativeValue(this.getValue());this.eAngleValue.setValue(e.toString())},t.prototype.positionChildCircle=function(e){var r=this.parentCircleRect||{width:24,height:24},n=this.eChildCircle,o=r.width/2,a=r.height/2;n.style.left=o+Math.cos(e)*8+"px",n.style.top=a+Math.sin(e)*8+"px"},t.prototype.calculatePolar=function(){var e=this.offsetX,r=this.offsetY,n=Math.atan2(r,e);this.degrees=this.toDegrees(n),this.radius=Math.sqrt(e*e+r*r),this.positionChildCircle(n)},t.prototype.calculateCartesian=function(){var e=this.toRadians(this.getValue()),r=this.getRadius();this.setOffsetX(Math.cos(e)*r).setOffsetY(Math.sin(e)*r)},t.prototype.setOffsetX=function(e){return this.offsetX!==e&&(this.offsetX=e,this.calculatePolar()),this},t.prototype.setOffsetY=function(e){return this.offsetY!==e&&(this.offsetY=e,this.calculatePolar()),this},t.prototype.calculateAngleDrag=function(e){var r=this.parentCircleRect,n=r.width/2,o=r.height/2,a=e.clientX-r.left,s=e.clientY-r.top,l=a-n,u=s-o,c=Math.atan2(u,l);this.setValue(c,!0)},t.prototype.toDegrees=function(e){return e/Math.PI*180},t.prototype.toRadians=function(e){return e/180*Math.PI},t.prototype.normalizeNegativeValue=function(e){return e<0?360+e:e},t.prototype.normalizeAngle180=function(e){return e%=Math.PI*2,e<-Math.PI?e+=Math.PI*2:e>=Math.PI&&(e-=Math.PI*2),e},t.prototype.getRadius=function(){return this.radius},t.prototype.setRadius=function(e){return this.radius===e?this:(this.radius=e,this.calculateCartesian(),this)},t.prototype.onValueChange=function(e){var r=this;return this.addManagedListener(this,ei.EVENT_CHANGED,function(){e(r.degrees)}),this},t.prototype.getValue=function(e){return e?this.toRadians(this.degrees):this.degrees},t.prototype.setValue=function(e,r){var n;return r?n=e:n=this.normalizeAngle180(this.toRadians(e)),e=this.toDegrees(n),this.degrees!==e&&(this.degrees=Math.floor(e),this.calculateCartesian(),this.positionChildCircle(n),this.dispatchEvent({type:ei.EVENT_CHANGED})),this},t.prototype.setWidth=function(e){return Li(this.getGui(),e),this},t.prototype.destroy=function(){this.dragService.removeDragSource(this.dragListener),i.prototype.destroy.call(this)},t.TEMPLATE=`<div class="ag-angle-select">
            <div ref="eLabel"></div>
            <div class="ag-wrapper ag-angle-select-wrapper">
                <div ref="eAngleSelectField" class="ag-angle-select-field">
                    <div ref="eParentCircle" class="ag-angle-select-parent-circle">
                        <div ref="eChildCircle" class="ag-angle-select-child-circle"></div>
                    </div>
                </div>
                <ag-input-number-field ref="eAngleValue"></ag-input-number-field>
            </div>
        </div>`,yu([I("eLabel")],t.prototype,"eLabel",void 0),yu([I("eParentCircle")],t.prototype,"eParentCircle",void 0),yu([I("eChildCircle")],t.prototype,"eChildCircle",void 0),yu([I("eAngleValue")],t.prototype,"eAngleValue",void 0),yu([y("dragService")],t.prototype,"dragService",void 0),t}(zC);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var v2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ki=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},m2=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},y2=function(i){v2(t,i);function t(e){var r=i.call(this,t.TEMPLATE)||this;return r.H=1,r.S=1,r.B=1,r.A=1,r.isSpectrumDragging=!1,r.isSpectrumHueDragging=!1,r.isSpectrumAlphaDragging=!1,r.colorChanged=!1,r.picker=e.picker,r}return t.prototype.postConstruct=function(){var e=this.getGui();this.initRecentColors(),this.addManagedListener(this.spectrumVal,"mousedown",this.onSpectrumDraggerDown.bind(this)),this.addManagedListener(e,"mousemove",this.onSpectrumDraggerMove.bind(this)),this.addManagedListener(this.spectrumHue,"mousedown",this.onSpectrumHueDown.bind(this)),this.addManagedListener(e,"mousemove",this.onSpectrumHueMove.bind(this)),this.addManagedListener(this.spectrumAlpha,"mousedown",this.onSpectrumAlphaDown.bind(this)),this.addManagedListener(e,"mousemove",this.onSpectrumAlphaMove.bind(this)),this.addManagedListener(document,"mouseup",this.onMouseUp.bind(this)),this.addManagedListener(this.recentColors,"click",this.onRecentColorClick.bind(this))},t.prototype.refreshSpectrumRect=function(){return this.spectrumValRect=this.spectrumVal.getBoundingClientRect()},t.prototype.refreshHueRect=function(){return this.spectrumHueRect=this.spectrumHue.getBoundingClientRect()},t.prototype.refreshAlphaRect=function(){return this.spectrumAlphaRect=this.spectrumAlpha.getBoundingClientRect()},t.prototype.onSpectrumDraggerDown=function(e){this.refreshSpectrumRect(),this.isSpectrumDragging=!0,this.moveDragger(e)},t.prototype.onSpectrumDraggerMove=function(e){this.isSpectrumDragging&&this.moveDragger(e)},t.prototype.onSpectrumHueDown=function(e){this.refreshHueRect(),this.isSpectrumHueDragging=!0,this.moveHueSlider(e)},t.prototype.onSpectrumHueMove=function(e){this.isSpectrumHueDragging&&this.moveHueSlider(e)},t.prototype.onSpectrumAlphaDown=function(e){this.refreshAlphaRect(),this.isSpectrumAlphaDragging=!0,this.moveAlphaSlider(e)},t.prototype.onSpectrumAlphaMove=function(e){this.isSpectrumAlphaDragging&&this.moveAlphaSlider(e)},t.prototype.onMouseUp=function(){this.isSpectrumDragging=!1,this.isSpectrumHueDragging=!1,this.isSpectrumAlphaDragging=!1},t.prototype.moveDragger=function(e){var r=this.spectrumValRect;if(r){var n=e.clientX-r.left,o=e.clientY-r.top;n=Math.max(n,0),n=Math.min(n,r.width),o=Math.max(o,0),o=Math.min(o,r.height),this.setSpectrumValue(n/r.width,1-o/r.height)}},t.prototype.moveHueSlider=function(e){var r=this.spectrumHueRect;if(r){var n=this.spectrumHueSlider,o=n.getBoundingClientRect(),a=e.clientX-r.left;a=Math.max(a,0),a=Math.min(a,r.width),this.H=1-a/r.width,n.style.left=a+o.width/2+"px",this.update()}},t.prototype.moveAlphaSlider=function(e){var r=this.spectrumAlphaRect;if(r){var n=this.spectrumAlphaSlider,o=n.getBoundingClientRect(),a=e.clientX-r.left;a=Math.max(a,0),a=Math.min(a,r.width),this.A=a/r.width,n.style.left=a+o.width/2+"px",this.update()}},t.prototype.update=function(){var e=Bo.fromHSB(this.H*360,this.S,this.B,this.A),r=Bo.fromHSB(this.H*360,1,1),n=e.toRgbaString(),o=this.picker,a=Bo.fromString(o.getValue());a.toRgbaString()!==n&&(this.colorChanged=!0),o.setValue(n),this.spectrumColor.style.backgroundColor=r.toRgbaString(),this.spectrumDragger.style.backgroundColor=n},t.prototype.setSpectrumValue=function(e,r){var n=this.spectrumValRect||this.refreshSpectrumRect();if(n){var o=this.spectrumDragger,a=o.getBoundingClientRect();e=Math.max(0,e),e=Math.min(1,e),r=Math.max(0,r),r=Math.min(1,r),this.S=e,this.B=r,o.style.left=e*n.width-a.width/2+"px",o.style.top=(1-r)*n.height-a.height/2+"px",this.update()}},t.prototype.initRecentColors=function(){var e=t.recentColors,r=e.map(function(n,o){return'<div class="ag-recent-color" id='+o+' style="background-color: '+n+'; width: 15px; height: 15px;" recent-color="'+n+'"></div>'});this.recentColors.innerHTML=r.join("")},t.prototype.setValue=function(e){var r=Bo.fromString(e),n=m2(r.toHSB(),3),o=n[0],a=n[1],s=n[2];this.H=(isNaN(o)?0:o)/360,this.A=r.a;var l=this.spectrumHueRect||this.refreshHueRect(),u=this.spectrumAlphaRect||this.refreshAlphaRect();this.spectrumHueSlider.style.left=(this.H-1)*-l.width+"px",this.spectrumAlphaSlider.style.left=this.A*u.width+"px",this.setSpectrumValue(a,s)},t.prototype.onRecentColorClick=function(e){var r=e.target;if(!!q(r.id)){var n=parseInt(r.id,10);this.setValue(t.recentColors[n]),this.destroy()}},t.prototype.addRecentColor=function(){var e=Bo.fromHSB(this.H*360,this.S,this.B,this.A),r=e.toRgbaString(),n=t.recentColors;!this.colorChanged||n[0]===r||(n=n.filter(function(o){return o!=r}),n=[r].concat(n),n.length>t.maxRecentColors&&(n=n.slice(0,t.maxRecentColors)),t.recentColors=n)},t.prototype.destroy=function(){this.addRecentColor(),i.prototype.destroy.call(this)},t.maxRecentColors=8,t.recentColors=[],t.TEMPLATE=`<div class="ag-color-panel">
            <div ref="spectrumColor" class="ag-spectrum-color">
                <div class="ag-spectrum-sat ag-spectrum-fill">
                    <div ref="spectrumVal" class="ag-spectrum-val ag-spectrum-fill">
                        <div ref="spectrumDragger" class="ag-spectrum-dragger"></div>
                    </div>
                </div>
            </div>
            <div class="ag-spectrum-tools">
                <div ref="spectrumHue" class="ag-spectrum-hue ag-spectrum-tool">
                    <div class="ag-spectrum-hue-background"></div>
                    <div ref="spectrumHueSlider" class="ag-spectrum-slider"></div>
                </div>
                <div ref="spectrumAlpha" class="ag-spectrum-alpha ag-spectrum-tool">
                    <div class="ag-spectrum-alpha-background"></div>
                    <div ref="spectrumAlphaSlider" class="ag-spectrum-slider"></div>
                </div>
                <div ref="recentColors" class="ag-recent-colors"></div>
            </div>
        </div>`,ki([I("spectrumColor")],t.prototype,"spectrumColor",void 0),ki([I("spectrumVal")],t.prototype,"spectrumVal",void 0),ki([I("spectrumDragger")],t.prototype,"spectrumDragger",void 0),ki([I("spectrumHue")],t.prototype,"spectrumHue",void 0),ki([I("spectrumHueSlider")],t.prototype,"spectrumHueSlider",void 0),ki([I("spectrumAlpha")],t.prototype,"spectrumAlpha",void 0),ki([I("spectrumAlphaSlider")],t.prototype,"spectrumAlphaSlider",void 0),ki([I("recentColors")],t.prototype,"recentColors",void 0),ki([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var C2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Cu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},lf=function(i){C2(t,i);function t(e){var r=i.call(this,t.getTemplate(e))||this;return r.closable=!0,r.config=e,r}return t.getTemplate=function(e){var r=e&&e.cssIdentifier||"default";return'<div class="ag-panel ag-'+r+`-panel" tabindex="-1">
            <div ref="eTitleBar" class="ag-panel-title-bar ag-`+r+`-panel-title-bar ag-unselectable">
                <span ref="eTitle" class="ag-panel-title-bar-title ag-`+r+`-panel-title-bar-title"></span>
                <div ref="eTitleBarButtons" class="ag-panel-title-bar-buttons ag-`+r+`-panel-title-bar-buttons"></div>
            </div>
            <div ref="eContentWrapper" class="ag-panel-content-wrapper ag-`+r+`-panel-content-wrapper"></div>
        </div>`},t.prototype.postConstruct=function(){var e=this,r=this.config,n=r.component,o=r.closable,a=r.hideTitleBar,s=r.title,l=r.minWidth,u=l===void 0?250:l,c=r.width,p=r.minHeight,h=p===void 0?250:p,d=r.height,f=r.centered,g=r.popup,v=r.x,m=r.y;this.positionableFeature=new Qc(this.getGui(),{minWidth:u,width:c,minHeight:h,height:d,centered:f,x:v,y:m,popup:g,calculateTopBuffer:function(){return e.positionableFeature.getHeight()-e.getBodyHeight()}}),this.createManagedBean(this.positionableFeature);var C=this.getGui();n&&this.setBodyComponent(n),a?this.eTitleBar.classList.add("ag-hidden"):(s&&this.setTitle(s),this.setClosable(o!=null?o:this.closable)),this.addManagedListener(this.eTitleBar,"mousedown",function(w){var b=e.gridOptionsWrapper.getDocument();if(C.contains(w.relatedTarget)||C.contains(b.activeElement)||e.eTitleBarButtons.contains(w.target)){w.preventDefault();return}var S=e.eContentWrapper.querySelector("button, [href], input, select, textarea, [tabindex]");S&&S.focus()}),!(g&&this.positionableFeature.isPositioned())&&(this.renderComponent&&this.renderComponent(),this.positionableFeature.initialisePosition(),this.eContentWrapper.style.height="0")},t.prototype.renderComponent=function(){var e=this,r=this.getGui();r.focus(),this.close=function(){r.parentElement.removeChild(r),e.destroy()}},t.prototype.getHeight=function(){return this.positionableFeature.getHeight()},t.prototype.setHeight=function(e){this.positionableFeature.setHeight(e)},t.prototype.getWidth=function(){return this.positionableFeature.getWidth()},t.prototype.setWidth=function(e){this.positionableFeature.setWidth(e)},t.prototype.setClosable=function(e){if(e!==this.closable&&(this.closable=e),e){var r=this.closeButtonComp=new ee(t.CLOSE_BTN_TEMPLATE);this.getContext().createBean(r);var n=r.getGui(),o=Ft("close",this.gridOptionsWrapper);o.classList.add("ag-panel-title-bar-button-icon"),n.appendChild(o),this.addTitleBarButton(r),r.addManagedListener(n,"click",this.onBtClose.bind(this))}else if(this.closeButtonComp){var n=this.closeButtonComp.getGui();n.parentElement.removeChild(n),this.closeButtonComp=this.destroyBean(this.closeButtonComp)}},t.prototype.setBodyComponent=function(e){e.setParentComponent(this),this.eContentWrapper.appendChild(e.getGui())},t.prototype.addTitleBarButton=function(e,r){var n=this.eTitleBarButtons,o=n.children,a=o.length;r==null&&(r=a),r=Math.max(0,Math.min(r,a)),e.addCssClass("ag-panel-title-bar-button");var s=e.getGui();r===0?n.insertAdjacentElement("afterbegin",s):r===a?n.insertAdjacentElement("beforeend",s):o[r-1].insertAdjacentElement("afterend",s),e.setParentComponent(this)},t.prototype.getBodyHeight=function(){return Uc(this.eContentWrapper)},t.prototype.getBodyWidth=function(){return tf(this.eContentWrapper)},t.prototype.setTitle=function(e){this.eTitle.innerText=e},t.prototype.onBtClose=function(){this.close()},t.prototype.destroy=function(){this.closeButtonComp&&(this.closeButtonComp=this.destroyBean(this.closeButtonComp));var e=this.getGui();e&&e.offsetParent&&this.close(),i.prototype.destroy.call(this)},t.CLOSE_BTN_TEMPLATE='<div class="ag-button"></div>',Cu([I("eContentWrapper")],t.prototype,"eContentWrapper",void 0),Cu([I("eTitleBar")],t.prototype,"eTitleBar",void 0),Cu([I("eTitleBarButtons")],t.prototype,"eTitleBarButtons",void 0),Cu([I("eTitle")],t.prototype,"eTitle",void 0),Cu([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var w2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ad=globalThis&&globalThis.__assign||function(){return ad=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},ad.apply(this,arguments)},b2=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Oc=function(i){w2(t,i);function t(e){var r=i.call(this,ad(ad({},e),{popup:!0}))||this;return r.isMaximizable=!1,r.isMaximized=!1,r.maximizeListeners=[],r.resizeListenerDestroy=null,r.lastPosition={x:0,y:0,width:0,height:0},r}return t.prototype.postConstruct=function(){var e=this,r=this.getGui(),n=this.config,o=n.movable,a=n.resizable,s=n.maximizable;this.addCssClass("ag-dialog"),i.prototype.postConstruct.call(this),this.addManagedListener(r,"focusin",function(l){r.contains(l.relatedTarget)||e.popupService.bringPopupToFront(r)}),o&&this.setMovable(o),s&&this.setMaximizable(s),a&&this.setResizable(a)},t.prototype.renderComponent=function(){var e=this.getGui(),r=this.config,n=r.alwaysOnTop,o=r.modal,a=r.title,s=this.gridOptionsWrapper.getLocaleTextFunc(),l=this.popupService.addPopup({modal:o,eChild:e,closeOnEsc:!0,closedCallback:this.destroy.bind(this),alwaysOnTop:n,ariaLabel:a||s("ariaLabelDialog","Dialog")});l&&(this.close=l.hideFunc)},t.prototype.toggleMaximize=function(){var e=this.positionableFeature.getPosition();if(this.isMaximized){var r=this.lastPosition,n=r.x,o=r.y,a=r.width,s=r.height;this.setWidth(a),this.setHeight(s),this.positionableFeature.offsetElement(n,o)}else this.lastPosition.width=this.getWidth(),this.lastPosition.height=this.getHeight(),this.lastPosition.x=e.x,this.lastPosition.y=e.y,this.positionableFeature.offsetElement(0,0),this.setHeight("100%"),this.setWidth("100%");this.isMaximized=!this.isMaximized,this.refreshMaximizeIcon()},t.prototype.refreshMaximizeIcon=function(){pt(this.maximizeIcon,!this.isMaximized),pt(this.minimizeIcon,this.isMaximized)},t.prototype.clearMaximizebleListeners=function(){this.maximizeListeners.length&&(this.maximizeListeners.forEach(function(e){return e()}),this.maximizeListeners.length=0),this.resizeListenerDestroy&&(this.resizeListenerDestroy(),this.resizeListenerDestroy=null)},t.prototype.destroy=function(){this.maximizeButtonComp=this.destroyBean(this.maximizeButtonComp),this.clearMaximizebleListeners(),i.prototype.destroy.call(this)},t.prototype.setResizable=function(e){this.positionableFeature.setResizable(e)},t.prototype.setMovable=function(e){this.positionableFeature.setMovable(e,this.eTitleBar)},t.prototype.setMaximizable=function(e){var r=this;if(!e){this.clearMaximizebleListeners(),this.maximizeButtonComp&&(this.destroyBean(this.maximizeButtonComp),this.maximizeButtonComp=this.maximizeIcon=this.minimizeIcon=void 0);return}var n=this.eTitleBar;if(!(!n||e===this.isMaximizable)){var o=this.maximizeButtonComp=this.createBean(new ee('<div class="ag-dialog-button"></span>')),a=o.getGui();a.appendChild(this.maximizeIcon=Ft("maximize",this.gridOptionsWrapper)),this.maximizeIcon.classList.add("ag-panel-title-bar-button-icon"),a.appendChild(this.minimizeIcon=Ft("minimize",this.gridOptionsWrapper)),this.minimizeIcon.classList.add("ag-panel-title-bar-button-icon","ag-hidden"),o.addManagedListener(a,"click",this.toggleMaximize.bind(this)),this.addTitleBarButton(o,0),this.maximizeListeners.push(this.addManagedListener(n,"dblclick",this.toggleMaximize.bind(this))),this.resizeListenerDestroy=this.addManagedListener(this,"resize",function(){r.isMaximized=!1,r.refreshMaximizeIcon()})}},b2([y("popupService")],t.prototype,"popupService",void 0),t}(lf);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var _2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),fR=function(i){_2(t,i);function t(e){var r=i.call(this,e,"ag-color-picker","colorPicker")||this;return e&&e.color&&(r.value=e.color),r}return t.prototype.postConstruct=function(){i.prototype.postConstruct.call(this),this.value&&this.setValue(this.value)},t.prototype.showPicker=function(){var e=this,r=this.getGui().getBoundingClientRect(),n=this.createBean(new Oc({closable:!1,modal:!0,hideTitleBar:!0,minWidth:190,width:190,height:250,x:r.right-190,y:r.top-250}));this.isPickerDisplayed=!0,n.addCssClass("ag-color-dialog"),dn(this.eWrapper,!0);var o=this.createBean(new y2({picker:this}));return o.addDestroyFunc(function(){n.isAlive()&&e.destroyBean(n)}),n.setParentComponent(this),n.setBodyComponent(o),o.setValue(this.getValue()),n.addDestroyFunc(function(){e.isDestroyingPicker?e.isDestroyingPicker=!1:(e.isDestroyingPicker=!0,o.isAlive()&&e.destroyBean(o)),e.isAlive()&&(dn(e.eWrapper,!1),e.getFocusableElement().focus()),e.isPickerDisplayed=!1}),n},t.prototype.setValue=function(e){return this.value===e?this:(this.eDisplayField.style.backgroundColor=e,i.prototype.setValue.call(this,e))},t.prototype.getValue=function(){return this.value},t}(pE);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var S2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),xo=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Qn=function(i){S2(t,i);function t(e){e===void 0&&(e={});var r=i.call(this,t.getTemplate(e))||this;r.suppressEnabledCheckbox=!0,r.suppressOpenCloseIcons=!1;var n=e.title,o=e.enabled,a=e.items,s=e.suppressEnabledCheckbox,l=e.suppressOpenCloseIcons;return r.title=n,r.cssIdentifier=e.cssIdentifier||"default",r.enabled=o!=null?o:!0,r.items=a||[],r.alignItems=e.alignItems||"center",s!=null&&(r.suppressEnabledCheckbox=s),l!=null&&(r.suppressOpenCloseIcons=l),r}return t.getTemplate=function(e){var r=e.cssIdentifier||"default",n=e.direction||"vertical";return'<div class="ag-group ag-'+r+`-group" role="presentation">
            <div class="ag-group-title-bar ag-`+r+`-group-title-bar ag-unselectable" ref="eTitleBar" role="button">
                <span class="ag-group-title-bar-icon ag-`+r+`-group-title-bar-icon" ref="eGroupOpenedIcon" role="presentation"></span>
                <span class="ag-group-title-bar-icon ag-`+r+`-group-title-bar-icon" ref="eGroupClosedIcon" role="presentation"></span>
                <span ref="eTitle" class="ag-group-title ag-`+r+`-group-title"></span>
            </div>
            <div ref="eToolbar" class="ag-group-toolbar ag-`+r+`-group-toolbar">
                <ag-checkbox ref="cbGroupEnabled"></ag-checkbox>
            </div>
            <div ref="eContainer" class="ag-group-container ag-group-container-`+n+" ag-"+r+`-group-container"></div>
        </div>`},t.prototype.postConstruct=function(){if(this.items.length){var e=this.items;this.items=[],this.addItems(e)}var r=this.gridOptionsWrapper.getLocaleTextFunc();this.cbGroupEnabled.setLabel(r("enabled","Enabled")),this.title&&this.setTitle(this.title),this.enabled&&this.setEnabled(this.enabled),this.setAlignItems(this.alignItems),this.hideEnabledCheckbox(this.suppressEnabledCheckbox),this.hideOpenCloseIcons(this.suppressOpenCloseIcons),this.setupExpandContract(),this.refreshAriaStatus(),this.refreshChildDisplay()},t.prototype.setupExpandContract=function(){var e=this;this.eGroupClosedIcon.appendChild(Tn("columnSelectClosed",this.gridOptionsWrapper,null)),this.eGroupOpenedIcon.appendChild(Tn("columnSelectOpen",this.gridOptionsWrapper,null)),this.addManagedListener(this.eTitleBar,"click",function(){return e.toggleGroupExpand()}),this.addManagedListener(this.eTitleBar,"keydown",function(r){switch(r.key){case j.ENTER:case j.SPACE:r.preventDefault(),e.toggleGroupExpand();break;case j.RIGHT:case j.LEFT:r.preventDefault(),e.toggleGroupExpand(r.key===j.RIGHT);break}})},t.prototype.refreshAriaStatus=function(){this.suppressOpenCloseIcons||dn(this.eTitleBar,this.expanded)},t.prototype.refreshChildDisplay=function(){var e=!this.suppressOpenCloseIcons;pt(this.eToolbar,this.expanded&&!this.suppressEnabledCheckbox),pt(this.eGroupOpenedIcon,e&&this.expanded),pt(this.eGroupClosedIcon,e&&!this.expanded)},t.prototype.isExpanded=function(){return this.expanded},t.prototype.setAlignItems=function(e){this.alignItems!==e&&this.removeCssClass("ag-group-item-alignment-"+this.alignItems),this.alignItems=e;var r="ag-group-item-alignment-"+this.alignItems;return this.addCssClass(r),this},t.prototype.toggleGroupExpand=function(e){return this.suppressOpenCloseIcons?(this.expanded=!0,this.refreshChildDisplay(),pt(this.eContainer,!0),this):(e=e!=null?e:!this.expanded,this.expanded===e?this:(this.expanded=e,this.refreshAriaStatus(),this.refreshChildDisplay(),pt(this.eContainer,e),this.dispatchEvent({type:this.expanded?t.EVENT_EXPANDED:t.EVENT_COLLAPSED}),this))},t.prototype.addItems=function(e){var r=this;e.forEach(function(n){return r.addItem(n)})},t.prototype.addItem=function(e){var r=this.eContainer,n=e instanceof ee?e.getGui():e;n.classList.add("ag-group-item","ag-"+this.cssIdentifier+"-group-item"),r.appendChild(n),this.items.push(n)},t.prototype.hideItem=function(e,r){var n=this.items[r];n.classList.toggle("ag-hidden",e)},t.prototype.setTitle=function(e){return this.eTitle.innerText=e,this},t.prototype.addCssClassToTitleBar=function(e){this.eTitleBar.classList.add(e)},t.prototype.setEnabled=function(e,r){return this.enabled=e,this.refreshDisabledStyles(),this.toggleGroupExpand(e),r||this.cbGroupEnabled.setValue(e),this},t.prototype.isEnabled=function(){return this.enabled},t.prototype.onEnableChange=function(e){var r=this;return this.cbGroupEnabled.onValueChange(function(n){r.setEnabled(n,!0),e(n)}),this},t.prototype.hideEnabledCheckbox=function(e){return this.suppressEnabledCheckbox=e,this.refreshChildDisplay(),this.refreshDisabledStyles(),this},t.prototype.hideOpenCloseIcons=function(e){return this.suppressOpenCloseIcons=e,e&&this.toggleGroupExpand(!0),this},t.prototype.refreshDisabledStyles=function(){this.addOrRemoveCssClass("ag-disabled",!this.enabled),this.suppressEnabledCheckbox&&!this.enabled?(this.eTitleBar.classList.add("ag-disabled-group-title-bar"),this.eTitleBar.removeAttribute("tabindex")):(this.eTitleBar.classList.remove("ag-disabled-group-title-bar"),this.eTitleBar.setAttribute("tabindex","0")),this.eContainer.classList.toggle("ag-disabled-group-container",!this.enabled)},t.EVENT_EXPANDED="expanded",t.EVENT_COLLAPSED="collapsed",xo([I("eTitleBar")],t.prototype,"eTitleBar",void 0),xo([I("eGroupOpenedIcon")],t.prototype,"eGroupOpenedIcon",void 0),xo([I("eGroupClosedIcon")],t.prototype,"eGroupClosedIcon",void 0),xo([I("eToolbar")],t.prototype,"eToolbar",void 0),xo([I("cbGroupEnabled")],t.prototype,"cbGroupEnabled",void 0),xo([I("eTitle")],t.prototype,"eTitle",void 0),xo([I("eContainer")],t.prototype,"eContainer",void 0),xo([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var x2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),F_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},gR=function(i){x2(t,i);function t(e){var r=i.call(this)||this;r.skipTabGuardFocus=!1;var n=e.comp,o=e.eTopGuard,a=e.eBottomGuard,s=e.focusInnerElement,l=e.onFocusIn,u=e.onFocusOut,c=e.shouldStopEventPropagation,p=e.onTabKeyDown,h=e.handleKeyDown,d=e.eFocusableElement;return r.comp=n,r.eTopGuard=o,r.eBottomGuard=a,r.providedFocusInnerElement=s,r.eFocusableElement=d,r.providedFocusIn=l,r.providedFocusOut=u,r.providedShouldStopEventPropagation=c,r.providedOnTabKeyDown=p,r.providedHandleKeyDown=h,r}return t.prototype.postConstruct=function(){var e=this;this.createManagedBean(new ai(this.eFocusableElement,{shouldStopEventPropagation:function(){return e.shouldStopEventPropagation()},onTabKeyDown:function(r){return e.onTabKeyDown(r)},handleKeyDown:function(r){return e.handleKeyDown(r)},onFocusIn:function(r){return e.onFocusIn(r)},onFocusOut:function(r){return e.onFocusOut(r)}})),this.activateTabGuards(),[this.eTopGuard,this.eBottomGuard].forEach(function(r){return e.addManagedListener(r,"focus",e.onFocus.bind(e))})},t.prototype.handleKeyDown=function(e){this.providedHandleKeyDown&&this.providedHandleKeyDown(e)},t.prototype.tabGuardsAreActive=function(){return!!this.eTopGuard&&this.eTopGuard.hasAttribute("tabIndex")},t.prototype.shouldStopEventPropagation=function(){return this.providedShouldStopEventPropagation?this.providedShouldStopEventPropagation():!1},t.prototype.activateTabGuards=function(){this.comp.setTabIndex(this.getGridTabIndex())},t.prototype.deactivateTabGuards=function(){this.comp.setTabIndex()},t.prototype.onFocus=function(e){if(this.skipTabGuardFocus){this.skipTabGuardFocus=!1;return}var r=e.target===this.eBottomGuard;this.providedFocusInnerElement?this.providedFocusInnerElement(r):this.focusInnerElement(r)},t.prototype.onFocusIn=function(e){this.providedFocusIn&&this.providedFocusIn(e)||this.deactivateTabGuards()},t.prototype.onFocusOut=function(e){this.providedFocusOut&&this.providedFocusOut(e)||this.eFocusableElement.contains(e.relatedTarget)||this.activateTabGuards()},t.prototype.onTabKeyDown=function(e){var r=this;if(this.providedOnTabKeyDown){this.providedOnTabKeyDown(e);return}if(!e.defaultPrevented){var n=this.tabGuardsAreActive();n&&this.deactivateTabGuards();var o=this.getNextFocusableElement(e.shiftKey);n&&setTimeout(function(){return r.activateTabGuards()},0),o&&(o.focus(),e.preventDefault())}},t.prototype.getGridTabIndex=function(){return this.gridOptionsWrapper.getGridTabIndex()},t.prototype.focusInnerElement=function(e){e===void 0&&(e=!1);var r=this.focusService.findFocusableElements(this.eFocusableElement);this.tabGuardsAreActive()&&(r.splice(0,1),r.splice(r.length-1,1)),r.length&&r[e?r.length-1:0].focus()},t.prototype.getNextFocusableElement=function(e){return this.focusService.findNextFocusableElement(this.eFocusableElement,!1,e)},t.prototype.forceFocusOutOfContainer=function(e){e===void 0&&(e=!1);var r=e?this.eTopGuard:this.eBottomGuard;this.activateTabGuards(),this.skipTabGuardFocus=!0,r.focus()},F_([y("focusService")],t.prototype,"focusService",void 0),F_([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var T2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),O2=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},E2=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(O2(arguments[t]));return i},bs=function(i){T2(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.initialiseTabGuard=function(e){this.eTopGuard=this.createTabGuard("top"),this.eBottomGuard=this.createTabGuard("bottom"),this.eFocusableElement=this.getFocusableElement();var r=[this.eTopGuard,this.eBottomGuard],n={setTabIndex:function(o){r.forEach(function(a){return o!=null?a.setAttribute("tabIndex",o):a.removeAttribute("tabIndex")})}};this.addTabGuards(this.eTopGuard,this.eBottomGuard),this.tabGuardCtrl=this.createManagedBean(new gR({comp:n,eTopGuard:this.eTopGuard,eBottomGuard:this.eBottomGuard,eFocusableElement:this.eFocusableElement,onFocusIn:e.onFocusIn,onFocusOut:e.onFocusOut,focusInnerElement:e.focusInnerElement,handleKeyDown:e.handleKeyDown,onTabKeyDown:e.onTabKeyDown,shouldStopEventPropagation:e.shouldStopEventPropagation}))},t.prototype.createTabGuard=function(e){var r=document.createElement("div");return r.classList.add("ag-tab-guard","ag-tab-guard-"+e),_r(r,"presentation"),r},t.prototype.addTabGuards=function(e,r){this.eFocusableElement.insertAdjacentElement("afterbegin",e),this.eFocusableElement.insertAdjacentElement("beforeend",r)},t.prototype.removeAllChildrenExceptTabGuards=function(){var e=[this.eTopGuard,this.eBottomGuard];gn(this.getFocusableElement()),this.addTabGuards.apply(this,E2(e))},t.prototype.forceFocusOutOfContainer=function(e){e===void 0&&(e=!1),this.tabGuardCtrl.forceFocusOutOfContainer(e)},t.prototype.appendChild=function(e,r){$c(e)||(e=e.getGui());var n=this.eBottomGuard;n?n.insertAdjacentElement("beforebegin",e):i.prototype.appendChild.call(this,e,r)},t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var R2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),sd=globalThis&&globalThis.__assign||function(){return sd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},sd.apply(this,arguments)},I_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ep=function(i){R2(t,i);function t(e){e===void 0&&(e=1);var r=i.call(this,'<div class="ag-menu-list" role="tree"></div>')||this;return r.level=e,r.menuItems=[],r}return t.prototype.postConstruct=function(){var e=this;this.initialiseTabGuard({onTabKeyDown:function(r){return e.onTabKeyDown(r)},handleKeyDown:function(r){return e.handleKeyDown(r)}})},t.prototype.onTabKeyDown=function(e){var r=this.getParentComponent(),n=r&&r.getGui(),o=n&&n.classList.contains("ag-focus-managed");o||e.preventDefault(),e.shiftKey&&this.closeIfIsChild(e)},t.prototype.handleKeyDown=function(e){switch(e.key){case j.UP:case j.RIGHT:case j.DOWN:case j.LEFT:e.preventDefault(),this.handleNavKey(e.key);break;case j.ESCAPE:var r=this.findTopMenu();r&&this.focusService.focusInto(r.getGui());break}},t.prototype.clearActiveItem=function(){this.activeMenuItem&&(this.activeMenuItem.deactivate(),this.activeMenuItem=null)},t.prototype.addMenuItems=function(e){var r=this;e!=null&&e.forEach(function(n){n==="separator"?r.addSeparator():typeof n=="string"?console.warn("AG Grid: unrecognised menu item "+n):r.addItem(n)})},t.prototype.addItem=function(e){var r=this,n=this.createManagedBean(new Dn(sd(sd({},e),{isAnotherSubMenuOpen:function(){return r.menuItems.some(function(o){return o.isSubMenuOpen()})}})));n.setParentComponent(this),UO(n.getGui(),this.level),this.menuItems.push(n),this.appendChild(n.getGui()),this.addManagedListener(n,Dn.EVENT_MENU_ITEM_SELECTED,function(o){r.dispatchEvent(o)}),this.addManagedListener(n,Dn.EVENT_MENU_ITEM_ACTIVATED,function(o){r.activeMenuItem&&r.activeMenuItem!==o.menuItem&&r.activeMenuItem.deactivate(),r.activeMenuItem=o.menuItem})},t.prototype.activateFirstItem=function(){var e=this.menuItems.filter(function(r){return!r.isDisabled()})[0];!e||e.activate()},t.prototype.addSeparator=function(){var e=`
            <div class="ag-menu-separator" aria-hidden="true">
                <div class="ag-menu-separator-part"></div>
                <div class="ag-menu-separator-part"></div>
                <div class="ag-menu-separator-part"></div>
                <div class="ag-menu-separator-part"></div>
            </div>`;this.appendChild(Kt(e))},t.prototype.findTopMenu=function(){var e=this.getParentComponent();if(!e&&this instanceof t)return this;for(;;){var r=e&&e.getParentComponent&&e.getParentComponent();if(!r||!(r instanceof t||r instanceof Dn))break;e=r}return e instanceof t?e:void 0},t.prototype.handleNavKey=function(e){switch(e){case j.UP:case j.DOWN:var r=this.findNextItem(e===j.UP);r&&r!==this.activeMenuItem&&r.activate();return}var n=this.gridOptionsWrapper.isEnableRtl()?j.RIGHT:j.LEFT;e===n?this.closeIfIsChild():this.openChild()},t.prototype.closeIfIsChild=function(e){var r=this.getParentComponent();r&&r instanceof Dn&&(e&&e.preventDefault(),r.closeSubMenu(),r.getGui().focus())},t.prototype.openChild=function(){this.activeMenuItem&&this.activeMenuItem.openSubMenu(!0)},t.prototype.findNextItem=function(e){var r=this.menuItems.filter(function(l){return!l.isDisabled()});if(!!r.length){if(!this.activeMenuItem)return e?rt(r):r[0];e&&r.reverse();for(var n,o=!1,a=0;a<r.length;a++){var s=r[a];if(!o){s===this.activeMenuItem&&(o=!0);continue}n=s;break}return n||this.activeMenuItem}},t.prototype.destroy=function(){this.clearActiveItem(),i.prototype.destroy.call(this)},I_([y("focusService")],t.prototype,"focusService",void 0),I_([B],t.prototype,"postConstruct",null),t}(bs);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var P2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),A2=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},vR=function(i){P2(t,i);function t(e){var r=i.call(this)||this;return r.wrappedComponent=e,r.setTemplateFromElement(e.getGui()),r}return t.prototype.postConstruct=function(){var e=this;this.initialiseTabGuard({onTabKeyDown:function(r){return e.onTabKeyDown(r)},handleKeyDown:function(r){return e.handleKeyDown(r)}})},t.prototype.handleKeyDown=function(e){e.key===j.ESCAPE&&this.closePanel()},t.prototype.onTabKeyDown=function(e){e.defaultPrevented||(this.closePanel(),e.preventDefault())},t.prototype.closePanel=function(){var e=this.parentComponent;e.closeSubMenu(),setTimeout(function(){return e.getGui().focus()},0)},A2([B],t.prototype,"postConstruct",null),t}(bs);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var D2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),L_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Dn=function(i){D2(t,i);function t(e){var r=i.call(this)||this;return r.params=e,r.isActive=!1,r.subMenuIsOpen=!1,r.setTemplate('<div class="'+r.getClassName()+'" tabindex="-1" role="treeitem"></div>'),r}return t.prototype.init=function(){var e=this;this.addIcon(),this.addName(),this.addShortcut(),this.addSubMenu(),this.addTooltip();var r=this.getGui();this.params.disabled?(this.addCssClass(this.getClassName("disabled")),GC(r,!0)):(this.addGuiEventListener("click",function(n){return e.onItemSelected(n)}),this.addGuiEventListener("keydown",function(n){(n.key===j.ENTER||n.key===j.SPACE)&&(n.preventDefault(),e.onItemSelected(n))}),this.addGuiEventListener("mousedown",function(n){n.stopPropagation(),n.preventDefault()}),this.addGuiEventListener("mouseenter",function(){return e.onMouseEnter()}),this.addGuiEventListener("mouseleave",function(){return e.onMouseLeave()})),this.params.cssClasses&&this.params.cssClasses.forEach(function(n){return e.addCssClass(n)})},t.prototype.isDisabled=function(){return!!this.params.disabled},t.prototype.openSubMenu=function(e){var r=this;if(e===void 0&&(e=!1),this.closeSubMenu(),!!this.params.subMenu){var n=Kt('<div class="ag-menu" role="presentation"></div>'),o;if(this.params.subMenu instanceof Array){var a=BO(this.getGui()),s=isNaN(a)?1:a+1,l=this.createBean(new ep(s));l.setParentComponent(this),l.addMenuItems(this.params.subMenu),n.appendChild(l.getGui()),this.addManagedListener(l,t.EVENT_MENU_ITEM_SELECTED,function(C){return r.dispatchEvent(C)}),l.addGuiEventListener("mouseenter",function(){return r.cancelDeactivate()}),o=function(){return r.destroyBean(l)},e&&setTimeout(function(){return l.activateFirstItem()},0)}else{var u=this.params.subMenu,c=this.createBean(new vR(u));c.setParentComponent(this);var p=c.getGui(),h="mouseenter",d=function(){return r.cancelDeactivate()};p.addEventListener(h,d),o=function(){return p.removeEventListener(h,d)},n.appendChild(p),u.afterGuiAttached&&setTimeout(function(){return u.afterGuiAttached()},0)}var f=this.getGui(),g=this.popupService.positionPopupForMenu.bind(this.popupService,{eventSource:f,ePopup:n}),v=this.gridOptionsWrapper.getLocaleTextFunc(),m=this.popupService.addPopup({modal:!0,eChild:n,positionCallback:g,anchorToElement:f,ariaLabel:v("ariaLabelSubMenu","SubMenu")});this.subMenuIsOpen=!0,dn(f,!0),this.hideSubMenu=function(){m&&m.hideFunc(),r.subMenuIsOpen=!1,dn(f,!1),o()}}},t.prototype.closeSubMenu=function(){!this.hideSubMenu||(this.hideSubMenu(),this.hideSubMenu=null,dn(this.getGui(),!1))},t.prototype.isSubMenuOpen=function(){return this.subMenuIsOpen},t.prototype.activate=function(e){var r=this;this.cancelActivate(),!this.params.disabled&&(this.isActive=!0,this.addCssClass(this.getClassName("active")),this.getGui().focus(),e&&this.params.subMenu&&window.setTimeout(function(){r.isAlive()&&r.isActive&&r.openSubMenu()},300),this.onItemActivated())},t.prototype.deactivate=function(){this.cancelDeactivate(),this.removeCssClass(this.getClassName("active")),this.isActive=!1,this.subMenuIsOpen&&this.hideSubMenu()},t.prototype.addIcon=function(){if(!(!this.params.checked&&!this.params.icon&&this.params.isCompact)){var e=Kt('<span ref="eIcon" class="'+this.getClassName("part")+" "+this.getClassName("icon")+'" role="presentation"></span>');this.params.checked?e.appendChild(Ft("check",this.gridOptionsWrapper)):this.params.icon&&($c(this.params.icon)?e.appendChild(this.params.icon):typeof this.params.icon=="string"?e.innerHTML=this.params.icon:console.warn("AG Grid: menu item icon must be DOM node or string")),this.getGui().appendChild(e)}},t.prototype.addName=function(){if(!(!this.params.name&&this.params.isCompact)){var e=Kt('<span ref="eName" class="'+this.getClassName("part")+" "+this.getClassName("text")+'">'+(this.params.name||"")+"</span>");this.getGui().appendChild(e)}},t.prototype.addTooltip=function(){!this.params.tooltip||(this.tooltip=this.params.tooltip,this.gridOptionsWrapper.isEnableBrowserTooltips()?this.getGui().setAttribute("title",this.tooltip):this.createManagedBean(new nf(this)))},t.prototype.getTooltipParams=function(){return{location:"menu",value:this.tooltip}},t.prototype.addShortcut=function(){if(!(!this.params.shortcut&&this.params.isCompact)){var e=Kt('<span ref="eShortcut" class="'+this.getClassName("part")+" "+this.getClassName("shortcut")+'">'+(this.params.shortcut||"")+"</span>");this.getGui().appendChild(e)}},t.prototype.addSubMenu=function(){if(!(!this.params.subMenu&&this.params.isCompact)){var e=Kt('<span ref="ePopupPointer" class="'+this.getClassName("part")+" "+this.getClassName("popup-pointer")+'"></span>'),r=this.getGui();if(this.params.subMenu){var n=this.gridOptionsWrapper.isEnableRtl()?"smallLeft":"smallRight";dn(r,!1),e.appendChild(Ft(n,this.gridOptionsWrapper))}r.appendChild(e)}},t.prototype.onItemSelected=function(e){if(this.params.action?this.params.action():this.openSubMenu(e&&e.type==="keydown"),!(this.params.subMenu&&!this.params.action)){var r={type:t.EVENT_MENU_ITEM_SELECTED,action:this.params.action,checked:this.params.checked,cssClasses:this.params.cssClasses,disabled:this.params.disabled,icon:this.params.icon,name:this.params.name,shortcut:this.params.shortcut,subMenu:this.params.subMenu,tooltip:this.params.tooltip,event:e};this.dispatchEvent(r)}},t.prototype.onItemActivated=function(){var e={type:t.EVENT_MENU_ITEM_ACTIVATED,menuItem:this};this.dispatchEvent(e)},t.prototype.cancelActivate=function(){this.activateTimeoutId&&(window.clearTimeout(this.activateTimeoutId),this.activateTimeoutId=0)},t.prototype.cancelDeactivate=function(){this.deactivateTimeoutId&&(window.clearTimeout(this.deactivateTimeoutId),this.deactivateTimeoutId=0)},t.prototype.onMouseEnter=function(){var e=this;this.cancelDeactivate(),this.params.isAnotherSubMenuOpen()?this.activateTimeoutId=window.setTimeout(function(){return e.activate(!0)},t.ACTIVATION_DELAY):this.activate(!0)},t.prototype.onMouseLeave=function(){var e=this;this.cancelActivate(),this.isSubMenuOpen()?this.deactivateTimeoutId=window.setTimeout(function(){return e.deactivate()},t.ACTIVATION_DELAY):this.deactivate()},t.prototype.getClassName=function(e){var r=this.params.isCompact?"ag-compact-menu-option":"ag-menu-option";return e?r+"-"+e:r},t.EVENT_MENU_ITEM_SELECTED="menuItemSelected",t.EVENT_MENU_ITEM_ACTIVATED="menuItemActivated",t.ACTIVATION_DELAY=80,L_([y("popupService")],t.prototype,"popupService",void 0),L_([B],t.prototype,"init",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var M2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),wn=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},F2=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},I2=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(F2(arguments[t]));return i},Sl=function(i){M2(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}e=t,t.addKeyboardModeEvents=function(r,n){var o=e.instancesMonitored.get(r);o&&o.length>0?o.indexOf(n)===-1&&o.push(n):(e.instancesMonitored.set(r,[n]),r.addEventListener("keydown",e.toggleKeyboardMode),r.addEventListener("mousedown",e.toggleKeyboardMode))},t.removeKeyboardModeEvents=function(r,n){var o=e.instancesMonitored.get(r),a=[];o&&o.length&&(a=I2(o).filter(function(s){return s!==n}),e.instancesMonitored.set(r,a)),a.length===0&&(r.removeEventListener("keydown",e.toggleKeyboardMode),r.removeEventListener("mousedown",e.toggleKeyboardMode))},t.toggleKeyboardMode=function(r){var n=e.keyboardModeActive,o=r.type==="keydown";if(!(o&&(r.ctrlKey||r.metaKey||r.altKey))&&!(n&&o||!n&&!o)){e.keyboardModeActive=o;var a=r.target.ownerDocument;if(!!a){var s=e.instancesMonitored.get(a);s&&s.forEach(function(l){l.dispatchEvent({type:o?x.EVENT_KEYBOARD_FOCUS:x.EVENT_MOUSE_FOCUS})})}}},t.prototype.init=function(){var r=this,n=this.clearFocusedCell.bind(this);this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,n),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.onColumnEverythingChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_GROUP_OPENED,n),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,n),this.ctrlsService.whenReady(function(o){r.gridCtrl=o.gridCtrl;var a=r.gridOptionsWrapper.getDocument();e.addKeyboardModeEvents(a,r.gridCtrl),r.addDestroyFunc(function(){return r.unregisterGridCompController(r.gridCtrl)})})},t.prototype.unregisterGridCompController=function(r){var n=this.gridOptionsWrapper.getDocument();e.removeKeyboardModeEvents(n,r)},t.prototype.onColumnEverythingChanged=function(){if(!!this.focusedCellPosition){var r=this.focusedCellPosition.column,n=this.columnModel.getGridColumn(r.getId());r!==n&&this.clearFocusedCell()}},t.prototype.isKeyboardMode=function(){return e.keyboardModeActive},t.prototype.getFocusCellToUseAfterRefresh=function(){var r=this.gridOptionsWrapper.getDocument();return this.gridOptionsWrapper.isSuppressFocusAfterRefresh()||!this.focusedCellPosition||this.isDomDataMissingInHierarchy(r.activeElement,ra.DOM_DATA_KEY_ROW_CTRL)?null:this.focusedCellPosition},t.prototype.getFocusHeaderToUseAfterRefresh=function(){var r=this.gridOptionsWrapper.getDocument();return this.gridOptionsWrapper.isSuppressFocusAfterRefresh()||!this.focusedHeaderPosition||this.isDomDataMissingInHierarchy(r.activeElement,Zc.DOM_DATA_KEY_HEADER_CTRL)?null:this.focusedHeaderPosition},t.prototype.isDomDataMissingInHierarchy=function(r,n){for(var o=r;o;){var a=this.gridOptionsWrapper.getDomData(o,n);if(a)return!1;o=o.parentNode}return!0},t.prototype.clearFocusedCell=function(){this.focusedCellPosition=null,this.onCellFocused(!1)},t.prototype.getFocusedCell=function(){return this.focusedCellPosition},t.prototype.setFocusedCell=function(r,n,o,a){a===void 0&&(a=!1);var s=this.columnModel.getGridColumn(n);if(!s){this.focusedCellPosition=null;return}this.focusedCellPosition=s?{rowIndex:r,rowPinned:Jo(o),column:s}:null,this.onCellFocused(a)},t.prototype.isCellFocused=function(r){return this.focusedCellPosition==null?!1:this.focusedCellPosition.column===r.column&&this.isRowFocused(r.rowIndex,r.rowPinned)},t.prototype.isRowNodeFocused=function(r){return this.isRowFocused(r.rowIndex,r.rowPinned)},t.prototype.isHeaderWrapperFocused=function(r){if(this.focusedHeaderPosition==null)return!1;var n=r.getColumnGroupChild(),o=r.getRowIndex(),a=r.getPinned(),s=this.focusedHeaderPosition,l=s.column,u=s.headerRowIndex;return n===l&&o===u&&a==l.getPinned()},t.prototype.clearFocusedHeader=function(){this.focusedHeaderPosition=null},t.prototype.getFocusedHeader=function(){return this.focusedHeaderPosition},t.prototype.setFocusedHeader=function(r,n){this.focusedHeaderPosition={headerRowIndex:r,column:n}},t.prototype.focusHeaderPosition=function(r){var n=r.direction,o=r.fromTab,a=r.allowUserOverride,s=r.event,l=r.headerPosition;if(a){var u=this.gridOptionsWrapper,c=this.getFocusedHeader(),p=this.headerNavigationService.getHeaderRowCount();if(o){var h=u.getTabToNextHeaderFunc();if(h){var d={backwards:n==="Before",previousHeaderPosition:c,nextHeaderPosition:l,headerRowCount:p};l=h(d)}}else{var h=u.getNavigateToNextHeaderFunc();if(h&&s){var f={key:s.key,previousHeaderPosition:c,nextHeaderPosition:l,headerRowCount:p,event:s};l=h(f)}}}if(!l)return!1;if(l.headerRowIndex===-1)return this.focusGridView(l.column);this.headerNavigationService.scrollToColumn(l.column,n);var g=this.ctrlsService.getHeaderRowContainerCtrl(l.column.getPinned()),v=g.focusHeader(l.headerRowIndex,l.column,s);return v},t.prototype.focusFirstHeader=function(){var r=this.columnModel.getAllDisplayedColumns()[0];return r?(r.getParent()&&(r=this.columnModel.getColumnGroupAtLevel(r,0)),this.focusHeaderPosition({headerPosition:{headerRowIndex:0,column:r}})):!1},t.prototype.focusLastHeader=function(r){var n=this.headerNavigationService.getHeaderRowCount()-1,o=rt(this.columnModel.getAllDisplayedColumns());return this.focusHeaderPosition({headerPosition:{headerRowIndex:n,column:o},event:r})},t.prototype.isAnyCellFocused=function(){return!!this.focusedCellPosition},t.prototype.isRowFocused=function(r,n){return this.focusedCellPosition==null?!1:this.focusedCellPosition.rowIndex===r&&this.focusedCellPosition.rowPinned===Jo(n)},t.prototype.findFocusableElements=function(r,n,o){o===void 0&&(o=!1);var a=L.FOCUSABLE_SELECTOR,s=L.FOCUSABLE_EXCLUDE;n&&(s+=", "+n),o&&(s+=', [tabindex="-1"]');var l=Array.prototype.slice.apply(r.querySelectorAll(a)),u=Array.prototype.slice.apply(r.querySelectorAll(s));if(!u.length)return l;var c=function(p,h){return p.filter(function(d){return h.indexOf(d)===-1})};return c(l,u)},t.prototype.focusInto=function(r,n,o){n===void 0&&(n=!1),o===void 0&&(o=!1);var a=this.findFocusableElements(r,null,o),s=n?rt(a):a[0];return s?(s.focus(),!0):!1},t.prototype.findNextFocusableElement=function(r,n,o){r===void 0&&(r=this.eGridDiv);var a=this.findFocusableElements(r,n?':not([tabindex="-1"])':null),s=this.gridOptionsWrapper.getDocument(),l=s.activeElement,u;n?u=a.findIndex(function(p){return p.contains(l)}):u=a.indexOf(l);var c=u+(o?-1:1);return c<0||c>=a.length?null:a[c]},t.prototype.isFocusUnderManagedComponent=function(r){var n=this.gridOptionsWrapper.getDocument(),o=r.querySelectorAll("."+ai.FOCUS_MANAGED_CLASS);if(!o.length)return!1;for(var a=0;a<o.length;a++)if(o[a].contains(n.activeElement))return!0;return!1},t.prototype.findTabbableParent=function(r,n){n===void 0&&(n=5);for(var o=0;r&&Kh(r)===null&&++o<=n;)r=r.parentElement;return Kh(r)===null?null:r},t.prototype.onCellFocused=function(r){var n={type:x.EVENT_CELL_FOCUSED,forceBrowserFocus:r,rowIndex:null,column:null,floating:null,api:this.gridApi,columnApi:this.columnApi,rowPinned:null,isFullWidthCell:!1};if(this.focusedCellPosition){var o=n.rowIndex=this.focusedCellPosition.rowIndex,a=n.rowPinned=this.focusedCellPosition.rowPinned;n.column=this.focusedCellPosition.column;var s=this.rowRenderer.getRowByPosition({rowIndex:o,rowPinned:a});s&&(n.isFullWidthCell=s.isFullWidth())}this.eventService.dispatchEvent(n)},t.prototype.focusGridView=function(r,n){if(this.gridOptionsWrapper.isSuppressCellFocus())return n?this.focusLastHeader():this.focusNextGridCoreContainer(!1);var o=n?this.rowPositionUtils.getLastRow():this.rowPositionUtils.getFirstRow();if(!o)return!1;var a=o.rowIndex,s=o.rowPinned,l=this.getFocusedHeader();if(!r&&l&&(r=l.column),a==null||!r)return!1;if(this.navigationService.ensureCellVisible({rowIndex:a,column:r,rowPinned:s}),this.setFocusedCell(a,r,Jo(s),!0),this.rangeService){var u={rowIndex:a,rowPinned:s,column:r};this.rangeService.setRangeToCell(u)}return!0},t.prototype.focusNextGridCoreContainer=function(r){return this.gridCtrl.focusNextInnerContainer(r)?!0:(!r&&!this.gridCtrl.isDetailGrid()&&this.gridCtrl.forceFocusOutOfContainer(),!1)};var e;return t.AG_KEYBOARD_FOCUS="ag-keyboard-focus",t.keyboardModeActive=!1,t.instancesMonitored=new Map,wn([y("eGridDiv")],t.prototype,"eGridDiv",void 0),wn([y("columnModel")],t.prototype,"columnModel",void 0),wn([y("headerNavigationService")],t.prototype,"headerNavigationService",void 0),wn([y("columnApi")],t.prototype,"columnApi",void 0),wn([y("gridApi")],t.prototype,"gridApi",void 0),wn([y("rowRenderer")],t.prototype,"rowRenderer",void 0),wn([y("rowPositionUtils")],t.prototype,"rowPositionUtils",void 0),wn([De("rangeService")],t.prototype,"rangeService",void 0),wn([y("navigationService")],t.prototype,"navigationService",void 0),wn([y("ctrlsService")],t.prototype,"ctrlsService",void 0),wn([B],t.prototype,"init",null),t=e=wn([Q("focusService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var L2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),wu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Io;(function(i){i[i.vertical=0]="vertical",i[i.horizontal=1]="horizontal"})(Io||(Io={}));var N2=0,mR=function(i){L2(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.popupList=[],e}return t.prototype.postConstruct=function(){var e=this;this.ctrlsService.whenReady(function(r){e.gridCtrl=r.gridCtrl,e.addManagedListener(e.gridCtrl,x.EVENT_KEYBOARD_FOCUS,function(){e.popupList.forEach(function(n){return n.element.classList.add(Sl.AG_KEYBOARD_FOCUS)})}),e.addManagedListener(e.gridCtrl,x.EVENT_MOUSE_FOCUS,function(){e.popupList.forEach(function(n){return n.element.classList.remove(Sl.AG_KEYBOARD_FOCUS)})})})},t.prototype.getPopupParent=function(){var e=this.gridOptionsWrapper.getPopupParent();return e||this.gridCtrl.getGui()},t.prototype.positionPopupForMenu=function(e){var r=e.eventSource.getBoundingClientRect(),n=this.getParentRect(),o=this.keepXYWithinBounds(e.ePopup,r.top-n.top,Io.vertical),a=e.ePopup.clientWidth>0?e.ePopup.clientWidth:200;e.ePopup.style.minWidth=a+"px";var s=n.right-n.left,l=s-a,u;this.gridOptionsWrapper.isEnableRtl()?(u=p(),u<0&&(u=c()),u>l&&(u=0)):(u=c(),u>l&&(u=p()),u<0&&(u=0)),e.ePopup.style.left=u+"px",e.ePopup.style.top=o+"px";function c(){return r.right-n.left-2}function p(){return r.left-n.left-a}},t.prototype.positionPopupUnderMouseEvent=function(e){var r=e.ePopup,n=e.nudgeX,o=e.nudgeY,a=this.calculatePointerAlign(e.mouseEvent),s=a.x,l=a.y;this.positionPopup({ePopup:r,x:s,y:l,nudgeX:n,nudgeY:o,keepWithinBounds:!0}),this.callPostProcessPopup(e.type,e.ePopup,null,e.mouseEvent,e.column,e.rowNode)},t.prototype.calculatePointerAlign=function(e){var r=this.getParentRect();return{x:e.clientX-r.left,y:e.clientY-r.top}},t.prototype.positionPopupUnderComponent=function(e){var r=e.eventSource.getBoundingClientRect(),n=e.alignSide||"left",o=this.getParentRect(),a=r.left-o.left;n==="right"&&(a-=e.ePopup.offsetWidth-r.width),this.positionPopup({ePopup:e.ePopup,nudgeX:e.nudgeX,nudgeY:e.nudgeY,x:a,y:r.top-o.top+r.height,keepWithinBounds:e.keepWithinBounds}),this.callPostProcessPopup(e.type,e.ePopup,e.eventSource,null,e.column,e.rowNode)},t.prototype.positionPopupOverComponent=function(e){var r=e.eventSource.getBoundingClientRect(),n=this.getParentRect();this.positionPopup({ePopup:e.ePopup,nudgeX:e.nudgeX,nudgeY:e.nudgeY,x:r.left-n.left,y:r.top-n.top,keepWithinBounds:e.keepWithinBounds}),this.callPostProcessPopup(e.type,e.ePopup,e.eventSource,null,e.column,e.rowNode)},t.prototype.callPostProcessPopup=function(e,r,n,o,a,s){var l=this.gridOptionsWrapper.getPostProcessPopupFunc();if(l){var u={column:a,rowNode:s,ePopup:r,type:e,eventSource:n,mouseEvent:o};l(u)}},t.prototype.positionPopup=function(e){var r=e.ePopup,n=e.keepWithinBounds,o=e.nudgeX,a=e.nudgeY,s=e.x,l=e.y;o&&(s+=o),a&&(l+=a),n&&(s=this.keepXYWithinBounds(r,s,Io.horizontal),l=this.keepXYWithinBounds(r,l,Io.vertical)),r.style.left=s+"px",r.style.top=l+"px"},t.prototype.getActivePopups=function(){return this.popupList.map(function(e){return e.element})},t.prototype.getPopupList=function(){return this.popupList},t.prototype.getParentRect=function(){var e=this.gridOptionsWrapper.getDocument(),r=this.getPopupParent();r===e.body&&(r=e.documentElement);var n=getComputedStyle(r),o=r.getBoundingClientRect();return{top:o.top+parseFloat(n.borderTopWidth)||0,left:o.left+parseFloat(n.borderLeftWidth)||0,right:o.right+parseFloat(n.borderRightWidth)||0,bottom:o.bottom+parseFloat(n.borderBottomWidth)||0}},t.prototype.keepXYWithinBounds=function(e,r,n){var o=n===Io.vertical,a=o?"clientHeight":"clientWidth",s=o?"top":"left",l=o?"offsetHeight":"offsetWidth",u=o?"scrollTop":"scrollLeft",c=this.gridOptionsWrapper.getDocument(),p=c.documentElement,h=this.getPopupParent(),d=h.getBoundingClientRect(),f=c.documentElement.getBoundingClientRect(),g=h===c.body,v=e[l],m=o?PC:rf,C=g?m(p)+p[u]:h[a];g&&(C-=Math.abs(f[s]-d[s]));var w=C-v;return Math.min(Math.max(r,0),Math.abs(w))},t.prototype.keepPopupPositionedRelativeTo=function(e){var r=this,n=this.getPopupParent(),o=n.getBoundingClientRect(),a=e.element.getBoundingClientRect(),s=o.top-a.top,l=o.left-a.left,u=s,c=l,p=e.ePopup.style.top,h=parseInt(p.substring(0,p.length-1),10),d=e.ePopup.style.left,f=parseInt(d.substring(0,d.length-1),10);return new ct(function(g){r.getFrameworkOverrides().setInterval(function(){var v=n.getBoundingClientRect(),m=e.element.getBoundingClientRect(),C=m.top==0&&m.left==0&&m.height==0&&m.width==0;if(C){e.hidePopup();return}var w=v.top-m.top;if(w!=u){var b=r.keepXYWithinBounds(e.ePopup,h+s-w,Io.vertical);e.ePopup.style.top=b+"px"}u=w;var S=v.left-m.left;if(S!=c){var T=r.keepXYWithinBounds(e.ePopup,f+l-S,Io.horizontal);e.ePopup.style.left=T+"px"}c=S},200).then(function(v){var m=function(){v!=null&&window.clearInterval(v)};g(m)})})},t.prototype.addPopup=function(e){var r=this,n=e.modal,o=e.eChild,a=e.closeOnEsc,s=e.closedCallback,l=e.click,u=e.alwaysOnTop,c=e.afterGuiAttached,p=e.positionCallback,h=e.anchorToElement,d=e.ariaLabel,f=this.gridOptionsWrapper.getDocument(),g=new ct(function(M){return M(function(){})});if(!f)return console.warn("ag-grid: could not find the document, document is empty"),{hideFunc:function(){},stopAnchoringPromise:g};var v=this.popupList.findIndex(function(M){return M.element===o});if(v!==-1){var m=this.popupList[v];return{hideFunc:m.hideFunc,stopAnchoringPromise:m.stopAnchoringPromise}}var C=this.getPopupParent();o.style.top==null&&(o.style.top="0px"),o.style.left==null&&(o.style.left="0px");var w=document.createElement("div"),b=this.environment.getTheme().theme;b&&w.classList.add(b),w.classList.add("ag-popup"),o.classList.add(this.gridOptionsWrapper.isEnableRtl()?"ag-rtl":"ag-ltr","ag-popup-child"),o.hasAttribute("role")||_r(o,"dialog"),Cs(o,d),this.focusService.isKeyboardMode()&&o.classList.add(Sl.AG_KEYBOARD_FOCUS),w.appendChild(o),C.appendChild(w),u?this.setAlwaysOnTop(w,!0):this.bringPopupToFront(w);var S=!1,T=function(M){if(!!w.contains(f.activeElement)){var N=M.key;N===j.ESCAPE&&R({keyboardEvent:M})}},E=function(M){return R({mouseEvent:M})},P=function(M){return R({touchEvent:M})},R=function(M){M===void 0&&(M={});var N=M.mouseEvent,W=M.touchEvent,G=M.keyboardEvent;r.isEventFromCurrentPopup({mouseEvent:N,touchEvent:W},o)||r.isEventSameChainAsOriginalEvent({originalMouseEvent:l,mouseEvent:N,touchEvent:W})||S||(S=!0,C.removeChild(w),f.removeEventListener("keydown",T),f.removeEventListener("mousedown",E),f.removeEventListener("touchstart",P),f.removeEventListener("contextmenu",E),r.eventService.removeEventListener(x.EVENT_DRAG_STARTED,E),s&&s(N||W||G),r.popupList=r.popupList.filter(function(k){return k.element!==o}),g&&g.then(function(k){return k&&k()}))};return c&&c({hidePopup:R}),window.setTimeout(function(){a&&f.addEventListener("keydown",T),n&&(f.addEventListener("mousedown",E),r.eventService.addEventListener(x.EVENT_DRAG_STARTED,E),f.addEventListener("touchstart",P),f.addEventListener("contextmenu",E))},0),p&&p(),h&&(g=this.keepPopupPositionedRelativeTo({element:h,ePopup:o,hidePopup:R})),this.popupList.push({element:o,wrapper:w,hideFunc:R,stopAnchoringPromise:g,instanceId:N2++,isAnchored:!!h}),{hideFunc:R,stopAnchoringPromise:g}},t.prototype.hasAnchoredPopup=function(){return this.popupList.some(function(e){return e.isAnchored})},t.prototype.isEventFromCurrentPopup=function(e,r){var n=e.mouseEvent,o=e.touchEvent,a=n||o;if(!a)return!1;var s=this.popupList.findIndex(function(c){return c.element===r});if(s===-1)return!1;for(var l=s;l<this.popupList.length;l++){var u=this.popupList[l];if($h(u.element,a))return!0}return this.isElementWithinCustomPopup(a.target)},t.prototype.isElementWithinCustomPopup=function(e){for(var r=this.gridOptionsWrapper.getDocument();e&&e!==r.body;){if(e.classList.contains("ag-custom-component-popup")||e.parentElement===null)return!0;e=e.parentElement}return!1},t.prototype.isEventSameChainAsOriginalEvent=function(e){var r=e.originalMouseEvent,n=e.mouseEvent,o=e.touchEvent,a=null;if(n?a=n:o&&(a=o.touches[0]),a&&r){var s=n?n.screenX:0,l=n?n.screenY:0,u=Math.abs(r.screenX-s)<5,c=Math.abs(r.screenY-l)<5;if(u&&c)return!0}return!1},t.prototype.getWrapper=function(e){for(;!e.classList.contains("ag-popup")&&e.parentElement;)e=e.parentElement;return e.classList.contains("ag-popup")?e:null},t.prototype.setAlwaysOnTop=function(e,r){var n=this.getWrapper(e);!n||(n.classList.toggle("ag-always-on-top",!!r),r&&this.bringPopupToFront(n))},t.prototype.bringPopupToFront=function(e){var r=this.getPopupParent(),n=Array.prototype.slice.call(r.querySelectorAll(".ag-popup")),o=n.length,a=Array.prototype.slice.call(r.querySelectorAll(".ag-popup.ag-always-on-top")),s=a.length,l=this.getWrapper(e);if(!(!l||o<=1||!r.contains(e))){var u=n.indexOf(l),c=l.querySelectorAll("div"),p=[];if(c.forEach(function(g){g.scrollTop!==0&&p.push([g,g.scrollTop])}),s){var h=l.classList.contains("ag-always-on-top");h?u!==o-1&&rt(a).insertAdjacentElement("afterend",l):u!==o-s-1&&a[0].insertAdjacentElement("beforebegin",l)}else u!==o-1&&rt(n).insertAdjacentElement("afterend",l);for(;p.length;){var d=p.pop();d[0].scrollTop=d[1]}var f={type:"popupToFront",api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),eWrapper:l};this.eventService.dispatchEvent(f)}},wu([y("environment")],t.prototype,"environment",void 0),wu([y("focusService")],t.prototype,"focusService",void 0),wu([y("ctrlsService")],t.prototype,"ctrlsService",void 0),wu([B],t.prototype,"postConstruct",null),t=wu([Q("popupService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var G2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),xp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},tp=function(i){G2(t,i);function t(e,r,n){e===void 0&&(e="default"),r===void 0&&(r="listbox");var o=i.call(this,t.getTemplate(e))||this;return o.cssIdentifier=e,o.ariaRole=r,o.listName=n,o.renderedRows=new Map,o.rowHeight=20,o.isDestroyed=!1,o}return t.prototype.postConstruct=function(){var e=this;this.addScrollListener(),this.rowHeight=this.getItemHeight(),this.addResizeObserver(),this.initialiseTabGuard({onFocusIn:function(r){return e.onFocusIn(r)},onFocusOut:function(r){return e.onFocusOut(r)},focusInnerElement:function(r){return e.focusInnerElement(r)},onTabKeyDown:function(r){return e.onTabKeyDown(r)},handleKeyDown:function(r){return e.handleKeyDown(r)}}),this.setAriaProperties()},t.prototype.setAriaProperties=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("ariaDefaultListName",this.listName||"List"),n=this.eContainer;_r(n,this.ariaRole),Cs(n,r)},t.prototype.addResizeObserver=function(){var e=this.drawVirtualRows.bind(this),r=this.resizeObserverService.observeResize(this.getGui(),e);this.addDestroyFunc(r)},t.prototype.focusInnerElement=function(e){this.focusRow(e?this.model.getRowCount()-1:0)},t.prototype.onFocusIn=function(e){var r=e.target;return r.classList.contains("ag-virtual-list-item")&&(this.lastFocusedRowIndex=HO(r)-1),!1},t.prototype.onFocusOut=function(e){return this.getFocusableElement().contains(e.relatedTarget)||(this.lastFocusedRowIndex=null),!1},t.prototype.handleKeyDown=function(e){switch(e.key){case j.UP:case j.DOWN:this.navigate(e.key===j.UP)&&e.preventDefault();break}},t.prototype.onTabKeyDown=function(e){this.navigate(e.shiftKey)?e.preventDefault():this.focusService.focusInto(this.getGui(),!e.shiftKey)},t.prototype.navigate=function(e){if(this.lastFocusedRowIndex==null)return!1;var r=this.lastFocusedRowIndex+(e?-1:1);return r<0||r>=this.model.getRowCount()?!1:(this.focusRow(r),!0)},t.prototype.getLastFocusedRow=function(){return this.lastFocusedRowIndex},t.prototype.focusRow=function(e){var r=this;this.ensureIndexVisible(e),window.setTimeout(function(){var n=r.renderedRows.get(e);n&&n.eDiv.focus()},10)},t.prototype.getComponentAt=function(e){var r=this.renderedRows.get(e);return r&&r.rowComponent},t.getTemplate=function(e){return`
            <div class="ag-virtual-list-viewport ag-`+e+`-virtual-list-viewport" role="presentation">
                <div class="ag-virtual-list-container ag-`+e+`-virtual-list-container" ref="eContainer"></div>
            </div>`},t.prototype.getItemHeight=function(){return this.gridOptionsWrapper.getListItemHeight()},t.prototype.ensureIndexVisible=function(e){var r=this.model.getRowCount();if(typeof e!="number"||e<0||e>=r){console.warn("AG Grid: invalid row index for ensureIndexVisible: "+e);return}var n=e*this.rowHeight,o=n+this.rowHeight,a=this.getGui(),s=a.scrollTop,l=a.offsetHeight,u=s+l,c=s>n,p=u<o;if(c)a.scrollTop=n;else if(p){var h=o-l;a.scrollTop=h}},t.prototype.setComponentCreator=function(e){this.componentCreator=e},t.prototype.getRowHeight=function(){return this.rowHeight},t.prototype.getScrollTop=function(){return this.getGui().scrollTop},t.prototype.setRowHeight=function(e){this.rowHeight=e,this.refresh()},t.prototype.refresh=function(){var e=this;if(!(this.model==null||this.isDestroyed)){var r=this.model.getRowCount();this.eContainer.style.height=r*this.rowHeight+"px",lO(function(){return e.eContainer.clientHeight>=r*e.rowHeight},function(){e.isDestroyed||(e.clearVirtualRows(),e.drawVirtualRows())})}},t.prototype.clearVirtualRows=function(){var e=this;this.renderedRows.forEach(function(r,n){return e.removeRow(n)})},t.prototype.drawVirtualRows=function(){var e=this.getGui(),r=e.scrollTop,n=r+e.offsetHeight,o=Math.floor(r/this.rowHeight),a=Math.floor(n/this.rowHeight);this.ensureRowsRendered(o,a)},t.prototype.ensureRowsRendered=function(e,r){var n=this;this.renderedRows.forEach(function(a,s){(s<e||s>r)&&s!==n.lastFocusedRowIndex&&n.removeRow(s)});for(var o=e;o<=r;o++)this.renderedRows.has(o)||o<this.model.getRowCount()&&this.insertRow(o)},t.prototype.insertRow=function(e){var r=this,n=this.model.getRow(e),o=document.createElement("div");if(o.classList.add("ag-virtual-list-item","ag-"+this.cssIdentifier+"-virtual-list-item"),_r(o,this.ariaRole==="tree"?"treeitem":"option"),kC(o,this.model.getRowCount()),WC(o,e+1),o.setAttribute("tabindex","-1"),typeof this.model.isRowSelected=="function"){var a=this.model.isRowSelected(e);Dl(o,!!a),QO(o,a)}o.style.height=this.rowHeight+"px",o.style.top=this.rowHeight*e+"px";var s=this.componentCreator(n,o);s.addGuiEventListener("focusin",function(){return r.lastFocusedRowIndex=e}),o.appendChild(s.getGui()),this.renderedRows.has(e-1)?this.renderedRows.get(e-1).eDiv.insertAdjacentElement("afterend",o):this.renderedRows.has(e+1)?this.renderedRows.get(e+1).eDiv.insertAdjacentElement("beforebegin",o):this.eContainer.appendChild(o),this.renderedRows.set(e,{rowComponent:s,eDiv:o})},t.prototype.removeRow=function(e){var r=this.renderedRows.get(e);this.eContainer.removeChild(r.eDiv),this.destroyBean(r.rowComponent),this.renderedRows.delete(e)},t.prototype.addScrollListener=function(){var e=this;this.addGuiEventListener("scroll",function(){return e.drawVirtualRows()})},t.prototype.setModel=function(e){this.model=e},t.prototype.destroy=function(){this.isDestroyed||(this.clearVirtualRows(),this.isDestroyed=!0,i.prototype.destroy.call(this))},xp([y("resizeObserverService")],t.prototype,"resizeObserverService",void 0),xp([y("focusService")],t.prototype,"focusService",void 0),xp([I("eContainer")],t.prototype,"eContainer",void 0),xp([B],t.prototype,"postConstruct",null),t}(bs);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var k2=["mouseover","mouseout","mouseenter","mouseleave"],W2=["touchstart","touchend","touchmove","touchcancel"],yR=function(){function i(){this.isOutsideAngular=function(t){return fn(k2,t)}}return i.prototype.setTimeout=function(t,e){window.setTimeout(t,e)},i.prototype.setInterval=function(t,e){return new ct(function(r){r(window.setInterval(t,e))})},i.prototype.addEventListener=function(t,e,r,n){var o=fn(W2,e);t.addEventListener(e,r,{capture:!!n,passive:o})},i.prototype.dispatchEvent=function(t,e,r){e()},i.prototype.frameworkComponent=function(t){return null},i.prototype.isFrameworkComponent=function(t){return!1},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var V2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),bu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},CR=function(i){V2(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getNextCellToFocus=function(e,r,n){return n===void 0&&(n=!1),n?this.getNextCellToFocusWithCtrlPressed(e,r):this.getNextCellToFocusWithoutCtrlPressed(e,r)},t.prototype.getNextCellToFocusWithCtrlPressed=function(e,r){var n=e===j.UP,o=e===j.DOWN,a=e===j.LEFT,s,l;if(n||o)l=n?this.paginationProxy.getPageFirstRow():this.paginationProxy.getPageLastRow(),s=r.column;else{var u=this.columnModel.getAllDisplayedColumns(),c=this.gridOptionsWrapper.isEnableRtl();l=r.rowIndex,s=a!==c?u[0]:rt(u)}return{rowIndex:l,rowPinned:null,column:s}},t.prototype.getNextCellToFocusWithoutCtrlPressed=function(e,r){for(var n=r,o=!1;!o;){switch(e){case j.UP:n=this.getCellAbove(n);break;case j.DOWN:n=this.getCellBelow(n);break;case j.RIGHT:this.gridOptionsWrapper.isEnableRtl()?n=this.getCellToLeft(n):n=this.getCellToRight(n);break;case j.LEFT:this.gridOptionsWrapper.isEnableRtl()?n=this.getCellToRight(n):n=this.getCellToLeft(n);break;default:n=null,console.warn("AG Grid: unknown key for navigation "+e);break}n?o=this.isCellGoodToFocusOn(n):o=!0}return n},t.prototype.isCellGoodToFocusOn=function(e){var r=e.column,n;switch(e.rowPinned){case L.PINNED_TOP:n=this.pinnedRowModel.getPinnedTopRow(e.rowIndex);break;case L.PINNED_BOTTOM:n=this.pinnedRowModel.getPinnedBottomRow(e.rowIndex);break;default:n=this.rowModel.getRow(e.rowIndex);break}if(!n)return!1;var o=r.isSuppressNavigable(n);return!o},t.prototype.getCellToLeft=function(e){if(!e)return null;var r=this.columnModel.getDisplayedColBefore(e.column);return r?{rowIndex:e.rowIndex,column:r,rowPinned:e.rowPinned}:null},t.prototype.getCellToRight=function(e){if(!e)return null;var r=this.columnModel.getDisplayedColAfter(e.column);return r?{rowIndex:e.rowIndex,column:r,rowPinned:e.rowPinned}:null},t.prototype.getRowBelow=function(e){var r=e.rowIndex,n=e.rowPinned;if(this.isLastRowInContainer(e))switch(n){case L.PINNED_BOTTOM:return null;case L.PINNED_TOP:return this.rowModel.isRowsToRender()?{rowIndex:this.paginationProxy.getPageFirstRow(),rowPinned:null}:this.pinnedRowModel.isRowsToRender(L.PINNED_BOTTOM)?{rowIndex:0,rowPinned:L.PINNED_BOTTOM}:null;default:return this.pinnedRowModel.isRowsToRender(L.PINNED_BOTTOM)?{rowIndex:0,rowPinned:L.PINNED_BOTTOM}:null}return{rowIndex:r+1,rowPinned:n}},t.prototype.getCellBelow=function(e){if(!e)return null;var r=this.getRowBelow(e);return r?{rowIndex:r.rowIndex,column:e.column,rowPinned:r.rowPinned}:null},t.prototype.isLastRowInContainer=function(e){var r=e.rowPinned,n=e.rowIndex;if(r===L.PINNED_TOP){var o=this.pinnedRowModel.getPinnedTopRowData().length-1;return o<=n}if(r===L.PINNED_BOTTOM){var a=this.pinnedRowModel.getPinnedBottomRowData().length-1;return a<=n}var s=this.paginationProxy.getPageLastRow();return s<=n},t.prototype.getRowAbove=function(e){var r=e.rowIndex,n=e.rowPinned,o=n?r===0:r===this.paginationProxy.getPageFirstRow();return o?n===L.PINNED_TOP?null:n?this.rowModel.isRowsToRender()?this.getLastBodyCell():this.pinnedRowModel.isRowsToRender(L.PINNED_TOP)?this.getLastFloatingTopRow():null:this.pinnedRowModel.isRowsToRender(L.PINNED_TOP)?this.getLastFloatingTopRow():null:{rowIndex:r-1,rowPinned:n}},t.prototype.getCellAbove=function(e){if(!e)return null;var r=this.getRowAbove({rowIndex:e.rowIndex,rowPinned:e.rowPinned});return r?{rowIndex:r.rowIndex,column:e.column,rowPinned:r.rowPinned}:null},t.prototype.getLastBodyCell=function(){var e=this.paginationProxy.getPageLastRow();return{rowIndex:e,rowPinned:null}},t.prototype.getLastFloatingTopRow=function(){var e=this.pinnedRowModel.getPinnedTopRowData().length-1;return{rowIndex:e,rowPinned:L.PINNED_TOP}},t.prototype.getNextTabbedCell=function(e,r){return r?this.getNextTabbedCellBackwards(e):this.getNextTabbedCellForwards(e)},t.prototype.getNextTabbedCellForwards=function(e){var r=this.columnModel.getAllDisplayedColumns(),n=e.rowIndex,o=e.rowPinned,a=this.columnModel.getDisplayedColAfter(e.column);if(!a){a=r[0];var s=this.getRowBelow(e);if(me(s)||!s.rowPinned&&!this.paginationProxy.isRowInPage(s))return null;n=s?s.rowIndex:null,o=s?s.rowPinned:null}return{rowIndex:n,column:a,rowPinned:o}},t.prototype.getNextTabbedCellBackwards=function(e){var r=this.columnModel.getAllDisplayedColumns(),n=e.rowIndex,o=e.rowPinned,a=this.columnModel.getDisplayedColBefore(e.column);if(!a){a=rt(r);var s=this.getRowAbove({rowIndex:e.rowIndex,rowPinned:e.rowPinned});if(me(s)||!s.rowPinned&&!this.paginationProxy.isRowInPage(s))return null;n=s?s.rowIndex:null,o=s?s.rowPinned:null}return{rowIndex:n,column:a,rowPinned:o}},bu([y("columnModel")],t.prototype,"columnModel",void 0),bu([y("rowModel")],t.prototype,"rowModel",void 0),bu([y("pinnedRowModel")],t.prototype,"pinnedRowModel",void 0),bu([y("paginationProxy")],t.prototype,"paginationProxy",void 0),t=bu([Q("cellNavigationService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var B2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),_u=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},H2=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},wR=function(i){B2(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.consuming=!1,e}return t.prototype.setBeans=function(e){this.logger=e.create("AlignedGridsService")},t.prototype.init=function(){this.addManagedListener(this.eventService,x.EVENT_COLUMN_MOVED,this.fireColumnEvent.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VISIBLE,this.fireColumnEvent.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PINNED,this.fireColumnEvent.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_GROUP_OPENED,this.fireColumnEvent.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_RESIZED,this.fireColumnEvent.bind(this)),this.addManagedListener(this.eventService,x.EVENT_BODY_SCROLL,this.fireScrollEvent.bind(this))},t.prototype.fireEvent=function(e){if(!this.consuming){var r=this.gridOptionsWrapper.getAlignedGrids();r&&r.forEach(function(n){if(n.api){var o=n.api.__getAlignedGridService();e(o)}})}},t.prototype.onEvent=function(e){this.consuming=!0,e(),this.consuming=!1},t.prototype.fireColumnEvent=function(e){this.fireEvent(function(r){r.onColumnEvent(e)})},t.prototype.fireScrollEvent=function(e){e.direction==="horizontal"&&this.fireEvent(function(r){r.onScrollEvent(e)})},t.prototype.onScrollEvent=function(e){var r=this;this.onEvent(function(){var n=r.ctrlsService.getGridBodyCtrl();n.getScrollFeature().setHorizontalScrollPosition(e.left)})},t.prototype.getMasterColumns=function(e){var r=[];return e.columns?e.columns.forEach(function(n){r.push(n)}):e.column&&r.push(e.column),r},t.prototype.getColumnIds=function(e){var r=[];return e.columns?e.columns.forEach(function(n){r.push(n.getColId())}):e.column&&r.push(e.column.getColId()),r},t.prototype.onColumnEvent=function(e){var r=this;this.onEvent(function(){switch(e.type){case x.EVENT_COLUMN_MOVED:case x.EVENT_COLUMN_VISIBLE:case x.EVENT_COLUMN_PINNED:case x.EVENT_COLUMN_RESIZED:var n=e;r.processColumnEvent(n);break;case x.EVENT_COLUMN_GROUP_OPENED:var o=e;r.processGroupOpenedEvent(o);break;case x.EVENT_COLUMN_PIVOT_CHANGED:console.warn("AG Grid: pivoting is not supported with aligned grids. You can only use one of these features at a time in a grid.");break}})},t.prototype.processGroupOpenedEvent=function(e){var r=e.columnGroup,n=null;if(r){var o=r.getGroupId();n=this.columnModel.getProvidedColumnGroup(o)}r&&!n||(this.logger.log("onColumnEvent-> processing "+e+" expanded = "+r.isExpanded()),this.columnModel.setColumnGroupOpened(n,r.isExpanded(),"alignedGridChanged"))},t.prototype.processColumnEvent=function(e){var r=this,n=e.column,o=null;if(n&&(o=this.columnModel.getPrimaryColumn(n.getColId())),!(n&&!o)){var a=this.getMasterColumns(e);switch(e.type){case x.EVENT_COLUMN_MOVED:{var s=e,l=e.columnApi.getColumnState(),u=l.map(function(v){return{colId:v.colId}});this.columnModel.applyColumnState({state:u,applyOrder:!0},"alignedGridChanged"),this.logger.log("onColumnEvent-> processing "+e.type+" toIndex = "+s.toIndex)}break;case x.EVENT_COLUMN_VISIBLE:{var c=e,l=e.columnApi.getColumnState(),u=l.map(function(C){return{colId:C.colId,hide:C.hide}});this.columnModel.applyColumnState({state:u},"alignedGridChanged"),this.logger.log("onColumnEvent-> processing "+e.type+" visible = "+c.visible)}break;case x.EVENT_COLUMN_PINNED:{var p=e,l=e.columnApi.getColumnState(),u=l.map(function(C){return{colId:C.colId,pinned:C.pinned}});this.columnModel.applyColumnState({state:u},"alignedGridChanged"),this.logger.log("onColumnEvent-> processing "+e.type+" pinned = "+p.pinned)}break;case x.EVENT_COLUMN_RESIZED:var h=e;a.forEach(function(v){r.logger.log("onColumnEvent-> processing "+e.type+" actualWidth = "+v.getActualWidth());var m=[{key:v.getColId(),newWidth:v.getActualWidth()}];r.columnModel.setColumnWidths(m,!1,h.finished,"alignedGridChanged")});break}var d=this.ctrlsService.getGridBodyCtrl(),f=d.isVerticalScrollShowing(),g=this.gridOptionsWrapper.getAlignedGrids();g&&g.forEach(function(v){v.api&&v.api.setAlwaysShowVerticalScroll(f)})}},_u([y("columnModel")],t.prototype,"columnModel",void 0),_u([y("ctrlsService")],t.prototype,"ctrlsService",void 0),_u([H2(0,Yt("loggerFactory"))],t.prototype,"setBeans",null),_u([B],t.prototype,"init",null),t=_u([Q("alignedGridsService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var j2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Gs=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},U2=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},bR=function(i){j2(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.setBeans=function(e){this.logger=e.create("selectionService"),this.reset()},t.prototype.init=function(){this.groupSelectsChildren=this.gridOptionsWrapper.isGroupSelectsChildren(),this.addManagedListener(this.eventService,x.EVENT_ROW_SELECTED,this.onRowSelected.bind(this))},t.prototype.setLastSelectedNode=function(e){this.lastSelectedNode=e},t.prototype.getLastSelectedNode=function(){return this.lastSelectedNode},t.prototype.getSelectedNodes=function(){var e=[];return Ke(this.selectedNodes,function(r,n){n&&e.push(n)}),e},t.prototype.getSelectedRows=function(){var e=[];return Ke(this.selectedNodes,function(r,n){n&&n.data&&e.push(n.data)}),e},t.prototype.removeGroupsFromSelection=function(){var e=this;Ke(this.selectedNodes,function(r,n){n&&n.group&&(e.selectedNodes[n.id]=void 0)})},t.prototype.updateGroupsFromChildrenSelections=function(e){if(!!this.gridOptionsWrapper.isGroupSelectsChildren()&&this.rowModel.getType()===L.ROW_MODEL_TYPE_CLIENT_SIDE){var r=this.rowModel,n=r.getRootNode();e||(e=new Jc(!0,n),e.setInactive()),e.forEachChangedNodeDepthFirst(function(o){o!==n&&o.calculateSelectedFromChildren()})}},t.prototype.getNodeForIdIfSelected=function(e){return this.selectedNodes[e]},t.prototype.clearOtherNodes=function(e){var r=this,n={},o=0;return Ke(this.selectedNodes,function(a,s){if(s&&s.id!==e.id){var l=r.selectedNodes[s.id];o+=l.setSelectedParams({newValue:!1,clearSelection:!1,suppressFinishActions:!0}),r.groupSelectsChildren&&s.parent&&(n[s.parent.id]=s.parent)}}),Ke(n,function(a,s){s.calculateSelectedFromChildren()}),o},t.prototype.onRowSelected=function(e){var r=e.node;this.groupSelectsChildren&&r.group||(r.isSelected()?this.selectedNodes[r.id]=r:this.selectedNodes[r.id]=void 0)},t.prototype.syncInRowNode=function(e,r){this.syncInOldRowNode(e,r),this.syncInNewRowNode(e)},t.prototype.syncInOldRowNode=function(e,r){var n=q(r)&&e.id!==r.id;if(n&&r){var o=r.id,a=this.selectedNodes[o]==e;a&&(this.selectedNodes[r.id]=r)}},t.prototype.syncInNewRowNode=function(e){q(this.selectedNodes[e.id])?(e.setSelectedInitialValue(!0),this.selectedNodes[e.id]=e):e.setSelectedInitialValue(!1)},t.prototype.reset=function(){this.logger.log("reset"),this.selectedNodes={},this.lastSelectedNode=null},t.prototype.getBestCostNodeSelection=function(){if(this.rowModel.getType()!==L.ROW_MODEL_TYPE_CLIENT_SIDE){console.warn("AG Grid: `getBestCostNodeSelection` is only available when using normal row model");return}var e=this.rowModel,r=e.getTopLevelNodes();if(r===null){console.warn("AG Grid: `selectAll` not available doing `rowModel=virtual`");return}var n=[];function o(a){for(var s=0,l=a.length;s<l;s++){var u=a[s];if(u.isSelected())n.push(u);else{var c=u;c.group&&c.children&&o(c.children)}}}return o(r),n},t.prototype.setRowModel=function(e){this.rowModel=e},t.prototype.isEmpty=function(){var e=0;return Ke(this.selectedNodes,function(r,n){n&&e++}),e===0},t.prototype.deselectAllRowNodes=function(e){e===void 0&&(e=!1);var r=function(s){return s.selectThisNode(!1)},n=this.rowModel.getType()===L.ROW_MODEL_TYPE_CLIENT_SIDE;if(e){if(!n){console.error("AG Grid: selecting just filtered only works with In Memory Row Model");return}var o=this.rowModel;o.forEachNodeAfterFilter(r)}else Ke(this.selectedNodes,function(s,l){l&&r(l)}),this.reset();n&&this.groupSelectsChildren&&this.updateGroupsFromChildrenSelections();var a={type:x.EVENT_SELECTION_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(a)},t.prototype.selectAllRowNodes=function(e){if(e===void 0&&(e=!1),this.rowModel.getType()!==L.ROW_MODEL_TYPE_CLIENT_SIDE)throw new Error("selectAll only available with normal row model, ie not "+this.rowModel.getType());var r=this.rowModel,n=function(a){return a.selectThisNode(!0)};e?r.forEachNodeAfterFilter(n):r.forEachNode(n),this.rowModel.getType()===L.ROW_MODEL_TYPE_CLIENT_SIDE&&this.groupSelectsChildren&&this.updateGroupsFromChildrenSelections();var o={type:x.EVENT_SELECTION_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(o)},t.prototype.selectNode=function(e,r){e&&e.setSelectedParams({newValue:!0,clearSelection:!r})},t.prototype.deselectIndex=function(e){var r=this.rowModel.getRow(e);this.deselectNode(r)},t.prototype.deselectNode=function(e){e&&e.setSelectedParams({newValue:!1,clearSelection:!1})},t.prototype.selectIndex=function(e,r){var n=this.rowModel.getRow(e);this.selectNode(n,r)},Gs([y("rowModel")],t.prototype,"rowModel",void 0),Gs([y("columnApi")],t.prototype,"columnApi",void 0),Gs([y("gridApi")],t.prototype,"gridApi",void 0),Gs([U2(0,Yt("loggerFactory"))],t.prototype,"setBeans",null),Gs([B],t.prototype,"init",null),t=Gs([Q("selectionService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Lg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},_R=function(){function i(){}return i.prototype.sizeColumnsToFit=function(t){typeof t=="undefined"&&console.error("AG Grid: missing parameter to columnApi.sizeColumnsToFit(gridWidth)"),this.columnModel.sizeColumnsToFit(t,"api")},i.prototype.setColumnGroupOpened=function(t,e){this.columnModel.setColumnGroupOpened(t,e,"api")},i.prototype.getColumnGroup=function(t,e){return this.columnModel.getColumnGroup(t,e)},i.prototype.getProvidedColumnGroup=function(t){return this.columnModel.getProvidedColumnGroup(t)},i.prototype.getDisplayNameForColumn=function(t,e){return this.columnModel.getDisplayNameForColumn(t,e)||""},i.prototype.getDisplayNameForColumnGroup=function(t,e){return this.columnModel.getDisplayNameForColumnGroup(t,e)||""},i.prototype.getColumn=function(t){return this.columnModel.getPrimaryColumn(t)},i.prototype.applyColumnState=function(t){return this.columnModel.applyColumnState(t,"api")},i.prototype.getColumnState=function(){return this.columnModel.getColumnState()},i.prototype.resetColumnState=function(){this.columnModel.resetColumnState("api")},i.prototype.getColumnGroupState=function(){return this.columnModel.getColumnGroupState()},i.prototype.setColumnGroupState=function(t){this.columnModel.setColumnGroupState(t,"api")},i.prototype.resetColumnGroupState=function(){this.columnModel.resetColumnGroupState("api")},i.prototype.isPinning=function(){return this.columnModel.isPinningLeft()||this.columnModel.isPinningRight()},i.prototype.isPinningLeft=function(){return this.columnModel.isPinningLeft()},i.prototype.isPinningRight=function(){return this.columnModel.isPinningRight()},i.prototype.getDisplayedColAfter=function(t){return this.columnModel.getDisplayedColAfter(t)},i.prototype.getDisplayedColBefore=function(t){return this.columnModel.getDisplayedColBefore(t)},i.prototype.setColumnVisible=function(t,e){this.columnModel.setColumnVisible(t,e,"api")},i.prototype.setColumnsVisible=function(t,e){this.columnModel.setColumnsVisible(t,e,"api")},i.prototype.setColumnPinned=function(t,e){this.columnModel.setColumnPinned(t,e,"api")},i.prototype.setColumnsPinned=function(t,e){this.columnModel.setColumnsPinned(t,e,"api")},i.prototype.getAllColumns=function(){return this.columnModel.getAllPrimaryColumns()},i.prototype.getAllGridColumns=function(){return this.columnModel.getAllGridColumns()},i.prototype.getDisplayedLeftColumns=function(){return this.columnModel.getDisplayedLeftColumns()},i.prototype.getDisplayedCenterColumns=function(){return this.columnModel.getDisplayedCenterColumns()},i.prototype.getDisplayedRightColumns=function(){return this.columnModel.getDisplayedRightColumns()},i.prototype.getAllDisplayedColumns=function(){return this.columnModel.getAllDisplayedColumns()},i.prototype.getAllDisplayedVirtualColumns=function(){return this.columnModel.getViewportColumns()},i.prototype.moveColumn=function(t,e){typeof t=="number"?(console.warn("AG Grid: you are using moveColumn(fromIndex, toIndex) - moveColumn takes a column key and a destination index, not two indexes, to move with indexes use moveColumnByIndex(from,to) instead"),this.columnModel.moveColumnByIndex(t,e,"api")):this.columnModel.moveColumn(t,e,"api")},i.prototype.moveColumnByIndex=function(t,e){this.columnModel.moveColumnByIndex(t,e,"api")},i.prototype.moveColumns=function(t,e){this.columnModel.moveColumns(t,e,"api")},i.prototype.moveRowGroupColumn=function(t,e){this.columnModel.moveRowGroupColumn(t,e)},i.prototype.setColumnAggFunc=function(t,e){this.columnModel.setColumnAggFunc(t,e)},i.prototype.setColumnWidth=function(t,e,r,n){r===void 0&&(r=!0),this.columnModel.setColumnWidths([{key:t,newWidth:e}],!1,r,n)},i.prototype.setColumnWidths=function(t,e,r){e===void 0&&(e=!0),this.columnModel.setColumnWidths(t,!1,e,r)},i.prototype.setPivotMode=function(t){this.columnModel.setPivotMode(t)},i.prototype.isPivotMode=function(){return this.columnModel.isPivotMode()},i.prototype.getSecondaryPivotColumn=function(t,e){return this.columnModel.getSecondaryPivotColumn(t,e)},i.prototype.setValueColumns=function(t){this.columnModel.setValueColumns(t,"api")},i.prototype.getValueColumns=function(){return this.columnModel.getValueColumns()},i.prototype.removeValueColumn=function(t){this.columnModel.removeValueColumn(t,"api")},i.prototype.removeValueColumns=function(t){this.columnModel.removeValueColumns(t,"api")},i.prototype.addValueColumn=function(t){this.columnModel.addValueColumn(t,"api")},i.prototype.addValueColumns=function(t){this.columnModel.addValueColumns(t,"api")},i.prototype.setRowGroupColumns=function(t){this.columnModel.setRowGroupColumns(t,"api")},i.prototype.removeRowGroupColumn=function(t){this.columnModel.removeRowGroupColumn(t,"api")},i.prototype.removeRowGroupColumns=function(t){this.columnModel.removeRowGroupColumns(t,"api")},i.prototype.addRowGroupColumn=function(t){this.columnModel.addRowGroupColumn(t,"api")},i.prototype.addRowGroupColumns=function(t){this.columnModel.addRowGroupColumns(t,"api")},i.prototype.getRowGroupColumns=function(){return this.columnModel.getRowGroupColumns()},i.prototype.setPivotColumns=function(t){this.columnModel.setPivotColumns(t,"api")},i.prototype.removePivotColumn=function(t){this.columnModel.removePivotColumn(t,"api")},i.prototype.removePivotColumns=function(t){this.columnModel.removePivotColumns(t,"api")},i.prototype.addPivotColumn=function(t){this.columnModel.addPivotColumn(t,"api")},i.prototype.addPivotColumns=function(t){this.columnModel.addPivotColumns(t,"api")},i.prototype.getPivotColumns=function(){return this.columnModel.getPivotColumns()},i.prototype.getLeftDisplayedColumnGroups=function(){return this.columnModel.getDisplayedTreeLeft()},i.prototype.getCenterDisplayedColumnGroups=function(){return this.columnModel.getDisplayedTreeCentre()},i.prototype.getRightDisplayedColumnGroups=function(){return this.columnModel.getDisplayedTreeRight()},i.prototype.getAllDisplayedColumnGroups=function(){return this.columnModel.getAllDisplayedTrees()},i.prototype.autoSizeColumn=function(t,e){return this.columnModel.autoSizeColumn(t,e,"api")},i.prototype.autoSizeColumns=function(t,e){this.columnModel.autoSizeColumns({columns:t,skipHeader:e})},i.prototype.autoSizeAllColumns=function(t){this.columnModel.autoSizeAllColumns(t,"api")},i.prototype.setSecondaryColumns=function(t){this.columnModel.setSecondaryColumns(t,"api")},i.prototype.getSecondaryColumns=function(){return this.columnModel.getSecondaryColumns()},i.prototype.getPrimaryColumns=function(){return this.columnModel.getAllPrimaryColumns()},i.prototype.cleanDownReferencesToAvoidMemoryLeakInCaseApplicationIsKeepingReferenceToDestroyedGrid=function(){setTimeout(_.removeAllReferences.bind(window,this,"Column API"),100)},i.prototype.columnGroupOpened=function(t,e){console.error("AG Grid: columnGroupOpened no longer exists, use setColumnGroupOpened"),this.setColumnGroupOpened(t,e)},i.prototype.hideColumns=function(t,e){console.error("AG Grid: hideColumns is deprecated, use setColumnsVisible"),this.columnModel.setColumnsVisible(t,!e,"api")},i.prototype.hideColumn=function(t,e){console.error("AG Grid: hideColumn is deprecated, use setColumnVisible"),this.columnModel.setColumnVisible(t,!e,"api")},i.prototype.setState=function(t){return console.error("AG Grid: setState is deprecated, use setColumnState"),this.setColumnState(t)},i.prototype.getState=function(){return console.error("AG Grid: getState is deprecated, use getColumnState"),this.getColumnState()},i.prototype.resetState=function(){console.error("AG Grid: resetState is deprecated, use resetColumnState"),this.resetColumnState()},i.prototype.getAggregationColumns=function(){return console.error("AG Grid: getAggregationColumns is deprecated, use getValueColumns"),this.columnModel.getValueColumns()},i.prototype.removeAggregationColumn=function(t){console.error("AG Grid: removeAggregationColumn is deprecated, use removeValueColumn"),this.columnModel.removeValueColumn(t,"api")},i.prototype.removeAggregationColumns=function(t){console.error("AG Grid: removeAggregationColumns is deprecated, use removeValueColumns"),this.columnModel.removeValueColumns(t,"api")},i.prototype.addAggregationColumn=function(t){console.error("AG Grid: addAggregationColumn is deprecated, use addValueColumn"),this.columnModel.addValueColumn(t,"api")},i.prototype.addAggregationColumns=function(t){console.error("AG Grid: addAggregationColumns is deprecated, use addValueColumns"),this.columnModel.addValueColumns(t,"api")},i.prototype.setColumnAggFunction=function(t,e){console.error("AG Grid: setColumnAggFunction is deprecated, use setColumnAggFunc"),this.columnModel.setColumnAggFunc(t,e,"api")},i.prototype.getDisplayNameForCol=function(t){return console.error("AG Grid: getDisplayNameForCol is deprecated, use getDisplayNameForColumn"),this.getDisplayNameForColumn(t,null)},i.prototype.setColumnState=function(t){return console.error("AG Grid: setColumnState is deprecated, use applyColumnState"),this.columnModel.applyColumnState({state:t,applyOrder:!0},"api")},i.prototype.getOriginalColumnGroup=function(t){return console.error("AG Grid: getOriginalColumnGroup is deprecated, use getProvidedColumnGroup"),this.columnModel.getProvidedColumnGroup(t)},Lg([y("columnModel")],i.prototype,"columnModel",void 0),Lg([nt],i.prototype,"cleanDownReferencesToAvoidMemoryLeakInCaseApplicationIsKeepingReferenceToDestroyedGrid",null),i=Lg([Q("columnApi")],i),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var z2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Su=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},SR=function(i){z2(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.initialised=!1,e}return t.prototype.init=function(){var e=this;this.cellExpressions=this.gridOptionsWrapper.isEnableCellExpressions(),this.initialised=!0,this.eventService.addEventListener(x.EVENT_CELL_VALUE_CHANGED,function(r){return e.callColumnCellValueChangedHandler(r)},this.gridOptionsWrapper.useAsyncEvents())},t.prototype.getValue=function(e,r,n,o){if(n===void 0&&(n=!1),o===void 0&&(o=!1),this.initialised||this.init(),!!r){var a=e.getColDef(),s=a.field,l=e.getId(),u=r.data,c,p=r.groupData&&r.groupData[l]!==void 0,h=!o&&r.aggData&&r.aggData[l]!==void 0;if(n&&a.filterValueGetter?c=this.executeFilterValueGetter(a.filterValueGetter,u,e,r):this.gridOptionsWrapper.isTreeData()&&h?c=r.aggData[l]:this.gridOptionsWrapper.isTreeData()&&a.valueGetter?c=this.executeValueGetter(a.valueGetter,u,e,r):this.gridOptionsWrapper.isTreeData()&&s&&u?c=Uh(u,s,e.isFieldContainsDots()):p?c=r.groupData[l]:h?c=r.aggData[l]:a.valueGetter?c=this.executeValueGetter(a.valueGetter,u,e,r):s&&u&&(c=Uh(u,s,e.isFieldContainsDots())),this.cellExpressions&&typeof c=="string"&&c.indexOf("=")===0){var d=c.substring(1);c=this.executeValueGetter(d,u,e,r)}if(c==null){var f=this.getOpenedGroup(r,e);if(f!=null)return f}return c}},t.prototype.getOpenedGroup=function(e,r){if(!!this.gridOptionsWrapper.isShowOpenedGroup()){var n=r.getColDef();if(!!n.showRowGroup)for(var o=r.getColDef().showRowGroup,a=e.parent;a!=null;){if(a.rowGroupColumn&&(o===!0||o===a.rowGroupColumn.getId()))return a.key;a=a.parent}}},t.prototype.setValue=function(e,r,n,o){var a=this.columnModel.getPrimaryColumn(r);if(!e||!a)return!1;me(e.data)&&(e.data={});var s=a.getColDef(),l=s.field,u=s.newValueHandler,c=s.valueSetter;if(me(l)&&me(u)&&me(c))return console.warn("AG Grid: you need either field or valueSetter set on colDef for editing to work"),!1;var p={node:e,data:e.data,oldValue:this.getValue(a,e),newValue:n,colDef:a.getColDef(),column:a,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()};p.newValue=n;var h;if(u&&q(u)?h=u(p):q(c)?h=this.expressionService.evaluate(c,p):h=this.setValueUsingField(e.data,l,n,a.isFieldContainsDots()),h===void 0&&(h=!0),!h)return!1;e.resetQuickFilterAggregateText(),this.valueCache.onDataChanged(),p.newValue=this.getValue(a,e);var d={type:x.EVENT_CELL_VALUE_CHANGED,event:null,rowIndex:e.rowIndex,rowPinned:e.rowPinned,column:p.column,api:p.api,columnApi:p.columnApi,colDef:p.colDef,context:p.context,data:e.data,node:e,oldValue:p.oldValue,newValue:p.newValue,value:p.newValue,source:o};return this.eventService.dispatchEvent(d),!0},t.prototype.callColumnCellValueChangedHandler=function(e){var r=e.colDef.onCellValueChanged;typeof r=="function"&&r({node:e.node,data:e.data,oldValue:e.oldValue,newValue:e.newValue,colDef:e.colDef,column:e.column,api:e.api,columnApi:e.columnApi,context:e.context})},t.prototype.setValueUsingField=function(e,r,n,o){if(!r)return!1;var a=!1;if(!o)e[r]=n;else for(var s=r.split("."),l=e;s.length>0&&l;){var u=s.shift();s.length===0?l[u]=n:l=l[u]}return!a},t.prototype.executeFilterValueGetter=function(e,r,n,o){var a={data:r,node:o,column:n,colDef:n.getColDef(),api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext(),getValue:this.getValueCallback.bind(this,o)};return this.expressionService.evaluate(e,a)},t.prototype.executeValueGetter=function(e,r,n,o){var a=n.getId(),s=this.valueCache.getValue(o,a);if(s!==void 0)return s;var l={data:r,node:o,column:n,colDef:n.getColDef(),api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext(),getValue:this.getValueCallback.bind(this,o)},u=this.expressionService.evaluate(e,l);return this.valueCache.setValue(o,a,u),u},t.prototype.getValueCallback=function(e,r){var n=this.columnModel.getPrimaryColumn(r);return n?this.getValue(n,e):null},t.prototype.getKeyForNode=function(e,r){var n=this.getValue(e,r),o=e.getColDef().keyCreator,a=n;if(o){var s={value:n,colDef:e.getColDef(),column:e,node:r,data:r.data,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()};a=o(s)}return typeof a=="string"||a==null||(a=String(a),a==="[object Object]"&&Ye(function(){console.warn("AG Grid: a column you are grouping or pivoting by has objects as values. If you want to group by complex objects then either a) use a colDef.keyCreator (se AG Grid docs) or b) to toString() on the object to return a key")},"getKeyForNode - warn about [object,object]")),a},Su([y("expressionService")],t.prototype,"expressionService",void 0),Su([y("columnModel")],t.prototype,"columnModel",void 0),Su([y("valueCache")],t.prototype,"valueCache",void 0),Su([B],t.prototype,"init",null),t=Su([Q("valueService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var $2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),N_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},K2=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},xR=function(i){$2(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.expressionToFunctionCache={},e}return t.prototype.setBeans=function(e){this.logger=e.create("ExpressionService")},t.prototype.evaluate=function(e,r){if(typeof e=="function"){var n=e;return n(r)}else if(typeof e=="string"){var o=e;return this.evaluateExpression(o,r)}else console.error("AG Grid: value should be either a string or a function",e)},t.prototype.evaluateExpression=function(e,r){try{var n=this.createExpressionFunction(e),o=n(r.value,r.context,r.oldValue,r.newValue,r.value,r.node,r.data,r.colDef,r.rowIndex,r.api,r.columnApi,r.getValue,r.column,r.columnGroup);return o}catch(a){return console.log("Processing of the expression failed"),console.log("Expression = "+e),console.log("Params =",r),console.log("Exception = "+a),null}},t.prototype.createExpressionFunction=function(e){if(this.expressionToFunctionCache[e])return this.expressionToFunctionCache[e];var r=this.createFunctionBody(e),n=new Function("x, ctx, oldValue, newValue, value, node, data, colDef, rowIndex, api, columnApi, getValue, column, columnGroup",r);return this.expressionToFunctionCache[e]=n,n},t.prototype.createFunctionBody=function(e){return e.indexOf("return")>=0?e:"return "+e+";"},N_([K2(0,Yt("loggerFactory"))],t.prototype,"setBeans",null),t=N_([Q("expressionService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Y2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),X2=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},TR=function(i){Y2(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.templateCache={},e.waitingCallbacks={},e}return t.prototype.getTemplate=function(e,r){var n=this.templateCache[e];if(n)return n;var o=this.waitingCallbacks[e],a=this;if(!o){o=[],this.waitingCallbacks[e]=o;var s=new XMLHttpRequest;s.onload=function(){a.handleHttpResult(this,e)},s.open("GET",e),s.send()}return r&&o.push(r),null},t.prototype.handleHttpResult=function(e,r){if(e.status!==200||e.response===null){console.warn("AG Grid: Unable to get template error "+e.status+" - "+r);return}this.templateCache[r]=e.response||e.responseText;for(var n=this.waitingCallbacks[r],o=0;o<n.length;o++){var a=n[o];a()}},t=X2([Q("templateService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var q2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),G_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Z2=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},OR=function(i){q2(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.setBeans=function(e){this.logging=e.isDebug()},t.prototype.create=function(e){return new ld(e,this.isLogging.bind(this))},t.prototype.isLogging=function(){return this.logging},G_([Z2(0,Yt("gridOptionsWrapper"))],t.prototype,"setBeans",null),t=G_([Q("loggerFactory")],t),t}(H),ld=function(){function i(t,e){this.name=t,this.isLoggingFunc=e}return i.prototype.isLogging=function(){return this.isLoggingFunc()},i.prototype.log=function(t){this.isLoggingFunc()&&console.log("AG Grid."+this.name+": "+t)},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Q2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),_a=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ER=function(i){Q2(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.setComp=function(e,r,n){var o=this;this.view=e,this.eGridHostDiv=r,this.eGui=n,this.mouseEventService.stampTopLevelGridCompWithGridInstance(r),this.createManagedBean(new ZC(this.view)),this.addRtlSupport(),this.addManagedListener(this,x.EVENT_KEYBOARD_FOCUS,function(){o.view.addOrRemoveKeyboardFocusClass(!0)}),this.addManagedListener(this,x.EVENT_MOUSE_FOCUS,function(){o.view.addOrRemoveKeyboardFocusClass(!1)});var a=this.resizeObserverService.observeResize(this.eGridHostDiv,this.onGridSizeChanged.bind(this));this.addDestroyFunc(function(){return a()}),this.ctrlsService.registerGridCtrl(this)},t.prototype.isDetailGrid=function(){var e,r,n=this.focusService.findTabbableParent(this.getGui());return((r=(e=n)===null||e===void 0?void 0:e.getAttribute("row-id"))===null||r===void 0?void 0:r.startsWith("detail"))||!1},t.prototype.showDropZones=function(){return ge.isRegistered(oe.RowGroupingModule)},t.prototype.showSideBar=function(){return ge.isRegistered(oe.SideBarModule)},t.prototype.showStatusBar=function(){return ge.isRegistered(oe.StatusBarModule)},t.prototype.showWatermark=function(){return ge.isRegistered(oe.EnterpriseCoreModule)},t.prototype.onGridSizeChanged=function(){var e={type:x.EVENT_GRID_SIZE_CHANGED,api:this.gridApi,columnApi:this.columnApi,clientWidth:this.eGridHostDiv.clientWidth,clientHeight:this.eGridHostDiv.clientHeight};this.eventService.dispatchEvent(e)},t.prototype.addRtlSupport=function(){var e=this.gridOptionsWrapper.isEnableRtl()?"ag-rtl":"ag-ltr";this.view.setRtlClass(e)},t.prototype.destroyGridUi=function(){this.view.destroyGridUi()},t.prototype.getGui=function(){return this.eGui},t.prototype.setResizeCursor=function(e){this.view.setCursor(e?"ew-resize":null)},t.prototype.disableUserSelect=function(e){this.view.setUserSelect(e?"none":null)},t.prototype.focusNextInnerContainer=function(e){var r=this.gridOptionsWrapper.getDocument(),n=this.view.getFocusableContainers(),o=n.findIndex(function(s){return s.contains(r.activeElement)}),a=o+(e?-1:1);return a<=0||a>=n.length?!1:this.focusService.focusInto(n[a])},t.prototype.focusInnerElement=function(e){var r=this.view.getFocusableContainers();if(e){if(r.length>1)return this.focusService.focusInto(rt(r),!0);var n=rt(this.columnModel.getAllDisplayedColumns());if(this.focusService.focusGridView(n,!0))return!0}return this.focusService.focusFirstHeader()},t.prototype.forceFocusOutOfContainer=function(e){e===void 0&&(e=!1),this.view.forceFocusOutOfContainer(e)},_a([y("columnApi")],t.prototype,"columnApi",void 0),_a([y("gridApi")],t.prototype,"gridApi",void 0),_a([y("focusService")],t.prototype,"focusService",void 0),_a([y("resizeObserverService")],t.prototype,"resizeObserverService",void 0),_a([y("columnModel")],t.prototype,"columnModel",void 0),_a([y("ctrlsService")],t.prototype,"ctrlsService",void 0),_a([y("mouseEventService")],t.prototype,"mouseEventService",void 0),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var J2=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),xu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},RR=function(i){J2(t,i);function t(e){var r=i.call(this,void 0)||this;return r.eGridDiv=e,r}return t.prototype.postConstruct=function(){var e=this;this.logger=this.loggerFactory.create("GridComp");var r={destroyGridUi:function(){return e.destroyBean(e)},setRtlClass:function(o){return e.addCssClass(o)},addOrRemoveKeyboardFocusClass:function(o){return e.addOrRemoveCssClass(Sl.AG_KEYBOARD_FOCUS,o)},forceFocusOutOfContainer:this.forceFocusOutOfContainer.bind(this),updateLayoutClasses:this.updateLayoutClasses.bind(this),getFocusableContainers:this.getFocusableContainers.bind(this),setUserSelect:function(o){e.getGui().style.userSelect=o!=null?o:"",e.getGui().style.webkitUserSelect=o!=null?o:""},setCursor:function(o){e.getGui().style.cursor=o!=null?o:""}};this.ctrl=this.createManagedBean(new ER);var n=this.createTemplate();this.setTemplate(n),this.ctrl.setComp(r,this.eGridDiv,this.getGui()),this.insertGridIntoDom(),this.initialiseTabGuard({onTabKeyDown:function(){},focusInnerElement:function(o){return e.ctrl.focusInnerElement(o)}})},t.prototype.insertGridIntoDom=function(){var e=this,r=this.getGui();this.eGridDiv.appendChild(r),this.addDestroyFunc(function(){e.eGridDiv.removeChild(r),e.logger.log("Grid removed from DOM")})},t.prototype.updateLayoutClasses=function(e,r){var n=this.eRootWrapperBody.classList;n.toggle(Rt.AUTO_HEIGHT,r.autoHeight),n.toggle(Rt.NORMAL,r.normal),n.toggle(Rt.PRINT,r.print),this.addOrRemoveCssClass(Rt.AUTO_HEIGHT,r.autoHeight),this.addOrRemoveCssClass(Rt.NORMAL,r.normal),this.addOrRemoveCssClass(Rt.PRINT,r.print)},t.prototype.createTemplate=function(){var e=this.ctrl.showDropZones()?"<ag-grid-header-drop-zones></ag-grid-header-drop-zones>":"",r=this.ctrl.showSideBar()?'<ag-side-bar ref="sideBar"></ag-side-bar>':"",n=this.ctrl.showStatusBar()?'<ag-status-bar ref="statusBar"></ag-status-bar>':"",o=this.ctrl.showWatermark()?"<ag-watermark></ag-watermark>":"",a=`<div class="ag-root-wrapper">
                `+e+`
                <div class="ag-root-wrapper-body" ref="rootWrapperBody">
                    <ag-grid-body ref="gridBody"></ag-grid-body>
                    `+r+`
                </div>
                `+n+`
                <ag-pagination></ag-pagination>
                `+o+`
            </div>`;return a},t.prototype.getFocusableElement=function(){return this.eRootWrapperBody},t.prototype.getFocusableContainers=function(){var e=[this.gridBodyComp.getGui()];return this.sideBarComp&&e.push(this.sideBarComp.getGui()),e.filter(function(r){return jl(r)})},xu([y("loggerFactory")],t.prototype,"loggerFactory",void 0),xu([I("gridBody")],t.prototype,"gridBodyComp",void 0),xu([I("sideBar")],t.prototype,"sideBarComp",void 0),xu([I("rootWrapperBody")],t.prototype,"eRootWrapperBody",void 0),xu([B],t.prototype,"postConstruct",null),t}(bs);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var eW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Tp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},PR=function(i){eW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}e=t,t.prototype.progressSort=function(r,n,o){var a=this.getNextSortDirection(r);this.setSortForColumn(r,a,n,o)},t.prototype.setSortForColumn=function(r,n,o,a){n!==L.SORT_ASC&&n!==L.SORT_DESC&&(n=null),r.setSort(n,a);var s=(o||this.gridOptionsWrapper.isAlwaysMultiSort())&&!this.gridOptionsWrapper.isSuppressMultiSort();s||this.clearSortBarThisColumn(r,a),this.updateSortIndex(r),this.dispatchSortChangedEvents(a)},t.prototype.updateSortIndex=function(r){var n=this.getColumnsWithSortingOrdered(),o=0;n.forEach(function(s){s!==r&&(s.setSortIndex(o),o++)}),r.getSort()&&r.setSortIndex(o);var a=this.columnModel.getPrimaryAndSecondaryAndAutoColumns();a.filter(function(s){return s.getSort()==null}).forEach(function(s){return s.setSortIndex()})},t.prototype.onSortChanged=function(r){this.dispatchSortChangedEvents(r)},t.prototype.isSortActive=function(){var r=this.columnModel.getPrimaryAndSecondaryAndAutoColumns(),n=r.filter(function(o){return!!o.getSort()});return n&&n.length>0},t.prototype.dispatchSortChangedEvents=function(r){var n={type:x.EVENT_SORT_CHANGED,api:this.gridApi,columnApi:this.columnApi,source:r};this.eventService.dispatchEvent(n)},t.prototype.clearSortBarThisColumn=function(r,n){this.columnModel.getPrimaryAndSecondaryAndAutoColumns().forEach(function(o){o!==r&&o.setSort(void 0,n)})},t.prototype.getNextSortDirection=function(r){var n;if(r.getColDef().sortingOrder?n=r.getColDef().sortingOrder:this.gridOptionsWrapper.getSortingOrder()?n=this.gridOptionsWrapper.getSortingOrder():n=e.DEFAULT_SORTING_ORDER,!Array.isArray(n)||n.length<=0)return console.warn("AG Grid: sortingOrder must be an array with at least one element, currently it's "+n),null;var o=n.indexOf(r.getSort()),a=o<0,s=o==n.length-1,l;return a||s?l=n[0]:l=n[o+1],e.DEFAULT_SORTING_ORDER.indexOf(l)<0?(console.warn("AG Grid: invalid sort type "+l),null):l},t.prototype.getColumnsWithSortingOrdered=function(){var r=this.columnModel.getPrimaryAndSecondaryAndAutoColumns(),n=r.filter(function(a){return!!a.getSort()}),o={};return r.forEach(function(a,s){return o[a.getId()]=s}),n.sort(function(a,s){var l=a.getSortIndex(),u=s.getSortIndex();if(l!=null&&u!=null)return l-u;if(l==null&&u==null){var c=o[a.getId()],p=o[s.getId()];return c>p?1:-1}else return u==null?-1:1}),n},t.prototype.getSortModel=function(){return this.getColumnsWithSortingOrdered().map(function(r){return{sort:r.getSort(),colId:r.getId()}})},t.prototype.getSortOptions=function(){return this.getColumnsWithSortingOrdered().map(function(r){return{sort:r.getSort(),column:r}})};var e;return t.DEFAULT_SORTING_ORDER=[L.SORT_ASC,L.SORT_DESC,null],Tp([y("columnModel")],t.prototype,"columnModel",void 0),Tp([y("columnApi")],t.prototype,"columnApi",void 0),Tp([y("gridApi")],t.prototype,"gridApi",void 0),t=e=Tp([Q("sortController")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var tW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ng=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},rW=function(i){tW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.setMouseOver=function(e){this.selectedColumns=e;var r={type:x.EVENT_COLUMN_HOVER_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(r)},t.prototype.clearMouseOver=function(){this.selectedColumns=null;var e={type:x.EVENT_COLUMN_HOVER_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(e)},t.prototype.isHovered=function(e){return!!this.selectedColumns&&this.selectedColumns.indexOf(e)>=0},Ng([y("columnApi")],t.prototype,"columnApi",void 0),Ng([y("gridApi")],t.prototype,"gridApi",void 0),t=Ng([Q("columnHoverService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var nW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Gg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},iW=function(i){nW(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.executeNextFuncs=[],e.executeLaterFuncs=[],e.active=!1,e.animationThreadCount=0,e}return t.prototype.postConstruct=function(){var e=this;this.ctrlsService.whenReady(function(r){return e.gridBodyCtrl=r.gridBodyCtrl})},t.prototype.isActive=function(){return this.active},t.prototype.start=function(){this.active||this.gridOptionsWrapper.isSuppressColumnMoveAnimation()||this.gridOptionsWrapper.isEnableRtl()||(this.ensureAnimationCssClassPresent(),this.active=!0)},t.prototype.finish=function(){!this.active||(this.flush(),this.active=!1)},t.prototype.executeNextVMTurn=function(e){this.active?this.executeNextFuncs.push(e):e()},t.prototype.executeLaterVMTurn=function(e){this.active?this.executeLaterFuncs.push(e):e()},t.prototype.ensureAnimationCssClassPresent=function(){var e=this;this.animationThreadCount++;var r=this.animationThreadCount;this.gridBodyCtrl.setColumnMovingCss(!0),this.executeLaterFuncs.push(function(){e.animationThreadCount===r&&e.gridBodyCtrl.setColumnMovingCss(!1)})},t.prototype.flush=function(){var e=this.executeNextFuncs;this.executeNextFuncs=[];var r=this.executeLaterFuncs;this.executeLaterFuncs=[],!(e.length===0&&r.length===0)&&(window.setTimeout(function(){return e.forEach(function(n){return n()})},0),window.setTimeout(function(){return r.forEach(function(n){return n()})},300))},Gg([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Gg([B],t.prototype,"postConstruct",null),t=Gg([Q("columnAnimationService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var oW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),kg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},aW=function(i){oW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}e=t,t.prototype.createAutoGroupColumns=function(r,n){var o=this,a=[],s=this.gridOptionsWrapper.isTreeData(),l=this.gridOptionsWrapper.isGroupMultiAutoColumn();return s&&l&&(console.warn("AG Grid: you cannot mix groupMultiAutoColumn with treeData, only one column can be used to display groups when doing tree data"),l=!1),l?n.forEach(function(u,c){a.push(o.createOneAutoGroupColumn(r,u,c))}):a.push(this.createOneAutoGroupColumn(r)),a},t.prototype.createOneAutoGroupColumn=function(r,n,o){var a=this.generateDefaultColDef(n),s;n?s=L.GROUP_AUTO_COLUMN_ID+"-"+n.getId():s=e.GROUP_AUTO_COLUMN_BUNDLE_ID;var l=this.gridOptionsWrapper.getAutoGroupColumnDef();if(Ln(a,l),a=this.columnFactory.mergeColDefs(a),a.colId=s,!this.gridOptionsWrapper.isTreeData()){var u=me(a.field)&&me(a.valueGetter)&&me(a.filterValueGetter);u&&(a.filter=!1)}o&&o>0&&(a.headerCheckboxSelection=!1);var c=r.find(function(h){return h.getId()==s});if(c)return c.setColDef(a,null),this.columnFactory.applyColumnState(c,a),c;var p=new Se(a,null,s,!0);return this.context.createBean(p),p},t.prototype.generateDefaultColDef=function(r){var n=this.gridOptionsWrapper.getAutoGroupColumnDef(),o=this.gridOptionsWrapper.getLocaleTextFunc(),a={headerName:o("group","Group")},s=n&&(n.cellRenderer||n.cellRendererFramework||n.cellRendererSelector);if(s||(a.cellRenderer="agGroupCellRenderer"),r){var l=r.getColDef();Object.assign(a,{headerName:this.columnModel.getDisplayNameForColumn(r,"header"),headerValueGetter:l.headerValueGetter}),(l.cellRenderer||l.cellRendererFramework)&&Object.assign(a,{cellRendererParams:{innerRenderer:l.cellRenderer,innerRendererFramework:l.cellRendererFramework,innerRendererParams:l.cellRendererParams}}),a.showRowGroup=r.getColId()}else a.showRowGroup=!0;return a};var e;return t.GROUP_AUTO_COLUMN_BUNDLE_ID=L.GROUP_AUTO_COLUMN_ID,kg([y("columnModel")],t.prototype,"columnModel",void 0),kg([y("columnFactory")],t.prototype,"columnFactory",void 0),t=e=kg([Q("autoGroupColService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var sW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Wg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},lW=function(i){sW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){var e=this;this.ctrlsService.whenReady(function(r){e.centerRowContainerCon=r.centerRowContainerCtrl,e.addManagedListener(e.eventService,x.EVENT_BODY_HEIGHT_CHANGED,e.onBodyHeightChanged.bind(e)),e.addManagedListener(e.eventService,x.EVENT_SCROLL_VISIBILITY_CHANGED,e.onScrollVisibilityChanged.bind(e)),e.checkPageSize()})},t.prototype.notActive=function(){return!this.gridOptionsWrapper.isPaginationAutoPageSize()},t.prototype.onScrollVisibilityChanged=function(){this.checkPageSize()},t.prototype.onBodyHeightChanged=function(){this.checkPageSize()},t.prototype.checkPageSize=function(){if(!this.notActive()){var e=this.gridOptionsWrapper.getRowHeightAsNumber(),r=this.centerRowContainerCon.getViewportSizeFeature().getBodyHeight();if(r>0){var n=Math.floor(r/e);this.gridOptionsWrapper.setProperty("paginationPageSize",n)}}},Wg([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Wg([B],t.prototype,"postConstruct",null),t=Wg([Q("paginationAutoPageSizeService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var uW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),k_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},AR=function(i){uW(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.cacheVersion=0,e}return t.prototype.init=function(){this.active=this.gridOptionsWrapper.isValueCache(),this.neverExpires=this.gridOptionsWrapper.isValueCacheNeverExpires()},t.prototype.onDataChanged=function(){this.neverExpires||this.expire()},t.prototype.expire=function(){this.cacheVersion++},t.prototype.setValue=function(e,r,n){this.active&&(e.__cacheVersion!==this.cacheVersion&&(e.__cacheVersion=this.cacheVersion,e.__cacheData={}),e.__cacheData[r]=n)},t.prototype.getValue=function(e,r){if(!(!this.active||e.__cacheVersion!==this.cacheVersion))return e.__cacheData[r]},k_([B],t.prototype,"init",null),t=k_([Q("valueCache")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var cW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Op=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},pW=function(i){cW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.init=function(){this.rowModel.getType()===L.ROW_MODEL_TYPE_CLIENT_SIDE&&(this.clientSideRowModel=this.rowModel),this.addManagedListener(this.eventService,x.EVENT_CELL_VALUE_CHANGED,this.onCellValueChanged.bind(this))},t.prototype.onCellValueChanged=function(e){e.source!==L.SOURCE_PASTE&&this.doChangeDetection(e.node,e.column)},t.prototype.doChangeDetection=function(e,r){if(!this.gridOptionsWrapper.isSuppressChangeDetection()){if(this.clientSideRowModel&&!e.isRowPinned()){var n=this.gridOptionsWrapper.isAggregateOnlyChangedColumns(),o=new Jc(n,this.clientSideRowModel.getRootNode());o.addParentNode(e.parent,[r]),this.clientSideRowModel.doAggregate(o)}this.rowRenderer.refreshCells()}},Op([y("rowModel")],t.prototype,"rowModel",void 0),Op([y("rowRenderer")],t.prototype,"rowRenderer",void 0),Op([B],t.prototype,"init",null),t=Op([Q("changeDetectionService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var hW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),W_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},dW=function(i){hW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.adaptFunction=function(e,r){var n=this.componentMetadataProvider.retrieve(e);return n&&n.functionAdapter?n.functionAdapter(r):null},t.prototype.adaptCellRendererFunction=function(e){var r=function(){function n(){}return n.prototype.refresh=function(o){return!1},n.prototype.getGui=function(){return this.eGui},n.prototype.init=function(o){var a=e(o),s=typeof a;if(s==="string"||s==="number"||s==="boolean"){this.eGui=Kt("<span>"+a+"</span>");return}if(a==null){this.eGui=Kt("<span></span>");return}this.eGui=a},n}();return r},t.prototype.doesImplementIComponent=function(e){return e?e.prototype&&"getGui"in e.prototype:!1},W_([y("componentMetadataProvider")],t.prototype,"componentMetadataProvider",void 0),t=W_([Q("agComponentUtils")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var fW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Vg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},gW=function(i){fW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){this.componentMetaData={dateComponent:{mandatoryMethodList:["getDate","setDate"],optionalMethodList:["afterGuiAttached","setInputPlaceholder","setInputAriaLabel"]},detailCellRenderer:{mandatoryMethodList:[],optionalMethodList:["refresh"],functionAdapter:this.agComponentUtils.adaptCellRendererFunction.bind(this.agComponentUtils)},headerComponent:{mandatoryMethodList:[],optionalMethodList:["refresh"]},headerGroupComponent:{mandatoryMethodList:[],optionalMethodList:[]},loadingCellRenderer:{mandatoryMethodList:[],optionalMethodList:[]},loadingOverlayComponent:{mandatoryMethodList:[],optionalMethodList:[]},noRowsOverlayComponent:{mandatoryMethodList:[],optionalMethodList:[]},floatingFilterComponent:{mandatoryMethodList:["onParentModelChanged"],optionalMethodList:["afterGuiAttached"]},floatingFilterWrapperComponent:{mandatoryMethodList:[],optionalMethodList:[]},cellRenderer:{mandatoryMethodList:[],optionalMethodList:["refresh","afterGuiAttached"],functionAdapter:this.agComponentUtils.adaptCellRendererFunction.bind(this.agComponentUtils)},cellEditor:{mandatoryMethodList:["getValue"],optionalMethodList:["isPopup","isCancelBeforeStart","isCancelAfterEnd","getPopupPosition","focusIn","focusOut","afterGuiAttached"]},innerRenderer:{mandatoryMethodList:[],optionalMethodList:["afterGuiAttached"],functionAdapter:this.agComponentUtils.adaptCellRendererFunction.bind(this.agComponentUtils)},fullWidthCellRenderer:{mandatoryMethodList:[],optionalMethodList:["refresh","afterGuiAttached"],functionAdapter:this.agComponentUtils.adaptCellRendererFunction.bind(this.agComponentUtils)},pinnedRowCellRenderer:{mandatoryMethodList:[],optionalMethodList:["refresh","afterGuiAttached"],functionAdapter:this.agComponentUtils.adaptCellRendererFunction.bind(this.agComponentUtils)},groupRowRenderer:{mandatoryMethodList:[],optionalMethodList:["afterGuiAttached"],functionAdapter:this.agComponentUtils.adaptCellRendererFunction.bind(this.agComponentUtils)},filter:{mandatoryMethodList:["isFilterActive","doesFilterPass","getModel","setModel"],optionalMethodList:["afterGuiAttached","onNewRowsLoaded","getModelAsString","onFloatingFilterChanged"]},filterComponent:{mandatoryMethodList:["isFilterActive","doesFilterPass","getModel","setModel"],optionalMethodList:["afterGuiAttached","onNewRowsLoaded","getModelAsString","onFloatingFilterChanged"]},statusPanel:{mandatoryMethodList:[],optionalMethodList:["afterGuiAttached"]},toolPanel:{mandatoryMethodList:[],optionalMethodList:["refresh","afterGuiAttached"]},tooltipComponent:{mandatoryMethodList:[],optionalMethodList:[]}}},t.prototype.retrieve=function(e){return this.componentMetaData[e]},Vg([y("agComponentUtils")],t.prototype,"agComponentUtils",void 0),Vg([B],t.prototype,"postConstruct",null),t=Vg([Q("componentMetadataProvider")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var vW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),V_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Bg=8,mW=4,Hg=4,jg=6,yW={"ag-theme-custom":{headerHeight:25,headerCellMinWidth:24,listItemHeight:mW*5,rowHeight:25,chartMenuPanelWidth:220},"ag-theme-material":{headerHeight:Bg*7,headerCellMinWidth:48,listItemHeight:Bg*4,rowHeight:Bg*6,chartMenuPanelWidth:240},"ag-theme-balham":{headerHeight:Hg*8,headerCellMinWidth:24,listItemHeight:Hg*6,rowHeight:Hg*7,chartMenuPanelWidth:220},"ag-theme-alpine":{headerHeight:jg*8,headerCellMinWidth:36,listItemHeight:jg*4,rowHeight:jg*7,chartMenuPanelWidth:240}},B_={headerHeight:["ag-header-row"],headerCellMinWidth:["ag-header-cell"],listItemHeight:["ag-virtual-list-item"],rowHeight:["ag-row"],chartMenuPanelWidth:["ag-chart-docked-container"]},Tu={},DR=function(i){vW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getSassVariable=function(e,r){var n="ag-theme-"+(e.match("material")?"material":e.match("balham")?"balham":e.match("alpine")?"alpine":"custom"),o=yW[n][r],a=0;Tu[e]||(Tu[e]={});var s=Tu[e][r];if(s!=null)return s;if(B_[r]){var l=B_[r],u=document.createElement("div");u.classList.add(e),u.style.position="absolute";var c=l.reduce(function(h,d){var f=document.createElement("div");return f.style.position="static",f.classList.add(d),h.appendChild(f),f},u);if(document.body){document.body.appendChild(u);var p=r.toLowerCase().indexOf("height")!==-1?"height":"width";a=parseInt(window.getComputedStyle(c)[p],10),document.body.removeChild(u)}}return Tu[e][r]=a||o,Tu[e][r]},t.prototype.isThemeDark=function(){var e=this.getTheme().theme;return!!e&&e.indexOf("dark")>=0},t.prototype.chartMenuPanelWidth=function(){var e=this.getTheme().themeFamily;return this.getSassVariable(e,"chartMenuPanelWidth")},t.prototype.getTheme=function(){for(var e=/\bag-(material|(?:theme-([\w\-]*)))\b/,r=this.eGridDiv,n=null;r&&(n=e.exec(r.className),!n);)r=r.parentElement||void 0;if(!n)return{};var o=n[0],a=n[2]===void 0;if(a){var s=o.replace("ag-","ag-theme-");Ye(function(){return console.warn("AG Grid: As of v19 old theme are no longer provided. Please replace "+o+" with "+s+".")},"using-old-theme")}return{theme:o,el:r,themeFamily:o.replace(/-dark$/,"")}},V_([y("eGridDiv")],t.prototype,"eGridDiv",void 0),t=V_([Q("environment")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var CW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ep=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},wW=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},bW=function(i){CW(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.scrollY=0,e.uiBodyHeight=0,e}return t.prototype.agWire=function(e){this.logger=e.create("RowContainerHeightService")},t.prototype.postConstruct=function(){this.addManagedListener(this.eventService,x.EVENT_BODY_HEIGHT_CHANGED,this.updateOffset.bind(this)),this.maxDivHeight=TO(),this.logger.log("maxDivHeight = "+this.maxDivHeight)},t.prototype.isStretching=function(){return this.stretching},t.prototype.getDivStretchOffset=function(){return this.divStretchOffset},t.prototype.updateOffset=function(){if(!!this.stretching){var e=this.ctrlsService.getGridBodyCtrl(),r=e.getScrollFeature().getVScrollPosition().top,n=this.getUiBodyHeight(),o=r!==this.scrollY||n!==this.uiBodyHeight;o&&(this.scrollY=r,this.uiBodyHeight=n,this.calculateOffset())}},t.prototype.calculateOffset=function(){this.setUiContainerHeight(this.maxDivHeight),this.pixelsToShave=this.modelHeight-this.uiContainerHeight,this.maxScrollY=this.uiContainerHeight-this.uiBodyHeight;var e=this.scrollY/this.maxScrollY,r=e*this.pixelsToShave;this.logger.log("Div Stretch Offset = "+r+" ("+this.pixelsToShave+" * "+e+")"),this.setDivStretchOffset(r)},t.prototype.setUiContainerHeight=function(e){e!==this.uiContainerHeight&&(this.uiContainerHeight=e,this.eventService.dispatchEvent({type:x.EVENT_ROW_CONTAINER_HEIGHT_CHANGED}))},t.prototype.clearOffset=function(){this.setUiContainerHeight(this.modelHeight),this.pixelsToShave=0,this.setDivStretchOffset(0)},t.prototype.setDivStretchOffset=function(e){var r=typeof e=="number"?Math.floor(e):null;this.divStretchOffset!==r&&(this.divStretchOffset=r,this.eventService.dispatchEvent({type:x.EVENT_HEIGHT_SCALE_CHANGED}))},t.prototype.setModelHeight=function(e){this.modelHeight=e,this.stretching=e!=null&&this.maxDivHeight>0&&e>this.maxDivHeight,this.stretching?this.calculateOffset():this.clearOffset()},t.prototype.getUiContainerHeight=function(){return this.uiContainerHeight},t.prototype.getRealPixelPosition=function(e){return e-this.divStretchOffset},t.prototype.getUiBodyHeight=function(){var e=this.ctrlsService.getGridBodyCtrl(),r=e.getScrollFeature().getVScrollPosition();return r.bottom-r.top},t.prototype.getScrollPositionForPixel=function(e){if(this.pixelsToShave<=0)return e;var r=this.modelHeight-this.getUiBodyHeight(),n=e/r,o=this.maxScrollY*n;return o},Ep([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Ep([wW(0,Yt("loggerFactory"))],t.prototype,"agWire",null),Ep([B],t.prototype,"postConstruct",null),t=Ep([Q("rowContainerHeightService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var _W=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),H_=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},MR=function(i){_W(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.init=function(){this.groupSelectsChildren=this.gridOptionsWrapper.isGroupSelectsChildren(),this.isRowSelectableFunc=this.gridOptionsWrapper.getIsRowSelectableFunc()},t.prototype.updateSelectableAfterGrouping=function(e){if(this.isRowSelectableFunc){var r=function(n){return n.childrenAfterGroup};this.recurseDown(e.childrenAfterGroup,r)}},t.prototype.recurseDown=function(e,r){var n=this;!e||e.forEach(function(o){if(!!o.group){o.hasChildren()&&n.recurseDown(r(o),r);var a;if(n.groupSelectsChildren){var s=(r(o)||[]).find(function(l){return l.selectable===!0});a=q(s)}else a=n.isRowSelectableFunc?n.isRowSelectableFunc(o):!1;o.setRowSelectable(a)}})},H_([B],t.prototype,"init",null),t=H_([Q("selectableService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var SW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),bn=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},xW=function(i){SW(t,i);function t(){var e=i.call(this)||this;return e.previousAndFirstButtonsDisabled=!1,e.nextButtonDisabled=!1,e.lastButtonDisabled=!1,e}return t.prototype.postConstruct=function(){var e=this,r=this.gridOptionsWrapper.isEnableRtl();this.setTemplate(this.getTemplate()),this.btFirst.insertAdjacentElement("afterbegin",Ft(r?"last":"first",this.gridOptionsWrapper)),this.btPrevious.insertAdjacentElement("afterbegin",Ft(r?"next":"previous",this.gridOptionsWrapper)),this.btNext.insertAdjacentElement("afterbegin",Ft(r?"previous":"next",this.gridOptionsWrapper)),this.btLast.insertAdjacentElement("afterbegin",Ft(r?"first":"last",this.gridOptionsWrapper));var n=this.gridOptionsWrapper.isPagination(),o=n&&!this.gridOptionsWrapper.isSuppressPaginationPanel();if(!o){this.setDisplayed(!1);return}this.addManagedListener(this.eventService,x.EVENT_PAGINATION_CHANGED,this.onPaginationChanged.bind(this)),[{el:this.btFirst,fn:this.onBtFirst.bind(this)},{el:this.btPrevious,fn:this.onBtPrevious.bind(this)},{el:this.btNext,fn:this.onBtNext.bind(this)},{el:this.btLast,fn:this.onBtLast.bind(this)}].forEach(function(a){var s=a.el,l=a.fn;e.addManagedListener(s,"click",l),e.addManagedListener(s,"keydown",function(u){(u.key===j.ENTER||u.key===j.SPACE)&&(u.preventDefault(),l())})}),this.onPaginationChanged()},t.prototype.onPaginationChanged=function(){this.enableOrDisableButtons(),this.updateRowLabels(),this.setCurrentPageLabel(),this.setTotalLabels()},t.prototype.onBtFirst=function(){this.previousAndFirstButtonsDisabled||this.paginationProxy.goToFirstPage()},t.prototype.setCurrentPageLabel=function(){var e=this.paginationProxy.getTotalPages()>0,r=this.paginationProxy.getCurrentPage(),n=e?r+1:0;this.lbCurrent.innerHTML=this.formatNumber(n)},t.prototype.formatNumber=function(e){var r=this.gridOptionsWrapper.getPaginationNumberFormatterFunc();if(r){var n={value:e};return r(n)}var o=this.gridOptionsWrapper.getLocaleTextFunc(),a=o("thousandSeparator",","),s=o("decimalSeparator",".");return HC(e,a,s)},t.prototype.getTemplate=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("page","Page"),n=e("to","to"),o=e("of","of"),a=e("firstPage","First Page"),s=e("previousPage","Previous Page"),l=e("nextPage","Next Page"),u=e("lastPage","Last Page"),c=this.getCompId();return'<div class="ag-paging-panel ag-unselectable" id="ag-'+c+`">
                <span class="ag-paging-row-summary-panel" role="status">
                    <span id="ag-`+c+`-first-row" ref="lbFirstRowOnPage" class="ag-paging-row-summary-panel-number"></span>
                    <span id="ag-`+c+'-to">'+n+`</span>
                    <span id="ag-`+c+`-last-row" ref="lbLastRowOnPage" class="ag-paging-row-summary-panel-number"></span>
                    <span id="ag-`+c+'-of">'+o+`</span>
                    <span id="ag-`+c+`-row-count" ref="lbRecordCount" class="ag-paging-row-summary-panel-number"></span>
                </span>
                <span class="ag-paging-page-summary-panel" role="presentation">
                    <div ref="btFirst" class="ag-paging-button" role="button" aria-label="`+a+`"></div>
                    <div ref="btPrevious" class="ag-paging-button" role="button" aria-label="`+s+`"></div>
                    <span class="ag-paging-description" role="status">
                        <span id="ag-`+c+'-start-page">'+r+`</span>
                        <span id="ag-`+c+`-start-page-number" ref="lbCurrent" class="ag-paging-number"></span>
                        <span id="ag-`+c+'-of-page">'+o+`</span>
                        <span id="ag-`+c+`-of-page-number" ref="lbTotal" class="ag-paging-number"></span>
                    </span>
                    <div ref="btNext" class="ag-paging-button" role="button" aria-label="`+l+`"></div>
                    <div ref="btLast" class="ag-paging-button" role="button" aria-label="`+u+`"></div>
                </span>
            </div>`},t.prototype.onBtNext=function(){this.nextButtonDisabled||this.paginationProxy.goToNextPage()},t.prototype.onBtPrevious=function(){this.previousAndFirstButtonsDisabled||this.paginationProxy.goToPreviousPage()},t.prototype.onBtLast=function(){this.lastButtonDisabled||this.paginationProxy.goToLastPage()},t.prototype.enableOrDisableButtons=function(){var e=this.paginationProxy.getCurrentPage(),r=this.paginationProxy.isLastPageFound(),n=this.paginationProxy.getTotalPages();this.previousAndFirstButtonsDisabled=e===0,this.toggleButtonDisabled(this.btFirst,this.previousAndFirstButtonsDisabled),this.toggleButtonDisabled(this.btPrevious,this.previousAndFirstButtonsDisabled);var o=this.isZeroPagesToDisplay(),a=r&&e===n-1;this.nextButtonDisabled=a||o,this.lastButtonDisabled=!r||o||e===n-1,this.toggleButtonDisabled(this.btNext,this.nextButtonDisabled),this.toggleButtonDisabled(this.btLast,this.lastButtonDisabled)},t.prototype.toggleButtonDisabled=function(e,r){GC(e,r),e.classList.toggle("ag-disabled",r),r?e.removeAttribute("tabindex"):e.setAttribute("tabindex","0")},t.prototype.updateRowLabels=function(){var e=this.paginationProxy.getCurrentPage(),r=this.paginationProxy.getPageSize(),n=this.paginationProxy.isLastPageFound(),o=this.paginationProxy.isLastPageFound()?this.paginationProxy.getMasterRowCount():null,a,s;this.isZeroPagesToDisplay()?a=s=0:(a=r*e+1,s=a+r-1,n&&s>o&&(s=o)),this.lbFirstRowOnPage.innerHTML=this.formatNumber(a),this.rowNodeBlockLoader.isLoading()?this.lbLastRowOnPage.innerHTML="?":this.lbLastRowOnPage.innerHTML=this.formatNumber(s)},t.prototype.isZeroPagesToDisplay=function(){var e=this.paginationProxy.isLastPageFound(),r=this.paginationProxy.getTotalPages();return e&&r===0},t.prototype.setTotalLabels=function(){var e=this.paginationProxy.isLastPageFound(),r=this.paginationProxy.getTotalPages(),n=e?this.paginationProxy.getMasterRowCount():null;if(n===1){var o=this.paginationProxy.getRow(0),a=o&&o.group&&!(o.groupData||o.aggData);if(a){this.setTotalLabelsToZero();return}}if(e)this.lbTotal.innerHTML=this.formatNumber(r),this.lbRecordCount.innerHTML=this.formatNumber(n);else{var s=this.gridOptionsWrapper.getLocaleTextFunc()("more","more");this.lbTotal.innerHTML=s,this.lbRecordCount.innerHTML=s}},t.prototype.setTotalLabelsToZero=function(){this.lbFirstRowOnPage.innerHTML=this.formatNumber(0),this.lbCurrent.innerHTML=this.formatNumber(0),this.lbLastRowOnPage.innerHTML=this.formatNumber(0),this.lbTotal.innerHTML=this.formatNumber(0),this.lbRecordCount.innerHTML=this.formatNumber(0)},bn([y("paginationProxy")],t.prototype,"paginationProxy",void 0),bn([y("rowNodeBlockLoader")],t.prototype,"rowNodeBlockLoader",void 0),bn([I("btFirst")],t.prototype,"btFirst",void 0),bn([I("btPrevious")],t.prototype,"btPrevious",void 0),bn([I("btNext")],t.prototype,"btNext",void 0),bn([I("btLast")],t.prototype,"btLast",void 0),bn([I("lbRecordCount")],t.prototype,"lbRecordCount",void 0),bn([I("lbFirstRowOnPage")],t.prototype,"lbFirstRowOnPage",void 0),bn([I("lbLastRowOnPage")],t.prototype,"lbLastRowOnPage",void 0),bn([I("lbCurrent")],t.prototype,"lbCurrent",void 0),bn([I("lbTotal")],t.prototype,"lbTotal",void 0),bn([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var TW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ks=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},dl;(function(i){i[i.Loading=0]="Loading",i[i.NoRows=1]="NoRows"})(dl||(dl={}));var OW=function(i){TW(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.inProgress=!1,e.destroyRequested=!1,e.manuallyDisplayed=!1,e}return t.prototype.updateLayoutClasses=function(e,r){var n=this.eOverlayWrapper.classList;n.toggle(Rt.AUTO_HEIGHT,r.autoHeight),n.toggle(Rt.NORMAL,r.normal),n.toggle(Rt.PRINT,r.print)},t.prototype.postConstruct=function(){this.createManagedBean(new ZC(this)),this.setDisplayed(!1),this.addManagedListener(this.eventService,x.EVENT_ROW_DATA_CHANGED,this.onRowDataChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_ROW_DATA_UPDATED,this.onRowDataChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.onNewColumnsLoaded.bind(this)),this.gridOptionsWrapper.isRowModelDefault()&&!this.gridOptionsWrapper.getRowData()&&this.showLoadingOverlay(),this.gridApi.registerOverlayWrapperComp(this)},t.prototype.setWrapperTypeClass=function(e){var r=this.eOverlayWrapper.classList;r.toggle("ag-overlay-loading-wrapper",e===dl.Loading),r.toggle("ag-overlay-no-rows-wrapper",e===dl.NoRows)},t.prototype.showLoadingOverlay=function(){if(!this.gridOptionsWrapper.isSuppressLoadingOverlay()){var e={},r=this.userComponentFactory.getLoadingOverlayCompDetails(e),n=r.newAgStackInstance();this.showOverlay(n,dl.Loading)}},t.prototype.showNoRowsOverlay=function(){if(!this.gridOptionsWrapper.isSuppressNoRowsOverlay()){var e={},r=this.userComponentFactory.getNoRowsOverlayCompDetails(e),n=r.newAgStackInstance();this.showOverlay(n,dl.NoRows)}},t.prototype.showOverlay=function(e,r){var n=this;this.inProgress||(this.setWrapperTypeClass(r),this.destroyActiveOverlay(),this.inProgress=!0,e&&e.then(function(o){n.inProgress=!1,n.eOverlayWrapper.appendChild(o.getGui()),n.activeOverlay=o,n.destroyRequested&&(n.destroyRequested=!1,n.destroyActiveOverlay())}),this.manuallyDisplayed=this.columnModel.isReady()&&!this.paginationProxy.isEmpty(),this.setDisplayed(!0))},t.prototype.destroyActiveOverlay=function(){if(this.inProgress){this.destroyRequested=!0;return}!this.activeOverlay||(this.activeOverlay=this.getContext().destroyBean(this.activeOverlay),gn(this.eOverlayWrapper))},t.prototype.hideOverlay=function(){this.manuallyDisplayed=!1,this.destroyActiveOverlay(),this.setDisplayed(!1)},t.prototype.destroy=function(){this.destroyActiveOverlay(),i.prototype.destroy.call(this)},t.prototype.showOrHideOverlay=function(){var e=this.paginationProxy.isEmpty(),r=this.gridOptionsWrapper.isSuppressNoRowsOverlay();e&&!r?this.showNoRowsOverlay():this.hideOverlay()},t.prototype.onRowDataChanged=function(){this.showOrHideOverlay()},t.prototype.onNewColumnsLoaded=function(){this.columnModel.isReady()&&!this.paginationProxy.isEmpty()&&!this.manuallyDisplayed&&this.hideOverlay()},t.TEMPLATE=`
        <div class="ag-overlay" aria-hidden="true">
            <div class="ag-overlay-panel">
                <div class="ag-overlay-wrapper" ref="eOverlayWrapper"></div>
            </div>
        </div>`,ks([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),ks([y("paginationProxy")],t.prototype,"paginationProxy",void 0),ks([y("gridApi")],t.prototype,"gridApi",void 0),ks([y("columnModel")],t.prototype,"columnModel",void 0),ks([I("eOverlayWrapper")],t.prototype,"eOverlayWrapper",void 0),ks([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var EW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Rp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},FR=function(i){EW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getFirstRow=function(){var e=0,r;return this.pinnedRowModel.getPinnedTopRowCount()?r=L.PINNED_TOP:this.rowModel.getRowCount()?(r=null,e=this.paginationProxy.getPageFirstRow()):this.pinnedRowModel.getPinnedBottomRowCount()&&(r=L.PINNED_BOTTOM),r===void 0?null:{rowIndex:e,rowPinned:r}},t.prototype.getLastRow=function(){var e,r=null,n=this.pinnedRowModel.getPinnedBottomRowCount(),o=this.pinnedRowModel.getPinnedTopRowCount();return n?(r=L.PINNED_BOTTOM,e=n-1):this.rowModel.getRowCount()?(r=null,e=this.paginationProxy.getPageLastRow()):o&&(r=L.PINNED_TOP,e=o-1),e===void 0?null:{rowIndex:e,rowPinned:r}},t.prototype.getRowNode=function(e){switch(e.rowPinned){case L.PINNED_TOP:return this.pinnedRowModel.getPinnedTopRowData()[e.rowIndex];case L.PINNED_BOTTOM:return this.pinnedRowModel.getPinnedBottomRowData()[e.rowIndex];default:return this.rowModel.getRow(e.rowIndex)}},t.prototype.sameRow=function(e,r){return!e&&!r?!0:e&&!r||!e&&r?!1:e.rowIndex===r.rowIndex&&e.rowPinned==r.rowPinned},t.prototype.before=function(e,r){switch(e.rowPinned){case L.PINNED_TOP:if(r.rowPinned!==L.PINNED_TOP)return!0;break;case L.PINNED_BOTTOM:if(r.rowPinned!==L.PINNED_BOTTOM)return!1;break;default:if(q(r.rowPinned))return r.rowPinned!==L.PINNED_TOP;break}return e.rowIndex<r.rowIndex},Rp([y("rowModel")],t.prototype,"rowModel",void 0),Rp([y("pinnedRowModel")],t.prototype,"pinnedRowModel",void 0),Rp([y("paginationProxy")],t.prototype,"paginationProxy",void 0),t=Rp([Q("rowPositionUtils")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var RW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),PW=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},IR=function(i){RW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.createId=function(e){var r=e.rowIndex,n=e.rowPinned,o=e.column;return this.createIdFromValues(r,o,n)},t.prototype.createIdFromValues=function(e,r,n){return e+"."+(n==null?"null":n)+"."+r.getId()},t.prototype.equals=function(e,r){var n=e.column===r.column,o=e.rowPinned===r.rowPinned,a=e.rowIndex===r.rowIndex;return n&&o&&a},t=PW([Q("cellPositionUtils")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var AW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),_h=function(){function i(t){this.cellValueChanges=t}return i}(),Ug=function(i){AW(t,i);function t(e,r,n){var o=i.call(this,e)||this;return o.initialRange=r,o.finalRange=n,o}return t}(_h),j_=function(){function i(t){this.actionStack=[],this.maxStackSize=t||i.DEFAULT_STACK_SIZE,this.actionStack=new Array(this.maxStackSize)}return i.prototype.pop=function(){return this.actionStack.pop()},i.prototype.push=function(t){var e=t.cellValueChanges&&t.cellValueChanges.length>0;!e||(this.actionStack.length===this.maxStackSize&&this.actionStack.shift(),this.actionStack.push(t))},i.prototype.clear=function(){this.actionStack=[]},i.prototype.getCurrentStackSize=function(){return this.actionStack.length},i.DEFAULT_STACK_SIZE=10,i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var DW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),To=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},MW=function(i){DW(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.cellValueChanges=[],e.activeCellEdit=null,e.activeRowEdit=null,e.isPasting=!1,e.isFilling=!1,e.onCellValueChanged=function(r){var n={column:r.column,rowIndex:r.rowIndex,rowPinned:r.rowPinned},o=e.activeCellEdit!==null&&e.cellPositionUtils.equals(e.activeCellEdit,n),a=e.activeRowEdit!==null&&e.rowPositionUtils.sameRow(e.activeRowEdit,n),s=o||a||e.isPasting||e.isFilling;if(!!s){var l=r.rowPinned,u=r.rowIndex,c=r.column,p=r.oldValue,h=r.value,d={rowPinned:l,rowIndex:u,columnId:c.getColId(),newValue:h,oldValue:p};e.cellValueChanges.push(d)}},e.clearStacks=function(){e.undoStack.clear(),e.redoStack.clear()},e}return t.prototype.init=function(){if(!!this.gridOptionsWrapper.isUndoRedoCellEditing()){var e=this.gridOptionsWrapper.getUndoRedoCellEditingLimit();e<=0||(this.undoStack=new j_(e),this.redoStack=new j_(e),this.addRowEditingListeners(),this.addCellEditingListeners(),this.addPasteListeners(),this.addFillListeners(),this.addManagedListener(this.eventService,x.EVENT_CELL_VALUE_CHANGED,this.onCellValueChanged),this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,this.clearStacks),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,this.clearStacks),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.clearStacks),this.addManagedListener(this.eventService,x.EVENT_COLUMN_GROUP_OPENED,this.clearStacks),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,this.clearStacks),this.addManagedListener(this.eventService,x.EVENT_COLUMN_MOVED,this.clearStacks),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PINNED,this.clearStacks),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VISIBLE,this.clearStacks),this.addManagedListener(this.eventService,x.EVENT_ROW_DRAG_END,this.clearStacks))}},t.prototype.getCurrentUndoStackSize=function(){return this.undoStack?this.undoStack.getCurrentStackSize():0},t.prototype.getCurrentRedoStackSize=function(){return this.redoStack?this.redoStack.getCurrentStackSize():0},t.prototype.undo=function(){if(!!this.undoStack){var e=this.undoStack.pop();!e||!e.cellValueChanges||(this.processAction(e,function(r){return r.oldValue}),e instanceof Ug?this.processRangeAndCellFocus(e.cellValueChanges,e.initialRange):this.processRangeAndCellFocus(e.cellValueChanges),this.redoStack.push(e))}},t.prototype.redo=function(){if(!!this.redoStack){var e=this.redoStack.pop();!e||!e.cellValueChanges||(this.processAction(e,function(r){return r.newValue}),e instanceof Ug?this.processRangeAndCellFocus(e.cellValueChanges,e.finalRange):this.processRangeAndCellFocus(e.cellValueChanges),this.undoStack.push(e))}},t.prototype.processAction=function(e,r){var n=this;e.cellValueChanges.forEach(function(o){var a=o.rowIndex,s=o.rowPinned,l=o.columnId,u={rowIndex:a,rowPinned:s},c=n.getRowNode(u);!c.displayed||c.setDataValue(l,r(o))})},t.prototype.processRangeAndCellFocus=function(e,r){var n;if(r){var o=r.startRow,a=r.endRow;n={rowPinned:o.rowPinned,rowIndex:o.rowIndex,columnId:r.startColumn.getColId()},this.setLastFocusedCell(n);var s={rowStartIndex:o.rowIndex,rowStartPinned:o.rowPinned,rowEndIndex:a.rowIndex,rowEndPinned:a.rowPinned,columnStart:r.startColumn,columns:r.columns};this.gridApi.addCellRange(s);return}var l=e[0],u=l.rowIndex,c=l.rowPinned,p={rowIndex:u,rowPinned:c},h=this.getRowNode(p);n={rowPinned:l.rowPinned,rowIndex:h.rowIndex,columnId:l.columnId},this.setLastFocusedCell(n)},t.prototype.setLastFocusedCell=function(e){var r=e.rowIndex,n=e.columnId,o=e.rowPinned;this.gridApi.ensureIndexVisible(r),this.gridApi.ensureColumnVisible(n),ge.isRegistered(oe.RangeSelectionModule)&&this.gridApi.clearRangeSelection(),this.focusService.setFocusedCell(r,n,o,!0)},t.prototype.addRowEditingListeners=function(){var e=this;this.addManagedListener(this.eventService,x.EVENT_ROW_EDITING_STARTED,function(r){e.activeRowEdit={rowIndex:r.rowIndex,rowPinned:r.rowPinned}}),this.addManagedListener(this.eventService,x.EVENT_ROW_EDITING_STOPPED,function(){var r=new _h(e.cellValueChanges);e.pushActionsToUndoStack(r),e.activeRowEdit=null})},t.prototype.addCellEditingListeners=function(){var e=this;this.addManagedListener(this.eventService,x.EVENT_CELL_EDITING_STARTED,function(r){e.activeCellEdit={column:r.column,rowIndex:r.rowIndex,rowPinned:r.rowPinned}}),this.addManagedListener(this.eventService,x.EVENT_CELL_EDITING_STOPPED,function(){e.activeCellEdit=null;var r=!e.activeRowEdit&&!e.isPasting&&!e.isFilling;if(r){var n=new _h(e.cellValueChanges);e.pushActionsToUndoStack(n)}})},t.prototype.addPasteListeners=function(){var e=this;this.addManagedListener(this.eventService,x.EVENT_PASTE_START,function(){e.isPasting=!0}),this.addManagedListener(this.eventService,x.EVENT_PASTE_END,function(){var r=new _h(e.cellValueChanges);e.pushActionsToUndoStack(r),e.isPasting=!1})},t.prototype.addFillListeners=function(){var e=this;this.addManagedListener(this.eventService,x.EVENT_FILL_START,function(){e.isFilling=!0}),this.addManagedListener(this.eventService,x.EVENT_FILL_END,function(r){var n=new Ug(e.cellValueChanges,r.initialRange,r.finalRange);e.pushActionsToUndoStack(n),e.isFilling=!1})},t.prototype.pushActionsToUndoStack=function(e){this.undoStack.push(e),this.cellValueChanges=[],this.redoStack.clear()},t.prototype.getRowNode=function(e){switch(e.rowPinned){case L.PINNED_TOP:return this.pinnedRowModel.getPinnedTopRowData()[e.rowIndex];case L.PINNED_BOTTOM:return this.pinnedRowModel.getPinnedBottomRowData()[e.rowIndex];default:return this.rowModel.getRow(e.rowIndex)}},To([y("focusService")],t.prototype,"focusService",void 0),To([y("gridApi")],t.prototype,"gridApi",void 0),To([y("rowModel")],t.prototype,"rowModel",void 0),To([y("pinnedRowModel")],t.prototype,"pinnedRowModel",void 0),To([y("cellPositionUtils")],t.prototype,"cellPositionUtils",void 0),To([y("rowPositionUtils")],t.prototype,"rowPositionUtils",void 0),To([B],t.prototype,"init",null),t=To([Q("undoRedoService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var FW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),zg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},LR=function(i){FW(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.findHeader=function(e,r){var n,o,a;if(e.column instanceof sr?(o="getDisplayedGroup"+r,n=this.columnModel[o](e.column)):(a="getDisplayedCol"+r,n=this.columnModel[a](e.column)),n)return{column:n,headerRowIndex:e.headerRowIndex}},t.prototype.findColAtEdgeForHeaderRow=function(e,r){var n=this.columnModel.getAllDisplayedColumns(),o=n[r==="start"?0:n.length-1];if(!!o){var a=this.ctrlsService.getHeaderRowContainerCtrl(o.getPinned()),s=a.getRowType(e);if(s==wr.COLUMN_GROUP){var l=this.columnModel.getColumnGroupAtLevel(o,e);return{headerRowIndex:e,column:l}}return{headerRowIndex:s==null?-1:e,column:o}}},zg([y("columnModel")],t.prototype,"columnModel",void 0),zg([y("ctrlsService")],t.prototype,"ctrlsService",void 0),t=zg([Q("headerPositionUtils")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var IW=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},LW=function(){function i(){}return i.prototype.buildColumnDefs=function(t,e,r){var n=this,o=[],a={};return t.forEach(function(s){for(var l=n.createDefFromColumn(s,e,r),u=!0,c=l,p=s.getOriginalParent();p;){var h=null;if(p.isPadding()){p=p.getOriginalParent();continue}var d=a[p.getGroupId()];if(d){d.children.push(c),u=!1;break}h=n.createDefFromGroup(p),h&&(h.children=[c],a[h.groupId]=h,c=h,p=p.getOriginalParent())}u&&o.push(c)}),o},i.prototype.createDefFromGroup=function(t){var e=jh(t.getColGroupDef(),["children"]);return e&&(e.groupId=t.getGroupId()),e},i.prototype.createDefFromColumn=function(t,e,r){var n=jh(t.getColDef());return n.colId=t.getColId(),n.width=t.getActualWidth(),n.rowGroup=t.isRowGroupActive(),n.rowGroupIndex=t.isRowGroupActive()?e.indexOf(t):null,n.pivot=t.isPivotActive(),n.pivotIndex=t.isPivotActive()?r.indexOf(t):null,n.aggFunc=t.isValueActive()?t.getAggFunc():null,n.hide=t.isVisible()?void 0:!0,n.pinned=t.isPinned()?t.getPinned():null,n.sort=t.getSort()?t.getSort():null,n.sortIndex=t.getSortIndex()!=null?t.getSortIndex():null,n},i=IW([Q("columnDefFactory")],i),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var $g=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},NW=function(){function i(){}return i.prototype.getInitialRowClasses=function(t){var e=[];return q(t.extraCssClass)&&e.push(t.extraCssClass),e.push("ag-row"),e.push(t.rowFocused?"ag-row-focus":"ag-row-no-focus"),t.fadeRowIn&&e.push("ag-opacity-zero"),e.push(t.rowIsEven?"ag-row-even":"ag-row-odd"),t.rowNode.isRowPinned()&&e.push("ag-row-pinned"),t.rowNode.isSelected()&&e.push("ag-row-selected"),t.rowNode.footer&&e.push("ag-row-footer"),e.push("ag-row-level-"+t.rowLevel),t.rowNode.stub&&e.push("ag-row-loading"),t.fullWidthRow&&e.push("ag-full-width-row"),t.expandable&&(e.push("ag-row-group"),e.push(t.rowNode.expanded?"ag-row-group-expanded":"ag-row-group-contracted")),t.rowNode.dragging&&e.push("ag-row-dragging"),Bm(e,this.processClassesFromGridOptions(t.rowNode)),Bm(e,this.preProcessRowClassRules(t.rowNode)),e.push(t.printLayout?"ag-row-position-relative":"ag-row-position-absolute"),t.firstRowOnPage&&e.push("ag-row-first"),t.lastRowOnPage&&e.push("ag-row-last"),t.fullWidthRow&&(t.pinned===L.PINNED_LEFT&&e.push("ag-cell-last-left-pinned"),t.pinned===L.PINNED_RIGHT&&e.push("ag-cell-first-right-pinned")),e},i.prototype.processClassesFromGridOptions=function(t){var e=[],r=function(l){typeof l=="string"?e.push(l):Array.isArray(l)&&l.forEach(function(u){return e.push(u)})},n=this.gridOptionsWrapper.getRowClass();if(n){if(typeof n=="function")return console.warn("AG Grid: rowClass should not be a function, please use getRowClass instead"),[];r(n)}var o=this.gridOptionsWrapper.getRowClassFunc();if(o){var a={data:t.data,node:t,rowIndex:t.rowIndex},s=o(a);r(s)}return e},i.prototype.preProcessRowClassRules=function(t){var e=[];return this.processRowClassRules(t,function(r){e.push(r)},function(r){}),e},i.prototype.processRowClassRules=function(t,e,r){var n={data:t.data,node:t,rowIndex:t.rowIndex,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()};this.stylingService.processClassRules(this.gridOptionsWrapper.rowClassRules(),n,e,r)},i.prototype.calculateRowLevel=function(t){return t.group?t.level:t.parent?t.parent.level+1:0},$g([y("stylingService")],i.prototype,"stylingService",void 0),$g([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),i=$g([Q("rowCssClassCalculator")],i),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Pp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},NR=function(){function i(){}return i.prototype.doFullSort=function(t,e){var r=function(o,a){return{currentPos:a,rowNode:o}},n=t.map(r);return n.sort(this.compareRowNodes.bind(this,e)),n.map(function(o){return o.rowNode})},i.prototype.compareRowNodes=function(t,e,r){for(var n=e.rowNode,o=r.rowNode,a=0,s=t.length;a<s;a++){var l=t[a],u=l.sort===L.SORT_DESC,c=this.getValue(n,l.column),p=this.getValue(o,l.column),h=void 0,d=this.getComparator(l,n);d?h=d(c,p,n,o,u):h=_.defaultComparator(c,p,this.gridOptionsWrapper.isAccentedSort());var f=!isNaN(h);if(f&&h!==0)return l.sort===L.SORT_ASC?h:h*-1}return e.currentPos-r.currentPos},i.prototype.getComparator=function(t,e){var r=t.column,n=r.getColDef().comparator;if(n!=null)return n;if(e.rowGroupColumn)return e.rowGroupColumn.getColDef().comparator;if(!!r.getColDef().showRowGroup){var o=!e.group&&r.getColDef().field;if(!!o){var a=this.columnModel.getPrimaryColumn(o);if(!!a)return a.getColDef().comparator}}},i.prototype.getValue=function(t,e){return this.valueService.getValue(e,t,!1,!1)},Pp([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),Pp([y("valueService")],i.prototype,"valueService",void 0),Pp([y("columnModel")],i.prototype,"columnModel",void 0),i=Pp([Q("rowNodeSorter")],i),i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var GW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),kW=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},GR=function(i){GW(t,i);function t(){var r=i!==null&&i.apply(this,arguments)||this;return r.ready=!1,r.readyCallbacks=[],r}e=t,t.prototype.checkReady=function(){if(this.ready=this.gridCtrl!=null&&this.gridBodyCtrl!=null&&this.centerRowContainerCtrl!=null&&this.leftRowContainerCtrl!=null&&this.rightRowContainerCtrl!=null&&this.bottomCenterRowContainerCtrl!=null&&this.bottomLeftRowContainerCtrl!=null&&this.bottomRightRowContainerCtrl!=null&&this.topCenterRowContainerCtrl!=null&&this.topLeftRowContainerCtrl!=null&&this.topRightRowContainerCtrl!=null&&this.centerHeaderRowContainerCtrl!=null&&this.leftHeaderRowContainerCtrl!=null&&this.rightHeaderRowContainerCtrl!=null&&this.fakeHScrollCtrl!=null&&this.gridHeaderCtrl!=null,this.ready){var r=this.createReadyParams();this.readyCallbacks.forEach(function(n){return n(r)}),this.readyCallbacks.length=0}},t.prototype.whenReady=function(r){this.ready?r(this.createReadyParams()):this.readyCallbacks.push(r)},t.prototype.createReadyParams=function(){return{centerRowContainerCtrl:this.centerRowContainerCtrl,leftRowContainerCtrl:this.leftRowContainerCtrl,rightRowContainerCtrl:this.rightRowContainerCtrl,bottomCenterRowContainerCtrl:this.bottomCenterRowContainerCtrl,bottomLeftRowContainerCtrl:this.bottomLeftRowContainerCtrl,bottomRightRowContainerCtrl:this.bottomRightRowContainerCtrl,topCenterRowContainerCtrl:this.topCenterRowContainerCtrl,topLeftRowContainerCtrl:this.topLeftRowContainerCtrl,topRightRowContainerCtrl:this.topRightRowContainerCtrl,centerHeaderRowContainerCtrl:this.centerHeaderRowContainerCtrl,leftHeaderRowContainerCtrl:this.leftHeaderRowContainerCtrl,rightHeaderRowContainerCtrl:this.rightHeaderRowContainerCtrl,fakeHScrollCtrl:this.fakeHScrollCtrl,gridBodyCtrl:this.gridBodyCtrl,gridCtrl:this.gridCtrl,gridHeaderCtrl:this.gridHeaderCtrl}},t.prototype.registerFakeHScrollCtrl=function(r){this.fakeHScrollCtrl=r,this.checkReady()},t.prototype.registerGridHeaderCtrl=function(r){this.gridHeaderCtrl=r,this.checkReady()},t.prototype.registerCenterRowContainerCtrl=function(r){this.centerRowContainerCtrl=r,this.checkReady()},t.prototype.registerLeftRowContainerCtrl=function(r){this.leftRowContainerCtrl=r,this.checkReady()},t.prototype.registerRightRowContainerCtrl=function(r){this.rightRowContainerCtrl=r,this.checkReady()},t.prototype.registerTopCenterRowContainerCtrl=function(r){this.topCenterRowContainerCtrl=r,this.checkReady()},t.prototype.registerTopLeftRowContainerCon=function(r){this.topLeftRowContainerCtrl=r,this.checkReady()},t.prototype.registerTopRightRowContainerCtrl=function(r){this.topRightRowContainerCtrl=r,this.checkReady()},t.prototype.registerBottomCenterRowContainerCtrl=function(r){this.bottomCenterRowContainerCtrl=r,this.checkReady()},t.prototype.registerBottomLeftRowContainerCtrl=function(r){this.bottomLeftRowContainerCtrl=r,this.checkReady()},t.prototype.registerBottomRightRowContainerCtrl=function(r){this.bottomRightRowContainerCtrl=r,this.checkReady()},t.prototype.registerHeaderContainer=function(r,n){switch(n){case L.PINNED_LEFT:this.leftHeaderRowContainerCtrl=r;break;case L.PINNED_RIGHT:this.rightHeaderRowContainerCtrl=r;break;default:this.centerHeaderRowContainerCtrl=r;break}this.checkReady()},t.prototype.registerGridBodyCtrl=function(r){this.gridBodyCtrl=r,this.checkReady()},t.prototype.registerGridCtrl=function(r){this.gridCtrl=r,this.checkReady()},t.prototype.getFakeHScrollCtrl=function(){return this.fakeHScrollCtrl},t.prototype.getGridHeaderCtrl=function(){return this.gridHeaderCtrl},t.prototype.getGridCtrl=function(){return this.gridCtrl},t.prototype.getCenterRowContainerCtrl=function(){return this.centerRowContainerCtrl},t.prototype.getTopCenterRowContainerCtrl=function(){return this.topCenterRowContainerCtrl},t.prototype.getBottomCenterRowContainerCtrl=function(){return this.bottomCenterRowContainerCtrl},t.prototype.getGridBodyCtrl=function(){return this.gridBodyCtrl},t.prototype.getHeaderRowContainerCtrls=function(){return[this.leftHeaderRowContainerCtrl,this.rightHeaderRowContainerCtrl,this.centerHeaderRowContainerCtrl]},t.prototype.getHeaderRowContainerCtrl=function(r){switch(r){case L.PINNED_LEFT:return this.leftHeaderRowContainerCtrl;case L.PINNED_RIGHT:return this.rightHeaderRowContainerCtrl;default:return this.centerHeaderRowContainerCtrl}};var e;return t.NAME="ctrlsService",t=e=kW([Q(e.NAME)],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var WW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),VW=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},BW=function(i){WW(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.registry={},e}return t.prototype.register=function(e){this.registry[e.controllerName]=e.controllerClass},t.prototype.getInstance=function(e){var r=this.registry[e];if(r!=null)return new r},t=VW([Q("ctrlsFactory")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var HW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ap=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},jW=function(i){HW(t,i);function t(){return i.call(this)||this}return t.prototype.setComp=function(e,r,n,o){this.view=e,this.eViewport=n,this.eContainer=o,this.eGui=r,this.addManagedListener(this.eventService,x.EVENT_SCROLL_VISIBILITY_CHANGED,this.onScrollVisibilityChanged.bind(this)),this.onScrollVisibilityChanged();var a=this.setFakeHScrollSpacerWidths.bind(this);this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,a),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,a),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_DOM_LAYOUT,a),this.setFakeHScrollSpacerWidths(),this.invisibleScrollbar&&(this.hideAndShowInvisibleScrollAsNeeded(),this.addActiveListenerToggles()),this.ctrlsService.registerFakeHScrollCtrl(this)},t.prototype.addActiveListenerToggles=function(){var e=this,r=["mouseenter","mousedown","touchstart"],n=["mouseleave","mouseup","touchend"];r.forEach(function(o){return e.addManagedListener(e.eGui,o,function(){return e.view.addOrRemoveCssClass("ag-scrollbar-active",!0)})}),n.forEach(function(o){return e.addManagedListener(e.eGui,o,function(){return e.view.addOrRemoveCssClass("ag-scrollbar-active",!1)})})},t.prototype.postConstruct=function(){this.enableRtl=this.gridOptionsWrapper.isEnableRtl(),this.invisibleScrollbar=RO()},t.prototype.onScrollVisibilityChanged=function(){this.setScrollVisible(),this.setFakeHScrollSpacerWidths()},t.prototype.hideAndShowInvisibleScrollAsNeeded=function(){var e=this;this.addManagedListener(this.eventService,x.EVENT_BODY_SCROLL,function(r){r.direction==="horizontal"&&e.view.addOrRemoveCssClass("ag-scrollbar-scrolling",!0)}),this.addManagedListener(this.eventService,x.EVENT_BODY_SCROLL_END,function(){return e.view.addOrRemoveCssClass("ag-scrollbar-scrolling",!1)})},t.prototype.setFakeHScrollSpacerWidths=function(){var e=this.scrollVisibleService.isVerticalScrollShowing(),r=this.columnModel.getDisplayedColumnsRightWidth(),n=!this.enableRtl&&e,o=this.gridOptionsWrapper.getScrollbarWidth();n&&(r+=o),this.view.setRightSpacerFixedWidth(r),this.view.includeRightSpacerScrollerCss("ag-scroller-corner",r<=o);var a=this.columnModel.getDisplayedColumnsLeftWidth(),s=this.enableRtl&&e;s&&(a+=o),this.view.setLeftSpacerFixedWidth(a),this.view.includeLeftSpacerScrollerCss("ag-scroller-corner",a<=o)},t.prototype.setScrollVisible=function(){var e=this.scrollVisibleService.isHorizontalScrollShowing(),r=this.invisibleScrollbar,n=this.gridOptionsWrapper.isSuppressHorizontalScroll(),o=e&&this.gridOptionsWrapper.getScrollbarWidth()||0,a=o===0&&r?15:o,s=n?0:a;this.view.addOrRemoveCssClass("ag-scrollbar-invisible",r),this.view.setHeight(s),this.view.setViewportHeight(s),this.view.setContainerHeight(s)},t.prototype.getViewport=function(){return this.eViewport},t.prototype.getContainer=function(){return this.eContainer},Ap([y("scrollVisibleService")],t.prototype,"scrollVisibleService",void 0),Ap([y("columnModel")],t.prototype,"columnModel",void 0),Ap([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Ap([B],t.prototype,"postConstruct",null),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var UW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ou=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},zW=function(i){UW(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.postConstruct=function(){var e=this,r={addOrRemoveCssClass:function(o,a){return e.addOrRemoveCssClass(o,a)},setHeight:function(o){return ts(e.getGui(),o)},setContainerHeight:function(o){return ts(e.eContainer,o)},setViewportHeight:function(o){return ts(e.eViewport,o)},setRightSpacerFixedWidth:function(o){return Li(e.eRightSpacer,o)},setLeftSpacerFixedWidth:function(o){return Li(e.eLeftSpacer,o)},includeLeftSpacerScrollerCss:function(o,a){return e.eLeftSpacer.classList.toggle(o,a)},includeRightSpacerScrollerCss:function(o,a){return e.eRightSpacer.classList.toggle(o,a)}},n=this.createManagedBean(new jW);n.setComp(r,this.getGui(),this.eViewport,this.eContainer),this.createManagedBean(new ew(function(o){return e.eContainer.style.width=o+"px"}))},t.TEMPLATE=`<div class="ag-body-horizontal-scroll" aria-hidden="true">
            <div class="ag-horizontal-left-spacer" ref="eLeftSpacer"></div>
            <div class="ag-body-horizontal-scroll-viewport" ref="eViewport">
                <div class="ag-body-horizontal-scroll-container" ref="eContainer"></div>
            </div>
            <div class="ag-horizontal-right-spacer" ref="eRightSpacer"></div>
        </div>`,Ou([I("eLeftSpacer")],t.prototype,"eLeftSpacer",void 0),Ou([I("eRightSpacer")],t.prototype,"eRightSpacer",void 0),Ou([I("eViewport")],t.prototype,"eViewport",void 0),Ou([I("eContainer")],t.prototype,"eContainer",void 0),Ou([B],t.prototype,"postConstruct",null),t}(ee);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var $W=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Kg=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},KW=function(i){$W(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){var e=this.checkContainerWidths.bind(this);this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_CHANGED,e),this.addManagedListener(this.eventService,x.EVENT_DISPLAYED_COLUMNS_WIDTH_CHANGED,e),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_DOM_LAYOUT,e)},t.prototype.checkContainerWidths=function(){var e=this.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_PRINT,r=e?0:this.columnModel.getDisplayedColumnsLeftWidth(),n=e?0:this.columnModel.getDisplayedColumnsRightWidth();r!=this.leftWidth&&(this.leftWidth=r,this.eventService.dispatchEvent({type:x.EVENT_LEFT_PINNED_WIDTH_CHANGED})),n!=this.rightWidth&&(this.rightWidth=n,this.eventService.dispatchEvent({type:x.EVENT_RIGHT_PINNED_WIDTH_CHANGED}))},t.prototype.getPinnedRightWidth=function(){return this.rightWidth},t.prototype.getPinnedLeftWidth=function(){return this.leftWidth},Kg([y("columnModel")],t.prototype,"columnModel",void 0),Kg([B],t.prototype,"postConstruct",null),t=Kg([Q("pinnedWidthService")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var YW=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Dp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},XW=function(i){YW(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.events=[],e}return t.prototype.postConstruct=function(){this.rowModel.getType()==L.ROW_MODEL_TYPE_CLIENT_SIDE&&(this.clientSideRowModel=this.rowModel)},t.prototype.dispatchExpanded=function(e){var r=this;if(this.clientSideRowModel==null){this.eventService.dispatchEvent(e);return}this.events.push(e);var n=function(){r.clientSideRowModel&&r.clientSideRowModel.onRowGroupOpened(),r.events.forEach(function(o){return r.eventService.dispatchEvent(o)}),r.events=[]};this.dispatchExpandedDebounced==null&&(this.dispatchExpandedDebounced=this.animationFrameService.debounce(n)),this.dispatchExpandedDebounced()},Dp([y("animationFrameService")],t.prototype,"animationFrameService",void 0),Dp([y("rowModel")],t.prototype,"rowModel",void 0),Dp([B],t.prototype,"postConstruct",null),t=Dp([Q("rowNodeEventThrottle")],t),t}(H);/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var qW=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},U_=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(qW(arguments[t]));return i},kR=function(){function i(t,e,r){if(!e){console.error("AG Grid: no gridOptions provided to the grid");return}this.gridOptions=e,new WR().create(t,e,function(n){var o=new RR(t);n.createBean(o)},void 0,r)}return i.prototype.destroy=function(){this.gridOptions&&this.gridOptions.api&&this.gridOptions.api.destroy()},i}(),WR=function(){function i(){}return i.prototype.create=function(t,e,r,n,o){var a=this,s=!!e.debug,l=this.getRegisteredModules(o),u=this.createBeansList(e.rowModelType,l),c=this.createProvidedBeans(t,e,o);if(!!u){var p={providedBeanInstances:c,beanClasses:u,debug:s},h=new ld("AG Grid",function(){return e.debug}),d=new ld("Context",function(){return p.debug}),f=new cO(p,d),g=f.getBean("beans");this.registerModuleUserComponents(g,l),this.registerStackComponents(g,l),this.registerControllers(g,l),r(f),g.ctrlsService.whenReady(function(){a.setColumnsAndData(g),a.dispatchGridReadyEvent(g,e);var v=ge.isRegistered(oe.EnterpriseCoreModule);h.log("initialised successfully, enterprise = "+v)}),n&&n(f)}},i.prototype.registerControllers=function(t,e){e.forEach(function(r){r.controllers&&r.controllers.forEach(function(n){return t.ctrlsFactory.register(n)})})},i.prototype.registerStackComponents=function(t,e){var r=this.createAgStackComponentsList(e);t.agStackComponentsRegistry.setupComponents(r)},i.prototype.getRegisteredModules=function(t){var e=t?t.modules:null,r=ge.getRegisteredModules(),n=[],o={};function a(s,l){function u(c){o[c.moduleName]||(o[c.moduleName]=!0,n.push(c),ge.register(c,s))}u(l),l.dependantModules&&l.dependantModules.forEach(a.bind(null,s))}return e&&e.forEach(a.bind(null,!0)),r&&r.forEach(a.bind(null,!ge.isPackageBased())),n},i.prototype.registerModuleUserComponents=function(t,e){var r=this.extractModuleEntity(e,function(n){return n.userComponents?n.userComponents:[]});r.forEach(function(n){t.userComponentRegistry.registerDefaultComponent(n.componentName,n.componentClass)})},i.prototype.createProvidedBeans=function(t,e,r){var n=r?r.frameworkOverrides:null;me(n)&&(n=new yR);var o={gridOptions:e,eGridDiv:t,globalEventListener:r?r.globalEventListener:null,frameworkOverrides:n};return r&&r.providedBeanInstances&&Object.assign(o,r.providedBeanInstances),o},i.prototype.createAgStackComponentsList=function(t){var e=[{componentName:"AgCheckbox",componentClass:ii},{componentName:"AgRadioButton",componentClass:nw},{componentName:"AgToggleButton",componentClass:sf},{componentName:"AgInputTextField",componentClass:iw},{componentName:"AgInputTextArea",componentClass:pR},{componentName:"AgInputNumberField",componentClass:hR},{componentName:"AgInputRange",componentClass:dR},{componentName:"AgSelect",componentClass:Kc},{componentName:"AgSlider",componentClass:zi},{componentName:"AgAngleSelect",componentClass:ow},{componentName:"AgColorPicker",componentClass:fR},{componentName:"AgGridBody",componentClass:IE},{componentName:"AgHeaderRoot",componentClass:QE},{componentName:"AgPagination",componentClass:xW},{componentName:"AgOverlayWrapper",componentClass:OW},{componentName:"AgGroupComponent",componentClass:Qn},{componentName:"AgPanel",componentClass:lf},{componentName:"AgDialog",componentClass:Oc},{componentName:"AgRowContainer",componentClass:VE},{componentName:"AgFakeHorizontalScroll",componentClass:zW}],r=this.extractModuleEntity(t,function(n){return n.agStackComponents?n.agStackComponents:[]});return e=e.concat(r),e},i.prototype.createBeansList=function(t,e){var r=this.getRowModelClass(t,e);if(!!r){var n=[r,JC,FR,IR,LR,lW,zE,_E,dW,gW,tR,xE,bW,JE,sR,TE,_O,oi,ce,mR,bR,OE,wO,qE,uR,oR,xR,mO,TR,wR,GE,AR,SR,OR,bO,iR,eR,ht,_R,Sl,NE,DR,CR,aR,cR,LE,PR,rW,iW,MR,aW,pW,rR,MW,SO,LW,NW,lR,NR,GR,KW,XW,BW],o=this.extractModuleEntity(e,function(s){return s.beans?s.beans:[]});n.push.apply(n,U_(o));var a=[];return n.forEach(function(s){a.indexOf(s)<0&&a.push(s)}),a}},i.prototype.extractModuleEntity=function(t,e){return[].concat.apply([],U_(t.map(e)))},i.prototype.setColumnsAndData=function(t){var e=t.gridOptionsWrapper.getColumnDefs();t.columnModel.setColumnDefs(e||[],"gridInitializing"),t.rowModel.start()},i.prototype.dispatchGridReadyEvent=function(t,e){var r={type:x.EVENT_GRID_READY,api:e.api,columnApi:e.columnApi};t.eventService.dispatchEvent(r)},i.prototype.getRowModelClass=function(t,e){t||(t=L.ROW_MODEL_TYPE_CLIENT_SIDE);var r={};e.forEach(function(o){Ke(o.rowModels,function(a,s){r[a]=s})});var n=r[t];if(q(n))return n;ge.isPackageBased()?[L.ROW_MODEL_TYPE_VIEWPORT,L.ROW_MODEL_TYPE_SERVER_SIDE].includes(t)?console.error('AG Grid: Row Model "'+t+`" not found. Please ensure the package 'ag-grid-enterprise' is imported. Please see: https://www.ag-grid.com/javascript-grid/packages/`):console.error("AG Grid: could not find row model for rowModelType "+t):t===L.ROW_MODEL_TYPE_INFINITE?console.error('AG Grid: Row Model "Infinite" not found. Please ensure the '+oe.InfiniteRowModelModule+" module is registered. Please see: https://www.ag-grid.com/javascript-grid/modules/"):t===L.ROW_MODEL_TYPE_VIEWPORT?console.error('AG Grid: Row Model "Viewport" not found. Please ensure the AG Grid Enterprise Module '+oe.ViewportRowModelModule+" module is registered. Please see: https://www.ag-grid.com/javascript-grid/modules/"):t===L.ROW_MODEL_TYPE_SERVER_SIDE?console.error('AG Grid: Row Model "Server Side" not found. Please ensure the AG Grid Enterprise Module '+oe.ServerSideRowModelModule+" module is registered. Please see: https://www.ag-grid.com/javascript-grid/modules/"):t===L.ROW_MODEL_TYPE_CLIENT_SIDE?console.error('AG Grid: Row Model "Client Side" not found. Please ensure the '+oe.ClientSideRowModelModule+" module is registered. Please see: https://www.ag-grid.com/javascript-grid/modules/"):console.error("AG Grid: could not find row model for rowModelType "+t)},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */function ZW(i,t,e,r,n){n===void 0&&(n=!1),console.warn("AG Grid: Since ag-grid 11.0.0 defaultGroupComparator is not necessary. You can remove this from your colDef");var o=q(e)&&e.group,a=q(r)&&r.group,s=o&&a,l=!o&&!a;return s?Gm(e.key,r.key,n):l?Gm(i,t,n):o?1:-1}/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var QW=function(){function i(){}return i.prototype.wrap=function(t,e,r,n){var o=this;r===void 0&&(r=[]);var a=this.createWrapper(t,n);return e.forEach(function(s){o.createMethod(a,s,!0)}),r.forEach(function(s){o.createMethod(a,s,!1)}),a},i.prototype.unwrap=function(t){return t},i.prototype.createMethod=function(t,e,r){t.addMethod(e,this.createMethodProxy(t,e,r))},i.prototype.createMethodProxy=function(t,e,r){return function(){return t.hasMethod(e)?t.callMethod(e,arguments):(r&&console.warn("AG Grid: Framework component is missing the method "+e+"()"),null)}},i}();/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var ay;(function(i){i.InsideBase="insideBase",i.InsideEnd="insideEnd",i.Center="center",i.OutsideEnd="outsideEnd"})(ay||(ay={}));/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */var Ul=typeof global=="undefined"?{}:global;Ul.HTMLElement=typeof HTMLElement=="undefined"?{}:HTMLElement;Ul.HTMLButtonElement=typeof HTMLButtonElement=="undefined"?{}:HTMLButtonElement;Ul.HTMLSelectElement=typeof HTMLSelectElement=="undefined"?{}:HTMLSelectElement;Ul.HTMLInputElement=typeof HTMLInputElement=="undefined"?{}:HTMLInputElement;Ul.Node=typeof Node=="undefined"?{}:Node;Ul.MouseEvent=typeof MouseEvent=="undefined"?{}:MouseEvent;var JW=Object.freeze(Object.defineProperty({__proto__:null,ColumnFactory:mO,ColumnModel:wO,ColumnKeyCreator:JT,ColumnUtils:bO,DisplayedGroupCreator:_O,GroupInstanceIdCreator:OC,ComponentUtil:ea,AgStackComponentsRegistry:SO,ColDefUtil:Um,UserComponentRegistry:_E,UserComponentFactory:xE,BeanStub:H,Context:cO,Autowired:y,PostConstruct:B,PreConstruct:ss,Optional:De,Bean:Q,Qualifier:Yt,PreDestroy:nt,QuerySelector:eL,RefSelector:I,get ExcelFactoryMode(){return to},DragAndDropService:ht,get DragSourceType(){return At},get HorizontalDirection(){return Pn},get VerticalDirection(){return la},DragService:TE,Column:Se,ColumnGroup:sr,ProvidedColumnGroup:$e,RowNode:Oe,get RowHighlightPosition(){return Di},FilterManager:OE,ProvidedFilter:us,SimpleFilter:Ot,ScalarFilter:Vt,NumberFilter:KC,TextFilter:Yc,DateFilter:$C,TextFloatingFilter:fE,HeaderFilterCellComp:EE,FloatingFilterMapper:SE,GridBodyComp:IE,GridBodyCtrl:ME,get RowAnimationCssClasses(){return cs},ScrollVisibleService:LE,MouseEventService:NE,NavigationService:GE,RowContainerComp:VE,get RowContainerName(){return re},RowContainerCtrl:id,get RowContainerType(){return Hr},getRowContainerTypeForName:FE,BodyDropPivotTarget:BE,BodyDropTarget:jE,CssClassApplier:qc,HeaderRowContainerComp:bh,GridHeaderComp:QE,GridHeaderCtrl:ZE,HeaderRowComp:UE,get HeaderRowType(){return wr},HeaderRowCtrl:wh,HeaderCellCtrl:KE,HeaderFilterCellCtrl:$E,HeaderGroupCellCtrl:YE,AbstractHeaderCellCtrl:Zc,HeaderRowContainerCtrl:XE,HorizontalResizeService:JE,MoveColumnFeature:HE,StandardMenuFactory:eR,TabbedLayout:rw,simpleHttpRequest:zk,ResizeObserverService:tR,AnimationFrameService:rR,LargeTextCellEditor:gE,PopupEditorWrapper:kE,PopupSelectCellEditor:vE,PopupTextCellEditor:mE,SelectCellEditor:YC,TextCellEditor:Qh,Beans:JC,AnimateShowChangeCellRenderer:yE,AnimateSlideCellRenderer:CE,GroupCellRenderer:ry,GroupCellRendererCtrl:bE,SetLeftFeature:af,PositionableFeature:Qc,AutoWidthCalculator:iR,CheckboxSelectionComponent:XC,CellComp:WE,CellCtrl:ta,RowCtrl:ra,RowRenderer:oR,ValueFormatterService:aR,PinnedRowModel:sR,get ServerSideTransactionResultStatus(){return hn},ChangedPath:Jc,RowNodeBlock:jr,RowNodeBlockLoader:lR,PaginationProxy:uR,get ClientSideRowModelSteps(){return ze},StylingService:cR,get LayoutCssClasses(){return Rt},AgAbstractField:ei,AgCheckbox:ii,AgRadioButton:nw,AgToggleButton:sf,AgInputTextField:iw,AgInputTextArea:pR,AgInputNumberField:hR,AgInputRange:dR,AgSelect:Kc,AgSlider:zi,AgAngleSelect:ow,AgColorPicker:fR,AgGroupComponent:Qn,AgMenuItemComponent:Dn,AgMenuList:ep,AgMenuPanel:vR,AgDialog:Oc,AgPanel:lf,Component:ee,CssClassManager:sE,ManagedFocusFeature:ai,TabGuardComp:bs,TabGuardCtrl:gR,PopupComponent:ws,PopupService:mR,TouchListener:Cr,VirtualList:tp,get CellRangeType(){return zr},get SelectionHandleType(){return co},AutoScrollService:Xc,VanillaFrameworkOverrides:yR,CellNavigationService:CR,AlignedGridsService:wR,Constants:L,KeyCode:j,Grid:kR,GridCoreCreator:WR,GridApi:zE,Events:x,FocusService:Sl,defaultGroupComparator:ZW,GridOptionsWrapper:ce,EventService:oi,SelectableService:MR,RowNodeSorter:NR,CtrlsService:GR,GridComp:RR,GridCtrl:ER,Logger:ld,LoggerFactory:OR,SelectionService:bR,SortController:PR,TemplateService:TR,ValueService:SR,ValueCache:AR,ExpressionService:xR,CellPositionUtils:IR,RowPositionUtils:FR,HeaderPositionUtils:LR,HeaderNavigationService:qE,get HeaderNavigationDirection(){return Xn},ColumnApi:_R,BaseComponentWrapper:QW,Environment:DR,CustomTooltipFeature:nf,get ModuleNames(){return oe},ModuleRegistry:ge,Color:Bo,stringToArray:xO,_,NumberSequence:Bt,get AgPromiseStatus(){return Ka},AgPromise:ct,Timer:qI,PropertyKeys:bi,get BarColumnLabelPlacement(){return ay}},Symbol.toStringTag,{value:"Module"})),jt=RD(JW),aw={},uf={};Object.defineProperty(uf,"__esModule",{value:!0});class eV{}class tV extends eV{constructor(t,e,r){super(),this.portal=null,this.reactComponent=t,this.portalManager=e,this.componentType=r,this.statelessComponent=this.isStateless(this.reactComponent)}getGui(){return this.eParentElement}destroy(){return this.componentInstance&&typeof this.componentInstance.destroy=="function"&&this.componentInstance.destroy(),this.portalManager.destroyPortal(this.portal)}createParentElement(t){const e=this.portalManager.getComponentWrappingElement(),r=document.createElement(e||"div");return r.classList.add("ag-react-container"),t.reactContainer=r,r}addParentContainerStyleAndClasses(){!this.componentInstance||(this.componentInstance.getReactContainerStyle&&this.componentInstance.getReactContainerStyle()&&Object.assign(this.eParentElement.style,this.componentInstance.getReactContainerStyle()),this.componentInstance.getReactContainerClasses&&this.componentInstance.getReactContainerClasses()&&this.componentInstance.getReactContainerClasses().forEach(e=>this.eParentElement.classList.add(e)))}statelessComponentRendered(){return this.eParentElement.childElementCount>0||this.eParentElement.childNodes.length>0}getFrameworkComponentInstance(){return this.componentInstance}isStatelessComponent(){return this.statelessComponent}getReactComponentName(){return this.reactComponent.name}getMemoType(){return this.hasSymbol()?Symbol.for("react.memo"):60115}hasSymbol(){return typeof Symbol=="function"&&Symbol.for}isStateless(t){return typeof t=="function"&&!(t.prototype&&t.prototype.isReactComponent)||typeof t=="object"&&t.$$typeof===this.getMemoType()}hasMethod(t){const e=this.getFrameworkComponentInstance();return!!e&&e[t]!==null||this.fallbackMethodAvailable(t)}callMethod(t,e){const r=this.getFrameworkComponentInstance();if(this.isStatelessComponent())return this.fallbackMethod(t,!!e&&e[0]?e[0]:{});if(!r){setTimeout(()=>this.callMethod(t,e));return}const n=r[t];if(n)return n.apply(r,e);if(this.fallbackMethodAvailable(t))return this.fallbackMethod(t,!!e&&e[0]?e[0]:{})}addMethod(t,e){this[t]=e}}uf.ReactComponent=tV;var ha={},zl={};/**
 * @license React
 * react-dom-server-legacy.browser.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var VR=we.exports;function Ne(i){for(var t="https://reactjs.org/docs/error-decoder.html?invariant="+i,e=1;e<arguments.length;e++)t+="&args[]="+encodeURIComponent(arguments[e]);return"Minified React error #"+i+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}var Vr=Object.prototype.hasOwnProperty,rV=/^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/,z_={},$_={};function BR(i){return Vr.call($_,i)?!0:Vr.call(z_,i)?!1:rV.test(i)?$_[i]=!0:(z_[i]=!0,!1)}function Mr(i,t,e,r,n,o,a){this.acceptsBooleans=t===2||t===3||t===4,this.attributeName=r,this.attributeNamespace=n,this.mustUseProperty=e,this.propertyName=i,this.type=t,this.sanitizeURL=o,this.removeEmptyString=a}var lr={};"children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach(function(i){lr[i]=new Mr(i,0,!1,i,null,!1,!1)});[["acceptCharset","accept-charset"],["className","class"],["htmlFor","for"],["httpEquiv","http-equiv"]].forEach(function(i){var t=i[0];lr[t]=new Mr(t,1,!1,i[1],null,!1,!1)});["contentEditable","draggable","spellCheck","value"].forEach(function(i){lr[i]=new Mr(i,2,!1,i.toLowerCase(),null,!1,!1)});["autoReverse","externalResourcesRequired","focusable","preserveAlpha"].forEach(function(i){lr[i]=new Mr(i,2,!1,i,null,!1,!1)});"allowFullScreen async autoFocus autoPlay controls default defer disabled disablePictureInPicture disableRemotePlayback formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach(function(i){lr[i]=new Mr(i,3,!1,i.toLowerCase(),null,!1,!1)});["checked","multiple","muted","selected"].forEach(function(i){lr[i]=new Mr(i,3,!0,i,null,!1,!1)});["capture","download"].forEach(function(i){lr[i]=new Mr(i,4,!1,i,null,!1,!1)});["cols","rows","size","span"].forEach(function(i){lr[i]=new Mr(i,6,!1,i,null,!1,!1)});["rowSpan","start"].forEach(function(i){lr[i]=new Mr(i,5,!1,i.toLowerCase(),null,!1,!1)});var sw=/[\-:]([a-z])/g;function lw(i){return i[1].toUpperCase()}"accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach(function(i){var t=i.replace(sw,lw);lr[t]=new Mr(t,1,!1,i,null,!1,!1)});"xlink:actuate xlink:arcrole xlink:role xlink:show xlink:title xlink:type".split(" ").forEach(function(i){var t=i.replace(sw,lw);lr[t]=new Mr(t,1,!1,i,"http://www.w3.org/1999/xlink",!1,!1)});["xml:base","xml:lang","xml:space"].forEach(function(i){var t=i.replace(sw,lw);lr[t]=new Mr(t,1,!1,i,"http://www.w3.org/XML/1998/namespace",!1,!1)});["tabIndex","crossOrigin"].forEach(function(i){lr[i]=new Mr(i,1,!1,i.toLowerCase(),null,!1,!1)});lr.xlinkHref=new Mr("xlinkHref",1,!1,"xlink:href","http://www.w3.org/1999/xlink",!0,!1);["src","href","action","formAction"].forEach(function(i){lr[i]=new Mr(i,1,!1,i.toLowerCase(),null,!0,!0)});var Sh={animationIterationCount:!0,aspectRatio:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,columns:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridArea:!0,gridRow:!0,gridRowEnd:!0,gridRowSpan:!0,gridRowStart:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnSpan:!0,gridColumnStart:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},nV=["Webkit","ms","Moz","O"];Object.keys(Sh).forEach(function(i){nV.forEach(function(t){t=t+i.charAt(0).toUpperCase()+i.substring(1),Sh[t]=Sh[i]})});var iV=/["'&<>]/;function Rr(i){if(typeof i=="boolean"||typeof i=="number")return""+i;i=""+i;var t=iV.exec(i);if(t){var e="",r,n=0;for(r=t.index;r<i.length;r++){switch(i.charCodeAt(r)){case 34:t="&quot;";break;case 38:t="&amp;";break;case 39:t="&#x27;";break;case 60:t="&lt;";break;case 62:t="&gt;";break;default:continue}n!==r&&(e+=i.substring(n,r)),n=r+1,e+=t}i=n!==r?e+i.substring(n,r):e}return i}var oV=/([A-Z])/g,aV=/^ms-/,sy=Array.isArray;function Wi(i,t){return{insertionMode:i,selectedValue:t}}function sV(i,t,e){switch(t){case"select":return Wi(1,e.value!=null?e.value:e.defaultValue);case"svg":return Wi(2,null);case"math":return Wi(3,null);case"foreignObject":return Wi(1,null);case"table":return Wi(4,null);case"thead":case"tbody":case"tfoot":return Wi(5,null);case"colgroup":return Wi(7,null);case"tr":return Wi(6,null)}return 4<=i.insertionMode||i.insertionMode===0?Wi(1,null):i}var K_=new Map;function HR(i,t,e){if(typeof e!="object")throw Error(Ne(62));t=!0;for(var r in e)if(Vr.call(e,r)){var n=e[r];if(n!=null&&typeof n!="boolean"&&n!==""){if(r.indexOf("--")===0){var o=Rr(r);n=Rr((""+n).trim())}else{o=r;var a=K_.get(o);a!==void 0||(a=Rr(o.replace(oV,"-$1").toLowerCase().replace(aV,"-ms-")),K_.set(o,a)),o=a,n=typeof n=="number"?n===0||Vr.call(Sh,r)?""+n:n+"px":Rr((""+n).trim())}t?(t=!1,i.push(' style="',o,":",n)):i.push(";",o,":",n)}}t||i.push('"')}function tn(i,t,e,r){switch(e){case"style":HR(i,t,r);return;case"defaultValue":case"defaultChecked":case"innerHTML":case"suppressContentEditableWarning":case"suppressHydrationWarning":return}if(!(2<e.length)||e[0]!=="o"&&e[0]!=="O"||e[1]!=="n"&&e[1]!=="N"){if(t=lr.hasOwnProperty(e)?lr[e]:null,t!==null){switch(typeof r){case"function":case"symbol":return;case"boolean":if(!t.acceptsBooleans)return}switch(e=t.attributeName,t.type){case 3:r&&i.push(" ",e,'=""');break;case 4:r===!0?i.push(" ",e,'=""'):r!==!1&&i.push(" ",e,'="',Rr(r),'"');break;case 5:isNaN(r)||i.push(" ",e,'="',Rr(r),'"');break;case 6:!isNaN(r)&&1<=r&&i.push(" ",e,'="',Rr(r),'"');break;default:t.sanitizeURL&&(r=""+r),i.push(" ",e,'="',Rr(r),'"')}}else if(BR(e)){switch(typeof r){case"function":case"symbol":return;case"boolean":if(t=e.toLowerCase().slice(0,5),t!=="data-"&&t!=="aria-")return}i.push(" ",e,'="',Rr(r),'"')}}}function xh(i,t,e){if(t!=null){if(e!=null)throw Error(Ne(60));if(typeof t!="object"||!("__html"in t))throw Error(Ne(61));t=t.__html,t!=null&&i.push(""+t)}}function lV(i){var t="";return VR.Children.forEach(i,function(e){e!=null&&(t+=e)}),t}function Yg(i,t,e,r){i.push(vi(e));var n=e=null,o;for(o in t)if(Vr.call(t,o)){var a=t[o];if(a!=null)switch(o){case"children":e=a;break;case"dangerouslySetInnerHTML":n=a;break;default:tn(i,r,o,a)}}return i.push(">"),xh(i,n,e),typeof e=="string"?(i.push(Rr(e)),null):e}var uV=/^[a-zA-Z][a-zA-Z:_\.\-\d]*$/,Y_=new Map;function vi(i){var t=Y_.get(i);if(t===void 0){if(!uV.test(i))throw Error(Ne(65,i));t="<"+i,Y_.set(i,t)}return t}function cV(i,t,e,r,n){switch(t){case"select":i.push(vi("select"));var o=null,a=null;for(c in e)if(Vr.call(e,c)){var s=e[c];if(s!=null)switch(c){case"children":o=s;break;case"dangerouslySetInnerHTML":a=s;break;case"defaultValue":case"value":break;default:tn(i,r,c,s)}}return i.push(">"),xh(i,a,o),o;case"option":a=n.selectedValue,i.push(vi("option"));var l=s=null,u=null,c=null;for(o in e)if(Vr.call(e,o)){var p=e[o];if(p!=null)switch(o){case"children":s=p;break;case"selected":u=p;break;case"dangerouslySetInnerHTML":c=p;break;case"value":l=p;default:tn(i,r,o,p)}}if(a!=null)if(e=l!==null?""+l:lV(s),sy(a)){for(r=0;r<a.length;r++)if(""+a[r]===e){i.push(' selected=""');break}}else""+a===e&&i.push(' selected=""');else u&&i.push(' selected=""');return i.push(">"),xh(i,c,s),s;case"textarea":i.push(vi("textarea")),c=a=o=null;for(s in e)if(Vr.call(e,s)&&(l=e[s],l!=null))switch(s){case"children":c=l;break;case"value":o=l;break;case"defaultValue":a=l;break;case"dangerouslySetInnerHTML":throw Error(Ne(91));default:tn(i,r,s,l)}if(o===null&&a!==null&&(o=a),i.push(">"),c!=null){if(o!=null)throw Error(Ne(92));if(sy(c)&&1<c.length)throw Error(Ne(93));o=""+c}return typeof o=="string"&&o[0]===`
`&&i.push(`
`),o!==null&&i.push(Rr(""+o)),null;case"input":i.push(vi("input")),l=c=s=o=null;for(a in e)if(Vr.call(e,a)&&(u=e[a],u!=null))switch(a){case"children":case"dangerouslySetInnerHTML":throw Error(Ne(399,"input"));case"defaultChecked":l=u;break;case"defaultValue":s=u;break;case"checked":c=u;break;case"value":o=u;break;default:tn(i,r,a,u)}return c!==null?tn(i,r,"checked",c):l!==null&&tn(i,r,"checked",l),o!==null?tn(i,r,"value",o):s!==null&&tn(i,r,"value",s),i.push("/>"),null;case"menuitem":i.push(vi("menuitem"));for(var h in e)if(Vr.call(e,h)&&(o=e[h],o!=null))switch(h){case"children":case"dangerouslySetInnerHTML":throw Error(Ne(400));default:tn(i,r,h,o)}return i.push(">"),null;case"title":i.push(vi("title")),o=null;for(p in e)if(Vr.call(e,p)&&(a=e[p],a!=null))switch(p){case"children":o=a;break;case"dangerouslySetInnerHTML":throw Error(Ne(434));default:tn(i,r,p,a)}return i.push(">"),o;case"listing":case"pre":i.push(vi(t)),a=o=null;for(l in e)if(Vr.call(e,l)&&(s=e[l],s!=null))switch(l){case"children":o=s;break;case"dangerouslySetInnerHTML":a=s;break;default:tn(i,r,l,s)}if(i.push(">"),a!=null){if(o!=null)throw Error(Ne(60));if(typeof a!="object"||!("__html"in a))throw Error(Ne(61));e=a.__html,e!=null&&(typeof e=="string"&&0<e.length&&e[0]===`
`?i.push(`
`,e):i.push(""+e))}return typeof o=="string"&&o[0]===`
`&&i.push(`
`),o;case"area":case"base":case"br":case"col":case"embed":case"hr":case"img":case"keygen":case"link":case"meta":case"param":case"source":case"track":case"wbr":i.push(vi(t));for(var d in e)if(Vr.call(e,d)&&(o=e[d],o!=null))switch(d){case"children":case"dangerouslySetInnerHTML":throw Error(Ne(399,t));default:tn(i,r,d,o)}return i.push("/>"),null;case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return Yg(i,e,t,r);case"html":return n.insertionMode===0&&i.push("<!DOCTYPE html>"),Yg(i,e,t,r);default:if(t.indexOf("-")===-1&&typeof e.is!="string")return Yg(i,e,t,r);i.push(vi(t)),a=o=null;for(u in e)if(Vr.call(e,u)&&(s=e[u],s!=null))switch(u){case"children":o=s;break;case"dangerouslySetInnerHTML":a=s;break;case"style":HR(i,r,s);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":break;default:BR(u)&&typeof s!="function"&&typeof s!="symbol"&&i.push(" ",u,'="',Rr(s),'"')}return i.push(">"),xh(i,a,o),o}}function X_(i,t,e){if(i.push('<!--$?--><template id="'),e===null)throw Error(Ne(395));return i.push(e),i.push('"></template>')}function pV(i,t,e,r){switch(e.insertionMode){case 0:case 1:return i.push('<div hidden id="'),i.push(t.segmentPrefix),t=r.toString(16),i.push(t),i.push('">');case 2:return i.push('<svg aria-hidden="true" style="display:none" id="'),i.push(t.segmentPrefix),t=r.toString(16),i.push(t),i.push('">');case 3:return i.push('<math aria-hidden="true" style="display:none" id="'),i.push(t.segmentPrefix),t=r.toString(16),i.push(t),i.push('">');case 4:return i.push('<table hidden id="'),i.push(t.segmentPrefix),t=r.toString(16),i.push(t),i.push('">');case 5:return i.push('<table hidden><tbody id="'),i.push(t.segmentPrefix),t=r.toString(16),i.push(t),i.push('">');case 6:return i.push('<table hidden><tr id="'),i.push(t.segmentPrefix),t=r.toString(16),i.push(t),i.push('">');case 7:return i.push('<table hidden><colgroup id="'),i.push(t.segmentPrefix),t=r.toString(16),i.push(t),i.push('">');default:throw Error(Ne(397))}}function hV(i,t){switch(t.insertionMode){case 0:case 1:return i.push("</div>");case 2:return i.push("</svg>");case 3:return i.push("</math>");case 4:return i.push("</table>");case 5:return i.push("</tbody></table>");case 6:return i.push("</tr></table>");case 7:return i.push("</colgroup></table>");default:throw Error(Ne(397))}}var dV=/[<\u2028\u2029]/g;function Xg(i){return JSON.stringify(i).replace(dV,function(t){switch(t){case"<":return"\\u003c";case"\u2028":return"\\u2028";case"\u2029":return"\\u2029";default:throw Error("escapeJSStringsForInstructionScripts encountered a match it does not know how to replace. this means the match regex and the replacement characters are no longer in sync. This is a bug in React")}})}function fV(i,t){return t=t===void 0?"":t,{bootstrapChunks:[],startInlineScript:"<script>",placeholderPrefix:t+"P:",segmentPrefix:t+"S:",boundaryPrefix:t+"B:",idPrefix:t,nextSuspenseID:0,sentCompleteSegmentFunction:!1,sentCompleteBoundaryFunction:!1,sentClientRenderFunction:!1,generateStaticMarkup:i}}function q_(i,t,e,r){return e.generateStaticMarkup?(i.push(Rr(t)),!1):(t===""?i=r:(r&&i.push("<!-- -->"),i.push(Rr(t)),i=!0),i)}var cc=Object.assign,gV=Symbol.for("react.element"),jR=Symbol.for("react.portal"),UR=Symbol.for("react.fragment"),zR=Symbol.for("react.strict_mode"),$R=Symbol.for("react.profiler"),KR=Symbol.for("react.provider"),YR=Symbol.for("react.context"),XR=Symbol.for("react.forward_ref"),qR=Symbol.for("react.suspense"),ZR=Symbol.for("react.suspense_list"),QR=Symbol.for("react.memo"),uw=Symbol.for("react.lazy"),vV=Symbol.for("react.scope"),mV=Symbol.for("react.debug_trace_mode"),yV=Symbol.for("react.legacy_hidden"),CV=Symbol.for("react.default_value"),Z_=Symbol.iterator;function ly(i){if(i==null)return null;if(typeof i=="function")return i.displayName||i.name||null;if(typeof i=="string")return i;switch(i){case UR:return"Fragment";case jR:return"Portal";case $R:return"Profiler";case zR:return"StrictMode";case qR:return"Suspense";case ZR:return"SuspenseList"}if(typeof i=="object")switch(i.$$typeof){case YR:return(i.displayName||"Context")+".Consumer";case KR:return(i._context.displayName||"Context")+".Provider";case XR:var t=i.render;return i=i.displayName,i||(i=t.displayName||t.name||"",i=i!==""?"ForwardRef("+i+")":"ForwardRef"),i;case QR:return t=i.displayName||null,t!==null?t:ly(i.type)||"Memo";case uw:t=i._payload,i=i._init;try{return ly(i(t))}catch{}}return null}var JR={};function Q_(i,t){if(i=i.contextTypes,!i)return JR;var e={},r;for(r in i)e[r]=t[r];return e}var Ya=null;function cf(i,t){if(i!==t){i.context._currentValue2=i.parentValue,i=i.parent;var e=t.parent;if(i===null){if(e!==null)throw Error(Ne(401))}else{if(e===null)throw Error(Ne(401));cf(i,e)}t.context._currentValue2=t.value}}function eP(i){i.context._currentValue2=i.parentValue,i=i.parent,i!==null&&eP(i)}function tP(i){var t=i.parent;t!==null&&tP(t),i.context._currentValue2=i.value}function rP(i,t){if(i.context._currentValue2=i.parentValue,i=i.parent,i===null)throw Error(Ne(402));i.depth===t.depth?cf(i,t):rP(i,t)}function nP(i,t){var e=t.parent;if(e===null)throw Error(Ne(402));i.depth===e.depth?cf(i,e):nP(i,e),t.context._currentValue2=t.value}function ud(i){var t=Ya;t!==i&&(t===null?tP(i):i===null?eP(t):t.depth===i.depth?cf(t,i):t.depth>i.depth?rP(t,i):nP(t,i),Ya=i)}var J_={isMounted:function(){return!1},enqueueSetState:function(i,t){i=i._reactInternals,i.queue!==null&&i.queue.push(t)},enqueueReplaceState:function(i,t){i=i._reactInternals,i.replace=!0,i.queue=[t]},enqueueForceUpdate:function(){}};function eS(i,t,e,r){var n=i.state!==void 0?i.state:null;i.updater=J_,i.props=e,i.state=n;var o={queue:[],replace:!1};i._reactInternals=o;var a=t.contextType;if(i.context=typeof a=="object"&&a!==null?a._currentValue2:r,a=t.getDerivedStateFromProps,typeof a=="function"&&(a=a(e,n),n=a==null?n:cc({},n,a),i.state=n),typeof t.getDerivedStateFromProps!="function"&&typeof i.getSnapshotBeforeUpdate!="function"&&(typeof i.UNSAFE_componentWillMount=="function"||typeof i.componentWillMount=="function"))if(t=i.state,typeof i.componentWillMount=="function"&&i.componentWillMount(),typeof i.UNSAFE_componentWillMount=="function"&&i.UNSAFE_componentWillMount(),t!==i.state&&J_.enqueueReplaceState(i,i.state,null),o.queue!==null&&0<o.queue.length)if(t=o.queue,a=o.replace,o.queue=null,o.replace=!1,a&&t.length===1)i.state=t[0];else{for(o=a?t[0]:i.state,n=!0,a=a?1:0;a<t.length;a++){var s=t[a];s=typeof s=="function"?s.call(i,o,e,r):s,s!=null&&(n?(n=!1,o=cc({},o,s)):cc(o,s))}i.state=o}else o.queue=null}var wV={id:1,overflow:""};function uy(i,t,e){var r=i.id;i=i.overflow;var n=32-Th(r)-1;r&=~(1<<n),e+=1;var o=32-Th(t)+n;if(30<o){var a=n-n%5;return o=(r&(1<<a)-1).toString(32),r>>=a,n-=a,{id:1<<32-Th(t)+n|e<<n|r,overflow:o+i}}return{id:1<<o|e<<n|r,overflow:i}}var Th=Math.clz32?Math.clz32:SV,bV=Math.log,_V=Math.LN2;function SV(i){return i>>>=0,i===0?32:31-(bV(i)/_V|0)|0}function xV(i,t){return i===t&&(i!==0||1/i===1/t)||i!==i&&t!==t}var TV=typeof Object.is=="function"?Object.is:xV,po=null,cw=null,Oh=null,vt=null,Ku=!1,cd=!1,Ec=0,Yo=null,pf=0;function Va(){if(po===null)throw Error(Ne(321));return po}function tS(){if(0<pf)throw Error(Ne(312));return{memoizedState:null,queue:null,next:null}}function pw(){return vt===null?Oh===null?(Ku=!1,Oh=vt=tS()):(Ku=!0,vt=Oh):vt.next===null?(Ku=!1,vt=vt.next=tS()):(Ku=!0,vt=vt.next),vt}function hw(){cw=po=null,cd=!1,Oh=null,pf=0,vt=Yo=null}function iP(i,t){return typeof t=="function"?t(i):t}function rS(i,t,e){if(po=Va(),vt=pw(),Ku){var r=vt.queue;if(t=r.dispatch,Yo!==null&&(e=Yo.get(r),e!==void 0)){Yo.delete(r),r=vt.memoizedState;do r=i(r,e.action),e=e.next;while(e!==null);return vt.memoizedState=r,[r,t]}return[vt.memoizedState,t]}return i=i===iP?typeof t=="function"?t():t:e!==void 0?e(t):t,vt.memoizedState=i,i=vt.queue={last:null,dispatch:null},i=i.dispatch=OV.bind(null,po,i),[vt.memoizedState,i]}function nS(i,t){if(po=Va(),vt=pw(),t=t===void 0?null:t,vt!==null){var e=vt.memoizedState;if(e!==null&&t!==null){var r=e[1];e:if(r===null)r=!1;else{for(var n=0;n<r.length&&n<t.length;n++)if(!TV(t[n],r[n])){r=!1;break e}r=!0}if(r)return e[0]}}return i=i(),vt.memoizedState=[i,t],i}function OV(i,t,e){if(25<=pf)throw Error(Ne(301));if(i===po)if(cd=!0,i={action:e,next:null},Yo===null&&(Yo=new Map),e=Yo.get(t),e===void 0)Yo.set(t,i);else{for(t=e;t.next!==null;)t=t.next;t.next=i}}function EV(){throw Error(Ne(394))}function Mp(){}var iS={readContext:function(i){return i._currentValue2},useContext:function(i){return Va(),i._currentValue2},useMemo:nS,useReducer:rS,useRef:function(i){po=Va(),vt=pw();var t=vt.memoizedState;return t===null?(i={current:i},vt.memoizedState=i):t},useState:function(i){return rS(iP,i)},useInsertionEffect:Mp,useLayoutEffect:function(){},useCallback:function(i,t){return nS(function(){return i},t)},useImperativeHandle:Mp,useEffect:Mp,useDebugValue:Mp,useDeferredValue:function(i){return Va(),i},useTransition:function(){return Va(),[!1,EV]},useId:function(){var i=cw.treeContext,t=i.overflow;i=i.id,i=(i&~(1<<32-Th(i)-1)).toString(32)+t;var e=Eh;if(e===null)throw Error(Ne(404));return t=Ec++,i=":"+e.idPrefix+"R"+i,0<t&&(i+="H"+t.toString(32)),i+":"},useMutableSource:function(i,t){return Va(),t(i._source)},useSyncExternalStore:function(i,t,e){if(e===void 0)throw Error(Ne(407));return e()}},Eh=null,qg=VR.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentDispatcher;function RV(i){return console.error(i),null}function Yu(){}function PV(i,t,e,r,n,o,a,s,l){var u=[],c=new Set;return t={destination:null,responseState:t,progressiveChunkSize:r===void 0?12800:r,status:0,fatalError:null,nextSegmentId:0,allPendingTasks:0,pendingRootTasks:0,completedRootSegment:null,abortableTasks:c,pingedTasks:u,clientRenderedBoundaries:[],completedBoundaries:[],partialBoundaries:[],onError:n===void 0?RV:n,onAllReady:o===void 0?Yu:o,onShellReady:a===void 0?Yu:a,onShellError:s===void 0?Yu:s,onFatalError:l===void 0?Yu:l},e=pd(t,0,null,e,!1,!1),e.parentFlushed=!0,i=dw(t,i,null,e,c,JR,null,wV),u.push(i),t}function dw(i,t,e,r,n,o,a,s){i.allPendingTasks++,e===null?i.pendingRootTasks++:e.pendingTasks++;var l={node:t,ping:function(){var u=i.pingedTasks;u.push(l),u.length===1&&sP(i)},blockedBoundary:e,blockedSegment:r,abortSet:n,legacyContext:o,context:a,treeContext:s};return n.add(l),l}function pd(i,t,e,r,n,o){return{status:0,id:-1,index:t,parentFlushed:!1,chunks:[],children:[],formatContext:r,boundary:e,lastPushedText:n,textEmbedded:o}}function Rc(i,t){if(i=i.onError(t),i!=null&&typeof i!="string")throw Error('onError returned something with a type other than "string". onError should return a string and may return null or undefined but must not return anything else. It received something of type "'+typeof i+'" instead');return i}function hd(i,t){var e=i.onShellError;e(t),e=i.onFatalError,e(t),i.destination!==null?(i.status=2,i.destination.destroy(t)):(i.status=1,i.fatalError=t)}function oS(i,t,e,r,n){for(po={},cw=t,Ec=0,i=e(r,n);cd;)cd=!1,Ec=0,pf+=1,vt=null,i=e(r,n);return hw(),i}function aS(i,t,e,r){var n=e.render(),o=r.childContextTypes;if(o!=null){var a=t.legacyContext;if(typeof e.getChildContext!="function")r=a;else{e=e.getChildContext();for(var s in e)if(!(s in o))throw Error(Ne(108,ly(r)||"Unknown",s));r=cc({},a,e)}t.legacyContext=r,sn(i,t,n),t.legacyContext=a}else sn(i,t,n)}function sS(i,t){if(i&&i.defaultProps){t=cc({},t),i=i.defaultProps;for(var e in i)t[e]===void 0&&(t[e]=i[e]);return t}return t}function cy(i,t,e,r,n){if(typeof e=="function")if(e.prototype&&e.prototype.isReactComponent){n=Q_(e,t.legacyContext);var o=e.contextType;o=new e(r,typeof o=="object"&&o!==null?o._currentValue2:n),eS(o,e,r,n),aS(i,t,o,e)}else{o=Q_(e,t.legacyContext),n=oS(i,t,e,r,o);var a=Ec!==0;if(typeof n=="object"&&n!==null&&typeof n.render=="function"&&n.$$typeof===void 0)eS(n,e,r,o),aS(i,t,n,e);else if(a){r=t.treeContext,t.treeContext=uy(r,1,0);try{sn(i,t,n)}finally{t.treeContext=r}}else sn(i,t,n)}else if(typeof e=="string"){switch(n=t.blockedSegment,o=cV(n.chunks,e,r,i.responseState,n.formatContext),n.lastPushedText=!1,a=n.formatContext,n.formatContext=sV(a,e,r),py(i,t,o),n.formatContext=a,e){case"area":case"base":case"br":case"col":case"embed":case"hr":case"img":case"input":case"keygen":case"link":case"meta":case"param":case"source":case"track":case"wbr":break;default:n.chunks.push("</",e,">")}n.lastPushedText=!1}else{switch(e){case yV:case mV:case zR:case $R:case UR:sn(i,t,r.children);return;case ZR:sn(i,t,r.children);return;case vV:throw Error(Ne(343));case qR:e:{e=t.blockedBoundary,n=t.blockedSegment,o=r.fallback,r=r.children,a=new Set;var s={id:null,rootSegmentID:-1,parentFlushed:!1,pendingTasks:0,forceClientRender:!1,completedSegments:[],byteSize:0,fallbackAbortableTasks:a,errorDigest:null},l=pd(i,n.chunks.length,s,n.formatContext,!1,!1);n.children.push(l),n.lastPushedText=!1;var u=pd(i,0,null,n.formatContext,!1,!1);u.parentFlushed=!0,t.blockedBoundary=s,t.blockedSegment=u;try{if(py(i,t,r),i.responseState.generateStaticMarkup||u.lastPushedText&&u.textEmbedded&&u.chunks.push("<!-- -->"),u.status=1,dd(s,u),s.pendingTasks===0)break e}catch(c){u.status=4,s.forceClientRender=!0,s.errorDigest=Rc(i,c)}finally{t.blockedBoundary=e,t.blockedSegment=n}t=dw(i,o,e,l,a,t.legacyContext,t.context,t.treeContext),i.pingedTasks.push(t)}return}if(typeof e=="object"&&e!==null)switch(e.$$typeof){case XR:if(r=oS(i,t,e.render,r,n),Ec!==0){e=t.treeContext,t.treeContext=uy(e,1,0);try{sn(i,t,r)}finally{t.treeContext=e}}else sn(i,t,r);return;case QR:e=e.type,r=sS(e,r),cy(i,t,e,r,n);return;case KR:if(n=r.children,e=e._context,r=r.value,o=e._currentValue2,e._currentValue2=r,a=Ya,Ya=r={parent:a,depth:a===null?0:a.depth+1,context:e,parentValue:o,value:r},t.context=r,sn(i,t,n),i=Ya,i===null)throw Error(Ne(403));r=i.parentValue,i.context._currentValue2=r===CV?i.context._defaultValue:r,i=Ya=i.parent,t.context=i;return;case YR:r=r.children,r=r(e._currentValue2),sn(i,t,r);return;case uw:n=e._init,e=n(e._payload),r=sS(e,r),cy(i,t,e,r,void 0);return}throw Error(Ne(130,e==null?e:typeof e,""))}}function sn(i,t,e){if(t.node=e,typeof e=="object"&&e!==null){switch(e.$$typeof){case gV:cy(i,t,e.type,e.props,e.ref);return;case jR:throw Error(Ne(257));case uw:var r=e._init;e=r(e._payload),sn(i,t,e);return}if(sy(e)){lS(i,t,e);return}if(e===null||typeof e!="object"?r=null:(r=Z_&&e[Z_]||e["@@iterator"],r=typeof r=="function"?r:null),r&&(r=r.call(e))){if(e=r.next(),!e.done){var n=[];do n.push(e.value),e=r.next();while(!e.done);lS(i,t,n)}return}throw i=Object.prototype.toString.call(e),Error(Ne(31,i==="[object Object]"?"object with keys {"+Object.keys(e).join(", ")+"}":i))}typeof e=="string"?(r=t.blockedSegment,r.lastPushedText=q_(t.blockedSegment.chunks,e,i.responseState,r.lastPushedText)):typeof e=="number"&&(r=t.blockedSegment,r.lastPushedText=q_(t.blockedSegment.chunks,""+e,i.responseState,r.lastPushedText))}function lS(i,t,e){for(var r=e.length,n=0;n<r;n++){var o=t.treeContext;t.treeContext=uy(o,r,n);try{py(i,t,e[n])}finally{t.treeContext=o}}}function py(i,t,e){var r=t.blockedSegment.formatContext,n=t.legacyContext,o=t.context;try{return sn(i,t,e)}catch(l){if(hw(),typeof l=="object"&&l!==null&&typeof l.then=="function"){e=l;var a=t.blockedSegment,s=pd(i,a.chunks.length,null,a.formatContext,a.lastPushedText,!0);a.children.push(s),a.lastPushedText=!1,i=dw(i,t.node,t.blockedBoundary,s,t.abortSet,t.legacyContext,t.context,t.treeContext).ping,e.then(i,i),t.blockedSegment.formatContext=r,t.legacyContext=n,t.context=o,ud(o)}else throw t.blockedSegment.formatContext=r,t.legacyContext=n,t.context=o,ud(o),l}}function AV(i){var t=i.blockedBoundary;i=i.blockedSegment,i.status=3,aP(this,t,i)}function oP(i,t,e){var r=i.blockedBoundary;i.blockedSegment.status=3,r===null?(t.allPendingTasks--,t.status!==2&&(t.status=2,t.destination!==null&&t.destination.push(null))):(r.pendingTasks--,r.forceClientRender||(r.forceClientRender=!0,i=e===void 0?Error(Ne(432)):e,r.errorDigest=t.onError(i),r.parentFlushed&&t.clientRenderedBoundaries.push(r)),r.fallbackAbortableTasks.forEach(function(n){return oP(n,t,e)}),r.fallbackAbortableTasks.clear(),t.allPendingTasks--,t.allPendingTasks===0&&(r=t.onAllReady,r()))}function dd(i,t){if(t.chunks.length===0&&t.children.length===1&&t.children[0].boundary===null){var e=t.children[0];e.id=t.id,e.parentFlushed=!0,e.status===1&&dd(i,e)}else i.completedSegments.push(t)}function aP(i,t,e){if(t===null){if(e.parentFlushed){if(i.completedRootSegment!==null)throw Error(Ne(389));i.completedRootSegment=e}i.pendingRootTasks--,i.pendingRootTasks===0&&(i.onShellError=Yu,t=i.onShellReady,t())}else t.pendingTasks--,t.forceClientRender||(t.pendingTasks===0?(e.parentFlushed&&e.status===1&&dd(t,e),t.parentFlushed&&i.completedBoundaries.push(t),t.fallbackAbortableTasks.forEach(AV,i),t.fallbackAbortableTasks.clear()):e.parentFlushed&&e.status===1&&(dd(t,e),t.completedSegments.length===1&&t.parentFlushed&&i.partialBoundaries.push(t)));i.allPendingTasks--,i.allPendingTasks===0&&(i=i.onAllReady,i())}function sP(i){if(i.status!==2){var t=Ya,e=qg.current;qg.current=iS;var r=Eh;Eh=i.responseState;try{var n=i.pingedTasks,o;for(o=0;o<n.length;o++){var a=n[o],s=i,l=a.blockedSegment;if(l.status===0){ud(a.context);try{sn(s,a,a.node),s.responseState.generateStaticMarkup||l.lastPushedText&&l.textEmbedded&&l.chunks.push("<!-- -->"),a.abortSet.delete(a),l.status=1,aP(s,a.blockedBoundary,l)}catch(f){if(hw(),typeof f=="object"&&f!==null&&typeof f.then=="function"){var u=a.ping;f.then(u,u)}else{a.abortSet.delete(a),l.status=4;var c=a.blockedBoundary,p=f,h=Rc(s,p);if(c===null?hd(s,p):(c.pendingTasks--,c.forceClientRender||(c.forceClientRender=!0,c.errorDigest=h,c.parentFlushed&&s.clientRenderedBoundaries.push(c))),s.allPendingTasks--,s.allPendingTasks===0){var d=s.onAllReady;d()}}}finally{}}}n.splice(0,o),i.destination!==null&&fw(i,i.destination)}catch(f){Rc(i,f),hd(i,f)}finally{Eh=r,qg.current=e,e===iS&&ud(t)}}}function Fp(i,t,e){switch(e.parentFlushed=!0,e.status){case 0:var r=e.id=i.nextSegmentId++;return e.lastPushedText=!1,e.textEmbedded=!1,i=i.responseState,t.push('<template id="'),t.push(i.placeholderPrefix),i=r.toString(16),t.push(i),t.push('"></template>');case 1:e.status=2;var n=!0;r=e.chunks;var o=0;e=e.children;for(var a=0;a<e.length;a++){for(n=e[a];o<n.index;o++)t.push(r[o]);n=hf(i,t,n)}for(;o<r.length-1;o++)t.push(r[o]);return o<r.length&&(n=t.push(r[o])),n;default:throw Error(Ne(390))}}function hf(i,t,e){var r=e.boundary;if(r===null)return Fp(i,t,e);if(r.parentFlushed=!0,r.forceClientRender)return i.responseState.generateStaticMarkup||(r=r.errorDigest,t.push("<!--$!-->"),t.push("<template"),r&&(t.push(' data-dgst="'),r=Rr(r),t.push(r),t.push('"')),t.push("></template>")),Fp(i,t,e),i=i.responseState.generateStaticMarkup?!0:t.push("<!--/$-->"),i;if(0<r.pendingTasks){r.rootSegmentID=i.nextSegmentId++,0<r.completedSegments.length&&i.partialBoundaries.push(r);var n=i.responseState,o=n.nextSuspenseID++;return n=n.boundaryPrefix+o.toString(16),r=r.id=n,X_(t,i.responseState,r),Fp(i,t,e),t.push("<!--/$-->")}if(r.byteSize>i.progressiveChunkSize)return r.rootSegmentID=i.nextSegmentId++,i.completedBoundaries.push(r),X_(t,i.responseState,r.id),Fp(i,t,e),t.push("<!--/$-->");if(i.responseState.generateStaticMarkup||t.push("<!--$-->"),e=r.completedSegments,e.length!==1)throw Error(Ne(391));return hf(i,t,e[0]),i=i.responseState.generateStaticMarkup?!0:t.push("<!--/$-->"),i}function uS(i,t,e){return pV(t,i.responseState,e.formatContext,e.id),hf(i,t,e),hV(t,e.formatContext)}function cS(i,t,e){for(var r=e.completedSegments,n=0;n<r.length;n++)lP(i,t,e,r[n]);if(r.length=0,i=i.responseState,r=e.id,e=e.rootSegmentID,t.push(i.startInlineScript),i.sentCompleteBoundaryFunction?t.push('$RC("'):(i.sentCompleteBoundaryFunction=!0,t.push('function $RC(a,b){a=document.getElementById(a);b=document.getElementById(b);b.parentNode.removeChild(b);if(a){a=a.previousSibling;var f=a.parentNode,c=a.nextSibling,e=0;do{if(c&&8===c.nodeType){var d=c.data;if("/$"===d)if(0===e)break;else e--;else"$"!==d&&"$?"!==d&&"$!"!==d||e++}d=c.nextSibling;f.removeChild(c);c=d}while(c);for(;b.firstChild;)f.insertBefore(b.firstChild,c);a.data="$";a._reactRetry&&a._reactRetry()}};$RC("')),r===null)throw Error(Ne(395));return e=e.toString(16),t.push(r),t.push('","'),t.push(i.segmentPrefix),t.push(e),t.push('")<\/script>')}function lP(i,t,e,r){if(r.status===2)return!0;var n=r.id;if(n===-1){if((r.id=e.rootSegmentID)===-1)throw Error(Ne(392));return uS(i,t,r)}return uS(i,t,r),i=i.responseState,t.push(i.startInlineScript),i.sentCompleteSegmentFunction?t.push('$RS("'):(i.sentCompleteSegmentFunction=!0,t.push('function $RS(a,b){a=document.getElementById(a);b=document.getElementById(b);for(a.parentNode.removeChild(a);a.firstChild;)b.parentNode.insertBefore(a.firstChild,b);b.parentNode.removeChild(b)};$RS("')),t.push(i.segmentPrefix),n=n.toString(16),t.push(n),t.push('","'),t.push(i.placeholderPrefix),t.push(n),t.push('")<\/script>')}function fw(i,t){try{var e=i.completedRootSegment;if(e!==null&&i.pendingRootTasks===0){hf(i,t,e),i.completedRootSegment=null;var r=i.responseState.bootstrapChunks;for(e=0;e<r.length-1;e++)t.push(r[e]);e<r.length&&t.push(r[e])}var n=i.clientRenderedBoundaries,o;for(o=0;o<n.length;o++){var a=n[o];r=t;var s=i.responseState,l=a.id,u=a.errorDigest,c=a.errorMessage,p=a.errorComponentStack;if(r.push(s.startInlineScript),s.sentClientRenderFunction?r.push('$RX("'):(s.sentClientRenderFunction=!0,r.push('function $RX(b,c,d,e){var a=document.getElementById(b);a&&(b=a.previousSibling,b.data="$!",a=a.dataset,c&&(a.dgst=c),d&&(a.msg=d),e&&(a.stck=e),b._reactRetry&&b._reactRetry())};$RX("')),l===null)throw Error(Ne(395));if(r.push(l),r.push('"'),u||c||p){r.push(",");var h=Xg(u||"");r.push(h)}if(c||p){r.push(",");var d=Xg(c||"");r.push(d)}if(p){r.push(",");var f=Xg(p);r.push(f)}if(!r.push(")<\/script>")){i.destination=null,o++,n.splice(0,o);return}}n.splice(0,o);var g=i.completedBoundaries;for(o=0;o<g.length;o++)if(!cS(i,t,g[o])){i.destination=null,o++,g.splice(0,o);return}g.splice(0,o);var v=i.partialBoundaries;for(o=0;o<v.length;o++){var m=v[o];e:{n=i,a=t;var C=m.completedSegments;for(s=0;s<C.length;s++)if(!lP(n,a,m,C[s])){s++,C.splice(0,s);var w=!1;break e}C.splice(0,s),w=!0}if(!w){i.destination=null,o++,v.splice(0,o);return}}v.splice(0,o);var b=i.completedBoundaries;for(o=0;o<b.length;o++)if(!cS(i,t,b[o])){i.destination=null,o++,b.splice(0,o);return}b.splice(0,o)}finally{i.allPendingTasks===0&&i.pingedTasks.length===0&&i.clientRenderedBoundaries.length===0&&i.completedBoundaries.length===0&&t.push(null)}}function DV(i,t){try{var e=i.abortableTasks;e.forEach(function(r){return oP(r,i,t)}),e.clear(),i.destination!==null&&fw(i,i.destination)}catch(r){Rc(i,r),hd(i,r)}}function MV(){}function uP(i,t,e,r){var n=!1,o=null,a="",s={push:function(u){return u!==null&&(a+=u),!0},destroy:function(u){n=!0,o=u}},l=!1;if(i=PV(i,fV(e,t?t.identifierPrefix:void 0),{insertionMode:1,selectedValue:null},1/0,MV,void 0,function(){l=!0},void 0,void 0),sP(i),DV(i,r),i.status===1)i.status=2,s.destroy(i.fatalError);else if(i.status!==2&&i.destination===null){i.destination=s;try{fw(i,s)}catch(u){Rc(i,u),hd(i,u)}}if(n)throw o;if(!l)throw Error(Ne(426));return a}zl.renderToNodeStream=function(){throw Error(Ne(207))};zl.renderToStaticMarkup=function(i,t){return uP(i,t,!0,'The server used "renderToStaticMarkup" which does not support Suspense. If you intended to have the server wait for the suspended component please switch to "renderToReadableStream" which supports Suspense on the server')};zl.renderToStaticNodeStream=function(){throw Error(Ne(208))};zl.renderToString=function(i,t){return uP(i,t,!1,'The server used "renderToString" which does not support Suspense. If you intended for this Suspense boundary to render the fallback content on the server consider throwing an Error somewhere within the Suspense boundary. If you intended to have the server wait for the suspended component please switch to "renderToReadableStream" which supports Suspense on the server')};zl.version="18.2.0";var gw={};/**
 * @license React
 * react-dom-server.browser.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var cP=we.exports;function Ve(i){for(var t="https://reactjs.org/docs/error-decoder.html?invariant="+i,e=1;e<arguments.length;e++)t+="&args[]="+encodeURIComponent(arguments[e]);return"Minified React error #"+i+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}var ln=null,un=0;function ve(i,t){if(t.length!==0)if(512<t.length)0<un&&(i.enqueue(new Uint8Array(ln.buffer,0,un)),ln=new Uint8Array(512),un=0),i.enqueue(t);else{var e=ln.length-un;e<t.length&&(e===0?i.enqueue(ln):(ln.set(t.subarray(0,e),un),i.enqueue(ln),t=t.subarray(e)),ln=new Uint8Array(512),un=0),ln.set(t,un),un+=t.length}}function bt(i,t){return ve(i,t),!0}function pS(i){ln&&0<un&&(i.enqueue(new Uint8Array(ln.buffer,0,un)),ln=null,un=0)}var pP=new TextEncoder;function Be(i){return pP.encode(i)}function he(i){return pP.encode(i)}function hP(i,t){typeof i.error=="function"?i.error(t):i.close()}var Br=Object.prototype.hasOwnProperty,FV=/^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/,hS={},dS={};function dP(i){return Br.call(dS,i)?!0:Br.call(hS,i)?!1:FV.test(i)?dS[i]=!0:(hS[i]=!0,!1)}function Fr(i,t,e,r,n,o,a){this.acceptsBooleans=t===2||t===3||t===4,this.attributeName=r,this.attributeNamespace=n,this.mustUseProperty=e,this.propertyName=i,this.type=t,this.sanitizeURL=o,this.removeEmptyString=a}var ur={};"children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach(function(i){ur[i]=new Fr(i,0,!1,i,null,!1,!1)});[["acceptCharset","accept-charset"],["className","class"],["htmlFor","for"],["httpEquiv","http-equiv"]].forEach(function(i){var t=i[0];ur[t]=new Fr(t,1,!1,i[1],null,!1,!1)});["contentEditable","draggable","spellCheck","value"].forEach(function(i){ur[i]=new Fr(i,2,!1,i.toLowerCase(),null,!1,!1)});["autoReverse","externalResourcesRequired","focusable","preserveAlpha"].forEach(function(i){ur[i]=new Fr(i,2,!1,i,null,!1,!1)});"allowFullScreen async autoFocus autoPlay controls default defer disabled disablePictureInPicture disableRemotePlayback formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach(function(i){ur[i]=new Fr(i,3,!1,i.toLowerCase(),null,!1,!1)});["checked","multiple","muted","selected"].forEach(function(i){ur[i]=new Fr(i,3,!0,i,null,!1,!1)});["capture","download"].forEach(function(i){ur[i]=new Fr(i,4,!1,i,null,!1,!1)});["cols","rows","size","span"].forEach(function(i){ur[i]=new Fr(i,6,!1,i,null,!1,!1)});["rowSpan","start"].forEach(function(i){ur[i]=new Fr(i,5,!1,i.toLowerCase(),null,!1,!1)});var vw=/[\-:]([a-z])/g;function mw(i){return i[1].toUpperCase()}"accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach(function(i){var t=i.replace(vw,mw);ur[t]=new Fr(t,1,!1,i,null,!1,!1)});"xlink:actuate xlink:arcrole xlink:role xlink:show xlink:title xlink:type".split(" ").forEach(function(i){var t=i.replace(vw,mw);ur[t]=new Fr(t,1,!1,i,"http://www.w3.org/1999/xlink",!1,!1)});["xml:base","xml:lang","xml:space"].forEach(function(i){var t=i.replace(vw,mw);ur[t]=new Fr(t,1,!1,i,"http://www.w3.org/XML/1998/namespace",!1,!1)});["tabIndex","crossOrigin"].forEach(function(i){ur[i]=new Fr(i,1,!1,i.toLowerCase(),null,!1,!1)});ur.xlinkHref=new Fr("xlinkHref",1,!1,"xlink:href","http://www.w3.org/1999/xlink",!0,!1);["src","href","action","formAction"].forEach(function(i){ur[i]=new Fr(i,1,!1,i.toLowerCase(),null,!0,!0)});var Rh={animationIterationCount:!0,aspectRatio:!0,borderImageOutset:!0,borderImageSlice:!0,borderImageWidth:!0,boxFlex:!0,boxFlexGroup:!0,boxOrdinalGroup:!0,columnCount:!0,columns:!0,flex:!0,flexGrow:!0,flexPositive:!0,flexShrink:!0,flexNegative:!0,flexOrder:!0,gridArea:!0,gridRow:!0,gridRowEnd:!0,gridRowSpan:!0,gridRowStart:!0,gridColumn:!0,gridColumnEnd:!0,gridColumnSpan:!0,gridColumnStart:!0,fontWeight:!0,lineClamp:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,tabSize:!0,widows:!0,zIndex:!0,zoom:!0,fillOpacity:!0,floodOpacity:!0,stopOpacity:!0,strokeDasharray:!0,strokeDashoffset:!0,strokeMiterlimit:!0,strokeOpacity:!0,strokeWidth:!0},IV=["Webkit","ms","Moz","O"];Object.keys(Rh).forEach(function(i){IV.forEach(function(t){t=t+i.charAt(0).toUpperCase()+i.substring(1),Rh[t]=Rh[i]})});var LV=/["'&<>]/;function ar(i){if(typeof i=="boolean"||typeof i=="number")return""+i;i=""+i;var t=LV.exec(i);if(t){var e="",r,n=0;for(r=t.index;r<i.length;r++){switch(i.charCodeAt(r)){case 34:t="&quot;";break;case 38:t="&amp;";break;case 39:t="&#x27;";break;case 60:t="&lt;";break;case 62:t="&gt;";break;default:continue}n!==r&&(e+=i.substring(n,r)),n=r+1,e+=t}i=n!==r?e+i.substring(n,r):e}return i}var NV=/([A-Z])/g,GV=/^ms-/,hy=Array.isArray,kV=he("<script>"),WV=he("<\/script>"),VV=he('<script src="'),BV=he('<script type="module" src="'),fS=he('" async=""><\/script>'),HV=/(<\/|<)(s)(cript)/gi;function jV(i,t,e,r){return""+t+(e==="s"?"\\u0073":"\\u0053")+r}function UV(i,t,e,r,n){i=i===void 0?"":i,t=t===void 0?kV:he('<script nonce="'+ar(t)+'">');var o=[];if(e!==void 0&&o.push(t,Be((""+e).replace(HV,jV)),WV),r!==void 0)for(e=0;e<r.length;e++)o.push(VV,Be(ar(r[e])),fS);if(n!==void 0)for(r=0;r<n.length;r++)o.push(BV,Be(ar(n[r])),fS);return{bootstrapChunks:o,startInlineScript:t,placeholderPrefix:he(i+"P:"),segmentPrefix:he(i+"S:"),boundaryPrefix:i+"B:",idPrefix:i,nextSuspenseID:0,sentCompleteSegmentFunction:!1,sentCompleteBoundaryFunction:!1,sentClientRenderFunction:!1}}function mi(i,t){return{insertionMode:i,selectedValue:t}}function zV(i){return mi(i==="http://www.w3.org/2000/svg"?2:i==="http://www.w3.org/1998/Math/MathML"?3:0,null)}function $V(i,t,e){switch(t){case"select":return mi(1,e.value!=null?e.value:e.defaultValue);case"svg":return mi(2,null);case"math":return mi(3,null);case"foreignObject":return mi(1,null);case"table":return mi(4,null);case"thead":case"tbody":case"tfoot":return mi(5,null);case"colgroup":return mi(7,null);case"tr":return mi(6,null)}return 4<=i.insertionMode||i.insertionMode===0?mi(1,null):i}var yw=he("<!-- -->");function gS(i,t,e,r){return t===""?r:(r&&i.push(yw),i.push(Be(ar(t))),!0)}var vS=new Map,KV=he(' style="'),mS=he(":"),YV=he(";");function fP(i,t,e){if(typeof e!="object")throw Error(Ve(62));t=!0;for(var r in e)if(Br.call(e,r)){var n=e[r];if(n!=null&&typeof n!="boolean"&&n!==""){if(r.indexOf("--")===0){var o=Be(ar(r));n=Be(ar((""+n).trim()))}else{o=r;var a=vS.get(o);a!==void 0||(a=he(ar(o.replace(NV,"-$1").toLowerCase().replace(GV,"-ms-"))),vS.set(o,a)),o=a,n=typeof n=="number"?n===0||Br.call(Rh,r)?Be(""+n):Be(n+"px"):Be(ar((""+n).trim()))}t?(t=!1,i.push(KV,o,mS,n)):i.push(YV,o,mS,n)}}t||i.push(Ba)}var Mo=he(" "),ll=he('="'),Ba=he('"'),yS=he('=""');function rn(i,t,e,r){switch(e){case"style":fP(i,t,r);return;case"defaultValue":case"defaultChecked":case"innerHTML":case"suppressContentEditableWarning":case"suppressHydrationWarning":return}if(!(2<e.length)||e[0]!=="o"&&e[0]!=="O"||e[1]!=="n"&&e[1]!=="N"){if(t=ur.hasOwnProperty(e)?ur[e]:null,t!==null){switch(typeof r){case"function":case"symbol":return;case"boolean":if(!t.acceptsBooleans)return}switch(e=Be(t.attributeName),t.type){case 3:r&&i.push(Mo,e,yS);break;case 4:r===!0?i.push(Mo,e,yS):r!==!1&&i.push(Mo,e,ll,Be(ar(r)),Ba);break;case 5:isNaN(r)||i.push(Mo,e,ll,Be(ar(r)),Ba);break;case 6:!isNaN(r)&&1<=r&&i.push(Mo,e,ll,Be(ar(r)),Ba);break;default:t.sanitizeURL&&(r=""+r),i.push(Mo,e,ll,Be(ar(r)),Ba)}}else if(dP(e)){switch(typeof r){case"function":case"symbol":return;case"boolean":if(t=e.toLowerCase().slice(0,5),t!=="data-"&&t!=="aria-")return}i.push(Mo,Be(e),ll,Be(ar(r)),Ba)}}}var Fo=he(">"),CS=he("/>");function Ph(i,t,e){if(t!=null){if(e!=null)throw Error(Ve(60));if(typeof t!="object"||!("__html"in t))throw Error(Ve(61));t=t.__html,t!=null&&i.push(Be(""+t))}}function XV(i){var t="";return cP.Children.forEach(i,function(e){e!=null&&(t+=e)}),t}var Zg=he(' selected=""');function Qg(i,t,e,r){i.push(yi(e));var n=e=null,o;for(o in t)if(Br.call(t,o)){var a=t[o];if(a!=null)switch(o){case"children":e=a;break;case"dangerouslySetInnerHTML":n=a;break;default:rn(i,r,o,a)}}return i.push(Fo),Ph(i,n,e),typeof e=="string"?(i.push(Be(ar(e))),null):e}var Jg=he(`
`),qV=/^[a-zA-Z][a-zA-Z:_\.\-\d]*$/,wS=new Map;function yi(i){var t=wS.get(i);if(t===void 0){if(!qV.test(i))throw Error(Ve(65,i));t=he("<"+i),wS.set(i,t)}return t}var ZV=he("<!DOCTYPE html>");function QV(i,t,e,r,n){switch(t){case"select":i.push(yi("select"));var o=null,a=null;for(c in e)if(Br.call(e,c)){var s=e[c];if(s!=null)switch(c){case"children":o=s;break;case"dangerouslySetInnerHTML":a=s;break;case"defaultValue":case"value":break;default:rn(i,r,c,s)}}return i.push(Fo),Ph(i,a,o),o;case"option":a=n.selectedValue,i.push(yi("option"));var l=s=null,u=null,c=null;for(o in e)if(Br.call(e,o)){var p=e[o];if(p!=null)switch(o){case"children":s=p;break;case"selected":u=p;break;case"dangerouslySetInnerHTML":c=p;break;case"value":l=p;default:rn(i,r,o,p)}}if(a!=null)if(e=l!==null?""+l:XV(s),hy(a)){for(r=0;r<a.length;r++)if(""+a[r]===e){i.push(Zg);break}}else""+a===e&&i.push(Zg);else u&&i.push(Zg);return i.push(Fo),Ph(i,c,s),s;case"textarea":i.push(yi("textarea")),c=a=o=null;for(s in e)if(Br.call(e,s)&&(l=e[s],l!=null))switch(s){case"children":c=l;break;case"value":o=l;break;case"defaultValue":a=l;break;case"dangerouslySetInnerHTML":throw Error(Ve(91));default:rn(i,r,s,l)}if(o===null&&a!==null&&(o=a),i.push(Fo),c!=null){if(o!=null)throw Error(Ve(92));if(hy(c)&&1<c.length)throw Error(Ve(93));o=""+c}return typeof o=="string"&&o[0]===`
`&&i.push(Jg),o!==null&&i.push(Be(ar(""+o))),null;case"input":i.push(yi("input")),l=c=s=o=null;for(a in e)if(Br.call(e,a)&&(u=e[a],u!=null))switch(a){case"children":case"dangerouslySetInnerHTML":throw Error(Ve(399,"input"));case"defaultChecked":l=u;break;case"defaultValue":s=u;break;case"checked":c=u;break;case"value":o=u;break;default:rn(i,r,a,u)}return c!==null?rn(i,r,"checked",c):l!==null&&rn(i,r,"checked",l),o!==null?rn(i,r,"value",o):s!==null&&rn(i,r,"value",s),i.push(CS),null;case"menuitem":i.push(yi("menuitem"));for(var h in e)if(Br.call(e,h)&&(o=e[h],o!=null))switch(h){case"children":case"dangerouslySetInnerHTML":throw Error(Ve(400));default:rn(i,r,h,o)}return i.push(Fo),null;case"title":i.push(yi("title")),o=null;for(p in e)if(Br.call(e,p)&&(a=e[p],a!=null))switch(p){case"children":o=a;break;case"dangerouslySetInnerHTML":throw Error(Ve(434));default:rn(i,r,p,a)}return i.push(Fo),o;case"listing":case"pre":i.push(yi(t)),a=o=null;for(l in e)if(Br.call(e,l)&&(s=e[l],s!=null))switch(l){case"children":o=s;break;case"dangerouslySetInnerHTML":a=s;break;default:rn(i,r,l,s)}if(i.push(Fo),a!=null){if(o!=null)throw Error(Ve(60));if(typeof a!="object"||!("__html"in a))throw Error(Ve(61));e=a.__html,e!=null&&(typeof e=="string"&&0<e.length&&e[0]===`
`?i.push(Jg,Be(e)):i.push(Be(""+e)))}return typeof o=="string"&&o[0]===`
`&&i.push(Jg),o;case"area":case"base":case"br":case"col":case"embed":case"hr":case"img":case"keygen":case"link":case"meta":case"param":case"source":case"track":case"wbr":i.push(yi(t));for(var d in e)if(Br.call(e,d)&&(o=e[d],o!=null))switch(d){case"children":case"dangerouslySetInnerHTML":throw Error(Ve(399,t));default:rn(i,r,d,o)}return i.push(CS),null;case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return Qg(i,e,t,r);case"html":return n.insertionMode===0&&i.push(ZV),Qg(i,e,t,r);default:if(t.indexOf("-")===-1&&typeof e.is!="string")return Qg(i,e,t,r);i.push(yi(t)),a=o=null;for(u in e)if(Br.call(e,u)&&(s=e[u],s!=null))switch(u){case"children":o=s;break;case"dangerouslySetInnerHTML":a=s;break;case"style":fP(i,r,s);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":break;default:dP(u)&&typeof s!="function"&&typeof s!="symbol"&&i.push(Mo,Be(u),ll,Be(ar(s)),Ba)}return i.push(Fo),Ph(i,a,o),o}}var JV=he("</"),e3=he(">"),t3=he('<template id="'),r3=he('"></template>'),n3=he("<!--$-->"),i3=he('<!--$?--><template id="'),o3=he('"></template>'),a3=he("<!--$!-->"),s3=he("<!--/$-->"),l3=he("<template"),u3=he('"'),c3=he(' data-dgst="');he(' data-msg="');he(' data-stck="');var p3=he("></template>");function bS(i,t,e){if(ve(i,i3),e===null)throw Error(Ve(395));return ve(i,e),bt(i,o3)}var h3=he('<div hidden id="'),d3=he('">'),f3=he("</div>"),g3=he('<svg aria-hidden="true" style="display:none" id="'),v3=he('">'),m3=he("</svg>"),y3=he('<math aria-hidden="true" style="display:none" id="'),C3=he('">'),w3=he("</math>"),b3=he('<table hidden id="'),_3=he('">'),S3=he("</table>"),x3=he('<table hidden><tbody id="'),T3=he('">'),O3=he("</tbody></table>"),E3=he('<table hidden><tr id="'),R3=he('">'),P3=he("</tr></table>"),A3=he('<table hidden><colgroup id="'),D3=he('">'),M3=he("</colgroup></table>");function F3(i,t,e,r){switch(e.insertionMode){case 0:case 1:return ve(i,h3),ve(i,t.segmentPrefix),ve(i,Be(r.toString(16))),bt(i,d3);case 2:return ve(i,g3),ve(i,t.segmentPrefix),ve(i,Be(r.toString(16))),bt(i,v3);case 3:return ve(i,y3),ve(i,t.segmentPrefix),ve(i,Be(r.toString(16))),bt(i,C3);case 4:return ve(i,b3),ve(i,t.segmentPrefix),ve(i,Be(r.toString(16))),bt(i,_3);case 5:return ve(i,x3),ve(i,t.segmentPrefix),ve(i,Be(r.toString(16))),bt(i,T3);case 6:return ve(i,E3),ve(i,t.segmentPrefix),ve(i,Be(r.toString(16))),bt(i,R3);case 7:return ve(i,A3),ve(i,t.segmentPrefix),ve(i,Be(r.toString(16))),bt(i,D3);default:throw Error(Ve(397))}}function I3(i,t){switch(t.insertionMode){case 0:case 1:return bt(i,f3);case 2:return bt(i,m3);case 3:return bt(i,w3);case 4:return bt(i,S3);case 5:return bt(i,O3);case 6:return bt(i,P3);case 7:return bt(i,M3);default:throw Error(Ve(397))}}var L3=he('function $RS(a,b){a=document.getElementById(a);b=document.getElementById(b);for(a.parentNode.removeChild(a);a.firstChild;)b.parentNode.insertBefore(a.firstChild,b);b.parentNode.removeChild(b)};$RS("'),N3=he('$RS("'),G3=he('","'),k3=he('")<\/script>'),W3=he('function $RC(a,b){a=document.getElementById(a);b=document.getElementById(b);b.parentNode.removeChild(b);if(a){a=a.previousSibling;var f=a.parentNode,c=a.nextSibling,e=0;do{if(c&&8===c.nodeType){var d=c.data;if("/$"===d)if(0===e)break;else e--;else"$"!==d&&"$?"!==d&&"$!"!==d||e++}d=c.nextSibling;f.removeChild(c);c=d}while(c);for(;b.firstChild;)f.insertBefore(b.firstChild,c);a.data="$";a._reactRetry&&a._reactRetry()}};$RC("'),V3=he('$RC("'),B3=he('","'),H3=he('")<\/script>'),j3=he('function $RX(b,c,d,e){var a=document.getElementById(b);a&&(b=a.previousSibling,b.data="$!",a=a.dataset,c&&(a.dgst=c),d&&(a.msg=d),e&&(a.stck=e),b._reactRetry&&b._reactRetry())};$RX("'),U3=he('$RX("'),z3=he('"'),$3=he(")<\/script>"),ev=he(","),K3=/[<\u2028\u2029]/g;function tv(i){return JSON.stringify(i).replace(K3,function(t){switch(t){case"<":return"\\u003c";case"\u2028":return"\\u2028";case"\u2029":return"\\u2029";default:throw Error("escapeJSStringsForInstructionScripts encountered a match it does not know how to replace. this means the match regex and the replacement characters are no longer in sync. This is a bug in React")}})}var pc=Object.assign,Y3=Symbol.for("react.element"),gP=Symbol.for("react.portal"),vP=Symbol.for("react.fragment"),mP=Symbol.for("react.strict_mode"),yP=Symbol.for("react.profiler"),CP=Symbol.for("react.provider"),wP=Symbol.for("react.context"),bP=Symbol.for("react.forward_ref"),_P=Symbol.for("react.suspense"),SP=Symbol.for("react.suspense_list"),xP=Symbol.for("react.memo"),Cw=Symbol.for("react.lazy"),X3=Symbol.for("react.scope"),q3=Symbol.for("react.debug_trace_mode"),Z3=Symbol.for("react.legacy_hidden"),Q3=Symbol.for("react.default_value"),_S=Symbol.iterator;function dy(i){if(i==null)return null;if(typeof i=="function")return i.displayName||i.name||null;if(typeof i=="string")return i;switch(i){case vP:return"Fragment";case gP:return"Portal";case yP:return"Profiler";case mP:return"StrictMode";case _P:return"Suspense";case SP:return"SuspenseList"}if(typeof i=="object")switch(i.$$typeof){case wP:return(i.displayName||"Context")+".Consumer";case CP:return(i._context.displayName||"Context")+".Provider";case bP:var t=i.render;return i=i.displayName,i||(i=t.displayName||t.name||"",i=i!==""?"ForwardRef("+i+")":"ForwardRef"),i;case xP:return t=i.displayName||null,t!==null?t:dy(i.type)||"Memo";case Cw:t=i._payload,i=i._init;try{return dy(i(t))}catch{}}return null}var TP={};function SS(i,t){if(i=i.contextTypes,!i)return TP;var e={},r;for(r in i)e[r]=t[r];return e}var Xa=null;function df(i,t){if(i!==t){i.context._currentValue=i.parentValue,i=i.parent;var e=t.parent;if(i===null){if(e!==null)throw Error(Ve(401))}else{if(e===null)throw Error(Ve(401));df(i,e)}t.context._currentValue=t.value}}function OP(i){i.context._currentValue=i.parentValue,i=i.parent,i!==null&&OP(i)}function EP(i){var t=i.parent;t!==null&&EP(t),i.context._currentValue=i.value}function RP(i,t){if(i.context._currentValue=i.parentValue,i=i.parent,i===null)throw Error(Ve(402));i.depth===t.depth?df(i,t):RP(i,t)}function PP(i,t){var e=t.parent;if(e===null)throw Error(Ve(402));i.depth===e.depth?df(i,e):PP(i,e),t.context._currentValue=t.value}function fd(i){var t=Xa;t!==i&&(t===null?EP(i):i===null?OP(t):t.depth===i.depth?df(t,i):t.depth>i.depth?RP(t,i):PP(t,i),Xa=i)}var xS={isMounted:function(){return!1},enqueueSetState:function(i,t){i=i._reactInternals,i.queue!==null&&i.queue.push(t)},enqueueReplaceState:function(i,t){i=i._reactInternals,i.replace=!0,i.queue=[t]},enqueueForceUpdate:function(){}};function TS(i,t,e,r){var n=i.state!==void 0?i.state:null;i.updater=xS,i.props=e,i.state=n;var o={queue:[],replace:!1};i._reactInternals=o;var a=t.contextType;if(i.context=typeof a=="object"&&a!==null?a._currentValue:r,a=t.getDerivedStateFromProps,typeof a=="function"&&(a=a(e,n),n=a==null?n:pc({},n,a),i.state=n),typeof t.getDerivedStateFromProps!="function"&&typeof i.getSnapshotBeforeUpdate!="function"&&(typeof i.UNSAFE_componentWillMount=="function"||typeof i.componentWillMount=="function"))if(t=i.state,typeof i.componentWillMount=="function"&&i.componentWillMount(),typeof i.UNSAFE_componentWillMount=="function"&&i.UNSAFE_componentWillMount(),t!==i.state&&xS.enqueueReplaceState(i,i.state,null),o.queue!==null&&0<o.queue.length)if(t=o.queue,a=o.replace,o.queue=null,o.replace=!1,a&&t.length===1)i.state=t[0];else{for(o=a?t[0]:i.state,n=!0,a=a?1:0;a<t.length;a++){var s=t[a];s=typeof s=="function"?s.call(i,o,e,r):s,s!=null&&(n?(n=!1,o=pc({},o,s)):pc(o,s))}i.state=o}else o.queue=null}var J3={id:1,overflow:""};function fy(i,t,e){var r=i.id;i=i.overflow;var n=32-Ah(r)-1;r&=~(1<<n),e+=1;var o=32-Ah(t)+n;if(30<o){var a=n-n%5;return o=(r&(1<<a)-1).toString(32),r>>=a,n-=a,{id:1<<32-Ah(t)+n|e<<n|r,overflow:o+i}}return{id:1<<o|e<<n|r,overflow:i}}var Ah=Math.clz32?Math.clz32:rB,eB=Math.log,tB=Math.LN2;function rB(i){return i>>>=0,i===0?32:31-(eB(i)/tB|0)|0}function nB(i,t){return i===t&&(i!==0||1/i===1/t)||i!==i&&t!==t}var iB=typeof Object.is=="function"?Object.is:nB,ho=null,ww=null,Dh=null,mt=null,Xu=!1,gd=!1,Pc=0,Xo=null,ff=0;function Ha(){if(ho===null)throw Error(Ve(321));return ho}function OS(){if(0<ff)throw Error(Ve(312));return{memoizedState:null,queue:null,next:null}}function bw(){return mt===null?Dh===null?(Xu=!1,Dh=mt=OS()):(Xu=!0,mt=Dh):mt.next===null?(Xu=!1,mt=mt.next=OS()):(Xu=!0,mt=mt.next),mt}function _w(){ww=ho=null,gd=!1,Dh=null,ff=0,mt=Xo=null}function AP(i,t){return typeof t=="function"?t(i):t}function ES(i,t,e){if(ho=Ha(),mt=bw(),Xu){var r=mt.queue;if(t=r.dispatch,Xo!==null&&(e=Xo.get(r),e!==void 0)){Xo.delete(r),r=mt.memoizedState;do r=i(r,e.action),e=e.next;while(e!==null);return mt.memoizedState=r,[r,t]}return[mt.memoizedState,t]}return i=i===AP?typeof t=="function"?t():t:e!==void 0?e(t):t,mt.memoizedState=i,i=mt.queue={last:null,dispatch:null},i=i.dispatch=oB.bind(null,ho,i),[mt.memoizedState,i]}function RS(i,t){if(ho=Ha(),mt=bw(),t=t===void 0?null:t,mt!==null){var e=mt.memoizedState;if(e!==null&&t!==null){var r=e[1];e:if(r===null)r=!1;else{for(var n=0;n<r.length&&n<t.length;n++)if(!iB(t[n],r[n])){r=!1;break e}r=!0}if(r)return e[0]}}return i=i(),mt.memoizedState=[i,t],i}function oB(i,t,e){if(25<=ff)throw Error(Ve(301));if(i===ho)if(gd=!0,i={action:e,next:null},Xo===null&&(Xo=new Map),e=Xo.get(t),e===void 0)Xo.set(t,i);else{for(t=e;t.next!==null;)t=t.next;t.next=i}}function aB(){throw Error(Ve(394))}function Ip(){}var PS={readContext:function(i){return i._currentValue},useContext:function(i){return Ha(),i._currentValue},useMemo:RS,useReducer:ES,useRef:function(i){ho=Ha(),mt=bw();var t=mt.memoizedState;return t===null?(i={current:i},mt.memoizedState=i):t},useState:function(i){return ES(AP,i)},useInsertionEffect:Ip,useLayoutEffect:function(){},useCallback:function(i,t){return RS(function(){return i},t)},useImperativeHandle:Ip,useEffect:Ip,useDebugValue:Ip,useDeferredValue:function(i){return Ha(),i},useTransition:function(){return Ha(),[!1,aB]},useId:function(){var i=ww.treeContext,t=i.overflow;i=i.id,i=(i&~(1<<32-Ah(i)-1)).toString(32)+t;var e=Mh;if(e===null)throw Error(Ve(404));return t=Pc++,i=":"+e.idPrefix+"R"+i,0<t&&(i+="H"+t.toString(32)),i+":"},useMutableSource:function(i,t){return Ha(),t(i._source)},useSyncExternalStore:function(i,t,e){if(e===void 0)throw Error(Ve(407));return e()}},Mh=null,rv=cP.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentDispatcher;function sB(i){return console.error(i),null}function qu(){}function lB(i,t,e,r,n,o,a,s,l){var u=[],c=new Set;return t={destination:null,responseState:t,progressiveChunkSize:r===void 0?12800:r,status:0,fatalError:null,nextSegmentId:0,allPendingTasks:0,pendingRootTasks:0,completedRootSegment:null,abortableTasks:c,pingedTasks:u,clientRenderedBoundaries:[],completedBoundaries:[],partialBoundaries:[],onError:n===void 0?sB:n,onAllReady:o===void 0?qu:o,onShellReady:a===void 0?qu:a,onShellError:s===void 0?qu:s,onFatalError:l===void 0?qu:l},e=vd(t,0,null,e,!1,!1),e.parentFlushed=!0,i=Sw(t,i,null,e,c,TP,null,J3),u.push(i),t}function Sw(i,t,e,r,n,o,a,s){i.allPendingTasks++,e===null?i.pendingRootTasks++:e.pendingTasks++;var l={node:t,ping:function(){var u=i.pingedTasks;u.push(l),u.length===1&&FP(i)},blockedBoundary:e,blockedSegment:r,abortSet:n,legacyContext:o,context:a,treeContext:s};return n.add(l),l}function vd(i,t,e,r,n,o){return{status:0,id:-1,index:t,parentFlushed:!1,chunks:[],children:[],formatContext:r,boundary:e,lastPushedText:n,textEmbedded:o}}function Ac(i,t){if(i=i.onError(t),i!=null&&typeof i!="string")throw Error('onError returned something with a type other than "string". onError should return a string and may return null or undefined but must not return anything else. It received something of type "'+typeof i+'" instead');return i}function md(i,t){var e=i.onShellError;e(t),e=i.onFatalError,e(t),i.destination!==null?(i.status=2,hP(i.destination,t)):(i.status=1,i.fatalError=t)}function AS(i,t,e,r,n){for(ho={},ww=t,Pc=0,i=e(r,n);gd;)gd=!1,Pc=0,ff+=1,mt=null,i=e(r,n);return _w(),i}function DS(i,t,e,r){var n=e.render(),o=r.childContextTypes;if(o!=null){var a=t.legacyContext;if(typeof e.getChildContext!="function")r=a;else{e=e.getChildContext();for(var s in e)if(!(s in o))throw Error(Ve(108,dy(r)||"Unknown",s));r=pc({},a,e)}t.legacyContext=r,cn(i,t,n),t.legacyContext=a}else cn(i,t,n)}function MS(i,t){if(i&&i.defaultProps){t=pc({},t),i=i.defaultProps;for(var e in i)t[e]===void 0&&(t[e]=i[e]);return t}return t}function gy(i,t,e,r,n){if(typeof e=="function")if(e.prototype&&e.prototype.isReactComponent){n=SS(e,t.legacyContext);var o=e.contextType;o=new e(r,typeof o=="object"&&o!==null?o._currentValue:n),TS(o,e,r,n),DS(i,t,o,e)}else{o=SS(e,t.legacyContext),n=AS(i,t,e,r,o);var a=Pc!==0;if(typeof n=="object"&&n!==null&&typeof n.render=="function"&&n.$$typeof===void 0)TS(n,e,r,o),DS(i,t,n,e);else if(a){r=t.treeContext,t.treeContext=fy(r,1,0);try{cn(i,t,n)}finally{t.treeContext=r}}else cn(i,t,n)}else if(typeof e=="string"){switch(n=t.blockedSegment,o=QV(n.chunks,e,r,i.responseState,n.formatContext),n.lastPushedText=!1,a=n.formatContext,n.formatContext=$V(a,e,r),vy(i,t,o),n.formatContext=a,e){case"area":case"base":case"br":case"col":case"embed":case"hr":case"img":case"input":case"keygen":case"link":case"meta":case"param":case"source":case"track":case"wbr":break;default:n.chunks.push(JV,Be(e),e3)}n.lastPushedText=!1}else{switch(e){case Z3:case q3:case mP:case yP:case vP:cn(i,t,r.children);return;case SP:cn(i,t,r.children);return;case X3:throw Error(Ve(343));case _P:e:{e=t.blockedBoundary,n=t.blockedSegment,o=r.fallback,r=r.children,a=new Set;var s={id:null,rootSegmentID:-1,parentFlushed:!1,pendingTasks:0,forceClientRender:!1,completedSegments:[],byteSize:0,fallbackAbortableTasks:a,errorDigest:null},l=vd(i,n.chunks.length,s,n.formatContext,!1,!1);n.children.push(l),n.lastPushedText=!1;var u=vd(i,0,null,n.formatContext,!1,!1);u.parentFlushed=!0,t.blockedBoundary=s,t.blockedSegment=u;try{if(vy(i,t,r),u.lastPushedText&&u.textEmbedded&&u.chunks.push(yw),u.status=1,yd(s,u),s.pendingTasks===0)break e}catch(c){u.status=4,s.forceClientRender=!0,s.errorDigest=Ac(i,c)}finally{t.blockedBoundary=e,t.blockedSegment=n}t=Sw(i,o,e,l,a,t.legacyContext,t.context,t.treeContext),i.pingedTasks.push(t)}return}if(typeof e=="object"&&e!==null)switch(e.$$typeof){case bP:if(r=AS(i,t,e.render,r,n),Pc!==0){e=t.treeContext,t.treeContext=fy(e,1,0);try{cn(i,t,r)}finally{t.treeContext=e}}else cn(i,t,r);return;case xP:e=e.type,r=MS(e,r),gy(i,t,e,r,n);return;case CP:if(n=r.children,e=e._context,r=r.value,o=e._currentValue,e._currentValue=r,a=Xa,Xa=r={parent:a,depth:a===null?0:a.depth+1,context:e,parentValue:o,value:r},t.context=r,cn(i,t,n),i=Xa,i===null)throw Error(Ve(403));r=i.parentValue,i.context._currentValue=r===Q3?i.context._defaultValue:r,i=Xa=i.parent,t.context=i;return;case wP:r=r.children,r=r(e._currentValue),cn(i,t,r);return;case Cw:n=e._init,e=n(e._payload),r=MS(e,r),gy(i,t,e,r,void 0);return}throw Error(Ve(130,e==null?e:typeof e,""))}}function cn(i,t,e){if(t.node=e,typeof e=="object"&&e!==null){switch(e.$$typeof){case Y3:gy(i,t,e.type,e.props,e.ref);return;case gP:throw Error(Ve(257));case Cw:var r=e._init;e=r(e._payload),cn(i,t,e);return}if(hy(e)){FS(i,t,e);return}if(e===null||typeof e!="object"?r=null:(r=_S&&e[_S]||e["@@iterator"],r=typeof r=="function"?r:null),r&&(r=r.call(e))){if(e=r.next(),!e.done){var n=[];do n.push(e.value),e=r.next();while(!e.done);FS(i,t,n)}return}throw i=Object.prototype.toString.call(e),Error(Ve(31,i==="[object Object]"?"object with keys {"+Object.keys(e).join(", ")+"}":i))}typeof e=="string"?(r=t.blockedSegment,r.lastPushedText=gS(t.blockedSegment.chunks,e,i.responseState,r.lastPushedText)):typeof e=="number"&&(r=t.blockedSegment,r.lastPushedText=gS(t.blockedSegment.chunks,""+e,i.responseState,r.lastPushedText))}function FS(i,t,e){for(var r=e.length,n=0;n<r;n++){var o=t.treeContext;t.treeContext=fy(o,r,n);try{vy(i,t,e[n])}finally{t.treeContext=o}}}function vy(i,t,e){var r=t.blockedSegment.formatContext,n=t.legacyContext,o=t.context;try{return cn(i,t,e)}catch(l){if(_w(),typeof l=="object"&&l!==null&&typeof l.then=="function"){e=l;var a=t.blockedSegment,s=vd(i,a.chunks.length,null,a.formatContext,a.lastPushedText,!0);a.children.push(s),a.lastPushedText=!1,i=Sw(i,t.node,t.blockedBoundary,s,t.abortSet,t.legacyContext,t.context,t.treeContext).ping,e.then(i,i),t.blockedSegment.formatContext=r,t.legacyContext=n,t.context=o,fd(o)}else throw t.blockedSegment.formatContext=r,t.legacyContext=n,t.context=o,fd(o),l}}function uB(i){var t=i.blockedBoundary;i=i.blockedSegment,i.status=3,MP(this,t,i)}function DP(i,t,e){var r=i.blockedBoundary;i.blockedSegment.status=3,r===null?(t.allPendingTasks--,t.status!==2&&(t.status=2,t.destination!==null&&t.destination.close())):(r.pendingTasks--,r.forceClientRender||(r.forceClientRender=!0,i=e===void 0?Error(Ve(432)):e,r.errorDigest=t.onError(i),r.parentFlushed&&t.clientRenderedBoundaries.push(r)),r.fallbackAbortableTasks.forEach(function(n){return DP(n,t,e)}),r.fallbackAbortableTasks.clear(),t.allPendingTasks--,t.allPendingTasks===0&&(r=t.onAllReady,r()))}function yd(i,t){if(t.chunks.length===0&&t.children.length===1&&t.children[0].boundary===null){var e=t.children[0];e.id=t.id,e.parentFlushed=!0,e.status===1&&yd(i,e)}else i.completedSegments.push(t)}function MP(i,t,e){if(t===null){if(e.parentFlushed){if(i.completedRootSegment!==null)throw Error(Ve(389));i.completedRootSegment=e}i.pendingRootTasks--,i.pendingRootTasks===0&&(i.onShellError=qu,t=i.onShellReady,t())}else t.pendingTasks--,t.forceClientRender||(t.pendingTasks===0?(e.parentFlushed&&e.status===1&&yd(t,e),t.parentFlushed&&i.completedBoundaries.push(t),t.fallbackAbortableTasks.forEach(uB,i),t.fallbackAbortableTasks.clear()):e.parentFlushed&&e.status===1&&(yd(t,e),t.completedSegments.length===1&&t.parentFlushed&&i.partialBoundaries.push(t)));i.allPendingTasks--,i.allPendingTasks===0&&(i=i.onAllReady,i())}function FP(i){if(i.status!==2){var t=Xa,e=rv.current;rv.current=PS;var r=Mh;Mh=i.responseState;try{var n=i.pingedTasks,o;for(o=0;o<n.length;o++){var a=n[o],s=i,l=a.blockedSegment;if(l.status===0){fd(a.context);try{cn(s,a,a.node),l.lastPushedText&&l.textEmbedded&&l.chunks.push(yw),a.abortSet.delete(a),l.status=1,MP(s,a.blockedBoundary,l)}catch(f){if(_w(),typeof f=="object"&&f!==null&&typeof f.then=="function"){var u=a.ping;f.then(u,u)}else{a.abortSet.delete(a),l.status=4;var c=a.blockedBoundary,p=f,h=Ac(s,p);if(c===null?md(s,p):(c.pendingTasks--,c.forceClientRender||(c.forceClientRender=!0,c.errorDigest=h,c.parentFlushed&&s.clientRenderedBoundaries.push(c))),s.allPendingTasks--,s.allPendingTasks===0){var d=s.onAllReady;d()}}}finally{}}}n.splice(0,o),i.destination!==null&&xw(i,i.destination)}catch(f){Ac(i,f),md(i,f)}finally{Mh=r,rv.current=e,e===PS&&fd(t)}}}function Lp(i,t,e){switch(e.parentFlushed=!0,e.status){case 0:var r=e.id=i.nextSegmentId++;return e.lastPushedText=!1,e.textEmbedded=!1,i=i.responseState,ve(t,t3),ve(t,i.placeholderPrefix),i=Be(r.toString(16)),ve(t,i),bt(t,r3);case 1:e.status=2;var n=!0;r=e.chunks;var o=0;e=e.children;for(var a=0;a<e.length;a++){for(n=e[a];o<n.index;o++)ve(t,r[o]);n=gf(i,t,n)}for(;o<r.length-1;o++)ve(t,r[o]);return o<r.length&&(n=bt(t,r[o])),n;default:throw Error(Ve(390))}}function gf(i,t,e){var r=e.boundary;if(r===null)return Lp(i,t,e);if(r.parentFlushed=!0,r.forceClientRender)r=r.errorDigest,bt(t,a3),ve(t,l3),r&&(ve(t,c3),ve(t,Be(ar(r))),ve(t,u3)),bt(t,p3),Lp(i,t,e);else if(0<r.pendingTasks){r.rootSegmentID=i.nextSegmentId++,0<r.completedSegments.length&&i.partialBoundaries.push(r);var n=i.responseState,o=n.nextSuspenseID++;n=he(n.boundaryPrefix+o.toString(16)),r=r.id=n,bS(t,i.responseState,r),Lp(i,t,e)}else if(r.byteSize>i.progressiveChunkSize)r.rootSegmentID=i.nextSegmentId++,i.completedBoundaries.push(r),bS(t,i.responseState,r.id),Lp(i,t,e);else{if(bt(t,n3),e=r.completedSegments,e.length!==1)throw Error(Ve(391));gf(i,t,e[0])}return bt(t,s3)}function IS(i,t,e){return F3(t,i.responseState,e.formatContext,e.id),gf(i,t,e),I3(t,e.formatContext)}function LS(i,t,e){for(var r=e.completedSegments,n=0;n<r.length;n++)IP(i,t,e,r[n]);if(r.length=0,i=i.responseState,r=e.id,e=e.rootSegmentID,ve(t,i.startInlineScript),i.sentCompleteBoundaryFunction?ve(t,V3):(i.sentCompleteBoundaryFunction=!0,ve(t,W3)),r===null)throw Error(Ve(395));return e=Be(e.toString(16)),ve(t,r),ve(t,B3),ve(t,i.segmentPrefix),ve(t,e),bt(t,H3)}function IP(i,t,e,r){if(r.status===2)return!0;var n=r.id;if(n===-1){if((r.id=e.rootSegmentID)===-1)throw Error(Ve(392));return IS(i,t,r)}return IS(i,t,r),i=i.responseState,ve(t,i.startInlineScript),i.sentCompleteSegmentFunction?ve(t,N3):(i.sentCompleteSegmentFunction=!0,ve(t,L3)),ve(t,i.segmentPrefix),n=Be(n.toString(16)),ve(t,n),ve(t,G3),ve(t,i.placeholderPrefix),ve(t,n),bt(t,k3)}function xw(i,t){ln=new Uint8Array(512),un=0;try{var e=i.completedRootSegment;if(e!==null&&i.pendingRootTasks===0){gf(i,t,e),i.completedRootSegment=null;var r=i.responseState.bootstrapChunks;for(e=0;e<r.length-1;e++)ve(t,r[e]);e<r.length&&bt(t,r[e])}var n=i.clientRenderedBoundaries,o;for(o=0;o<n.length;o++){var a=n[o];r=t;var s=i.responseState,l=a.id,u=a.errorDigest,c=a.errorMessage,p=a.errorComponentStack;if(ve(r,s.startInlineScript),s.sentClientRenderFunction?ve(r,U3):(s.sentClientRenderFunction=!0,ve(r,j3)),l===null)throw Error(Ve(395));ve(r,l),ve(r,z3),(u||c||p)&&(ve(r,ev),ve(r,Be(tv(u||"")))),(c||p)&&(ve(r,ev),ve(r,Be(tv(c||"")))),p&&(ve(r,ev),ve(r,Be(tv(p)))),bt(r,$3)}n.splice(0,o);var h=i.completedBoundaries;for(o=0;o<h.length;o++)LS(i,t,h[o]);h.splice(0,o),pS(t),ln=new Uint8Array(512),un=0;var d=i.partialBoundaries;for(o=0;o<d.length;o++){var f=d[o];e:{n=i,a=t;var g=f.completedSegments;for(s=0;s<g.length;s++)if(!IP(n,a,f,g[s])){s++,g.splice(0,s);var v=!1;break e}g.splice(0,s),v=!0}if(!v){i.destination=null,o++,d.splice(0,o);return}}d.splice(0,o);var m=i.completedBoundaries;for(o=0;o<m.length;o++)LS(i,t,m[o]);m.splice(0,o)}finally{pS(t),i.allPendingTasks===0&&i.pingedTasks.length===0&&i.clientRenderedBoundaries.length===0&&i.completedBoundaries.length===0&&t.close()}}function NS(i,t){try{var e=i.abortableTasks;e.forEach(function(r){return DP(r,i,t)}),e.clear(),i.destination!==null&&xw(i,i.destination)}catch(r){Ac(i,r),md(i,r)}}gw.renderToReadableStream=function(i,t){return new Promise(function(e,r){var n,o,a=new Promise(function(c,p){o=c,n=p}),s=lB(i,UV(t?t.identifierPrefix:void 0,t?t.nonce:void 0,t?t.bootstrapScriptContent:void 0,t?t.bootstrapScripts:void 0,t?t.bootstrapModules:void 0),zV(t?t.namespaceURI:void 0),t?t.progressiveChunkSize:void 0,t?t.onError:void 0,o,function(){var c=new ReadableStream({type:"bytes",pull:function(p){if(s.status===1)s.status=2,hP(p,s.fatalError);else if(s.status!==2&&s.destination===null){s.destination=p;try{xw(s,p)}catch(h){Ac(s,h),md(s,h)}}},cancel:function(){NS(s)}},{highWaterMark:0});c.allReady=a,e(c)},function(c){a.catch(function(){}),r(c)},n);if(t&&t.signal){var l=t.signal,u=function(){NS(s,l.reason),l.removeEventListener("abort",u)};l.addEventListener("abort",u)}FP(s)})};gw.version="18.2.0";var $l,LP;$l=zl,LP=gw;ha.version=$l.version;ha.renderToString=$l.renderToString;ha.renderToStaticMarkup=$l.renderToStaticMarkup;ha.renderToNodeStream=$l.renderToNodeStream;ha.renderToStaticNodeStream=$l.renderToStaticNodeStream;ha.renderToReadableStream=LP.renderToReadableStream;var vf={};Object.defineProperty(vf,"__esModule",{value:!0});let cB=0;function pB(){return`agPortalKey_${++cB}`}vf.default=pB;var hB=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(aw,"__esModule",{value:!0});const GS=we.exports,dB=Jy.exports,fB=jt,gB=uf,vB=ha,mB=hB(vf);class mf extends gB.ReactComponent{constructor(t,e,r,n){super(t,r,n),this.staticMarkup=null,this.staticRenderTime=0,this.parentComponent=e}init(t){return this.eParentElement=this.createParentElement(t),this.renderStaticMarkup(t),new fB.AgPromise(e=>this.createReactComponent(t,e))}createReactComponent(t,e){this.isStatelessComponent()||(t.ref=o=>{this.componentInstance=o,this.addParentContainerStyleAndClasses(),this.removeStaticMarkup()});const r=GS.createElement(this.reactComponent,t),n=dB.createPortal(r,this.eParentElement,mB.default());this.portal=n,this.portalManager.mountReactPortal(n,this,o=>{e(o),this.isStatelessComponent()&&(this.isSlowRenderer()&&this.removeStaticMarkup(),setTimeout(()=>{this.removeStaticMarkup()}))})}fallbackMethodAvailable(t){return!1}fallbackMethod(t,e){}isSlowRenderer(){return this.staticRenderTime>=mf.SLOW_RENDERING_THRESHOLD}isNullValue(){return this.staticMarkup===""}renderStaticMarkup(t){if(this.parentComponent.isDisableStaticMarkup()||!this.componentType.cellRenderer)return;const e=console.error,r=GS.createElement(this.reactComponent,t);try{console.error=()=>{};const n=Date.now(),o=vB.renderToStaticMarkup(r);this.staticRenderTime=Date.now()-n,console.error=e,o===""?this.staticMarkup=o:o&&(this.staticMarkup=document.createElement("span"),this.staticMarkup.innerHTML=o,this.eParentElement.appendChild(this.staticMarkup))}catch{}finally{console.error=e}}removeStaticMarkup(){this.parentComponent.isDisableStaticMarkup()||!this.componentType.cellRenderer||this.staticMarkup&&(this.staticMarkup.remove?(this.staticMarkup.remove(),this.staticMarkup=null):this.eParentElement.removeChild&&(this.eParentElement.removeChild(this.staticMarkup),this.staticMarkup=null))}rendered(){return this.isNullValue()||!!this.staticMarkup||this.isStatelessComponent()&&this.statelessComponentRendered()||!!(!this.isStatelessComponent()&&this.getFrameworkComponentInstance())}}aw.LegacyReactComponent=mf;mf.SLOW_RENDERING_THRESHOLD=3;var rp={},yB=de&&de.__rest||function(i,t){var e={};for(var r in i)Object.prototype.hasOwnProperty.call(i,r)&&t.indexOf(r)<0&&(e[r]=i[r]);if(i!=null&&typeof Object.getOwnPropertySymbols=="function")for(var n=0,r=Object.getOwnPropertySymbols(i);n<r.length;n++)t.indexOf(r[n])<0&&Object.prototype.propertyIsEnumerable.call(i,r[n])&&(e[r[n]]=i[r[n]]);return e},NP=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t},CB=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(rp,"__esModule",{value:!0});const nv=NP(we.exports),Kl=CB(Zd.exports),Yl=NP(jt);class xl extends nv.Component{constructor(t){super(t),this.props=t}render(){return null}static mapChildColumnDefs(t){return nv.default.Children.map(t,e=>e?xl.toColDef(e.props):null)}static toColDef(t){const{children:e}=t,r=yB(t,["children"]);return xl.hasChildColumns(e)&&(r.children=xl.mapChildColumnDefs(e)),r}static hasChildColumns(t){return nv.default.Children.count(t)>0}}rp.AgGridColumn=xl;Xl(Yl.ColDefUtil.BOOLEAN_PROPERTIES,Kl.default.bool);Xl(Yl.ColDefUtil.STRING_PROPERTIES,Kl.default.string);Xl(Yl.ColDefUtil.OBJECT_PROPERTIES,Kl.default.object);Xl(Yl.ColDefUtil.ARRAY_PROPERTIES,Kl.default.array);Xl(Yl.ColDefUtil.NUMBER_PROPERTIES,Kl.default.number);Xl(Yl.ColDefUtil.FUNCTION_PROPERTIES,Kl.default.func);function Xl(i,t){i.forEach(e=>{xl[e]=t})}var Tw={};(function(i){Object.defineProperty(i,"__esModule",{value:!0});var t;(function(o){o.IdentityCheck="IdentityCheck",o.DeepValueCheck="DeepValueCheck",o.NoCheck="NoCheck"})(t=i.ChangeDetectionStrategyType||(i.ChangeDetectionStrategyType={}));class e{constructor(a){this.strategy=a}areEqual(a,s){return this.strategy(a,s)}}class r{areEqual(a,s){return r.areEquivalent(r.copy(a),r.copy(s))}static unwrapStringOrNumber(a){return a instanceof Number||a instanceof String?a.valueOf():a}static copy(a){if(!a)return a;if(Array.isArray(a)){const s=[];for(let l=0;l<a.length;l++)s.push(this.copy(a[l]));return s}return typeof a!="object"?a:[{},a].reduce((s,l)=>(Object.keys(l).forEach(function(u){s[u]=l[u]}),s),{})}static isNaN(a){return Number.isNaN?Number.isNaN(a):typeof a=="number"&&isNaN(a)}static areEquivalent(a,s){if(a=r.unwrapStringOrNumber(a),s=r.unwrapStringOrNumber(s),a===s)return!0;if(a===null||s===null||typeof a!=typeof s)return!1;if(r.isNaN(a)&&r.isNaN(s))return!0;if(a instanceof Date)return s instanceof Date&&a.valueOf()===s.valueOf();if(typeof a=="function")return!1;if(typeof a!="object"||a.$$typeof&&a.$$typeof.toString()==="Symbol(react.element)")return a==s;if(Object.isFrozen(a)||Object.isFrozen(s))return a===s;const l=a.areEquivPropertyTracking===void 0,u=s.areEquivPropertyTracking===void 0;try{let c;if(l)a.areEquivPropertyTracking=[];else if(a.areEquivPropertyTracking.some(function(h){return h===s}))return!0;if(u)s.areEquivPropertyTracking=[];else if(s.areEquivPropertyTracking.some(h=>h===a))return!0;a.areEquivPropertyTracking.push(s),s.areEquivPropertyTracking.push(a);const p={};for(c in a)c!="areEquivPropertyTracking"&&(p[c]=null);for(c in s)c!="areEquivPropertyTracking"&&(p[c]=null);for(c in p)if(!this.areEquivalent(a[c],s[c]))return!1;return!0}finally{l&&delete a.areEquivPropertyTracking,u&&delete s.areEquivPropertyTracking}}}class n{constructor(){this.strategyMap={[t.DeepValueCheck]:new r,[t.IdentityCheck]:new e((a,s)=>a===s),[t.NoCheck]:new e((a,s)=>!1)}}getStrategy(a){return this.strategyMap[a]}}i.ChangeDetectionService=n})(Tw);var yf={},wB=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(yf,"__esModule",{value:!0});const kS=we.exports,bB=Jy.exports,_B=jt,SB=uf,xB=ha,WS=wB(vf);class TB extends SB.ReactComponent{constructor(t,e,r){super(t,e,r),this.oldPortal=null,this.key=WS.default(),this.portalKey=WS.default()}init(t){return this.eParentElement=this.createParentElement(t),this.params=t,this.createOrUpdatePortal(t),new _B.AgPromise(e=>this.createReactComponent(e))}createOrUpdatePortal(t){this.isStatelessComponent()||(t.ref=e=>{this.componentInstance=e,this.addParentContainerStyleAndClasses()}),this.reactElement=kS.createElement(this.reactComponent,Object.assign(Object.assign({},t),{key:this.key})),this.portal=bB.createPortal(this.reactElement,this.eParentElement,this.portalKey)}createReactComponent(t){this.portalManager.mountReactPortal(this.portal,this,e=>{t(e)})}isNullValue(){return this.valueRenderedIsNull(this.params)}rendered(){return this.isStatelessComponent()&&this.statelessComponentRendered()||!!(!this.isStatelessComponent()&&this.getFrameworkComponentInstance())}valueRenderedIsNull(t){if(!this.componentType.cellRenderer)return!1;const e=console.error;try{return console.error=()=>{},xB.renderToStaticMarkup(kS.createElement(this.reactComponent,t))===""}catch{}finally{console.error=e}return!1}refreshComponent(t){this.oldPortal=this.portal,this.createOrUpdatePortal(t),this.portalManager.updateReactPortal(this.oldPortal,this.portal)}fallbackMethod(t,e){const r=this[`${t}Component`];if(r)return r.bind(this)(e)}fallbackMethodAvailable(t){return!!this[`${t}Component`]}}yf.NewReactComponent=TB;var Cf={};Object.defineProperty(Cf,"__esModule",{value:!0});class wf{constructor(t,e,r){this.destroyed=!1,this.portals=[],this.hasPendingPortalUpdate=!1,this.wrappingElement=e||"div",this.parent=t,this.maxComponentCreationTimeMs=r||wf.MAX_COMPONENT_CREATION_TIME_IN_MS}getPortals(){return this.portals}destroy(){this.destroyed=!0}destroyPortal(t){this.portals=this.portals.filter(e=>e!==t),this.batchUpdate()}getComponentWrappingElement(){return this.wrappingElement}mountReactPortal(t,e,r){this.portals=[...this.portals,t],this.waitForInstance(e,r),this.batchUpdate()}updateReactPortal(t,e){this.portals[this.portals.indexOf(t)]=e,this.batchUpdate()}batchUpdate(){this.hasPendingPortalUpdate||(setTimeout(()=>{this.destroyed||this.parent.forceUpdate(()=>{this.hasPendingPortalUpdate=!1})}),this.hasPendingPortalUpdate=!0)}waitForInstance(t,e,r=Date.now()){if(this.destroyed){e(null);return}if(t.rendered())e(t);else{if(Date.now()-r>=this.maxComponentCreationTimeMs&&!this.hasPendingPortalUpdate){if(t.isNullValue()){e(t);return}console.error(`AG Grid: React Component '${t.getReactComponentName()}' not created within ${this.maxComponentCreationTimeMs}ms`);return}window.setTimeout(()=>{this.waitForInstance(t,e,r)})}}}Cf.PortalManager=wf;wf.MAX_COMPONENT_CREATION_TIME_IN_MS=1e3;var bf={},Ow={},qt={},OB=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(qt,"__esModule",{value:!0});const EB=OB(we.exports);qt.BeansContext=EB.default.createContext({});var mo={};Object.defineProperty(mo,"__esModule",{value:!0});mo.showJsComp=(i,t,e,r)=>{if(!i||i.componentFromFramework)return;const o=i.newAgStackInstance();if(!o)return;let a,s,l=!1;return o.then(u=>{if(l){t.destroyBean(u);return}a=u,s=a.getGui(),e.appendChild(s),VS(r,a)}),()=>{l=!0,a&&(s&&s.parentElement&&s.parentElement.removeChild(s),t.destroyBean(a),r&&VS(r,void 0))}};const VS=(i,t)=>{if(!!i)if(i instanceof Function)i(t);else{const e=i;e.current=t}};mo.createSyncJsComp=i=>{const t=i.newAgStackInstance();if(!!t)return t.resolveNow(null,e=>e)};var Ut={};Object.defineProperty(Ut,"__esModule",{value:!0});const Eu=we.exports;Ut.useEffectOnce=i=>{const t=Eu.useRef(),e=Eu.useRef(!1),r=Eu.useRef(!1),[n,o]=Eu.useState(0);e.current&&(r.current=!0),Eu.useEffect(()=>(e.current||(t.current=i(),e.current=!0),o(a=>a+1),()=>{!r.current||t.current&&t.current()}),[])};var Sr={};Object.defineProperty(Sr,"__esModule",{value:!0});Sr.classesList=(...i)=>i.filter(e=>e!=null&&e!=="").join(" ");class Ew{constructor(...t){this.classesMap={},t.forEach(e=>{this.classesMap[e]=!0})}setClass(t,e){if(!!this.classesMap[t]==e)return this;const n=new Ew;return n.classesMap=Object.assign({},this.classesMap),n.classesMap[t]=e,n}toString(){return Object.keys(this.classesMap).filter(e=>this.classesMap[e]).join(" ")}}Sr.CssClasses=Ew;Sr.isComponentStateless=i=>{const t=()=>typeof Symbol=="function"&&Symbol.for,e=()=>t()?Symbol.for("react.memo"):60115;return typeof i=="function"&&!(i.prototype&&i.prototype.isReactComponent)||typeof i=="object"&&i.$$typeof===e()};var RB=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t};Object.defineProperty(Ow,"__esModule",{value:!0});const PB=jt,et=RB(we.exports),AB=qt,DB=mo,MB=Ut,Np=Sr,GP=et.forwardRef((i,t)=>{const e=et.useContext(AB.BeansContext).context,r=et.useRef(null),n=et.useRef(null),o=et.useRef(null),a=et.useRef(null),s=et.useRef(null),[l,u]=et.useState(),[c,p]=et.useState(),[h,d]=et.useState(),[f,g]=et.useState(new Np.CssClasses),[v,m]=et.useState(new Np.CssClasses("ag-hidden")),[C,w]=et.useState(new Np.CssClasses("ag-hidden")),[b,S]=et.useState(new Np.CssClasses("ag-invisible"));et.useImperativeHandle(t,()=>({refresh(){return!1}})),et.useEffect(()=>DB.showJsComp(l,e,n.current),[l]),MB.useEffectOnce(()=>{const G={setInnerRenderer:(V,U)=>{u(V),d(U)},setChildCount:V=>p(V),addOrRemoveCssClass:(V,U)=>g(J=>J.setClass(V,U)),setContractedDisplayed:V=>w(U=>U.setClass("ag-hidden",!V)),setExpandedDisplayed:V=>m(U=>U.setClass("ag-hidden",!V)),setCheckboxVisible:V=>S(U=>U.setClass("ag-invisible",!V))},k=e.createBean(new PB.GroupCellRendererCtrl);return k.init(G,r.current,o.current,a.current,s.current,GP,i),()=>{e.destroyBean(k)}});const T=et.useMemo(()=>`ag-cell-wrapper ${f.toString()}`,[f]),E=et.useMemo(()=>`ag-group-expanded ${v.toString()}`,[v]),P=et.useMemo(()=>`ag-group-contracted ${C.toString()}`,[C]),R=et.useMemo(()=>`ag-group-checkbox ${b.toString()}`,[b]),M=l&&l.componentFromFramework,N=M?l.componentClass:void 0,W=l==null&&h!=null;return et.default.createElement("span",Object.assign({className:T,ref:r},i.colDef?{}:{role:"gridcell"}),et.default.createElement("span",{className:E,ref:a}),et.default.createElement("span",{className:P,ref:s}),et.default.createElement("span",{className:R,ref:o}),et.default.createElement("span",{className:"ag-group-value",ref:n},W&&et.default.createElement(et.default.Fragment,null,h),M&&et.default.createElement(N,Object.assign({},l.params))),et.default.createElement("span",{className:"ag-group-child-count"},c))});Ow.default=GP;var Rw={},_f={},Pw={},Aw={},Dw={},Mw={},Fw={},Iw={},FB=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t};Object.defineProperty(Iw,"__esModule",{value:!0});const Dt=FB(we.exports),IB=qt,LB=jt,NB=Sr,GB=mo,kB=Ut,WB=i=>{const{context:t}=Dt.useContext(IB.BeansContext),[e,r]=Dt.useState(),[n,o]=Dt.useState(),[a,s]=Dt.useState(),[l,u]=Dt.useState(),[c,p]=Dt.useState(),[h,d]=Dt.useState(),f=Dt.useRef(null),g=Dt.useRef(null),v=Dt.useRef(),{ctrl:m}=i,C=Dt.useMemo(()=>new LB.CssClassManager(()=>f.current),[]);kB.useEffectOnce(()=>{const E={setWidth:R=>r(R),addOrRemoveCssClass:(R,M)=>C.addOrRemoveCssClass(R,M),setColId:R=>s(R),setTitle:R=>o(R),setAriaDescription:R=>p(R),setAriaSort:R=>u(R),setUserCompDetails:R=>d(R),getUserCompInstance:()=>v.current||void 0};m.setComp(E,f.current,g.current);const P=m.getSelectAllGui();g.current.insertAdjacentElement("afterend",P)}),Dt.useEffect(()=>GB.showJsComp(h,t,f.current,v),[h]),Dt.useEffect(()=>{let E;f.current.childNodes.forEach(P=>{P!=null&&P!==g.current&&(E=P)}),m.setDragSource(E)},[h]);const w=Dt.useMemo(()=>({width:e}),[e]),b=Dt.useMemo(()=>{var E;return!!(((E=h)===null||E===void 0?void 0:E.componentFromFramework)&&NB.isComponentStateless(h.componentClass))},[h]),S=h&&h.componentFromFramework,T=h&&h.componentClass;return Dt.default.createElement("div",{ref:f,className:"ag-header-cell",style:w,title:n,"col-id":a,"aria-sort":l,role:"columnheader",tabIndex:-1,"aria-description":c},Dt.default.createElement("div",{ref:g,className:"ag-header-cell-resize",role:"presentation"}),S&&b&&Dt.default.createElement(T,Object.assign({},h.params)),S&&!b&&Dt.default.createElement(T,Object.assign({},h.params,{ref:v})))};Iw.default=Dt.memo(WB);var Lw={},VB=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t};Object.defineProperty(Lw,"__esModule",{value:!0});const Wt=VB(we.exports),BB=qt,HB=mo,jB=Ut,BS=Sr,UB=i=>{const{context:t}=Wt.useContext(BB.BeansContext),[e,r]=Wt.useState(new BS.CssClasses),[n,o]=Wt.useState(new BS.CssClasses),[a,s]=Wt.useState(),[l,u]=Wt.useState(),[c,p]=Wt.useState(),[h,d]=Wt.useState(),[f,g]=Wt.useState(),v=Wt.useRef(null),m=Wt.useRef(null),{ctrl:C}=i;jB.useEffectOnce(()=>{const P={setWidth:R=>s(R),addOrRemoveCssClass:(R,M)=>r(N=>N.setClass(R,M)),setColId:R=>p(R),setTitle:R=>u(R),setUserCompDetails:R=>g(R),addOrRemoveResizableCssClass:(R,M)=>o(N=>N.setClass(R,M)),setAriaExpanded:R=>d(R)};C.setComp(P,v.current,m.current)}),Wt.useEffect(()=>HB.showJsComp(f,t,v.current),[f]),Wt.useEffect(()=>{let P;v.current.childNodes.forEach(R=>{R!=null&&R!==m.current&&(P=R)}),P&&C.setDragSource(P)},[f]);const w=Wt.useMemo(()=>({width:a}),[a]),b=Wt.useMemo(()=>"ag-header-group-cell "+e.toString(),[e]),S=Wt.useMemo(()=>"ag-header-cell-resize "+n.toString(),[n]),T=f&&f.componentFromFramework,E=f&&f.componentClass;return Wt.default.createElement("div",{ref:v,className:b,style:w,title:l,"col-id":c,role:"columnheader",tabIndex:-1,"aria-expanded":h},T&&Wt.default.createElement(E,Object.assign({},f.params)),Wt.default.createElement("div",{ref:m,className:S}))};Lw.default=Wt.memo(UB);var Nw={},zB=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t};Object.defineProperty(Nw,"__esModule",{value:!0});const ft=zB(we.exports),$B=qt,KB=jt,Gp=Sr,YB=mo,HS=Ut,XB=i=>{const{context:t}=ft.useContext($B.BeansContext),[e,r]=ft.useState(new Gp.CssClasses),[n,o]=ft.useState(new Gp.CssClasses),[a,s]=ft.useState(new Gp.CssClasses),[l,u]=ft.useState(),[c,p]=ft.useState(),h=ft.useRef(null),d=ft.useRef(null),f=ft.useRef(null),g=ft.useRef(null),v=ft.useRef(!1),m=ft.useRef(),C=ft.useRef();HS.useEffectOnce(()=>{C.current=new KB.AgPromise(W=>{m.current=W})});const w=W=>{v.current||W!=null&&(m.current&&m.current(W),v.current=!0)},{ctrl:b}=i;HS.useEffectOnce(()=>{const W={addOrRemoveCssClass:(G,k)=>r(V=>V.setClass(G,k)),addOrRemoveBodyCssClass:(G,k)=>o(V=>V.setClass(G,k)),addOrRemoveButtonWrapperCssClass:(G,k)=>s(V=>V.setClass(G,k)),setWidth:G=>u(G),setCompDetails:G=>p(G),getFloatingFilterComp:()=>C.current?C.current:null,setMenuIcon:G=>g.current.appendChild(G)};b.setComp(W,h.current,g.current,d.current)}),ft.useEffect(()=>YB.showJsComp(c,t,d.current,w),[c]);const S=ft.useMemo(()=>({width:l}),[l]),T=ft.useMemo(()=>"ag-header-cell ag-floating-filter "+e.toString(),[e]),E=ft.useMemo(()=>n.toString(),[n]),P=ft.useMemo(()=>"ag-floating-filter-button "+a.toString(),[n]),R=ft.useMemo(()=>!!(c&&c.componentFromFramework&&Gp.isComponentStateless(c.componentClass)),[c]),M=c&&c.componentFromFramework,N=c&&c.componentClass;return ft.default.createElement("div",{ref:h,className:T,style:S,role:"gridcell",tabIndex:-1},ft.default.createElement("div",{ref:d,className:E,role:"presentation"},M&&R&&ft.default.createElement(N,Object.assign({},c.params)),M&&!R&&ft.default.createElement(N,Object.assign({},c.params,{ref:w}))),ft.default.createElement("div",{ref:f,className:P,role:"presentation"},ft.default.createElement("button",{ref:g,type:"button","aria-label":"Open Filter Menu",className:"ag-floating-filter-button-button",tabIndex:-1})))};Nw.default=ft.memo(XB);var qB=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t},Gw=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(Fw,"__esModule",{value:!0});const Sa=jt,rr=qB(we.exports),ZB=qt,QB=Gw(Iw),JB=Gw(Lw),eH=Gw(Nw),tH=Ut,rH=i=>{const{gridOptionsWrapper:t}=rr.useContext(ZB.BeansContext),[e,r]=rr.useState(),[n,o]=rr.useState(),[a,s]=rr.useState(),[l,u]=rr.useState(),[c,p]=rr.useState(),[h,d]=rr.useState([]),f=rr.useRef(null),{ctrl:g}=i,v=g.getType()===Sa.HeaderRowType.COLUMN,m=g.getType()===Sa.HeaderRowType.COLUMN_GROUP,C=g.getType()===Sa.HeaderRowType.FLOATING_FILTER,w=rr.useCallback((E,P)=>{if(t.isEnsureDomOrder())return P;const R=Sa._.mapById(E,G=>G.getInstanceId()),M=Sa._.mapById(P,G=>G.getInstanceId()),N=E.filter(G=>M.has(G.getInstanceId())),W=P.filter(G=>!R.has(G.getInstanceId()));return[...N,...W]},[]);tH.useEffectOnce(()=>{const E={setTransform:P=>r(P),setHeight:P=>o(P),setTop:P=>s(P),setHeaderCtrls:P=>d(R=>w(R,P)),setWidth:P=>u(P),setAriaRowIndex:P=>p(P)};g.setComp(E)});const b=rr.useMemo(()=>({transform:e,height:n,top:a,width:l}),[e,n,a,l]),S=rr.useMemo(()=>{const E=["ag-header-row"];return v&&E.push("ag-header-row-column"),m&&E.push("ag-header-row-column-group"),C&&E.push("ag-header-row-column-filter"),E.join(" ")},[]),T=rr.useCallback(E=>{switch(g.getType()){case Sa.HeaderRowType.COLUMN_GROUP:return rr.default.createElement(JB.default,{ctrl:E,key:E.getInstanceId()});case Sa.HeaderRowType.FLOATING_FILTER:return rr.default.createElement(eH.default,{ctrl:E,key:E.getInstanceId()});default:return rr.default.createElement(QB.default,{ctrl:E,key:E.getInstanceId()})}},[]);return rr.default.createElement("div",{ref:f,className:S,role:"row",style:b,"aria-rowindex":c},h.map(T))};Fw.default=rr.memo(rH);var nH=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t},iH=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(Mw,"__esModule",{value:!0});const zt=nH(we.exports),oH=qt,iv=jt,aH=Sr,sH=iH(Fw),lH=Ut,uH=i=>{const[t,e]=zt.useState(new aH.CssClasses),[r,n]=zt.useState(),[o,a]=zt.useState(),[s,l]=zt.useState(),[u,c]=zt.useState([]),{context:p}=zt.useContext(oH.BeansContext),h=zt.useRef(null),d=i.pinned===iv.Constants.PINNED_LEFT,f=i.pinned===iv.Constants.PINNED_RIGHT,g=!d&&!f;lH.useEffectOnce(()=>{const b={addOrRemoveCssClass:(T,E)=>e(P=>P.setClass(T,E)),setCtrls:T=>c(T),setCenterWidth:T=>n(T),setContainerTransform:T=>a(T),setPinnedContainerWidth:T=>l(T)},S=p.createBean(new iv.HeaderRowContainerCtrl(i.pinned));return S.setComp(b,h.current),()=>{p.destroyBean(S)}});const v=zt.useMemo(()=>t.toString(),[t]),m=()=>u.map(b=>zt.default.createElement(sH.default,{ctrl:b,key:b.getInstanceId()})),C=zt.useMemo(()=>({width:r,transform:o}),[r,o]),w=zt.useMemo(()=>({width:s,minWidth:s,maxWidth:s}),[s]);return zt.default.createElement(zt.default.Fragment,null,d&&zt.default.createElement("div",{ref:h,className:"ag-pinned-left-header "+v,role:"presentation",style:w},m()),f&&zt.default.createElement("div",{ref:h,className:"ag-pinned-right-header "+v,role:"presentation",style:w},m()),g&&zt.default.createElement("div",{ref:h,className:"ag-header-viewport "+v,role:"presentation"},zt.default.createElement("div",{className:"ag-header-container",role:"rowgroup",style:C},m())))};Mw.default=zt.memo(uH);var cH=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t},pH=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(Dw,"__esModule",{value:!0});const Un=cH(we.exports),hH=qt,ov=jt,dH=Sr,av=pH(Mw),fH=Ut,gH=()=>{const[i,t]=Un.useState(new dH.CssClasses),[e,r]=Un.useState(),{context:n}=Un.useContext(hH.BeansContext),o=Un.useRef(null);fH.useEffectOnce(()=>{const l={addOrRemoveCssClass:(c,p)=>t(h=>h.setClass(c,p)),setHeightAndMinHeight:c=>r(c)},u=n.createBean(new ov.GridHeaderCtrl);return u.setComp(l,o.current,o.current),()=>{n.destroyBean(u)}});const a=Un.useMemo(()=>{let l=i.toString();return"ag-header "+l},[i]),s=Un.useMemo(()=>({height:e,minHeight:e}),[e]);return Un.default.createElement("div",{ref:o,className:a,style:s,role:"presentation"},Un.default.createElement(av.default,{pinned:ov.Constants.PINNED_LEFT}),Un.default.createElement(av.default,{pinned:null}),Un.default.createElement(av.default,{pinned:ov.Constants.PINNED_RIGHT}))};Dw.default=Un.memo(gH);var np={};Object.defineProperty(np,"__esModule",{value:!0});const vH=Ut,mH=(i,t)=>{vH.useEffectOnce(()=>{const e=t.current,r=e.parentElement;if(!r)return;const n=document.createComment(i);return r.insertBefore(n,e),()=>{r.removeChild(n)}})};np.default=mH;var kw={},Ww={},kP={},Vw={},yH=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t};Object.defineProperty(Vw,"__esModule",{value:!0});const CH=jt,Zu=yH(we.exports),wH=Jy.exports,bH=qt,_H=Ut,SH=i=>{const[t,e]=Zu.useState(),{context:r,popupService:n,gridOptionsWrapper:o}=Zu.useContext(bH.BeansContext);return _H.useEffectOnce(()=>{const{editDetails:a,cellCtrl:s,eParentCell:l}=i,{compDetails:u}=a,c=o.isStopEditingWhenCellsLoseFocus(),p=r.createBean(new CH.PopupEditorWrapper(u.params)),h=p.getGui();if(i.jsChildComp){const C=i.jsChildComp.getGui();C&&h.appendChild(C)}const d={column:s.getColumn(),rowNode:s.getRowNode(),type:"popupCellEditor",eventSource:l,ePopup:h,keepWithinBounds:!0},f=a.popupPosition==="under"?n.positionPopupUnderComponent.bind(n,d):n.positionPopupOverComponent.bind(n,d),g=o.getLocaleTextFunc(),v=n.addPopup({modal:c,eChild:h,closeOnEsc:!0,closedCallback:()=>{s.onPopupEditorClosed()},anchorToElement:l,positionCallback:f,ariaLabel:g("ariaLabelCellEditor","Cell Editor")}),m=v?v.hideFunc:void 0;return e(p),i.jsChildComp&&i.jsChildComp.afterGuiAttached&&i.jsChildComp.afterGuiAttached(),()=>{m!=null&&m(),r.destroyBean(p)}}),Zu.default.createElement(Zu.default.Fragment,null,t&&i.wrappedContent&&wH.createPortal(i.wrappedContent,t.getGui()))};Vw.default=Zu.memo(SH);var Bw={};Object.defineProperty(Bw,"__esModule",{value:!0});const sv=we.exports,xH=qt,TH=Ut,OH=(i,t,e,r,n,o)=>{const{context:a,userComponentFactory:s}=sv.useContext(xH.BeansContext),l=sv.useCallback(()=>{const u=n.current;if(!u)return;const c=u.getGui();c&&c.parentElement&&c.parentElement.removeChild(c),a.destroyBean(u),n.current=void 0},[]);sv.useEffect(()=>{const u=i!=null,c=i&&i.compDetails&&!i.compDetails.componentFromFramework,p=t&&e==null;if(!(u&&c&&!p)){l();return}const d=i.compDetails;if(n.current){const C=n.current,b=C.refresh!=null&&i.force==!1?C.refresh(d.params):!1;if(b===!0||b===void 0)return;l()}const f=d.newAgStackInstance();if(!f)return;const g=f.resolveNow(null,C=>C);if(!g)return;const v=g.getGui();if(!v)return;(t?e:o.current).appendChild(v),n.current=g},[i,t,r]),TH.useEffectOnce(()=>l)};Bw.default=OH;(function(i){var t=de&&de.__importStar||function(g){if(g&&g.__esModule)return g;var v={};if(g!=null)for(var m in g)Object.hasOwnProperty.call(g,m)&&(v[m]=g[m]);return v.default=g,v},e=de&&de.__importDefault||function(g){return g&&g.__esModule?g:{default:g}};Object.defineProperty(i,"__esModule",{value:!0});const r=jt,n=t(we.exports),o=Sr,a=e(Vw),s=e(Bw),l=qt,u=mo,c=Ut;(function(g){g[g.ShowValue=0]="ShowValue",g[g.EditValue=1]="EditValue"})(i.CellCompState||(i.CellCompState={}));const p=(g,v,m)=>{const C=m.getColumn();if(!g&&v.isPopup&&v.isPopup()){const w=`AG Grid: Found an issue in column ${C.getColId()}. If using React, specify an editor is a popup using colDef.cellEditorPopup=true. AG Grid React cannot depend on the editor component specifying if it's in a popup (via the isPopup() method on the editor), as React needs to know this information BEFORE the component is created.`;r._.doOnce(()=>console.warn(w),"jsEditorComp-isPopup-"+m.getColumn().getColId())}if(g&&v.getPopupPosition&&v.getPopupPosition()!=null){const w=`AG Grid: Found an issue in column ${C.getColId()}. If using React, specify an editor popup position using colDef.cellEditorPopupPosition=true. AG Grid React cannot depend on the editor component specifying it's position (via the getPopupPosition() method on the editor), as React needs to know this information BEFORE the component is created.`;r._.doOnce(()=>console.warn(w),"jsEditorComp-getPopupPosition-"+m.getColumn().getColId())}},h=(g,v,m,C,w,b)=>{const S=g.compDetails,T=S.componentClass,E=S.componentFromFramework&&!g.popup,P=S.componentFromFramework&&g.popup,R=!S.componentFromFramework&&g.popup;return n.default.createElement(n.default.Fragment,null,E&&n.default.createElement(T,Object.assign({},g.compDetails.params,{ref:v})),P&&n.default.createElement(a.default,{editDetails:g,cellCtrl:w,eParentCell:C,wrappedContent:n.default.createElement(T,Object.assign({},g.compDetails.params,{ref:m}))}),R&&b&&n.default.createElement(a.default,{editDetails:g,cellCtrl:w,eParentCell:C,jsChildComp:b}))},d=(g,v,m,C,w,b,S)=>{const{compDetails:T,value:E}=g,P=!T,R=T&&T.componentFromFramework,M=T&&T.componentClass,N=E&&E.toString?E.toString():E,W=()=>n.default.createElement(n.default.Fragment,null,P&&n.default.createElement(n.default.Fragment,null,N),R&&!b&&n.default.createElement(M,Object.assign({},T.params,{key:v,ref:C})),R&&b&&n.default.createElement(M,Object.assign({},T.params,{key:v})));return n.default.createElement(n.default.Fragment,null,w?n.default.createElement("span",{role:"presentation",id:`cell-${m}`,className:"ag-cell-value",ref:S},W()):W())},f=g=>{const{context:v}=n.useContext(l.BeansContext),{cellCtrl:m,printLayout:C,editingRow:w}=g,[b,S]=n.useState(),[T,E]=n.useState(),[P,R]=n.useState(1),[M,N]=n.useState(new o.CssClasses),[W,G]=n.useState(),[k,V]=n.useState(),[U,J]=n.useState(),[Z,le]=n.useState(),[Y,ae]=n.useState(),[z,X]=n.useState(),[ue,pe]=n.useState(!1),[be,_e]=n.useState(!1),[Te,Pe]=n.useState(!1),[Ee,Le]=n.useState(),Ze=n.useMemo(()=>m.isForceWrapper(),[]),Re=n.useRef(null),Ge=n.useRef(null),Xe=n.useRef(),ie=n.useRef(),He=n.useRef(),[Me,ne]=n.useState(0),ke=n.useCallback(Qe=>{He.current=Qe,ne(yt=>yt+1)},[]),qe=n.useRef(),[it,mn]=n.useState(0),Yr=n.useCallback(Qe=>{qe.current=Qe,mn(yt=>yt+1)},[]),xr=b!=null&&(ue||Te||be),Xr=Ze||xr,Bn=n.useCallback((Qe,yt)=>{ie.current=yt,yt&&(p(Qe,yt,m),yt.isCancelBeforeStart&&yt.isCancelBeforeStart()&&setTimeout(()=>m.stopEditing(),0))},[]),yo=n.useCallback(Qe=>Bn(!0,Qe),[]),Co=n.useCallback(Qe=>Bn(!1,Qe),[]);s.default(b,Xr,qe.current,it,Xe,Re);const Gi=n.useRef();n.useEffect(()=>{const Qe=Gi.current,yt=b;if(Gi.current=b,Qe==null||Qe.compDetails==null||yt==null||yt.compDetails==null)return;const We=Qe.compDetails,St=yt.compDetails;if(We.componentClass!=St.componentClass||Ge.current==null||Ge.current.refresh==null)return;Ge.current.refresh(St.params)!=!0&&R(va=>va+1)},[b]),n.useEffect(()=>{if(!(T&&!T.compDetails.componentFromFramework))return;const yt=T.compDetails,We=T.popup===!0,St=u.createSyncJsComp(yt);if(!St)return;const pr=St.getGui();return Bn(We,St),We||(Re.current.appendChild(pr),St.afterGuiAttached&&St.afterGuiAttached()),Le(St),()=>{v.destroyBean(St),Bn(We,void 0),Le(void 0),pr&&pr.parentElement&&pr.parentElement.removeChild(pr)}},[T]),n.useEffect(()=>{if(!m||!v||(J(He.current?`cell-${m.getInstanceId()}`:void 0),!He.current||!Xr))return;const Qe=[],yt=We=>{if(We){const St=We.getGui();He.current.insertAdjacentElement("afterbegin",St),Qe.push(()=>{v.destroyBean(We),r._.removeFromParent(St)})}return We};return ue&&yt(m.createSelectionCheckbox()),Te&&yt(m.createDndSource()),be&&yt(m.createRowDragComp()),()=>{Qe.forEach(We=>{We()})}},[Xr,Te,be,ue,Me]),c.useEffectOnce(()=>{if(!m)return;const Qe={addOrRemoveCssClass:(We,St)=>N(pr=>pr.setClass(We,St)),setUserStyles:We=>G(We),getFocusableElement:()=>Re.current,setTabIndex:We=>V(We),setRole:We=>le(We),setColId:We=>ae(We),setTitle:We=>X(We),setIncludeSelection:We=>pe(We),setIncludeRowDrag:We=>_e(We),setIncludeDndSource:We=>Pe(We),getCellEditor:()=>ie.current||null,getCellRenderer:()=>Ge.current?Ge.current:Xe.current,getParentOfValue:()=>qe.current?qe.current:He.current?He.current:Re.current,setRenderDetails:(We,St,pr)=>{S({value:St,compDetails:We,force:pr})},setEditDetails:(We,St,pr)=>{We?(E({compDetails:We,popup:St,popupPosition:pr}),St||S(void 0)):E(void 0)}},yt=He.current||void 0;m.setComp(Qe,Re.current,yt,C,w)});const wo=n.useMemo(()=>!!(b&&b.compDetails&&b.compDetails.componentFromFramework&&o.isComponentStateless(b.compDetails.componentClass)),[b]),fa=n.useMemo(()=>{let Qe=M.toString();return Xr||(Qe+=" ag-cell-value"),Qe},[M,xr]),ga=n.useMemo(()=>m.getInstanceId(),[]),bo=()=>n.default.createElement(n.default.Fragment,null,b!=null&&d(b,P,ga,Ge,Xr,wo,Yr),T!=null&&h(T,Co,yo,Re.current,m,Ee));return n.default.createElement("div",{ref:Re,className:fa,style:W,tabIndex:k,role:Z,"col-id":Y,title:z,"aria-describedby":U},Xr?n.default.createElement("div",{className:"ag-cell-wrapper",role:"presentation",ref:ke},bo()):bo())};i.default=n.memo(f)})(kP);var EH=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t},RH=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(Ww,"__esModule",{value:!0});const st=EH(we.exports),PH=jt,AH=mo,DH=Sr,MH=qt,FH=RH(kP),IH=Ut,LH=(i,t,e)=>{if(e){const l={list:t,instanceIdMap:new Map};return t.forEach(u=>l.instanceIdMap.set(u.getInstanceId(),u)),l}const r=[],n=[],o=new Map,a=new Map;return t.forEach(l=>a.set(l.getInstanceId(),l)),i.list.forEach(l=>{const u=l.getInstanceId();a.has(u)&&(r.push(l),o.set(u,l))}),t.forEach(l=>{const u=l.getInstanceId();i.instanceIdMap.has(u)||(n.push(l),o.set(u,l))}),{list:[...r,...n],instanceIdMap:o}},NH=i=>{const{context:t}=st.useContext(MH.BeansContext),{rowCtrl:e,containerType:r}=i,[n,o]=st.useState(),[a,s]=st.useState(),[l,u]=st.useState(),[c,p]=st.useState(),[h,d]=st.useState(),[f,g]=st.useState(),[v,m]=st.useState({list:[],instanceIdMap:new Map}),[C,w]=st.useState(),[b,S]=st.useState(!1),[T,E]=st.useState(e.getInitialRowTop()),[P,R]=st.useState(e.getInitialTransform()),M=st.useRef(null),N=st.useRef(),W=st.useRef(!1),[G,k]=st.useState(0);st.useEffect(()=>{var z;if(W.current||!C||G>10)return;const X=(z=M.current)===null||z===void 0?void 0:z.firstChild;X?(e.setupDetailRowAutoHeight(X),W.current=!0):k(ue=>ue+1)},[C,G]);const V=st.useMemo(()=>new PH.CssClassManager(()=>M.current),[]);IH.useEffectOnce(()=>{if(!e.isAlive())return;const z={setTop:X=>E(X),setTransform:X=>R(X),addOrRemoveCssClass:(X,ue)=>V.addOrRemoveCssClass(X,ue),setDomOrder:X=>S(X),setRowIndex:X=>o(X),setRowId:X=>s(X),setRowBusinessKey:X=>p(X),setTabIndex:X=>d(X),setUserStyles:X=>g(X),setRole:X=>u(X),setCellCtrls:X=>m(ue=>LH(ue,X,b)),showFullWidth:X=>w(X),getFullWidthCellRenderer:()=>N.current};e.setComp(z,M.current,r)}),st.useEffect(()=>AH.showJsComp(C,t,M.current,N),[C]);const U=st.useMemo(()=>{const z={top:T,transform:P};return Object.assign(z,f),z},[T,P,f]),J=C&&C.componentFromFramework,Z=v!=null,le=st.useMemo(()=>{var z;return!!(((z=C)===null||z===void 0?void 0:z.componentFromFramework)&&DH.isComponentStateless(C.componentClass))},[C]),Y=()=>v.list.map(z=>st.default.createElement(FH.default,{cellCtrl:z,editingRow:e.isEditing(),printLayout:e.isPrintLayout(),key:z.getInstanceId()})),ae=()=>{const z=C.componentClass;return st.default.createElement(st.default.Fragment,null,le&&st.default.createElement(z,Object.assign({},C.params)),!le&&st.default.createElement(z,Object.assign({},C.params,{ref:N})))};return st.default.createElement("div",{ref:M,role:l,style:U,"row-index":n,"row-id":a,"row-business-key":c,tabIndex:h},Z&&Y(),J&&ae())};Ww.default=st.memo(NH);var GH=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t},WP=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(kw,"__esModule",{value:!0});const Ws=jt,Ct=GH(we.exports),lv=Sr,kH=WP(np),WH=WP(Ww),VH=qt,BH=Ut,HH=i=>{const{context:t}=Ct.useContext(VH.BeansContext),[e,r]=Ct.useState(""),[n,o]=Ct.useState([]),[a,s]=Ct.useState([]),[l,u]=Ct.useState(!1),[c,p]=Ct.useState(""),{name:h}=i,d=Ct.useMemo(()=>Ws.getRowContainerTypeForName(h),[h]),f=Ct.useRef(null),g=Ct.useRef(null),v=Ct.useRef(null),m=Ct.useMemo(()=>Ws.RowContainerCtrl.getRowContainerCssClasses(h),[h]),C=Ct.useMemo(()=>lv.classesList(m.wrapper),[]),w=Ct.useMemo(()=>lv.classesList(m.viewport),[]),b=Ct.useMemo(()=>lv.classesList(m.container),[]),S=h===Ws.RowContainerName.CENTER,T=h===Ws.RowContainerName.TOP_CENTER||h===Ws.RowContainerName.BOTTOM_CENTER,E=!S&&!T,P=S?f:T?g:v;kH.default(" AG Row Container "+h+" ",P),Ct.useEffect(()=>{o(W=>{if(l)return a;const G=W.filter(U=>a.indexOf(U)>=0),k=a.filter(U=>G.indexOf(U)<0);return[...G,...k]})},[l,a]),BH.useEffectOnce(()=>{const W=[],G={setViewportHeight:r,setRowCtrls:V=>s(V),setDomOrder:V=>u(V),setContainerWidth:V=>p(V)},k=t.createBean(new Ws.RowContainerCtrl(h));return W.push(k),k.setComp(G,v.current,g.current,f.current),()=>{t.destroyBeans(W)}});const R=Ct.useMemo(()=>({height:e}),[e]),M=Ct.useMemo(()=>({width:c}),[c]),N=()=>Ct.default.createElement("div",{className:b,ref:v,role:a.length?"rowgroup":"presentation",style:M},n.map(W=>Ct.default.createElement(WH.default,{rowCtrl:W,containerType:d,key:W.getInstanceId()})));return Ct.default.createElement(Ct.default.Fragment,null,S&&Ct.default.createElement("div",{className:C,ref:f,role:"presentation"},Ct.default.createElement("div",{className:w,ref:g,role:"presentation",style:R},N())),T&&Ct.default.createElement("div",{className:w,ref:g,role:"presentation",style:R},N()),E&&N())};kw.default=Ct.memo(HH);var jH=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t},Hw=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(Aw,"__esModule",{value:!0});const Qr=jt,lt=jH(we.exports),UH=qt,zH=Hw(Dw),kp=Hw(np),$H=Hw(kw),KH=Ut,Wp=Sr,YH=()=>{const{context:i,agStackComponentsRegistry:t,resizeObserverService:e}=lt.useContext(UH.BeansContext),[r,n]=lt.useState(""),[o,a]=lt.useState(0),[s,l]=lt.useState(0),[u,c]=lt.useState(0),[p,h]=lt.useState(0),[d,f]=lt.useState(""),[g,v]=lt.useState(""),[m,C]=lt.useState(null),[w,b]=lt.useState(null),[S,T]=lt.useState(""),[E,P]=lt.useState(null),[R,M]=lt.useState("ag-layout-normal"),N=lt.useRef(null),W=lt.useRef(null),G=lt.useRef(null),k=lt.useRef(null);kp.default(" AG Grid Body ",N),kp.default(" AG Pinned Top ",W),kp.default(" AG Middle ",G),kp.default(" AG Pinned Bottom ",k),KH.useEffectOnce(()=>{const X=[],ue=[];if(!i)return;const pe=Te=>{const Pe=t.getComponentClass(Te),Ee=i.createBean(new Pe);return X.push(Ee),Ee};N.current.appendChild(document.createComment(" AG Fake Horizontal Scroll ")),N.current.appendChild(pe("AG-FAKE-HORIZONTAL-SCROLL").getGui()),N.current.appendChild(document.createComment(" AG Overlay Wrapper ")),N.current.appendChild(pe("AG-OVERLAY-WRAPPER").getGui());const be={setRowAnimationCssOnBodyViewport:n,setColumnCount:a,setRowCount:l,setTopHeight:c,setBottomHeight:h,setTopDisplay:f,setBottomDisplay:v,setColumnMovingCss:C,updateLayoutClasses:M,setAlwaysVerticalScrollClass:b,setPinnedTopBottomOverflowY:T,setCellSelectableCss:P,registerBodyViewportResizeListener:Te=>{const Pe=e.observeResize(G.current,Te);ue.push(()=>Pe())}},_e=i.createBean(new Qr.GridBodyCtrl);return X.push(_e),_e.setComp(be,N.current,G.current,W.current,k.current),()=>{i.destroyBeans(X),ue.forEach(Te=>Te())}});const V=lt.useMemo(()=>Wp.classesList("ag-root","ag-unselectable",m,R),[m,R]),U=lt.useMemo(()=>Wp.classesList("ag-body-viewport",r,R,w,E),[r,R,w,E]),J=lt.useMemo(()=>Wp.classesList("ag-floating-top",E),[E]),Z=lt.useMemo(()=>Wp.classesList("ag-floating-bottom",E),[E]),le=lt.useMemo(()=>({height:u,minHeight:u,display:d,overflowY:S}),[u,d,S]),Y=lt.useMemo(()=>({height:p,minHeight:p,display:g,overflowY:S}),[p,g,S]),ae=X=>lt.default.createElement($H.default,{name:X,key:`${X}-container`}),z=({section:X,children:ue,className:pe,style:be})=>lt.default.createElement("div",{ref:X,className:pe,role:"presentation",style:be},ue.map(ae));return lt.default.createElement("div",{ref:N,className:V,role:"grid","aria-colcount":o,"aria-rowcount":s},lt.default.createElement(zH.default,null),z({section:W,className:J,style:le,children:[Qr.RowContainerName.TOP_LEFT,Qr.RowContainerName.TOP_CENTER,Qr.RowContainerName.TOP_RIGHT,Qr.RowContainerName.TOP_FULL_WIDTH]}),z({section:G,className:U,children:[Qr.RowContainerName.LEFT,Qr.RowContainerName.CENTER,Qr.RowContainerName.RIGHT,Qr.RowContainerName.FULL_WIDTH]}),z({section:k,className:Z,style:Y,children:[Qr.RowContainerName.BOTTOM_LEFT,Qr.RowContainerName.BOTTOM_CENTER,Qr.RowContainerName.BOTTOM_RIGHT,Qr.RowContainerName.BOTTOM_FULL_WIDTH]}))};Aw.default=lt.memo(YH);var jw={},XH=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t};Object.defineProperty(jw,"__esModule",{value:!0});const $n=XH(we.exports),qH=jt,ZH=qt,QH=Ut,JH=(i,t)=>{const{children:e,eFocusableElement:r,onTabKeyDown:n,gridCtrl:o}=i,{context:a}=$n.useContext(ZH.BeansContext),s=$n.useRef(null),l=$n.useRef(null),u=$n.useRef(),[c,p]=$n.useState();$n.useImperativeHandle(t,()=>({forceFocusOutOfContainer(){u.current.forceFocusOutOfContainer()}})),QH.useEffectOnce(()=>{const d=s.current,f=l.current,g={setTabIndex:m=>m==null?p(void 0):p(parseInt(m,10))},v=u.current=a.createBean(new qH.TabGuardCtrl({comp:g,eTopGuard:d,eBottomGuard:f,eFocusableElement:r,onTabKeyDown:n,focusInnerElement:m=>o.focusInnerElement(m)}));return()=>{a.destroyBean(v)}});const h=d=>$n.default.createElement("div",{className:`ag-tab-guard ag-tab-guard-${d}`,role:"presentation",tabIndex:c,ref:d==="top"?s:l});return $n.default.createElement($n.default.Fragment,null,h("top"),e,h("bottom"))},ej=$n.forwardRef(JH);jw.default=$n.memo(ej);var tj=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t},Uw=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(Pw,"__esModule",{value:!0});const jS=jt,wt=tj(we.exports),rj=qt,nj=Uw(Aw),ij=Uw(np),oj=Uw(jw),US=Ut,zS=Sr,aj=({context:i})=>{const[t,e]=wt.useState(""),[r,n]=wt.useState(""),[o,a]=wt.useState(""),[s,l]=wt.useState(null),[u,c]=wt.useState(null),[p,h]=wt.useState(!1),[d,f]=wt.useState(),g=wt.useRef(null),v=wt.useRef(null),m=wt.useRef(),C=wt.useRef(null),w=wt.useRef(()=>{}),b=wt.useCallback(()=>{},[]),S=wt.useMemo(()=>i.getBean("beans"),[]);ij.default(" AG Grid ",v),US.useEffectOnce(()=>{const N=g.current=i.createBean(new jS.GridCtrl);return()=>{i.destroyBean(N),g.current=null}}),US.useEffectOnce(()=>{const N=g.current;w.current=N.focusInnerElement.bind(N);const W={destroyGridUi:()=>{},setRtlClass:e,addOrRemoveKeyboardFocusClass:G=>n(G?jS.FocusService.AG_KEYBOARD_FOCUS:""),forceFocusOutOfContainer:()=>{m.current.forceFocusOutOfContainer()},updateLayoutClasses:a,getFocusableContainers:()=>{const G=[],k=v.current.querySelector(".ag-root"),V=v.current.querySelector(".ag-side-bar");return k&&G.push(k),V&&G.push(V),G},setCursor:l,setUserSelect:c};N.setComp(W,v.current,v.current),h(!0)}),wt.useEffect(()=>{if(!d)return;const N=g.current,W=[],{agStackComponentsRegistry:G}=S,k=G.getComponentClass("AG-GRID-HEADER-DROP-ZONES"),V=G.getComponentClass("AG-SIDE-BAR"),U=G.getComponentClass("AG-STATUS-BAR"),J=G.getComponentClass("AG-WATERMARK"),Z=G.getComponentClass("AG-PAGINATION"),le=[],Y=v.current,ae=C.current;if(N.showDropZones()&&k){const z=i.createBean(new k),X=z.getGui();Y.insertAdjacentElement("afterbegin",X),le.push(X),W.push(z)}if(N.showSideBar()&&V){const z=i.createBean(new V),X=z.getGui(),ue=ae.querySelector(".ag-tab-guard-bottom");ue&&(ue.insertAdjacentElement("beforebegin",X),le.push(X)),W.push(z)}if(N.showStatusBar()&&U){const z=i.createBean(new U),X=z.getGui();Y.insertAdjacentElement("beforeend",X),le.push(X),W.push(z)}if(Z){const z=i.createBean(new Z),X=z.getGui();Y.insertAdjacentElement("beforeend",X),le.push(X),W.push(z)}if(N.showWatermark()&&J){const z=i.createBean(new J),X=z.getGui();Y.insertAdjacentElement("beforeend",X),le.push(X),W.push(z)}return()=>{i.destroyBeans(W),le.forEach(z=>{z.parentElement&&z.parentElement.removeChild(z)})}},[d]);const T=wt.useMemo(()=>zS.classesList("ag-root-wrapper",t,r,o),[t,r,o]),E=wt.useMemo(()=>zS.classesList("ag-root-wrapper-body","ag-focus-managed",o),[o]),P=wt.useMemo(()=>({userSelect:u!=null?u:"",WebkitUserSelect:u!=null?u:"",cursor:s!=null?s:""}),[u,s]),R=C.current,M=wt.useCallback(N=>{m.current=N,f(!0)},[]);return wt.default.createElement("div",{ref:v,className:T,style:P},wt.default.createElement("div",{className:E,ref:C},p&&R&&wt.default.createElement(rj.BeansContext.Provider,{value:S},wt.default.createElement(oj.default,{ref:M,eFocusableElement:R,onTabKeyDown:b,gridCtrl:g.current},wt.default.createElement(nj.default,null)))))};Pw.default=wt.memo(aj);var sj=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t},lj=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(_f,"__esModule",{value:!0});const Hi=jt,Vp=sj(we.exports),Bp=rp,uv=Tw,uj=yf,cj=Cf,pj=lj(Pw),hj=bf;class dj extends Vp.Component{constructor(t){super(t),this.props=t,this.destroyFuncs=[],this.changeDetectionService=new uv.ChangeDetectionService,this.eGui=Vp.default.createRef(),this.whenReadyFuncs=[],this.ready=!1,this.renderedAfterMount=!1,this.mounted=!1,this.state={context:void 0},this.portalManager=new cj.PortalManager(this,t.componentWrappingElement,t.maxComponentCreationTimeMs),this.destroyFuncs.push(()=>this.portalManager.destroy())}render(){return""+this.state.context,this.state.context&&(this.renderedAfterMount=!0),Vp.default.createElement("div",{style:this.createStyleForDiv(),className:this.props.className,ref:this.eGui},this.state.context&&Vp.default.createElement(pj.default,{context:this.state.context}),this.portalManager.getPortals())}createStyleForDiv(){return Object.assign({height:"100%"},this.props.containerStyle||{})}componentDidMount(){if(this.mounted)return;this.mounted=!0;const t=this.props.modules||[],e={providedBeanInstances:{frameworkComponentWrapper:new fj(this.portalManager)},modules:t,frameworkOverrides:new hj.ReactFrameworkOverrides(!0)};this.gridOptions=this.props.gridOptions||{};const{children:r}=this.props;Bp.AgGridColumn.hasChildColumns(r)&&(this.gridOptions.columnDefs=Bp.AgGridColumn.mapChildColumnDefs(r)),this.gridOptions=Hi.ComponentUtil.copyAttributesToGridOptions(this.gridOptions,this.props);const n=s=>{this.setState({context:s}),s.getBean(Hi.CtrlsService.NAME).whenReady(()=>{this.api=this.gridOptions.api,this.columnApi=this.gridOptions.columnApi,this.props.setGridApi(this.api,this.columnApi),this.destroyFuncs.push(()=>this.api.destroy())})},o=s=>{s.getBean(Hi.CtrlsService.NAME).whenReady(()=>{this.whenReadyFuncs.forEach(u=>u()),this.whenReadyFuncs.length=0,this.ready=!0})};new Hi.GridCoreCreator().create(this.eGui.current,this.gridOptions,n,o,e)}componentWillUnmount(){this.renderedAfterMount&&(this.destroyFuncs.forEach(t=>t()),this.destroyFuncs.length=0)}componentDidUpdate(t){this.processPropsChanges(t,this.props)}processPropsChanges(t,e){const r={};this.extractGridPropertyChanges(t,e,r),this.extractDeclarativeColDefChanges(e,r),this.processChanges(r)}extractDeclarativeColDefChanges(t,e){if(this.props.gridOptions&&this.props.gridOptions.columnDefs||this.props.columnDefs)return;const r=!!t.debug,n="columnDefs",o=this.gridOptions.columnDefs;if(Bp.AgGridColumn.hasChildColumns(t.children)){const a=this.changeDetectionService.getStrategy(this.getStrategyTypeForProp(n)),s=Bp.AgGridColumn.mapChildColumnDefs(t.children);a.areEqual(o,s)||(r&&console.log("agGridReact: colDefs definitions changed"),e[n]={previousValue:o,currentValue:s})}else o&&o.length>0&&(e[n]={previousValue:o,currentValue:[]})}extractGridPropertyChanges(t,e,r){const n=!!e.debug;Object.keys(e).forEach(o=>{Hi._.includes(Hi.ComponentUtil.ALL_PROPERTIES,o)&&(this.changeDetectionService.getStrategy(this.getStrategyTypeForProp(o)).areEqual(t[o],e[o])||(n&&console.log(`agGridReact: [${o}] property changed`),r[o]={previousValue:t[o],currentValue:e[o]}))}),Hi.ComponentUtil.getEventCallbacks().forEach(o=>{t[o]!==e[o]&&(n&&console.log(`agGridReact: [${o}] event callback changed`),r[o]={previousValue:t[o],currentValue:e[o]})})}processChanges(t){this.processWhenReady(()=>Hi.ComponentUtil.processOnChange(t,this.gridOptions,this.api,this.columnApi))}processWhenReady(t){this.ready?t():this.whenReadyFuncs.push(t)}getStrategyTypeForProp(t){if(t==="rowData"){if(this.props.rowDataChangeDetectionStrategy)return this.props.rowDataChangeDetectionStrategy;if(this.isImmutableDataActive())return uv.ChangeDetectionStrategyType.IdentityCheck}return uv.ChangeDetectionStrategyType.DeepValueCheck}isImmutableDataActive(){return this.props.deltaRowDataMode||this.props.immutableData||this.props.getRowId!=null||this.props.gridOptions&&(this.props.gridOptions.deltaRowDataMode||this.props.gridOptions.immutableData||this.props.gridOptions.getRowId!=null)}}_f.AgGridReactUi=dj;class fj extends Hi.BaseComponentWrapper{constructor(t){super(),this.parent=t}createWrapper(t,e){return new uj.NewReactComponent(t,this.parent,e)}}var gj=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t};Object.defineProperty(Rw,"__esModule",{value:!0});const kr=gj(we.exports),$S=Sr,vj=qt,mj=_f,KS=Ut,yj=(i,t)=>{const{ctrlsFactory:e,context:r,gridOptionsWrapper:n,resizeObserverService:o,clientSideRowModel:a,serverSideRowModel:s}=kr.useContext(vj.BeansContext),[l,u]=kr.useState(new $S.CssClasses),[c,p]=kr.useState(new $S.CssClasses),[h,d]=kr.useState(),[f,g]=kr.useState(),v=kr.useRef(),m=kr.useRef(null),C=kr.useMemo(()=>l.toString()+" ag-details-row",[l]),w=kr.useMemo(()=>c.toString()+" ag-details-grid",[c]);t&&kr.useImperativeHandle(t,()=>({refresh(){return v.current.refresh()}})),KS.useEffectOnce(()=>{i.template&&typeof i.template=="string"&&console.warn("AG Grid: detailCellRendererParams.template is not supported by React - this only works with frameworks that work against String templates. To change the template, please provide your own React Detail Cell Renderer.")}),KS.useEffectOnce(()=>{const S={addOrRemoveCssClass:(P,R)=>u(M=>M.setClass(P,R)),addOrRemoveDetailGridCssClass:(P,R)=>p(M=>M.setClass(P,R)),setDetailGrid:P=>d(P),setRowData:P=>g(P),getGui:()=>m.current},T=e.getInstance("detailCellRenderer");if(!T)return;r.createBean(T),T.init(S,i),v.current=T;let E;if(n.isDetailRowAutoHeight()){const P=()=>{if(m.current==null)return;const R=m.current.clientHeight;R!=null&&R>0&&setTimeout(()=>{i.node.setRowHeight(R),a?a.onRowHeightChanged():s&&s.onRowHeightChanged()},0)};E=o.observeResize(m.current,P),P()}return()=>{r.destroyBean(T),E&&E()}});const b=kr.useCallback((S,T)=>{v.current.registerDetailWithMaster(S,T)},[]);return kr.default.createElement("div",{className:C,ref:m},h&&kr.default.createElement(mj.AgGridReactUi,Object.assign({className:w},h,{rowData:f,setGridApi:b})))};Rw.default=kr.forwardRef(yj);var VP=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}};Object.defineProperty(bf,"__esModule",{value:!0});const Cj=jt,YS=VP(Ow),wj=VP(Rw);class bj extends Cj.VanillaFrameworkOverrides{constructor(t){super(),this.frameworkComponents={agGroupCellRenderer:YS.default,agGroupRowRenderer:YS.default,agDetailCellRenderer:wj.default},this.reactUi=t}frameworkComponent(t){if(!!this.reactUi)return this.frameworkComponents[t]}isFrameworkComponent(t){if(!t)return!1;const e=t.prototype;return!(e&&"getGui"in e)}}bf.ReactFrameworkOverrides=bj;var _j=de&&de.__importDefault||function(i){return i&&i.__esModule?i:{default:i}},Sj=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t};Object.defineProperty(CC,"__esModule",{value:!0});const Er=jt,da=_j(Zd.exports),XS=Sj(we.exports),xj=aw,Hp=rp,cv=Tw,Tj=yf,Oj=Cf,Ej=bf;class Ml extends XS.Component{constructor(t){super(t),this.props=t,this.changeDetectionService=new cv.ChangeDetectionService,this.api=null,this.destroyed=!1,this.SYNCHRONOUS_CHANGE_PROPERTIES=["context"],this.portalManager=new Oj.PortalManager(this,t.componentWrappingElement,t.maxComponentCreationTimeMs)}render(){return XS.default.createElement("div",{style:this.createStyleForDiv(),className:this.props.className,ref:t=>{this.eGridDiv=t}},this.portalManager.getPortals())}createStyleForDiv(){return Object.assign({height:"100%"},this.props.containerStyle||{})}componentDidMount(){const t=this.props.modules||[],e={providedBeanInstances:{agGridReact:this,frameworkComponentWrapper:new Rj(this,this.portalManager)},modules:t,frameworkOverrides:new Ej.ReactFrameworkOverrides(!1)},r=this.props.gridOptions||{},{children:n}=this.props;Hp.AgGridColumn.hasChildColumns(n)&&(r.columnDefs=Hp.AgGridColumn.mapChildColumnDefs(n)),this.gridOptions=Er.ComponentUtil.copyAttributesToGridOptions(r,this.props),new Er.Grid(this.eGridDiv,this.gridOptions,e),this.api=this.gridOptions.api,this.columnApi=this.gridOptions.columnApi,this.props.setGridApi(this.api,this.columnApi)}getStrategyTypeForProp(t){if(t==="rowData"){if(this.props.rowDataChangeDetectionStrategy)return this.props.rowDataChangeDetectionStrategy;if(this.isImmutableDataActive())return cv.ChangeDetectionStrategyType.IdentityCheck}return cv.ChangeDetectionStrategyType.DeepValueCheck}isImmutableDataActive(){return this.props.deltaRowDataMode||this.props.immutableData||this.props.getRowId!=null||this.props.gridOptions&&(this.props.gridOptions.deltaRowDataMode||this.props.gridOptions.immutableData||this.props.gridOptions.getRowId!=null)}shouldComponentUpdate(t){return this.processPropsChanges(this.props,t),!1}componentDidUpdate(t){this.processPropsChanges(t,this.props)}processPropsChanges(t,e){const r={};this.extractGridPropertyChanges(t,e,r),this.extractDeclarativeColDefChanges(e,r),this.processSynchronousChanges(r),this.processAsynchronousChanges(r)}extractDeclarativeColDefChanges(t,e){if(this.props.gridOptions&&this.props.gridOptions.columnDefs||this.props.columnDefs)return;const r=!!t.debug,n="columnDefs",o=this.gridOptions.columnDefs;if(Hp.AgGridColumn.hasChildColumns(t.children)){const a=this.changeDetectionService.getStrategy(this.getStrategyTypeForProp(n)),s=Hp.AgGridColumn.mapChildColumnDefs(t.children);a.areEqual(o,s)||(r&&console.log("agGridReact: colDefs definitions changed"),e[n]={previousValue:o,currentValue:s})}else o&&o.length>0&&(e[n]={previousValue:o,currentValue:[]})}extractGridPropertyChanges(t,e,r){const n=!!e.debug;Object.keys(e).forEach(o=>{Er._.includes(Er.ComponentUtil.ALL_PROPERTIES,o)&&(this.changeDetectionService.getStrategy(this.getStrategyTypeForProp(o)).areEqual(t[o],e[o])||(n&&console.log(`agGridReact: [${o}] property changed`),r[o]={previousValue:t[o],currentValue:e[o]}))}),Er.ComponentUtil.getEventCallbacks().forEach(o=>{t[o]!==e[o]&&(n&&console.log(`agGridReact: [${o}] event callback changed`),r[o]={previousValue:t[o],currentValue:e[o]})})}componentWillUnmount(){this.api&&(this.api.destroy(),this.api=null),this.destroyed=!0,this.portalManager.destroy()}isDisableStaticMarkup(){return this.props.disableStaticMarkup===!0}isLegacyComponentRendering(){return this.props.legacyComponentRendering===!0}processSynchronousChanges(t){const e=Object.assign({},t);if(Object.keys(e).length>0){const r={};this.SYNCHRONOUS_CHANGE_PROPERTIES.forEach(n=>{e[n]&&(r[n]=e[n],delete e[n])}),Object.keys(r).length>0&&!!this.api&&Er.ComponentUtil.processOnChange(r,this.gridOptions,this.api,this.columnApi)}return e}processAsynchronousChanges(t){Object.keys(t).length>0&&window.setTimeout(()=>{this.api&&Er.ComponentUtil.processOnChange(t,this.gridOptions,this.api,this.columnApi)})}}CC.AgGridReactLegacy=Ml;Ml.MAX_COMPONENT_CREATION_TIME_IN_MS=1e3;Ml.defaultProps={legacyComponentRendering:!1,disableStaticMarkup:!1,maxComponentCreationTimeMs:Ml.MAX_COMPONENT_CREATION_TIME_IN_MS};Ml.propTypes={gridOptions:da.default.object};_s(Er.ComponentUtil.getEventCallbacks(),da.default.func);_s(Er.ComponentUtil.BOOLEAN_PROPERTIES,da.default.bool);_s(Er.ComponentUtil.STRING_PROPERTIES,da.default.string);_s(Er.ComponentUtil.OBJECT_PROPERTIES,da.default.object);_s(Er.ComponentUtil.ARRAY_PROPERTIES,da.default.array);_s(Er.ComponentUtil.NUMBER_PROPERTIES,da.default.number);_s(Er.ComponentUtil.FUNCTION_PROPERTIES,da.default.func);function _s(i,t){i.forEach(e=>{Ml[e]=t})}class Rj extends Er.BaseComponentWrapper{constructor(t,e){super(),this.agGridReact=t,this.portalManager=e}createWrapper(t,e){return this.agGridReact.isLegacyComponentRendering()?new xj.LegacyReactComponent(t,this.agGridReact,this.portalManager,e):new Tj.NewReactComponent(t,this.portalManager,e)}}var Pj=de&&de.__importStar||function(i){if(i&&i.__esModule)return i;var t={};if(i!=null)for(var e in i)Object.hasOwnProperty.call(i,e)&&(t[e]=i[e]);return t.default=i,t};Object.defineProperty(yC,"__esModule",{value:!0});const qS=Pj(we.exports),Aj=CC,Dj=_f;class Mj extends qS.Component{constructor(){super(...arguments),this.setGridApi=(t,e)=>{this.api=t,this.columnApi=e}}render(){const t=this.props.suppressReactUi?Aj.AgGridReactLegacy:Dj.AgGridReactUi;return qS.default.createElement(t,Object.assign({},this.props,{setGridApi:this.setGridApi}))}}yC.AgGridReact=Mj;var Fj=yC.AgGridReact,Ij=rp.AgGridColumn,Lj=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Nj=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(Lj(arguments[t]));return i},Gj=function(){function i(t,e,r,n,o,a,s,l){this.nextId=0,this.allNodesMap={},this.rootNode=t,this.gridOptionsWrapper=e,this.eventService=r,this.columnModel=n,this.gridApi=o,this.columnApi=a,this.beans=l,this.selectionService=s,this.rootNode.group=!0,this.rootNode.level=-1,this.rootNode.id=i.ROOT_NODE_ID,this.rootNode.allLeafChildren=[],this.rootNode.childrenAfterGroup=[],this.rootNode.childrenAfterSort=[],this.rootNode.childrenAfterAggFilter=[],this.rootNode.childrenAfterFilter=[],this.postConstruct()}return i.prototype.postConstruct=function(){this.suppressParentsInRowNodes=this.gridOptionsWrapper.isSuppressParentsInRowNodes(),this.isRowMasterFunc=this.gridOptionsWrapper.getIsRowMasterFunc(),this.doingTreeData=this.gridOptionsWrapper.isTreeData(),this.doingMasterDetail=this.gridOptionsWrapper.isMasterDetail()},i.prototype.getCopyOfNodesMap=function(){return _.cloneObject(this.allNodesMap)},i.prototype.getRowNode=function(t){return this.allNodesMap[t]},i.prototype.setRowData=function(t){var e=this;if(typeof t=="string"){console.warn("AG Grid: rowData must be an array, however you passed in a string. If you are loading JSON, make sure you convert the JSON string to JavaScript objects first");return}var r=this.rootNode,n=this.rootNode.sibling;r.childrenAfterFilter=null,r.childrenAfterGroup=null,r.childrenAfterAggFilter=null,r.childrenAfterSort=null,r.childrenMapped=null,r.updateHasChildren(),this.nextId=0,this.allNodesMap={},t?r.allLeafChildren=t.map(function(o){return e.createNode(o,e.rootNode,i.TOP_LEVEL)}):(r.allLeafChildren=[],r.childrenAfterGroup=[]),n&&(n.childrenAfterFilter=r.childrenAfterFilter,n.childrenAfterGroup=r.childrenAfterGroup,n.childrenAfterAggFilter=r.childrenAfterAggFilter,n.childrenAfterSort=r.childrenAfterSort,n.childrenMapped=r.childrenMapped,n.allLeafChildren=r.allLeafChildren)},i.prototype.updateRowData=function(t,e){var r={remove:[],update:[],add:[]},n=[];return this.executeRemove(t,r,n),this.executeUpdate(t,r,n),this.executeAdd(t,r),this.updateSelection(n),e&&_.sortRowNodesByOrder(this.rootNode.allLeafChildren,e),r},i.prototype.updateSelection=function(t){var e=t.length>0;if(e&&t.forEach(function(n){n.setSelected(!1,!1,!0)}),this.selectionService.updateGroupsFromChildrenSelections(),e){var r={type:x.EVENT_SELECTION_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(r)}},i.prototype.executeAdd=function(t,e){var r=this,n=t.add,o=t.addIndex;if(!_.missingOrEmpty(n)){var a=n.map(function(h){return r.createNode(h,r.rootNode,i.TOP_LEVEL)}),s=typeof o=="number"&&o>=0,l,u;if(s){var c=this.rootNode.allLeafChildren,p=o===0?0:c.reduce(function(h,d,f){var g,v=d.rowIndex,m=(g=c[h])===null||g===void 0?void 0:g.rowIndex,C=v!=null&&m!=null&&v<o&&v>m;return C?f:h},0)+1;l=c.slice(0,p),u=c.slice(p,c.length)}else l=this.rootNode.allLeafChildren,u=[];this.rootNode.allLeafChildren=Nj(l,a,u),this.rootNode.sibling&&(this.rootNode.sibling.allLeafChildren=this.rootNode.allLeafChildren),e.add=a}},i.prototype.executeRemove=function(t,e,r){var n=this,o=t.remove;if(!_.missingOrEmpty(o)){var a={};o.forEach(function(s){var l=n.lookupRowNode(s);!l||(l.isSelected()&&r.push(l),l.clearRowTopAndRowIndex(),a[l.id]=!0,delete n.allNodesMap[l.id],e.remove.push(l))}),this.rootNode.allLeafChildren=this.rootNode.allLeafChildren.filter(function(s){return!a[s.id]}),this.rootNode.sibling&&(this.rootNode.sibling.allLeafChildren=this.rootNode.allLeafChildren)}},i.prototype.executeUpdate=function(t,e,r){var n=this,o=t.update;_.missingOrEmpty(o)||o.forEach(function(a){var s=n.lookupRowNode(a);!s||(s.updateData(a),!s.selectable&&s.isSelected()&&r.push(s),n.setMasterForRow(s,a,i.TOP_LEVEL,!1),e.update.push(s))})},i.prototype.lookupRowNode=function(t){var e=this.gridOptionsWrapper.getRowIdFunc(),r;if(e){var n=e({data:t,level:0});if(r=this.allNodesMap[n],!r)return console.error("AG Grid: could not find row id="+n+", data item was not found for this id"),null}else if(r=this.rootNode.allLeafChildren.find(function(o){return o.data===t}),!r)return console.error("AG Grid: could not find data item as object was not found",t),console.error("Consider using getRowId to help the Grid find matching row data"),null;return r||null},i.prototype.createNode=function(t,e,r){var n=new Oe(this.beans);return n.group=!1,this.setMasterForRow(n,t,r,!0),e&&!this.suppressParentsInRowNodes&&(n.parent=e),n.level=r,n.setDataAndId(t,this.nextId.toString()),this.allNodesMap[n.id]&&console.warn("AG Grid: duplicate node id '"+n.id+"' detected from getRowId callback, this could cause issues in your grid."),this.allNodesMap[n.id]=n,this.nextId++,n},i.prototype.setMasterForRow=function(t,e,r,n){if(this.doingTreeData)t.setMaster(!1),n&&(t.expanded=!1);else if(this.doingMasterDetail?this.isRowMasterFunc?t.setMaster(this.isRowMasterFunc(e)):t.setMaster(!0):t.setMaster(!1),n){var o=this.columnModel.getRowGroupColumns(),a=o?o.length:0,s=r+a;t.expanded=t.master?this.isExpanded(s):!1}},i.prototype.isExpanded=function(t){var e=this.gridOptionsWrapper.getGroupDefaultExpanded();return e===-1?!0:t<e},i.TOP_LEVEL=0,i.ROOT_NODE_ID="ROOT_NODE_ID",i}(),kj=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Qt=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Ci;(function(i){i[i.Normal=0]="Normal",i[i.AfterFilter=1]="AfterFilter",i[i.AfterFilterAndSort=2]="AfterFilterAndSort",i[i.PivotNodes=3]="PivotNodes"})(Ci||(Ci={}));var Wj=function(i){kj(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.init=function(){var e=this.refreshModel.bind(this,{step:ze.EVERYTHING}),r=!this.gridOptionsWrapper.isSuppressAnimationFrame(),n=this.refreshModel.bind(this,{step:ze.EVERYTHING,afterColumnsChanged:!0,keepRenderedRows:!0,animate:r});this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,n),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,e),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VALUE_CHANGED,this.onValueChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_CHANGED,this.refreshModel.bind(this,{step:ze.PIVOT})),this.addManagedListener(this.eventService,x.EVENT_FILTER_CHANGED,this.onFilterChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_SORT_CHANGED,this.onSortChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,e);var o=this.refreshModel.bind(this,{step:ze.MAP,keepRenderedRows:!0,animate:r});this.addManagedListener(this.gridOptionsWrapper,ce.PROP_GROUP_REMOVE_SINGLE_CHILDREN,o),this.addManagedListener(this.gridOptionsWrapper,ce.PROP_GROUP_REMOVE_LOWEST_SINGLE_CHILDREN,o),this.rootNode=new Oe(this.beans),this.nodeManager=new Gj(this.rootNode,this.gridOptionsWrapper,this.eventService,this.columnModel,this.gridApi,this.columnApi,this.selectionService,this.beans)},t.prototype.start=function(){var e=this.gridOptionsWrapper.getRowData();e&&this.setRowData(e)},t.prototype.ensureRowHeightsValid=function(e,r,n,o){var a,s=!1;do{a=!1;for(var l=this.getRowIndexAtPixel(e),u=this.getRowIndexAtPixel(r),c=Math.max(l,n),p=Math.min(u,o),h=c;h<=p;h++){var d=this.getRow(h);if(d.rowHeightEstimated){var f=this.gridOptionsWrapper.getRowHeightForNode(d);d.setRowHeight(f.height),a=!0,s=!0}}a&&this.setRowTopAndRowIndex()}while(a);return s},t.prototype.setRowTopAndRowIndex=function(){for(var e=this.gridOptionsWrapper.getDefaultRowHeight(),r=0,n=new Set,o=this.gridOptionsWrapper.getDomLayout()===L.DOM_LAYOUT_NORMAL,a=0;a<this.rowsToDisplay.length;a++){var s=this.rowsToDisplay[a];if(s.id!=null&&n.add(s.id),s.rowHeight==null){var l=this.gridOptionsWrapper.getRowHeightForNode(s,o,e);s.setRowHeight(l.height,l.estimated)}s.setRowTop(r),s.setRowIndex(a),r+=s.rowHeight}return n},t.prototype.clearRowTopAndRowIndex=function(e,r){var n=e.isActive(),o=function(s){s&&s.id!=null&&!r.has(s.id)&&s.clearRowTopAndRowIndex()},a=function(s){if(o(s),o(s.detailNode),o(s.sibling),s.hasChildren()&&s.childrenAfterGroup){var l=s.level==-1,u=n&&!l&&!s.expanded;u||s.childrenAfterGroup.forEach(a)}};a(this.rootNode)},t.prototype.ensureRowsAtPixel=function(e,r,n){var o=this;n===void 0&&(n=0);var a=this.getRowIndexAtPixel(r),s=this.getRow(a),l=!this.gridOptionsWrapper.isSuppressAnimationFrame();return s===e[0]?!1:(e.forEach(function(u){_.removeFromArray(o.rootNode.allLeafChildren,u)}),e.forEach(function(u,c){_.insertIntoArray(o.rootNode.allLeafChildren,u,Math.max(a+n,0)+c)}),this.refreshModel({step:ze.EVERYTHING,keepRenderedRows:!0,keepEditingRows:!0,animate:l}),!0)},t.prototype.highlightRowAtPixel=function(e,r){var n=r!=null?this.getRowIndexAtPixel(r):null,o=n!=null?this.getRow(n):null;if(!o||!e||o===e||r==null){this.lastHighlightedRow&&(this.lastHighlightedRow.setHighlighted(null),this.lastHighlightedRow=null);return}var a=this.getHighlightPosition(r,o);this.lastHighlightedRow&&this.lastHighlightedRow!==o&&(this.lastHighlightedRow.setHighlighted(null),this.lastHighlightedRow=null),o.setHighlighted(a),this.lastHighlightedRow=o},t.prototype.getHighlightPosition=function(e,r){if(!r){var n=this.getRowIndexAtPixel(e);if(r=this.getRow(n||0),!r)return Di.Below}var o=r.rowTop,a=r.rowHeight;return e-o<a/2?Di.Above:Di.Below},t.prototype.getLastHighlightedRowNode=function(){return this.lastHighlightedRow},t.prototype.isLastRowIndexKnown=function(){return!0},t.prototype.getRowCount=function(){return this.rowsToDisplay?this.rowsToDisplay.length:0},t.prototype.getTopLevelRowCount=function(){var e=this.rowsToDisplay&&this.rowsToDisplay[0]===this.rootNode;if(e)return 1;var r=this.rootNode.childrenAfterAggFilter;return r?r.length:0},t.prototype.getTopLevelRowDisplayedIndex=function(e){var r=this.rowsToDisplay&&this.rowsToDisplay[0]===this.rootNode;if(r)return e;var n=this.rootNode.childrenAfterSort[e];if(this.gridOptionsWrapper.isGroupHideOpenParents())for(;n.expanded&&n.childrenAfterSort&&n.childrenAfterSort.length>0;)n=n.childrenAfterSort[0];return n.rowIndex},t.prototype.getRowBounds=function(e){if(_.missing(this.rowsToDisplay))return null;var r=this.rowsToDisplay[e];return r?{rowTop:r.rowTop,rowHeight:r.rowHeight}:null},t.prototype.onRowGroupOpened=function(){var e=this.gridOptionsWrapper.isAnimateRows();this.refreshModel({step:ze.MAP,keepRenderedRows:!0,animate:e})},t.prototype.onFilterChanged=function(e){if(!e.afterDataChange){var r=this.gridOptionsWrapper.isAnimateRows(),n=e.columns.length===0||e.columns.some(function(a){return a.isPrimary()}),o=n?ze.FILTER:ze.FILTER_AGGREGATES;this.refreshModel({step:o,keepRenderedRows:!0,animate:r})}},t.prototype.onSortChanged=function(){var e=this.gridOptionsWrapper.isAnimateRows();this.refreshModel({step:ze.SORT,keepRenderedRows:!0,animate:e,keepEditingRows:!0})},t.prototype.getType=function(){return L.ROW_MODEL_TYPE_CLIENT_SIDE},t.prototype.onValueChanged=function(){this.columnModel.isPivotActive()?this.refreshModel({step:ze.PIVOT}):this.refreshModel({step:ze.AGGREGATE})},t.prototype.createChangePath=function(e){var r=_.missingOrEmpty(e),n=new Jc(!1,this.rootNode);return(r||this.gridOptionsWrapper.isTreeData())&&n.setInactive(),n},t.prototype.isSuppressModelUpdateAfterUpdateTransaction=function(e){if(!this.gridOptionsWrapper.isSuppressModelUpdateAfterUpdateTransaction()||e.rowNodeTransactions==null)return!1;var r=e.rowNodeTransactions.filter(function(o){return o.add!=null&&o.add.length>0||o.remove!=null&&o.remove.length>0}),n=r==null||r.length==0;return n},t.prototype.refreshModel=function(e){if(!this.isSuppressModelUpdateAfterUpdateTransaction(e)){var r=this.createChangePath(e.rowNodeTransactions);switch(e.step){case ze.EVERYTHING:this.doRowGrouping(e.groupState,e.rowNodeTransactions,e.rowNodeOrder,r,!!e.afterColumnsChanged);case ze.FILTER:this.doFilter(r);case ze.PIVOT:this.doPivot(r);case ze.AGGREGATE:this.doAggregate(r);case ze.FILTER_AGGREGATES:this.doFilterAggregates(r);case ze.SORT:this.doSort(e.rowNodeTransactions,r);case ze.MAP:this.doRowsToDisplay()}var n=this.setRowTopAndRowIndex();this.clearRowTopAndRowIndex(r,n);var o={type:x.EVENT_MODEL_UPDATED,api:this.gridApi,columnApi:this.columnApi,animate:e.animate,keepRenderedRows:e.keepRenderedRows,newData:e.newData,newPage:!1};this.eventService.dispatchEvent(o)}},t.prototype.isEmpty=function(){var e=_.missing(this.rootNode.allLeafChildren)||this.rootNode.allLeafChildren.length===0;return _.missing(this.rootNode)||e||!this.columnModel.isReady()},t.prototype.isRowsToRender=function(){return _.exists(this.rowsToDisplay)&&this.rowsToDisplay.length>0},t.prototype.getNodesInRangeForSelection=function(e,r){var n=!r,o=!1,a,s=[],l=this.gridOptionsWrapper.isGroupSelectsChildren();return this.forEachNodeAfterFilterAndSort(function(u){var c=n&&!o;n||(u===r||u===e)&&(n=!0);var p=u.group&&l;if(!p){var h=n&&!o,d=u.isParentOfNode(a);(h||d)&&s.push(u)}c&&(u===r||u===e)&&(o=!0,u===r?a=r:a=e)}),s},t.prototype.setDatasource=function(e){console.error("AG Grid: should never call setDatasource on clientSideRowController")},t.prototype.getTopLevelNodes=function(){return this.rootNode?this.rootNode.childrenAfterGroup:null},t.prototype.getRootNode=function(){return this.rootNode},t.prototype.getRow=function(e){return this.rowsToDisplay[e]},t.prototype.isRowPresent=function(e){return this.rowsToDisplay.indexOf(e)>=0},t.prototype.getRowIndexAtPixel=function(e){if(this.isEmpty())return-1;var r=0,n=this.rowsToDisplay.length-1;if(e<=0)return 0;var o=_.last(this.rowsToDisplay);if(o.rowTop<=e)return this.rowsToDisplay.length-1;for(;;){var a=Math.floor((r+n)/2),s=this.rowsToDisplay[a];if(this.isRowInPixel(s,e))return a;s.rowTop<e?r=a+1:s.rowTop>e&&(n=a-1)}},t.prototype.isRowInPixel=function(e,r){var n=e.rowTop,o=e.rowTop+e.rowHeight,a=n<=r&&o>r;return a},t.prototype.forEachLeafNode=function(e){this.rootNode.allLeafChildren&&this.rootNode.allLeafChildren.forEach(function(r,n){return e(r,n)})},t.prototype.forEachNode=function(e){this.recursivelyWalkNodesAndCallback(this.rootNode.childrenAfterGroup,e,Ci.Normal,0)},t.prototype.forEachNodeAfterFilter=function(e){this.recursivelyWalkNodesAndCallback(this.rootNode.childrenAfterAggFilter,e,Ci.AfterFilter,0)},t.prototype.forEachNodeAfterFilterAndSort=function(e){this.recursivelyWalkNodesAndCallback(this.rootNode.childrenAfterSort,e,Ci.AfterFilterAndSort,0)},t.prototype.forEachPivotNode=function(e){this.recursivelyWalkNodesAndCallback([this.rootNode],e,Ci.PivotNodes,0)},t.prototype.recursivelyWalkNodesAndCallback=function(e,r,n,o){if(!e)return o;for(var a=0;a<e.length;a++){var s=e[a];if(r(s,o++),s.hasChildren()){var l=null;switch(n){case Ci.Normal:l=s.childrenAfterGroup;break;case Ci.AfterFilter:l=s.childrenAfterAggFilter;break;case Ci.AfterFilterAndSort:l=s.childrenAfterSort;break;case Ci.PivotNodes:l=s.leafGroup?null:s.childrenAfterSort;break}l&&(o=this.recursivelyWalkNodesAndCallback(l,r,n,o))}}return o},t.prototype.doAggregate=function(e){this.aggregationStage&&this.aggregationStage.execute({rowNode:this.rootNode,changedPath:e})},t.prototype.doFilterAggregates=function(e){this.filterAggregatesStage?this.filterAggregatesStage.execute({rowNode:this.rootNode,changedPath:e}):this.rootNode.childrenAfterAggFilter=this.rootNode.childrenAfterFilter},t.prototype.expandOrCollapseAll=function(e){var r=this.gridOptionsWrapper.isTreeData(),n=this.columnModel.isPivotActive(),o=function(l){!l||l.forEach(function(u){var c=function(){u.expanded=e,o(u.childrenAfterGroup)};if(r){var p=_.exists(u.childrenAfterGroup);p&&c();return}if(n){var h=!u.leafGroup;h&&c();return}var d=u.group;d&&c()})};this.rootNode&&o(this.rootNode.childrenAfterGroup),this.refreshModel({step:ze.MAP});var a=e?"expandAll":"collapseAll",s={api:this.gridApi,columnApi:this.columnApi,type:x.EVENT_EXPAND_COLLAPSE_ALL,source:a};this.eventService.dispatchEvent(s)},t.prototype.doSort=function(e,r){this.sortStage.execute({rowNode:this.rootNode,rowNodeTransactions:e,changedPath:r})},t.prototype.doRowGrouping=function(e,r,n,o,a){this.groupStage?(r?this.groupStage.execute({rowNode:this.rootNode,rowNodeTransactions:r,rowNodeOrder:n,changedPath:o}):(this.groupStage.execute({rowNode:this.rootNode,changedPath:o,afterColumnsChanged:a}),this.restoreGroupState(e)),this.gridOptionsWrapper.isGroupSelectsChildren()&&this.selectionService.updateGroupsFromChildrenSelections(o)):(this.rootNode.childrenAfterGroup=this.rootNode.allLeafChildren,this.rootNode.sibling&&(this.rootNode.sibling.childrenAfterGroup=this.rootNode.childrenAfterGroup),this.rootNode.updateHasChildren())},t.prototype.restoreGroupState=function(e){!e||_.traverseNodesWithKey(this.rootNode.childrenAfterGroup,function(r,n){typeof e[n]=="boolean"&&(r.expanded=e[n])})},t.prototype.doFilter=function(e){this.filterStage.execute({rowNode:this.rootNode,changedPath:e})},t.prototype.doPivot=function(e){this.pivotStage&&this.pivotStage.execute({rowNode:this.rootNode,changedPath:e})},t.prototype.getGroupState=function(){if(!this.rootNode.childrenAfterGroup||!this.gridOptionsWrapper.isRememberGroupStateWhenNewData())return null;var e={};return _.traverseNodesWithKey(this.rootNode.childrenAfterGroup,function(r,n){return e[n]=r.expanded}),e},t.prototype.getCopyOfNodesMap=function(){return this.nodeManager.getCopyOfNodesMap()},t.prototype.getRowNode=function(e){var r=typeof e=="string"&&e.indexOf(Oe.ID_PREFIX_ROW_GROUP)==0;if(r){var n=void 0;return this.forEachNode(function(o){o.id===e&&(n=o)}),n}return this.nodeManager.getRowNode(e)},t.prototype.setRowData=function(e){var r=this.getGroupState();this.nodeManager.setRowData(e),this.selectionService.reset(),this.filterManager.onNewRowsLoaded("rowDataUpdated");var n={type:x.EVENT_ROW_DATA_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(n),this.refreshModel({step:ze.EVERYTHING,groupState:r,newData:!0})},t.prototype.batchUpdateRowData=function(e,r){var n=this;if(this.applyAsyncTransactionsTimeout==null){this.rowDataTransactionBatch=[];var o=this.gridOptionsWrapper.getAsyncTransactionWaitMillis();this.applyAsyncTransactionsTimeout=window.setTimeout(function(){n.executeBatchUpdateRowData()},o)}this.rowDataTransactionBatch.push({rowDataTransaction:e,callback:r})},t.prototype.flushAsyncTransactions=function(){this.applyAsyncTransactionsTimeout!=null&&(clearTimeout(this.applyAsyncTransactionsTimeout),this.executeBatchUpdateRowData())},t.prototype.executeBatchUpdateRowData=function(){var e=this;this.valueCache.onDataChanged();var r=[],n=[],o=!1;if(this.rowDataTransactionBatch&&this.rowDataTransactionBatch.forEach(function(s){var l=e.nodeManager.updateRowData(s.rowDataTransaction,void 0);n.push(l),s.callback&&r.push(s.callback.bind(null,l)),typeof s.rowDataTransaction.addIndex=="number"&&(o=!0)}),this.commonUpdateRowData(n,void 0,o),r.length>0&&window.setTimeout(function(){r.forEach(function(s){return s()})},0),n.length>0){var a={api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),type:x.EVENT_ASYNC_TRANSACTIONS_FLUSHED,results:n};this.eventService.dispatchEvent(a)}this.rowDataTransactionBatch=null,this.applyAsyncTransactionsTimeout=void 0},t.prototype.updateRowData=function(e,r){this.valueCache.onDataChanged();var n=this.nodeManager.updateRowData(e,r),o=typeof e.addIndex=="number";return this.commonUpdateRowData([n],r,o),n},t.prototype.createRowNodeOrder=function(){var e=this.gridOptionsWrapper.isSuppressMaintainUnsortedOrder();if(!e){var r={};if(this.rootNode&&this.rootNode.allLeafChildren)for(var n=0;n<this.rootNode.allLeafChildren.length;n++){var o=this.rootNode.allLeafChildren[n];r[o.id]=n}return r}},t.prototype.commonUpdateRowData=function(e,r,n){var o=!this.gridOptionsWrapper.isSuppressAnimationFrame();n&&(r=this.createRowNodeOrder()),this.refreshModel({step:ze.EVERYTHING,rowNodeTransactions:e,rowNodeOrder:r,keepRenderedRows:!0,keepEditingRows:!0,animate:o});var a={type:x.EVENT_ROW_DATA_UPDATED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(a)},t.prototype.doRowsToDisplay=function(){this.rowsToDisplay=this.flattenStage.execute({rowNode:this.rootNode})},t.prototype.onRowHeightChanged=function(){this.refreshModel({step:ze.MAP,keepRenderedRows:!0,keepEditingRows:!0})},t.prototype.resetRowHeights=function(){var e=!1;this.forEachNode(function(r){r.setRowHeight(r.rowHeight,!0);var n=r.detailNode;n&&n.setRowHeight(n.rowHeight,!0),e=!0}),e&&this.onRowHeightChanged()},Qt([y("columnModel")],t.prototype,"columnModel",void 0),Qt([y("selectionService")],t.prototype,"selectionService",void 0),Qt([y("filterManager")],t.prototype,"filterManager",void 0),Qt([y("valueCache")],t.prototype,"valueCache",void 0),Qt([y("columnApi")],t.prototype,"columnApi",void 0),Qt([y("gridApi")],t.prototype,"gridApi",void 0),Qt([y("animationFrameService")],t.prototype,"animationFrameService",void 0),Qt([y("beans")],t.prototype,"beans",void 0),Qt([y("filterStage")],t.prototype,"filterStage",void 0),Qt([y("sortStage")],t.prototype,"sortStage",void 0),Qt([y("flattenStage")],t.prototype,"flattenStage",void 0),Qt([De("groupStage")],t.prototype,"groupStage",void 0),Qt([De("aggregationStage")],t.prototype,"aggregationStage",void 0),Qt([De("pivotStage")],t.prototype,"pivotStage",void 0),Qt([De("filterAggregatesStage")],t.prototype,"filterAggregatesStage",void 0),Qt([B],t.prototype,"init",null),t=Qt([Q("rowModel")],t),t}(H),Vj=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ZS=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Bj=function(i){Vj(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.execute=function(e){var r=e.changedPath;this.filterService.filter(r)},ZS([y("filterService")],t.prototype,"filterService",void 0),t=ZS([Q("filterStage")],t),t}(H),Hj=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),jp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},jj=function(i){Hj(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.execute=function(e){var r=this.sortController.getSortOptions(),n=_.exists(r)&&r.length>0,o=n&&_.exists(e.rowNodeTransactions)&&this.gridOptionsWrapper.isDeltaSort(),a=r.some(function(s){return!!s.column.getColDef().showRowGroup});this.sortService.sort(r,n,o,e.rowNodeTransactions,e.changedPath,a)},jp([y("sortService")],t.prototype,"sortService",void 0),jp([y("sortController")],t.prototype,"sortController",void 0),jp([y("columnModel")],t.prototype,"columnModel",void 0),t=jp([Q("sortStage")],t),t}(H),Uj=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),pv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},zj=function(i){Uj(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.execute=function(e){var r=e.rowNode,n=[],o={value:0},a=this.columnModel.isPivotMode(),s=a&&r.leafGroup,l=s?[r]:r.childrenAfterSort;this.recursivelyAddToRowsToDisplay(l,n,o,a,0);var u=n.length>0,c=!s&&u&&this.gridOptionsWrapper.isGroupIncludeTotalFooter();return c&&(this.ensureFooterNodeExists(r),this.addRowNodeToRowsToDisplay(r.sibling,n,o,0)),n},t.prototype.recursivelyAddToRowsToDisplay=function(e,r,n,o,a){if(!_.missingOrEmpty(e))for(var s=this.gridOptionsWrapper.isGroupHideOpenParents(),l=this.gridOptionsWrapper.isGroupRemoveSingleChildren(),u=!l&&this.gridOptionsWrapper.isGroupRemoveLowestSingleChildren(),c=0;c<e.length;c++){var p=e[c],h=p.hasChildren(),d=o&&!h,f=l&&h&&p.childrenAfterGroup.length===1,g=u&&h&&p.leafGroup&&p.childrenAfterGroup.length===1,v=o&&p.leafGroup,m=s&&p.expanded&&!p.master&&!v,C=!d&&!m&&!f&&!g;if(C&&this.addRowNodeToRowsToDisplay(p,r,n,a),!(o&&p.leafGroup)){if(h){var w=f||g;if(p.expanded||w){var b=w?a:a+1;this.recursivelyAddToRowsToDisplay(p.childrenAfterSort,r,n,o,b),this.gridOptionsWrapper.isGroupIncludeFooter()&&(this.ensureFooterNodeExists(p),this.addRowNodeToRowsToDisplay(p.sibling,r,n,a))}}else if(p.master&&p.expanded){var S=this.createDetailNode(p);this.addRowNodeToRowsToDisplay(S,r,n,a)}}}},t.prototype.addRowNodeToRowsToDisplay=function(e,r,n,o){var a=this.gridOptionsWrapper.isGroupMultiAutoColumn();r.push(e),e.setUiLevel(a?0:o)},t.prototype.ensureFooterNodeExists=function(e){if(!_.exists(e.sibling)){var r=new Oe(this.beans);Object.keys(e).forEach(function(n){r[n]=e[n]}),r.footer=!0,r.setRowTop(null),r.setRowIndex(null),r.oldRowTop=null,_.exists(r.id)&&(r.id="rowGroupFooter_"+r.id),r.sibling=e,e.sibling=r}},t.prototype.createDetailNode=function(e){if(_.exists(e.detailNode))return e.detailNode;var r=new Oe(this.beans);return r.detail=!0,r.selectable=!1,r.parent=e,_.exists(e.id)&&(r.id="detail_"+e.id),r.data=e.data,r.level=e.level+1,e.detailNode=r,r},pv([y("columnModel")],t.prototype,"columnModel",void 0),pv([y("beans")],t.prototype,"beans",void 0),t=pv([Q("flattenStage")],t),t}(H),$j=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Up=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Kj=function(i){$j(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.init=function(){this.postSortFunc=this.gridOptionsWrapper.getPostSortFunc()},t.prototype.sort=function(e,r,n,o,a,s){var l=this,u=this.gridOptionsWrapper.isGroupMaintainOrder(),c=this.columnModel.getAllGridColumns().some(function(f){return f.isRowGroupActive()}),p={};n&&o&&(p=this.calculateDirtyNodes(o));var h=this.columnModel.isPivotMode(),d=function(f){l.pullDownGroupDataForHideOpenParents(f.childrenAfterAggFilter,!0);var g=h&&f.leafGroup,v=u&&c&&!f.leafGroup&&!s;if(!r||v||g){var m=f.childrenAfterAggFilter.slice(0);if(u&&f.childrenAfterSort){var C=f.childrenAfterSort.reduce(function(b,S,T){return b[S.id]=T,b},{});m.sort(function(b,S){return(C[b.id]||0)-(C[S.id]||0)})}f.childrenAfterSort=m}else n?f.childrenAfterSort=l.doDeltaSort(f,p,a,e):f.childrenAfterSort=l.rowNodeSorter.doFullSort(f.childrenAfterAggFilter,e);if(f.sibling&&(f.sibling.childrenAfterSort=f.childrenAfterSort),l.updateChildIndexes(f),l.postSortFunc){var w={nodes:f.childrenAfterSort};l.postSortFunc(w)}};a&&a.forEachChangedNodeDepthFirst(d),this.updateGroupDataForHideOpenParents(a)},t.prototype.calculateDirtyNodes=function(e){var r={},n=function(o){o&&o.forEach(function(a){return r[a.id]=!0})};return e&&e.forEach(function(o){n(o.add),n(o.update),n(o.remove)}),r},t.prototype.doDeltaSort=function(e,r,n,o){var a=this,s=e.childrenAfterAggFilter,l=e.childrenAfterSort;if(!l)return this.rowNodeSorter.doFullSort(s,o);var u={},c=[];s.forEach(function(f){r[f.id]||!n.canSkip(f)?c.push(f):u[f.id]=!0});var p=l.filter(function(f){return u[f.id]}),h=function(f,g){return{currentPos:g,rowNode:f}},d=c.map(h).sort(function(f,g){return a.rowNodeSorter.compareRowNodes(o,f,g)});return this.mergeSortedArrays(o,d,p.map(h)).map(function(f){var g=f.rowNode;return g})},t.prototype.mergeSortedArrays=function(e,r,n){for(var o=[],a=0,s=0;a<r.length&&s<n.length;){var l=this.rowNodeSorter.compareRowNodes(e,r[a],n[s]);l<0?o.push(r[a++]):o.push(n[s++])}for(;a<r.length;)o.push(r[a++]);for(;s<n.length;)o.push(n[s++]);return o},t.prototype.updateChildIndexes=function(e){if(!_.missing(e.childrenAfterSort))for(var r=e.childrenAfterSort,n=0;n<r.length;n++){var o=r[n],a=n===0,s=n===e.childrenAfterSort.length-1;o.setFirstChild(a),o.setLastChild(s),o.setChildIndex(n)}},t.prototype.updateGroupDataForHideOpenParents=function(e){var r=this;if(!!this.gridOptionsWrapper.isGroupHideOpenParents()){if(this.gridOptionsWrapper.isTreeData()){var n="AG Grid: The property hideOpenParents dose not work with Tree Data. This is because Tree Data has values at the group level, it doesn't make sense to hide them (as opposed to Row Grouping, which only has Aggregated Values at the group level).";return _.doOnce(function(){return console.warn(n)},"sortService.hideOpenParentsWithTreeData"),!1}var o=function(a){r.pullDownGroupDataForHideOpenParents(a.childrenAfterSort,!1),a.childrenAfterSort.forEach(function(s){s.hasChildren()&&o(s)})};e&&e.executeFromRootNode(function(a){return o(a)})}},t.prototype.pullDownGroupDataForHideOpenParents=function(e,r){var n=this;!this.gridOptionsWrapper.isGroupHideOpenParents()||_.missing(e)||e.forEach(function(o){var a=n.columnModel.getGroupDisplayColumns();a.forEach(function(s){var l=s.getColDef().showRowGroup;if(typeof l!="string"){console.error("AG Grid: groupHideOpenParents only works when specifying specific columns for colDef.showRowGroup");return}var u=l,c=n.columnModel.getPrimaryColumn(u),p=c===o.rowGroupColumn;if(!p)if(r)o.setGroupValue(s.getId(),void 0);else{var h=o.getFirstChildOfFirstChild(c);h&&o.setGroupValue(s.getId(),h.key)}})})},Up([y("columnModel")],t.prototype,"columnModel",void 0),Up([y("rowNodeSorter")],t.prototype,"rowNodeSorter",void 0),Up([B],t.prototype,"init",null),t=Up([Q("sortService")],t),t}(H),Yj=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),QS=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Xj=function(i){Yj(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.filter=function(e){var r=this.filterManager.isColumnFilterPresent()||this.filterManager.isQuickFilterPresent()||this.gridOptionsWrapper.isExternalFilterPresent();this.filterNodes(r,e)},t.prototype.filterNodes=function(e,r){var n=this,o=function(u,c){u.hasChildren()&&e&&!c?u.childrenAfterFilter=u.childrenAfterGroup.filter(function(p){var h=p.childrenAfterFilter&&p.childrenAfterFilter.length>0,d=p.data&&n.filterManager.doesRowPassFilter({rowNode:p});return h||d}):u.childrenAfterFilter=u.childrenAfterGroup,u.sibling&&(u.sibling.childrenAfterFilter=u.childrenAfterFilter)};if(this.doingTreeDataFiltering()){var a=function(u,c){if(u.childrenAfterGroup)for(var p=0;p<u.childrenAfterGroup.length;p++){var h=u.childrenAfterGroup[p],d=c||n.filterManager.doesRowPassFilter({rowNode:h});h.childrenAfterGroup?a(u.childrenAfterGroup[p],d):o(h,d)}o(u,c)},s=function(u){return a(u,!1)};r.executeFromRootNode(s)}else{var l=function(u){return o(u,!1)};r.forEachChangedNodeDepthFirst(l,!0)}},t.prototype.doingTreeDataFiltering=function(){return this.gridOptionsWrapper.isTreeData()&&!this.gridOptionsWrapper.isExcludeChildrenWhenTreeDataFiltering()},QS([y("filterManager")],t.prototype,"filterManager",void 0),t=QS([Q("filterService")],t),t}(H),qj=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Vs=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Zj=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Qj=function(i){qj(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){this.rowModel.getType()===L.ROW_MODEL_TYPE_CLIENT_SIDE&&(this.clientSideRowModel=this.rowModel)},t.prototype.isActive=function(){return this.gridOptionsWrapper.isImmutableData()},t.prototype.setRowData=function(e){var r=this.createTransactionForRowData(e);if(!!r){var n=Zj(r,2),o=n[0],a=n[1],s=this.clientSideRowModel.updateRowData(o,a);s&&this.rowRenderer.refreshFullWidthRows(s.update);var l={type:x.EVENT_ROW_DATA_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(l)}},t.prototype.createTransactionForRowData=function(e){if(_.missing(this.clientSideRowModel)){console.error("AG Grid: ImmutableService only works with ClientSideRowModel");return}var r=this.gridOptionsWrapper.getRowIdFunc();if(r==null){console.error("AG Grid: ImmutableService requires getRowId() callback to be implemented, your row data needs IDs!");return}var n={remove:[],update:[],add:[]},o=this.clientSideRowModel.getCopyOfNodesMap(),a=this.gridOptionsWrapper.isSuppressMaintainUnsortedOrder(),s=a?void 0:{};return _.exists(e)&&e.forEach(function(l,u){var c=r({data:l,level:0}),p=o[c];if(s&&(s[c]=u),p){var h=p.data!==l;h&&n.update.push(l),o[c]=void 0}else n.add.push(l)}),_.iterateObject(o,function(l,u){u&&n.remove.push(u.data)}),[n,s]},Vs([y("rowModel")],t.prototype,"rowModel",void 0),Vs([y("rowRenderer")],t.prototype,"rowRenderer",void 0),Vs([y("columnApi")],t.prototype,"columnApi",void 0),Vs([y("gridApi")],t.prototype,"gridApi",void 0),Vs([B],t.prototype,"postConstruct",null),t=Vs([Q("immutableService")],t),t}(H),Jj={moduleName:oe.ClientSideRowModelModule,beans:[Bj,jj,zj,Kj,Xj,Qj],rowModels:{clientSide:Wj}},e4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),hv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},JS=function(i){e4(t,i);function t(e,r,n){var o=i.call(this,e)||this;return o.parentCache=r,o.params=n,o.startRow=e*n.blockSize,o.endRow=o.startRow+n.blockSize,o}return t.prototype.postConstruct=function(){this.createRowNodes()},t.prototype.getBlockStateJson=function(){return{id:""+this.getId(),state:{blockNumber:this.getId(),startRow:this.getStartRow(),endRow:this.getEndRow(),pageStatus:this.getState()}}},t.prototype.setDataAndId=function(e,r,n){!e.id&&e.alreadyRendered&&(e.alreadyRendered=!1),_.exists(r)?e.setDataAndId(r,n.toString()):e.setDataAndId(void 0,void 0)},t.prototype.loadFromDatasource=function(){var e=this,r=this.createLoadParams();if(_.missing(this.params.datasource.getRows)){console.warn("AG Grid: datasource is missing getRows method");return}window.setTimeout(function(){e.params.datasource.getRows(r)},0)},t.prototype.processServerFail=function(){},t.prototype.createLoadParams=function(){var e={startRow:this.getStartRow(),endRow:this.getEndRow(),successCallback:this.pageLoaded.bind(this,this.getVersion()),failCallback:this.pageLoadFailed.bind(this,this.getVersion()),sortModel:this.params.sortModel,filterModel:this.params.filterModel,context:this.gridOptionsWrapper.getContext()};return e},t.prototype.forEachNode=function(e,r,n){var o=this;this.rowNodes.forEach(function(a,s){var l=o.startRow+s;l<n&&e(a,r.next())})},t.prototype.getLastAccessed=function(){return this.lastAccessed},t.prototype.getRow=function(e,r){r===void 0&&(r=!1),r||(this.lastAccessed=this.params.lastAccessedSequence.next());var n=e-this.startRow;return this.rowNodes[n]},t.prototype.getStartRow=function(){return this.startRow},t.prototype.getEndRow=function(){return this.endRow},t.prototype.createRowNodes=function(){this.rowNodes=[];for(var e=0;e<this.params.blockSize;e++){var r=this.startRow+e,n=new Oe(this.beans);n.setRowHeight(this.params.rowHeight),n.uiLevel=0,n.setRowIndex(r),n.setRowTop(this.params.rowHeight*r),this.rowNodes.push(n)}},t.prototype.processServerResult=function(e){var r=this;this.rowNodes.forEach(function(o,a){var s=e.rowData?e.rowData[a]:void 0;r.setDataAndId(o,s,r.startRow+a)});var n=e.rowCount!=null&&e.rowCount>=0?e.rowCount:void 0;this.parentCache.pageLoaded(this,n)},t.prototype.destroyRowNodes=function(){this.rowNodes.forEach(function(e){e.clearRowTopAndRowIndex()})},hv([y("beans")],t.prototype,"beans",void 0),hv([B],t.prototype,"postConstruct",null),hv([nt],t.prototype,"destroyRowNodes",null),t}(jr),t4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),zp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},r4=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},n4=function(i){t4(t,i);function t(e){var r=i.call(this)||this;return r.lastRowIndexKnown=!1,r.blocks={},r.blockCount=0,r.rowCount=e.initialRowCount,r.params=e,r}return t.prototype.setBeans=function(e){this.logger=e.create("InfiniteCache")},t.prototype.getRow=function(e,r){r===void 0&&(r=!1);var n=Math.floor(e/this.params.blockSize),o=this.blocks[n];if(!o){if(r)return;o=this.createBlock(n)}return o.getRow(e)},t.prototype.createBlock=function(e){var r=this.createBean(new JS(e,this,this.params));return this.blocks[r.getId()]=r,this.blockCount++,this.purgeBlocksIfNeeded(r),this.params.rowNodeBlockLoader.addBlock(r),r},t.prototype.refreshCache=function(){var e=this.blockCount==0;if(e){this.purgeCache();return}this.getBlocksInOrder().forEach(function(r){return r.setStateWaitingToLoad()}),this.params.rowNodeBlockLoader.checkBlockToLoad()},t.prototype.destroyAllBlocks=function(){var e=this;this.getBlocksInOrder().forEach(function(r){return e.destroyBlock(r)})},t.prototype.getRowCount=function(){return this.rowCount},t.prototype.isLastRowIndexKnown=function(){return this.lastRowIndexKnown},t.prototype.pageLoaded=function(e,r){!this.isAlive()||(this.logger.log("onPageLoaded: page = "+e.getId()+", lastRow = "+r),this.checkRowCount(e,r),this.onCacheUpdated())},t.prototype.purgeBlocksIfNeeded=function(e){var r=this,n=this.getBlocksInOrder().filter(function(u){return u!=e}),o=function(u,c){return c.getLastAccessed()-u.getLastAccessed()};n.sort(o);var a=this.params.maxBlocksInCache>0,s=a?this.params.maxBlocksInCache-1:null,l=t.MAX_EMPTY_BLOCKS_TO_KEEP-1;n.forEach(function(u,c){var p=u.getState()===JS.STATE_WAITING_TO_LOAD&&c>=l,h=a?c>=s:!1;if(p||h){if(r.isBlockCurrentlyDisplayed(u)||r.isBlockFocused(u))return;r.removeBlockFromCache(u)}})},t.prototype.isBlockFocused=function(e){var r=this.focusService.getFocusCellToUseAfterRefresh();if(!r||r.rowPinned!=null)return!1;var n=e.getStartRow(),o=e.getEndRow(),a=r.rowIndex>=n&&r.rowIndex<o;return a},t.prototype.isBlockCurrentlyDisplayed=function(e){var r=e.getStartRow(),n=e.getEndRow()-1;return this.rowRenderer.isRangeInRenderedViewport(r,n)},t.prototype.removeBlockFromCache=function(e){!e||this.destroyBlock(e)},t.prototype.checkRowCount=function(e,r){if(typeof r=="number"&&r>=0)this.rowCount=r,this.lastRowIndexKnown=!0;else if(!this.lastRowIndexKnown){var n=(e.getId()+1)*this.params.blockSize,o=n+this.params.overflowSize;this.rowCount<o&&(this.rowCount=o)}},t.prototype.setRowCount=function(e,r){this.rowCount=e,_.exists(r)&&(this.lastRowIndexKnown=r),this.lastRowIndexKnown||this.rowCount%this.params.blockSize===0&&this.rowCount++,this.onCacheUpdated()},t.prototype.forEachNodeDeep=function(e){var r=this,n=new Bt;this.getBlocksInOrder().forEach(function(o){return o.forEachNode(e,n,r.rowCount)})},t.prototype.getBlocksInOrder=function(){var e=function(n,o){return n.getId()-o.getId()},r=_.getAllValuesInObject(this.blocks).sort(e);return r},t.prototype.destroyBlock=function(e){delete this.blocks[e.getId()],this.destroyBean(e),this.blockCount--,this.params.rowNodeBlockLoader.removeBlock(e)},t.prototype.onCacheUpdated=function(){if(this.isAlive()){this.destroyAllBlocksPastVirtualRowCount();var e={type:x.EVENT_STORE_UPDATED};this.eventService.dispatchEvent(e)}},t.prototype.destroyAllBlocksPastVirtualRowCount=function(){var e=this,r=[];this.getBlocksInOrder().forEach(function(n){var o=n.getId()*e.params.blockSize;o>=e.rowCount&&r.push(n)}),r.length>0&&r.forEach(function(n){return e.destroyBlock(n)})},t.prototype.purgeCache=function(){var e=this;this.getBlocksInOrder().forEach(function(r){return e.removeBlockFromCache(r)}),this.lastRowIndexKnown=!1,this.rowCount===0&&(this.rowCount=this.params.initialRowCount),this.onCacheUpdated()},t.prototype.getRowNodesInRange=function(e,r){var n=this,o=[],a=-1,s=!1,l=new Bt;_.missing(e)&&(s=!0);var u=!1;this.getBlocksInOrder().forEach(function(p){if(!u){if(s&&a+1!==p.getId()){u=!0;return}a=p.getId(),p.forEachNode(function(h){var d=h===e||h===r;(s||d)&&o.push(h),d&&(s=!s)},l,n.rowCount)}});var c=u||s;return c?[]:o},t.MAX_EMPTY_BLOCKS_TO_KEEP=2,zp([y("rowRenderer")],t.prototype,"rowRenderer",void 0),zp([y("focusService")],t.prototype,"focusService",void 0),zp([r4(0,Yt("loggerFactory"))],t.prototype,"setBeans",null),zp([nt],t.prototype,"destroyAllBlocks",null),t}(H),i4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),hi=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},o4=function(i){i4(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getRowBounds=function(e){return{rowHeight:this.rowHeight,rowTop:this.rowHeight*e}},t.prototype.ensureRowHeightsValid=function(e,r,n,o){return!1},t.prototype.init=function(){var e=this;!this.gridOptionsWrapper.isRowModelInfinite()||(this.rowHeight=this.gridOptionsWrapper.getRowHeightAsNumber(),this.addEventListeners(),this.addDestroyFunc(function(){return e.destroyCache()}),this.verifyProps())},t.prototype.verifyProps=function(){if(this.gridOptionsWrapper.getInitialGroupOrderComparator()!=null){var e="AG Grid: initialGroupOrderComparator cannot be used with Infinite Row Model. If using Infinite Row Model, then sorting is done on the server side, nothing to do with the client.";_.doOnce(function(){return console.warn(e)},"IRM.InitialGroupOrderComparator")}},t.prototype.start=function(){this.setDatasource(this.gridOptionsWrapper.getDatasource())},t.prototype.destroyDatasource=function(){this.datasource&&(this.getContext().destroyBean(this.datasource),this.rowRenderer.datasourceChanged(),this.datasource=null)},t.prototype.addEventListeners=function(){this.addManagedListener(this.eventService,x.EVENT_FILTER_CHANGED,this.onFilterChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_SORT_CHANGED,this.onSortChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.onColumnEverything.bind(this)),this.addManagedListener(this.eventService,x.EVENT_STORE_UPDATED,this.onCacheUpdated.bind(this))},t.prototype.onFilterChanged=function(){this.reset()},t.prototype.onSortChanged=function(){this.reset()},t.prototype.onColumnEverything=function(){var e;this.cacheParams?e=this.isSortModelDifferent():e=!0,e&&this.reset()},t.prototype.isSortModelDifferent=function(){return!_.jsonEquals(this.cacheParams.sortModel,this.sortController.getSortModel())},t.prototype.getType=function(){return L.ROW_MODEL_TYPE_INFINITE},t.prototype.setDatasource=function(e){this.destroyDatasource(),this.datasource=e,e&&this.reset()},t.prototype.isEmpty=function(){return!this.infiniteCache},t.prototype.isRowsToRender=function(){return!!this.infiniteCache},t.prototype.getNodesInRangeForSelection=function(e,r){return this.infiniteCache?this.infiniteCache.getRowNodesInRange(e,r):[]},t.prototype.reset=function(){if(!!this.datasource){var e=this.gridOptionsWrapper.getRowIdFunc(),r=e!=null;r||this.selectionService.reset(),this.resetCache();var n=this.createModelUpdatedEvent();this.eventService.dispatchEvent(n)}},t.prototype.createModelUpdatedEvent=function(){return{type:x.EVENT_MODEL_UPDATED,api:this.gridApi,columnApi:this.columnApi,newPage:!1,newData:!1,keepRenderedRows:!0,animate:!1}},t.prototype.resetCache=function(){this.destroyCache(),this.cacheParams={datasource:this.datasource,filterModel:this.filterManager.getFilterModel(),sortModel:this.sortController.getSortModel(),rowNodeBlockLoader:this.rowNodeBlockLoader,initialRowCount:this.defaultIfInvalid(this.gridOptionsWrapper.getInfiniteInitialRowCount(),1),maxBlocksInCache:this.gridOptionsWrapper.getMaxBlocksInCache(),rowHeight:this.gridOptionsWrapper.getRowHeightAsNumber(),overflowSize:this.defaultIfInvalid(this.gridOptionsWrapper.getCacheOverflowSize(),1),blockSize:this.defaultIfInvalid(this.gridOptionsWrapper.getCacheBlockSize(),100),lastAccessedSequence:new Bt},this.infiniteCache=this.createBean(new n4(this.cacheParams))},t.prototype.defaultIfInvalid=function(e,r){return e>0?e:r},t.prototype.destroyCache=function(){this.infiniteCache&&(this.infiniteCache=this.destroyBean(this.infiniteCache))},t.prototype.onCacheUpdated=function(){var e=this.createModelUpdatedEvent();this.eventService.dispatchEvent(e)},t.prototype.getRow=function(e){if(!!this.infiniteCache&&!(e>=this.infiniteCache.getRowCount()))return this.infiniteCache.getRow(e)},t.prototype.getRowNode=function(e){var r;return this.forEachNode(function(n){n.id===e&&(r=n)}),r},t.prototype.forEachNode=function(e){this.infiniteCache&&this.infiniteCache.forEachNodeDeep(e)},t.prototype.getTopLevelRowCount=function(){return this.getRowCount()},t.prototype.getTopLevelRowDisplayedIndex=function(e){return e},t.prototype.getRowIndexAtPixel=function(e){if(this.rowHeight!==0){var r=Math.floor(e/this.rowHeight),n=this.getRowCount()-1;return r>n?n:r}return 0},t.prototype.getRowCount=function(){return this.infiniteCache?this.infiniteCache.getRowCount():0},t.prototype.isRowPresent=function(e){var r=this.getRowNode(e.id);return!!r},t.prototype.refreshCache=function(){this.infiniteCache&&this.infiniteCache.refreshCache()},t.prototype.purgeCache=function(){this.infiniteCache&&this.infiniteCache.purgeCache()},t.prototype.isLastRowIndexKnown=function(){return this.infiniteCache?this.infiniteCache.isLastRowIndexKnown():!1},t.prototype.setRowCount=function(e,r){this.infiniteCache&&this.infiniteCache.setRowCount(e,r)},hi([y("filterManager")],t.prototype,"filterManager",void 0),hi([y("sortController")],t.prototype,"sortController",void 0),hi([y("selectionService")],t.prototype,"selectionService",void 0),hi([y("gridApi")],t.prototype,"gridApi",void 0),hi([y("columnApi")],t.prototype,"columnApi",void 0),hi([y("rowRenderer")],t.prototype,"rowRenderer",void 0),hi([y("rowNodeBlockLoader")],t.prototype,"rowNodeBlockLoader",void 0),hi([B],t.prototype,"init",null),hi([nt],t.prototype,"destroyDatasource",null),t=hi([Q("rowModel")],t),t}(H),a4={moduleName:oe.InfiniteRowModelModule,rowModels:{infinite:o4}},BP=function(){function i(){}return i.prototype.setBeans=function(t){this.beans=t},i.prototype.getFileName=function(t){var e=this.getDefaultFileExtension();return(t==null||!t.length)&&(t=this.getDefaultFileName()),t.indexOf(".")===-1?t+"."+e:t},i.prototype.getData=function(t){var e=this.createSerializingSession(t),r=this.beans.gridSerializer.serialize(e,t);return r},i}(),HP=function(){function i(t){this.groupColumns=[];var e=t.columnModel,r=t.valueService,n=t.gridOptionsWrapper,o=t.processCellCallback,a=t.processHeaderCallback,s=t.processGroupHeaderCallback,l=t.processRowGroupCallback;this.columnModel=e,this.valueService=r,this.gridOptionsWrapper=n,this.processCellCallback=o,this.processHeaderCallback=a,this.processGroupHeaderCallback=s,this.processRowGroupCallback=l}return i.prototype.prepare=function(t){this.groupColumns=t.filter(function(e){return!!e.getColDef().showRowGroup})},i.prototype.extractHeaderValue=function(t){var e=this.getHeaderName(this.processHeaderCallback,t);return e!=null?e:""},i.prototype.extractRowCellValue=function(t,e,r,n,o){var a=this.gridOptionsWrapper.isGroupMultiAutoColumn()?o.rowGroupIndex:0,s=o&&o.group&&(e===a&&this.groupColumns.indexOf(t)!==-1||e===0&&this.gridOptionsWrapper.isGroupUseEntireRow(this.columnModel.isPivotMode())),l;s?l=this.createValueForGroupNode(o):l=this.valueService.getValue(t,o);var u=this.processCell(r,o,t,l,this.processCellCallback,n);return u!=null?u:""},i.prototype.getHeaderName=function(t,e){return t?t({column:e,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()}):this.columnModel.getDisplayNameForColumn(e,"csv",!0)},i.prototype.createValueForGroupNode=function(t){if(this.processRowGroupCallback)return this.processRowGroupCallback({node:t,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()});var e=[t.key];if(!this.gridOptionsWrapper.isGroupMultiAutoColumn())for(;t.parent;)t=t.parent,e.push(t.key);return e.reverse().join(" -> ")},i.prototype.processCell=function(t,e,r,n,o,a){return o?o({accumulatedRowIndex:t,column:r,node:e,value:n,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext(),type:a}):n!=null?n:""},i}(),zw=function(){function i(){}return i.download=function(t,e){var r=document.defaultView||window;if(!r){console.warn("AG Grid: There is no `window` associated with the current `document`");return}var n=document.createElement("a"),o=r.URL.createObjectURL(e);n.setAttribute("href",o),n.setAttribute("download",t),n.style.display="none",document.body.appendChild(n),n.dispatchEvent(new MouseEvent("click",{bubbles:!1,cancelable:!0,view:r})),document.body.removeChild(n),r.setTimeout(function(){r.URL.revokeObjectURL(o)},0)},i}(),s4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ex=`\r
`,l4=function(i){s4(t,i);function t(e){var r=i.call(this,e)||this;r.isFirstLine=!0,r.result="";var n=e.suppressQuotes,o=e.columnSeparator;return r.suppressQuotes=n,r.columnSeparator=o,r}return t.prototype.addCustomContent=function(e){var r=this;!e||(typeof e=="string"?(/^\s*\n/.test(e)||this.beginNewLine(),e=e.replace(/\r?\n/g,ex),this.result+=e):e.forEach(function(n){r.beginNewLine(),n.forEach(function(o,a){a!==0&&(r.result+=r.columnSeparator),r.result+=r.putInQuotes(o.data.value||""),o.mergeAcross&&r.appendEmptyCells(o.mergeAcross)})}))},t.prototype.onNewHeaderGroupingRow=function(){return this.beginNewLine(),{onColumn:this.onNewHeaderGroupingRowColumn.bind(this)}},t.prototype.onNewHeaderGroupingRowColumn=function(e,r,n){r!=0&&(this.result+=this.columnSeparator),this.result+=this.putInQuotes(e),this.appendEmptyCells(n)},t.prototype.appendEmptyCells=function(e){for(var r=1;r<=e;r++)this.result+=this.columnSeparator+this.putInQuotes("")},t.prototype.onNewHeaderRow=function(){return this.beginNewLine(),{onColumn:this.onNewHeaderRowColumn.bind(this)}},t.prototype.onNewHeaderRowColumn=function(e,r){r!=0&&(this.result+=this.columnSeparator),this.result+=this.putInQuotes(this.extractHeaderValue(e))},t.prototype.onNewBodyRow=function(){return this.beginNewLine(),{onColumn:this.onNewBodyRowColumn.bind(this)}},t.prototype.onNewBodyRowColumn=function(e,r,n){r!=0&&(this.result+=this.columnSeparator),this.result+=this.putInQuotes(this.extractRowCellValue(e,r,r,L.EXPORT_TYPE_CSV,n))},t.prototype.putInQuotes=function(e){if(this.suppressQuotes)return e;if(e==null)return'""';var r;typeof e=="string"?r=e:typeof e.toString=="function"?r=e.toString():(console.warn("AG Grid: unknown value type during csv conversion"),r="");var n=r.replace(/"/g,'""');return'"'+n+'"'},t.prototype.parse=function(){return this.result},t.prototype.beginNewLine=function(){this.isFirstLine||(this.result+=ex),this.isFirstLine=!1},t}(HP),u4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Bs=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},jP=function(i){u4(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){this.setBeans({gridSerializer:this.gridSerializer,gridOptionsWrapper:this.gridOptionsWrapper})},t.prototype.getMergedParams=function(e){var r=this.gridOptionsWrapper.getDefaultExportParams("csv");return Object.assign({},r,e)},t.prototype.export=function(e){if(this.isExportSuppressed())return console.warn("AG Grid: Export cancelled. Export is not allowed as per your configuration."),"";var r=this.getMergedParams(e),n=this.getData(r),o=new Blob(["\uFEFF",n],{type:"text/plain"});return zw.download(this.getFileName(r.fileName),o),n},t.prototype.exportDataAsCsv=function(e){return this.export(e)},t.prototype.getDataAsCsv=function(e){var r=this.getMergedParams(e);return this.getData(r)},t.prototype.getDefaultFileName=function(){return"export.csv"},t.prototype.getDefaultFileExtension=function(){return"csv"},t.prototype.createSerializingSession=function(e){var r=this,n=r.columnModel,o=r.valueService,a=r.gridOptionsWrapper,s=e,l=s.processCellCallback,u=s.processHeaderCallback,c=s.processGroupHeaderCallback,p=s.processRowGroupCallback,h=s.suppressQuotes,d=s.columnSeparator;return new l4({columnModel:n,valueService:o,gridOptionsWrapper:a,processCellCallback:l||void 0,processHeaderCallback:u||void 0,processGroupHeaderCallback:c||void 0,processRowGroupCallback:p||void 0,suppressQuotes:h||!1,columnSeparator:d||","})},t.prototype.isExportSuppressed=function(){return this.gridOptionsWrapper.isSuppressCsvExport()},Bs([y("columnModel")],t.prototype,"columnModel",void 0),Bs([y("valueService")],t.prototype,"valueService",void 0),Bs([y("gridSerializer")],t.prototype,"gridSerializer",void 0),Bs([y("gridOptionsWrapper")],t.prototype,"gridOptionsWrapper",void 0),Bs([B],t.prototype,"postConstruct",null),t=Bs([Q("csvCreator")],t),t}(BP),c4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),xa=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},rs;(function(i){i[i.HEADER_GROUPING=0]="HEADER_GROUPING",i[i.HEADER=1]="HEADER",i[i.BODY=2]="BODY"})(rs||(rs={}));var UP=function(i){c4(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.serialize=function(e,r){r===void 0&&(r={});var n=this.getColumnsToExport(r.allColumns,r.columnKeys),o=_.compose(this.prepareSession(n),this.prependContent(r),this.exportColumnGroups(r,n),this.exportHeaders(r,n),this.processPinnedTopRows(r,n),this.processRows(r,n),this.processPinnedBottomRows(r,n),this.appendContent(r));return o(e).parse()},t.prototype.processRow=function(e,r,n,o){var a=this,s=r.shouldRowBeSkipped||function(){return!1},l=this.gridOptionsWrapper,u=l.getContext(),c=l.getApi(),p=l.getColumnApi(),h=l.isGroupRemoveSingleChildren(),d=l.isGroupHideOpenParents(),f=l.isGroupRemoveLowestSingleChildren(),g=this.columnModel.isPivotMode()?o.leafGroup:!o.group,v=r.skipGroups||r.skipRowGroups,m=f&&o.leafGroup,C=o.allChildrenCount===1&&(h||m);v&&r.skipGroups&&_.doOnce(function(){return console.warn("AG Grid: Since v25.2 `skipGroups` has been renamed to `skipRowGroups`.")},"gridSerializer-skipGroups");var w={rowIndex:o.rowIndex,rowPinned:o.rowPinned};if(!(!g&&(r.skipRowGroups||C||d)||r.onlySelected&&!o.isSelected()||r.rowNodes&&!r.rowNodes.some(function(P){return a.rowPositionUtils.sameRow(P,w)})||r.skipPinnedTop&&o.rowPinned==="top"||r.skipPinnedBottom&&o.rowPinned==="bottom")){var b=o.level===-1;if(!(b&&!o.leafGroup)){var S=s({node:o,api:c,columnApi:p,context:u});if(!S){var T=e.onNewBodyRow();if(n.forEach(function(P,R){T.onColumn(P,R,o)}),r.getCustomContentBelowRow){var E=r.getCustomContentBelowRow({node:o,api:c,columnApi:p,context:u});E&&e.addCustomContent(E)}}}}},t.prototype.appendContent=function(e){return function(r){var n=e.customFooter||e.appendContent;return n&&(e.customFooter&&_.doOnce(function(){return console.warn("AG Grid: Since version 25.2.0 the `customFooter` param has been deprecated. Use `appendContent` instead.")},"gridSerializer-customFooter"),r.addCustomContent(n)),r}},t.prototype.prependContent=function(e){return function(r){var n=e.customHeader||e.prependContent;return n&&(e.customHeader&&_.doOnce(function(){return console.warn("AG Grid: Since version 25.2.0 the `customHeader` param has been deprecated. Use `prependContent` instead.")},"gridSerializer-customHeader"),r.addCustomContent(n)),r}},t.prototype.prepareSession=function(e){return function(r){return r.prepare(e),r}},t.prototype.exportColumnGroups=function(e,r){var n=this;return function(o){if(e.skipColumnGroupHeaders)e.columnGroups&&_.doOnce(function(){return console.warn("AG Grid: Since v25.2 the `columnGroups` param has deprecated, and groups are exported by default.")},"gridSerializer-columnGroups");else{var a=new OC,s=n.displayedGroupCreator.createDisplayedGroups(r,n.columnModel.getGridBalancedTree(),a,null);n.recursivelyAddHeaderGroups(s,o,e.processGroupHeaderCallback)}return o}},t.prototype.exportHeaders=function(e,r){return function(n){if(!e.skipHeader&&!e.skipColumnHeaders){var o=n.onNewHeaderRow();r.forEach(function(a,s){o.onColumn(a,s,void 0)})}else e.skipHeader&&_.doOnce(function(){return console.warn("AG Grid: Since v25.2 the `skipHeader` param has been renamed to `skipColumnHeaders`.")},"gridSerializer-skipHeader");return n}},t.prototype.processPinnedTopRows=function(e,r){var n=this;return function(o){var a=n.processRow.bind(n,o,e,r);return n.pinnedRowModel.forEachPinnedTopRow(a),o}},t.prototype.processRows=function(e,r){var n=this;return function(o){var a=n.rowModel,s=a.getType(),l=s===L.ROW_MODEL_TYPE_CLIENT_SIDE,u=s===L.ROW_MODEL_TYPE_SERVER_SIDE,c=!l&&e.onlySelected,p=n.processRow.bind(n,o,e,r);if(n.columnModel.isPivotMode())l?a.forEachPivotNode(p):a.forEachNode(p);else if(e.onlySelectedAllPages||c){var h=n.selectionService.getSelectedNodes();h.forEach(p)}else l||u?a.forEachNodeAfterFilterAndSort(p):a.forEachNode(p);return o}},t.prototype.processPinnedBottomRows=function(e,r){var n=this;return function(o){var a=n.processRow.bind(n,o,e,r);return n.pinnedRowModel.forEachPinnedBottomRow(a),o}},t.prototype.getColumnsToExport=function(e,r){e===void 0&&(e=!1);var n=this.columnModel.isPivotMode();if(r&&r.length)return this.columnModel.getGridColumns(r);if(e&&!n){var o=this.gridOptionsWrapper.isTreeData()?this.columnModel.getGridColumns([L.GROUP_AUTO_COLUMN_ID]):[];return o.concat(this.columnModel.getAllPrimaryColumns()||[])}return this.columnModel.getAllDisplayedColumns()},t.prototype.recursivelyAddHeaderGroups=function(e,r,n){var o=[];e.forEach(function(a){var s=a;!s.getChildren||s.getChildren().forEach(function(l){return o.push(l)})}),e.length>0&&e[0]instanceof sr&&this.doAddHeaderHeader(r,e,n),o&&o.length>0&&this.recursivelyAddHeaderGroups(o,r,n)},t.prototype.doAddHeaderHeader=function(e,r,n){var o=this,a=e.onNewHeaderGroupingRow(),s=0;r.forEach(function(l){var u=l,c;n?c=n({columnGroup:u,api:o.gridOptionsWrapper.getApi(),columnApi:o.gridOptionsWrapper.getColumnApi(),context:o.gridOptionsWrapper.getContext()}):c=o.columnModel.getDisplayNameForColumnGroup(u,"header");var p=u.getLeafColumns().reduce(function(h,d,f,g){var v=_.last(h),m=d.getColumnGroupShow()==="open";return m?(!v||v[1]!=null)&&(v=[f],h.push(v)):v&&v[1]==null&&(v[1]=f-1),f===g.length-1&&v&&v[1]==null&&(v[1]=f),h},[]);a.onColumn(c||"",s++,u.getLeafColumns().length-1,p)})},xa([y("displayedGroupCreator")],t.prototype,"displayedGroupCreator",void 0),xa([y("columnModel")],t.prototype,"columnModel",void 0),xa([y("rowModel")],t.prototype,"rowModel",void 0),xa([y("pinnedRowModel")],t.prototype,"pinnedRowModel",void 0),xa([y("selectionService")],t.prototype,"selectionService",void 0),xa([y("rowPositionUtils")],t.prototype,"rowPositionUtils",void 0),t=xa([Q("gridSerializer")],t),t}(H),$w={moduleName:oe.CsvExportModule,beans:[jP,UP]},$p=`\r
`,my=function(){function i(){}return i.createHeader=function(t){t===void 0&&(t={});var e="<?",r="?>",n=["version"];t.version||(t.version="1.0"),t.encoding&&n.push("encoding"),t.standalone&&n.push("standalone");var o=n.map(function(a){return a+'="'+t[a]+'"'}).join(" ");return e+"xml "+o+" "+r},i.createXml=function(t,e){var r=this,n="";t.properties&&(t.properties.prefixedAttributes&&t.properties.prefixedAttributes.forEach(function(a){Object.keys(a.map).forEach(function(s){n+=r.returnAttributeIfPopulated(a.prefix+s,a.map[s],e)})}),t.properties.rawMap&&Object.keys(t.properties.rawMap).forEach(function(a){n+=r.returnAttributeIfPopulated(a,t.properties.rawMap[a],e)}));var o="<"+t.name+n;return!t.children&&t.textNode==null?o+"/>"+$p:t.textNode!=null?o+">"+t.textNode+"</"+t.name+">"+$p:(o+=">"+$p,t.children&&t.children.forEach(function(a){o+=r.createXml(a,e)}),o+"</"+t.name+">"+$p)},i.returnAttributeIfPopulated=function(t,e,r){if(!e&&e!==""&&e!==0)return"";var n=e;return typeof e=="boolean"&&r&&(n=r(e))," "+t+'="'+n+'"'},i}(),p4=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},h4=new Uint32Array([0,1996959894,3993919788,2567524794,124634137,1886057615,3915621685,2657392035,249268274,2044508324,3772115230,2547177864,162941995,2125561021,3887607047,2428444049,498536548,1789927666,4089016648,2227061214,450548861,1843258603,4107580753,2211677639,325883990,1684777152,4251122042,2321926636,335633487,1661365465,4195302755,2366115317,997073096,1281953886,3579855332,2724688242,1006888145,1258607687,3524101629,2768942443,901097722,1119000684,3686517206,2898065728,853044451,1172266101,3705015759,2882616665,651767980,1373503546,3369554304,3218104598,565507253,1454621731,3485111705,3099436303,671266974,1594198024,3322730930,2970347812,795835527,1483230225,3244367275,3060149565,1994146192,31158534,2563907772,4023717930,1907459465,112637215,2680153253,3904427059,2013776290,251722036,2517215374,3775830040,2137656763,141376813,2439277719,3865271297,1802195444,476864866,2238001368,4066508878,1812370925,453092731,2181625025,4111451223,1706088902,314042704,2344532202,4240017532,1658658271,366619977,2362670323,4224994405,1303535960,984961486,2747007092,3569037538,1256170817,1037604311,2765210733,3554079995,1131014506,879679996,2909243462,3663771856,1141124467,855842277,2852801631,3708648649,1342533948,654459306,3188396048,3373015174,1466479909,544179635,3110523913,3462522015,1591671054,702138776,2966460450,3352799412,1504918807,783551873,3082640443,3233442989,3988292384,2596254646,62317068,1957810842,3939845945,2647816111,81470997,1943803523,3814918930,2489596804,225274430,2053790376,3826175755,2466906013,167816743,2097651377,4027552580,2265490386,503444072,1762050814,4150417245,2154129355,426522225,1852507879,4275313526,2312317920,282753626,1742555852,4189708143,2394877945,397917763,1622183637,3604390888,2714866558,953729732,1340076626,3518719985,2797360999,1068828381,1219638859,3624741850,2936675148,906185462,1090812512,3747672003,2825379669,829329135,1181335161,3412177804,3160834842,628085408,1382605366,3423369109,3138078467,570562233,1426400815,3317316542,2998733608,733239954,1555261956,3268935591,3050360625,752459403,1541320221,2607071920,3965973030,1969922972,40735498,2617837225,3943577151,1913087877,83908371,2512341634,3803740692,2075208622,213261112,2463272603,3855990285,2094854071,198958881,2262029012,4057260610,1759359992,534414190,2176718541,4139329115,1873836001,414664567,2282248934,4279200368,1711684554,285281116,2405801727,4167216745,1634467795,376229701,2685067896,3608007406,1308918612,956543938,2808555105,3495958263,1231636301,1047427035,2932959818,3654703836,1088359270,936918e3,2847714899,3736837829,1202900863,817233897,3183342108,3401237130,1404277552,615818150,3134207493,3453421203,1423857449,601450431,3009837614,3294710456,1567103746,711928724,3020668471,3272380065,1510334235,755167117]),dr=function(){function i(){}return i.addFolders=function(t){t.forEach(this.addFolder.bind(this))},i.addFolder=function(t){this.folders.push({path:t,created:new Date,isBase64:!1})},i.addFile=function(t,e,r){r===void 0&&(r=!1),this.files.push({path:t,created:new Date,content:e,isBase64:r})},i.getContent=function(t){t===void 0&&(t="application/zip");var e=this.buildFileStream(),r=this.buildUint8Array(e);return this.clearStream(),new Blob([r],{type:t})},i.clearStream=function(){this.folders=[],this.files=[]},i.buildFileStream=function(t){var e,r;t===void 0&&(t="");var n=this.folders.concat(this.files),o=n.length,a="",s=0,l=0;try{for(var u=p4(n),c=u.next();!c.done;c=u.next()){var p=c.value,h=this.getHeader(p,s),d=h.fileHeader,f=h.folderHeader,g=h.content;s+=d.length+g.length,l+=f.length,t+=d+g,a+=f}}catch(m){e={error:m}}finally{try{c&&!c.done&&(r=u.return)&&r.call(u)}finally{if(e)throw e.error}}var v=this.buildFolderEnd(o,l,s);return t+a+v},i.getHeader=function(t,e){var r=t.content,n=t.path,o=t.created,a=t.isBase64,s=_.utf8_encode,l=_.decToHex,u=s(n),c=u!==n,p=this.convertTime(o),h=this.convertDate(o),d="";if(c){var f=l(1,1)+l(this.getFromCrc32Table(u),4)+u;d="up"+l(f.length,2)+f}var g=r?this.getConvertedContent(r,a):{size:0,content:""},v=g.size,m=g.content,C=`
\0`+(c?"\0\b":"\0\0")+"\0\0"+l(p,2)+l(h,2)+l(v?this.getFromCrc32Table(m):0,4)+l(v,4)+l(v,4)+l(u.length,2)+l(d.length,2),w="PK"+C+u+d,b="PK\0"+C+"\0\0\0\0\0\0"+(r?"\0\0\0\0":"\0\0\0")+l(e,4)+u+d;return{fileHeader:w,folderHeader:b,content:m||""}},i.getConvertedContent=function(t,e){return e===void 0&&(e=!1),e&&(t=t.split(";base64,")[1]),t=e?atob(t):t,{size:t.length,content:t}},i.buildFolderEnd=function(t,e,r){var n=_.decToHex;return"PK\0\0\0\0"+n(t,2)+n(t,2)+n(e,4)+n(r,4)+"\0\0"},i.buildUint8Array=function(t){for(var e=new Uint8Array(t.length),r=0;r<e.length;r++)e[r]=t.charCodeAt(r);return e},i.getFromCrc32Table=function(t){if(!t.length)return 0;for(var e=t.length,r=new Uint8Array(e),n=0;n<e;n++)r[n]=t.charCodeAt(n);for(var o=-1,a=0,s=0,l=0,n=0;n<e;n++)a=r[n],s=(o^a)&255,l=h4[s],o=o>>>8^l;return o^-1},i.convertTime=function(t){var e=t.getHours();return e<<=6,e=e|t.getMinutes(),e<<=5,e=e|t.getSeconds()/2,e},i.convertDate=function(t){var e=t.getFullYear()-1980;return e<<=4,e=e|t.getMonth()+1,e<<=5,e=e|t.getDate(),e},i.folders=[],i.files=[],i}(),d4=[Jj,a4,$w],f4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),dv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},g4=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},zP=function(i){f4(t,i);function t(){var r=i!==null&&i.apply(this,arguments)||this;return r.watermarkMessage=void 0,r}e=t,t.prototype.validateLicense=function(){if(_.missingOrEmpty(e.licenseKey))this.outputMissingLicenseKey();else if(e.licenseKey.length>32){var r=e.extractLicenseComponents(e.licenseKey),n=r.md5,o=r.license,a=r.version,s=r.isTrial;n===this.md5.md5(o)?_.exists(a)&&a?this.validateLicenseKeyForVersion(a,!!s,o):this.validateLegacyKey(o):this.outputInvalidLicenseKey()}else this.outputInvalidLicenseKey()},t.extractExpiry=function(r){var n=r.substring(r.lastIndexOf("_")+1,r.length);return new Date(parseInt(e.decode(n),10))},t.extractLicenseComponents=function(r){var n=r.replace(/[\u200B-\u200D\uFEFF]/g,"");n=n.replace(/\r?\n|\r/g,"");var o=n.length-32,a=n.substring(o),s=n.substring(0,o),l=g4(e.extractBracketedInformation(n),2),u=l[0],c=l[1];return{md5:a,license:s,version:u,isTrial:c}},t.prototype.getLicenseDetails=function(r){var n=e.extractLicenseComponents(r),o=n.md5,a=n.license,s=n.version,l=n.isTrial,u=o===this.md5.md5(a),c=null;return u&&(c=e.extractExpiry(a),u=!isNaN(c.getTime())),{licenseKey:r,valid:u,expiry:u?e.formatDate(c):null,version:s||"legacy",isTrial:l}},t.prototype.isDisplayWatermark=function(){return!_.missingOrEmpty(this.watermarkMessage)},t.prototype.getWatermarkMessage=function(){return this.watermarkMessage||""},t.formatDate=function(r){var n=["January","February","March","April","May","June","July","August","September","October","November","December"],o=r.getDate(),a=r.getMonth(),s=r.getFullYear();return o+" "+n[a]+" "+s},t.getGridReleaseDate=function(){return new Date(parseInt(e.decode(e.RELEASE_INFORMATION),10))},t.decode=function(r){for(var n="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",o="",a,s,l,u,c,p,h,d=0,f=r.replace(/[^A-Za-z0-9+/=]/g,"");d<f.length;)u=n.indexOf(f.charAt(d++)),c=n.indexOf(f.charAt(d++)),p=n.indexOf(f.charAt(d++)),h=n.indexOf(f.charAt(d++)),a=u<<2|c>>4,s=(c&15)<<4|p>>2,l=(p&3)<<6|h,o=o+String.fromCharCode(a),p!=64&&(o=o+String.fromCharCode(s)),h!=64&&(o=o+String.fromCharCode(l));return o=e.utf8_decode(o),o},t.utf8_decode=function(r){r=r.replace(/rn/g,"n");for(var n="",o=0;o<r.length;o++){var a=r.charCodeAt(o);a<128?n+=String.fromCharCode(a):a>127&&a<2048?(n+=String.fromCharCode(a>>6|192),n+=String.fromCharCode(a&63|128)):(n+=String.fromCharCode(a>>12|224),n+=String.fromCharCode(a>>6&63|128),n+=String.fromCharCode(a&63|128))}return n},t.setLicenseKey=function(r){e.licenseKey=r},t.extractBracketedInformation=function(r){var n=r.split("[").filter(function(s){return s.indexOf("]")>-1}).map(function(s){return s.split("]")[0]});if(!n||n.length===0)return[null,null];var o=n.filter(function(s){return s==="TRIAL"}).length===1,a=n.filter(function(s){return s.indexOf("v")===0}).map(function(s){return s.replace(/^v/,"")})[0];return[a,o]},t.prototype.validateLicenseKeyForVersion=function(r,n,o){r==="2"&&(n?this.validateForTrial(o):this.validateLegacyKey(o))},t.prototype.validateLegacyKey=function(r){var n=e.getGridReleaseDate(),o=e.extractExpiry(r),a=!1,s=!1;if(isNaN(o.getTime())||(a=!0,s=n<o),!a)this.outputInvalidLicenseKey();else if(!s){var l=e.formatDate(o),u=e.formatDate(n);this.outputIncompatibleVersion(l,u)}},t.prototype.validateForTrial=function(r){var n=e.extractExpiry(r),o=new Date,a=!1,s=!1;if(isNaN(n.getTime())||(a=!0,s=n>o),!a)this.outputInvalidLicenseKey();else if(!s){var l=e.formatDate(n);this.outputExpiredTrialKey(l)}},t.prototype.outputInvalidLicenseKey=function(){console.error("*****************************************************************************************************************"),console.error("***************************************** AG Grid Enterprise License ********************************************"),console.error("********************************************* Invalid License ***************************************************"),console.error("* Your license for AG Grid Enterprise is not valid - please contact info@ag-grid.com to obtain a valid license. *"),console.error("*****************************************************************************************************************"),console.error("*****************************************************************************************************************"),this.watermarkMessage="Invalid License"},t.prototype.outputExpiredTrialKey=function(r){console.error("****************************************************************************************************************"),console.error("***************************************** AG Grid Enterprise License *******************************************"),console.error("*****************************************   Trial Period Expired.    *******************************************"),console.error("* Your license for AG Grid Enterprise expired on "+r+".                                                *"),console.error("* Please email info@ag-grid.com to purchase a license.                                                         *"),console.error("****************************************************************************************************************"),console.error("****************************************************************************************************************"),this.watermarkMessage="Trial Period Expired"},t.prototype.outputMissingLicenseKey=function(){console.error("****************************************************************************************************************"),console.error("***************************************** AG Grid Enterprise License *******************************************"),console.error("****************************************** License Key Not Found ***********************************************"),console.error("* All AG Grid Enterprise features are unlocked.                                                                *"),console.error("* This is an evaluation only version, it is not licensed for development projects intended for production.     *"),console.error("* If you want to hide the watermark, please email info@ag-grid.com for a trial license.                        *"),console.error("****************************************************************************************************************"),console.error("****************************************************************************************************************"),this.watermarkMessage="For Trial Use Only"},t.prototype.outputIncompatibleVersion=function(r,n){console.error("****************************************************************************************************************************"),console.error("****************************************************************************************************************************"),console.error("*                                             AG Grid Enterprise License                                                   *"),console.error("*                           License not compatible with installed version of AG Grid Enterprise.                           *"),console.error("*                                                                                                                          *"),console.error("* Your AG Grid License entitles you to all versions of AG Grid that we release within the time covered by your license     *"),console.error("* - typically we provide one year licenses which entitles you to all releases / updates of AG Grid within that year.       *"),console.error("* Your license has an end (expiry) date which stops the license key working with versions of AG Grid released after the    *"),console.error("* license end date. The license key that you have expires on "+r+", however the version of AG Grid you    *"),console.error("* are trying to use was released on "+n+".                                                               *"),console.error("*                                                                                                                          *"),console.error("* Please contact info@ag-grid.com to renew your subscription to new versions and get a new license key to work with this   *"),console.error("* version of AG Grid.                                                                                                      *"),console.error("****************************************************************************************************************************"),console.error("****************************************************************************************************************************"),this.watermarkMessage="License Expired"};var e;return t.RELEASE_INFORMATION="MTY1MjQ0MzczMDM1MQ==",dv([y("md5")],t.prototype,"md5",void 0),dv([ss],t.prototype,"validateLicense",null),t=e=dv([Q("licenseManager")],t),t}(H),v4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),tx=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},m4=function(i){v4(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.ieCompatibility=!1,e}return t.prototype.init=function(){this.ieCompatibility=this.md5("hello")!="5d41402abc4b2a76b9719d911017c592"},t.prototype.md5cycle=function(e,r){var n=e[0],o=e[1],a=e[2],s=e[3];n=this.ff(n,o,a,s,r[0],7,-680876936),s=this.ff(s,n,o,a,r[1],12,-389564586),a=this.ff(a,s,n,o,r[2],17,606105819),o=this.ff(o,a,s,n,r[3],22,-1044525330),n=this.ff(n,o,a,s,r[4],7,-176418897),s=this.ff(s,n,o,a,r[5],12,1200080426),a=this.ff(a,s,n,o,r[6],17,-1473231341),o=this.ff(o,a,s,n,r[7],22,-45705983),n=this.ff(n,o,a,s,r[8],7,1770035416),s=this.ff(s,n,o,a,r[9],12,-1958414417),a=this.ff(a,s,n,o,r[10],17,-42063),o=this.ff(o,a,s,n,r[11],22,-1990404162),n=this.ff(n,o,a,s,r[12],7,1804603682),s=this.ff(s,n,o,a,r[13],12,-40341101),a=this.ff(a,s,n,o,r[14],17,-1502002290),o=this.ff(o,a,s,n,r[15],22,1236535329),n=this.gg(n,o,a,s,r[1],5,-165796510),s=this.gg(s,n,o,a,r[6],9,-1069501632),a=this.gg(a,s,n,o,r[11],14,643717713),o=this.gg(o,a,s,n,r[0],20,-373897302),n=this.gg(n,o,a,s,r[5],5,-701558691),s=this.gg(s,n,o,a,r[10],9,38016083),a=this.gg(a,s,n,o,r[15],14,-660478335),o=this.gg(o,a,s,n,r[4],20,-405537848),n=this.gg(n,o,a,s,r[9],5,568446438),s=this.gg(s,n,o,a,r[14],9,-1019803690),a=this.gg(a,s,n,o,r[3],14,-187363961),o=this.gg(o,a,s,n,r[8],20,1163531501),n=this.gg(n,o,a,s,r[13],5,-1444681467),s=this.gg(s,n,o,a,r[2],9,-51403784),a=this.gg(a,s,n,o,r[7],14,1735328473),o=this.gg(o,a,s,n,r[12],20,-1926607734),n=this.hh(n,o,a,s,r[5],4,-378558),s=this.hh(s,n,o,a,r[8],11,-2022574463),a=this.hh(a,s,n,o,r[11],16,1839030562),o=this.hh(o,a,s,n,r[14],23,-35309556),n=this.hh(n,o,a,s,r[1],4,-1530992060),s=this.hh(s,n,o,a,r[4],11,1272893353),a=this.hh(a,s,n,o,r[7],16,-155497632),o=this.hh(o,a,s,n,r[10],23,-1094730640),n=this.hh(n,o,a,s,r[13],4,681279174),s=this.hh(s,n,o,a,r[0],11,-358537222),a=this.hh(a,s,n,o,r[3],16,-722521979),o=this.hh(o,a,s,n,r[6],23,76029189),n=this.hh(n,o,a,s,r[9],4,-640364487),s=this.hh(s,n,o,a,r[12],11,-421815835),a=this.hh(a,s,n,o,r[15],16,530742520),o=this.hh(o,a,s,n,r[2],23,-995338651),n=this.ii(n,o,a,s,r[0],6,-198630844),s=this.ii(s,n,o,a,r[7],10,1126891415),a=this.ii(a,s,n,o,r[14],15,-1416354905),o=this.ii(o,a,s,n,r[5],21,-57434055),n=this.ii(n,o,a,s,r[12],6,1700485571),s=this.ii(s,n,o,a,r[3],10,-1894986606),a=this.ii(a,s,n,o,r[10],15,-1051523),o=this.ii(o,a,s,n,r[1],21,-2054922799),n=this.ii(n,o,a,s,r[8],6,1873313359),s=this.ii(s,n,o,a,r[15],10,-30611744),a=this.ii(a,s,n,o,r[6],15,-1560198380),o=this.ii(o,a,s,n,r[13],21,1309151649),n=this.ii(n,o,a,s,r[4],6,-145523070),s=this.ii(s,n,o,a,r[11],10,-1120210379),a=this.ii(a,s,n,o,r[2],15,718787259),o=this.ii(o,a,s,n,r[9],21,-343485551),e[0]=this.add32(n,e[0]),e[1]=this.add32(o,e[1]),e[2]=this.add32(a,e[2]),e[3]=this.add32(s,e[3])},t.prototype.cmn=function(e,r,n,o,a,s){return r=this.add32(this.add32(r,e),this.add32(o,s)),this.add32(r<<a|r>>>32-a,n)},t.prototype.ff=function(e,r,n,o,a,s,l){return this.cmn(r&n|~r&o,e,r,a,s,l)},t.prototype.gg=function(e,r,n,o,a,s,l){return this.cmn(r&o|n&~o,e,r,a,s,l)},t.prototype.hh=function(e,r,n,o,a,s,l){return this.cmn(r^n^o,e,r,a,s,l)},t.prototype.ii=function(e,r,n,o,a,s,l){return this.cmn(n^(r|~o),e,r,a,s,l)},t.prototype.md51=function(e){var r=e.length,n=[1732584193,-271733879,-1732584194,271733878],o;for(o=64;o<=e.length;o+=64)this.md5cycle(n,this.md5blk(e.substring(o-64,o)));e=e.substring(o-64);var a=[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];for(o=0;o<e.length;o++)a[o>>2]|=e.charCodeAt(o)<<(o%4<<3);if(a[o>>2]|=128<<(o%4<<3),o>55)for(this.md5cycle(n,a),o=0;o<16;o++)a[o]=0;return a[14]=r*8,this.md5cycle(n,a),n},t.prototype.md5blk=function(e){for(var r=[],n=0;n<64;n+=4)r[n>>2]=e.charCodeAt(n)+(e.charCodeAt(n+1)<<8)+(e.charCodeAt(n+2)<<16)+(e.charCodeAt(n+3)<<24);return r},t.prototype.rhex=function(e){for(var r="0123456789abcdef".split(""),n="",o=0;o<4;o++)n+=r[e>>o*8+4&15]+r[e>>o*8&15];return n},t.prototype.hex=function(e){for(var r=0;r<e.length;r++)e[r]=this.rhex(e[r]);return e.join("")},t.prototype.md5=function(e){return this.hex(this.md51(e))},t.prototype.add32=function(e,r){return this.ieCompatibility?this.add32Compat(e,r):this.add32Std(e,r)},t.prototype.add32Std=function(e,r){return e+r&4294967295},t.prototype.add32Compat=function(e,r){var n=(e&65535)+(r&65535),o=(e>>16)+(r>>16)+(n>>16);return o<<16|n&65535},tx([B],t.prototype,"init",null),t=tx([Q("md5")],t),t}(H),y4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),fv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},C4=function(i){y4(t,i);function t(){return i.call(this,`<div class="ag-watermark">
                    <div ref="eLicenseTextRef" class="ag-watermark-text"></div>
               </div>`)||this}return t.prototype.postConstruct=function(){var e=this,r=this.shouldDisplayWatermark();this.addOrRemoveCssClass("ag-hidden",!r),r&&(this.eLicenseTextRef.innerText=this.licenseManager.getWatermarkMessage(),window.setTimeout(function(){return e.addCssClass("ag-opacity-zero")},0),window.setTimeout(function(){return e.addCssClass("ag-hidden")},5e3))},t.prototype.shouldDisplayWatermark=function(){var e=this.gridOptionsWrapper.getDocument(),r=e.defaultView||window,n=r.location,o=n.hostname,a=o===void 0?"":o,s=n.pathname,l=this.licenseManager.isDisplayWatermark(),u=a.match("^(?:127.0.0.1|localhost|(?:w+.)?ag-grid.com)$")!=null,c=s?s.indexOf("forceWatermark")!==-1:!1;return c||l&&!u},fv([y("licenseManager")],t.prototype,"licenseManager",void 0),fv([I("eLicenseTextRef")],t.prototype,"eLicenseTextRef",void 0),fv([B],t.prototype,"postConstruct",null),t}(ee),cr={moduleName:oe.EnterpriseCoreModule,beans:[zP,m4],agStackComponents:[{componentName:"AgWatermark",componentClass:C4}]},w4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Jt=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},b4=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},_4=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(b4(arguments[t]));return i},S4=function(i){w4(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.navigatorApiFailed=!1,e}return t.prototype.init=function(){var e=this;this.logger=this.loggerFactory.create("ClipboardService"),this.rowModel.getType()===L.ROW_MODEL_TYPE_CLIENT_SIDE&&(this.clientSideRowModel=this.rowModel),this.ctrlsService.whenReady(function(r){e.gridCtrl=r.gridCtrl})},t.prototype.pasteFromClipboard=function(){var e=this;this.logger.log("pasteFromClipboard");var r=!this.gridOptionsWrapper.isSuppressClipboardApi();r&&!this.navigatorApiFailed&&navigator.clipboard&&navigator.clipboard.readText?navigator.clipboard.readText().then(this.processClipboardData.bind(this)).catch(function(n){_.doOnce(function(){console.warn(n),console.warn("AG Grid: Unable to use the Clipboard API (navigator.clipboard.readText()). The reason why it could not be used has been logged in the previous line. For this reason the grid has defaulted to using a workaround which doesn't perform as well. Either fix why Clipboard API is blocked, OR stop this message from appearing by setting grid property suppressClipboardApi=true (which will default the grid to using the workaround rather than the API")},"clipboardApiError"),e.navigatorApiFailed=!0,e.pasteFromClipboardLegacy()}):this.pasteFromClipboardLegacy()},t.prototype.pasteFromClipboardLegacy=function(){var e=this;this.executeOnTempElement(function(r){return r.focus({preventScroll:!0})},function(r){var n=r.value;e.processClipboardData(n)})},t.prototype.processClipboardData=function(e){var r=this;if(e!=null){var n=_.stringToArray(e,this.gridOptionsWrapper.getClipboardDelimiter()),o=this.gridOptionsWrapper.getProcessDataFromClipboardFunc();if(o&&(n=o({data:n})),n!=null){this.gridOptionsWrapper.isSuppressLastEmptyLineOnPaste()&&this.removeLastLineIfBlank(n);var a=function(s,l,u,c){var p=r.rangeService&&r.rangeService.isMoreThanOneCell(),h=p&&!r.hasOnlyOneValueToPaste(n);h?r.pasteIntoActiveRange(n,s,l,c):r.pasteStartingFromFocusedCell(n,s,l,u,c)};this.doPasteOperation(a)}}},t.prototype.doPasteOperation=function(e){var r=this.gridOptionsWrapper.getApi(),n=this.gridOptionsWrapper.getColumnApi(),o="clipboard";this.eventService.dispatchEvent({type:x.EVENT_PASTE_START,api:r,columnApi:n,source:o});var a;if(this.clientSideRowModel){var s=this.gridOptionsWrapper.isAggregateOnlyChangedColumns();a=new Jc(s,this.clientSideRowModel.getRootNode())}var l={},u=[],c=this.focusService.getFocusedCell();e(l,u,c,a),a&&this.clientSideRowModel.doAggregate(a),this.rowRenderer.refreshCells(),this.dispatchFlashCells(l),this.fireRowChanged(u),c&&this.focusService.setFocusedCell(c.rowIndex,c.column,c.rowPinned,!0),this.eventService.dispatchEvent({type:x.EVENT_PASTE_END,api:r,columnApi:n,source:o})},t.prototype.pasteIntoActiveRange=function(e,r,n,o){var a=this,s=this.getRangeSize()%e.length!=0,l=0,u=0,c=function(p,h,d,f){var g=f-l>=e.length;if(g){if(s)return;l+=u,u=0}var v=e[f-l];n.push(h);var m=a.gridOptionsWrapper.getProcessCellFromClipboardFunc();d.forEach(function(C,w){if(!(!C.isCellEditable(h)||C.isSuppressPaste(h))){w>=v.length&&(w=w%v.length);var b=a.processCell(h,C,v[w],L.EXPORT_TYPE_DRAG_COPY,m);h.setDataValue(C,b,L.SOURCE_PASTE),o&&o.addParentNode(h.parent,[C]);var S=a.cellPositionUtils.createIdFromValues(p.rowIndex,C,p.rowPinned);r[S]=!0}}),u++};this.iterateActiveRanges(!1,c)},t.prototype.pasteStartingFromFocusedCell=function(e,r,n,o,a){if(!!o){var s={rowIndex:o.rowIndex,rowPinned:o.rowPinned},l=this.columnModel.getDisplayedColumnsStartingAt(o.column);this.isPasteSingleValueIntoRange(e)?this.pasteSingleValueIntoRange(e,n,r,a):this.pasteMultipleValues(e,s,n,l,r,L.EXPORT_TYPE_CLIPBOARD,a)}},t.prototype.isPasteSingleValueIntoRange=function(e){return this.hasOnlyOneValueToPaste(e)&&this.rangeService!=null&&!this.rangeService.isEmpty()},t.prototype.pasteSingleValueIntoRange=function(e,r,n,o){var a=this,s=e[0][0],l=function(u,c,p){r.push(c),p.forEach(function(h){return a.updateCellValue(c,h,s,n,L.EXPORT_TYPE_CLIPBOARD,o)})};this.iterateActiveRanges(!1,l)},t.prototype.hasOnlyOneValueToPaste=function(e){return e.length===1&&e[0].length===1},t.prototype.copyRangeDown=function(){var e=this;if(!(!this.rangeService||this.rangeService.isEmpty())){var r=[],n=function(o,a,s,l){var u=e.gridOptionsWrapper.getProcessCellForClipboardFunc(),c=e.gridOptionsWrapper.getProcessCellFromClipboardFunc(),p=function(h,d,f){r.length?(a.push(d),f.forEach(function(g,v){if(!(!g.isCellEditable(d)||g.isSuppressPaste(d))){var m=e.processCell(d,g,r[v],L.EXPORT_TYPE_DRAG_COPY,c);d.setDataValue(g,m,L.SOURCE_PASTE),l&&l.addParentNode(d.parent,[g]);var C=e.cellPositionUtils.createIdFromValues(h.rowIndex,g,h.rowPinned);o[C]=!0}})):f.forEach(function(g){var v=e.processCell(d,g,e.valueService.getValue(g,d),L.EXPORT_TYPE_DRAG_COPY,u);r.push(v)})};e.iterateActiveRanges(!0,p)};this.doPasteOperation(n)}},t.prototype.removeLastLineIfBlank=function(e){var r=_.last(e),n=r&&r.length===1&&r[0]==="";n&&_.removeFromArray(e,r)},t.prototype.fireRowChanged=function(e){var r=this;!this.gridOptionsWrapper.isFullRowEdit()||e.forEach(function(n){var o={type:x.EVENT_ROW_VALUE_CHANGED,node:n,data:n.data,rowIndex:n.rowIndex,rowPinned:n.rowPinned,context:r.gridOptionsWrapper.getContext(),api:r.gridOptionsWrapper.getApi(),columnApi:r.gridOptionsWrapper.getColumnApi()};r.eventService.dispatchEvent(o)})},t.prototype.pasteMultipleValues=function(e,r,n,o,a,s,l){var u=this,c=r,p=this.clientSideRowModel!=null&&!this.gridOptionsWrapper.isTreeData(),h=function(){for(;;){if(!c)return null;var d=u.rowPositionUtils.getRowNode(c);if(c=u.cellNavigationService.getRowBelow({rowPinned:c.rowPinned,rowIndex:c.rowIndex}),d==null)return null;var f=d.detail||d.footer||p&&d.group;if(!f)return d}};e.forEach(function(d){var f=h();!f||(d.forEach(function(g,v){return u.updateCellValue(f,o[v],g,a,s,l)}),n.push(f))})},t.prototype.updateCellValue=function(e,r,n,o,a,s){if(!(!e||!r||!r.isCellEditable(e)||r.isSuppressPaste(e))){var l=this.processCell(e,r,n,a,this.gridOptionsWrapper.getProcessCellFromClipboardFunc());e.setDataValue(r,l,L.SOURCE_PASTE);var u=this.cellPositionUtils.createIdFromValues(e.rowIndex,r,e.rowPinned);o[u]=!0,s&&s.addParentNode(e.parent,[r])}},t.prototype.copyToClipboard=function(e){e===void 0&&(e={});var r=e.includeHeaders,n=e.includeGroupHeaders;this.logger.log("copyToClipboard: includeHeaders = "+r),r==null&&(r=this.gridOptionsWrapper.isCopyHeadersToClipboard()),n==null&&(n=this.gridOptionsWrapper.isCopyGroupHeadersToClipboard());var o={includeHeaders:r,includeGroupHeaders:n},a=!this.gridOptionsWrapper.isSuppressCopyRowsToClipboard();this.rangeService&&!this.rangeService.isEmpty()&&!this.shouldSkipSingleCellRange()?this.copySelectedRangeToClipboard(o):a&&!this.selectionService.isEmpty()?this.copySelectedRowsToClipboard(o):this.focusService.isAnyCellFocused()&&this.copyFocusedCellToClipboard(o)},t.prototype.shouldSkipSingleCellRange=function(){return this.gridOptionsWrapper.isSuppressCopySingleCellRanges()&&!this.rangeService.isMoreThanOneCell()},t.prototype.iterateActiveRanges=function(e,r,n){var o=this;if(!(!this.rangeService||this.rangeService.isEmpty())){var a=this.rangeService.getCellRanges();e?this.iterateActiveRange(a[0],r,n,!0):a.forEach(function(s,l){return o.iterateActiveRange(s,r,n,l===a.length-1)})}},t.prototype.iterateActiveRange=function(e,r,n,o){if(!!this.rangeService){var a=this.rangeService.getRangeStartRow(e),s=this.rangeService.getRangeEndRow(e);n&&e.columns&&n(e.columns);for(var l=0,u=!1;!u&&a!=null;){var c=this.rowPositionUtils.getRowNode(a);u=this.rowPositionUtils.sameRow(a,s),r(a,c,e.columns,l++,u&&o),a=this.cellNavigationService.getRowBelow(a)}}},t.prototype.copySelectedRangeToClipboard=function(e){if(e===void 0&&(e={}),!(!this.rangeService||this.rangeService.isEmpty())){var r=this.rangeService.areAllRangesAbleToMerge(),n=r?this.buildDataFromMergedRanges(e):this.buildDataFromRanges(e),o=n.data,a=n.cellsToFlash;this.copyDataToClipboard(o),this.dispatchFlashCells(a)}},t.prototype.buildDataFromMergedRanges=function(e){var r=this,n=new Set,o=this.rangeService.getCellRanges(),a=[],s={};o.forEach(function(p){p.columns.forEach(function(g){return n.add(g)});var h=r.getRangeRowPositionsAndCellsToFlash(p),d=h.rowPositions,f=h.cellsToFlash;a.push.apply(a,_4(d)),Object.assign(s,f)});var l=this.columnModel.getAllDisplayedColumns(),u=Array.from(n);u.sort(function(p,h){var d=l.indexOf(p),f=l.indexOf(h);return d-f});var c=this.buildExportParams({columns:u,rowPositions:a,includeHeaders:e.includeHeaders,includeGroupHeaders:e.includeGroupHeaders});return{data:c,cellsToFlash:s}},t.prototype.buildDataFromRanges=function(e){var r=this,n=this.rangeService.getCellRanges(),o=[],a={};return n.forEach(function(s){var l=r.getRangeRowPositionsAndCellsToFlash(s),u=l.rowPositions,c=l.cellsToFlash;Object.assign(a,c),o.push(r.buildExportParams({columns:s.columns,rowPositions:u,includeHeaders:e.includeHeaders,includeGroupHeaders:e.includeGroupHeaders}))}),{data:o.join(`
`),cellsToFlash:a}},t.prototype.getRangeRowPositionsAndCellsToFlash=function(e){for(var r=this,n=[],o={},a=this.rangeService.getRangeStartRow(e),s=this.rangeService.getRangeEndRow(e),l=a;l&&(n.push(l),e.columns.forEach(function(u){var c=r.cellPositionUtils.createIdFromValues(l.rowIndex,u,l.rowPinned);o[c]=!0}),!this.rowPositionUtils.sameRow(l,s));)l=this.cellNavigationService.getRowBelow(l);return{rowPositions:n,cellsToFlash:o}},t.prototype.copyFocusedCellToClipboard=function(e){var r;e===void 0&&(e={});var n=this.focusService.getFocusedCell();if(n!=null){var o=this.cellPositionUtils.createId(n),a={rowPinned:n.rowPinned,rowIndex:n.rowIndex},s=n.column,l=this.buildExportParams({columns:[s],rowPositions:[a],includeHeaders:e.includeHeaders,includeGroupHeaders:e.includeGroupHeaders});this.copyDataToClipboard(l),this.dispatchFlashCells((r={},r[o]=!0,r))}},t.prototype.copySelectedRowsToClipboard=function(e){e===void 0&&(e={});var r=e.columnKeys,n=e.includeHeaders,o=e.includeGroupHeaders,a=this.buildExportParams({columns:r,includeHeaders:n,includeGroupHeaders:o});this.copyDataToClipboard(a)},t.prototype.buildExportParams=function(e){var r=e.columns,n=e.rowPositions,o=e.includeHeaders,a=o===void 0?!1:o,s=e.includeGroupHeaders,l=s===void 0?!1:s,u={columnKeys:r,rowNodes:n,skipColumnHeaders:!a,skipColumnGroupHeaders:!l,suppressQuotes:!0,columnSeparator:this.gridOptionsWrapper.getClipboardDelimiter(),onlySelected:!n,processCellCallback:this.gridOptionsWrapper.getProcessCellForClipboardFunc(),processRowGroupCallback:function(c){return c.node.key},processHeaderCallback:this.gridOptionsWrapper.getProcessHeaderForClipboardFunc(),processGroupHeaderCallback:this.gridOptionsWrapper.getProcessGroupHeaderForClipboardFunc()};return this.csvCreator.getDataAsCsv(u)},t.prototype.dispatchFlashCells=function(e){var r=this;window.setTimeout(function(){var n={type:x.EVENT_FLASH_CELLS,cells:e,api:r.gridApi,columnApi:r.columnApi};r.eventService.dispatchEvent(n)},0)},t.prototype.processCell=function(e,r,n,o,a){if(a){var s={column:r,node:e,value:n,type:o};return a(s)}return n},t.prototype.copyDataToClipboard=function(e){var r=this,n=this.gridOptionsWrapper.getSendToClipboardFunc();if(n){n({data:e});return}var o=!this.gridOptionsWrapper.isSuppressClipboardApi();if(o&&navigator.clipboard){navigator.clipboard.writeText(e).catch(function(a){_.doOnce(function(){console.warn(a),console.warn("AG Grid: Unable to use the Clipboard API (navigator.clipboard.writeText()). The reason why it could not be used has been logged in the previous line. For this reason the grid has defaulted to using a workaround which doesn't perform as well. Either fix why Clipboard API is blocked, OR stop this message from appearing by setting grid property suppressClipboardApi=true (which will default the grid to using the workaround rather than the API.")},"clipboardApiError"),r.copyDataToClipboardLegacy(e)});return}this.copyDataToClipboardLegacy(e)},t.prototype.copyDataToClipboardLegacy=function(e){var r=this;this.executeOnTempElement(function(n){var o=r.gridOptionsWrapper.getDocument(),a=o.activeElement;n.value=e||" ",n.select(),n.focus({preventScroll:!0});var s=o.execCommand("copy");s||console.warn("AG Grid: Browser did not allow document.execCommand('copy'). Ensure api.copySelectedRowsToClipboard() is invoked via a user event, i.e. button click, otherwise the browser will prevent it for security reasons."),a!=null&&a.focus!=null&&a.focus({preventScroll:!0})})},t.prototype.executeOnTempElement=function(e,r){var n=this.gridOptionsWrapper.getDocument(),o=n.createElement("textarea");o.style.width="1px",o.style.height="1px",o.style.top=n.documentElement.scrollTop+"px",o.style.left=n.documentElement.scrollLeft+"px",o.style.position="absolute",o.style.opacity="0";var a=this.gridCtrl.getGui();a.appendChild(o);try{e(o)}catch{console.warn("AG Grid: Browser does not support document.execCommand('copy') for clipboard operations")}r?window.setTimeout(function(){r(o),a.removeChild(o)},100):a.removeChild(o)},t.prototype.getRangeSize=function(){var e=this.rangeService.getCellRanges(),r=0,n=0;return e.length>0&&(r=this.rangeService.getRangeStartRow(e[0]).rowIndex,n=this.rangeService.getRangeEndRow(e[0]).rowIndex),r-n+1},Jt([y("csvCreator")],t.prototype,"csvCreator",void 0),Jt([y("loggerFactory")],t.prototype,"loggerFactory",void 0),Jt([y("selectionService")],t.prototype,"selectionService",void 0),Jt([De("rangeService")],t.prototype,"rangeService",void 0),Jt([y("rowModel")],t.prototype,"rowModel",void 0),Jt([y("ctrlsService")],t.prototype,"ctrlsService",void 0),Jt([y("valueService")],t.prototype,"valueService",void 0),Jt([y("focusService")],t.prototype,"focusService",void 0),Jt([y("rowRenderer")],t.prototype,"rowRenderer",void 0),Jt([y("columnModel")],t.prototype,"columnModel",void 0),Jt([y("cellNavigationService")],t.prototype,"cellNavigationService",void 0),Jt([y("columnApi")],t.prototype,"columnApi",void 0),Jt([y("gridApi")],t.prototype,"gridApi",void 0),Jt([y("cellPositionUtils")],t.prototype,"cellPositionUtils",void 0),Jt([y("rowPositionUtils")],t.prototype,"rowPositionUtils",void 0),Jt([B],t.prototype,"init",null),t=Jt([Q("clipboardService")],t),t}(H),x4={moduleName:oe.ClipboardModule,beans:[S4],dependantModules:[cr,$w]},T4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ru=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Si;(function(i){i[i.EXPANDED=0]="EXPANDED",i[i.COLLAPSED=1]="COLLAPSED",i[i.INDETERMINATE=2]="INDETERMINATE"})(Si||(Si={}));var O4=function(i){T4(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.postConstruct=function(){var e=this;this.createExpandIcons(),this.addManagedListener(this.eExpand,"click",this.onExpandClicked.bind(this)),this.addManagedListener(this.eExpand,"keydown",function(n){n.key===j.SPACE&&e.onExpandClicked()}),this.addManagedListener(this.eSelect.getInputElement(),"click",this.onSelectClicked.bind(this)),this.eFilterTextField.onValueChange(function(){return e.onFilterTextChanged()}),this.addManagedListener(this.eFilterTextField.getInputElement(),"keypress",this.onMiniFilterKeyPress.bind(this)),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.showOrHideOptions.bind(this));var r=this.gridOptionsWrapper.getLocaleTextFunc();this.eSelect.setInputAriaLabel(r("ariaColumnSelectAll","Toggle Select All Columns")),this.eFilterTextField.setInputAriaLabel(r("ariaFilterColumnsInput","Filter Columns Input"))},t.prototype.init=function(e){this.params=e,this.columnModel.isReady()&&this.showOrHideOptions()},t.prototype.createExpandIcons=function(){this.eExpand.appendChild(this.eExpandChecked=_.createIconNoSpan("columnSelectOpen",this.gridOptionsWrapper)),this.eExpand.appendChild(this.eExpandUnchecked=_.createIconNoSpan("columnSelectClosed",this.gridOptionsWrapper)),this.eExpand.appendChild(this.eExpandIndeterminate=_.createIconNoSpan("columnSelectIndeterminate",this.gridOptionsWrapper)),this.setExpandState(Si.EXPANDED)},t.prototype.showOrHideOptions=function(){var e=!this.params.suppressColumnFilter,r=!this.params.suppressColumnSelectAll,n=!this.params.suppressColumnExpandAll,o=this.columnModel.isPrimaryColumnGroupsPresent(),a=this.gridOptionsWrapper.getLocaleTextFunc();this.eFilterTextField.setInputPlaceholder(a("searchOoo","Search...")),_.setDisplayed(this.eFilterTextField.getGui(),e),_.setDisplayed(this.eSelect.getGui(),r),_.setDisplayed(this.eExpand,n&&o)},t.prototype.onFilterTextChanged=function(){var e=this;this.onFilterTextChangedDebounced||(this.onFilterTextChangedDebounced=_.debounce(function(){var r=e.eFilterTextField.getValue();e.dispatchEvent({type:"filterChanged",filterText:r})},t.DEBOUNCE_DELAY)),this.onFilterTextChangedDebounced()},t.prototype.onMiniFilterKeyPress=function(e){var r=this;e.key===j.ENTER&&setTimeout(function(){return r.onSelectClicked()},t.DEBOUNCE_DELAY)},t.prototype.onSelectClicked=function(){this.dispatchEvent({type:this.selectState?"unselectAll":"selectAll"})},t.prototype.onExpandClicked=function(){this.dispatchEvent({type:this.expandState===Si.EXPANDED?"collapseAll":"expandAll"})},t.prototype.setExpandState=function(e){this.expandState=e,_.setDisplayed(this.eExpandChecked,this.expandState===Si.EXPANDED),_.setDisplayed(this.eExpandUnchecked,this.expandState===Si.COLLAPSED),_.setDisplayed(this.eExpandIndeterminate,this.expandState===Si.INDETERMINATE)},t.prototype.setSelectionState=function(e){this.selectState=e,this.eSelect.setValue(this.selectState)},t.DEBOUNCE_DELAY=300,t.TEMPLATE=`<div class="ag-column-select-header" role="presentation" tabindex="-1">
            <div ref="eExpand" class="ag-column-select-header-icon" tabindex="0"></div>
            <ag-checkbox ref="eSelect" class="ag-column-select-header-checkbox"></ag-checkbox>
            <ag-input-text-field class="ag-column-select-header-filter-wrapper" ref="eFilterTextField"></ag-input-text-field>
        </div>`,Ru([y("columnModel")],t.prototype,"columnModel",void 0),Ru([I("eExpand")],t.prototype,"eExpand",void 0),Ru([I("eSelect")],t.prototype,"eSelect",void 0),Ru([I("eFilterTextField")],t.prototype,"eFilterTextField",void 0),Ru([B],t.prototype,"postConstruct",null),t}(ee),Qu=function(){function i(t,e,r,n,o){n===void 0&&(n=!1),this.eventService=new oi,this.displayName=t,this.dept=r,this.group=n,n?(this.columnGroup=e,this.expanded=o,this.children=[]):this.column=e}return i.prototype.isGroup=function(){return this.group},i.prototype.getDisplayName=function(){return this.displayName},i.prototype.getColumnGroup=function(){return this.columnGroup},i.prototype.getColumn=function(){return this.column},i.prototype.getDept=function(){return this.dept},i.prototype.isExpanded=function(){return!!this.expanded},i.prototype.getChildren=function(){return this.children},i.prototype.isPassesFilter=function(){return this.passesFilter},i.prototype.setExpanded=function(t){t!==this.expanded&&(this.expanded=t,this.eventService.dispatchEvent({type:i.EVENT_EXPANDED_CHANGED}))},i.prototype.setPassesFilter=function(t){this.passesFilter=t},i.prototype.addEventListener=function(t,e){this.eventService.addEventListener(t,e)},i.prototype.removeEventListener=function(t,e){this.eventService.removeEventListener(t,e)},i.EVENT_EXPANDED_CHANGED="expandedChanged",i}(),E4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Kp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},R4=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},P4=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(R4(arguments[t]));return i},A4=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},$P=function(i){E4(t,i);function t(e,r,n){var o=i.call(this,'<div class="ag-menu"></div>')||this;return o.column=e,o.mouseEvent=r,o.parentEl=n,o.displayName=null,o}return t.prototype.postConstruct=function(){this.initializeProperties(this.column),this.buildMenuItemMap(),this.column instanceof Se?this.displayName=this.columnModel.getDisplayNameForColumn(this.column,"columnToolPanel"):this.displayName=this.columnModel.getDisplayNameForProvidedColumnGroup(null,this.column,"columnToolPanel"),this.isActive()&&(this.mouseEvent.preventDefault(),this.displayContextMenu())},t.prototype.initializeProperties=function(e){e instanceof $e?this.columns=e.getLeafColumns():this.columns=[e],this.allowGrouping=this.columns.some(function(r){return r.isPrimary()&&r.isAllowRowGroup()}),this.allowValues=this.columns.some(function(r){return r.isPrimary()&&r.isAllowValue()}),this.allowPivoting=this.columnModel.isPivotMode()&&this.columns.some(function(r){return r.isPrimary()&&r.isAllowPivot()})},t.prototype.buildMenuItemMap=function(){var e=this,r=this.gridOptionsWrapper.getLocaleTextFunc();this.menuItemMap=new Map,this.menuItemMap.set("rowGroup",{allowedFunction:function(n){return n.isPrimary()&&n.isAllowRowGroup()},activeFunction:function(n){return n.isRowGroupActive()},activateLabel:function(){return r("groupBy","Group by")+" "+e.displayName},deactivateLabel:function(){return r("ungroupBy","Un-Group by")+" "+e.displayName},activateFunction:function(){var n=e.columnModel.getRowGroupColumns();e.columnModel.setRowGroupColumns(e.addColumnsToList(n),"toolPanelUi")},deActivateFunction:function(){var n=e.columnModel.getRowGroupColumns();e.columnModel.setRowGroupColumns(e.removeColumnsFromList(n),"toolPanelUi")},addIcon:"menuAddRowGroup",removeIcon:"menuRemoveRowGroup"}),this.menuItemMap.set("value",{allowedFunction:function(n){return n.isPrimary()&&n.isAllowValue()},activeFunction:function(n){return n.isValueActive()},activateLabel:function(){return r("addToValues","Add "+e.displayName+" to values",[e.displayName])},deactivateLabel:function(){return r("removeFromValues","Remove "+e.displayName+" from values",[e.displayName])},activateFunction:function(){var n=e.columnModel.getValueColumns();e.columnModel.setValueColumns(e.addColumnsToList(n),"toolPanelUi")},deActivateFunction:function(){var n=e.columnModel.getValueColumns();e.columnModel.setValueColumns(e.removeColumnsFromList(n),"toolPanelUi")},addIcon:"valuePanel",removeIcon:"valuePanel"}),this.menuItemMap.set("pivot",{allowedFunction:function(n){return e.columnModel.isPivotMode()&&n.isPrimary()&&n.isAllowPivot()},activeFunction:function(n){return n.isPivotActive()},activateLabel:function(){return r("addToLabels","Add "+e.displayName+" to labels",[e.displayName])},deactivateLabel:function(){return r("removeFromLabels","Remove "+e.displayName+" from labels",[e.displayName])},activateFunction:function(){var n=e.columnModel.getPivotColumns();e.columnModel.setPivotColumns(e.addColumnsToList(n),"toolPanelUi")},deActivateFunction:function(){var n=e.columnModel.getPivotColumns();e.columnModel.setPivotColumns(e.removeColumnsFromList(n),"toolPanelUi")},addIcon:"pivotPanel",removeIcon:"pivotPanel"})},t.prototype.addColumnsToList=function(e){return P4(e).concat(this.columns.filter(function(r){return e.indexOf(r)===-1}))},t.prototype.removeColumnsFromList=function(e){var r=this;return e.filter(function(n){return r.columns.indexOf(n)===-1})},t.prototype.displayContextMenu=function(){var e=this,r=this.getGui(),n=this.createBean(new ep),o=this.getMappedMenuItems(),a=function(){};r.appendChild(n.getGui()),n.addMenuItems(o),n.addManagedListener(n,Dn.EVENT_MENU_ITEM_SELECTED,function(){e.parentEl.focus(),a()});var s=this.popupService.addPopup({modal:!0,eChild:r,closeOnEsc:!0,afterGuiAttached:function(){return e.focusService.focusInto(n.getGui())},ariaLabel:"Foo",closedCallback:function(l){l instanceof KeyboardEvent&&e.parentEl.focus(),e.destroyBean(n)}});s&&(a=s.hideFunc),this.popupService.positionPopupUnderMouseEvent({type:"columnContextMenu",mouseEvent:this.mouseEvent,ePopup:r})},t.prototype.isActive=function(){return this.allowGrouping||this.allowValues||this.allowPivoting},t.prototype.getMappedMenuItems=function(){var e,r,n=[],o=function(c){var p=a.columns.some(function(d){return c.allowedFunction(d)&&!c.activeFunction(d)}),h=a.columns.some(function(d){return c.allowedFunction(d)&&c.activeFunction(d)});p&&n.push({name:c.activateLabel(a.displayName),icon:_.createIconNoSpan(c.addIcon,a.gridOptionsWrapper,null),action:function(){return c.activateFunction()}}),h&&n.push({name:c.deactivateLabel(a.displayName),icon:_.createIconNoSpan(c.removeIcon,a.gridOptionsWrapper,null),action:function(){return c.deActivateFunction()}})},a=this;try{for(var s=A4(this.menuItemMap.values()),l=s.next();!l.done;l=s.next()){var u=l.value;o(u)}}catch(c){e={error:c}}finally{try{l&&!l.done&&(r=s.return)&&r.call(s)}finally{if(e)throw e.error}}return n},Kp([y("columnModel")],t.prototype,"columnModel",void 0),Kp([y("popupService")],t.prototype,"popupService",void 0),Kp([y("focusService")],t.prototype,"focusService",void 0),Kp([B],t.prototype,"postConstruct",null),t}(ee),D4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Vi=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},KP=function(i){D4(t,i);function t(e,r,n,o){var a=i.call(this)||this;return a.modelItem=e,a.allowDragging=r,a.eventType=n,a.focusWrapper=o,a.processingColumnStateChange=!1,a.modelItem=e,a.columnGroup=e.getColumnGroup(),a.columnDept=e.getDept(),a.allowDragging=r,a}return t.prototype.init=function(){var e=this;this.setTemplate(t.TEMPLATE),this.eDragHandle=_.createIconNoSpan("columnDrag",this.gridOptionsWrapper),this.eDragHandle.classList.add("ag-drag-handle","ag-column-select-column-group-drag-handle");var r=this.cbSelect.getGui(),n=this.cbSelect.getInputElement();r.insertAdjacentElement("afterend",this.eDragHandle),n.setAttribute("tabindex","-1"),this.displayName=this.columnModel.getDisplayNameForProvidedColumnGroup(null,this.columnGroup,this.eventType),_.missing(this.displayName)&&(this.displayName=">>"),this.eLabel.innerHTML=this.displayName?this.displayName:"",this.setupExpandContract(),this.addCssClass("ag-column-select-indent-"+this.columnDept),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,this.onColumnStateChanged.bind(this)),this.addManagedListener(this.eLabel,"click",this.onLabelClicked.bind(this)),this.addManagedListener(this.cbSelect,ii.EVENT_CHANGED,this.onCheckboxChanged.bind(this)),this.addManagedListener(this.modelItem,Qu.EVENT_EXPANDED_CHANGED,this.onExpandChanged.bind(this)),this.addManagedListener(this.focusWrapper,"keydown",this.handleKeyDown.bind(this)),this.addManagedListener(this.focusWrapper,"contextmenu",this.onContextMenu.bind(this)),this.setOpenClosedIcons(),this.setupDragging(),this.onColumnStateChanged(),this.addVisibilityListenersToAllChildren(),this.refreshAriaExpanded(),this.refreshAriaLabel(),this.setupTooltip();var o=qc.getToolPanelClassesFromColDef(this.columnGroup.getColGroupDef(),this.gridOptionsWrapper,null,this.columnGroup);o.forEach(function(a){return e.addOrRemoveCssClass(a,!0)})},t.prototype.getColumns=function(){return this.columnGroup.getLeafColumns()},t.prototype.setupTooltip=function(){var e=this,r=this.columnGroup.getColGroupDef();if(!!r){var n=function(){var o=r.headerTooltip;e.setTooltip(o)};n(),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,n)}},t.prototype.getTooltipParams=function(){var e=i.prototype.getTooltipParams.call(this);return e.location="columnToolPanelColumnGroup",e},t.prototype.handleKeyDown=function(e){switch(e.key){case j.LEFT:e.preventDefault(),this.modelItem.setExpanded(!1);break;case j.RIGHT:e.preventDefault(),this.modelItem.setExpanded(!0);break;case j.SPACE:e.preventDefault(),this.isSelectable()&&this.onSelectAllChanged(!this.isSelected());break}},t.prototype.onContextMenu=function(e){var r=this,n=this,o=n.columnGroup,a=n.gridOptionsWrapper;if(!a.isFunctionsReadOnly()){var s=this.createBean(new $P(o,e,this.focusWrapper));this.addDestroyFunc(function(){s.isAlive()&&r.destroyBean(s)})}},t.prototype.addVisibilityListenersToAllChildren=function(){var e=this;this.columnGroup.getLeafColumns().forEach(function(r){e.addManagedListener(r,Se.EVENT_VISIBLE_CHANGED,e.onColumnStateChanged.bind(e)),e.addManagedListener(r,Se.EVENT_VALUE_CHANGED,e.onColumnStateChanged.bind(e)),e.addManagedListener(r,Se.EVENT_PIVOT_CHANGED,e.onColumnStateChanged.bind(e)),e.addManagedListener(r,Se.EVENT_ROW_GROUP_CHANGED,e.onColumnStateChanged.bind(e))})},t.prototype.setupDragging=function(){var e=this;if(!this.allowDragging){_.setDisplayed(this.eDragHandle,!1);return}var r={type:At.ToolPanel,eElement:this.eDragHandle,dragItemName:this.displayName,getDragItem:function(){return e.createDragItem()},onDragStarted:function(){var n={type:x.EVENT_COLUMN_PANEL_ITEM_DRAG_START,column:e.columnGroup};e.eventService.dispatchEvent(n)},onDragStopped:function(){var n={type:x.EVENT_COLUMN_PANEL_ITEM_DRAG_END};e.eventService.dispatchEvent(n)}};this.dragAndDropService.addDragSource(r,!0),this.addDestroyFunc(function(){return e.dragAndDropService.removeDragSource(r)})},t.prototype.createDragItem=function(){var e={};return this.columnGroup.getLeafColumns().forEach(function(r){e[r.getId()]=r.isVisible()}),{columns:this.columnGroup.getLeafColumns(),visibleState:e}},t.prototype.setupExpandContract=function(){this.eGroupClosedIcon.appendChild(_.createIcon("columnSelectClosed",this.gridOptionsWrapper,null)),this.eGroupOpenedIcon.appendChild(_.createIcon("columnSelectOpen",this.gridOptionsWrapper,null)),this.addManagedListener(this.eGroupClosedIcon,"click",this.onExpandOrContractClicked.bind(this)),this.addManagedListener(this.eGroupOpenedIcon,"click",this.onExpandOrContractClicked.bind(this));var e=new Cr(this.eColumnGroupIcons,!0);this.addManagedListener(e,Cr.EVENT_TAP,this.onExpandOrContractClicked.bind(this)),this.addDestroyFunc(e.destroy.bind(e))},t.prototype.onLabelClicked=function(){var e=!this.cbSelect.getValue();this.onChangeCommon(e)},t.prototype.onCheckboxChanged=function(e){this.onChangeCommon(e.selected)},t.prototype.getVisibleLeafColumns=function(){var e=[],r=function(n){n.forEach(function(o){!o.isPassesFilter()||(o.isGroup()?r(o.getChildren()):e.push(o.getColumn()))})};return r(this.modelItem.getChildren()),e},t.prototype.onChangeCommon=function(e){this.refreshAriaLabel(),!this.processingColumnStateChange&&this.modelItemUtils.selectAllChildren(this.modelItem.getChildren(),e,this.eventType)},t.prototype.refreshAriaLabel=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("ariaColumnGroup","Column Group"),n=this.cbSelect.getValue()?e("ariaVisible","visible"):e("ariaHidden","hidden"),o=e("ariaToggleVisibility","Press SPACE to toggle visibility");_.setAriaLabel(this.focusWrapper,this.displayName+" "+r),this.cbSelect.setInputAriaLabel(o+" ("+n+")"),_.setAriaDescribedBy(this.focusWrapper,this.cbSelect.getInputElement().id)},t.prototype.onColumnStateChanged=function(){var e=this.workOutSelectedValue(),r=this.workOutReadOnlyValue();this.processingColumnStateChange=!0,this.cbSelect.setValue(e),this.cbSelect.setReadOnly(r),this.addOrRemoveCssClass("ag-column-select-column-group-readonly",r),this.processingColumnStateChange=!1},t.prototype.workOutSelectedValue=function(){var e=this,r=this.columnModel.isPivotMode(),n=this.getVisibleLeafColumns(),o=0,a=0;if(n.forEach(function(s){!r&&s.getColDef().lockVisible||(e.isColumnChecked(s,r)?o++:a++)}),!(o>0&&a>0))return o>0},t.prototype.workOutReadOnlyValue=function(){var e=this.columnModel.isPivotMode(),r=0;return this.columnGroup.getLeafColumns().forEach(function(n){e?n.isAnyFunctionAllowed()&&r++:n.getColDef().lockVisible||r++}),r===0},t.prototype.isColumnChecked=function(e,r){if(r){var n=e.isPivotActive(),o=e.isRowGroupActive(),a=e.isValueActive();return n||o||a}return e.isVisible()},t.prototype.onExpandOrContractClicked=function(){var e=this.modelItem.isExpanded();this.modelItem.setExpanded(!e)},t.prototype.onExpandChanged=function(){this.setOpenClosedIcons(),this.refreshAriaExpanded()},t.prototype.setOpenClosedIcons=function(){var e=this.modelItem.isExpanded();_.setDisplayed(this.eGroupClosedIcon,!e),_.setDisplayed(this.eGroupOpenedIcon,e)},t.prototype.refreshAriaExpanded=function(){_.setAriaExpanded(this.focusWrapper,this.modelItem.isExpanded())},t.prototype.getDisplayName=function(){return this.displayName},t.prototype.onSelectAllChanged=function(e){var r=this.cbSelect.getValue(),n=this.cbSelect.isReadOnly();!n&&(e&&!r||!e&&r)&&this.cbSelect.toggle()},t.prototype.isSelected=function(){return this.cbSelect.getValue()},t.prototype.isSelectable=function(){return!this.cbSelect.isReadOnly()},t.prototype.setSelected=function(e){this.cbSelect.setValue(e,!0)},t.TEMPLATE=`<div class="ag-column-select-column-group" aria-hidden="true">
            <span class="ag-column-group-icons" ref="eColumnGroupIcons" >
                <span class="ag-column-group-closed-icon" ref="eGroupClosedIcon"></span>
                <span class="ag-column-group-opened-icon" ref="eGroupOpenedIcon"></span>
            </span>
            <ag-checkbox ref="cbSelect" class="ag-column-select-checkbox"></ag-checkbox>
            <span class="ag-column-select-column-label" ref="eLabel"></span>
        </div>`,Vi([y("columnModel")],t.prototype,"columnModel",void 0),Vi([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),Vi([y("modelItemUtils")],t.prototype,"modelItemUtils",void 0),Vi([I("cbSelect")],t.prototype,"cbSelect",void 0),Vi([I("eLabel")],t.prototype,"eLabel",void 0),Vi([I("eGroupOpenedIcon")],t.prototype,"eGroupOpenedIcon",void 0),Vi([I("eGroupClosedIcon")],t.prototype,"eGroupClosedIcon",void 0),Vi([I("eColumnGroupIcons")],t.prototype,"eColumnGroupIcons",void 0),Vi([B],t.prototype,"init",null),t}(ee),M4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),gv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},vv="ag-list-item-hovered",F4=function(i){M4(t,i);function t(e,r){var n=i.call(this)||this;return n.comp=e,n.virtualList=r,n.currentDragColumn=null,n.lastHoveredColumnItem=null,n}return t.prototype.postConstruct=function(){this.addManagedListener(this.eventService,x.EVENT_COLUMN_PANEL_ITEM_DRAG_START,this.columnPanelItemDragStart.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PANEL_ITEM_DRAG_END,this.columnPanelItemDragEnd.bind(this)),this.createDropTarget(),this.createAutoScrollService()},t.prototype.columnPanelItemDragStart=function(e){var r=e.column;this.currentDragColumn=r;var n=this.getCurrentColumns(),o=n.find(function(a){var s=a.getColDef();return!!s.suppressMovable||!!s.lockPosition});o&&(this.moveBlocked=!0)},t.prototype.columnPanelItemDragEnd=function(){var e=this;window.setTimeout(function(){e.currentDragColumn=null,e.moveBlocked=!1},10)},t.prototype.createDropTarget=function(){var e=this,r={isInterestedIn:function(n){return n===At.ToolPanel},getIconName:function(){return ht[e.moveBlocked?"ICON_NOT_ALLOWED":"ICON_MOVE"]},getContainer:function(){return e.comp.getGui()},onDragging:function(n){return e.onDragging(n)},onDragStop:function(){return e.onDragStop()},onDragLeave:function(){return e.onDragLeave()}};this.dragAndDropService.addDropTarget(r)},t.prototype.createAutoScrollService=function(){var e=this.virtualList.getGui();this.autoScrollService=new Xc({scrollContainer:e,scrollAxis:"y",getVerticalPosition:function(){return e.scrollTop},setVerticalPosition:function(r){return e.scrollTop=r}})},t.prototype.onDragging=function(e){if(!(!this.currentDragColumn||this.moveBlocked)){var r=this.getDragColumnItem(e),n=this.virtualList.getComponentAt(r.rowIndex);if(!!n){var o=n.getGui().parentElement;this.lastHoveredColumnItem&&this.lastHoveredColumnItem.rowIndex===r.rowIndex&&this.lastHoveredColumnItem.position===r.position||(this.autoScrollService.check(e.event),this.clearHoveredItems(),this.lastHoveredColumnItem=r,_.radioCssClass(o,""+vv),_.radioCssClass(o,"ag-item-highlight-"+r.position))}}},t.prototype.getDragColumnItem=function(e){var r=this.virtualList.getGui(),n=parseFloat(window.getComputedStyle(r).paddingTop),o=this.virtualList.getRowHeight(),a=this.virtualList.getScrollTop(),s=Math.max(0,(e.y-n+a)/o),l=this.comp.getDisplayedColsList().length-1,u=Math.min(l,s)|0;return{rowIndex:u,position:Math.round(s)>s||s>l?"bottom":"top",component:this.virtualList.getComponentAt(u)}},t.prototype.onDragStop=function(){if(!this.moveBlocked){var e=this.getTargetIndex(),r=this.getCurrentColumns();e!=null&&this.columnModel.moveColumns(r,e),this.clearHoveredItems(),this.autoScrollService.ensureCleared()}},t.prototype.getMoveDiff=function(e){var r=this.columnModel.getAllGridColumns(),n=this.getCurrentColumns(),o=n[0],a=n.length,s=r.indexOf(o);return s<e?a:0},t.prototype.getCurrentColumns=function(){return this.currentDragColumn instanceof $e?this.currentDragColumn.getLeafColumns():[this.currentDragColumn]},t.prototype.getTargetIndex=function(){if(!this.lastHoveredColumnItem)return null;var e=this.lastHoveredColumnItem.component,r=this.lastHoveredColumnItem.position==="top",n;if(e instanceof KP){var o=e.getColumns();n=o[0],r=!0}else n=e.getColumn();var a=this.columnModel.getAllGridColumns().indexOf(n),s=r?a:a+1,l=this.getMoveDiff(s);return s-l},t.prototype.onDragLeave=function(){this.clearHoveredItems(),this.autoScrollService.ensureCleared()},t.prototype.clearHoveredItems=function(){var e=this.virtualList.getGui();e.querySelectorAll("."+vv).forEach(function(r){[vv,"ag-item-highlight-top","ag-item-highlight-bottom"].forEach(function(n){r.classList.remove(n)})}),this.lastHoveredColumnItem=null},gv([y("columnModel")],t.prototype,"columnModel",void 0),gv([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),gv([B],t.prototype,"postConstruct",null),t}(H),I4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Hs=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},L4=function(i){I4(t,i);function t(e,r,n,o,a){var s=i.call(this)||this;return s.column=e,s.columnDept=r,s.allowDragging=n,s.groupsExist=o,s.focusWrapper=a,s.processingColumnStateChange=!1,s}return t.prototype.init=function(){var e=this;this.setTemplate(t.TEMPLATE),this.eDragHandle=_.createIconNoSpan("columnDrag",this.gridOptionsWrapper),this.eDragHandle.classList.add("ag-drag-handle","ag-column-select-column-drag-handle");var r=this.cbSelect.getGui(),n=this.cbSelect.getInputElement();r.insertAdjacentElement("afterend",this.eDragHandle),n.setAttribute("tabindex","-1"),this.displayName=this.columnModel.getDisplayNameForColumn(this.column,"columnToolPanel");var o=_.escapeString(this.displayName);this.eLabel.innerHTML=o;var a=this.columnDept;this.groupsExist&&this.addCssClass("ag-column-select-add-group-indent"),this.addCssClass("ag-column-select-indent-"+a),this.setupDragging(),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,this.onColumnStateChanged.bind(this)),this.addManagedListener(this.column,Se.EVENT_VALUE_CHANGED,this.onColumnStateChanged.bind(this)),this.addManagedListener(this.column,Se.EVENT_PIVOT_CHANGED,this.onColumnStateChanged.bind(this)),this.addManagedListener(this.column,Se.EVENT_ROW_GROUP_CHANGED,this.onColumnStateChanged.bind(this)),this.addManagedListener(this.column,Se.EVENT_VISIBLE_CHANGED,this.onColumnStateChanged.bind(this)),this.addManagedListener(this.focusWrapper,"keydown",this.handleKeyDown.bind(this)),this.addManagedListener(this.focusWrapper,"contextmenu",this.onContextMenu.bind(this)),this.addManagedListener(this.gridOptionsWrapper,"functionsReadOnly",this.onColumnStateChanged.bind(this)),this.addManagedListener(this.cbSelect,ii.EVENT_CHANGED,this.onCheckboxChanged.bind(this)),this.addManagedListener(this.eLabel,"click",this.onLabelClicked.bind(this)),this.onColumnStateChanged(),this.refreshAriaLabel(),this.setupTooltip();var s=qc.getToolPanelClassesFromColDef(this.column.getColDef(),this.gridOptionsWrapper,this.column,null);s.forEach(function(l){return e.addOrRemoveCssClass(l,!0)})},t.prototype.getColumn=function(){return this.column},t.prototype.setupTooltip=function(){var e=this,r=function(){var n=e.column.getColDef().headerTooltip;e.setTooltip(n)};r(),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,r)},t.prototype.getTooltipParams=function(){var e=i.prototype.getTooltipParams.call(this);return e.location="columnToolPanelColumn",e.colDef=this.column.getColDef(),e},t.prototype.onContextMenu=function(e){var r=this,n=this,o=n.column,a=n.gridOptionsWrapper;if(!a.isFunctionsReadOnly()){var s=this.createBean(new $P(o,e,this.focusWrapper));this.addDestroyFunc(function(){s.isAlive()&&r.destroyBean(s)})}},t.prototype.handleKeyDown=function(e){e.key===j.SPACE&&(e.preventDefault(),this.isSelectable()&&this.onSelectAllChanged(!this.isSelected()))},t.prototype.onLabelClicked=function(){if(!this.gridOptionsWrapper.isFunctionsReadOnly()){var e=!this.cbSelect.getValue();this.onChangeCommon(e)}},t.prototype.onCheckboxChanged=function(e){this.onChangeCommon(e.selected)},t.prototype.onChangeCommon=function(e){this.cbSelect.isReadOnly()||(this.refreshAriaLabel(),!this.processingColumnStateChange&&this.modelItemUtils.setColumn(this.column,e,"toolPanelUi"))},t.prototype.refreshAriaLabel=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("ariaColumn","Column"),n=this.cbSelect.getValue()?e("ariaVisible","visible"):e("ariaHidden","hidden"),o=e("ariaToggleVisibility","Press SPACE to toggle visibility");_.setAriaLabel(this.focusWrapper,this.displayName+" "+r),this.cbSelect.setInputAriaLabel(o+" ("+n+")"),_.setAriaDescribedBy(this.focusWrapper,this.cbSelect.getInputElement().id)},t.prototype.setupDragging=function(){var e=this;if(!this.allowDragging){_.setDisplayed(this.eDragHandle,!1);return}var r={type:At.ToolPanel,eElement:this.eDragHandle,dragItemName:this.displayName,getDragItem:function(){return e.createDragItem()},onDragStarted:function(){var n={type:x.EVENT_COLUMN_PANEL_ITEM_DRAG_START,column:e.column};e.eventService.dispatchEvent(n)},onDragStopped:function(){var n={type:x.EVENT_COLUMN_PANEL_ITEM_DRAG_END};e.eventService.dispatchEvent(n)}};this.dragAndDropService.addDragSource(r,!0),this.addDestroyFunc(function(){return e.dragAndDropService.removeDragSource(r)})},t.prototype.createDragItem=function(){var e={};return e[this.column.getId()]=this.column.isVisible(),{columns:[this.column],visibleState:e}},t.prototype.onColumnStateChanged=function(){this.processingColumnStateChange=!0;var e=this.columnModel.isPivotMode();if(e){var r=this.column.isAnyFunctionActive();this.cbSelect.setValue(r)}else this.cbSelect.setValue(this.column.isVisible());var n=!0,o=!0;if(e){var a=this.gridOptionsWrapper.isFunctionsReadOnly(),s=!this.column.isAnyFunctionAllowed();n=!a&&!s,o=n}else{var l=this.column.getColDef(),u=l.enableRowGroup,c=l.enableValue,p=l.lockPosition,h=l.suppressMovable,d=l.lockVisible,f=!!u||!!c,g=!!p||!!h;n=!d,o=f||!g}this.cbSelect.setReadOnly(!n),this.eDragHandle.classList.toggle("ag-column-select-column-readonly",!o),this.addOrRemoveCssClass("ag-column-select-column-readonly",!o&&!n);var v=e&&this.gridOptionsWrapper.isFunctionsPassive();this.cbSelect.setPassive(v),this.processingColumnStateChange=!1},t.prototype.getDisplayName=function(){return this.displayName},t.prototype.onSelectAllChanged=function(e){e!==this.cbSelect.getValue()&&(this.cbSelect.isReadOnly()||this.cbSelect.toggle())},t.prototype.isSelected=function(){return this.cbSelect.getValue()},t.prototype.isSelectable=function(){return!this.cbSelect.isReadOnly()},t.prototype.isExpandable=function(){return!1},t.prototype.setExpanded=function(e){console.warn("AG Grid: can not expand a column item that does not represent a column group header")},t.TEMPLATE=`<div class="ag-column-select-column" aria-hidden="true">
            <ag-checkbox ref="cbSelect" class="ag-column-select-checkbox"></ag-checkbox>
            <span class="ag-column-select-column-label" ref="eLabel"></span>
        </div>`,Hs([y("columnModel")],t.prototype,"columnModel",void 0),Hs([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),Hs([y("modelItemUtils")],t.prototype,"modelItemUtils",void 0),Hs([I("eLabel")],t.prototype,"eLabel",void 0),Hs([I("cbSelect")],t.prototype,"cbSelect",void 0),Hs([B],t.prototype,"init",null),t}(ee),N4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Yp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},G4=function(){function i(t){this.items=t}return i.prototype.getRowCount=function(){return this.items.length},i.prototype.getRow=function(t){return this.items[t]},i}(),k4="ag-column-select-list",W4=function(i){N4(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.destroyColumnItemFuncs=[],e}return t.prototype.destroyColumnTree=function(){this.allColsTree=[],this.destroyColumnItemFuncs.forEach(function(e){return e()}),this.destroyColumnItemFuncs=[]},t.prototype.init=function(e,r,n){var o=this;this.params=e,this.allowDragging=r,this.eventType=n,this.params.suppressSyncLayoutWithGrid||this.addManagedListener(this.eventService,x.EVENT_COLUMN_MOVED,this.onColumnsChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.onColumnsChanged.bind(this));var a=[x.EVENT_COLUMN_PIVOT_CHANGED,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,x.EVENT_COLUMN_ROW_GROUP_CHANGED,x.EVENT_COLUMN_VALUE_CHANGED,x.EVENT_COLUMN_VISIBLE,x.EVENT_NEW_COLUMNS_LOADED];a.forEach(function(u){o.addManagedListener(o.eventService,u,o.fireSelectionChangedEvent.bind(o))}),this.expandGroupsByDefault=!this.params.contractColumnSelection;var s=this.gridOptionsWrapper.getLocaleTextFunc(),l=s("ariaColumnList","Column List");this.virtualList=this.createManagedBean(new tp("column-select","tree",l)),this.appendChild(this.virtualList.getGui()),this.virtualList.setComponentCreator(function(u,c){return _.setAriaLevel(c,u.getDept()+1),o.createComponentFromItem(u,c)}),this.columnModel.isReady()&&this.onColumnsChanged(),!e.suppressColumnMove&&!this.gridOptionsWrapper.isSuppressMovableColumns()&&this.createManagedBean(new F4(this,this.virtualList))},t.prototype.createComponentFromItem=function(e,r){if(e.isGroup()){var n=new KP(e,this.allowDragging,this.eventType,r);return this.getContext().createBean(n),n}var o=new L4(e.getColumn(),e.getDept(),this.allowDragging,this.groupsExist,r);return this.getContext().createBean(o),o},t.prototype.onColumnsChanged=function(){var e=this.getExpandedStates(),r=this.columnModel.isPivotMode(),n=!this.params.suppressSyncLayoutWithGrid&&!r;n?this.buildTreeFromWhatGridIsDisplaying():this.buildTreeFromProvidedColumnDefs(),this.setExpandedStates(e),this.markFilteredColumns(),this.flattenAndFilterModel()},t.prototype.getDisplayedColsList=function(){return this.displayedColsList},t.prototype.getExpandedStates=function(){if(!this.allColsTree)return{};var e={};return this.forEachItem(function(r){if(!!r.isGroup()){var n=r.getColumnGroup();n&&(e[n.getId()]=r.isExpanded())}}),e},t.prototype.setExpandedStates=function(e){!this.allColsTree||this.forEachItem(function(r){if(!!r.isGroup()){var n=r.getColumnGroup();if(n){var o=e[n.getId()],a=o!=null;a&&r.setExpanded(o)}}})},t.prototype.buildTreeFromWhatGridIsDisplaying=function(){this.colDefService.syncLayoutWithGrid(this.setColumnLayout.bind(this))},t.prototype.setColumnLayout=function(e){var r=this.colDefService.createColumnTree(e);this.buildListModel(r),this.groupsExist=e.some(function(n){return n&&typeof n.children!="undefined"}),this.markFilteredColumns(),this.flattenAndFilterModel()},t.prototype.buildTreeFromProvidedColumnDefs=function(){this.buildListModel(this.columnModel.getPrimaryColumnTree()),this.groupsExist=this.columnModel.isPrimaryColumnGroupsPresent()},t.prototype.buildListModel=function(e){var r=this,n=this.onColumnExpanded.bind(this),o=function(u){u.addEventListener(Qu.EVENT_EXPANDED_CHANGED,n);var c=u.removeEventListener.bind(u,Qu.EVENT_EXPANDED_CHANGED,n);r.destroyColumnItemFuncs.push(c)},a=function(u,c,p){u.forEach(function(h){h instanceof $e?s(h,c,p):l(h,c,p)})},s=function(u,c,p){var h=u.getColGroupDef(),d=h&&h.suppressColumnsToolPanel;if(!d){if(u.isPadding()){a(u.getChildren(),c,p);return}var f=r.columnModel.getDisplayNameForProvidedColumnGroup(null,u,r.eventType),g=new Qu(f,u,c,!0,r.expandGroupsByDefault);p.push(g),o(g),a(u.getChildren(),c+1,g.getChildren())}},l=function(u,c,p){var h=u.getColDef()&&u.getColDef().suppressColumnsToolPanel;if(!h){var d=r.columnModel.getDisplayNameForColumn(u,"columnToolPanel");p.push(new Qu(d,u,c))}};this.destroyColumnTree(),a(e,0,this.allColsTree)},t.prototype.onColumnExpanded=function(){this.flattenAndFilterModel()},t.prototype.flattenAndFilterModel=function(){var e=this;this.displayedColsList=[];var r=function(o){!o.isPassesFilter()||(e.displayedColsList.push(o),o.isGroup()&&o.isExpanded()&&o.getChildren().forEach(r))};this.allColsTree.forEach(r),this.virtualList.setModel(new G4(this.displayedColsList));var n=this.virtualList.getLastFocusedRow();this.virtualList.refresh(),n!=null&&this.focusRowIfAlive(n),this.notifyListeners()},t.prototype.focusRowIfAlive=function(e){var r=this;window.setTimeout(function(){r.isAlive()&&r.virtualList.focusRow(e)},0)},t.prototype.forEachItem=function(e){var r=function(n){n.forEach(function(o){e(o),o.isGroup()&&r(o.getChildren())})};r(this.allColsTree)},t.prototype.doSetExpandedAll=function(e){this.forEachItem(function(r){r.isGroup()&&r.setExpanded(e)})},t.prototype.setGroupsExpanded=function(e,r){if(!r){this.doSetExpandedAll(e);return}var n=[];this.forEachItem(function(a){if(!!a.isGroup()){var s=a.getColumnGroup().getId();r.indexOf(s)>=0&&(a.setExpanded(e),n.push(s))}});var o=r.filter(function(a){return!_.includes(n,a)});o.length>0&&console.warn("AG Grid: unable to find group(s) for supplied groupIds:",o)},t.prototype.getExpandState=function(){var e=0,r=0;return this.forEachItem(function(n){!n.isGroup()||(n.isExpanded()?e++:r++)}),e>0&&r>0?Si.INDETERMINATE:r>0?Si.COLLAPSED:Si.EXPANDED},t.prototype.doSetSelectedAll=function(e){this.modelItemUtils.selectAllChildren(this.allColsTree,e,this.eventType)},t.prototype.getSelectionState=function(){var e=0,r=0,n=this.columnModel.isPivotMode();if(this.forEachItem(function(o){if(!o.isGroup()&&!!o.isPassesFilter()){var a=o.getColumn(),s=a.getColDef(),l;if(n){var u=!a.isAllowPivot()&&!a.isAllowRowGroup()&&!a.isAllowValue();if(u)return;l=a.isValueActive()||a.isPivotActive()||a.isRowGroupActive()}else{if(s.lockVisible)return;l=a.isVisible()}l?e++:r++}}),!(e>0&&r>0))return!(e===0||r>0)},t.prototype.setFilterText=function(e){this.filterText=_.exists(e)?e.toLowerCase():null,this.markFilteredColumns(),this.flattenAndFilterModel()},t.prototype.markFilteredColumns=function(){var e=this,r=function(o){if(!_.exists(e.filterText))return!0;var a=o.getDisplayName();return a==null||a.toLowerCase().indexOf(e.filterText)!==-1},n=function(o,a){var s=!1;if(o.isGroup()){var l=r(o);o.getChildren().forEach(function(c){var p=n(c,l||a);p&&(s=p)})}var u=a||s?!0:r(o);return o.setPassesFilter(u),u};this.allColsTree.forEach(function(o){return n(o,!1)})},t.prototype.notifyListeners=function(){this.fireGroupExpandedEvent(),this.fireSelectionChangedEvent()},t.prototype.fireGroupExpandedEvent=function(){var e=this.getExpandState();this.dispatchEvent({type:"groupExpanded",state:e})},t.prototype.fireSelectionChangedEvent=function(){var e=this.getSelectionState();this.dispatchEvent({type:"selectionChanged",state:e})},t.TEMPLATE='<div class="'+k4+'" role="presentation"></div>',Yp([y("columnModel")],t.prototype,"columnModel",void 0),Yp([y("toolPanelColDefService")],t.prototype,"colDefService",void 0),Yp([y("modelItemUtils")],t.prototype,"modelItemUtils",void 0),Yp([nt],t.prototype,"destroyColumnTree",null),t}(ee),V4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),mv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},rx=function(i){V4(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.createTemplate=function(){return`<div class="ag-pivot-mode-panel">
                <ag-toggle-button ref="cbPivotMode" class="ag-pivot-mode-select"></ag-toggle-button>
            </div>`},t.prototype.init=function(){this.setTemplate(this.createTemplate()),this.cbPivotMode.setValue(this.columnModel.isPivotMode());var e=this.gridOptionsWrapper.getLocaleTextFunc();this.cbPivotMode.setLabel(e("pivotMode","Pivot Mode")),this.addManagedListener(this.cbPivotMode,ii.EVENT_CHANGED,this.onBtPivotMode.bind(this)),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.onPivotModeChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,this.onPivotModeChanged.bind(this))},t.prototype.onBtPivotMode=function(){var e=!!this.cbPivotMode.getValue();if(e!==this.columnModel.isPivotMode()){this.columnModel.setPivotMode(e,"toolPanelUi");var r=this.gridOptionsWrapper.getApi();r&&r.refreshHeader()}},t.prototype.onPivotModeChanged=function(){var e=this.columnModel.isPivotMode();this.cbPivotMode.setValue(e)},mv([y("columnModel")],t.prototype,"columnModel",void 0),mv([I("cbPivotMode")],t.prototype,"cbPivotMode",void 0),mv([ss],t.prototype,"init",null),t}(ee),B4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ta=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},H4=function(i){B4(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.execute=function(e){var r=_.missingOrEmpty(this.columnModel.getValueColumns()),n=!this.gridOptionsWrapper.getGroupRowAggFunc(),o=e.changedPath&&e.changedPath.isActive();if(!(r&&n&&o)){var a=this.createAggDetails(e);this.recursivelyCreateAggData(a)}},t.prototype.createAggDetails=function(e){var r=this.columnModel.isPivotActive(),n=this.columnModel.getValueColumns(),o=r?this.columnModel.getPivotColumns():[],a={changedPath:e.changedPath,valueColumns:n,pivotColumns:o};return a},t.prototype.recursivelyCreateAggData=function(e){var r=this;this.filteredOnly=!this.gridOptionsWrapper.isSuppressAggFilteredOnly();var n=function(o){var a=!o.hasChildren();if(a){o.aggData&&o.setAggData(null);return}var s=o.level===-1;if(s){var l=!r.columnModel.isPivotMode(),u=r.gridOptionsWrapper.isSuppressAggAtRootLevel();if(u&&l)return}r.aggregateRowNode(o,e)};e.changedPath.forEachChangedNodeDepthFirst(n,!0)},t.prototype.aggregateRowNode=function(e,r){var n=r.valueColumns.length===0,o=r.pivotColumns.length===0,a=this.gridOptionsWrapper.getGroupRowAggFunc(),s;if(a){var l={nodes:e.childrenAfterFilter};s=a(l)}else n?s=null:o?s=this.aggregateRowNodeUsingValuesOnly(e,r):s=this.aggregateRowNodeUsingValuesAndPivot(e);e.setAggData(s),e.sibling&&e.sibling.setAggData(s)},t.prototype.aggregateRowNodeUsingValuesAndPivot=function(e){var r=this,n={},o=this.pivotStage.getPivotColumnDefs();return o.filter(function(a){return!_.exists(a.pivotTotalColumnIds)}).forEach(function(a){var s=a.pivotKeys||[],l,u=a.pivotValueColumn,c=a.colId;e.leafGroup?l=r.getValuesFromMappedSet(e.childrenMapped,s,u):l=r.getValuesPivotNonLeaf(e,c),n[c]=r.aggregateValues(l,u.getAggFunc(),u,e)}),o.filter(function(a){return _.exists(a.pivotTotalColumnIds)}).forEach(function(a){var s=[],l=a.pivotValueColumn,u=a.pivotTotalColumnIds,c=a.colId;!u||!u.length||(u.forEach(function(p){s.push(n[p])}),n[c]=r.aggregateValues(s,l.getAggFunc(),l,e))}),n},t.prototype.aggregateRowNodeUsingValuesOnly=function(e,r){var n=this,o={},a=r.changedPath.isActive()?r.changedPath.getValueColumnsForNode(e,r.valueColumns):r.valueColumns,s=r.changedPath.isActive()?r.changedPath.getNotValueColumnsForNode(e,r.valueColumns):null,l=this.getValuesNormal(e,a),u=e.aggData;return a.forEach(function(c,p){o[c.getId()]=n.aggregateValues(l[p],c.getAggFunc(),c,e)}),s&&u&&s.forEach(function(c){o[c.getId()]=u[c.getId()]}),o},t.prototype.getValuesPivotNonLeaf=function(e,r){var n=[];return e.childrenAfterFilter.forEach(function(o){var a=o.aggData[r];n.push(a)}),n},t.prototype.getValuesFromMappedSet=function(e,r,n){var o=this,a=e;if(r.forEach(function(l){return a=a?a[l]:null}),!a)return[];var s=[];return a.forEach(function(l){var u=o.valueService.getValue(n,l);s.push(u)}),s},t.prototype.getValuesNormal=function(e,r){var n=[];r.forEach(function(){return n.push([])});for(var o=r.length,a=this.filteredOnly?e.childrenAfterFilter:e.childrenAfterGroup,s=a.length,l=0;l<s;l++)for(var u=a[l],c=0;c<o;c++){var p=r[c],h=this.valueService.getValue(p,u);n[c].push(h)}return n},t.prototype.aggregateValues=function(e,r,n,o){var a=typeof r=="string"?this.aggFuncService.getAggFunc(r):r;if(typeof a!="function")return console.error("AG Grid: unrecognised aggregation function "+r),null;var s=function(){_.doOnce(function(){console.warn("AG Grid: since v24.0, custom aggregation functions take a params object. Please alter your aggregation function to use params.values")},"aggregationStage.aggregateValues Deprecation")},l=a,u={values:e,column:n,colDef:n?n.getColDef():void 0,rowNode:o,data:o?o.data:void 0,api:this.gridApi,columnApi:this.columnApi,context:this.gridOptionsWrapper.getContext(),forEach:function(c,p){return s(),e.forEach(c,p)},get length(){return s(),e.length},set length(c){s(),e.length=c}};return l(u)},Ta([y("columnModel")],t.prototype,"columnModel",void 0),Ta([y("valueService")],t.prototype,"valueService",void 0),Ta([y("pivotStage")],t.prototype,"pivotStage",void 0),Ta([y("aggFuncService")],t.prototype,"aggFuncService",void 0),Ta([y("gridApi")],t.prototype,"gridApi",void 0),Ta([y("columnApi")],t.prototype,"columnApi",void 0),t=Ta([Q("aggregationStage")],t),t}(H),yv=function(){function i(){this.allSets={},this.allParents=[]}return i.prototype.removeFromChildrenAfterGroup=function(t,e){var r=this.getSet(t);r.removeFromChildrenAfterGroup[e.id]=!0},i.prototype.removeFromAllLeafChildren=function(t,e){var r=this.getSet(t);r.removeFromAllLeafChildren[e.id]=!0},i.prototype.getSet=function(t){return this.allSets[t.id]||(this.allSets[t.id]={removeFromAllLeafChildren:{},removeFromChildrenAfterGroup:{}},this.allParents.push(t)),this.allSets[t.id]},i.prototype.getAllParents=function(){return this.allParents},i.prototype.flush=function(){var t=this;this.allParents.forEach(function(e){var r=t.allSets[e.id];e.childrenAfterGroup=e.childrenAfterGroup.filter(function(n){return!r.removeFromChildrenAfterGroup[n.id]}),e.allLeafChildren=e.allLeafChildren.filter(function(n){return!r.removeFromAllLeafChildren[n.id]}),e.updateHasChildren(),e.sibling&&(e.sibling.childrenAfterGroup=e.childrenAfterGroup,e.sibling.allLeafChildren=e.allLeafChildren)}),this.allSets={},this.allParents.length=0},i}(),j4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Oa=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},U4=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},z4=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(U4(arguments[t]));return i},$4=function(i){j4(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.groupIdSequence=new Bt,e}return t.prototype.postConstruct=function(){this.usingTreeData=this.gridOptionsWrapper.isTreeData(),this.usingTreeData&&(this.getDataPath=this.gridOptionsWrapper.getDataPathFunc())},t.prototype.execute=function(e){var r=this.createGroupingDetails(e);if(r.transactions)this.handleTransaction(r);else{var n=e.afterColumnsChanged===!0;this.shotgunResetEverything(r,n)}this.positionLeafsAboveGroups(e.changedPath),this.orderGroups(r.rootNode),this.selectableService.updateSelectableAfterGrouping(r.rootNode)},t.prototype.positionLeafsAboveGroups=function(e){this.usingTreeData||e.forEachChangedNodeDepthFirst(function(r){if(r.childrenAfterGroup){var n=[],o=[];r.childrenAfterGroup.forEach(function(a){var s;!((s=a.childrenAfterGroup)===null||s===void 0)&&s.length?o.push(a):n.push(a)}),r.childrenAfterGroup=z4(n,o)}},!1)},t.prototype.createGroupingDetails=function(e){var r=e.rowNode,n=e.changedPath,o=e.rowNodeTransactions,a=e.rowNodeOrder,s=this.usingTreeData?null:this.columnModel.getRowGroupColumns(),l={includeParents:!this.gridOptionsWrapper.isSuppressParentsInRowNodes(),expandByDefault:this.gridOptionsWrapper.getGroupDefaultExpanded(),groupedCols:s,rootNode:r,pivotMode:this.columnModel.isPivotMode(),groupedColCount:this.usingTreeData||!s?0:s.length,rowNodeOrder:a,transactions:o,changedPath:n};return l},t.prototype.handleTransaction=function(e){var r=this;e.transactions.forEach(function(n){var o=r.usingTreeData?void 0:new yv;if(_.existsAndNotEmpty(n.remove)&&r.removeNodes(n.remove,e,o),_.existsAndNotEmpty(n.update)&&r.moveNodesInWrongPath(n.update,e,o),_.existsAndNotEmpty(n.add)&&r.insertNodes(n.add,e,!1),o){var a=o.getAllParents().slice();o.flush(),r.removeEmptyGroups(a,e)}}),e.rowNodeOrder&&this.sortChildren(e)},t.prototype.sortChildren=function(e){e.changedPath.forEachChangedNodeDepthFirst(function(r){_.sortRowNodesByOrder(r.childrenAfterGroup,e.rowNodeOrder)})},t.prototype.orderGroups=function(e){if(this.usingTreeData)return;var r=this.gridOptionsWrapper.getInitialGroupOrderComparator();_.exists(r)&&n(e);function n(o){var a=_.exists(o.childrenAfterGroup)&&!o.leafGroup;a&&(o.childrenAfterGroup.sort(function(s,l){return r({nodeA:s,nodeB:l})}),o.childrenAfterGroup.forEach(function(s){return n(s)}))}},t.prototype.getExistingPathForNode=function(e,r){for(var n=[],o=this.usingTreeData?e:e.parent;o&&o!==r.rootNode;)n.push({key:o.key,rowGroupColumn:o.rowGroupColumn,field:o.field}),o=o.parent;return n.reverse(),n},t.prototype.moveNodesInWrongPath=function(e,r,n){var o=this;e.forEach(function(a){r.changedPath.isActive()&&r.changedPath.addParentNode(a.parent);var s=function(p){return p.key},l=o.getExistingPathForNode(a,r).map(s),u=o.getGroupInfo(a,r).map(s),c=_.areEqual(l,u);c||o.moveNode(a,r,n)})},t.prototype.moveNode=function(e,r,n){if(this.removeNodesInStages([e],r,n),this.insertOneNode(e,r,!0),e.setData(e.data),r.changedPath.isActive()){var o=e.parent;r.changedPath.addParentNode(o)}},t.prototype.removeNodes=function(e,r,n){this.removeNodesInStages(e,r,n),r.changedPath.isActive()&&e.forEach(function(o){return r.changedPath.addParentNode(o.parent)})},t.prototype.removeNodesInStages=function(e,r,n){if(this.removeNodesFromParents(e,r,n),this.usingTreeData){this.postRemoveCreateFillerNodes(e,r);var o=e.map(function(a){return a.parent});this.removeEmptyGroups(o,r)}},t.prototype.forEachParentGroup=function(e,r,n){for(var o=r;o&&o!==e.rootNode;)n(o),o=o.parent},t.prototype.removeNodesFromParents=function(e,r,n){var o=this,a=n==null,s=n||new yv;e.forEach(function(l){o.removeFromParent(l,s),o.forEachParentGroup(r,l.parent,function(u){s.removeFromAllLeafChildren(u,l)})}),a&&s.flush()},t.prototype.postRemoveCreateFillerNodes=function(e,r){var n=this;e.forEach(function(o){var a=o.hasChildren();if(a){var s=n.getExistingPathForNode(o,r),l=n.findParentForNode(o,s,r);l.expanded=o.expanded,l.allLeafChildren=o.allLeafChildren,l.childrenAfterGroup=o.childrenAfterGroup,l.childrenMapped=o.childrenMapped,l.updateHasChildren(),l.childrenAfterGroup.forEach(function(u){return u.parent=l})}})},t.prototype.removeEmptyGroups=function(e,r){for(var n=this,o=!0,a=function(l){var u=n.getChildrenMappedKey(l.key,l.rowGroupColumn),c=l.parent,p=c&&c.childrenMapped?!c.childrenMapped[u]:!0;return p?!1:!!l.isEmptyRowGroupNode()},s=function(){o=!1;var l=new yv;e.forEach(function(u){n.forEachParentGroup(r,u,function(c){a(c)&&(o=!0,n.removeFromParent(c,l),c.setSelected(!1))})}),l.flush()};o;)s()},t.prototype.removeFromParent=function(e,r){e.parent&&(r?r.removeFromChildrenAfterGroup(e.parent,e):(_.removeFromArray(e.parent.childrenAfterGroup,e),e.parent.updateHasChildren()));var n=this.getChildrenMappedKey(e.key,e.rowGroupColumn);e.parent&&e.parent.childrenMapped&&(e.parent.childrenMapped[n]=void 0),e.setRowTop(null),e.setRowIndex(null)},t.prototype.addToParent=function(e,r){var n=this.getChildrenMappedKey(e.key,e.rowGroupColumn);if(r){var o=r.childrenMapped!=null;o&&(r.childrenMapped[n]=e),r.childrenAfterGroup.push(e),r.updateHasChildren()}},t.prototype.areGroupColsEqual=function(e,r){return e==null||r==null||e.pivotMode!==r.pivotMode?!1:_.areEqual(e.groupedCols,r.groupedCols)},t.prototype.checkAllGroupDataAfterColsChanged=function(e){var r=this,n=function(o){!o||o.forEach(function(a){var s=!r.usingTreeData&&!a.group;if(!s){var l={field:a.field,key:a.key,rowGroupColumn:a.rowGroupColumn};r.setGroupData(a,l),n(a.childrenAfterGroup)}})};n(e.rootNode.childrenAfterGroup)},t.prototype.shotgunResetEverything=function(e,r){if(!this.noChangeInGroupingColumns(e,r)){this.selectionService.removeGroupsFromSelection();var n=e.rootNode,o=e.groupedCols;n.leafGroup=this.usingTreeData?!1:o.length===0,n.childrenAfterGroup=[],n.childrenMapped={},n.updateHasChildren();var a=n.sibling;a&&(a.childrenAfterGroup=n.childrenAfterGroup,a.childrenMapped=n.childrenMapped),this.insertNodes(n.allLeafChildren,e,!1)}},t.prototype.noChangeInGroupingColumns=function(e,r){var n=!1,o=this.columnModel.getGroupDisplayColumns(),a=o?o.map(function(s){return s.getId()}).join("-"):"";return r&&(n=this.usingTreeData||this.areGroupColsEqual(e,this.oldGroupingDetails),this.oldGroupDisplayColIds!==a&&this.checkAllGroupDataAfterColsChanged(e)),this.oldGroupingDetails=e,this.oldGroupDisplayColIds=a,n},t.prototype.insertNodes=function(e,r,n){var o=this;e.forEach(function(a){o.insertOneNode(a,r,n),r.changedPath.isActive()&&r.changedPath.addParentNode(a.parent)})},t.prototype.insertOneNode=function(e,r,n){var o=this.getGroupInfo(e,r),a=this.findParentForNode(e,o,r);a.group||console.warn("AG Grid: duplicate group keys for row data, keys should be unique",[a.data,e.data]),this.usingTreeData?this.swapGroupWithUserNode(a,e,n):(e.parent=a,e.level=o.length,a.childrenAfterGroup.push(e),a.updateHasChildren())},t.prototype.findParentForNode=function(e,r,n){var o=this,a=n.rootNode;return r.forEach(function(s,l){a=o.getOrCreateNextNode(a,s,l,n),a.allLeafChildren.push(e)}),a},t.prototype.swapGroupWithUserNode=function(e,r,n){r.parent=e.parent,r.key=e.key,r.field=e.field,r.groupData=e.groupData,r.level=e.level,n||(r.expanded=e.expanded),r.leafGroup=e.leafGroup,r.rowGroupIndex=e.rowGroupIndex,r.allLeafChildren=e.allLeafChildren,r.childrenAfterGroup=e.childrenAfterGroup,r.childrenMapped=e.childrenMapped,r.updateHasChildren(),this.removeFromParent(e),r.childrenAfterGroup.forEach(function(o){return o.parent=r}),this.addToParent(r,e.parent)},t.prototype.getOrCreateNextNode=function(e,r,n,o){var a=this.getChildrenMappedKey(r.key,r.rowGroupColumn),s=e.childrenMapped?e.childrenMapped[a]:void 0;return s||(s=this.createGroup(r,e,n,o),this.addToParent(s,e)),s},t.prototype.createGroup=function(e,r,n,o){var a=new Oe(this.beans);return a.group=!0,a.field=e.field,a.rowGroupColumn=e.rowGroupColumn,this.setGroupData(a,e),a.id=Oe.ID_PREFIX_ROW_GROUP+this.groupIdSequence.next(),a.key=e.key,a.level=n,a.leafGroup=this.usingTreeData?!1:n===o.groupedColCount-1,a.allLeafChildren=[],a.setAllChildrenCount(0),a.rowGroupIndex=this.usingTreeData?null:n,a.childrenAfterGroup=[],a.childrenMapped={},a.updateHasChildren(),a.parent=o.includeParents?r:null,this.setExpandedInitialValue(o,a),a},t.prototype.setGroupData=function(e,r){var n=this;e.groupData={};var o=this.columnModel.getGroupDisplayColumns();o.forEach(function(a){var s=n.usingTreeData||(e.rowGroupColumn?a.isRowGroupDisplayed(e.rowGroupColumn.getId()):!1);s&&(e.groupData[a.getColId()]=r.key)})},t.prototype.getChildrenMappedKey=function(e,r){return r?r.getId()+"-"+e:e},t.prototype.setExpandedInitialValue=function(e,r){if(e.pivotMode&&r.leafGroup){r.expanded=!1;return}var n=this.gridOptionsWrapper.getIsGroupOpenByDefaultFunc();if(n){var o={rowNode:r,field:r.field,key:r.key,level:r.level,rowGroupColumn:r.rowGroupColumn};r.expanded=n(o)==!0;return}var a=e.expandByDefault;if(e.expandByDefault===-1){r.expanded=!0;return}r.expanded=r.level<a},t.prototype.getGroupInfo=function(e,r){return this.usingTreeData?this.getGroupInfoFromCallback(e):this.getGroupInfoFromGroupColumns(e,r)},t.prototype.getGroupInfoFromCallback=function(e){var r=this.getDataPath?this.getDataPath(e.data):null;(r==null||r.length===0)&&_.doOnce(function(){return console.warn("AG Grid: getDataPath() should not return an empty path for data",e.data)},"groupStage.getGroupInfoFromCallback");var n=function(o){return{key:o,field:null,rowGroupColumn:null}};return r?r.map(n):[]},t.prototype.getGroupInfoFromGroupColumns=function(e,r){var n=this,o=[];return r.groupedCols.forEach(function(a){var s=n.valueService.getKeyForNode(a,e),l=s!=null;if(r.pivotMode&&!l&&(s=" ",l=!0),l){var u={key:s,field:a.getColDef().field,rowGroupColumn:a};o.push(u)}}),o},Oa([y("columnModel")],t.prototype,"columnModel",void 0),Oa([y("selectableService")],t.prototype,"selectableService",void 0),Oa([y("valueService")],t.prototype,"valueService",void 0),Oa([y("beans")],t.prototype,"beans",void 0),Oa([y("selectionService")],t.prototype,"selectionService",void 0),Oa([B],t.prototype,"postConstruct",null),t=Oa([Q("groupStage")],t),t}(H),K4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),fl=globalThis&&globalThis.__assign||function(){return fl=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},fl.apply(this,arguments)},nx=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Y4=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Cv=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(Y4(arguments[t]));return i},X4=function(i){K4(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}e=t,t.prototype.createPivotColumnDefs=function(r){var n=this.columnModel.getPivotColumns(),o=this.columnModel.getValueColumns(),a=n.length,s=this.recursiveBuildGroup(0,r,[],a,n);function l(p,h){return h===void 0&&(h=[]),p.forEach(function(d){d.children!==void 0?l(d.children,h):h.push(d)}),h}var u=l(s);this.addRowGroupTotals(s,u,o),this.addExpandablePivotGroups(s,u),this.addPivotTotalsToGroups(s,u);var c=u.map(function(p){return _.cloneObject(p)});return{pivotColumnGroupDefs:s,pivotColumnDefs:c}},t.prototype.recursiveBuildGroup=function(r,n,o,a,s){var l=this,u=this.columnModel.getValueColumns();if(r>=a)return this.buildMeasureCols(o);var c=s[r].getColDef(),p=this.headerNameComparator.bind(this,c.pivotComparator);if(u.length===1&&this.gridOptionsWrapper.isRemovePivotHeaderRowWhenSingleValueColumn()&&r===a-1){var h=[];return _.iterateObject(n,function(f){var g=Cv(o,[f]);h.push(fl(fl({},l.createColDef(u[0],f,g)),{columnGroupShow:"open"}))}),h.sort(p),h}var d=[];return _.iterateObject(n,function(f,g){var v=Cv(o,[f]);d.push({children:l.recursiveBuildGroup(r+1,g,v,a,s),headerName:f,pivotKeys:v,columnGroupShow:"open",groupId:l.generateColumnGroupId(v)})}),d.sort(p),d},t.prototype.buildMeasureCols=function(r){var n=this,o=this.columnModel.getValueColumns();return o.length===0?[this.createColDef(null,"-",r)]:o.map(function(a){var s=n.columnModel.getDisplayNameForColumn(a,"header");return fl(fl({},n.createColDef(a,s,r)),{columnGroupShow:"open"})})},t.prototype.addExpandablePivotGroups=function(r,n){var o=this;if(!(this.gridOptionsWrapper.isSuppressExpandablePivotGroups()||this.gridOptionsWrapper.getPivotColumnGroupTotals())){var a=function(s,l,u){var c=s;if(c.children){var p=new Map;c.children.forEach(function(v){a(v,l,p)});var h=!c.children.some(function(v){return v.children});o.columnModel.getValueColumns().forEach(function(v){var m=o.columnModel.getDisplayNameForColumn(v,"header"),C=o.createColDef(v,m,s.pivotKeys);if(C.pivotTotalColumnIds=p.get(v.getColId()),C.columnGroupShow="closed",C.aggFunc=v.getAggFunc(),!h){var w=s.children;w.push(C),l.push(C)}}),o.merge(u,p)}else{var d=s;if(!d.pivotValueColumn)return;var f=d.pivotValueColumn.getColId(),g=u.has(f)?u.get(f):[];g.push(d.colId),u.set(f,g)}};r.forEach(function(s){a(s,n,new Map)})}},t.prototype.addPivotTotalsToGroups=function(r,n){var o=this;if(!!this.gridOptionsWrapper.getPivotColumnGroupTotals()){var a=this.gridOptionsWrapper.getPivotColumnGroupTotals()==="after",s=this.columnModel.getValueColumns(),l=s.map(function(c){return c.getAggFunc()});if(!(!l||l.length<1||!this.sameAggFuncs(l))){var u=s[0];r.forEach(function(c){o.recursivelyAddPivotTotal(c,n,u,a)})}}},t.prototype.recursivelyAddPivotTotal=function(r,n,o,a){var s=this,l=r;if(!l.children){var u=r;return u.colId?[u.colId]:null}var c=[];if(l.children.forEach(function(g){var v=s.recursivelyAddPivotTotal(g,n,o,a);v&&(c=c.concat(v))}),l.children.length>1){var p=this.gridOptionsWrapper.getLocaleTextFunc(),h=p("pivotColumnGroupTotals","Total"),d=this.createColDef(o,h,r.pivotKeys,!0);d.pivotTotalColumnIds=c,d.aggFunc=o.getAggFunc();var f=r.children;a?f.push(d):f.unshift(d),n.push(d)}return c},t.prototype.addRowGroupTotals=function(r,n,o){var a=this;if(!!this.gridOptionsWrapper.getPivotRowTotals())for(var s=this.gridOptionsWrapper.getPivotRowTotals()==="after",l=s?o.slice():o.slice().reverse(),u=function(h){var d=l[h],f=[];r.forEach(function(g){f=f.concat(a.extractColIdsForValueColumn(g,d))}),c.createRowGroupTotal(r,n,d,f,s)},c=this,p=0;p<l.length;p++)u(p)},t.prototype.extractColIdsForValueColumn=function(r,n){var o=this,a=r;if(!a.children){var s=a;return s.pivotValueColumn===n&&s.colId?[s.colId]:[]}var l=[];return a.children.forEach(function(u){o.extractColIdsForValueColumn(u,n);var c=o.extractColIdsForValueColumn(u,n);l=l.concat(c)}),l},t.prototype.createRowGroupTotal=function(r,n,o,a,s){var l=this.columnModel.getValueColumns(),u;if(l.length===0)u=this.createColDef(null,"-",[]);else{var c=this.columnModel.getDisplayNameForColumn(o,"header");u=this.createColDef(o,c,[]),u.pivotTotalColumnIds=a}u.colId=e.PIVOT_ROW_TOTAL_PREFIX+u.colId,n.push(u),s?r.push(u):r.unshift(u)},t.prototype.createColDef=function(r,n,o,a){a===void 0&&(a=!1);var s={};if(r){var l=r.getColDef();Object.assign(s,l),s.hide=!1}return s.headerName=n,s.colId=this.generateColumnId(o||[],r&&!a?r.getColId():""),s.field=s.colId,s.pivotKeys=o,s.pivotValueColumn=r,s.filter===!0&&(s.filter="agNumberColumnFilter"),s},t.prototype.sameAggFuncs=function(r){if(r.length==1)return!0;for(var n=1;n<r.length;n++)if(r[n]!==r[0])return!1;return!0},t.prototype.headerNameComparator=function(r,n,o){return r?r(n.headerName,o.headerName):n.headerName&&!o.headerName?1:!n.headerName&&o.headerName?-1:!n.headerName||!o.headerName?0:n.headerName<o.headerName?-1:n.headerName>o.headerName?1:0},t.prototype.merge=function(r,n){n.forEach(function(o,a,s){var l=r.has(a)?r.get(a):[],u=Cv(l,o);r.set(a,u)})},t.prototype.generateColumnGroupId=function(r){var n=this.columnModel.getPivotColumns().map(function(o){return o.getColId()});return"pivotGroup_"+n.join("-")+"_"+r.join("-")},t.prototype.generateColumnId=function(r,n){var o=this.columnModel.getPivotColumns().map(function(a){return a.getColId()});return"pivot_"+o.join("-")+"_"+r.join("-")+"_"+n};var e;return t.PIVOT_ROW_TOTAL_PREFIX="PivotRowTotal_",nx([y("columnModel")],t.prototype,"columnModel",void 0),t=e=nx([Q("pivotColDefService")],t),t}(H),q4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Xp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Z4=function(i){q4(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.uniqueValues={},e}return t.prototype.execute=function(e){var r=e.rowNode,n=e.changedPath;this.columnModel.isPivotActive()?this.executePivotOn(r,n):this.executePivotOff(n)},t.prototype.executePivotOff=function(e){this.aggregationColumnsHashLastTime=null,this.uniqueValues={},this.columnModel.isSecondaryColumnsPresent()&&(this.columnModel.setSecondaryColumns(null,"rowModelUpdated"),e&&e.setInactive())},t.prototype.executePivotOn=function(e,r){var n=this.bucketUpRowNodes(e),o=this.setUniqueValues(n),a=this.columnModel.getValueColumns(),s=a.map(function(f){return f.getId()+"-"+f.getColDef().headerName}).join("#"),l=a.map(function(f){return f.getAggFunc().toString()}).join("#"),u=this.aggregationColumnsHashLastTime!==s,c=this.aggregationFuncsHashLastTime!==l;if(this.aggregationColumnsHashLastTime=s,this.aggregationFuncsHashLastTime=l,o||u||c){var p=this.pivotColDefService.createPivotColumnDefs(this.uniqueValues),h=p.pivotColumnGroupDefs,d=p.pivotColumnDefs;this.pivotColumnDefs=d,this.columnModel.setSecondaryColumns(h,"rowModelUpdated"),r&&r.setInactive()}},t.prototype.setUniqueValues=function(e){var r=JSON.stringify(e),n=JSON.stringify(this.uniqueValues),o=r!==n;return o?(this.uniqueValues=e,!0):!1},t.prototype.bucketUpRowNodes=function(e){var r=this,n={},o=function(a){a.leafGroup?r.bucketRowNode(a,n):a.childrenAfterFilter.forEach(function(s){o(s)})};return o(e),n},t.prototype.bucketRowNode=function(e,r){var n=this.columnModel.getPivotColumns();n.length===0?e.childrenMapped=null:e.childrenMapped=this.bucketChildren(e.childrenAfterFilter,n,0,r),e.sibling&&(e.sibling.childrenMapped=e.childrenMapped)},t.prototype.bucketChildren=function(e,r,n,o){var a=this,s={},l=r[n];if(e.forEach(function(c){var p=a.valueService.getKeyForNode(l,c);_.missing(p)&&(p=""),o[p]||(o[p]={}),s[p]||(s[p]=[]),s[p].push(c)}),n===r.length-1)return s;var u={};return _.iterateObject(s,function(c,p){u[c]=a.bucketChildren(p,r,n+1,o[c])}),u},t.prototype.getPivotColumnDefs=function(){return this.pivotColumnDefs},Xp([y("valueService")],t.prototype,"valueService",void 0),Xp([y("columnModel")],t.prototype,"columnModel",void 0),Xp([y("pivotColDefService")],t.prototype,"pivotColDefService",void 0),t=Xp([Q("pivotStage")],t),t}(H),Q4=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ix=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},_i=typeof BigInt=="undefined"?null:BigInt,J4=function(i){Q4(t,i);function t(){var r=i!==null&&i.apply(this,arguments)||this;return r.aggFuncsMap={},r.initialised=!1,r}e=t,t.prototype.init=function(){this.initialised||(this.initialiseWithDefaultAggregations(),this.addAggFuncs(this.gridOptionsWrapper.getAggFuncs()))},t.prototype.initialiseWithDefaultAggregations=function(){this.aggFuncsMap[e.AGG_SUM]=eU,this.aggFuncsMap[e.AGG_FIRST]=tU,this.aggFuncsMap[e.AGG_LAST]=rU,this.aggFuncsMap[e.AGG_MIN]=nU,this.aggFuncsMap[e.AGG_MAX]=iU,this.aggFuncsMap[e.AGG_COUNT]=oU,this.aggFuncsMap[e.AGG_AVG]=aU,this.initialised=!0},t.prototype.isAggFuncPossible=function(r,n){var o=this.getFuncNames(r),a=_.includes(o,n),s=_.exists(this.aggFuncsMap[n]);return a&&s},t.prototype.getDefaultAggFunc=function(r){var n=r.getColDef().defaultAggFunc;if(_.exists(n)&&this.isAggFuncPossible(r,n))return n;if(this.isAggFuncPossible(r,e.AGG_SUM))return e.AGG_SUM;var o=this.getFuncNames(r);return _.existsAndNotEmpty(o)?o[0]:null},t.prototype.addAggFuncs=function(r){_.iterateObject(r,this.addAggFunc.bind(this))},t.prototype.addAggFunc=function(r,n){this.init(),this.aggFuncsMap[r]=n},t.prototype.getAggFunc=function(r){return this.init(),this.aggFuncsMap[r]},t.prototype.getFuncNames=function(r){var n=r.getColDef().allowedAggFuncs;return n==null?Object.keys(this.aggFuncsMap).sort():n},t.prototype.clear=function(){this.aggFuncsMap={}};var e;return t.AGG_SUM="sum",t.AGG_FIRST="first",t.AGG_LAST="last",t.AGG_MIN="min",t.AGG_MAX="max",t.AGG_COUNT="count",t.AGG_AVG="avg",ix([B],t.prototype,"init",null),t=e=ix([Q("aggFuncService")],t),t}(H);function eU(i){for(var t=i.values,e=null,r=0;r<t.length;r++){var n=t[r];typeof n=="number"?e===null?e=n:_i?e+=typeof e=="number"?n:_i(n):e+=n:typeof n=="bigint"&&(e===null?e=n:e=(typeof e=="bigint"?e:_i(e))+n)}return e}function tU(i){return i.values.length>0?i.values[0]:null}function rU(i){return i.values.length>0?_.last(i.values):null}function nU(i){for(var t=i.values,e=null,r=0;r<t.length;r++){var n=t[r];(typeof n=="number"||typeof n=="bigint")&&(e===null||e>n)&&(e=n)}return e}function iU(i){for(var t=i.values,e=null,r=0;r<t.length;r++){var n=t[r];(typeof n=="number"||typeof n=="bigint")&&(e===null||e<n)&&(e=n)}return e}function oU(i){for(var t=i.values,e=0,r=0;r<t.length;r++){var n=t[r];e+=n!=null&&typeof n.value=="number"?n.value:1}return{value:e,toString:function(){return this.value.toString()},toNumber:function(){return this.value}}}function aU(i){for(var t=i.values,e=0,r=0,n=0;n<t.length;n++){var o=t[n],a=null;typeof o=="number"||typeof o=="bigint"?(a=o,r++):o!=null&&(typeof o.value=="number"||typeof o.value=="bigint")&&typeof o.count=="number"&&(_i?a=o.value*(typeof o.value=="number"?o.count:_i(o.count)):a=o.value*o.count,r+=o.count),typeof a=="number"?_i?e+=typeof e=="number"?a:_i(a):e+=a:typeof a=="bigint"&&(e=(typeof e=="bigint"?e:_i(e))+a)}var s=null;return r>0&&(_i?s=e/(typeof e=="number"?r:_i(r)):s=e/r),{count:r,value:s,toString:function(){return typeof this.value=="number"||typeof this.value=="bigint"?this.value.toString():""},toNumber:function(){return this.value}}}var YP=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),di=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ox=function(i){YP(t,i);function t(e,r,n,o,a){var s=i.call(this)||this;return s.column=e,s.dragSourceDropTarget=r,s.ghost=n,s.valueColumn=o,s.horizontal=a,s.popupShowing=!1,s}return t.prototype.init=function(){this.setTemplate(t.TEMPLATE);var e=this.getGui(),r=this.gridOptionsWrapper.isFunctionsReadOnly();this.addElementClasses(e),this.addElementClasses(this.eDragHandle,"drag-handle"),this.addElementClasses(this.eText,"text"),this.addElementClasses(this.eButton,"button"),this.eDragHandle.appendChild(_.createIconNoSpan("columnDrag",this.gridOptionsWrapper)),this.eButton.appendChild(_.createIconNoSpan("cancel",this.gridOptionsWrapper)),this.displayName=this.columnModel.getDisplayNameForColumn(this.column,"columnDrop"),this.setupComponents(),!this.ghost&&!r&&this.addDragSource();var n=this.gridOptionsWrapper.getLocaleTextFunc(),o=n("ariaDropZoneColumnComponentDescription","Press DELETE to remove"),a=this.getColumnAndAggFuncName(),s=a.name,l=a.aggFuncName,u="";this.valueColumn&&!r&&(u=n("ariaDropZoneColumnValueItemDescription","Press ENTER to change the aggregation type")),_.setAriaLabel(e,l+" "+s+" "+o+" "+u),this.setupTooltip()},t.prototype.setupTooltip=function(){var e=this,r=function(){var n=e.column.getColDef().headerTooltip;e.setTooltip(n)};r(),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,r)},t.prototype.addDragSource=function(){var e=this,r={type:At.ToolPanel,eElement:this.eDragHandle,defaultIconName:ht.ICON_HIDE,getDragItem:function(){return e.createDragItem()},dragItemName:this.displayName,dragSourceDropTarget:this.dragSourceDropTarget};this.dragAndDropService.addDragSource(r,!0),this.addDestroyFunc(function(){return e.dragAndDropService.removeDragSource(r)})},t.prototype.createDragItem=function(){var e={};return e[this.column.getId()]=this.column.isVisible(),{columns:[this.column],visibleState:e}},t.prototype.setupComponents=function(){this.setTextValue(),this.setupRemove(),this.ghost&&this.addCssClass("ag-column-drop-cell-ghost"),this.valueColumn&&!this.gridOptionsWrapper.isFunctionsReadOnly()&&this.addGuiEventListener("click",this.onShowAggFuncSelection.bind(this))},t.prototype.setupRemove=function(){var e=this;_.setDisplayed(this.eButton,!this.gridOptionsWrapper.isFunctionsReadOnly());var r={type:t.EVENT_COLUMN_REMOVE};this.addGuiEventListener("keydown",function(o){var a=o.key===j.ENTER,s=o.key===j.DELETE;s&&(o.preventDefault(),e.dispatchEvent(r)),a&&e.valueColumn&&!e.gridOptionsWrapper.isFunctionsReadOnly()&&(o.preventDefault(),e.onShowAggFuncSelection())}),this.addManagedListener(this.eButton,"click",function(o){e.dispatchEvent(r),o.stopPropagation()});var n=new Cr(this.eButton);this.addManagedListener(n,Cr.EVENT_TAP,function(){e.dispatchEvent(r)}),this.addDestroyFunc(n.destroy.bind(n))},t.prototype.getColumnAndAggFuncName=function(){var e=this.displayName,r="";if(this.valueColumn){var n=this.column.getAggFunc(),o=typeof n=="string"?n:"agg",a=this.gridOptionsWrapper.getLocaleTextFunc();r=a(o,o)}return{name:e,aggFuncName:r}},t.prototype.setTextValue=function(){var e=this.getColumnAndAggFuncName(),r=e.name,n=e.aggFuncName,o=this.valueColumn?n+"("+r+")":r,a=_.escapeString(o);this.eText.innerHTML=a},t.prototype.onShowAggFuncSelection=function(){var e=this;if(!this.popupShowing){this.popupShowing=!0;var r=new tp("select-agg-func"),n=this.aggFuncService.getFuncNames(this.column),o=this.getGui(),a=r.getGui();r.setModel({getRow:function(h){return n[h]},getRowCount:function(){return n.length}}),this.getContext().createBean(r);var s=_.loadTemplate('<div class="ag-select-agg-func-popup"></div>');s.style.top="0px",s.style.left="0px",s.appendChild(a),s.style.width=o.clientWidth+"px";var l=function(){e.destroyBean(r),e.popupShowing=!1,o.focus()},u=this.gridOptionsWrapper.getLocaleTextFunc(),c=this.popupService.addPopup({modal:!0,eChild:s,closeOnEsc:!0,closedCallback:l,ariaLabel:u("ariaLabelAggregationFunction","Aggregation Function")});c&&r.setComponentCreator(this.createAggSelect.bind(this,c.hideFunc)),r.addGuiEventListener("keydown",function(h){if(h.key===j.ENTER||h.key===j.SPACE){var d=r.getLastFocusedRow();if(d==null)return;var f=r.getComponentAt(d);f&&f.selectItem()}}),this.popupService.positionPopupUnderComponent({type:"aggFuncSelect",eventSource:o,ePopup:s,keepWithinBounds:!0,column:this.column}),r.refresh();var p=n.findIndex(function(h){return h===e.column.getAggFunc()});p===-1&&(p=0),r.focusRow(p)}},t.prototype.createAggSelect=function(e,r){var n=this,o=function(){if(e(),n.gridOptionsWrapper.isFunctionsPassive()){var c={type:x.EVENT_COLUMN_AGG_FUNC_CHANGE_REQUEST,columns:[n.column],aggFunc:r,api:n.gridApi,columnApi:n.columnApi};n.eventService.dispatchEvent(c)}else n.columnModel.setColumnAggFunc(n.column,r,"toolPanelDragAndDrop")},a=this.gridOptionsWrapper.getLocaleTextFunc(),s=r.toString(),l=a(s,s),u=new sU(o,l);return u},t.prototype.addElementClasses=function(e,r){r=r?"-"+r:"";var n=this.horizontal?"horizontal":"vertical";e.classList.add("ag-column-drop-cell"+r,"ag-column-drop-"+n+"-cell"+r)},t.EVENT_COLUMN_REMOVE="columnRemove",t.TEMPLATE=`<span role="option" tabindex="0">
          <span ref="eDragHandle" class="ag-drag-handle ag-column-drop-cell-drag-handle" role="presentation"></span>
          <span ref="eText" class="ag-column-drop-cell-text" aria-hidden="true"></span>
          <span ref="eButton" class="ag-column-drop-cell-button" role="presentation"></span>
        </span>`,di([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),di([y("columnModel")],t.prototype,"columnModel",void 0),di([y("popupService")],t.prototype,"popupService",void 0),di([De("aggFuncService")],t.prototype,"aggFuncService",void 0),di([y("columnApi")],t.prototype,"columnApi",void 0),di([y("gridApi")],t.prototype,"gridApi",void 0),di([I("eText")],t.prototype,"eText",void 0),di([I("eDragHandle")],t.prototype,"eDragHandle",void 0),di([I("eButton")],t.prototype,"eButton",void 0),di([B],t.prototype,"init",null),t}(ee),sU=function(i){YP(t,i);function t(e,r){var n=i.call(this,'<div class="ag-select-agg-func-item"/>')||this;return n.selectItem=e,n.getGui().innerText=r,n.addGuiEventListener("click",n.selectItem),n}return t}(ee),lU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),uU=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Kw=function(i){lU(t,i);function t(e,r){var n=i.call(this,'<div class="ag-unselectable" aria-role="presentation"></div>')||this;return n.horizontal=e,n.valueColumn=r,n.state=t.STATE_NOT_DRAGGING,n.guiDestroyFunctions=[],n.childColumnComponents=[],n.resizeEnabled=!1,n.addElementClasses(n.getGui()),n.eColumnDropList=document.createElement("div"),n.addElementClasses(n.eColumnDropList,"list"),_.setAriaRole(n.eColumnDropList,"listbox"),n}return t.prototype.isHorizontal=function(){return this.horizontal},t.prototype.toggleResizable=function(e){this.positionableFeature.setResizable(e?{bottom:!0}:!1),this.resizeEnabled=e},t.prototype.setBeans=function(e){this.beans=e},t.prototype.destroy=function(){this.destroyGui(),i.prototype.destroy.call(this)},t.prototype.destroyGui=function(){this.guiDestroyFunctions.forEach(function(e){return e()}),this.guiDestroyFunctions.length=0,this.childColumnComponents.length=0,_.clearElement(this.getGui()),_.clearElement(this.eColumnDropList)},t.prototype.init=function(e){this.params=e,this.createManagedBean(new ai(this.getFocusableElement(),{handleKeyDown:this.handleKeyDown.bind(this)})),this.addManagedListener(this.beans.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.refreshGui.bind(this)),this.addManagedListener(this.beans.gridOptionsWrapper,"functionsReadOnly",this.refreshGui.bind(this)),this.setupDropTarget(),this.positionableFeature=new Qc(this.getGui(),{minHeight:100}),this.createManagedBean(this.positionableFeature),this.refreshGui(),_.setAriaLabel(this.eColumnDropList,this.getAriaLabel())},t.prototype.handleKeyDown=function(e){var r=!this.horizontal,n=e.key===j.DOWN,o=e.key===j.UP;if(!r){var a=this.gridOptionsWrapper.isEnableRtl();n=!a&&e.key===j.RIGHT||a&&e.key===j.LEFT,o=!a&&e.key===j.LEFT||a&&e.key===j.RIGHT}if(!(!n&&!o)){var s=this.focusService.findNextFocusableElement(this.getFocusableElement(),!1,o);s&&(e.preventDefault(),s.focus())}},t.prototype.addElementClasses=function(e,r){r=r?"-"+r:"";var n=this.horizontal?"horizontal":"vertical";e.classList.add("ag-column-drop"+r,"ag-column-drop-"+n+r)},t.prototype.setupDropTarget=function(){this.dropTarget={getContainer:this.getGui.bind(this),getIconName:this.getIconName.bind(this),onDragging:this.onDragging.bind(this),onDragEnter:this.onDragEnter.bind(this),onDragLeave:this.onDragLeave.bind(this),onDragStop:this.onDragStop.bind(this),isInterestedIn:this.isInterestedIn.bind(this)},this.beans.dragAndDropService.addDropTarget(this.dropTarget)},t.prototype.isInterestedIn=function(e){return e===At.HeaderCell||e===At.ToolPanel},t.prototype.checkInsertIndex=function(e){var r=this.horizontal?this.getNewHorizontalInsertIndex(e):this.getNewVerticalInsertIndex(e);if(r<0)return!1;var n=r!==this.insertIndex;return n&&(this.insertIndex=r),n},t.prototype.getNewHorizontalInsertIndex=function(e){if(_.missing(e.hDirection))return-1;var r=0,n=e.event,o=this.beans.gridOptionsWrapper.isEnableRtl(),a=e.hDirection===Pn.Left,s=n.clientX;return this.childColumnComponents.forEach(function(l){var u=l.getGui().getBoundingClientRect(),c=a?u.right:u.left,p=o?s<=c:s>=c;p&&r++}),r},t.prototype.getNewVerticalInsertIndex=function(e){if(_.missing(e.vDirection))return-1;var r=0,n=e.event;return this.childColumnComponents.forEach(function(o){var a=o.getGui().getBoundingClientRect(),s=n.clientY>=(e.vDirection===la.Down?a.top:a.bottom);s&&r++}),r},t.prototype.checkDragStartedBySelf=function(e){this.state===t.STATE_NOT_DRAGGING&&(this.state=t.STATE_REARRANGE_COLUMNS,this.potentialDndColumns=e.dragSource.getDragItem().columns||[],this.refreshGui(),this.checkInsertIndex(e),this.refreshGui())},t.prototype.onDragging=function(e){this.checkDragStartedBySelf(e),this.checkInsertIndex(e)&&this.refreshGui()},t.prototype.onDragEnter=function(e){var r=e.dragSource.getDragItem().columns||[];this.state=t.STATE_NEW_COLUMNS_IN;var n=r.filter(this.isColumnDroppable.bind(this));n.length>0&&(this.potentialDndColumns=n,this.checkInsertIndex(e),this.refreshGui())},t.prototype.isPotentialDndColumns=function(){return _.existsAndNotEmpty(this.potentialDndColumns)},t.prototype.onDragLeave=function(e){if(this.state===t.STATE_REARRANGE_COLUMNS){var r=e.dragSource.getDragItem().columns||[];this.removeColumns(r)}this.isPotentialDndColumns()&&(this.potentialDndColumns=[],this.refreshGui()),this.state=t.STATE_NOT_DRAGGING},t.prototype.onDragStop=function(){if(this.isPotentialDndColumns()){var e=!1;this.state===t.STATE_NEW_COLUMNS_IN?(this.addColumns(this.potentialDndColumns),e=!0):e=this.rearrangeColumns(this.potentialDndColumns),this.potentialDndColumns=[],(!this.beans.gridOptionsWrapper.isFunctionsPassive()||!e)&&this.refreshGui()}this.state=t.STATE_NOT_DRAGGING},t.prototype.removeColumns=function(e){var r=this.getExistingColumns().filter(function(n){return!_.includes(e,n)});this.updateColumns(r)},t.prototype.addColumns=function(e){if(!!e){var r=this.getExistingColumns().slice(),n=e.filter(function(o){return r.indexOf(o)<0});_.insertArrayIntoArray(r,n,this.insertIndex),this.updateColumns(r)}},t.prototype.rearrangeColumns=function(e){var r=this.getNonGhostColumns().slice();return _.insertArrayIntoArray(r,e,this.insertIndex),_.areEqual(r,this.getExistingColumns())?!1:(this.updateColumns(r),!0)},t.prototype.refreshGui=function(){var e=this.eColumnDropList.scrollTop,r=this.resizeEnabled,n=this.getFocusedItem(),o=this.focusService.findNextFocusableElement();o||(o=this.focusService.findNextFocusableElement(void 0,!1,!0)),this.toggleResizable(!1),this.destroyGui(),this.addIconAndTitleToGui(),this.addEmptyMessageToGui(),this.addColumnsToGui(),this.isHorizontal()||(this.eColumnDropList.scrollTop=e),r&&this.toggleResizable(r),this.restoreFocus(n,o)},t.prototype.getFocusedItem=function(){var e=this.getGui(),r=this.gridOptionsWrapper.getDocument().activeElement;if(!e.contains(r))return-1;var n=Array.from(e.querySelectorAll(".ag-column-drop-cell"));return n.indexOf(r)},t.prototype.restoreFocus=function(e,r){var n=this.getGui(),o=Array.from(n.querySelectorAll(".ag-column-drop-cell"));if(e!==-1){o.length===0&&r.focus();var a=Math.min(o.length-1,e),s=o[a];s&&s.focus()}},t.prototype.getNonGhostColumns=function(){var e=this,r=this.getExistingColumns();return this.isPotentialDndColumns()?r.filter(function(n){return!_.includes(e.potentialDndColumns,n)}):r},t.prototype.addColumnsToGui=function(){var e=this,r=this.getNonGhostColumns(),n=this.isPotentialDndColumns(),o=[];r.forEach(function(a,s){if(!(n&&s>=e.insertIndex)){var l=e.createColumnComponent(a,!1);o.push(l)}}),this.isPotentialDndColumns()&&(this.potentialDndColumns.forEach(function(a){var s=e.createColumnComponent(a,!0);o.push(s)}),r.forEach(function(a,s){if(!(s<e.insertIndex)){var l=e.createColumnComponent(a,!1);o.push(l)}})),this.appendChild(this.eColumnDropList),o.forEach(function(a,s){s>0&&e.addArrow(e.eColumnDropList),e.eColumnDropList.appendChild(a.getGui())}),this.addAriaLabelsToComponents()},t.prototype.addAriaLabelsToComponents=function(){var e=this;this.childColumnComponents.forEach(function(r,n){var o=r.getGui();_.setAriaPosInSet(o,n+1),_.setAriaSetSize(o,e.childColumnComponents.length)})},t.prototype.createColumnComponent=function(e,r){var n=this,o=new ox(e,this.dropTarget,r,this.valueColumn,this.horizontal);return o.addEventListener(ox.EVENT_COLUMN_REMOVE,this.removeColumns.bind(this,[e])),this.beans.context.createBean(o),this.guiDestroyFunctions.push(function(){return n.destroyBean(o)}),r||this.childColumnComponents.push(o),o},t.prototype.addIconAndTitleToGui=function(){var e=this.params.icon,r=document.createElement("div");if(r.setAttribute("aria-hidden","true"),this.addElementClasses(r,"title-bar"),this.addElementClasses(e,"icon"),this.addOrRemoveCssClass("ag-column-drop-empty",this.isExistingColumnsEmpty()),r.appendChild(e),!this.horizontal){var n=document.createElement("span");this.addElementClasses(n,"title"),n.innerHTML=this.params.title,r.appendChild(n)}this.appendChild(r)},t.prototype.isExistingColumnsEmpty=function(){return this.getExistingColumns().length===0},t.prototype.addEmptyMessageToGui=function(){if(!(!this.isExistingColumnsEmpty()||this.isPotentialDndColumns())){var e=document.createElement("span");e.innerHTML=this.params.emptyMessage,this.addElementClasses(e,"empty-message"),this.eColumnDropList.appendChild(e)}},t.prototype.addArrow=function(e){if(this.horizontal){var r=this.beans.gridOptionsWrapper.isEnableRtl(),n=_.createIconNoSpan(r?"smallLeft":"smallRight",this.beans.gridOptionsWrapper);this.addElementClasses(n,"cell-separator"),e.appendChild(n)}},t.STATE_NOT_DRAGGING="notDragging",t.STATE_NEW_COLUMNS_IN="newColumnsIn",t.STATE_REARRANGE_COLUMNS="rearrangeColumns",uU([y("focusService")],t.prototype,"focusService",void 0),t}(ee),cU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),js=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},yy=function(i){cU(t,i);function t(e){return i.call(this,e,!1)||this}return t.prototype.passBeansUp=function(){i.prototype.setBeans.call(this,{gridOptionsWrapper:this.gridOptionsWrapper,eventService:this.eventService,context:this.getContext(),loggerFactory:this.loggerFactory,dragAndDropService:this.dragAndDropService});var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("rowGroupColumnsEmptyMessage","Drag here to set row groups"),n=e("groups","Row Groups");i.prototype.init.call(this,{dragAndDropIcon:ht.ICON_GROUP,icon:_.createIconNoSpan("rowGroupPanel",this.gridOptionsWrapper,null),emptyMessage:r,title:n}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,this.refreshGui.bind(this))},t.prototype.getAriaLabel=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("ariaRowGroupDropZonePanelLabel","Row Groups");return r},t.prototype.getTooltipParams=function(){var e=i.prototype.getTooltipParams.call(this);return e.location="rowGroupColumnsList",e},t.prototype.isColumnDroppable=function(e){return this.gridOptionsWrapper.isFunctionsReadOnly()||!e.isPrimary()?!1:e.isAllowRowGroup()&&!e.isRowGroupActive()},t.prototype.updateColumns=function(e){if(this.gridOptionsWrapper.isFunctionsPassive()){var r={type:x.EVENT_COLUMN_ROW_GROUP_CHANGE_REQUEST,columns:e,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(r)}else this.columnModel.setRowGroupColumns(e,"toolPanelUi")},t.prototype.getIconName=function(){return this.isPotentialDndColumns()?ht.ICON_GROUP:ht.ICON_NOT_ALLOWED},t.prototype.getExistingColumns=function(){return this.columnModel.getRowGroupColumns()},js([y("columnModel")],t.prototype,"columnModel",void 0),js([y("loggerFactory")],t.prototype,"loggerFactory",void 0),js([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),js([y("columnApi")],t.prototype,"columnApi",void 0),js([y("gridApi")],t.prototype,"gridApi",void 0),js([B],t.prototype,"passBeansUp",null),t}(Kw),pU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Us=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Cy=function(i){pU(t,i);function t(e){return i.call(this,e,!1)||this}return t.prototype.passBeansUp=function(){i.prototype.setBeans.call(this,{gridOptionsWrapper:this.gridOptionsWrapper,eventService:this.eventService,context:this.getContext(),loggerFactory:this.loggerFactory,dragAndDropService:this.dragAndDropService});var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("pivotColumnsEmptyMessage","Drag here to set column labels"),n=e("pivots","Column Labels");i.prototype.init.call(this,{dragAndDropIcon:ht.ICON_GROUP,icon:_.createIconNoSpan("pivotPanel",this.gridOptionsWrapper,null),emptyMessage:r,title:n}),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.refresh.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_CHANGED,this.refresh.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,this.checkVisibility.bind(this)),this.refresh()},t.prototype.getAriaLabel=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("ariaPivotDropZonePanelLabel","Column Labels");return r},t.prototype.getTooltipParams=function(){var e=i.prototype.getTooltipParams.call(this);return e.location="pivotColumnsList",e},t.prototype.refresh=function(){this.checkVisibility(),this.refreshGui()},t.prototype.checkVisibility=function(){var e=this.columnModel.isPivotMode();if(this.isHorizontal())switch(this.gridOptionsWrapper.getPivotPanelShow()){case"always":this.setDisplayed(e);break;case"onlyWhenPivoting":var r=this.columnModel.isPivotActive();this.setDisplayed(e&&r);break;default:this.setDisplayed(!1);break}else this.setDisplayed(e)},t.prototype.isColumnDroppable=function(e){return this.gridOptionsWrapper.isFunctionsReadOnly()||!e.isPrimary()?!1:e.isAllowPivot()&&!e.isPivotActive()},t.prototype.updateColumns=function(e){if(this.gridOptionsWrapper.isFunctionsPassive()){var r={type:x.EVENT_COLUMN_PIVOT_CHANGE_REQUEST,columns:e,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(r)}else this.columnModel.setPivotColumns(e,"toolPanelUi")},t.prototype.getIconName=function(){return this.isPotentialDndColumns()?ht.ICON_PIVOT:ht.ICON_NOT_ALLOWED},t.prototype.getExistingColumns=function(){return this.columnModel.getPivotColumns()},Us([y("columnModel")],t.prototype,"columnModel",void 0),Us([y("loggerFactory")],t.prototype,"loggerFactory",void 0),Us([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),Us([y("columnApi")],t.prototype,"columnApi",void 0),Us([y("gridApi")],t.prototype,"gridApi",void 0),Us([B],t.prototype,"passBeansUp",null),t}(Kw),hU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ax=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},dU=function(i){hU(t,i);function t(){return i.call(this)||this}return t.prototype.postConstruct=function(){this.setGui(this.createNorthPanel()),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,this.onRowGroupChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.onRowGroupChanged.bind(this)),this.onRowGroupChanged()},t.prototype.createNorthPanel=function(){var e=document.createElement("div"),r=this.onDropPanelVisible.bind(this);return e.classList.add("ag-column-drop-wrapper"),_.setAriaRole(e,"presentation"),this.rowGroupComp=new yy(!0),this.createManagedBean(this.rowGroupComp),this.pivotComp=new Cy(!0),this.createManagedBean(this.pivotComp),e.appendChild(this.rowGroupComp.getGui()),e.appendChild(this.pivotComp.getGui()),this.addManagedListener(this.rowGroupComp,ee.EVENT_DISPLAYED_CHANGED,r),this.addManagedListener(this.pivotComp,ee.EVENT_DISPLAYED_CHANGED,r),this.onDropPanelVisible(),e},t.prototype.onDropPanelVisible=function(){var e=this.rowGroupComp.isDisplayed()&&this.pivotComp.isDisplayed();this.rowGroupComp.addOrRemoveCssClass("ag-column-drop-horizontal-half-width",e),this.pivotComp.addOrRemoveCssClass("ag-column-drop-horizontal-half-width",e)},t.prototype.onRowGroupChanged=function(){if(!!this.rowGroupComp){var e=this.gridOptionsWrapper.getRowGroupPanelShow();if(e===L.ALWAYS)this.rowGroupComp.setDisplayed(!0);else if(e===L.ONLY_WHEN_GROUPING){var r=!this.columnModel.isRowGroupEmpty();this.rowGroupComp.setDisplayed(r)}else this.rowGroupComp.setDisplayed(!1)}},ax([y("columnModel")],t.prototype,"columnModel",void 0),ax([B],t.prototype,"postConstruct",null),t}(ee),fU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),wv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},gU=function(i){fU(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.execute=function(e){var r=this,n=this.columnModel.isPivotMode(),o=this.filterManager.isAggregateFilterPresent(),a=function(h){return!h.node.group},s=function(h){return h.node.leafGroup},l=this.gridOptionsWrapper.getGroupAggFiltering()||(n?s:a),u=e.changedPath,c=function(h,d){d===void 0&&(d=!1),h.childrenAfterFilter&&(h.childrenAfterAggFilter=h.childrenAfterFilter,d&&h.childrenAfterAggFilter.forEach(function(f){return c(f,d)}),r.setAllChildrenCount(h)),h.sibling&&(h.sibling.childrenAfterAggFilter=h.childrenAfterAggFilter)},p=function(h){var d;h.childrenAfterAggFilter=((d=h.childrenAfterFilter)===null||d===void 0?void 0:d.filter(function(f){var g,v=l({node:f});if(v){var m=r.filterManager.doesRowPassAggregateFilters({rowNode:f});if(m)return c(f,!0),!0}var C=(g=f.childrenAfterAggFilter)===null||g===void 0?void 0:g.length;return C}))||null,r.setAllChildrenCount(h),h.sibling&&(h.sibling.childrenAfterAggFilter=h.childrenAfterAggFilter)};u.forEachChangedNodeDepthFirst(o?p:c,!0)},t.prototype.setAllChildrenCountTreeData=function(e){var r=0;e.childrenAfterAggFilter.forEach(function(n){r++,r+=n.allChildrenCount}),e.setAllChildrenCount(r)},t.prototype.setAllChildrenCountGridGrouping=function(e){var r=0;e.childrenAfterAggFilter.forEach(function(n){n.group?r+=n.allChildrenCount:r++}),e.setAllChildrenCount(r)},t.prototype.setAllChildrenCount=function(e){if(!e.hasChildren()){e.setAllChildrenCount(null);return}this.gridOptionsWrapper.isTreeData()?this.setAllChildrenCountTreeData(e):this.setAllChildrenCountGridGrouping(e)},wv([y("filterManager")],t.prototype,"filterManager",void 0),wv([y("columnModel")],t.prototype,"columnModel",void 0),t=wv([Q("filterAggregatesStage")],t),t}(H),XP={moduleName:oe.RowGroupingModule,beans:[H4,gU,$4,X4,Z4,J4],agStackComponents:[{componentName:"AgGridHeaderDropZones",componentClass:dU}],dependantModules:[cr]},vU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),zs=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},sx=function(i){vU(t,i);function t(e){return i.call(this,e,!0)||this}return t.prototype.passBeansUp=function(){i.prototype.setBeans.call(this,{gridOptionsWrapper:this.gridOptionsWrapper,eventService:this.eventService,context:this.getContext(),loggerFactory:this.loggerFactory,dragAndDropService:this.dragAndDropService});var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("valueColumnsEmptyMessage","Drag here to aggregate"),n=e("values","Values");i.prototype.init.call(this,{dragAndDropIcon:ht.ICON_AGGREGATE,icon:_.createIconNoSpan("valuePanel",this.gridOptionsWrapper,null),emptyMessage:r,title:n}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VALUE_CHANGED,this.refreshGui.bind(this))},t.prototype.getAriaLabel=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("ariaValuesDropZonePanelLabel","Values");return r},t.prototype.getTooltipParams=function(){var e=i.prototype.getTooltipParams.call(this);return e.location="valueColumnsList",e},t.prototype.getIconName=function(){return this.isPotentialDndColumns()?ht.ICON_AGGREGATE:ht.ICON_NOT_ALLOWED},t.prototype.isColumnDroppable=function(e){return this.gridOptionsWrapper.isFunctionsReadOnly()||!e.isPrimary()?!1:e.isAllowValue()&&!e.isValueActive()},t.prototype.updateColumns=function(e){if(this.gridOptionsWrapper.isFunctionsPassive()){var r={type:x.EVENT_COLUMN_VALUE_CHANGE_REQUEST,columns:e,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(r)}else this.columnModel.setValueColumns(e,"toolPanelUi")},t.prototype.getExistingColumns=function(){return this.columnModel.getValueColumns()},zs([y("columnModel")],t.prototype,"columnModel",void 0),zs([y("loggerFactory")],t.prototype,"loggerFactory",void 0),zs([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),zs([y("columnApi")],t.prototype,"columnApi",void 0),zs([y("gridApi")],t.prototype,"gridApi",void 0),zs([B],t.prototype,"passBeansUp",null),t}(Kw),mU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),bv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Yw=function(i){mU(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.postConstruct=function(){this.initialiseTabGuard({})},t.prototype.init=function(e,r,n){this.allowDragging=e,this.params=r,this.eventType=n,this.primaryColsHeaderPanel.init(this.params);var o=this.params.suppressColumnFilter,a=this.params.suppressColumnSelectAll,s=this.params.suppressColumnExpandAll;s&&o&&a&&this.primaryColsHeaderPanel.setDisplayed(!1),this.addManagedListener(this.primaryColsListPanel,"groupExpanded",this.onGroupExpanded.bind(this)),this.addManagedListener(this.primaryColsListPanel,"selectionChanged",this.onSelectionChange.bind(this)),this.primaryColsListPanel.init(this.params,this.allowDragging,this.eventType),this.addManagedListener(this.primaryColsHeaderPanel,"expandAll",this.onExpandAll.bind(this)),this.addManagedListener(this.primaryColsHeaderPanel,"collapseAll",this.onCollapseAll.bind(this)),this.addManagedListener(this.primaryColsHeaderPanel,"selectAll",this.onSelectAll.bind(this)),this.addManagedListener(this.primaryColsHeaderPanel,"unselectAll",this.onUnselectAll.bind(this)),this.addManagedListener(this.primaryColsHeaderPanel,"filterChanged",this.onFilterChanged.bind(this)),this.positionableFeature=new Qc(this.getGui(),{minHeight:100}),this.createManagedBean(this.positionableFeature)},t.prototype.toggleResizable=function(e){this.positionableFeature.setResizable(e?{bottom:!0}:!1)},t.prototype.onExpandAll=function(){this.primaryColsListPanel.doSetExpandedAll(!0)},t.prototype.onCollapseAll=function(){this.primaryColsListPanel.doSetExpandedAll(!1)},t.prototype.expandGroups=function(e){this.primaryColsListPanel.setGroupsExpanded(!0,e)},t.prototype.collapseGroups=function(e){this.primaryColsListPanel.setGroupsExpanded(!1,e)},t.prototype.setColumnLayout=function(e){this.primaryColsListPanel.setColumnLayout(e)},t.prototype.onFilterChanged=function(e){this.primaryColsListPanel.setFilterText(e.filterText)},t.prototype.syncLayoutWithGrid=function(){this.primaryColsListPanel.onColumnsChanged()},t.prototype.onSelectAll=function(){this.primaryColsListPanel.doSetSelectedAll(!0)},t.prototype.onUnselectAll=function(){this.primaryColsListPanel.doSetSelectedAll(!1)},t.prototype.onGroupExpanded=function(e){this.primaryColsHeaderPanel.setExpandState(e.state)},t.prototype.onSelectionChange=function(e){this.primaryColsHeaderPanel.setSelectionState(e.state)},t.TEMPLATE=`<div class="ag-column-select">
            <ag-primary-cols-header ref="primaryColsHeaderPanel"></ag-primary-cols-header>
            <ag-primary-cols-list ref="primaryColsListPanel"></ag-primary-cols-list>
        </div>`,bv([I("primaryColsHeaderPanel")],t.prototype,"primaryColsHeaderPanel",void 0),bv([I("primaryColsListPanel")],t.prototype,"primaryColsListPanel",void 0),bv([B],t.prototype,"postConstruct",null),t}(bs),yU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),lx=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},CU=function(i){yU(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.initialised=!1,e.childDestroyFuncs=[],e}return t.prototype.setVisible=function(e){i.prototype.setDisplayed.call(this,e),e&&!this.initialised&&this.init(this.params)},t.prototype.init=function(e){var r=this,n={suppressColumnMove:!1,suppressColumnSelectAll:!1,suppressColumnFilter:!1,suppressColumnExpandAll:!1,contractColumnSelection:!1,suppressPivotMode:!1,suppressRowGroups:!1,suppressValues:!1,suppressPivots:!1,suppressSyncLayoutWithGrid:!1,api:this.gridApi,columnApi:this.columnApi,context:this.gridOptionsWrapper.getContext()};if(_.mergeDeep(n,e),this.params=n,this.isRowGroupingModuleLoaded()&&!this.params.suppressPivotMode&&(this.pivotModePanel=this.createBean(new rx),this.childDestroyFuncs.push(function(){return r.destroyBean(r.pivotModePanel)}),this.appendChild(this.pivotModePanel)),this.primaryColsPanel=this.createBean(new Yw),this.childDestroyFuncs.push(function(){return r.destroyBean(r.primaryColsPanel)}),this.primaryColsPanel.init(!0,this.params,"toolPanelUi"),this.primaryColsPanel.addCssClass("ag-column-panel-column-select"),this.appendChild(this.primaryColsPanel),this.isRowGroupingModuleLoaded()){this.params.suppressRowGroups||(this.rowGroupDropZonePanel=this.createBean(new yy(!1)),this.childDestroyFuncs.push(function(){return r.destroyBean(r.rowGroupDropZonePanel)}),this.appendChild(this.rowGroupDropZonePanel)),this.params.suppressValues||(this.valuesDropZonePanel=this.createBean(new sx(!1)),this.childDestroyFuncs.push(function(){return r.destroyBean(r.valuesDropZonePanel)}),this.appendChild(this.valuesDropZonePanel)),this.params.suppressPivots||(this.pivotDropZonePanel=this.createBean(new Cy(!1)),this.childDestroyFuncs.push(function(){return r.destroyBean(r.pivotDropZonePanel)}),this.appendChild(this.pivotDropZonePanel)),this.setLastVisible();var o=this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,function(){r.resetChildrenHeight(),r.setLastVisible()});this.childDestroyFuncs.push(function(){return o()})}this.initialised=!0},t.prototype.setPivotModeSectionVisible=function(e){var r=this;!this.isRowGroupingModuleLoaded()||(this.pivotModePanel?this.pivotModePanel.setDisplayed(e):e&&(this.pivotModePanel=this.createBean(new rx),this.getGui().insertBefore(this.pivotModePanel.getGui(),this.getGui().firstChild),this.childDestroyFuncs.push(function(){return r.destroyBean(r.pivotModePanel)})),this.setLastVisible())},t.prototype.setRowGroupsSectionVisible=function(e){!this.isRowGroupingModuleLoaded()||(this.rowGroupDropZonePanel?this.rowGroupDropZonePanel.setDisplayed(e):e&&(this.rowGroupDropZonePanel=this.createManagedBean(new yy(!1)),this.appendChild(this.rowGroupDropZonePanel)),this.setLastVisible())},t.prototype.setValuesSectionVisible=function(e){!this.isRowGroupingModuleLoaded()||(this.valuesDropZonePanel?this.valuesDropZonePanel.setDisplayed(e):e&&(this.valuesDropZonePanel=this.createManagedBean(new sx(!1)),this.appendChild(this.valuesDropZonePanel)),this.setLastVisible())},t.prototype.setPivotSectionVisible=function(e){!this.isRowGroupingModuleLoaded()||(this.pivotDropZonePanel?this.pivotDropZonePanel.setDisplayed(e):e&&(this.pivotDropZonePanel=this.createManagedBean(new Cy(!1)),this.appendChild(this.pivotDropZonePanel),this.pivotDropZonePanel.setDisplayed(e)),this.setLastVisible())},t.prototype.setResizers=function(){[this.primaryColsPanel,this.rowGroupDropZonePanel,this.valuesDropZonePanel,this.pivotDropZonePanel].forEach(function(e){if(!!e){var r=e.getGui();e.toggleResizable(!r.classList.contains("ag-last-column-drop")&&!r.classList.contains("ag-hidden"))}})},t.prototype.setLastVisible=function(){var e=this.getGui(),r=Array.prototype.slice.call(e.querySelectorAll(".ag-column-drop"));r.forEach(function(a){return a.classList.remove("ag-last-column-drop")});var n=e.querySelectorAll(".ag-column-drop:not(.ag-hidden)"),o=_.last(n);o&&o.classList.add("ag-last-column-drop"),this.setResizers()},t.prototype.resetChildrenHeight=function(){for(var e=this.getGui(),r=e.children,n=0;n<r.length;n++){var o=r[n];o.style.removeProperty("height"),o.style.removeProperty("flex")}},t.prototype.isRowGroupingModuleLoaded=function(){return ge.assertRegistered(oe.RowGroupingModule,"Row Grouping")},t.prototype.expandColumnGroups=function(e){this.primaryColsPanel.expandGroups(e)},t.prototype.collapseColumnGroups=function(e){this.primaryColsPanel.collapseGroups(e)},t.prototype.setColumnLayout=function(e){this.primaryColsPanel.setColumnLayout(e)},t.prototype.syncLayoutWithGrid=function(){this.primaryColsPanel.syncLayoutWithGrid()},t.prototype.destroyChildren=function(){this.childDestroyFuncs.forEach(function(e){return e()}),this.childDestroyFuncs.length=0,_.clearElement(this.getGui())},t.prototype.refresh=function(){this.destroyChildren(),this.init(this.params)},t.prototype.destroy=function(){this.destroyChildren(),i.prototype.destroy.call(this)},t.TEMPLATE='<div class="ag-column-panel"></div>',lx([y("gridApi")],t.prototype,"gridApi",void 0),lx([y("columnApi")],t.prototype,"columnApi",void 0),t}(ee),wU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ux=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},qP=function(i){wU(t,i);function t(){var e=i.call(this,'<div class="ag-tool-panel-horizontal-resize"></div>')||this;return e.minWidth=100,e.maxWidth=null,e}return t.prototype.setElementToResize=function(e){this.elementToResize=e},t.prototype.postConstruct=function(){var e=this.horizontalResizeService.addResizeBar({eResizeBar:this.getGui(),dragStartPixels:1,onResizeStart:this.onResizeStart.bind(this),onResizing:this.onResizing.bind(this),onResizeEnd:this.onResizing.bind(this)});this.addDestroyFunc(e),this.setInverted(this.gridOptionsWrapper.isEnableRtl())},t.prototype.onResizeStart=function(){this.startingWidth=this.elementToResize.offsetWidth},t.prototype.onResizing=function(e){var r=this.inverted?-1:1,n=Math.max(this.minWidth,Math.floor(this.startingWidth-e*r));this.maxWidth!=null&&(n=Math.min(this.maxWidth,n)),this.elementToResize.style.width=n+"px"},t.prototype.setInverted=function(e){this.inverted=e},t.prototype.setMaxWidth=function(e){this.maxWidth=e},t.prototype.setMinWidth=function(e){e!=null?this.minWidth=e:this.minWidth=100},ux([y("horizontalResizeService")],t.prototype,"horizontalResizeService",void 0),ux([B],t.prototype,"postConstruct",null),t}(ee),bU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),_v=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},cx=function(i){bU(t,i);function t(e){var r=i.call(this)||this;return r.toolPanelDef=e,r}return t.prototype.getToolPanelId=function(){return this.toolPanelDef.id},t.prototype.postConstruct=function(){var e=this.createTemplate();this.setTemplate(e),this.eIconWrapper.insertAdjacentElement("afterbegin",_.createIconNoSpan(this.toolPanelDef.iconKey,this.gridOptionsWrapper)),this.addManagedListener(this.eToggleButton,"click",this.onButtonPressed.bind(this))},t.prototype.createTemplate=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=this.toolPanelDef,n=e(r.labelKey,r.labelDefault),o=`<div class="ag-side-button" role="presentation">
                <button type="button" ref="eToggleButton" tabindex="-1" role="tab" class="ag-side-button-button">
                    <div ref="eIconWrapper" class="ag-side-button-icon-wrapper" aria-hidden="true"></div>
                    <span class="ag-side-button-label">`+n+`</span>
                </button>
            </div>`;return o},t.prototype.onButtonPressed=function(){this.dispatchEvent({type:t.EVENT_TOGGLE_BUTTON_CLICKED})},t.prototype.setSelected=function(e){this.addOrRemoveCssClass("ag-selected",e)},t.EVENT_TOGGLE_BUTTON_CLICKED="toggleButtonClicked",_v([I("eToggleButton")],t.prototype,"eToggleButton",void 0),_v([I("eIconWrapper")],t.prototype,"eIconWrapper",void 0),_v([B],t.prototype,"postConstruct",null),t}(ee),_U=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),qp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ZP=function(i){_U(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.buttonComps=[],e}return t.prototype.postConstruct=function(){this.addManagedListener(this.getFocusableElement(),"keydown",this.handleKeyDown.bind(this))},t.prototype.handleKeyDown=function(e){if(!(e.key!==j.TAB||!e.shiftKey)){var r=_.last(this.columnModel.getAllDisplayedColumns());this.focusService.focusGridView(r,!0)&&e.preventDefault()}},t.prototype.setToolPanelDefs=function(e){e.forEach(this.addButtonComp.bind(this))},t.prototype.setActiveButton=function(e){this.buttonComps.forEach(function(r){r.setSelected(e===r.getToolPanelId())})},t.prototype.addButtonComp=function(e){var r=this,n=this.createBean(new cx(e));this.buttonComps.push(n),this.appendChild(n),n.addEventListener(cx.EVENT_TOGGLE_BUTTON_CLICKED,function(){r.dispatchEvent({type:t.EVENT_SIDE_BAR_BUTTON_CLICKED,toolPanelId:e.id})})},t.prototype.clearButtons=function(){this.buttonComps=this.destroyBeans(this.buttonComps),_.clearElement(this.getGui())},t.EVENT_SIDE_BAR_BUTTON_CLICKED="sideBarButtonClicked",t.TEMPLATE='<div class="ag-side-buttons" role="tablist"></div>',qp([y("focusService")],t.prototype,"focusService",void 0),qp([y("columnModel")],t.prototype,"columnModel",void 0),qp([B],t.prototype,"postConstruct",null),qp([nt],t.prototype,"clearButtons",null),t}(ee),SU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),px=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},xU=function(i){SU(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.setupResize=function(){var e=this.getGui(),r=this.resizeBar=new qP;this.getContext().createBean(r),r.setElementToResize(e),this.appendChild(r)},t.prototype.getToolPanelId=function(){return this.toolPanelId},t.prototype.setToolPanelDef=function(e){var r=e.id,n=e.minWidth,o=e.maxWidth,a=e.width;this.toolPanelId=r,this.width=a;var s={},l=this.userComponentFactory.getToolPanelCompDetails(e,s),u=l.newAgStackInstance();if(u==null){console.warn("AG Grid: error processing tool panel component "+r+". You need to specify either 'toolPanel' or 'toolPanelFramework'");return}u.then(this.setToolPanelComponent.bind(this)),n!=null&&this.resizeBar.setMinWidth(n),o!=null&&this.resizeBar.setMaxWidth(o)},t.prototype.setToolPanelComponent=function(e){var r=this;this.toolPanelCompInstance=e,this.appendChild(e.getGui()),this.addDestroyFunc(function(){r.destroyBean(e)}),this.width&&(this.getGui().style.width=this.width+"px")},t.prototype.getToolPanelInstance=function(){return this.toolPanelCompInstance},t.prototype.setResizerSizerSide=function(e){var r=this.gridOptionsWrapper.isEnableRtl(),n=e==="left",o=r?n:!n;this.resizeBar.setInverted(o)},t.prototype.refresh=function(){this.toolPanelCompInstance.refresh()},t.TEMPLATE='<div class="ag-tool-panel-wrapper"/>',px([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),px([B],t.prototype,"setupResize",null),t}(ee),TU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Zp=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},OU=function(i){TU(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.toolPanelWrappers=[],e}return t.prototype.postConstruct=function(){var e=this;this.sideBarButtonsComp.addEventListener(ZP.EVENT_SIDE_BAR_BUTTON_CLICKED,this.onToolPanelButtonClicked.bind(this)),this.setSideBarDef(),this.gridOptionsWrapper.addEventListener("sideBar",function(){e.clearDownUi(),e.setSideBarDef()}),this.gridApi.registerSideBarComp(this),this.createManagedBean(new ai(this.getFocusableElement(),{onTabKeyDown:this.onTabKeyDown.bind(this),handleKeyDown:this.handleKeyDown.bind(this)}))},t.prototype.onTabKeyDown=function(e){if(!e.defaultPrevented){var r=this,n=r.focusService,o=r.sideBarButtonsComp,a=this.getGui(),s=o.getGui(),l=this.gridOptionsWrapper.getDocument(),u=l.activeElement,c=a.querySelector(".ag-tool-panel-wrapper:not(.ag-hidden)");if(!!c){if(s.contains(u))n.focusInto(c,e.shiftKey)&&e.preventDefault();else if(!n.isFocusUnderManagedComponent(c)&&e.shiftKey){var p=n.findFocusableElements(c)[0],h=this.gridOptionsWrapper.getDocument();if(h.activeElement===p){var d=s.querySelector(".ag-selected button");d&&(e.preventDefault(),d.focus())}}}}},t.prototype.handleKeyDown=function(e){var r=this.gridOptionsWrapper.getDocument();if(!!this.sideBarButtonsComp.getGui().contains(r.activeElement)){var n=this.sideBarButtonsComp.getGui(),o=Array.prototype.slice.call(n.querySelectorAll(".ag-side-button")),a=r.activeElement,s=o.findIndex(function(c){return c.contains(a)}),l=null;switch(e.key){case j.LEFT:case j.UP:l=Math.max(0,s-1);break;case j.RIGHT:case j.DOWN:l=Math.min(s+1,o.length-1);break}if(l!==null){var u=o[l].querySelector("button");u&&(u.focus(),e.preventDefault())}}},t.prototype.onToolPanelButtonClicked=function(e){var r=e.toolPanelId,n=this.openedItem();n===r?this.openToolPanel(void 0):this.openToolPanel(r)},t.prototype.clearDownUi=function(){this.sideBarButtonsComp.clearButtons(),this.destroyToolPanelWrappers()},t.prototype.setSideBarDef=function(){this.setDisplayed(!1);var e=this.gridOptionsWrapper.getSideBar(),r=!!e&&!!e.toolPanels;if(!!r){var n=r&&!e.hiddenByDefault;this.setDisplayed(n);var o=e.toolPanels;this.sideBarButtonsComp.setToolPanelDefs(o),this.setupToolPanels(o),this.setSideBarPosition(e.position),e.hiddenByDefault||this.openToolPanel(e.defaultToolPanel)}},t.prototype.setSideBarPosition=function(e){e||(e="right");var r=e==="left",n=r?"right":"left";return this.addOrRemoveCssClass("ag-side-bar-left",r),this.addOrRemoveCssClass("ag-side-bar-right",!r),this.toolPanelWrappers.forEach(function(o){o.setResizerSizerSide(n)}),this},t.prototype.setupToolPanels=function(e){var r=this;e.forEach(function(n){if(n.id==null){console.warn("AG Grid: please review all your toolPanel components, it seems like at least one of them doesn't have an id");return}if(n.toolPanel==="agColumnsToolPanel"){var o=!ge.assertRegistered(oe.ColumnToolPanelModule,"Column Tool Panel");if(o)return}if(n.toolPanel==="agFiltersToolPanel"){var o=!ge.assertRegistered(oe.FiltersToolPanelModule,"Filters Tool Panel");if(o)return}var a=new xU;r.getContext().createBean(a),a.setToolPanelDef(n),a.setDisplayed(!1),r.getGui().appendChild(a.getGui()),r.toolPanelWrappers.push(a)})},t.prototype.refresh=function(){this.toolPanelWrappers.forEach(function(e){return e.refresh()})},t.prototype.openToolPanel=function(e){var r=this.openedItem();if(r!==e){this.toolPanelWrappers.forEach(function(a){var s=e===a.getToolPanelId();a.setDisplayed(s)});var n=this.openedItem(),o=r!==n;o&&(this.sideBarButtonsComp.setActiveButton(e),this.raiseToolPanelVisibleEvent(e))}},t.prototype.getToolPanelInstance=function(e){var r=this.toolPanelWrappers.filter(function(n){return n.getToolPanelId()===e})[0];if(!r){console.warn("AG Grid: unable to lookup Tool Panel as invalid key supplied: "+e);return}return r.getToolPanelInstance()},t.prototype.raiseToolPanelVisibleEvent=function(e){var r={type:x.EVENT_TOOL_PANEL_VISIBLE_CHANGED,source:e,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi()};this.eventService.dispatchEvent(r)},t.prototype.close=function(){this.openToolPanel(void 0)},t.prototype.isToolPanelShowing=function(){return!!this.openedItem()},t.prototype.openedItem=function(){var e=null;return this.toolPanelWrappers.forEach(function(r){r.isDisplayed()&&(e=r.getToolPanelId())}),e},t.prototype.destroyToolPanelWrappers=function(){var e=this;this.toolPanelWrappers.forEach(function(r){_.removeFromParent(r.getGui()),e.destroyBean(r)}),this.toolPanelWrappers.length=0},t.prototype.destroy=function(){this.destroyToolPanelWrappers(),i.prototype.destroy.call(this)},t.TEMPLATE=`<div class="ag-side-bar ag-unselectable">
            <ag-side-bar-buttons ref="sideBarButtons"></ag-side-bar-buttons>
        </div>`,Zp([y("gridApi")],t.prototype,"gridApi",void 0),Zp([y("focusService")],t.prototype,"focusService",void 0),Zp([I("sideBarButtons")],t.prototype,"sideBarButtonsComp",void 0),Zp([B],t.prototype,"postConstruct",null),t}(ee),EU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),hx=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},RU=function(i){EU(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.isColGroupDef=function(r){return r&&typeof r.children!="undefined"},e.getId=function(r){return e.isColGroupDef(r)?r.groupId:r.colId},e}return t.prototype.createColumnTree=function(e){var r=this,n=[],o=function(s,l){if(r.isColGroupDef(s)){var u=s,c=typeof u.groupId!="undefined"?u.groupId:u.headerName,p=new $e(u,c,!1,l),h=[];return u.children.forEach(function(v){var m=o(v,l+1);m&&h.push(m)}),p.setChildren(h),p}else{var d=s,f=d.colId?d.colId:d.field,g=r.columnModel.getPrimaryColumn(f);return g||n.push(d),g}},a=[];return e.forEach(function(s){var l=o(s,0);l&&a.push(l)}),n.length>0&&console.warn("AG Grid: unable to find grid columns for the supplied colDef(s):",n),a},t.prototype.syncLayoutWithGrid=function(e){var r=this.getLeafPathTrees(),n=this.mergeLeafPathTrees(r);e(n)},t.prototype.getLeafPathTrees=function(){var e=function(o,a){var s;if(o instanceof $e)if(o.isPadding())s=a;else{var l=Object.assign({},o.getColGroupDef());l.groupId=o.getGroupId(),l.children=[a],s=l}else{var u=Object.assign({},o.getColDef());u.colId=o.getColId(),s=u}var c=o.getOriginalParent();return c?e(c,s):s},r=this.columnModel.getAllGridColumns(),n=r.filter(function(o){var a=o.getColDef();return o.isPrimary()&&!a.showRowGroup});return n.map(function(o){return e(o,o.getColDef())})},t.prototype.mergeLeafPathTrees=function(e){for(var r=this,n=function(c,p){var h=r.isColGroupDef(c)&&r.isColGroupDef(p);return h&&r.getId(c)===r.getId(p)},o=function(c,p){if(!r.isColGroupDef(p))return c;var h=c,d=p;if(d.children&&d.groupId){var f=r.addChildrenToGroup(h,d.groupId,d.children[0]);if(f)return h}return d.children.forEach(function(g){return o(h,g)}),h},a=[],s=1;s<=e.length;s++){var l=e[s-1],u=e[s];n(l,u)?e[s]=o(l,u):a.push(l)}return a},t.prototype.addChildrenToGroup=function(e,r,n){var o=this,a=function(p,h){var d=p.children.map(o.getId),f=_.includes(d,o.getId(h)),g=_.last(p.children),v=g&&o.getId(g)!==o.getId(h);return f&&v};if(!this.isColGroupDef(e))return!0;var s=e,l=n;if(a(s,l))return s.children.push(l),!0;if(s.groupId===r){var u=s.children.map(this.getId),c=_.includes(u,this.getId(l));if(!c)return s.children.push(l),!0}return s.children.forEach(function(p){return o.addChildrenToGroup(p,r,n)}),!1},hx([y("columnModel")],t.prototype,"columnModel",void 0),t=hx([Q("toolPanelColDefService")],t),t}(H),Xw={moduleName:oe.SideBarModule,beans:[RU],agStackComponents:[{componentName:"AgHorizontalResize",componentClass:qP},{componentName:"AgSideBar",componentClass:OU},{componentName:"AgSideBarButtons",componentClass:ZP}],dependantModules:[cr]},Ea=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},PU=function(){function i(){}return i.prototype.selectAllChildren=function(t,e,r){var n=this.extractAllLeafColumns(t);this.setAllColumns(n,e,r)},i.prototype.setColumn=function(t,e,r){this.setAllColumns([t],e,r)},i.prototype.setAllColumns=function(t,e,r){this.columnModel.isPivotMode()?this.setAllPivot(t,e,r):this.setAllVisible(t,e,r)},i.prototype.extractAllLeafColumns=function(t){var e=[],r=function(n){n.forEach(function(o){!o.isPassesFilter()||(o.isGroup()?r(o.getChildren()):e.push(o.getColumn()))})};return r(t),e},i.prototype.setAllVisible=function(t,e,r){var n=[];t.forEach(function(o){o.getColDef().lockVisible||o.isVisible()!=e&&n.push({colId:o.getId(),hide:!e})}),n.length>0&&this.columnModel.applyColumnState({state:n},r)},i.prototype.setAllPivot=function(t,e,r){this.gridOptionsWrapper.isFunctionsPassive()?this.setAllPivotPassive(t,e):this.setAllPivotActive(t,e,r)},i.prototype.setAllPivotPassive=function(t,e){var r=this.columnModel.getPivotColumns().slice(),n=this.columnModel.getValueColumns().slice(),o=this.columnModel.getRowGroupColumns().slice(),a=!1,s=!1,l=!1,u=function(g){g.isAnyFunctionActive()||(g.isAllowValue()?(n.push(g),s=!0):g.isAllowRowGroup()?(o.push(g),a=!0):g.isAllowPivot()&&(r.push(g),l=!0))},c=function(g){!g.isAnyFunctionActive()||(r.indexOf(g)>=0&&(_.removeFromArray(r,g),a=!0),n.indexOf(g)>=0&&(_.removeFromArray(n,g),s=!0),o.indexOf(g)>=0&&(_.removeFromArray(o,g),l=!0))},p=e?u:c;if(t.forEach(p),a){var h={type:x.EVENT_COLUMN_PIVOT_CHANGE_REQUEST,columns:r,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(h)}if(l){var d={type:x.EVENT_COLUMN_ROW_GROUP_CHANGE_REQUEST,columns:o,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(d)}if(s){var f={type:x.EVENT_COLUMN_VALUE_CHANGE_REQUEST,columns:o,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(f)}},i.prototype.setAllPivotActive=function(t,e,r){var n=this,o=[],a=function(u){if(!u.isAnyFunctionActive())if(u.isAllowValue()){var c=typeof u.getAggFunc()=="string"?u.getAggFunc():n.aggFuncService.getDefaultAggFunc(u);o.push({colId:u.getId(),aggFunc:c})}else u.isAllowRowGroup()?o.push({colId:u.getId(),rowGroup:!0}):u.isAllowPivot()&&o.push({colId:u.getId(),pivot:!0})},s=function(u){var c=u.isPivotActive()||u.isRowGroupActive()||u.isValueActive();c&&o.push({colId:u.getId(),pivot:!1,rowGroup:!1,aggFunc:null})},l=e?a:s;t.forEach(l),o.length>0&&this.columnModel.applyColumnState({state:o},r)},Ea([y("aggFuncService")],i.prototype,"aggFuncService",void 0),Ea([y("columnModel")],i.prototype,"columnModel",void 0),Ea([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),Ea([y("columnApi")],i.prototype,"columnApi",void 0),Ea([y("gridApi")],i.prototype,"gridApi",void 0),Ea([y("eventService")],i.prototype,"eventService",void 0),i=Ea([Q("modelItemUtils")],i),i}(),AU={moduleName:oe.ColumnToolPanelModule,beans:[PU],agStackComponents:[{componentName:"AgPrimaryColsHeader",componentClass:O4},{componentName:"AgPrimaryColsList",componentClass:W4},{componentName:"AgPrimaryCols",componentClass:Yw}],userComponents:[{componentName:"agColumnsToolPanel",componentClass:CU}],dependantModules:[cr,XP,Xw]},DU={getTemplate:function(){return{name:"Workbook",properties:{prefixedAttributes:[{prefix:"xmlns:",map:{o:"urn:schemas-microsoft-com:office:office",x:"urn:schemas-microsoft-com:office:excel",ss:"urn:schemas-microsoft-com:office:spreadsheet",html:"http://www.w3.org/TR/REC-html40"}}],rawMap:{xmlns:"urn:schemas-microsoft-com:office:spreadsheet"}}}}},MU={getTemplate:function(){return{name:"ExcelWorkbook",properties:{rawMap:{xmlns:"urn:schemas-microsoft-com:office:excel"}},children:[{name:"WindowHeight",textNode:"8130"},{name:"WindowWidth",textNode:"15135"},{name:"WindowHeight",textNode:"8130"},{name:"WindowTopX",textNode:"120"},{name:"WindowTopY",textNode:"45"},{name:"ProtectStructure",textNode:"False"},{name:"ProtectWindow",textNode:"False"}]}}},FU={getTemplate:function(i){var t=i.width;return{name:"Column",properties:{prefixedAttributes:[{prefix:"ss:",map:{Width:t}}]}}}},IU={getTemplate:function(i){var t=i.mergeAcross,e=i.styleId,r=i.data,n={};return t&&(n.MergeAcross=t),e&&(n.StyleID=e),{name:"Cell",properties:{prefixedAttributes:[{prefix:"ss:",map:n}]},children:[{name:"Data",properties:{prefixedAttributes:[{prefix:"ss:",map:{Type:r.type}}]},textNode:r.value}]}}},LU={getTemplate:function(i){var t=i.cells;return{name:"Row",children:t.map(function(e){return IU.getTemplate(e)})}}},NU={getTemplate:function(i){var t=i.table,e=i.name,r=t.columns,n=t.rows,o=r.map(function(s){return FU.getTemplate(s)}),a=n.map(function(s){return LU.getTemplate(s)});return{name:"Worksheet",children:[{name:"Table",children:o.concat(a)}],properties:{prefixedAttributes:[{prefix:"ss:",map:{Name:e}}]}}}},GU={getTemplate:function(){return{name:"DocumentProperties",properties:{rawMap:{xmlns:"urn:schemas-microsoft-com:office:office"}},children:[{name:"Version",textNode:"12.00"}]}}},kU={getTemplate:function(i){var t=i.alignment,e=t.vertical,r=t.horizontal,n=t.indent,o=t.readingOrder,a=t.rotate,s=t.shrinkToFit,l=t.verticalText,u=t.wrapText;return{name:"Alignment",properties:{prefixedAttributes:[{prefix:"ss:",map:{Vertical:e,Horizontal:r,Indent:n,ReadingOrder:o,Rotate:a,ShrinkToFit:s,VerticalText:l,WrapText:u}}]}}}},WU={getTemplate:function(i){var t=i.borders,e=t.borderBottom,r=t.borderLeft,n=t.borderRight,o=t.borderTop;return{name:"Borders",children:[e,r,n,o].map(function(a,s){var l=s==0?"Bottom":s==1?"Left":s==2?"Right":"Top";return{name:"Border",properties:{prefixedAttributes:[{prefix:"ss:",map:{Position:l,LineStyle:a.lineStyle,Weight:a.weight,Color:a.color}}]}}})}}},VU={getTemplate:function(i){var t=i.font,e=t.bold,r=t.fontName,n=t.italic,o=t.color,a=t.outline,s=t.shadow,l=t.size,u=t.strikeThrough,c=t.underline,p=t.verticalAlign,h=t.charSet,d=t.family;return{name:"Font",properties:{prefixedAttributes:[{prefix:"ss:",map:{Bold:e,FontName:r,Italic:n,Color:o,Outline:a,Shadow:s,Size:l,StrikeThrough:u,Underline:c,VerticalAlign:p}},{prefix:"x:",map:{CharSet:h,Family:d}}]}}}},BU={getTemplate:function(i){var t=i.interior,e=t.color,r=t.pattern,n=t.patternColor;return{name:"Interior",properties:{prefixedAttributes:[{prefix:"ss:",map:{Color:e,Pattern:r,PatternColor:n}}]}}}},HU={getTemplate:function(i){return{name:"Protection",properties:{prefixedAttributes:[{prefix:"ss:",map:{Protected:i.protection.protected,HideFormula:i.protection.hideFormula}}]}}}},jU={getTemplate:function(i){var t=i.numberFormat.format;return{name:"NumberFormat",properties:{prefixedAttributes:[{prefix:"ss:",map:{Format:t}}]}}}},UU={getTemplate:function(i){var t=i.id,e=i.name;return{name:"Style",properties:{prefixedAttributes:[{prefix:"ss:",map:{ID:t,Name:e||t}}]}}}},wy=function(){function i(){}return i.createExcel=function(t,e){var r=this.excelXmlHeader(),n=GU.getTemplate(),o=MU.getTemplate(),a=this.workbook(n,o,t,e);return""+r+my.createXml(a,function(s){return s?"1":"0"})},i.workbook=function(t,e,r,n){var o=[t,e,this.stylesXmlElement(r)].concat(NU.getTemplate(n));return Object.assign({},DU.getTemplate(),{children:o})},i.excelXmlHeader=function(){return`<?xml version="1.0" ?>
        <?mso-application progid="Excel.Sheet" ?>
        `},i.stylesXmlElement=function(t){var e=this;return{name:"Styles",children:t?t.map(function(r){return e.styleXmlElement(r)}):[]}},i.styleXmlElement=function(t){var e=_.compose(this.addProperty("alignment",t),this.addProperty("borders",t),this.addProperty("font",t),this.addProperty("interior",t),this.addProperty("protection",t),this.addProperty("numberFormat",t))([]);return Object.assign({},UU.getTemplate(t),{children:e})},i.addProperty=function(t,e){return function(r){if(!e[t])return r;var n={alignment:kU,borders:WU,font:VU,interior:BU,numberFormat:jU,protection:HU};return r.concat(n[t].getTemplate(e))}},i.factoryMode=to.SINGLE_SHEET,i}(),zU=9525,dx={"0":1,"0.00":2,"#,##0":3,"#,##0.00":4,"0%":9,"0.00%":10,"0.00E+00":11,"# ?/?":12,"# ??/??":13,"mm-dd-yy":14,"d-mmm-yy":15,"d-mmm":16,"mmm-yy":17,"h:mm AM/PM":18,"h:mm:ss AM/PM":19,"h:mm":20,"h:mm:ss":21,"m/d/yy h:mm":22,"#,##0 ;(#,##0)":37,"#,##0 ;[Red](#,##0)":38,"#,##0.00;(#,##0.00)":39,"#,##0.00;[Red](#,##0.00)":40,"mm:ss":45,"[h]:mm:ss":46,"mmss.0":47,"##0.0E+0":48,"@":49},$U=function(i){return Math.round(i*72/96)},KU=function(i){return Math.round(i*96/72)},$s=function(i){return Math.ceil(i*zU)},YU=function(i){if(i!==void 0){var t=["Automatic","Roman","Swiss","Modern","Script","Decorative"],e=t.indexOf(i||"Automatic");return Math.max(e,0)}},Fh=function(i,t){if(!!t){var e;if(typeof t=="number")e=t;else{var r=t;e=r({rowIndex:i})}return $U(e)}},XU=function(i,t){var e=i.position,r=e.colSpan,n=e.column;if(i.width)if(r)for(var o=t.slice(n-1,n+r-1),a=0,s=0;s<o.length;s++){var l=o[s].getActualWidth();if(i.width<a+l){i.position.colSpan=s+1,i.totalWidth=i.width,i.width=i.totalWidth-a;break}a+=l}else i.totalWidth=i.width},qU=function(i,t){var e=i.position,r=e.rowSpan,n=e.row;if(i.height)if(r)for(var o=0,a=0,s=n;s<n+r;s++){var l=KU(Fh(s,t)||20);if(i.height<o+l){i.position.rowSpan=a+1,i.totalHeight=i.height,i.height=i.totalHeight-o;break}o+=l,a++}else i.totalHeight=i.height},_n=function(i){var t=my.createHeader({encoding:"UTF-8",standalone:"yes"}),e=my.createXml(i);return""+t+e},Dc=function(i){var t=65,e=26,r=String.fromCharCode,n=Math.floor(i/e),o=i%e;return!n||i===e?r(t+i-1):o?n<e?r(t+n-1)+r(t+o-1):Dc(n)+r(t+o-1):Dc(n-1)+"Z"},ZU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),hc=globalThis&&globalThis.__assign||function(){return hc=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},hc.apply(this,arguments)},QU=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},fx=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(QU(arguments[t]));return i},QP=function(i){ZU(t,i);function t(e){var r=i.call(this,e)||this;return r.mixedStyles={},r.mixedStyleCounter=0,r.rows=[],r.config=Object.assign({},e),r.stylesByIds={},r.config.baseExcelStyles.forEach(function(n){r.stylesByIds[n.id]=n}),r.excelStyles=fx(r.config.baseExcelStyles),r}return t.prototype.addCustomContent=function(e){var r=this;e.forEach(function(n){var o=r.rows.length+1;r.rows.push({height:Fh(o,r.config.rowHeight),cells:n.map(function(a,s){var l=r.addImage(o,r.columnsToExport[s],a.data.value),u=hc({},a);return l&&(u.data={},l.value!=null?u.data.value=l.value:(u.data.type="e",u.data.value=null)),u})})})},t.prototype.onNewHeaderGroupingRow=function(){var e=this,r=[];return this.rows.push({cells:r,height:Fh(this.rows.length+1,this.config.headerRowHeight)}),{onColumn:function(n,o,a,s){var l=e.config.styleLinker(rs.HEADER_GROUPING,1,"grouping-"+n,void 0,void 0);r.push(hc(hc({},e.createMergedCell(e.getStyleId(l),e.getDataTypeForValue("string"),n,a)),{collapsibleRanges:s}))}}},t.prototype.onNewHeaderRow=function(){return this.onNewRow(this.onNewHeaderColumn,this.config.headerRowHeight)},t.prototype.onNewBodyRow=function(){return this.onNewRow(this.onNewBodyColumn,this.config.rowHeight)},t.prototype.prepare=function(e){var r=this;i.prototype.prepare.call(this,e),this.columnsToExport=fx(e),this.cols=e.map(function(n,o){return r.convertColumnToExcel(n,o)})},t.prototype.parse=function(){for(var e=this.rows.reduce(function(n,o){return Math.max(n,o.cells.length)},0);this.cols.length<e;)this.cols.push(this.convertColumnToExcel(null,this.cols.length+1));var r={name:this.config.sheetName,table:{columns:this.cols,rows:this.rows}};return this.createExcel(r)},t.prototype.isFormula=function(e){return e==null?!1:this.config.autoConvertFormulas&&e.toString().startsWith("=")},t.prototype.getStyleById=function(e){return e==null?null:this.stylesByIds[e]||null},t.prototype.convertColumnToExcel=function(e,r){var n=this.config.columnWidth;if(n)return typeof n=="number"?{width:n}:{width:n({column:e,index:r})};if(e){var o=75;return{width:Math.max(e.getActualWidth(),o)}}return{}},t.prototype.onNewHeaderColumn=function(e,r){var n=this;return function(o,a){var s=n.extractHeaderValue(o),l=n.config.styleLinker(rs.HEADER,e,s,o,void 0);r.push(n.createCell(n.getStyleId(l),n.getDataTypeForValue("string"),s))}},t.prototype.onNewRow=function(e,r){var n=[];return this.rows.push({cells:n,height:Fh(this.rows.length+1,r)}),{onColumn:e.bind(this,this.rows.length,n)()}},t.prototype.onNewBodyColumn=function(e,r){var n=this,o=0;return function(a,s,l){if(o>0){o-=1;return}var u=n.extractRowCellValue(a,s,e,L.EXPORT_TYPE_EXCEL,l),c=n.config.styleLinker(rs.BODY,e,u,a,l),p=n.getStyleId(c),h=a.getColSpan(l),d=n.addImage(e,a,u);d?r.push(n.createCell(p,n.getDataTypeForValue(d.value),d.value==null?"":d.value)):h>1?(o=h-1,r.push(n.createMergedCell(p,n.getDataTypeForValue(u),u,h-1))):r.push(n.createCell(p,n.getDataTypeForValue(u),u))}},t.prototype.getStyleId=function(e){if(!e||!e.length)return null;if(e.length===1)return e[0];var r=e.join("-");return this.mixedStyles[r]||this.addNewMixedStyle(e),this.mixedStyles[r].excelID},t.prototype.addNewMixedStyle=function(e){var r=this;this.mixedStyleCounter+=1;var n="mixedStyle"+this.mixedStyleCounter,o={};e.forEach(function(s){r.excelStyles.forEach(function(l){l.id===s&&_.mergeDeep(o,_.deepCloneObject(l))})}),o.id=n,o.name=n;var a=e.join("-");this.mixedStyles[a]={excelID:n,key:a,result:o},this.excelStyles.push(o),this.stylesByIds[n]=o},t}(HP),JU=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ez=function(i){JU(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.createExcel=function(e){return wy.createExcel(this.excelStyles,e)},t.prototype.getDataTypeForValue=function(e){return _.isNumeric(e)?"Number":"String"},t.prototype.getType=function(e,r,n){if(this.isFormula(n))return"Formula";if(r&&r.dataType)switch(r.dataType.toLocaleLowerCase()){case"string":return"Formula";case"number":return"Number";case"datetime":return"DateTime";case"error":return"Error";case"boolean":return"Boolean";default:console.warn("AG Grid: Unrecognized data type for excel export ["+r.id+".dataType="+r.dataType+"]")}return e},t.prototype.addImage=function(){},t.prototype.createCell=function(e,r,n){var o=this.getStyleById(e),a=this.getType(r,o,n)||r;return{styleId:o?e:void 0,data:{type:a,value:this.getValueTransformed(a,n)}}},t.prototype.getValueTransformed=function(e,r){var n=this,o=function(s){if(n.config.suppressTextAsCDATA)return _.escapeString(s);var l="<![CDATA[",u="]]>",c=new RegExp(u,"g");return l+String(s).replace(c,"]]"+u+l+">")+u},a=function(s){return!s||s==="0"||s==="false"?"0":"1"};switch(e){case"String":return o(r);case"Number":return Number(r).valueOf()+"";case"Boolean":return a(r);default:return r}},t.prototype.createMergedCell=function(e,r,n,o){return{styleId:this.getStyleById(e)?e:void 0,data:{type:r,value:n},mergeAcross:o}},t}(QP),tz={getTemplate:function(i){var t=new Date,e=t.toJSON();return{name:"cp:coreProperties",properties:{prefixedAttributes:[{prefix:"xmlns:",map:{cp:"http://schemas.openxmlformats.org/package/2006/metadata/core-properties",dc:"http://purl.org/dc/elements/1.1/",dcterms:"http://purl.org/dc/terms/",dcmitype:"http://purl.org/dc/dcmitype/",xsi:"http://www.w3.org/2001/XMLSchema-instance"}}]},children:[{name:"dc:creator",textNode:i},{name:"dc:title",textNode:"Workbook"},{name:"dcterms:created",properties:{rawMap:{"xsi:type":"dcterms:W3CDTF"}},textNode:e},{name:"dcterms:modified",properties:{rawMap:{"xsi:type":"dcterms:W3CDTF"}},textNode:e}]}}},rz={getTemplate:function(i){var t=i.name,e=i.ContentType,r=i.Extension,n=i.PartName;return{name:t,properties:{rawMap:{Extension:r,PartName:n,ContentType:e}}}}},nz=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},iz=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(nz(arguments[t]));return i},oz={getTemplate:function(i){var t=new Array(i).fill(void 0).map(function(s,l){return{name:"Override",ContentType:"application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml",PartName:"/xl/worksheets/sheet"+(l+1)+".xml"}}),e=ut.worksheetImages.size,r={};ut.workbookImageIds.forEach(function(s){r[s.type]=!0});var n=new Array(e).fill(void 0).map(function(s,l){return{name:"Override",ContentType:"application/vnd.openxmlformats-officedocument.drawing+xml",PartName:"/xl/drawings/drawing"+(l+1)+".xml"}}),o=Object.keys(r).map(function(s){return{name:"Default",ContentType:"image/"+s,Extension:s}}),a=iz(o,[{name:"Default",Extension:"rels",ContentType:"application/vnd.openxmlformats-package.relationships+xml"},{name:"Default",ContentType:"application/xml",Extension:"xml"},{name:"Override",ContentType:"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml",PartName:"/xl/workbook.xml"}],t,[{name:"Override",ContentType:"application/vnd.openxmlformats-officedocument.theme+xml",PartName:"/xl/theme/theme1.xml"},{name:"Override",ContentType:"application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml",PartName:"/xl/styles.xml"},{name:"Override",ContentType:"application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml",PartName:"/xl/sharedStrings.xml"}],n,[{name:"Override",ContentType:"application/vnd.openxmlformats-package.core-properties+xml",PartName:"/docProps/core.xml"}]).map(function(s){return rz.getTemplate(s)});return{name:"Types",properties:{rawMap:{xmlns:"http://schemas.openxmlformats.org/package/2006/content-types"}},children:a}}},gx=function(i,t){return{name:"xdr:"+i,children:[{name:"xdr:col",textNode:t.col.toString()},{name:"xdr:colOff",textNode:t.offsetX.toString()},{name:"xdr:row",textNode:t.row.toString()},{name:"xdr:rowOff",textNode:t.offsetY.toString()}]}},az=function(i){var t=[{name:"a:ext",properties:{rawMap:{uri:"{FF2B5EF4-FFF2-40B4-BE49-F238E27FC236}"}},children:[{name:"a16:creationId",properties:{rawMap:{id:"{822E6D20-D7BC-2841-A643-D49A6EF008A2}","xmlns:a16":"http://schemas.microsoft.com/office/drawing/2014/main"}}}]}],e=i.recolor&&i.recolor.toLowerCase();switch(e){case"grayscale":case"sepia":case"washout":t.push({name:"a:ext",properties:{rawMap:{uri:"{C183D7F6-B498-43B3-948B-1728B52AA6E4}"}},children:[{name:"adec:decorative",properties:{rawMap:{val:"0","xmlns:adec":"http://schemas.microsoft.com/office/drawing/2017/decorative"}}}]})}return{name:"a:extLst",children:t}},sz=function(i,t){return{name:"xdr:nvPicPr",children:[{name:"xdr:cNvPr",properties:{rawMap:{id:t,name:i.id,descr:i.altText!=null?i.altText:void 0}},children:[az(i)]},{name:"xdr:cNvPicPr",properties:{rawMap:{preferRelativeResize:"0"}},children:[{name:"a:picLocks"}]}]}},vx=function(i){if(!(!i.saturation&&!i.tint)){var t=[];return i.saturation&&t.push({name:"a:satMod",properties:{rawMap:{val:i.saturation*1e3}}}),i.tint&&t.push({name:"a:tint",properties:{rawMap:{val:i.tint*1e3}}}),t}},lz=function(i,t){return{name:"a:duotone",children:[{name:"a:prstClr",properties:{rawMap:{val:i.color}},children:vx(i)},{name:"a:srgbClr",properties:{rawMap:{val:t.color}},children:vx(t)}]}},uz=function(i,t){var e;if(i.transparency){var r=Math.min(Math.max(i.transparency,0),100);e=[{name:"a:alphaModFix",properties:{rawMap:{amt:1e5-Math.round(r*1e3)}}}]}if(i.recolor)switch(e||(e=[]),i.recolor.toLocaleLowerCase()){case"grayscale":e.push({name:"a:grayscl"});break;case"sepia":e.push(lz({color:"black"},{color:"D9C3A5",tint:50,saturation:180}));break;case"washout":e.push({name:"a:lum",properties:{rawMap:{bright:"70000",contrast:"-70000"}}});break}return{name:"xdr:blipFill",children:[{name:"a:blip",properties:{rawMap:{cstate:"print","r:embed":"rId"+t,"xmlns:r":"http://schemas.openxmlformats.org/officeDocument/2006/relationships"}},children:e},{name:"a:stretch",children:[{name:"a:fillRect"}]}]}},cz=function(i,t){var e={name:"a:xfrm",children:[{name:"a:off",properties:{rawMap:{x:0,y:0}}},{name:"a:ext",properties:{rawMap:{cx:t.width,cy:t.height}}}]};if(i.rotation){var r=i.rotation;e.properties={rawMap:{rot:Math.min(Math.max(r,0),360)*6e4}}}var n={name:"a:prstGeom",properties:{rawMap:{prst:"rect"}},children:[{name:"a:avLst"}]},o={name:"xdr:spPr",children:[e,n]};return o},pz=function(i){i.fitCell=!!i.fitCell||!i.width||!i.height;var t=i.position,e=t===void 0?{}:t,r=i.fitCell,n=i.width,o=n===void 0?0:n,a=i.height,s=a===void 0?0:a,l=i.totalHeight,u=i.totalWidth,c=e.offsetX,p=c===void 0?0:c,h=e.offsetY,d=h===void 0?0:h,f=e.row,g=f===void 0?1:f,v=e.rowSpan,m=v===void 0?1:v,C=e.column,w=C===void 0?1:C,b=e.colSpan,S=b===void 0?1:b;return{from:{row:g-1,col:w-1,offsetX:$s(p),offsetY:$s(d)},to:{row:g-1+(r?1:m-1),col:w-1+(r?1:S-1),offsetX:$s(o+p),offsetY:$s(s+d)},height:$s(l||s),width:$s(u||o)}},hz=function(i,t,e,r){return{name:"xdr:pic",children:[sz(i,t+1),uz(i,e+1),cz(i,r)]}},dz={getTemplate:function(i){var t=i.sheetIndex,e=ut.worksheetImages.get(t),r=ut.worksheetImageIds.get(t),n=e.map(function(o,a){var s=pz(o);return{name:"xdr:twoCellAnchor",properties:{rawMap:{editAs:"absolute"}},children:[gx("from",s.from),gx("to",s.to),hz(o,a,r.get(o.id).index,s),{name:"xdr:clientData"}]}});return{name:"xdr:wsDr",properties:{rawMap:{"xmlns:a":"http://schemas.openxmlformats.org/drawingml/2006/main","xmlns:xdr":"http://schemas.openxmlformats.org/drawingml/2006/spreadsheetDrawing"}},children:n}}},fz=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Sn=function(i){var t=fz(i,4),e=t[0],r=t[1],n=t[2],o=t[3];return{name:"a:"+e,children:[{name:"a:"+r,properties:{rawMap:{val:n,lastClr:o}}}]}},gz={getTemplate:function(){return{name:"a:clrScheme",properties:{rawMap:{name:"Office"}},children:[Sn(["dk1","sysClr","windowText","000000"]),Sn(["lt1","sysClr","window","FFFFFF"]),Sn(["dk2","srgbClr","44546A"]),Sn(["lt2","srgbClr","E7E6E6"]),Sn(["accent1","srgbClr","4472C4"]),Sn(["accent2","srgbClr","ED7D31"]),Sn(["accent3","srgbClr","A5A5A5"]),Sn(["accent4","srgbClr","FFC000"]),Sn(["accent5","srgbClr","5B9BD5"]),Sn(["accent6","srgbClr","70AD47"]),Sn(["hlink","srgbClr","0563C1"]),Sn(["folHlink","srgbClr","954F72"])]}}},vz=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},te=function(i){var t=vz(i,4),e=t[0],r=t[1],n=t[2],o=t[3];return{name:"a:"+e,properties:{rawMap:{script:n,typeface:r,panose:o}}}},mz={getTemplate:function(){var i=_.utf8_encode;return{name:"a:fontScheme",properties:{rawMap:{name:"Office"}},children:[{name:"a:majorFont",children:[te(["latin","Calibri Light",void 0,"020F0302020204030204"]),te(["ea",""]),te(["cs",""]),te(["font",i("\u6E38\u30B4\u30B7\u30C3\u30AF Light"),"Jpan"]),te(["font",i("\uB9D1\uC740 \uACE0\uB515"),"Hang"]),te(["font",i("\u7B49\u7EBF Light"),"Hans"]),te(["font",i("\u65B0\u7D30\u660E\u9AD4"),"Hant"]),te(["font","Times New Roman","Arab"]),te(["font","Times New Roman","Hebr"]),te(["font","Tahoma","Thai"]),te(["font","Nyala","Ethi"]),te(["font","Vrinda","Beng"]),te(["font","Shruti","Gujr"]),te(["font","MoolBoran","Khmr"]),te(["font","Tunga","Knda"]),te(["font","Raavi","Guru"]),te(["font","Euphemia","Cans"]),te(["font","Plantagenet Cherokee","Cher"]),te(["font","Microsoft Yi Baiti","Yiii"]),te(["font","Microsoft Himalaya","Tibt"]),te(["font","MV Boli","Thaa"]),te(["font","Mangal","Deva"]),te(["font","Gautami","Telu"]),te(["font","Latha","Taml"]),te(["font","Estrangelo Edessa","Syrc"]),te(["font","Kalinga","Orya"]),te(["font","Kartika","Mlym"]),te(["font","DokChampa","Laoo"]),te(["font","Iskoola Pota","Sinh"]),te(["font","Mongolian Baiti","Mong"]),te(["font","Times New Roman","Viet"]),te(["font","Microsoft Uighur","Uigh"]),te(["font","Sylfaen","Geor"]),te(["font","Arial","Armn"]),te(["font","Leelawadee UI","Bugi"]),te(["font","Microsoft JhengHei","Bopo"]),te(["font","Javanese Text","Java"]),te(["font","Segoe UI","Lisu"]),te(["font","Myanmar Text","Mymr"]),te(["font","Ebrima","Nkoo"]),te(["font","Nirmala UI","Olck"]),te(["font","Ebrima","Osma"]),te(["font","Phagspa","Phag"]),te(["font","Estrangelo Edessa","Syrn"]),te(["font","Estrangelo Edessa","Syrj"]),te(["font","Estrangelo Edessa","Syre"]),te(["font","Nirmala UI","Sora"]),te(["font","Microsoft Tai Le","Tale"]),te(["font","Microsoft New Tai Lue","Talu"]),te(["font","Ebrima","Tfng"])]},{name:"a:minorFont",children:[te(["latin","Calibri",void 0,"020F0502020204030204"]),te(["ea",""]),te(["cs",""]),te(["font",i("\u6E38\u30B4\u30B7\u30C3\u30AF"),"Jpan"]),te(["font",i("\uB9D1\uC740 \uACE0\uB515"),"Hang"]),te(["font",i("\u7B49\u7EBF"),"Hans"]),te(["font",i("\u65B0\u7D30\u660E\u9AD4"),"Hant"]),te(["font","Arial","Arab"]),te(["font","Arial","Hebr"]),te(["font","Tahoma","Thai"]),te(["font","Nyala","Ethi"]),te(["font","Vrinda","Beng"]),te(["font","Shruti","Gujr"]),te(["font","DaunPenh","Khmr"]),te(["font","Tunga","Knda"]),te(["font","Raavi","Guru"]),te(["font","Euphemia","Cans"]),te(["font","Plantagenet Cherokee","Cher"]),te(["font","Microsoft Yi Baiti","Yiii"]),te(["font","Microsoft Himalaya","Tibt"]),te(["font","MV Boli","Thaa"]),te(["font","Mangal","Deva"]),te(["font","Gautami","Telu"]),te(["font","Latha","Taml"]),te(["font","Estrangelo Edessa","Syrc"]),te(["font","Kalinga","Orya"]),te(["font","Kartika","Mlym"]),te(["font","DokChampa","Laoo"]),te(["font","Iskoola Pota","Sinh"]),te(["font","Mongolian Baiti","Mong"]),te(["font","Arial","Viet"]),te(["font","Microsoft Uighur","Uigh"]),te(["font","Sylfaen","Geor"]),te(["font","Arial","Armn"]),te(["font","Leelawadee UI","Bugi"]),te(["font","Microsoft JhengHei","Bopo"]),te(["font","Javanese Text","Java"]),te(["font","Segoe UI","Lisu"]),te(["font","Myanmar Text","Mymr"]),te(["font","Ebrima","Nkoo"]),te(["font","Nirmala UI","Olck"]),te(["font","Ebrima","Osma"]),te(["font","Phagspa","Phag"]),te(["font","Estrangelo Edessa","Syrn"]),te(["font","Estrangelo Edessa","Syrj"]),te(["font","Estrangelo Edessa","Syre"]),te(["font","Nirmala UI","Sora"]),te(["font","Microsoft Tai Le","Tale"]),te(["font","Microsoft New Tai Lue","Talu"]),te(["font","Ebrima","Tfng"])]}]}}},Mc=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Ei=function(i,t,e){return{name:"a:"+i,properties:{rawMap:{val:t}},children:e}},Sv=function(i){var t=Mc(i,6),e=t[0],r=t[1],n=t[2],o=t[3],a=t[4],s=t[5],l=[];return l.push(Ei("satMod",n)),o&&l.push(Ei("lumMod",o)),a&&l.push(Ei("tint",a)),s&&l.push(Ei("shade",s)),{name:"a:gs",properties:{rawMap:{pos:e}},children:[{name:"a:schemeClr",properties:{rawMap:{val:r}},children:l}]}},Cd=function(i,t){return{name:"a:solidFill",children:[Ei("schemeClr",i,t)]}},by=function(i){var t=Mc(i,5),e=t[0],r=t[1],n=t[2],o=t[3],a=t[4],s=Mc(a,2),l=s[0],u=s[1];return{name:"a:gradFill",properties:{rawMap:{rotWithShape:e}},children:[{name:"a:gsLst",children:[Sv(r),Sv(n),Sv(o)]},{name:"a:lin",properties:{rawMap:{ang:l,scaled:u}}}]}},xv=function(i){var t=Mc(i,4),e=t[0],r=t[1],n=t[2],o=t[3];return{name:"a:ln",properties:{rawMap:{w:e,cap:r,cmpd:n,algn:o}},children:[Cd("phClr"),Ei("prstDash","solid"),{name:"a:miter",properties:{rawMap:{lim:"800000"}}}]}},Tv=function(i){var t=[];if(i){var e=Mc(i,5),r=e[0],n=e[1],o=e[2],a=e[3],s=e[4];t.push({name:"a:outerShdw",properties:{rawMap:{blurRad:r,dist:n,dir:o,algn:a,rotWithShape:s}},children:[Ei("srgbClr","000000",[Ei("alpha","63000")])]})}return{name:"a:effectStyle",children:[Object.assign({},{name:"a:effectLst"},t.length?{children:t}:{})]}},yz=function(){return{name:"a:fillStyleLst",children:[Cd("phClr"),by(["1",["0","phClr","105000","110000","67000"],["50000","phClr","103000","105000","73000"],["100000","phClr","109000","105000","81000"],["5400000","0"]]),by(["1",["0","phClr","103000","102000","94000"],["50000","phClr","110000","100000",void 0,"100000"],["100000","phClr","120000","99000",void 0,"78000"],["5400000","0"]])]}},Cz=function(){return{name:"a:lnStyleLst",children:[xv(["6350","flat","sng","ctr"]),xv(["12700","flat","sng","ctr"]),xv(["19050","flat","sng","ctr"])]}},wz=function(){return{name:"a:effectStyleLst",children:[Tv(),Tv(),Tv(["57150","19050","5400000","ctr","0"])]}},bz=function(){return{name:"a:bgFillStyleLst",children:[Cd("phClr"),Cd("phClr",[Ei("tint","95000"),Ei("satMod","170000")]),by(["1",["0","phClr","150000","102000","93000","98000"],["50000","phClr","130000","103000","98000","90000"],["100000","phClr","120000",void 0,void 0,"63000"],["5400000","0"]])]}},_z={getTemplate:function(){return{name:"a:fmtScheme",properties:{rawMap:{name:"Office"}},children:[yz(),Cz(),wz(),bz()]}}},Sz={getTemplate:function(){return{name:"a:themeElements",children:[gz.getTemplate(),mz.getTemplate(),_z.getTemplate()]}}},xz={getTemplate:function(){return{name:"a:theme",properties:{prefixedAttributes:[{prefix:"xmlns:",map:{a:"http://schemas.openxmlformats.org/drawingml/2006/main"}}],rawMap:{name:"Office Theme"}},children:[Sz.getTemplate(),{name:"a:objectDefaults"},{name:"a:extraClrSchemeLst"}]}}},Tz=function(i){var t=[];return i.forEach(function(e,r){var n=r.toString(),o={name:"t",textNode:_.utf8_encode(_.escapeString(n))},a=n.trim().length!==n.length;a&&(o.properties={rawMap:{"xml:space":"preserve"}}),t.push({name:"si",children:[o]})}),t},Oz={getTemplate:function(i){return{name:"sst",properties:{rawMap:{xmlns:"http://schemas.openxmlformats.org/spreadsheetml/2006/main",count:i.size,uniqueCount:i.size}},children:Tz(i)}}},Ez={getTemplate:function(i){var t=i.formatCode,e=i.numFmtId;return{name:"numFmt",properties:{rawMap:{formatCode:t,numFmtId:e}}}}},Rz={getTemplate:function(i){return{name:"numFmts",properties:{rawMap:{count:i.length}},children:i.map(function(t){return Ez.getTemplate(t)})}}},Pz={getTemplate:function(i){var t=i.size,e=i.colorTheme,r=i.color,n=r===void 0?"FF000000":r,o=i.fontName,a=o===void 0?"Calibri":o,s=i.family,l=i.scheme,u=i.italic,c=i.bold,p=i.strikeThrough,h=i.outline,d=i.shadow,f=i.underline,g=i.verticalAlign,v=[{name:"sz",properties:{rawMap:{val:t}}},{name:"color",properties:{rawMap:{theme:e,rgb:n}}},{name:"name",properties:{rawMap:{val:a}}}];return s&&v.push({name:"family",properties:{rawMap:{val:s}}}),l&&v.push({name:"scheme",properties:{rawMap:{val:l}}}),u&&v.push({name:"i"}),c&&v.push({name:"b"}),p&&v.push({name:"strike"}),h&&v.push({name:"outline"}),d&&v.push({name:"shadow"}),f&&v.push({name:"u",properties:{rawMap:{val:f}}}),g&&v.push({name:"vertAlign",properties:{rawMap:{val:g}}}),{name:"font",children:v}}},Az={getTemplate:function(i){return{name:"fonts",properties:{rawMap:{count:i.length}},children:i.map(function(t){return Pz.getTemplate(t)})}}},Dz={getTemplate:function(i){var t=i.patternType,e=i.fgTheme,r=i.fgTint,n=i.fgRgb,o=i.bgRgb,a=i.bgIndexed,s={name:"patternFill",properties:{rawMap:{patternType:t}}};return(e||r||n)&&(s.children=[{name:"fgColor",properties:{rawMap:{theme:e,tint:r,rgb:n}}}]),(a||o)&&(s.children||(s.children=[]),s.children.push({name:"bgColor",properties:{rawMap:{indexed:a,rgb:o}}})),{name:"fill",children:[s]}}},Mz={getTemplate:function(i){return{name:"fills",properties:{rawMap:{count:i.length}},children:i.map(function(t){return Dz.getTemplate(t)})}}},Fz=function(i){switch(i){case 1:return"thin";case 2:return"medium";case 3:return"thick";default:return"hair"}},Iz={None:"None",Dot:"Dotted",Dash:"Dashed",Double:"Double",DashDot:"DashDot",DashDotDot:"DashDotDot",SlantDashDot:"SlantDashDot"},Lz=["Dashed","DashDot","DashDotDot"],Nz={None:"none",Solid:"solid",Gray50:"mediumGray",Gray75:"darkGray",Gray25:"lightGray",HorzStripe:"darkHorizontal",VertStripe:"darkVertical",ReverseDiagStripe:"darkDown",DiagStripe:"darkUp",DiagCross:"darkGrid",ThickDiagCross:"darkTrellis",ThinHorzStripe:"lightHorizontal",ThinVertStripe:"lightVertical",ThinReverseDiagStripe:"lightDown",ThinDiagStripe:"lightUp",ThinHorzCross:"lightGrid",ThinDiagCross:"lightTrellis",Gray125:"gray125",Gray0625:"gray0625"},Gz={Automatic:"general",Left:"left",Center:"center",Right:"right",Fill:"fill",Justify:"justify",CenterAcrossSelection:"centerContinuous",Distributed:"distributed",JustifyDistributed:"justify"},kz={Automatic:void 0,Top:"top",Bottom:"bottom",Center:"center",Justify:"justify",Distributed:"distributed",JustifyDistributed:"justify"},Wz=function(i){return i?Nz[i]||i:"none"},qo=function(i){return i==null?i:(i.charAt(0)==="#"&&(i=i.substr(1)),i.length===6?"FF"+i:i)},Qp=function(i,t){if(!i)return"thin";var e=Fz(t),r=Iz[i];return i==="Continuous"?e:e==="medium"&&Lz.indexOf(r)!==-1?"medium"+r:r.charAt(0).toLowerCase()+r.substr(1)},Vz=function(i){return Gz[i]||"general"},Bz=function(i){return kz[i]||void 0},Pu=function(i){return{name:"color",properties:{rawMap:{rgb:qo(i||"#000000")}}}},Hz={getTemplate:function(i){var t=i.left,e=i.right,r=i.top,n=i.bottom,o=i.diagonal,a=t?[Pu(t.color)]:void 0,s=e?[Pu(e.color)]:void 0,l=r?[Pu(r.color)]:void 0,u=n?[Pu(n.color)]:void 0,c=o?[Pu(o.color)]:void 0;return{name:"border",children:[{name:"left",properties:{rawMap:{style:t&&t.style}},children:a},{name:"right",properties:{rawMap:{style:e&&e.style}},children:s},{name:"top",properties:{rawMap:{style:r&&r.style}},children:l},{name:"bottom",properties:{rawMap:{style:n&&n.style}},children:u},{name:"diagonal",properties:{rawMap:{style:o&&o.style}},children:c}]}}},jz={getTemplate:function(i){return{name:"borders",properties:{rawMap:{count:i.length}},children:i.map(function(t){return Hz.getTemplate(t)})}}},Uz=function(i){var t=["Context","LeftToRight","RightToLeft"],e=t.indexOf(i);return Math.max(e,0)},zz={getTemplate:function(i){var t=i.horizontal,e=i.indent,r=i.readingOrder,n=i.rotate,o=i.shrinkToFit,a=i.vertical,s=i.wrapText;return{name:"alignment",properties:{rawMap:{horizontal:t&&Vz(t),indent:e,readingOrder:r&&Uz(r),textRotation:n,shrinkToFit:o,vertical:a&&Bz(a),wrapText:s}}}}},$z={getTemplate:function(i){var t=i.protected===!1?0:1,e=i.hideFormula===!0?1:0;return{name:"protection",properties:{rawMap:{hidden:e,locked:t}}}}},JP={getTemplate:function(i){var t=i.alignment,e=i.borderId,r=i.fillId,n=i.fontId,o=i.numFmtId,a=i.protection,s=i.xfId,l=[];return t&&l.push(zz.getTemplate(t)),a&&l.push($z.getTemplate(a)),{name:"xf",properties:{rawMap:{applyAlignment:t?1:void 0,applyProtection:a?1:void 0,applyBorder:e?1:void 0,applyFill:r?1:void 0,borderId:e,fillId:r,applyFont:n?1:void 0,fontId:n,applyNumberFormat:o?1:void 0,numFmtId:o,xfId:s}},children:l.length?l:void 0}}},Kz={getTemplate:function(i){return{name:"cellStyleXfs",properties:{rawMap:{count:i.length}},children:i.map(function(t){return JP.getTemplate(t)})}}},Yz={getTemplate:function(i){return{name:"cellXfs",properties:{rawMap:{count:i.length}},children:i.map(function(t){return JP.getTemplate(t)})}}},Xz={getTemplate:function(i){var t=i.builtinId,e=i.name,r=i.xfId;return{name:"cellStyle",properties:{rawMap:{builtinId:t,name:e,xfId:r}}}}},qz={getTemplate:function(i){return{name:"cellStyles",properties:{rawMap:{count:i.length}},children:i.map(function(t){return Xz.getTemplate(t)})}}},wd=globalThis&&globalThis.__assign||function(){return wd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},wd.apply(this,arguments)},bd,gl,dc,fc,gc,eA,_d,tA,_y,rA=function(i,t){return i.indexOf("mixedStyle")!==-1&&t>1&&(i+="_"+t),i},nA=function(){bd={base:0},gl=[],dc=[{fontName:"Calibri",colorTheme:"1",family:"2",scheme:"minor"}],fc=[{patternType:"none"},{patternType:"gray125"}],gc=[{left:void 0,right:void 0,top:void 0,bottom:void 0,diagonal:void 0}],eA=[{borderId:0,fillId:0,fontId:0,numFmtId:0}],_d=[{borderId:0,fillId:0,fontId:0,numFmtId:0,xfId:0}],tA=[{builtinId:0,name:"Normal",xfId:0}]},Zz=function(i){var t=Wz(i.pattern),e=qo(i.color),r=qo(i.patternColor),n=fc.findIndex(function(o){var a=o.patternType,s=o.fgRgb,l=o.bgRgb;return!(a!=t||s!=e||l!=r)});return n===-1&&(n=fc.length,fc.push({patternType:t,fgRgb:e,bgRgb:r})),n},Qz=function(i){if(i=_.utf8_encode(i),dx[i])return dx[i];var t=gl.findIndex(function(e){return e.formatCode===i});return t===-1?(t=gl.length+164,gl.push({formatCode:i,numFmtId:t})):t=gl[t].numFmtId,t},Jz=function(i){var t=i.borderBottom,e=i.borderTop,r=i.borderLeft,n=i.borderRight,o,a,s,l,u,c,p,h;r&&(s=Qp(r.lineStyle,r.weight),p=qo(r.color)),n&&(l=Qp(n.lineStyle,n.weight),h=qo(n.color)),t&&(o=Qp(t.lineStyle,t.weight),u=qo(t.color)),e&&(a=Qp(e.lineStyle,e.weight),c=qo(e.color));var d=gc.findIndex(function(f){var g=f.left,v=f.right,m=f.top,C=f.bottom;if(!g&&(s||p)||!v&&(l||h)||!m&&(a||c)||!C&&(o||u))return!1;var w=g||{},b=w.style,S=w.color,T=v||{},E=T.style,P=T.color,R=m||{},M=R.style,N=R.color,W=C||{},G=W.style,k=W.color;return!(b!=s||S!=p||E!=l||P!=h||M!=a||N!=c||G!=o||k!=u)});return d===-1&&(d=gc.length,gc.push({left:{style:s,color:p},right:{style:l,color:h},top:{style:a,color:c},bottom:{style:o,color:u},diagonal:{style:void 0,color:void 0}})),d},e5=function(i){var t=i.fontName,e=t===void 0?"Calibri":t,r=i.color,n=i.size,o=i.bold,a=i.italic,s=i.outline,l=i.shadow,u=i.strikeThrough,c=i.underline,p=i.family,h=i.verticalAlign,d=e&&_.utf8_encode(e),f=qo(r),g=YU(p),v=c?c.toLocaleLowerCase():void 0,m=h?h.toLocaleLowerCase():void 0,C=dc.findIndex(function(w){return!(w.fontName!=d||w.color!=f||w.size!=n||w.bold!=o||w.italic!=a||w.outline!=s||w.shadow!=l||w.strikeThrough!=u||w.underline!=v||w.verticalAlign!=m||w.family!=g)});return C===-1&&(C=dc.length,dc.push({fontName:d,color:f,size:n,bold:o,italic:a,outline:s,shadow:l,strikeThrough:u,underline:v,verticalAlign:m,family:g!=null?g.toString():void 0})),C},t5=function(i){var t=i.alignment,e=i.borders,r=i.font,n=i.interior,o=i.numberFormat,a=i.protection,s=i.id,l=0,u=0,c=0,p=0;!s||(s=rA(s,_y),bd[s]==null&&(n&&(l=Zz(n)),e&&(u=Jz(e)),r&&(c=e5(r)),o&&(p=Qz(o.format)),bd[s]=_d.length,_d.push({alignment:t,borderId:u||0,fillId:l||0,fontId:c||0,numFmtId:p||0,protection:a,xfId:0})))},r5={getTemplate:function(i){var t=Rz.getTemplate(gl),e=Az.getTemplate(dc.map(function(l){return wd(wd({},l),{size:l.size!=null?l.size:i})})),r=Mz.getTemplate(fc),n=jz.getTemplate(gc),o=Kz.getTemplate(eA),a=Yz.getTemplate(_d),s=qz.getTemplate(tA);return nA(),{name:"styleSheet",properties:{rawMap:{"mc:Ignorable":"x14ac x16r2 xr",xmlns:"http://schemas.openxmlformats.org/spreadsheetml/2006/main","xmlns:mc":"http://schemas.openxmlformats.org/markup-compatibility/2006","xmlns:x14ac":"http://schemas.microsoft.com/office/spreadsheetml/2009/9/ac","xmlns:x16r2":"http://schemas.microsoft.com/office/spreadsheetml/2015/02/main","xmlns:xr":"http://schemas.microsoft.com/office/spreadsheetml/2014/revision"}},children:[t,e,r,n,o,a,s,{name:"tableStyles",properties:{rawMap:{count:0,defaultPivotStyle:"PivotStyleLight16",defaultTableStyle:"TableStyleMedium2"}}}]}}},n5=function(i,t){return bd[rA(i,t)]||0},i5=function(i,t){_y=t,_y===1&&nA(),i.forEach(t5)},o5={getTemplate:function(i,t){var e=(t+1).toString();return{name:"sheet",properties:{rawMap:{name:i,sheetId:e,"r:id":"rId"+e}}}}},a5={getTemplate:function(i){return{name:"sheets",children:i.map(function(t,e){return o5.getTemplate(t,e)})}}},s5={getTemplate:function(i){return{name:"workbook",properties:{prefixedAttributes:[{prefix:"xmlns:",map:{r:"http://schemas.openxmlformats.org/officeDocument/2006/relationships"}}],rawMap:{xmlns:"http://schemas.openxmlformats.org/spreadsheetml/2006/main"}},children:[a5.getTemplate(i)]}}},l5=function(i){return Math.ceil((i-12)/7+1)},u5={getTemplate:function(i){var t=i.min,e=i.max,r=i.outlineLevel,n=i.s,o=i.width,a=i.hidden,s=i.bestFit,l=1,u="0";return o>1&&(l=l5(o),u="1"),{name:"col",properties:{rawMap:{min:t,max:e,outlineLevel:r!=null?r:void 0,width:l,style:n,hidden:a?"1":"0",bestFit:s?"1":"0",customWidth:u}}}}},c5=function(i){var t=i.charAt(0).toLowerCase();return t==="s"?"inlineStr":t},p5={getTemplate:function(i,t,e){var r=i.ref,n=i.data,o=i.styleId,a=n.type,s=n.value,l=a;a==="f"?l="str":a.charAt(0)===a.charAt(0).toUpperCase()&&(l=c5(a));var u={name:"c",properties:{rawMap:{r,t:l==="empty"?void 0:l,s:o?n5(o,e):void 0}}};if(l==="empty")return u;var c;return l==="str"&&a==="f"?c=[{name:"f",textNode:_.escapeString(_.utf8_encode(s))}]:l==="inlineStr"?c=[{name:"is",children:[{name:"t",textNode:_.escapeString(_.utf8_encode(s))}]}]:c=[{name:"v",textNode:s}],Object.assign({},u,{children:c})}},h5=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},d5=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(h5(arguments[t]));return i},f5=function(i,t){for(var e=[],r=0,n=0;n<i.length;n++){var o=i[n];o.mergeAcross&&(e.push({pos:n,excelPos:r}),r+=o.mergeAcross),r++}if(e.length)for(var n=e.length-1;n>=0;n--){for(var a=[],o=i[e[n].pos],s=1;s<=o.mergeAcross;s++)a.push({ref:""+Dc(e[n].excelPos+1+s)+(t+1),styleId:o.styleId,data:{type:"empty",value:null}});a.length&&i.splice.apply(i,d5([e[n].pos+1,0],a))}},g5=function(i){return i.data.value!==""||i.styleId!==void 0},v5={getTemplate:function(i,t,e){var r=i.index,n=i.collapsed,o=i.hidden,a=i.height,s=i.s,l=i.cells,u=l===void 0?[]:l;f5(u,t);var c=u.filter(g5).map(function(p,h){return p5.getTemplate(p,h,e)});return{name:"row",properties:{rawMap:{r,collapsed:n,hidden:o?"1":"0",ht:a,customHeight:a!=null?"1":"0",s,customFormat:s!=null?"1":"0"}},children:c}}},m5={getTemplate:function(i){return{name:"mergeCell",properties:{rawMap:{ref:i}}}}},y5=function(i,t){var e=[],r=[];i.forEach(function(a,s){var l=a.cells,u=0;a.index=s+1;var c;l.forEach(function(p,h){var d=h+u+1,f=Dc(d),g=s+1;if(p.mergeAcross){u+=p.mergeAcross;var v=Dc(h+u+1);e.push(""+f+g+":"+v+g)}t[d-1]||(t[d-1]={});var m=p.collapsibleRanges;m&&m.forEach(function(C){r.push([d+C[0],d+C[1]])}),c=t[d-1],c.min=d,c.max=d,p.ref=""+f+g})}),r.sort(function(a,s){return a[0]!==s[0]?a[0]-s[0]:s[1]-a[1]});var n=new Map,o=new Map;return r.filter(function(a){var s=a.toString(),l=n.get(s);return l?!1:(n.set(s,!0),!0)}).forEach(function(a){var s=t.find(function(u){return u.min==a[0]&&u.max==a[1]}),l=o.get(a[0]);t.push({min:a[0],max:a[1],outlineLevel:l||1,width:(s||{width:100}).width}),o.set(a[0],(l||0)+1)}),e},C5=function(i){return!i||i!=="Portrait"&&i!=="Landscape"?"portrait":i.toLocaleLowerCase()},w5=function(i){if(i==null)return 1;var t=["Letter","Letter Small","Tabloid","Ledger","Legal","Statement","Executive","A3","A4","A4 Small","A5","A6","B4","B5","Folio","Envelope","Envelope DL","Envelope C5","Envelope B5","Envelope C3","Envelope C4","Envelope C6","Envelope Monarch","Japanese Postcard","Japanese Double Postcard"],e=t.indexOf(i);return e===-1?1:e+1},b5=function(i){return function(t){return i.length&&t.push({name:"cols",children:i.map(function(e){return u5.getTemplate(e)})}),t}},_5=function(i,t){return function(e){return i.length&&e.push({name:"sheetData",children:i.map(function(r,n){return v5.getTemplate(r,n,t)})}),e}},S5=function(i){return function(t){return i.length&&t.push({name:"mergeCells",properties:{rawMap:{count:i.length}},children:i.map(function(e){return m5.getTemplate(e)})}),t}},x5=function(i){return function(t){var e=i.top,r=e===void 0?.75:e,n=i.right,o=n===void 0?.7:n,a=i.bottom,s=a===void 0?.75:a,l=i.left,u=l===void 0?.7:l,c=i.header,p=c===void 0?.3:c,h=i.footer,d=h===void 0?.3:h;return t.push({name:"pageMargins",properties:{rawMap:{bottom:s,footer:d,header:p,left:u,right:o,top:r}}}),t}},T5=function(i){return function(t){return i&&t.push({name:"pageSetup",properties:{rawMap:{horizontalDpi:0,verticalDpi:0,orientation:C5(i.orientation),paperSize:w5(i.pageSize)}}}),t}},O5=function(i){var t={"&[Page]":"&P","&[Pages]":"&N","&[Date]":"&D","&[Time]":"&T","&[Tab]":"&A","&[Path]":"&Z","&[File]":"&F"};return _.iterateObject(t,function(e,r){i=i.replace(e,r)}),i},E5=function(i){return i==="Center"?"C":i==="Right"?"R":"L"},R5=function(i,t){return t&&(i+="&amp;&quot;",i+=t.fontName||"Calibri",t.bold!==t.italic?i+=t.bold?",Bold":",Italic":t.bold?i+=",Bold Italic":i+=",Regular",i+="&quot;",t.size&&(i+="&amp;"+t.size),t.strikeThrough&&(i+="&amp;S"),t.underline&&(i+="&amp;"+(t.underline==="Double"?"E":"U")),t.color&&(i+="&amp;K"+t.color.replace("#","").toUpperCase())),i},P5=function(i){return i.reduce(function(t,e){var r=E5(e.position),n=R5(t+"&amp;"+r,e.font);return""+n+_.escapeString(O5(e.value))},"")},A5=function(i){var t=["all","first","even"],e=[];return t.forEach(function(r){var n=i[r],o=r==="all"?"odd":r;!n||!n.header&&!n.footer||_.iterateObject(n,function(a,s){var l=""+a.charAt(0).toUpperCase()+a.slice(1);s&&e.push({name:""+o+l,properties:{rawMap:{"xml:space":"preserve"}},textNode:P5(s)})})}),e},D5=function(i){return function(t){if(!i)return t;var e=i.first!=null?1:0,r=i.even!=null?1:0;return t.push({name:"headerFooter",properties:{rawMap:{differentFirst:e,differentOddEven:r}},children:A5(i)}),t}},M5=function(i){return function(t){return ut.worksheetImages.get(i)&&t.push({name:"drawing",properties:{rawMap:{"r:id":"rId1"}}}),t}},F5={getTemplate:function(i){var t=i.worksheet,e=i.currentSheet,r=i.margins,n=r===void 0?{}:r,o=i.pageSetup,a=i.headerFooterConfig,s=t.table,l=s.rows,u=s.columns,c=u&&u.length?y5(l,u):[],p=_.compose(b5(u),_5(l,e+1),S5(c),x5(n),T5(o),D5(a),M5(e)),h=p([]);return{name:"worksheet",properties:{prefixedAttributes:[{prefix:"xmlns:",map:{r:"http://schemas.openxmlformats.org/officeDocument/2006/relationships"}}],rawMap:{xmlns:"http://schemas.openxmlformats.org/spreadsheetml/2006/main"}},children:h}}},I5={getTemplate:function(i){var t=i.Id,e=i.Type,r=i.Target;return{name:"Relationship",properties:{rawMap:{Id:t,Type:e,Target:r}}}}},Jp={getTemplate:function(i){var t=i.map(function(e){return I5.getTemplate(e)});return{name:"Relationships",properties:{rawMap:{xmlns:"http://schemas.openxmlformats.org/package/2006/relationships"}},children:t}}},L5=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},N5=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(L5(arguments[t]));return i},ut=function(){function i(){}return i.createExcel=function(t,e,r,n,o){return this.addSheetName(e),i5(t,this.sheetNames.length),this.createWorksheet(e,r,n,o)},i.buildImageMap=function(t,e,r,n,o){var a=this.sheetNames.length,s=this.images.get(t.id);(!t.position||!t.position.row||!t.position.column)&&(t.position||(t.position={}),t.position=Object.assign({},t.position,{row:e,column:n.indexOf(r)+1}));var l=t;if(XU(l,n),qU(l,o),s){var u=s.find(function(c){return c.sheetId===a});u?u.image.push(l):s.push({sheetId:a,image:[l]})}else this.images.set(l.id,[{sheetId:a,image:[l]}]),this.workbookImageIds.set(l.id,{type:l.imageType,index:this.workbookImageIds.size});this.buildSheetImageMap(a,l)},i.buildSheetImageMap=function(t,e){var r=this.worksheetImageIds.get(t);r||(r=new Map,this.worksheetImageIds.set(t,r));var n=this.worksheetImages.get(t);n?(n.push(e),r.get(e.id)||r.set(e.id,{index:r.size,type:e.imageType})):(this.worksheetImages.set(t,[e]),r.set(e.id,{index:0,type:e.imageType}))},i.addSheetName=function(t){for(var e=_.escapeString(t.name)||"",r="";this.sheetNames.indexOf(""+e+r)!==-1;)if(r==="")r="_1";else{var n=parseInt(r.slice(1),10);r="_"+(n+1)}t.name=""+e+r,this.sheetNames.push(t.name)},i.getStringPosition=function(t){return this.sharedStrings.has(t)?this.sharedStrings.get(t):(this.sharedStrings.set(t,this.sharedStrings.size),this.sharedStrings.size-1)},i.resetFactory=function(){this.sharedStrings=new Map,this.images=new Map,this.worksheetImages=new Map,this.workbookImageIds=new Map,this.worksheetImageIds=new Map,this.sheetNames=[],this.factoryMode=to.SINGLE_SHEET},i.createWorkbook=function(){return _n(s5.getTemplate(this.sheetNames))},i.createStylesheet=function(t){return _n(r5.getTemplate(t))},i.createSharedStrings=function(){return _n(Oz.getTemplate(this.sharedStrings))},i.createCore=function(t){return _n(tz.getTemplate(t))},i.createContentTypes=function(t){return _n(oz.getTemplate(t))},i.createRels=function(){var t=Jp.getTemplate([{Id:"rId1",Type:"http://schemas.openxmlformats.org/officeDocument/2006/relationships/officeDocument",Target:"xl/workbook.xml"},{Id:"rId2",Type:"http://schemas.openxmlformats.org/package/2006/relationships/metadata/core-properties",Target:"docProps/core.xml"}]);return _n(t)},i.createTheme=function(){return _n(xz.getTemplate())},i.createWorkbookRels=function(t){var e=new Array(t).fill(void 0).map(function(n,o){return{Id:"rId"+(o+1),Type:"http://schemas.openxmlformats.org/officeDocument/2006/relationships/worksheet",Target:"worksheets/sheet"+(o+1)+".xml"}}),r=Jp.getTemplate(N5(e,[{Id:"rId"+(t+1),Type:"http://schemas.openxmlformats.org/officeDocument/2006/relationships/theme",Target:"theme/theme1.xml"},{Id:"rId"+(t+2),Type:"http://schemas.openxmlformats.org/officeDocument/2006/relationships/styles",Target:"styles.xml"},{Id:"rId"+(t+3),Type:"http://schemas.openxmlformats.org/officeDocument/2006/relationships/sharedStrings",Target:"sharedStrings.xml"}]));return _n(r)},i.createDrawing=function(t){return _n(dz.getTemplate({sheetIndex:t}))},i.createDrawingRel=function(t){var e=this,r=this.worksheetImageIds.get(t),n=[];return r.forEach(function(o,a){n.push({Id:"rId"+(o.index+1),Type:"http://schemas.openxmlformats.org/officeDocument/2006/relationships/image",Target:"../media/image"+(e.workbookImageIds.get(a).index+1)+"."+o.type})}),_n(Jp.getTemplate(n))},i.createWorksheetDrawingRel=function(t){var e=Jp.getTemplate([{Id:"rId1",Type:"http://schemas.openxmlformats.org/officeDocument/2006/relationships/drawing",Target:"../drawings/drawing"+(t+1)+".xml"}]);return _n(e)},i.createWorksheet=function(t,e,r,n){return _n(F5.getTemplate({worksheet:t,currentSheet:this.sheetNames.length-1,margins:e,pageSetup:r,headerFooterConfig:n}))},i.sharedStrings=new Map,i.sheetNames=[],i.images=new Map,i.worksheetImages=new Map,i.workbookImageIds=new Map,i.worksheetImageIds=new Map,i.factoryMode=to.SINGLE_SHEET,i}(),G5=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),k5=function(i){G5(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.createExcel=function(e){var r=this,n=r.excelStyles,o=r.config,a=o.margins,s=o.pageSetup,l=o.headerFooterConfig;return ut.createExcel(n,e,a,s,l)},t.prototype.getDataTypeForValue=function(e){return e===void 0?"empty":_.isNumeric(e)?"n":"s"},t.prototype.getType=function(e,r,n){if(this.isFormula(n))return"f";if(r&&r.dataType)switch(r.dataType.toLocaleLowerCase()){case"formula":return"f";case"string":return"s";case"number":return"n";case"datetime":return"d";case"error":return"e";case"boolean":return"b";default:console.warn("AG Grid: Unrecognized data type for excel export ["+r.id+".dataType="+r.dataType+"]")}return e},t.prototype.addImage=function(e,r,n){if(!!this.config.addImageToCell){var o=this.config.addImageToCell(e,r,n);if(!!o)return ut.buildImageMap(o.image,e,r,this.columnsToExport,this.config.rowHeight),o}},t.prototype.createCell=function(e,r,n){var o=this.getStyleById(e),a=this.getType(r,o,n)||r;return{styleId:o?e:void 0,data:{type:a,value:this.getCellValue(a,n)}}},t.prototype.createMergedCell=function(e,r,n,o){var a=n==null?"":n;return{styleId:this.getStyleById(e)?e:void 0,data:{type:r,value:r==="s"?ut.getStringPosition(a).toString():n},mergeAcross:o}},t.prototype.getCellValue=function(e,r){if(r==null)return ut.getStringPosition("").toString();switch(e){case"s":return r===""?"":ut.getStringPosition(r).toString();case"f":return r.slice(1);case"n":return Number(r).toString();default:return r}},t}(QP),W5=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Sd=globalThis&&globalThis.__assign||function(){return Sd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Sd.apply(this,arguments)},Oo=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Sy=function(i){var t=i.data,e=i.fontSize,r=e===void 0?11:e,n=i.author,o=n===void 0?"AG Grid":n,a=ut.images.size>0;if(dr.addFolders(["_rels/","docProps/","xl/","xl/theme/","xl/_rels/","xl/worksheets/"]),a){dr.addFolders(["xl/worksheets/_rels","xl/drawings/","xl/drawings/_rels","xl/media/"]);var s=0;ut.images.forEach(function(p){var h=p[0].image[0],d=h.imageType;dr.addFile("xl/media/image"+ ++s+"."+d,h.base64,!0)})}if(!t||t.length===0){console.warn("AG Grid: Invalid params supplied to getMultipleSheetsAsExcel() - `ExcelExportParams.data` is empty."),ut.resetFactory();return}var l=t.length,u=0;t.forEach(function(p,h){dr.addFile("xl/worksheets/sheet"+(h+1)+".xml",p),a&&ut.worksheetImages.get(h)&&B5(h,u++)}),dr.addFile("xl/workbook.xml",ut.createWorkbook()),dr.addFile("xl/styles.xml",ut.createStylesheet(r)),dr.addFile("xl/sharedStrings.xml",ut.createSharedStrings()),dr.addFile("xl/theme/theme1.xml",ut.createTheme()),dr.addFile("xl/_rels/workbook.xml.rels",ut.createWorkbookRels(l)),dr.addFile("docProps/core.xml",ut.createCore(o)),dr.addFile("[Content_Types].xml",ut.createContentTypes(l)),dr.addFile("_rels/.rels",ut.createRels()),ut.resetFactory();var c=i.mimeType||"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";return dr.getContent(c)},V5=function(i){var t=i.fileName,e=t===void 0?"export.xlsx":t,r=Sy(i);r&&zw.download(e,r)},B5=function(i,t){var e="xl/drawings",r=e+"/drawing"+(t+1)+".xml",n=e+"/_rels/drawing"+(t+1)+".xml.rels",o="xl/worksheets/_rels/sheet"+(i+1)+".xml.rels";dr.addFile(n,ut.createDrawingRel(i)),dr.addFile(r,ut.createDrawing(i)),dr.addFile(o,ut.createWorksheetDrawingRel(t))},H5=function(i){W5(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.exportMode="xlsx",e}return t.prototype.postConstruct=function(){this.setBeans({gridSerializer:this.gridSerializer,gridOptionsWrapper:this.gridOptionsWrapper})},t.prototype.getMergedParams=function(e){var r=this.gridOptionsWrapper.getDefaultExportParams("excel");return Object.assign({},r,e)},t.prototype.getData=function(e){return this.setExportMode(e.exportMode||"xlsx"),i.prototype.getData.call(this,e)},t.prototype.export=function(e){if(this.isExportSuppressed())return console.warn("AG Grid: Export cancelled. Export is not allowed as per your configuration."),"";var r=this.getMergedParams(e),n=this.getData(r),o={data:[n],fontSize:r.fontSize,author:r.author,mimeType:r.mimeType},a=this.packageFile(o);return a&&zw.download(this.getFileName(r.fileName),a),n},t.prototype.exportDataAsExcel=function(e){return this.export(e)},t.prototype.getDataAsExcel=function(e){var r=this.getMergedParams(e),n=this.getData(r);if(e&&e.exportMode==="xml")return n;var o={data:[n],fontSize:r.fontSize,author:r.author,mimeType:r.mimeType};return this.packageFile(o)},t.prototype.setFactoryMode=function(e,r){r===void 0&&(r="xlsx");var n=r==="xlsx"?ut:wy;n.factoryMode=e},t.prototype.getFactoryMode=function(e){var r=e==="xlsx"?ut:wy;return r.factoryMode},t.prototype.getSheetDataForExcel=function(e){var r=this.getMergedParams(e),n=this.getData(r);return n},t.prototype.getMultipleSheetsAsExcel=function(e){return Sy(e)},t.prototype.exportMultipleSheetsAsExcel=function(e){return V5(e)},t.prototype.getDefaultFileName=function(){return"export."+this.getExportMode()},t.prototype.getDefaultFileExtension=function(){return this.getExportMode()},t.prototype.createSerializingSession=function(e){var r=this,n=r.columnModel,o=r.valueService,a=r.gridOptionsWrapper,s=this.getExportMode()==="xlsx",l="ag-grid";e.sheetName!=null&&(l=_.utf8_encode(e.sheetName.toString().substr(0,31)));var u=Sd(Sd({},e),{sheetName:l,columnModel:n,valueService:o,gridOptionsWrapper:a,headerRowHeight:e.headerRowHeight||e.rowHeight,baseExcelStyles:this.gridOptions.excelStyles||[],styleLinker:this.styleLinker.bind(this)});return new(s?k5:ez)(u)},t.prototype.styleLinker=function(e,r,n,o,a){if(e===rs.HEADER)return["header"];if(e===rs.HEADER_GROUPING)return["header","headerGroup"];var s=this.gridOptions.excelStyles,l=["cell"];if(!s||!s.length)return l;var u=s.map(function(c){return c.id});return this.stylingService.processAllCellClasses(o.getColDef(),{value:n,data:a.data,node:a,colDef:o.getColDef(),rowIndex:r,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext()},function(c){u.indexOf(c)>-1&&l.push(c)}),l.sort(function(c,p){return u.indexOf(c)<u.indexOf(p)?-1:1})},t.prototype.isExportSuppressed=function(){return this.gridOptionsWrapper.isSuppressExcelExport()},t.prototype.setExportMode=function(e){this.exportMode=e},t.prototype.getExportMode=function(){return this.exportMode},t.prototype.packageFile=function(e){if(this.getExportMode()==="xml"){var r=e.mimeType||"application/vnd.ms-excel";return new Blob(["\uFEFF",e.data[0]],{type:r})}return Sy(e)},Oo([y("columnModel")],t.prototype,"columnModel",void 0),Oo([y("valueService")],t.prototype,"valueService",void 0),Oo([y("gridOptions")],t.prototype,"gridOptions",void 0),Oo([y("stylingService")],t.prototype,"stylingService",void 0),Oo([y("gridSerializer")],t.prototype,"gridSerializer",void 0),Oo([y("gridOptionsWrapper")],t.prototype,"gridOptionsWrapper",void 0),Oo([B],t.prototype,"postConstruct",null),t=Oo([Q("excelCreator")],t),t}(BP),j5={moduleName:oe.ExcelExportModule,beans:[H5,UP,jP],dependantModules:[$w,cr]},U5=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Au=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},xi;(function(i){i[i.EXPANDED=0]="EXPANDED",i[i.COLLAPSED=1]="COLLAPSED",i[i.INDETERMINATE=2]="INDETERMINATE"})(xi||(xi={}));var z5=function(i){U5(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.preConstruct=function(){this.setTemplate(`<div class="ag-filter-toolpanel-search" role="presentation">
                <div ref="eExpand" class="ag-filter-toolpanel-expand"></div>
                <ag-input-text-field ref="eFilterTextField" class="ag-filter-toolpanel-search-input"></ag-input-text-field>
            </div>`)},t.prototype.postConstruct=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc();this.eFilterTextField.onValueChange(this.onSearchTextChanged.bind(this)),this.eFilterTextField.setInputAriaLabel(e("ariaFilterColumnsInput","Filter Columns Input")),this.createExpandIcons(),this.setExpandState(xi.EXPANDED),this.addManagedListener(this.eExpand,"click",this.onExpandClicked.bind(this)),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.showOrHideOptions.bind(this))},t.prototype.init=function(e){this.params=e,this.columnModel.isReady()&&this.showOrHideOptions()},t.prototype.createExpandIcons=function(){this.eExpand.appendChild(this.eExpandChecked=_.createIconNoSpan("columnSelectOpen",this.gridOptionsWrapper)),this.eExpand.appendChild(this.eExpandUnchecked=_.createIconNoSpan("columnSelectClosed",this.gridOptionsWrapper)),this.eExpand.appendChild(this.eExpandIndeterminate=_.createIconNoSpan("columnSelectIndeterminate",this.gridOptionsWrapper))},t.prototype.showOrHideOptions=function(){var e=!this.params.suppressFilterSearch,r=!this.params.suppressExpandAll,n=this.gridOptionsWrapper.getLocaleTextFunc();this.eFilterTextField.setInputPlaceholder(n("searchOoo","Search..."));var o=function(s){return s.getOriginalParent()&&s.isFilterAllowed()},a=this.columnModel.getAllGridColumns().some(o);_.setDisplayed(this.eFilterTextField.getGui(),e),_.setDisplayed(this.eExpand,r&&a)},t.prototype.onSearchTextChanged=function(){var e=this;this.onSearchTextChangedDebounced||(this.onSearchTextChangedDebounced=_.debounce(function(){e.dispatchEvent({type:"searchChanged",searchText:e.eFilterTextField.getValue()})},300)),this.onSearchTextChangedDebounced()},t.prototype.onExpandClicked=function(){var e=this.currentExpandState===xi.EXPANDED?{type:"collapseAll"}:{type:"expandAll"};this.dispatchEvent(e)},t.prototype.setExpandState=function(e){this.currentExpandState=e,_.setDisplayed(this.eExpandChecked,this.currentExpandState===xi.EXPANDED),_.setDisplayed(this.eExpandUnchecked,this.currentExpandState===xi.COLLAPSED),_.setDisplayed(this.eExpandIndeterminate,this.currentExpandState===xi.INDETERMINATE)},Au([y("columnModel")],t.prototype,"columnModel",void 0),Au([I("eExpand")],t.prototype,"eExpand",void 0),Au([I("eFilterTextField")],t.prototype,"eFilterTextField",void 0),Au([ss],t.prototype,"preConstruct",null),Au([B],t.prototype,"postConstruct",null),t}(ee),$5=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Eo=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},iA=function(i){$5(t,i);function t(e){e===void 0&&(e=!1);var r=i.call(this,t.TEMPLATE)||this;return r.expanded=!1,r.hideHeader=e,r}return t.prototype.postConstruct=function(){this.eExpandChecked=_.createIconNoSpan("columnSelectOpen",this.gridOptionsWrapper),this.eExpandUnchecked=_.createIconNoSpan("columnSelectClosed",this.gridOptionsWrapper),this.eExpand.appendChild(this.eExpandChecked),this.eExpand.appendChild(this.eExpandUnchecked)},t.prototype.setColumn=function(e){var r=this;this.column=e,this.eFilterName.innerText=this.columnModel.getDisplayNameForColumn(this.column,"filterToolPanel",!1)||"",this.addManagedListener(this.eFilterToolPanelHeader,"click",this.toggleExpanded.bind(this)),this.addManagedListener(this.eFilterToolPanelHeader,"keydown",function(n){n.key===j.ENTER&&r.toggleExpanded()}),this.addManagedListener(this.eventService,x.EVENT_FILTER_OPENED,this.onFilterOpened.bind(this)),this.addInIcon("filter",this.eFilterIcon,this.column),this.eFilterIcon.classList.toggle("ag-hidden",!this.isFilterActive()),this.eExpandChecked.classList.add("ag-hidden"),this.hideHeader?(this.eFilterToolPanelHeader.classList.toggle("ag-hidden",!0),this.eFilterToolPanelHeader.removeAttribute("tabindex")):this.eFilterToolPanelHeader.setAttribute("tabindex","0"),this.addManagedListener(this.column,Se.EVENT_FILTER_CHANGED,this.onFilterChanged.bind(this))},t.prototype.getColumn=function(){return this.column},t.prototype.getColumnFilterName=function(){return this.columnModel.getDisplayNameForColumn(this.column,"filterToolPanel",!1)},t.prototype.addCssClassToTitleBar=function(e){this.eFilterToolPanelHeader.classList.add(e)},t.prototype.addInIcon=function(e,r,n){if(r!=null){var o=_.createIconNoSpan(e,this.gridOptionsWrapper,n);r.appendChild(o)}},t.prototype.isFilterActive=function(){return this.filterManager.isFilterActive(this.column)},t.prototype.onFilterChanged=function(){this.eFilterIcon.classList.toggle("ag-hidden",!this.isFilterActive()),this.dispatchEvent({type:Se.EVENT_FILTER_CHANGED})},t.prototype.toggleExpanded=function(){this.expanded?this.collapse():this.expand()},t.prototype.expand=function(){var e=this,r;if(!this.expanded){this.expanded=!0,_.setAriaExpanded(this.eFilterToolPanelHeader,!0);var n=_.loadTemplate('<div class="ag-filter-toolpanel-instance-filter"></div>'),o=(r=this.filterManager.getOrCreateFilterWrapper(this.column,"TOOLBAR"))===null||r===void 0?void 0:r.filterPromise;o&&o.then(function(a){e.underlyingFilter=a,a&&(n.appendChild(a.getGui()),e.agFilterToolPanelBody.appendChild(n),a.afterGuiAttached&&a.afterGuiAttached({container:"toolPanel"}))}),_.setDisplayed(this.eExpandChecked,!0),_.setDisplayed(this.eExpandUnchecked,!1)}},t.prototype.collapse=function(){!this.expanded||(this.expanded=!1,_.setAriaExpanded(this.eFilterToolPanelHeader,!1),this.agFilterToolPanelBody.removeChild(this.agFilterToolPanelBody.children[0]),_.setDisplayed(this.eExpandChecked,!1),_.setDisplayed(this.eExpandUnchecked,!0))},t.prototype.refreshFilter=function(){if(!!this.expanded){var e=this.underlyingFilter;!e||typeof e.refreshVirtualList=="function"&&e.refreshVirtualList()}},t.prototype.onFilterOpened=function(e){e.source==="COLUMN_MENU"&&e.column===this.column&&(!this.expanded||this.collapse())},t.TEMPLATE=`
        <div class="ag-filter-toolpanel-instance">
            <div class="ag-filter-toolpanel-header ag-filter-toolpanel-instance-header" ref="eFilterToolPanelHeader" role="button" aria-expanded="false">
                <div ref="eExpand" class="ag-filter-toolpanel-expand"></div>
                <span ref="eFilterName" class="ag-header-cell-text"></span>
                <span ref="eFilterIcon" class="ag-header-icon ag-filter-icon ag-filter-toolpanel-instance-header-icon" aria-hidden="true"></span>
            </div>
            <div class="ag-filter-toolpanel-instance-body ag-filter" ref="agFilterToolPanelBody"></div>
        </div>`,Eo([I("eFilterToolPanelHeader")],t.prototype,"eFilterToolPanelHeader",void 0),Eo([I("eFilterName")],t.prototype,"eFilterName",void 0),Eo([I("agFilterToolPanelBody")],t.prototype,"agFilterToolPanelBody",void 0),Eo([I("eFilterIcon")],t.prototype,"eFilterIcon",void 0),Eo([I("eExpand")],t.prototype,"eExpand",void 0),Eo([y("filterManager")],t.prototype,"filterManager",void 0),Eo([y("columnModel")],t.prototype,"columnModel",void 0),Eo([B],t.prototype,"postConstruct",null),t}(ee),K5=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),eh=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Ks=function(i){K5(t,i);function t(e,r,n,o,a){var s=i.call(this)||this;return s.columnGroup=e,s.childFilterComps=r,s.depth=o,s.expandedCallback=n,s.showingColumn=a,s}return t.prototype.preConstruct=function(){var e={cssIdentifier:"filter-toolpanel",direction:"vertical"};this.setTemplate(t.TEMPLATE,{filterGroupComp:e})},t.prototype.init=function(){var e=this;this.setGroupTitle(),this.filterGroupComp.setAlignItems("stretch"),this.filterGroupComp.addCssClass("ag-filter-toolpanel-group-level-"+this.depth),this.filterGroupComp.addCssClassToTitleBar("ag-filter-toolpanel-group-level-"+this.depth+"-header"),this.childFilterComps.forEach(function(r){e.filterGroupComp.addItem(r),r.addCssClassToTitleBar("ag-filter-toolpanel-group-level-"+(e.depth+1)+"-header")}),this.refreshFilterClass(),this.addExpandCollapseListeners(),this.addFilterChangedListeners(),this.setupTooltip()},t.prototype.setupTooltip=function(){var e=this;if(!!this.showingColumn){var r=function(){var n=e.columnGroup.getColDef().headerTooltip;e.setTooltip(n)};r(),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,r)}},t.prototype.getTooltipParams=function(){var e=i.prototype.getTooltipParams.call(this);return e.location="filterToolPanelColumnGroup",e},t.prototype.addCssClassToTitleBar=function(e){this.filterGroupComp.addCssClassToTitleBar(e)},t.prototype.refreshFilters=function(){this.childFilterComps.forEach(function(e){e instanceof t?e.refreshFilters():e.refreshFilter()})},t.prototype.isColumnGroup=function(){return this.columnGroup instanceof $e},t.prototype.isExpanded=function(){return this.filterGroupComp.isExpanded()},t.prototype.getChildren=function(){return this.childFilterComps},t.prototype.getFilterGroupName=function(){return this.filterGroupName?this.filterGroupName:""},t.prototype.getFilterGroupId=function(){return this.columnGroup.getId()},t.prototype.hideGroupItem=function(e,r){this.filterGroupComp.hideItem(e,r)},t.prototype.hideGroup=function(e){this.addOrRemoveCssClass("ag-hidden",e)},t.prototype.forEachToolPanelFilterChild=function(e){this.childFilterComps.forEach(function(r){r instanceof iA&&e(r)})},t.prototype.addExpandCollapseListeners=function(){var e=this,r=this.isColumnGroup()?function(){return e.expandedCallback()}:function(){return e.forEachToolPanelFilterChild(function(o){return o.expand()})},n=this.isColumnGroup()?function(){return e.expandedCallback()}:function(){return e.forEachToolPanelFilterChild(function(o){return o.collapse()})};this.addManagedListener(this.filterGroupComp,Qn.EVENT_EXPANDED,r),this.addManagedListener(this.filterGroupComp,Qn.EVENT_COLLAPSED,n)},t.prototype.getColumns=function(){return this.columnGroup instanceof $e?this.columnGroup.getLeafColumns():[this.columnGroup]},t.prototype.addFilterChangedListeners=function(){var e=this;this.getColumns().forEach(function(r){e.addManagedListener(r,Se.EVENT_FILTER_CHANGED,function(){return e.refreshFilterClass()})}),this.columnGroup instanceof $e||this.addManagedListener(this.eventService,x.EVENT_FILTER_OPENED,this.onFilterOpened.bind(this))},t.prototype.refreshFilterClass=function(){var e=this.getColumns(),r=function(){return e.some(function(n){return n.isFilterActive()})};this.filterGroupComp.addOrRemoveCssClass("ag-has-filter",r())},t.prototype.onFilterOpened=function(e){e.source==="COLUMN_MENU"&&e.column===this.columnGroup&&(!this.isExpanded()||this.collapse())},t.prototype.expand=function(){this.filterGroupComp.toggleGroupExpand(!0)},t.prototype.collapse=function(){this.filterGroupComp.toggleGroupExpand(!1)},t.prototype.setGroupTitle=function(){this.filterGroupName=this.columnGroup instanceof $e?this.getColumnGroupName(this.columnGroup):this.getColumnName(this.columnGroup),this.filterGroupComp.setTitle(this.filterGroupName||"")},t.prototype.getColumnGroupName=function(e){return this.columnModel.getDisplayNameForProvidedColumnGroup(null,e,"filterToolPanel")},t.prototype.getColumnName=function(e){return this.columnModel.getDisplayNameForColumn(e,"filterToolPanel",!1)},t.prototype.destroyFilters=function(){this.childFilterComps=this.destroyBeans(this.childFilterComps),_.clearElement(this.getGui())},t.prototype.destroy=function(){this.destroyFilters(),i.prototype.destroy.call(this)},t.TEMPLATE=`<div class="ag-filter-toolpanel-group-wrapper">
            <ag-group-component ref="filterGroupComp"></ag-group-component>
        </div>`,eh([I("filterGroupComp")],t.prototype,"filterGroupComp",void 0),eh([y("columnModel")],t.prototype,"columnModel",void 0),eh([ss],t.prototype,"preConstruct",null),eh([B],t.prototype,"init",null),t}(ee),Y5=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),th=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},X5=function(i){Y5(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.initialised=!1,e.filterGroupComps=[],e}return t.prototype.init=function(e){var r=this;this.initialised=!0;var n={suppressExpandAll:!1,suppressFilterSearch:!1,suppressSyncLayoutWithGrid:!1,api:this.gridApi,columnApi:this.columnApi,context:this.gridOptionsWrapper.getContext()};_.mergeDeep(n,e),this.params=n,this.params.suppressSyncLayoutWithGrid||this.addManagedListener(this.eventService,x.EVENT_COLUMN_MOVED,function(){return r.onColumnsChanged()}),this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,function(){return r.onColumnsChanged()}),this.addManagedListener(this.eventService,x.EVENT_TOOL_PANEL_VISIBLE_CHANGED,function(o){o.source==="filters"&&r.refreshFilters()}),this.columnModel.isReady()&&this.onColumnsChanged()},t.prototype.onColumnsChanged=function(){var e=this.columnModel.isPivotMode(),r=!this.params.suppressSyncLayoutWithGrid&&!e;r?this.syncFilterLayout():this.buildTreeFromProvidedColumnDefs()},t.prototype.syncFilterLayout=function(){this.toolPanelColDefService.syncLayoutWithGrid(this.setFiltersLayout.bind(this))},t.prototype.buildTreeFromProvidedColumnDefs=function(){var e=this;this.destroyFilters();var r=this.columnModel.getPrimaryColumnTree();this.filterGroupComps=this.recursivelyAddComps(r,0);var n=this.filterGroupComps.length;n&&(this.filterGroupComps.forEach(function(o){return e.appendChild(o)}),this.setFirstAndLastVisible(0,n-1)),_.exists(this.searchFilterText)&&this.searchFilters(this.searchFilterText),this.fireExpandedEvent()},t.prototype.setFiltersLayout=function(e){var r=this;this.destroyFilters();var n=this.toolPanelColDefService.createColumnTree(e);this.filterGroupComps=this.recursivelyAddComps(n,0);var o=this.filterGroupComps.length;o&&(this.filterGroupComps.forEach(function(a){return r.appendChild(a)}),this.setFirstAndLastVisible(0,o-1)),_.exists(this.searchFilterText)&&this.searchFilters(this.searchFilterText),this.fireExpandedEvent()},t.prototype.recursivelyAddComps=function(e,r){var n=this;return _.flatten(e.map(function(o){if(o instanceof $e)return _.flatten(n.recursivelyAddFilterGroupComps(o,r));var a=o;if(!n.shouldDisplayFilter(a))return[];var s=r===0,l=new iA(s);if(n.getContext().createBean(l),l.setColumn(a),r>0)return l;var u=n.createBean(new Ks(a,[l],n.onGroupExpanded.bind(n),r,!0));return u.addCssClassToTitleBar("ag-filter-toolpanel-header"),u.collapse(),u}))},t.prototype.recursivelyAddFilterGroupComps=function(e,r){if(!!this.filtersExistInChildren(e.getChildren())){var n=e.getColGroupDef();if(n&&n.suppressFiltersToolPanel)return[];var o=e.isPadding()?r:r+1,a=_.flatten(this.recursivelyAddComps(e.getChildren(),o));if(e.isPadding())return a;var s=new Ks(e,a,this.onGroupExpanded.bind(this),r,!1);return this.getContext().createBean(s),s.addCssClassToTitleBar("ag-filter-toolpanel-header"),[s]}},t.prototype.filtersExistInChildren=function(e){var r=this;return e.some(function(n){return n instanceof $e?r.filtersExistInChildren(n.getChildren()):r.shouldDisplayFilter(n)})},t.prototype.shouldDisplayFilter=function(e){var r=e.getColDef()&&e.getColDef().suppressFiltersToolPanel;return e.isFilterAllowed()&&!r},t.prototype.refresh=function(){},t.prototype.setVisible=function(e){i.prototype.setDisplayed.call(this,e),e&&!this.initialised&&this.init(this.params)},t.prototype.expandFilterGroups=function(e,r){var n=[],o=function(s){var l=s.getFilterGroupId(),u=!r||_.includes(r,l);u&&(e&&s.isColumnGroup()?s.expand():s.collapse(),n.push(l)),s.getChildren().forEach(function(c){c instanceof Ks&&o(c)})};if(this.filterGroupComps.forEach(o),this.onGroupExpanded(),r){var a=r.filter(function(s){return n.indexOf(s)<0});a.length>0&&console.warn("AG Grid: unable to find groups for these supplied groupIds:",a)}},t.prototype.expandFilters=function(e,r){var n=[],o=function(s){if(s instanceof Ks){var l=!1;return s.getChildren().forEach(function(p){var h=o(p);h&&(e?(s.expand(),l=!0):s.isColumnGroup()||s.collapse())}),l}var u=s.getColumn().getColId(),c=!r||_.includes(r,u);return c&&(e?s.expand():s.collapse(),n.push(u)),c};if(this.filterGroupComps.forEach(o),this.onGroupExpanded(),r){var a=r.filter(function(s){return n.indexOf(s)<0});a.length>0&&console.warn("AG Grid: unable to find columns for these supplied colIds:",a)}},t.prototype.onGroupExpanded=function(){this.fireExpandedEvent()},t.prototype.fireExpandedEvent=function(){var e=0,r=0,n=function(a){!a.isColumnGroup()||(a.isExpanded()?e++:r++,a.getChildren().forEach(function(s){s instanceof Ks&&n(s)}))};this.filterGroupComps.forEach(n);var o;e>0&&r>0?o=xi.INDETERMINATE:r>0?o=xi.COLLAPSED:o=xi.EXPANDED,this.dispatchEvent({type:"groupExpanded",state:o})},t.prototype.performFilterSearch=function(e){this.searchFilterText=_.exists(e)?e.toLowerCase():null,this.searchFilters(this.searchFilterText)},t.prototype.searchFilters=function(e){var r=function(s){return!_.exists(e)||s.toLowerCase().indexOf(e)!==-1},n=function(s,l){if(!(s instanceof Ks))return r(s.getColumnFilterName()||"");var u=s.getChildren(),c=r(s.getFilterGroupName()),p=l||c;if(p){s.hideGroup(!1);for(var h=0;h<u.length;h++)n(u[h],p),s.hideGroupItem(!1,h);return!0}var d=!1;return u.forEach(function(f,g){var v=n(f,l);s.hideGroupItem(!v,g),v&&(d=!0)}),s.hideGroup(!d),d},o,a;this.filterGroupComps.forEach(function(s,l){n(s,!1),o===void 0?s.containsCssClass("ag-hidden")||(o=l,a=l):!s.containsCssClass("ag-hidden")&&a!==l&&(a=l)}),this.setFirstAndLastVisible(o,a)},t.prototype.setFirstAndLastVisible=function(e,r){this.filterGroupComps.forEach(function(n,o){n.removeCssClass("ag-first-group-visible"),n.removeCssClass("ag-last-group-visible"),o===e&&n.addCssClass("ag-first-group-visible"),o===r&&n.addCssClass("ag-last-group-visible")})},t.prototype.refreshFilters=function(){this.filterGroupComps.forEach(function(e){return e.refreshFilters()})},t.prototype.destroyFilters=function(){this.filterGroupComps=this.destroyBeans(this.filterGroupComps),_.clearElement(this.getGui())},t.prototype.destroy=function(){this.destroyFilters(),i.prototype.destroy.call(this)},t.TEMPLATE='<div class="ag-filter-list-panel"></div>',th([y("gridApi")],t.prototype,"gridApi",void 0),th([y("columnApi")],t.prototype,"columnApi",void 0),th([y("toolPanelColDefService")],t.prototype,"toolPanelColDefService",void 0),th([y("columnModel")],t.prototype,"columnModel",void 0),t}(ee),q5=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),rh=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Z5=function(i){q5(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.initialised=!1,e.listenerDestroyFuncs=[],e}return t.prototype.init=function(e){this.initialised&&(this.listenerDestroyFuncs.forEach(function(a){return a()}),this.listenerDestroyFuncs=[]),this.initialised=!0;var r={suppressExpandAll:!1,suppressFilterSearch:!1,suppressSyncLayoutWithGrid:!1,api:this.gridApi,columnApi:this.columnApi,context:this.gridOptionsWrapper.getContext()};_.mergeDeep(r,e),this.params=r,this.filtersToolPanelHeaderPanel.init(this.params),this.filtersToolPanelListPanel.init(this.params);var n=this.params.suppressExpandAll,o=this.params.suppressFilterSearch;n&&o&&this.filtersToolPanelHeaderPanel.setDisplayed(!1),this.listenerDestroyFuncs.push(this.addManagedListener(this.filtersToolPanelHeaderPanel,"expandAll",this.onExpandAll.bind(this)),this.addManagedListener(this.filtersToolPanelHeaderPanel,"collapseAll",this.onCollapseAll.bind(this)),this.addManagedListener(this.filtersToolPanelHeaderPanel,"searchChanged",this.onSearchChanged.bind(this)),this.addManagedListener(this.filtersToolPanelListPanel,"groupExpanded",this.onGroupExpanded.bind(this)))},t.prototype.setVisible=function(e){i.prototype.setDisplayed.call(this,e),e&&!this.initialised&&this.init(this.params)},t.prototype.onExpandAll=function(){this.filtersToolPanelListPanel.expandFilterGroups(!0)},t.prototype.onCollapseAll=function(){this.filtersToolPanelListPanel.expandFilterGroups(!1)},t.prototype.onSearchChanged=function(e){this.filtersToolPanelListPanel.performFilterSearch(e.searchText)},t.prototype.setFilterLayout=function(e){this.filtersToolPanelListPanel.setFiltersLayout(e)},t.prototype.onGroupExpanded=function(e){this.filtersToolPanelHeaderPanel.setExpandState(e.state)},t.prototype.expandFilterGroups=function(e){this.filtersToolPanelListPanel.expandFilterGroups(!0,e)},t.prototype.collapseFilterGroups=function(e){this.filtersToolPanelListPanel.expandFilterGroups(!1,e)},t.prototype.expandFilters=function(e){this.filtersToolPanelListPanel.expandFilters(!0,e)},t.prototype.collapseFilters=function(e){this.filtersToolPanelListPanel.expandFilters(!1,e)},t.prototype.syncLayoutWithGrid=function(){this.filtersToolPanelListPanel.syncFilterLayout()},t.prototype.refresh=function(){this.init(this.params)},t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.TEMPLATE=`<div class="ag-filter-toolpanel">
            <ag-filters-tool-panel-header ref="filtersToolPanelHeaderPanel"></ag-filters-tool-panel-header>
            <ag-filters-tool-panel-list ref="filtersToolPanelListPanel"></ag-filters-tool-panel-list>
         </div>`,rh([I("filtersToolPanelHeaderPanel")],t.prototype,"filtersToolPanelHeaderPanel",void 0),rh([I("filtersToolPanelListPanel")],t.prototype,"filtersToolPanelListPanel",void 0),rh([y("gridApi")],t.prototype,"gridApi",void 0),rh([y("columnApi")],t.prototype,"columnApi",void 0),t}(ee),Q5={moduleName:oe.FiltersToolPanelModule,beans:[],agStackComponents:[{componentName:"AgFiltersToolPanelHeader",componentClass:z5},{componentName:"AgFiltersToolPanelList",componentClass:X5}],userComponents:[{componentName:"agFiltersToolPanel",componentClass:Z5}],dependantModules:[Xw,cr]},J5=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ys=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},e8=function(i){J5(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getData=function(e){if(e.crossFiltering){if(e.grouping)return console.warn("AG Grid: crossing filtering with row grouping is not supported."),{chartData:[],columnNames:{}};if(!this.gridOptionsWrapper.isRowModelDefault())return console.warn("AG Grid: crossing filtering is only supported in the client side row model."),{chartData:[],columnNames:{}}}var r=this.gridOptionsWrapper.isRowModelServerSide();r&&e.pivoting&&this.updatePivotKeysForSSRM();var n=this.extractRowsFromGridRowModel(e);return n.chartData=this.aggregateRowsByDimension(e,n.chartData),n},t.prototype.extractRowsFromGridRowModel=function(e){var r=this,n=[],o={},a={},s={},l={},u=[],c;if(e.crossFiltering)l=this.getFilteredRowNodes(),u=this.getAllRowNodes(),c=u.length;else{var p=this.gridRowModel.getRowCount()-1,h=e.endRow>=0?Math.min(e.endRow,p):p;c=h-e.startRow+1}for(var d=function(C){var w={},b=e.crossFiltering?u[C]:f.gridRowModel.getRow(C+e.startRow);e.dimensionCols.forEach(function(S){var T=S.colId,E=r.columnModel.getGridColumn(T);if(E){var P=r.valueService.getValue(E,b);if(e.grouping){var R=P&&P.toString?String(P.toString()):"",M=t.getGroupLabels(b,R);w[T]={labels:M,toString:function(){return this.labels.filter(function(W){return!!W}).reverse().join(" - ")}},b.group&&(a[M.toString()]=C);var N=M.slice(1,M.length).toString();N&&(s[N]=a[N])}else w[T]=P}else w[Sf.DEFAULT_CATEGORY]=C+1}),e.valueCols.forEach(function(S){var T=[],E=S.getColDef().pivotKeys;E&&(T=E.slice());var P=S.getColDef().headerName;P&&T.push(P),T.length>0&&(o[S.getId()]=T);var R=S.getColId();if(e.crossFiltering){var M=R+"-filtered-out",N=r.valueService.getValue(S,b),W=N!=null&&typeof N.toNumber=="function"?N.toNumber():N;l[b.id]?(w[R]=W,w[M]=e.aggFunc||e.isScatter?void 0:0):(w[R]=e.aggFunc||e.isScatter?void 0:0,w[M]=W)}else{var N=r.valueService.getValue(S,b);w[R]=N!=null&&typeof N.toNumber=="function"?N.toNumber():N}}),b.footer&&(w.footer=!0),n.push(w)},f=this,g=0;g<c;g++)d(g);if(e.grouping){var v=_.values(s),m=function(C,w){return!C.footer&&!_.includes(v,w)};n=n.filter(m)}return{chartData:n,columnNames:o}},t.prototype.aggregateRowsByDimension=function(e,r){var n=this,o=e.dimensionCols;if(!e.aggFunc||o.length===0)return r;var a=_.last(o),s=a&&a.colId,l={},u=[];return r.forEach(function(c){var p=l;o.forEach(function(h){var d=h.colId,f=c[d];if(d===s){var g=p[f];g||(g={__children:[]},o.forEach(function(v){var m=v.colId;g[m]=c[m]}),p[f]=g,u.push(g)),g.__children.push(c)}else p[f]||(p[f]={}),p=p[f]})}),ge.assertRegistered(oe.RowGroupingModule,"Charting Aggregation")&&u.forEach(function(c){return e.valueCols.forEach(function(p){if(e.crossFiltering)e.valueCols.forEach(function(f){var g=f.getColId(),v=c.__children.filter(function(S){return typeof S[g]!="undefined"}).map(function(S){return S[g]}),m=n.aggregationStage.aggregateValues(v,e.aggFunc);c[f.getId()]=m&&typeof m.value!="undefined"?m.value:m;var C=g+"-filtered-out",w=c.__children.filter(function(S){return typeof S[C]!="undefined"}).map(function(S){return S[C]}),b=n.aggregationStage.aggregateValues(w,e.aggFunc);c[C]=b&&typeof b.value!="undefined"?b.value:b});else{var h=c.__children.map(function(f){return f[p.getId()]}),d=0;ge.assertRegistered(oe.RowGroupingModule,"Charting Aggregation")&&(d=n.aggregationStage.aggregateValues(h,e.aggFunc)),c[p.getId()]=d&&typeof d.value!="undefined"?d.value:d}})}),u},t.prototype.updatePivotKeysForSSRM=function(){var e=this.columnModel.getSecondaryColumns();if(!!e){var r=this.extractPivotKeySeparator(e);e.forEach(function(n){var o=n.getColId().split(r);n.getColDef().pivotKeys=o.slice(0,o.length-1)})}},t.prototype.extractPivotKeySeparator=function(e){if(e.length===0)return"";var r=function(o,a){var s=o.getGroupId();return o.getParent()?r(o.getParent(),s):a.split(s)[1][0]},n=e[0];return r(n.getParent(),n.getColId())},t.getGroupLabels=function(e,r){for(var n=[r];e&&e.level!==0;)e=e.parent,e&&n.push(e.key);return n},t.prototype.getFilteredRowNodes=function(){var e={};return this.gridRowModel.forEachNodeAfterFilterAndSort(function(r){e[r.id]=r}),e},t.prototype.getAllRowNodes=function(){var e=[];return this.gridRowModel.forEachNode(function(r){e.push(r)}),this.sortRowNodes(e)},t.prototype.sortRowNodes=function(e){var r=this.sortController.getSortOptions(),n=!r||r.length==0;return n?e:this.rowNodeSorter.doFullSort(e,r)},Ys([y("rowModel")],t.prototype,"gridRowModel",void 0),Ys([y("valueService")],t.prototype,"valueService",void 0),Ys([y("columnModel")],t.prototype,"columnModel",void 0),Ys([y("rowNodeSorter")],t.prototype,"rowNodeSorter",void 0),Ys([y("sortController")],t.prototype,"sortController",void 0),Ys([De("aggregationStage")],t.prototype,"aggregationStage",void 0),t}(H),t8=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Xs=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},r8=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},nh=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(r8(arguments[t]));return i},Sf=function(i){t8(t,i);function t(e){var r=i.call(this)||this;return r.unlinked=!1,r.chartData=[],r.valueColState=[],r.dimensionColState=[],r.columnNames={},r.grouping=!1,r.crossFiltering=!1,r.suppressComboChartWarnings=!1,r.chartId=e.chartId,r.chartType=e.chartType,r.pivotChart=e.pivotChart,r.chartThemeName=e.chartThemeName,r.aggFunc=e.aggFunc,r.referenceCellRange=e.cellRange,r.suppliedCellRange=e.cellRange,r.suppressChartRanges=e.suppressChartRanges,r.unlinked=!!e.unlinkChart,r.crossFiltering=!!e.crossFiltering,r.seriesChartTypes=e.seriesChartTypes||[],r.initComboCharts(e),r}return t.prototype.initComboCharts=function(e){var r=this.seriesChartTypes&&this.seriesChartTypes.length>0,n=e.chartType==="customCombo"||r;n&&(this.chartType="customCombo",this.savedCustomSeriesChartTypes=this.seriesChartTypes||[])},t.prototype.init=function(){this.datasource=this.createManagedBean(new e8),this.updateCellRanges()},t.prototype.updateCellRanges=function(e){this.valueCellRange&&(this.referenceCellRange=this.valueCellRange);var r=this.getAllChartColumns(),n=r.dimensionCols,o=r.valueCols,a=this.getAllColumnsFromRanges();e&&this.updateColumnState(e),this.setDimensionCellRange(n,a,e),this.setValueCellRange(o,a,e),e||(this.resetColumnState(),this.syncDimensionCellRange()),this.updateSeriesChartTypes(),this.updateData()},t.prototype.updateSeriesChartTypes=function(){if(!!this.isComboChart()){if(this.seriesChartTypes=this.seriesChartTypes.map(function(e){var r=["groupedColumn","stackedColumn","stackedArea"].includes(e.chartType);return e.secondaryAxis=r?!1:e.secondaryAxis,e}),this.chartType==="customCombo"){this.updateSeriesChartTypesForCustomCombo();return}this.updateChartSeriesTypesForBuiltInCombos()}},t.prototype.updateSeriesChartTypesForCustomCombo=function(){var e=this,r=this.seriesChartTypes&&this.seriesChartTypes.length>0;!r&&!this.suppressComboChartWarnings&&console.warn("AG Grid: 'seriesChartTypes' are required when the 'customCombo' chart type is specified."),this.seriesChartTypes=this.seriesChartTypes.map(function(a){return t.SUPPORTED_COMBO_CHART_TYPES.includes(a.chartType)||(console.warn("AG Grid: invalid chartType '"+a.chartType+"' supplied in 'seriesChartTypes', converting to 'line' instead."),a.chartType="line"),a});var n=function(a){(!e.savedCustomSeriesChartTypes||e.savedCustomSeriesChartTypes.length===0)&&(e.savedCustomSeriesChartTypes=e.seriesChartTypes);var s=e.savedCustomSeriesChartTypes.find(function(l){return l.colId===a.colId});return s||(a.selected&&!e.suppressComboChartWarnings&&console.warn("AG Grid: no 'seriesChartType' found for colId = '"+a.colId+"', defaulting to 'line'."),{colId:a.colId,chartType:"line",secondaryAxis:!1})},o=this.valueColState.map(n);this.seriesChartTypes=o,this.savedCustomSeriesChartTypes=o,this.suppressComboChartWarnings=!0},t.prototype.updateChartSeriesTypesForBuiltInCombos=function(){var e=this.chartType==="columnLineCombo"?"groupedColumn":"stackedArea",r=this.chartType==="columnLineCombo"?"line":"groupedColumn",n=this.valueColState.filter(function(a){return a.selected}),o=Math.ceil(n.length/2);this.seriesChartTypes=n.map(function(a,s){var l=s>=o?r:e;return{colId:a.colId,chartType:l,secondaryAxis:!1}})},t.prototype.updateData=function(){var e=this.getRowIndexes(),r=e.startRow,n=e.endRow;this.pivotChart&&this.resetColumnState(),this.grouping=this.isGrouping();var o={aggFunc:this.aggFunc,dimensionCols:[this.getSelectedDimension()],grouping:this.grouping,pivoting:this.isPivotActive(),crossFiltering:this.crossFiltering,valueCols:this.getSelectedValueCols(),startRow:r,endRow:n,isScatter:_.includes(["scatter","bubble"],this.chartType)},a=this.datasource.getData(o),s=a.chartData,l=a.columnNames;this.chartData=s,this.columnNames=l},t.prototype.isGrouping=function(){var e=this.gridOptionsWrapper.isTreeData(),r=e?null:this.columnModel.getRowGroupColumns(),n=e||r&&r.length>0,o=this.getSelectedDimension().colId,a=this.columnModel.getGroupDisplayColumns(),s=a.map(function(l){return l.getColId()}).some(function(l){return l===o});return!!n&&s},t.prototype.getSelectedValueCols=function(){return this.valueColState.filter(function(e){return e.selected}).map(function(e){return e.column})},t.prototype.getSelectedDimension=function(){return this.dimensionColState.filter(function(e){return e.selected})[0]},t.prototype.getColDisplayName=function(e){return this.columnModel.getDisplayNameForColumn(e,"chart")},t.prototype.isPivotMode=function(){return this.columnModel.isPivotMode()},t.prototype.isPivotActive=function(){return this.columnModel.isPivotActive()},t.prototype.createCellRange=function(e){for(var r=[],n=1;n<arguments.length;n++)r[n-1]=arguments[n];return{id:this.chartId,startRow:this.referenceCellRange.startRow,endRow:this.referenceCellRange.endRow,columns:r,startColumn:e===zr.DIMENSION?r[0]:this.referenceCellRange.startColumn,type:e}},t.prototype.getAllColumnsFromRanges=function(){if(this.pivotChart)return _.convertToSet(this.columnModel.getAllDisplayedColumns());var e=this.dimensionCellRange||this.valueCellRange?[]:this.referenceCellRange.columns;return this.dimensionCellRange&&e.push.apply(e,nh(this.dimensionCellRange.columns)),this.valueCellRange&&e.push.apply(e,nh(this.valueCellRange.columns)),_.convertToSet(e)},t.prototype.getRowIndexes=function(){var e=0,r=0,n=this.rangeService,o=this.valueCellRange;if(n&&o){e=n.getRangeStartRow(o).rowIndex;var a=n.getRangeEndRow(o);r=a.rowPinned===L.PINNED_BOTTOM?-1:a.rowIndex}return{startRow:e,endRow:r}},t.prototype.getAllChartColumns=function(){var e=this,r=this.columnModel.getAllDisplayedColumns(),n=new Set,o=new Set;return r.forEach(function(a){var s=a.getColDef(),l=s.chartDataType;if(l)switch(l){case"category":case"time":n.add(a);return;case"series":o.add(a);return;case"excluded":return;default:console.warn("AG Grid: unexpected chartDataType value '"+l+"' supplied, instead use 'category', 'series' or 'excluded'");break}if(s.colId==="ag-Grid-AutoColumn"){n.add(a);return}if(!a.isPrimary()){o.add(a);return}(e.isNumberCol(a)?o:n).add(a)}),{dimensionCols:n,valueCols:o}},t.prototype.isNumberCol=function(e){if(e.getColId()==="ag-Grid-AutoColumn")return!1;var r=this.rowRenderer.getRowNode({rowIndex:0,rowPinned:null});if(!r)return!1;var n=this.valueService.getValue(e,r);return n==null&&(n=this.extractLeafData(r,e)),n!=null&&typeof n.toNumber=="function"&&(n=n.toNumber()),typeof n=="number"},t.prototype.extractLeafData=function(e,r){if(!e.allLeafChildren)return null;for(var n=0;n<e.allLeafChildren.length;n++){var o=e.allLeafChildren[n],a=this.valueService.getValue(r,o);if(a!=null)return a}return null},t.prototype.resetColumnState=function(){var e=this,r=this.getAllChartColumns(),n=r.dimensionCols,o=r.valueCols,a=this.getAllColumnsFromRanges(),s=this.valueColState.length<1;this.dimensionColState=[],this.valueColState=[];var l=!1,u=1,c=this.suppliedCellRange.columns[0];n.forEach(function(d){var f=d.getColId()==="ag-Grid-AutoColumn",g=!1;e.crossFiltering&&e.aggFunc?c.getColId()===d.getColId()&&(g=!0):g=f?!0:!l&&a.has(d),e.dimensionColState.push({column:d,colId:d.getColId(),displayName:e.getColDisplayName(d),selected:g,order:u++}),g&&(l=!0)});var p={colId:t.DEFAULT_CATEGORY,displayName:this.chartTranslationService.translate("defaultCategory"),selected:!l,order:0};this.dimensionColState.unshift(p);var h=this.referenceCellRange.columns.filter(function(d){return o.has(d)});o.forEach(function(d){s&&_.includes(e.referenceCellRange.columns,d)&&(d=h.shift()),e.valueColState.push({column:d,colId:d.getColId(),displayName:e.getColDisplayName(d),selected:a.has(d),order:u++})})},t.prototype.updateColumnState=function(e){var r=function(u){return u.colId===e.colId},n=this,o=n.dimensionColState,a=n.valueColState;o.filter(r).length>0?o.forEach(function(u){return u.selected=r(u)}):a.filter(r).forEach(function(u){return u.selected=e.selected});var s=nh(o,a),l=[];s.forEach(function(u,c){c===e.order&&l.push(e.colId),u.colId!==e.colId&&l.push(u.colId)}),s.forEach(function(u){var c=l.indexOf(u.colId);u.order=c>=0?l.indexOf(u.colId):s.length-1}),this.reorderColState()},t.prototype.reorderColState=function(){var e=function(r,n){return r.order-n.order};this.dimensionColState.sort(e),this.valueColState.sort(e)},t.prototype.setDimensionCellRange=function(e,r,n){var o=this;if(this.dimensionCellRange=void 0,!n&&!this.dimensionColState.length){e.forEach(function(l){o.dimensionCellRange||!r.has(l)||(o.dimensionCellRange=o.createCellRange(zr.DIMENSION,l))});return}var a=n;if(this.crossFiltering&&this.aggFunc){var s=this.suppliedCellRange.columns[0];a=this.dimensionColState.filter(function(l){return l.colId===s.getColId()})[0]}else(!a||!e.has(a.column))&&(a=this.dimensionColState.filter(function(l){return l.selected})[0]);a&&a.colId!==t.DEFAULT_CATEGORY&&(this.dimensionCellRange=this.createCellRange(zr.DIMENSION,a.column))},t.prototype.setValueCellRange=function(e,r,n){this.valueCellRange=void 0;var o=[];if(e.forEach(function(s){n&&n.colId===s.getColId()?n.selected&&o.push(n.column):r.has(s)&&o.push(s)}),o.length>0){var a=[];this.valueColState.length>0?a=this.valueColState.map(function(s){return s.colId}):r.forEach(function(s){return a.push(s.getColId())}),o.sort(function(s,l){return a.indexOf(s.getColId())-a.indexOf(l.getColId())}),this.valueCellRange=this.createCellRange.apply(this,nh([zr.VALUE],o))}},t.prototype.syncDimensionCellRange=function(){var e=this.getSelectedDimension();e&&e.column&&(this.dimensionCellRange=this.createCellRange(zr.DIMENSION,e.column))},t.prototype.isComboChart=function(){return["columnLineCombo","areaColumnCombo","customCombo"].includes(this.chartType)},t.DEFAULT_CATEGORY="AG-GRID-DEFAULT-CATEGORY",t.SUPPORTED_COMBO_CHART_TYPES=["line","groupedColumn","stackedColumn","area","stackedArea"],Xs([y("columnModel")],t.prototype,"columnModel",void 0),Xs([y("valueService")],t.prototype,"valueService",void 0),Xs([y("rangeService")],t.prototype,"rangeService",void 0),Xs([y("rowRenderer")],t.prototype,"rowRenderer",void 0),Xs([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),Xs([B],t.prototype,"init",null),t}(H),xd=globalThis&&globalThis.__assign||function(){return xd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},xd.apply(this,arguments)},Ir=function(){function i(){this.allPropertyListeners=new Map,this.allEventListeners=new Map}return i.prototype.addPropertyListener=function(t,e,r){r===void 0&&(r=this);var n=this.allPropertyListeners,o=n.get(t);if(o||(o=new Map,n.set(t,o)),!o.has(e)){var a=new Set;o.set(e,a)}var s=o.get(e);s&&s.add(r)},i.prototype.removePropertyListener=function(t,e,r){r===void 0&&(r=this);var n=this.allPropertyListeners,o=n.get(t);if(o)if(e){var a=o.get(e);a&&(a.delete(r),a.size||o.delete(e))}else o.clear()},i.prototype.notifyPropertyListeners=function(t,e,r){var n=this,o=this.allPropertyListeners,a=o.get(t);a&&a.forEach(function(s,l){s.forEach(function(u){return l.call(u,{type:t,source:n,value:r,oldValue:e})})})},i.prototype.addEventListener=function(t,e,r){r===void 0&&(r=this);var n=this.allEventListeners,o=n.get(t);if(o||(o=new Map,n.set(t,o)),!o.has(e)){var a=new Set;o.set(e,a)}var s=o.get(e);s&&s.add(r)},i.prototype.removeEventListener=function(t,e,r){r===void 0&&(r=this);var n=this.allEventListeners,o=n.get(t);if(o)if(e){var a=o.get(e);a&&(a.delete(r),a.size||o.delete(e))}else o.clear()},i.prototype.notifyEventListeners=function(t){var e=this,r=this.allEventListeners;t.forEach(function(n){var o=r.get(n);o&&o.forEach(function(a,s){a.forEach(function(l){return s.call(l,{type:n,source:e})})})})},i.prototype.fireEvent=function(t){var e=this,r=this.allEventListeners.get(t.type);r&&r.forEach(function(n,o){n.forEach(function(a){return o.call(a,xd(xd({},t),{source:e}))})})},i.privateKeyPrefix="_",i}();function $(){for(var i=[],t=0;t<arguments.length;t++)i[t]=arguments[t];var e=i.indexOf("debugger")>=0;return function(r,n){var o=Ir.privateKeyPrefix+n,a=o+"Events";r[n]||(i&&(r[a]=i),Object.defineProperty(r,n,{set:function(s){var l=this[o];if(e)debugger;if(s!==l||typeof s=="object"&&s!==null){this[o]=s,this.notifyPropertyListeners(n,l,s);var u=this[a];u&&this.notifyEventListeners(u)}},get:function(){return this[o]},enumerable:!0,configurable:!0}))}}var n8=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ih=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Td=function(i){n8(t,i);function t(e,r,n,o){e===void 0&&(e=0),r===void 0&&(r=e),n===void 0&&(n=e),o===void 0&&(o=r);var a=i.call(this)||this;return a.top=e,a.right=r,a.bottom=n,a.left=o,a}return t.prototype.clear=function(){this.top=this.right=this.bottom=this.left=0},ih([$("layoutChange")],t.prototype,"top",void 0),ih([$("layoutChange")],t.prototype,"right",void 0),ih([$("layoutChange")],t.prototype,"bottom",void 0),ih([$("layoutChange")],t.prototype,"left",void 0),t}(Ir),Lt=function(){function i(t,e,r,n){this.x=t,this.y=e,this.width=r,this.height=n}return i.prototype.isValid=function(){return isFinite(this.x)&&isFinite(this.y)&&isFinite(this.width)&&isFinite(this.height)},i.prototype.dilate=function(t){this.x-=t,this.y-=t,this.width+=t*2,this.height+=t*2},i.prototype.containsPoint=function(t,e){return t>=this.x&&t<=this.x+this.width&&e>=this.y&&e<=this.y+this.height},i.prototype.render=function(t,e){e===void 0&&(e=i.noParams),t.save(),e.resetTransform&&t.setTransform(1,0,0,1,0,0),t.strokeStyle=e.strokeStyle||"cyan",t.lineWidth=e.lineWidth||1,t.strokeRect(this.x,this.y,this.width,this.height),e.label&&(t.fillStyle=e.fillStyle||"black",t.textBaseline="bottom",t.fillText(e.label,this.x,this.y)),t.restore()},i.noParams={},i}(),Du=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Zo=function(){function i(t){t===void 0&&(t=[1,0,0,1,0,0]),this.elements=t}return i.prototype.setElements=function(t){var e=this.elements;return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[4]=t[4],e[5]=t[5],this},i.prototype.setIdentityElements=function(){var t=this.elements;return t[0]=1,t[1]=0,t[2]=0,t[3]=1,t[4]=0,t[5]=0,this},Object.defineProperty(i.prototype,"identity",{get:function(){var t=this.elements;return t[0]===1&&t[1]===0&&t[2]===0&&t[3]===1&&t[4]===0&&t[5]===0},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"a",{get:function(){return this.elements[0]},set:function(t){this.elements[0]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"b",{get:function(){return this.elements[1]},set:function(t){this.elements[1]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"c",{get:function(){return this.elements[2]},set:function(t){this.elements[2]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"d",{get:function(){return this.elements[3]},set:function(t){this.elements[3]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"e",{get:function(){return this.elements[4]},set:function(t){this.elements[4]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"f",{get:function(){return this.elements[5]},set:function(t){this.elements[5]=t},enumerable:!0,configurable:!0}),i.prototype.AxB=function(t,e,r){var n=Du(t,6),o=n[0],a=n[1],s=n[2],l=n[3],u=n[4],c=n[5],p=Du(e,6),h=p[0],d=p[1],f=p[2],g=p[3],v=p[4],m=p[5];r=r||t,r[0]=o*h+s*d,r[1]=a*h+l*d,r[2]=o*f+s*g,r[3]=a*f+l*g,r[4]=o*v+s*m+u,r[5]=a*v+l*m+c},i.prototype.multiplySelf=function(t){return this.AxB(this.elements,t.elements),this},i.prototype.multiply=function(t){var e=new Array(6);return this.AxB(this.elements,t.elements,e),new i(e)},i.prototype.preMultiplySelf=function(t){return this.AxB(t.elements,this.elements,this.elements),this},i.prototype.inverse=function(){var t=Du(this.elements,6),e=t[0],r=t[1],n=t[2],o=t[3],a=t[4],s=t[5],l=1/(e*o-r*n);return e*=l,r*=l,n*=l,o*=l,new i([o,-r,-n,e,n*s-o*a,r*a-e*s])},i.prototype.inverseTo=function(t){var e=Du(this.elements,6),r=e[0],n=e[1],o=e[2],a=e[3],s=e[4],l=e[5],u=1/(r*a-n*o);return r*=u,n*=u,o*=u,a*=u,t.setElements([a,-n,-o,r,o*l-a*s,n*s-r*l]),this},i.prototype.invertSelf=function(){var t=this.elements,e=Du(t,6),r=e[0],n=e[1],o=e[2],a=e[3],s=e[4],l=e[5],u=1/(r*a-n*o);return r*=u,n*=u,o*=u,a*=u,t[0]=a,t[1]=-n,t[2]=-o,t[3]=r,t[4]=o*l-a*s,t[5]=n*s-r*l,this},i.prototype.clone=function(){return new i(this.elements.slice())},i.prototype.transformPoint=function(t,e){var r=this.elements;return{x:t*r[0]+e*r[2]+r[4],y:t*r[1]+e*r[3]+r[5]}},i.prototype.transformBBox=function(t,e,r){e===void 0&&(e=0);var n=this.elements,o=n[0],a=n[1],s=n[2],l=n[3],u=t.width*.5,c=t.height*.5,p=t.x+u,h=t.y+c,d,f;if(e){u-=e,c-=e;var g=Math.sqrt(o*o+s*s),v=Math.sqrt(a*a+l*l);d=Math.abs(u*o)+Math.abs(c*s)+Math.abs(g*e),f=Math.abs(u*a)+Math.abs(c*l)+Math.abs(v*e)}else d=Math.abs(u*o)+Math.abs(c*s),f=Math.abs(u*a)+Math.abs(c*l);return r||(r=new Lt(0,0,0,0)),r.x=p*o+h*s+n[4]-d,r.y=p*a+h*l+n[5]-f,r.width=d+d,r.height=f+f,r},i.prototype.toContext=function(t){if(!this.identity){var e=this.elements;t.transform(e[0],e[1],e[2],e[3],e[4],e[5])}},i.flyweight=function(t){return t?t instanceof i?i.matrix.setElements(t.elements):i.matrix.setElements(t):i.matrix.setIdentityElements(),i.matrix},i.matrix=new i,i}();function ql(i){var t=i.constructor,e=t.className;if(!e)throw new Error("The "+t+" is missing the 'className' property.");return e+"-"+(t.id=(t.id||0)+1)}var i8=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Wn;(function(i){i[i.All=0]="All",i[i.None=1]="None"})(Wn||(Wn={}));var Yi=function(){function i(){this.id=ql(this),this.tag=NaN,this.isContainerNode=!1,this._children=[],this.childSet={},this.matrix=new Zo,this.inverseMatrix=new Zo,this._dirtyTransform=!1,this._scalingX=1,this._scalingY=1,this._scalingCenterX=null,this._scalingCenterY=null,this._rotationCenterX=null,this._rotationCenterY=null,this._rotation=0,this._translationX=0,this._translationY=0,this._dirty=!0,this._visible=!0,this.dirtyZIndex=!1,this._zIndex=0,this.pointerEvents=Wn.All}return i.isNode=function(t){return t?t.matrix!==void 0:!1},i.prototype._setScene=function(t){this._scene=t;for(var e=this.children,r=e.length,n=0;n<r;n++)e[n]._setScene(t)},Object.defineProperty(i.prototype,"scene",{get:function(){return this._scene},enumerable:!0,configurable:!0}),i.prototype._setParent=function(t){this._parent=t},Object.defineProperty(i.prototype,"parent",{get:function(){return this._parent},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"children",{get:function(){return this._children},enumerable:!0,configurable:!0}),i.prototype.countChildren=function(t){if(t===void 0&&(t=i.MAX_SAFE_INTEGER),t<=0)return 0;for(var e=this.children,r=e.length,n=r,o=0;o<r;o++)n+=e[o].countChildren(t-1);return n},i.prototype.append=function(t){i.isNode(t)&&(t=[t]);for(var e=t.length,r=0;r<e;r++){var n=t[r];if(n.parent)throw new Error(n+" already belongs to another parent: "+n.parent+".");if(n.scene)throw new Error(n+" already belongs a scene: "+n.scene+".");if(this.childSet[n.id])throw new Error("Duplicate "+n.constructor.name+" node: "+n);this._children.push(n),this.childSet[n.id]=!0,n._setParent(this),n._setScene(this.scene)}this.dirty=!0},i.prototype.appendChild=function(t){if(t.parent)throw new Error(t+" already belongs to another parent: "+t.parent+".");if(t.scene)throw new Error(t+" already belongs to a scene: "+t.scene+".");if(this.childSet[t.id])throw new Error("Duplicate "+t.constructor.name+" node: "+t);return this._children.push(t),this.childSet[t.id]=!0,t._setParent(this),t._setScene(this.scene),this.dirty=!0,t},i.prototype.removeChild=function(t){if(t.parent===this){var e=this.children.indexOf(t);if(e>=0)return this._children.splice(e,1),delete this.childSet[t.id],t._setParent(),t._setScene(),this.dirty=!0,t}throw new Error("The node to be removed is not a child of this node.")},i.prototype.insertBefore=function(t,e){var r=t.parent;if(t.parent&&t.parent.removeChild(t),e&&e.parent===this){var n=this.children.indexOf(e);if(n>=0)this._children.splice(n,0,t),this.childSet[t.id]=!0,t._setParent(this),t._setScene(this.scene);else throw new Error(e+" has "+r+" as the parent, but is not in its list of children.");this.dirty=!0}else this.append(t);return t},Object.defineProperty(i.prototype,"nextSibling",{get:function(){var t=this.parent;if(t){var e=t.children,r=e.indexOf(this);if(r>=0&&r<=e.length-1)return e[r+1]}},enumerable:!0,configurable:!0}),i.prototype.transformPoint=function(t,e){for(var r=Zo.flyweight(this.matrix),n=this.parent;n;)r.preMultiplySelf(n.matrix),n=n.parent;return r.invertSelf().transformPoint(t,e)},i.prototype.inverseTransformPoint=function(t,e){for(var r=Zo.flyweight(this.matrix),n=this.parent;n;)r.preMultiplySelf(n.matrix),n=n.parent;return r.transformPoint(t,e)},Object.defineProperty(i.prototype,"dirtyTransform",{get:function(){return this._dirtyTransform},set:function(t){this._dirtyTransform=t,t&&(this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"scalingX",{get:function(){return this._scalingX},set:function(t){this._scalingX!==t&&(this._scalingX=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"scalingY",{get:function(){return this._scalingY},set:function(t){this._scalingY!==t&&(this._scalingY=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"scalingCenterX",{get:function(){return this._scalingCenterX},set:function(t){this._scalingCenterX!==t&&(this._scalingCenterX=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"scalingCenterY",{get:function(){return this._scalingCenterY},set:function(t){this._scalingCenterY!==t&&(this._scalingCenterY=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"rotationCenterX",{get:function(){return this._rotationCenterX},set:function(t){this._rotationCenterX!==t&&(this._rotationCenterX=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"rotationCenterY",{get:function(){return this._rotationCenterY},set:function(t){this._rotationCenterY!==t&&(this._rotationCenterY=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"rotation",{get:function(){return this._rotation},set:function(t){this._rotation!==t&&(this._rotation=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"rotationDeg",{get:function(){return this.rotation/Math.PI*180},set:function(t){this.rotation=t/180*Math.PI},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"translationX",{get:function(){return this._translationX},set:function(t){this._translationX!==t&&(this._translationX=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"translationY",{get:function(){return this._translationY},set:function(t){this._translationY!==t&&(this._translationY=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),i.prototype.containsPoint=function(t,e){return!1},i.prototype.pickNode=function(t,e){if(!(!this.visible||this.pointerEvents===Wn.None||!this.containsPoint(t,e))){var r=this.children;if(r.length)for(var n=r.length-1;n>=0;n--){var o=r[n].pickNode(t,e);if(o)return o}else if(!this.isContainerNode)return this}},i.prototype.computeBBox=function(){},i.prototype.computeBBoxCenter=function(){var t=this.computeBBox&&this.computeBBox();return t?[t.x+t.width*.5,t.y+t.height*.5]:[0,0]},i.prototype.computeTransformMatrix=function(){var t=i8([0,0],2),e=t[0],r=t[1],n=this.scalingX,o=this.scalingY,a,s;n===1&&o===1?(a=0,s=0):(a=this.scalingCenterX===null?e:this.scalingCenterX,s=this.scalingCenterY===null?r:this.scalingCenterY);var l=this.rotation,u=Math.cos(l),c=Math.sin(l),p,h;l===0?(p=0,h=0):(p=this.rotationCenterX===null?e:this.rotationCenterX,h=this.rotationCenterY===null?r:this.rotationCenterY);var d=this.translationX,f=this.translationY,g=a*(1-n)-p,v=s*(1-o)-h;this.dirtyTransform=!1,this.matrix.setElements([u*n,c*n,-c*o,u*o,u*g-c*v+p+d,c*g+u*v+h+f]).inverseTo(this.inverseMatrix)},Object.defineProperty(i.prototype,"dirty",{get:function(){return this._dirty},set:function(t){this._dirty=t,t&&(this.parent?this.parent.dirty=!0:this.scene&&(this.scene.dirty=!0))},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"visible",{get:function(){return this._visible},set:function(t){this._visible!==t&&(this._visible=t,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"zIndex",{get:function(){return this._zIndex},set:function(t){this._zIndex!==t&&(this._zIndex=t,this.parent&&(this.parent.dirtyZIndex=!0),this.dirty=!0)},enumerable:!0,configurable:!0}),i.MAX_SAFE_INTEGER=Math.pow(2,53)-1,i}();function xf(i,t){var e=Object.create(i);for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r]);return e}function o8(i,t,e){var r=Array.isArray(t)?t:t.split("."),n=i;try{r.forEach(function(o){n=n[o]})}catch(o){if(arguments.length===3)n=e;else throw o}return n}function a8(i){return Array.isArray(i)?[]:{}}function Fc(i,t){return t.clone!==!1&&t.isMergeableObject(i)?On(a8(i),i,t):i}function s8(i,t,e){return i.concat(t).map(function(r){return Fc(r,e)})}function l8(i,t){if(!t.customMerge)return On;var e=t.customMerge(i);return typeof e=="function"?e:On}function u8(i){return Object.getOwnPropertySymbols?Object.getOwnPropertySymbols(i).filter(function(t){return i.propertyIsEnumerable(t)}):[]}function mx(i){return Object.keys(i).concat(u8(i))}function oA(i,t){try{return t in i}catch{return!1}}function c8(i,t){return oA(i,t)&&!(Object.hasOwnProperty.call(i,t)&&Object.propertyIsEnumerable.call(i,t))}function p8(i,t,e){var r={};return e.isMergeableObject(i)&&mx(i).forEach(function(n){r[n]=Fc(i[n],e)}),mx(t).forEach(function(n){c8(i,n)||(oA(i,n)&&e.isMergeableObject(t[n])?r[n]=l8(n,e)(i[n],t[n],e):r[n]=Fc(t[n],e))}),r}function aA(i){return h8(i)&&!d8(i)}function h8(i){return!!i&&typeof i=="object"}function d8(i){var t=Object.prototype.toString.call(i);return t==="[object RegExp]"||t==="[object Date]"}function On(i,t,e){e=e||{},e.arrayMerge=e.arrayMerge||s8,e.isMergeableObject=e.isMergeableObject||aA,e.cloneUnlessOtherwiseSpecified=Fc;var r=Array.isArray(t),n=Array.isArray(i),o=r===n;return o?r?e.arrayMerge(i,t,e):p8(i,t,e):Fc(t,e)}function f8(i){return typeof i=="object"&&!Array.isArray(i)}var g8=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ua=function(i){g8(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.lastInstanceId=0,e._fillOpacity=1,e._strokeOpacity=1,e._fill=t.defaultStyles.fill,e._stroke=t.defaultStyles.stroke,e._strokeWidth=t.defaultStyles.strokeWidth,e._lineDash=t.defaultStyles.lineDash,e._lineDashOffset=t.defaultStyles.lineDashOffset,e._lineCap=t.defaultStyles.lineCap,e._lineJoin=t.defaultStyles.lineJoin,e._opacity=t.defaultStyles.opacity,e.onShadowChange=function(){e.dirty=!0},e._fillShadow=t.defaultStyles.fillShadow,e._strokeShadow=t.defaultStyles.strokeShadow,e}return t.createInstance=function(e){var r=Object.create(e);return r._setParent(void 0),r.id=e.id+"-Instance-"+String(++e.lastInstanceId),r},t.prototype.restoreOwnStyles=function(){for(var e=this.constructor.defaultStyles,r=Object.getOwnPropertyNames(e),n=0,o=r.length;n<o;n++){var a=r[n];this[a]=e[a]}},t.prototype.restoreAllStyles=function(){var e=this.constructor.defaultStyles;for(var r in e)this[r]=e[r]},t.prototype.restoreOverriddenStyles=function(){var e=this.constructor.defaultStyles,r=Object.getPrototypeOf(e);for(var n in e)e.hasOwnProperty(n)&&r.hasOwnProperty(n)&&(this[n]=e[n])},Object.defineProperty(t.prototype,"fillOpacity",{get:function(){return this._fillOpacity},set:function(e){this._fillOpacity!==e&&(this._fillOpacity=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeOpacity",{get:function(){return this._strokeOpacity},set:function(e){this._strokeOpacity!==e&&(this._strokeOpacity=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fill",{get:function(){return this._fill},set:function(e){this._fill!==e&&(this._fill=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"stroke",{get:function(){return this._stroke},set:function(e){this._stroke!==e&&(this._stroke=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeWidth",{get:function(){return this._strokeWidth},set:function(e){this._strokeWidth!==e&&(this._strokeWidth=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"alignment",{get:function(){return Math.floor(this.strokeWidth)%2/2},enumerable:!0,configurable:!0}),t.prototype.align=function(e,r,n){return n!=null?Math.floor(n)+Math.floor(r%1+n%1):Math.floor(r)+e},Object.defineProperty(t.prototype,"lineDash",{get:function(){return this._lineDash},set:function(e){var r=this._lineDash;if(r!==e){if(r&&e&&r.length===e.length){for(var n=!0,o=e.length,a=0;a<o;a++)if(r[a]!==e[a]){n=!1;break}if(n)return}this._lineDash=e,this.dirty=!0}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineDashOffset",{get:function(){return this._lineDashOffset},set:function(e){this._lineDashOffset!==e&&(this._lineDashOffset=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineCap",{get:function(){return this._lineCap},set:function(e){this._lineCap!==e&&(this._lineCap=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineJoin",{get:function(){return this._lineJoin},set:function(e){this._lineJoin!==e&&(this._lineJoin=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"opacity",{get:function(){return this._opacity},set:function(e){e=Math.min(1,Math.max(0,e)),this._opacity!==e&&(this._opacity=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fillShadow",{get:function(){return this._fillShadow},set:function(e){var r=this._fillShadow;r!==e&&(r&&r.removeEventListener("change",this.onShadowChange),e&&e.addEventListener("change",this.onShadowChange),this._fillShadow=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeShadow",{get:function(){return this._strokeShadow},set:function(e){var r=this._strokeShadow;r!==e&&(r&&r.removeEventListener("change",this.onShadowChange),e&&e.addEventListener("change",this.onShadowChange),this._strokeShadow=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.fillStroke=function(e){if(!!this.scene){var r=this.scene.canvas.pixelRatio||1,n=e.globalAlpha;if(this.fill){e.fillStyle=this.fill,e.globalAlpha=n*this.opacity*this.fillOpacity;var o=this.fillShadow;o&&o.enabled&&(e.shadowColor=o.color,e.shadowOffsetX=o.xOffset*r,e.shadowOffsetY=o.yOffset*r,e.shadowBlur=o.blur*r),e.fill()}if(e.shadowColor="rgba(0, 0, 0, 0)",this.stroke&&this.strokeWidth){e.strokeStyle=this.stroke,e.globalAlpha=n*this.opacity*this.strokeOpacity,e.lineWidth=this.strokeWidth,this.lineDash&&e.setLineDash(this.lineDash),this.lineDashOffset&&(e.lineDashOffset=this.lineDashOffset),this.lineCap&&(e.lineCap=this.lineCap),this.lineJoin&&(e.lineJoin=this.lineJoin);var a=this.strokeShadow;a&&a.enabled&&(e.shadowColor=a.color,e.shadowOffsetX=a.xOffset*r,e.shadowOffsetY=a.yOffset*r,e.shadowBlur=a.blur*r),e.stroke()}}},t.prototype.containsPoint=function(e,r){return this.isPointInPath(e,r)},t.defaultStyles=xf({},{fill:"black",stroke:void 0,strokeWidth:0,lineDash:void 0,lineDashOffset:0,lineCap:void 0,lineJoin:void 0,opacity:1,fillShadow:void 0,strokeShadow:void 0}),t}(Yi),ns=function(){function i(t,e,r){t===void 0&&(t=window.document),e===void 0&&(e=600),r===void 0&&(r=300),this._container=void 0,this._pixelRatio=NaN,this._width=0,this._height=0,this.document=t,this.element=t.createElement("canvas"),this.context=this.element.getContext("2d"),this.element.style.userSelect="none",this.element.style.display="block",this.setPixelRatio(),this.resize(e,r)}return Object.defineProperty(i.prototype,"container",{get:function(){return this._container},set:function(t){this._container!==t&&(this.remove(),t&&t.appendChild(this.element),this._container=t)},enumerable:!0,configurable:!0}),i.prototype.remove=function(){var t=this.element.parentNode;t!=null&&t.removeChild(this.element)},i.prototype.destroy=function(){this.element.remove(),this._canvas=void 0,Object.freeze(this)},i.prototype.toImage=function(){var t=this.document.createElement("img");return t.src=this.getDataURL(),t},i.prototype.getDataURL=function(t){return this.element.toDataURL(t)},i.prototype.download=function(t){t=((t||"").trim()||"image")+".png";var e="image/png",r=this.getDataURL(e),n=this.document,o=n.createElement("a");o.href=r,o.download=t,o.style.display="none",n.body.appendChild(o),o.click(),n.body.removeChild(o)},Object.defineProperty(i.prototype,"pixelRatio",{get:function(){return this._pixelRatio},enumerable:!0,configurable:!0}),i.prototype.setPixelRatio=function(t){var e=t||window.devicePixelRatio;e!==this.pixelRatio&&(i.overrideScale(this.context,e),this._pixelRatio=e,this.resize(this.width,this.height))},Object.defineProperty(i.prototype,"pixelated",{get:function(){return this.element.style.imageRendering==="pixelated"},set:function(t){this.element.style.imageRendering=t?"pixelated":"auto"},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"width",{get:function(){return this._width},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"height",{get:function(){return this._height},enumerable:!0,configurable:!0}),i.prototype.resize=function(t,e){if(t>0&&e>0){var r=this,n=r.element,o=r.context,a=r.pixelRatio;n.width=Math.round(t*a),n.height=Math.round(e*a),n.style.width=t+"px",n.style.height=e+"px",o.resetTransform(),this._width=t,this._height=e}},Object.defineProperty(i,"textMeasuringContext",{get:function(){if(this._textMeasuringContext)return this._textMeasuringContext;var t=document.createElement("canvas");return this._textMeasuringContext=t.getContext("2d")},enumerable:!0,configurable:!0}),Object.defineProperty(i,"svgText",{get:function(){if(this._svgText)return this._svgText;var t="http://www.w3.org/2000/svg",e=document.createElementNS(t,"svg");e.setAttribute("width","100"),e.setAttribute("height","100"),e.classList?e.classList.add("text-measuring-svg"):e.setAttribute("class","text-measuring-svg"),e.style.position="absolute",e.style.top="-1000px",e.style.visibility="hidden";var r=document.createElementNS(t,"text");return r.setAttribute("x","0"),r.setAttribute("y","30"),r.setAttribute("text","black"),e.appendChild(r),document.body.appendChild(e),this._svgText=r,r},enumerable:!0,configurable:!0}),Object.defineProperty(i,"has",{get:function(){if(this._has)return this._has;var t=navigator.userAgent.indexOf("Chrome")>-1,e=navigator.userAgent.indexOf("Firefox")>-1,r=!t&&navigator.userAgent.indexOf("Safari")>-1;return this._has=Object.freeze({textMetrics:this.textMeasuringContext.measureText("test").actualBoundingBoxDescent!==void 0&&!e&&!r,getTransform:this.textMeasuringContext.getTransform!==void 0})},enumerable:!0,configurable:!0}),i.measureText=function(t,e,r,n){var o=this.textMeasuringContext;return o.font=e,o.textBaseline=r,o.textAlign=n,o.measureText(t)},i.getTextSize=function(t,e){if(this.has.textMetrics){var r=this.textMeasuringContext;r.font=e;var n=r.measureText(t);return{width:n.width,height:n.actualBoundingBoxAscent+n.actualBoundingBoxDescent}}else return this.measureSvgText(t,e)},i.measureSvgText=function(t,e){var r=this.textSizeCache,n=r[e];if(n){var o=n[t];if(o)return o}else r[e]={};var a=this.svgText;a.style.font=e,a.textContent=t;var s=a.getBBox(),l={width:s.width,height:s.height};return r[e][t]=l,l},i.overrideScale=function(t,e){var r=0,n={save:function(){this.$save(),r++},restore:function(){r>0&&(this.$restore(),r--)},setTransform:function(a,s,l,u,c,p){this.$setTransform(a*e,s*e,l*e,u*e,c*e,p*e)},resetTransform:function(){this.$setTransform(e,0,0,e,0,0),this.save(),r=0}};for(var o in n)n.hasOwnProperty(o)&&(t["$"+o]||(t["$"+o]=t[o]),t[o]=n[o])},i.textSizeCache={},i}(),v8=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Pr=function(i){v8(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._x=0,e._y=0,e.lineBreakRegex=/\r?\n/g,e.lines=[],e._text="",e._dirtyFont=!0,e._fontSize=10,e._fontFamily="sans-serif",e._textAlign=t.defaultStyles.textAlign,e._textBaseline=t.defaultStyles.textBaseline,e._lineHeight=14,e}return Object.defineProperty(t.prototype,"x",{get:function(){return this._x},set:function(e){this._x!==e&&(this._x=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this._y},set:function(e){this._y!==e&&(this._y=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.splitText=function(){this.lines=this._text.split(this.lineBreakRegex)},Object.defineProperty(t.prototype,"text",{get:function(){return this._text},set:function(e){var r=String(e);this._text!==r&&(this._text=r,this.splitText(),this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"font",{get:function(){return this.dirtyFont&&(this.dirtyFont=!1,this._font=sA(this.fontSize,this.fontFamily,this.fontStyle,this.fontWeight)),this._font},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dirtyFont",{get:function(){return this._dirtyFont},set:function(e){this._dirtyFont!==e&&(this._dirtyFont=e,e&&(this.dirty=!0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontStyle",{get:function(){return this._fontStyle},set:function(e){this._fontStyle!==e&&(this._fontStyle=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontWeight",{get:function(){return this._fontWeight},set:function(e){this._fontWeight!==e&&(this._fontWeight=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontSize",{get:function(){return this._fontSize},set:function(e){isFinite(e)||(e=10),this._fontSize!==e&&(this._fontSize=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontFamily",{get:function(){return this._fontFamily},set:function(e){this._fontFamily!==e&&(this._fontFamily=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"textAlign",{get:function(){return this._textAlign},set:function(e){this._textAlign!==e&&(this._textAlign=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"textBaseline",{get:function(){return this._textBaseline},set:function(e){this._textBaseline!==e&&(this._textBaseline=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineHeight",{get:function(){return this._lineHeight},set:function(e){this._lineHeight!==e&&(this._lineHeight=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.computeBBox=function(){return ns.has.textMetrics?this.getPreciseBBox():this.getApproximateBBox()},t.prototype.getPreciseBBox=function(){var e=ns.measureText(this.text,this.font,this.textBaseline,this.textAlign);return new Lt(this.x-e.actualBoundingBoxLeft,this.y-e.actualBoundingBoxAscent,e.width,e.actualBoundingBoxAscent+e.actualBoundingBoxDescent)},t.prototype.getApproximateBBox=function(){var e=ns.getTextSize(this.text,this.font),r=this,n=r.x,o=r.y;switch(this.textAlign){case"end":case"right":n-=e.width;break;case"center":n-=e.width/2}switch(this.textBaseline){case"alphabetic":o-=e.height*.7;break;case"middle":o-=e.height*.45;break;case"ideographic":o-=e.height;break;case"hanging":o-=e.height*.2;break;case"bottom":o-=e.height;break}return new Lt(n,o,e.width,e.height)},t.prototype.isPointInPath=function(e,r){var n=this.transformPoint(e,r),o=this.computeBBox();return o?o.containsPoint(n.x,n.y):!1},t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.render=function(e){if(!(!this.lines.length||!this.scene)){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this,n=r.fill,o=r.stroke,a=r.strokeWidth;e.font=this.font,e.textAlign=this.textAlign,e.textBaseline=this.textBaseline;var s=this.scene.canvas.pixelRatio||1,l=e.globalAlpha;if(n){e.fillStyle=n,e.globalAlpha=l*this.opacity*this.fillOpacity;var u=this,c=u.fillShadow,p=u.text,h=u.x,d=u.y;c&&c.enabled&&(e.shadowColor=c.color,e.shadowOffsetX=c.xOffset*s,e.shadowOffsetY=c.yOffset*s,e.shadowBlur=c.blur*s),e.fillText(p,h,d)}if(o&&a){e.strokeStyle=o,e.lineWidth=a,e.globalAlpha=l*this.opacity*this.strokeOpacity;var f=this,g=f.lineDash,v=f.lineDashOffset,m=f.lineCap,C=f.lineJoin,w=f.strokeShadow,p=f.text,h=f.x,d=f.y;g&&e.setLineDash(g),v&&(e.lineDashOffset=v),m&&(e.lineCap=m),C&&(e.lineJoin=C),w&&w.enabled&&(e.shadowColor=w.color,e.shadowOffsetX=w.xOffset*s,e.shadowOffsetY=w.yOffset*s,e.shadowBlur=w.blur*s),e.strokeText(p,h,d)}this.dirty=!1}},t.className="Text",t.defaultStyles=xf(ua.defaultStyles,{textAlign:"start",fontStyle:void 0,fontWeight:void 0,fontSize:10,fontFamily:"sans-serif",textBaseline:"alphabetic"}),t}(ua);function sA(i,t,e,r){return[e||"",r||"",i+"px",t].join(" ").trim()}var m8=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),yx=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},xy=function(i){m8(t,i);function t(){var e=i.call(this)||this;e.node=new Pr,e.enabled=!1,e.padding=new Td(10);var r=e.node;return r.textAlign="center",r.textBaseline="top",r.pointerEvents=Wn.None,e}return Object.defineProperty(t.prototype,"text",{get:function(){return this.node.text},set:function(e){this.node.text!==e&&(this.node.text=e,this.fireEvent({type:"change"}))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontStyle",{get:function(){return this.node.fontStyle},set:function(e){this.node.fontStyle!==e&&(this.node.fontStyle=e,this.fireEvent({type:"change"}))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontWeight",{get:function(){return this.node.fontWeight},set:function(e){this.node.fontWeight!==e&&(this.node.fontWeight=e,this.fireEvent({type:"change"}))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontSize",{get:function(){return this.node.fontSize},set:function(e){this.node.fontSize!==e&&(this.node.fontSize=e,this.fireEvent({type:"change"}))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontFamily",{get:function(){return this.node.fontFamily},set:function(e){this.node.fontFamily!==e&&(this.node.fontFamily=e,this.fireEvent({type:"change"}))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return this.node.fill},set:function(e){this.node.fill!==e&&(this.node.fill=e,this.fireEvent({type:"change"}))},enumerable:!0,configurable:!0}),yx([$("change")],t.prototype,"enabled",void 0),yx([$("change")],t.prototype,"padding",void 0),t}(Ir),y8=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}();function Od(i,t,e){var r=Ic(i,t,e);return i=Math.ceil(i/r)*r,t=Math.floor(t/r)*r+r/2,C8(i,t,r)}var Ty=Math.sqrt(50),Oy=Math.sqrt(10),Ey=Math.sqrt(2);function Ic(i,t,e){var r=Math.abs(t-i)/Math.max(0,e),n=Math.pow(10,Math.floor(Math.log(r)/Math.LN10)),o=r/n;return o>=Ty?n*=10:o>=Oy?n*=5:o>=Ey&&(n*=2),t<i?-n:n}function Ov(i,t,e){var r=(t-i)/Math.max(0,e),n=Math.floor(Math.log(r)/Math.LN10),o=r/Math.pow(10,n);return n>=0?(o>=Ty?10:o>=Oy?5:o>=Ey?2:1)*Math.pow(10,n):-Math.pow(10,-n)/(o>=Ty?10:o>=Oy?5:o>=Ey?2:1)}var lA=function(i){y8(t,i);function t(e,r){var n=i.call(this)||this;if(r)for(var o=0,a=r.length;o<a;o++)n[o]=r[o];return n.fractionDigits=e,n}return t}(Array);function C8(i,t,e){e===void 0&&(e=1);for(var r=Math.abs(e),n=r>0&&r<1?Math.abs(Math.floor(Math.log(r)/Math.LN10)):0,o=Math.pow(10,n),a=Math.max(0,Math.ceil((t-i)/e))||0,s=new lA(n),l=0;l<a;l++){var u=i+e*l;s[l]=Math.round(u*o)/o}return s}function w8(i,t){var e=Math.floor(i[0]),r=i[1];e=b8(e,r,t);var n=S8(e,r,t),o=t-1;r=e+o*n;var a=[e,r],s=_8(e,n,t);return[a,s]}function b8(i,t,e){var r=Math.abs(t-i)/(e-1),n=Math.floor(Math.log10(r)),o=Math.pow(10,n);return Math.floor(i/o)*o}function _8(i,t,e){for(var r=Math.floor(Math.log10(t)),n=t>0&&t<1?Math.abs(r):0,o=Math.pow(10,n),a=new lA(n),s=0;s<e;s++){var l=i+t*s;a[s]=Math.round(l*o)/o}return a}function S8(i,t,e){var r=e-1,n=(t-i)/r;return x8(n)}function x8(i){var t=Math.floor(Math.log10(i)),e=Math.pow(10,t),r=i/e*10;return r>0&&r<=1?e/10:r>1&&r<=2?2*e/10:r>1&&r<=5?5*e/10:r>5&&r<=10?10*e/10:r>10&&r<=20?20*e/10:r>20&&r<=40?40*e/10:r>40&&r<=50?50*e/10:r>50&&r<=100?100*e/10:r}var T8=function(i){return function(){return i}};function qa(i,t){return i=+i,t=+t,function(e){return i*(1-e)+t*e}}function O8(i,t){var e=new Date,r=+i,n=+t;return function(o){return e.setTime(r*(1-o)+n*o),e}}function E8(i,t){var e=t?t.length:0,r=i?Math.min(e,i.length):0,n=new Array(r),o=new Array(e),a;for(a=0;a<r;++a)n[a]=qw(i[a],t[a]);for(;a<e;++a)o[a]=t[a];return function(s){for(a=0;a<r;++a)o[a]=n[a](s);return o}}function R8(i,t){var e={},r={},n;(i===null||typeof i!="object")&&(i={}),(t===null||typeof t!="object")&&(t={});for(n in t)n in i?e[n]=qw(i[n],t[n]):r[n]=t[n];return function(o){for(n in e)r[n]=e[n](o);return r}}var Cx=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Xi=function(){function i(t,e,r,n){n===void 0&&(n=1),this.r=Math.min(1,Math.max(0,t||0)),this.g=Math.min(1,Math.max(0,e||0)),this.b=Math.min(1,Math.max(0,r||0)),this.a=Math.min(1,Math.max(0,n||0))}return i.fromString=function(t){if(t.indexOf("#")>=0)return i.fromHexString(t);var e=i.nameToHex[t];if(e)return i.fromHexString(e);if(t.indexOf("rgb")>=0)return i.fromRgbaString(t);throw new Error("Invalid color string: '"+t+"'")},i.parseHex=function(t){t=t.replace(/ /g,"").slice(1);var e;switch(t.length){case 6:case 8:e=[];for(var r=0;r<t.length;r+=2)e.push(parseInt(""+t[r]+t[r+1],16));break;case 3:case 4:e=t.split("").map(function(n){return parseInt(n,16)}).map(function(n){return n+n*16});break}if(e.length>=3&&e.every(function(n){return n>=0}))return e.length===3&&e.push(255),e},i.fromHexString=function(t){var e=i.parseHex(t);if(e){var r=Cx(e,4),n=r[0],o=r[1],a=r[2],s=r[3];return new i(n/255,o/255,a/255,s/255)}throw new Error("Malformed hexadecimal color string: '"+t+"'")},i.stringToRgba=function(t){for(var e=Cx([NaN,NaN],2),r=e[0],n=e[1],o=0;o<t.length;o++){var a=t[o];if(!r&&a==="(")r=o;else if(a===")"){n=o;break}}var s=r&&n&&t.substring(r+1,n);if(!!s){for(var l=s.split(","),u=[],o=0;o<l.length;o++){var c=l[o],p=parseFloat(c);if(isNaN(p))return;c.indexOf("%")>=0?(p=Math.max(0,Math.min(100,p)),p/=100):o===3?p=Math.max(0,Math.min(1,p)):(p=Math.max(0,Math.min(255,p)),p/=255),u.push(p)}return u}},i.fromRgbaString=function(t){var e=i.stringToRgba(t);if(e){if(e.length===3)return new i(e[0],e[1],e[2]);if(e.length===4)return new i(e[0],e[1],e[2],e[3])}throw new Error("Malformed rgb/rgba color string: '"+t+"'")},i.fromArray=function(t){if(t.length===4)return new i(t[0],t[1],t[2],t[3]);if(t.length===3)return new i(t[0],t[1],t[2]);throw new Error("The given array should contain 3 or 4 color components (numbers).")},i.fromHSB=function(t,e,r,n){n===void 0&&(n=1);var o=i.HSBtoRGB(t,e,r);return new i(o[0],o[1],o[2],n)},i.padHex=function(t){return t.length===1?"0"+t:t},i.prototype.toHexString=function(){var t="#"+i.padHex(Math.round(this.r*255).toString(16))+i.padHex(Math.round(this.g*255).toString(16))+i.padHex(Math.round(this.b*255).toString(16));return this.a<1&&(t+=i.padHex(Math.round(this.a*255).toString(16))),t},i.prototype.toRgbaString=function(t){t===void 0&&(t=3);var e=[Math.round(this.r*255),Math.round(this.g*255),Math.round(this.b*255)],r=Math.pow(10,t);return this.a!==1?(e.push(Math.round(this.a*r)/r),"rgba("+e.join(", ")+")"):"rgb("+e.join(", ")+")"},i.prototype.toString=function(){return this.a===1?this.toHexString():this.toRgbaString()},i.prototype.toHSB=function(){return i.RGBtoHSB(this.r,this.g,this.b)},i.RGBtoHSB=function(t,e,r){var n=Math.min(t,e,r),o=Math.max(t,e,r),a=o!==0?(o-n)/o:0,s=NaN;if(n!==o){var l=o-n,u=(o-t)/l,c=(o-e)/l,p=(o-r)/l;t===o?s=p-c:e===o?s=2+u-p:s=4+c-u,s/=6,s<0&&(s=s+1)}return[s*360,a,o]},i.HSBtoRGB=function(t,e,r){isNaN(t)&&(t=0),t=(t%360+360)%360/360;var n=0,o=0,a=0;if(e===0)n=o=a=r;else{var s=(t-Math.floor(t))*6,l=s-Math.floor(s),u=r*(1-e),c=r*(1-e*l),p=r*(1-e*(1-l));switch(s>>0){case 0:n=r,o=p,a=u;break;case 1:n=c,o=r,a=u;break;case 2:n=u,o=r,a=p;break;case 3:n=u,o=c,a=r;break;case 4:n=p,o=u,a=r;break;case 5:n=r,o=u,a=c;break}}return[n,o,a]},i.prototype.derive=function(t,e,r,n){var o=i.RGBtoHSB(this.r,this.g,this.b),a=o[2];a==0&&r>1&&(a=.05);var s=((o[0]+t)%360+360)%360,l=Math.max(Math.min(o[1]*e,1),0);a=Math.max(Math.min(a*r,1),0);var u=Math.max(Math.min(this.a*n,1),0),c=i.HSBtoRGB(s,l,a);return c.push(u),i.fromArray(c)},i.prototype.brighter=function(){return this.derive(0,1,1/.7,1)},i.prototype.darker=function(){return this.derive(0,1,.7,1)},i.nameToHex=Object.freeze({aliceblue:"#F0F8FF",antiquewhite:"#FAEBD7",aqua:"#00FFFF",aquamarine:"#7FFFD4",azure:"#F0FFFF",beige:"#F5F5DC",bisque:"#FFE4C4",black:"#000000",blanchedalmond:"#FFEBCD",blue:"#0000FF",blueviolet:"#8A2BE2",brown:"#A52A2A",burlywood:"#DEB887",cadetblue:"#5F9EA0",chartreuse:"#7FFF00",chocolate:"#D2691E",coral:"#FF7F50",cornflowerblue:"#6495ED",cornsilk:"#FFF8DC",crimson:"#DC143C",cyan:"#00FFFF",darkblue:"#00008B",darkcyan:"#008B8B",darkgoldenrod:"#B8860B",darkgray:"#A9A9A9",darkgreen:"#006400",darkgrey:"#A9A9A9",darkkhaki:"#BDB76B",darkmagenta:"#8B008B",darkolivegreen:"#556B2F",darkorange:"#FF8C00",darkorchid:"#9932CC",darkred:"#8B0000",darksalmon:"#E9967A",darkseagreen:"#8FBC8F",darkslateblue:"#483D8B",darkslategray:"#2F4F4F",darkslategrey:"#2F4F4F",darkturquoise:"#00CED1",darkviolet:"#9400D3",deeppink:"#FF1493",deepskyblue:"#00BFFF",dimgray:"#696969",dimgrey:"#696969",dodgerblue:"#1E90FF",firebrick:"#B22222",floralwhite:"#FFFAF0",forestgreen:"#228B22",fuchsia:"#FF00FF",gainsboro:"#DCDCDC",ghostwhite:"#F8F8FF",gold:"#FFD700",goldenrod:"#DAA520",gray:"#808080",green:"#008000",greenyellow:"#ADFF2F",grey:"#808080",honeydew:"#F0FFF0",hotpink:"#FF69B4",indianred:"#CD5C5C",indigo:"#4B0082",ivory:"#FFFFF0",khaki:"#F0E68C",lavender:"#E6E6FA",lavenderblush:"#FFF0F5",lawngreen:"#7CFC00",lemonchiffon:"#FFFACD",lightblue:"#ADD8E6",lightcoral:"#F08080",lightcyan:"#E0FFFF",lightgoldenrodyellow:"#FAFAD2",lightgray:"#D3D3D3",lightgreen:"#90EE90",lightgrey:"#D3D3D3",lightpink:"#FFB6C1",lightsalmon:"#FFA07A",lightseagreen:"#20B2AA",lightskyblue:"#87CEFA",lightslategray:"#778899",lightslategrey:"#778899",lightsteelblue:"#B0C4DE",lightyellow:"#FFFFE0",lime:"#00FF00",limegreen:"#32CD32",linen:"#FAF0E6",magenta:"#FF00FF",maroon:"#800000",mediumaquamarine:"#66CDAA",mediumblue:"#0000CD",mediumorchid:"#BA55D3",mediumpurple:"#9370DB",mediumseagreen:"#3CB371",mediumslateblue:"#7B68EE",mediumspringgreen:"#00FA9A",mediumturquoise:"#48D1CC",mediumvioletred:"#C71585",midnightblue:"#191970",mintcream:"#F5FFFA",mistyrose:"#FFE4E1",moccasin:"#FFE4B5",navajowhite:"#FFDEAD",navy:"#000080",oldlace:"#FDF5E6",olive:"#808000",olivedrab:"#6B8E23",orange:"#FFA500",orangered:"#FF4500",orchid:"#DA70D6",palegoldenrod:"#EEE8AA",palegreen:"#98FB98",paleturquoise:"#AFEEEE",palevioletred:"#DB7093",papayawhip:"#FFEFD5",peachpuff:"#FFDAB9",peru:"#CD853F",pink:"#FFC0CB",plum:"#DDA0DD",powderblue:"#B0E0E6",purple:"#800080",rebeccapurple:"#663399",red:"#FF0000",rosybrown:"#BC8F8F",royalblue:"#4169E1",saddlebrown:"#8B4513",salmon:"#FA8072",sandybrown:"#F4A460",seagreen:"#2E8B57",seashell:"#FFF5EE",sienna:"#A0522D",silver:"#C0C0C0",skyblue:"#87CEEB",slateblue:"#6A5ACD",slategray:"#708090",slategrey:"#708090",snow:"#FFFAFA",springgreen:"#00FF7F",steelblue:"#4682B4",tan:"#D2B48C",teal:"#008080",thistle:"#D8BFD8",tomato:"#FF6347",turquoise:"#40E0D0",violet:"#EE82EE",wheat:"#F5DEB3",white:"#FFFFFF",whitesmoke:"#F5F5F5",yellow:"#FFFF00",yellowgreen:"#9ACD32"}),i}();function wx(i,t){if(typeof i=="string")try{i=Xi.fromString(i)}catch{i=Xi.fromArray([0,0,0])}if(typeof t=="string")try{t=Xi.fromString(t)}catch{t=Xi.fromArray([0,0,0])}var e=qa(i.r,t.r),r=qa(i.g,t.g),n=qa(i.b,t.b),o=qa(i.a,t.a);return function(a){return Xi.fromArray([e(a),r(a),n(a),o(a)]).toRgbaString()}}function qw(i,t){var e=typeof t,r;if(t==null||e==="boolean")return T8(t);if(e==="number")return qa(i,t);if(e==="string")try{return r=Xi.fromString(t),t=r,wx(i,t)}catch{}return t instanceof Xi?wx(i,t):t instanceof Date?O8(i,t):Array.isArray(t)?E8(i,t):typeof t.valueOf!="function"&&typeof t.toString!="function"||isNaN(t)?R8(i,t):qa(i,t)}function uA(i,t){return i<t?-1:i>t?1:i>=t?0:NaN}function P8(i,t,e,r,n){for(r===void 0&&(r=0),n===void 0&&(n=i.length);r<n;){var o=r+n>>>1;e(i[o],t)>0?n=o:r=o+1}return r}function A8(i,t,e,r,n){r===void 0&&(r=0),n===void 0&&(n=i.length);for(var o=D8(e);r<n;){var a=r+n>>>1;o(i[a],t)<0?r=a+1:n=a}return r}function D8(i){return function(t,e){return uA(i(t),e)}}var M8=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},F8=function(i){return function(){return i}},qs=function(i){return i};function Tl(i){var t,e=i[0],r=i[i.length-1];return e>r&&(t=M8([r,e],2),e=t[0],r=t[1]),function(n){return Math.max(e,Math.min(r,n))}}var vr=function(){function i(){this.unknown=void 0,this.clamper=Tl,this._clamp=qs,this._domain=[0,1],this._range=[0,1],this.transform=qs,this.untransform=qs,this._interpolate=qw,this.rescale()}return Object.defineProperty(i.prototype,"clamp",{get:function(){return this._clamp!==qs},set:function(t){this._clamp=t?this.clamper(this.domain):qs},enumerable:!0,configurable:!0}),i.prototype.setDomain=function(t){this._domain=Array.prototype.map.call(t,function(e){return+e}),this._clamp!==qs&&(this._clamp=this.clamper(this.domain)),this.rescale()},i.prototype.getDomain=function(){return this._domain.slice()},Object.defineProperty(i.prototype,"domain",{get:function(){return this.getDomain()},set:function(t){this.setDomain(t)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"range",{get:function(){return this._range.slice()},set:function(t){this._range=Array.prototype.slice.call(t),this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"interpolate",{get:function(){return this._interpolate},set:function(t){this._interpolate=t,this.rescale()},enumerable:!0,configurable:!0}),i.prototype.rescale=function(){Math.min(this.domain.length,this.range.length)>2?this.piecewise=this.polymap:this.piecewise=this.bimap,this.output=void 0,this.input=void 0},i.prototype.normalize=function(t,e){return(e-=t=+t)?function(r){return(r-t)/e}:F8(isNaN(e)?NaN:.5)},i.prototype.bimap=function(t,e,r){var n=t[0],o=t[1],a=e[0],s=e[1],l,u;return o<n?(l=this.normalize(o,n),u=r(s,a)):(l=this.normalize(n,o),u=r(a,s)),function(c){return u(l(c))}},i.prototype.polymap=function(t,e,r){var n=this,o=Math.min(t.length,e.length)-1;t[o]<t[0]&&(t=t.slice().reverse(),e=e.slice().reverse());var a=Array.from({length:o},function(l,u){return n.normalize(t[u],t[u+1])}),s=Array.from({length:o},function(l,u){return r(e[u],e[u+1])});return function(l){var u=P8(t,l,uA,1,o)-1;return s[u](a[u](l))}},i.prototype.convert=function(t,e){if(t=+t,isNaN(t))return this.unknown;this.output||(this.output=this.piecewise(this.domain.map(this.transform),this.range,this.interpolate));var r=e?e(this.domain):this._clamp;return this.output(this.transform(r(t)))},i.prototype.invert=function(t){return this.input||(this.input=this.piecewise(this.range,this.domain.map(this.transform),qa)),this._clamp(this.untransform(this.input(t)))},i}();function I8(i,t){var e=i.toPrecision(t);e:for(var r=e.length,n=1,o=-1,a=0;n<r;++n)switch(e[n]){case".":o=a=n;break;case"0":o===0&&(o=n),a=n;break;case"e":break e;default:o>0&&(o=0);break}return o>0?e.slice(0,o)+e.slice(a+1):e}var bx={"":I8,"%":function(i,t){return(i*100).toFixed(t)},b:function(i){return Math.round(i).toString(2)},c:function(i){return String(i)},d:k8,e:function(i,t){return i.toExponential(t)},f:function(i,t){return i.toFixed(t)},g:function(i,t){return i.toPrecision(t)},o:function(i){return Math.round(i).toString(8)},p:function(i,t){return Sx(i*100,t)},r:Sx,s:G8,X:function(i){return Math.round(i).toString(16).toUpperCase()},x:function(i){return Math.round(i).toString(16)}},_x=["y","z","a","f","p","n","\xB5","m","","k","M","G","T","P","E","Z","Y"],Ev=function(){function i(t){t instanceof i?(this.fill=t.fill,this.align=t.align,this.sign=t.sign,this.symbol=t.symbol,this.zero=t.zero,this.width=t.width,this.comma=t.comma,this.precision=t.precision,this.trim=t.trim,this.type=t.type,this.string=t.string):(this.fill=t.fill===void 0?" ":String(t.fill),this.align=t.align===void 0?">":String(t.align),this.sign=t.sign===void 0?"-":String(t.sign),this.symbol=t.symbol===void 0?"":String(t.symbol),this.zero=!!t.zero,this.width=t.width===void 0?void 0:+t.width,this.comma=!!t.comma,this.precision=t.precision===void 0?void 0:+t.precision,this.trim=!!t.trim,this.type=t.type===void 0?"":String(t.type),this.string=t.string)}return i}(),L8=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i,cA=/(#\{(.*?)\})/g;function Ry(i){if(i instanceof Ev)return new Ev(i);var t=!1,e=i.replace(cA,function(){return t||(i=arguments[2],t=!0),"#{}"}),r=L8.exec(i);if(!r)throw new Error("Invalid format: "+i);return new Ev({fill:r[1],align:r[2],sign:r[3],symbol:r[4],zero:r[5],width:r[6],comma:r[7],precision:r[8]&&r[8].slice(1),trim:r[9],type:r[10],string:t?e:void 0})}function N8(i,t,e,r){var n=Ic(i,t,e),o=Ry(r==null?",f":r),a;switch(o.type){case"s":{var s=Math.max(Math.abs(i),Math.abs(t));return o.precision==null&&!isNaN(a=U8(n,s))&&(o.precision=a),pA(o,s)}case"":case"e":case"g":case"p":case"r":{o.precision==null&&!isNaN(a=z8(n,Math.max(Math.abs(i),Math.abs(t))))&&(o.precision=a-+(o.type==="e"));break}case"f":case"%":{o.precision==null&&!isNaN(a=j8(n))&&(o.precision=a-+(o.type==="%")*2);break}}return Zw(o)}var Py;function G8(i,t){t===void 0&&(t=0);var e=Ed(i,t);if(!e)return String(i);var r=e[0],n=e[1];Py=Math.max(-8,Math.min(8,Math.floor(n/3)))*3;var o=n-Py+1,a=r.length;if(o===a)return r;if(o>a)return r+new Array(o-a+1).join("0");if(o>0)return r.slice(0,o)+"."+r.slice(o);var s=Ed(i,Math.max(0,t+o-1));return"0."+new Array(1-o).join("0")+s[0]}function k8(i){return Math.abs(i=Math.round(i))>=1e21?i.toLocaleString("en").replace(/,/g,""):i.toString(10)}function W8(i,t){return function(e,r){for(var n=[],o=e.length,a=0,s=i[0],l=0;o>0&&s>0&&(l+s+1>r&&(s=Math.max(1,r-l)),n.push(e.substring(o-=s,o+s)),!((l+=s+1)>r));)s=i[a=(a+1)%i.length];return n.reverse().join(t)}}function V8(i){return function(t){return t.replace(/[0-9]/g,function(e){return i[+e]})}}function B8(i){e:for(var t=i.length,e=1,r=-1,n=0;e<t;++e)switch(i[e]){case".":r=n=e;break;case"0":r===0&&(r=e),n=e;break;default:if(!+i[e])break e;r>0&&(r=0);break}return r>0?i.slice(0,r)+i.slice(n+1):i}function Sx(i,t){var e=Ed(i,t);if(!e)return String(i);var r=e[0],n=e[1];return n<0?"0."+new Array(-n).join("0")+r:r.length>n+1?r.slice(0,n+1)+"."+r.slice(n+1):r+new Array(n-r.length+2).join("0")}function Ed(i,t){var e=t?i.toExponential(t-1):i.toExponential(),r=e.indexOf("e");if(!(r<0)){var n=e.slice(0,r);return[n.length>1?n[0]+n.slice(2):n,+e.slice(r+1)]}}function xx(i){return i}var Rv,Zw,pA;H8({thousands:",",grouping:[3],currency:["$",""]});function H8(i){Rv=$8(i),Zw=Rv.format,pA=Rv.formatPrefix}function Fl(i){var t=Ed(Math.abs(i));return t?t[1]:NaN}function j8(i){return Math.max(0,-Fl(Math.abs(i)))}function U8(i,t){var e=Math.floor(Fl(t)/3);return e=Math.min(8,e),e=Math.max(-8,e),Math.max(0,e*3-Fl(Math.abs(i)))}function z8(i,t){return i=Math.abs(i),t=Math.abs(t)-i,Math.max(0,Fl(t)-Fl(i))+1}function $8(i){var t=i.grouping===void 0||i.thousands===void 0?xx:W8(Array.prototype.map.call(i.grouping,Number),String(i.thousands)),e=i.currency===void 0?"":String(i.currency[0]),r=i.currency===void 0?"":String(i.currency[1]),n=i.decimal===void 0?".":String(i.decimal),o=i.numerals===void 0?xx:V8(Array.prototype.map.call(i.numerals,String)),a=i.percent===void 0?"%":String(i.percent),s=i.minus===void 0?"\u2212":String(i.minus),l=i.nan===void 0?"NaN":String(i.nan);function u(p){var h=Ry(p),d=h.fill,f=h.align,g=h.sign,v=h.symbol,m=h.zero,C=h.width,w=h.comma,b=h.precision,S=h.trim,T=h.type;T==="n"?(w=!0,T="g"):bx[T]||(b===void 0&&(b=12),S=!0,T="g"),(m||d==="0"&&f==="=")&&(m=!0,d="0",f="=");var E=v==="$"?e:v==="#"&&/[boxX]/.test(T)?"0"+T.toLowerCase():"",P=v==="$"?r:/[%p]/.test(T)?a:"",R=bx[T],M=/[defgprs%]/.test(T);b===void 0?b=6:/[gprs]/.test(T)?b=Math.max(1,Math.min(21,b)):b=Math.max(0,Math.min(20,b));function N(W){var G=E,k=P,V;if(T==="c")k=R(+W)+k,V="";else{var U=+W,J=W<0||1/U<0;V=isNaN(U)?l:R(Math.abs(U),b),S&&(V=B8(V)),J&&+V==0&&g!=="+"&&(J=!1);var Z=J?g==="("?g:s:g==="-"||g==="("?"":g,le=J&&g==="("?")":"";if(G=Z+G,k=(T==="s"?_x[8+Py/3]:"")+k+le,M)for(var Y=0,ae=V.length;Y<ae;Y++){var z=V.charCodeAt(Y);if(48>z||z>57){k=(z===46?n+V.slice(Y+1):V.slice(Y))+k,V=V.slice(0,Y);break}}}w&&!m&&(V=t(V,1/0));var X=G.length+V.length+k.length,ue=X<C?new Array(C-X+1).join(d):"";switch(w&&m&&(V=t(ue+V,ue.length?C-k.length:1/0),ue=""),f){case"<":V=G+V+k+ue;break;case"=":V=G+ue+V+k;break;case"^":V=ue.slice(0,X=ue.length>>1)+G+V+k+ue.slice(X);break;default:V=ue+G+V+k;break}var pe=h.string;return pe?pe.replace(cA,function(){return o(V)}):o(V)}return N}function c(p,h){var d=Ry(p);d.type="f";var f=u(d),g=Math.max(-8,Math.min(8,Math.floor(Fl(h)/3)))*3,v=Math.pow(10,-g),m=_x[8+g/3];return function(C){return f(v*+C)+m}}return{format:u,formatPrefix:c}}var K8=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Nt=function(i){K8(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.type="linear",e}return t.prototype.ticks=function(e){e===void 0&&(e=10);var r=this._domain;return Od(r[0],r[r.length-1],e)},t.prototype.nice=function(e){e===void 0&&(e=10);var r=this.domain,n=0,o=r.length-1,a=r[n],s=r[o],l;s<a&&(l=a,a=s,s=l,l=n,n=o,o=l),l=Ov(a,s,e),l>0?(a=Math.floor(a/l)*l,s=Math.ceil(s/l)*l,l=Ov(a,s,e)):l<0&&(a=Math.ceil(a*l)/l,s=Math.floor(s*l)/l,l=Ov(a,s,e)),l>0?(r[n]=Math.floor(a/l)*l,r[o]=Math.ceil(s/l)*l,this.domain=r):l<0&&(r[n]=Math.ceil(a*l)/l,r[o]=Math.floor(s*l)/l,this.domain=r)},t.prototype.tickFormat=function(e,r){var n=this.domain;return N8(n[0],n[n.length-1],e==null?10:e,r)},t}(vr),Y8=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")};function Ay(i,t){for(var e=0;e<i.length;e++){var r=i[e];if(t(r,e,i))return r}}function X8(i){return i}function ti(i,t,e){for(var r=e||X8,n=i.length,o=-1,a,s,l;++o<n;)if(a=i[o],t(a))for(s=l=a;++o<n;)a=i[o],t(a)&&(s>a&&(s=a),l<a&&(l=a));return s===void 0||l===void 0?void 0:[r(s),r(l)]}function q8(i){var t,e,r=0,n=0;try{for(var o=Y8(i),a=o.next();!a.done;a=o.next()){var s=a.value;s<0?r+=s:n+=s}}catch(l){t={error:l}}finally{try{a&&!a.done&&(e=o.return)&&e.call(o)}finally{if(t)throw t.error}}return{min:r,max:n}}function ca(i){return typeof i!="number"?!1:Number.isFinite(i)}function Z8(i){return!!i&&i.hasOwnProperty("valueOf")&&ca(i.valueOf())}function Q8(i){return ca(i)||Z8(i)}function J8(i){return i instanceof Date&&!isNaN(+i)}function hA(i){return typeof i=="string"}function e6(i){return!!i&&i.hasOwnProperty("toString")&&hA(i.toString())}function Pv(i){return hA(i)||e6(i)}function Ar(i){return Q8(i)||J8(i)}var t6=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),It=function(i){t6(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.isContainerNode=!0,e._opacity=1,e}return Object.defineProperty(t.prototype,"opacity",{get:function(){return this._opacity},set:function(e){e=Math.min(1,Math.max(0,e)),this._opacity!==e&&(this._opacity=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.containsPoint=function(e,r){return!0},t.prototype.computeBBox=function(){var e=1/0,r=-1/0,n=1/0,o=-1/0;return this.dirtyTransform&&this.computeTransformMatrix(),this.children.forEach(function(a){if(!!a.visible){var s=a.computeBBox();if(!!s){if(!(a instanceof t)){a.dirtyTransform&&a.computeTransformMatrix();for(var l=Zo.flyweight(a.matrix),u=a.parent;u;)l.preMultiplySelf(u.matrix),u=u.parent;l.transformBBox(s,0,s)}var c=s.x,p=s.y;c<e&&(e=c),p<n&&(n=p),c+s.width>r&&(r=c+s.width),p+s.height>o&&(o=p+s.height)}}}),new Lt(e,n,r-e,o-n)},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this.children,n=r.length;e.globalAlpha*=this.opacity,this.dirtyZIndex&&(this.dirtyZIndex=!1,r.sort(function(s,l){return s.zIndex-l.zIndex}));for(var o=0;o<n;o++){var a=r[o];a.visible&&(e.save(),a.render(e),e.restore())}},t.className="Group",t}(Yi),Tx=function(){function i(t,e){this.next=null,this.scene=t.scene,this.parent=t,this.datum=e}return i.prototype.appendChild=function(t){if(this.next===null)return this.parent.insertBefore(t,null);if(!Yi.isNode(this.next))throw new Error(this.next+" is not a Node.");return this.parent.insertBefore(t,this.next)},i.prototype.insertBefore=function(t,e){return this.parent.insertBefore(t,e)},i}(),Ht=function(){function i(t,e){this.groups=t,this.parents=e}return i.select=function(t){return new i([[typeof t=="function"?t():t]],[void 0])},i.selectAll=function(t){return new i([t==null?[]:t],[void 0])},i.prototype.append=function(t){return this.select(function(e){return e.appendChild(new t)})},i.prototype.appendFn=function(t){return this.select(function(e,r,n,o){return e.appendChild(t(e,r,n,o))})},i.prototype.select=function(t){for(var e=this.groups,r=e.length,n=[],o=0;o<r;o++)for(var a=e[o],s=a.length,l=n[o]=new Array(s),u=0;u<s;u++){var c=a[u];if(c){var p=t(c,c.datum,u,a);p&&(p.datum=c.datum),l[u]=p}}return new i(n,this.parents)},i.prototype.selectByClass=function(t){return this.select(function(e){if(Yi.isNode(e))for(var r=e.children,n=r.length,o=0;o<n;o++){var a=r[o];if(a instanceof t)return a}})},i.prototype.selectByTag=function(t){return this.select(function(e){if(Yi.isNode(e))for(var r=e.children,n=r.length,o=0;o<n;o++){var a=r[o];if(a.tag===t)return a}})},i.prototype.selectAllByClass=function(t){return this.selectAll(function(e){var r=[];if(Yi.isNode(e))for(var n=e.children,o=n.length,a=0;a<o;a++){var s=n[a];s instanceof t&&r.push(s)}return r})},i.prototype.selectAllByTag=function(t){return this.selectAll(function(e){var r=[];if(Yi.isNode(e))for(var n=e.children,o=n.length,a=0;a<o;a++){var s=n[a];s.tag===t&&r.push(s)}return r})},i.prototype.selectNone=function(){return[]},i.prototype.selectAll=function(t){t||(t=this.selectNone);for(var e=[],r=[],n=this.groups,o=n.length,a=0;a<o;a++)for(var s=n[a],l=s.length,u=0;u<l;u++){var c=s[u];c&&(e.push(t(c,c.datum,u,s)),r.push(c))}return new i(e,r)},i.prototype.each=function(t){for(var e=this.groups,r=e.length,n=0;n<r;n++)for(var o=e[n],a=o.length,s=0;s<a;s++){var l=o[s];l&&t(l,l.datum,s,o)}return this},i.prototype.remove=function(){return this.each(function(t){if(Yi.isNode(t)){var e=t.parent;e&&e.removeChild(t)}})},i.prototype.merge=function(t){for(var e=this.groups,r=t.groups,n=e.length,o=r.length,a=Math.min(n,o),s=new Array(n),l=0;l<a;l++)for(var u=e[l],c=r[l],p=u.length,h=s[l]=new Array(p),d=0;d<p;d++){var f=u[d]||c[d];h[d]=f||void 0}for(;l<n;l++)s[l]=e[l];return new i(s,this.parents)},i.prototype.node=function(){for(var t=this.groups,e=t.length,r=0;r<e;r++)for(var n=t[r],o=n.length,a=0;a<o;a++){var s=n[a];if(s)return s}return null},i.prototype.attr=function(t,e){return this.each(function(r){r[t]=e}),this},i.prototype.attrFn=function(t,e){return this.each(function(r,n,o,a){r[t]=e(r,n,o,a)}),this},i.prototype.call=function(t){return t(this),this},Object.defineProperty(i.prototype,"size",{get:function(){var t=0;return this.each(function(){return t++}),t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"data",{get:function(){var t=[];return this.each(function(e,r){return t.push(r)}),t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"enter",{get:function(){return new i(this.enterGroups?this.enterGroups:[[]],this.parents)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"exit",{get:function(){return new i(this.exitGroups?this.exitGroups:[[]],this.parents)},enumerable:!0,configurable:!0}),i.prototype.setDatum=function(t){return this.each(function(e){e.datum=t})},Object.defineProperty(i.prototype,"datum",{get:function(){var t=this.node();return t?t.datum:null},enumerable:!0,configurable:!0}),i.prototype.setData=function(t,e){if(typeof t!="function"){var r=t;t=function(){return r}}for(var n=this.groups,o=this.parents,a=n.length,s=new Array(a),l=new Array(a),u=new Array(a),c=0;c<a;c++){var p=n[c],h=o[c];if(!h)throw new Error("Group #"+c+" has no parent: "+p);var d=p.length,f=t(h,h.datum,c,o),g=f.length,v=l[c]=new Array(g),m=s[c]=new Array(g),C=u[c]=new Array(d);e?this.bindKey(h,p,v,m,C,f,e):this.bindIndex(h,p,v,m,C,f);for(var w=0,b=0;w<g;w++){var S=v[w];if(S){w>=b&&(b=w+1);for(var T=void 0;!(T=m[b])&&b<g;)b++;S.next=T||null}}}var E=new i(s,o);return E.enterGroups=l,E.exitGroups=u,E},i.prototype.bindIndex=function(t,e,r,n,o,a){for(var s=e.length,l=a.length,u=0;u<l;u++){var c=e[u];c?(c.datum=a[u],n[u]=c):r[u]=new Tx(t,a[u])}for(;u<s;u++){var c=e[u];c&&(o[u]=c)}},i.prototype.bindKey=function(t,e,r,n,o,a,s){for(var l=e.length,u=a.length,c=new Array(l),p={},h=0;h<l;h++){var d=e[h];if(d){var f=c[h]=i.keyPrefix+s(d,d.datum,h,e);f in p?o[h]=d:p[f]=d}}for(var h=0;h<u;h++){var f=i.keyPrefix+s(t,a[h],h,a),d=p[f];d?(n[h]=d,d.datum=a[h],p[f]=void 0):r[h]=new Tx(t,a[h])}for(var h=0;h<l;h++){var d=e[h];d&&p[c[h]]===d&&(o[h]=d)}},i.keyPrefix="$",i}(),r6=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ri=function(i){r6(t,i);function t(){var e=i.call(this)||this;return e._x1=0,e._y1=0,e._x2=0,e._y2=0,e.restoreOwnStyles(),e}return Object.defineProperty(t.prototype,"x1",{get:function(){return this._x1},set:function(e){this._x1!==e&&(this._x1=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y1",{get:function(){return this._y1},set:function(e){this._y1!==e&&(this._y1=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"x2",{get:function(){return this._x2},set:function(e){this._x2!==e&&(this._x2=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y2",{get:function(){return this._y2},set:function(e){this._y2!==e&&(this._y2=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.computeBBox=function(){return new Lt(this.x1,this.y1,this.x2-this.x1,this.y2-this.y1)},t.prototype.isPointInPath=function(e,r){return!1},t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this.x1,n=this.y1,o=this.x2,a=this.y2;if(r===o){var s=Math.round(r)+Math.floor(this.strokeWidth)%2/2;r=s,o=s}else if(n===a){var l=Math.round(n)+Math.floor(this.strokeWidth)%2/2;n=l,a=l}e.beginPath(),e.moveTo(r,n),e.lineTo(o,a),this.fillStroke(e),this.dirty=!1},t.className="Line",t.defaultStyles=xf(ua.defaultStyles,{fill:void 0,strokeWidth:1}),t}(ua),Ox=Math.sign?Math.sign:function(i){return i=+i,i===0||isNaN(i)?i:i>0?1:-1};function n6(i,t){var e=-t/i;return i!==0&&e>=0&&e<=1?[e]:[]}function i6(i,t,e){if(i===0)return n6(t,e);var r=t*t-4*i*e,n=[];if(r===0){var o=-t/(2*i);o>=0&&o<=1&&n.push(o)}else if(r>0){var a=Math.sqrt(r),s=(-t-a)/(2*i),l=(-t+a)/(2*i);s>=0&&s<=1&&n.push(s),l>=0&&l<=1&&n.push(l)}return n}function o6(i,t,e,r){if(i===0)return i6(t,e,r);var n=t/i,o=e/i,a=r/i,s=(3*o-n*n)/9,l=(9*n*o-27*a-2*n*n*n)/54,u=s*s*s+l*l,c=1/3,p=[];if(u>=0){var h=Math.sqrt(u),d=Ox(l+h)*Math.pow(Math.abs(l+h),c),f=Ox(l-h)*Math.pow(Math.abs(l-h),c),g=Math.abs(Math.sqrt(3)*(d-f)/2),v=-c*n+(d+f);if(v>=0&&v<=1&&p.push(v),g===0){var m=-c*n-(d+f)/2;m>=0&&m<=1&&p.push(m)}}else{var C=Math.acos(l/Math.sqrt(-s*s*s)),w=c*n,b=2*Math.sqrt(-s),S=b*Math.cos(c*C)-w,T=b*Math.cos(c*(C+2*Math.PI))-w,E=b*Math.cos(c*(C+4*Math.PI))-w;S>=0&&S<=1&&p.push(S),T>=0&&T<=1&&p.push(T),E>=0&&E<=1&&p.push(E)}return p}function Av(i,t,e,r,n,o,a,s){var l=(e-i)*(s-o)-(r-t)*(a-n);if(l===0)return null;var u=((a-n)*(t-o)-(i-n)*(s-o))/l,c=((e-i)*(t-o)-(r-t)*(i-n))/l;return u>=0&&u<=1&&c>=0&&c<=1?{x:i+u*(e-i),y:t+u*(r-t)}:null}function a6(i,t,e,r,n,o,a,s,l,u,c,p){for(var h=[],d=u-p,f=c-l,g=l*(p-u)-u*(c-l),v=Ex(i,e,n,a),m=Ex(t,r,o,s),C=d*v[0]+f*m[0],w=d*v[1]+f*m[1],b=d*v[2]+f*m[2],S=d*v[3]+f*m[3]+g,T=o6(C,w,b,S),E=0;E<T.length;E++){var P=T[E],R=P*P,M=P*R,N=v[0]*M+v[1]*R+v[2]*P+v[3],W=m[0]*M+m[1]*R+m[2]*P+m[3],G=void 0;l!==c?G=(N-l)/(c-l):G=(W-u)/(p-u),G>=0&&G<=1&&h.push({x:N,y:W})}return h}function Ex(i,t,e,r){return[-i+3*t-3*e+r,3*i-6*t+3*e,-3*i+3*t,i]}var dA=function(){function i(){this.commands=[],this.params=[],this._closedPath=!1}return i.prototype.draw=function(t){var e=this.commands,r=this.params,n=e.length,o=0;t.beginPath();for(var a=0;a<n;a++)switch(e[a]){case"M":t.moveTo(r[o++],r[o++]);break;case"L":t.lineTo(r[o++],r[o++]);break;case"C":t.bezierCurveTo(r[o++],r[o++],r[o++],r[o++],r[o++],r[o++]);break;case"Z":t.closePath();break}},i.prototype.moveTo=function(t,e){this.xy?(this.xy[0]=t,this.xy[1]=e):this.xy=[t,e],this.commands.push("M"),this.params.push(t,e)},i.prototype.lineTo=function(t,e){this.xy?(this.commands.push("L"),this.params.push(t,e),this.xy[0]=t,this.xy[1]=e):this.moveTo(t,e)},i.prototype.rect=function(t,e,r,n){this.moveTo(t,e),this.lineTo(t+r,e),this.lineTo(t+r,e+n),this.lineTo(t,e+n),this.closePath()},i.prototype.arcTo=function(t,e,r,n,o,a,s){var l=this.xy;if(!!l){t<0&&(t=-t),e<0&&(e=-e);var u=l[0],c=l[1],p=(u-a)/2,h=(c-s)/2,d=Math.sin(r),f=Math.cos(r),g=f*p+d*h,v=-d*p+f*h,m=g/t,C=v/e,w=m*m+C*C,b=(u+a)/2,S=(c+s)/2,T=0,E=0;w>=1?(w=Math.sqrt(w),t*=w,e*=w):(w=Math.sqrt(1/w-1),n===o&&(w=-w),T=w*t*C,E=-w*e*m,b+=f*T-d*E,S+=d*T+f*E);var P=Math.atan2((v-E)/e,(g-T)/t),R=Math.atan2((-v-E)/e,(-g-T)/t)-P;this.cubicArc(b,S,t,e,r,P,P+R,1-o)}},i.cubicArc=function(t,e,r,n,o,a,s,l,u,c){if(c){var p=l;l=u,u=p}var h=e.length,d=.5522847498307935,f=Math.sin(l),g=Math.cos(l),v=Math.sin(s),m=Math.cos(s),C=Math.PI/2,w=m*g*o-v*f*a,b=v*g*o+m*f*a,S=-m*f*o-v*g*a,T=-v*f*o+m*g*a;for(u-=l,u<0&&(u+=Math.PI*2),t.push("M"),e.push(w+r,b+n);u>=C;){u-=C,t.push("C");var E=S+r;e.push(w+S*d+r,b+T*d+n,w*d+S+r,b*d+T+n,Math.abs(E)<1e-8?0:E,T+n);var p=w;w=S,S=-p,p=b,b=T,T=-p}if(u){var P=1.3333333333333333*Math.tan(u/4),R=Math.sin(u),M=Math.cos(u),N=M+P*R,W=R-P*M;t.push("C");var E=w*M+S*R+r;e.push(w+S*P+r,b+T*P+n,w*N+S*W+r,b*N+T*W+n,Math.abs(E)<1e-8?0:E,b*M+T*R+n)}if(c)for(var G=h,k=e.length-2;G<k;G+=2,k-=2){var p=e[G];e[G]=e[k],e[k]=p,p=e[G+1],e[G+1]=e[k+1],e[k+1]=p}},i.prototype.cubicArc=function(t,e,r,n,o,a,s,l){var u=this.commands,c=this.params,p=u.length;i.cubicArc(u,c,t,e,r,n,o,a,s,l);var h=c[c.length-2],d=c[c.length-1];this.xy?(u[p]="L",this.xy[0]=h,this.xy[1]=d):this.xy=[h,d]},i.prototype.deCasteljau=function(t,e){var r=t.length;if(r<2||r%2===1)throw new Error("Fewer than two points or not an even count.");if(r===2||e===0)return t.slice(0,2);if(e===1)return t.slice(-2);for(var n=[],o=r-2,a=0;a<o;a+=2)n.push((1-e)*t[a]+e*t[a+2],(1-e)*t[a+1]+e*t[a+3]);return this.deCasteljau(n,e)},i.prototype.approximateCurve=function(t,e){var r=this.deCasteljau(t,0);this.moveTo(r[0],r[1]);for(var n=1/e,o=n;o<=1;o+=n){var a=this.deCasteljau(t,o);this.lineTo(a[0],a[1])}},i.prototype.quadraticCurveTo=function(t,e,r,n){this.xy||this.moveTo(t,e),this.cubicCurveTo((this.xy[0]+2*t)/3,(this.xy[1]+2*e)/3,(2*t+r)/3,(2*e+n)/3,r,n)},i.prototype.cubicCurveTo=function(t,e,r,n,o,a){this.xy||this.moveTo(t,e),this.commands.push("C"),this.params.push(t,e,r,n,o,a),this.xy[0]=o,this.xy[1]=a},Object.defineProperty(i.prototype,"closedPath",{get:function(){return this._closedPath},enumerable:!0,configurable:!0}),i.prototype.closePath=function(){this.xy&&(this.xy=void 0,this.commands.push("Z"),this._closedPath=!0)},i.prototype.clear=function(){this.commands.length=0,this.params.length=0,this.xy=void 0,this._closedPath=!1},i.prototype.isPointInPath=function(t,e){for(var r=this.commands,n=this.params,o=r.length,a=-1e4,s=-1e4,l=NaN,u=NaN,c=0,p=0,h=0,d=0,f=0;d<o;d++)switch(r[d]){case"M":isNaN(l)||Av(l,u,c,p,a,s,t,e)&&h++,l=c=n[f++],u=p=n[f++];break;case"L":Av(c,p,c=n[f++],p=n[f++],a,s,t,e)&&h++;break;case"C":h+=a6(c,p,n[f++],n[f++],n[f++],n[f++],c=n[f++],p=n[f++],a,s,t,e).length;break;case"Z":isNaN(l)||Av(l,u,c,p,a,s,t,e)&&h++;break}return h%2===1},i.fromString=function(t){var e=new i;return e.setFromString(t),e},i.parseSvgPath=function(t){return t.trim().split(i.splitCommandsRe).map(function(e){var r=e.match(i.matchParamsRe);return{command:e.substr(0,1),params:r?r.map(parseFloat):[]}})},i.prettifySvgPath=function(t){return i.parseSvgPath(t).map(function(e){return e.command+e.params.join(",")}).join(`
`)},i.prototype.setFromString=function(t){var e=this;this.clear();var r=i.parseSvgPath(t),n,o,a,s,l;function u(){l.match(i.quadraticCommandRe)||(a=n,s=o)}function c(){l.match(i.cubicCommandRe)||(a=n,s=o)}r.forEach(function(p){var h=p.params,d=h.length,f=0;switch(p.command){case"M":for(e.moveTo(n=h[f++],o=h[f++]);f<d;)e.lineTo(n=h[f++],o=h[f++]);break;case"m":for(e.moveTo(n+=h[f++],o+=h[f++]);f<d;)e.lineTo(n+=h[f++],o+=h[f++]);break;case"L":for(;f<d;)e.lineTo(n=h[f++],o=h[f++]);break;case"l":for(;f<d;)e.lineTo(n+=h[f++],o+=h[f++]);break;case"C":for(;f<d;)e.cubicCurveTo(h[f++],h[f++],a=h[f++],s=h[f++],n=h[f++],o=h[f++]);break;case"c":for(;f<d;)e.cubicCurveTo(n+h[f++],o+h[f++],a=n+h[f++],s=o+h[f++],n+=h[f++],o+=h[f++]);break;case"S":for(c();f<d;)e.cubicCurveTo(n+n-a,o+o-s,a=h[f++],s=h[f++],n=h[f++],o=h[f++]);break;case"s":for(c();f<d;)e.cubicCurveTo(n+n-a,o+o-s,a=n+h[f++],s=o+h[f++],n+=h[f++],o+=h[f++]);break;case"Q":for(;f<d;)e.quadraticCurveTo(a=h[f++],s=h[f++],n=h[f++],o=h[f++]);break;case"q":for(;f<d;)e.quadraticCurveTo(a=n+h[f++],s=o+h[f++],n+=h[f++],o+=h[f++]);break;case"T":for(u();f<d;)e.quadraticCurveTo(a=n+n-a,s=o+o-s,n=h[f++],o=h[f++]);break;case"t":for(u();f<d;)e.quadraticCurveTo(a=n+n-a,s=o+o-s,n+=h[f++],o+=h[f++]);break;case"A":for(;f<d;)e.arcTo(h[f++],h[f++],h[f++]*Math.PI/180,h[f++],h[f++],n=h[f++],o=h[f++]);break;case"a":for(;f<d;)e.arcTo(h[f++],h[f++],h[f++]*Math.PI/180,h[f++],h[f++],n+=h[f++],o+=h[f++]);break;case"Z":case"z":e.closePath();break;case"H":for(;f<d;)e.lineTo(n=h[f++],o);break;case"h":for(;f<d;)e.lineTo(n+=h[f++],o);break;case"V":for(;f<d;)e.lineTo(n,o=h[f++]);break;case"v":for(;f<d;)e.lineTo(n,o+=h[f++]);break}l=p.command})},i.prototype.toString=function(){for(var t=this.commands,e=this.params,r=t.length,n=[],o=0,a=0;o<r;o++)switch(t[o]){case"M":n.push("M"+e[a++]+","+e[a++]);break;case"L":n.push("L"+e[a++]+","+e[a++]);break;case"C":n.push("C"+e[a++]+","+e[a++]+" "+e[a++]+","+e[a++]+" "+e[a++]+","+e[a++]);break;case"Z":n.push("Z");break}return n.join("")},i.prototype.toPrettyString=function(){return i.prettifySvgPath(this.toString())},i.prototype.toSvg=function(){return i.xmlDeclaration+`
<svg width="100%" height="100%" viewBox="0 0 50 50" version="1.1" xmlns="`+i.xmlns+`">
    <path d="`+this.toString()+`" style="fill:none;stroke:#000;stroke-width:0.5;"/>
</svg>`},i.prototype.toDebugSvg=function(){var t=i.prettifySvgPath(this.toString());return i.xmlDeclaration+`
<svg width="100%" height="100%" viewBox="0 0 100 100" version="1.1" xmlns="`+i.xmlns+`">
    <path d="`+t+`" style="fill:none;stroke:#000;stroke-width:0.5;"/>
</svg>`},i.prototype.toCubicPaths=function(){var t=[],e=this.params,r,n,o,a,s,l=0;return this.commands.forEach(function(u){switch(u){case"M":r=[n=a=e[l++],o=s=e[l++]],t.push(r);break;case"L":var c=e[l++],p=e[l++];r.push((a+a+c)/3,(s+s+p)/3,(a+c+c)/3,(s+p+p)/3,a=c,s=p);break;case"C":r.push(e[l++],e[l++],e[l++],e[l++],a=e[l++],s=e[l++]);break;case"Z":r.push((a+a+n)/3,(s+s+o)/3,(a+n+n)/3,(s+o+o)/3,a=n,s=o);break}}),t},i.cubicPathToString=function(t){var e=t.length;if(!(e%2===0&&(e/2-1)/2>=1))throw new Error("Invalid path.");return"M"+t.slice(0,2).join(",")+"C"+t.slice(2).join(",")},i.splitCommandsRe=/(?=[AaCcHhLlMmQqSsTtVvZz])/g,i.matchParamsRe=/-?[0-9]*\.?\d+/g,i.quadraticCommandRe=/[QqTt]/,i.cubicCommandRe=/[CcSs]/,i.xmlDeclaration='<?xml version="1.0" encoding="UTF-8"?>',i.xmlns="http://www.w3.org/2000/svg",i}(),s6=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),$r=function(i){s6(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.path=new dA,e._dirtyPath=!0,e._svgPath="",e}return Object.defineProperty(t.prototype,"dirtyPath",{get:function(){return this._dirtyPath},set:function(e){this._dirtyPath!==e&&(this._dirtyPath=e,e&&(this.dirty=!0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"svgPath",{get:function(){return this._svgPath},set:function(e){this._svgPath!==e&&(this._svgPath=e,this.path.setFromString(e),this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.isPointInPath=function(e,r){var n=this.transformPoint(e,r);return this.path.closedPath&&this.path.isPointInPath(n.x,n.y)},t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.updatePath=function(){},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e),this.dirtyPath&&(this.updatePath(),this.dirtyPath=!1),this.path.draw(e),this.fillStroke(e),this.dirty=!1},t.className="Path",t}(ua),Ri=Math.PI*2;function Mn(i){return i%=Ri,i+=Ri,i%=Ri,i}function l6(i){return i%=Ri,i+=Ri,i!==Ri&&(i%=Ri),i}function u6(i){return i%=Ri,i<-Math.PI?i+=Ri:i>=Math.PI&&(i-=Ri),i}function Mt(i){return i/180*Math.PI}function fA(i,t,e){return e===void 0&&(e=1e-10),Math.abs(i-t)<e}function Dy(i,t){t===void 0&&(t=2);var e=Math.floor(Math.log(Math.abs(i))/Math.LN10);return e>=0||!isFinite(e)?i.toFixed(t):i.toFixed(Math.abs(e)-1+t)}function c6(i,t){var e=i%t;return e>=0?e:e+t}var p6=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),vl;(function(i){i[i.Open=0]="Open",i[i.Chord=1]="Chord",i[i.Round=2]="Round"})(vl||(vl={}));var Qw=function(i){p6(t,i);function t(){var e=i.call(this)||this;return e._centerX=0,e._centerY=0,e._radiusX=10,e._radiusY=10,e._startAngle=0,e._endAngle=Math.PI*2,e._counterClockwise=!1,e._type=vl.Open,e.restoreOwnStyles(),e}return Object.defineProperty(t.prototype,"centerX",{get:function(){return this._centerX},set:function(e){this._centerX!==e&&(this._centerX=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"centerY",{get:function(){return this._centerY},set:function(e){this._centerY!==e&&(this._centerY=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"radiusX",{get:function(){return this._radiusX},set:function(e){this._radiusX!==e&&(this._radiusX=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"radiusY",{get:function(){return this._radiusY},set:function(e){this._radiusY!==e&&(this._radiusY=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"startAngle",{get:function(){return this._startAngle},set:function(e){this._startAngle!==e&&(this._startAngle=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"endAngle",{get:function(){return this._endAngle},set:function(e){this._endAngle!==e&&(this._endAngle=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fullPie",{get:function(){return fA(Mn(this.startAngle),Mn(this.endAngle))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"counterClockwise",{get:function(){return this._counterClockwise},set:function(e){this._counterClockwise!==e&&(this._counterClockwise=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"type",{get:function(){return this._type},set:function(e){this._type!==e&&(this._type=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),t.prototype.updatePath=function(){var e=this.path;e.clear(),e.cubicArc(this.centerX,this.centerY,this.radiusX,this.radiusY,0,this.startAngle,this.endAngle,this.counterClockwise?1:0),this.type===vl.Chord?e.closePath():this.type===vl.Round&&!this.fullPie&&(e.lineTo(this.centerX,this.centerY),e.closePath())},t.prototype.computeBBox=function(){return new Lt(this.centerX-this.radiusX,this.centerY-this.radiusY,this.radiusX*2,this.radiusY*2)},t.prototype.isPointInPath=function(e,r){var n=this.transformPoint(e,r),o=this.computeBBox();return this.type!==vl.Open&&o.containsPoint(n.x,n.y)&&this.path.isPointInPath(n.x,n.y)},t.className="Arc",t.defaultStyles=xf(ua.defaultStyles,{lineWidth:1,fillStyle:null}),t}($r),Rx={};function Rd(i,t){Rx[t]||(i(),Rx[t]=!0)}var Px=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},h6=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},Lo;(function(i){i[i.Tick=0]="Tick",i[i.GridLine=1]="GridLine"})(Lo||(Lo={}));var d6=function(){function i(){this.width=1,this.size=6,this.color="rgba(195, 195, 195, 1)",this.count=void 0}return i}(),gA=function(){function i(){this.fontStyle=void 0,this.fontWeight=void 0,this.fontSize=12,this.fontFamily="Verdana, sans-serif",this.padding=5,this.color="rgba(87, 87, 87, 1)",this.rotation=void 0,this.autoRotate=void 0,this.autoRotateAngle=335,this.mirrored=!1,this.parallel=!1,this.formatter=void 0,this.onFormatChange=void 0}return Object.defineProperty(i.prototype,"format",{get:function(){return this._format},set:function(t){this._format!==t&&(this._format=t,this.onFormatChange&&this.onFormatChange(t))},enumerable:!0,configurable:!0}),i}(),f6=function(){function i(t){this.id=ql(this),this.lineNode=new ri,this.group=new It,this.line={width:1,color:"rgba(195, 195, 195, 1)"},this.tick=new d6,this.label=new gA,this.translation={x:0,y:0},this.rotation=0,this._labelAutoRotated=!1,this._calculatedTickCount=void 0,this.requestedRange=[0,1],this._visibleRange=[0,1],this._title=void 0,this._gridLength=0,this.gridStyle=[{stroke:"rgba(219, 219, 219, 1)",lineDash:[4,2]}],this._radialGrid=!1,this.fractionDigits=0,this.thickness=0,this.scale=t,this.groupSelection=Ht.select(this.group).selectAll(),this.label.onFormatChange=this.onLabelFormatChange.bind(this),this.group.append(this.lineNode)}return Object.defineProperty(i.prototype,"scale",{get:function(){return this._scale},set:function(t){this._scale=t,this.requestedRange=t.range.slice(),this.onLabelFormatChange()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"ticks",{get:function(){return this._ticks},set:function(t){this._ticks=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"labelAutoRotated",{get:function(){return this._labelAutoRotated},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"calculatedTickCount",{get:function(){var t;return t=this._calculatedTickCount,t!=null?t:this.tick.count},enumerable:!0,configurable:!0}),i.prototype.calculateTickCount=function(t){},i.prototype.getMeta=function(){},i.prototype.updateRange=function(){var t=this,e=t.requestedRange,r=t.visibleRange,n=t.scale,o=(e[1]-e[0])/(r[1]-r[0]),a=o*r[0],s=e[0]-a;n.range=[s,s+o]},i.prototype.inRange=function(t,e,r){return e===void 0&&(e=0),r===void 0&&(r=0),this.inRangeEx(t,e,r)===0},i.prototype.inRangeEx=function(t,e,r){e===void 0&&(e=0),r===void 0&&(r=0);var n=this.range,o=Math.min(n[0],n[1]),a=Math.max(n[0],n[1]);return t+e<o-r?-1:t>a+r?1:0},Object.defineProperty(i.prototype,"range",{get:function(){return this.requestedRange.slice()},set:function(t){this.requestedRange=t.slice(),this.updateRange()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"visibleRange",{get:function(){return this._visibleRange.slice()},set:function(t){if(t&&t.length===2){var e=Px(t,2),r=e[0],n=e[1];r=Math.max(0,r),n=Math.min(1,n),r=Math.min(r,n),n=Math.max(r,n),this._visibleRange=[r,n],this.updateRange()}},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"domain",{get:function(){return this.scale.domain.slice()},set:function(t){this.scale.domain=t.slice(),this.onLabelFormatChange(this.label.format)},enumerable:!0,configurable:!0}),i.prototype.onLabelFormatChange=function(t){if(t&&this.scale&&this.scale.tickFormat)try{this.labelFormatter=this.scale.tickFormat(this.calculatedTickCount,t)}catch{this.labelFormatter=void 0,Rd(function(){return console.warn("AG Charts - the axis label format string "+t+" is invalid. No formatting will be applied")},"invalid axis label format string "+t)}else this.labelFormatter=void 0},Object.defineProperty(i.prototype,"title",{get:function(){return this._title},set:function(t){var e=this._title;e!==t&&(e&&this.group.removeChild(e.node),t&&(t.node.rotation=-Math.PI/2,this.group.appendChild(t.node)),this._title=t,this.positionTitle())},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"gridLength",{get:function(){return this._gridLength},set:function(t){(this._gridLength&&!t||!this._gridLength&&t)&&(this.groupSelection=this.groupSelection.remove().setData([])),this._gridLength=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"radialGrid",{get:function(){return this._radialGrid},set:function(t){this._radialGrid!==t&&(this._radialGrid=t,this.groupSelection=this.groupSelection.remove().setData([]))},enumerable:!0,configurable:!0}),i.prototype.update=function(){var t=this,e=this,r=e.group,n=e.scale,o=e.tick,a=e.label,s=e.gridStyle,l=e.requestedRange,u=Math.min(l[0],l[1]),c=Math.max(l[0],l[1]),p=Mt(this.rotation),h=a.rotation?Mn(Mt(a.rotation)):0,d=a.parallel,f=0;r.translationX=this.translation.x,r.translationY=this.translation.y,r.rotation=p;var g=(n.bandwidth||0)/2,v=a.mirrored?1:-1,m=Mn(p),C=!h&&m>=0&&m<=Math.PI?-1:1,w=Mn(p-Math.PI/2),b=!h&&w>=0&&w<=Math.PI?-1:1,S=this.ticks||n.ticks(this.calculatedTickCount),T=this.groupSelection.setData(S);T.exit.remove();var E=T.enter.append(It);E.append(ri).each(function(ie){return ie.tag=Lo.Tick}),this.gridLength&&(this.radialGrid?E.append(Qw).each(function(ie){return ie.tag=Lo.GridLine}):E.append(ri).each(function(ie){return ie.tag=Lo.GridLine})),E.append(Pr);var P=T.merge(E),R=!1;if(P.attrFn("translationY",function(ie,He){return Math.round(n.convert(He)+g)}).attrFn("visible",function(ie){var He=Math.floor(u),Me=Math.ceil(c),ne=He!==Me&&ie.translationY>=He&&ie.translationY<=Me;return R=ne||R,ne}),this.group.visible=R,!R){this.groupSelection=P;return}this.fractionDigits=S.fractionDigits>=0?S.fractionDigits:0;var M=new Map,N=0,W=!1,G=!1,k=c-u,V=P.selectByClass(Pr).each(function(ie,He,Me){ie.fontStyle=a.fontStyle,ie.fontWeight=a.fontWeight,ie.fontSize=a.fontSize,ie.fontFamily=a.fontFamily,ie.fill=a.color,ie.text=t.formatTickDatum(He,Me),ie.visible=ie.parent.visible,ie.visible===!0&&(M.set(Me,ie.computeBBox()),!(ie.text===""||ie.text==null)&&(N++,Me===0&&ie.translationY===n.range[0]?W=!0:Me===S.length-1&&ie.translationY===n.range[1]&&(G=!0)))}),U=v*(o.size+a.padding),J=k/N,Z=function(ie,He){var Me,ne,ke=0,qe=!1,it=ie.size-1,mn=12;try{for(var Yr=h6(ie.entries()),xr=Yr.next();!xr.done;xr=Yr.next()){var Xr=Px(xr.value,2),Bn=Xr[0],yo=Xr[1],Co=Bn===0&&W||Bn===it&&G?2:1,Gi=He?yo.width/Co:yo.height/Co,wo=Gi<=0?0:Gi+mn;ke+=wo,wo>J&&(qe=!0)}}catch(fa){Me={error:fa}}finally{try{xr&&!xr.done&&(ne=Yr.return)&&ne.call(Yr)}finally{if(Me)throw Me.error}}return{totalLength:ke,rotate:qe}},le=d,Y=Z(M,le),ae=Y.totalLength,z=Y.rotate;this._labelAutoRotated=!1,a.rotation===void 0&&a.autoRotate===!0&&z&&(f=Mn(Mt(a.autoRotateAngle)),this._labelAutoRotated=!0),(h||f)&&(d?le=h===Math.PI||f===Math.PI:le=h===Math.PI/2||h===Math.PI+Math.PI/2||f===Math.PI/2||f===Math.PI+Math.PI/2,ae=Z(M,le).totalLength);var X=d?C*Math.PI/2:b===-1?Math.PI:0,ue=d&&!h?v*C===-1?"hanging":"bottom":"middle",pe=h>0&&h<=Math.PI||f>0&&f<=Math.PI?-1:1,be=d?h||f?v*pe===-1?"end":"start":"center":v*b===-1?"end":"start";if(V.each(function(ie){if(ie.text===""||ie.text==null){ie.visible=!1;return}ie.textBaseline=ue,ie.textAlign=be,ie.x=U,ie.rotationCenterX=U,ie.rotation=X+h+f}),ae>k){var _e=n instanceof vr,Te=ae/N,Pe=Math.floor(k/Te),Ee=Pe>2?Math.ceil(N/Pe):N,Le=0;V.each(function(ie,He,Me){if(ie.visible===!0){var ne=_e&&t.tick.count===void 0?Me===0||Me===N-1:!1;ie.visible=!!(ne||Le%Ee===0),Le++,ie.visible||M.delete(Me)}})}if(P.selectByTag(Lo.Tick).each(function(ie,He,Me){ie.strokeWidth=o.width,ie.stroke=o.color,ie.visible=M.has(Me)}).attr("x1",v*o.size).attr("x2",0).attr("y1",0).attr("y2",0),this.gridLength&&s.length){var Ze=s.length,Re=void 0;if(this.radialGrid){var Ge=l6(Mt(this.gridLength));Re=P.selectByTag(Lo.GridLine).each(function(ie,He,Me){var ne=Math.round(n.convert(He)+g);ie.centerX=0,ie.centerY=n.range[0]-ne,ie.endAngle=Ge,ie.radiusX=ne,ie.radiusY=ne,ie.visible=M.has(Me)})}else Re=P.selectByTag(Lo.GridLine).each(function(ie,He,Me){ie.x1=0,ie.x2=-v*t.gridLength,ie.y1=0,ie.y2=0,ie.visible=Math.abs(ie.parent.translationY-n.range[0])>1&&M.has(Me)});Re.each(function(ie,He,Me){var ne=s[Me%Ze];ie.stroke=ne.stroke,ie.strokeWidth=o.width,ie.lineDash=ne.lineDash,ie.fill=void 0})}this.groupSelection=P;var Xe=this.lineNode;Xe.x1=0,Xe.x2=0,Xe.y1=l[0],Xe.y2=l[1],Xe.strokeWidth=this.line.width,Xe.stroke=this.line.color,Xe.visible=S.length>0,this.positionTitle()},i.prototype.positionTitle=function(){var t=this,e=t.title,r=t.lineNode;if(!!e){var n=!1;if(e.enabled&&r.visible){n=!0;var o=this,a=o.label,s=o.rotation,l=o.requestedRange,u=a.mirrored?1:-1,c=Mn(s),p=e.padding.bottom,h=e.node,d=this.computeBBox({excludeTitle:!0}),f=u===-1&&c>Math.PI&&c<Math.PI*2?-1:1;h.rotation=f*u*Math.PI/2,h.x=f*u*(l[0]+l[1])/2,u===-1?h.y=f*(-p-d.width+Math.max(d.x+d.width,0)):h.y=-p-d.width-Math.min(d.x,0),h.textBaseline=f===1?"bottom":"top"}e.node.visible=n}},i.prototype.formatTickDatum=function(t,e){var r=this,n=r.label,o=r.labelFormatter,a=r.fractionDigits,s=this.getMeta();return n.formatter?n.formatter({value:a>=0?t:String(t),index:e,fractionDigits:a,formatter:o,axis:s}):o?o(t):typeof t=="number"&&a>=0?t.toFixed(a):String(t)},i.prototype.formatDatum=function(t){return String(t)},i.prototype.computeBBox=function(t){var e=this,r=e.title,n=e.lineNode,o=this.groupSelection.selectByClass(Pr),a=1/0,s=-1/0,l=1/0,u=-1/0;if(o.each(function(f){f.computeTransformMatrix();var g=Zo.flyweight(f.matrix),v=f.parent;v.computeTransformMatrix(),g.preMultiplySelf(v.matrix);var m=f.computeBBox();if(m){var C=g.transformBBox(m);a=Math.min(a,C.x),s=Math.max(s,C.x+C.width),l=Math.min(l,C.y),u=Math.max(u,C.y+C.height)}}),r&&r.enabled&&n.visible&&(!t||!t.excludeTitle)){var c=r.node;c.computeTransformMatrix();var p=Zo.flyweight(c.matrix),h=c.computeBBox();if(h){var d=p.transformBBox(h);a=Math.min(a,d.x),s=Math.max(s,d.x+d.width),l=Math.min(l,d.y),u=Math.max(u,d.y+d.height)}}return a=Math.min(a,0),s=Math.max(s,0),l=Math.min(l,n.y1,n.y2),u=Math.max(u,n.y1,n.y2),new Lt(a,l,s-a,u-l)},i}(),g6=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),tt;(function(i){i.X="x",i.Y="y"})(tt||(tt={}));function Ax(i){return i===tt.X?tt.Y:tt.X}var Ae;(function(i){i.Top="top",i.Right="right",i.Bottom="bottom",i.Left="left",i.Angle="angle",i.Radius="radius"})(Ae||(Ae={}));var Tf=function(i){g6(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.keys=[],e.direction=tt.Y,e.boundSeries=[],e._position=Ae.Left,e}return Object.defineProperty(t.prototype,"type",{get:function(){return this.constructor.type||""},enumerable:!0,configurable:!0}),t.prototype.getMeta=function(){return{id:this.id,direction:this.direction,boundSeries:this.boundSeries}},t.prototype.useCalculatedTickCount=function(){return this.scale instanceof Nt},t.prototype.calculateTickCount=function(e){if(!!this.useCalculatedTickCount()){var r=600,n=70,o=Math.pow(Math.log(e)/Math.log(r),2),a=n*o;this._calculatedTickCount=this.tick.count||Math.max(2,Math.floor(e/a))}},Object.defineProperty(t.prototype,"position",{get:function(){return this._position},set:function(e){if(this._position!==e)switch(this._position=e,e){case Ae.Top:this.direction=tt.X,this.rotation=-90,this.label.mirrored=!0,this.label.parallel=!0;break;case Ae.Right:this.direction=tt.Y,this.rotation=0,this.label.mirrored=!0,this.label.parallel=!1;break;case Ae.Bottom:this.direction=tt.X,this.rotation=-90,this.label.mirrored=!1,this.label.parallel=!0;break;case Ae.Left:this.direction=tt.Y,this.rotation=0,this.label.mirrored=!1,this.label.parallel=!1;break}},enumerable:!0,configurable:!0}),t}(f6),v6=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),vA=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o};function mA(i){var t,e=i[0],r=i[i.length-1];return e>r&&(t=vA([r,e],2),e=t[0],r=t[1]),function(n){return n>=e&&n<=r?n:NaN}}var Zl=function(i){v6(t,i);function t(){var e=i.call(this,new Nt)||this;return e._nice=!0,e._min=NaN,e._max=NaN,e.scale.clamper=mA,e}return Object.defineProperty(t.prototype,"nice",{get:function(){return this._nice},set:function(e){this._nice!==e&&(this._nice=e,e&&this.scale.nice&&this.scale.nice(typeof this.calculatedTickCount=="number"?this.calculatedTickCount:void 0))},enumerable:!0,configurable:!0}),t.prototype.setDomain=function(e,r){var n=this,o=n.scale,a=n.min,s=n.max;if(e.length>2&&(e=ti(e,Ar,Number)||[0,1]),e=[isNaN(a)?e[0]:a,isNaN(s)?e[1]:s],r){var l=vA(w8(e,r),2),u=l[0],c=l[1];o.domain=u,this.ticks=c;return}else o.domain=e,this.onLabelFormatChange(this.label.format),this.scale.clamp=!0,this.nice&&this.scale.nice&&this.scale.nice(typeof this.calculatedTickCount=="number"?this.calculatedTickCount:void 0)},Object.defineProperty(t.prototype,"domain",{get:function(){return this.scale.domain},set:function(e){this.setDomain(e)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"min",{get:function(){return this._min},set:function(e){this._min!==e&&(this._min=e,isNaN(e)||(this.scale.domain=[e,this.scale.domain[1]]))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"max",{get:function(){return this._max},set:function(e){this._max!==e&&(this._max=e,isNaN(e)||(this.scale.domain=[this.scale.domain[0],e]))},enumerable:!0,configurable:!0}),t.prototype.formatDatum=function(e){return typeof e=="number"?e.toFixed(2):(Rd(function(){return console.warn("AG Charts - Data contains Date objects which are being plotted against a number axis, please only use a number axis for numbers.")},"number axis config used with Date objects"),String(e))},t.className="NumberAxis",t.type="number",t}(Tf),Dx=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Ni=function(){function i(){this.type="band",this.index=new Map,this.ordinalRange=[],this._domain=[],this._range=[0,1],this._bandwidth=1,this._paddingInner=0,this._paddingOuter=0,this._round=!1,this._align=.5}return Object.defineProperty(i.prototype,"domain",{get:function(){return this._domain},set:function(t){var e=this._domain;e.length=0,this.index=new Map;var r=this.index;t.forEach(function(n){r.get(n)===void 0&&r.set(n,e.push(n)-1)}),this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"range",{get:function(){return this._range},set:function(t){this._range[0]=t[0],this._range[1]=t[1],this.rescale()},enumerable:!0,configurable:!0}),i.prototype.ticks=function(){return this._domain},i.prototype.convert=function(t){var e=this.index.get(t);if(e===void 0)return NaN;var r=this.ordinalRange[e];return r===void 0?NaN:r},Object.defineProperty(i.prototype,"bandwidth",{get:function(){return this._bandwidth},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"padding",{get:function(){return this._paddingInner},set:function(t){t=Math.max(0,Math.min(1,t)),this._paddingInner=t,this._paddingOuter=t,this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"paddingInner",{get:function(){return this._paddingInner},set:function(t){this._paddingInner=Math.max(0,Math.min(1,t)),this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"paddingOuter",{get:function(){return this._paddingOuter},set:function(t){this._paddingOuter=Math.max(0,Math.min(1,t)),this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"round",{get:function(){return this._round},set:function(t){this._round=t,this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"align",{get:function(){return this._align},set:function(t){this._align=Math.max(0,Math.min(1,t)),this.rescale()},enumerable:!0,configurable:!0}),i.prototype.rescale=function(){var t,e=this._domain.length;if(!!e){var r=Dx(this._range,2),n=r[0],o=r[1],a=o<n;a&&(t=Dx([o,n],2),n=t[0],o=t[1]);var s=(o-n)/Math.max(1,e-this._paddingInner+this._paddingOuter*2);this._round&&(s=Math.floor(s)),n+=(o-n-s*(e-this._paddingInner))*this._align,this._bandwidth=s*(1-this._paddingInner),this._round&&(n=Math.round(n),this._bandwidth=Math.round(this._bandwidth));for(var l=[],u=0;u<e;u++)l.push(n+s*u);this.ordinalRange=a?l.reverse():l}},i}(),m6=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ps=function(i){m6(t,i);function t(){var e=i.call(this,new Ni)||this;return e.scale.paddingInner=.2,e.scale.paddingOuter=.3,e}return Object.defineProperty(t.prototype,"paddingInner",{get:function(){return this.scale.paddingInner},set:function(e){this.scale.paddingInner=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"paddingOuter",{get:function(){return this.scale.paddingOuter},set:function(e){this.scale.paddingOuter=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"domain",{get:function(){return this.scale.domain.slice()},set:function(e){this.scale.domain=e.filter(function(r,n,o){return o.indexOf(r)===n})},enumerable:!0,configurable:!0}),t.className="CategoryAxis",t.type="category",t}(Tf),yA=function(){function i(t,e,r){t===void 0&&(t=""),r===void 0&&(r=0),this.x=0,this.y=0,this.subtreeLeft=NaN,this.subtreeRight=NaN,this.screenX=0,this.screenY=0,this.children=[],this.leafCount=0,this.prelim=0,this.mod=0,this.ancestor=this,this.change=0,this.shift=0,this.label=t,this.parent=e,this.depth=e?e.depth+1:0,this.number=r}return i.prototype.getLeftSibling=function(){return this.number>0&&this.parent?this.parent.children[this.number-1]:void 0},i.prototype.getLeftmostSibling=function(){return this.number>0&&this.parent?this.parent.children[0]:void 0},i.prototype.nextLeft=function(){return this.children?this.children[0]:this.thread},i.prototype.nextRight=function(){return this.children?this.children[this.children.length-1]:this.thread},i.prototype.getSiblings=function(){var t=this;return this.parent?this.parent.children.filter(function(e,r){return r!==t.number}):[]},i}();function y6(i,t){t===void 0&&(t=!0);var e=new yA,r=0;return t&&i.forEach(function(n){return r=Math.max(r,n.labels.length)}),i.forEach(function(n){if(t)for(;n.labels.length<r;)n.labels.unshift("");C6(e,n)}),e}function C6(i,t){var e=t.labels.slice().reverse(),r=e.length-1;e.forEach(function(n,o){var a=i.children,s=Ay(a,function(c){return c.label===n}),l=o!==r;if(s&&l)i=s;else{var u=new yA(n,i);u.number=a.length,a.push(u),l&&(i=u)}})}function w6(i,t,e){var r=t.number-i.number,n=e/r;t.change-=n,t.shift+=e,i.change+=n,t.prelim+=e,t.mod+=e}function b6(i,t,e){return t.getSiblings().indexOf(i.ancestor)>=0?i.ancestor:e}function _6(i){var t=i.children;if(t)for(var e=0,r=0,n=t.length-1;n>=0;n--){var o=t[n];o.prelim+=e,o.mod+=e,r+=o.change,e+=o.shift+r}}function S6(i,t,e){var r=i.getLeftSibling();if(r){for(var n=i,o=i,a=r,s=o.getLeftmostSibling(),l=o.mod,u=n.mod,c=a.mod,p=s.mod;a.nextRight()&&o.nextLeft();){a=a.nextRight(),o=o.nextLeft(),s=s.nextLeft(),n=n.nextRight(),n.ancestor=i;var h=a.prelim+c-(o.prelim+l)+e;h>0&&(w6(b6(a,i,t),i,h),l+=h,u+=h),c+=a.mod,l+=o.mod,p+=s.mod,u+=n.mod}a.nextRight()&&!n.nextRight()?(n.thread=a.nextRight(),n.mod+=c-u):(o.nextLeft()&&!s.nextLeft()&&(s.thread=o.nextLeft(),s.mod+=l-p),t=i)}return t}function CA(i,t){var e=i.children;if(e.length){var r=e[0];e.forEach(function(a){CA(a,t),r=S6(a,r,t)}),_6(i);var n=(e[0].prelim+e[e.length-1].prelim)/2,o=i.getLeftSibling();o?(i.prelim=o.prelim+t,i.mod=i.prelim-n):i.prelim=n}else{var o=i.getLeftSibling();i.prelim=o?o.prelim+t:0}}var Mx=function(){function i(){this.top=1/0,this.right=-1/0,this.bottom=-1/0,this.left=1/0}return i.prototype.update=function(t,e){var r=e(t),n=r.x,o=r.y;n>this.right&&(this.right=n),n<this.left&&(this.left=n),o>this.bottom&&(this.bottom=o),o<this.top&&(this.top=o)},i}();function wA(i,t,e){i.x=i.prelim+t,i.y=i.depth,e.update(i),i.children.forEach(function(r){return wA(r,t+i.mod,e)})}function bA(i){var t=i.children,e=0;t.forEach(function(r){bA(r),r.children.length?e+=r.leafCount:e++}),i.leafCount=e,t.length?(i.subtreeLeft=t[0].subtreeLeft,i.subtreeRight=t[i.children.length-1].subtreeRight,i.x=(i.subtreeLeft+i.subtreeRight)/2):(i.subtreeLeft=i.x,i.subtreeRight=i.x)}function x6(i){var t=new T6;return CA(i,1),wA(i,-i.prelim,t),bA(i),t}var T6=function(){function i(){this.dimensions=new Mx,this.leafCount=0,this.nodes=[],this.leafNodes=[],this.nonLeafNodes=[],this.depth=0}return i.prototype.update=function(t){this.dimensions.update(t,function(e){return{x:e.x,y:e.y}}),t.children.length?this.nonLeafNodes.push(t):(this.leafCount++,this.leafNodes.push(t)),t.depth>this.depth&&(this.depth=t.depth),this.nodes.push(t)},i.prototype.resize=function(t,e,r,n,o){r===void 0&&(r=0),n===void 0&&(n=0),o===void 0&&(o=!1);var a=this.leafCount-1,s=this.depth,l=this.dimensions,u=1,c=1;if(t>0&&a){var p=(l.right-l.left)/a,h=t/a;u=h/p,o&&(u=-u)}if(e>0&&s){var d=(l.bottom-l.top)/s,f=e/s;c=f/d}var g=new Mx;this.nodes.forEach(function(C){C.screenX=C.x*u,C.screenY=C.y*c,g.update(C,function(w){return{x:w.screenX,y:w.screenY}})});var v=-g.left,m=-g.top;this.nodes.forEach(function(C){C.screenX+=v+r,C.screenY+=m+n})},i}(),_A=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),O6=function(i){_A(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.grid=!1,e}return t}(gA),Il=function(i){_A(t,i);function t(){var e=i.call(this,new Ni)||this;e.tickScale=new Ni,e.longestSeparatorLength=0,e.translation={x:0,y:0},e.rotation=0,e.line={width:1,color:"rgba(195, 195, 195, 1)"},e.label=new O6,e.labelColor="rgba(87, 87, 87, 1)";var r=e,n=r.group,o=r.tickScale,a=r.scale;return a.paddingOuter=.1,a.paddingInner=a.paddingOuter*2,e.requestedRange=a.range.slice(),e.scale=a,o.paddingInner=1,o.paddingOuter=0,e.gridLineSelection=Ht.select(n).selectAll(),e.axisLineSelection=Ht.select(n).selectAll(),e.separatorSelection=Ht.select(n).selectAll(),e.labelSelection=Ht.select(n).selectAll(),e}return Object.defineProperty(t.prototype,"domain",{get:function(){return this.scale.domain},set:function(e){var r=e.filter(function(a,s,l){return l.indexOf(a)===s});this.scale.domain=r;var n=y6(r);this.tickTreeLayout=x6(n);var o=r.slice();o.push(""),this.tickScale.domain=o,this.resizeTickTree()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"range",{get:function(){return this.requestedRange.slice()},set:function(e){this.requestedRange=e.slice(),this.updateRange()},enumerable:!0,configurable:!0}),t.prototype.updateRange=function(){var e=this,r=e.requestedRange,n=e.visibleRange,o=e.scale,a=(r[1]-r[0])/(n[1]-n[0]),s=a*n[0],l=r[0]-s;this.tickScale.range=o.range=[l,l+a],this.resizeTickTree()},t.prototype.resizeTickTree=function(){var e=this.scale,r=e.domain.length?[e.convert(e.domain[0]),e.convert(e.domain[e.domain.length-1])]:e.range,n=this.tickTreeLayout,o=this.lineHeight;n&&n.resize(Math.abs(r[1]-r[0]),n.depth*o,(Math.min(r[0],r[1])||0)+(e.bandwidth||0)/2,-n.depth*o,r[1]-r[0]<0)},Object.defineProperty(t.prototype,"lineHeight",{get:function(){return this.label.fontSize*1.5},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"gridLength",{get:function(){return this._gridLength},set:function(e){(this._gridLength&&!e||!this._gridLength&&e)&&(this.gridLineSelection=this.gridLineSelection.remove().setData([]),this.labelSelection=this.labelSelection.remove().setData([])),this._gridLength=e},enumerable:!0,configurable:!0}),t.prototype.update=function(){var e=this,r=this,n=r.group,o=r.scale,a=r.label,s=r.tickScale,l=r.requestedRange,u=o.range[0],c=o.range[1],p=Math.abs(c-u),h=p/o.domain.length||0,d=a.parallel,f=Mt(this.rotation),g=Math.abs(Math.cos(f))<1e-8,v=this.label.rotation?Mn(Mt(this.label.rotation)):0;n.translationX=this.translation.x,n.translationY=this.translation.y,n.rotation=f;var m=this.title;m&&(m.node.visible=!1);var C=this.lineHeight,w=this.tickTreeLayout,b=o.ticks(),S=w?w.nodes:[],T=w?w.depth>1:!1,E=s.ticks(),P=a.mirrored?1:-1,R=Mn(f),M=!v&&R>=0&&R<=Math.PI?-1:1,N=Mn(f-Math.PI/2),W=!v&&N>=0&&N<=Math.PI?-1:1,G=this.gridLineSelection.setData(this.gridLength?E:[]);G.exit.remove();var k=G.enter.append(ri),V=G.merge(k),U=this.labelSelection.setData(S);U.exit.remove();var J=U.enter.append(Pr),Z=U.merge(J),le=a.formatter,Y=new Map,ae=0;Z.each(function(ne,ke,qe){ne.fontStyle=a.fontStyle,ne.fontWeight=a.fontWeight,ne.fontSize=a.fontSize,ne.fontFamily=a.fontFamily,ne.fill=a.color,ne.textBaseline=M===-1?"bottom":"hanging",ne.textAlign="center",ne.translationX=ke.screenY-a.fontSize*.25,ne.translationY=ke.screenX,qe===0?m&&m.enabled&&b.length>0?(ne.visible=!0,ne.text=m.text,ne.fontSize=m.fontSize,ne.fontStyle=m.fontStyle,ne.fontWeight=m.fontWeight,ne.fontFamily=m.fontFamily,ne.textBaseline="hanging"):ne.visible=!1:(ne.text=le?le({value:String(ke.label),index:qe}):String(ke.label),ne.visible=ke.screenX>=l[0]&&ke.screenX<=l[1]);var it=ne.computeBBox();Y.set(ne.id,it),it.width>ae&&(ae=it.width)});var z=P*a.padding,X=d?M*Math.PI/2:W===-1?Math.PI:0,ue=this.label.grid,pe=[];Z.each(function(ne,ke,qe){if(ne.x=z,ne.rotationCenterX=z,ke.children.length){ne.translationX-=ae-C+e.label.padding;var mn=ke.leafCount*h,it=Y.get(ne.id);it&&it.width>mn?ne.visible=!1:g?ne.rotation=X:ne.rotation=-Math.PI/2}else{ne.rotation=v,ne.textAlign="end",ne.textBaseline="middle";var it=Y.get(ne.id);it&&it.height>h&&(ne.visible=!1)}if(ke.parent&&T){var Yr=ke.children.length?ke.screenX-ke.leafCount*h/2:ke.screenX-h/2;if(!ke.children.length)(ke.number!==ke.children.length-1||ue)&&pe.push({y:Yr,x1:0,x2:-ae-e.label.padding*2,toString:function(){return String(qe)}});else{var xr=-ae-e.label.padding*2+ke.screenY;pe.push({y:Yr,x1:xr+C,x2:xr,toString:function(){return String(qe)}})}}});var be=0;pe.forEach(function(ne){return be=Math.min(be,ne.x2)}),this.longestSeparatorLength=Math.abs(be),pe.push({y:Math.max(u,c),x1:0,x2:be,toString:function(){return String(pe.length)}});var _e=this.separatorSelection.setData(pe);_e.exit.remove();var Te=_e.enter.append(ri),Pe=_e.merge(Te);this.separatorSelection=Pe;var Ee=1e-7;Pe.each(function(ne,ke,qe){ne.x1=ke.x1,ne.x2=ke.x2,ne.y1=ke.y,ne.y2=ke.y,ne.visible=ke.y>=l[0]-Ee&&ke.y<=l[1]+Ee,ne.stroke=e.tick.color,ne.fill=void 0,ne.strokeWidth=1}),this.gridLineSelection=V,this.labelSelection=Z;for(var Le=w?w.depth+1:1,Ze=[],Re=0;Re<Le;Re++)Ze.push(Re);var Ge=this.axisLineSelection.setData(Ze);Ge.exit.remove();var Xe=Ge.enter.append(ri),ie=Ge.merge(Xe);if(this.axisLineSelection=ie,ie.each(function(ne,ke,qe){var it=qe>0?-ae-e.label.padding*2-(qe-1)*C:0;ne.x1=it,ne.x2=it,ne.y1=l[0],ne.y2=l[1],ne.strokeWidth=e.line.width,ne.stroke=e.line.color,ne.visible=b.length>0&&(qe===0||ue&&T)}),this.gridLength){var He=this.gridStyle,Me=He.length;V.each(function(ne,ke,qe){var it=Math.round(s.convert(ke));ne.x1=0,ne.x2=-P*e.gridLength,ne.y1=it,ne.y2=it,ne.visible=it>=l[0]&&it<=l[1]&&Math.abs(ne.parent.translationY-u)>1;var mn=He[qe%Me];ne.stroke=mn.stroke,ne.strokeWidth=e.tick.width,ne.lineDash=mn.lineDash,ne.fill=void 0})}},t.prototype.computeBBox=function(e){var r=!e||!e.excludeTitle,n=1/0,o=-1/0,a=1/0,s=-1/0;return this.labelSelection.each(function(l,u,c){if(c>0||r){l.computeTransformMatrix();var p=Zo.flyweight(l.matrix),h=l.computeBBox();if(h){var d=p.transformBBox(h);n=Math.min(n,d.x),o=Math.max(o,d.x+d.width),a=Math.min(a,d.y),s=Math.max(s,d.y+d.height)}}}),new Lt(n,a,Math.max(o-n,this.longestSeparatorLength),s-a)},t.className="GroupedCategoryAxis",t.type="groupedCategory",t}(Tf),E6=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Dv=new Date,Mv=new Date,SA=function(){function i(t,e){this._floor=t,this._offset=e}return i.prototype.floor=function(t){return t=new Date(+t),this._floor(t),t},i.prototype.ceil=function(t){return t=new Date(+t-1),this._floor(t),this._offset(t,1),this._floor(t),t},i.prototype.round=function(t){var e=this.floor(t),r=this.ceil(t),n=+t;return n-e.getTime()<r.getTime()-n?e:r},i.prototype.offset=function(t,e){return e===void 0&&(e=1),t=new Date(+t),this._offset(t,Math.floor(e)),t},i.prototype.range=function(t,e,r){r===void 0&&(r=1);var n=[];if(t=this.ceil(t),r=Math.floor(r),t>e||r<=0)return n;var o;do o=new Date(+t),n.push(o),this._offset(t,r),this._floor(t);while(o<t&&t<e);return n},i.prototype.filter=function(t){var e=this,r=function(o){if(o>=o)for(;e._floor(o),!t(o);)o.setTime(o.getTime()-1);return o},n=function(o,a){if(o>=o)if(a<0)for(;++a<=0;)do e._offset(o,-1);while(!t(o));else for(;--a>=0;)do e._offset(o,1);while(!t(o));return o};return new i(r,n)},i}(),Kr=function(i){E6(t,i);function t(e,r,n,o){var a=i.call(this,e,r)||this;return a._count=n,a._field=o,a}return t.prototype.count=function(e,r){return Dv.setTime(+e),Mv.setTime(+r),this._floor(Dv),this._floor(Mv),Math.floor(this._count(Dv,Mv))},t.prototype.every=function(e){var r=this,n;if(e=Math.floor(e),isFinite(e)&&e>0)if(e>1){var o=this._field;o?n=this.filter(function(a){return o(a)%e===0}):n=this.filter(function(a){return r.count(0,a)%e===0})}else n=this;return n},t}(SA);function R6(i){return i}function P6(i,t){i.setTime(i.getTime()+t)}function A6(i,t){return t.getTime()-i.getTime()}var D6=new Kr(R6,P6,A6),Qi=1e3,Nn=Qi*60,Ji=Nn*60,hs=Ji*24,Jw=hs*7,Fx=hs*30,Fv=hs*365;function M6(i){i.setTime(i.getTime()-i.getMilliseconds())}function F6(i,t){i.setTime(i.getTime()+t*Qi)}function I6(i,t){return(t.getTime()-i.getTime())/Qi}var L6=new Kr(M6,F6,I6);function N6(i){i.setTime(i.getTime()-i.getMilliseconds()-i.getSeconds()*Qi)}function G6(i,t){i.setTime(i.getTime()+t*Nn)}function k6(i,t){return(t.getTime()-i.getTime())/Nn}function W6(i){return i.getMinutes()}var V6=new Kr(N6,G6,k6,W6);function B6(i){i.setTime(i.getTime()-i.getMilliseconds()-i.getSeconds()*Qi-i.getMinutes()*Nn)}function H6(i,t){i.setTime(i.getTime()+t*Ji)}function j6(i,t){return(t.getTime()-i.getTime())/Ji}function U6(i){return i.getHours()}var z6=new Kr(B6,H6,j6,U6);function $6(i){i.setHours(0,0,0,0)}function K6(i,t){i.setDate(i.getDate()+t)}function Y6(i,t){var e=t.getTimezoneOffset()-i.getTimezoneOffset();return(t.getTime()-i.getTime()-e*Nn)/hs}function X6(i){return i.getDate()-1}var My=new Kr($6,K6,Y6,X6);function Ss(i){function t(n){n.setDate(n.getDate()-(n.getDay()+7-i)%7),n.setHours(0,0,0,0)}function e(n,o){n.setDate(n.getDate()+o*7)}function r(n,o){var a=o.getTime()-n.getTime(),s=o.getTimezoneOffset()-n.getTimezoneOffset();return(a-s*Nn)/Jw}return new Kr(t,e,r)}var xA=Ss(0),Iv=Ss(1);Ss(2);Ss(3);var Lv=Ss(4);Ss(5);Ss(6);function q6(i){i.setDate(1),i.setHours(0,0,0,0)}function Z6(i,t){i.setMonth(i.getMonth()+t)}function Q6(i,t){return t.getMonth()-i.getMonth()+(t.getFullYear()-i.getFullYear())*12}function J6(i){return i.getMonth()}var e$=new Kr(q6,Z6,Q6,J6);function t$(i){i.setMonth(0,1),i.setHours(0,0,0,0)}function r$(i,t){i.setFullYear(i.getFullYear()+t)}function n$(i,t){return t.getFullYear()-i.getFullYear()}function i$(i){return i.getFullYear()}var Ju=new Kr(t$,r$,n$,i$);function o$(i){i.setUTCHours(0,0,0,0)}function a$(i,t){i.setUTCDate(i.getUTCDate()+t)}function s$(i,t){return(t.getTime()-i.getTime())/hs}function l$(i){return i.getUTCDate()-1}var Ix=new Kr(o$,a$,s$,l$);function u$(i){i.setUTCMonth(0,1),i.setUTCHours(0,0,0,0)}function c$(i,t){i.setUTCFullYear(i.getUTCFullYear()+t)}function p$(i,t){return t.getUTCFullYear()-i.getUTCFullYear()}function h$(i){return i.getUTCFullYear()}var oh=new Kr(u$,c$,p$,h$);function xs(i){function t(n){n.setUTCDate(n.getUTCDate()-(n.getUTCDay()+7-i)%7),n.setHours(0,0,0,0)}function e(n,o){n.setUTCDate(n.getUTCDate()+o*7)}function r(n,o){return(o.getTime()-n.getTime())/Jw}return new Kr(t,e,r)}var d$=xs(0),Nv=xs(1);xs(2);xs(3);var Gv=xs(4);xs(5);xs(6);function f$(i){if(i.y>=0&&i.y<100){var t=new Date(-1,i.m,i.d,i.H,i.M,i.S,i.L);return t.setFullYear(i.y),t}return new Date(i.y,i.m,i.d,i.H,i.M,i.S,i.L)}function ah(i){if(i.y>=0&&i.y<100){var t=new Date(Date.UTC(-1,i.m,i.d,i.H,i.M,i.S,i.L));return t.setUTCFullYear(i.y),t}return new Date(Date.UTC(i.y,i.m,i.d,i.H,i.M,i.S,i.L))}function Mu(i){for(var t={},e=0,r=i.length;e<r;e++)t[i[e].toLowerCase()]=e;return t}function Fu(i){return{y:i,m:0,d:1,H:0,M:0,S:0,L:0}}var Lx=37,er=/^\s*\d+/,g$=/^%/,v$=/[\\^$*+?|[\]().{}]/g,m$=function(i){return i.replace(v$,"\\$&")},Iu=function(i){return new RegExp("^(?:"+i.map(m$).join("|")+")","i")},Nx={"-":"",_:" ","0":"0"};function ot(i,t,e){var r=i<0?"-":"",n=String(r?-i:i),o=n.length;return r+(o<e?new Array(e-o+1).join(t)+n:n)}function y$(i){var t=i.dateTime,e=i.date,r=i.time,n=i.periods,o=i.days,a=i.shortDays,s=i.months,l=i.shortMonths,u=Iu(n),c=Mu(n),p=Iu(o),h=Mu(o),d=Iu(a),f=Mu(a),g=Iu(s),v=Mu(s),m=Iu(l),C=Mu(l),w={a:U,A:J,b:pe,B:be,c:void 0,d:X,e:X,f:R,H:k,I:G,j:ue,L:M,m:_e,M:W,p:V,Q:bo,s:Qe,S:N,u:Z,U:le,V:Y,w:ae,W:z,x:void 0,X:void 0,y:Te,Y:Pe,Z:Ee,"%":ga},b={a:mn,A:Yr,b:qe,B:it,c:void 0,d:Me,e:Me,f:Le,H:ie,I:Xe,j:ne,L:Ze,m:ke,M:Ge,p:He,Q:bo,s:Qe,S:Re,u:xr,U:Xr,V:Bn,w:yo,W:Co,x:void 0,X:void 0,y:Gi,Y:wo,Z:fa,"%":ga},S={a:If,A:Lf,b:Bf,B:Hf,c:Uf,d:ru,e:ru,f:yt,H:va,I:va,j:Ff,L:We,m:jf,M:pr,p:Mf,Q:Kf,s:Yf,S:St,u:Nf,U:Gf,V:kf,w:Vf,W:Wf,x:zf,X:$f,y:Xf,Y:qf,Z:Zf,"%":Qf};w.x=E(e,w),w.X=E(r,w),w.c=E(t,w),b.x=E(e,b),b.X=E(r,b),b.c=E(t,b);function T(O,D){return function(F){var A=Fu(1900),Zt=P(A,O,F+="",0);if(Zt==F.length){if("Q"in A)return new Date(A.Q);if("p"in A&&(A.H=A.H%12+A.p*12),"V"in A){if(A.V<1||A.V>53)return;if("w"in A||(A.w=1),"Z"in A){var ye=ah(Fu(A.y)),Je=ye.getUTCDay();ye=Je>4||Je===0?Nv.ceil(ye):Nv.floor(ye),ye=Ix.offset(ye,(A.V-1)*7),A.y=ye.getUTCFullYear(),A.m=ye.getUTCMonth(),A.d=ye.getUTCDate()+(A.w+6)%7}else{var ye=D(Fu(A.y)),Je=ye.getDay();ye=Je>4||Je===0?Iv.ceil(ye):Iv.floor(ye),ye=My.offset(ye,(A.V-1)*7),A.y=ye.getFullYear(),A.m=ye.getMonth(),A.d=ye.getDate()+(A.w+6)%7}}else if("W"in A||"U"in A){"w"in A||(A.w="u"in A?A.u%7:"W"in A?1:0);var Je="Z"in A?ah(Fu(A.y)).getUTCDay():D(Fu(A.y)).getDay();A.m=0,A.d="W"in A?(A.w+6)%7+A.W*7-(Je+5)%7:A.w+A.U*7-(Je+6)%7}return"Z"in A?(A.H+=A.Z/100|0,A.M+=A.Z%100,ah(A)):D(A)}}}function E(O,D){return function(F){var A=[],Zt=O.length,ye=-1,Je=0;for(F instanceof Date||(F=new Date(+F));++ye<Zt;)if(O.charCodeAt(ye)===Lx){A.push(O.slice(Je,ye));var Gt=O.charAt(++ye),qr=Nx[Gt];qr!=null?Gt=O.charAt(++ye):qr=Gt==="e"?" ":"0";var Hn=D[Gt];Hn&&(Gt=Hn(F,qr)),A.push(Gt),Je=ye+1}return A.push(O.slice(Je,ye)),A.join("")}}function P(O,D,F,A){for(var Zt=0,ye=D.length,Je=F.length;Zt<ye;){if(A>=Je)return-1;var Gt=D.charCodeAt(Zt++);if(Gt===Lx){var qr=D.charAt(Zt++),Hn=S[qr in Nx?D.charAt(Zt++):qr];if(!Hn||(A=Hn(O,F,A))<0)return-1}else if(Gt!=F.charCodeAt(A++))return-1}return A}function R(O,D){return M(O,D)+"000"}function M(O,D){return ot(O.getMilliseconds(),D,3)}function N(O,D){return ot(O.getSeconds(),D,2)}function W(O,D){return ot(O.getMinutes(),D,2)}function G(O,D){return ot(O.getHours()%12||12,D,2)}function k(O,D){return ot(O.getHours(),D,2)}function V(O){return n[O.getHours()>=12?1:0]}function U(O){return a[O.getDay()]}function J(O){return o[O.getDay()]}function Z(O){var D=O.getDay();return D===0?7:D}function le(O,D){return ot(xA.count(Ju.floor(O),O),D,2)}function Y(O,D){var F=O.getDay();O=F>=4||F===0?Lv.floor(O):Lv.ceil(O);var A=Ju.floor(O);return ot(Lv.count(A,O)+(A.getDay()===4?1:0),D,2)}function ae(O){return O.getDay()}function z(O,D){return ot(Iv.count(Ju.floor(O),O),D,2)}function X(O,D){return ot(O.getDate(),D,2)}function ue(O,D){return ot(1+My.count(Ju.floor(O),O),D,3)}function pe(O){return l[O.getMonth()]}function be(O){return s[O.getMonth()]}function _e(O,D){return ot(O.getMonth()+1,D,2)}function Te(O,D){return ot(O.getFullYear()%100,D,2)}function Pe(O,D){return ot(O.getFullYear()%1e4,D,4)}function Ee(O){var D=O.getTimezoneOffset();return(D>0?"-":(D*=-1,"+"))+ot(Math.floor(D/60),"0",2)+ot(D%60,"0",2)}function Le(O,D){return Ze(O,D)+"000"}function Ze(O,D){return ot(O.getUTCMilliseconds(),D,3)}function Re(O,D){return ot(O.getUTCSeconds(),D,2)}function Ge(O,D){return ot(O.getUTCMinutes(),D,2)}function Xe(O,D){return ot(O.getUTCHours()%12||12,D,2)}function ie(O,D){return ot(O.getUTCHours(),D,2)}function He(O){return n[O.getUTCHours()>=12?1:0]}function Me(O,D){return ot(O.getUTCDate(),D,2)}function ne(O,D){return ot(1+Ix.count(oh.floor(O),O),D,3)}function ke(O,D){return ot(O.getUTCMonth()+1,D,2)}function qe(O){return l[O.getUTCMonth()]}function it(O){return s[O.getUTCMonth()]}function mn(O){return a[O.getUTCDay()]}function Yr(O){return o[O.getUTCDay()]}function xr(O){var D=O.getUTCDay();return D===0?7:D}function Xr(O,D){return ot(d$.count(oh.floor(O),O),D,2)}function Bn(O,D){var F=O.getUTCDay();O=F>=4||F===0?Gv.floor(O):Gv.ceil(O);var A=oh.floor(O);return ot(Gv.count(A,O)+(A.getUTCDay()===4?1:0),D,4)}function yo(O){return O.getUTCDay()}function Co(O,D){return ot(Nv.count(oh.floor(O),O),D,2)}function Gi(O,D){return ot(O.getUTCFullYear()%100,D,2)}function wo(O,D){return ot(O.getUTCFullYear()%1e4,D,4)}function fa(){return"+0000"}function ga(O){return"%"}function bo(O){return O.getTime()}function Qe(O){return Math.floor(O.getTime()/1e3)}function yt(O,D,F){var A=er.exec(D.slice(F,F+6));return A?(O.L=Math.floor(parseFloat(A[0])/1e3),F+A[0].length):-1}function We(O,D,F){var A=er.exec(D.slice(F,F+3));return A?(O.L=+A[0],F+A[0].length):-1}function St(O,D,F){var A=er.exec(D.slice(F,F+2));return A?(O.S=+A[0],F+A[0].length):-1}function pr(O,D,F){var A=er.exec(D.slice(F,F+2));return A?(O.M=+A[0],F+A[0].length):-1}function va(O,D,F){var A=er.exec(D.slice(F,F+2));return A?(O.H=+A[0],F+A[0].length):-1}function Mf(O,D,F){var A=u.exec(D.slice(F));return A?(O.p=c[A[0].toLowerCase()],F+A[0].length):-1}function ru(O,D,F){var A=er.exec(D.slice(F,F+2));return A?(O.d=+A[0],F+A[0].length):-1}function Ff(O,D,F){var A=er.exec(D.slice(F,F+3));return A?(O.m=0,O.d=+A[0],F+A[0].length):-1}function If(O,D,F){var A=d.exec(D.slice(F));return A?(O.w=f[A[0].toLowerCase()],F+A[0].length):-1}function Lf(O,D,F){var A=p.exec(D.slice(F));return A?(O.w=h[A[0].toLowerCase()],F+A[0].length):-1}function Nf(O,D,F){var A=er.exec(D.slice(F,F+1));return A?(O.u=+A[0],F+A[0].length):-1}function Gf(O,D,F){var A=er.exec(D.slice(F,F+2));return A?(O.U=+A[0],F+A[0].length):-1}function kf(O,D,F){var A=er.exec(D.slice(F,F+2));return A?(O.V=+A[0],F+A[0].length):-1}function Wf(O,D,F){var A=er.exec(D.slice(F,F+2));return A?(O.W=+A[0],F+A[0].length):-1}function Vf(O,D,F){var A=er.exec(D.slice(F,F+1));return A?(O.w=+A[0],F+A[0].length):-1}function Bf(O,D,F){var A=m.exec(D.slice(F));return A?(O.m=C[A[0].toLowerCase()],F+A[0].length):-1}function Hf(O,D,F){var A=g.exec(D.slice(F));return A?(O.m=v[A[0].toLowerCase()],F+A[0].length):-1}function jf(O,D,F){var A=er.exec(D.slice(F,F+2));return A?(O.m=parseFloat(A[0])-1,F+A[0].length):-1}function Uf(O,D,F){return P(O,t,D,F)}function zf(O,D,F){return P(O,e,D,F)}function $f(O,D,F){return P(O,r,D,F)}function Kf(O,D,F){var A=er.exec(D.slice(F));return A?(O.Q=+A[0],F+A[0].length):-1}function Yf(O,D,F){var A=er.exec(D.slice(F));return A?(O.Q=+A[0]*1e3,F+A[0].length):-1}function Xf(O,D,F){var A=er.exec(D.slice(F,F+2));return A?(O.y=+A[0]+(+A[0]>68?1900:2e3),F+A[0].length):-1}function qf(O,D,F){var A=er.exec(D.slice(F,F+4));return A?(O.y=+A[0],F+A[0].length):-1}function Zf(O,D,F){var A=/^(Z)|^([+-]\d\d)(?::?(\d\d))?/.exec(D.slice(F,F+6));return A?(O.Z=A[1]?0:-(A[2]+(A[3]||"00")),F+A[0].length):-1}function Qf(O,D,F){var A=g$.exec(D.slice(F,F+1));return A?F+A[0].length:-1}return{format:function(O){var D=E(O,w);return D.toString=function(){return O},D},parse:function(O){var D=T(O,f$);return D.toString=function(){return O},D},utcFormat:function(O){var D=E(O,b);return D.toString=function(){return O},D},utcParse:function(O){var D=T(O,ah);return D.toString=function(){return O},D}}}var eb;C$({dateTime:"%x, %X",date:"%-m/%-d/%Y",time:"%-I:%M:%S %p",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});function C$(i){return eb=y$(i)}var w$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),kv=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},b$=function(i){w$(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.type="time",e.year=Ju,e.month=e$,e.week=xA,e.day=My,e.hour=z6,e.minute=V6,e.second=L6,e.millisecond=D6,e.format=eb.format,e.tickIntervals=[[e.second,1,Qi],[e.second,5,5*Qi],[e.second,15,15*Qi],[e.second,30,30*Qi],[e.minute,1,Nn],[e.minute,5,5*Nn],[e.minute,15,15*Nn],[e.minute,30,30*Nn],[e.hour,1,Ji],[e.hour,3,3*Ji],[e.hour,6,6*Ji],[e.hour,12,12*Ji],[e.day,1,hs],[e.day,2,2*hs],[e.week,1,Jw],[e.month,1,Fx],[e.month,3,3*Fx],[e.year,1,Fv]],e.formatMillisecond=e.format(".%L"),e.formatSecond=e.format(":%S"),e.formatMinute=e.format("%I:%M"),e.formatHour=e.format("%I %p"),e.formatDay=e.format("%a %d"),e.formatWeek=e.format("%b %d"),e.formatMonth=e.format("%B"),e.formatYear=e.format("%Y"),e._domain=[new Date(2e3,0,1),new Date(2e3,0,2)],e}return t.prototype.defaultTickFormat=function(e){return(this.second.floor(e)<e?this.formatMillisecond:this.minute.floor(e)<e?this.formatSecond:this.hour.floor(e)<e?this.formatMinute:this.day.floor(e)<e?this.formatHour:this.month.floor(e)<e?this.week.floor(e)<e?this.formatDay:this.formatWeek:this.year.floor(e)<e?this.formatMonth:this.formatYear)(e)},t.prototype.tickInterval=function(e,r,n,o){var a;if(typeof e=="number"){var s=e,l=this.tickIntervals,u=Math.abs(n-r)/s,c=A8(l,u,function(p){return p[2]});c===l.length?(o=Ic(r/Fv,n/Fv,s),e=this.year):c?(a=kv(l[u/l[c-1][2]<l[c][2]/u?c-1:c],2),e=a[0],o=a[1]):(o=Math.max(Ic(r,n,e),1),e=this.millisecond)}return o==null?e:e.every(o)},Object.defineProperty(t.prototype,"domain",{get:function(){return i.prototype.getDomain.call(this).map(function(e){return new Date(e)})},set:function(e){i.prototype.setDomain.call(this,Array.prototype.map.call(e,function(r){return r instanceof Date?+r:+new Date(+r)}))},enumerable:!0,configurable:!0}),t.prototype.invert=function(e){return new Date(i.prototype.invert.call(this,e))},t.prototype.ticks=function(e){e===void 0&&(e=10);var r=i.prototype.getDomain.call(this),n=r[0],o=r[r.length-1],a=o<n;if(a){var s=n;n=o,o=s}var l=this.tickInterval(e,n,o),u=l?l.range(n,o+1):[];return a?u.reverse():u},t.prototype.tickFormat=function(e,r){return r==null?this.defaultTickFormat.bind(this):this.format(r)},t.prototype.nice=function(e){e===void 0&&(e=10);var r=i.prototype.getDomain.call(this),n=this.tickInterval(e,r[0],r[r.length-1]);n&&(this.domain=this._nice(r,n))},t.prototype._nice=function(e,r){var n,o;e=e.slice();var a=0,s=e.length-1,l=e[a],u=e[s];return u<l&&(n=kv([s,a],2),a=n[0],s=n[1],o=kv([u,l],2),l=o[0],u=o[1]),e[a]=r.floor(l),e[s]=r.ceil(u),e},t}(vr),_$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Lc=function(i){_$(t,i);function t(){var e=i.call(this,new b$)||this;e.datumFormat="%m/%d/%y, %H:%M:%S",e._nice=!0;var r=e.scale;return r.clamp=!0,e.scale=r,e.datumFormatter=r.tickFormat(e.calculatedTickCount,e.datumFormat),e}return Object.defineProperty(t.prototype,"nice",{get:function(){return this._nice},set:function(e){this._nice!==e&&(this._nice=e,e&&this.scale.nice&&this.scale.nice(10))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"domain",{get:function(){return this.scale.domain},set:function(e){e.length>2&&(e=(ti(e,Ar,Number)||[0,1e3]).map(function(r){return new Date(r)})),this.scale.domain=e,this.nice&&this.scale.nice&&this.scale.nice(10)},enumerable:!0,configurable:!0}),t.prototype.onLabelFormatChange=function(e){e?i.prototype.onLabelFormatChange.call(this,e):this.labelFormatter=this.scale.tickFormat(this.calculatedTickCount,void 0)},t.prototype.formatDatum=function(e){return this.datumFormatter(e)},t.className="TimeAxis",t.type="time",t}(Tf),S$=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},x$=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(S$(arguments[t]));return i},TA=function(){function i(t,e,r){var n=this;t===void 0&&(t=window.document),this.id=ql(this),this._dirty=!1,this.animationFrameId=0,this._root=null,this.debug={renderFrameIndex:!1,renderBoundingBoxes:!1},this._frameIndex=0,this.render=function(){var o,a=n,s=a.ctx,l=a.root,u=a.pendingSize;if(n.animationFrameId=0,u&&((o=n.canvas).resize.apply(o,x$(u)),n.pendingSize=void 0),l&&!l.visible){n.dirty=!1;return}s.clearRect(0,0,n.width,n.height),l&&(s.save(),l.visible&&l.render(s),s.restore()),n._frameIndex++,n.debug.renderFrameIndex&&(s.fillStyle="white",s.fillRect(0,0,40,15),s.fillStyle="black",s.fillText(n.frameIndex.toString(),2,10)),n.dirty=!1},this.canvas=new ns(t,e,r),this.ctx=this.canvas.context}return Object.defineProperty(i.prototype,"container",{get:function(){return this.canvas.container},set:function(t){this.canvas.container=t},enumerable:!0,configurable:!0}),i.prototype.download=function(t){this.canvas.download(t)},i.prototype.getDataURL=function(t){return this.canvas.getDataURL(t)},Object.defineProperty(i.prototype,"width",{get:function(){return this.pendingSize?this.pendingSize[0]:this.canvas.width},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"height",{get:function(){return this.pendingSize?this.pendingSize[1]:this.canvas.height},enumerable:!0,configurable:!0}),i.prototype.resize=function(t,e){t=Math.round(t),e=Math.round(e),!(t===this.width&&e===this.height)&&(this.pendingSize=[t,e],this.dirty=!0)},Object.defineProperty(i.prototype,"dirty",{get:function(){return this._dirty},set:function(t){t?this._dirty||(this.animationFrameId=requestAnimationFrame(this.render)):this.animationFrameId&&(cancelAnimationFrame(this.animationFrameId),this.animationFrameId=0),this._dirty=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"root",{get:function(){return this._root},set:function(t){t!==this._root&&(this._root&&this._root._setScene(),this._root=t,t&&(t.parent===null&&t.scene&&t.scene!==this&&(t.scene.root=null),t._setScene(this)),this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"frameIndex",{get:function(){return this._frameIndex},enumerable:!0,configurable:!0}),i.className="Scene",i}(),T$=function(){function i(){this.stops=[]}return i}(),O$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),E$=function(i){O$(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.angle=0,e}return t.prototype.createGradient=function(e,r){var n=this.stops,o=this.angle%360*Math.PI/180,a=Math.cos(o),s=Math.sin(o),l=r.width,u=r.height,c=r.x+l*.5,p=r.y+u*.5;if(l>0&&u>0){var h=Math.sqrt(u*u+l*l)*Math.abs(Math.cos(o-Math.atan(u/l)))/2,d=e.createLinearGradient(c+a*h,p+s*h,c-a*h,p-s*h);return n.forEach(function(f){d.addColorStop(f.offset,f.color)}),d}return"black"},t}(T$),R$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),vc;(function(i){i[i.Content=0]="Content",i[i.Border=1]="Border"})(vc||(vc={}));var fo=function(i){R$(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._x=0,e._y=0,e._width=10,e._height=10,e._radius=0,e._crisp=!1,e._gradient=!1,e.effectiveStrokeWidth=ua.defaultStyles.strokeWidth,e._sizing=vc.Content,e}return Object.defineProperty(t.prototype,"x",{get:function(){return this._x},set:function(e){this._x!==e&&(this._x=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this._y},set:function(e){this._y!==e&&(this._y=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this._width},set:function(e){this._width!==e&&(this._width=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return this._height},set:function(e){this._height!==e&&(this._height=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"radius",{get:function(){return this._radius},set:function(e){this._radius!==e&&(this._radius=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"crisp",{get:function(){return this._crisp},set:function(e){this._crisp!==e&&(this._crisp=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"gradient",{get:function(){return this._gradient},set:function(e){this._gradient!==e&&(this._gradient=e,this.updateGradientInstance(),this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.updateGradientInstance=function(){if(this.gradient){var e=this.fill;if(e){var r=new E$;r.angle=270,r.stops=[{offset:0,color:Xi.fromString(e).brighter().toString()},{offset:1,color:Xi.fromString(e).darker().toString()}],this.gradientInstance=r}}else this.gradientInstance=void 0},Object.defineProperty(t.prototype,"fill",{get:function(){return this._fill},set:function(e){this._fill!==e&&(this._fill=e,this.updateGradientInstance(),this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeWidth",{get:function(){return this._strokeWidth},set:function(e){this._strokeWidth!==e&&(this._strokeWidth=e,this.crisp||this.sizing===vc.Border?this.dirtyPath=!0:(this.effectiveStrokeWidth=e,this.dirty=!0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"sizing",{get:function(){return this._sizing},set:function(e){this._sizing!==e&&(this._sizing=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),t.prototype.updatePath=function(){var e=this.sizing===vc.Border,r=this.path;r.clear();var n=this.x,o=this.y,a=this.width,s=this.height,l;if(e){var u=a/2,c=s/2;l=Math.min(this.strokeWidth,u,c),n=Math.min(n+l/2,n+u),o=Math.min(o+l/2,o+c),a=Math.max(a-l,0),s=Math.max(s-l,0)}else l=this.strokeWidth;if(this.effectiveStrokeWidth=l,this.crisp&&!e){var p=this,h=p.alignment,d=p.align;r.rect(d(h,n),d(h,o),d(h,n,a),d(h,o,s))}else r.rect(n,o,a,s)},t.prototype.computeBBox=function(){var e=this,r=e.x,n=e.y,o=e.width,a=e.height;return new Lt(r,n,o,a)},t.prototype.isPointInPath=function(e,r){var n=this.transformPoint(e,r),o=this.computeBBox();return o.containsPoint(n.x,n.y)},t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.fillStroke=function(e){if(!!this.scene){var r=this.scene.canvas.pixelRatio||1;if(this.fill){this.gradientInstance?e.fillStyle=this.gradientInstance.createGradient(e,this.computeBBox()):e.fillStyle=this.fill,e.globalAlpha=this.opacity*this.fillOpacity;var n=this.fillShadow;n&&n.enabled&&(e.shadowColor=n.color,e.shadowOffsetX=n.xOffset*r,e.shadowOffsetY=n.yOffset*r,e.shadowBlur=n.blur*r),e.fill()}if(e.shadowColor="rgba(0, 0, 0, 0)",this.stroke&&this.effectiveStrokeWidth){e.strokeStyle=this.stroke,e.globalAlpha=this.opacity*this.strokeOpacity,e.lineWidth=this.effectiveStrokeWidth,this.lineDash&&e.setLineDash(this.lineDash),this.lineDashOffset&&(e.lineDashOffset=this.lineDashOffset),this.lineCap&&(e.lineCap=this.lineCap),this.lineJoin&&(e.lineJoin=this.lineJoin);var o=this.strokeShadow;o&&o.enabled&&(e.shadowColor=o.color,e.shadowOffsetX=o.xOffset*r,e.shadowOffsetY=o.yOffset*r,e.shadowBlur=o.blur*r),e.stroke()}}},t.className="Rect",t}($r),P$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ts=function(i){P$(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._x=0,e._y=0,e._size=12,e}return Object.defineProperty(t.prototype,"x",{get:function(){return this._x},set:function(e){this._x!==e&&(this._x=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this._y},set:function(e){this._y!==e&&(this._y=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return this._size},set:function(e){this._size!==e&&(this._size=Math.abs(e),this.dirtyPath=!0)},enumerable:!0,configurable:!0}),t.prototype.computeBBox=function(){var e=this,r=e.x,n=e.y,o=e.size,a=o/2;return new Lt(r-a,n-a,o,o)},t}($r),A$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ih=function(i){A$(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.updatePath=function(){var e=this,r=e.path,n=e.x,o=e.y,a=this.size/2,s=this,l=s.alignment,u=s.align;r.clear(),r.moveTo(u(l,n-a),u(l,o-a)),r.lineTo(u(l,n+a),u(l,o-a)),r.lineTo(u(l,n+a),u(l,o+a)),r.lineTo(u(l,n-a),u(l,o+a)),r.closePath()},t.className="Square",t}(Ts),D$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),M$=function(i){D$(t,i);function t(){var e=i.call(this)||this;e.label=new Pr,e._marker=new Ih,e._markerSize=15,e._spacing=8;var r=e.label;return r.textBaseline="middle",r.fontSize=12,r.fontFamily="Verdana, sans-serif",r.fill="black",r.y=ns.has.textMetrics?1:0,e.append([e.marker,r]),e.update(),e}return Object.defineProperty(t.prototype,"text",{get:function(){return this.label.text},set:function(e){this.label.text=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontStyle",{get:function(){return this.label.fontStyle},set:function(e){this.label.fontStyle=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontWeight",{get:function(){return this.label.fontWeight},set:function(e){this.label.fontWeight=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontSize",{get:function(){return this.label.fontSize},set:function(e){this.label.fontSize=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontFamily",{get:function(){return this.label.fontFamily},set:function(e){this.label.fontFamily=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"color",{get:function(){return this.label.fill},set:function(e){this.label.fill=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"marker",{get:function(){return this._marker},set:function(e){this._marker!==e&&(this.removeChild(this._marker),this._marker=e,this.appendChild(e),this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"markerSize",{get:function(){return this._markerSize},set:function(e){this._markerSize!==e&&(this._markerSize=e,this.update())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"markerFill",{get:function(){return this.marker.fill},set:function(e){this.marker.fill=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"markerStroke",{get:function(){return this.marker.stroke},set:function(e){this.marker.stroke=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"markerStrokeWidth",{get:function(){return this.marker.strokeWidth},set:function(e){this.marker.strokeWidth=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"markerFillOpacity",{get:function(){return this.marker.fillOpacity},set:function(e){this.marker.fillOpacity=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"markerStrokeOpacity",{get:function(){return this.marker.strokeOpacity},set:function(e){this.marker.strokeOpacity=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"opacity",{get:function(){return this.marker.opacity},set:function(e){this.marker.opacity=e,this.label.opacity=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"spacing",{get:function(){return this._spacing},set:function(e){this._spacing!==e&&(this._spacing=e,this.update())},enumerable:!0,configurable:!0}),t.prototype.update=function(){var e=this.marker,r=this.markerSize;e.size=r,this.label.x=r/2+this.spacing},t.className="MarkerLabel",t}(It),F$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),OA=function(i){F$(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.updatePath=function(){var e=this,r=e.x,n=e.y,o=e.path,a=e.size,s=a/2;o.clear(),o.cubicArc(r,n,s,s,0,0,Math.PI*2,0),o.closePath()},t.className="Circle",t}(Ts),I$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),L$=function(i){I$(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.updatePath=function(){var e=this,r=e.x,n=e.y,o=this,a=o.path,s=o.size,l=s/4.2;a.clear(),a.moveTo(r-=l,n),a.lineTo(r-=l,n-=l),a.lineTo(r+=l,n-=l),a.lineTo(r+=l,n+=l),a.lineTo(r+=l,n-=l),a.lineTo(r+=l,n+=l),a.lineTo(r-=l,n+=l),a.lineTo(r+=l,n+=l),a.lineTo(r-=l,n+=l),a.lineTo(r-=l,n-=l),a.lineTo(r-=l,n+=l),a.lineTo(r-=l,n-=l),a.closePath()},t.className="Cross",t}(Ts),N$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),G$=function(i){N$(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.updatePath=function(){var e=this,r=e.x,n=e.y,o=this,a=o.path,s=o.size,l=s/2;a.clear(),a.moveTo(r,n-=l),a.lineTo(r+=l,n+=l),a.lineTo(r-=l,n+=l),a.lineTo(r-=l,n-=l),a.lineTo(r+=l,n-=l),a.closePath()},t.className="Diamond",t}(Ts),k$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),W$=function(i){k$(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.rad=function(e){return e/180*Math.PI},t.prototype.updatePath=function(){var e=this,r=e.x,n=e.path,o=e.size,a=e.rad,s=o/4,l=this.y+s/2;n.clear(),n.cubicArc(r-s,l-s,s,s,0,a(130),a(330),0),n.cubicArc(r+s,l-s,s,s,0,a(220),a(50),0),n.lineTo(r,l+s),n.closePath()},t.className="Heart",t}(Ts),V$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),B$=function(i){V$(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.updatePath=function(){var e=this,r=e.x,n=e.y,o=this,a=o.path,s=o.size,l=s/3,u=l/2;a.clear(),a.moveTo(r-=u,n-=u),a.lineTo(r,n-=l),a.lineTo(r+=l,n),a.lineTo(r,n+=l),a.lineTo(r+=l,n),a.lineTo(r,n+=l),a.lineTo(r-=l,n),a.lineTo(r,n+=l),a.lineTo(r-=l,n),a.lineTo(r,n-=l),a.lineTo(r-=l,n),a.lineTo(r,n-=l),a.closePath()},t.className="Plus",t}(Ts),H$=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),j$=function(i){H$(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.updatePath=function(){var e=this,r=e.x,n=e.y,o=this,a=o.path,s=o.size,l=s*1.1;a.clear(),a.moveTo(r,n-=l*.48),a.lineTo(r+=l*.5,n+=l*.87),a.lineTo(r-=l,n),a.closePath()},t.className="Triangle",t}(Ts);function Ll(i){if(i===void 0&&(i=Ih),typeof i=="string")switch(i){case"circle":return OA;case"cross":return L$;case"diamond":return G$;case"heart":return W$;case"plus":return B$;case"triangle":return j$;default:return Ih}return typeof i=="function"?i:Ih}var Of=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),or=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ja;(function(i){i[i.Vertical=0]="Vertical",i[i.Horizontal=1]="Horizontal"})(ja||(ja={}));var Za;(function(i){i.Top="top",i.Right="right",i.Bottom="bottom",i.Left="left"})(Za||(Za={}));var U$=function(i){Of(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.color="black",e.fontSize=12,e.fontFamily="Verdana, sans-serif",e}return or([$("change")],t.prototype,"color",void 0),or([$("layoutChange")],t.prototype,"fontStyle",void 0),or([$("layoutChange")],t.prototype,"fontWeight",void 0),or([$("layoutChange")],t.prototype,"fontSize",void 0),or([$("layoutChange")],t.prototype,"fontFamily",void 0),or([$()],t.prototype,"formatter",void 0),t}(Ir),z$=function(i){Of(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.size=15,e.padding=8,e.strokeWidth=1,e}return or([$("layoutChange")],t.prototype,"size",void 0),or([$("layoutChange")],t.prototype,"shape",void 0),or([$("layoutChange")],t.prototype,"padding",void 0),or([$("change")],t.prototype,"strokeWidth",void 0),t}(Ir),$$=function(i){Of(t,i);function t(){var e=i.call(this)||this;e.marker=new z$,e.label=new U$,e.paddingX=16,e.paddingY=8;var r=function(){return e.fireEvent({type:"change"})};e.marker.addEventListener("change",r),e.label.addEventListener("change",r);var n=function(){return e.fireEvent({type:"layoutChange"})};return e.marker.addEventListener("layoutChange",n),e.label.addEventListener("layoutChange",n),e}return or([$("layoutChange")],t.prototype,"paddingX",void 0),or([$("layoutChange")],t.prototype,"paddingY",void 0),t}(Ir),K$=function(i){Of(t,i);function t(){var e=i.call(this)||this;return e.id=ql(e),e.group=new It,e.itemSelection=Ht.select(e.group).selectAll(),e.oldSize=[0,0],e.item=new $$,e.data=[],e.enabled=!0,e.orientation=ja.Vertical,e.position=Za.Right,e.spacing=20,e._size=[0,0],e.addPropertyListener("data",e.onDataChange),e.addPropertyListener("enabled",e.onEnabledChange),e.addPropertyListener("position",e.onPositionChange),e.item.marker.addPropertyListener("shape",e.onMarkerShapeChange,e),e.addEventListener("change",e.update),e.item.addEventListener("change",function(){return e.fireEvent({type:"change"})}),e.item.addEventListener("layoutChange",function(){return e.fireEvent({type:"layoutChange"})}),e}return Object.defineProperty(t.prototype,"size",{get:function(){return this._size},enumerable:!0,configurable:!0}),t.prototype.onDataChange=function(e){this.group.visible=e.value.length>0&&this.enabled},t.prototype.onEnabledChange=function(e){this.group.visible=e.value&&this.data.length>0},t.prototype.onPositionChange=function(e){switch(e.value){case"right":case"left":this.orientation=ja.Vertical;break;case"bottom":case"top":this.orientation=ja.Horizontal;break}},t.prototype.onMarkerShapeChange=function(){this.itemSelection=this.itemSelection.setData([]),this.itemSelection.exit.remove(),this.group.scene&&(this.group.scene.dirty=!1)},t.prototype.performLayout=function(e,r){var n=this.item,o=n.marker,a=n.paddingX,s=n.paddingY,l=this.itemSelection.setData(this.data,function(Z,le){var Y=Ll(o.shape||le.marker.shape);return le.id+"-"+le.itemId+"-"+Y.name});l.exit.remove();var u=l.enter.append(M$).each(function(Z,le){var Y=Ll(o.shape||le.marker.shape);Z.marker=new Y}),c=this.itemSelection=l.merge(u),p=c.size,h=[],d=this.item.marker,f=this.item.label,g=25,v="...";c.each(function(Z,le){Z.markerSize=d.size,Z.spacing=d.padding,Z.fontStyle=f.fontStyle,Z.fontWeight=f.fontWeight,Z.fontSize=f.fontSize,Z.fontFamily=f.fontFamily;var Y=le.label.text;Y.length>g&&(Y=""+Y.substring(0,g-v.length)+v),Z.text=Y,h.push(Z.computeBBox())});var m=h.length&&h[0].height,C=0,w=0,b=0,S=0;switch(e=Math.max(1,e),r=Math.max(1,r),this.orientation){case ja.Horizontal:if(!(isFinite(e)&&e>0))return!1;C=0;var T=0;do{var E=0;T=0,w=0,C++;for(var P=0;P<p;){var R=h[P];R.width>w&&(w=R.width),P++,P%C===0&&(E+=w,w=0,T++)}P%C!==0&&(E+=w,T++),b=E+(T-1)*a}while(b>e&&T>1);S=m*C+(C-1)*s;break;case ja.Vertical:if(!(isFinite(r)&&r>0))return!1;C=p*2;do{C=(C>>1)+C%2,w=0;for(var E=0,M=0,N=0,P=0;P<p;){var R=h[P];N||(M+=R.height),R.width>w&&(w=R.width),P++,P%C===0&&(E+=w,w=0,N++)}P%C!==0&&(E+=w,N++),b=E+(N-1)*a,S=M+(C-1)*s}while(S>r&&C>1);break}var W=(e-b)/2,G=(r-S)/2,k=0,V=0;w=0,c.each(function(Z,le,Y){Z.translationX=Math.floor(W+k),Z.translationY=Math.floor(G+V);var ae=h[Y];ae.width>w&&(w=ae.width),(Y+1)%C===0?(k+=w+a,V=0,w=0):V+=ae.height+s}),this.update();var U=this._size,J=this.oldSize;U[0]=b,U[1]=S,(U[0]!==J[0]||U[1]!==J[1])&&(J[0]=U[0],J[1]=U[1])},t.prototype.update=function(){var e=this;this.itemSelection.each(function(r,n){var o=n.marker;r.markerFill=o.fill,r.markerStroke=o.stroke,r.markerStrokeWidth=e.item.marker.strokeWidth,r.markerFillOpacity=o.fillOpacity,r.markerStrokeOpacity=o.strokeOpacity,r.opacity=n.enabled?1:.5,r.color=e.item.label.color})},t.prototype.getDatumForPoint=function(e,r){var n=this.group.pickNode(e,r);if(n&&n.parent)return n.parent.datum},t.className="Legend",or([$("layoutChange")],t.prototype,"data",void 0),or([$("layoutChange")],t.prototype,"enabled",void 0),or([$("layoutChange")],t.prototype,"orientation",void 0),or([$("layoutChange")],t.prototype,"position",void 0),or([$("layoutChange")],t.prototype,"spacing",void 0),t}(Ir),Y$=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},Wv=function(){function i(){}return i.init=function(){var t=this,e=window.ResizeObserver;if(e)this.resizeObserver=new e(function(n){var o,a;try{for(var s=Y$(n),l=s.next();!l.done;l=s.next()){var u=l.value,c=u.contentRect,p=c.width,h=c.height;t.checkSize(t.elements.get(u.target),u.target,p,h)}}catch(d){o={error:d}}finally{try{l&&!l.done&&(a=s.return)&&a.call(s)}finally{if(o)throw o.error}}});else{var r=function(){t.elements.forEach(function(n,o){t.checkClientSize(o,n)})};window.setInterval(r,100)}this.ready=!0},i.checkSize=function(t,e,r,n){t&&(!t.size||r!==t.size.width||n!==t.size.height)&&(t.size={width:r,height:n},t.cb(t.size,e))},i.observe=function(t,e){this.ready||this.init(),this.unobserve(t),this.resizeObserver&&this.resizeObserver.observe(t),this.elements.set(t,{cb:e}),this.checkClientSize(t,{cb:e})},i.unobserve=function(t){this.resizeObserver&&this.resizeObserver.unobserve(t),this.elements.delete(t)},i.checkClientSize=function(t,e){var r=t.clientWidth?t.clientWidth:0,n=t.clientHeight?t.clientHeight:0;this.checkSize(e,t,r,n)},i.elements=new Map,i.ready=!1,i}();function X$(i,t,e,r,n,o,a){var s=i<r?r:i>r+o?r+o:i,l=t<n?n:t>n+a?n+a:t,u=i-s,c=t-l,p=Math.sqrt(u*u+c*c);return p<=e}function q$(i,t,e,r,n,o,a,s){var l=i+e>n&&i<n+a,u=t+r>o&&t<o+s;return l&&u}function Z$(i,t,e,r,n,o,a,s){return n+a<i+e&&n>i&&o>t&&o+s<t+r}function Q$(i,t,e){e===void 0&&(e=5);var r=[];i=i.map(function(c){return c.slice().sort(function(p,h){return h.size-p.size})});for(var n=0;n<i.length;n++){var o=r[n]=[],a=i[n];if(!!(a&&a.length&&a[0].label))for(var s=function(c,p){var h=a[c],d=h.label,f=h.size*.5,g=h.point.x-d.width*.5,v=h.point.y-f-d.height-e,m=d.width,C=d.height,w=!t||Z$(t.x,t.y,t.width,t.height,g,v,m,C);if(!w)return"continue";var b=i.some(function(T){return T.some(function(E){return X$(E.point.x,E.point.y,E.size*.5,g,v,m,C)})});if(b)return"continue";var S=r.some(function(T){return T.some(function(E){return q$(E.x,E.y,E.width,E.height,g,v,m,C)})});if(S)return"continue";o.push({index:c,text:d.text,x:g,y:v,width:m,height:C})},l=0,u=a.length;l<u;l++)s(l,u)}return r}var EA=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Pd=globalThis&&globalThis.__assign||function(){return Pd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Pd.apply(this,arguments)},ml=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},J$=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},e9=`
.ag-chart-tooltip {
    display: table;
    position: absolute;
    user-select: none;
    pointer-events: none;
    white-space: nowrap;
    z-index: 99999;
    font: 12px Verdana, sans-serif;
    color: black;
    background: rgb(244, 244, 244);
    border-radius: 5px;
    box-shadow: 0 0 1px rgba(3, 3, 3, 0.7), 0.5vh 0.5vh 1vh rgba(3, 3, 3, 0.25);
}

.ag-chart-tooltip-hidden {
    top: -10000px !important;
}

.ag-chart-tooltip-title {
    font-weight: bold;
    padding: 7px;
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
    color: white;
    background-color: #888888;
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
}

.ag-chart-tooltip-content {
    padding: 7px;
    line-height: 1.7em;
    border-bottom-left-radius: 5px;
    border-bottom-right-radius: 5px;
    overflow: hidden;
}

.ag-chart-tooltip-content:empty {
    padding: 0;
    height: 7px;
}

.ag-chart-tooltip-arrow::before {
    content: "";

    position: absolute;
    top: 100%;
    left: 50%;
    transform: translateX(-50%);

    border: 6px solid #989898;

    border-left-color: transparent;
    border-right-color: transparent;
    border-top-color: #989898;
    border-bottom-color: transparent;

    width: 0;
    height: 0;

    margin: 0 auto;
}

.ag-chart-tooltip-arrow::after {
    content: "";

    position: absolute;
    top: 100%;
    left: 50%;
    transform: translateX(-50%);

    border: 5px solid black;

    border-left-color: transparent;
    border-right-color: transparent;
    border-top-color: rgb(244, 244, 244);
    border-bottom-color: transparent;

    width: 0;
    height: 0;

    margin: 0 auto;
}

.ag-chart-wrapper {
    box-sizing: border-box;
    overflow: hidden;
}
`;function br(i,t){if(typeof i=="string")return i;t=t||{};var e=i.content,r=e===void 0?t.content||"":e,n=i.title,o=n===void 0?t.title||void 0:n,a=i.color,s=a===void 0?t.color||"white":a,l=i.backgroundColor,u=l===void 0?t.backgroundColor||"#888":l,c=o?'<div class="'+An.defaultTooltipClass+`-title"
        style="color: `+s+"; background-color: "+u+'">'+o+"</div>":"";return c+'<div class="'+An.defaultTooltipClass+'-content">'+r+"</div>"}var t9=function(i){EA(t,i);function t(e,r){var n=i.call(this)||this;n.enabled=!0,n.class=An.defaultTooltipClass,n.delay=0,n.tracking=!0,n.showTimeout=0,n.constrained=!1,n.chart=e,n.class="";var o=r.body,a=r.createElement("div");if(n.element=o.appendChild(a),window.IntersectionObserver){var s=n.chart.scene.canvas.element,l=new IntersectionObserver(function(u){var c,p;try{for(var h=J$(u),d=h.next();!d.done;d=h.next()){var f=d.value;f.target===s&&f.intersectionRatio===0&&n.toggle(!1)}}catch(g){c={error:g}}finally{try{d&&!d.done&&(p=h.return)&&p.call(h)}finally{if(c)throw c.error}}},{root:o});l.observe(s),n.observer=l}return n}return t.prototype.destroy=function(){var e=this.element.parentNode;e&&e.removeChild(this.element),this.observer&&this.observer.unobserve(this.chart.scene.canvas.element)},t.prototype.isVisible=function(){var e=this.element;if(e.classList)return!e.classList.contains(An.defaultTooltipClass+"-hidden");var r=e.getAttribute("class");return r?r.split(" ").indexOf(An.defaultTooltipClass+"-hidden")<0:!1},t.prototype.updateClass=function(e,r){var n=[An.defaultTooltipClass,this.class];e!==!0&&n.push(An.defaultTooltipClass+"-hidden"),r!==!0&&n.push(An.defaultTooltipClass+"-arrow"),this.element.setAttribute("class",n.join(" "))},t.prototype.show=function(e,r,n){var o=this;n===void 0&&(n=!1);var a=this.element;if(r!==void 0)a.innerHTML=r;else if(!a.innerHTML)return;var s=e.pageX-a.clientWidth/2,l=e.pageY-a.clientHeight-8;if(this.constrained=!1,this.chart.container){var u=a.getBoundingClientRect(),c=0,p=window.innerWidth-u.width-1;s<c?(s=c,this.updateClass(!0,this.constrained=!0)):s>p&&(s=p,this.updateClass(!0,this.constrained=!0)),l<window.pageYOffset&&(l=e.pageY+20,this.updateClass(!0,this.constrained=!0))}if(a.style.left=Math.round(s)+"px",a.style.top=Math.round(l)+"px",this.delay>0&&!n){this.toggle(!1),this.showTimeout=window.setTimeout(function(){o.toggle(!0)},this.delay);return}this.toggle(!0)},t.prototype.toggle=function(e){e||(window.clearTimeout(this.showTimeout),this.chart.lastPick&&!this.delay&&(this.chart.dehighlightDatum(),this.chart.lastPick=void 0)),this.updateClass(e,this.constrained)},ml([$()],t.prototype,"enabled",void 0),ml([$()],t.prototype,"class",void 0),ml([$()],t.prototype,"delay",void 0),ml([$()],t.prototype,"tracking",void 0),t}(Ir),An=function(i){EA(t,i);function t(e){e===void 0&&(e=window.document);var r=i.call(this)||this;r.id=ql(r),r.background=new fo,r.legend=new K$,r.legendAutoPadding=new Td,r.captionAutoPadding=0,r._container=void 0,r._data=[],r._autoSize=!1,r.padding=new Td(20),r._axes=[],r._series=[],r._axesChanged=!1,r._seriesChanged=!1,r.layoutCallbackId=0,r._performLayout=function(){r.layoutCallbackId=0,r.background.width=r.width,r.background.height=r.height,r.performLayout(),r.layoutPending||r.fireEvent({type:"layoutDone"})},r.dataCallbackId=0,r.nodeData=new Map,r.updateCallbackId=0,r.legendBBox=new Lt(0,0,0,0),r._onMouseDown=r.onMouseDown.bind(r),r._onMouseMove=r.onMouseMove.bind(r),r._onMouseUp=r.onMouseUp.bind(r),r._onMouseOut=r.onMouseOut.bind(r),r._onClick=r.onClick.bind(r),r.pointerInsideLegend=!1,r.pointerOverLegendDatum=!1;var n=new It,o=r.background;o.fill="white",n.appendChild(o);var a=r._element=e.createElement("div");a.setAttribute("class","ag-chart-wrapper");var s=new TA(e);r.scene=s,r.autoSize=!0,s.root=n,s.container=a,r.padding.addEventListener("layoutChange",r.scheduleLayout,r);var l=r.legend;if(l.addEventListener("layoutChange",r.scheduleLayout,r),l.item.label.addPropertyListener("formatter",r.updateLegend,r),l.addPropertyListener("position",r.onLegendPositionChange,r),r.tooltip=new t9(r,e),r.tooltip.addPropertyListener("class",function(){return r.tooltip.toggle()}),t.tooltipDocuments.indexOf(e)<0){var u=e.createElement("style");u.innerHTML=e9,e.head.insertBefore(u,e.head.querySelector("style")),t.tooltipDocuments.push(e)}return r.setupDomListeners(s.canvas.element),r.addPropertyListener("title",r.onCaptionChange),r.addPropertyListener("subtitle",r.onCaptionChange),r.addEventListener("layoutChange",r.scheduleLayout),r}return Object.defineProperty(t.prototype,"container",{get:function(){return this._container},set:function(e){if(this._container!==e){var r=this.element.parentNode;r!=null&&r.removeChild(this.element),e&&e.appendChild(this.element),this._container=e}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"data",{get:function(){return this._data},set:function(e){this._data=e,this.series.forEach(function(r){return r.data=e})},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this.scene.width},set:function(e){this.autoSize=!1,this.width!==e&&(this.scene.resize(e,this.height),this.fireEvent({type:"layoutChange"}))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return this.scene.height},set:function(e){this.autoSize=!1,this.height!==e&&(this.scene.resize(this.width,e),this.fireEvent({type:"layoutChange"}))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"autoSize",{get:function(){return this._autoSize},set:function(e){if(this._autoSize!==e){this._autoSize=e;var r=this.element.style;if(e){var n=this;Wv.observe(this.element,function(o){(o.width!==n.width||o.height!==n.height)&&(n.scene.resize(o.width,o.height),n.fireEvent({type:"layoutChange"}))}),r.display="block",r.width="100%",r.height="100%"}else Wv.unobserve(this.element),r.display="inline-block",r.width="auto",r.height="auto"}},enumerable:!0,configurable:!0}),t.prototype.download=function(e){this.scene.download(e)},t.prototype.destroy=function(){this.tooltip.destroy(),Wv.unobserve(this.element),this.container=void 0,this.cleanupDomListeners(this.scene.canvas.element),this.scene.container=void 0},t.prototype.onLegendPositionChange=function(){this.legendAutoPadding.clear(),this.layoutPending=!0},t.prototype.onCaptionChange=function(e){var r=e.value,n=e.oldValue;n&&(n.removeEventListener("change",this.scheduleLayout,this),this.scene.root.removeChild(n.node)),r&&(r.addEventListener("change",this.scheduleLayout,this),this.scene.root.appendChild(r.node))},Object.defineProperty(t.prototype,"element",{get:function(){return this._element},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"axes",{get:function(){return this._axes},set:function(e){var r=this;this._axes.forEach(function(n){return r.detachAxis(n)}),this._axes=e.filter(function(n){return!n.linkedTo}).concat(e.filter(function(n){return n.linkedTo})),this._axes.forEach(function(n){return r.attachAxis(n)}),this.axesChanged=!0},enumerable:!0,configurable:!0}),t.prototype.attachAxis=function(e){this.scene.root.insertBefore(e.group,this.seriesRoot)},t.prototype.detachAxis=function(e){this.scene.root.removeChild(e.group)},Object.defineProperty(t.prototype,"series",{get:function(){return this._series},set:function(e){var r=this;this.removeAllSeries(),e.forEach(function(n){return r.addSeries(n)})},enumerable:!0,configurable:!0}),t.prototype.scheduleLayout=function(){this.layoutPending=!0},t.prototype.scheduleData=function(){this.dehighlightDatum(),this.dataPending=!0},t.prototype.addSeries=function(e,r){var n=this,o=n.series,a=n.seriesRoot,s=o.indexOf(e)<0;if(s){var l=r?o.indexOf(r):-1;return l>=0?(o.splice(l,0,e),a.insertBefore(e.group,r.group)):(o.push(e),a.append(e.group)),this.initSeries(e),this.seriesChanged=!0,this.axesChanged=!0,!0}return!1},t.prototype.initSeries=function(e){e.chart=this,e.data||(e.data=this.data),e.addEventListener("layoutChange",this.scheduleLayout,this),e.addEventListener("dataChange",this.scheduleData,this),e.addEventListener("legendChange",this.updateLegend,this),e.addEventListener("nodeClick",this.onSeriesNodeClick,this)},t.prototype.freeSeries=function(e){e.chart=void 0,e.removeEventListener("layoutChange",this.scheduleLayout,this),e.removeEventListener("dataChange",this.scheduleData,this),e.removeEventListener("legendChange",this.updateLegend,this),e.removeEventListener("nodeClick",this.onSeriesNodeClick,this)},t.prototype.addSeriesAfter=function(e,r){var n=this,o=n.series,a=n.seriesRoot,s=o.indexOf(e)<0;if(s){var l=r?this.series.indexOf(r):-1;l>=0?(l+1<o.length?a.insertBefore(e.group,o[l+1].group):a.append(e.group),this.initSeries(e),o.splice(l+1,0,e)):(o.length>0?a.insertBefore(e.group,o[0].group):a.append(e.group),this.initSeries(e),o.unshift(e)),this.seriesChanged=!0,this.axesChanged=!0}return!1},t.prototype.removeSeries=function(e){var r=this.series.indexOf(e);return r>=0?(this.series.splice(r,1),this.freeSeries(e),this.seriesRoot.removeChild(e.group),this.seriesChanged=!0,!0):!1},t.prototype.removeAllSeries=function(){var e=this;this.series.forEach(function(r){e.freeSeries(r),e.seriesRoot.removeChild(r.group)}),this._series=[],this.seriesChanged=!0},t.prototype.assignSeriesToAxes=function(){var e=this;this.axes.forEach(function(r){var n=r.direction+"Axis",o=[];e.series.forEach(function(a){a[n]===r&&o.push(a)}),r.boundSeries=o}),this.seriesChanged=!1},t.prototype.assignAxesToSeries=function(e){var r=this;e===void 0&&(e=!1);var n={};this.axes.forEach(function(o){var a=o.direction,s=n[a]||(n[a]=[]);s.push(o)}),this.series.forEach(function(o){o.directions.forEach(function(a){var s=a+"Axis";if(!o[s]||e){var l=n[a];if(l){var u=r.findMatchingAxis(l,o.getKeys(a));u&&(o[s]=u)}}})}),this.axesChanged=!1},t.prototype.findMatchingAxis=function(e,r){for(var n=0;n<e.length;n++){var o=e[n],a=o.keys;if(a.length){if(r){for(var s=0;s<r.length;s++)if(a.indexOf(r[s])>=0)return o}}else return o}},Object.defineProperty(t.prototype,"axesChanged",{get:function(){return this._axesChanged},set:function(e){this._axesChanged=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"seriesChanged",{get:function(){return this._seriesChanged},set:function(e){this._seriesChanged=e,e&&(this.dataPending=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"layoutPending",{get:function(){return!!this.layoutCallbackId},set:function(e){e?this.layoutCallbackId||this.dataPending||(this.layoutCallbackId=requestAnimationFrame(this._performLayout),this.series.forEach(function(r){return r.nodeDataPending=!0})):this.layoutCallbackId&&(cancelAnimationFrame(this.layoutCallbackId),this.layoutCallbackId=0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dataPending",{get:function(){return!!this.dataCallbackId},set:function(e){var r=this;this.dataCallbackId&&(clearTimeout(this.dataCallbackId),this.dataCallbackId=0),e&&(this.dataCallbackId=window.setTimeout(function(){r.dataPending=!1,r.processData()},0))},enumerable:!0,configurable:!0}),t.prototype.processData=function(){this.layoutPending=!1,this.axesChanged&&(this.assignAxesToSeries(!0),this.assignSeriesToAxes()),this.seriesChanged&&this.assignSeriesToAxes(),this.series.forEach(function(e){return e.processData()}),this.updateLegend(),this.layoutPending=!0},t.prototype.createNodeData=function(){var e=this;this.nodeData.clear(),this.series.forEach(function(r){var n=r.visible?r.createNodeData():[];e.nodeData.set(r,n)})},t.prototype.placeLabels=function(){var e=[],r=[];this.nodeData.forEach(function(a,s){s.visible&&s.label.enabled&&(e.push(s),r.push(s.getLabelData()))});var n=this.seriesRect,o=n?Q$(r,{x:0,y:0,width:n.width,height:n.height}):[];return new Map(o.map(function(a,s){return[e[s],a]}))},t.prototype.updateLegend=function(){var e=[];this.series.filter(function(n){return n.showInLegend}).forEach(function(n){return n.listSeriesItems(e)});var r=this.legend.item.label.formatter;r&&e.forEach(function(n){return n.label.text=r({id:n.id,itemId:n.itemId,value:n.label.text})}),this.legend.data=e},Object.defineProperty(t.prototype,"updatePending",{get:function(){return!!this.updateCallbackId},set:function(e){var r=this;this.updateCallbackId&&(clearTimeout(this.updateCallbackId),this.updateCallbackId=0),e&&!this.layoutPending&&(this.updateCallbackId=window.setTimeout(function(){r.update()},0))},enumerable:!0,configurable:!0}),t.prototype.update=function(){this.updatePending=!1,this.series.forEach(function(e){e.updatePending&&e.update()})},t.prototype.positionCaptions=function(){var e=this,r=e.title,n=e.subtitle,o=!1,a=!1,s=10,l=s;if(r&&r.enabled){r.node.x=this.width/2,r.node.y=l,o=!0;var u=r.node.computeBBox();if(u&&(l=u.y+u.height),n&&n.enabled){n.node.x=this.width/2,n.node.y=l+s,a=!0;var c=n.node.computeBBox();c&&(l=c.y+c.height)}}r&&(r.node.visible=o),n&&(n.node.visible=a),this.captionAutoPadding=Math.floor(l)},t.prototype.positionLegend=function(){if(!(!this.legend.enabled||!this.legend.data.length)){var e=this,r=e.legend,n=e.captionAutoPadding,o=e.legendAutoPadding,a=this.width,s=this.height-n,l=r.group,u=r.spacing,c=0,p=0,h;switch(r.position){case"bottom":r.performLayout(a-u*2,0),h=l.computeBBox(),l.visible=h.height<Math.floor(s*.5),l.visible?(c=(a-h.width)/2-h.x,p=n+s-h.height-h.y-u,o.bottom=h.height):o.bottom=0;break;case"top":r.performLayout(a-u*2,0),h=l.computeBBox(),l.visible=h.height<Math.floor(s*.5),l.visible?(c=(a-h.width)/2-h.x,p=n+u-h.y,o.top=h.height):o.top=0;break;case"left":r.performLayout(0,s-u*2),h=l.computeBBox(),l.visible=h.width<Math.floor(a*.5),l.visible?(c=u-h.x,p=n+(s-h.height)/2-h.y,o.left=h.width):o.left=0;break;default:r.performLayout(0,s-u*2),h=l.computeBBox(),l.visible=h.width<Math.floor(a*.5),l.visible?(c=a-h.width-h.x-u,p=n+(s-h.height)/2-h.y,o.right=h.width):o.right=0;break}l.visible&&(l.translationX=Math.floor(c+l.translationX),l.translationY=Math.floor(p+l.translationY),this.legendBBox=l.computeBBox())}},t.prototype.setupDomListeners=function(e){e.addEventListener("mousedown",this._onMouseDown),e.addEventListener("mousemove",this._onMouseMove),e.addEventListener("mouseup",this._onMouseUp),e.addEventListener("mouseout",this._onMouseOut),e.addEventListener("click",this._onClick)},t.prototype.cleanupDomListeners=function(e){e.removeEventListener("mousedown",this._onMouseDown),e.removeEventListener("mousemove",this._onMouseMove),e.removeEventListener("mouseup",this._onMouseUp),e.removeEventListener("mouseout",this._onMouseOut),e.removeEventListener("click",this._onClick)},t.prototype.getSeriesRect=function(){return this.seriesRect},t.prototype.pickSeriesNode=function(e,r){if(!!(this.seriesRect&&this.seriesRect.containsPoint(e,r)))for(var n=this.series,o=void 0,a=n.length-1;a>=0;a--){var s=n[a];if(!(!s.visible||!s.group.visible)&&(o=s.pickGroup.pickNode(e,r),o))return{series:s,node:o}}},t.prototype.pickClosestSeriesNodeDatum=function(e,r){if(!this.seriesRect||!this.seriesRect.containsPoint(e,r))return;var n=this.series;function o(c,p){return Math.sqrt(Math.pow(c.x-p.x,2)+Math.pow(c.y-p.y,2))}for(var a=1/0,s,l=function(c){var p=n[c];if(!p.visible||!p.group.visible)return"continue";var h=p.group.transformPoint(e,r);p.getNodeData().forEach(function(d){var f,g;if(!!d.point){var v=p.xAxis,m=p.yAxis,C=((f=v)===null||f===void 0?void 0:f.inRange(d.point.x))&&((g=m)===null||g===void 0?void 0:g.inRange(d.point.y));if(!!C){var w=o(h,d.point);w<a&&(a=w,s=d)}}})},u=n.length-1;u>=0;u--)l(u);return s},t.prototype.onMouseMove=function(e){this.handleLegendMouseMove(e),this.tooltip.enabled&&(this.tooltip.delay>0&&this.tooltip.toggle(!1),this.handleTooltip(e))},t.prototype.handleTooltip=function(e){var r=this,n=r.lastPick,o=r.tooltip.tracking,a=e.offsetX,s=e.offsetY,l=this.pickSeriesNode(a,s),u;if(l&&l.node instanceof ua){var c=l.node;if(u=c.datum,n&&n.datum===u&&(n.node=c,n.event=e),!c.datum.point||!o){!n||n.node!==c?this.onSeriesDatumPick(e,c.datum,c,e):l.series.tooltip.enabled&&this.tooltip.show(e);return}}var p=!1;if(o){var h=this.pickClosestSeriesNodeDatum(a,s);if(h&&h.point){var d=h.point,f=d.x,g=d.y,v=this.scene.canvas,m=h.series.group.inverseTransformPoint(f,g),C=v.element.getBoundingClientRect();this.onSeriesDatumPick({pageX:Math.round(C.left+window.pageXOffset+m.x),pageY:Math.round(C.top+window.pageYOffset+m.y)},h,u===h&&l?l.node:void 0,e)}else p=!0}n&&(p||!o)&&(this.dehighlightDatum(),this.tooltip.toggle(!1),this.lastPick=void 0)},t.prototype.onMouseDown=function(e){},t.prototype.onMouseUp=function(e){},t.prototype.onMouseOut=function(e){this.tooltip.toggle(!1)},t.prototype.onClick=function(e){this.checkSeriesNodeClick()||this.checkLegendClick(e)||this.fireEvent({type:"click",event:e})},t.prototype.checkSeriesNodeClick=function(){var e=this.lastPick;if(e&&e.event&&e.node){var r=e.event,n=e.datum;return n.series.fireNodeClickEvent(r,n),!0}return!1},t.prototype.onSeriesNodeClick=function(e){this.fireEvent(Pd(Pd({},e),{type:"seriesNodeClick"}))},t.prototype.checkLegendClick=function(e){var r=this.legend.getDatumForPoint(e.offsetX,e.offsetY);if(r){var n=r.id,o=r.itemId,a=r.enabled,s=Ay(this.series,function(l){return l.id===n});if(s)return s.toggleSeriesItem(o,!a),a&&this.tooltip.toggle(!1),this.legend.fireEvent({type:"click",event:e,itemId:o,enabled:!a}),!0}return!1},t.prototype.handleLegendMouseMove=function(e){if(!!this.legend.enabled){var r=e.offsetX,n=e.offsetY,o=this.legend.getDatumForPoint(r,n),a=this.legendBBox.containsPoint(r,n),s=a&&o!==void 0;if(!a&&this.pointerInsideLegend){this.pointerInsideLegend=!1,this.element.style.cursor="default",this.dehighlightDatum();return}s&&!this.pointerOverLegendDatum&&(this.element.style.cursor="pointer"),!s&&this.pointerOverLegendDatum&&(this.element.style.cursor="default"),this.pointerInsideLegend=a,this.pointerOverLegendDatum=s;var l=this.highlightedDatum;if(o){var u=o.id,c=o.itemId,p=o.enabled;if(p){var h=Ay(this.series,function(d){return d.id===u});h&&(this.highlightedDatum={series:h,itemId:c,datum:void 0})}}(this.highlightedDatum&&!l||this.highlightedDatum&&l&&(this.highlightedDatum.series!==l.series||this.highlightedDatum.itemId!==l.itemId))&&this.series.forEach(function(d){return d.updatePending=!0})}},t.prototype.onSeriesDatumPick=function(e,r,n,o){var a=this.lastPick;if(a){if(a.datum===r)return;this.dehighlightDatum()}this.lastPick={datum:r,node:n,event:o},this.highlightDatum(r);var s=r.series.tooltip.enabled&&r.series.getTooltipHtml(r);s&&this.tooltip.show(e,s)},t.prototype.highlightDatum=function(e){this.element.style.cursor=e.series.cursor,this.highlightedDatum=e,this.series.forEach(function(r){return r.updatePending=!0})},t.prototype.dehighlightDatum=function(){this.highlightedDatum&&(this.highlightedDatum=void 0,this.series.forEach(function(e){return e.updatePending=!0}))},t.defaultTooltipClass="ag-chart-tooltip",t.tooltipDocuments=[],ml([$("layoutChange")],t.prototype,"title",void 0),ml([$("layoutChange")],t.prototype,"subtitle",void 0),t}(Ir),r9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ip=function(i){r9(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.isContainerNode=!0,e.path=new dA,e._enabled=!0,e._dirtyPath=!0,e._x=0,e._y=0,e._width=10,e._height=10,e}return t.prototype.containsPoint=function(e,r){var n=this.transformPoint(e,r);return n.x>=this.x&&n.x<=this.x+this.width&&n.y>=this.y&&n.y<=this.y+this.height},Object.defineProperty(t.prototype,"enabled",{get:function(){return this._enabled},set:function(e){this._enabled!==e&&(this._enabled=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dirtyPath",{get:function(){return this._dirtyPath},set:function(e){this._dirtyPath!==e&&(this._dirtyPath=e,e&&(this.dirty=!0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"x",{get:function(){return this._x},set:function(e){this._x!==e&&(this._x=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this._y},set:function(e){this._y!==e&&(this._y=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this._width},set:function(e){this._width!==e&&(this._width=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return this._height},set:function(e){this._height!==e&&(this._height=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),t.prototype.updatePath=function(){var e=this.path;e.clear(),e.rect(this.x,this.y,this.width,this.height),this.dirtyPath=!1},t.prototype.computeBBox=function(){var e=this,r=e.x,n=e.y,o=e.width,a=e.height;return new Lt(r,n,o,a)},t.prototype.render=function(e){this.enabled&&(this.dirtyPath&&this.updatePath(),this.path.draw(e),e.clip());for(var r=this.children,n=r.length,o=0;o<n;o++){e.save();var a=r[o];a.visible&&a.render(e),e.restore()}},t.className="ClipRect",t}(Yi),n9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Gx=function(i){n9(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._fill="#f2f2f2",e._stroke="#999999",e._strokeWidth=1,e._lineCap="square",e._centerX=0,e._centerY=0,e._width=8,e._gripLineGap=2,e._gripLineLength=8,e._height=16,e}return Object.defineProperty(t.prototype,"centerX",{get:function(){return this._centerX},set:function(e){this._centerX!==e&&(this._centerX=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"centerY",{get:function(){return this._centerY},set:function(e){this._centerY!==e&&(this._centerY=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this._width},set:function(e){this._width!==e&&(this._width=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"gripLineGap",{get:function(){return this._gripLineGap},set:function(e){this._gripLineGap!==e&&(this._gripLineGap=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"gripLineLength",{get:function(){return this._gripLineLength},set:function(e){this._gripLineLength!==e&&(this._gripLineLength=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return this._height},set:function(e){this._height!==e&&(this._height=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),t.prototype.computeBBox=function(){var e=this,r=e.centerX,n=e.centerY,o=e.width,a=e.height,s=r-o/2,l=n-a/2;return new Lt(s,l,o,a)},t.prototype.isPointInPath=function(e,r){var n=this.transformPoint(e,r),o=this.computeBBox();return o.containsPoint(n.x,n.y)},t.prototype.updatePath=function(){var e=this,r=e.path,n=e.centerX,o=e.centerY,a=e.width,s=e.height,l=this,u=l.alignment,c=l.align;r.clear();var p=n-a/2,h=o-s/2,d=c(u,p),f=c(u,h),g=d+c(u,p,a),v=f+c(u,h,s);r.moveTo(d,f),r.lineTo(g,f),r.lineTo(g,v),r.lineTo(d,v),r.lineTo(d,f);var m=this.gripLineGap/2,C=this.gripLineLength/2;r.moveTo(c(u,n-m),c(u,o-C)),r.lineTo(c(u,n-m),c(u,o+C)),r.moveTo(c(u,n+m),c(u,o-C)),r.lineTo(c(u,n+m),c(u,o+C))},t.className="RangeHandle",t}($r),i9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),o9=function(i){i9(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._stroke="#999999",e._strokeWidth=1,e._fill="#999999",e._fillOpacity=.2,e._lineCap="square",e._x=0,e._y=0,e._width=200,e._height=30,e.minRange=.05,e._min=0,e._max=1,e}return Object.defineProperty(t.prototype,"x",{get:function(){return this._x},set:function(e){this._x!==e&&(this._x=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this._y},set:function(e){this._y!==e&&(this._y=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this._width},set:function(e){this._width!==e&&(this._width=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return this._height},set:function(e){this._height!==e&&(this._height=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"min",{get:function(){return this._min},set:function(e){e=Math.min(Math.max(e,0),this.max-this.minRange),!isNaN(e)&&this._min!==e&&(this._min=e,this.dirtyPath=!0,this.onRangeChange&&this.onRangeChange(e,this.max))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"max",{get:function(){return this._max},set:function(e){e=Math.max(Math.min(e,1),this.min+this.minRange),!isNaN(e)&&this._max!==e&&(this._max=e,this.dirtyPath=!0,this.onRangeChange&&this.onRangeChange(this.min,e))},enumerable:!0,configurable:!0}),t.prototype.computeBBox=function(){var e=this,r=e.x,n=e.y,o=e.width,a=e.height;return new Lt(r,n,o,a)},t.prototype.computeVisibleRangeBBox=function(){var e=this,r=e.x,n=e.y,o=e.width,a=e.height,s=e.min,l=e.max,u=r+o*s,c=r+o*l;return new Lt(u,n,c-u,a)},t.prototype.updatePath=function(){var e=this,r=e.path,n=e.x,o=e.y,a=e.width,s=e.height,l=e.min,u=e.max,c=this,p=c.alignment,h=c.align;r.clear();var d=h(p,n),f=h(p,o),g=d+h(p,n,a),v=f+h(p,o,s);r.moveTo(d,f),r.lineTo(g,f),r.lineTo(g,v),r.lineTo(d,v),r.lineTo(d,f);var m=h(p,n+a*l),C=h(p,n+a*u);r.moveTo(m,f),r.lineTo(m,v),r.lineTo(C,v),r.lineTo(C,f),r.lineTo(m,f)},t.className="RangeMask",t}($r),a9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),s9=function(i){a9(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.isContainerNode=!0,e.minHandle=new Gx,e.maxHandle=new Gx,e.mask=function(){var r=t.defaults,n=r.x,o=r.y,a=r.width,s=r.height,l=r.min,u=r.max,c=new o9;c.x=n,c.y=o,c.width=a,c.height=s,c.min=l,c.max=u;var p=e,h=p.minHandle,d=p.maxHandle;return h.centerX=n,d.centerX=n+a,h.centerY=d.centerY=o+s/2,e.append([c,h,d]),c.onRangeChange=function(f,g){e.updateHandles(),e.onRangeChange&&e.onRangeChange(f,g)},c}(),e._x=t.defaults.x,e._y=t.defaults.y,e._width=t.defaults.width,e._height=t.defaults.height,e._min=t.defaults.min,e._max=t.defaults.max,e}return Object.defineProperty(t.prototype,"x",{get:function(){return this.mask.x},set:function(e){this.mask.x=e,this.updateHandles()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this.mask.y},set:function(e){this.mask.y=e,this.updateHandles()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this.mask.width},set:function(e){this.mask.width=e,this.updateHandles()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return this.mask.height},set:function(e){this.mask.height=e,this.updateHandles()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"min",{get:function(){return this.mask.min},set:function(e){this.mask.min=e},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"max",{get:function(){return this.mask.max},set:function(e){this.mask.max=e},enumerable:!0,configurable:!0}),t.prototype.updateHandles=function(){var e=this,r=e.minHandle,n=e.maxHandle,o=e.x,a=e.y,s=e.width,l=e.height,u=e.mask;r.centerX=o+s*u.min,n.centerX=o+s*u.max,r.centerY=n.centerY=a+l/2},t.prototype.computeBBox=function(){return this.mask.computeBBox()},t.prototype.computeVisibleRangeBBox=function(){return this.mask.computeVisibleRangeBBox()},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this,n=r.mask,o=r.minHandle,a=r.maxHandle;[n,o,a].forEach(function(s){e.save(),s.visible&&s.render(e),e.restore()})},t.className="Range",t.defaults={x:0,y:0,width:200,height:30,min:0,max:1},t}(It),l9=function(){function i(t){this.rm=t}return Object.defineProperty(i.prototype,"fill",{get:function(){return this.rm.fill},set:function(t){this.rm.fill=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"stroke",{get:function(){return this.rm.stroke},set:function(t){this.rm.stroke=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"strokeWidth",{get:function(){return this.rm.strokeWidth},set:function(t){this.rm.strokeWidth=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"fillOpacity",{get:function(){return this.rm.fillOpacity},set:function(t){this.rm.fillOpacity=t},enumerable:!0,configurable:!0}),i}(),kx=function(){function i(t){this.rh=t}return Object.defineProperty(i.prototype,"fill",{get:function(){return this.rh.fill},set:function(t){this.rh.fill=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"stroke",{get:function(){return this.rh.stroke},set:function(t){this.rh.stroke=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"strokeWidth",{get:function(){return this.rh.strokeWidth},set:function(t){this.rh.strokeWidth=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"width",{get:function(){return this.rh.width},set:function(t){this.rh.width=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"height",{get:function(){return this.rh.height},set:function(t){this.rh.height=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"gripLineGap",{get:function(){return this.rh.gripLineGap},set:function(t){this.rh.gripLineGap=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"gripLineLength",{get:function(){return this.rh.gripLineLength},set:function(t){this.rh.gripLineLength=t},enumerable:!0,configurable:!0}),i}(),u9=function(){function i(t){var e=this;this.rs=new s9,this.mask=new l9(this.rs.mask),this.minHandle=new kx(this.rs.minHandle),this.maxHandle=new kx(this.rs.maxHandle),this.minHandleDragging=!1,this.maxHandleDragging=!1,this.panHandleOffset=NaN,this.changedCursor=!1,this._margin=10,this.chart=t,t.scene.root.append(this.rs),this.rs.onRangeChange=function(r,n){return e.updateAxes(r,n)}}return Object.defineProperty(i.prototype,"enabled",{get:function(){return this.rs.visible},set:function(t){this.rs.visible=t,this.chart.layoutPending=!0},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"x",{get:function(){return this.rs.x},set:function(t){this.rs.x=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"y",{get:function(){return this.rs.y},set:function(t){this.rs.y=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"width",{get:function(){return this.rs.width},set:function(t){this.rs.width=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"height",{get:function(){return this.rs.height},set:function(t){this.rs.height=t,this.chart.layoutPending=!0},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"margin",{get:function(){return this._margin},set:function(t){this._margin=t,this.chart.layoutPending=!0},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"min",{get:function(){return this.rs.min},set:function(t){this.rs.min=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"max",{get:function(){return this.rs.max},set:function(t){this.rs.max=t},enumerable:!0,configurable:!0}),i.prototype.updateAxes=function(t,e){var r=this,n=this.chart,o=!1;n.axes.forEach(function(a){if(a.direction===tt.X){!o&&(t>0||e<1)&&(o=!0),a.visibleRange=[t,e];var s=a.labelAutoRotated;a.update(),a.labelAutoRotated!==s&&(r.chart.layoutPending=!0)}}),n.seriesRoot.enabled=o,n.series.forEach(function(a){return a.nodeDataPending=!0})},i.prototype.onDragStart=function(t){if(!!this.enabled){var e=t.offsetX,r=t.offsetY,n=this.rs,o=n.minHandle,a=n.maxHandle,s=n.x,l=n.width,u=n.min,c=n.computeVisibleRangeBBox();this.minHandleDragging||this.maxHandleDragging||(o.containsPoint(e,r)?this.minHandleDragging=!0:a.containsPoint(e,r)?this.maxHandleDragging=!0:c.containsPoint(e,r)&&(this.panHandleOffset=(e-s)/l-u))}},i.prototype.onDrag=function(t){if(!this.enabled)return;var e=this,r=e.rs,n=e.panHandleOffset,o=r.x,a=r.y,s=r.width,l=r.height,u=r.minHandle,c=r.maxHandle,p=this.chart.element.style,h=t.offsetX,d=t.offsetY,f=o+s*r.min,g=o+s*r.max,v=new Lt(f,a,g-f,l);function m(){return Math.min(Math.max((h-o)/s,0),1)}if(u.containsPoint(h,d)?(this.changedCursor=!0,p.cursor="ew-resize"):c.containsPoint(h,d)?(this.changedCursor=!0,p.cursor="ew-resize"):v.containsPoint(h,d)?(this.changedCursor=!0,p.cursor="grab"):this.changedCursor&&(this.changedCursor=!1,p.cursor="default"),this.minHandleDragging)r.min=m();else if(this.maxHandleDragging)r.max=m();else if(!isNaN(n)){var C=r.max-r.min,w=Math.min(m()-n,1-C);w<=r.min?(r.min=w,r.max=r.min+C):(r.max=w+C,r.min=r.max-C)}},i.prototype.onDragStop=function(){this.stopHandleDragging()},i.prototype.stopHandleDragging=function(){this.minHandleDragging=this.maxHandleDragging=!1,this.panHandleOffset=NaN},i}(),c9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),p9=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},h9=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(p9(arguments[t]));return i},RA=function(i){c9(t,i);function t(e){e===void 0&&(e=window.document);var r=i.call(this,e)||this;r._seriesRoot=new ip,r.navigator=new u9(r),r.scene.root.visible=!1;var n=r.scene.root;return n.append(r.seriesRoot),n.append(r.legend.group),r.navigator.enabled=!1,r}return Object.defineProperty(t.prototype,"seriesRoot",{get:function(){return this._seriesRoot},enumerable:!0,configurable:!0}),t.prototype.performLayout=function(){if(!this.dataPending){this.scene.root.visible=!0;var e=this,r=e.width,n=e.height,o=e.axes,a=e.legend,s=e.navigator,l=new Lt(0,0,r,n);if(this.positionCaptions(),this.positionLegend(),a.enabled&&a.data.length){var u=this.legendAutoPadding,c=this.legend.spacing;switch(l.x+=u.left,l.y+=u.top,l.width-=u.left+u.right,l.height-=u.top+u.bottom,this.legend.position){case"right":l.width-=c;break;case"bottom":l.height-=c;break;case"left":l.x+=c,l.width-=c;break;case"top":l.y+=c,l.height-=c;break}}var p=this,h=p.captionAutoPadding,d=p.padding;l.x+=d.left,l.width-=d.left+d.right,l.y+=d.top+h,l.height-=d.top+h+d.bottom,s.enabled&&(l.height-=s.height+s.margin),o.forEach(function(m){var C=0;switch(m.position){case Ae.Top:case Ae.Bottom:C=l.width;break;case Ae.Left:case Ae.Right:C=l.height;break}m.calculateTickCount(C)}),this.updateAxes();var f=0,g={top:!1,right:!1,bottom:!1,left:!1,angle:!1,radius:!1};o.forEach(function(m){m.group.visible=!0;var C=Math.floor(m.thickness||m.computeBBox().width),w=m.title&&m.title.padding.top||15;switch(g[m.position]&&(C+=w),m.position){case Ae.Top:g[Ae.Top]=!0,l.y+=C,l.height-=C,m.translation.y=Math.floor(l.y+1),m.label.mirrored=!0;break;case Ae.Right:g[Ae.Right]=!0,l.width-=C,m.translation.x=Math.max(Math.floor(l.x),Math.floor(l.x+l.width)),m.label.mirrored=!0;break;case Ae.Bottom:g[Ae.Bottom]=!0,l.height-=C,f+=C,m.translation.y=Math.max(Math.floor(l.y),Math.floor(l.y+l.height+1));break;case Ae.Left:g[Ae.Left]=!0,l.x+=C,l.width-=C,m.translation.x=Math.floor(l.x);break}}),l.width=Math.max(0,l.width),l.height=Math.max(0,l.height),o.forEach(function(m){switch(m.position){case Ae.Top:case Ae.Bottom:m.translation.x=Math.floor(l.x),m.range=[0,l.width],m.gridLength=l.height;break;case Ae.Left:case Ae.Right:m.translation.y=Math.floor(l.y),m instanceof ps||m instanceof Il?m.range=[0,l.height]:m.range=[l.height,0],m.gridLength=l.width;break}}),this.createNodeData(),this.seriesRect=l,this.series.forEach(function(m){m.group.translationX=Math.floor(l.x),m.group.translationY=Math.floor(l.y),m.update()});var v=this.seriesRoot;v.x=l.x,v.y=l.y,v.width=l.width,v.height=l.height,s.enabled&&(s.x=l.x,s.y=l.y+l.height+f+s.margin,s.width=l.width),this.axes.forEach(function(m){return m.update()})}},t.prototype.initSeries=function(e){i.prototype.initSeries.call(this,e)},t.prototype.freeSeries=function(e){i.prototype.freeSeries.call(this,e)},t.prototype.setupDomListeners=function(e){i.prototype.setupDomListeners.call(this,e),this._onTouchStart=this.onTouchStart.bind(this),this._onTouchMove=this.onTouchMove.bind(this),this._onTouchEnd=this.onTouchEnd.bind(this),this._onTouchCancel=this.onTouchCancel.bind(this),e.addEventListener("touchstart",this._onTouchStart,{passive:!0}),e.addEventListener("touchmove",this._onTouchMove,{passive:!0}),e.addEventListener("touchend",this._onTouchEnd,{passive:!0}),e.addEventListener("touchcancel",this._onTouchCancel,{passive:!0})},t.prototype.cleanupDomListeners=function(e){i.prototype.cleanupDomListeners.call(this,e),e.removeEventListener("touchstart",this._onTouchStart),e.removeEventListener("touchmove",this._onTouchMove),e.removeEventListener("touchend",this._onTouchEnd),e.removeEventListener("touchcancel",this._onTouchCancel)},t.prototype.getTouchOffset=function(e){var r=this.scene.canvas.element.getBoundingClientRect(),n=e.touches[0];return n?{offsetX:n.clientX-r.left,offsetY:n.clientY-r.top}:void 0},t.prototype.onTouchStart=function(e){var r=this.getTouchOffset(e);r&&this.navigator.onDragStart(r)},t.prototype.onTouchMove=function(e){var r=this.getTouchOffset(e);r&&this.navigator.onDrag(r)},t.prototype.onTouchEnd=function(e){this.navigator.onDragStop()},t.prototype.onTouchCancel=function(e){this.navigator.onDragStop()},t.prototype.onMouseDown=function(e){i.prototype.onMouseDown.call(this,e),this.navigator.onDragStart(e)},t.prototype.onMouseMove=function(e){i.prototype.onMouseMove.call(this,e),this.navigator.onDrag(e)},t.prototype.onMouseUp=function(e){i.prototype.onMouseUp.call(this,e),this.navigator.onDragStop()},t.prototype.onMouseOut=function(e){i.prototype.onMouseOut.call(this,e),this.navigator.onDragStop()},t.prototype.assignAxesToSeries=function(e){e===void 0&&(e=!1),i.prototype.assignAxesToSeries.call(this,e),this.series.forEach(function(r){r.xAxis||console.warn("Could not find a matching xAxis for the "+r.id+" series."),r.yAxis||console.warn("Could not find a matching yAxis for the "+r.id+" series.")})},t.prototype.updateAxes=function(){var e=this,r=this.navigator,n=!1,o;this.axes.forEach(function(a){var s,l=a.direction,u=a.boundSeries;if(u.length===0&&e._series.length>0&&console.warn("AG Charts - chart series not initialised; check series and axes configuration."),a.linkedTo)a.domain=a.linkedTo.domain;else{var c=[];u.filter(function(d){return d.visible}).forEach(function(d){c.push(d.getDomain(l))});var p=(s=new Array).concat.apply(s,h9(c)),h=a.direction==="y";a instanceof Zl&&h?(a.setDomain(p,o),o=o||a.scale.ticks(a.calculatedTickCount).length):a.domain=p}a.direction===tt.X&&(a.visibleRange=[r.min,r.max]),!n&&(a.visibleRange[0]>0||a.visibleRange[1]<1)&&(n=!0),a.update()}),this.seriesRoot.enabled=n},t.className="CartesianChart",t.type="cartesian",t}(An),d9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),f9=function(i){d9(t,i);function t(e){e===void 0&&(e=window.document);var r=i.call(this,e)||this;r._data={},r._seriesRoot=new ip,r.scene.root.visible=!1;var n=r.scene.root;return n.append(r.seriesRoot),n.append(r.legend.group),r}return Object.defineProperty(t.prototype,"seriesRoot",{get:function(){return this._seriesRoot},enumerable:!0,configurable:!0}),t.prototype.performLayout=function(){if(!this.dataPending){this.scene.root.visible=!0;var e=this,r=e.width,n=e.height,o=e.legend,a=new Lt(0,0,r,n);if(this.positionCaptions(),this.positionLegend(),o.enabled&&o.data.length){var s=this.legendAutoPadding,l=this.legend.spacing;switch(a.x+=s.left,a.y+=s.top,a.width-=s.left+s.right,a.height-=s.top+s.bottom,this.legend.position){case"right":a.width-=l;break;case"bottom":a.height-=l;break;case"left":a.x+=l,a.width-=l;break;case"top":a.y+=l,a.height-=l;break}}var u=this,c=u.captionAutoPadding,p=u.padding;a.x+=p.left,a.width-=p.left+p.right,a.y+=p.top+c,a.height-=p.top+c+p.bottom,this.seriesRect=a,this.series.forEach(function(d){d.group.translationX=Math.floor(a.x),d.group.translationY=Math.floor(a.y),d.update()});var h=this.seriesRoot;h.x=a.x,h.y=a.y,h.width=a.width,h.height=a.height}},t.className="HierarchyChart",t.type="hierarchy",t}(An),g9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),v9=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},m9=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(v9(arguments[t]));return i},y9=function(i){g9(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.updateAxes=function(){this.axes.forEach(function(e){var r,n=e.direction,o=e.boundSeries,a=[],s=void 0;o.filter(function(u){return u.visible}).forEach(function(u){if(n===tt.X)if(s===void 0){var c=u.getDomain(n);a.push(c),s=typeof c[0]=="number"}else s&&a.push(u.getDomain(n));else a.push(u.getDomain(n))});var l=(r=new Array).concat.apply(r,m9(a));e.domain=ti(l,Ar)||l,e.update()})},t.className="GroupedCategoryChart",t.type="groupedCategory",t}(RA),C9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Zs=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Gn=function(i){C9(t,i);function t(){var e=i.call(this)||this;return e.enabled=!0,e.fontSize=12,e.fontFamily="Verdana, sans-serif",e.color="rgba(70, 70, 70, 1)",e}return t.prototype.getFont=function(){return sA(this.fontSize,this.fontFamily,this.fontStyle,this.fontWeight)},Zs([$("change","dataChange")],t.prototype,"enabled",void 0),Zs([$("change")],t.prototype,"fontSize",void 0),Zs([$("change")],t.prototype,"fontFamily",void 0),Zs([$("change")],t.prototype,"fontStyle",void 0),Zs([$("change")],t.prototype,"fontWeight",void 0),Zs([$("change")],t.prototype,"color",void 0),t}(Ir),PA=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Lh=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},AA=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},w9=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(AA(arguments[t]));return i},b9=function(){function i(){this.fill="yellow",this.stroke=void 0,this.strokeWidth=void 0}return i}(),_9=function(){function i(){this.strokeWidth=void 0,this.dimOpacity=void 0}return i}(),DA=function(){function i(){this.fill=void 0,this.stroke=void 0,this.strokeWidth=void 0,this.item=new b9,this.series=new _9}return i}(),Os=function(i){PA(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.enabled=!0,e}return Lh([$("change")],t.prototype,"enabled",void 0),t}(Ir),Ql=function(i){PA(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.id=ql(e),e.group=new It,e.pickGroup=e.group.appendChild(new It),e.directions=[tt.X,tt.Y],e.directionKeys={},e.label=new Gn,e.data=void 0,e.visible=!0,e.showInLegend=!0,e.cursor="default",e._nodeDataPending=!0,e._updatePending=!1,e.highlightStyle=new DA,e.scheduleLayout=function(){e.fireEvent({type:"layoutChange"})},e.scheduleData=function(){e.fireEvent({type:"dataChange"})},e}return Object.defineProperty(t.prototype,"type",{get:function(){return this.constructor.type||""},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"grouped",{set:function(e){if(e===!0)throw new Error("AG Charts - grouped: true is unsupported for series of type: "+this.type)},enumerable:!0,configurable:!0}),t.prototype.setColors=function(e,r){},t.prototype.getKeys=function(e){var r=this,n=this.directionKeys,o=n&&n[e],a=[];return o&&o.forEach(function(s){var l=r[s];l&&(Array.isArray(l)?a.push.apply(a,w9(l)):a.push(l))}),a},t.prototype.createNodeData=function(){return[]},t.prototype.getNodeData=function(){return[]},t.prototype.getLabelData=function(){return[]},Object.defineProperty(t.prototype,"nodeDataPending",{get:function(){return this._nodeDataPending},set:function(e){this._nodeDataPending!==e&&(this._nodeDataPending=e,this.updatePending=!0,e&&this.chart&&(this.chart.updatePending=e))},enumerable:!0,configurable:!0}),t.prototype.scheduleNodeDate=function(){this.nodeDataPending=!0},Object.defineProperty(t.prototype,"updatePending",{get:function(){return this._updatePending},set:function(e){this._updatePending!==e&&(this._updatePending=e,e&&this.chart&&(this.chart.updatePending=e))},enumerable:!0,configurable:!0}),t.prototype.scheduleUpdate=function(){this.updatePending=!0},t.prototype.getOpacity=function(e){var r=this,n=r.chart,o=r.highlightStyle.series.dimOpacity,a=o===void 0?1:o;return!n||!n.highlightedDatum||n.highlightedDatum.series===this&&(!e||n.highlightedDatum.itemId===e.itemId)?1:a},t.prototype.getStrokeWidth=function(e,r){var n=this,o=n.chart,a=n.highlightStyle.series.strokeWidth;return o&&o.highlightedDatum&&o.highlightedDatum.series===this&&(!r||o.highlightedDatum.itemId===r.itemId)&&a!==void 0?a:e},t.prototype.fireNodeClickEvent=function(e,r){},t.prototype.toggleSeriesItem=function(e,r){this.visible=r},t.prototype.onHighlightChange=function(){},t.prototype.fixNumericExtent=function(e,r,n){if(!e)return[0,1];var o=AA(e,2),a=o[0],s=o[1];if(a=+a,s=+s,a===0&&s===0)return[0,1];if(a===s)if(n instanceof Lc)a-=1,s+=1;else{var l=Math.abs(a*.01);a-=l,s+=l}return ca(a)&&ca(s)?[a,s]:[0,1]},t.highlightedZIndex=1e12,Lh([$("dataChange")],t.prototype,"data",void 0),Lh([$("dataChange")],t.prototype,"visible",void 0),Lh([$("layoutChange")],t.prototype,"showInLegend",void 0),t}(Ir),S9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),fi=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},MA=function(i){S9(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.enabled=!0,e.shape=OA,e.size=6,e.maxSize=30,e.strokeWidth=1,e}return fi([$("change")],t.prototype,"enabled",void 0),fi([$("change")],t.prototype,"shape",void 0),fi([$("change")],t.prototype,"size",void 0),fi([$("change")],t.prototype,"maxSize",void 0),fi([$("change")],t.prototype,"domain",void 0),fi([$("change")],t.prototype,"fill",void 0),fi([$("change")],t.prototype,"stroke",void 0),fi([$("change")],t.prototype,"strokeWidth",void 0),fi([$("change")],t.prototype,"fillOpacity",void 0),fi([$("change")],t.prototype,"strokeOpacity",void 0),t}(Ir),FA=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),IA=function(i){FA(t,i);function t(){var e,r=i!==null&&i.apply(this,arguments)||this;return r.directionKeys=(e={},e[tt.X]=["angleKey"],e[tt.Y]=["radiusKey"],e),r.centerX=0,r.centerY=0,r.radius=0,r}return t}(Ql);(function(i){FA(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t})(MA);var x9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),T9=function(i){x9(t,i);function t(e){e===void 0&&(e=window.document);var r=i.call(this,e)||this;return r.padding=new Td(40),r.scene.root.append(r.legend.group),r.padding.addEventListener("layoutChange",r.scheduleLayout,r),r}return Object.defineProperty(t.prototype,"seriesRoot",{get:function(){return this.scene.root},enumerable:!0,configurable:!0}),t.prototype.performLayout=function(){var e=new Lt(0,0,this.width,this.height);this.positionCaptions(),this.positionLegend();var r=this.captionAutoPadding;if(e.y+=r,e.height-=r,this.legend.enabled&&this.legend.data.length){var n=this.legendAutoPadding;e.x+=n.left,e.y+=n.top,e.width-=n.left+n.right,e.height-=n.top+n.bottom;var o=this.legend.spacing;switch(this.legend.position){case"right":e.width-=o;break;case"bottom":e.height-=o;break;case"left":e.x+=o,e.width-=o;break;case"top":e.y+=o,e.height-=o;break}}var a=this.padding;e.x+=a.left,e.y+=a.top,e.width-=a.left+a.right,e.height-=a.top+a.bottom,this.seriesRect=e;var s=e.x+e.width/2,l=e.y+e.height/2,u=Math.max(0,Math.min(e.width,e.height)/2);this.series.forEach(function(c){c instanceof IA&&(c.centerX=s,c.centerY=l,c.radius=u,c.update())})},t.className="PolarChart",t.type="polar",t}(An),LA=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),op=function(i){LA(t,i);function t(){var e,r=i!==null&&i.apply(this,arguments)||this;return r.directionKeys=(e={},e[tt.X]=["xKey"],e[tt.Y]=["yKey"],e),r}return t.prototype.checkDomainXY=function(e,r,n,o){var a=(n&&Ar(e)||!n&&Pv(e))&&(o&&Ar(r)||!o&&Pv(r));return a?[e,r]:void 0},t.prototype.checkDatum=function(e,r){if(r&&Ar(e))return e;if(!r)return Pv(e)?e:String(e)},t.prototype.checkRangeXY=function(e,r,n,o){return!isNaN(e)&&!isNaN(r)&&n.inRange(e)&&o.inRange(r)},t}(Ql),tb=function(i){LA(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.formatter=void 0,e}return t}(MA);function Ad(i,t){if(i===t)return!0;if(i&&t&&typeof i=="object"&&typeof t=="object"){if(i.constructor!==t.constructor)return!1;var e,r=void 0;if(Array.isArray(i)){if(e=i.length,e!=t.length)return!1;for(r=e;r--!==0;)if(!Ad(i[r],t[r]))return!1;return!0}if(i.constructor===RegExp)return i.source===t.source&&i.flags===t.flags;if(i.valueOf!==Object.prototype.valueOf)return i.valueOf()===t.valueOf();if(i.toString!==Object.prototype.toString)return i.toString()===t.toString();var n=Object.keys(i);if(e=n.length,e!==Object.keys(t).length)return!1;for(r=e;r--!==0;)if(!Object.prototype.hasOwnProperty.call(t,n[r]))return!1;for(r=e;r--!==0;){var o=n[r];if(!Ad(i[o],t[o]))return!1}return!0}return i!==i&&t!==t}var O9=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},E9=/(#\{(.*?)\})/g;function NA(i,t,e){return i.replace(E9,function(){var r=arguments[2],n=O9(r.split(":"),2),o=n[0],a=n[1],s=t[o];if(typeof s=="number"){var l=a&&e&&e[a];if(l){var u=l,c=u.locales,p=u.options;return s.toLocaleString(c,p)}return String(s)}if(s instanceof Date){var l=a&&e&&e[a];if(typeof l=="string"){var h=eb.format(l);return h(s)}return s.toDateString()}return typeof s=="string"||s&&s.toString?String(s):""})}var sh=null;function ir(i){return sh=sh||document.createElement("div"),i?(sh.innerText=i,sh.innerHTML):""}var rb=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),on=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},lh=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},R9=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},P9=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(R9(arguments[t]));return i},A9=function(i){rb(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return on([$("change")],t.prototype,"formatter",void 0),t}(Gn),D9=function(i){rb(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return on([$("change")],t.prototype,"renderer",void 0),on([$("change")],t.prototype,"format",void 0),t}(Os),nb=function(i){rb(t,i);function t(){var e=i.call(this)||this;e.tooltip=new D9,e.areaGroup=e.group.insertBefore(new It,e.pickGroup),e.strokeGroup=e.group.insertBefore(new It,e.pickGroup),e.markerGroup=e.pickGroup.appendChild(new It),e.labelGroup=e.group.appendChild(new It),e.fillSelection=Ht.select(e.areaGroup).selectAll(),e.strokeSelection=Ht.select(e.strokeGroup).selectAll(),e.markerSelection=Ht.select(e.markerGroup).selectAll(),e.labelSelection=Ht.select(e.labelGroup).selectAll(),e.seriesItemEnabled=new Map,e.xData=[],e.yData=[],e.fillSelectionData=[],e.strokeSelectionData=[],e.markerSelectionData=[],e.labelSelectionData=[],e.yDomain=[],e.xDomain=[],e.directionKeys={x:["xKey"],y:["yKeys"]},e.marker=new tb,e.label=new A9,e.fills=["#c16068","#a2bf8a","#ebcc87","#80a0c3","#b58dae","#85c0d1"],e.strokes=["#874349","#718661","#a48f5f","#5a7088","#7f637a","#5d8692"],e.fillOpacity=1,e.strokeOpacity=1,e.lineDash=[0],e.lineDashOffset=0,e._xKey="",e.xName="",e._yKeys=[],e.yNames=[],e.strokeWidth=2,e.addEventListener("update",e.scheduleUpdate);var r=e,n=r.marker,o=r.label;return n.enabled=!1,n.addPropertyListener("shape",e.onMarkerShapeChange,e),n.addEventListener("change",e.scheduleUpdate,e),o.enabled=!1,o.addEventListener("change",e.scheduleUpdate,e),e}return t.prototype.onMarkerShapeChange=function(){this.markerSelection=this.markerSelection.setData([]),this.markerSelection.exit.remove(),this.fireEvent({type:"legendChange"})},Object.defineProperty(t.prototype,"xKey",{get:function(){return this._xKey},set:function(e){this._xKey!==e&&(this._xKey=e,this.xData=[],this.scheduleData())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"yKeys",{get:function(){return this._yKeys},set:function(e){if(!Ad(this._yKeys,e)){this._yKeys=e,this.yData=[];var r=this.seriesItemEnabled;r.clear(),e.forEach(function(n){return r.set(n,!0)}),this.scheduleData()}},enumerable:!0,configurable:!0}),t.prototype.setColors=function(e,r){this.fills=e,this.strokes=r},Object.defineProperty(t.prototype,"normalizedTo",{get:function(){return this._normalizedTo},set:function(e){var r=e?Math.abs(e):void 0;this._normalizedTo!==r&&(this._normalizedTo=r,this.scheduleData())},enumerable:!0,configurable:!0}),t.prototype.processData=function(){var e,r,n,o,a,s,l=this,u=this,c=u.xKey,p=u.yKeys,h=u.seriesItemEnabled,d=u.xAxis,f=u.yAxis,g=u.normalizedTo,v=c&&p.length&&this.data?this.data:[];if(!d||!f)return!1;var m=d.scale instanceof vr,C=f.scale instanceof vr,w=g&&isFinite(g),b=[],S=[],T=[],E=function(pe){if(!(c in pe))return Rd(function(){return console.warn("The key '"+c+"' was not found in the data: ",pe)},c+" not found in data"),"continue";var be=P.checkDatum(pe[c],m);if(m&&be===void 0)return"continue";T.push(be),S.push({xDatum:be,seriesDatum:pe}),p.forEach(function(_e,Te){if(!(_e in pe)){Rd(function(){return console.warn("The key '"+_e+"' was not found in the data: ",pe)},_e+" not found in data");return}var Pe=pe[_e],Ee=b[Te]||(b[Te]=[]);if(!h.get(_e))Ee.push(0);else{var Le=l.checkDatum(Pe,C);Ee.push(Le)}})},P=this;try{for(var R=lh(v),M=R.next();!M.done;M=R.next()){var N=M.value;E(N)}}catch(pe){e={error:pe}}finally{try{M&&!M.done&&(r=R.return)&&r.call(R)}finally{if(e)throw e.error}}this.yData=b,this.xData=S,this.xDomain=m?this.fixNumericExtent(ti(T,Ar),"x",d):T;for(var W=0,G=0,k=0;k<S.length;k++){var V={sum:0,absSum:0};try{for(var U=(n=void 0,lh(b)),J=U.next();!J.done;J=U.next()){var Z=J.value;if(Z[k]!==void 0){var le=+Z[k];V.absSum+=Math.abs(le),V.sum+=le,V.sum>G?G=V.sum:V.sum<W&&(W=V.sum)}}}catch(pe){n={error:pe}}finally{try{J&&!J.done&&(o=U.return)&&o.call(U)}finally{if(n)throw n.error}}if(!!(w&&g)){var Y=0;try{for(var ae=(a=void 0,lh(b)),z=ae.next();!z.done;z=ae.next()){var Z=z.value,X=+Z[k]/V.absSum*g;Z[k]=X,Y+=X,Y>G?G=Y:Y<W&&(W=Y)}}catch(pe){a={error:pe}}finally{try{z&&!z.done&&(s=ae.return)&&s.call(ae)}finally{if(a)throw a.error}}}}if(w&&g){var ue=.5;W=W<-g*ue?-g:W,G=G>g*ue?g:G}return this.yDomain=this.fixNumericExtent([W,G],"y",f),this.fireEvent({type:"dataProcessed"}),!0},t.prototype.findLargestMinMax=function(e){var r,n,o=0,a=0;try{for(var s=lh(e),l=s.next();!l.done;l=s.next()){var u=l.value;u.min<o&&(o=u.min),u.max>a&&(a=u.max)}}catch(c){r={error:c}}finally{try{l&&!l.done&&(n=s.return)&&n.call(s)}finally{if(r)throw r.error}}return{min:o,max:a}},t.prototype.getDomain=function(e){return e===tt.X?this.xDomain:this.yDomain},t.prototype.update=function(){this.updatePending=!1,this.updateSelections(),this.updateNodes()},t.prototype.updateSelections=function(){!this.nodeDataPending||(this.nodeDataPending=!1,this.createSelectionData(),this.updateFillSelection(),this.updateStrokeSelection(),this.updateMarkerSelection(),this.updateLabelSelection())},t.prototype.updateNodes=function(){this.group.visible=this.visible&&this.xData.length>0&&this.yData.length>0,this.updateFillNodes(),this.updateStrokeNodes(),this.updateMarkerNodes(),this.updateLabelNodes()},t.prototype.createSelectionData=function(){var e=this,r=this,n=r.data,o=r.xAxis,a=r.yAxis,s=r.xData,l=r.yData,u=r.labelSelectionData,c=r.markerSelectionData,p=r.strokeSelectionData,h=r.fillSelectionData;if(r.xKey,!(!n||!o||!a||!s.length||!l.length)){var d=this,f=d.yKeys,g=d.marker,v=d.label,m=d.fills,C=d.strokes,w=o.scale,b=a.scale,S=b instanceof vr,T=(w.bandwidth||0)/2;c.length=0,u.length=0,p.length=0,h.length=0;var E=new Array(s.length).fill(null).map(function(){return{left:0,right:0}}),P=new Array(s.length).fill(0),R=function(N,W,G,k){var V=w.convert(N)+T,U=E[G][k],J=E[G][k]+W,Z=b.convert(U,S?Tl:void 0),le=b.convert(J,S?Tl:void 0);return E[G][k]=J,[{x:V,y:le},{x:V,y:Z}]},M=function(N,W,G,k){var V,U=e.normalizedTo&&isFinite(e.normalizedTo),J=U&&S&&Ar(k),Z=!U&&!isNaN(W)||J;Z&&(V=P[G]+=W);var le=w.convert(N)+T,Y=b.convert(V,S?Tl:void 0);return{x:le,y:Y}};l.forEach(function(N,W){var G=f[W],k=h[W]||(h[W]={itemId:G,points:[]}),V=k.points,U=[],J=p[W]||(p[W]={itemId:G,points:[],yValues:[]}),Z=J.points,le=J.yValues;N.forEach(function(Y,ae){var z,X=s[ae],ue=X.xDatum,pe=X.seriesDatum,be=(z=s[ae+1])===null||z===void 0?void 0:z.xDatum,_e=N[ae+1],Te=M(ue,+Y,ae,pe[G]);g&&c.push({index:ae,series:e,itemId:G,datum:pe,yValue:Y,yKey:G,point:Te,fill:m[W%m.length],stroke:C[W%C.length]});var Pe;v.formatter?Pe=v.formatter({value:Y}):Pe=ca(Y)?Number(Y).toFixed(2):String(Y),v&&u.push({index:ae,itemId:G,point:Te,label:e.seriesItemEnabled.get(G)&&Pe?{text:Pe,fontStyle:v.fontStyle,fontWeight:v.fontWeight,fontSize:v.fontSize,fontFamily:v.fontFamily,textAlign:"center",textBaseline:"bottom",fill:v.color}:void 0});var Ee=[ue,be],Le=[Y,_e];if(!Ee.some(function(Ge){return Ge==null})){Le.some(function(Ge){return Ge==null})&&(Le[0]=0,Le[1]=0);var Ze=R(Ee[0],+Le[0],ae,"right");V.push(Ze[0]),U.push(Ze[1]);var Re=R(Ee[1],+Le[1],ae,"left");V.push(Re[0]),U.push(Re[1]),Z.push({x:NaN,y:NaN}),le.push(void 0),Z.push(Ze[0]),le.push(Y),_e!==void 0&&(Z.push(Re[0]),le.push(Y))}}),V.push.apply(V,P9(U.slice().reverse()))})}},t.prototype.updateFillSelection=function(){var e=this.fillSelection.setData(this.fillSelectionData);e.exit.remove();var r=e.enter.append($r).each(function(n){n.lineJoin="round",n.stroke=void 0,n.pointerEvents=Wn.None});this.fillSelection=e.merge(r)},t.prototype.updateFillNodes=function(){var e=this,r=this,n=r.fills,o=r.fillOpacity,a=r.strokeOpacity,s=r.strokeWidth,l=r.shadow,u=r.seriesItemEnabled;this.fillSelection.each(function(c,p,h){c.fill=n[h%n.length],c.fillOpacity=o,c.strokeOpacity=a,c.strokeWidth=s,c.lineDash=e.lineDash,c.lineDashOffset=e.lineDashOffset,c.fillShadow=l,c.visible=!!u.get(p.itemId),c.opacity=e.getOpacity(p);var d=p.points,f=c.path;f.clear(),d.forEach(function(g,v){var m=g.x,C=g.y;v>0?f.lineTo(m,C):f.moveTo(m,C)}),f.closePath()})},t.prototype.updateStrokeSelection=function(){var e=this.strokeSelection.setData(this.strokeSelectionData);e.exit.remove();var r=e.enter.append($r).each(function(n){n.fill=void 0,n.lineJoin=n.lineCap="round",n.pointerEvents=Wn.None});this.strokeSelection=e.merge(r)},t.prototype.updateStrokeNodes=function(){var e=this;if(!!this.data){var r=this,n=r.strokes,o=r.strokeOpacity,a=r.seriesItemEnabled,s=!0;this.strokeSelection.each(function(l,u,c){l.visible=!!a.get(u.itemId),l.opacity=e.getOpacity(u),l.stroke=n[c%n.length],l.strokeWidth=e.getStrokeWidth(e.strokeWidth,u),l.strokeOpacity=o,l.lineDash=e.lineDash,l.lineDashOffset=e.lineDashOffset;var p=u.points,h=u.yValues,d=l.path;d.clear();for(var f=0;f<p.length;f++){var g=p[f],v=g.x,m=g.y;h[f]===void 0?s=!0:s?(d.moveTo(v,m),s=!1):d.lineTo(v,m)}})}},t.prototype.updateMarkerSelection=function(){var e=Ll(this.marker.shape),r=e?this.markerSelectionData:[],n=this.markerSelection.setData(r);n.exit.remove();var o=n.enter.append(e);this.markerSelection=n.merge(o)},t.prototype.updateMarkerNodes=function(){var e=this;if(!!this.chart){var r=this,n=r.xKey,o=r.marker,a=r.seriesItemEnabled,s=r.yKeys,l=r.fills,u=r.strokes,c=r.chart.highlightedDatum,p=r.highlightStyle,h=p.fill,d=p.stroke,f=p.strokeWidth,g=p.item,v=g.fill,m=v===void 0?h:v,C=g.stroke,w=C===void 0?d:C,b=g.strokeWidth,S=b===void 0?f:b,T=o.size,E=o.formatter,P=o.strokeWidth!==void 0?o.strokeWidth:this.strokeWidth;this.markerSelection.each(function(R,M){var N=s.indexOf(M.yKey),W=M===c,G=W&&m!==void 0?m:o.fill||l[N%l.length],k=W&&w!==void 0?w:o.stroke||u[N%l.length],V=W&&S!==void 0?S:P,U=void 0;E&&(U=E({datum:M.datum,xKey:n,yKey:M.yKey,fill:G,stroke:k,strokeWidth:V,size:T,highlighted:W})),R.fill=U&&U.fill||G,R.stroke=U&&U.stroke||k,R.strokeWidth=U&&U.strokeWidth!==void 0?U.strokeWidth:V,R.size=U&&U.size!==void 0?U.size:T,R.translationX=M.point.x,R.translationY=M.point.y,R.visible=o.enabled&&R.size>0&&!!a.get(M.yKey)&&!isNaN(M.point.x)&&!isNaN(M.point.y),R.opacity=e.getOpacity(M)})}},t.prototype.updateLabelSelection=function(){var e=this.labelSelection.setData(this.labelSelectionData);e.exit.remove();var r=e.enter.append(Pr);this.labelSelection=e.merge(r)},t.prototype.updateLabelNodes=function(){var e=this;if(!!this.chart){var r=this.label,n=r.enabled,o=r.fontStyle,a=r.fontWeight,s=r.fontSize,l=r.fontFamily,u=r.color;this.labelSelection.each(function(c,p){var h=p.point,d=p.label;d&&n?(c.fontStyle=o,c.fontWeight=a,c.fontSize=s,c.fontFamily=l,c.textAlign=d.textAlign,c.textBaseline=d.textBaseline,c.text=d.text,c.x=h.x,c.y=h.y-10,c.fill=u,c.visible=!0,c.opacity=e.getOpacity(p)):c.visible=!1})}},t.prototype.getNodeData=function(){return this.markerSelectionData},t.prototype.fireNodeClickEvent=function(e,r){this.fireEvent({type:"nodeClick",event:e,series:this,datum:r.datum,xKey:this.xKey,yKey:r.yKey})},t.prototype.getTooltipHtml=function(e){var r=this.xKey,n=e.yKey;if(!(r&&n)||!this.seriesItemEnabled.get(n))return"";var o=e.datum,a=o[r],s=o[n],l=this,u=l.xAxis,c=l.yAxis;if(!(u&&c&&ca(s)))return"";var p=this,h=p.xName,d=p.yKeys,f=p.yNames,g=p.yData,v=p.fills,m=p.strokes,C=p.tooltip,w=p.marker,b=w.size,S=w.formatter,T=w.strokeWidth,E=w.fill,P=w.stroke,R=u.formatDatum(a),M=c.formatDatum(s),N=d.indexOf(n),W=g[N],G=W[e.index],k=f[N],V=ir(k),U=ir(R+": "+M),J=T!==void 0?T:this.strokeWidth,Z=E||v[N%v.length],le=P||m[N%v.length],Y=void 0;S&&(Y=S({datum:o,xKey:r,yKey:n,fill:Z,stroke:le,strokeWidth:J,size:b,highlighted:!1}));var ae=Y&&Y.fill||Z,z={title:V,backgroundColor:ae,content:U},X=C.renderer,ue=C.format;if(ue||X){var pe={datum:o,xKey:r,xName:h,xValue:a,yKey:n,yValue:s,processedYValue:G,yName:k,color:ae};if(ue)return br({content:NA(ue,pe)},z);if(X)return br(X(pe),z)}return br(z)},t.prototype.listSeriesItems=function(e){var r=this,n=r.data,o=r.id,a=r.xKey,s=r.yKeys,l=r.yNames,u=r.seriesItemEnabled,c=r.marker,p=r.fills,h=r.strokes,d=r.fillOpacity,f=r.strokeOpacity;n&&n.length&&a&&s.length&&s.forEach(function(g,v){e.push({id:o,itemId:g,enabled:u.get(g)||!1,label:{text:l[v]||s[v]},marker:{shape:c.shape,fill:c.fill||p[v%p.length],stroke:c.stroke||h[v%h.length],fillOpacity:d,strokeOpacity:f}})})},t.prototype.toggleSeriesItem=function(e,r){this.seriesItemEnabled.set(e,r),this.scheduleData()},t.className="AreaSeries",t.type="area",on([$("dataChange")],t.prototype,"fills",void 0),on([$("dataChange")],t.prototype,"strokes",void 0),on([$("update")],t.prototype,"fillOpacity",void 0),on([$("update")],t.prototype,"strokeOpacity",void 0),on([$("update")],t.prototype,"lineDash",void 0),on([$("update")],t.prototype,"lineDashOffset",void 0),on([$("update")],t.prototype,"xName",void 0),on([$("update")],t.prototype,"yNames",void 0),on([$("update")],t.prototype,"strokeWidth",void 0),on([$("update")],t.prototype,"shadow",void 0),t}(op),ib=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),En=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Wx=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},Dd;(function(i){i[i.Bar=0]="Bar",i[i.Label=1]="Label"})(Dd||(Dd={}));var na;(function(i){i.Inside="inside",i.Outside="outside"})(na||(na={}));var M9=function(i){ib(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.placement=na.Inside,e}return En([$("change")],t.prototype,"formatter",void 0),En([$("change")],t.prototype,"placement",void 0),t}(Gn),F9=function(i){ib(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return En([$("change")],t.prototype,"renderer",void 0),t}(Os);function GA(i,t){return t===void 0&&(t=[]),i.forEach(function(e){Array.isArray(e)?GA(e,t):t.push(e)}),t}function I9(i){return i.length>0&&Array.isArray(i[0])}var Fy=function(i){ib(t,i);function t(){var e,r=i.call(this)||this;return r.rectGroup=r.pickGroup.appendChild(new It),r.labelGroup=r.group.appendChild(new It),r.rectSelection=Ht.select(r.rectGroup).selectAll(),r.labelSelection=Ht.select(r.labelGroup).selectAll(),r.nodeData=[],r.xData=[],r.yData=[],r.yDomain=[],r.label=new M9,r.seriesItemEnabled=new Map,r.tooltip=new F9,r.flipXY=!1,r.fills=["#c16068","#a2bf8a","#ebcc87","#80a0c3","#b58dae","#85c0d1"],r.strokes=["#874349","#718661","#a48f5f","#5a7088","#7f637a","#5d8692"],r.fillOpacity=1,r.strokeOpacity=1,r.lineDash=[0],r.lineDashOffset=0,r.groupScale=new Ni,r.directionKeys=(e={},e[tt.X]=["xKey"],e[tt.Y]=["yKeys"],e),r._xKey="",r._xName="",r.cumYKeyCount=[],r.flatYKeys=void 0,r.hideInLegend=[],r._yKeys=[],r._grouped=!1,r._yNames={},r._strokeWidth=1,r.addEventListener("update",r.scheduleUpdate),r.label.enabled=!1,r.label.addEventListener("change",r.scheduleUpdate,r),r}return t.prototype.getKeys=function(e){var r=this,n=this.directionKeys,o=n&&n[this.flipXY?Ax(e):e],a=[];return o&&o.forEach(function(s){var l=r[s];l&&(Array.isArray(l)?a=a.concat(GA(l)):a.push(l))}),a},Object.defineProperty(t.prototype,"xKey",{get:function(){return this._xKey},set:function(e){this._xKey!==e&&(this._xKey=e,this.xData=[],this.scheduleData())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"xName",{get:function(){return this._xName},set:function(e){this._xName!==e&&(this._xName=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"yKeys",{get:function(){return this._yKeys},set:function(e){var r=this,n=void 0;if(I9(e)||(n=e,e=this.grouped?n.map(function(u){return[u]}):[n]),!Ad(this._yKeys,e)){this.flatYKeys=n||void 0,this._yKeys=e;var o=0;this.cumYKeyCount=[];var a=[];e.forEach(function(u,c){u.length>0&&a.push(String(c)),r.cumYKeyCount.push(o),o+=u.length}),this.yData=[];var s=this.seriesItemEnabled;s.clear(),e.forEach(function(u){u.forEach(function(c){return s.set(c,!0)})});var l=this.groupScale;l.domain=a,l.padding=.1,l.round=!0,this.scheduleData()}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"grouped",{get:function(){return this._grouped},set:function(e){this._grouped!==e&&(this._grouped=e,this.flatYKeys&&(this.yKeys=this.flatYKeys))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"yNames",{get:function(){return this._yNames},set:function(e){if(Array.isArray(e)&&this.flatYKeys){var r={};this.flatYKeys.forEach(function(n,o){r[n]=e[o]}),e=r}this._yNames=e,this.scheduleData()},enumerable:!0,configurable:!0}),t.prototype.setColors=function(e,r){this.fills=e,this.strokes=r},Object.defineProperty(t.prototype,"normalizedTo",{get:function(){return this._normalizedTo},set:function(e){var r=e?Math.abs(e):void 0;this._normalizedTo!==r&&(this._normalizedTo=r,this.scheduleData())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeWidth",{get:function(){return this._strokeWidth},set:function(e){this._strokeWidth!==e&&(this._strokeWidth=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"shadow",{get:function(){return this._shadow},set:function(e){this._shadow!==e&&(this._shadow=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),t.prototype.onHighlightChange=function(){this.updateRectNodes()},t.prototype.processData=function(){var e=this,r=this,n=r.xKey,o=r.yKeys,a=r.seriesItemEnabled,s=r.xAxis,l=n&&o.length&&this.data?this.data:[];if(!s)return!1;var u=!0;this.xData=l.map(function(C){return u&&!(n in C)&&(u=!1,console.warn("The key '"+n+"' was not found in the data: ",C)),e.checkDatum(C[n],!1)}),this.yData=l.map(function(C){return o.map(function(w){return w.map(function(b){u&&!(b in C)&&(u=!1,console.warn("The key '"+b+"' was not found in the data: ",C));var S=e.checkDatum(C[b],!0);return!a.get(b)||S===void 0?0:S})})});var c=this.yData.map(function(C){return C.map(function(w){return q8(w)})}),p=this,h=p.yData,d=p.normalizedTo,f=this.yData.map(function(C){return C.map(function(w){return w.reduce(function(b,S){return b+=Math.abs(S),b},0)})}),g=this.findLargestMinMax(c),v,m;return d&&isFinite(d)?(v=g.min<0?-d:0,m=g.max>0?d:0,h.forEach(function(C,w){C.forEach(function(b,S){b.forEach(function(T,E){b[E]=T/f[w][S]*d})})})):(v=g.min,m=g.max),this.yDomain=this.fixNumericExtent([v,m],"y",this.yAxis),this.fireEvent({type:"dataProcessed"}),!0},t.prototype.findLargestMinMax=function(e){var r,n,o,a,s=0,l=0;try{for(var u=Wx(e),c=u.next();!c.done;c=u.next()){var p=c.value;try{for(var h=(o=void 0,Wx(p)),d=h.next();!d.done;d=h.next()){var f=d.value;f.min<s&&(s=f.min),f.max>l&&(l=f.max)}}catch(g){o={error:g}}finally{try{d&&!d.done&&(a=h.return)&&a.call(h)}finally{if(o)throw o.error}}}}catch(g){r={error:g}}finally{try{c&&!c.done&&(n=u.return)&&n.call(u)}finally{if(r)throw r.error}}return{min:s,max:l}},t.prototype.getDomain=function(e){return this.flipXY&&(e=Ax(e)),e===tt.X?this.xData:this.yDomain},t.prototype.fireNodeClickEvent=function(e,r){this.fireEvent({type:"nodeClick",event:e,series:this,datum:r.datum,xKey:this.xKey,yKey:r.yKey})},t.prototype.getCategoryAxis=function(){return this.flipXY?this.yAxis:this.xAxis},t.prototype.getValueAxis=function(){return this.flipXY?this.xAxis:this.yAxis},t.prototype.createNodeData=function(){var e=this,r=this,n=r.chart,o=r.data,a=r.visible,s=this.getCategoryAxis(),l=this.getValueAxis();if(!(n&&o&&a&&s&&l)||n.layoutPending||n.dataPending)return[];var u=this.flipXY,c=s.scale,p=l.scale,h=this,d=h.groupScale,f=h.yKeys,g=h.cumYKeyCount,v=h.fills,m=h.strokes,C=h.strokeWidth,w=h.seriesItemEnabled,b=h.xData,S=h.yData,T=h.label,E=T.fontStyle,P=T.fontWeight,R=T.fontSize,M=T.fontFamily,N=T.color,W=T.formatter,G=T.placement;d.range=[0,c.bandwidth];var k=d.bandwidth,V=[];return b.forEach(function(U,J){for(var Z=o[J],le=c.convert(U),Y=S[J],ae=0;ae<Y.length;ae++)for(var z=Y[ae],X=0,ue=0,pe=0;pe<z.length;pe++){var be=+z[pe],_e=f[ae][pe],Te=le+d.convert(String(ae));if(!!s.inRange(Te,k)){var Pe=be<0?X:ue,Ee=p instanceof vr,Le=p.convert(Pe+be,Ee?Tl:void 0),Ze=p.convert(Pe,Ee?Tl:void 0),Re=Z[_e],Ge=void 0;W?Ge=W({value:ca(Re)?Re:void 0}):Ge=ca(Re)?Re.toFixed(2):"";var Xe=void 0,ie=void 0;u?(ie=Te+k/2,G===na.Inside?Xe=Le+(Re>=0?-1:1)*Math.abs(Ze-Le)/2:Xe=Le+(Re>=0?1:-1)*4):(Xe=Te+k/2,G===na.Inside?ie=Le+(Re>=0?1:-1)*Math.abs(Ze-Le)/2:ie=Le+(Re>=0?-3:4));var He=void 0,Me=void 0;G===na.Inside?(He="center",Me="middle"):(He=u?Re>=0?"start":"end":"center",Me=u?"middle":Re>=0?"bottom":"top");var ne=g[ae]+pe;V.push({index:J,series:e,itemId:_e,datum:Z,yValue:Re,yKey:_e,x:u?Math.min(Le,Ze):Te,y:u?Te:Math.min(Le,Ze),width:u?Math.abs(Ze-Le):k,height:u?k:Math.abs(Ze-Le),fill:v[ne%v.length],stroke:m[ne%m.length],strokeWidth:C,label:w.get(_e)&&Ge?{text:Ge,fontStyle:E,fontWeight:P,fontSize:R,fontFamily:M,textAlign:He,textBaseline:Me,fill:N,x:Xe,y:ie}:void 0}),be<0?X+=be:ue+=be}}}),this.nodeData=V},t.prototype.update=function(){this.updatePending=!1,this.updateSelections(),this.updateNodes()},t.prototype.updateSelections=function(){!this.nodeDataPending||(this.nodeDataPending=!1,this.createNodeData(),this.updateRectSelection(),this.updateLabelSelection())},t.prototype.updateNodes=function(){this.group.visible=this.visible,this.updateRectNodes(),this.updateLabelNodes()},t.prototype.updateRectSelection=function(){var e=this.rectSelection.setData(this.nodeData);e.exit.remove();var r=e.enter.append(fo).each(function(n){n.tag=Dd.Bar,n.crisp=!0});this.rectSelection=e.merge(r)},t.prototype.updateRectNodes=function(){var e=this;if(!!this.chart){var r=this,n=r.fills,o=r.strokes,a=r.fillOpacity,s=r.strokeOpacity,l=r.shadow,u=r.formatter,c=r.xKey,p=r.flipXY,h=r.yKeys,d=r.chart.highlightedDatum,f=r.highlightStyle,g=f.fill,v=f.stroke,m=f.strokeWidth,C=f.item,w=C.fill,b=w===void 0?g:w,S=C.stroke,T=S===void 0?v:S,E=C.strokeWidth,P=E===void 0?m:E;this.rectSelection.each(function(R,M,N){for(var W=0,G=0,k=0;k<h.length;k++){var V=h[k];if(G=V.indexOf(M.yKey),G>=0){W+=G;break}W+=V.length}var U=M===d,J=U&&b!==void 0?b:n[W%n.length],Z=U&&T!==void 0?T:o[W%n.length],le=U&&P!==void 0?P:e.getStrokeWidth(e.strokeWidth,M),Y=void 0;u&&(Y=u({datum:M.datum,fill:J,stroke:Z,strokeWidth:le,highlighted:U,xKey:c,yKey:M.yKey})),R.x=M.x,R.y=M.y,R.width=M.width,R.height=M.height,R.fill=Y&&Y.fill||J,R.stroke=Y&&Y.stroke||Z,R.strokeWidth=Y&&Y.strokeWidth!==void 0?Y.strokeWidth:le,R.fillOpacity=a,R.strokeOpacity=s,R.lineDash=e.lineDash,R.lineDashOffset=e.lineDashOffset,R.fillShadow=l,R.visible=p?M.width>0:M.height>0,R.zIndex=U?Ql.highlightedZIndex:N,R.opacity=e.getOpacity(M)})}},t.prototype.updateLabelSelection=function(){var e=this.labelSelection.setData(this.nodeData);e.exit.remove();var r=e.enter.append(Pr).each(function(n){n.tag=Dd.Label,n.pointerEvents=Wn.None});this.labelSelection=e.merge(r)},t.prototype.updateLabelNodes=function(){var e=this;if(!!this.chart){var r=this;r.chart.highlightedDatum;var n=r.label,o=n.enabled,a=n.fontStyle,s=n.fontWeight,l=n.fontSize,u=n.fontFamily,c=n.color;this.labelSelection.each(function(p,h,d){var f=h.label;f&&o?(p.fontStyle=a,p.fontWeight=s,p.fontSize=l,p.fontFamily=u,p.textAlign=f.textAlign,p.textBaseline=f.textBaseline,p.text=f.text,p.x=f.x,p.y=f.y,p.fill=c,p.visible=!0,p.opacity=e.getOpacity(h)):p.visible=!1})}},t.prototype.getTooltipHtml=function(e){var r=this,n=r.xKey,o=r.yKeys,a=r.yData,s=this.getCategoryAxis(),l=this.getValueAxis(),u=e.yKey;if(!a.length||!n||!u||!s||!l)return"";for(var c=a[e.index],p=0,h=0,d=0;d<o.length;d++){var f=o[d];if(h=f.indexOf(u),h>=0){p+=h;break}p+=f.length}var g=this,v=g.xName,m=g.yNames,C=g.fills,w=g.strokes,b=g.tooltip,S=g.formatter,T=b.renderer,E=e.datum,P=m[u],R=C[p%C.length],M=w[p%C.length],N=this.getStrokeWidth(this.strokeWidth,E),W=E[n],G=E[u],k=c[d][h],V=ir(s.formatDatum(W)),U=ir(l.formatDatum(G)),J=ir(P),Z=V+": "+U,le=void 0;S&&(le=S({datum:E,fill:R,stroke:M,strokeWidth:N,highlighted:!1,xKey:n,yKey:u}));var Y=le&&le.fill||R,ae={title:J,backgroundColor:Y,content:Z};return T?br(T({datum:E,xKey:n,xValue:W,xName:v,yKey:u,yValue:G,processedYValue:k,yName:P,color:Y}),ae):br(ae)},t.prototype.listSeriesItems=function(e){var r=this,n=r.id,o=r.data,a=r.xKey,s=r.yKeys,l=r.yNames,u=r.cumYKeyCount,c=r.seriesItemEnabled,p=r.hideInLegend,h=r.fills,d=r.strokes,f=r.fillOpacity,g=r.strokeOpacity;o&&o.length&&a&&s.length&&this.yKeys.forEach(function(v,m){v.forEach(function(C,w){if(p.indexOf(C)<0){var b=u[m]+w;e.push({id:n,itemId:C,enabled:c.get(C)||!1,label:{text:l[C]||C},marker:{fill:h[b%h.length],stroke:d[b%d.length],fillOpacity:f,strokeOpacity:g}})}})})},t.prototype.toggleSeriesItem=function(e,r){var n=this.seriesItemEnabled;n.set(e,r);var o=this.yKeys.map(function(s){return s.slice()});n.forEach(function(s,l){s||o.forEach(function(u){var c=u.indexOf(l);c>=0&&u.splice(c,1)})});var a=[];o.forEach(function(s,l){s.length>0&&a.push(String(l))}),this.groupScale.domain=a,this.scheduleData()},t.className="BarSeries",t.type="bar",En([$("dataChange")],t.prototype,"flipXY",void 0),En([$("dataChange")],t.prototype,"fills",void 0),En([$("dataChange")],t.prototype,"strokes",void 0),En([$("update")],t.prototype,"fillOpacity",void 0),En([$("update")],t.prototype,"strokeOpacity",void 0),En([$("update")],t.prototype,"lineDash",void 0),En([$("update")],t.prototype,"lineDashOffset",void 0),En([$("update")],t.prototype,"formatter",void 0),En([$("layoutChange")],t.prototype,"hideInLegend",void 0),t}(op),ob=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Yn=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},L9=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},N9=function(i){ob(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return Yn([$("change")],t.prototype,"formatter",void 0),t}(Gn),G9=function(i){ob(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return Yn([$("change")],t.prototype,"renderer",void 0),Yn([$("change")],t.prototype,"format",void 0),t}(Os),ab=function(i){ob(t,i);function t(){var e=i.call(this)||this;e.xDomain=[],e.yDomain=[],e.xData=[],e.yData=[],e.lineNode=new $r,e.nodeSelection=Ht.select(e.pickGroup).selectAll(),e.nodeData=[],e.marker=new tb,e.label=new N9,e.stroke="#874349",e.lineDash=[0],e.lineDashOffset=0,e.strokeWidth=2,e.strokeOpacity=1,e.tooltip=new G9,e._xKey="",e.xName="",e._yKey="",e.yName="";var r=e.lineNode;r.fill=void 0,r.lineJoin="round",r.pointerEvents=Wn.None,e.group.insertBefore(r,e.pickGroup),e.addEventListener("update",e.scheduleUpdate);var n=e,o=n.marker,a=n.label;return o.fill="#c16068",o.stroke="#874349",o.addPropertyListener("shape",e.onMarkerShapeChange,e),o.addEventListener("change",e.scheduleUpdate,e),a.enabled=!1,a.addEventListener("change",e.scheduleUpdate,e),e}return t.prototype.onMarkerShapeChange=function(){this.nodeSelection=this.nodeSelection.setData([]),this.nodeSelection.exit.remove(),this.fireEvent({type:"legendChange"})},t.prototype.setColors=function(e,r){this.stroke=e[0],this.marker.stroke=r[0],this.marker.fill=e[0]},Object.defineProperty(t.prototype,"xKey",{get:function(){return this._xKey},set:function(e){this._xKey!==e&&(this._xKey=e,this.xData=[],this.scheduleData())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"yKey",{get:function(){return this._yKey},set:function(e){this._yKey!==e&&(this._yKey=e,this.yData=[],this.scheduleData())},enumerable:!0,configurable:!0}),t.prototype.processData=function(){var e=this,r=e.xAxis,n=e.yAxis,o=e.xKey,a=e.yKey,s=e.xData,l=e.yData,u=o&&a&&this.data?this.data:[];if(!r||!n)return!1;var c=r.scale instanceof vr,p=n.scale instanceof vr;s.length=0,l.length=0;for(var h=0,d=u.length;h<d;h++){var f=u[h],g=f[o],v=f[a],m=this.checkDatum(g,c);if(!(c&&m===void 0)){s.push(m);var C=this.checkDatum(v,p);l.push(C)}}return this.xDomain=c?this.fixNumericExtent(ti(s,Ar),"x",r):s,this.yDomain=p?this.fixNumericExtent(ti(l,Ar),"y",n):l,!0},t.prototype.getDomain=function(e){return e===tt.X?this.xDomain:this.yDomain},t.prototype.onHighlightChange=function(){this.updateNodes()},t.prototype.resetHighlight=function(){this.lineNode.strokeWidth=this.strokeWidth},t.prototype.update=function(){this.updatePending=!1,this.updateSelections(),this.updateNodes()},t.prototype.updateSelections=function(){!this.nodeDataPending||(this.nodeDataPending=!1,this.updateLinePath(),this.updateNodeSelection())},t.prototype.updateLinePath=function(){var e,r=this,n=r.data,o=r.xAxis,a=r.yAxis;if(!(!n||!o||!a)){var s=this,l=s.xData,u=s.yData,c=s.lineNode,p=s.label,h=s.xKey,d=s.yKey,f=o.scale,g=a.scale,v=(f.bandwidth||0)/2,m=(g.bandwidth||0)/2,C=c.path,w=[];C.clear();for(var b=!0,S=void 0,T=void 0,E=0;E<l.length;E++){var P=T||[l[E],u[E]];if(P[1]===void 0)S=void 0,b=!0;else{var R=L9(P,2),M=R[0],N=R[1],W=f.convert(M)+v;if(isNaN(W)){S=void 0,b=!0;continue}var G=(f.bandwidth||this.marker.size*.5+(this.marker.strokeWidth||0))+1;T=u[E+1]===void 0?void 0:[l[E+1],u[E+1]];var k=o.inRangeEx(W,0,G),V=T&&o.inRangeEx(f.convert(T[0])+v,0,G);if(k===-1&&V===-1){b=!0;continue}if(k===1&&S===1){b=!0;continue}S=k;var U=g.convert(N)+m;b?(C.moveTo(W,U),b=!1):C.lineTo(W,U);var J=void 0;p.formatter?J=p.formatter({value:N}):J=typeof N=="number"&&isFinite(N)?N.toFixed(2):N?String(N):"";var Z=(e={},e[h]=M,e[d]=N,e);w.push({series:this,datum:Z,point:{x:W,y:U},label:J?{text:J,fontStyle:p.fontStyle,fontWeight:p.fontWeight,fontSize:p.fontSize,fontFamily:p.fontFamily,textAlign:"center",textBaseline:"bottom",fill:p.color}:void 0})}}this.nodeData=w}},t.prototype.updateNodeSelection=function(){var e=this.marker,r=e.shape?this.nodeData:[],n=Ll(e.shape),o=this.nodeSelection.setData(r);o.exit.remove();var a=o.enter.append(It);a.append(n),a.append(Pr),this.nodeSelection=o.merge(a)},t.prototype.updateNodes=function(){this.group.visible=this.visible,this.updateLineNode(),this.updateMarkerNodes(),this.updateTextNodes()},t.prototype.updateLineNode=function(){var e=this.lineNode;e.stroke=this.stroke,e.strokeWidth=this.getStrokeWidth(this.strokeWidth),e.strokeOpacity=this.strokeOpacity,e.lineDash=this.lineDash,e.lineDashOffset=this.lineDashOffset,e.opacity=this.getOpacity()},t.prototype.updateMarkerNodes=function(){var e=this;if(!!this.chart){var r=this,n=r.marker,o=r.xKey,a=r.yKey,s=r.stroke,l=r.chart.highlightedDatum,u=r.highlightStyle,c=u.fill,p=u.stroke,h=u.strokeWidth,d=u.item,f=d.fill,g=f===void 0?c:f,v=d.stroke,m=v===void 0?p:v,C=d.strokeWidth,w=C===void 0?h:C,b=n.size,S=n.formatter,T=n.strokeWidth!==void 0?n.strokeWidth:this.strokeWidth,E=Ll(n.shape);this.nodeSelection.selectByClass(E).each(function(P,R){var M=R===l,N=M&&g!==void 0?g:n.fill,W=M&&m!==void 0?m:n.stroke||s,G=M&&w!==void 0?w:T,k=void 0;S&&(k=S({datum:R.datum,xKey:o,yKey:a,fill:N,stroke:W,strokeWidth:G,size:b,highlighted:M})),P.fill=k&&k.fill||N,P.stroke=k&&k.stroke||W,P.strokeWidth=k&&k.strokeWidth!==void 0?k.strokeWidth:G,P.size=k&&k.size!==void 0?k.size:b,P.translationX=R.point.x,P.translationY=R.point.y,P.opacity=e.getOpacity(R),P.visible=n.enabled&&P.size>0})}},t.prototype.updateTextNodes=function(){var e=this;this.nodeSelection.selectByClass(Pr).each(function(r,n){var o=n.point,a=n.label,s=e.label,l=s.enabled,u=s.fontStyle,c=s.fontWeight,p=s.fontSize,h=s.fontFamily,d=s.color;a&&l?(r.fontStyle=u,r.fontWeight=c,r.fontSize=p,r.fontFamily=h,r.textAlign=a.textAlign,r.textBaseline=a.textBaseline,r.text=a.text,r.x=o.x,r.y=o.y-10,r.fill=d,r.visible=!0):r.visible=!1})},t.prototype.getNodeData=function(){return this.nodeData},t.prototype.fireNodeClickEvent=function(e,r){this.fireEvent({type:"nodeClick",event:e,series:this,datum:r.datum,xKey:this.xKey,yKey:this.yKey})},t.prototype.getTooltipHtml=function(e){var r=this,n=r.xKey,o=r.yKey,a=r.xAxis,s=r.yAxis;if(!n||!o||!a||!s)return"";var l=this,u=l.xName,c=l.yName,p=l.tooltip,h=l.marker,d=p.renderer,f=p.format,g=e.datum,v=g[n],m=g[o],C=a.formatDatum(v),w=s.formatDatum(m),b=ir(this.title||c),S=ir(C+": "+w),T=h.formatter,E=h.fill,P=h.stroke,R=h.strokeWidth,M=h.size,N=R!==void 0?R:this.strokeWidth,W=void 0;T&&(W=T({datum:g,xKey:n,yKey:o,fill:E,stroke:P,strokeWidth:N,size:M,highlighted:!1}));var G=W&&W.fill||E,k={title:b,backgroundColor:G,content:S};if(f||d){var V={datum:g,xKey:n,xValue:v,xName:u,yKey:o,yValue:m,yName:c,title:b,color:G};if(f)return br({content:NA(f,V)},k);if(d)return br(d(V),k)}return br(k)},t.prototype.listSeriesItems=function(e){var r=this,n=r.id,o=r.data,a=r.xKey,s=r.yKey,l=r.yName,u=r.visible,c=r.title,p=r.marker,h=r.stroke,d=r.strokeOpacity;o&&o.length&&a&&s&&e.push({id:n,itemId:void 0,enabled:u,label:{text:c||l||s},marker:{shape:p.shape,fill:p.fill||"rgba(0, 0, 0, 0)",stroke:p.stroke||h||"rgba(0, 0, 0, 0)",fillOpacity:1,strokeOpacity:d}})},t.className="LineSeries",t.type="line",Yn([$("layoutChange")],t.prototype,"title",void 0),Yn([$("update")],t.prototype,"stroke",void 0),Yn([$("update")],t.prototype,"lineDash",void 0),Yn([$("update")],t.prototype,"lineDashOffset",void 0),Yn([$("update")],t.prototype,"strokeWidth",void 0),Yn([$("update")],t.prototype,"strokeOpacity",void 0),Yn([$("update")],t.prototype,"xName",void 0),Yn([$("update")],t.prototype,"yName",void 0),t}(op),kA=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Iy=globalThis&&globalThis.__assign||function(){return Iy=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Iy.apply(this,arguments)},ul=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},k9=function(i){kA(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return ul([$("change")],t.prototype,"renderer",void 0),t}(Os),WA=function(i){kA(t,i);function t(){var e=i.call(this)||this;e.xDomain=[],e.yDomain=[],e.xData=[],e.yData=[],e.validData=[],e.sizeData=[],e.sizeScale=new Nt,e.nodeData=[],e.markerSelection=Ht.select(e.pickGroup).selectAll(),e.labelSelection=Ht.select(e.group).selectAll(),e.marker=new tb,e.label=new Gn,e._fill="#c16068",e._stroke="#874349",e._strokeWidth=2,e._fillOpacity=1,e._strokeOpacity=1,e.xKey="",e.yKey="",e.xName="",e.yName="",e.sizeName="Size",e.labelName="Label",e.tooltip=new k9;var r=e,n=r.marker,o=r.label;return n.addPropertyListener("shape",e.onMarkerShapeChange,e),n.addEventListener("change",e.scheduleUpdate,e),e.addPropertyListener("xKey",function(){return e.xData=[]}),e.addPropertyListener("yKey",function(){return e.yData=[]}),e.addPropertyListener("sizeKey",function(){return e.sizeData=[]}),o.enabled=!1,o.addEventListener("change",e.scheduleUpdate,e),o.addEventListener("dataChange",e.scheduleData,e),e}return Object.defineProperty(t.prototype,"fill",{get:function(){return this._fill},set:function(e){this._fill!==e&&(this._fill=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"stroke",{get:function(){return this._stroke},set:function(e){this._stroke!==e&&(this._stroke=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeWidth",{get:function(){return this._strokeWidth},set:function(e){this._strokeWidth!==e&&(this._strokeWidth=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fillOpacity",{get:function(){return this._fillOpacity},set:function(e){this._fillOpacity!==e&&(this._fillOpacity=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeOpacity",{get:function(){return this._strokeOpacity},set:function(e){this._strokeOpacity!==e&&(this._strokeOpacity=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),t.prototype.onHighlightChange=function(){this.updateMarkerNodes()},t.prototype.onMarkerShapeChange=function(){this.markerSelection=this.markerSelection.setData([]),this.markerSelection.exit.remove(),this.fireEvent({type:"legendChange"})},t.prototype.setColors=function(e,r){this.fill=e[0],this.stroke=r[0],this.marker.fill=e[0],this.marker.stroke=r[0]},t.prototype.processData=function(){var e=this,r=this,n=r.xKey,o=r.yKey,a=r.sizeKey;r.labelKey;var s=r.xAxis,l=r.yAxis,u=r.marker;if(r.label,!s||!l)return!1;var c=n&&o&&this.data?this.data:[],p=s.scale,h=l.scale,d=p instanceof vr,f=h instanceof vr;return this.validData=c.filter(function(g){return e.checkDatum(g[n],d)!==void 0&&e.checkDatum(g[o],f)!==void 0}),this.xData=this.validData.map(function(g){return g[n]}),this.yData=this.validData.map(function(g){return g[o]}),this.sizeData=a?this.validData.map(function(g){return g[a]}):[],this.sizeScale.domain=u.domain?u.domain:ti(this.sizeData,Ar)||[1,1],s.scale instanceof vr?this.xDomain=this.fixNumericExtent(ti(this.xData,Ar),"x",s):this.xDomain=this.xData,l.scale instanceof vr?this.yDomain=this.fixNumericExtent(ti(this.yData,Ar),"y",l):this.yDomain=this.yData,!0},t.prototype.getDomain=function(e){return e===tt.X?this.xDomain:this.yDomain},t.prototype.getNodeData=function(){return this.nodeData},t.prototype.getLabelData=function(){return this.nodeData},t.prototype.fireNodeClickEvent=function(e,r){this.fireEvent({type:"nodeClick",event:e,series:this,datum:r.datum,xKey:this.xKey,yKey:this.yKey,sizeKey:this.sizeKey})},t.prototype.createNodeData=function(){var e=this,r=e.chart,n=e.data,o=e.visible,a=e.xAxis,s=e.yAxis,l=e.label,u=e.labelKey;if(!(r&&n&&o&&a&&s)||r.layoutPending||r.dataPending)return[];var c=a.scale,p=s.scale,h=c instanceof vr,d=p instanceof vr,f=(c.bandwidth||0)/2,g=(p.bandwidth||0)/2,v=this,m=v.xData,C=v.yData,w=v.validData,b=v.sizeData,S=v.sizeScale,T=v.marker,E=[];S.range=[T.size,T.maxSize];for(var P=l.getFont(),R=0;R<m.length;R++){var M=this.checkDomainXY(m[R],C[R],h,d);if(!!M){var N=c.convert(M[0])+f,W=p.convert(M[1])+g;if(!!this.checkRangeXY(N,W,a,s)){var G=u?String(w[R][u]):"",k=ns.getTextSize(G,P);E.push({series:this,datum:w[R],point:{x:N,y:W},size:b.length?S.convert(b[R]):T.size,label:Iy({text:G},k)})}}}return this.nodeData=E},t.prototype.update=function(){this.updatePending=!1,this.updateSelections(),this.updateNodes()},t.prototype.updateSelections=function(){!this.nodeDataPending||(this.nodeDataPending=!1,this.createNodeData(),this.updateMarkerSelection(),this.updateLabelSelection())},t.prototype.updateNodes=function(){this.group.visible=this.visible,this.updateMarkerNodes(),this.updateLabelNodes()},t.prototype.updateLabelSelection=function(){var e=this.chart&&this.chart.placeLabels().get(this)||[],r=this.labelSelection.setData(e);r.exit.remove();var n=r.enter.append(Pr);this.labelSelection=r.merge(n)},t.prototype.updateMarkerSelection=function(){var e=Ll(this.marker.shape),r=this.markerSelection.setData(this.nodeData);r.exit.remove();var n=r.enter.append(e);this.markerSelection=r.merge(n)},t.prototype.updateLabelNodes=function(){var e=this.label;this.labelSelection.each(function(r,n){r.text=n.text,r.fill=e.color,r.x=n.x,r.y=n.y,r.fontStyle=e.fontStyle,r.fontWeight=e.fontWeight,r.fontSize=e.fontSize,r.fontFamily=e.fontFamily,r.textAlign="left",r.textBaseline="top"})},t.prototype.updateMarkerNodes=function(){var e=this;if(!!this.chart){var r=this,n=r.marker,o=r.xKey,a=r.yKey,s=r.strokeWidth,l=r.fillOpacity,u=r.strokeOpacity,c=r.fill,p=r.stroke,h=r.chart.highlightedDatum,d=r.sizeScale,f=r.sizeData,g=r.highlightStyle,v=g.fill,m=g.stroke,C=g.strokeWidth,w=g.item,b=w.fill,S=b===void 0?v:b,T=w.stroke,E=T===void 0?m:T,P=w.strokeWidth,R=P===void 0?C:P,M=n.strokeWidth!==void 0?n.strokeWidth:s,N=n.formatter;d.range=[n.size,n.maxSize],this.markerSelection.each(function(W,G,k){var V=G===h,U=V&&S!==void 0?S:n.fill||c,J=V&&E!==void 0?E:n.stroke||p,Z=V&&R!==void 0?R:e.getStrokeWidth(M,G),le=f.length?d.convert(f[k]):n.size,Y=void 0;N&&(Y=N({datum:G.datum,xKey:o,yKey:a,fill:U,stroke:J,strokeWidth:Z,size:le,highlighted:V})),W.fill=Y&&Y.fill||U,W.stroke=Y&&Y.stroke||J,W.strokeWidth=Y&&Y.strokeWidth!==void 0?Y.strokeWidth:Z,W.size=Y&&Y.size!==void 0?Y.size:le,W.fillOpacity=n.fillOpacity!==void 0?n.fillOpacity:l,W.strokeOpacity=n.strokeOpacity!==void 0?n.strokeOpacity:u,W.translationX=G.point.x,W.translationY=G.point.y,W.opacity=e.getOpacity(G),W.zIndex=V?Ql.highlightedZIndex:k,W.visible=n.enabled&&W.size>0})}},t.prototype.getTooltipHtml=function(e){var r=this,n=r.xKey,o=r.yKey,a=r.xAxis,s=r.yAxis;if(!n||!o||!a||!s)return"";var l=this,u=l.fill,c=l.stroke,p=l.marker,h=l.tooltip,d=l.xName,f=l.yName,g=l.sizeKey,v=l.sizeName,m=l.labelKey,C=l.labelName,w=p.fill||u,b=p.stroke||c,S=this.getStrokeWidth(p.strokeWidth||this.strokeWidth,e),T=this.marker.formatter,E=void 0;T&&(E=T({datum:e,xKey:n,yKey:o,fill:w,stroke:b,strokeWidth:S,size:e.size,highlighted:!1}));var P=E&&E.fill||w||"gray",R=this.title||f,M=e.datum,N=M[n],W=M[o],G=ir(a.formatDatum(N)),k=ir(s.formatDatum(W)),V="<b>"+ir(d||n)+"</b>: "+G+"<br>"+("<b>"+ir(f||o)+"</b>: "+k);g&&(V+="<br><b>"+ir(v||g)+"</b>: "+ir(M[g])),m&&(V="<b>"+ir(C||m)+"</b>: "+ir(M[m])+"<br>"+V);var U={title:R,backgroundColor:P,content:V},J=h.renderer;return J?br(J({datum:M,xKey:n,xValue:N,xName:d,yKey:o,yValue:W,yName:f,sizeKey:g,sizeName:v,labelKey:m,labelName:C,title:R,color:P}),U):br(U)},t.prototype.listSeriesItems=function(e){var r=this,n=r.id,o=r.data,a=r.xKey,s=r.yKey,l=r.yName,u=r.title,c=r.visible,p=r.marker,h=r.fill,d=r.stroke,f=r.fillOpacity,g=r.strokeOpacity;o&&o.length&&a&&s&&e.push({id:n,itemId:void 0,enabled:c,label:{text:u||l||s},marker:{shape:p.shape,fill:p.fill||h||"rgba(0, 0, 0, 0)",stroke:p.stroke||d||"rgba(0, 0, 0, 0)",fillOpacity:p.fillOpacity!==void 0?p.fillOpacity:f,strokeOpacity:p.strokeOpacity!==void 0?p.strokeOpacity:g}})},t.className="ScatterSeries",t.type="scatter",ul([$("layoutChange")],t.prototype,"title",void 0),ul([$("dataChange")],t.prototype,"xKey",void 0),ul([$("dataChange")],t.prototype,"yKey",void 0),ul([$("dataChange")],t.prototype,"sizeKey",void 0),ul([$("dataChange")],t.prototype,"labelKey",void 0),t}(op),sb=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),No=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Nc=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Vx=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(Nc(arguments[t]));return i},Md;(function(i){i[i.Bin=0]="Bin",i[i.Label=1]="Label"})(Md||(Md={}));var W9=function(i){sb(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return No([$("change")],t.prototype,"formatter",void 0),t}(Gn),Bx=10,Ly={count:function(i){return i.data.length},sum:function(i,t){return i.data.reduce(function(e,r){return e+r[t]},0)},mean:function(i,t){return Ly.sum(i,t)/Ly.count(i,t)}},Hx=function(){function i(t){var e=Nc(t,2),r=e[0],n=e[1];this.data=[],this.aggregatedValue=0,this.frequency=0,this.domain=[r,n]}return i.prototype.addDatum=function(t){this.data.push(t),this.frequency++},Object.defineProperty(i.prototype,"domainWidth",{get:function(){var t=Nc(this.domain,2),e=t[0],r=t[1];return r-e},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"relativeHeight",{get:function(){return this.aggregatedValue/this.domainWidth},enumerable:!0,configurable:!0}),i.prototype.calculateAggregatedValue=function(t,e){e||(t="count");var r=Ly[t];this.aggregatedValue=r(this,e)},i.prototype.getY=function(t){return t?this.relativeHeight:this.aggregatedValue},i}(),V9=function(i){sb(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return No([$("change")],t.prototype,"renderer",void 0),t}(Os),VA=function(i){sb(t,i);function t(){var e,r=i.call(this)||this;return r.rectGroup=r.pickGroup.appendChild(new It),r.textGroup=r.group.appendChild(new It),r.rectSelection=Ht.select(r.rectGroup).selectAll(),r.textSelection=Ht.select(r.textGroup).selectAll(),r.binnedData=[],r.xDomain=[],r.yDomain=[],r.label=new W9,r.seriesItemEnabled=!0,r.tooltip=new V9,r.fill=void 0,r.stroke=void 0,r.fillOpacity=1,r.strokeOpacity=1,r.lineDash=[0],r.lineDashOffset=0,r.directionKeys=(e={},e[tt.X]=["xKey"],e[tt.Y]=["yKey"],e),r._xKey="",r._areaPlot=!1,r._bins=void 0,r._aggregation="count",r._binCount=void 0,r._xName="",r._yKey="",r._yName="",r._strokeWidth=1,r.label.enabled=!1,r.label.addEventListener("change",r.scheduleUpdate,r),r}return t.prototype.getKeys=function(e){var r=this,n=this.directionKeys,o=n&&n[e],a=[];return o&&o.forEach(function(s){var l=r[s];l&&(Array.isArray(l)?a.push.apply(a,Vx(l)):a.push(l))}),a},Object.defineProperty(t.prototype,"xKey",{get:function(){return this._xKey},set:function(e){this._xKey!==e&&(this._xKey=e,this.scheduleData())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"areaPlot",{get:function(){return this._areaPlot},set:function(e){this._areaPlot=e,this.scheduleData()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"bins",{get:function(){return this._bins},set:function(e){this._bins=e,this.scheduleData()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"aggregation",{get:function(){return this._aggregation},set:function(e){this._aggregation=e,this.scheduleData()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"binCount",{get:function(){return this._binCount},set:function(e){this._binCount=e,this.scheduleData()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"xName",{get:function(){return this._xName},set:function(e){this._xName!==e&&(this._xName=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"yKey",{get:function(){return this._yKey},set:function(e){this._yKey=e,this.seriesItemEnabled=!0,this.scheduleData()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"yName",{get:function(){return this._yName},set:function(e){this._yName=e,this.scheduleData()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeWidth",{get:function(){return this._strokeWidth},set:function(e){this._strokeWidth!==e&&(this._strokeWidth=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"shadow",{get:function(){return this._shadow},set:function(e){this._shadow!==e&&(this._shadow=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),t.prototype.onHighlightChange=function(){this.updateRectNodes()},t.prototype.setColors=function(e,r){this.fill=e[0],this.stroke=r[0]},t.prototype.deriveBins=function(){var e=this,r=this,n=r.bins;if(r.binCount,!this.data)return[];if(n)return n;var o=this.data.map(function(p){return p[e.xKey]}),a=this.fixNumericExtent(ti(o,Ar),"x"),s=Od(a[0],a[1],this.binCount||Bx),l=Ic(a[0],a[1],this.binCount||Bx),u=s[0],c=function(p){return[p,p+l]};return Vx([[u-l,u]],s.map(c))},t.prototype.placeDataInBins=function(e){var r=this,n=this.xKey,o=this.deriveBins(),a=e.slice().sort(function(d,f){return d[n]<f[n]?-1:d[n]>f[n]?1:0}),s=0,l=[new Hx(o[0])];e:for(var u=0,c=a.length;u<c;u++){for(var p=a[u];p[n]>o[s][1];){s++;var h=o[s];if(!h)break e;l.push(new Hx(h))}l[s].addDatum(p)}return l.forEach(function(d){return d.calculateAggregatedValue(r._aggregation,r.yKey)}),l},Object.defineProperty(t.prototype,"xMax",{get:function(){var e=this;return this.data&&this.data.reduce(function(r,n){return Math.max(r,n[e.xKey])},Number.NEGATIVE_INFINITY)},enumerable:!0,configurable:!0}),t.prototype.processData=function(){var e=this,r=this,n=r.xKey,o=r.data;this.binnedData=this.placeDataInBins(n&&o?o:[]);var a=this.binnedData.map(function(h){return h.getY(e.areaPlot)}),s=ti(a,Ar);this.yDomain=this.fixNumericExtent([0,s?s[1]:1],"y");var l=this.binnedData[0],u=this.binnedData[this.binnedData.length-1],c=l.domain[0],p=u.domain[1];return this.xDomain=[c,p],this.fireEvent({type:"dataProcessed"}),!0},t.prototype.getDomain=function(e){return e===tt.X?this.xDomain:this.yDomain},t.prototype.fireNodeClickEvent=function(e,r){this.fireEvent({type:"nodeClick",event:e,series:this,datum:r.datum,xKey:this.xKey})},t.prototype.update=function(){this.updatePending=!1,this.updateSelections(),this.updateNodes()},t.prototype.updateSelections=function(){if(!!this.nodeDataPending){this.nodeDataPending=!1;var e=this.createNodeData();this.updateRectSelection(e),this.updateTextSelection(e)}},t.prototype.updateNodes=function(){this.group.visible=this.visible,this.updateRectNodes(),this.updateTextNodes()},t.prototype.createNodeData=function(){var e=this,r=this,n=r.xAxis,o=r.yAxis;if(!this.seriesItemEnabled||!n||!o)return[];var a=n.scale,s=o.scale,l=this,u=l.fill,c=l.stroke,p=l.strokeWidth,h=[],d=function(T){return String(T.value)},f=this.label,g=f.formatter,v=g===void 0?d:g,m=f.fontStyle,C=f.fontWeight,w=f.fontSize,b=f.fontFamily,S=f.color;return this.binnedData.forEach(function(T){var E=T.aggregatedValue,P=T.frequency,R=Nc(T.domain,2),M=R[0],N=R[1],W=T.relativeHeight,G=a.convert(M),k=a.convert(N),V=e.areaPlot?W:e.yKey?E:P,U=s.convert(0),J=s.convert(V),Z=k-G,le=Math.abs(J-U),Y=V!==0?{text:v({value:T.aggregatedValue}),fontStyle:m,fontWeight:C,fontSize:w,fontFamily:b,fill:S,x:G+Z/2,y:J+le/2}:void 0;h.push({series:e,datum:T,x:G,y:J,width:Z,height:le,fill:u,stroke:c,strokeWidth:p,label:Y})}),h},t.prototype.updateRectSelection=function(e){var r=this.rectSelection.setData(e);r.exit.remove();var n=r.enter.append(fo).each(function(o){o.tag=Md.Bin,o.crisp=!0});this.rectSelection=r.merge(n)},t.prototype.updateRectNodes=function(){var e=this;if(!!this.chart){var r=this,n=r.fillOpacity,o=r.strokeOpacity,a=r.shadow,s=r.chart.highlightedDatum,l=r.highlightStyle,u=l.fill,c=l.stroke,p=l.strokeWidth,h=l.item,d=h.fill,f=d===void 0?u:d,g=h.stroke,v=g===void 0?c:g,m=h.strokeWidth,C=m===void 0?p:m;this.rectSelection.each(function(w,b,S){var T=b===s,E=T&&C!==void 0?C:e.getStrokeWidth(b.strokeWidth,b);w.x=b.x,w.y=b.y,w.width=b.width,w.height=b.height,w.fill=T&&f!==void 0?f:b.fill,w.stroke=T&&v!==void 0?v:b.stroke,w.fillOpacity=n,w.strokeOpacity=o,w.strokeWidth=E,w.lineDash=e.lineDash,w.lineDashOffset=e.lineDashOffset,w.fillShadow=a,w.zIndex=T?Ql.highlightedZIndex:S,w.visible=b.height>0,w.opacity=e.getOpacity(b)})}},t.prototype.updateTextSelection=function(e){var r=this.textSelection.setData(e);r.exit.remove();var n=r.enter.append(Pr).each(function(o){o.tag=Md.Label,o.pointerEvents=Wn.None,o.textAlign="center",o.textBaseline="middle"});this.textSelection=r.merge(n)},t.prototype.updateTextNodes=function(){var e=this.label.enabled;this.textSelection.each(function(r,n){var o=n.label;o&&e?(r.text=o.text,r.x=o.x,r.y=o.y,r.fontStyle=o.fontStyle,r.fontWeight=o.fontWeight,r.fontSize=o.fontSize,r.fontFamily=o.fontFamily,r.fill=o.fill,r.visible=!0):r.visible=!1})},t.prototype.getTooltipHtml=function(e){var r=this,n=r.xKey,o=r.yKey,a=r.xAxis,s=r.yAxis;if(!n||!a||!s)return"";var l=this,u=l.xName,c=l.yName,p=l.fill,h=l.tooltip,d=l.aggregation,f=h.renderer,g=e.datum,v=g.aggregatedValue,m=g.frequency,C=Nc(g.domain,2),w=C[0],b=C[1],S=ir(u||n)+": "+a.formatDatum(w)+" - "+a.formatDatum(b),T=o?"<b>"+ir(c||o)+" ("+d+")</b>: "+s.formatDatum(v)+"<br>":"";T+="<b>Frequency</b>: "+m;var E={title:S,backgroundColor:p,content:T};return f?br(f({datum:g,xKey:n,xValue:g.domain,xName:u,yKey:o,yValue:g.aggregatedValue,yName:c,color:p}),E):br(E)},t.prototype.listSeriesItems=function(e){var r=this,n=r.id,o=r.data,a=r.yKey,s=r.yName,l=r.seriesItemEnabled,u=r.fill,c=r.stroke,p=r.fillOpacity,h=r.strokeOpacity;o&&o.length&&e.push({id:n,itemId:a,enabled:l,label:{text:s||a||"Frequency"},marker:{fill:u||"rgba(0, 0, 0, 0)",stroke:c||"rgba(0, 0, 0, 0)",fillOpacity:p,strokeOpacity:h}})},t.prototype.toggleSeriesItem=function(e,r){e===this.yKey&&(this.seriesItemEnabled=r),this.scheduleData()},t.className="HistogramSeries",t.type="histogram",No([$("dataChange")],t.prototype,"fill",void 0),No([$("dataChange")],t.prototype,"stroke",void 0),No([$("layoutChange")],t.prototype,"fillOpacity",void 0),No([$("layoutChange")],t.prototype,"strokeOpacity",void 0),No([$("update")],t.prototype,"lineDash",void 0),No([$("update")],t.prototype,"lineDashOffset",void 0),t}(op),B9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),H9=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},j9=function(i){B9(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.data=void 0,e}return H9([$("dataChange")],t.prototype,"data",void 0),t}(Ql),U9=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Lu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},BA=function(i){U9(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.enabled=!0,e.color="rgba(0, 0, 0, 0.5)",e.xOffset=0,e.yOffset=0,e.blur=5,e}return Lu([$("change")],t.prototype,"enabled",void 0),Lu([$("change")],t.prototype,"color",void 0),Lu([$("change")],t.prototype,"xOffset",void 0),Lu([$("change")],t.prototype,"yOffset",void 0),Lu([$("change")],t.prototype,"blur",void 0),t}(Ir);function z9(i,t,e,r,n){var o=i.children,a=i.value&&(n-e)/i.value;o.forEach(function(s){s.x0=t,s.x1=r,s.y0=e,s.y1=e+=s.value*a})}function $9(i,t,e,r,n){var o=i.children,a=i.value&&(r-t)/i.value;o.forEach(function(s){s.x0=t,s.x1=t+=s.value*a,s.y0=e,s.y1=n})}function K9(i){i.x0=Math.round(i.x0),i.y0=Math.round(i.y0),i.x1=Math.round(i.x1),i.y1=Math.round(i.y1)}function Y9(i,t,e,r,n,o){for(var a=[],s=t.children,l=s.length,u=t.value,c=0,p=0,h,d,f,g,v,m,C,w,b,S;c<l;){h=n-e,d=o-r;do g=s[p++].value;while(!g&&p<l);for(v=m=g,b=Math.max(d/h,h/d)/(u*i),S=g*g*b,w=Math.max(m/S,S/v);p<l;++p){if(f=s[p].value,g+=f,f<v&&(v=f),f>m&&(m=f),S=g*g*b,C=Math.max(m/S,S/v),C>w){g-=f;break}w=C}var T={value:g,dice:h<d,children:s.slice(c,p)};a.push(T),T.dice?$9(T,e,r,n,u?r+=d*g/u:o):z9(T,e,r,u?e+=h*g/u:n,o),u-=g,c=p}return a}var X9=(1+Math.sqrt(5))/2,q9=function i(t){function e(r,n,o,a,s){Y9(t,r,n,o,a,s)}return e.ratio=function(r){return i((r=+r)>1?r:1)},e}(X9),Z9=function(){function i(){this.paddingStack=[0],this.dx=1,this.dy=1,this.round=!0,this.tile=q9,this.paddingInner=function(t){return 0},this.paddingTop=function(t){return 0},this.paddingRight=function(t){return 0},this.paddingBottom=function(t){return 0},this.paddingLeft=function(t){return 0}}return Object.defineProperty(i.prototype,"size",{get:function(){return[this.dx,this.dy]},set:function(t){this.dx=t[0],this.dy=t[1]},enumerable:!0,configurable:!0}),i.prototype.processData=function(t){return t.x0=0,t.y0=0,t.x1=this.dx,t.y1=this.dy,t.eachBefore(this.positionNode.bind(this)),this.paddingStack=[0],this.round&&t.eachBefore(K9),t},i.prototype.positionNode=function(t){var e=this.paddingStack[t.depth],r=t.x0+e,n=t.y0+e,o=t.x1-e,a=t.y1-e;o<r&&(r=o=(r+o)/2),a<n&&(n=a=(n+a)/2),t.x0=r,t.y0=n,t.x1=o,t.y1=a,t.children&&(e=this.paddingStack[t.depth+1]=this.paddingInner(t)/2,r+=this.paddingLeft(t)-e,n+=this.paddingTop(t)-e,o-=this.paddingRight(t)-e,a-=this.paddingBottom(t)-e,o<r&&(r=o=(r+o)/2),a<n&&(n=a=(n+a)/2),this.tile(t,r,n,o,a))},i}(),jx=function(){function i(t){this.value=0,this.depth=0,this.height=0,this.parent=void 0,this.children=void 0,this.datum=t}return i.prototype.countFn=function(t){var e=0,r=t.children;if(!r||!r.length)e=1;else for(var n=r.length;--n>=0;)e+=r[n].value;t.value=e},i.prototype.count=function(){return this.eachAfter(this.countFn)},i.prototype.each=function(t,e){var r=this,n=-1;return this.iterator(function(o){t.call(e,o,++n,r)}),this},i.prototype.eachAfter=function(t,e){for(var r=this,n=[r],o=[];r=n.pop();){o.push(r);var a=r.children;if(a)for(var s=0,l=a.length;s<l;++s)n.push(a[s])}for(var u=-1;r=o.pop();)t.call(e,r,++u,this);return this},i.prototype.eachBefore=function(t,e){for(var r=this,n=[r],o=-1;r=n.pop();){t.call(e,r,++o,this);var a=r.children;if(a)for(var s=a.length-1;s>=0;--s){var l=a[s];n.push(l)}}return this},i.prototype.find=function(t,e){var r=this,n=-1,o;return this.iterator(function(a){if(t.call(e,a,++n,r))return o=a,!1}),o},i.prototype.sum=function(t){return this.eachAfter(function(e){var r=+t(e.datum)||0,n=e.children;if(n)for(var o=n.length;--o>=0;)r+=n[o].value;e.value=r})},i.prototype.sort=function(t){return this.eachBefore(function(e){e.children&&e.children.sort(t)})},i.prototype.path=function(t){for(var e=this,r=t7(e,t),n=[e];e!==r;)e=e.parent,n.push(e);for(var o=n.length;t!==r;)n.splice(o,0,t),t=t.parent;return n},i.prototype.ancestors=function(){for(var t=this,e=[t];t=t.parent;)e.push(t);return e},i.prototype.descendants=function(){var t=[];return this.iterator(function(e){return t.push(e)}),t},i.prototype.leaves=function(){var t=[];return this.eachBefore(function(e){e.children||t.push(e)}),t},i.prototype.links=function(){var t=this,e=[];return t.each(function(r){r!==t&&e.push({source:r.parent,target:r})}),e},i.prototype.copy=function(){},i.prototype.iterator=function(t){var e=this,r=[e],n;e:do for(n=r.reverse(),r=[];e=n.pop();){if(t(e)===!1)break e;var o=e.children;if(o)for(var a=0,s=o.length;a<s;++a)r.push(o[a])}while(r.length)},i}();function Ux(i,t){i instanceof Map?(i=[void 0,i],t===void 0&&(t=J9)):t===void 0&&(t=e7);for(var e=new jx(i),r=[e],n,o,a,s,l;n=r.pop();)if((a=t(n.datum))&&(l=(a=Array.from(a)).length))for(n.children=a,s=l-1;s>=0;--s)r.push(o=a[s]=new jx(a[s])),o.parent=n,o.depth=n.depth+1;return e.eachBefore(Q9)}function Q9(i){var t=0;do i.height=t;while((i=i.parent)&&i.height<++t)}function J9(i){return Array.isArray(i)?i[1]:void 0}function e7(i){return i.children}function t7(i,t){if(!!(i&&t)){if(i===t)return i;var e=i.ancestors(),r=t.ancestors(),n=void 0;for(i=e.pop(),t=r.pop();i===t;)n=i,i=e.pop(),t=r.pop();return n}}var lb=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ji=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},r7=function(i){lb(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return ji([$("change")],t.prototype,"renderer",void 0),t}(Os),zx=function(i){lb(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.padding=10,e}return ji([$("change")],t.prototype,"padding",void 0),t}(Gn),yl;(function(i){i[i.Name=0]="Name",i[i.Value=1]="Value"})(yl||(yl={}));var n7=function(i){lb(t,i);function t(){var e=i.call(this)||this;return e.groupSelection=Ht.select(e.pickGroup).selectAll(),e.labelMap=new Map,e.layout=new Z9,e.title=function(){var r=new zx;return r.color="white",r.fontWeight="bold",r.fontSize=12,r.fontFamily="Verdana, sans-serif",r.padding=15,r}(),e.subtitle=function(){var r=new zx;return r.color="white",r.fontSize=9,r.fontFamily="Verdana, sans-serif",r.padding=13,r}(),e.labels={large:function(){var r=new Gn;return r.color="white",r.fontWeight="bold",r.fontSize=18,r}(),medium:function(){var r=new Gn;return r.color="white",r.fontWeight="bold",r.fontSize=14,r}(),small:function(){var r=new Gn;return r.color="white",r.fontWeight="bold",r.fontSize=10,r}(),color:function(){var r=new Gn;return r.color="white",r}()},e._nodePadding=2,e.labelKey="label",e.sizeKey="size",e.colorKey="color",e.colorDomain=[-5,5],e.colorRange=["#cb4b3f","#6acb64"],e.colorParents=!1,e.gradient=!0,e.colorName="Change",e.rootName="Root",e._shadow=function(){var r=new BA;return r.color="rgba(0, 0, 0, 0.4)",r.xOffset=1.5,r.yOffset=1.5,r}(),e.tooltip=new r7,e.shadow.addEventListener("change",e.scheduleUpdate,e),e.title.addEventListener("change",e.scheduleUpdate,e),e.subtitle.addEventListener("change",e.scheduleUpdate,e),e.labels.small.addEventListener("change",e.scheduleUpdate,e),e.labels.medium.addEventListener("change",e.scheduleUpdate,e),e.labels.large.addEventListener("change",e.scheduleUpdate,e),e.labels.color.addEventListener("change",e.scheduleUpdate,e),e}return Object.defineProperty(t.prototype,"nodePadding",{get:function(){return this._nodePadding},set:function(e){this._nodePadding!==e&&(this._nodePadding=e,this.updateLayoutPadding(),this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"shadow",{get:function(){return this._shadow},set:function(e){this._shadow!==e&&(this._shadow=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),t.prototype.onHighlightChange=function(){this.updateNodes()},t.prototype.updateLayoutPadding=function(){var e=this,r=e.title,n=e.subtitle,o=e.nodePadding,a=e.labelKey;this.layout.paddingRight=function(s){return o},this.layout.paddingBottom=function(s){return o},this.layout.paddingLeft=function(s){return o},this.layout.paddingTop=function(s){var l=s.datum[a]||"";s.children&&(l=l.toUpperCase());var u=s.depth>1?n:r,c=ns.getTextSize(l,[u.fontWeight,u.fontSize+"px",u.fontFamily].join(" ").trim()),p=s.x1-s.x0-o*2,h=s.depth>0&&s.children&&c.width<=p;return s.hasTitle=!!h,h?c.height+o*2:o}},t.prototype.processData=function(){if(!this.data)return!1;var e=this,r=e.data,n=e.sizeKey,o=e.labelKey,a=e.colorKey,s=e.colorDomain,l=e.colorRange,u=e.colorParents,c;n?c=Ux(r).sum(function(f){return f.children?1:f[n]}):c=Ux(r).sum(function(f){return f.children?0:1}),this.dataRoot=c;var p=new Nt;p.domain=s,p.range=l;var h=this;function d(f,g){g===void 0&&(g=0);var v=f.children,m=f.datum,C=m[o],w=a?m[a]:g;f.series=h,f.fill=!v||u?p.convert(w):"#272931",f.colorValue=w,C?f.label=v?C.toUpperCase():C:f.label="",v&&v.forEach(function(b){return d(b,g+1)})}return d(this.dataRoot),!0},t.prototype.getLabelCenterX=function(e){return(e.x0+e.x1)/2},t.prototype.getLabelCenterY=function(e){return(e.y0+e.y1)/2+2},t.prototype.update=function(){this.updatePending=!1,this.updateSelections(),this.updateNodes()},t.prototype.updateSelections=function(){if(!!this.nodeDataPending){this.nodeDataPending=!1;var e=this,r=e.chart,n=e.dataRoot;if(!(!r||!n)){var o=r.getSeriesRect();if(!!o){this.layout.size=[o.width,o.height],this.updateLayoutPadding();var a=this.layout.processData(n).descendants(),s=this.groupSelection.setData(a);s.exit.remove();var l=s.enter.append(It);l.append(fo),l.append(Pr).each(function(u){return u.tag=yl.Name}),l.append(Pr).each(function(u){return u.tag=yl.Value}),this.groupSelection=s.merge(l)}}}},t.prototype.updateNodes=function(){var e=this;if(!!this.chart){var r=this,n=r.colorKey,o=r.labelMap,a=r.nodePadding,s=r.title,l=r.subtitle,u=r.labels,c=r.shadow,p=r.gradient,h=r.chart.highlightedDatum,d=r.highlightStyle,f=d.fill,g=d.stroke,v=d.strokeWidth,m=d.item,C=m.fill,w=C===void 0?f:C,b=m.stroke,S=b===void 0?g:b,T=m.strokeWidth,E=T===void 0?v:T;this.groupSelection.selectByClass(fo).each(function(P,R){var M=R===h,N=M&&w!==void 0?w:R.fill,W=M&&S!==void 0?S:R.depth<2?void 0:"black",G=M&&E!==void 0?E:e.getStrokeWidth(1,R);P.fill=N,P.stroke=W,P.strokeWidth=G,P.crisp=!0,P.gradient=p,P.x=R.x0,P.y=R.y0,P.width=R.x1-R.x0,P.height=R.y1-R.y0}),this.groupSelection.selectByTag(yl.Name).each(function(P,R,M){var N=!R.children,W=R.x1-R.x0-a*2,G=R.y1-R.y0-a*2,k=R.hasTitle,V=R===h,U;N?W>40&&G>40?U=u.large:W>20&&G>20?U=u.medium:U=u.small:R.depth>1?U=l:U=s,P.fontWeight=U.fontWeight,P.fontSize=U.fontSize,P.fontFamily=U.fontFamily,P.textBaseline=N?"bottom":k?"top":"middle",P.textAlign=k?"left":"center",P.text=R.label;var J=P.computeBBox(),Z=N&&!!J&&J.width<=W&&J.height*2+8<=G;o.set(M,P),P.fill=V?"black":U.color,P.fillShadow=Z&&!V?c:void 0,P.visible=k||Z,k?(P.x=R.x0+a,P.y=R.y0+a):(P.x=e.getLabelCenterX(R),P.y=e.getLabelCenterY(R))}),this.groupSelection.selectByTag(yl.Value).each(function(P,R,M){var N=!R.children,W=R.x1-R.x0-a*2,G=R===h,k=R.colorValue,V=u.color;P.fontSize=V.fontSize,P.fontFamily=V.fontFamily,P.fontStyle=V.fontStyle,P.fontWeight=V.fontWeight,P.textBaseline="top",P.textAlign="center",P.text=typeof k=="number"&&isFinite(k)?String(Dy(R.colorValue))+"%":"";var U=P.computeBBox(),J=o.get(M),Z=!!J&&J.visible,le=N&&!!n&&Z&&!!U&&U.width<W;P.fill=G?"black":V.color,P.fillShadow=G?void 0:c,P.visible=le,le?(P.x=e.getLabelCenterX(R),P.y=e.getLabelCenterY(R)):J&&!(R.children&&R.children.length)&&(J.textBaseline="middle",J.y=e.getLabelCenterY(R))})}},t.prototype.getDomain=function(e){return[0,1]},t.prototype.fireNodeClickEvent=function(e,r){this.fireEvent({type:"nodeClick",event:e,series:this,datum:r.datum,labelKey:this.labelKey,sizeKey:this.sizeKey,colorKey:this.colorKey})},t.prototype.getTooltipHtml=function(e){var r=this,n=r.tooltip,o=r.sizeKey,a=r.labelKey,s=r.colorKey,l=r.colorName,u=r.rootName,c=e.datum,p=n.renderer,h=e.depth?c[a]:u||c[a],d=void 0,f=e.fill||"gray";if(s&&l){var g=c[s];typeof g=="number"&&isFinite(g)&&(d="<b>"+l+"</b>: "+Dy(c[s]))}var v={title:h,backgroundColor:f,content:d};return p?br(p({datum:e,sizeKey:o,labelKey:a,colorKey:s,title:h,color:f}),v):br(v)},t.prototype.listSeriesItems=function(e){},t.className="TreemapSeries",t.type="treemap",ji([$("dataChange")],t.prototype,"labelKey",void 0),ji([$("dataChange")],t.prototype,"sizeKey",void 0),ji([$("dataChange")],t.prototype,"colorKey",void 0),ji([$("dataChange")],t.prototype,"colorDomain",void 0),ji([$("dataChange")],t.prototype,"colorRange",void 0),ji([$("dataChange")],t.prototype,"colorParents",void 0),ji([$("update")],t.prototype,"gradient",void 0),t}(j9),i7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),HA=function(i){i7(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._centerX=0,e._centerY=0,e._centerOffset=0,e._innerRadius=10,e._outerRadius=20,e._startAngle=0,e._endAngle=Math.PI*2,e._angleOffset=0,e}return Object.defineProperty(t.prototype,"centerX",{get:function(){return this._centerX},set:function(e){this._centerX!==e&&(this._centerX=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"centerY",{get:function(){return this._centerY},set:function(e){this._centerY!==e&&(this._centerY=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"centerOffset",{get:function(){return this._centerOffset},set:function(e){this._centerOffset!==e&&(this._centerOffset=Math.max(0,e),this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"innerRadius",{get:function(){return this._innerRadius},set:function(e){this._innerRadius!==e&&(this._innerRadius=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"outerRadius",{get:function(){return this._outerRadius},set:function(e){this._outerRadius!==e&&(this._outerRadius=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"startAngle",{get:function(){return this._startAngle},set:function(e){this._startAngle!==e&&(this._startAngle=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"endAngle",{get:function(){return this._endAngle},set:function(e){this._endAngle!==e&&(this._endAngle=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"angleOffset",{get:function(){return this._angleOffset},set:function(e){this._angleOffset!==e&&(this._angleOffset=e,this.dirtyPath=!0)},enumerable:!0,configurable:!0}),t.prototype.computeBBox=function(){var e=this.outerRadius;return new Lt(this.centerX-e,this.centerY-e,e*2,e*2)},t.prototype.isFullPie=function(){return fA(Mn(this.startAngle),Mn(this.endAngle))},t.prototype.updatePath=function(){var e=this.path,r=this.angleOffset,n=Math.min(this.startAngle,this.endAngle)+r,o=Math.max(this.startAngle,this.endAngle)+r,a=(n+o)*.5,s=Math.min(this.innerRadius,this.outerRadius),l=Math.max(this.innerRadius,this.outerRadius),u=this.centerOffset,c=this.isFullPie(),p=this.centerX,h=this.centerY;if(e.clear(),u&&(p+=u*Math.cos(a),h+=u*Math.sin(a)),c||(e.moveTo(p+s*Math.cos(n),h+s*Math.sin(n)),e.lineTo(p+l*Math.cos(n),h+l*Math.sin(n))),e.cubicArc(p,h,l,l,0,n,o,0),c)e.moveTo(p+s*Math.cos(o),h+s*Math.sin(o));else{var d=p+s*Math.cos(o);e.lineTo(Math.abs(d)<1e-8?0:d,h+s*Math.sin(o))}e.cubicArc(p,h,s,s,0,o,n,1),e.closePath(),this.dirtyPath=!1},t.className="Sector",t}($r),Jl=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),gt=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},o7=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},$x=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(o7(arguments[t]));return i},a7=function(i){Jl(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t}(DA),Go;(function(i){i[i.Sector=0]="Sector",i[i.Callout=1]="Callout",i[i.Label=2]="Label"})(Go||(Go={}));var s7=function(i){Jl(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.offset=3,e.minAngle=20,e}return gt([$("change")],t.prototype,"offset",void 0),gt([$("dataChange")],t.prototype,"minAngle",void 0),gt([$("dataChange")],t.prototype,"formatter",void 0),t}(Gn),l7=function(i){Jl(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.colors=["#874349","#718661","#a48f5f","#5a7088","#7f637a","#5d8692"],e.length=10,e.strokeWidth=1,e}return gt([$("change")],t.prototype,"colors",void 0),gt([$("change")],t.prototype,"length",void 0),gt([$("change")],t.prototype,"strokeWidth",void 0),t}(Ir),u7=function(i){Jl(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return gt([$("change")],t.prototype,"renderer",void 0),t}(Os),c7=function(i){Jl(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.showInLegend=!1,e}return gt([$()],t.prototype,"showInLegend",void 0),t}(xy),jA=function(i){Jl(t,i);function t(){var e=i.call(this)||this;return e.radiusScale=new Nt,e.groupSelection=Ht.select(e.pickGroup).selectAll(),e.groupSelectionData=[],e.angleScale=function(){var r=new Nt;return r.domain=[0,1],r.range=[-Math.PI,Math.PI].map(function(n){return n+Math.PI/2}),r}(),e.seriesItemEnabled=[],e.label=new s7,e.callout=new l7,e.tooltip=new u7,e.angleKey="",e.angleName="",e._fills=["#c16068","#a2bf8a","#ebcc87","#80a0c3","#b58dae","#85c0d1"],e._strokes=["#874349","#718661","#a48f5f","#5a7088","#7f637a","#5d8692"],e.fillOpacity=1,e.strokeOpacity=1,e.lineDash=[0],e.lineDashOffset=0,e.rotation=0,e.outerRadiusOffset=0,e.innerRadiusOffset=0,e.strokeWidth=1,e.highlightStyle=new a7,e.addEventListener("update",e.scheduleUpdate,e),e.label.addEventListener("change",e.scheduleUpdate,e),e.label.addEventListener("dataChange",e.scheduleData,e),e.callout.addEventListener("change",e.scheduleLayout,e),e.addPropertyListener("data",function(r){r.value&&(r.source.seriesItemEnabled=r.value.map(function(){return!0}))}),e}return Object.defineProperty(t.prototype,"title",{get:function(){return this._title},set:function(e){var r=this._title;function n(){this.fireEvent({type:"legendChange"})}r!==e&&(r&&(r.removeEventListener("change",this.scheduleUpdate,this),r.removePropertyListener("showInLegend",n,this),this.group.removeChild(r.node)),e&&(e.node.textBaseline="bottom",e.addEventListener("change",this.scheduleUpdate,this),e.addPropertyListener("showInLegend",n,this),this.group.appendChild(e.node)),this._title=e,this.scheduleUpdate())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fills",{get:function(){return this._fills},set:function(e){this._fills=e,this.scheduleUpdate()},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokes",{get:function(){return this._strokes},set:function(e){this._strokes=e,this.scheduleUpdate()},enumerable:!0,configurable:!0}),t.prototype.onHighlightChange=function(){this.updateNodes()},t.prototype.setColors=function(e,r){this.fills=e,this.strokes=r,this.callout.colors=r},t.prototype.getDomain=function(e){return e===tt.X?this.angleScale.domain:this.radiusScale.domain},t.prototype.processData=function(){var e=this,r=this,n=r.angleKey,o=r.radiusKey,a=r.seriesItemEnabled,s=r.angleScale,l=r.groupSelectionData,u=r.label,c=n&&this.data?this.data:[],p=c.map(function(k,V){return a[V]&&Math.abs(+k[n])||0}),h=p.reduce(function(k,V){return k+V},0),d=function(){var k=0;return p.map(function(V){return k+=V/h})}(),f=u.formatter,g=u.enabled&&this.labelKey,v=[],m=[];if(g&&(f?v=c.map(function(k){return f({value:k[g]})}):v=c.map(function(k){return String(k[g])})),o){var C=this,w=C.radiusMin,b=C.radiusMax,S=c.map(function(k){return Math.abs(k[o])}),T=w!==void 0?w:Math.min.apply(Math,$x(S)),E=b!==void 0?b:Math.max.apply(Math,$x(S)),P=E-T;m=S.map(function(k){return P?(k-T)/P:1})}l.length=0;var R=Mt(this.rotation),M=Math.PI/2,N=0,W=[{textAlign:"center",textBaseline:"bottom"},{textAlign:"left",textBaseline:"middle"},{textAlign:"center",textBaseline:"hanging"},{textAlign:"right",textBaseline:"middle"}],G=0;return d.forEach(function(k){if(!isNaN(k)){var V=o?m[N]:1,U=s.convert(k)+R,J=s.convert(G)+R,Z=(U+J)/2,le=Math.abs(J-U),Y=Math.cos(Z),ae=Math.sin(Z),z=Mt(u.minAngle),X=g&&le>z,ue=u6(Z),pe=-3*Math.PI/4,be=ue-pe,_e=Math.floor(be/M),Te=c6(_e,W.length),Pe=W[Te],Ee=Pe.textAlign,Le=Pe.textBaseline;l.push({series:e,datum:c[N],itemId:N,index:N,radius:V,startAngle:U,endAngle:J,midAngle:Z,midCos:Y,midSin:ae,label:X?{text:v[N],textAlign:Ee,textBaseline:Le}:void 0}),N++,G=k}}),!0},t.prototype.update=function(){this.updatePending=!1;var e=this,r=e.radius,n=e.innerRadiusOffset,o=e.outerRadiusOffset,a=e.title;if(this.radiusScale.range=[n?r+n:0,r+(o||0)],this.group.translationX=this.centerX,this.group.translationY=this.centerY,a){var s=Math.max(0,this.radiusScale.range[1]);s===0?a.node.visible=!1:(a.node.translationY=-r-o-2,a.node.visible=a.enabled)}this.updateSelections(),this.updateNodes()},t.prototype.updateSelections=function(){!this.nodeDataPending||(this.nodeDataPending=!1,this.updateGroupSelection())},t.prototype.updateGroupSelection=function(){var e=this.groupSelection.setData(this.groupSelectionData);e.exit.remove();var r=e.enter.append(It);r.append(HA).each(function(n){return n.tag=Go.Sector}),r.append(ri).each(function(n){n.tag=Go.Callout,n.pointerEvents=Wn.None}),r.append(Pr).each(function(n){n.tag=Go.Label,n.pointerEvents=Wn.None}),this.groupSelection=e.merge(r)},t.prototype.updateNodes=function(){var e=this;if(!!this.chart){this.group.visible=this.visible&&this.seriesItemEnabled.indexOf(!0)>=0;var r=this,n=r.fills,o=r.strokes,a=r.fillOpacity,s=r.strokeOpacity,l=r.radiusScale,u=r.callout,c=r.shadow,p=r.chart.highlightedDatum,h=r.highlightStyle,d=h.fill,f=h.stroke,g=h.strokeWidth,v=h.item,m=v.fill,C=m===void 0?d:m,w=v.stroke,b=w===void 0?f:w,S=v.strokeWidth,T=S===void 0?g:S,E=r.angleKey,P=r.radiusKey,R=r.formatter,M=[],N=l.convert(0);this.groupSelection.selectByTag(Go.Sector).each(function(z,X,ue){var pe=l.convert(X.radius),be=!!p&&p.series===e&&X.itemId===p.itemId,_e=be&&C!==void 0?C:n[ue%n.length],Te=be&&b!==void 0?b:o[ue%o.length],Pe=be&&T!==void 0?T:e.getStrokeWidth(e.strokeWidth),Ee=void 0;R&&(Ee=R({datum:X.datum,angleKey:E,radiusKey:P,fill:_e,stroke:Te,strokeWidth:Pe,highlighted:be}));var Le=z.parent&&z.parent.parent;be&&Le&&(Le.removeChild(z.parent),Le.appendChild(z.parent)),z.innerRadius=Math.max(0,N),z.outerRadius=Math.max(0,pe),z.startAngle=X.startAngle,z.endAngle=X.endAngle,z.fill=Ee&&Ee.fill||_e,z.stroke=Ee&&Ee.stroke||Te,z.strokeWidth=Ee&&Ee.strokeWidth!==void 0?Ee.strokeWidth:Pe,z.fillOpacity=a,z.strokeOpacity=s,z.lineDash=e.lineDash,z.lineDashOffset=e.lineDashOffset,z.fillShadow=c,z.lineJoin="round",z.opacity=e.getOpacity(),M.push(z.centerOffset)});var W=u.colors,G=u.length,k=u.strokeWidth;this.groupSelection.selectByTag(Go.Callout).each(function(z,X,ue){var pe=l.convert(X.radius),be=Math.max(0,pe);X.label&&be!==0?(z.strokeWidth=k,z.stroke=W[ue%W.length],z.x1=X.midCos*be,z.y1=X.midSin*be,z.x2=X.midCos*(be+G),z.y2=X.midSin*(be+G)):z.stroke=void 0});{var V=this.label,U=V.offset,J=V.fontStyle,Z=V.fontWeight,le=V.fontSize,Y=V.fontFamily,ae=V.color;this.groupSelection.selectByTag(Go.Label).each(function(z,X,ue){var pe=X.label,be=l.convert(X.radius),_e=Math.max(0,be);if(pe&&_e!==0){var Te=M[ue]+_e+G+U;z.fontStyle=J,z.fontWeight=Z,z.fontSize=le,z.fontFamily=Y,z.text=pe.text,z.x=X.midCos*Te,z.y=X.midSin*Te,z.fill=ae,z.textAlign=pe.textAlign,z.textBaseline=pe.textBaseline}else z.fill=void 0})}}},t.prototype.fireNodeClickEvent=function(e,r){this.fireEvent({type:"nodeClick",event:e,series:this,datum:r.datum,angleKey:this.angleKey,labelKey:this.labelKey,radiusKey:this.radiusKey})},t.prototype.getTooltipHtml=function(e){var r=this.angleKey;if(!r)return"";var n=this,o=n.fills,a=n.tooltip,s=n.angleName,l=n.radiusKey,u=n.radiusName,c=n.labelKey,p=n.labelName,h=a.renderer,d=o[e.index%o.length],f=e.datum,g=c?f[c]+": ":"",v=f[r],m=typeof v=="number"?Dy(v):v.toString(),C=this.title?this.title.text:void 0,w=g+m,b={title:C,backgroundColor:d,content:w};return h?br(h({datum:f,angleKey:r,angleValue:v,angleName:s,radiusKey:l,radiusValue:l?f[l]:void 0,radiusName:u,labelKey:c,labelName:p,title:C,color:d}),b):br(b)},t.prototype.listSeriesItems=function(e){var r=this,n=this,o=n.labelKey,a=n.data;if(a&&a.length&&o){var s=this,l=s.fills,u=s.strokes,c=s.id,p=this.title&&this.title.showInLegend&&this.title.text;a.forEach(function(h,d){var f=[];p&&f.push(p),f.push(String(h[o])),e.push({id:c,itemId:d,enabled:r.seriesItemEnabled[d],label:{text:f.join(" - ")},marker:{fill:l[d%l.length],stroke:u[d%u.length],fillOpacity:r.fillOpacity,strokeOpacity:r.strokeOpacity}})})}},t.prototype.toggleSeriesItem=function(e,r){this.seriesItemEnabled[e]=r,this.scheduleData()},t.className="PieSeries",t.type="pie",gt([$("dataChange")],t.prototype,"angleKey",void 0),gt([$("update")],t.prototype,"angleName",void 0),gt([$("dataChange")],t.prototype,"radiusKey",void 0),gt([$("update")],t.prototype,"radiusName",void 0),gt([$("dataChange")],t.prototype,"radiusMin",void 0),gt([$("dataChange")],t.prototype,"radiusMax",void 0),gt([$("dataChange")],t.prototype,"labelKey",void 0),gt([$("update")],t.prototype,"labelName",void 0),gt([$("layoutChange")],t.prototype,"fillOpacity",void 0),gt([$("layoutChange")],t.prototype,"strokeOpacity",void 0),gt([$("update")],t.prototype,"lineDash",void 0),gt([$("update")],t.prototype,"lineDashOffset",void 0),gt([$("update")],t.prototype,"formatter",void 0),gt([$("dataChange")],t.prototype,"rotation",void 0),gt([$("layoutChange")],t.prototype,"outerRadiusOffset",void 0),gt([$("dataChange")],t.prototype,"innerRadiusOffset",void 0),gt([$("layoutChange")],t.prototype,"strokeWidth",void 0),gt([$("layoutChange")],t.prototype,"shadow",void 0),t}(IA);function p7(i){i.setUTCSeconds(0,0)}function h7(i,t){i.setTime(i.getTime()+t*Nn)}function d7(i,t){return(t.getTime()-i.getTime())/Nn}function f7(i){return i.getUTCMinutes()}new Kr(p7,h7,d7,f7);function g7(i){i.setUTCMinutes(0,0,0)}function v7(i,t){i.setTime(i.getTime()+t*Ji)}function m7(i,t){return(t.getTime()-i.getTime())/Ji}function y7(i){return i.getUTCHours()}new Kr(g7,v7,m7,y7);function C7(i){i.setUTCDate(1),i.setUTCHours(0,0,0,0)}function w7(i,t){i.setUTCMonth(i.getUTCMonth()+t)}function b7(i,t){return t.getUTCMonth()-i.getUTCMonth()+(t.getUTCFullYear()-i.getUTCFullYear())*12}function _7(i){return i.getUTCMonth()}new Kr(C7,w7,b7,_7);var Ie=globalThis&&globalThis.__assign||function(){return Ie=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Ie.apply(this,arguments)},S7=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},x7={fills:["#f3622d","#fba71b","#57b757","#41a9c9","#4258c9","#9a42c8","#c84164","#888888"],strokes:["#aa4520","#b07513","#3d803d","#2d768d","#2e3e8d","#6c2e8c","#8c2d46","#5f5f5f"]};function T7(i,t,e){return t}function O7(i){return aA(i)&&!(i instanceof SA)}var Bi={arrayMerge:T7,isMergeableObject:O7},Kx="bold",E7="inside",R7="right",Es=function(){function i(t){t=On({},t||{},Bi);var e=t.overrides,r=e===void 0?null:e,n=t.palette,o=n===void 0?null:n,a=this.createChartConfigPerSeries(this.getDefaults());if(r){var s=r.common,l=r.cartesian,u=r.polar,c=r.hierarchy,p=function(d,f,g){g&&(a[d]=On(a[d],g,Bi),f.forEach(function(v){a[v]=On(a[v],g,Bi)}))};p("common",Object.keys(a),s),p("cartesian",i.cartesianSeriesTypes,l),p("polar",i.polarSeriesTypes,u),p("hierarchy",i.hierarchySeriesTypes,c);var h={};i.seriesTypes.forEach(function(d){var f=r[d];f&&(f.series&&(h[d]=f.series,f.series=h),a[d]=On(a[d],f,Bi))})}this.palette=o!=null?o:this.getPalette(),this.config=Object.freeze(a)}return i.prototype.getPalette=function(){return x7},i.getAxisDefaults=function(){return{top:{},right:{},bottom:{},left:{},thickness:0,title:{enabled:!1,padding:{top:10,right:10,bottom:10,left:10},text:"Axis Title",fontStyle:void 0,fontWeight:Kx,fontSize:12,fontFamily:this.fontFamily,color:"rgb(70, 70, 70)"},label:{fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:this.fontFamily,padding:5,rotation:void 0,color:"rgb(87, 87, 87)",formatter:void 0,autoRotate:!1},line:{width:1,color:"rgb(195, 195, 195)"},tick:{width:1,size:6,color:"rgb(195, 195, 195)"},gridStyle:[{stroke:"rgb(219, 219, 219)",lineDash:[4,2]}]}},i.getSeriesDefaults=function(){return{tooltip:{enabled:!0,renderer:void 0},visible:!0,showInLegend:!0,cursor:"default",highlightStyle:{item:{fill:"yellow"},series:{dimOpacity:1}}}},i.getBarSeriesDefaults=function(){return Ie(Ie({},this.getSeriesDefaults()),{flipXY:!1,fillOpacity:1,strokeOpacity:1,xKey:"",xName:"",yKeys:[],yNames:[],grouped:!1,normalizedTo:void 0,strokeWidth:1,lineDash:[0],lineDashOffset:0,label:{enabled:!1,fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:this.fontFamily,color:"rgb(70, 70, 70)",formatter:void 0,placement:E7},shadow:{enabled:!1,color:"rgba(0, 0, 0, 0.5)",xOffset:3,yOffset:3,blur:5}})},i.getLineSeriesDefaults=function(){var t=this.getSeriesDefaults();return Ie(Ie({},t),{tooltip:Ie(Ie({},t.tooltip),{format:void 0})})},i.getCartesianSeriesMarkerDefaults=function(){return{enabled:!0,shape:"circle",size:6,maxSize:30,strokeWidth:1,formatter:void 0}},i.getChartDefaults=function(){return{background:{visible:!0,fill:"white"},padding:{top:20,right:20,bottom:20,left:20},title:{enabled:!1,padding:{top:10,right:10,bottom:10,left:10},text:"Title",fontStyle:void 0,fontWeight:Kx,fontSize:16,fontFamily:this.fontFamily,color:"rgb(70, 70, 70)"},subtitle:{enabled:!1,padding:{top:10,right:10,bottom:10,left:10},text:"Subtitle",fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:this.fontFamily,color:"rgb(140, 140, 140)"},legend:{enabled:!0,position:R7,spacing:20,item:{paddingX:16,paddingY:8,marker:{shape:void 0,size:15,strokeWidth:1,padding:8},label:{color:"black",fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:this.fontFamily,formatter:void 0}}},tooltip:{enabled:!0,tracking:!0,delay:0,class:An.defaultTooltipClass}}},i.prototype.createChartConfigPerSeries=function(t){var e={cartesian:i.cartesianSeriesTypes,polar:i.polarSeriesTypes,hierarchy:i.hierarchySeriesTypes,groupedCategory:[]};return Object.entries(e).forEach(function(r){var n=S7(r,2),o=n[0],a=n[1];a.forEach(function(s){t[s]||(t[s]=On({},t[o],Bi))})}),t},i.prototype.getConfig=function(t,e){var r=o8(this.config,t,e);return Array.isArray(r)?On([],r,Bi):f8(r)?On({},r,Bi):r},i.prototype.getDefaults=function(){return On({},i.defaults,Bi)},i.prototype.mergeWithParentDefaults=function(t,e){return On(t,e,Bi)},i.fontFamily="Verdana, sans-serif",i.cartesianDefaults=Ie(Ie({},i.getChartDefaults()),{axes:{number:Ie({},i.getAxisDefaults()),log:Ie(Ie({},i.getAxisDefaults()),{base:10}),category:Ie(Ie({},i.getAxisDefaults()),{label:Ie(Ie({},i.getAxisDefaults().label),{autoRotate:!0})}),groupedCategory:Ie({},i.getAxisDefaults()),time:Ie({},i.getAxisDefaults())},series:{column:Ie(Ie({},i.getBarSeriesDefaults()),{flipXY:!1}),bar:Ie(Ie({},i.getBarSeriesDefaults()),{flipXY:!0}),line:Ie(Ie({},i.getLineSeriesDefaults()),{title:void 0,xKey:"",xName:"",yKey:"",yName:"",strokeWidth:2,strokeOpacity:1,lineDash:[0],lineDashOffset:0,marker:Ie({},i.getCartesianSeriesMarkerDefaults()),label:{enabled:!1,fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:i.fontFamily,color:"rgb(70, 70, 70)",formatter:void 0}}),scatter:Ie(Ie({},i.getSeriesDefaults()),{title:void 0,xKey:"",yKey:"",sizeKey:void 0,labelKey:void 0,xName:"",yName:"",sizeName:"Size",labelName:"Label",strokeWidth:2,fillOpacity:1,strokeOpacity:1,marker:Ie({},i.getCartesianSeriesMarkerDefaults()),label:{enabled:!1,fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:i.fontFamily,color:"rgb(70, 70, 70)"}}),area:Ie(Ie({},i.getSeriesDefaults()),{xKey:"",xName:"",yKeys:[],yNames:[],normalizedTo:void 0,fillOpacity:.8,strokeOpacity:1,strokeWidth:2,lineDash:[0],lineDashOffset:0,shadow:{enabled:!1,color:"rgba(0, 0, 0, 0.5)",xOffset:3,yOffset:3,blur:5},marker:Ie(Ie({},i.getCartesianSeriesMarkerDefaults()),{enabled:!1}),label:{enabled:!1,fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:i.fontFamily,color:"rgb(70, 70, 70)",formatter:void 0}}),histogram:Ie(Ie({},i.getSeriesDefaults()),{xKey:"",yKey:"",xName:"",yName:"",strokeWidth:1,fillOpacity:1,strokeOpacity:1,lineDash:[0],lineDashOffset:0,areaPlot:!1,binCount:10,bins:void 0,aggregation:"sum",label:{enabled:!1,fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:i.fontFamily,color:"rgb(70, 70, 70)",formatter:void 0},shadow:{enabled:!0,color:"rgba(0, 0, 0, 0.5)",xOffset:0,yOffset:0,blur:5}})},navigator:{enabled:!1,height:30,mask:{fill:"#999999",stroke:"#999999",strokeWidth:1,fillOpacity:.2},minHandle:{fill:"#f2f2f2",stroke:"#999999",strokeWidth:1,width:8,height:16,gripLineGap:2,gripLineLength:8},maxHandle:{fill:"#f2f2f2",stroke:"#999999",strokeWidth:1,width:8,height:16,gripLineGap:2,gripLineLength:8}}}),i.polarDefaults=Ie(Ie({},i.getChartDefaults()),{series:{pie:Ie(Ie({},i.getSeriesDefaults()),{title:{enabled:!0,padding:{top:10,right:10,bottom:10,left:10},text:"",fontStyle:void 0,fontWeight:"bold",fontSize:14,fontFamily:i.fontFamily,color:"rgb(70, 70, 70)"},angleKey:"",angleName:"",radiusKey:void 0,radiusName:void 0,labelKey:void 0,labelName:void 0,label:{enabled:!0,fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:i.fontFamily,color:"rgb(70, 70, 70)",offset:3,minAngle:20},callout:{length:10,strokeWidth:2},fillOpacity:1,strokeOpacity:1,strokeWidth:1,lineDash:[0],lineDashOffset:0,rotation:0,outerRadiusOffset:0,innerRadiusOffset:0,shadow:{enabled:!1,color:"rgba(0, 0, 0, 0.5)",xOffset:3,yOffset:3,blur:5}})}}),i.hierarchyDefaults=Ie(Ie({},i.getChartDefaults()),{series:{treemap:Ie(Ie({},i.getSeriesDefaults()),{showInLegend:!1,labelKey:"label",sizeKey:"size",colorKey:"color",colorDomain:[-5,5],colorRange:["#cb4b3f","#6acb64"],colorParents:!1,gradient:!0,nodePadding:2,title:{enabled:!0,color:"white",fontStyle:void 0,fontWeight:"bold",fontSize:12,fontFamily:"Verdana, sans-serif",padding:15},subtitle:{enabled:!0,color:"white",fontStyle:void 0,fontWeight:void 0,fontSize:9,fontFamily:"Verdana, sans-serif",padding:13},labels:{large:{enabled:!0,fontStyle:void 0,fontWeight:"bold",fontSize:18,fontFamily:"Verdana, sans-serif",color:"white"},medium:{enabled:!0,fontStyle:void 0,fontWeight:"bold",fontSize:14,fontFamily:"Verdana, sans-serif",color:"white"},small:{enabled:!0,fontStyle:void 0,fontWeight:"bold",fontSize:10,fontFamily:"Verdana, sans-serif",color:"white"},color:{enabled:!0,fontStyle:void 0,fontWeight:void 0,fontSize:12,fontFamily:"Verdana, sans-serif",color:"white"}}})}}),i.defaults={cartesian:i.cartesianDefaults,groupedCategory:i.cartesianDefaults,polar:i.polarDefaults,hierarchy:i.hierarchyDefaults},i.cartesianSeriesTypes=["line","area","bar","column","scatter","histogram"],i.polarSeriesTypes=["pie"],i.hierarchySeriesTypes=["treemap"],i.seriesTypes=i.cartesianSeriesTypes.concat(i.polarSeriesTypes).concat(i.hierarchySeriesTypes),i}(),P7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),xt=globalThis&&globalThis.__assign||function(){return xt=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},xt.apply(this,arguments)},ap=function(i){P7(t,i);function t(e){return i.call(this,e)||this}return t.prototype.getDefaults=function(){var e="rgb(200, 200, 200)",r="rgb(150, 150, 150)",n={title:{color:e},label:{color:e},gridStyle:[{stroke:"rgb(88, 88, 88)",lineDash:[4,2]}]},o={label:{color:e}},a={axes:{number:xt({},n),category:xt({},n),time:xt({},n)}},s={background:{fill:"rgb(34, 38, 41)"},title:{color:e},subtitle:{color:r},legend:{item:{label:{color:e}}}};return this.mergeWithParentDefaults(i.prototype.getDefaults.call(this),{cartesian:xt(xt(xt({},s),a),{series:{bar:xt({},o),column:xt({},o),histogram:xt({},o)}}),groupedCategory:xt(xt(xt({},s),a),{series:{bar:xt({},o),column:xt({},o),histogram:xt({},o)}}),polar:xt(xt({},s),{series:{pie:xt(xt({},o),{title:{color:e}})}}),hierarchy:xt(xt({},s),{series:{treemap:{title:{color:e},subtitle:{color:r},labels:{large:{color:e},medium:{color:e},small:{color:e},color:{color:e}}}}})})},t}(Es),A7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),D7={fills:["#f44336","#e91e63","#9c27b0","#673ab7","#3f51b5","#2196f3","#03a9f4","#00bcd4","#009688","#4caf50","#8bc34a","#cddc39","#ffeb3b","#ffc107","#ff9800","#ff5722"],strokes:["#ab2f26","#a31545","#6d1b7b","#482980","#2c397f","#1769aa","#0276ab","#008494","#00695f","#357a38","#618834","#909a28","#b3a429","#b38705","#b36a00","#b33d18"]},M7=function(i){A7(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getPalette=function(){return D7},t}(Es),F7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),I7={fills:["#f44336","#e91e63","#9c27b0","#673ab7","#3f51b5","#2196f3","#03a9f4","#00bcd4","#009688","#4caf50","#8bc34a","#cddc39","#ffeb3b","#ffc107","#ff9800","#ff5722"],strokes:["#ab2f26","#a31545","#6d1b7b","#482980","#2c397f","#1769aa","#0276ab","#008494","#00695f","#357a38","#618834","#909a28","#b3a429","#b38705","#b36a00","#b33d18"]},L7=function(i){F7(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getPalette=function(){return I7},t}(ap),N7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),G7={fills:["#c16068","#a2bf8a","#ebcc87","#80a0c3","#b58dae","#85c0d1"],strokes:["#874349","#718661","#a48f5f","#5a7088","#7f637a","#5d8692"]},k7=function(i){N7(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getPalette=function(){return G7},t}(Es),W7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),V7={fills:["#c16068","#a2bf8a","#ebcc87","#80a0c3","#b58dae","#85c0d1"],strokes:["#874349","#718661","#a48f5f","#5a7088","#7f637a","#5d8692"]},B7=function(i){W7(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getPalette=function(){return V7},t}(ap),H7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),j7={fills:["#febe76","#ff7979","#badc58","#f9ca23","#f0932b","#eb4c4b","#6ab04c","#7ed6df","#e056fd","#686de0"],strokes:["#b28553","#b35555","#829a3e","#ae8d19","#a8671e","#a43535","#4a7b35","#58969c","#9d3cb1","#494c9d"]},U7=function(i){H7(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getPalette=function(){return j7},t}(Es),z7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),$7={fills:["#febe76","#ff7979","#badc58","#f9ca23","#f0932b","#eb4c4b","#6ab04c","#7ed6df","#e056fd","#686de0"],strokes:["#b28553","#b35555","#829a3e","#ae8d19","#a8671e","#a43535","#4a7b35","#58969c","#9d3cb1","#494c9d"]},K7=function(i){z7(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getPalette=function(){return $7},t}(ap),Y7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),X7={fills:["#5BC0EB","#FDE74C","#9BC53D","#E55934","#FA7921","#fa3081"],strokes:["#4086a4","#b1a235","#6c8a2b","#a03e24","#af5517","#af225a"]},q7=function(i){Y7(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getPalette=function(){return X7},t}(Es),Z7=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Q7={fills:["#5BC0EB","#FDE74C","#9BC53D","#E55934","#FA7921","#fa3081"],strokes:["#4086a4","#b1a235","#6c8a2b","#a03e24","#af5517","#af225a"]},J7=function(i){Z7(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.getPalette=function(){return Q7},t}(ap),Fd=globalThis&&globalThis.__assign||function(){return Fd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Fd.apply(this,arguments)},Nh=new Es,Vv=new ap,eK={undefined:Nh,null:Nh,"ag-default":Nh,"ag-material":new M7,"ag-pastel":new k7,"ag-solar":new U7,"ag-vivid":new q7},tK={undefined:Vv,null:Vv,"ag-default-dark":Vv,"ag-material-dark":new L7,"ag-pastel-dark":new B7,"ag-solar-dark":new K7,"ag-vivid-dark":new J7},UA=Fd(Fd({},tK),eK);function Ol(i){if(i instanceof Es)return i;var t=UA[i];if(t)return t;if(i=i,i.baseTheme||i.overrides||i.palette){var e=Ol(i.baseTheme);return new e.constructor(i)}return Nh}var rK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Bv=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Yx=function(i){return i},nK=function(i){rK(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.type="log",e._domain=[1,10],e.baseLog=Yx,e.basePow=Yx,e._base=10,e}return t.prototype.setDomain=function(e){var r=e[0],n=e[e.length-1];(r===0||n===0||r<0&&n>0||r>0&&n<0)&&(console.warn("Log scale domain should not start at, end at or cross zero."),r===0&&n>0?r=Number.EPSILON:n===0&&r<0?n=-Number.EPSILON:r<0&&n>0?Math.abs(n)>=Math.abs(r)?r=Number.EPSILON:n=-Number.EPSILON:r>0&&n<0&&(Math.abs(n)>=Math.abs(r)?r=-Number.EPSILON:n=Number.EPSILON),e=e.slice(),e[0]=r,e[e.length-1]=n),i.prototype.setDomain.call(this,e)},t.prototype.getDomain=function(){return i.prototype.getDomain.call(this)},Object.defineProperty(t.prototype,"base",{get:function(){return this._base},set:function(e){this._base!==e&&(this._base=e,this.rescale())},enumerable:!0,configurable:!0}),t.prototype.rescale=function(){var e=this.base,r=t.makeLogFn(e),n=t.makePowFn(e);this.domain[0]<0?(r=this.reflect(r),n=this.reflect(n),this.transform=function(o){return-Math.log(-o)},this.untransform=function(o){return-Math.exp(-o)}):(this.transform=function(o){return Math.log(o)},this.untransform=function(o){return Math.exp(o)}),this.baseLog=r,this.basePow=n,i.prototype.rescale.call(this)},t.prototype.reflect=function(e){return function(r){return-e(-r)}},t.prototype.nice=function(){var e,r,n=this.domain,o=0,a=n.length-1,s=n[o],l=n[a];l<s&&(e=Bv([a,o],2),o=e[0],a=e[1],r=Bv([l,s],2),s=r[0],l=r[1]),n[o]=this.basePow(Math.floor(this.baseLog(s))),n[a]=this.basePow(Math.ceil(this.baseLog(l))),this.domain=n},t.pow10=function(e){return isFinite(e)?+("1e"+e):e<0?0:e},t.makePowFn=function(e){return e===10?t.pow10:e===Math.E?Math.exp:function(r){return Math.pow(e,r)}},t.makeLogFn=function(e){if(e===Math.E)return Math.log;if(e===10)return Math.log10;if(e===2)return Math.log2;var r=Math.log(e);return function(n){return Math.log(n)/r}},t.prototype.ticks=function(e){var r;e===void 0&&(e=10);var n=e==null?10:+e,o=this.base,a=this.domain,s=a[0],l=a[a.length-1],u=l<s;u&&(r=Bv([l,s],2),s=r[0],l=r[1]);var c=this.baseLog(s),p=this.baseLog(l),h=[];if(!(o%1)&&p-c<n){if(c=Math.round(c)-1,p=Math.round(p)+1,s>0)for(;c<p;++c)for(var d=1,f=this.basePow(c);d<o;++d){var g=f*d;if(!(g<s)){if(g>l)break;h.push(g)}}else for(;c<p;++c)for(var d=o-1,f=this.basePow(c);d>=1;--d){var g=f*d;if(!(g<s)){if(g>l)break;h.push(g)}}h.length*2<n&&(h=Od(s,l,n))}else h=Od(c,p,Math.min(p-c,n)).map(this.basePow);return u?h.reverse():h},t.prototype.tickFormat=function(e,r){var n=this,o=this.base;if(r==null&&(r=o===10?".0e":","),typeof r!="function"&&(r=Zw(r)),e===1/0)return r;e==null&&(e=10);var a=Math.max(1,o*e/this.ticks().length);return function(s){var l=s/n.basePow(Math.round(n.baseLog(s)));return l*o<o-.5&&(l*=o),l<=a?r(s):""}},t}(vr),iK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Xx=function(i){iK(t,i);function t(){var e=i.call(this)||this;return e.scale=new nK,e.scale.clamper=mA,e}return Object.defineProperty(t.prototype,"base",{get:function(){return this.scale.base},set:function(e){this.scale.base=e},enumerable:!0,configurable:!0}),t.className="LogAxis",t.type="log",t}(Zl),Cl=globalThis&&globalThis.__assign||function(){return Cl=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Cl.apply(this,arguments)},oK=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},is=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(oK(arguments[t]));return i},zA=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")};function Gc(i,t,e){var r,n,o=(e||{}).stringify,a=o===void 0?[]:o,s=kn(i),l=kn(t);if(l==="array")return s!=="array"||i.length!==t.length||t.some(function(C,w){var b;return Gc((b=i)===null||b===void 0?void 0:b[w],C)!=null})?is(t):null;var u=i||{},c=t||{},p=new Set(is(Object.keys(u),Object.keys(c))),h=0,d={},f=function(C){if(u[C]===c[C])return"continue";var w=function(E){d[C]=E,h++};if(a.includes(C))return JSON.stringify(u[C]!==JSON.stringify(c[C]))&&w(c[C]),"continue";var b=kn(u[C]),S=kn(c[C]);if(b!==S||S==="primitive"||S===null||S==="array"&&u[C].length!==c[C].length||S==="class-instance"||S==="function"&&u[C]!==c[C])return w(c[C]),"continue";var T=Gc(u[C],c[C],{stringify:a});T!==null&&w(T)};try{for(var g=zA(p),v=g.next();!v.done;v=g.next()){var m=v.value;f(m)}}catch(C){r={error:C}}finally{try{v&&!v.done&&(n=g.return)&&n.call(g)}finally{if(r)throw r.error}}return h===0?null:d}var Qo=Symbol("<delete-property>"),qx=Symbol("<unspecified-property>");function ni(){for(var i,t,e=[],r=0;r<arguments.length;r++)e[r]=arguments[r];var n=e.map(function(h){return kn(h)});if(n.some(function(h){return h==="array"})){var o=e[e.length-1];return o instanceof Array?o.map(function(h){var d=kn(h);return d==="array"?ni([],h):d==="object"?ni({},h):h}):o}var a={},s=new Set(e.map(function(h){return h!=null?Object.keys(h):[]}).reduce(function(h,d){return h.concat(d)},[])),l=function(h){var d=e.map(function(m){return m!=null&&h in m?m[h]:qx}).filter(function(m){return m!==qx});if(d.length===0)return"continue";var f=d[d.length-1];if(f===Qo)return"continue";var g=d.map(function(m){return kn(m)}),v=g[0];if(g.some(function(m){return m!==v&&m!==null}))return a[h]=f,"continue";v==="array"||v==="object"?a[h]=ni.apply(void 0,is(d)):a[h]=f};try{for(var u=zA(s),c=u.next();!c.done;c=u.next()){var p=c.value;l(p)}}catch(h){i={error:h}}finally{try{c&&!c.done&&(t=u.return)&&t.call(u)}finally{if(i)throw i.error}}return a}function ds(i,t,e){e===void 0&&(e={});var r,n,o,a=e.path,s=a===void 0?void 0:a,l=e.matcherPath,u=l===void 0?s?s.replace(/(\[[0-9+]{1,}\])/i,"[]"):void 0:l,c=e.skip,p=c===void 0?[]:c,h=e.constructors,d=h===void 0?{}:h,f=e.allowedTypes,g=f===void 0?{}:f;if(i==null)throw new Error("AG Charts - target is uninitialised: "+(s||"<root>"));if(t==null)return i;var v=kn(i);for(var m in t)if(!(p.indexOf(m)>=0)){var C=t[m],w=(s?s+".":"")+m,b=(u?u+".":"")+m,S=i,T=S.constructor,E=S[m],P=d[m];try{var R=kn(E),M=kn(C);if(v==="class-instance"&&!(m in i||S.hasOwnProperty(m))){console.warn("AG Charts - unable to set ["+w+"] in "+((r=T)===null||r===void 0?void 0:r.name)+" - property is unknown");continue}var N=g[b]||[R];if(!(R==="class-instance"&&M==="object")){if(R!=null&&M!=null&&!N.includes(M)){console.warn("AG Charts - unable to set ["+w+"] in "+((n=T)===null||n===void 0?void 0:n.name)+" - can't apply type of ["+M+"], allowed types are: ["+N+"]");continue}}M==="array"||M==="class-instance"?S[m]=C:M==="object"?E!=null?ds(E,C,Cl(Cl({},e),{path:w,matcherPath:b})):P!=null?S[m]=ds(new P,C,Cl(Cl({},e),{path:w,matcherPath:b})):S[m]=C:S[m]=C}catch(W){console.warn("AG Charts - unable to set ["+w+"] in ["+((o=T)===null||o===void 0?void 0:o.name)+"]; nested error is: "+W.message);continue}}return i}function Ny(i,t,e){for(var r=[],n=3;n<arguments.length;n++)r[n-3]=arguments[n];var o,a=kn(i),s=e.skip||[];if(a==="array"){i.forEach(function(c,p){var h;Ny.apply(void 0,is([c,t,e],(h=r)===null||h===void 0?void 0:h.map(function(d){var f;return(f=d)===null||f===void 0?void 0:f[p]})))});return}else if(a!=="object")return;t.apply(void 0,is([a,i],r));var l=function(c){if(s.indexOf(c)>=0)return"continue";var p=i[c],h=(o=r)===null||o===void 0?void 0:o.map(function(f){var g;return(g=f)===null||g===void 0?void 0:g[c]}),d=kn(p);(d==="object"||d==="array")&&Ny.apply(void 0,is([p,t,e],h))};for(var u in i)l(u)}function kn(i){return i==null?null:i instanceof HTMLElement?"primitive":i instanceof Array?"array":i instanceof Date?"primitive":typeof i=="object"&&i.constructor===Object?"object":typeof i=="function"?"function":typeof i=="object"&&i.constructor!=null?"class-instance":"primitive"}var aK={type:"cartesian",axes:[{type:Zl.type,position:Ae.Left},{type:ps.type,position:Ae.Bottom}]},sK={axes:[{type:"number",position:Ae.Bottom},{type:"category",position:Ae.Left}]},Zx={axes:[{type:"number",position:Ae.Bottom},{type:"number",position:Ae.Left}]},Nl=globalThis&&globalThis.__assign||function(){return Nl=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Nl.apply(this,arguments)};function lK(i,t){var e={};for(var r in i){var n=t[r]||function(o){return o};e[r]=n(i[r],i)}return e}function $A(i){return i!=null&&i instanceof Array&&i[0]instanceof Array}function uK(i,t){if(i==null)return{};if(!(i instanceof Array))return i;var e=t.yKeys;if(e==null||$A(e))throw new Error("AG Charts - yNames and yKeys mismatching configuration.");var r={};return e.forEach(function(n,o){r[n]=i[o]}),r}function cK(i,t){return i==null?[[]]:$A(i)?i:t.grouped?i.map(function(e){return[e]}):[i]}function pK(i){if(i!=null){var t=i.placement;return Nl(Nl({},i),{placement:t==="inside"?na.Inside:t==="outside"?na.Outside:void 0})}}function Qx(i){var t=Nl(Nl({},i),{yKeys:i.yKeys||[i.yKey]});return delete t.yKey,lK(t,{yNames:uK,yKeys:cK,label:pK})}function Qs(i){return i}var hK={area:Qs,bar:Qx,column:Qx,histogram:Qs,line:Qs,pie:Qs,scatter:Qs,treemap:Qs};function dK(i){var t=i.type,e=hK[t||"line"];return e(i)}var Id=globalThis&&globalThis.__assign||function(){return Id=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Id.apply(this,arguments)},ub=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},fK=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Hv=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(fK(arguments[t]));return i};function gK(i){var t,e,r={},n=[];try{for(var o=ub(i),a=o.next();!a.done;a=o.next()){var s=a.value;if(s.type!=="column"&&s.type!=="bar"&&(s.type!=="area"||s.stacked!==!0)){n.push([s]);continue}var l=s.type||"line",u=s.stacked?"stacked":s.grouped?"grouped":s.yKeys?"stacked":"grouped",c=l+"-"+s.xKey+"-"+u;r[c]==null&&(r[c]=[],n.push(r[c])),r[c].push(s)}}catch(p){t={error:p}}finally{try{a&&!a.done&&(e=o.return)&&e.call(o)}finally{if(t)throw t.error}}return n}function Jx(i,t){var e,r,n={},o=["yKeys","fills","strokes","yNames","hideInChart","hideInLegend"],a=["yKey","fill","stroke","yName"];try{for(var s=ub(i),l=s.next();!l.done;l=s.next()){var u=l.value;for(var c in u){var p=o.indexOf(c)>-1,h=a.indexOf(c)>-1;p&&u[c].length>0?n[c]=Hv(n[c]||[],u[c]):h?n[c+"s"]=Hv(n[c+"s"]||[],[u[c]]):t&&c==="showInLegend"?u[c]===!1&&(n.hideInLegend=Hv(n.hideInLegend||[],u.yKey?[u.yKey]:u.yKeys)):t&&c==="grouped"?u[c]===!0&&(n[c]=u[c]):n[c]=u[c]}}}catch(d){e={error:d}}finally{try{l&&!l.done&&(r=s.return)&&r.call(s)}finally{if(e)throw e.error}}return n}function vK(i){var t,e,r=[],n=i.map(function(l){return(l.type==="bar"||l.type==="column")&&l.yKey!=null&&!l.stacked?Id(Id({},l),{grouped:l.grouped!=null?l.grouped:!0}):l});try{for(var o=ub(gK(n)),a=o.next();!a.done;a=o.next()){var s=a.value;switch(s[0].type){case"column":case"bar":r.push(Jx(s,!0));break;case"area":r.push(Jx(s,!1));break;case"line":default:s.length>1&&console.warn("AG Charts - unexpected grouping of series type: "+s[0].type),r.push(s[0]);break}}}catch(l){t={error:l}}finally{try{a&&!a.done&&(e=o.return)&&e.call(o)}finally{if(t)throw t.error}}return r}var ko=globalThis&&globalThis.__assign||function(){return ko=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},ko.apply(this,arguments)},mK=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},yK=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},KA=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(yK(arguments[t]));return i};function eu(i){var t,e;return i.type||((e=(t=i.series)===null||t===void 0?void 0:t[0])===null||e===void 0?void 0:e.type)||"cartesian"}function ia(i){var t=eu(i);if(t==null)return!0;switch(t){case"cartesian":case"area":case"bar":case"column":case"groupedCategory":case"histogram":case"line":case"scatter":return!0;default:return!1}}function Ef(i){var t=eu(i);if(t==null)return!1;switch(i.type){case"hierarchy":case"treemap":return!0;default:return!1}}function Rf(i){var t=eu(i);if(t==null)return!1;switch(i.type){case"polar":case"pie":return!0;default:return!1}}function CK(i){return i==null?!1:["line","bar","column","histogram","scatter","area","pie","treemap"].indexOf(i)>=0}function YA(i){var t,e,r=0;try{for(var n=mK(i),o=n.next();!o.done;o=n.next()){var a=o.value;a instanceof Array&&(r+=YA(a)),a!=null&&r++}}catch(s){t={error:s}}finally{try{o&&!o.done&&(e=n.return)&&e.call(n)}finally{if(t)throw t.error}}return r}function Ro(i,t,e){for(var r=[],n=0;n<e;n++)r.push(t[(n+i.colourIndex)%t.length]);return r}function e0(i,t){var e=t==null?i:ni(t,i);wK(e);var r=e.type,n=eu(e);e=ko(ko({},e),{type:n});var o=ia(e)?"line":Ef(e)?"treemap":Rf(e)?"pie":"line",a=n==="bar"?sK:n==="scatter"||n==="histogram"?Zx:ia(e)?aK:{},s=bK(a,e),l=s.context,u=s.mergedOptions,c=s.axesThemes,p=s.seriesThemes;return u.series=vK(u.series||[]).map(function(h){var d=h.type?h.type:CK(r)?r:o,f=ko(ko({},h),{type:d});return SK(l,f,p[d]||{})}),ia(u)&&(u.axes||[]).forEach(function(h,d){var f=h.type||"number",g=ko(ko({},h),{type:f}),v=ni(c[f],c[f][h.position||"unknown"]||{});u.axes[d]=TK(g,v)}),OK(e,u),u}function wK(i){var t,e;!((t=i.series)===null||t===void 0)&&t.some(function(r){return r.yKeys!=null&&r.yKey!=null})&&console.warn("AG Charts - series options yKeys and yKey are mutually exclusive, please only use yKey for future compatibility."),!((e=i.series)===null||e===void 0)&&e.some(function(r){return r.yNames!=null&&r.yName!=null})&&console.warn("AG Charts - series options yNames and yName are mutually exclusive, please only use yName for future compatibility.")}function bK(i,t){var e=_K(t),r=e.theme,n=e.cleanedTheme,o=e.axesThemes,a=e.seriesThemes,s={colourIndex:0,palette:r.palette},l=ni(i,n,t);return{context:s,mergedOptions:l,axesThemes:o,seriesThemes:a}}function _K(i){var t=Ol(i.theme),e=t.getConfig(eu(i)||"cartesian");return{theme:t,axesThemes:e.axes||{},seriesThemes:e.series||{},cleanedTheme:ni(e,{axes:Qo,series:Qo})}}function SK(i,t){for(var e=[],r=2;r<arguments.length;r++)e[r-2]=arguments[r];var n=xK(i,t),o={stacked:Qo},a=ni.apply(void 0,KA(e,[n,t,o]));return dK(a)}function xK(i,t){var e={},r=i.palette,n=r.fills,o=r.strokes,a=t,s=YA(a.yKeys||[])||1;switch(t.type){case"pie":s=Math.max(n.length,o.length);case"area":case"bar":case"column":e.fills=Ro(i,n,s),e.strokes=Ro(i,o,s);break;case"histogram":e.fill=Ro(i,n,1)[0],e.stroke=Ro(i,o,1)[0];break;case"scatter":e.fill=Ro(i,n,1)[0];case"line":e.stroke=Ro(i,n,1)[0],e.marker={stroke:Ro(i,o,1)[0],fill:Ro(i,n,1)[0]};break;case"treemap":break;default:throw new Error("AG Charts - unknown series type: "+t.type)}return i.colourIndex+=s,e}function TK(i){for(var t=[],e=1;e<arguments.length;e++)t[e-1]=arguments[e];var r={top:Qo,bottom:Qo,left:Qo,right:Qo};return ni.apply(void 0,KA(t,[i,r]))}function OK(i,t){Ny(i,function(e,r,n){!n||"enabled"in n&&r.enabled==null&&(n.enabled=!0)},{skip:["data"]},t)}var $t=globalThis&&globalThis.__assign||function(){return $t=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},$t.apply(this,arguments)},EK=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Pf=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(EK(arguments[t]));return i},XA=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")};function t0(i){if(ia(i))return"cartesian";if(Rf(i))return"polar";if(Ef(i))return"hierarchy";throw new Error("AG Chart - unknown type of chart for options with type: "+i.type)}var Dr=function(){function i(){}return i.createComponent=function(t,e){if(e.indexOf(".series")>=0){var r=$t($t({},t),{type:t.type||e.split(".")[0]});return qA([r])[0]}return null},i.create=function(t,e,r){return Gy.create(t)},i.update=function(t,e,r,n){return Gy.update(t,e)},i}(),Gy=function(){function i(){}return i.create=function(t){mc("user options",t);var e=e0(t),r=ia(e)?e.type==="groupedCategory"?new y9(document):new RA(document):Ef(e)?new f9(document):Rf(e)?new T9(document):void 0;if(!r)throw new Error("AG Charts - couldn't apply configuration, check type of options: "+e.type);return i.updateDelta(r,e,t),r},i.update=function(t,e){mc("user options",e);var r=e0(e,t.userOptions);if(t0(r)!==t0(t.options)){t.destroy(),console.warn("AG Charts - options supplied require a different type of chart, please recreate the chart.");return}var n=Gc(t.options,r,{stringify:["data"]});n!=null&&i.updateDelta(t,n,e)},i.updateDelta=function(t,e,r){e.type==null&&(e=$t($t({},e),{type:t.options.type||eu(e)})),mc("delta update",e),RK(t,e,r)},i.DEBUG=!1,i}();function mc(i){for(var t=[],e=1;e<arguments.length;e++)t[e-1]=arguments[e];Gy.DEBUG&&console.log.apply(console,Pf([i],t))}function RK(i,t,e){if(ia(t))jv(i,t,{skip:["type","data","series","axes","autoSize","listeners","theme"]});else if(Rf(t))jv(i,t,{skip:["type","data","series","autoSize","listeners","theme"]});else if(Ef(t))jv(i,t,{skip:["type","data","series","autoSize","listeners","theme"]});else throw new Error("AG Charts - couldn't apply configuration, check type of options and chart: "+t.type);var r=!1;t.series&&t.series.length>0&&PK(i,t),ia(t)&&t.axes&&(r=AK(i,t)),t.data&&(i.data=t.data,r=!0),t.autoSize!=null&&(i.autoSize=t.autoSize),t.listeners&&ZA(i,t.listeners),i.layoutPending=!0,r&&i.processData(),i.performLayout(),i.options=ni(i.options||{},t),i.userOptions=ni(i.userOptions||{},e)}function PK(i,t){var e=t.series;if(!!e){var r=i.series.length===e.length&&i.series.every(function(n,o){var a;return n.type===((a=e[o])===null||a===void 0?void 0:a.type)});if(r){i.series.forEach(function(n,o){var a,s,l=((s=(a=i.options)===null||a===void 0?void 0:a.series)===null||s===void 0?void 0:s[o])||{},u=Gc(l,e[o]||{});mc("applying series diff idx "+o,u),ds(n,u)});return}i.series=qA(e)}}function AK(i,t){var e=t.axes;if(!e)return!1;var r=i.axes.length===e.length&&i.axes.every(function(o,a){return o.type===e[a].type});if(r){var n=i.options;if(ia(n))return i.axes.forEach(function(o,a){var s,l=((s=n.axes)===null||s===void 0?void 0:s[a])||{},u=Gc(l,e[a]);mc("applying axis diff idx "+a,u),ds(o,u)}),!0}return i.axes=DK(e),!0}function qA(i){var t,e,r=[],n=["listeners"],o=0;try{for(var a=XA(i||[]),s=a.next();!s.done;s=a.next()){var l=s.value,u="series["+o+++"]";switch(l.type){case"area":r.push(Ra(new nb,l,{path:u,skip:n}));break;case"bar":case"column":r.push(Ra(new Fy,l,{path:u,skip:n}));break;case"histogram":r.push(Ra(new VA,l,{path:u,skip:n}));break;case"line":r.push(Ra(new ab,l,{path:u,skip:n}));break;case"scatter":r.push(Ra(new WA,l,{path:u,skip:n}));break;case"pie":r.push(Ra(new jA,l,{path:u,skip:n}));break;case"treemap":r.push(Ra(new n7,l,{path:u,skip:n}));break;default:throw new Error("AG Charts - unknown series type: "+l.type)}}}catch(c){t={error:c}}finally{try{s&&!s.done&&(e=a.return)&&e.call(a)}finally{if(t)throw t.error}}return r.forEach(function(c,p){var h,d,f=(d=(h=i)===null||h===void 0?void 0:h[p])===null||d===void 0?void 0:d.listeners;f!=null&&ZA(c,f)}),r}function DK(i){var t,e,r=[],n=0;try{for(var o=XA(i||[]),a=o.next();!a.done;a=o.next()){var s=a.value,l="axis["+n+++"]";switch(s.type){case"number":r.push(Nu(new Zl,s,{path:l}));break;case Xx.type:r.push(Nu(new Xx,s,{path:l}));break;case ps.type:r.push(Nu(new ps,s,{path:l}));break;case Il.type:r.push(Nu(new Il,s,{path:l}));break;case Lc.type:r.push(Nu(new Lc,s,{path:l}));break;default:throw new Error("AG Charts - unknown axis type: "+s.type)}}}catch(u){t={error:u}}finally{try{a&&!a.done&&(e=o.return)&&e.call(o)}finally{if(t)throw t.error}}return r}function ZA(i,t){for(var e in t)i.addEventListener(e,t[e])}var Ld={constructors:{title:xy,subtitle:xy,shadow:BA},allowedTypes:{"series[].marker.shape":["primitive","function"],"axis[].tick.count":["primitive","class-instance"]}};function jv(i,t,e){var r=e===void 0?{}:e,n=r.skip,o=r.path,a=$t($t($t({},Ld),{skip:Pf(["type"],n||[])}),o?{path:o}:{});return ds(i,t,a)}function Ra(i,t,e){var r=e===void 0?{}:e,n=r.skip,o=r.path,a,s=((a=Ld)===null||a===void 0?void 0:a.constructors)||{},l={constructors:$t($t({},s),{title:i.type==="pie"?c7:s.title})},u=$t($t($t($t({},Ld),l),{skip:Pf(["type"],n||[])}),o?{path:o}:{});return ds(i,t,u)}function Nu(i,t,e){var r=e===void 0?{}:e,n=r.skip,o=r.path,a=$t($t($t({},Ld),{skip:Pf(["type"],n||[])}),o?{path:o}:{});return ds(i,t,a)}function Mi(i){switch(i){case"bar":case"groupedBar":case"stackedBar":case"normalizedBar":return"bar";case"column":case"groupedColumn":case"stackedColumn":case"normalizedColumn":return"column";case"line":return"line";case"area":case"stackedArea":case"normalizedArea":return"area";case"scatter":case"bubble":return"scatter";case"histogram":return"histogram";case"pie":case"doughnut":return"pie";default:return"cartesian"}}var MK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),uh=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},fs=function(i){MK(t,i);function t(e){var r=i.call(this)||this;return r.model=e,r}return t.prototype.init=function(){var e=this;this.setChartRange(),this.addManagedListener(this.eventService,x.EVENT_RANGE_SELECTION_CHANGED,function(r){r.id&&r.id===e.model.chartId&&e.updateForRangeChange()}),this.model.unlinked&&this.rangeService&&this.rangeService.setCellRanges([]),this.addManagedListener(this.eventService,x.EVENT_COLUMN_MOVED,this.updateForGridChange.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PINNED,this.updateForGridChange.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VISIBLE,this.updateForGridChange.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,this.updateForGridChange.bind(this)),this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,this.updateForGridChange.bind(this)),this.addManagedListener(this.eventService,x.EVENT_CELL_VALUE_CHANGED,this.updateForDataChange.bind(this))},t.prototype.updateForGridChange=function(){this.model.unlinked||(this.model.updateCellRanges(),this.setChartRange())},t.prototype.updateForDataChange=function(){this.model.unlinked||(this.model.updateData(),this.raiseChartUpdatedEvent())},t.prototype.updateForRangeChange=function(){this.updateForGridChange(),this.raiseChartRangeSelectionChangedEvent()},t.prototype.updateForPanelChange=function(e){this.model.updateCellRanges(e),this.setChartRange(),this.raiseChartRangeSelectionChangedEvent()},t.prototype.getChartModel=function(){var e=this.model.pivotChart?"pivot":"range",r=this.isComboChart()?this.model.seriesChartTypes:void 0;return{modelType:e,chartId:this.model.chartId,chartType:this.model.chartType,chartThemeName:this.model.chartThemeName,chartOptions:this.chartProxy.getChartOptions(),chartPalette:this.chartProxy.getChartPalette(),cellRange:this.getCellRangeParams(),suppressChartRanges:this.model.suppressChartRanges,aggFunc:this.model.aggFunc,unlinkChart:this.model.unlinked,seriesChartTypes:r}},t.prototype.getChartId=function(){return this.model.chartId},t.prototype.getChartData=function(){return this.model.chartData},t.prototype.getChartType=function(){return this.model.chartType},t.prototype.setChartType=function(e){this.model.chartType=e,this.model.updateSeriesChartTypes(),this.raiseChartUpdatedEvent(),this.raiseChartOptionsChangedEvent()},t.prototype.setChartThemeName=function(e){this.model.chartThemeName=e,this.raiseChartUpdatedEvent(),this.raiseChartOptionsChangedEvent()},t.prototype.getChartThemeName=function(){return this.model.chartThemeName},t.prototype.isPivotChart=function(){return this.model.pivotChart},t.prototype.isPivotMode=function(){return this.model.isPivotMode()},t.prototype.isGrouping=function(){return this.model.isGrouping()},t.prototype.getThemes=function(){return this.gridOptionsWrapper.getChartThemes()},t.prototype.getPalettes=function(){var e=this,r=this.gridOptionsWrapper.getChartThemes();return r.map(function(n){var o=e.chartProxy.isStockTheme(n),a=o?n:e.chartProxy.lookupCustomChartTheme(n);return Ol(a).palette})},t.prototype.getValueColState=function(){return this.model.valueColState.map(this.displayNameMapper.bind(this))},t.prototype.getSelectedValueColState=function(){return this.getValueColState().filter(function(e){return e.selected})},t.prototype.getDimensionColState=function(){return this.model.dimensionColState},t.prototype.getSelectedDimension=function(){return this.model.getSelectedDimension()},t.prototype.displayNameMapper=function(e){var r=this.model.columnNames[e.colId];return e.displayName=r?r.join(" - "):this.model.getColDisplayName(e.column),e},t.prototype.getColStateForMenu=function(){return{dimensionCols:this.model.dimensionColState,valueCols:this.getValueColState()}},t.prototype.isDefaultCategorySelected=function(){return this.model.getSelectedDimension().colId===Sf.DEFAULT_CATEGORY},t.prototype.setChartRange=function(e){e===void 0&&(e=!1),this.rangeService&&!this.model.suppressChartRanges&&!this.model.unlinked&&this.rangeService.setCellRanges(this.getCellRanges()),e||this.raiseChartUpdatedEvent()},t.prototype.detachChartRange=function(){this.model.unlinked=!this.model.unlinked,this.model.unlinked?this.rangeService&&this.rangeService.setCellRanges([]):this.updateForGridChange()},t.prototype.setChartProxy=function(e){this.chartProxy=e},t.prototype.getChartProxy=function(){return this.chartProxy},t.prototype.isActiveXYChart=function(){return _.includes(["scatter","bubble"],this.getChartType())},t.prototype.isChartLinked=function(){return!this.model.unlinked},t.prototype.customComboExists=function(){return this.model.savedCustomSeriesChartTypes&&this.model.savedCustomSeriesChartTypes.length>0},t.prototype.getSeriesChartTypes=function(){return this.model.seriesChartTypes},t.prototype.isComboChart=function(){return this.model.isComboChart()},t.prototype.updateSeriesChartType=function(e,r,n){var o=this.model.seriesChartTypes.find(function(l){return l.colId===e});if(o){var a=this.model.chartType!=="customCombo";a&&(this.model.chartType="customCombo");var s=o.chartType;r!=null&&(o.chartType=r),n!=null&&(o.secondaryAxis=n),this.model.savedCustomSeriesChartTypes=this.model.seriesChartTypes,this.model.updateSeriesChartTypes(),this.updateForDataChange(),a&&this.dispatchEvent(Object.freeze({type:t.EVENT_CHART_TYPE_CHANGED})),s!==r&&this.dispatchEvent(Object.freeze({type:t.EVENT_CHART_SERIES_CHART_TYPE_CHANGED}))}},t.prototype.getActiveSeriesChartTypes=function(){var e=this.getSelectedValueColState().map(function(r){return r.colId});return this.getSeriesChartTypes().filter(function(r){return e.includes(r.colId)})},t.prototype.getChartSeriesTypes=function(){var e=["line","column","area"];return this.isComboChart()?e:[Mi(this.getChartType())]},t.prototype.getCellRanges=function(){return[this.model.dimensionCellRange,this.model.valueCellRange].filter(function(e){return e})},t.prototype.getCellRangeParams=function(){var e=this.getCellRanges(),r=e[0],n=r&&r.startRow||null,o=r&&r.endRow||null;return{rowStartIndex:n&&n.rowIndex,rowStartPinned:n&&n.rowPinned,rowEndIndex:o&&o.rowIndex,rowEndPinned:o&&o.rowPinned,columns:e.reduce(function(a,s){return a.concat(s.columns.map(function(l){return l.getId()}))},[])}},t.prototype.raiseChartUpdatedEvent=function(){var e=Object.freeze({type:t.EVENT_CHART_UPDATED});this.dispatchEvent(e)},t.prototype.raiseChartOptionsChangedEvent=function(){var e=this.getChartModel(),r=e.chartId,n=e.chartType,o=Object.freeze({type:x.EVENT_CHART_OPTIONS_CHANGED,chartId:r,chartType:n,chartThemeName:this.model.chartThemeName,chartOptions:this.chartProxy.getChartOptions(),api:this.gridApi,columnApi:this.columnApi});this.eventService.dispatchEvent(o)},t.prototype.raiseChartRangeSelectionChangedEvent=function(){var e=Object.freeze({type:x.EVENT_CHART_RANGE_SELECTION_CHANGED,id:this.model.chartId,chartId:this.model.chartId,cellRange:this.getCellRangeParams(),api:this.gridApi,columnApi:this.columnApi});this.eventService.dispatchEvent(e)},t.prototype.destroy=function(){i.prototype.destroy.call(this),this.rangeService&&this.rangeService.setCellRanges([])},t.EVENT_CHART_UPDATED="chartUpdated",t.EVENT_CHART_TYPE_CHANGED="chartTypeChanged",t.EVENT_CHART_SERIES_CHART_TYPE_CHANGED="chartSeriesChartTypeChanged",uh([y("rangeService")],t.prototype,"rangeService",void 0),uh([y("gridApi")],t.prototype,"gridApi",void 0),uh([y("columnApi")],t.prototype,"columnApi",void 0),uh([B],t.prototype,"init",null),t}(H),FK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Uv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},IK=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},r0=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(IK(arguments[t]));return i},LK=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},NK=function(i){FK(t,i);function t(e,r){var n=i.call(this,t.TEMPLATE)||this;return n.chartController=e,n.chartOptionsService=r,n.columnComps=new Map,n}return t.prototype.init=function(){this.updatePanels(),this.addManagedListener(this.chartController,fs.EVENT_CHART_UPDATED,this.updatePanels.bind(this)),this.createAutoScrollService()},t.prototype.destroy=function(){this.clearComponents(),i.prototype.destroy.call(this)},t.prototype.updatePanels=function(){var e=this,r=this.chartType,n=this.chartController.getColStateForMenu(),o=n.dimensionCols,a=n.valueCols,s=o.map(function(c){return c.colId}).concat(a.map(function(c){return c.colId}));this.chartType=this.chartController.getChartType();var l=this.getGroupExpandedState();if(_.areEqual(_.keys(this.columnComps),s)&&this.chartType===r){if(r0(o,a).forEach(function(c){e.columnComps.get(c.colId).setValue(c.selected,!0)}),this.chartController.isActiveXYChart()){var u=this.generateGetSeriesLabel();a.forEach(function(c){e.columnComps.get(c.colId).setLabel(u(c))})}_.removeFromParent(this.getGui().querySelector("#seriesChartTypeGroup")),this.seriesChartTypeGroupComp=this.destroyBean(this.seriesChartTypeGroupComp),this.createSeriesChartTypeGroup(a)}else this.clearComponents(),this.createCategoriesGroup(o),this.createSeriesGroup(a),this.createSeriesChartTypeGroup(a);this.restoreGroupExpandedState(l)},t.prototype.getGroupExpandedState=function(){return[this.categoriesGroupComp,this.seriesGroupComp,this.seriesChartTypeGroupComp].map(function(e){return e?e.isExpanded():!0})},t.prototype.restoreGroupExpandedState=function(e){[this.categoriesGroupComp,this.seriesGroupComp,this.seriesChartTypeGroupComp].forEach(function(r,n){!r||r.toggleGroupExpand(e[n])})},t.prototype.createAutoScrollService=function(){var e=this.getGui();this.autoScrollService=new Xc({scrollContainer:e,scrollAxis:"y",getVerticalPosition:function(){return e.scrollTop},setVerticalPosition:function(r){return e.scrollTop=r}})},t.prototype.addComponent=function(e,r,n){var o=document.createElement("div");o.id=n,o.className="ag-chart-data-section",o.appendChild(r.getGui()),e.appendChild(o)},t.prototype.addChangeListener=function(e,r){var n=this;this.addManagedListener(e,ei.EVENT_CHANGED,function(){r.selected=e.getValue(),n.chartController.updateForPanelChange(r)})},t.prototype.createCategoriesGroup=function(e){var r=this;this.categoriesGroupComp=this.createBean(new Qn({title:this.getCategoryGroupTitle(),enabled:!0,suppressEnabledCheckbox:!0,suppressOpenCloseIcons:!1,cssIdentifier:"charts-data"}));var n="chartDimension"+this.getCompId();e.forEach(function(o){var a=r.categoriesGroupComp.createManagedBean(new nw);a.setLabel(_.escapeString(o.displayName)),a.setValue(o.selected),a.setInputName(n),r.addChangeListener(a,o),r.categoriesGroupComp.addItem(a),r.columnComps.set(o.colId,a)}),this.addComponent(this.getGui(),this.categoriesGroupComp,"categoriesGroup")},t.prototype.createSeriesGroup=function(e){var r=this;if(this.seriesGroupComp=this.createManagedBean(new Qn({title:this.getSeriesGroupTitle(),enabled:!0,suppressEnabledCheckbox:!0,suppressOpenCloseIcons:!1,cssIdentifier:"charts-data"})),this.chartController.isActiveXYChart()){var n=this.seriesGroupComp.createManagedBean(new sf);n.setLabel(this.chartTranslationService.translate("paired")).setLabelAlignment("left").setLabelWidth("flex").setInputWidth(45).setValue(this.chartOptionsService.getPairedMode()).onValueChange(function(s){r.chartOptionsService.setPairedMode(!!s),r.chartController.updateForGridChange()}),this.seriesGroupComp.addItem(n)}var o=this.generateGetSeriesLabel();e.forEach(function(s){var l=r.seriesGroupComp.createManagedBean(new ii);l.addCssClass("ag-data-select-checkbox");var u=o(s);l.setLabel(u),l.setValue(s.selected),r.addChangeListener(l,s),r.seriesGroupComp.addItem(l),r.columnComps.set(s.colId,l),r.addDragHandle(l,s)}),this.addComponent(this.getGui(),this.seriesGroupComp,"seriesGroup");var a={getIconName:function(){return ht.ICON_MOVE},getContainer:function(){return r.seriesGroupComp.getGui()},onDragging:function(s){return r.onDragging(s)},onDragLeave:function(){return r.onDragLeave()},isInterestedIn:this.isInterestedIn.bind(this),targetContainsSource:!0};this.dragAndDropService.addDropTarget(a)},t.prototype.createSeriesChartTypeGroup=function(e){var r=this;if(!!this.chartController.isComboChart()){this.seriesChartTypeGroupComp=this.createManagedBean(new Qn({title:this.chartTranslationService.translate("seriesChartType"),enabled:!0,suppressEnabledCheckbox:!0,suppressOpenCloseIcons:!1,cssIdentifier:"charts-data"}));var n=this.chartController.getSeriesChartTypes();e.forEach(function(o){if(!!o.selected){var a=n.filter(function(h){return h.colId===o.colId})[0];if(!!a){var s=r.seriesChartTypeGroupComp.createManagedBean(new Qn({title:o.displayName,enabled:!0,suppressEnabledCheckbox:!0,suppressOpenCloseIcons:!0,cssIdentifier:"charts-format-sub-level"})),l=r.seriesChartTypeGroupComp.createManagedBean(new ii).setLabel(r.chartTranslationService.translate("secondaryAxis")).setLabelWidth("flex").setDisabled(["groupedColumn","stackedColumn","stackedArea"].includes(a.chartType)).setValue(!!a.secondaryAxis).onValueChange(function(h){return r.chartController.updateSeriesChartType(o.colId,void 0,h)});s.addItem(l);var u=function(h,d){return r.chartTranslationService.translate(h,d)},c=[{value:"line",text:u("line","Line")},{value:"area",text:u("area","Area")},{value:"stackedArea",text:u("stackedArea","StackedArea")},{value:"groupedColumn",text:u("groupedColumn","Grouped Column")},{value:"stackedColumn",text:u("stackedColumn","Stacked Column")}],p=s.createManagedBean(new Kc);p.setLabelAlignment("left").setLabelWidth("flex").addOptions(c).setValue(a.chartType).onValueChange(function(h){return r.chartController.updateSeriesChartType(o.colId,h)}),s.addItem(p),r.seriesChartTypeGroupComp.addItem(s)}}}),this.addComponent(this.getGui(),this.seriesChartTypeGroupComp,"seriesChartTypeGroup")}},t.prototype.addDragHandle=function(e,r){var n=this,o=_.createIconNoSpan("columnDrag",this.gridOptionsWrapper);o.classList.add("ag-drag-handle","ag-chart-data-column-drag-handle"),e.getGui().insertAdjacentElement("beforeend",o);var a={type:At.ChartPanel,eElement:o,dragItemName:r.displayName,getDragItem:function(){return{columns:[r.column]}},onDragStopped:function(){return n.onDragStop()}};this.dragAndDropService.addDragSource(a,!0),this.addDestroyFunc(function(){return n.dragAndDropService.removeDragSource(a)})},t.prototype.generateGetSeriesLabel=function(){if(!this.chartController.isActiveXYChart())return function(a){return _.escapeString(a.displayName)};var e=this.chartType==="bubble",r=this.isInPairedMode(),n=0,o=new Map;return o.set(0,"X"),o.set(1,"Y"),o.set(2,"size"),function(a){var s=_.escapeString(a.displayName);if(!a.selected)return s;var l;return r?l=o.get(n%(e?3:2)):n===0?l="X":l=e&&n%2===0?"size":"Y",n++,s+" ("+l+")"}},t.prototype.getCategoryGroupTitle=function(){return this.chartTranslationService.translate(this.chartController.isActiveXYChart()?"labels":"categories")},t.prototype.getSeriesGroupTitle=function(){return this.chartTranslationService.translate(this.chartController.isActiveXYChart()?"xyValues":"series")},t.prototype.isInPairedMode=function(){return this.chartController.isActiveXYChart()&&this.chartOptionsService.getSeriesOption("paired","scatter")},t.prototype.clearComponents=function(){_.clearElement(this.getGui()),this.categoriesGroupComp=this.destroyBean(this.categoriesGroupComp),this.seriesGroupComp=this.destroyBean(this.seriesGroupComp),this.seriesChartTypeGroupComp=this.destroyBean(this.seriesChartTypeGroupComp),this.columnComps.clear()},t.prototype.onDragging=function(e){var r=this.checkHoveredItem(e);if(!!r){this.lastDraggedColumn=e.dragItem.columns[0];var n=r.comp,o=r.position,a=this.lastHoveredItem||{},s=a.comp,l=a.position;if(!(n===s&&o===l)){this.autoScrollService.check(e.event),this.clearHoveredItems(),this.lastHoveredItem={comp:n,position:o};var u=n.getGui();u.classList.add("ag-list-item-hovered","ag-item-highlight-"+o)}}},t.prototype.checkHoveredItem=function(e){var r,n;if(_.missing(e.vDirection))return null;var o=e.event;try{for(var a=LK(this.columnComps.values()),s=a.next();!s.done;s=a.next()){var l=s.value,u=l.getGui();if(!!u.querySelector(".ag-chart-data-column-drag-handle")){var c=u.getBoundingClientRect(),p=o.clientY>=c.top&&o.clientY<=c.bottom;if(p){var h=u.clientHeight,d=o.clientY>c.top+h/2?"bottom":"top";return{comp:l,position:d}}}}}catch(f){r={error:f}}finally{try{s&&!s.done&&(n=a.return)&&n.call(a)}finally{if(r)throw r.error}}return null},t.prototype.onDragLeave=function(){this.clearHoveredItems()},t.prototype.onDragStop=function(){var e=this;if(this.lastHoveredItem){var r=this.chartController.getColStateForMenu(),n=r.dimensionCols,o=r.valueCols,a=r0(n,o).find(function(l){return l.column===e.lastDraggedColumn});if(a){var s=Array.from(this.columnComps.values()).indexOf(this.lastHoveredItem.comp);this.lastHoveredItem.position==="bottom"&&s++,a.order=s,this.chartController.updateForPanelChange(a)}}this.clearHoveredItems(),this.lastDraggedColumn=void 0,this.autoScrollService.ensureCleared()},t.prototype.clearHoveredItems=function(){this.columnComps.forEach(function(e){e.getGui().classList.remove("ag-list-item-hovered","ag-item-highlight-top","ag-item-highlight-bottom")}),this.lastHoveredItem=void 0},t.prototype.isInterestedIn=function(e){return e===At.ChartPanel},t.TEMPLATE='<div class="ag-chart-data-wrapper ag-scrollable-container"></div>',Uv([y("dragAndDropService")],t.prototype,"dragAndDropService",void 0),Uv([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),Uv([B],t.prototype,"init",null),t}(ee),GK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Pa=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Af=function(i){GK(t,i);function t(e){var r=i.call(this)||this;return r.activeComps=[],r.params=e,r}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-sub-level",direction:"vertical",suppressOpenCloseIcons:!0};this.setTemplate(t.TEMPLATE,{fontGroup:e}),this.initGroup(),this.initFontFamilySelect(),this.initFontWeightStyleSelect(),this.initFontSizeSelect(),this.initFontColorPicker()},t.prototype.addCompToPanel=function(e){this.fontGroup.addItem(e),this.activeComps.push(e)},t.prototype.setEnabled=function(e){this.fontGroup.setEnabled(e)},t.prototype.initGroup=function(){var e=this;this.fontGroup.setTitle(this.params.name||this.chartTranslationService.translate("font")).setEnabled(this.params.enabled).hideEnabledCheckbox(!!this.params.suppressEnabledCheckbox).hideOpenCloseIcons(!0).onEnableChange(function(r){e.params.setEnabled&&e.params.setEnabled(r)})},t.prototype.initFontFamilySelect=function(){var e=this,r=["Arial, sans-serif","Aria Black, sans-serif","Book Antiqua,  serif","Charcoal, sans-serif","Comic Sans MS, cursive","Courier, monospace","Courier New, monospace","Gadget, sans-serif","Geneva, sans-serif","Helvetica, sans-serif","Impact, sans-serif","Lucida Console, monospace","Lucida Grande, sans-serif","Lucida Sans Unicode,  sans-serif","Monaco, monospace","Palatino Linotype, serif","Palatino, serif","Times New Roman, serif","Times, serif","Verdana, sans-serif"],n=this.params.initialFont.family,o=r[0];if(n){var a=r.map(function(c){return c.toLowerCase()}),s=a.indexOf(n.toLowerCase());if(s>=0)o=r[s];else{var l=_.capitalise(n);r.push(l),o=l}}var u=r.sort().map(function(c){return{value:c,text:c}});this.familySelect.addOptions(u).setInputWidth("flex").setValue(""+o).onValueChange(function(c){return e.params.setFont({family:c})})},t.prototype.initFontSizeSelect=function(){var e=this,r=[8,10,12,14,16,18,20,22,24,26,28,30,32,34,36],n=this.params.initialFont.size;_.includes(r,n)||r.push(n);var o=r.sort(function(a,s){return a-s}).map(function(a){return{value:""+a,text:""+a}});this.sizeSelect.addOptions(o).setInputWidth("flex").setValue(""+n).onValueChange(function(a){return e.params.setFont({size:parseInt(a,10)})}),this.sizeSelect.setLabel(this.chartTranslationService.translate("size"))},t.prototype.initFontWeightStyleSelect=function(){var e=this,r=this.params.initialFont,n=r.weight,o=n===void 0?"normal":n,a=r.style,s=a===void 0?"normal":a,l=[{name:"normal",weight:"normal",style:"normal"},{name:"bold",weight:"bold",style:"normal"},{name:"italic",weight:"normal",style:"italic"},{name:"boldItalic",weight:"bold",style:"italic"}],u=l.find(function(p){return p.weight===o&&p.style===s});u||(u={name:"predefined",weight:o,style:s},l.unshift(u));var c=l.map(function(p){return{value:p.name,text:e.chartTranslationService.translate(p.name)}});this.weightStyleSelect.addOptions(c).setInputWidth("flex").setValue(u.name).onValueChange(function(p){var h=l.find(function(d){return d.name===p});e.params.setFont({weight:h.weight,style:h.style})})},t.prototype.initFontColorPicker=function(){var e=this;this.colorPicker.setLabel(this.chartTranslationService.translate("color")).setInputWidth(45).setValue(""+this.params.initialFont.color).onValueChange(function(r){return e.params.setFont({color:r})})},t.prototype.destroyActiveComps=function(){var e=this;this.activeComps.forEach(function(r){_.removeFromParent(r.getGui()),e.destroyBean(r)})},t.prototype.destroy=function(){this.destroyActiveComps(),i.prototype.destroy.call(this)},t.TEMPLATE=`<div class="ag-font-panel">
            <ag-group-component ref="fontGroup">
                <ag-select ref="familySelect"></ag-select>
                <ag-select ref="weightStyleSelect"></ag-select>
                <div class="ag-charts-font-size-color">
                    <ag-select ref="sizeSelect"></ag-select>
                    <ag-color-picker ref="colorPicker"></ag-color-picker>
                </div>
            </ag-group-component>
        </div>`,Pa([I("fontGroup")],t.prototype,"fontGroup",void 0),Pa([I("familySelect")],t.prototype,"familySelect",void 0),Pa([I("weightStyleSelect")],t.prototype,"weightStyleSelect",void 0),Pa([I("sizeSelect")],t.prototype,"sizeSelect",void 0),Pa([I("colorPicker")],t.prototype,"colorPicker",void 0),Pa([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),Pa([B],t.prototype,"init",null),t}(ee),kK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),gi=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},WK=function(i){kK(t,i);function t(e){var r=i.call(this)||this;return r.chartOptionsService=e,r.activePanels=[],r}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-top-level",direction:"vertical"};this.setTemplate(t.TEMPLATE,{legendGroup:e}),this.initLegendGroup(),this.initLegendPosition(),this.initLegendPadding(),this.initLegendItems(),this.initLabelPanel()},t.prototype.initLegendGroup=function(){var e=this;this.legendGroup.setTitle(this.chartTranslationService.translate("legend")).hideEnabledCheckbox(!1).setEnabled(this.chartOptionsService.getChartOption("legend.enabled")||!1).toggleGroupExpand(!1).onEnableChange(function(r){e.chartOptionsService.setChartOption("legend.enabled",r),e.legendGroup.toggleGroupExpand(!0)})},t.prototype.initLegendPosition=function(){var e=this,r=[Za.Top,Za.Right,Za.Bottom,Za.Left];this.legendPositionSelect.setLabel(this.chartTranslationService.translate("position")).setLabelWidth("flex").setInputWidth(80).addOptions(r.map(function(n){return{value:n,text:e.chartTranslationService.translate(n)}})).setValue(this.chartOptionsService.getChartOption("legend.position")).onValueChange(function(n){return e.chartOptionsService.setChartOption("legend.position",n)})},t.prototype.initLegendPadding=function(){var e=this,r=this.chartOptionsService.getChartOption("legend.spacing");this.legendPaddingSlider.setLabel(this.chartTranslationService.translate("spacing")).setMaxValue(mr(r,200)).setValue(""+r).setTextFieldWidth(45).onValueChange(function(n){return e.chartOptionsService.setChartOption("legend.spacing",n)})},t.prototype.initLegendItems=function(){var e=this,r=function(n,o,a,s){var l=e.chartOptionsService.getChartOption("legend."+n);a.setLabel(e.chartTranslationService.translate(o)).setMaxValue(mr(l,s)).setValue(""+l).setTextFieldWidth(45).onValueChange(function(u){e.chartOptionsService.setChartOption("legend."+n,u)})};r("item.marker.size","markerSize",this.markerSizeSlider,40),r("item.marker.strokeWidth","markerStroke",this.markerStrokeSlider,10),r("item.marker.padding","itemSpacing",this.markerPaddingSlider,20),r("item.paddingX","layoutHorizontalSpacing",this.itemPaddingXSlider,50),r("item.paddingY","layoutVerticalSpacing",this.itemPaddingYSlider,50)},t.prototype.initLabelPanel=function(){var e=this,r=this.chartOptionsService,n={family:r.getChartOption("legend.item.label.fontFamily"),style:r.getChartOption("legend.item.label.fontStyle"),weight:r.getChartOption("legend.item.label.fontWeight"),size:r.getChartOption("legend.item.label.fontSize"),color:r.getChartOption("legend.item.label.color")},o=function(l){var u=e.chartOptionsService;l.family&&u.setChartOption("legend.item.label.fontFamily",l.family),l.weight&&u.setChartOption("legend.item.label.fontWeight",l.weight),l.style&&u.setChartOption("legend.item.label.fontStyle",l.style),l.size&&u.setChartOption("legend.item.label.fontSize",l.size),l.color&&u.setChartOption("legend.item.label.color",l.color)},a={enabled:!0,suppressEnabledCheckbox:!0,initialFont:n,setFont:o},s=this.createBean(new Af(a));this.legendGroup.addItem(s),this.activePanels.push(s)},t.prototype.destroyActivePanels=function(){var e=this;this.activePanels.forEach(function(r){_.removeFromParent(r.getGui()),e.destroyBean(r)})},t.prototype.destroy=function(){this.destroyActivePanels(),i.prototype.destroy.call(this)},t.TEMPLATE=`<div>
            <ag-group-component ref="legendGroup">
                <ag-select ref="legendPositionSelect"></ag-select>
                <ag-slider ref="legendPaddingSlider"></ag-slider>
                <ag-slider ref="markerSizeSlider"></ag-slider>
                <ag-slider ref="markerStrokeSlider"></ag-slider>
                <ag-slider ref="markerPaddingSlider"></ag-slider>
                <ag-slider ref="itemPaddingXSlider"></ag-slider>
                <ag-slider ref="itemPaddingYSlider"></ag-slider>
            </ag-group-component>
        </div>`,gi([I("legendGroup")],t.prototype,"legendGroup",void 0),gi([I("legendPositionSelect")],t.prototype,"legendPositionSelect",void 0),gi([I("legendPaddingSlider")],t.prototype,"legendPaddingSlider",void 0),gi([I("markerSizeSlider")],t.prototype,"markerSizeSlider",void 0),gi([I("markerStrokeSlider")],t.prototype,"markerStrokeSlider",void 0),gi([I("markerPaddingSlider")],t.prototype,"markerPaddingSlider",void 0),gi([I("itemPaddingXSlider")],t.prototype,"itemPaddingXSlider",void 0),gi([I("itemPaddingYSlider")],t.prototype,"itemPaddingYSlider",void 0),gi([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),gi([B],t.prototype,"init",null),t}(ee),VK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Js=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},BK=function(i){VK(t,i);function t(e){var r=i.call(this)||this;return r.chartOptionsService=e,r}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-sub-level",direction:"vertical",suppressOpenCloseIcons:!0};this.setTemplate(t.TEMPLATE,{axisTicksGroup:e}),this.initAxisTicks()},t.prototype.initAxisTicks=function(){var e=this;this.axisTicksGroup.setTitle(this.chartTranslationService.translate("ticks")).hideOpenCloseIcons(!0).hideEnabledCheckbox(!0),this.axisTicksColorPicker.setLabel(this.chartTranslationService.translate("color")).setLabelWidth("flex").setInputWidth(45).setValue(this.chartOptionsService.getAxisProperty("tick.color")).onValueChange(function(n){return e.chartOptionsService.setAxisProperty("tick.color",n)});var r=function(n,o,a,s){var l=e.chartOptionsService.getAxisProperty(n);o.setLabel(a).setMaxValue(mr(l,s)).setValue(""+l).setTextFieldWidth(45).onValueChange(function(u){return e.chartOptionsService.setAxisProperty(n,u)})};r("tick.width",this.axisTicksWidthSlider,this.chartTranslationService.translate("width"),10),r("tick.size",this.axisTicksSizeSlider,this.chartTranslationService.translate("length"),30)},t.TEMPLATE=`<div>
            <ag-group-component ref="axisTicksGroup">
                <ag-color-picker ref="axisTicksColorPicker"></ag-color-picker>
                <ag-slider ref="axisTicksWidthSlider"></ag-slider>
                <ag-slider ref="axisTicksSizeSlider"></ag-slider>
            </ag-group-component>
        </div>`,Js([I("axisTicksGroup")],t.prototype,"axisTicksGroup",void 0),Js([I("axisTicksColorPicker")],t.prototype,"axisTicksColorPicker",void 0),Js([I("axisTicksWidthSlider")],t.prototype,"axisTicksWidthSlider",void 0),Js([I("axisTicksSizeSlider")],t.prototype,"axisTicksSizeSlider",void 0),Js([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),Js([B],t.prototype,"init",null),t}(ee),HK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),el=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Aa=function(i){HK(t,i);function t(e,r){var n=i.call(this)||this;return n.chartController=e,n.chartOptionsService=r,n.activePanels=[],n.axisLabelUpdateFuncs=[],n}return t.prototype.init=function(){var e=this,r={cssIdentifier:"charts-format-top-level",direction:"vertical"};this.setTemplate(t.TEMPLATE,{axisGroup:r}),this.initAxis(),this.initAxisTicks(),this.initAxisLabels();var n=function(){return e.axisLabelUpdateFuncs.forEach(function(o){return o()})};this.addManagedListener(this.chartController,fs.EVENT_CHART_UPDATED,n)},t.prototype.initAxis=function(){var e=this,r=this.chartTranslationService;this.axisGroup.setTitle(r.translate("axis")).toggleGroupExpand(!1).hideEnabledCheckbox(!0),this.axisColorInput.setLabel(r.translate("color")).setLabelWidth("flex").setInputWidth(45).setValue(this.chartOptionsService.getAxisProperty("line.color")).onValueChange(function(a){return e.chartOptionsService.setAxisProperty("line.color",a)});var n=this.chartOptionsService.getAxisProperty("line.width");if(this.axisLineWidthSlider.setMaxValue(mr(n,10)).setLabel(r.translate("thickness")).setTextFieldWidth(45).setValue(""+n).onValueChange(function(a){return e.chartOptionsService.setAxisProperty("line.width",a)}),_.includes(["line","scatter","bubble"],this.chartController.getChartType())&&!this.chartController.isGrouping()){var o=[{value:"",text:r.translate("automatic")}];["category","time","number"].forEach(function(a){o.push({value:a,text:r.translate(a)})}),this.xAxisTypeSelect.setLabel(r.translate("xType")).setLabelWidth("flex").addOptions(o).setValue(this.chartOptionsService.getChartOption("xAxis.type")||"").onValueChange(function(a){e.chartOptionsService.setChartOption("xAxis.type",typeof a=="string"&&a.length&&a),e.chartController.updateForDataChange()})}else this.xAxisTypeSelect.setDisplayed(!1)},t.prototype.initAxisTicks=function(){var e=this.createBean(new BK(this.chartOptionsService));this.axisGroup.addItem(e),this.activePanels.push(e)},t.prototype.initAxisLabels=function(){var e=this,r={family:this.chartOptionsService.getAxisProperty("label.fontFamily"),style:this.chartOptionsService.getAxisProperty("label.fontStyle"),weight:this.chartOptionsService.getAxisProperty("label.fontWeight"),size:this.chartOptionsService.getAxisProperty("label.fontSize"),color:this.chartOptionsService.getAxisProperty("label.color")},n=function(s){s.family&&e.chartOptionsService.setAxisProperty("label.fontFamily",s.family),s.weight&&e.chartOptionsService.setAxisProperty("label.fontWeight",s.weight),s.style&&e.chartOptionsService.setAxisProperty("label.fontStyle",s.style),s.size&&e.chartOptionsService.setAxisProperty("label.fontSize",s.size),s.color&&e.chartOptionsService.setAxisProperty("label.color",s.color),e.chartController.getChartProxy().getChart().layoutPending=!0},o={enabled:!0,suppressEnabledCheckbox:!0,initialFont:r,setFont:n},a=this.createBean(new Af(o));this.axisGroup.addItem(a),this.activePanels.push(a),this.addAdditionalLabelComps(a)},t.prototype.addAdditionalLabelComps=function(e){var r=this,n=function(c,p){var h=r.chartOptionsService.getLabelRotation(p),d=new ow().setLabel(c).setLabelWidth("flex").setValue(h||0).onValueChange(function(g){return r.chartOptionsService.setLabelRotation(p,g)});r.axisLabelUpdateFuncs.push(function(){var g=r.chartOptionsService.getLabelRotation(p);d.setValue(g)});var f=r.createBean(d);e.addCompToPanel(f)},o=String.fromCharCode(176),a=this.chartTranslationService.translate("xRotation")+" "+o,s=this.chartTranslationService.translate("yRotation")+" "+o;n(a,"xAxis"),n(s,"yAxis");var l=this.createBean(new zi),u=this.chartOptionsService.getAxisProperty("label.padding");l.setLabel(this.chartTranslationService.translate("padding")).setMaxValue(mr(u,30)).setValue(""+u).setTextFieldWidth(45).onValueChange(function(c){return r.chartOptionsService.setAxisProperty("label.padding",c)}),e.addCompToPanel(l)},t.prototype.destroyActivePanels=function(){var e=this;this.activePanels.forEach(function(r){_.removeFromParent(r.getGui()),e.destroyBean(r)})},t.prototype.destroy=function(){this.destroyActivePanels(),i.prototype.destroy.call(this)},t.TEMPLATE=`<div>
            <ag-group-component ref="axisGroup">
                <ag-color-picker ref="axisColorInput"></ag-color-picker>
                <ag-slider ref="axisLineWidthSlider"></ag-slider>
                <ag-select ref="xAxisTypeSelect"></ag-select>
            </ag-group-component>
        </div>`,el([I("axisGroup")],t.prototype,"axisGroup",void 0),el([I("axisColorInput")],t.prototype,"axisColorInput",void 0),el([I("axisLineWidthSlider")],t.prototype,"axisLineWidthSlider",void 0),el([I("xAxisTypeSelect")],t.prototype,"xAxisTypeSelect",void 0),el([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),el([B],t.prototype,"init",null),t}(ee),jK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ch=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Da=function(i){jK(t,i);function t(e){var r=i.call(this)||this;return r.chartOptionsService=e,r}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-top-level",direction:"vertical"};this.setTemplate(t.TEMPLATE,{navigatorGroup:e}),this.initNavigator()},t.prototype.initNavigator=function(){var e=this,r=this.chartTranslationService;this.navigatorGroup.setTitle(r.translate("navigator")).toggleGroupExpand(!1).hideEnabledCheckbox(!1).setEnabled(this.chartOptionsService.getChartOption("navigator.enabled")||!1).onEnableChange(function(o){e.chartOptionsService.setChartOption("navigator.enabled",o),e.navigatorGroup.toggleGroupExpand(!0)});var n=this.chartOptionsService.getChartOption("navigator.height");this.navigatorHeightSlider.setLabel(r.translate("height")).setMinValue(10).setMaxValue(mr(n,60)).setTextFieldWidth(45).setValue(""+(n||30)).onValueChange(function(o){return e.chartOptionsService.setChartOption("navigator.height",o)})},t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.TEMPLATE=`<div>
            <ag-group-component ref="navigatorGroup">
                <ag-slider ref="navigatorHeightSlider"></ag-slider>
            </ag-group-component>
        </div>`,ch([I("navigatorGroup")],t.prototype,"navigatorGroup",void 0),ch([I("navigatorHeightSlider")],t.prototype,"navigatorHeightSlider",void 0),ch([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),ch([B],t.prototype,"init",null),t}(ee),UK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ma=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},zK=function(i){UK(t,i);function t(e){var r=i.call(this)||this;return r.chartOptionsService=e,r}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-sub-level",direction:"vertical",suppressOpenCloseIcons:!0};this.setTemplate(t.TEMPLATE,{chartPaddingGroup:e}),this.initGroup(),this.initChartPaddingItems()},t.prototype.initGroup=function(){this.chartPaddingGroup.setTitle(this.chartTranslationService.translate("padding")).hideOpenCloseIcons(!0).hideEnabledCheckbox(!0)},t.prototype.initChartPaddingItems=function(){var e=this,r=function(n,o){var a=e.chartOptionsService.getChartOption("padding."+n);o.setLabel(e.chartTranslationService.translate(n)).setMaxValue(mr(a,200)).setValue(""+a).setTextFieldWidth(45).onValueChange(function(s){return e.chartOptionsService.setChartOption("padding."+n,s)})};r("top",this.paddingTopSlider),r("right",this.paddingRightSlider),r("bottom",this.paddingBottomSlider),r("left",this.paddingLeftSlider)},t.TEMPLATE=`<div>
            <ag-group-component ref="chartPaddingGroup">
                <ag-slider ref="paddingTopSlider"></ag-slider>
                <ag-slider ref="paddingRightSlider"></ag-slider>
                <ag-slider ref="paddingBottomSlider"></ag-slider>
                <ag-slider ref="paddingLeftSlider"></ag-slider>
            </ag-group-component>
        <div>`,Ma([I("chartPaddingGroup")],t.prototype,"chartPaddingGroup",void 0),Ma([I("paddingTopSlider")],t.prototype,"paddingTopSlider",void 0),Ma([I("paddingRightSlider")],t.prototype,"paddingRightSlider",void 0),Ma([I("paddingBottomSlider")],t.prototype,"paddingBottomSlider",void 0),Ma([I("paddingLeftSlider")],t.prototype,"paddingLeftSlider",void 0),Ma([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),Ma([B],t.prototype,"init",null),t}(ee),$K=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ph=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},KK=function(i){$K(t,i);function t(e){var r=i.call(this)||this;return r.chartOptionsService=e,r}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-sub-level",direction:"vertical",suppressOpenCloseIcons:!0};this.setTemplate(t.TEMPLATE,{chartBackgroundGroup:e}),this.initGroup(),this.initColorPicker()},t.prototype.initGroup=function(){var e=this;this.group.setTitle(this.chartTranslationService.translate("background")).setEnabled(this.chartOptionsService.getChartOption("background.visible")).hideOpenCloseIcons(!0).hideEnabledCheckbox(!1).onEnableChange(function(r){return e.chartOptionsService.setChartOption("background.visible",r)})},t.prototype.initColorPicker=function(){var e=this;this.colorPicker.setLabel(this.chartTranslationService.translate("color")).setLabelWidth("flex").setInputWidth(45).setValue(this.chartOptionsService.getChartOption("background.fill")).onValueChange(function(r){return e.chartOptionsService.setChartOption("background.fill",r)})},t.TEMPLATE=`<div>
            <ag-group-component ref="chartBackgroundGroup">
                <ag-color-picker ref="colorPicker"></ag-color-picker>
            </ag-group-component>
        <div>`,ph([I("chartBackgroundGroup")],t.prototype,"group",void 0),ph([I("colorPicker")],t.prototype,"colorPicker",void 0),ph([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),ph([B],t.prototype,"init",null),t}(ee),YK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),n0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},XK=function(i){YK(t,i);function t(e){var r=i.call(this,t.TEMPLATE)||this;return r.chartOptionsService=e,r.activePanels=[],r}return t.prototype.init=function(){this.initFontPanel(),this.titlePlaceholder=this.chartTranslationService.translate("titlePlaceholder")},t.prototype.hasTitle=function(){var e=this.getOption("title");return e&&e.enabled&&e.text&&e.text.length>0},t.prototype.initFontPanel=function(){var e=this,r=this.hasTitle,n=function(l){l.family&&e.setOption("title.fontFamily",l.family),l.weight&&e.setOption("title.fontWeight",l.weight),l.style&&e.setOption("title.fontStyle",l.style),l.size&&e.setOption("title.fontSize",l.size),l.color&&e.setOption("title.color",l.color)},o={family:this.getOption("title.fontFamily"),style:this.getOption("title.fontStyle"),weight:this.getOption("title.fontWeight"),size:this.getOption("title.fontSize"),color:this.getOption("title.color")};r||n(o);var a={name:this.chartTranslationService.translate("title"),enabled:this.hasTitle(),suppressEnabledCheckbox:!1,initialFont:o,setFont:n,setEnabled:function(l){e.setOption("title.enabled",l);var u=e.getOption("title.text");l&&u==="Title"&&e.setOption("title.text",e.titlePlaceholder)}},s=this.createBean(new Af(a));this.getGui().appendChild(s.getGui()),this.activePanels.push(s),this.addManagedListener(this.eventService,"chartTitleEdit",function(){s.setEnabled(e.hasTitle())})},t.prototype.getOption=function(e){return this.chartOptionsService.getChartOption(e)},t.prototype.setOption=function(e,r){this.chartOptionsService.setChartOption(e,r)},t.prototype.destroyActivePanels=function(){var e=this;this.activePanels.forEach(function(r){_.removeFromParent(r.getGui()),e.destroyBean(r)})},t.prototype.destroy=function(){this.destroyActivePanels(),i.prototype.destroy.call(this)},t.TEMPLATE="<div></div>",n0([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),n0([B],t.prototype,"init",null),t}(ee),qK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),zv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ZK=function(i){qK(t,i);function t(e){var r=i.call(this)||this;return r.chartOptionsService=e,r.activePanels=[],r}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-top-level",direction:"vertical"};this.setTemplate(t.TEMPLATE,{chartGroup:e}),this.initGroup(),this.initTitles(),this.initPaddingPanel(),this.initBackgroundPanel()},t.prototype.initGroup=function(){this.chartGroup.setTitle(this.chartTranslationService.translate("chart")).toggleGroupExpand(!1).hideEnabledCheckbox(!0)},t.prototype.initTitles=function(){var e=this.createBean(new XK(this.chartOptionsService));this.chartGroup.addItem(e),this.activePanels.push(e)},t.prototype.initPaddingPanel=function(){var e=this.createBean(new zK(this.chartOptionsService));this.chartGroup.addItem(e),this.activePanels.push(e)},t.prototype.initBackgroundPanel=function(){var e=this.createBean(new KK(this.chartOptionsService));this.chartGroup.addItem(e),this.activePanels.push(e)},t.prototype.destroyActivePanels=function(){var e=this;this.activePanels.forEach(function(r){_.removeFromParent(r.getGui()),e.destroyBean(r)})},t.prototype.destroy=function(){this.destroyActivePanels(),i.prototype.destroy.call(this)},t.TEMPLATE=`<div>
            <ag-group-component ref="chartGroup"></ag-group-component>
        </div>`,zv([I("chartGroup")],t.prototype,"chartGroup",void 0),zv([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),zv([B],t.prototype,"init",null),t}(ee),QK=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Fa=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},JK=function(i){QK(t,i);function t(e,r){var n=i.call(this)||this;return n.chartOptionsService=e,n.getSelectedSeries=r,n}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-sub-level",direction:"vertical",suppressOpenCloseIcons:!0};this.setTemplate(t.TEMPLATE,{shadowGroup:e}),this.shadowBlurSlider.setTextFieldWidth(45),this.shadowXOffsetSlider.setTextFieldWidth(45),this.shadowYOffsetSlider.setTextFieldWidth(45),this.initSeriesShadow()},t.prototype.initSeriesShadow=function(){var e=this;this.shadowGroup.setTitle(this.chartTranslationService.translate("shadow")).setEnabled(this.chartOptionsService.getSeriesOption("shadow.enabled",this.getSelectedSeries())).hideOpenCloseIcons(!0).hideEnabledCheckbox(!1).onEnableChange(function(n){return e.chartOptionsService.setSeriesOption("shadow.enabled",n,e.getSelectedSeries())}),this.shadowColorPicker.setLabel(this.chartTranslationService.translate("color")).setLabelWidth("flex").setInputWidth(45).setValue(this.chartOptionsService.getSeriesOption("shadow.color",this.getSelectedSeries())).onValueChange(function(n){return e.chartOptionsService.setSeriesOption("shadow.color",n,e.getSelectedSeries())});var r=function(n,o,a,s){var l=e.chartOptionsService.getSeriesOption("shadow."+o,e.getSelectedSeries());n.setLabel(e.chartTranslationService.translate(o)).setMinValue(a).setMaxValue(mr(l,s)).setValue(""+l).onValueChange(function(u){return e.chartOptionsService.setSeriesOption("shadow."+o,u,e.getSelectedSeries())})};r(this.shadowBlurSlider,"blur",0,20),r(this.shadowXOffsetSlider,"xOffset",-10,10),r(this.shadowYOffsetSlider,"yOffset",-10,10)},t.TEMPLATE=`<div>
            <ag-group-component ref="shadowGroup">
                <ag-color-picker ref="shadowColorPicker"></ag-color-picker>
                <ag-slider ref="shadowBlurSlider"></ag-slider>
                <ag-slider ref="shadowXOffsetSlider"></ag-slider>
                <ag-slider ref="shadowYOffsetSlider"></ag-slider>
            </ag-group-component>
        </div>`,Fa([I("shadowGroup")],t.prototype,"shadowGroup",void 0),Fa([I("shadowColorPicker")],t.prototype,"shadowColorPicker",void 0),Fa([I("shadowBlurSlider")],t.prototype,"shadowBlurSlider",void 0),Fa([I("shadowXOffsetSlider")],t.prototype,"shadowXOffsetSlider",void 0),Fa([I("shadowYOffsetSlider")],t.prototype,"shadowYOffsetSlider",void 0),Fa([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),Fa([B],t.prototype,"init",null),t}(ee);function eY(i,t,e){var r={family:t.getSeriesOption("label.fontFamily",e()),style:t.getSeriesOption("label.fontStyle",e()),weight:t.getSeriesOption("label.fontWeight",e()),size:t.getSeriesOption("label.fontSize",e()),color:t.getSeriesOption("label.color",e())},n=function(a){a.family&&t.setSeriesOption("label.fontFamily",a.family,e()),a.weight&&t.setSeriesOption("label.fontWeight",a.weight,e()),a.style&&t.setSeriesOption("label.fontStyle",a.style,e()),a.size&&t.setSeriesOption("label.fontSize",a.size,e()),a.color&&t.setSeriesOption("label.color",a.color,e())},o={name:i.translate("labels"),enabled:t.getSeriesOption("label.enabled",e())||!1,setEnabled:function(a){return t.setSeriesOption("label.enabled",a,e())},suppressEnabledCheckbox:!1,initialFont:r,setFont:n};return o}var tY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ia=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},rY=function(i){tY(t,i);function t(e,r){var n=i.call(this)||this;return n.chartOptionsService=e,n.getSelectedSeries=r,n}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-sub-level",direction:"vertical"};this.setTemplate(t.TEMPLATE,{seriesMarkersGroup:e}),this.initMarkers()},t.prototype.initMarkers=function(){var e=this,r=[{value:"square",text:"Square"},{value:"circle",text:"Circle"},{value:"cross",text:"Cross"},{value:"diamond",text:"Diamond"},{value:"plus",text:"Plus"},{value:"triangle",text:"Triangle"},{value:"heart",text:"Heart"}];this.seriesMarkerShapeSelect.addOptions(r).setLabel(this.chartTranslationService.translate("shape")).setValue(this.getSeriesOption("marker.shape")).onValueChange(function(s){return e.setSeriesOption("marker.shape",s)});var n=this.chartOptionsService.getChartType(),o=_.includes(["scatter","bubble"],n);this.seriesMarkersGroup.setTitle(this.chartTranslationService.translate("markers")).hideEnabledCheckbox(o).setEnabled(this.getSeriesOption("marker.enabled")||!1).hideOpenCloseIcons(!0).onEnableChange(function(s){return e.setSeriesOption("marker.enabled",s)});var a=function(s,l,u,c){var p=e.getSeriesOption(s);l.setLabel(e.chartTranslationService.translate(u)).setMaxValue(mr(p,c)).setValue(""+p).setTextFieldWidth(45).onValueChange(function(h){return e.setSeriesOption(s,h)})};n==="bubble"?(a("marker.maxSize",this.seriesMarkerMinSizeSlider,"maxSize",60),a("marker.size",this.seriesMarkerSizeSlider,"minSize",60)):(this.seriesMarkerMinSizeSlider.setDisplayed(!1),a("marker.size",this.seriesMarkerSizeSlider,"size",60)),a("marker.strokeWidth",this.seriesMarkerStrokeWidthSlider,"strokeWidth",10)},t.prototype.getSeriesOption=function(e){return this.chartOptionsService.getSeriesOption(e,this.getSelectedSeries())},t.prototype.setSeriesOption=function(e,r){this.chartOptionsService.setSeriesOption(e,r,this.getSelectedSeries())},t.TEMPLATE=`<div>
            <ag-group-component ref="seriesMarkersGroup">
                <ag-select ref="seriesMarkerShapeSelect"></ag-select>
                <ag-slider ref="seriesMarkerMinSizeSlider"></ag-slider>
                <ag-slider ref="seriesMarkerSizeSlider"></ag-slider>
                <ag-slider ref="seriesMarkerStrokeWidthSlider"></ag-slider>
            </ag-group-component>
        </div>`,Ia([I("seriesMarkersGroup")],t.prototype,"seriesMarkersGroup",void 0),Ia([I("seriesMarkerShapeSelect")],t.prototype,"seriesMarkerShapeSelect",void 0),Ia([I("seriesMarkerSizeSlider")],t.prototype,"seriesMarkerSizeSlider",void 0),Ia([I("seriesMarkerMinSizeSlider")],t.prototype,"seriesMarkerMinSizeSlider",void 0),Ia([I("seriesMarkerStrokeWidthSlider")],t.prototype,"seriesMarkerStrokeWidthSlider",void 0),Ia([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),Ia([B],t.prototype,"init",null),t}(ee),nY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),tl=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},iY=function(i){nY(t,i);function t(e,r){var n=i.call(this)||this;return n.chartOptionsService=e,n.getSelectedSeries=r,n}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-sub-level",direction:"vertical"};this.setTemplate(t.TEMPLATE,{calloutGroup:e}),this.initCalloutOptions()},t.prototype.initCalloutOptions=function(){var e=this;this.calloutGroup.setTitle(this.chartTranslationService.translate("callout")).setEnabled(!0).hideOpenCloseIcons(!0).hideEnabledCheckbox(!0);var r=function(n,o,a,s){var l=e.chartOptionsService.getSeriesOption(n,e.getSelectedSeries());o.setLabel(e.chartTranslationService.translate(a)).setMaxValue(mr(l,s)).setValue(""+l).setTextFieldWidth(45).onValueChange(function(u){return e.chartOptionsService.setSeriesOption(n,u,e.getSelectedSeries())})};r("callout.length",this.calloutLengthSlider,"length",40),r("callout.strokeWidth",this.calloutStrokeWidthSlider,"strokeWidth",10),r("label.offset",this.labelOffsetSlider,"offset",30)},t.TEMPLATE=`<div>
            <ag-group-component ref="calloutGroup">
                <ag-slider ref="calloutLengthSlider"></ag-slider>
                <ag-slider ref="calloutStrokeWidthSlider"></ag-slider>
                <ag-slider ref="labelOffsetSlider"></ag-slider>
            </ag-group-component>
        </div>`,tl([I("calloutGroup")],t.prototype,"calloutGroup",void 0),tl([I("calloutLengthSlider")],t.prototype,"calloutLengthSlider",void 0),tl([I("calloutStrokeWidthSlider")],t.prototype,"calloutStrokeWidthSlider",void 0),tl([I("labelOffsetSlider")],t.prototype,"labelOffsetSlider",void 0),tl([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),tl([B],t.prototype,"init",null),t}(ee),oY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),$v=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Po=function(i){oY(t,i);function t(e,r,n){var o=i.call(this)||this;return o.chartController=e,o.chartOptionsService=r,o.activePanels=[],o.widgetFuncs={lineWidth:function(){return o.initLineWidth()},strokeWidth:function(){return o.initStrokeWidth()},lineDash:function(){return o.initLineDash()},lineOpacity:function(){return o.initLineOpacity()},fillOpacity:function(){return o.initFillOpacity()},markers:function(){return o.initMarkers()},labels:function(){return o.initLabels()},shadow:function(){return o.initShadow()},tooltips:function(){return o.initTooltips()},bins:function(){return o.initBins()}},o.seriesWidgetMappings={area:["tooltips","lineWidth","lineDash","lineOpacity","fillOpacity","markers","labels","shadow"],bar:["tooltips","strokeWidth","lineDash","lineOpacity","fillOpacity","labels","shadow"],column:["tooltips","strokeWidth","lineDash","lineOpacity","fillOpacity","labels","shadow"],line:["tooltips","lineWidth","lineDash","lineOpacity","markers","labels"],histogram:["tooltips","bins","strokeWidth","lineDash","lineOpacity","fillOpacity","labels","shadow"],scatter:["tooltips","markers","labels"],pie:["tooltips","strokeWidth","lineOpacity","fillOpacity","labels","shadow"]},o.seriesType=n||o.getChartSeriesType(),o}return t.prototype.init=function(){var e={cssIdentifier:"charts-format-top-level",direction:"vertical"};this.setTemplate(t.TEMPLATE,{seriesGroup:e}),this.seriesGroup.setTitle(this.translate("series")).toggleGroupExpand(!1).hideEnabledCheckbox(!0),this.addManagedListener(this.chartController,fs.EVENT_CHART_SERIES_CHART_TYPE_CHANGED,this.refreshWidgets.bind(this)),this.refreshWidgets()},t.prototype.refreshWidgets=function(){var e=this;this.destroyActivePanels(),this.chartController.isComboChart()&&(this.updateSeriesType(),this.initSeriesSelect()),this.seriesWidgetMappings[this.seriesType].forEach(function(r){return e.widgetFuncs[r]()})},t.prototype.initSeriesSelect=function(){var e=this,r=this.seriesGroup.createManagedBean(new Kc);r.setLabel(this.translate("seriesType")).setLabelAlignment("left").setLabelWidth("flex").setInputWidth(100).addOptions(this.getSeriesSelectOptions()).setValue(""+this.seriesType).onValueChange(function(n){e.seriesType=n,e.refreshWidgets()}),this.seriesGroup.addItem(r),this.activePanels.push(r)},t.prototype.initTooltips=function(){var e=this,r=this.createBean(new sf);r.setLabel(this.translate("tooltips")).setLabelAlignment("left").setLabelWidth("flex").setInputWidth(45).setValue(this.getSeriesOption("tooltip.enabled")||!1).onValueChange(function(n){return e.setSeriesOption("tooltip.enabled",n)}),this.addWidget(r)},t.prototype.initStrokeWidth=function(){var e=this,r=this.getSeriesOption("strokeWidth"),n=this.createBean(new zi);n.setLabel(this.translate("strokeWidth")).setMaxValue(mr(r,10)).setTextFieldWidth(45).setValue(""+r).onValueChange(function(o){return e.setSeriesOption("strokeWidth",o)}),this.addWidget(n)},t.prototype.initLineWidth=function(){var e=this,r=this.getSeriesOption("strokeWidth"),n=this.createBean(new zi);n.setLabel(this.translate("lineWidth")).setMaxValue(mr(r,10)).setTextFieldWidth(45).setValue(""+r).onValueChange(function(o){return e.setSeriesOption("strokeWidth",o)}),this.addWidget(n)},t.prototype.initLineDash=function(){var e=this,r=this.getSeriesOption("lineDash"),n=r?r[0]:0,o=this.createBean(new zi);o.setLabel(this.translate("lineDash")).setMaxValue(mr(n,30)).setTextFieldWidth(45).setValue(""+n).onValueChange(function(a){return e.setSeriesOption("lineDash",[a])}),this.addWidget(o)},t.prototype.initLineOpacity=function(){var e=this,r=this.getSeriesOption("strokeOpacity"),n=this.createBean(new zi);n.setLabel(this.translate("strokeOpacity")).setStep(.05).setMaxValue(mr(r,1)).setTextFieldWidth(45).setValue(""+r).onValueChange(function(o){return e.setSeriesOption("strokeOpacity",o)}),this.addWidget(n)},t.prototype.initFillOpacity=function(){var e=this,r=this.getSeriesOption("fillOpacity"),n=this.createBean(new zi);n.setLabel(this.translate("fillOpacity")).setStep(.05).setMaxValue(mr(r,1)).setTextFieldWidth(45).setValue(""+r).onValueChange(function(o){return e.setSeriesOption("fillOpacity",o)}),this.addWidget(n)},t.prototype.initLabels=function(){var e=this,r=eY(this.chartTranslationService,this.chartOptionsService,function(){return e.seriesType}),n=this.createBean(new Af(r));if(this.seriesType==="pie"){var o=this.createBean(new iY(this.chartOptionsService,function(){return e.seriesType}));n.addCompToPanel(o),this.activePanels.push(o)}this.addWidget(n)},t.prototype.initShadow=function(){var e=this,r=this.createBean(new JK(this.chartOptionsService,function(){return e.seriesType}));this.addWidget(r)},t.prototype.initMarkers=function(){var e=this,r=this.createBean(new rY(this.chartOptionsService,function(){return e.seriesType}));this.addWidget(r)},t.prototype.initBins=function(){var e=this,r=this.getSeriesOption("binCount"),n=this.createBean(new zi);n.setLabel(this.translate("histogramBinCount")).setMinValue(4).setMaxValue(mr(r,100)).setTextFieldWidth(45).setValue(""+r).onValueChange(function(o){return e.setSeriesOption("binCount",o)}),this.addWidget(n)},t.prototype.addWidget=function(e){this.seriesGroup.addItem(e),this.activePanels.push(e)},t.prototype.getSeriesOption=function(e){return this.chartOptionsService.getSeriesOption(e,this.seriesType)},t.prototype.setSeriesOption=function(e,r){this.chartOptionsService.setSeriesOption(e,r,this.seriesType)},t.prototype.getChartSeriesType=function(){var e=this.chartController.getSeriesChartTypes()[0].chartType;return e==="columnLineCombo"?"column":e==="areaColumnCombo"?"area":Mi(e)},t.prototype.getSeriesSelectOptions=function(){var e=this;this.seriesSelectOptions||(this.seriesSelectOptions=new Map([["area",{value:"area",text:this.translate("area","Area")}],["bar",{value:"bar",text:this.translate("bar","Bar")}],["column",{value:"column",text:this.translate("column","Column")}],["line",{value:"line",text:this.translate("line","Line")}],["scatter",{value:"scatter",text:this.translate("scatter","Scatter")}],["histogram",{value:"histogram",text:this.translate("histogram","Histogram")}],["pie",{value:"pie",text:this.translate("pie","Pie")}]]));var r=new Set;return this.chartController.getActiveSeriesChartTypes().forEach(function(n){var o=Mi(n.chartType);r.add(e.seriesSelectOptions.get(o))}),Array.from(r)},t.prototype.updateSeriesType=function(){var e=this.chartController.getActiveSeriesChartTypes().map(function(n){return Mi(n.chartType)}),r=!e.includes(this.seriesType);r&&e.length>0&&(this.seriesType=e[0])},t.prototype.translate=function(e,r){return this.chartTranslationService.translate(e,r)},t.prototype.destroyActivePanels=function(){var e=this;this.activePanels.forEach(function(r){_.removeFromParent(r.getGui()),e.destroyBean(r)})},t.prototype.destroy=function(){this.destroyActivePanels(),i.prototype.destroy.call(this)},t.TEMPLATE=`<div>
            <ag-group-component ref="seriesGroup">                
            </ag-group-component>
        </div>`,$v([I("seriesGroup")],t.prototype,"seriesGroup",void 0),$v([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),$v([B],t.prototype,"init",null),t}(ee),aY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),sY=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o};function mr(i,t){return Math.max(i,t)}var lY=function(i){aY(t,i);function t(e,r){var n=i.call(this,t.TEMPLATE)||this;return n.chartController=e,n.chartOptionsService=r,n.panels=[],n}return t.prototype.init=function(){this.createPanels(),this.addManagedListener(this.chartController,fs.EVENT_CHART_UPDATED,this.createPanels.bind(this))},t.prototype.createPanels=function(){var e=this.chartController.getChartType(),r=this.chartController.isGrouping();if(!(e===this.chartType&&r===this.isGrouping)){switch(this.destroyPanels(),this.addComponent(new ZK(this.chartOptionsService)),this.addComponent(new WK(this.chartOptionsService)),e){case"groupedColumn":case"stackedColumn":case"normalizedColumn":this.addComponent(new Aa(this.chartController,this.chartOptionsService)),this.addComponent(new Po(this.chartController,this.chartOptionsService,"column")),this.addComponent(new Da(this.chartOptionsService));break;case"groupedBar":case"stackedBar":case"normalizedBar":this.addComponent(new Aa(this.chartController,this.chartOptionsService)),this.addComponent(new Po(this.chartController,this.chartOptionsService,"bar")),this.addComponent(new Da(this.chartOptionsService));break;case"pie":case"doughnut":this.addComponent(new Po(this.chartController,this.chartOptionsService,"pie"));break;case"line":this.addComponent(new Aa(this.chartController,this.chartOptionsService)),this.addComponent(new Po(this.chartController,this.chartOptionsService,"line")),this.addComponent(new Da(this.chartOptionsService));break;case"scatter":case"bubble":this.addComponent(new Aa(this.chartController,this.chartOptionsService)),this.addComponent(new Po(this.chartController,this.chartOptionsService,"scatter")),this.addComponent(new Da(this.chartOptionsService));break;case"area":case"stackedArea":case"normalizedArea":this.addComponent(new Aa(this.chartController,this.chartOptionsService)),this.addComponent(new Po(this.chartController,this.chartOptionsService,"area")),this.addComponent(new Da(this.chartOptionsService));break;case"histogram":this.addComponent(new Aa(this.chartController,this.chartOptionsService)),this.addComponent(new Po(this.chartController,this.chartOptionsService,"histogram")),this.addComponent(new Da(this.chartOptionsService));break;case"columnLineCombo":case"areaColumnCombo":case"customCombo":this.addComponent(new Aa(this.chartController,this.chartOptionsService)),this.addComponent(new Po(this.chartController,this.chartOptionsService)),this.addComponent(new Da(this.chartOptionsService));break;default:console.warn("AG Grid: ChartFormattingPanel - unexpected chart type index: "+e+" supplied")}this.chartType=e,this.isGrouping=r}},t.prototype.addComponent=function(e){this.createBean(e),this.panels.push(e),e.addCssClass("ag-chart-format-section"),this.getGui().appendChild(e.getGui())},t.prototype.destroyPanels=function(){var e=this;this.panels.forEach(function(r){_.removeFromParent(r.getGui()),e.destroyBean(r)})},t.prototype.destroy=function(){this.destroyPanels(),i.prototype.destroy.call(this)},t.TEMPLATE='<div class="ag-chart-format-wrapper"></div>',sY([B],t.prototype,"init",null),t}(ee),uY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),i0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},cb=function(i){uY(t,i);function t(e,r){var n=i.call(this)||this;n.size=58,n.padding=5,n.root=new It;var o=new TA(window.document,n.size,n.size);return o.canvas.element.classList.add("ag-chart-mini-thumbnail-canvas"),o.root=n.root,o.container=e,n.scene=o,n.tooltipName=r,n}return t.prototype.init=function(){this.scene.canvas.element.title=this.chartTranslationService.translate(this.tooltipName)},i0([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),i0([B],t.prototype,"init",null),t}(ee),cY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),pY=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Vn=function(i){cY(t,i);function t(e,r){var n=i.call(this,e,r)||this;return n.stroke="gray",n.axisOvershoot=3,n}return t.prototype.addAxes=function(){var e=this.size,r=this.padding,n=new ri;n.x1=r,n.y1=r,n.x2=r,n.y2=e-r+this.axisOvershoot,n.stroke=this.stroke;var o=new ri;o.x1=r-this.axisOvershoot+1,o.y1=e-r,o.x2=e-r+1,o.y2=e-r,o.stroke=this.stroke;var a=this.root;a.append(n),a.append(o)},pY([B],t.prototype,"addAxes",null),t}(cb);function sp(i){var t=i.stacked,e=i.size,r=i.padding,n=i.xScalePadding,o=i.xScaleDomain,a=i.yScaleDomain,s=new Ni;s.domain=o,s.range=[r,e-r],s.paddingInner=n,s.paddingOuter=n;var l=new Nt;l.domain=a,l.range=[e-r,r];var u=function(c,p,h){return c.map(function(d,f){var g=h.convert(d),v=new fo;return v.x=p.convert(f),v.y=g,v.width=p.bandwidth,v.height=h.convert(0)-g,v.strokeWidth=1,v.crisp=!0,v})};return t?i.data.map(function(c){return u(c,s,l)}):u(i.data,s,l)}function pb(i,t,e,r){var n=new Nt;n.domain=[0,4],n.range=[r,e-r];var o=new Nt;o.domain=[0,10],o.range=[e-r,r];var a=t.map(function(l){var u=new $r;return u.strokeWidth=3,u.lineCap="round",u.fill=void 0,l.forEach(function(c,p){u.path[p>0?"lineTo":"moveTo"](n.convert(p),o.convert(c))}),u}),s=new ip;return s.x=s.y=r,s.width=s.height=e-r*2,s.append(a),i.append(s),a}var hY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),dY=function(i){hY(t,i);function t(e,r,n){var o=i.call(this,e,"groupedColumnTooltip")||this;o.columnData=[2,3,4];var a=o,s=a.root,l=a.columnData,u=a.size,c=a.padding;return o.columns=sp({stacked:!1,root:s,data:l,size:u,padding:c,xScaleDomain:[0,1,2],yScaleDomain:[0,4],xScalePadding:.3}),s.append(o.columns),o.updateColors(r,n),o}return t.prototype.updateColors=function(e,r){this.columns.forEach(function(n,o){n.fill=e[o],n.stroke=r[o]})},t.chartType="groupedColumn",t}(Vn),fY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),QA=function(i){fY(t,i);function t(e,r,n,o,a,s){o===void 0&&(o=t.data),a===void 0&&(a=[0,16]),s===void 0&&(s="stackedColumnTooltip");var l=i.call(this,e,s)||this,u=l,c=u.root,p=u.size,h=u.padding;return l.stackedColumns=sp({stacked:!0,root:c,data:o,size:p,padding:h,xScaleDomain:[0,1,2],yScaleDomain:a,xScalePadding:.3}),c.append([].concat.apply([],l.stackedColumns)),l.updateColors(r,n),l}return t.prototype.updateColors=function(e,r){this.stackedColumns.forEach(function(n,o){return n.forEach(function(a){a.fill=e[o],a.stroke=r[o]})})},t.chartType="stackedColumn",t.data=[[8,12,16],[6,9,12],[2,3,4]],t}(Vn),gY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),vY=function(i){gY(t,i);function t(e,r,n){return i.call(this,e,r,n,t.data,[0,10],"normalizedColumnTooltip")||this}return t.chartType="normalizedColumn",t.data=[[10,10,10],[6,7,8],[2,4,6]],t}(QA),mY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),yY=function(i){mY(t,i);function t(e,r,n){var o=i.call(this,e,"groupedBarTooltip")||this,a=o.padding,s=o.size,l=[2,3,4],u=new Ni;u.domain=[0,1,2],u.range=[a,s-a],u.paddingInner=.3,u.paddingOuter=.3;var c=new Nt;c.domain=[0,4],c.range=[s-a,a];var p=c.convert(0),h=u.bandwidth;return o.bars=l.map(function(d,f){var g=new fo;return g.x=a,g.y=u.convert(f),g.width=p-c.convert(d),g.height=h,g.strokeWidth=1,g.crisp=!0,g}),o.updateColors(r,n),o.root.append(o.bars),o}return t.prototype.updateColors=function(e,r){this.bars.forEach(function(n,o){n.fill=e[o],n.stroke=r[o]})},t.chartType="groupedBar",t}(Vn),CY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),JA=function(i){CY(t,i);function t(e,r,n,o,a,s){o===void 0&&(o=t.data),a===void 0&&(a=[0,16]),s===void 0&&(s="stackedBarTooltip");var l=i.call(this,e,s)||this,u=l.size,c=l.padding,p=new Ni;p.domain=[0,1,2],p.range=[c,u-c],p.paddingInner=.3,p.paddingOuter=.3;var h=new Nt;h.domain=a,h.range=[u-c,c];var d=h.convert(0),f=p.bandwidth;return l.bars=o.map(function(g){return g.map(function(v,m){var C=new fo;return C.x=c,C.y=p.convert(m),C.width=d-h.convert(v),C.height=f,C.strokeWidth=1,C.crisp=!0,C})}),l.updateColors(r,n),l.root.append([].concat.apply([],l.bars)),l}return t.prototype.updateColors=function(e,r){this.bars.forEach(function(n,o){return n.forEach(function(a){a.fill=e[o],a.stroke=r[o]})})},t.chartType="stackedBar",t.data=[[8,12,16],[6,9,12],[2,3,4]],t}(Vn),wY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),bY=function(i){wY(t,i);function t(e,r,n){return i.call(this,e,r,n,t.data,[0,10],"normalizedBarTooltip")||this}return t.chartType="normalizedBar",t.data=[[10,10,10],[6,7,8],[2,4,6]],t}(JA),_Y=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),SY=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},eD=function(i){_Y(t,i);function t(e,r,n,o,a){o===void 0&&(o=.6),a===void 0&&(a="doughnutTooltip");var s=i.call(this,e,a)||this,l=(s.size-s.padding*2)/2,u=l+s.padding,c=[[Mt(-90),Mt(30)],[Mt(30),Mt(120)],[Mt(120),Mt(180)],[Mt(180),Mt(210)],[Mt(210),Mt(240)],[Mt(240),Mt(270)]];return s.sectors=c.map(function(p){var h=SY(p,2),d=h[0],f=h[1],g=new HA;return g.centerX=u,g.centerY=u,g.innerRadius=l*o,g.outerRadius=l,g.startAngle=d,g.endAngle=f,g.stroke=void 0,g.strokeWidth=1,g}),s.updateColors(r,n),s.root.append(s.sectors),s}return t.prototype.updateColors=function(e,r){this.sectors.forEach(function(n,o){n.fill=e[o%e.length],n.stroke=r[o%r.length]})},t.chartType="doughnut",t}(cb),xY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),TY=function(i){xY(t,i);function t(e,r,n){return i.call(this,e,r,n,0,"pieTooltip")||this}return t.chartType="pie",t}(eD),OY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),EY=function(i){OY(t,i);function t(e,r,n){var o=i.call(this,e,"lineTooltip")||this;return o.data=[[9,7,8,5,6],[5,6,3,4,1],[1,3,4,8,7]],o.lines=pb(o.root,o.data,o.size,o.padding),o.updateColors(r,n),o}return t.prototype.updateColors=function(e,r){this.lines.forEach(function(n,o){n.stroke=e[o]})},t.chartType="line",t}(Vn),RY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),PY=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},AY=function(i){RY(t,i);function t(e,r,n){var o=i.call(this,e,"scatterTooltip")||this,a=o.size,s=o.padding,l=[[[.3,3],[1.1,.9],[2,.4],[3.4,2.4]],[[0,.3],[1,2],[2.4,1.4],[3,0]]],u=new Nt;u.domain=[-.5,4],u.range=[s*2,a-s];var c=new Nt;c.domain=[-.5,3.5],c.range=[a-s,s];var p=[];l.forEach(function(d){d.forEach(function(f){var g=PY(f,2),v=g[0],m=g[1],C=new Qw;C.strokeWidth=1,C.centerX=u.convert(v),C.centerY=c.convert(m),C.radiusX=C.radiusY=2.5,p.push(C)})}),o.points=p,o.updateColors(r,n);var h=new ip;return h.x=h.y=s,h.width=h.height=a-s*2,h.append(o.points),o.root.append(h),o}return t.prototype.updateColors=function(e,r){this.points.forEach(function(n,o){n.stroke=r[o%r.length],n.fill=e[o%e.length]})},t.chartType="scatter",t}(Vn),DY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),MY=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},FY=function(i){DY(t,i);function t(e,r,n){var o=i.call(this,e,"bubbleTooltip")||this,a=o.size,s=o.padding,l=[[[.1,.3,5],[.5,.4,7],[.2,.8,7]],[[.8,.7,5],[.7,.3,9]]],u=new Nt;u.domain=[0,1],u.range=[s*2,a-s];var c=new Nt;c.domain=[0,1],c.range=[a-s,s];var p=[];l.forEach(function(d){d.forEach(function(f){var g=MY(f,3),v=g[0],m=g[1],C=g[2],w=new Qw;w.strokeWidth=1,w.centerX=u.convert(v),w.centerY=c.convert(m),w.radiusX=w.radiusY=C,w.fillOpacity=.7,p.push(w)})}),o.points=p,o.updateColors(r,n);var h=new ip;return h.x=s,h.y=s,h.width=a-s*2,h.height=a-s*2,h.append(o.points),o.root.append(h),o}return t.prototype.updateColors=function(e,r){this.points.forEach(function(n,o){n.stroke=r[o%r.length],n.fill=e[o%e.length]})},t.chartType="bubble",t}(Vn),IY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),LY=function(i){IY(t,i);function t(e,r,n,o){o===void 0&&(o=t.data);var a=i.call(this,e,"groupedAreaTooltip")||this,s=a.size,l=a.padding,u=new Ni;u.domain=[0,1,2],u.paddingInner=1,u.paddingOuter=0,u.range=[l+.5,s-l-.5];var c=new Nt;c.domain=[0,6],c.range=[s-l+.5,l];var p=o.length,h=p*2-1,d=[],f=c.convert(0);return o.forEach(function(g,v){var m=u.convert(v);g.forEach(function(C,w){var b=c.convert(C),S=d[w]||(d[w]=[]);S[v]={x:m,y:b},S[h-v]={x:m,y:f}})}),a.areas=d.reverse().map(function(g){var v=new $r;v.strokeWidth=1,v.fillOpacity=.7;var m=v.path;return m.clear(),g.forEach(function(C,w){return m[w>0?"lineTo":"moveTo"](C.x,C.y)}),m.closePath(),v}),a.updateColors(r,n),a.root.append(a.areas),a}return t.prototype.updateColors=function(e,r){this.areas.forEach(function(n,o){n.fill=e[o],n.stroke=r[o]})},t.chartType="area",t.data=[[1,3,5],[2,6,4],[5,3,1]],t}(Vn),NY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ky=function(i){NY(t,i);function t(e,r,n,o,a){o===void 0&&(o=t.data),a===void 0&&(a="stackedAreaTooltip");var s=i.call(this,e,a)||this,l=s.size,u=s.padding,c=new Ni;c.domain=[0,1,2],c.paddingInner=1,c.paddingOuter=0,c.range=[u+.5,l-u-.5];var p=new Nt;p.domain=[0,16],p.range=[l-u+.5,u+.5];var h=o.length,d=h*2-1,f=[];return o.forEach(function(g,v){var m=c.convert(v),C=0;g.forEach(function(w,b){var S=p.convert(C+w),T=f[b]||(f[b]=[]);T[v]={x:m,y:S},T[d-v]={x:m,y:p.convert(C)},C+=w})}),s.areas=f.map(function(g){var v=new $r;v.strokeWidth=1;var m=v.path;return m.clear(),g.forEach(function(C,w){return m[w>0?"lineTo":"moveTo"](C.x,C.y)}),m.closePath(),v}),s.updateColors(r,n),s.root.append(s.areas),s}return t.prototype.updateColors=function(e,r){this.areas.forEach(function(n,o){n.fill=e[o],n.stroke=r[o]})},t.chartType="stackedArea",t.data=[[2,3,2],[3,6,5],[6,2,2]],t}(Vn),GY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),kY=function(i){GY(t,i);function t(e,r,n,o){return o===void 0&&(o=t.data),i.call(this,e,r,n,o,"normalizedAreaTooltip")||this}return t.chartType="normalizedArea",t.data=ky.data.map(function(e){var r=e.reduce(function(n,o){return n+o},0);return e.map(function(n){return n/r*16})}),t}(ky),WY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),o0=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},VY=function(i){WY(t,i);function t(e,r,n){var o=i.call(this,e,"histogramTooltip")||this,a=o.padding,s=o.size,l=[2,5,11,13,10,6,1],u=new Nt;u.domain=[0,l.length],u.range=[a,s-a];var c=new Nt;c.domain=[0,l.reduce(function(h,d){return Math.max(h,d)},0)],c.range=[s-a,a];var p=c.convert(0);return o.bars=l.map(function(h,d){var f=c.convert(h),g=u.convert(d),v=u.convert(d+1),m=new fo;return m.x=g,m.y=f,m.width=v-g,m.height=p-f,m.strokeWidth=1,m.crisp=!0,m}),o.updateColors(r,n),o.root.append(o.bars),o}return t.prototype.updateColors=function(e,r){var n=o0(e,1),o=n[0],a=o0(r,1),s=a[0];this.bars.forEach(function(l){l.fill=o,l.stroke=s})},t.chartType="histogram",t}(Vn),BY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),HY=function(i){BY(t,i);function t(e,r,n){var o=i.call(this,e,"columnLineComboTooltip")||this;o.columnData=[3,4],o.lineData=[[5,4,6,5,4]];var a=o,s=a.root,l=a.columnData,u=a.lineData,c=a.size,p=a.padding;return o.columns=sp({stacked:!1,root:s,data:l,size:c,padding:p,xScaleDomain:[0,1],yScaleDomain:[0,4],xScalePadding:.5}),s.append(o.columns),o.lines=pb(s,u,c,p),o.updateColors(r,n),o}return t.prototype.updateColors=function(e,r){this.columns.forEach(function(n,o){n.fill=e[o],n.stroke=r[o]}),this.lines.forEach(function(n,o){n.stroke=e[o+2]})},t.chartType="columnLineCombo",t}(Vn),jY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),UY=function(i){jY(t,i);function t(e,r,n){var o=i.call(this,e,"areaColumnComboTooltip")||this;o.columnData=[3,4.5],o.areaData=[[5,4,6,5,4]];var a=o,s=a.root,l=a.columnData,u=a.areaData,c=a.size,p=a.padding;o.columns=sp({stacked:!1,root:s,data:l,size:c,padding:p,xScaleDomain:[0,1],yScaleDomain:[0,6],xScalePadding:.5});var h=new Ni;h.range=[p,c-p],h.domain=[0,1,2,3,4],h.paddingInner=1,h.paddingOuter=0;var d=new Nt;d.range=[c-p,p],d.domain=[0,6];var f=[],g=d.convert(0),v=h.convert(0);return u.forEach(function(m,C){var w=f[C]||(f[C]=[]);m.forEach(function(S,T){var E=S,P=T,R=h.convert(P),M=d.convert(E);w[T]={x:R,y:M}});var b=h.convert(m.length-1);f[C].push({x:b,y:g},{x:v,y:g})}),o.areas=f.map(function(m){var C=new $r;C.strokeWidth=1,C.fillOpacity=.8;var w=C.path;return m.forEach(function(b,S){return w[S>0?"lineTo":"moveTo"](b.x,b.y)}),C}),s.append(o.areas),s.append([].concat.apply([],o.columns)),o.updateColors(r,n),o}return t.prototype.updateColors=function(e,r){this.areas.forEach(function(n,o){n.fill=e[o],n.stroke=r[o]}),this.columns.forEach(function(n,o){n.fill=e[o+1],n.stroke=r[o+1]})},t.chartType="areaColumnCombo",t}(Vn),zY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),a0=function(i){zY(t,i);function t(e,r,n){var o=i.call(this,e,"customComboTooltip")||this;o.columnData=[3,4],o.lineData=[[5,4,6,5,4]];var a=o,s=a.root,l=a.columnData,u=a.lineData,c=a.size,p=a.padding;o.columns=sp({stacked:!1,root:s,data:l,size:c,padding:p,xScaleDomain:[0,1],yScaleDomain:[0,4],xScalePadding:.5}),s.append(o.columns),o.lines=pb(s,u,c,p);var h="grey",d=3,f=new ri;f.x1=p,f.y1=p,f.x2=p,f.y2=c-p+d,f.stroke=h;var g=new ri;g.x1=p-d+1,g.y1=c-p,g.x2=c-p+1,g.y2=c-p,g.stroke=h;var v=new $r;return v.svgPath="M25.76,43.46l5.51,5.07M49.86,22a3.26,3.26,0,0,0-3-.59,6.78,6.78,0,0,0-3.35,2.14l-18,20.25-.08.09-2.42,8-.18.57,8.19-3.6,18-20.34a6.83,6.83,0,0,0,1.73-3.59A3.29,3.29,0,0,0,49.86,22Zm-8.1,3.5,5.58,5m-6.6-3.85,5.51,5.06",v.fill="whitesmoke",v.stroke="darkslategrey",v.strokeWidth=1,s.append([g,f,v]),o.updateColors(r,n),o}return t.prototype.updateColors=function(e,r){this.columns.forEach(function(n,o){n.fill=e[o],n.stroke=r[o]}),this.lines.forEach(function(n,o){n.stroke=e[o+2]})},t.chartType="customCombo",t}(cb),$Y=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),s0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},KY=function(i){$Y(t,i);function t(e,r,n){var o=i.call(this,t.TEMPLATE)||this;return o.wrappers={},o.chartGroups={columnGroup:[dY,QA,vY],barGroup:[yY,JA,bY],pieGroup:[TY,eD],lineGroup:[EY],scatterGroup:[AY,FY],areaGroup:[LY,ky,kY],histogramGroup:[VY],combinationGroup:[HY,UY,a0]},o.chartController=e,o.fills=r,o.strokes=n,o}return t.prototype.init=function(){var e=this;this.chartController.customComboExists()||(this.chartGroups.combinationGroup=this.chartGroups.combinationGroup.filter(function(n){return n!==a0}));var r=this.getGui();Object.keys(this.chartGroups).forEach(function(n){var o=e.chartGroups[n],a=e.createBean(new Qn({title:e.chartTranslationService.translate(n),suppressEnabledCheckbox:!0,enabled:!0,suppressOpenCloseIcons:!0,cssIdentifier:"charts-settings",direction:"horizontal"}));o.forEach(function(s){var l=document.createElement("div");l.classList.add("ag-chart-mini-thumbnail"),e.addManagedListener(l,"click",function(){e.chartController.setChartType(s.chartType),e.updateSelectedMiniChart()}),e.wrappers[s.chartType]=l,e.createBean(new s(l,e.fills,e.strokes)),a.addItem(l)}),r.appendChild(a.getGui())}),this.updateSelectedMiniChart()},t.prototype.updateSelectedMiniChart=function(){var e=this.chartController.getChartType();for(var r in this.wrappers){var n=this.wrappers[r],o=r===e;n.classList.toggle("ag-selected",o)}},t.TEMPLATE='<div class="ag-chart-settings-mini-wrapper"></div>',s0([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),s0([B],t.prototype,"init",null),t}(ee),YY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),La=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},XY=function(i){YY(t,i);function t(e){var r=i.call(this,t.TEMPLATE)||this;return r.miniCharts=[],r.cardItems=[],r.activePaletteIndex=0,r.palettes=[],r.themes=[],r.chartController=e,r}return t.prototype.postConstruct=function(){var e=this;this.resetPalettes(),this.ePrevBtn.insertAdjacentElement("afterbegin",_.createIconNoSpan("previous",this.gridOptionsWrapper)),this.eNextBtn.insertAdjacentElement("afterbegin",_.createIconNoSpan("next",this.gridOptionsWrapper)),this.addManagedListener(this.ePrevBtn,"click",function(){return e.setActivePalette(e.getPrev(),"left")}),this.addManagedListener(this.eNextBtn,"click",function(){return e.setActivePalette(e.getNext(),"right")}),this.addManagedListener(this.chartController,fs.EVENT_CHART_TYPE_CHANGED,function(){return e.resetPalettes(!0)}),this.scrollSelectedIntoView()},t.prototype.scrollSelectedIntoView=function(){var e=this;setTimeout(function(){var r=e.miniCharts.find(function(o){return!o.getGui().classList.contains("ag-hidden")}),n=r.getGui().querySelector(".ag-selected");n&&n.scrollIntoView({block:"nearest"})},250)},t.prototype.resetPalettes=function(e){var r=this,n=this.chartController.getPalettes();_.shallowCompare(n,this.palettes)&&!e||this.isAnimating||(this.palettes=n,this.themes=this.chartController.getThemes(),this.activePaletteIndex=this.themes.findIndex(function(o){return o===r.chartController.getChartThemeName()}),this.cardItems=[],_.clearElement(this.eCardSelector),this.destroyMiniCharts(),this.palettes.forEach(function(o,a){var s=r.activePaletteIndex===a,l=o.fills,u=o.strokes,c=r.createBean(new KY(r.chartController,l,u));r.miniCharts.push(c),r.eMiniChartsContainer.appendChild(c.getGui()),r.addCardLink(a),s?c.updateSelectedMiniChart():c.addCssClass("ag-hidden")}),this.eNavBar.classList.toggle("ag-hidden",this.palettes.length<=1),_.radioCssClass(this.cardItems[this.activePaletteIndex],"ag-selected","ag-not-selected"))},t.prototype.addCardLink=function(e){var r=this,n=document.createElement("div");n.classList.add("ag-chart-settings-card-item"),this.addManagedListener(n,"click",function(){r.setActivePalette(e,e<r.activePaletteIndex?"left":"right")}),this.eCardSelector.appendChild(n),this.cardItems.push(n)},t.prototype.getPrev=function(){var e=this.activePaletteIndex-1;return e<0&&(e=this.palettes.length-1),e},t.prototype.getNext=function(){var e=this.activePaletteIndex+1;return e>=this.palettes.length&&(e=0),e},t.prototype.setActivePalette=function(e,r){var n=this;if(!(this.isAnimating||this.activePaletteIndex===e)){_.radioCssClass(this.cardItems[e],"ag-selected","ag-not-selected");var o=this.miniCharts[this.activePaletteIndex],a=o.getGui(),s=this.miniCharts[e],l=s.getGui();o.updateSelectedMiniChart(),s.updateSelectedMiniChart();var u=r==="left"?-1:1,c=l.style.left=_.getAbsoluteWidth(this.getGui())*u+"px";this.activePaletteIndex=e,this.isAnimating=!0;var p="ag-animating";s.removeCssClass("ag-hidden"),o.addCssClass(p),s.addCssClass(p),this.chartController.setChartThemeName(this.themes[e]),window.setTimeout(function(){a.style.left=-parseFloat(c)+"px",l.style.left="0px"},0),window.setTimeout(function(){n.isAnimating=!1,o.removeCssClass(p),s.removeCssClass(p),o.addCssClass("ag-hidden")},300)}},t.prototype.destroyMiniCharts=function(){_.clearElement(this.eMiniChartsContainer),this.miniCharts=this.destroyBeans(this.miniCharts)},t.prototype.destroy=function(){this.destroyMiniCharts(),i.prototype.destroy.call(this)},t.TEMPLATE=`<div class="ag-chart-settings-wrapper">
            <div ref="eMiniChartsContainer" class="ag-chart-settings-mini-charts-container ag-scrollable-container"></div>
            <div ref="eNavBar" class="ag-chart-settings-nav-bar">
                <div ref="ePrevBtn" class="ag-chart-settings-prev">
                    <button type="button" class="ag-chart-settings-prev-button"></button>
                </div>
                <div ref="eCardSelector" class="ag-chart-settings-card-selector"></div>
                <div ref="eNextBtn" class="ag-chart-settings-next">
                    <button type="button" class="ag-chart-settings-next-button"></button>
                </div>
            </div>
        </div>`,La([y("resizeObserverService")],t.prototype,"resizeObserverService",void 0),La([I("eMiniChartsContainer")],t.prototype,"eMiniChartsContainer",void 0),La([I("eNavBar")],t.prototype,"eNavBar",void 0),La([I("eCardSelector")],t.prototype,"eCardSelector",void 0),La([I("ePrevBtn")],t.prototype,"ePrevBtn",void 0),La([I("eNextBtn")],t.prototype,"eNextBtn",void 0),La([B],t.prototype,"postConstruct",null),t}(ee),qY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),l0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ZY=function(i){qY(t,i);function t(e){var r=i.call(this)||this;r.tabs=[];var n=e.controller,o=e.panels,a=e.chartOptionsService;return r.chartController=n,r.chartOptionsService=a,r.panels=o,r}return t.prototype.init=function(){var e=this;this.panels.forEach(function(r){var n=r.replace("chart","").toLowerCase(),o=e.createTab(r,n,e.getPanelClass(n)),a=o.comp,s=o.tab;e.tabs.push(s),e.addDestroyFunc(function(){return e.destroyBean(a)})}),this.tabbedLayout=new rw({items:this.tabs,cssClass:"ag-chart-tabbed-menu",keepScrollPosition:!0}),this.getContext().createBean(this.tabbedLayout)},t.prototype.createTab=function(e,r,n){var o=document.createElement("div");o.classList.add("ag-chart-tab","ag-chart-"+r);var a=new n(this.chartController,this.chartOptionsService);this.getContext().createBean(a),o.appendChild(a.getGui());var s=document.createElement("div"),l=this.chartTranslationService.translate(r);return s.innerText=l,{comp:a,tab:{title:s,titleLabel:l,bodyPromise:ct.resolve(o),getScrollableContainer:function(){var u=o.querySelector(".ag-scrollable-container");return u||o},name:e}}},t.prototype.showTab=function(e){var r=this.tabs[e];this.tabbedLayout.showItem(r)},t.prototype.getGui=function(){return this.tabbedLayout&&this.tabbedLayout.getGui()},t.prototype.destroy=function(){this.parentComponent&&this.parentComponent.isAlive()&&this.destroyBean(this.parentComponent),i.prototype.destroy.call(this)},t.prototype.getPanelClass=function(e){switch(e){case t.TAB_DATA:return NK;case t.TAB_FORMAT:return lY;default:return XY}},t.TAB_DATA="data",t.TAB_FORMAT="format",l0([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),l0([B],t.prototype,"init",null),t}(ee),QY=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),u0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},JY=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},c0=function(i){QY(t,i);function t(e,r,n,o){var a=i.call(this,t.TEMPLATE)||this;return a.eChartContainer=e,a.eMenuPanelContainer=r,a.chartController=n,a.chartOptionsService=o,a.buttons={chartSettings:["menu",function(){return a.showMenu("chartSettings")}],chartData:["menu",function(){return a.showMenu("chartData")}],chartFormat:["menu",function(){return a.showMenu("chartFormat")}],chartLink:["linked",function(s){return a.toggleDetached(s)}],chartUnlink:["unlinked",function(s){return a.toggleDetached(s)}],chartDownload:["save",function(){return a.saveChart()}]},a.tabs=[],a.menuVisible=!1,a}return t.prototype.postConstruct=function(){this.createButtons(),this.refreshMenuClasses()},t.prototype.isVisible=function(){return this.menuVisible},t.prototype.getToolbarOptions=function(){var e=this,r=["chartSettings","chartData","chartFormat",this.chartController.isChartLinked()?"chartLink":"chartUnlink","chartDownload"],n=this.gridOptionsWrapper.getChartToolbarItemsFunc();if(n){var o={defaultItems:r};r=n(o).filter(function(s){return e.buttons[s]?!0:(console.warn("AG Grid: '"+s+" is not a valid Chart Toolbar Option"),!1)})}this.chartController.isPivotChart()&&(r=r.filter(function(s){return s!=="chartData"}));var a=["chartUnlink","chartLink","chartDownload"];return this.tabs=r.filter(function(s){return a.indexOf(s)===-1}),r.filter(function(s){return a.indexOf(s)!==-1||e.tabs.length&&s===e.tabs[0]})},t.prototype.toggleDetached=function(e){var r=e.target,n=r.classList.contains("ag-icon-linked");r.classList.toggle("ag-icon-linked",!n),r.classList.toggle("ag-icon-unlinked",n);var o=n?"chartUnlinkToolbarTooltip":"chartLinkToolbarTooltip",a=this.chartTranslationService.translate(o);a&&(r.title=a),this.chartController.detachChartRange()},t.prototype.createButtons=function(){var e=this,r=this.getToolbarOptions(),n=this.getGui();r.forEach(function(o){var a=e.buttons[o],s=JY(a,2),l=s[0],u=s[1],c=_.createIconNoSpan(l,e.gridOptionsWrapper,void 0,!0);c.classList.add("ag-chart-menu-icon");var p=e.chartTranslationService.translate(o+"ToolbarTooltip");p&&(c.title=p),e.addManagedListener(c,"click",u),n.appendChild(c)})},t.prototype.saveChart=function(){var e={type:t.EVENT_DOWNLOAD_CHART};this.dispatchEvent(e)},t.prototype.createMenuPanel=function(e){var r=this,n=this.gridOptionsWrapper.chartMenuPanelWidth(),o=this.menuPanel=this.createBean(new lf({minWidth:n,width:n,height:"100%",closable:!0,hideTitleBar:!0,cssIdentifier:"chart-menu"}));return o.setParentComponent(this),this.eMenuPanelContainer.appendChild(o.getGui()),this.tabbedMenu=this.createBean(new ZY({controller:this.chartController,type:this.chartController.getChartType(),panels:this.tabs,chartOptionsService:this.chartOptionsService})),this.addManagedListener(o,ee.EVENT_DESTROYED,function(){return r.destroyBean(r.tabbedMenu)}),new ct(function(a){window.setTimeout(function(){o.setBodyComponent(r.tabbedMenu),r.tabbedMenu.showTab(e),r.addManagedListener(r.eChartContainer,"click",function(s){r.getGui().contains(s.target)||r.menuVisible&&r.hideMenu()}),a(o)},100)})},t.prototype.showContainer=function(){!this.menuPanel||(this.menuVisible=!0,this.showParent(this.menuPanel.getWidth()),this.refreshMenuClasses())},t.prototype.showMenu=function(e){var r=this.tabs.indexOf(e);this.menuPanel?this.showContainer():this.createMenuPanel(r).then(this.showContainer.bind(this))},t.prototype.hideMenu=function(){var e=this;this.hideParent(),window.setTimeout(function(){e.menuVisible=!1,e.refreshMenuClasses()},500)},t.prototype.refreshMenuClasses=function(){this.eChartContainer.classList.toggle("ag-chart-menu-visible",this.menuVisible),this.eChartContainer.classList.toggle("ag-chart-menu-hidden",!this.menuVisible)},t.prototype.showParent=function(e){this.eMenuPanelContainer.style.minWidth=e+"px"},t.prototype.hideParent=function(){this.eMenuPanelContainer.style.minWidth="0"},t.prototype.destroy=function(){i.prototype.destroy.call(this),this.menuPanel&&this.menuPanel.isAlive()&&this.destroyBean(this.menuPanel)},t.EVENT_DOWNLOAD_CHART="downloadChart",t.TEMPLATE='<div class="ag-chart-menu"></div>',u0([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),u0([B],t.prototype,"postConstruct",null),t}(ee),eX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Nd=globalThis&&globalThis.__assign||function(){return Nd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Nd.apply(this,arguments)},p0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},tX=function(i){eX(t,i);function t(e){var r=i.call(this,t.TEMPLATE)||this;return r.chartMenu=e,r}return t.prototype.init=function(){var e=this;this.addManagedListener(this.getGui(),"keypress",function(r){r.key==="Enter"&&e.endEditing()}),this.addManagedListener(this.getGui(),"blur",this.endEditing.bind(this))},t.prototype.refreshTitle=function(e,r){var n=this;this.chartController=e,this.chartOptionsService=r;var o=this.chartController.getChartProxy();if(o){for(var a=0;a++;a<this.destroyableChartListeners.length)this.destroyableChartListeners[a]();this.destroyableChartListeners=[]}var s=o.getChart(),l=s.scene.canvas.element,u=this.addManagedListener(l,"dblclick",function(h){var d=s.title;if(d&&d.node.containsPoint(h.offsetX,h.offsetY)){var f=d.node.computeBBox(),g=d.node.inverseTransformPoint(f.x,f.y);n.startEditing(Nd(Nd({},f),g))}}),c=!1,p=this.addManagedListener(l,"mousemove",function(h){var d=s.title,f=!!(d&&d.enabled&&d.node.containsPoint(h.offsetX,h.offsetY));c!==f&&(l.style.cursor=f?"pointer":""),c=f});this.destroyableChartListeners=[u,p]},t.prototype.startEditing=function(e){if(!(this.chartMenu&&this.chartMenu.isVisible())){var r=300,n=this.chartController.getChartProxy().getChart().width,o=Math.max(Math.min(e.width+20,n),r),a=this.getGui();a.classList.add("currently-editing");var s=a.style;s.fontFamily=this.chartOptionsService.getChartOption("title.fontFamily"),s.fontWeight=this.chartOptionsService.getChartOption("title.fontWeight"),s.fontStyle=this.chartOptionsService.getChartOption("title.fontStyle"),s.fontSize=this.chartOptionsService.getChartOption("title.fontSize")+"px",s.color=this.chartOptionsService.getChartOption("title.color");var l=this.chartOptionsService.getChartOption("title.text"),u=l===this.chartTranslationService.translate("titlePlaceholder")?"":l;a.value=u;var c=a.getBoundingClientRect();s.left=Math.round(e.x+e.width/2-o/2)+"px",s.top=Math.round(e.y+e.height/2-c.height/2)+"px",s.width=Math.round(o)+"px",a.focus()}},t.prototype.endEditing=function(){var e=this.getGui().value;this.chartOptionsService.setChartOption("title.text",e),this.eventService.dispatchEvent({type:"chartTitleEdit"}),this.removeCssClass("currently-editing")},t.TEMPLATE=`<input
            class="ag-chart-title-edit"
            style="padding:0; border:none; border-radius: 0; min-height: 0; text-align: center;" />
        `,p0([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),p0([B],t.prototype,"init",null),t}(ee);function rX(i){return Array.isArray(i)?[]:{}}function kc(i,t){return t.clone!==!1&&t.isMergeableObject(i)?Pt(rX(i),i,t):i}function nX(i,t,e){return i.concat(t).map(function(r){return kc(r,e)})}function iX(i,t){if(!t.customMerge)return Pt;var e=t.customMerge(i);return typeof e=="function"?e:Pt}function oX(i){return Object.getOwnPropertySymbols?Object.getOwnPropertySymbols(i).filter(function(t){return i.propertyIsEnumerable(t)}):[]}function h0(i){return Object.keys(i).concat(oX(i))}function tD(i,t){try{return t in i}catch{return!1}}function aX(i,t){return tD(i,t)&&!(Object.hasOwnProperty.call(i,t)&&Object.propertyIsEnumerable.call(i,t))}function sX(i,t,e){var r={};return e.isMergeableObject(i)&&h0(i).forEach(function(n){r[n]=kc(i[n],e)}),h0(t).forEach(function(n){aX(i,n)||(tD(i,n)&&e.isMergeableObject(t[n])?r[n]=iX(n,e)(i[n],t[n],e):r[n]=kc(t[n],e))}),r}function lX(i){return uX(i)&&!cX(i)}function uX(i){return!!i&&typeof i=="object"}function cX(i){var t=Object.prototype.toString.call(i);return t==="[object RegExp]"||t==="[object Date]"}function Pt(i,t,e){e=e||{},e.arrayMerge=e.arrayMerge||nX,e.isMergeableObject=e.isMergeableObject||lX,e.cloneUnlessOtherwiseSpecified=kc;var r=Array.isArray(t),n=Array.isArray(i),o=r===n;return o?r?e.arrayMerge(i,t,e):sX(i,t,e):kc(t,e)}var yc=globalThis&&globalThis.__assign||function(){return yc=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},yc.apply(this,arguments)},rD=function(){function i(t){if(this.chartProxyParams=t,this.chartType=t.chartType,this.crossFiltering=t.crossFiltering,this.crossFilterCallback=t.crossFilterCallback,this.standaloneChartType=Mi(this.chartType),this.chartProxyParams.chartOptionsToRestore){this.chartOptions=this.chartProxyParams.chartOptionsToRestore,this.chartPalette=this.chartProxyParams.chartPaletteToRestore;var e={overrides:this.chartOptions,palette:this.chartPalette};this.chartTheme=Ol(yc({baseTheme:this.getSelectedTheme()},e));return}this.chartTheme=this.createChartTheme(),this.chartOptions=this.convertConfigToOverrides(this.chartTheme.config),this.chartPalette=this.chartTheme.palette}return i.prototype.recreateChart=function(){var t=this;if(this.chart&&this.destroyChart(),this.chart=this.createChart(),this.crossFiltering){var e=!0;this.chart.addEventListener("click",function(r){return t.crossFilterCallback(r,e)})}},i.prototype.getChart=function(){return this.chart},i.prototype.createChartTheme=function(){var t=this,e=this.getSelectedTheme(),r=this.isStockTheme(e),n=this.chartProxyParams.getGridOptionsChartThemeOverrides(),o=this.chartProxyParams.apiChartThemeOverrides;if(n||o){var a={overrides:i.mergeThemeOverrides(n,o)},s=function(){return Pt(t.lookupCustomChartTheme(e),a)};return Ol(r?yc({baseTheme:e},a):s())}return Ol(r?e:this.lookupCustomChartTheme(e))},i.prototype.isStockTheme=function(t){return _.includes(Object.keys(UA),t)},i.prototype.getSelectedTheme=function(){var t=this.chartProxyParams.getChartThemeName(),e=this.chartProxyParams.getChartThemes();return _.includes(e,t)||(t=e[0]),t},i.prototype.lookupCustomChartTheme=function(t){var e=this.chartProxyParams.customChartThemes,r=e&&e[t];return r||console.warn("AG Grid: no stock theme exists with the name '"+t+"' and no custom chart theme with that name was supplied to 'customChartThemes'"),r},i.mergeThemeOverrides=function(t,e){return t?e?Pt(t,e):t:e},i.prototype.downloadChart=function(){var t=this.chart,e=t.title?t.title.text:"chart";t.scene.download(e)},i.prototype.getChartImageDataURL=function(t){return this.chart.scene.getDataURL(t)},i.prototype.getChartOptions=function(){return this.chartOptions},i.prototype.getChartPalette=function(){return this.chartPalette},i.prototype.transformData=function(t,e){var r=this.chartProxyParams.grouping||this.chart.axes.filter(function(n){return n instanceof ps}).length<1;return r?t:t.map(function(n,o){var a=n[e],s=a&&a.toString?a.toString():"",l=yc({},n);return l[e]={id:o,value:a,toString:function(){return s}},l})},i.prototype.convertConfigToOverrides=function(t){var e=["columnLineCombo","areaColumnCombo","customCombo"].includes(this.chartType),r=e?["line","area","column","cartesian"]:[this.standaloneChartType],n={};return r.forEach(function(o){var a=Pt({},t[o]);a.series=a.series[o],o==="scatter"&&(a.paired=!0),n[o]=a}),n},i.prototype.destroy=function(){this.destroyChart()},i.prototype.destroyChart=function(){this.chart&&(this.chart.destroy(),this.chart=void 0)},i}(),pX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),tu=function(i){pX(t,i);function t(e){var r=i.call(this,e)||this;return r.axisTypeToClassMap={number:Zl,category:ps,groupedCategory:Il,time:Lc},r}return t.prototype.updateAxes=function(e){if(e.grouping){this.axisTypeToClassMap[this.xAxisType]!==Il&&(this.xAxisType="groupedCategory",this.recreateChart());return}var r=t.isTimeAxis(e)?"time":"category";r!==this.xAxisType&&(this.xAxisType=r,this.recreateChart())},t.prototype.getAxesOptions=function(e){return e===void 0&&(e=this.standaloneChartType),this.chartOptions[e].axes},t.prototype.processDataForCrossFiltering=function(e,r,n){var o=r,a=!1;if(this.crossFiltering){e.forEach(function(l){l[r+"-total"]=l[r]+l[r+"-filtered-out"],l[r+"-filtered-out"]>0&&(a=!0)});var s=n.getCrossFilteringContext().lastSelectedChartId;s===n.chartId&&(o=r+"-total")}return{yKey:o,atLeastOneSelectedPoint:a}},t.prototype.updateSeriesForCrossFiltering=function(e,r,n,o,a){if(this.crossFiltering){e.marker.enabled=!0,e.marker.formatter=function(c){return{fill:c.highlighted?"yellow":c.fill,size:c.highlighted?12:c.datum[r]>0?8:0}},n.tooltip.delay=500;var s=o.getCrossFilteringContext(),l=s.lastSelectedChartId===o.chartId,u=l&&a;e instanceof nb&&(e.fillOpacity=u?.3:1),e instanceof ab&&(e.strokeOpacity=u?.3:1),e.addEventListener("nodeClick",this.crossFilterCallback)}},t.isTimeAxis=function(e){if(e.category&&e.category.chartDataType)return e.category.chartDataType==="time";var r=e.data[0];return(r&&r[e.category.id])instanceof Date},t}(rD);function Gl(i,t){var e=parseInt(i.slice(1,3),16),r=parseInt(i.slice(3,5),16),n=parseInt(i.slice(5,7),16);return t?"rgba("+e+", "+r+", "+n+", "+t+")":"rgba("+e+", "+r+", "+n+")"}function Kv(i,t){return i.map(function(e){var r=Bo.fromString(e);return new Bo(r.r,r.g,r.b,t).toHexString()})}var hX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Kn=globalThis&&globalThis.__assign||function(){return Kn=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Kn.apply(this,arguments)},dX=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},fX=function(i){hX(t,i);function t(e){var r=i.call(this,e)||this;return r.xAxisType=e.grouping?"groupedCategory":"category",r.yAxisType="number",r.recreateChart(),r}return t.prototype.createChart=function(){var e=dX([this.standaloneChartType==="bar",this.isNormalised()],2),r=e[0],n=e[1];return Dr.create({container:this.chartProxyParams.parentElement,theme:this.chartTheme,axes:this.getAxes(r,n),series:this.getSeries(n)})},t.prototype.update=function(e){this.updateAxes(e);var r=this.chart.series[0];this.crossFiltering?this.updateCrossFilteringSeries(r,e):(r.fills=this.chartTheme.palette.fills,r.strokes=this.chartTheme.palette.strokes),r.data=this.transformData(e.data,e.category.id),r.xKey=e.category.id,r.xName=e.category.name,r.yKeys=e.fields.map(function(n){return n.colId}),r.yNames=e.fields.map(function(n){return n.displayName})},t.prototype.updateCrossFilteringSeries=function(e,r){var n=r.fields;n.forEach(function(u){var c=Kn({},u);c.colId=u.colId+"-filtered-out",n.push(c)});var o=this.chartTheme.palette,a=[];o.fills.forEach(function(u){a.push(u),a.push(Gl(u,"0.3"))}),e.fills=a;var s=[];o.strokes.forEach(function(u){a.push(u),a.push(Gl(u,"0.3"))}),e.strokes=s,e.highlightStyle.fill=void 0;var l=r.fields.map(function(u){return u.colId});e.hideInLegend=l.filter(function(u){return u.indexOf("-filtered-out")!==-1}),this.chart.legend.addEventListener("click",function(u){e.toggleSeriesItem(u.itemId+"-filtered-out",u.enabled)}),this.chart.tooltip.delay=500,e.addEventListener("nodeClick",this.crossFilterCallback)},t.prototype.getAxes=function(e,r){var n=this.getAxesOptions(),o=[Kn(Kn({},Pt(n[this.xAxisType],n[this.xAxisType].bottom)),{type:this.xAxisType,position:e?Ae.Left:Ae.Bottom}),Kn(Kn({},Pt(n[this.yAxisType],n[this.yAxisType].left)),{type:this.yAxisType,position:e?Ae.Bottom:Ae.Left})];if(r){var a=o[1];a.label=Kn(Kn({},a.label),{formatter:function(s){return Math.round(s.value)+"%"}})}return o},t.prototype.getSeries=function(e){var r=["groupedColumn","groupedBar"],n=!this.crossFiltering&&_.includes(r,this.chartType);return[Kn(Kn({},this.chartOptions[this.standaloneChartType].series),{type:this.standaloneChartType,grouped:n,normalizedTo:e?100:void 0})]},t.prototype.isNormalised=function(){var e=["normalizedColumn","normalizedBar"];return!this.crossFiltering&&_.includes(e,this.chartType)},t}(tu),gX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Tr=globalThis&&globalThis.__assign||function(){return Tr=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Tr.apply(this,arguments)},vX=function(i){gX(t,i);function t(e){var r=i.call(this,e)||this;return r.xAxisType=e.grouping?"groupedCategory":"category",r.yAxisType="number",r.recreateChart(),r}return t.prototype.createChart=function(){return Dr.create({type:"area",container:this.chartProxyParams.parentElement,theme:this.chartTheme,axes:this.getAxes()})},t.prototype.update=function(e){if(this.updateAxes(e),this.chartType==="area")this.updateAreaChart(e);else{var r=this.chart.series[0];if(!r){var n=Tr(Tr({},this.chartOptions[this.standaloneChartType].series),{type:"area",normalizedTo:this.chartType==="normalizedArea"?100:void 0});if(r=Dr.createComponent(Tr({},n),"area.series"),!r)return;this.chart.addSeries(r)}r.data=this.transformData(e.data,e.category.id),r.xKey=e.category.id,r.xName=e.category.name,r.yKeys=e.fields.map(function(o){return o.colId}),r.yNames=e.fields.map(function(o){return o.displayName}),r.fills=this.chartTheme.palette.fills,r.strokes=this.chartTheme.palette.strokes}},t.prototype.updateAreaChart=function(e){var r=this,n=this.chart;if(e.fields.length===0){n.removeAllSeries();return}var o=e.fields.map(function(h){return h.colId}),a=n.series.reduceRight(function(h,d,f){var g=d.yKeys[0];return o.indexOf(g)===f?h.set(g,d):n.removeSeries(d),h},new Map),s=this.transformData(e.data,e.category.id),l,u=this.chartTheme.palette,c=u.fills,p=u.strokes;e.fields.forEach(function(h,d){var f=r.processDataForCrossFiltering(s,h.colId,e),g=f.yKey,v=f.atLeastOneSelectedPoint,m=a.get(h.colId),C=c[d%c.length],w=p[d%p.length];if(m)m.data=s,m.xKey=e.category.id,m.xName=e.category.name,m.yKeys=[g],m.yNames=[h.displayName],m.fills=[C],m.strokes=[w];else{var b=r.chartOptions[r.standaloneChartType].series,S=Tr(Tr({},b),{type:"area",normalizedTo:r.chartType==="normalizedArea"?100:void 0}),T=Tr(Tr({},S),{data:s,xKey:e.category.id,xName:e.category.name,yKeys:[g],yNames:[h.displayName],fills:[C],strokes:[w],marker:Tr(Tr({},S.marker),{fill:C,stroke:w})});m=Dr.createComponent(T,"area.series"),n.addSeriesAfter(m,l)}r.updateSeriesForCrossFiltering(m,h.colId,n,e,v),l=m})},t.prototype.getAxes=function(){var e=this.getAxesOptions(),r=[Tr(Tr({},Pt(e[this.xAxisType],e[this.xAxisType].bottom)),{type:this.xAxisType,position:Ae.Bottom,paddingInner:1,paddingOuter:0}),Tr(Tr({},Pt(e[this.yAxisType],e[this.yAxisType].left)),{type:this.yAxisType,position:Ae.Left})];return this.xAxisType==="time"&&(delete r[0].paddingInner,delete r[0].paddingOuter),r},t}(tu),mX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),wi=globalThis&&globalThis.__assign||function(){return wi=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},wi.apply(this,arguments)},yX=function(i){mX(t,i);function t(e){var r=i.call(this,e)||this;return r.xAxisType=e.grouping?"groupedCategory":"category",r.yAxisType="number",r.recreateChart(),r}return t.prototype.createChart=function(){return Dr.create({type:"line",container:this.chartProxyParams.parentElement,theme:this.chartTheme,axes:this.getAxes()})},t.prototype.update=function(e){var r=this;if(e.fields.length===0){this.chart.removeAllSeries();return}this.updateAxes(e);var n=this.chart,o=e.fields,a=o.map(function(d){return d.colId}),s=this.transformData(e.data,e.category.id),l=n.series.reduceRight(function(d,f,g){var v=f.yKey;return a.indexOf(v)===g?d.set(v,f):n.removeSeries(f),d},new Map),u,c=this.chartTheme.palette,p=c.fills,h=c.strokes;o.forEach(function(d,f){var g=r.processDataForCrossFiltering(s,d.colId,e),v=g.yKey,m=g.atLeastOneSelectedPoint,C=l.get(d.colId),w=p[f%p.length],b=h[f%h.length];if(C)C.title=d.displayName,C.data=s,C.xKey=e.category.id,C.xName=e.category.name,C.yKey=v,C.yName=d.displayName,C.marker.fill=w,C.marker.stroke=b,C.stroke=w;else{var S=r.chartOptions[r.standaloneChartType].series,T=wi(wi({},S),{type:"line",title:d.displayName,data:s,xKey:e.category.id,xName:e.category.name,yKey:v,yName:d.displayName,stroke:w,marker:wi(wi({},S.marker),{fill:w,stroke:b})});C=Dr.createComponent(T,"line.series"),n.addSeriesAfter(C,u)}r.updateSeriesForCrossFiltering(C,d.colId,n,e,m),u=C})},t.prototype.getAxes=function(){var e=this.getAxesOptions();return[wi(wi({},Pt(e[this.xAxisType],e[this.xAxisType].bottom)),{type:this.xAxisType,position:Ae.Bottom}),wi(wi({},Pt(e[this.yAxisType],e[this.yAxisType].left)),{type:this.yAxisType,position:Ae.Left})]},t}(tu),CX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),nD=function(i){CX(t,i);function t(e){return i.call(this,e)||this}return t.prototype.addCrossFilteringTooltipRenderer=function(e){e.tooltip.renderer=function(r){var n=r.datum[r.labelKey],o=r.datum[r.radiusKey],a=r.angleValue,s=a*o;return{content:n+": "+s}}},t}(rD),wX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Cc=globalThis&&globalThis.__assign||function(){return Cc=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Cc.apply(this,arguments)},bX=function(i){wX(t,i);function t(e){var r=i.call(this,e)||this;return r.recreateChart(),r}return t.prototype.createChart=function(){return Dr.create({type:"pie",container:this.chartProxyParams.parentElement,theme:this.chartTheme})},t.prototype.update=function(e){var r=this.chart;if(e.fields.length===0){r.removeAllSeries();return}var n=e.fields[0],o=n;if(this.crossFiltering){var a=e.fields;a.forEach(function(h){var d=Cc({},h);d.colId=h.colId+"-filtered-out",a.push(d)});var s=a[1];e.data.forEach(function(h){h[n.colId+"-total"]=h[n.colId]+h[s.colId],h[n.colId]=h[n.colId]/h[n.colId+"-total"],h[s.colId]=1});var l=r.series[1],u=s;l=this.updateSeries(r,l,o,u,e,void 0),u=o;var c=r.series[0];this.updateSeries(r,c,o,u,e,l)}else{var p=r.series[0];this.updateSeries(r,p,o,o,e,void 0)}},t.prototype.updateSeries=function(e,r,n,o,a,s){var l=r&&r.angleKey,u=this.chartOptions[this.standaloneChartType].series,c=r;if(l!==o.colId){e.removeSeries(r);var p=Cc(Cc({},u),{type:"pie",angleKey:this.crossFiltering?n.colId+"-total":n.colId,radiusKey:this.crossFiltering?o.colId:void 0});c=Dr.createComponent(p,"pie.series"),c.fills=this.chartTheme.palette.fills,c.strokes=this.chartTheme.palette.strokes,c.callout.colors=this.chartTheme.palette.strokes,this.crossFiltering&&c&&!c.tooltip.renderer&&this.addCrossFilteringTooltipRenderer(c)}if(c.angleName=o.displayName,c.labelKey=a.category.id,c.labelName=a.category.name,c.data=a.data,this.crossFiltering){c.radiusMin=0,c.radiusMax=1;var h=!s;h?(c.fills=Kv(c.fills,.3),c.strokes=Kv(c.strokes,.3),c.callout.colors=Kv(c.strokes,.3),c.showInLegend=!1):e.legend.addEventListener("click",function(d){s&&s.toggleSeriesItem(d.itemId,d.enabled)}),e.tooltip.delay=500,c.highlightStyle.fill=void 0,c.addEventListener("nodeClick",this.crossFilterCallback)}return e.addSeries(c),c},t}(nD),_X=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ua=globalThis&&globalThis.__assign||function(){return Ua=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Ua.apply(this,arguments)},SX=function(i){_X(t,i);function t(e){var r=i.call(this,e)||this;return r.recreateChart(),r}return t.prototype.createChart=function(){return Dr.create({type:"pie",container:this.chartProxyParams.parentElement,theme:this.chartTheme})},t.prototype.update=function(e){var r=this;if(e.fields.length===0){this.chart.removeAllSeries();return}var n=this.chart,o=e.fields.map(function(h){return h.colId}),a={};n.series.forEach(function(h){var d=h,f=d.angleKey;_.includes(o,f)&&(a[f]=d)});var s=this.chartTheme.palette.fills,l=this.chartTheme.palette.strokes,u=this.chartOptions[this.standaloneChartType].series,c=e.fields.length,p=0;this.crossFiltering?e.fields.forEach(function(h,d){var f=Ua({},h);f.colId=h.colId+"-filtered-out",e.data.forEach(function(C){C[h.colId+"-total"]=C[h.colId]+C[f.colId],C[h.colId]=C[h.colId]/C[h.colId+"-total"],C[f.colId]=1});var g=r.updateSeries({seriesMap:a,angleField:h,field:f,seriesDefaults:u,index:d,params:e,fills:s,strokes:l,doughnutChart:n,offset:p,numFields:c,opaqueSeries:void 0}),v=g.updatedOffset,m=g.pieSeries;r.updateSeries({seriesMap:a,angleField:h,field:h,seriesDefaults:u,index:d,params:e,fills:s,strokes:l,doughnutChart:n,offset:p,numFields:c,opaqueSeries:m}),p=v}):e.fields.forEach(function(h,d){var f=r.updateSeries({seriesMap:a,angleField:h,field:h,seriesDefaults:u,index:d,params:e,fills:s,strokes:l,doughnutChart:n,offset:p,numFields:c,opaqueSeries:void 0}).updatedOffset;p=f}),n.series=_.values(a)},t.prototype.updateSeries=function(e){var r=e.seriesMap[e.field.colId],n=Ua(Ua({},e.seriesDefaults),{type:"pie",angleKey:this.crossFiltering?e.angleField.colId+"-total":e.angleField.colId,radiusKey:this.crossFiltering?e.field.colId:void 0,title:Ua(Ua({},e.seriesDefaults.title),{text:e.seriesDefaults.title.text||e.field.displayName})}),o=r||Dr.createComponent(n,"pie.series");if(o.title&&(o.title.showInLegend=e.numFields>1),r||this.crossFiltering&&!o.tooltip.renderer&&this.addCrossFilteringTooltipRenderer(o),o.angleName=e.field.displayName,o.labelKey=e.params.category.id,o.labelName=e.params.category.name,o.data=e.params.data,this.crossFiltering){o.radiusMin=0,o.radiusMax=1;var a=!e.opaqueSeries;a?(o.fills=e.fills.map(function(l){return Gl(l,"0.3")}),o.strokes=e.strokes.map(function(l){return Gl(l,"0.3")}),o.showInLegend=!1):(e.doughnutChart.legend.addEventListener("click",function(l){e.opaqueSeries&&e.opaqueSeries.toggleSeriesItem(l.itemId,l.enabled)}),o.fills=e.fills,o.strokes=e.strokes,o.callout.colors=e.strokes),o.highlightStyle.fill=void 0,o.addEventListener("nodeClick",this.crossFilterCallback),e.doughnutChart.tooltip.delay=500}else o.fills=e.fills,o.strokes=e.strokes,o.callout.colors=e.strokes;var s=e.numFields>1?20:40;return o.outerRadiusOffset=e.offset,e.offset-=s,o.innerRadiusOffset=e.offset,e.offset-=s,r||(e.seriesMap[e.field.colId]=o),{updatedOffset:e.offset,pieSeries:o}},t}(nD),xX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ui=globalThis&&globalThis.__assign||function(){return Ui=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Ui.apply(this,arguments)},TX=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},d0=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(TX(arguments[t]));return i},OX=function(i){xX(t,i);function t(e){var r=i.call(this,e)||this;return r.xAxisType="number",r.yAxisType="number",r.recreateChart(),r}return t.prototype.createChart=function(){return Dr.create({type:"scatter",container:this.chartProxyParams.parentElement,theme:this.chartTheme,axes:this.getAxes()})},t.prototype.update=function(e){var r=this;if(e.fields.length<2){this.chart.removeAllSeries();return}var n=e.fields;this.crossFiltering&&n.forEach(function(C){var w=Ui({},C);w.colId=C.colId+"-filtered-out",n.push(w)});var o=this.chartOptions[this.standaloneChartType].paired,a=this.getSeriesDefinitions(n,o),s;this.crossFiltering&&(s=this.getCrossFilteringDataDomain(a,e));var l=this.chart,u=l.series.reduceRight(function(C,w,b){var S=a.findIndex(function(T){return T.xField.colId===w.xKey&&T.yField.colId===w.yKey&&(!T.sizeField&&!w.sizeKey||T.sizeField&&T.sizeField.colId===w.sizeKey)});return S===b?C.set(w.yKey,w):l.removeSeries(w),C},new Map),c=this.chartTheme.palette,p=c.fills,h=c.strokes;if(this.crossFiltering){var d=[];p.forEach(function(C){d.push(C),d.push(Gl(C,"0.3"))}),p=d;var f=[];h.forEach(function(C){f.push(C),f.push(Gl(C,"0.3"))}),h=f}var g=e.category.id===Sf.DEFAULT_CATEGORY?void 0:e.category,v,m=this.chartOptions[this.standaloneChartType].series;a.forEach(function(C,w){var b=u.get(C.yField.colId),S=b||Dr.createComponent(Ui(Ui({},m),{type:"scatter"}),"scatter.series");if(!!S){var T=C,E=T.xField,P=T.yField,R=T.sizeField;S.title=P.displayName+" vs "+E.displayName,S.xKey=E.colId,S.xName=E.displayName,S.yKey=P.colId,S.yName=P.displayName,S.data=e.data,S.fill=p[w%p.length],S.stroke=h[w%h.length],R?(S.sizeKey=R.colId,S.sizeName=R.displayName):S.sizeKey=void 0,g?(S.labelKey=g.id,S.labelName=g.name):S.labelKey=S.yKey;var M=P.colId.indexOf("-filtered-out")>-1;r.crossFiltering&&(M||l.legend.addEventListener("click",function(N){S.toggleSeriesItem(N.itemId+"-filtered-out",N.enabled)}),s&&(S.marker.domain=s),l.tooltip.delay=500,M&&(S.showInLegend=!1),S.addEventListener("nodeClick",r.crossFilterCallback)),b||l.addSeriesAfter(S,v),v=S}})},t.prototype.getSeriesDefinitions=function(e,r){if(e.length<2)return[];var n=this.chartType==="bubble";if(r)return n?e.map(function(a,s){return s%3===0?{xField:a,yField:e[s+1],sizeField:e[s+2]}:null}).filter(function(a){return a&&a.yField&&a.sizeField}):e.map(function(a,s){return s%2===0?{xField:a,yField:e[s+1]}:null}).filter(function(a){return a&&a.yField});var o=e[0];return n?e.map(function(a,s){return s%2===1?{xField:o,yField:a,sizeField:e[s+1]}:null}).filter(function(a){return a&&a.sizeField}):e.filter(function(a,s){return s>0}).map(function(a){return{xField:o,yField:a}})},t.prototype.getCrossFilteringDataDomain=function(e,r){var n;if(e[0]&&e[0].sizeField){var o=e[0].sizeField.colId,a=[];r.data.forEach(function(s){typeof s[o]!="undefined"&&a.push(s[o]),typeof s[o+"-filtered-out"]!="undefined"&&a.push(s[o+"-filtered-out"])}),a.length>0&&(n=[Math.min.apply(Math,d0(a)),Math.max.apply(Math,d0(a))])}return n},t.prototype.getAxes=function(){var e=this.getAxesOptions();return[Ui(Ui({},Pt(e[this.xAxisType],e[this.xAxisType].bottom)),{type:this.xAxisType,position:Ae.Bottom}),Ui(Ui({},Pt(e[this.yAxisType],e[this.yAxisType].left)),{type:this.yAxisType,position:Ae.Left})]},t}(tu),EX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Wo=globalThis&&globalThis.__assign||function(){return Wo=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Wo.apply(this,arguments)},RX=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},PX=function(i){EX(t,i);function t(e){var r=i.call(this,e)||this;return r.xAxisType="number",r.yAxisType="number",r.recreateChart(),r}return t.prototype.createChart=function(){return Dr.create({container:this.chartProxyParams.parentElement,theme:this.chartTheme,axes:this.getAxes(),series:[Wo(Wo({},this.chartOptions[this.standaloneChartType].series),{type:"histogram"})]})},t.prototype.update=function(e){var r=RX(e.fields,1),n=r[0],o=this.chart,a=o.series[0];a.data=e.data,a.xKey=n.colId,a.xName=n.displayName,a.areaPlot=!1,a.fill=this.chartTheme.palette.fills[0],a.stroke=this.chartTheme.palette.strokes[0]},t.prototype.getAxes=function(){var e=this.getAxesOptions();return[Wo(Wo({},Pt(e[this.xAxisType],e[this.xAxisType].bottom)),{type:this.xAxisType,position:Ae.Bottom}),Wo(Wo({},Pt(e[this.yAxisType],e[this.yAxisType].left)),{type:this.yAxisType,position:Ae.Left})]},t}(tu),AX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),f0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},DX=function(i){AX(t,i);function t(e){var r=i.call(this)||this;return r.chartController=e,r}return t.prototype.getChartType=function(){return this.chartController.getChartType()},t.prototype.getChartOption=function(e){return _.get(this.getChart(),e,void 0)},t.prototype.setChartOption=function(e,r){var n=this;this.chartController.getChartSeriesTypes().forEach(function(o){var a=_.get(n.getChartOptions(),""+o,void 0);_.set(a,e,r)}),_.set(this.getChart(),e,r),this.raiseChartOptionsChangedEvent()},t.prototype.getAxisProperty=function(e){return _.get(this.getChart().axes[0],e,void 0)},t.prototype.setAxisProperty=function(e,r){var n=this,o=this.getChart();o.axes.forEach(function(a){n.updateAxisOptions(a,e,r),_.set(a,e,r)}),o.layoutPending=!0,this.raiseChartOptionsChangedEvent()},t.prototype.getLabelRotation=function(e){var r=this.getAxis(e);return _.get(r,"label.rotation",void 0)},t.prototype.setLabelRotation=function(e,r){var n="label.rotation",o=this.getAxis(e);_.set(o,n,r),this.getChart().layoutPending=!0,o&&!this.chartController.isDefaultCategorySelected()&&(this.updateAxisOptions(o,n,r),this.raiseChartOptionsChangedEvent())},t.prototype.getSeriesOption=function(e,r){var n=this.getChart().series.find(function(o){return t.isMatchingSeries(r,o)});return _.get(n,e,void 0)},t.prototype.setSeriesOption=function(e,r,n){var o=this.getChartOptions();o[n]||(o[n]={}),_.set(o[n].series,e,r),this.getChart().series.forEach(function(a){t.isMatchingSeries(n,a)&&_.set(a,e,r)}),this.raiseChartOptionsChangedEvent()},t.prototype.getPairedMode=function(){var e=Mi(this.getChartType());return _.get(this.getChartOptions(),e+".paired",void 0)},t.prototype.setPairedMode=function(e){var r=Mi(this.getChartType()),n=_.get(this.getChartOptions(),""+r,void 0);_.set(n,"paired",e)},t.prototype.getChart=function(){return this.chartController.getChartProxy().getChart()},t.prototype.getChartOptions=function(){return this.chartController.getChartProxy().getChartOptions()},t.prototype.getAxis=function(e){var r=this.getChart();if(!(!r.axes||r.axes.length<1))return e==="xAxis"?r.axes&&r.axes[0].direction==="x"?r.axes[0]:r.axes[1]:r.axes&&r.axes[1].direction==="y"?r.axes[1]:r.axes[0]},t.prototype.updateAxisOptions=function(e,r,n){var o=Mi(this.getChartType()),a=this.getChartOptions()[o].axes;e instanceof Zl?_.set(a.number,r,n):e instanceof ps?_.set(a.category,r,n):e instanceof Lc?_.set(a.time,r,n):e instanceof Il&&_.set(a.groupedCategory,r,n)},t.prototype.raiseChartOptionsChangedEvent=function(){var e=this.chartController.getChartModel(),r=Object.freeze({type:x.EVENT_CHART_OPTIONS_CHANGED,chartId:e.chartId,chartType:e.chartType,chartThemeName:e.chartThemeName,chartOptions:e.chartOptions,api:this.gridApi,columnApi:this.columnApi});this.eventService.dispatchEvent(r)},t.isMatchingSeries=function(e,r){return e==="area"&&r instanceof nb||e==="bar"&&r instanceof Fy||e==="column"&&r instanceof Fy||e==="histogram"&&r instanceof VA||e==="line"&&r instanceof ab||e==="pie"&&r instanceof jA?!0:e==="scatter"&&r instanceof WA},t.prototype.destroy=function(){i.prototype.destroy.call(this)},f0([y("gridApi")],t.prototype,"gridApi",void 0),f0([y("columnApi")],t.prototype,"columnApi",void 0),t}(H),MX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),nn=globalThis&&globalThis.__assign||function(){return nn=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},nn.apply(this,arguments)},FX=function(i){MX(t,i);function t(e){var r=i.call(this,e)||this;return r.xAxisType=e.grouping?"groupedCategory":"category",r.yAxisType="number",r.recreateChart(),r}return t.prototype.createChart=function(){return Dr.create({container:this.chartProxyParams.parentElement,theme:this.chartTheme})},t.prototype.update=function(e){var r=e.category,n=e.data,o={data:this.transformData(n,r.id)};this.seriesChanged(e)&&(o.series=this.getSeriesOptions(e),o.axes=this.getAxes(e)),Dr.update(this.chart,o)},t.prototype.seriesChanged=function(e){var r=e.seriesChartTypes,n=!_.areEqual(this.prevSeriesChartTypes,r,function(u,c){return u.colId===c.colId&&u.chartType===c.chartType&&u.secondaryAxis===c.secondaryAxis});this.prevSeriesChartTypes=r.map(function(u){return nn({},u)});var o=e.fields.map(function(u){return u.colId}).join(),a=this.prevFields!==o;this.prevFields=o;var s=e.category.id,l=this.prevCategoryId!==s;return this.prevCategoryId=s,n||a||l},t.prototype.getSeriesOptions=function(e){var r=this,n=e.fields,o=e.category,a=e.seriesChartTypes;return n.map(function(s){var l=a.find(function(c){return c.colId===s.colId});if(l){var u=l.chartType;return nn(nn({},r.extractSeriesOverrides(l)),{type:Mi(u),xKey:o.id,yKey:s.colId,yName:s.displayName,grouped:["groupedColumn","groupedBar","groupedArea"].includes(u),stacked:["stackedArea","stackedColumn"].includes(u)})}})},t.prototype.getAxes=function(e){var r=this;this.xAxisType=e.grouping?"groupedCategory":"category";var n=e?e.fields:[],o=new Map(n.map(function(f){return[f.colId,f]})),a=this.getYKeys(n,e.seriesChartTypes),s=a.primaryYKeys,l=a.secondaryYKeys,u=this.getAxisOptions(),c=u.bottomOptions,p=u.leftOptions,h=u.rightOptions,d=[nn(nn({},c),{type:this.xAxisType,position:Ae.Bottom,gridStyle:[{strokeWidth:0}]})];return s.length>0&&d.push(nn(nn({},p),{type:this.yAxisType,keys:s,position:Ae.Left,title:nn({},Pt(p.title,{enabled:!0,text:s.map(function(f){var g=o.get(f);return g?g.displayName:f}).join(" / ")}))})),l.length>0&&l.forEach(function(f,g){var v=o.get(f),m=v&&v.colId===f;if(!!m){var C=nn(nn({},h),{type:r.yAxisType,keys:[f],position:Ae.Right,title:nn({},Pt(h.title,{enabled:!0,text:v?v.displayName:f}))}),w=s.some(function(S){return!!o.get(S)}),b=g===l.length-1;!w&&b||(C.gridStyle=[{strokeWidth:0}]),d.push(C)}}),d},t.prototype.getAxisOptions=function(){var e=this.getAxesOptions("cartesian");return{bottomOptions:Pt(e[this.xAxisType],e[this.xAxisType].bottom),leftOptions:Pt(e[this.yAxisType],e[this.yAxisType].left),rightOptions:Pt(e[this.yAxisType],e[this.yAxisType].right)}},t.prototype.getYKeys=function(e,r){var n=[],o=[];return e.forEach(function(a){var s=a.colId,l=r.find(function(u){return u.colId===s});l&&(l.secondaryAxis?o.push(s):n.push(s))}),{primaryYKeys:n,secondaryYKeys:o}},t.prototype.extractSeriesOverrides=function(e){var r=this.chartOptions[Mi(e.chartType)].series;return delete r.yKeys,delete r.yNames,r},t}(tu),IX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Jr=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},g0=function(i){IX(t,i);function t(e){var r=i.call(this,t.TEMPLATE)||this;return r.params=e,r}return t.prototype.init=function(){var e=this.gridOptionsWrapper.getChartThemes();if(e.length<1)throw new Error("Cannot create chart: no chart themes are available to be used.");var r=this.params.chartThemeName;_.includes(e,r)||(r=e[0]);var n={chartId:this.params.chartId,pivotChart:this.params.pivotChart,chartType:this.params.chartType,chartThemeName:r,aggFunc:this.params.aggFunc,cellRange:this.params.cellRange,suppressChartRanges:this.params.suppressChartRanges,unlinkChart:this.params.unlinkChart,crossFiltering:this.params.crossFiltering,seriesChartTypes:this.params.seriesChartTypes},o=this.gridOptionsWrapper.isEnableRtl();this.addCssClass(o?"ag-rtl":"ag-ltr");var a=this.createBean(new Sf(n));this.chartController=this.createManagedBean(new fs(a)),this.validateCustomThemes(),this.createChart(),this.params.insideDialog&&this.addDialog(),this.addMenu(),this.addTitleEditComp(),this.addManagedListener(this.getGui(),"focusin",this.setActiveChartCellRange.bind(this)),this.addManagedListener(this.chartController,fs.EVENT_CHART_UPDATED,this.refresh.bind(this)),this.chartMenu&&this.addManagedListener(this.chartMenu,c0.EVENT_DOWNLOAD_CHART,this.downloadChart.bind(this)),this.refresh(),this.raiseChartCreatedEvent()},t.prototype.validateCustomThemes=function(){var e=this.gridOptionsWrapper.getChartThemes(),r=this.gridOptionsWrapper.getCustomChartThemes();r&&_.getAllKeysInObjects([r]).forEach(function(n){_.includes(e,n)||console.warn("AG Grid: a custom chart theme with the name '"+n+"' has been supplied but not added to the 'chartThemes' list")})},t.prototype.createChart=function(){var e=this;this.chartProxy&&this.chartProxy.destroy();var r=function(s,l){var u=e.params.crossFilteringContext;u.lastSelectedChartId=l?"":e.chartController.getChartId(),e.crossFilterService.filter(s,l)},n=this.chartController.getChartType(),o={chartType:n,getChartThemeName:this.getChartThemeName.bind(this),getChartThemes:this.getChartThemes.bind(this),customChartThemes:this.gridOptionsWrapper.getCustomChartThemes(),getGridOptionsChartThemeOverrides:this.getGridOptionsChartThemeOverrides.bind(this),apiChartThemeOverrides:this.params.chartThemeOverrides,crossFiltering:this.params.crossFiltering,crossFilterCallback:r,parentElement:this.eChart,grouping:this.chartController.isGrouping(),chartOptionsToRestore:this.params.chartOptionsToRestore,chartPaletteToRestore:this.params.chartPaletteToRestore,seriesChartTypes:this.chartController.getSeriesChartTypes()};if(this.params.chartOptionsToRestore=void 0,this.chartType=n,this.chartThemeName=this.chartController.getChartThemeName(),this.chartProxy=t.createChartProxy(o),!this.chartProxy){console.warn("AG Grid: invalid chart type supplied: ",o.chartType);return}var a=this.eChart.querySelector("canvas");a&&a.classList.add("ag-charts-canvas"),this.chartController.setChartProxy(this.chartProxy),this.chartOptionsService=this.createBean(new DX(this.chartController)),this.titleEdit&&this.titleEdit.refreshTitle(this.chartController,this.chartOptionsService)},t.prototype.getChartThemeName=function(){return this.chartController.getChartThemeName()},t.prototype.getChartThemes=function(){return this.chartController.getThemes()},t.prototype.getGridOptionsChartThemeOverrides=function(){return this.gridOptionsWrapper.getChartThemeOverrides()},t.createChartProxy=function(e){switch(e.chartType){case"column":case"bar":case"groupedColumn":case"stackedColumn":case"normalizedColumn":case"groupedBar":case"stackedBar":case"normalizedBar":return new fX(e);case"pie":return new bX(e);case"doughnut":return new SX(e);case"area":case"stackedArea":case"normalizedArea":return new vX(e);case"line":return new yX(e);case"scatter":case"bubble":return new OX(e);case"histogram":return new PX(e);case"columnLineCombo":case"areaColumnCombo":case"customCombo":return new FX(e);default:throw"AG Grid: Unable to create chart as an invalid chartType = '"+e.chartType+"' was supplied."}},t.prototype.addDialog=function(){var e=this,r=this.chartTranslationService.translate(this.params.pivotChart?"pivotChartTitle":"rangeChartTitle"),n=this.getBestDialogSize(),o=n.width,a=n.height;this.chartDialog=new Oc({resizable:!0,movable:!0,maximizable:!0,title:r,width:o,height:a,component:this,centered:!0,closable:!0}),this.getContext().createBean(this.chartDialog),this.chartDialog.addEventListener(Oc.EVENT_DESTROYED,function(){return e.destroy()})},t.prototype.getBestDialogSize=function(){var e=this.popupService.getPopupParent(),r=_.getAbsoluteWidth(e)*.75,n=_.getAbsoluteHeight(e)*.75,o=.553,a=this.chartProxy.getChart(),s=this.params.insideDialog?850:a.width,l=this.params.insideDialog?470:a.height;return(s>r||l>n)&&(s=Math.min(s,r),l=Math.round(s*o),l>n&&(l=n,s=Math.min(s,Math.round(l/o)))),{width:s,height:l}},t.prototype.addMenu=function(){this.params.crossFiltering||(this.chartMenu=this.createBean(new c0(this.eChartContainer,this.eMenuContainer,this.chartController,this.chartOptionsService)),this.eChartContainer.appendChild(this.chartMenu.getGui()))},t.prototype.addTitleEditComp=function(){this.titleEdit=this.createBean(new tX(this.chartMenu)),this.eTitleEditContainer.appendChild(this.titleEdit.getGui()),this.chartProxy&&this.titleEdit.refreshTitle(this.chartController,this.chartOptionsService)},t.prototype.refresh=function(){this.shouldRecreateChart()&&this.createChart(),this.updateChart()},t.prototype.shouldRecreateChart=function(){return this.chartType!==this.chartController.getChartType()||this.chartThemeName!==this.chartController.getChartThemeName()},t.prototype.getCurrentChartType=function(){return this.chartType},t.prototype.getChartModel=function(){return this.chartController.getChartModel()},t.prototype.getChartImageDataURL=function(e){return this.chartProxy.getChartImageDataURL(e)},t.prototype.updateChart=function(){var e=this,r=this.chartProxy,n=this.chartController.getSelectedValueColState(),o=n.map(function(c){return{colId:c.colId,displayName:c.displayName}}),a=this.chartController.getChartData(),s=this.handleEmptyChart(a,o);if(!s){var l=this.chartController.getSelectedDimension(),u={data:a,grouping:this.chartController.isGrouping(),category:{id:l.colId,name:l.displayName,chartDataType:this.getChartDataType(l.colId)},fields:o,chartId:this.chartController.getChartId(),getCrossFilteringContext:function(){return e.params.crossFilteringContext},seriesChartTypes:this.chartController.getSeriesChartTypes()};r.update(u),this.titleEdit.refreshTitle(this.chartController,this.chartOptionsService)}},t.prototype.getChartDataType=function(e){var r=this.columnModel.getPrimaryColumn(e);return r?r.getColDef().chartDataType:void 0},t.prototype.handleEmptyChart=function(e,r){var n=this.chartProxy.getChart().container,o=this.chartController.isPivotChart()&&!this.chartController.isPivotMode(),a=1;this.chartController.isActiveXYChart()&&(a=this.chartController.getChartType()==="bubble"?3:2);var s=r.length<a||e.length===0;if(n){var l=o||s;_.setDisplayed(this.eChart,!l),_.setDisplayed(this.eEmpty,l)}return o?(this.eEmpty.innerText=this.chartTranslationService.translate("pivotChartRequiresPivotMode"),!0):s?(this.eEmpty.innerText=this.chartTranslationService.translate("noDataToChart"),!0):!1},t.prototype.downloadChart=function(){this.chartProxy.downloadChart()},t.prototype.getChartId=function(){return this.chartController.getChartId()},t.prototype.getUnderlyingChart=function(){return this.chartProxy.getChart()},t.prototype.refreshCanvasSize=function(){if(!!this.params.insideDialog){var e=this.eChart;if(this.chartMenu.isVisible()){var r=this.chartProxy.getChart();r.height=_.getInnerHeight(e),r.width=_.getInnerWidth(e)}}},t.prototype.setActiveChartCellRange=function(e){this.getGui().contains(e.relatedTarget)||(this.chartController.setChartRange(!0),this.gridApi.focusService.clearFocusedCell())},t.prototype.raiseChartCreatedEvent=function(){var e=Object.freeze({type:x.EVENT_CHART_CREATED,chartId:this.chartController.getChartId(),api:this.gridApi,columnApi:this.columnApi});this.eventService.dispatchEvent(e)},t.prototype.raiseChartDestroyedEvent=function(){var e=Object.freeze({type:x.EVENT_CHART_DESTROYED,chartId:this.chartController.getChartId(),api:this.gridApi,columnApi:this.columnApi});this.eventService.dispatchEvent(e)},t.prototype.destroy=function(){i.prototype.destroy.call(this),this.chartProxy&&this.chartProxy.destroy(),this.destroyBean(this.chartMenu),this.chartDialog&&this.chartDialog.isAlive()&&this.destroyBean(this.chartDialog);var e=this.getGui();_.clearElement(e),_.removeFromParent(e),this.raiseChartDestroyedEvent()},t.TEMPLATE=`<div class="ag-chart" tabindex="-1">
            <div ref="eChartContainer" tabindex="-1" class="ag-chart-components-wrapper">
                <div ref="eChart" class="ag-chart-canvas-wrapper"></div>
                <div ref="eEmpty" class="ag-chart-empty-text ag-unselectable"></div>
            </div>
            <div ref="eTitleEditContainer"></div>
            <div ref="eMenuContainer" class="ag-chart-docked-container"></div>
        </div>`,Jr([I("eChart")],t.prototype,"eChart",void 0),Jr([I("eChartContainer")],t.prototype,"eChartContainer",void 0),Jr([I("eMenuContainer")],t.prototype,"eMenuContainer",void 0),Jr([I("eEmpty")],t.prototype,"eEmpty",void 0),Jr([I("eTitleEditContainer")],t.prototype,"eTitleEditContainer",void 0),Jr([y("environment")],t.prototype,"environment",void 0),Jr([y("columnModel")],t.prototype,"columnModel",void 0),Jr([y("chartCrossFilterService")],t.prototype,"crossFilterService",void 0),Jr([y("chartTranslationService")],t.prototype,"chartTranslationService",void 0),Jr([y("gridApi")],t.prototype,"gridApi",void 0),Jr([y("columnApi")],t.prototype,"columnApi",void 0),Jr([y("popupService")],t.prototype,"popupService",void 0),Jr([B],t.prototype,"init",null),t}(ee),v0="27.3.0";function LX(i){return i.version==null&&(i.version="27.1.0"),NX(i.version),i}function NX(i){var t=typeof i=="string"?i.split(".").map(function(e){return Number(e)}):[];if(t.length!==3||t.some(function(e){return isNaN(e)}))throw new Error("AG Grid - Illegal version string: "+i);return{major:t[0],minor:t[1],patch:t[2]}}var GX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Gd=globalThis&&globalThis.__assign||function(){return Gd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Gd.apply(this,arguments)},Gu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},kX=function(i){GX(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.activeCharts=new Set,e.activeChartComps=new Set,e.crossFilteringContext={lastSelectedChartId:""},e}return t.prototype.getChartModels=function(){var e=[],r=function(n){return Gd(Gd({},n),{version:v0})};return this.activeChartComps.forEach(function(n){return e.push(r(n.getChartModel()))}),e},t.prototype.getChartRef=function(e){var r;return this.activeCharts.forEach(function(n){n.chartId===e&&(r=n)}),r},t.prototype.getChartImageDataURL=function(e){var r;return this.activeChartComps.forEach(function(n){n.getChartId()===e.chartId&&(r=n.getChartImageDataURL(e.fileFormat))}),r},t.prototype.createChartFromCurrentRange=function(e){e===void 0&&(e="groupedColumn");var r=this.getSelectedRange();return this.createChart(r,e)},t.prototype.restoreChart=function(e,r){var n=this;if(!e){console.warn("AG Grid - unable to restore chart as no chart model is provided");return}e.version!==v0&&(e=LX(e));var o={cellRange:e.cellRange,chartType:e.chartType,chartThemeName:e.chartThemeName,chartContainer:r,suppressChartRanges:e.suppressChartRanges,aggFunc:e.aggFunc,unlinkChart:e.unlinkChart,seriesChartTypes:e.seriesChartTypes},a=function(p){return n.rangeService?n.rangeService.createCellRangeFromCellRangeParams(p):void 0};if(e.modelType==="pivot"){this.columnModel.isPivotMode()||this.columnModel.setPivotMode(!0,"pivotChart");var s=this.columnModel.getAllDisplayedColumns().map(function(p){return p.getColId()}),l={rowStartIndex:null,rowEndIndex:null,columns:s},u=a(l);if(!u){console.warn("AG Grid - unable to create chart as there are no columns in the grid.");return}return this.createChart(u,o.chartType,o.chartThemeName,!0,!0,o.chartContainer,void 0,void 0,o.unlinkChart,!1,e.chartOptions)}var c=a(o.cellRange);if(!c){console.warn("AG Grid - unable to create chart as no range is selected");return}return this.createChart(c,o.chartType,o.chartThemeName,!1,o.suppressChartRanges,o.chartContainer,o.aggFunc,void 0,o.unlinkChart,!1,e.chartOptions,e.chartPalette,o.seriesChartTypes)},t.prototype.createRangeChart=function(e){var r=this.rangeService?this.rangeService.createCellRangeFromCellRangeParams(e.cellRange):void 0;if(!r){console.warn("AG Grid - unable to create chart as no range is selected");return}return this.createChart(r,e.chartType,e.chartThemeName,!1,e.suppressChartRanges,e.chartContainer,e.aggFunc,e.chartThemeOverrides,e.unlinkChart,void 0,void 0,void 0,e.seriesChartTypes)},t.prototype.createPivotChart=function(e){this.columnModel.isPivotMode()||this.columnModel.setPivotMode(!0,"pivotChart");var r={rowStartIndex:null,rowEndIndex:null,columns:this.columnModel.getAllDisplayedColumns().map(function(o){return o.getColId()})},n=this.rangeService?this.rangeService.createCellRangeFromCellRangeParams(r):void 0;if(!n){console.warn("AG Grid - unable to create chart as there are no columns in the grid.");return}return this.createChart(n,e.chartType,e.chartThemeName,!0,!0,e.chartContainer,void 0,e.chartThemeOverrides,e.unlinkChart)},t.prototype.createCrossFilterChart=function(e){var r=this.rangeService?this.rangeService.createCellRangeFromCellRangeParams(e.cellRange):void 0;if(!r){console.warn("AG Grid - unable to create chart as no range is selected");return}var n=!0,o=typeof e.suppressChartRanges!="undefined"&&e.suppressChartRanges!==null,a=o?e.suppressChartRanges:!0;return this.createChart(r,e.chartType,e.chartThemeName,!1,a,e.chartContainer,e.aggFunc,e.chartThemeOverrides,e.unlinkChart,n)},t.prototype.createChart=function(e,r,n,o,a,s,l,u,c,p,h,d,f){var g=this;o===void 0&&(o=!1),a===void 0&&(a=!1),c===void 0&&(c=!1),p===void 0&&(p=!1);var v=this.gridOptionsWrapper.getCreateChartContainerFunc(),m={chartId:this.generateId(),pivotChart:o,cellRange:e,chartType:r,chartThemeName:n,insideDialog:!(s||v),suppressChartRanges:a,aggFunc:l,chartThemeOverrides:u,unlinkChart:c,crossFiltering:p,crossFilteringContext:this.crossFilteringContext,chartOptionsToRestore:h,chartPaletteToRestore:d,seriesChartTypes:f},C=new g0(m);this.context.createBean(C);var w=this.createChartRef(C);if(s){s.appendChild(C.getGui());var b=this.environment.getTheme();b.el&&!b.el.contains(s)&&s.classList.add(b.theme)}else v?v(w):C.addEventListener(g0.EVENT_DESTROYED,function(){g.activeChartComps.delete(C),g.activeCharts.delete(w)});return w},t.prototype.createChartRef=function(e){var r=this,n={destroyChart:function(){r.activeCharts.has(n)&&(r.context.destroyBean(e),r.activeChartComps.delete(e),r.activeCharts.delete(n))},chartElement:e.getGui(),chart:e.getUnderlyingChart(),chartId:e.getChartModel().chartId};return this.activeCharts.add(n),this.activeChartComps.add(e),n},t.prototype.getSelectedRange=function(){var e=this.rangeService.getCellRanges();return e.length>0?e[0]:{}},t.prototype.generateId=function(){return"id-"+Math.random().toString(36).substr(2,16)},t.prototype.destroyAllActiveCharts=function(){this.activeCharts.forEach(function(e){return e.destroyChart()})},Gu([De("rangeService")],t.prototype,"rangeService",void 0),Gu([y("columnModel")],t.prototype,"columnModel",void 0),Gu([y("environment")],t.prototype,"environment",void 0),Gu([nt],t.prototype,"destroyAllActiveCharts",null),t=Gu([Q("chartService")],t),t}(H),WX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),VX=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},BX=function(i){WX(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}e=t,t.prototype.translate=function(r,n){var o=this.gridOptionsWrapper.getLocaleTextFunc(),a=e.DEFAULT_TRANSLATIONS[r]||n;return o(r,a)};var e;return t.DEFAULT_TRANSLATIONS={pivotChartTitle:"Pivot Chart",rangeChartTitle:"Range Chart",settings:"Settings",data:"Data",format:"Format",categories:"Categories",defaultCategory:"(None)",series:"Series",xyValues:"X Y Values",paired:"Paired Mode",axis:"Axis",navigator:"Navigator",color:"Color",thickness:"Thickness",xType:"X Type",automatic:"Automatic",category:"Category",number:"Number",time:"Time",xRotation:"X Rotation",yRotation:"Y Rotation",ticks:"Ticks",width:"Width",height:"Height",length:"Length",padding:"Padding",spacing:"Spacing",chart:"Chart",title:"Title",titlePlaceholder:"Chart title - double click to edit",background:"Background",font:"Font",top:"Top",right:"Right",bottom:"Bottom",left:"Left",labels:"Labels",size:"Size",shape:"Shape",minSize:"Minimum Size",maxSize:"Maximum Size",legend:"Legend",position:"Position",markerSize:"Marker Size",markerStroke:"Marker Stroke",markerPadding:"Marker Padding",itemSpacing:"Item Spacing",itemPaddingX:"Item Padding X",itemPaddingY:"Item Padding Y",layoutHorizontalSpacing:"Horizontal Spacing",layoutVerticalSpacing:"Vertical Spacing",strokeWidth:"Stroke Width",offset:"Offset",offsets:"Offsets",tooltips:"Tooltips",callout:"Callout",markers:"Markers",shadow:"Shadow",blur:"Blur",xOffset:"X Offset",yOffset:"Y Offset",lineWidth:"Line Width",lineDash:"Line Dash",normal:"Normal",bold:"Bold",italic:"Italic",boldItalic:"Bold Italic",predefined:"Predefined",fillOpacity:"Fill Opacity",strokeOpacity:"Line Opacity",histogramBinCount:"Bin count",columnGroup:"Column",barGroup:"Bar",pieGroup:"Pie",lineGroup:"Line",scatterGroup:"X Y (Scatter)",areaGroup:"Area",histogramGroup:"Histogram",combinationGroup:"Combination",groupedColumnTooltip:"Grouped",stackedColumnTooltip:"Stacked",normalizedColumnTooltip:"100% Stacked",groupedBarTooltip:"Grouped",stackedBarTooltip:"Stacked",normalizedBarTooltip:"100% Stacked",pieTooltip:"Pie",doughnutTooltip:"Doughnut",lineTooltip:"Line",groupedAreaTooltip:"Area",stackedAreaTooltip:"Stacked",normalizedAreaTooltip:"100% Stacked",scatterTooltip:"Scatter",bubbleTooltip:"Bubble",histogramTooltip:"Histogram",columnLineComboTooltip:"Column & Line",areaColumnComboTooltip:"Area & Column",customComboTooltip:"Custom Combination",noDataToChart:"No data available to be charted.",pivotChartRequiresPivotMode:"Pivot Chart requires Pivot Mode enabled.",chartSettingsToolbarTooltip:"Menu",chartLinkToolbarTooltip:"Linked to Grid",chartUnlinkToolbarTooltip:"Unlinked from Grid",chartDownloadToolbarTooltip:"Download Chart",seriesChartType:"Series Chart Type",seriesType:"Series Type",secondaryAxis:"Secondary Axis"},t=e=VX([Q("chartTranslationService")],t),t}(H),HX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),hh=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},jX=function(i){HX(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}e=t,t.prototype.filter=function(r,n){n===void 0&&(n=!1);var o=this.gridApi.getFilterModel();if(n){this.resetFilters(o);return}var a=e.extractFilterColId(r);this.isValidColumnFilter(a)?this.updateFilters(o,r,a):console.warn("AG Grid: cross filtering requires a 'agSetColumnFilter' or 'agMultiColumnFilter' to be defined on the column with id: '"+a+"'")},t.prototype.resetFilters=function(r){var n=Object.keys(r).length>0;n&&(this.gridApi.setFilterModel(null),this.gridApi.onFilterChanged())},t.prototype.updateFilters=function(r,n,o){var a,s=e.extractFilterColId(n),l=n.datum[s];if(l!==void 0){var u=l.toString();if(n.event.metaKey||n.event.ctrlKey){var c=this.getCurrentGridValuesForCategory(o),p=_.includes(c,u),h=void 0;p?h=c.filter(function(d){return d!==u}):(h=c,h.push(u)),r[o]=this.getUpdatedFilterModel(o,h)}else{var h=[u];r=(a={},a[o]=this.getUpdatedFilterModel(o,h),a)}this.gridApi.setFilterModel(r)}},t.prototype.getUpdatedFilterModel=function(r,n){var o=this.getColumnFilterType(r);return o==="agMultiColumnFilter"?{filterType:"multi",filterModels:[null,{filterType:"set",values:n}]}:{filterType:"set",values:n}},t.prototype.getCurrentGridValuesForCategory=function(r){var n=this,o=[],a=this.getColumnById(r);return this.gridApi.forEachNodeAfterFilter(function(s){if(a&&!s.group){var l=n.valueService.getValue(a,s)+"";o.includes(l)||o.push(l)}}),o},t.extractFilterColId=function(r){return r.xKey?r.xKey:r.labelKey},t.prototype.isValidColumnFilter=function(r){r.indexOf("-filtered-out")&&(r=r.replace("-filtered-out",""));var n=this.getColumnFilterType(r);return typeof n=="boolean"?n:_.includes(["agSetColumnFilter","agMultiColumnFilter"],n)},t.prototype.getColumnFilterType=function(r){var n=this.getColumnById(r);if(n){var o=n.getColDef();return o.filter!=null?o.filter:o.filterFramework}},t.prototype.getColumnById=function(r){return this.columnModel.getGridColumn(r)};var e;return hh([y("gridApi")],t.prototype,"gridApi",void 0),hh([y("columnModel")],t.prototype,"columnModel",void 0),hh([y("valueService")],t.prototype,"valueService",void 0),t=e=hh([Q("chartCrossFilterService")],t),t}(H),UX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),xn=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},zX=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Yv=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(zX(arguments[t]));return i},$X=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},KX=function(i){UX(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.cellRanges=[],e.bodyScrollListener=e.onBodyScroll.bind(e),e.dragging=!1,e}return t.prototype.init=function(){var e=this;this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,function(){return e.onColumnsChanged()}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VISIBLE,this.onColumnsChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_VALUE_CHANGED,this.onColumnsChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,function(){return e.removeAllCellRanges()}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,function(){return e.removeAllCellRanges()}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_CHANGED,function(){return e.removeAllCellRanges()}),this.addManagedListener(this.eventService,x.EVENT_COLUMN_GROUP_OPENED,this.refreshLastRangeStart.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_MOVED,this.refreshLastRangeStart.bind(this)),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PINNED,this.refreshLastRangeStart.bind(this)),this.ctrlsService.whenReady(function(){var r=e.ctrlsService.getGridBodyCtrl();e.autoScrollService=new Xc({scrollContainer:r.getBodyViewportElement(),scrollAxis:"xy",getVerticalPosition:function(){return r.getScrollFeature().getVScrollPosition().top},setVerticalPosition:function(n){return r.getScrollFeature().setVerticalScrollPosition(n)},getHorizontalPosition:function(){return r.getScrollFeature().getHScrollPosition().left},setHorizontalPosition:function(n){return r.getScrollFeature().setHorizontalScrollPosition(n)},shouldSkipVerticalScroll:function(){return e.gridOptionsWrapper.getDomLayout()!==L.DOM_LAYOUT_NORMAL},shouldSkipHorizontalScroll:function(){return r.getScrollFeature().isHorizontalScrollShowing()}})})},t.prototype.onColumnsChanged=function(){var e=this;this.refreshLastRangeStart();var r=this.columnModel.getAllDisplayedColumns();this.cellRanges.forEach(function(o){var a=o.columns;o.columns=o.columns.filter(function(l){return l.isVisible()&&r.indexOf(l)!==-1});var s=!_.areEqual(a,o.columns);s&&e.dispatchChangedEvent(!1,!0,o.id)});var n=this.cellRanges.length;this.cellRanges=this.cellRanges.filter(function(o){return o.columns.length>0}),n>this.cellRanges.length&&this.dispatchChangedEvent(!1,!0)},t.prototype.refreshLastRangeStart=function(){var e=_.last(this.cellRanges);!e||this.refreshRangeStart(e)},t.prototype.isContiguousRange=function(e){var r=e.columns;if(!r.length)return!1;var n=this.columnModel.getAllDisplayedColumns(),o=r.map(function(a){return n.indexOf(a)}).sort(function(a,s){return a-s});return _.last(o)-o[0]+1===r.length},t.prototype.getRangeStartRow=function(e){if(e.startRow&&e.endRow)return this.rowPositionUtils.before(e.startRow,e.endRow)?e.startRow:e.endRow;var r=this.pinnedRowModel.getPinnedTopRowCount()>0?L.PINNED_TOP:null;return{rowIndex:0,rowPinned:r}},t.prototype.getRangeEndRow=function(e){if(e.startRow&&e.endRow)return this.rowPositionUtils.before(e.startRow,e.endRow)?e.endRow:e.startRow;var r=this.pinnedRowModel.getPinnedBottomRowCount(),n=r>0;return n?{rowIndex:r-1,rowPinned:L.PINNED_BOTTOM}:{rowIndex:this.rowModel.getRowCount()-1,rowPinned:null}},t.prototype.setRangeToCell=function(e,r){if(r===void 0&&(r=!1),!!this.gridOptionsWrapper.isEnableRangeSelection()){var n=this.calculateColumnsBetween(e.column,e.column);if(!!n){var o=this.gridOptionsWrapper.isSuppressMultiRangeSelection();(o||!r||_.missing(this.cellRanges))&&this.removeAllCellRanges(!0);for(var a={rowIndex:e.rowIndex,rowPinned:e.rowPinned},s,l=0;l<this.cellRanges.length;l++){var u=this.cellRanges[l],c=u.columns&&u.columns.length===1&&u.columns[0]===e.column&&this.rowPositionUtils.sameRow(a,u.startRow)&&this.rowPositionUtils.sameRow(a,u.endRow);if(c){s=u;break}}s?_.last(this.cellRanges)!==s&&(_.removeFromArray(this.cellRanges,s),this.cellRanges.push(s)):(s={startRow:a,endRow:a,columns:n,startColumn:e.column},this.cellRanges.push(s)),this.setNewestRangeStartCell(e),this.onDragStop(),this.dispatchChangedEvent(!0,!0,s.id)}}},t.prototype.extendLatestRangeToCell=function(e){if(!(this.isEmpty()||!this.newestRangeStartCell)){var r=_.last(this.cellRanges);this.updateRangeEnd(r,e)}},t.prototype.updateRangeEnd=function(e,r,n){n===void 0&&(n=!1);var o=r.column,a=this.calculateColumnsBetween(e.startColumn,o);!a||this.isLastCellOfRange(e,r)||(e.columns=a,e.endRow={rowIndex:r.rowIndex,rowPinned:r.rowPinned},n||this.dispatchChangedEvent(!0,!0,e.id))},t.prototype.refreshRangeStart=function(e){var r=e.startColumn,n=e.columns,o=function(p,h){var d=e.columns.filter(function(f){return f!==p});p?(e.startColumn=p,e.columns=h?Yv([p],d):Yv(d,[p])):e.columns=d},a=this.getRangeEdgeColumns(e),s=a.left,l=a.right,u=r===n[0]&&r!==s;if(u){o(s,!0);return}var c=r===_.last(n)&&r===l;if(c){o(l,!1);return}},t.prototype.getRangeEdgeColumns=function(e){var r=this.columnModel.getAllDisplayedColumns(),n=e.columns.map(function(o){return r.indexOf(o)}).filter(function(o){return o>-1}).sort(function(o,a){return o-a});return{left:r[n[0]],right:r[_.last(n)]}},t.prototype.extendLatestRangeInDirection=function(e){if(!(this.isEmpty()||!this.newestRangeStartCell)){var r=e.key,n=e.ctrlKey||e.metaKey,o=_.last(this.cellRanges),a=this.newestRangeStartCell,s=o.columns[0],l=_.last(o.columns),u=o.endRow.rowIndex,c=o.endRow.rowPinned,p=a.column===s?l:s,h={column:p,rowIndex:u,rowPinned:c},d=this.cellNavigationService.getNextCellToFocus(r,h,n);if(!!d)return this.setCellRange({rowStartIndex:a.rowIndex,rowStartPinned:a.rowPinned,rowEndIndex:d.rowIndex,rowEndPinned:d.rowPinned,columnStart:a.column,columnEnd:d.column}),d}},t.prototype.setCellRange=function(e){!this.gridOptionsWrapper.isEnableRangeSelection()||(this.removeAllCellRanges(!0),this.addCellRange(e))},t.prototype.setCellRanges=function(e){var r=this;_.shallowCompare(this.cellRanges,e)||(this.removeAllCellRanges(!0),e.forEach(function(n){n.columns&&n.startRow&&r.setNewestRangeStartCell({rowIndex:n.startRow.rowIndex,rowPinned:n.startRow.rowPinned,column:n.columns[0]}),r.cellRanges.push(n)}),this.dispatchChangedEvent(!1,!0))},t.prototype.setNewestRangeStartCell=function(e){this.newestRangeStartCell=e},t.prototype.createCellRangeFromCellRangeParams=function(e){var r=this,n,o=!1;if(e.columns)n=e.columns.map(function(c){return r.columnModel.getColumnWithValidation(c)}).filter(function(c){return c});else{var a=this.columnModel.getColumnWithValidation(e.columnStart),s=this.columnModel.getColumnWithValidation(e.columnEnd);if(!a||!s)return;n=this.calculateColumnsBetween(a,s),n&&n.length&&(o=n[0]!==a)}if(!!n){var l=e.rowStartIndex!=null?{rowIndex:e.rowStartIndex,rowPinned:e.rowStartPinned||null}:void 0,u=e.rowEndIndex!=null?{rowIndex:e.rowEndIndex,rowPinned:e.rowEndPinned||null}:void 0;return{startRow:l,endRow:u,columns:n,startColumn:o?_.last(n):n[0]}}},t.prototype.addCellRange=function(e){if(!!this.gridOptionsWrapper.isEnableRangeSelection()){var r=this.createCellRangeFromCellRangeParams(e);r&&(r.startRow&&this.setNewestRangeStartCell({rowIndex:r.startRow.rowIndex,rowPinned:r.startRow.rowPinned,column:r.startColumn}),this.cellRanges.push(r),this.dispatchChangedEvent(!1,!0,r.id))}},t.prototype.getCellRanges=function(){return this.cellRanges},t.prototype.isEmpty=function(){return this.cellRanges.length===0},t.prototype.isMoreThanOneCell=function(){var e=this.cellRanges.length;if(e===0)return!1;if(e>1)return!0;var r=this.cellRanges[0],n=this.getRangeStartRow(r),o=this.getRangeEndRow(r);return n.rowPinned!==o.rowPinned||n.rowIndex!==o.rowIndex||r.columns.length!==1},t.prototype.areAllRangesAbleToMerge=function(){var e,r,n=this,o=new Map,a=this.cellRanges.length;if(a<=1)return!0;this.cellRanges.forEach(function(h){for(var d=n.getRangeStartRow(h),f=n.getRangeEndRow(h),g=d,v=function(){var C=(g.rowPinned||"normal")+"_"+g.rowIndex,w=o.get(C),b=h.columns.map(function(T){return T.getId()});if(w){var S=b.filter(function(T){return w.indexOf(T)===-1});w.push.apply(w,Yv(S))}else o.set(C,b);if(n.rowPositionUtils.sameRow(g,f))return"break";g=n.cellNavigationService.getRowBelow(g)};g;){var m=v();if(m==="break")break}});var s;try{for(var l=$X(o.values()),u=l.next();!u.done;u=l.next()){var c=u.value,p=c.sort().join();if(s===void 0){s=p;continue}if(s!==p)return!1}}catch(h){e={error:h}}finally{try{u&&!u.done&&(r=l.return)&&r.call(l)}finally{if(e)throw e.error}}return!0},t.prototype.removeAllCellRanges=function(e){this.isEmpty()||(this.onDragStop(),this.cellRanges.length=0,e||this.dispatchChangedEvent(!1,!0))},t.prototype.onBodyScroll=function(){this.dragging&&this.lastMouseEvent&&this.onDragging(this.lastMouseEvent)},t.prototype.isCellInAnyRange=function(e){return this.getCellRangeCount(e)>0},t.prototype.isCellInSpecificRange=function(e,r){var n=r.columns!==null&&_.includes(r.columns,e.column),o=this.isRowInRange(e.rowIndex,e.rowPinned,r);return n&&o},t.prototype.isLastCellOfRange=function(e,r){var n=e.startRow,o=e.endRow,a=this.rowPositionUtils.before(n,o)?o:n,s=r.rowIndex===a.rowIndex&&r.rowPinned===a.rowPinned,l=e.columns[0],u=_.last(e.columns),c=e.startColumn===l?u:l,p=r.column===c;return p&&s},t.prototype.isBottomRightCell=function(e,r){var n=this.columnModel.getAllDisplayedColumns(),o=e.columns.map(function(p){return n.indexOf(p)}).sort(function(p,h){return p-h}),a=e.startRow,s=e.endRow,l=this.rowPositionUtils.before(a,s)?s:a,u=n.indexOf(r.column)===_.last(o),c=r.rowIndex===l.rowIndex&&_.makeNull(r.rowPinned)===_.makeNull(l.rowPinned);return u&&c},t.prototype.getCellRangeCount=function(e){var r=this;return this.isEmpty()?0:this.cellRanges.filter(function(n){return r.isCellInSpecificRange(e,n)}).length},t.prototype.isRowInRange=function(e,r,n){var o=this.getRangeStartRow(n),a=this.getRangeEndRow(n),s={rowIndex:e,rowPinned:r||null},l=s.rowIndex===o.rowIndex&&s.rowPinned==o.rowPinned,u=s.rowIndex===a.rowIndex&&s.rowPinned==a.rowPinned;if(l||u)return!0;var c=!this.rowPositionUtils.before(s,o),p=this.rowPositionUtils.before(s,a);return c&&p},t.prototype.getDraggingRange=function(){return this.draggingRange},t.prototype.onDragStart=function(e){if(!!this.gridOptionsWrapper.isEnableRangeSelection()){var r=e.ctrlKey,n=e.metaKey,o=e.shiftKey,a=r||n,s=!this.gridOptionsWrapper.isSuppressMultiRangeSelection(),l=s?a:!1,u=o&&_.existsAndNotEmpty(this.cellRanges);if(!l&&(!u||_.exists(_.last(this.cellRanges).type))&&this.removeAllCellRanges(!0),this.cellHoverListener=this.addManagedListener(this.ctrlsService.getGridCtrl().getGui(),"mousemove",this.updateValuesOnMove.bind(this)),this.updateValuesOnMove(e),!!this.lastCellHovered){if(this.dragging=!0,this.lastMouseEvent=e,u||this.setNewestRangeStartCell(this.lastCellHovered),this.cellRanges.length>0)this.draggingRange=_.last(this.cellRanges);else{var c={rowIndex:this.lastCellHovered.rowIndex,rowPinned:this.lastCellHovered.rowPinned};this.draggingRange={startRow:c,endRow:c,columns:[this.lastCellHovered.column],startColumn:this.newestRangeStartCell.column},this.cellRanges.push(this.draggingRange)}this.ctrlsService.getGridBodyCtrl().addScrollEventListener(this.bodyScrollListener),this.dispatchChangedEvent(!0,!1,this.draggingRange.id)}}},t.prototype.updateValuesOnMove=function(e){var r=this.mouseEventService.getCellPositionForEvent(e);this.cellHasChanged=!1,!(!r||this.lastCellHovered&&this.cellPositionUtils.equals(r,this.lastCellHovered))&&(this.lastCellHovered&&(this.cellHasChanged=!0),this.lastCellHovered=r)},t.prototype.onDragging=function(e){var r=this;if(!(!this.dragging||!e)){this.lastMouseEvent=e;var n=this.lastCellHovered,o=function(l){return n&&n.rowPinned===l&&r.newestRangeStartCell.rowPinned===l},a=o("top")||o("bottom");if(this.autoScrollService.check(e,a),!!this.cellHasChanged){var s=this.calculateColumnsBetween(this.newestRangeStartCell.column,n.column);!s||(this.draggingRange.endRow={rowIndex:n.rowIndex,rowPinned:n.rowPinned},this.draggingRange.columns=s,this.dispatchChangedEvent(!1,!1,this.draggingRange.id))}}},t.prototype.onDragStop=function(){if(this.cellHoverListener&&(this.cellHoverListener(),this.cellHoverListener=void 0),!!this.dragging){var e=this.draggingRange.id;this.autoScrollService.ensureCleared(),this.ctrlsService.getGridBodyCtrl().removeScrollEventListener(this.bodyScrollListener),this.lastMouseEvent=null,this.dragging=!1,this.draggingRange=void 0,this.lastCellHovered=void 0,this.dispatchChangedEvent(!1,!0,e)}},t.prototype.dispatchChangedEvent=function(e,r,n){var o=Object.freeze({type:x.EVENT_RANGE_SELECTION_CHANGED,api:this.gridApi,columnApi:this.columnApi,started:e,finished:r,id:n});this.eventService.dispatchEvent(o)},t.prototype.calculateColumnsBetween=function(e,r){var n=this.columnModel.getAllDisplayedColumns(),o=e===r,a=n.indexOf(e);if(a<0){console.warn("AG Grid: column "+e.getId()+" is not visible");return}var s=o?a:n.indexOf(r);if(s<0){console.warn("AG Grid: column "+r.getId()+" is not visible");return}if(o)return[e];for(var l=Math.min(a,s),u=l===a?s:a,c=[],p=l;p<=u;p++)c.push(n[p]);return c},xn([y("rowModel")],t.prototype,"rowModel",void 0),xn([y("columnModel")],t.prototype,"columnModel",void 0),xn([y("mouseEventService")],t.prototype,"mouseEventService",void 0),xn([y("columnApi")],t.prototype,"columnApi",void 0),xn([y("gridApi")],t.prototype,"gridApi",void 0),xn([y("cellNavigationService")],t.prototype,"cellNavigationService",void 0),xn([y("pinnedRowModel")],t.prototype,"pinnedRowModel",void 0),xn([y("rowPositionUtils")],t.prototype,"rowPositionUtils",void 0),xn([y("cellPositionUtils")],t.prototype,"cellPositionUtils",void 0),xn([y("ctrlsService")],t.prototype,"ctrlsService",void 0),xn([B],t.prototype,"init",null),t=xn([Q("rangeService")],t),t}(H),YX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),jn=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},iD=function(i){YX(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.changedCalculatedValues=!1,e.dragging=!1,e.shouldDestroyOnEndDragging=!1,e}return t.prototype.init=function(){var e=this;this.dragService.addDragSource({dragStartPixels:0,eElement:this.getGui(),onDragStart:this.onDragStart.bind(this),onDragging:function(r){e.dragging=!0,e.rangeService.autoScrollService.check(r),e.changedCalculatedValues&&(e.onDrag(r),e.changedCalculatedValues=!1)},onDragStop:function(r){e.dragging=!1,e.onDragEnd(r),e.clearValues(),e.rangeService.autoScrollService.ensureCleared(),document.body.classList.remove(e.getDraggingCssClass()),e.shouldDestroyOnEndDragging&&e.destroy()}}),this.addManagedListener(this.getGui(),"mousedown",this.preventRangeExtension.bind(this))},t.prototype.isDragging=function(){return this.dragging},t.prototype.getCellCtrl=function(){return this.cellCtrl},t.prototype.setCellCtrl=function(e){this.cellCtrl=e},t.prototype.getCellRange=function(){return this.cellRange},t.prototype.setCellRange=function(e){this.cellRange=e},t.prototype.getRangeStartRow=function(){return this.rangeStartRow},t.prototype.setRangeStartRow=function(e){this.rangeStartRow=e},t.prototype.getRangeEndRow=function(){return this.rangeEndRow},t.prototype.setRangeEndRow=function(e){this.rangeEndRow=e},t.prototype.getLastCellHovered=function(){return this.lastCellHovered},t.prototype.preventRangeExtension=function(e){e.preventDefault(),e.stopPropagation()},t.prototype.onDragStart=function(e){this.cellHoverListener=this.addManagedListener(this.ctrlsService.getGridCtrl().getGui(),"mousemove",this.updateValuesOnMove.bind(this)),document.body.classList.add(this.getDraggingCssClass())},t.prototype.getDraggingCssClass=function(){return"ag-dragging-"+(this.type===co.FILL?"fill":"range")+"-handle"},t.prototype.updateValuesOnMove=function(e){var r=this.mouseEventService.getCellPositionForEvent(e);!r||this.lastCellHovered&&this.cellPositionUtils.equals(r,this.lastCellHovered)||(this.lastCellHovered=r,this.changedCalculatedValues=!0)},t.prototype.getType=function(){return this.type},t.prototype.refresh=function(e){var r=this.getCellCtrl(),n=this.getGui(),o=_.last(this.rangeService.getCellRanges()),a=o.startRow,s=o.endRow;if(a&&s){var l=this.rowPositionUtils.before(s,a);l?(this.setRangeStartRow(s),this.setRangeEndRow(a)):(this.setRangeStartRow(a),this.setRangeEndRow(s))}if(r!==e||!_.isVisible(n)){this.setCellCtrl(e);var u=e.getComp().getParentOfValue();u&&u.appendChild(n)}this.setCellRange(o)},t.prototype.clearValues=function(){this.lastCellHovered=void 0,this.removeListeners()},t.prototype.removeListeners=function(){this.cellHoverListener&&(this.cellHoverListener(),this.cellHoverListener=void 0)},t.prototype.destroy=function(){if(!this.shouldDestroyOnEndDragging&&this.isDragging()){_.setDisplayed(this.getGui(),!1),this.shouldDestroyOnEndDragging=!0;return}this.shouldDestroyOnEndDragging=!1,i.prototype.destroy.call(this),this.removeListeners();var e=this.getGui();e.parentElement&&e.parentElement.removeChild(e)},jn([y("rowRenderer")],t.prototype,"rowRenderer",void 0),jn([y("dragService")],t.prototype,"dragService",void 0),jn([y("rangeService")],t.prototype,"rangeService",void 0),jn([y("mouseEventService")],t.prototype,"mouseEventService",void 0),jn([y("columnModel")],t.prototype,"columnModel",void 0),jn([y("cellNavigationService")],t.prototype,"cellNavigationService",void 0),jn([y("navigationService")],t.prototype,"navigationService",void 0),jn([y("rowPositionUtils")],t.prototype,"rowPositionUtils",void 0),jn([y("cellPositionUtils")],t.prototype,"cellPositionUtils",void 0),jn([y("ctrlsService")],t.prototype,"ctrlsService",void 0),jn([B],t.prototype,"init",null),t}(ee),XX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),wl=globalThis&&globalThis.__assign||function(){return wl=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},wl.apply(this,arguments)},qX=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},ZX=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},QX=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(ZX(arguments[t]));return i},oD=function(i){XX(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.markedCells=[],e.cellValues=[],e.isUp=!1,e.isLeft=!1,e.isReduce=!1,e.type=co.FILL,e}return t.prototype.updateValuesOnMove=function(e){i.prototype.updateValuesOnMove.call(this,e),this.initialXY||(this.initialXY=this.mouseEventService.getNormalisedPosition(e));var r=this.initialXY,n=r.x,o=r.y,a=this.mouseEventService.getNormalisedPosition(e),s=a.x,l=a.y,u=Math.abs(n-s),c=Math.abs(o-l),p=this.gridOptionsWrapper.getFillHandleDirection(),h;p==="xy"?h=u>c?"x":"y":h=p,h!==this.dragAxis&&(this.dragAxis=h,this.changedCalculatedValues=!0)},t.prototype.onDrag=function(e){if(!this.initialPosition){var r=this.getCellCtrl();if(!r)return;this.initialPosition=r.getCellPosition()}var n=this.getLastCellHovered();n&&this.markPathFrom(this.initialPosition,n)},t.prototype.onDragEnd=function(e){if(this.initialXY=null,!!this.markedCells.length){var r=this.dragAxis==="x",n=this.getCellRange(),o=n.columns.length,a=this.getRangeStartRow(),s=this.getRangeEndRow(),l;if(!this.isUp&&!this.isLeft)l=this.rangeService.createCellRangeFromCellRangeParams({rowStartIndex:a.rowIndex,rowStartPinned:a.rowPinned,columnStart:n.columns[0],rowEndIndex:r?s.rowIndex:this.lastCellMarked.rowIndex,rowEndPinned:r?s.rowPinned:this.lastCellMarked.rowPinned,columnEnd:r?this.lastCellMarked.column:n.columns[o-1]});else{var u=r?a:this.lastCellMarked;l=this.rangeService.createCellRangeFromCellRangeParams({rowStartIndex:u.rowIndex,rowStartPinned:u.rowPinned,columnStart:r?this.lastCellMarked.column:n.columns[0],rowEndIndex:s.rowIndex,rowEndPinned:s.rowPinned,columnEnd:n.columns[o-1]})}l&&(this.raiseFillStartEvent(),this.handleValueChanged(n,l,e),this.rangeService.setCellRanges([l]),this.raiseFillEndEvent(n,l))}},t.prototype.raiseFillStartEvent=function(){var e={type:x.EVENT_FILL_START,columnApi:this.gridOptionsWrapper.getColumnApi(),api:this.gridOptionsWrapper.getApi()};this.eventService.dispatchEvent(e)},t.prototype.raiseFillEndEvent=function(e,r){var n={type:x.EVENT_FILL_END,columnApi:this.gridOptionsWrapper.getColumnApi(),api:this.gridOptionsWrapper.getApi(),initialRange:e,finalRange:r};this.eventService.dispatchEvent(n)},t.prototype.handleValueChanged=function(e,r,n){var o=this,a=this.rangeService.getRangeEndRow(e),s=this.rangeService.getRangeStartRow(e),l=this.rangeService.getRangeEndRow(r),u=this.rangeService.getRangeStartRow(r),c=this.dragAxis==="y";if(this.isReduce&&!this.gridOptionsWrapper.isSuppressClearOnFillReduction()){var p=c?e.columns:e.columns.filter(function(b){return r.columns.indexOf(b)<0}),h=c?this.cellNavigationService.getRowBelow(l):u;h&&this.clearCellsInRange(h,a,p);return}var d=!0,f=[],g=[],v=0,m=function(){f.length=0,g.length=0,v=0},C=function(b,S){var T=o.isUp?a:s,E=!1;c&&(d=!0,m());for(var P=function(){var M=o.rowPositionUtils.getRowNode(T);if(!M)return"break";c&&b?w(f,b,M,function(){return!o.rowPositionUtils.sameRow(T,o.isUp?s:a)}):S&&(d=!0,m(),S.forEach(function(N){return w(f,N,M,function(){return N!==(o.isLeft?e.columns[0]:_.last(e.columns))})})),E=o.rowPositionUtils.sameRow(T,o.isUp?u:l),T=o.isUp?o.cellNavigationService.getRowAbove(T):o.cellNavigationService.getRowBelow(T)};!E&&T;){var R=P();if(R==="break")break}},w=function(b,S,T,E){var P,R=!1;if(d)P=o.valueService.getValue(S,T),g.push(P),d=E();else{var M=o.processValues(n,b,g,S,T,v++),N=M.value,W=M.fromUserFunction;if(P=N,S.isCellEditable(T)){var G=o.valueService.getValue(S,T);!W||G!==P?T.setDataValue(S,P):R=!0}}R||b.push(P)};if(c)e.columns.forEach(function(b){C(b)});else{var p=this.isLeft?QX(r.columns).reverse():r.columns;C(void 0,p)}},t.prototype.clearCellsInRange=function(e,r,n){for(var o=e,a=!1,s=function(){var c=l.rowPositionUtils.getRowNode(o);if(!c)return"break";n.forEach(function(p){p.isCellEditable(c)&&c.setDataValue(p,null)}),a=l.rowPositionUtils.sameRow(o,r),o=l.cellNavigationService.getRowBelow(o)},l=this;!a&&o;){var u=s();if(u==="break")break}},t.prototype.processValues=function(e,r,n,o,a,s){var l=this.gridOptionsWrapper.getFillOperation(),u=this.dragAxis==="y",c;if(u?c=this.isUp?"up":"down":c=this.isLeft?"left":"right",l){var p={event:e,values:r,initialValues:n,currentIndex:s,currentCellValue:this.valueService.getValue(o,a),direction:c,column:o,rowNode:a},h=l(p);if(h!==!1)return{value:h,fromUserFunction:!0}}var d=!r.some(function(g){var v=parseFloat(g);return isNaN(v)||v.toString()!==g.toString()});if(e.altKey||!d){if(d&&n.length===1){var f=this.isUp||this.isLeft?-1:1;return{value:parseFloat(_.last(r))+1*f,fromUserFunction:!1}}return{value:r[s%r.length],fromUserFunction:!1}}return{value:_.last(_.findLineByLeastSquares(r.map(Number))),fromUserFunction:!1}},t.prototype.clearValues=function(){this.clearMarkedPath(),this.clearCellValues(),this.lastCellMarked=void 0,i.prototype.clearValues.call(this)},t.prototype.clearMarkedPath=function(){this.markedCells.forEach(function(e){var r=e.getComp();r.addOrRemoveCssClass("ag-selection-fill-top",!1),r.addOrRemoveCssClass("ag-selection-fill-right",!1),r.addOrRemoveCssClass("ag-selection-fill-bottom",!1),r.addOrRemoveCssClass("ag-selection-fill-left",!1)}),this.markedCells.length=0,this.isUp=!1,this.isLeft=!1,this.isReduce=!1},t.prototype.clearCellValues=function(){this.cellValues.length=0},t.prototype.markPathFrom=function(e,r){if(this.clearMarkedPath(),this.clearCellValues(),this.dragAxis==="y"){if(this.rowPositionUtils.sameRow(r,e))return;var n=this.rowPositionUtils.before(r,e),o=this.getRangeStartRow(),a=this.getRangeEndRow();n&&(r.rowPinned==o.rowPinned&&r.rowIndex>=o.rowIndex||o.rowPinned!=a.rowPinned&&r.rowPinned==a.rowPinned&&r.rowIndex<=a.rowIndex)?(this.reduceVertical(e,r),this.isReduce=!0):(this.extendVertical(e,r,n),this.isReduce=!1)}else{var s=e.column,l=r.column;if(s===l)return;var u=this.columnModel.getAllDisplayedColumns(),c=u.indexOf(s),p=u.indexOf(l);p<=c&&p>=u.indexOf(this.getCellRange().columns[0])?(this.reduceHorizontal(e,r),this.isReduce=!0):(this.extendHorizontal(e,r,p<c),this.isReduce=!1)}this.lastCellMarked=r},t.prototype.extendVertical=function(e,r,n){var o=this,a=o.navigationService,s=o.rangeService,l=e;do{for(var u=this.getCellRange(),c=u.columns.length,p=0;p<c;p++){var h=u.columns[p],d={rowIndex:l.rowIndex,rowPinned:l.rowPinned},f=wl(wl({},d),{column:h}),g=s.isCellInSpecificRange(f,u),v=this.rowPositionUtils.sameRow(l,e);if(n&&(this.isUp=!0),!v){var m=a.getCellByPosition(f);if(m){this.markedCells.push(m);var C=m.getComp();g||(C.addOrRemoveCssClass("ag-selection-fill-left",p===0),C.addOrRemoveCssClass("ag-selection-fill-right",p===c-1)),C.addOrRemoveCssClass(n?"ag-selection-fill-top":"ag-selection-fill-bottom",this.rowPositionUtils.sameRow(l,r))}}}if(this.rowPositionUtils.sameRow(l,r))break}while(l=n?this.cellNavigationService.getRowAbove(l):this.cellNavigationService.getRowBelow(l))},t.prototype.reduceVertical=function(e,r){var n=e;do{for(var o=this.getCellRange(),a=o.columns.length,s=this.rowPositionUtils.sameRow(n,r),l=0;l<a;l++){var u={rowIndex:n.rowIndex,rowPinned:n.rowPinned},c=wl(wl({},u),{column:o.columns[l]}),p=this.navigationService.getCellByPosition(c);if(p){this.markedCells.push(p);var h=p.getComp();h.addOrRemoveCssClass("ag-selection-fill-bottom",this.rowPositionUtils.sameRow(n,r))}}if(s)break}while(n=this.cellNavigationService.getRowAbove(n))},t.prototype.extendHorizontal=function(e,r,n){var o=this,a=this.columnModel.getAllDisplayedColumns(),s=a.indexOf(n?r.column:e.column),l=a.indexOf(n?this.getCellRange().columns[0]:r.column),u=n?0:1,c=a.slice(s+u,l+u),p=this.getRangeStartRow(),h=this.getRangeEndRow();c.forEach(function(d){var f=p,g=!1;do{g=o.rowPositionUtils.sameRow(f,h);var v=o.navigationService.getCellByPosition({rowIndex:f.rowIndex,rowPinned:f.rowPinned,column:d});if(v){o.markedCells.push(v);var m=v.getComp();m.addOrRemoveCssClass("ag-selection-fill-top",o.rowPositionUtils.sameRow(f,p)),m.addOrRemoveCssClass("ag-selection-fill-bottom",o.rowPositionUtils.sameRow(f,h)),n?(o.isLeft=!0,m.addOrRemoveCssClass("ag-selection-fill-left",d===c[0])):m.addOrRemoveCssClass("ag-selection-fill-right",d===_.last(c))}f=o.cellNavigationService.getRowBelow(f)}while(!g)})},t.prototype.reduceHorizontal=function(e,r){var n=this,o=this.columnModel.getAllDisplayedColumns(),a=o.indexOf(r.column),s=o.indexOf(e.column),l=o.slice(a,s),u=this.getRangeStartRow(),c=this.getRangeEndRow();l.forEach(function(p){var h=u,d=!1;do{d=n.rowPositionUtils.sameRow(h,c);var f=n.navigationService.getCellByPosition({rowIndex:h.rowIndex,rowPinned:h.rowPinned,column:p});if(f){n.markedCells.push(f);var g=f.getComp();g.addOrRemoveCssClass("ag-selection-fill-right",p===l[0])}h=n.cellNavigationService.getRowBelow(h)}while(!d)})},t.prototype.refresh=function(e){var r=this.rangeService.getCellRanges()[0],n=!r.startRow||!r.endRow;if(n){this.destroy();return}i.prototype.refresh.call(this,e)},t.TEMPLATE='<div class="ag-fill-handle"></div>',qX([y("valueService")],t.prototype,"valueService",void 0),t}(iD),JX=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),kd=globalThis&&globalThis.__assign||function(){return kd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},kd.apply(this,arguments)},aD=function(i){JX(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.type=co.RANGE,e.rangeFixed=!1,e}return t.prototype.onDrag=function(e){var r=this.getLastCellHovered();if(!!r){var n=this.rangeService.getCellRanges(),o=_.last(n);if(this.rangeFixed||(this.fixRangeStartEnd(o),this.rangeFixed=!0),this.endPosition={rowIndex:r.rowIndex,rowPinned:r.rowPinned,column:r.column},n.length===2&&n[0].type===zr.DIMENSION&&o.type===zr.VALUE){var a=!this.rowPositionUtils.sameRow(this.endPosition,this.rangeService.getRangeEndRow(o));a&&this.rangeService.updateRangeEnd(n[0],kd(kd({},this.endPosition),{column:n[0].columns[0]}),!0)}this.rangeService.extendLatestRangeToCell(this.endPosition)}},t.prototype.onDragEnd=function(e){var r=_.last(this.rangeService.getCellRanges());this.fixRangeStartEnd(r),this.rangeFixed=!1},t.prototype.fixRangeStartEnd=function(e){var r=this.rangeService.getRangeStartRow(e),n=this.rangeService.getRangeEndRow(e),o=e.columns[0];e.startRow=r,e.endRow=n,e.startColumn=o},t.TEMPLATE='<div class="ag-range-handle"></div>',t}(iD),eq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),tq=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},rq=function(i){eq(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.createSelectionHandle=function(e){return this.createBean(e===co.RANGE?new aD:new oD)},t=tq([Q("selectionHandleFactory")],t),t}(H),sD={moduleName:oe.RangeSelectionModule,beans:[KX,rq],agStackComponents:[{componentName:"AgFillHandle",componentClass:oD},{componentName:"AgRangeHandle",componentClass:aD}],dependantModules:[cr]},nq={moduleName:oe.GridChartsModule,beans:[kX,BX,jX],dependantModules:[sD,cr]},iq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Wy=globalThis&&globalThis.__assign||function(){return Wy=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Wy.apply(this,arguments)},Xv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},lD=function(i){iq(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.loadRowDataVersion=0,e.needRefresh=!1,e}return t.prototype.init=function(e,r){var n=this;this.params=r,this.comp=e;var o=r.pinned!=null;o||(this.setAutoHeightClasses(),this.setupRefreshStrategy(),this.addThemeToDetailGrid(),this.createDetailGrid(),this.loadRowData(),this.addManagedListener(r.node.parent,Oe.EVENT_DATA_CHANGED,function(){n.needRefresh=!0}),this.addManagedListener(this.eventService,x.EVENT_FULL_WIDTH_ROW_FOCUSED,this.onFullWidthRowFocused.bind(this)))},t.prototype.onFullWidthRowFocused=function(e){var r=this.params,n={rowIndex:r.node.rowIndex,rowPinned:r.node.rowPinned},o={rowIndex:e.rowIndex,rowPinned:e.rowPinned},a=this.rowPositionUtils.sameRow(n,o);!a||this.focusService.focusInto(this.comp.getGui(),e.fromBelow)},t.prototype.setAutoHeightClasses=function(){var e=this.gridOptionsWrapper.isDetailRowAutoHeight(),r=e?"ag-details-row-auto-height":"ag-details-row-fixed-height",n=e?"ag-details-grid-auto-height":"ag-details-grid-fixed-height";this.comp.addOrRemoveCssClass(r,!0),this.comp.addOrRemoveDetailGridCssClass(n,!0)},t.prototype.setupRefreshStrategy=function(){if(this.params.suppressRefresh){console.warn("AG Grid: as of v23.2.0, cellRendererParams.suppressRefresh for Detail Cell Renderer is no longer used. Please set cellRendererParams.refreshStrategy = 'nothing' instead."),this.refreshStrategy="nothing";return}var e=this.params.refreshStrategy,r=e=="everything"||e=="nothing"||e=="rows";if(r){this.refreshStrategy=e;return}e!=null&&console.warn("AG Grid: invalid cellRendererParams.refreshStrategy = '"+e+"' supplied, defaulting to refreshStrategy = 'rows'."),this.refreshStrategy="rows"},t.prototype.addThemeToDetailGrid=function(){var e=this.environment.getTheme().theme;e&&this.comp.addOrRemoveDetailGridCssClass(e,!0)},t.prototype.createDetailGrid=function(){if(_.missing(this.params.detailGridOptions)){console.warn("AG Grid: could not find detail grid options for master detail, please set gridOptions.detailCellRendererParams.detailGridOptions");return}var e=this.gridOptionsWrapper.isDetailRowAutoHeight(),r=Wy({},this.params.detailGridOptions);e&&(r.domLayout="autoHeight"),this.comp.setDetailGrid(r)},t.prototype.registerDetailWithMaster=function(e,r){var n=this.params.node.id,o=this.params.api,a={id:n,api:e,columnApi:r},s=this.params.node;o.addDetailGridInfo(n,a),s.detailGridInfo=a,this.addDestroyFunc(function(){s.detailGridInfo===a&&(o.removeDetailGridInfo(n),s.detailGridInfo=null)})},t.prototype.loadRowData=function(){var e=this;this.loadRowDataVersion++;var r=this.loadRowDataVersion,n=this.params.getDetailRowData;if(!n){console.warn("AG Grid: could not find getDetailRowData for master / detail, please set gridOptions.detailCellRendererParams.getDetailRowData");return}var o=function(s){var l=e.loadRowDataVersion===r;l&&e.comp.setRowData(s)},a={node:this.params.node,data:this.params.node.data,successCallback:o,context:this.gridOptionsWrapper.getContext()};n(a)},t.prototype.refresh=function(){var e=!1,r=!0,n=!this.needRefresh||this.refreshStrategy==="nothing";return n?r:(this.needRefresh=!1,this.refreshStrategy==="everything"?e:(this.loadRowData(),r))},Xv([y("environment")],t.prototype,"environment",void 0),Xv([y("rowPositionUtils")],t.prototype,"rowPositionUtils",void 0),Xv([y("focusService")],t.prototype,"focusService",void 0),t}(H),oq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),aq=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},sq=function(i){oq(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.init=function(e){var r=this;this.params=e,this.selectAndSetTemplate();var n={addOrRemoveCssClass:function(o,a){return r.addOrRemoveCssClass(o,a)},addOrRemoveDetailGridCssClass:function(o,a){return r.eDetailGrid.classList.toggle(o,a)},setDetailGrid:function(o){return r.setDetailGrid(o)},setRowData:function(o){return r.setRowData(o)},getGui:function(){return r.eDetailGrid}};this.ctrl=this.createManagedBean(new lD),this.ctrl.init(n,e)},t.prototype.refresh=function(){return this.ctrl&&this.ctrl.refresh()},t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.selectAndSetTemplate=function(){var e=this;if(this.params.pinned){this.setTemplate('<div class="ag-details-row"></div>');return}var r=function(){e.setTemplate(t.TEMPLATE)};if(_.missing(this.params.template))r();else if(typeof this.params.template=="string")this.setTemplate(this.params.template);else if(typeof this.params.template=="function"){var n=this.params.template,o=n(this.params);this.setTemplate(o)}else console.warn("AG Grid: detailCellRendererParams.template should be function or string"),r();this.eDetailGrid==null&&console.warn('AG Grid: reference to eDetailGrid was missing from the details template. Please add ref="eDetailGrid" to the template.')},t.prototype.setDetailGrid=function(e){if(!!this.eDetailGrid){var r=this.context.getBean("agGridReact"),n=r?_.cloneObject(r):void 0,o=this.context.getBean("frameworkComponentWrapper"),a=this.getFrameworkOverrides();new kR(this.eDetailGrid,e,{frameworkOverrides:a,providedBeanInstances:{agGridReact:n,frameworkComponentWrapper:o}}),this.detailApi=e.api,this.ctrl.registerDetailWithMaster(e.api,e.columnApi),this.addDestroyFunc(function(){e.api&&e.api.destroy()})}},t.prototype.setRowData=function(e){this.detailApi&&this.detailApi.setRowData(e)},t.TEMPLATE=`<div class="ag-details-row">
            <div ref="eDetailGrid" class="ag-details-grid"></div>
        </div>`,aq([I("eDetailGrid")],t.prototype,"eDetailGrid",void 0),t}(ee),lq={moduleName:oe.MasterDetailModule,beans:[],userComponents:[{componentName:"agDetailCellRenderer",componentClass:sq}],controllers:[{controllerName:"detailCellRenderer",controllerClass:lD}],dependantModules:[cr]},uD=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Rn=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},uq=function(i){uD(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.hideActiveMenu=function(){this.destroyBean(this.activeMenu)},t.prototype.showMenuAfterMouseEvent=function(e,r,n){var o=this;this.showMenu(e,function(a){var s=a.getGui();o.popupService.positionPopupUnderMouseEvent({type:"columnMenu",column:e,mouseEvent:r,ePopup:s}),n&&a.showTab(n)},"columnMenu",n,void 0,r.target)},t.prototype.showMenuAfterButtonClick=function(e,r,n,o,a){var s=this,l=-1,u="left";this.gridOptionsWrapper.isEnableRtl()&&(l=1,u="right"),this.showMenu(e,function(c){var p=c.getGui();s.popupService.positionPopupUnderComponent({type:n,column:e,eventSource:r,ePopup:p,alignSide:u,nudgeX:9*l,nudgeY:-23,keepWithinBounds:!0}),o&&c.showTab(o)},n,o,a,r)},t.prototype.showMenu=function(e,r,n,o,a,s){var l=this,u=this.createBean(new qv(e,this.lastSelectedTab,a)),c=u.getGui(),p=s||this.ctrlsService.getGridBodyCtrl().getGui(),h=[];h.push(function(v){l.destroyBean(u),e.setMenuVisible(!1,"contextMenu");var m=v instanceof KeyboardEvent;if(m&&s&&_.isVisible(s)){var C=l.focusService.findTabbableParent(s);C&&C.focus()}});var d=this.gridOptionsWrapper.getLocaleTextFunc(),f=this.popupService.addPopup({modal:!0,eChild:c,closeOnEsc:!0,closedCallback:function(v){h.forEach(function(m){return m(v)})},afterGuiAttached:function(v){return u.afterGuiAttached(Object.assign({},{container:n},v))},positionCallback:function(){return r(u)},anchorToElement:p,ariaLabel:d("ariaLabelColumnMenu","Column Menu")});if(f){var g=f.stopAnchoringPromise;g&&g.then(function(v){e.addEventListener(Se.EVENT_LEFT_CHANGED,v),e.addEventListener(Se.EVENT_VISIBLE_CHANGED,v),h.push(function(){e.removeEventListener(Se.EVENT_LEFT_CHANGED,v),e.removeEventListener(Se.EVENT_VISIBLE_CHANGED,v)})})}o||(u.showTabBasedOnPreviousSelection(),r(u)),u.addEventListener(qv.EVENT_TAB_SELECTED,function(v){l.lastSelectedTab=v.key}),e.setMenuVisible(!0,"contextMenu"),this.activeMenu=u,u.addEventListener(H.EVENT_DESTROYED,function(){l.activeMenu===u&&(l.activeMenu=null)})},t.prototype.isMenuEnabled=function(e){return e.getMenuTabs(qv.TABS_DEFAULT).length>0},Rn([y("popupService")],t.prototype,"popupService",void 0),Rn([y("focusService")],t.prototype,"focusService",void 0),Rn([y("ctrlsService")],t.prototype,"ctrlsService",void 0),t=Rn([Q("menuFactory")],t),t}(H),qv=function(i){uD(t,i);function t(e,r,n){var o=i.call(this)||this;return o.tabFactories={},o.includeChecks={},o.column=e,o.initialSelection=r,o.tabFactories[t.TAB_GENERAL]=o.createMainPanel.bind(o),o.tabFactories[t.TAB_FILTER]=o.createFilterPanel.bind(o),o.tabFactories[t.TAB_COLUMNS]=o.createColumnsPanel.bind(o),o.includeChecks[t.TAB_GENERAL]=function(){return!0},o.includeChecks[t.TAB_FILTER]=function(){return e.isFilterAllowed()},o.includeChecks[t.TAB_COLUMNS]=function(){return!0},o.restrictTo=n,o}return t.prototype.init=function(){var e=this,r=this.getTabsToCreate().map(function(n){return e.createTab(n)});this.tabbedLayout=new rw({items:r,cssClass:"ag-menu",onActiveItemClicked:this.onHidePopup.bind(this),onItemClicked:this.onTabItemClicked.bind(this)}),this.createBean(this.tabbedLayout),this.mainMenuList&&this.mainMenuList.setParentComponent(this.tabbedLayout)},t.prototype.getTabsToCreate=function(){var e=this;return this.restrictTo?this.restrictTo:this.column.getMenuTabs(t.TABS_DEFAULT).filter(function(r){return e.isValidMenuTabItem(r)}).filter(function(r){return e.isNotSuppressed(r)}).filter(function(r){return e.isModuleLoaded(r)})},t.prototype.isModuleLoaded=function(e){return e===t.TAB_COLUMNS?ge.isRegistered(oe.ColumnToolPanelModule):!0},t.prototype.isValidMenuTabItem=function(e){var r=!0,n=t.TABS_DEFAULT;return this.restrictTo!=null&&(r=this.restrictTo.indexOf(e)>-1,n=this.restrictTo),r=r&&t.TABS_DEFAULT.indexOf(e)>-1,r||console.warn("AG Grid: Trying to render an invalid menu item '"+e+"'. Check that your 'menuTabs' contains one of ["+n+"]"),r},t.prototype.isNotSuppressed=function(e){return this.includeChecks[e]()},t.prototype.createTab=function(e){return this.tabFactories[e]()},t.prototype.showTabBasedOnPreviousSelection=function(){this.showTab(this.initialSelection)},t.prototype.showTab=function(e){this.tabItemColumns&&e===t.TAB_COLUMNS?this.tabbedLayout.showItem(this.tabItemColumns):this.tabItemFilter&&e===t.TAB_FILTER?this.tabbedLayout.showItem(this.tabItemFilter):this.tabItemGeneral&&e===t.TAB_GENERAL?this.tabbedLayout.showItem(this.tabItemGeneral):this.tabbedLayout.showFirstItem()},t.prototype.onTabItemClicked=function(e){var r=null;switch(e.item){case this.tabItemColumns:r=t.TAB_COLUMNS;break;case this.tabItemFilter:r=t.TAB_FILTER;break;case this.tabItemGeneral:r=t.TAB_GENERAL;break}r&&this.activateTab(r)},t.prototype.activateTab=function(e){var r={type:t.EVENT_TAB_SELECTED,key:e};this.dispatchEvent(r)},t.prototype.getMenuItems=function(){var e=this.getDefaultMenuOptions(),r,n=this.gridOptionsWrapper.getMainMenuItemsFunc();return n?r=n({column:this.column,defaultItems:e}):r=e,_.removeRepeatsFromArray(r,t.MENU_ITEM_SEPARATOR),r},t.prototype.getDefaultMenuOptions=function(){var e=[],r=!this.column.getColDef().lockPinned,n=this.columnModel.getRowGroupColumns().length,o=n>0,a=this.columnModel.getRowGroupColumns().indexOf(this.column)>=0,s=this.column.isAllowValue(),l=this.column.isAllowRowGroup(),u=this.column.isPrimary(),c=this.columnModel.isPivotMode(),p=this.rowModel.getType()===L.ROW_MODEL_TYPE_CLIENT_SIDE,h=this.gridOptionsWrapper.isTreeData(),d=u&&o&&s||!u;r&&e.push("pinSubMenu"),d&&e.push("valueAggSubMenu"),(r||d)&&e.push(t.MENU_ITEM_SEPARATOR),e.push("autoSizeThis"),e.push("autoSizeAll"),e.push(t.MENU_ITEM_SEPARATOR),l&&this.column.isPrimary()&&(a?e.push("rowUnGroup"):e.push("rowGroup")),e.push(t.MENU_ITEM_SEPARATOR),e.push("resetColumns");var f=p&&(h||n>(c?1:0));return f&&(e.push("expandAll"),e.push("contractAll")),e},t.prototype.createMainPanel=function(){this.mainMenuList=this.createManagedBean(new ep);var e=this.getMenuItems(),r=this.menuItemMapper.mapWithStockItems(e,this.column);return this.mainMenuList.addMenuItems(r),this.mainMenuList.addEventListener(Dn.EVENT_MENU_ITEM_SELECTED,this.onHidePopup.bind(this)),this.tabItemGeneral={title:_.createIconNoSpan("menu",this.gridOptionsWrapper,this.column),titleLabel:t.TAB_GENERAL.replace("MenuTab",""),bodyPromise:ct.resolve(this.mainMenuList.getGui()),name:t.TAB_GENERAL},this.tabItemGeneral},t.prototype.onHidePopup=function(e){var r;e&&e.event&&e.event instanceof KeyboardEvent&&(r=e.event),this.hidePopupFunc(r&&{keyboardEvent:r});var n=this.focusService.getFocusedCell(),o=this.gridOptionsWrapper.getDocument();if(o.activeElement===o.body&&n){var a=n.rowIndex,s=n.rowPinned,l=n.column;this.focusService.setFocusedCell(a,l,s,!0)}},t.prototype.createFilterPanel=function(){var e,r=this.filterManager.getOrCreateFilterWrapper(this.column,"COLUMN_MENU");if(!r)throw new Error("AG Grid - Unable to instantiate filter");var n=function(o){var a;!(!((a=r)===null||a===void 0)&&a.filterPromise)||r.filterPromise.then(function(s){s&&s.afterGuiAttached&&s.afterGuiAttached(o)})};return this.tabItemFilter={title:_.createIconNoSpan("filter",this.gridOptionsWrapper,this.column),titleLabel:t.TAB_FILTER.replace("MenuTab",""),bodyPromise:(e=r)===null||e===void 0?void 0:e.guiPromise,afterAttachedCallback:n,name:t.TAB_FILTER},this.tabItemFilter},t.prototype.createColumnsPanel=function(){var e=document.createElement("div");e.classList.add("ag-menu-column-select-wrapper"),this.columnSelectPanel=this.createManagedBean(new Yw);var r=this.column.getColDef().columnsMenuParams;r||(r={}),this.columnSelectPanel.init(!1,{suppressColumnMove:!1,suppressValues:!1,suppressPivots:!1,suppressRowGroups:!1,suppressPivotMode:!1,contractColumnSelection:!!r.contractColumnSelection,suppressColumnExpandAll:!!r.suppressColumnExpandAll,suppressColumnFilter:!!r.suppressColumnFilter,suppressColumnSelectAll:!!r.suppressColumnSelectAll,suppressSyncLayoutWithGrid:!!r.suppressSyncLayoutWithGrid,api:this.gridApi,columnApi:this.columnApi,context:this.gridOptionsWrapper.getContext()},"columnMenu");var n=this.columnSelectPanel.getGui();return n.classList.add("ag-menu-column-select"),e.appendChild(n),this.tabItemColumns={title:_.createIconNoSpan("columns",this.gridOptionsWrapper,this.column),titleLabel:t.TAB_COLUMNS.replace("MenuTab",""),bodyPromise:ct.resolve(e),name:t.TAB_COLUMNS},this.tabItemColumns},t.prototype.afterGuiAttached=function(e){var r=e.container,n=e.hidePopup;this.tabbedLayout.setAfterAttachedParams({container:r,hidePopup:n}),n&&(this.hidePopupFunc=n,this.addDestroyFunc(n))},t.prototype.getGui=function(){return this.tabbedLayout.getGui()},t.EVENT_TAB_SELECTED="tabSelected",t.TAB_FILTER="filterMenuTab",t.TAB_GENERAL="generalMenuTab",t.TAB_COLUMNS="columnsMenuTab",t.TABS_DEFAULT=[t.TAB_GENERAL,t.TAB_FILTER,t.TAB_COLUMNS],t.MENU_ITEM_SEPARATOR="separator",Rn([y("columnModel")],t.prototype,"columnModel",void 0),Rn([y("filterManager")],t.prototype,"filterManager",void 0),Rn([y("gridApi")],t.prototype,"gridApi",void 0),Rn([y("columnApi")],t.prototype,"columnApi",void 0),Rn([y("menuItemMapper")],t.prototype,"menuItemMapper",void 0),Rn([y("rowModel")],t.prototype,"rowModel",void 0),Rn([y("focusService")],t.prototype,"focusService",void 0),Rn([B],t.prototype,"init",null),t}(H),cD=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),qi=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},cq="ag-menu",m0="ag-context-menu-open",pq=function(i){cD(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.hideActiveMenu=function(){this.destroyBean(this.activeMenu)},t.prototype.getMenuItems=function(e,r,n){var o=[];if(_.exists(e)&&ge.isRegistered(oe.ClipboardModule)&&r&&o.push("copy","copyWithHeaders","copyWithGroupHeaders","paste","separator"),this.gridOptionsWrapper.isEnableCharts()&&ge.isRegistered(oe.RangeSelectionModule)&&ge.isRegistered(oe.GridChartsModule)&&(this.columnModel.isPivotMode()&&o.push("pivotChart"),this.rangeService&&!this.rangeService.isEmpty()&&o.push("chartRange")),_.exists(e)){var a=!ge.isRegistered(oe.CsvExportModule),s=!ge.isRegistered(oe.ExcelExportModule),l=this.gridOptionsWrapper.isSuppressExcelExport()||s,u=this.gridOptionsWrapper.isSuppressCsvExport()||a,c=_.isIOSUserAgent(),p=!c&&(!l||!u);p&&o.push("export")}if(this.gridOptionsWrapper.getContextMenuItemsFunc()){var h=this.gridOptionsWrapper.getContextMenuItemsFunc(),d={node:e,column:r,value:n,defaultItems:o.length?o:void 0};return h?h(d):void 0}return o},t.prototype.onContextMenu=function(e,r,n,o,a,s){if(!(!this.gridOptionsWrapper.isAllowContextMenuWithControlKey()&&e&&(e.ctrlKey||e.metaKey))&&(e&&this.blockMiddleClickScrollsIfNeeded(e),!this.gridOptionsWrapper.isSuppressContextMenu())){var l=e||r.touches[0];if(this.showMenu(n,o,a,l,s)){var u=e||r;u.preventDefault()}}},t.prototype.blockMiddleClickScrollsIfNeeded=function(e){var r=this.gridOptionsWrapper,n=e.which;r.isSuppressMiddleClickScrolls()&&n===2&&e.preventDefault()},t.prototype.showMenu=function(e,r,n,o,a){var s=this,l=this.getMenuItems(e,r,n),u=this.ctrlsService.getGridBodyCtrl().getGui();if(l===void 0||_.missingOrEmpty(l))return!1;var c=new hq(l);this.createBean(c);var p=c.getGui(),h={column:r,rowNode:e,type:"contextMenu",mouseEvent:o,ePopup:p,nudgeY:1},d=this.gridOptionsWrapper.getLocaleTextFunc(),f=this.popupService.addPopup({modal:!0,eChild:p,closeOnEsc:!0,closedCallback:function(){u.classList.remove(m0),s.destroyBean(c)},click:o,positionCallback:function(){s.popupService.positionPopupUnderMouseEvent(Object.assign({},{nudgeX:s.gridOptionsWrapper.isEnableRtl()?(p.offsetWidth+1)*-1:1},h))},anchorToElement:a,ariaLabel:d("ariaLabelContextMenu","Context Menu")});return f&&(u.classList.add(m0),c.afterGuiAttached({container:"contextMenu",hidePopup:f.hideFunc})),this.activeMenu&&this.hideActiveMenu(),this.activeMenu=c,c.addEventListener(H.EVENT_DESTROYED,function(){s.activeMenu===c&&(s.activeMenu=null)}),f&&c.addEventListener(Dn.EVENT_MENU_ITEM_SELECTED,f.hideFunc),!0},qi([y("popupService")],t.prototype,"popupService",void 0),qi([De("rangeService")],t.prototype,"rangeService",void 0),qi([y("ctrlsService")],t.prototype,"ctrlsService",void 0),qi([y("columnModel")],t.prototype,"columnModel",void 0),t=qi([Q("contextMenuFactory")],t),t}(H),hq=function(i){cD(t,i);function t(e){var r=i.call(this,'<div class="'+cq+'" role="presentation"></div>')||this;return r.menuList=null,r.focusedCell=null,r.menuItems=e,r}return t.prototype.addMenuItems=function(){var e=this,r=this.createBean(new ep),n=this.menuItemMapper.mapWithStockItems(this.menuItems,null);r.addMenuItems(n),this.appendChild(r),this.menuList=r,r.addEventListener(Dn.EVENT_MENU_ITEM_SELECTED,function(o){return e.dispatchEvent(o)})},t.prototype.afterGuiAttached=function(e){e.hidePopup&&this.addDestroyFunc(e.hidePopup),this.focusedCell=this.focusService.getFocusedCell(),this.menuList&&this.focusService.focusInto(this.menuList.getGui())},t.prototype.restoreFocusedCell=function(){var e=this.focusService.getFocusedCell();if(e&&this.focusedCell&&this.cellPositionUtils.equals(e,this.focusedCell)){var r=this.focusedCell,n=r.rowIndex,o=r.rowPinned,a=r.column,s=this.gridOptionsWrapper.getDocument();s.activeElement===s.body&&this.focusService.setFocusedCell(n,a,o,!0)}},t.prototype.destroy=function(){this.restoreFocusedCell(),i.prototype.destroy.call(this)},qi([y("menuItemMapper")],t.prototype,"menuItemMapper",void 0),qi([y("focusService")],t.prototype,"focusService",void 0),qi([y("cellPositionUtils")],t.prototype,"cellPositionUtils",void 0),qi([B],t.prototype,"addMenuItems",null),t}(ee),dq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),rl=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},fq=function(i){dq(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.mapWithStockItems=function(e,r){var n=this;if(!e)return[];var o=[];return e.forEach(function(a){var s;if(typeof a=="string"?s=n.getStockMenuItem(a,r):s=a,!!s){var l=s,u=l.subMenu;u&&u instanceof Array&&(l.subMenu=n.mapWithStockItems(l.subMenu,r)),s!=null&&o.push(s)}}),o},t.prototype.getStockMenuItem=function(e,r){var n=this,o=this.gridOptionsWrapper.getLocaleTextFunc(),a=this.gridOptionsWrapper.isSkipHeaderOnAutoSize();switch(e){case"pinSubMenu":return{name:o("pinColumn","Pin Column"),icon:_.createIconNoSpan("menuPin",this.gridOptionsWrapper,null),subMenu:["pinLeft","pinRight","clearPinned"]};case"pinLeft":return{name:o("pinLeft","Pin Left"),action:function(){return n.columnModel.setColumnPinned(r,L.PINNED_LEFT,"contextMenu")},checked:!!r&&r.isPinnedLeft()};case"pinRight":return{name:o("pinRight","Pin Right"),action:function(){return n.columnModel.setColumnPinned(r,L.PINNED_RIGHT,"contextMenu")},checked:!!r&&r.isPinnedRight()};case"clearPinned":return{name:o("noPin","No Pin"),action:function(){return n.columnModel.setColumnPinned(r,null,"contextMenu")},checked:!!r&&!r.isPinned()};case"valueAggSubMenu":return ge.assertRegistered(oe.RowGroupingModule,"Aggregation from Menu")?{name:o("valueAggregation","Value Aggregation"),icon:_.createIconNoSpan("menuValue",this.gridOptionsWrapper,null),subMenu:this.createAggregationSubMenu(r)}:null;case"autoSizeThis":return{name:o("autosizeThiscolumn","Autosize This Column"),action:function(){return n.columnModel.autoSizeColumn(r,a,"contextMenu")}};case"autoSizeAll":return{name:o("autosizeAllColumns","Autosize All Columns"),action:function(){return n.columnModel.autoSizeAllColumns(a,"contextMenu")}};case"rowGroup":return{name:o("groupBy","Group by")+" "+_.escapeString(this.columnModel.getDisplayNameForColumn(r,"header")),action:function(){return n.columnModel.addRowGroupColumn(r,"contextMenu")},icon:_.createIconNoSpan("menuAddRowGroup",this.gridOptionsWrapper,null)};case"rowUnGroup":return{name:o("ungroupBy","Un-Group by")+" "+_.escapeString(this.columnModel.getDisplayNameForColumn(r,"header")),action:function(){return n.columnModel.removeRowGroupColumn(r,"contextMenu")},icon:_.createIconNoSpan("menuRemoveRowGroup",this.gridOptionsWrapper,null)};case"resetColumns":return{name:o("resetColumns","Reset Columns"),action:function(){return n.columnModel.resetColumnState("contextMenu")}};case"expandAll":return{name:o("expandAll","Expand All"),action:function(){return n.gridApi.expandAll()}};case"contractAll":return{name:o("collapseAll","Collapse All"),action:function(){return n.gridApi.collapseAll()}};case"copy":return ge.assertRegistered(oe.ClipboardModule,"Copy from Menu")?{name:o("copy","Copy"),shortcut:o("ctrlC","Ctrl+C"),icon:_.createIconNoSpan("clipboardCopy",this.gridOptionsWrapper,null),action:function(){return n.clipboardService.copyToClipboard()}}:null;case"copyWithHeaders":return ge.assertRegistered(oe.ClipboardModule,"Copy with Headers from Menu")?{name:o("copyWithHeaders","Copy with Headers"),icon:_.createIconNoSpan("clipboardCopy",this.gridOptionsWrapper,null),action:function(){return n.clipboardService.copyToClipboard({includeHeaders:!0})}}:null;case"copyWithGroupHeaders":return ge.assertRegistered(oe.ClipboardModule,"Copy with Group Headers from Menu")?{name:o("copyWithGroupHeaders","Copy with Group Headers"),icon:_.createIconNoSpan("clipboardCopy",this.gridOptionsWrapper,null),action:function(){return n.clipboardService.copyToClipboard({includeHeaders:!0,includeGroupHeaders:!0})}}:null;case"paste":return ge.assertRegistered(oe.ClipboardModule,"Paste from Clipboard")?{name:o("paste","Paste"),shortcut:o("ctrlV","Ctrl+V"),disabled:!0,icon:_.createIconNoSpan("clipboardPaste",this.gridOptionsWrapper,null),action:function(){return n.clipboardService.pasteFromClipboard()}}:null;case"export":var s=[],l=ge.isRegistered(oe.CsvExportModule),u=ge.isRegistered(oe.ExcelExportModule);return!this.gridOptionsWrapper.isSuppressCsvExport()&&l&&s.push("csvExport"),!this.gridOptionsWrapper.isSuppressExcelExport()&&u&&s.push("excelExport"),{name:o("export","Export"),subMenu:s,icon:_.createIconNoSpan("save",this.gridOptionsWrapper,null)};case"csvExport":return{name:o("csvExport","CSV Export"),icon:_.createIconNoSpan("csvExport",this.gridOptionsWrapper,null),action:function(){return n.gridApi.exportDataAsCsv({})}};case"excelExport":return{name:o("excelExport","Excel Export"),icon:_.createIconNoSpan("excelExport",this.gridOptionsWrapper,null),action:function(){return n.gridApi.exportDataAsExcel()}};case"separator":return"separator";default:var c=this.getChartItems(e);return c||(console.warn("AG Grid: unknown menu item type "+e),null)}},t.prototype.getChartItems=function(e){var r=this,n=this.gridOptionsWrapper.getLocaleTextFunc(),o=function(s,l,u){return{name:n(s,l),action:function(){return r.chartService.createPivotChart({chartType:u})}}},a=function(s,l,u){return{name:n(s,l),action:function(){return r.chartService.createChartFromCurrentRange(u)}}};switch(e){case"pivotChart":return{name:n("pivotChart","Pivot Chart"),subMenu:["pivotColumnChart","pivotBarChart","pivotPieChart","pivotLineChart","pivotXYChart","pivotAreaChart"],icon:_.createIconNoSpan("chart",this.gridOptionsWrapper,null)};case"chartRange":return{name:n("chartRange","Chart Range"),subMenu:["rangeColumnChart","rangeBarChart","rangePieChart","rangeLineChart","rangeXYChart","rangeAreaChart","rangeHistogramChart","rangeCombinationChart"],icon:_.createIconNoSpan("chart",this.gridOptionsWrapper,null)};case"pivotColumnChart":return{name:n("columnChart","Column"),subMenu:["pivotGroupedColumn","pivotStackedColumn","pivotNormalizedColumn"]};case"pivotGroupedColumn":return o("groupedColumn","Grouped&lrm;","groupedColumn");case"pivotStackedColumn":return o("stackedColumn","Stacked&lrm;","stackedColumn");case"pivotNormalizedColumn":return o("normalizedColumn","100% Stacked&lrm;","normalizedColumn");case"rangeColumnChart":return{name:n("columnChart","Column"),subMenu:["rangeGroupedColumn","rangeStackedColumn","rangeNormalizedColumn"]};case"rangeGroupedColumn":return a("groupedColumn","Grouped&lrm;","groupedColumn");case"rangeStackedColumn":return a("stackedColumn","Stacked&lrm;","stackedColumn");case"rangeNormalizedColumn":return a("normalizedColumn","100% Stacked&lrm;","normalizedColumn");case"pivotBarChart":return{name:n("barChart","Bar"),subMenu:["pivotGroupedBar","pivotStackedBar","pivotNormalizedBar"]};case"pivotGroupedBar":return o("groupedBar","Grouped&lrm;","groupedBar");case"pivotStackedBar":return o("stackedBar","Stacked&lrm;","stackedBar");case"pivotNormalizedBar":return o("normalizedBar","100% Stacked&lrm;","normalizedBar");case"rangeBarChart":return{name:n("barChart","Bar"),subMenu:["rangeGroupedBar","rangeStackedBar","rangeNormalizedBar"]};case"rangeGroupedBar":return a("groupedBar","Grouped&lrm;","groupedBar");case"rangeStackedBar":return a("stackedBar","Stacked&lrm;","stackedBar");case"rangeNormalizedBar":return a("normalizedBar","100% Stacked&lrm;","normalizedBar");case"pivotPieChart":return{name:n("pieChart","Pie"),subMenu:["pivotPie","pivotDoughnut"]};case"pivotPie":return o("pie","Pie&lrm;","pie");case"pivotDoughnut":return o("doughnut","Doughnut&lrm;","doughnut");case"rangePieChart":return{name:n("pieChart","Pie"),subMenu:["rangePie","rangeDoughnut"]};case"rangePie":return a("pie","Pie&lrm;","pie");case"rangeDoughnut":return a("doughnut","Doughnut&lrm;","doughnut");case"pivotLineChart":return o("line","Line&lrm;","line");case"rangeLineChart":return a("line","Line&lrm;","line");case"pivotXYChart":return{name:n("xyChart","X Y (Scatter)"),subMenu:["pivotScatter","pivotBubble"]};case"pivotScatter":return o("scatter","Scatter&lrm;","scatter");case"pivotBubble":return o("bubble","Bubble&lrm;","bubble");case"rangeXYChart":return{name:n("xyChart","X Y (Scatter)"),subMenu:["rangeScatter","rangeBubble"]};case"rangeScatter":return a("scatter","Scatter&lrm;","scatter");case"rangeBubble":return a("bubble","Bubble&lrm;","bubble");case"pivotAreaChart":return{name:n("areaChart","Area"),subMenu:["pivotArea","pivotStackedArea","pivotNormalizedArea"]};case"pivotArea":return o("area","Area&lrm;","area");case"pivotStackedArea":return o("stackedArea","Stacked&lrm;","stackedArea");case"pivotNormalizedArea":return o("normalizedArea","100% Stacked&lrm;","normalizedArea");case"rangeAreaChart":return{name:n("areaChart","Area"),subMenu:["rangeArea","rangeStackedArea","rangeNormalizedArea"]};case"rangeArea":return a("area","Area&lrm;","area");case"rangeStackedArea":return a("stackedArea","Stacked&lrm;","stackedArea");case"rangeNormalizedArea":return a("normalizedArea","100% Stacked&lrm;","normalizedArea");case"rangeHistogramChart":return a("histogramChart","Histogram&lrm;","histogram");case"rangeColumnLineCombo":return a("columnLineCombo","Column & Line&lrm;","columnLineCombo");case"rangeAreaColumnCombo":return a("AreaColumnCombo","Area & Column&lrm;","areaColumnCombo");case"rangeCombinationChart":return{name:n("combinationChart","Combination"),subMenu:["rangeColumnLineCombo","rangeAreaColumnCombo"]};default:return null}},t.prototype.createAggregationSubMenu=function(e){var r=this,n=this.gridOptionsWrapper.getLocaleTextFunc(),o=e.isValueActive(),a=this.aggFuncService.getFuncNames(e),s;if(e.isPrimary())s=e;else{var l=e.getColDef().pivotValueColumn;s=_.exists(l)?l:void 0}var u=[];return a.forEach(function(c){u.push({name:n(c,c),action:function(){r.columnModel.setColumnAggFunc(s,c,"contextMenu"),r.columnModel.addValueColumn(s,"contextMenu")},checked:o&&s.getAggFunc()===c})}),u},rl([y("columnModel")],t.prototype,"columnModel",void 0),rl([y("gridApi")],t.prototype,"gridApi",void 0),rl([De("clipboardService")],t.prototype,"clipboardService",void 0),rl([De("aggFuncService")],t.prototype,"aggFuncService",void 0),rl([De("chartService")],t.prototype,"chartService",void 0),t=rl([Q("menuItemMapper")],t),t}(H),gq={moduleName:oe.MenuModule,beans:[uq,pq,fq],dependantModules:[cr]},vq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),bl=globalThis&&globalThis.__assign||function(){return bl=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},bl.apply(this,arguments)},Zv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Vy=function(i){vq(t,i);function t(){var e=i.call(this,'<div class="ag-multi-filter ag-menu-list-compact"></div>')||this;return e.filterDefs=[],e.filters=[],e.guiDestroyFuncs=[],e.activeFilterIndices=[],e.lastActivatedMenuItem=null,e.afterFiltersReadyFuncs=[],e}return t.prototype.postConstruct=function(){var e=this;this.initialiseTabGuard({onFocusIn:function(r){return e.onFocusIn(r)}})},t.getFilterDefs=function(e){var r=e.filters;return r&&r.length>0?r:[{filter:"agTextColumnFilter"},{filter:"agSetColumnFilter"}]},t.prototype.init=function(e){var r=this;this.params=e,this.filterDefs=t.getFilterDefs(e);var n=e.column,o=e.filterChangedCallback;this.column=n,this.filterChangedCallback=o;var a=[];return this.filterDefs.forEach(function(s,l){var u=r.createFilter(s,l);u!=null&&a.push(u)}),ct.all(a).then(function(s){r.filters=s,r.refreshGui("columnMenu"),r.afterFiltersReadyFuncs.forEach(function(l){return l()}),r.afterFiltersReadyFuncs.length=0})},t.prototype.refreshGui=function(e){var r=this;e!==this.lastOpenedInContainer&&(this.removeAllChildrenExceptTabGuards(),this.destroyChildren(),this.filters.forEach(function(n,o){o>0&&r.appendChild(_.loadTemplate('<div class="ag-filter-separator"></div>'));var a=r.filterDefs[o],s=r.getFilterTitle(n,a),l;if(a.display==="subMenu"&&e!=="toolPanel"){var u=r.insertFilterMenu(n,s);l=u.getGui()}else if(a.display==="subMenu"||a.display==="accordion"){var c=r.insertFilterGroup(n,s);l=c.getGui()}else l=n.getGui();r.appendChild(l)}),this.lastOpenedInContainer=e)},t.prototype.getFilterTitle=function(e,r){if(r.title!=null)return r.title;var n=e;return typeof n.getFilterTitle=="function"?n.getFilterTitle():"Filter"},t.prototype.destroyChildren=function(){this.guiDestroyFuncs.forEach(function(e){return e()}),this.guiDestroyFuncs.length=0},t.prototype.insertFilterMenu=function(e,r){var n=this,o=this.createBean(new Dn({name:r,subMenu:e,cssClasses:["ag-multi-filter-menu-item"],isCompact:!0,isAnotherSubMenuOpen:function(){return!1}}));return o.setParentComponent(this),this.guiDestroyFuncs.push(function(){return n.destroyBean(o)}),this.addManagedListener(o,Dn.EVENT_MENU_ITEM_ACTIVATED,function(a){n.lastActivatedMenuItem&&n.lastActivatedMenuItem!==a.menuItem&&n.lastActivatedMenuItem.deactivate(),n.lastActivatedMenuItem=a.menuItem}),o.addGuiEventListener("focusin",function(){return o.activate()}),o.addGuiEventListener("focusout",function(){o.isSubMenuOpen()||o.deactivate()}),o},t.prototype.insertFilterGroup=function(e,r){var n=this,o=this.createBean(new Qn({title:r,cssIdentifier:"multi-filter"}));if(this.guiDestroyFuncs.push(function(){return n.destroyBean(o)}),o.addItem(e.getGui()),o.toggleGroupExpand(!1),e.afterGuiAttached){var a={container:this.lastOpenedInContainer,suppressFocus:!0};o.addManagedListener(o,Qn.EVENT_EXPANDED,function(){return e.afterGuiAttached(a)})}return o},t.prototype.isFilterActive=function(){return this.filters.some(function(e){return e.isFilterActive()})},t.prototype.getLastActiveFilterIndex=function(){return this.activeFilterIndices.length>0?this.activeFilterIndices[this.activeFilterIndices.length-1]:null},t.prototype.doesFilterPass=function(e,r){var n=!0;return this.filters.forEach(function(o){!n||o===r||!o.isFilterActive()||(n=o.doesFilterPass(e))}),n},t.prototype.getFilterType=function(){return"multi"},t.prototype.getModelFromUi=function(){var e={filterType:this.getFilterType(),filterModels:this.filters.map(function(r){var n=r;return typeof n.getModelFromUi=="function"?n.getModelFromUi():null})};return e},t.prototype.getModel=function(){if(!this.isFilterActive())return null;var e={filterType:this.getFilterType(),filterModels:this.filters.map(function(r){return r.isFilterActive()?r.getModel():null})};return e},t.prototype.setModel=function(e){var r=this,n=function(a,s){return new ct(function(l){var u=a.setModel(s);u?u.then(function(){return l()}):l()})},o=[];return e==null?o=this.filters.map(function(a,s){var l=n(a,null);return r.updateActiveList(s),l}):this.filters.forEach(function(a,s){var l=e.filterModels.length>s?e.filterModels[s]:null,u=n(a,l);o.push(u),r.updateActiveList(s)}),ct.all(o).then(function(){})},t.prototype.applyModel=function(){var e=!1;return this.filters.forEach(function(r){r instanceof us&&(e=r.applyModel()||e)}),e},t.prototype.getChildFilterInstance=function(e){return this.filters[e]},t.prototype.afterGuiAttached=function(e){e&&this.refreshGui(e.container);var r=this.params.filters,n=r&&r.some(function(s){return s.display&&s.display!=="inline"});this.executeFunctionIfExists("afterGuiAttached",bl(bl({},e||{}),{suppressFocus:n}));var o=this.gridOptionsWrapper.getDocument(),a=o.activeElement;n&&(a===o.body||this.getGui().contains(a))&&this.forceFocusOutOfContainer(!0)},t.prototype.onAnyFilterChanged=function(){this.executeFunctionIfExists("onAnyFilterChanged")},t.prototype.onNewRowsLoaded=function(){this.executeFunctionIfExists("onNewRowsLoaded")},t.prototype.destroy=function(){var e=this;this.filters.forEach(function(r){r.setModel(null),e.destroyBean(r)}),this.filters.length=0,this.destroyChildren(),i.prototype.destroy.call(this)},t.prototype.executeFunctionIfExists=function(e){for(var r=[],n=1;n<arguments.length;n++)r[n-1]=arguments[n];_.forEachReverse(this.filters,function(o){var a=o[e];typeof a=="function"&&a.apply(o,r)})},t.prototype.createFilter=function(e,r){var n=this,o=this.params,a=o.filterModifiedCallback,s=o.doesRowPassOtherFilter,l,u=bl(bl({},this.filterManager.createFilterParams(this.column,this.column.getColDef())),{filterModifiedCallback:a,filterChangedCallback:function(h){n.executeWhenAllFiltersReady(function(){return n.filterChanged(r,h)})},doesRowPassOtherFilter:function(h){return s(h)&&n.doesFilterPass({node:h,data:h.data},l)}}),c=this.userComponentFactory.getFilterDetails(e,u,"agTextColumnFilter");if(!c)return null;var p=c.newAgStackInstance();return p&&p.then(function(h){return l=h}),p},t.prototype.executeWhenAllFiltersReady=function(e){this.filters&&this.filters.length>0?e():this.afterFiltersReadyFuncs.push(e)},t.prototype.updateActiveList=function(e){var r=this.filters[e];_.removeFromArray(this.activeFilterIndices,e),r.isFilterActive()&&this.activeFilterIndices.push(e)},t.prototype.filterChanged=function(e,r){this.updateActiveList(e),this.filterChangedCallback(r);var n=this.filters[e];this.filters.forEach(function(o){o!==n&&typeof o.onAnyFilterChanged=="function"&&o.onAnyFilterChanged()})},t.prototype.onFocusIn=function(e){return this.lastActivatedMenuItem!=null&&!this.lastActivatedMenuItem.getGui().contains(e.target)&&(this.lastActivatedMenuItem.deactivate(),this.lastActivatedMenuItem=null),!0},Zv([y("filterManager")],t.prototype,"filterManager",void 0),Zv([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),Zv([B],t.prototype,"postConstruct",null),t}(bs),mq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Wd=globalThis&&globalThis.__assign||function(){return Wd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Wd.apply(this,arguments)},yq=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Cq=function(i){mq(t,i);function t(){var e=i.call(this,'<div class="ag-multi-floating-filter ag-floating-filter-input"></div>')||this;return e.floatingFilters=[],e}return t.prototype.init=function(e){var r=this;this.params=e;var n=e.filterParams,o=[];return Vy.getFilterDefs(n).forEach(function(a,s){var l=Wd(Wd({},e),{parentFilterInstance:function(c){r.parentMultiFilterInstance(function(p){var h=p.getChildFilterInstance(s);h!=null&&c(h)})}}),u=r.createFloatingFilter(a,l);u!=null&&o.push(u)}),ct.all(o).then(function(a){a.forEach(function(s,l){r.floatingFilters.push(s);var u=s.getGui();r.appendChild(u),l>0&&_.setDisplayed(u,!1)})})},t.prototype.onParentModelChanged=function(e,r){var n=this;r&&r.afterFloatingFilter||this.parentMultiFilterInstance(function(o){if(e==null)n.floatingFilters.forEach(function(s,l){s.onParentModelChanged(null,r),_.setDisplayed(s.getGui(),l===0)});else{var a=o.getLastActiveFilterIndex();n.floatingFilters.forEach(function(s,l){var u=e.filterModels.length>l?e.filterModels[l]:null;s.onParentModelChanged(u,r);var c=a==null?l===0:l===a;_.setDisplayed(s.getGui(),c)})}})},t.prototype.destroy=function(){this.destroyBeans(this.floatingFilters),this.floatingFilters.length=0,i.prototype.destroy.call(this)},t.prototype.createFloatingFilter=function(e,r){var n=this.userComponentFactory.getDefaultFloatingFilterType(e)||"agTextColumnFloatingFilter",o=this.userComponentFactory.getFloatingFilterCompDetails(e,r,n);return o?o.newAgStackInstance():null},t.prototype.parentMultiFilterInstance=function(e){this.params.parentFilterInstance(function(r){if(!(r instanceof Vy))throw new Error("AG Grid - MultiFloatingFilterComp expects MultiFilter as it's parent");e(r)})},yq([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),t}(ee),wq={moduleName:oe.MultiFilterModule,beans:[],userComponents:[{componentName:"agMultiColumnFilter",componentClass:Vy},{componentName:"agMultiColumnFloatingFilter",componentClass:Cq}],dependantModules:[cr]},bq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),_q=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Sq=function(i){bq(t,i);function t(e){var r=i.call(this,'<div class="ag-rich-select-row"></div>')||this;return r.params=e,r}return t.prototype.setState=function(e,r,n){var o=this.populateWithRenderer(e,r);o||this.populateWithoutRenderer(e,r),this.addOrRemoveCssClass("ag-rich-select-row-selected",n)},t.prototype.populateWithoutRenderer=function(e,r){var n=r!=null,o=n?r:e;_.exists(o)&&o!==""?this.getGui().textContent=o.toString():this.getGui().innerHTML="&nbsp;"},t.prototype.populateWithRenderer=function(e,r){var n=this,o={value:e,valueFormatted:r,api:this.gridOptionsWrapper.getApi()},a=this.userComponentFactory.getCellRendererDetails(this.params,o),s=a?a.newAgStackInstance():void 0;return s!=null?_.bindCellRendererToHtmlElement(s,this.getGui()):this.getGui().innerText=o.valueFormatted!=null?o.valueFormatted:o.value,s?(s.then(function(l){n.addDestroyFunc(function(){n.getContext().destroyBean(l)})}),!0):!1},_q([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),t}(ee),xq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Qv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},y0=function(i){xq(t,i);function t(){var e=i.call(this,t.TEMPLATE)||this;return e.selectionConfirmed=!1,e.searchString="",e}return t.prototype.init=function(e){this.params=e,this.selectedValue=e.value,this.originalSelectedValue=e.value,this.focusAfterAttached=e.cellStartedEdit;var r=_.createIconNoSpan("smallDown",this.gridOptionsWrapper);if(r.classList.add("ag-rich-select-value-icon"),this.eValue.appendChild(r),this.virtualList=this.getContext().createBean(new tp("rich-select")),this.virtualList.setComponentCreator(this.createRowComponent.bind(this)),this.eList.appendChild(this.virtualList.getGui()),_.exists(this.params.cellHeight)&&this.virtualList.setRowHeight(this.params.cellHeight),this.renderSelectedValue(),_.missing(e.values)){console.warn("AG Grid: richSelectCellEditor requires values for it to work");return}var n=e.values;this.virtualList.setModel({getRowCount:function(){return n.length},getRow:function(s){return n[s]}}),this.addGuiEventListener("keydown",this.onKeyDown.bind(this));var o=this.virtualList.getGui();this.addManagedListener(o,"click",this.onClick.bind(this)),this.addManagedListener(o,"mousemove",this.onMouseMove.bind(this));var a=_.exists(e.searchDebounceDelay)?e.searchDebounceDelay:300;this.clearSearchString=_.debounce(this.clearSearchString,a),_.exists(e.charPress)&&this.searchText(e.charPress)},t.prototype.onKeyDown=function(e){var r=e.key;switch(e.preventDefault(),r){case j.ENTER:this.onEnterKeyDown();break;case j.TAB:this.confirmSelection();break;case j.DOWN:case j.UP:this.onNavigationKeyPressed(e,r);break;default:this.searchText(e)}},t.prototype.confirmSelection=function(){this.selectionConfirmed=!0},t.prototype.onEnterKeyDown=function(){this.confirmSelection(),this.params.stopEditing()},t.prototype.onNavigationKeyPressed=function(e,r){e.preventDefault();var n=this.params.values.indexOf(this.selectedValue),o=r===j.UP?n-1:n+1;if(o>=0&&o<this.params.values.length){var a=this.params.values[o];this.setSelectedValue(a)}},t.prototype.searchText=function(e){if(typeof e!="string"){var r=e.key;if(r===j.BACKSPACE)this.searchString=this.searchString.slice(0,-1),r="";else if(!_.isEventFromPrintableCharacter(e))return;this.searchText(r);return}this.searchString+=e,this.runSearch(),this.clearSearchString()},t.prototype.runSearch=function(){var e=this,r=this.params.values,n;if((typeof r[0]=="number"||typeof r[0]=="string")&&(n=r.map(String)),typeof r[0]=="object"&&this.params.colDef.keyCreator&&(n=r.map(function(l){var u={value:l,colDef:e.params.colDef,column:e.params.column,node:e.params.node,data:e.params.data,api:e.gridOptionsWrapper.getApi(),columnApi:e.gridOptionsWrapper.getColumnApi(),context:e.gridOptionsWrapper.getContext()};return e.params.colDef.keyCreator(u)})),!!n){var o=_.fuzzySuggestions(this.searchString,n,!0,!0)[0];if(!!o){var a=n.indexOf(o),s=r[a];this.setSelectedValue(s)}}},t.prototype.clearSearchString=function(){this.searchString=""},t.prototype.renderSelectedValue=function(){var e=this,r=this.params.formatValue(this.selectedValue),n=this.eValue,o={value:this.selectedValue,valueFormatted:r,api:this.gridOptionsWrapper.getApi()},a=this.userComponentFactory.getCellRendererDetails(this.params,o),s=a?a.newAgStackInstance():void 0;s?(_.bindCellRendererToHtmlElement(s,n),s.then(function(l){e.addDestroyFunc(function(){return e.getContext().destroyBean(l)})})):_.exists(this.selectedValue)?n.innerHTML=r:_.clearElement(n)},t.prototype.setSelectedValue=function(e){if(this.selectedValue!==e){var r=this.params.values.indexOf(e);r!==-1&&(this.selectedValue=e,this.virtualList.ensureIndexVisible(r),this.virtualList.refresh())}},t.prototype.createRowComponent=function(e){var r=this.params.formatValue(e),n=new Sq(this.params);return this.getContext().createBean(n),n.setState(e,r,e===this.selectedValue),n},t.prototype.onMouseMove=function(e){var r=this.virtualList.getGui().getBoundingClientRect(),n=this.virtualList.getScrollTop(),o=e.clientY-r.top+n,a=Math.floor(o/this.virtualList.getRowHeight()),s=this.params.values[a];s!==void 0&&this.setSelectedValue(s)},t.prototype.onClick=function(){this.confirmSelection(),this.params.stopEditing()},t.prototype.afterGuiAttached=function(){var e=this.params.values.indexOf(this.selectedValue);this.virtualList.refresh(),e>=0&&this.virtualList.ensureIndexVisible(e),this.virtualList.refresh(),this.focusAfterAttached&&this.getGui().focus()},t.prototype.getValue=function(){return this.selectionConfirmed?this.selectedValue:this.originalSelectedValue},t.TEMPLATE=`<div class="ag-rich-select" tabindex="-1">
            <div ref="eValue" class="ag-rich-select-value"></div>
            <div ref="eList" class="ag-rich-select-list"></div>
        </div>`,Qv([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),Qv([I("eValue")],t.prototype,"eValue",void 0),Qv([I("eList")],t.prototype,"eList",void 0),t}(ws),Tq={moduleName:oe.RichSelectModule,beans:[],userComponents:[{componentName:"agRichSelect",componentClass:y0},{componentName:"agRichSelectCellEditor",componentClass:y0}],dependantModules:[cr]},Oq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Nr=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Eq=function(i){Oq(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.pauseStoreUpdateListening=!1,e.started=!1,e}return t.prototype.ensureRowHeightsValid=function(){return!1},t.prototype.start=function(){this.started=!0;var e=this.gridOptionsWrapper.getServerSideDatasource();e&&this.setDatasource(e)},t.prototype.destroyDatasource=function(){!this.datasource||(this.datasource.destroy&&this.datasource.destroy(),this.rowRenderer.datasourceChanged(),this.datasource=void 0)},t.prototype.addEventListeners=function(){this.addManagedListener(this.eventService,x.EVENT_NEW_COLUMNS_LOADED,this.onColumnEverything.bind(this)),this.addManagedListener(this.eventService,x.EVENT_STORE_UPDATED,this.onStoreUpdated.bind(this));var e=this.resetRootStore.bind(this);this.addManagedListener(this.eventService,x.EVENT_COLUMN_VALUE_CHANGED,e),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_CHANGED,e),this.addManagedListener(this.eventService,x.EVENT_COLUMN_ROW_GROUP_CHANGED,e),this.addManagedListener(this.eventService,x.EVENT_COLUMN_PIVOT_MODE_CHANGED,e),this.verifyProps()},t.prototype.verifyProps=function(){if(this.gridOptionsWrapper.getInitialGroupOrderComparator()!=null){var e="AG Grid: initialGroupOrderComparator cannot be used with Server Side Row Model. If using Full Store, then provide the rows to the grid in the desired sort order. If using Partial Store, then sorting is done on the server side, nothing to do with the client.";_.doOnce(function(){return console.warn(e)},"SSRM.InitialGroupOrderComparator")}if(this.gridOptionsWrapper.isRowSelection()&&this.gridOptionsWrapper.getRowIdFunc()==null){var r="AG Grid: getRowId callback must be provided for Server Side Row Model selection to work correctly.";_.doOnce(function(){return console.warn(r)},"SSRM.SelectionNeedsRowNodeIdFunc")}},t.prototype.setDatasource=function(e){!this.started||(this.destroyDatasource(),this.datasource=e,this.resetRootStore())},t.prototype.isLastRowIndexKnown=function(){var e=this.getRootStore();return e?e.isLastRowIndexKnown():!1},t.prototype.onColumnEverything=function(){if(!this.gridOptionsWrapper.isSuppressEnterpriseResetOnNewColumns()){if(!this.storeParams){this.resetRootStore();return}var e=this.columnsToValueObjects(this.columnModel.getRowGroupColumns()),r=this.columnsToValueObjects(this.columnModel.getValueColumns()),n=this.columnsToValueObjects(this.columnModel.getPivotColumns()),o=!_.jsonEquals(this.storeParams.sortModel,this.sortController.getSortModel()),a=!_.jsonEquals(this.storeParams.rowGroupCols,e),s=!_.jsonEquals(this.storeParams.pivotCols,n),l=!_.jsonEquals(this.storeParams.valueCols,r),u=o||a||s||l;u&&this.resetRootStore()}},t.prototype.destroyRootStore=function(){!this.rootNode||!this.rootNode.childStore||(this.rootNode.childStore=this.destroyBean(this.rootNode.childStore),this.nodeManager.clear())},t.prototype.refreshAfterSort=function(e,r){this.storeParams&&(this.storeParams.sortModel=e);var n=this.getRootStore();!n||(n.refreshAfterSort(r),this.onStoreUpdated())},t.prototype.resetRootStore=function(){this.destroyRootStore(),this.rootNode=new Oe(this.beans),this.rootNode.group=!0,this.rootNode.level=-1,this.datasource&&(this.storeParams=this.createStoreParams(),this.rootNode.childStore=this.createBean(this.storeFactory.createStore(this.storeParams,this.rootNode)),this.updateRowIndexesAndBounds());var e={type:x.EVENT_ROW_DATA_CHANGED,api:this.gridApi,columnApi:this.columnApi};this.eventService.dispatchEvent(e),this.dispatchModelUpdated(!0)},t.prototype.columnsToValueObjects=function(e){var r=this;return e.map(function(n){return{id:n.getId(),aggFunc:n.getAggFunc(),displayName:r.columnModel.getDisplayNameForColumn(n,"model"),field:n.getColDef().field}})},t.prototype.createStoreParams=function(){var e=this.columnsToValueObjects(this.columnModel.getRowGroupColumns()),r=this.columnsToValueObjects(this.columnModel.getValueColumns()),n=this.columnsToValueObjects(this.columnModel.getPivotColumns()),o=this.gridOptionsWrapper.isDynamicRowHeight(),a={valueCols:r,rowGroupCols:e,pivotCols:n,pivotMode:this.columnModel.isPivotMode(),filterModel:this.filterManager.getFilterModel(),sortModel:this.sortListener.extractSortModel(),datasource:this.datasource,lastAccessedSequence:new Bt,dynamicRowHeight:o};return a},t.prototype.getParams=function(){return this.storeParams},t.prototype.dispatchModelUpdated=function(e){e===void 0&&(e=!1);var r={type:x.EVENT_MODEL_UPDATED,api:this.gridApi,columnApi:this.columnApi,animate:!e,keepRenderedRows:!e,newPage:!1,newData:!1};this.eventService.dispatchEvent(r)},t.prototype.onStoreUpdated=function(){this.pauseStoreUpdateListening||(this.updateRowIndexesAndBounds(),this.dispatchModelUpdated())},t.prototype.onRowHeightChanged=function(){this.updateRowIndexesAndBounds(),this.dispatchModelUpdated()},t.prototype.updateRowIndexesAndBounds=function(){var e=this.getRootStore();!e||e.setDisplayIndexes(new Bt,{value:0})},t.prototype.retryLoads=function(){var e=this.getRootStore();!e||(e.retryLoads(),this.onStoreUpdated())},t.prototype.getRow=function(e){var r=this.getRootStore();if(!!r)return r.getRowUsingDisplayIndex(e)},t.prototype.expandAll=function(e){this.pauseStoreUpdateListening=!0,this.forEachNode(function(r){r.group&&r.setExpanded(e)}),this.pauseStoreUpdateListening=!1,this.onStoreUpdated()},t.prototype.refreshAfterFilter=function(e,r){this.storeParams&&(this.storeParams.filterModel=e);var n=this.getRootStore();!n||(n.refreshAfterFilter(r),this.onStoreUpdated())},t.prototype.getRootStore=function(){if(this.rootNode&&this.rootNode.childStore)return this.rootNode.childStore},t.prototype.getRowCount=function(){var e=this.getRootStore();return e?e.getDisplayIndexEnd():0},t.prototype.getTopLevelRowCount=function(){var e=this.getRootStore();return e?e.getRowCount():1},t.prototype.getTopLevelRowDisplayedIndex=function(e){var r=this.getRootStore();return r?r.getTopLevelRowDisplayedIndex(e):e},t.prototype.getRowBounds=function(e){var r=this.getRootStore();if(!r){var n=this.gridOptionsWrapper.getRowHeightAsNumber();return{rowTop:0,rowHeight:n}}return r.getRowBounds(e)},t.prototype.getRowIndexAtPixel=function(e){var r=this.getRootStore();return e<=0||!r?0:r.getRowIndexAtPixel(e)},t.prototype.isEmpty=function(){return!1},t.prototype.isRowsToRender=function(){return this.getRootStore()!=null&&this.getRowCount()>0},t.prototype.getType=function(){return L.ROW_MODEL_TYPE_SERVER_SIDE},t.prototype.forEachNode=function(e){var r=this.getRootStore();!r||r.forEachNodeDeep(e)},t.prototype.forEachNodeAfterFilterAndSort=function(e){var r=this.getRootStore();!r||r.forEachNodeDeepAfterFilterAndSort(e)},t.prototype.executeOnStore=function(e,r){var n=this.getRootStore();if(!!n){var o=n.getChildStore(e);o&&r(o)}},t.prototype.refreshStore=function(e){e===void 0&&(e={});var r=e.route?e.route:[];this.executeOnStore(r,function(n){return n.refreshStore(e.purge==!0)})},t.prototype.getStoreState=function(){var e=[],r=this.getRootStore();return r&&r.addStoreStates(e),e},t.prototype.getNodesInRangeForSelection=function(e,r){return _.exists(r)&&e.parent!==r.parent?[]:e.parent.childStore.getRowNodesInRange(r,e)},t.prototype.getRowNode=function(e){var r;return this.forEachNode(function(n){n.id===e&&(r=n),n.detailNode&&n.detailNode.id===e&&(r=n.detailNode)}),r},t.prototype.isRowPresent=function(e){var r=this.getRowNode(e.id);return!!r},Nr([y("columnModel")],t.prototype,"columnModel",void 0),Nr([y("filterManager")],t.prototype,"filterManager",void 0),Nr([y("sortController")],t.prototype,"sortController",void 0),Nr([y("gridApi")],t.prototype,"gridApi",void 0),Nr([y("columnApi")],t.prototype,"columnApi",void 0),Nr([y("rowRenderer")],t.prototype,"rowRenderer",void 0),Nr([y("ssrmSortService")],t.prototype,"sortListener",void 0),Nr([y("ssrmNodeManager")],t.prototype,"nodeManager",void 0),Nr([y("ssrmStoreFactory")],t.prototype,"storeFactory",void 0),Nr([y("beans")],t.prototype,"beans",void 0),Nr([nt],t.prototype,"destroyDatasource",null),Nr([B],t.prototype,"addEventListeners",null),Nr([nt],t.prototype,"destroyRootStore",null),t=Nr([Q("rowModel")],t),t}(H),Rq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Jv=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Pq=function(i){Rq(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.loadFromDatasource=function(e){var r=e.storeParams,n=e.parentBlock,o=e.parentNode,a=o.getGroupKeys();if(!!r.datasource){var s={startRow:e.startRow,endRow:e.endRow,rowGroupCols:r.rowGroupCols,valueCols:r.valueCols,pivotCols:r.pivotCols,pivotMode:r.pivotMode,groupKeys:a,filterModel:r.filterModel,sortModel:r.sortModel},l={successCallback:e.successCallback,success:e.success,failCallback:e.failCallback,fail:e.fail,request:s,parentNode:e.parentNode,api:this.gridApi,columnApi:this.columnApi,context:this.gridOptionsWrapper.getContext()};window.setTimeout(function(){if(!r.datasource||!n.isAlive()){e.failCallback();return}r.datasource.getRows(l)},0)}},t.prototype.getChildStore=function(e,r,n){if(_.missingOrEmpty(e))return r;var o=e[0],a=n(o);if(a){var s=e.slice(1,e.length),l=a.childStore;return l?l.getChildStore(s):null}return null},t.prototype.isServerRefreshNeeded=function(e,r,n){if(n.alwaysReset||n.valueColChanged||n.secondaryColChanged)return!0;var o=e.level+1,a=o<r.length,s=!a;if(s)return!0;var l=r[o].id,u=n.changedColumns.indexOf(l)>-1;return!!u},Jv([y("columnApi")],t.prototype,"columnApi",void 0),Jv([y("gridApi")],t.prototype,"gridApi",void 0),t=Jv([Q("ssrmStoreUtils")],t),t}(H),Aq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),nl=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Dq=function(i){Aq(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){this.rowHeight=this.gridOptionsWrapper.getRowHeightAsNumber(),this.usingTreeData=this.gridOptionsWrapper.isTreeData(),this.usingMasterDetail=this.gridOptionsWrapper.isMasterDetail()},t.prototype.createRowNode=function(e){var r=new Oe(this.beans),n=e.rowHeight!=null?e.rowHeight:this.rowHeight;return r.setRowHeight(n),r.group=e.group,r.leafGroup=e.leafGroup,r.level=e.level,r.uiLevel=e.level,r.parent=e.parent,r.stub=!0,r.group&&(r.expanded=!1,r.field=e.field,r.rowGroupColumn=e.rowGroupColumn),r},t.prototype.destroyRowNodes=function(e){e&&e.forEach(this.destroyRowNode.bind(this))},t.prototype.destroyRowNode=function(e){e.childStore&&(this.destroyBean(e.childStore),e.childStore=null),e.clearRowTopAndRowIndex(),e.id!=null&&this.nodeManager.removeNode(e)},t.prototype.setTreeGroupInfo=function(e){var r=this.gridOptionsWrapper.getIsServerSideGroupFunc(),n=this.gridOptionsWrapper.getServerSideGroupKeyFunc();r!=null&&(e.setGroup(r(e.data)),e.group&&n!=null&&(e.key=n(e.data))),!e.group&&e.childStore!=null&&(this.destroyBean(e.childStore),e.childStore=null)},t.prototype.setRowGroupInfo=function(e){e.key=this.valueService.getValue(e.rowGroupColumn,e),(e.key===null||e.key===void 0)&&_.doOnce(function(){console.warn("AG Grid: null and undefined values are not allowed for server side row model keys"),e.rowGroupColumn&&console.warn("column = "+e.rowGroupColumn.getId()),console.warn("data is ",e.data)},"ServerSideBlock-CannotHaveNullOrUndefinedForKey")},t.prototype.setMasterDetailInfo=function(e){var r=this.gridOptionsWrapper.getIsRowMasterFunc();r!=null?e.master=r(e.data):e.master=!0},t.prototype.updateDataIntoRowNode=function(e,r){e.updateData(r),this.usingTreeData?this.setTreeGroupInfo(e):e.group||this.usingMasterDetail},t.prototype.setDataIntoRowNode=function(e,r,n,o){e.stub=!1,_.exists(r)?(e.setDataAndId(r,n),this.usingTreeData?this.setTreeGroupInfo(e):e.group?this.setRowGroupInfo(e):this.usingMasterDetail&&this.setMasterDetailInfo(e)):(e.setDataAndId(void 0,void 0),e.key=null),(this.usingTreeData||e.group)&&(this.setGroupDataIntoRowNode(e),this.setChildCountIntoRowNode(e)),_.exists(r)&&e.setRowHeight(this.gridOptionsWrapper.getRowHeightForNode(e,!1,o).height)},t.prototype.setChildCountIntoRowNode=function(e){var r=this.gridOptionsWrapper.getChildCountFunc();r&&(e.allChildrenCount=r(e.data))},t.prototype.setGroupDataIntoRowNode=function(e){var r=this,n=this.columnModel.getGroupDisplayColumns(),o=this.gridOptionsWrapper.isTreeData();n.forEach(function(a){if(e.groupData==null&&(e.groupData={}),o)e.groupData[a.getColId()]=e.key;else if(a.isRowGroupDisplayed(e.rowGroupColumn.getId())){var s=r.valueService.getValue(e.rowGroupColumn,e);e.groupData[a.getColId()]=s}})},t.prototype.clearDisplayIndex=function(e){e.clearRowTopAndRowIndex();var r=e.group&&_.exists(e.childStore);if(r){var n=e.childStore;n.clearDisplayIndexes()}var o=e.master&&e.detailNode;o&&e.detailNode.clearRowTopAndRowIndex()},t.prototype.setDisplayIndex=function(e,r,n){e.setRowIndex(r.next()),e.setRowTop(n.value),n.value+=e.rowHeight;var o=e.master;o&&(e.expanded&&e.detailNode?(e.detailNode.setRowIndex(r.next()),e.detailNode.setRowTop(n.value),n.value+=e.detailNode.rowHeight):e.detailNode&&e.detailNode.clearRowTopAndRowIndex());var a=e.group&&_.exists(e.childStore);if(a){var s=e.childStore;e.expanded?s.setDisplayIndexes(r,n):s.clearDisplayIndexes()}},t.prototype.binarySearchForDisplayIndex=function(e,r){var n=0,o=r.length-1;if(_.missing(o)||_.missing(n)){console.warn("AG Grid: error: topPointer = "+o+", bottomPointer = "+n);return}for(;;){var a=Math.floor((n+o)/2),s=r[a];if(s.rowIndex===e)return s;var l=s.master&&s.expanded,u=s.detailNode;if(l&&u&&u.rowIndex===e)return s.detailNode;var c=s.childStore;if(s.expanded&&c&&c.isDisplayIndexInStore(e))return c.getRowUsingDisplayIndex(e);if(s.rowIndex<e)n=a+1;else if(s.rowIndex>e)o=a-1;else{console.warn("AG Grid: error: unable to locate rowIndex = "+e+" in cache");return}}},t.prototype.extractRowBounds=function(e,r){var n=function(a){return{rowHeight:a.rowHeight,rowTop:a.rowTop}};if(e.rowIndex===r)return n(e);if(e.group&&e.expanded&&_.exists(e.childStore)){var o=e.childStore;if(o.isDisplayIndexInStore(r))return o.getRowBounds(r)}else if(e.master&&e.expanded&&_.exists(e.detailNode)&&e.detailNode.rowIndex===r)return n(e.detailNode)},t.prototype.getIndexAtPixel=function(e,r){if(e.isPixelInRange(r))return e.rowIndex;var n=e.master&&e.expanded,o=e.detailNode;if(n&&o&&o.isPixelInRange(r))return e.detailNode.rowIndex;if(e.group&&e.expanded&&_.exists(e.childStore)){var a=e.childStore;if(a.isPixelInRange(r))return a.getRowIndexAtPixel(r)}return null},t.prototype.createNodeIdPrefix=function(e){for(var r=[],n=e;n&&n.level>=0;)r.push(n.key),n=n.parent;if(r.length>0)return r.reverse().join("-")},t.prototype.checkOpenByDefault=function(e){if(!!e.isExpandable()){var r=this.gridOptionsWrapper.getIsServerSideGroupOpenByDefaultFunc();if(!!r){var n={data:e.data,rowNode:e},o=r(n);o&&window.setTimeout(function(){return e.setExpanded(!0)},0)}}},nl([y("valueService")],t.prototype,"valueService",void 0),nl([y("columnModel")],t.prototype,"columnModel",void 0),nl([y("ssrmNodeManager")],t.prototype,"nodeManager",void 0),nl([y("beans")],t.prototype,"beans",void 0),nl([B],t.prototype,"postConstruct",null),t=nl([Q("ssrmBlockUtils")],t),t}(H),C0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Mq=function(){function i(){this.rowNodes={}}return i.prototype.addRowNode=function(t){var e=t.id;this.rowNodes[e]&&(console.warn("AG Grid: Duplicate node id "+t.id+". Row ID's are provided via the getRowId() callback. Please modify the getRowId() callback code to provide unique row id values."),console.warn("first instance",this.rowNodes[e].data),console.warn("second instance",t.data)),this.rowNodes[e]=t},i.prototype.removeNode=function(t){var e=t.id;this.rowNodes[e]&&(this.rowNodes[e]=void 0)},i.prototype.clear=function(){this.rowNodes={}},C0([nt],i.prototype,"clear",null),i=C0([Q("ssrmNodeManager")],i),i}(),Fq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),ku=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Iq=function(i){Fq(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.asyncTransactions=[],e}return t.prototype.postConstruct=function(){this.gridOptionsWrapper.isRowModelServerSide()},t.prototype.applyTransactionAsync=function(e,r){this.asyncTransactionsTimeout==null&&this.scheduleExecuteAsync(),this.asyncTransactions.push({transaction:e,callback:r})},t.prototype.scheduleExecuteAsync=function(){var e=this,r=this.gridOptionsWrapper.getAsyncTransactionWaitMillis();this.asyncTransactionsTimeout=window.setTimeout(function(){e.executeAsyncTransactions()},r)},t.prototype.executeAsyncTransactions=function(){var e=this;if(!!this.asyncTransactions){var r=[],n=[],o=[],a=!1;if(this.asyncTransactions.forEach(function(l){var u;e.serverSideRowModel.executeOnStore(l.transaction.route,function(p){u=p.applyTransaction(l.transaction)}),u==null&&(u={status:hn.StoreNotFound}),n.push(u);var c=u.status==hn.StoreLoading;if(c){o.push(l);return}l.callback&&r.push(function(){return l.callback(u)}),u.status===hn.Applied&&(a=!0)}),r.length>0&&window.setTimeout(function(){r.forEach(function(l){return l()})},0),this.asyncTransactionsTimeout=void 0,this.asyncTransactions=o,a&&(this.valueCache.onDataChanged(),this.eventService.dispatchEvent({type:x.EVENT_STORE_UPDATED})),n.length>0){var s={api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),type:x.EVENT_ASYNC_TRANSACTIONS_FLUSHED,results:n};this.eventService.dispatchEvent(s)}}},t.prototype.flushAsyncTransactions=function(){this.asyncTransactionsTimeout!=null&&clearTimeout(this.asyncTransactionsTimeout),this.executeAsyncTransactions()},t.prototype.applyTransaction=function(e){var r;return this.serverSideRowModel.executeOnStore(e.route,function(n){r=n.applyTransaction(e)}),r?(this.valueCache.onDataChanged(),this.eventService.dispatchEvent({type:x.EVENT_STORE_UPDATED}),r):{status:hn.StoreNotFound}},ku([y("rowNodeBlockLoader")],t.prototype,"rowNodeBlockLoader",void 0),ku([y("valueCache")],t.prototype,"valueCache",void 0),ku([y("rowModel")],t.prototype,"serverSideRowModel",void 0),ku([B],t.prototype,"postConstruct",null),t=ku([Q("ssrmTransactionManager")],t),t}(H),Lq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Wu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Nq=function(i){Lq(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){!this.gridOptionsWrapper.isRowModelServerSide()||this.addManagedListener(this.eventService,x.EVENT_ROW_GROUP_OPENED,this.onRowGroupOpened.bind(this))},t.prototype.onRowGroupOpened=function(e){var r=e.node;if(r.expanded){if(r.master)this.createDetailNode(r);else if(_.missing(r.childStore)){var n=this.serverSideRowModel.getParams();r.childStore=this.createBean(this.storeFactory.createStore(n,r))}}else this.gridOptionsWrapper.isPurgeClosedRowNodes()&&_.exists(r.childStore)&&(r.childStore=this.destroyBean(r.childStore));var o={type:x.EVENT_STORE_UPDATED};this.eventService.dispatchEvent(o)},t.prototype.createDetailNode=function(e){if(_.exists(e.detailNode))return e.detailNode;var r=new Oe(this.beans);r.detail=!0,r.selectable=!1,r.parent=e,_.exists(e.id)&&(r.id="detail_"+e.id),r.data=e.data,r.level=e.level+1;var n=200,o=this.gridOptionsWrapper.getRowHeightForNode(r).height;return r.rowHeight=o||n,e.detailNode=r,r},Wu([y("rowModel")],t.prototype,"serverSideRowModel",void 0),Wu([y("ssrmStoreFactory")],t.prototype,"storeFactory",void 0),Wu([y("beans")],t.prototype,"beans",void 0),Wu([B],t.prototype,"postConstruct",null),t=Wu([Q("ssrmExpandListener")],t),t}(H),Gq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),il=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},kq=function(i){Gq(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){!this.gridOptionsWrapper.isRowModelServerSide()||this.addManagedListener(this.eventService,x.EVENT_SORT_CHANGED,this.onSortChanged.bind(this))},t.prototype.extractSortModel=function(){var e=this.sortController.getSortModel();return this.gridOptionsWrapper.isTreeData()||(this.replaceAutoGroupColumnWithActualRowGroupColumns(e),this.removeMultiColumnPrefixOnColumnIds(e)),e},t.prototype.removeMultiColumnPrefixOnColumnIds=function(e){if(this.gridOptionsWrapper.isGroupMultiAutoColumn())for(var r=L.GROUP_AUTO_COLUMN_ID+"-",n=0;n<e.length;++n)e[n].colId.indexOf(r)>-1&&(e[n].colId=e[n].colId.substr(r.length))},t.prototype.replaceAutoGroupColumnWithActualRowGroupColumns=function(e){var r=e.find(function(l){return l.colId==L.GROUP_AUTO_COLUMN_ID});if(r){var n=e.indexOf(r);_.removeFromArray(e,r);var o=function(l){return e.filter(function(u){return u.colId===l.getColId()}).length==0},a=function(l){return{colId:l.getId(),sort:r.sort}},s=this.columnModel.getRowGroupColumns().filter(o).map(a);_.insertArrayIntoArray(e,s,n)}},t.prototype.onSortChanged=function(){var e=this.serverSideRowModel.getParams();if(!!e){var r=this.extractSortModel(),n=e.sortModel,o=this.findChangedColumnsInSort(r,n),a=this.listenerUtils.isSortingWithValueColumn(o),s=this.listenerUtils.isSortingWithSecondaryColumn(o),l=this.gridOptionsWrapper.isServerSideSortingAlwaysResets(),u={valueColChanged:a,secondaryColChanged:s,alwaysReset:l,changedColumns:o};this.serverSideRowModel.refreshAfterSort(r,u)}},t.prototype.findChangedColumnsInSort=function(e,r){var n=[];[e,r].forEach(function(s){if(s){var l=s.map(function(u){return u.colId});n=n.concat(l)}});var o=function(s,l){var u=s?s.sort:null,c=l?l.sort:null;return u!==c},a=function(s,l){var u=s?r.indexOf(s):-1,c=l?e.indexOf(l):-1;return u!==c};return n.filter(function(s){var l=r.find(function(c){return c.colId===s}),u=e.find(function(c){return c.colId===s});return o(l,u)||a(l,u)})},il([y("sortController")],t.prototype,"sortController",void 0),il([y("columnModel")],t.prototype,"columnModel",void 0),il([y("rowModel")],t.prototype,"serverSideRowModel",void 0),il([y("ssrmListenerUtils")],t.prototype,"listenerUtils",void 0),il([B],t.prototype,"postConstruct",null),t=il([Q("ssrmSortService")],t),t}(H),Wq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Vu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Vq=function(i){Wq(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){!this.gridOptionsWrapper.isRowModelServerSide()||this.addManagedListener(this.eventService,x.EVENT_FILTER_CHANGED,this.onFilterChanged.bind(this))},t.prototype.onFilterChanged=function(){var e=this.serverSideRowModel.getParams();if(!!e){var r=this.filterManager.getFilterModel(),n=e?e.filterModel:{},o=this.findChangedColumns(r,n),a=this.listenerUtils.isSortingWithValueColumn(o),s=this.listenerUtils.isSortingWithSecondaryColumn(o),l=this.gridOptionsWrapper.isServerSideFilteringAlwaysResets(),u={valueColChanged:a,secondaryColChanged:s,alwaysReset:l,changedColumns:o};this.serverSideRowModel.refreshAfterFilter(r,u)}},t.prototype.findChangedColumns=function(e,r){var n={};Object.keys(e).forEach(function(a){return n[a]=!0}),Object.keys(r).forEach(function(a){return n[a]=!0});var o=[];return Object.keys(n).forEach(function(a){var s=JSON.stringify(e[a]),l=JSON.stringify(r[a]),u=s!=l;u&&o.push(a)}),o},Vu([y("rowModel")],t.prototype,"serverSideRowModel",void 0),Vu([y("filterManager")],t.prototype,"filterManager",void 0),Vu([y("ssrmListenerUtils")],t.prototype,"listenerUtils",void 0),Vu([B],t.prototype,"postConstruct",null),t=Vu([Q("ssrmFilterListener")],t),t}(H),Bq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ao=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Hq=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},w0=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},b0=function(i){Bq(t,i);function t(e,r,n,o,a){var s=i.call(this,e)||this;return s.ssrmParams=n,s.storeParams=o,s.parentRowNode=r,s.startRow=e*o.cacheBlockSize,s.parentStore=a,s.level=r.level+1,s.groupLevel=n.rowGroupCols?s.level<n.rowGroupCols.length:void 0,s.leafGroup=n.rowGroupCols?s.level===n.rowGroupCols.length-1:!1,s}return t.prototype.postConstruct=function(){if(this.usingTreeData=this.gridOptionsWrapper.isTreeData(),!this.usingTreeData&&this.groupLevel){var e=this.ssrmParams.rowGroupCols[this.level];this.groupField=e.field,this.rowGroupColumn=this.columnModel.getRowGroupColumns()[this.level]}this.nodeIdPrefix=this.blockUtils.createNodeIdPrefix(this.parentRowNode),this.setData([])},t.prototype.isDisplayIndexInBlock=function(e){return e>=this.displayIndexStart&&e<this.displayIndexEnd},t.prototype.isBlockBefore=function(e){return e>=this.displayIndexEnd},t.prototype.getDisplayIndexStart=function(){return this.displayIndexStart},t.prototype.getDisplayIndexEnd=function(){return this.displayIndexEnd},t.prototype.getBlockHeightPx=function(){return this.blockHeightPx},t.prototype.getBlockTopPx=function(){return this.blockTopPx},t.prototype.isGroupLevel=function(){return this.groupLevel},t.prototype.getGroupField=function(){return this.groupField},t.prototype.prefixId=function(e){return this.nodeIdPrefix!=null?this.nodeIdPrefix+"-"+e:e.toString()},t.prototype.getBlockStateJson=function(){return{id:this.prefixId(this.getId()),state:{blockNumber:this.getId(),startRow:this.startRow,endRow:this.startRow+this.storeParams.cacheBlockSize,pageStatus:this.getState()}}},t.prototype.isAnyNodeOpen=function(){var e=this.rowNodes.filter(function(r){return r.expanded}).length;return e>0},t.prototype.forEachNode=function(e,r,n,o){r===void 0&&(r=new Bt),this.rowNodes.forEach(function(a){if(e(a,r.next()),n&&a.childStore){var s=a.childStore;o?s.forEachNodeDeepAfterFilterAndSort(e,r):s.forEachNodeDeep(e,r)}})},t.prototype.forEachNodeDeep=function(e,r){this.forEachNode(e,r,!0,!1)},t.prototype.forEachNodeAfterFilterAndSort=function(e,r){this.forEachNode(e,r,!0,!0)},t.prototype.forEachNodeShallow=function(e,r){this.forEachNode(e,r,!1,!1)},t.prototype.getLastAccessed=function(){return this.lastAccessed},t.prototype.getRowUsingLocalIndex=function(e){return this.rowNodes[e-this.startRow]},t.prototype.touchLastAccessed=function(){this.lastAccessed=this.ssrmParams.lastAccessedSequence.next()},t.prototype.processServerFail=function(){this.parentStore.onBlockLoadFailed(this)},t.prototype.retryLoads=function(){this.getState()===jr.STATE_FAILED&&(this.setStateWaitingToLoad(),this.rowNodeBlockLoader.checkBlockToLoad(),this.setData()),this.forEachNodeShallow(function(e){e.childStore&&e.childStore.retryLoads()})},t.prototype.processServerResult=function(e){this.parentStore.onBlockLoaded(this,e)},t.prototype.setData=function(e,r){e===void 0&&(e=[]),r===void 0&&(r=!1),this.destroyRowNodes();for(var n=this.parentStore.getRowCount(),o=this.getId()*this.storeParams.cacheBlockSize,a=Math.min(o+this.storeParams.cacheBlockSize,n),s=a-o,l=this.columnModel.getAllDisplayedAutoHeightCols().length>0,u=l?this.parentStore.getCachedBlockHeight(this.getId()):void 0,c=u?Math.round(u/s):void 0,p=0;p<s;p++){var h=this.blockUtils.createRowNode({field:this.groupField,group:this.groupLevel,leafGroup:this.leafGroup,level:this.level,parent:this.parentRowNode,rowGroupColumn:this.rowGroupColumn,rowHeight:c}),d=p<e.length;if(d){var f=e[p],g=this.prefixId(this.startRow+p);this.blockUtils.setDataIntoRowNode(h,f,g,c);var v=h.id;this.parentStore.removeDuplicateNode(v),this.nodeManager.addRowNode(h),this.allNodesMap[h.id]=h,this.blockUtils.checkOpenByDefault(h)}this.rowNodes.push(h),r&&(h.failedLoad=!0)}},t.prototype.removeDuplicateNode=function(e){if(this.getState()!=jr.STATE_LOADED){var r=this.allNodesMap[e];if(!!r){this.blockUtils.destroyRowNode(r);var n=this.rowNodes.indexOf(r),o=this.blockUtils.createRowNode({field:this.groupField,group:this.groupLevel,leafGroup:this.leafGroup,level:this.level,parent:this.parentRowNode,rowGroupColumn:this.rowGroupColumn});this.rowNodes[n]=o}}},t.prototype.refresh=function(){this.getState()!==jr.STATE_WAITING_TO_LOAD&&this.setStateWaitingToLoad()},t.prototype.destroyRowNodes=function(){this.blockUtils.destroyRowNodes(this.rowNodes),this.rowNodes=[],this.allNodesMap={}},t.prototype.setBeans=function(e){this.logger=e.create("ServerSideBlock")},t.prototype.getRowUsingDisplayIndex=function(e){this.touchLastAccessed();var r=this.blockUtils.binarySearchForDisplayIndex(e,this.rowNodes);return r},t.prototype.loadFromDatasource=function(){this.storeUtils.loadFromDatasource({startRow:this.startRow,endRow:this.startRow+this.storeParams.cacheBlockSize,parentBlock:this,parentNode:this.parentRowNode,storeParams:this.ssrmParams,successCallback:this.pageLoaded.bind(this,this.getVersion()),success:this.success.bind(this,this.getVersion()),failCallback:this.pageLoadFailed.bind(this,this.getVersion()),fail:this.pageLoadFailed.bind(this,this.getVersion())})},t.prototype.isPixelInRange=function(e){return e>=this.blockTopPx&&e<this.blockTopPx+this.blockHeightPx},t.prototype.getRowBounds=function(e){var r,n;this.touchLastAccessed();var o;try{for(var a=w0(this.rowNodes),s=a.next();!s.done;s=a.next()){var l=s.value;if(o=this.blockUtils.extractRowBounds(l,e),o!=null)break}}catch(u){r={error:u}}finally{try{s&&!s.done&&(n=a.return)&&n.call(a)}finally{if(r)throw r.error}}return o},t.prototype.getRowIndexAtPixel=function(e){var r,n;this.touchLastAccessed();var o=null;try{for(var a=w0(this.rowNodes),s=a.next();!s.done;s=a.next()){var l=s.value;if(o=this.blockUtils.getIndexAtPixel(l,e),o!=null)break}}catch(u){r={error:u}}finally{try{s&&!s.done&&(n=a.return)&&n.call(a)}finally{if(r)throw r.error}}return o},t.prototype.clearDisplayIndexes=function(){var e=this;this.displayIndexEnd=void 0,this.displayIndexStart=void 0,this.rowNodes.forEach(function(r){return e.blockUtils.clearDisplayIndex(r)})},t.prototype.setDisplayIndexes=function(e,r){var n=this;this.displayIndexStart=e.peek(),this.blockTopPx=r.value,this.rowNodes.forEach(function(o){return n.blockUtils.setDisplayIndex(o,e,r)}),this.displayIndexEnd=e.peek(),this.blockHeightPx=r.value-this.blockTopPx},Ao([y("columnModel")],t.prototype,"columnModel",void 0),Ao([y("ssrmStoreUtils")],t.prototype,"storeUtils",void 0),Ao([y("ssrmBlockUtils")],t.prototype,"blockUtils",void 0),Ao([y("ssrmNodeManager")],t.prototype,"nodeManager",void 0),Ao([y("rowNodeBlockLoader")],t.prototype,"rowNodeBlockLoader",void 0),Ao([B],t.prototype,"postConstruct",null),Ao([nt],t.prototype,"destroyRowNodes",null),Ao([Hq(0,Yt("loggerFactory"))],t.prototype,"setBeans",null),t}(jr),jq=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Do=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Uq=globalThis&&globalThis.__param||function(i,t){return function(e,r){t(e,r,i)}},nr;(function(i){i[i.FOUND=0]="FOUND",i[i.CONTINUE_FIND=1]="CONTINUE_FIND",i[i.BREAK_FIND=2]="BREAK_FIND"})(nr||(nr={}));var zq=function(i){jq(t,i);function t(e,r,n){var o=i.call(this)||this;return o.blocks={},o.blockHeights={},o.lastRowIndexKnown=!1,o.displayIndexStart=0,o.displayIndexEnd=0,o.cacheTopPixel=0,o.info={},o.ssrmParams=e,o.storeParams=r,o.parentRowNode=n,o.rowCount=t.INITIAL_ROW_COUNT,o}return t.prototype.postConstruct=function(){this.defaultRowHeight=this.gridOptionsWrapper.getRowHeightAsNumber()},t.prototype.destroyAllBlocks=function(){var e=this;this.getBlocksInOrder().forEach(function(r){return e.destroyBlock(r)})},t.prototype.setBeans=function(e){this.logger=e.create("ServerSideCache")},t.prototype.getRowCount=function(){return this.rowCount},t.prototype.isLastRowIndexKnown=function(){return this.lastRowIndexKnown},t.prototype.retryLoads=function(){this.getBlocksInOrder().forEach(function(e){return e.retryLoads()})},t.prototype.onBlockLoadFailed=function(e){e.setData([],!0),this.fireCacheUpdatedEvent()},t.prototype.onBlockLoaded=function(e,r){if(this.logger.log("onPageLoaded: page = "+e.getId()+", lastRow = "+r.rowCount),r.storeInfo&&Object.assign(this.info,r.storeInfo),!r.rowData){var n='AG Grid: "params.rowData" is missing from Server-Side Row Model success() callback. Please use the "rowData" attribute. If no data is returned, set an empty list.';_.doOnce(function(){return console.warn(n,r)},"InfiniteStore.noData")}var o=r.rowCount!=null&&r.rowCount>=0?r.rowCount:void 0;!this.isAlive()||(this.checkRowCount(e,o),e.setData(r.rowData),this.destroyAllBlocksPastVirtualRowCount(),this.fireCacheUpdatedEvent())},t.prototype.purgeBlocksIfNeeded=function(e){var r=this,n=this.getBlocksInOrder().filter(function(u){return u!=e}),o=function(u,c){return c.getLastAccessed()-u.getLastAccessed()};n.sort(o);var a=this.storeParams.maxBlocksInCache>0,s=a?this.storeParams.maxBlocksInCache-1:null,l=t.MAX_EMPTY_BLOCKS_TO_KEEP-1;n.forEach(function(u,c){var p=u.getState()===b0.STATE_WAITING_TO_LOAD&&c>=l,h=a?c>=s:!1;if(p||h){if(u.isAnyNodeOpen()||r.isBlockCurrentlyDisplayed(u)||r.isBlockFocused(u))return;r.destroyBlock(u)}})},t.prototype.isBlockFocused=function(e){var r=this.focusService.getFocusCellToUseAfterRefresh();if(!r||r.rowPinned!=null)return!1;var n=e.getDisplayIndexStart(),o=e.getDisplayIndexEnd();if(o==null||n==null)return!1;var a=r.rowIndex>=n&&r.rowIndex<o;return a},t.prototype.isBlockCurrentlyDisplayed=function(e){var r=e.getDisplayIndexStart(),n=e.getDisplayIndexEnd()-1;return this.rowRenderer.isRangeInRenderedViewport(r,n)},t.prototype.removeDuplicateNode=function(e){this.getBlocksInOrder().forEach(function(r){return r.removeDuplicateNode(e)})},t.prototype.checkRowCount=function(e,r){if(typeof r=="number"&&r>=0)this.rowCount=r,this.lastRowIndexKnown=!0;else if(!this.lastRowIndexKnown){var n=(e.getId()+1)*this.storeParams.cacheBlockSize,o=n+t.OVERFLOW_SIZE;this.rowCount<o&&(this.rowCount=o)}},t.prototype.forEachNodeDeep=function(e,r){r===void 0&&(r=new Bt),this.getBlocksInOrder().forEach(function(n){return n.forEachNodeDeep(e,r)})},t.prototype.forEachNodeDeepAfterFilterAndSort=function(e,r){r===void 0&&(r=new Bt),this.getBlocksInOrder().forEach(function(n){return n.forEachNodeAfterFilterAndSort(e,r)})},t.prototype.getBlocksInOrder=function(){var e=function(n,o){return n.getId()-o.getId()},r=_.getAllValuesInObject(this.blocks).sort(e);return r},t.prototype.destroyBlock=function(e){delete this.blocks[e.getId()],this.destroyBean(e),this.rowNodeBlockLoader.removeBlock(e)},t.prototype.fireCacheUpdatedEvent=function(){var e={type:x.EVENT_STORE_UPDATED};this.eventService.dispatchEvent(e)},t.prototype.destroyAllBlocksPastVirtualRowCount=function(){var e=this,r=[];this.getBlocksInOrder().forEach(function(n){var o=n.getId()*e.storeParams.cacheBlockSize;o>=e.rowCount&&r.push(n)}),r.length>0&&r.forEach(function(n){return e.destroyBlock(n)})},t.prototype.refreshStore=function(e){var r=this.getRowCount()==0;r||e?this.resetStore():this.refreshBlocks(),this.fireCacheUpdatedEvent()},t.prototype.refreshBlocks=function(){this.getBlocksInOrder().forEach(function(e){e.refresh()}),this.lastRowIndexKnown=!1,this.rowNodeBlockLoader.checkBlockToLoad()},t.prototype.resetStore=function(){this.destroyAllBlocks(),this.lastRowIndexKnown=!1,(this.columnModel.isAutoRowHeightActive||this.rowCount===0)&&(this.rowCount=t.INITIAL_ROW_COUNT)},t.prototype.getRowNodesInRange=function(e,r){var n=[],o=-1,a=!1;_.missing(e)&&(a=!0);var s=!1;this.getBlocksInOrder().forEach(function(u){if(!s){if(a&&o+1!==u.getId()){s=!0;return}o=u.getId(),u.forEachNodeShallow(function(c){var p=c===e||c===r;(a||p)&&n.push(c),p&&(a=!a)})}});var l=s||a;return l?[]:n},t.prototype.findBlockAndExecute=function(e,r,n){var o=!1,a=!1,s=null,l;return this.getBlocksInOrder().forEach(function(u){if(!(o||a)){var c=e(u);c==nr.FOUND?(l=r(u),o=!0):c==nr.CONTINUE_FIND?s=u:c==nr.BREAK_FIND&&(a=!0)}}),o||(l=n(s)),l},t.prototype.getRowBounds=function(e){var r=this,n=function(s){return s.isDisplayIndexInBlock(e)?nr.FOUND:s.isBlockBefore(e)?nr.CONTINUE_FIND:nr.BREAK_FIND},o=function(s){return s.getRowBounds(e)},a=function(s){var l,u;s!==null?(l=s.getBlockTopPx()+s.getBlockHeightPx(),u=s.getDisplayIndexEnd()):(l=r.cacheTopPixel,u=r.displayIndexStart);var c=e-u;return{rowHeight:r.defaultRowHeight,rowTop:l+c*r.defaultRowHeight}};return this.findBlockAndExecute(n,o,a)},t.prototype.getRowIndexAtPixel=function(e){var r=this,n=function(u){return u.isPixelInRange(e)?nr.FOUND:u.getBlockTopPx()<e?nr.CONTINUE_FIND:nr.BREAK_FIND},o=function(u){return u.getRowIndexAtPixel(e)},a=function(u){var c,p;u?(c=u.getBlockTopPx()+u.getBlockHeightPx(),p=u.getDisplayIndexEnd()):(c=r.cacheTopPixel,p=r.displayIndexStart);for(var h=r.storeParams.cacheBlockSize,d=r.defaultRowHeight*h,f=u?u.getId()+1:0,g=function(b){var S=r.getCachedBlockHeight(b),T=S!=null?S:d,E=e<=T+c;return{height:T,pixelInBlock:E}},v=g(f);!v.pixelInBlock;)c+=v.height,p+=h,f++,v=g(f);var m=e-c,C=v.height/h,w=Math.floor(m/C)|0;return p+w},s=this.findBlockAndExecute(n,o,a),l=this.getDisplayIndexEnd()-1;return s=Math.min(s,l),s},t.prototype.clearDisplayIndexes=function(){this.displayIndexStart=void 0,this.displayIndexEnd=void 0,this.getBlocksInOrder().forEach(function(e){return e.clearDisplayIndexes()})},t.prototype.setDisplayIndexes=function(e,r){var n=this;this.displayIndexStart=e.peek(),this.cacheTopPixel=r.value;var o=-1,a=this.storeParams.cacheBlockSize;this.getBlocksInOrder().forEach(function(c){var p=c.getId(),h=p-o-1,d=h*a;d>0&&e.skip(d);for(var f=1;f<=h;f++){var g=p-f;_.exists(n.blockHeights[g])?r.value+=n.blockHeights[g]:r.value+=a*n.defaultRowHeight}o=p,c.setDisplayIndexes(e,r),n.blockHeights[p]=c.getBlockHeightPx()});var s=(o+1)*a-1,l=this.getRowCount(),u=l-s-1;u>0&&(e.skip(u),r.value+=u*this.defaultRowHeight),this.displayIndexEnd=e.peek(),this.cacheHeightPixels=r.value-this.cacheTopPixel},t.prototype.getRowUsingDisplayIndex=function(e,r){var n=this;if(r===void 0&&(r=!1),!!this.isDisplayIndexInStore(e)){var o=function(l){return l.isDisplayIndexInBlock(e)?nr.FOUND:l.isBlockBefore(e)?nr.CONTINUE_FIND:nr.BREAK_FIND},a=function(l){return l.getRowUsingDisplayIndex(e)},s=function(l){if(!r){var u,c,p,h=n.storeParams.cacheBlockSize;if(l){u=l.getId()+1,c=l.getDisplayIndexEnd(),p=l.getBlockHeightPx()+l.getBlockTopPx();for(var d=function(){return e>=c&&e<c+h};!d();){c+=h;var f=n.blockHeights[u];_.exists(f)?p+=f:p+=n.defaultRowHeight*h,u++}}else{var g=e-n.displayIndexStart;u=Math.floor(g/h),c=n.displayIndexStart+u*h,p=n.cacheTopPixel+u*h*n.defaultRowHeight}n.logger.log("block missing, rowIndex = "+e+", creating #"+u+", displayIndexStart = "+c);var v=n.createBlock(u,c,{value:p});return v.getRowUsingDisplayIndex(e)}};return this.findBlockAndExecute(o,a,s)}},t.prototype.getTopLevelRowDisplayedIndex=function(e){var r=this.storeParams.cacheBlockSize,n=Math.floor(e/r),o=function(l){return l.getId()===n?nr.FOUND:l.getId()<n?nr.CONTINUE_FIND:nr.BREAK_FIND},a=function(l){var u=l.getRowUsingLocalIndex(e);return u.rowIndex},s=function(l){if(!l)return e;var u=r*(l.getId()+1)-1,c=l.getRowUsingLocalIndex(u),p;if(c.expanded&&c.childStore){var h=c.childStore;p=h.getDisplayIndexEnd()-1}else c.expanded&&c.detailNode?p=c.detailNode.rowIndex:p=c.rowIndex;var d=e-u;return p+d};return this.findBlockAndExecute(o,a,s)},t.prototype.addStoreStates=function(e){e.push({type:"partial",route:this.parentRowNode.getGroupKeys(),rowCount:this.rowCount,lastRowIndexKnown:this.lastRowIndexKnown,info:this.info,maxBlocksInCache:this.storeParams.maxBlocksInCache,cacheBlockSize:this.storeParams.cacheBlockSize}),this.forEachChildStoreShallow(function(r){return r.addStoreStates(e)})},t.prototype.getCachedBlockHeight=function(e){return this.blockHeights[e]},t.prototype.createBlock=function(e,r,n){var o=this.createBean(new b0(e,this.parentRowNode,this.ssrmParams,this.storeParams,this));return o.setDisplayIndexes(new Bt(r),n),this.blocks[o.getId()]=o,this.purgeBlocksIfNeeded(o),this.rowNodeBlockLoader.addBlock(o),o},t.prototype.getDisplayIndexEnd=function(){return this.displayIndexEnd},t.prototype.isDisplayIndexInStore=function(e){return this.getRowCount()===0?!1:e>=this.displayIndexStart&&e<this.displayIndexEnd},t.prototype.applyTransaction=function(e){return{status:hn.StoreWrongType}},t.prototype.getChildStore=function(e){var r=this,n=function(o){var a=null;return r.getBlocksInOrder().forEach(function(s){s.forEachNodeShallow(function(l){l.key===o&&(a=l)},new Bt)}),a};return this.storeUtils.getChildStore(e,this,n)},t.prototype.isPixelInRange=function(e){return this.getRowCount()===0?!1:e>=this.cacheTopPixel&&e<this.cacheTopPixel+this.cacheHeightPixels},t.prototype.refreshAfterFilter=function(e){this.resetStore()},t.prototype.refreshAfterSort=function(e){if(this.storeUtils.isServerRefreshNeeded(this.parentRowNode,this.ssrmParams.rowGroupCols,e)){this.resetStore();return}this.forEachChildStoreShallow(function(r){return r.refreshAfterSort(e)})},t.prototype.forEachChildStoreShallow=function(e){this.getBlocksInOrder().forEach(function(r){if(r.isGroupLevel()){var n=function(o){var a=o.childStore;a&&e(a)};r.forEachNodeShallow(n,new Bt)}})},t.MAX_EMPTY_BLOCKS_TO_KEEP=2,t.INITIAL_ROW_COUNT=1,t.OVERFLOW_SIZE=1,Do([y("rowRenderer")],t.prototype,"rowRenderer",void 0),Do([y("rowNodeBlockLoader")],t.prototype,"rowNodeBlockLoader",void 0),Do([y("ssrmStoreUtils")],t.prototype,"storeUtils",void 0),Do([y("focusService")],t.prototype,"focusService",void 0),Do([y("columnModel")],t.prototype,"columnModel",void 0),Do([B],t.prototype,"postConstruct",null),Do([nt],t.prototype,"destroyAllBlocks",null),Do([Uq(0,Yt("loggerFactory"))],t.prototype,"setBeans",null),t}(H),$q=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),en=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Kq=function(i){$q(t,i);function t(e,r,n){var o=i.call(this,0)||this;return o.nodeIdSequence=new Bt,o.info={},o.ssrmParams=e,o.parentRowNode=n,o.level=n.level+1,o.groupLevel=e.rowGroupCols?o.level<e.rowGroupCols.length:void 0,o.leafGroup=e.rowGroupCols?o.level===e.rowGroupCols.length-1:!1,o}return t.prototype.postConstruct=function(){var e=this;if(this.usingTreeData=this.gridOptionsWrapper.isTreeData(),this.nodeIdPrefix=this.blockUtils.createNodeIdPrefix(this.parentRowNode),!this.usingTreeData&&this.groupLevel){var r=this.ssrmParams.rowGroupCols[this.level];this.groupField=r.field,this.rowGroupColumn=this.columnModel.getRowGroupColumns()[this.level]}this.initialiseRowNodes(),this.rowNodeBlockLoader.addBlock(this),this.addDestroyFunc(function(){return e.rowNodeBlockLoader.removeBlock(e)})},t.prototype.destroyRowNodes=function(){this.blockUtils.destroyRowNodes(this.allRowNodes),this.allRowNodes=[],this.nodesAfterSort=[],this.nodesAfterFilter=[],this.allNodesMap={}},t.prototype.initialiseRowNodes=function(e,r){e===void 0&&(e=1),r===void 0&&(r=!1),this.destroyRowNodes();for(var n=0;n<e;n++){var o=this.blockUtils.createRowNode({field:this.groupField,group:this.groupLevel,leafGroup:this.leafGroup,level:this.level,parent:this.parentRowNode,rowGroupColumn:this.rowGroupColumn});r&&(o.failedLoad=!0),this.allRowNodes.push(o),this.nodesAfterFilter.push(o),this.nodesAfterSort.push(o)}},t.prototype.getBlockStateJson=function(){return{id:this.nodeIdPrefix?this.nodeIdPrefix:"",state:this.getState()}},t.prototype.loadFromDatasource=function(){this.storeUtils.loadFromDatasource({startRow:void 0,endRow:void 0,parentBlock:this,parentNode:this.parentRowNode,storeParams:this.ssrmParams,successCallback:this.pageLoaded.bind(this,this.getVersion()),success:this.success.bind(this,this.getVersion()),failCallback:this.pageLoadFailed.bind(this,this.getVersion()),fail:this.pageLoadFailed.bind(this,this.getVersion())})},t.prototype.getStartRow=function(){return 0},t.prototype.getEndRow=function(){return this.nodesAfterSort.length},t.prototype.createDataNode=function(e,r){var n=this.blockUtils.createRowNode({field:this.groupField,group:this.groupLevel,leafGroup:this.leafGroup,level:this.level,parent:this.parentRowNode,rowGroupColumn:this.rowGroupColumn});r!=null?_.insertIntoArray(this.allRowNodes,n,r):this.allRowNodes.push(n);var o=this.prefixId(this.nodeIdSequence.next());return this.blockUtils.setDataIntoRowNode(n,e,o,void 0),this.nodeManager.addRowNode(n),this.blockUtils.checkOpenByDefault(n),this.allNodesMap[n.id]=n,n},t.prototype.prefixId=function(e){return this.nodeIdPrefix?this.nodeIdPrefix+"-"+e:e.toString()},t.prototype.processServerFail=function(){this.initialiseRowNodes(1,!0),this.fireStoreUpdatedEvent(),this.flushAsyncTransactions()},t.prototype.processServerResult=function(e){if(!!this.isAlive()){e.storeInfo&&Object.assign(this.info,e.storeInfo);var r=this.allRowNodes.length>0?this.allNodesMap:void 0;if(this.allRowNodes=[],this.nodesAfterSort=[],this.nodesAfterFilter=[],this.allNodesMap={},!e.rowData){var n='AG Grid: "params.data" is missing from Server-Side Row Model success() callback. Please use the "data" attribute. If no data is returned, set an empty list.';_.doOnce(function(){return console.warn(n,e)},"FullStore.noData")}this.createOrRecycleNodes(r,e.rowData),r&&this.blockUtils.destroyRowNodes(_.getAllValuesInObject(r)),this.filterAndSortNodes(),this.fireStoreUpdatedEvent(),this.flushAsyncTransactions()}},t.prototype.createOrRecycleNodes=function(e,r){var n=this;if(!!r){var o=function(s){if(!!e){var l=n.gridOptionsWrapper.getRowIdFunc();if(!!l){var u=n.parentRowNode.getGroupKeys(),c=n.level,p=l({data:s,parentKeys:u.length>0?u:void 0,level:c}),h=e[p];if(!!h)return delete e[p],h}}},a=function(s,l){n.allNodesMap[s.id]=s,n.blockUtils.updateDataIntoRowNode(s,l),n.allRowNodes.push(s)};r.forEach(function(s){var l=o(s);l?a(l,s):n.createDataNode(s)})}},t.prototype.flushAsyncTransactions=function(){var e=this;window.setTimeout(function(){return e.transactionManager.flushAsyncTransactions()},0)},t.prototype.filterAndSortNodes=function(){this.filterRowNodes(),this.sortRowNodes()},t.prototype.sortRowNodes=function(){var e=this.sortController.getSortOptions(),r=!e||e.length==0,n=this.gridOptionsWrapper.isServerSideSortingAlwaysResets();if(r||n){this.nodesAfterSort=this.nodesAfterFilter;return}this.nodesAfterSort=this.rowNodeSorter.doFullSort(this.nodesAfterFilter,e)},t.prototype.filterRowNodes=function(){var e=this,r=this.gridOptionsWrapper.isServerSideFilteringAlwaysResets(),n=this.groupLevel,o=r||n;if(o){this.nodesAfterFilter=this.allRowNodes;return}this.nodesAfterFilter=this.allRowNodes.filter(function(a){return e.filterManager.doesRowPassFilter({rowNode:a})})},t.prototype.clearDisplayIndexes=function(){var e=this;this.displayIndexStart=void 0,this.displayIndexEnd=void 0,this.allRowNodes.forEach(function(r){return e.blockUtils.clearDisplayIndex(r)})},t.prototype.getDisplayIndexEnd=function(){return this.displayIndexEnd},t.prototype.isDisplayIndexInStore=function(e){return this.getRowCount()===0?!1:e>=this.displayIndexStart&&e<this.displayIndexEnd},t.prototype.setDisplayIndexes=function(e,r){var n=this;this.displayIndexStart=e.peek(),this.topPx=r.value;var o={};this.nodesAfterSort.forEach(function(a){n.blockUtils.setDisplayIndex(a,e,r),o[a.id]=!0}),this.allRowNodes.forEach(function(a){o[a.id]||n.blockUtils.clearDisplayIndex(a)}),this.displayIndexEnd=e.peek(),this.heightPx=r.value-this.topPx},t.prototype.forEachNodeDeep=function(e,r){r===void 0&&(r=new Bt),this.allRowNodes.forEach(function(n){e(n,r.next());var o=n.childStore;o&&o.forEachNodeDeep(e,r)})},t.prototype.forEachNodeDeepAfterFilterAndSort=function(e,r){r===void 0&&(r=new Bt),this.nodesAfterSort.forEach(function(n){e(n,r.next());var o=n.childStore;o&&o.forEachNodeDeepAfterFilterAndSort(e,r)})},t.prototype.getRowUsingDisplayIndex=function(e){if(!!this.isDisplayIndexInStore(e)){var r=this.blockUtils.binarySearchForDisplayIndex(e,this.nodesAfterSort);return r}},t.prototype.getRowBounds=function(e){for(var r=0;r<this.nodesAfterSort.length;r++){var n=this.nodesAfterSort[r],o=this.blockUtils.extractRowBounds(n,e);if(o)return o}return null},t.prototype.isPixelInRange=function(e){return e>=this.topPx&&e<this.topPx+this.heightPx},t.prototype.getRowIndexAtPixel=function(e){var r=this,n=e<=this.topPx;if(n){var o=this.nodesAfterSort[0];return o.rowIndex}var a=e>=this.topPx+this.heightPx;if(a){var s=this.nodesAfterSort[this.nodesAfterSort.length-1],l=s.rowTop+s.rowHeight;if(e>=l&&s.expanded){if(s.childStore&&s.childStore.getRowCount()>0)return s.childStore.getRowIndexAtPixel(e);if(s.detailNode)return s.detailNode.rowIndex}return s.rowIndex}var u=null;this.nodesAfterSort.forEach(function(p){var h=r.blockUtils.getIndexAtPixel(p,e);h!=null&&(u=h)});var c=u==null;return c?this.displayIndexEnd-1:u},t.prototype.getChildStore=function(e){var r=this;return this.storeUtils.getChildStore(e,this,function(n){var o=r.allRowNodes.find(function(a){return a.key==n});return o})},t.prototype.forEachChildStoreShallow=function(e){this.allRowNodes.forEach(function(r){var n=r.childStore;n&&e(n)})},t.prototype.refreshAfterFilter=function(e){if(e.alwaysReset||this.gridOptionsWrapper.isTreeData()){this.refreshStore(!0);return}this.filterAndSortNodes(),this.forEachChildStoreShallow(function(r){return r.refreshAfterFilter(e)})},t.prototype.refreshAfterSort=function(e){if(e.alwaysReset){this.refreshStore(!0);return}this.sortRowNodes(),this.forEachChildStoreShallow(function(r){return r.refreshAfterSort(e)})},t.prototype.applyTransaction=function(e){switch(this.getState()){case jr.STATE_FAILED:return{status:hn.StoreLoadingFailed};case jr.STATE_LOADING:return{status:hn.StoreLoading};case jr.STATE_WAITING_TO_LOAD:return{status:hn.StoreWaitingToLoad}}var r=this.gridOptionsWrapper.getIsApplyServerSideTransactionFunc();if(r){var n={transaction:e,parentNode:this.parentRowNode,storeInfo:this.info},o=r(n);if(!o)return{status:hn.Cancelled}}var a={status:hn.Applied,remove:[],update:[],add:[]},s=[];return this.executeAdd(e,a),this.executeRemove(e,a,s),this.executeUpdate(e,a,s),this.filterAndSortNodes(),this.updateSelection(s),a},t.prototype.updateSelection=function(e){var r=e.length>0;if(r){e.forEach(function(o){o.setSelected(!1,!1,!0)});var n={type:x.EVENT_SELECTION_CHANGED,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi()};this.eventService.dispatchEvent(n)}},t.prototype.executeAdd=function(e,r){var n=this,o=e.add,a=e.addIndex;if(!_.missingOrEmpty(o)){var s=typeof a=="number"&&a>=0;s?o.reverse().forEach(function(l){var u=n.createDataNode(l,a);r.add.push(u)}):o.forEach(function(l){var u=n.createDataNode(l);r.add.push(u)})}},t.prototype.executeRemove=function(e,r,n){var o=this,a=e.remove;if(a!=null){var s={};a.forEach(function(l){var u=o.lookupRowNode(l);!u||(u.isSelected()&&n.push(u),u.clearRowTopAndRowIndex(),s[u.id]=!0,delete o.allNodesMap[u.id],r.remove.push(u),o.nodeManager.removeNode(u))}),this.allRowNodes=this.allRowNodes.filter(function(l){return!s[l.id]})}},t.prototype.executeUpdate=function(e,r,n){var o=this,a=e.update;a!=null&&a.forEach(function(s){var l=o.lookupRowNode(s);!l||(l.updateData(s),!l.selectable&&l.isSelected()&&n.push(l),r.update.push(l))})},t.prototype.lookupRowNode=function(e){var r=this.gridOptionsWrapper.getRowIdFunc(),n;if(r!=null){var o=this.level,a=this.parentRowNode.getGroupKeys(),s=r({data:e,parentKeys:a.length>0?a:void 0,level:o});if(n=this.allNodesMap[s],!n)return console.error("AG Grid: could not find row id="+s+", data item was not found for this id"),null}else if(n=this.allRowNodes.find(function(l){return l.data===e}),!n)return console.error("AG Grid: could not find data item as object was not found",e),null;return n},t.prototype.addStoreStates=function(e){e.push({type:"full",route:this.parentRowNode.getGroupKeys(),rowCount:this.allRowNodes.length,info:this.info}),this.forEachChildStoreShallow(function(r){return r.addStoreStates(e)})},t.prototype.refreshStore=function(e){if(e){var r=this.nodesAfterSort?this.nodesAfterSort.length:1;this.initialiseRowNodes(r)}this.scheduleLoad(),this.fireStoreUpdatedEvent()},t.prototype.retryLoads=function(){this.getState()===jr.STATE_FAILED&&(this.initialiseRowNodes(1),this.scheduleLoad()),this.forEachChildStoreShallow(function(e){return e.retryLoads()})},t.prototype.scheduleLoad=function(){this.setStateWaitingToLoad(),this.rowNodeBlockLoader.checkBlockToLoad()},t.prototype.fireStoreUpdatedEvent=function(){var e={type:x.EVENT_STORE_UPDATED};this.eventService.dispatchEvent(e)},t.prototype.getRowCount=function(){return this.nodesAfterSort.length},t.prototype.getTopLevelRowDisplayedIndex=function(e){var r=this.nodesAfterSort[e];return r.rowIndex},t.prototype.isLastRowIndexKnown=function(){return this.getState()==jr.STATE_LOADED},t.prototype.getRowNodesInRange=function(e,r){var n=[],o=!1;_.missing(e)&&(o=!0),this.nodesAfterSort.forEach(function(s){var l=s===e||s===r;(o||l)&&n.push(s),l&&(o=!o)});var a=o;return a?[]:n},en([y("ssrmStoreUtils")],t.prototype,"storeUtils",void 0),en([y("ssrmBlockUtils")],t.prototype,"blockUtils",void 0),en([y("columnModel")],t.prototype,"columnModel",void 0),en([y("rowNodeBlockLoader")],t.prototype,"rowNodeBlockLoader",void 0),en([y("rowNodeSorter")],t.prototype,"rowNodeSorter",void 0),en([y("sortController")],t.prototype,"sortController",void 0),en([y("ssrmNodeManager")],t.prototype,"nodeManager",void 0),en([y("filterManager")],t.prototype,"filterManager",void 0),en([y("ssrmTransactionManager")],t.prototype,"transactionManager",void 0),en([y("columnApi")],t.prototype,"columnApi",void 0),en([y("gridApi")],t.prototype,"gridApi",void 0),en([B],t.prototype,"postConstruct",null),en([nt],t.prototype,"destroyRowNodes",null),t}(jr),em=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Yq=function(){function i(){}return i.prototype.createStore=function(t,e){var r=this.getStoreParams(t,e),n=r.storeType==="partial"?zq:Kq;return new n(t,r,e)},i.prototype.getStoreParams=function(t,e){var r=this.getLevelSpecificParams(e),n=this.getStoreType(r),o=this.getBlockSize(n,r),a=this.getMaxBlocksInCache(n,t,r),s={storeType:n,cacheBlockSize:o,maxBlocksInCache:a};return s},i.prototype.getMaxBlocksInCache=function(t,e,r){if(t!="full"){var n=r&&r.maxBlocksInCache!=null?r.maxBlocksInCache:this.gridOptionsWrapper.getMaxBlocksInCache(),o=n!=null&&n>=0;if(!!o){if(e.dynamicRowHeight){var a="AG Grid: Server Side Row Model does not support Dynamic Row Height and Cache Purging. Either a) remove getRowHeight() callback or b) remove maxBlocksInCache property. Purging has been disabled.";_.doOnce(function(){return console.warn(a)},"storeFactory.maxBlocksInCache.dynamicRowHeight");return}if(this.columnModel.isAutoRowHeightActive()){var s="AG Grid: Server Side Row Model does not support Auto Row Height and Cache Purging. Either a) remove colDef.autoHeight or b) remove maxBlocksInCache property. Purging has been disabled.";_.doOnce(function(){return console.warn(s)},"storeFactory.maxBlocksInCache.autoRowHeightActive");return}return n}}},i.prototype.getBlockSize=function(t,e){if(t!="full"){var r=e&&e.cacheBlockSize!=null?e.cacheBlockSize:this.gridOptionsWrapper.getCacheBlockSize();return r!=null&&r>0?r:100}},i.prototype.getLevelSpecificParams=function(t){var e=this.gridOptionsWrapper.getServerSideStoreParamsFunc();if(!!e){var r={level:t.level+1,parentRowNode:t.level>=0?t:void 0,rowGroupColumns:this.columnModel.getRowGroupColumns(),pivotColumns:this.columnModel.getPivotColumns(),pivotMode:this.columnModel.isPivotMode()};return e(r)}},i.prototype.getStoreType=function(t){var e=t&&t.storeType!=null?t.storeType:this.gridOptionsWrapper.getServerSideStoreType();switch(e){case"partial":case"full":return e;case null:case void 0:return"full";default:var r=["full","partial"],n=r.join(", ");return console.warn("AG Grid: invalid Server Side Store Type "+e+", valid types are ["+n+"]"),"partial"}},em([y("gridOptionsWrapper")],i.prototype,"gridOptionsWrapper",void 0),em([y("columnModel")],i.prototype,"columnModel",void 0),i=em([Q("ssrmStoreFactory")],i),i}(),_0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},Xq=function(){function i(){}return i.prototype.isSortingWithValueColumn=function(t){for(var e=this.columnModel.getValueColumns().map(function(n){return n.getColId()}),r=0;r<t.length;r++)if(e.indexOf(t[r])>-1)return!0;return!1},i.prototype.isSortingWithSecondaryColumn=function(t){if(!this.columnModel.getSecondaryColumns())return!1;for(var e=this.columnModel.getSecondaryColumns().map(function(n){return n.getColId()}),r=0;r<t.length;r++)if(e.indexOf(t[r])>-1)return!0;return!1},_0([y("columnModel")],i.prototype,"columnModel",void 0),i=_0([Q("ssrmListenerUtils")],i),i}(),qq={moduleName:oe.ServerSideRowModelModule,rowModels:{serverSide:Eq},beans:[Nq,kq,Pq,Dq,Mq,Iq,Vq,Yq,Xq],dependantModules:[cr]},Zq="__<ag-grid-pseudo-null>__",Qq=function(){function i(t,e,r){this.rowModel=t,this.filterParams=e,this.caseFormat=r}return i.prototype.extractUniqueValues=function(t){var e=this,r={},n=this.filterParams.colDef.keyCreator,o=function(a){var s=a!=null?e.caseFormat(a):Zq;s&&r[s]==null&&(r[s]=a)};return this.rowModel.forEachLeafNode(function(a){if(!(!a.data||!t(a))){var s=e.filterParams,l=s.api,u=s.colDef,c=s.column,p=s.columnApi,h=s.context,d=e.filterParams.valueGetter({api:l,colDef:u,column:c,columnApi:p,context:h,data:a.data,getValue:function(g){return a.data[g]},node:a});if(n){var f={value:d,colDef:e.filterParams.colDef,column:e.filterParams.column,node:a,data:a.data,api:e.filterParams.api,columnApi:e.filterParams.columnApi,context:e.filterParams.context};d=n(f)}d=_.makeNull(d),d!=null&&Array.isArray(d)?d.forEach(function(g){var v=_.toStringOrNull(_.makeNull(g));o(v)}):o(_.toStringOrNull(d))}}),_.values(r)},i}(),an;(function(i){i[i.PROVIDED_LIST=0]="PROVIDED_LIST",i[i.PROVIDED_CALLBACK=1]="PROVIDED_CALLBACK",i[i.TAKEN_FROM_GRID_VALUES=2]="TAKEN_FROM_GRID_VALUES"})(an||(an={}));var Jq="__<ag-grid-pseudo-null>__",pD=function(){function i(t,e,r,n,o){var a=this;this.filterParams=t,this.setIsLoading=e,this.valueFormatterService=r,this.translate=n,this.caseFormat=o,this.localEventService=new oi,this.miniFilterText=null,this.providedValues=null,this.allValues=[],this.availableValues=new Set,this.displayedValues=[],this.selectedValues=new Set,this.initialised=!1;var s=t.column,l=t.colDef,u=t.textFormatter,c=t.doesRowPassOtherFilter,p=t.suppressSorting,h=t.comparator,d=t.rowModel,f=t.values;t.caseSensitive,this.column=s,this.formatter=u||Yc.DEFAULT_FORMATTER,this.doesRowPassOtherFilters=c,this.suppressSorting=p||!1,this.comparator=h||l.comparator||_.defaultComparator,d.getType()===L.ROW_MODEL_TYPE_CLIENT_SIDE&&(this.clientSideValuesExtractor=new Qq(d,this.filterParams,this.caseFormat)),f==null?this.valuesType=an.TAKEN_FROM_GRID_VALUES:(this.valuesType=Array.isArray(f)?an.PROVIDED_LIST:an.PROVIDED_CALLBACK,this.providedValues=f),this.updateAllValues().then(function(g){return a.resetSelectionState(g||[])})}return i.prototype.addEventListener=function(t,e,r){this.localEventService.addEventListener(t,e,r)},i.prototype.removeEventListener=function(t,e,r){this.localEventService.removeEventListener(t,e,r)},i.prototype.refreshValues=function(){var t=this.getModel();return this.updateAllValues(),this.setModel(t)},i.prototype.overrideValues=function(t){var e=this;return new ct(function(r){e.allValuesPromise.then(function(){e.valuesType=an.PROVIDED_LIST,e.providedValues=t,e.refreshValues().then(function(){return r()})})})},i.prototype.refreshAfterAnyFilterChanged=function(){var t=this;return this.showAvailableOnly()?this.allValuesPromise.then(function(e){return t.updateAvailableValues(e||[])}):ct.resolve()},i.prototype.isInitialised=function(){return this.initialised},i.prototype.updateAllValues=function(){var t=this;return this.allValuesPromise=new ct(function(e){switch(t.valuesType){case an.TAKEN_FROM_GRID_VALUES:case an.PROVIDED_LIST:{var r=t.valuesType===an.TAKEN_FROM_GRID_VALUES?t.getValuesFromRows(!1):_.values(t.uniqueValues(_.toStrings(t.providedValues)||[])),n=t.sortValues(r||[]);t.allValues=n,e(n);break}case an.PROVIDED_CALLBACK:{t.setIsLoading(!0);var o=t.providedValues,a=t.filterParams,s=a.columnApi,l=a.api,u=a.context,c=a.column,p=a.colDef,h={success:function(d){var f=_.values(t.uniqueValues(_.toStrings(d)||[]));t.setIsLoading(!1);var g=t.sortValues(f||[]);t.allValues=g,e(g)},colDef:p,column:c,columnApi:s,api:l,context:u};window.setTimeout(function(){return o(h)},0);break}default:throw new Error("Unrecognised valuesType")}}),this.allValuesPromise.then(function(e){return t.updateAvailableValues(e||[])}).then(function(){return t.initialised=!0}),this.allValuesPromise},i.prototype.setValuesType=function(t){this.valuesType=t},i.prototype.getValuesType=function(){return this.valuesType},i.prototype.isValueAvailable=function(t){return this.availableValues.has(t)},i.prototype.showAvailableOnly=function(){return this.valuesType===an.TAKEN_FROM_GRID_VALUES},i.prototype.updateAvailableValues=function(t){var e=this.showAvailableOnly()?this.sortValues(this.getValuesFromRows(!0)):t;this.availableValues=_.convertToSet(e),this.localEventService.dispatchEvent({type:i.EVENT_AVAILABLE_VALUES_CHANGED}),this.updateDisplayedValues()},i.prototype.sortValues=function(t){return this.suppressSorting?t:!this.filterParams.excelMode||t.indexOf(null)<0?t.sort(this.comparator):t.filter(function(e){return e!=null}).sort(this.comparator).concat(null)},i.prototype.getValuesFromRows=function(t){var e=this;if(t===void 0&&(t=!1),!this.clientSideValuesExtractor)return console.error("AG Grid: Set Filter cannot initialise because you are using a row model that does not contain all rows in the browser. Either use a different filter type, or configure Set Filter such that you provide it with values"),[];var r=function(n){return!t||e.doesRowPassOtherFilters(n)};return this.clientSideValuesExtractor.extractUniqueValues(r)},i.prototype.setMiniFilter=function(t){return t=_.makeNull(t),this.miniFilterText===t?!1:(this.miniFilterText=t,this.updateDisplayedValues(),!0)},i.prototype.getMiniFilter=function(){return this.miniFilterText},i.prototype.updateDisplayedValues=function(){var t=this;if(this.miniFilterText==null){this.displayedValues=_.values(this.availableValues);return}this.displayedValues=[];var e=this.caseFormat(this.formatter(this.miniFilterText)||""),r=function(n){return n!=null&&t.caseFormat(n).indexOf(e)>=0};this.availableValues.forEach(function(n){if(n==null)t.filterParams.excelMode&&r(t.translate("blanks"))&&t.displayedValues.push(n);else{var o=t.formatter(n),a=t.valueFormatterService.formatValue(t.column,null,o,t.filterParams.valueFormatter,!1);(r(o)||r(a))&&t.displayedValues.push(n)}})},i.prototype.getDisplayedValueCount=function(){return this.displayedValues.length},i.prototype.getDisplayedValue=function(t){return this.displayedValues[t]},i.prototype.hasSelections=function(){return this.filterParams.defaultToNothingSelected?this.selectedValues.size>0:this.allValues.length!==this.selectedValues.size},i.prototype.getValues=function(){return this.allValues.slice()},i.prototype.selectAllMatchingMiniFilter=function(t){var e=this;t===void 0&&(t=!1),this.miniFilterText==null?this.selectedValues=_.convertToSet(this.allValues):(t&&this.selectedValues.clear(),this.displayedValues.forEach(function(r){return e.selectedValues.add(r)}))},i.prototype.deselectAllMatchingMiniFilter=function(){var t=this;this.miniFilterText==null?this.selectedValues.clear():this.displayedValues.forEach(function(e){return t.selectedValues.delete(e)})},i.prototype.selectValue=function(t){this.selectedValues.add(t)},i.prototype.deselectValue=function(t){this.filterParams.excelMode&&this.isEverythingVisibleSelected()&&this.resetSelectionState(this.displayedValues),this.selectedValues.delete(t)},i.prototype.isValueSelected=function(t){return this.selectedValues.has(t)},i.prototype.isEverythingVisibleSelected=function(){var t=this;return this.displayedValues.filter(function(e){return t.isValueSelected(e)}).length===this.displayedValues.length},i.prototype.isNothingVisibleSelected=function(){var t=this;return this.displayedValues.filter(function(e){return t.isValueSelected(e)}).length===0},i.prototype.getModel=function(){return this.hasSelections()?_.values(this.selectedValues):null},i.prototype.setModel=function(t){var e=this;return this.allValuesPromise.then(function(r){if(t==null)e.resetSelectionState(r||[]);else{e.selectedValues.clear();var n=e.uniqueValues(r||[]);t.forEach(function(o){var a=n[e.uniqueKey(o)];a!==void 0&&e.selectedValues.add(a)})}})},i.prototype.uniqueValues=function(t){var e=this,r={};return(t||[]).forEach(function(n){var o=_.makeNull(n),a=e.uniqueKey(o);r[a]===void 0&&(r[a]=o)}),r},i.prototype.uniqueKey=function(t){return t==null?Jq:this.caseFormat(t)},i.prototype.resetSelectionState=function(t){this.filterParams.defaultToNothingSelected?this.selectedValues.clear():this.selectedValues=_.convertToSet(t||[])},i.EVENT_AVAILABLE_VALUES_CHANGED="availableValuesChanged",i}(),eZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),dh=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},fh=function(i){eZ(t,i);function t(e,r,n,o){var a=i.call(this,t.TEMPLATE)||this;return a.value=e,a.params=r,a.translate=n,a.isSelected=o,a}return t.prototype.init=function(){var e=this;this.render(),this.eCheckbox.setValue(this.isSelected,!0),this.eCheckbox.setDisabled(!!this.params.readOnly),!this.params.readOnly&&this.eCheckbox.onValueChange(function(r){var n=r||!1;e.isSelected=n;var o={type:t.EVENT_SELECTION_CHANGED,isSelected:n};e.dispatchEvent(o)})},t.prototype.toggleSelected=function(){this.params.readOnly||(this.isSelected=!this.isSelected,this.eCheckbox.setValue(this.isSelected))},t.prototype.render=function(){var e=this.params.column,r=this.value,n=null;if(typeof r=="function"?r=r():n=this.getFormattedValue(this.params,e,r),this.params.showTooltips){var o=n!=null?n:r;this.setTooltip(o)}var a={value:r,valueFormatted:n,api:this.gridOptionsWrapper.getApi(),columnApi:this.gridOptionsWrapper.getColumnApi(),context:this.gridOptionsWrapper.getContext(),colDef:this.params.colDef,column:this.params.column};this.renderCell(a)},t.prototype.getTooltipParams=function(){var e=i.prototype.getTooltipParams.call(this);return e.location="setFilterValue",e.colDef=this.getComponentHolder(),e},t.prototype.getFormattedValue=function(e,r,n){var o=e&&e.valueFormatter;return this.valueFormatterService.formatValue(r,null,n,o,!1)},t.prototype.renderCell=function(e){var r=this,n=this.userComponentFactory.getSetFilterCellRendererDetails(this.params,e),o=n?n.newAgStackInstance():void 0;if(o==null){var a=e.valueFormatted==null?e.value:e.valueFormatted;this.eCheckbox.setLabel(a==null?this.translate("blanks"):a);return}o.then(function(s){s&&(r.eCheckbox.setLabel(s.getGui()),r.addDestroyFunc(function(){return r.destroyBean(s)}))})},t.prototype.getComponentHolder=function(){return this.params.column.getColDef()},t.EVENT_SELECTION_CHANGED="selectionChanged",t.TEMPLATE=`
        <div class="ag-set-filter-item">
            <ag-checkbox ref="eCheckbox" class="ag-set-filter-item-checkbox"></ag-checkbox>
        </div>`,dh([y("valueFormatterService")],t.prototype,"valueFormatterService",void 0),dh([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),dh([I("eCheckbox")],t.prototype,"eCheckbox",void 0),dh([B],t.prototype,"init",null),t}(ee),hD={loadingOoo:"Loading...",blanks:"(Blanks)",searchOoo:"Search...",selectAll:"(Select All)",selectAllSearchResults:"(Select All Search Results)",noMatches:"No matches."},tZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Bu=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},hb=function(i){tZ(t,i);function t(){var e=i.call(this,"setFilter")||this;return e.valueModel=null,e.setFilterParams=null,e.virtualList=null,e.caseSensitive=!1,e.appliedModelValues=null,e}return t.prototype.postConstruct=function(){i.prototype.postConstruct.call(this),this.positionableFeature=new Qc(this.eSetFilterList,{forcePopupParentAsOffsetParent:!0}),this.createBean(this.positionableFeature)},t.prototype.updateUiVisibility=function(){},t.prototype.createBodyTemplate=function(){return`
            <div class="ag-set-filter">
                <div ref="eFilterLoading" class="ag-filter-loading ag-hidden">`+this.translateForSetFilter("loadingOoo")+`</div>
                <ag-input-text-field class="ag-mini-filter" ref="eMiniFilter"></ag-input-text-field>
                <div ref="eFilterNoMatches" class="ag-filter-no-matches ag-hidden">`+this.translateForSetFilter("noMatches")+`</div>
                <div ref="eSetFilterList" class="ag-set-filter-list" role="presentation"></div>
            </div>`},t.prototype.handleKeyDown=function(e){if(i.prototype.handleKeyDown.call(this,e),!e.defaultPrevented)switch(e.key){case j.SPACE:this.handleKeySpace(e);break;case j.ENTER:this.handleKeyEnter(e);break}},t.prototype.handleKeySpace=function(e){var r=this.gridOptionsWrapper.getDocument();if(!(!this.eSetFilterList.contains(r.activeElement)||!this.virtualList)){var n=this.virtualList.getLastFocusedRow();if(n!=null){var o=this.virtualList.getComponentAt(n);if(o!=null){e.preventDefault();var a=(this.setFilterParams||{}).readOnly;a||o.toggleSelected()}}}},t.prototype.handleKeyEnter=function(e){if(!!this.setFilterParams){var r=this.setFilterParams||{},n=r.excelMode,o=r.readOnly;!n||!!o||(e.preventDefault(),this.onBtApply(!1,!1,e),this.setFilterParams.excelMode==="mac"&&this.eMiniFilter.getInputElement().select())}},t.prototype.getCssIdentifier=function(){return"set-filter"},t.prototype.setModelAndRefresh=function(e){var r=this;return this.valueModel?this.valueModel.setModel(e).then(function(){return r.refresh()}):ct.resolve()},t.prototype.resetUiToDefaults=function(){return this.setMiniFilter(null),this.setModelAndRefresh(null)},t.prototype.setModelIntoUi=function(e){this.setMiniFilter(null);var r=e==null?null:e.values;return this.setModelAndRefresh(r)},t.prototype.getModelFromUi=function(){if(!this.valueModel)throw new Error("Value model has not been created.");var e=this.valueModel.getModel();return e?{values:e,filterType:this.getFilterType()}:null},t.prototype.getFilterType=function(){return"set"},t.prototype.getValueModel=function(){return this.valueModel},t.prototype.areModelsEqual=function(e,r){return e==null&&r==null?!0:e!=null&&r!=null&&_.areEqual(e.values,r.values)},t.prototype.setParams=function(e){var r=this;this.applyExcelModeOptions(e),i.prototype.setParams.call(this,e),this.setFilterParams=e,this.caseSensitive=e.caseSensitive||!1,this.valueModel=new pD(e,function(n){return r.showOrHideLoadingScreen(n)},this.valueFormatterService,function(n){return r.translateForSetFilter(n)},function(n){return r.caseFormat(n)}),this.initialiseFilterBodyUi(),e.rowModel.getType()===L.ROW_MODEL_TYPE_CLIENT_SIDE&&!e.values&&this.addEventListenersForDataChanges()},t.prototype.applyExcelModeOptions=function(e){e.excelMode==="windows"?(e.buttons||(e.buttons=["apply","cancel"]),e.closeOnApply==null&&(e.closeOnApply=!0)):e.excelMode==="mac"&&(e.buttons||(e.buttons=["reset"]),e.applyMiniFilterWhileTyping==null&&(e.applyMiniFilterWhileTyping=!0),e.debounceMs==null&&(e.debounceMs=500))},t.prototype.addEventListenersForDataChanges=function(){var e=this;this.addManagedListener(this.eventService,x.EVENT_ROW_DATA_UPDATED,function(){return e.syncAfterDataChange()}),this.addManagedListener(this.eventService,x.EVENT_CELL_VALUE_CHANGED,function(r){e.setFilterParams&&r.column===e.setFilterParams.column&&e.syncAfterDataChange()})},t.prototype.syncAfterDataChange=function(e){var r=this;if(e===void 0&&(e=!0),!this.valueModel)throw new Error("Value model has not been created.");var n=ct.resolve();return e&&(n=this.valueModel.refreshValues()),n.then(function(){r.refresh(),r.onBtApply(!1,!0)})},t.prototype.showOrHideLoadingScreen=function(e){_.setDisplayed(this.eFilterLoading,e)},t.prototype.initialiseFilterBodyUi=function(){this.initVirtualList(),this.initMiniFilter()},t.prototype.initVirtualList=function(){var e=this;if(!this.setFilterParams)throw new Error("Set filter params have not been provided.");if(!this.valueModel)throw new Error("Value model has not been created.");var r=this.gridOptionsWrapper.getLocaleTextFunc(),n=r("ariaFilterList","Filter List"),o=this.virtualList=this.createBean(new tp("filter","listbox",n)),a=this.getRefElement("eSetFilterList");a&&a.appendChild(o.getGui());var s=this.setFilterParams.cellHeight;s!=null&&o.setRowHeight(s),o.setComponentCreator(function(u){return e.createSetListItem(u)});var l;this.setFilterParams.suppressSelectAll?l=new rZ(this.valueModel):l=new nZ(this.valueModel,function(){return e.isSelectAllSelected()}),o.setModel(l)},t.prototype.getSelectAllLabel=function(){if(!this.setFilterParams)throw new Error("Set filter params have not been provided.");if(!this.valueModel)throw new Error("Value model has not been created.");var e=this.valueModel.getMiniFilter()==null||!this.setFilterParams.excelMode?"selectAll":"selectAllSearchResults";return this.translateForSetFilter(e)},t.prototype.createSetListItem=function(e){var r=this;if(!this.setFilterParams)throw new Error("Set filter params have not been provided.");if(!this.valueModel)throw new Error("Value model has not been created.");var n;return e===t.SELECT_ALL_VALUE?(n=this.createBean(new fh(function(){return r.getSelectAllLabel()},this.setFilterParams,function(o){return r.translateForSetFilter(o)},this.isSelectAllSelected())),n.addEventListener(fh.EVENT_SELECTION_CHANGED,function(o){return r.onSelectAll(o.isSelected)}),n):(n=this.createBean(new fh(e,this.setFilterParams,function(o){return r.translateForSetFilter(o)},this.valueModel.isValueSelected(e))),n.addEventListener(fh.EVENT_SELECTION_CHANGED,function(o){return r.onItemSelected(e,o.isSelected)}),n)},t.prototype.initMiniFilter=function(){var e=this;if(!this.setFilterParams)throw new Error("Set filter params have not been provided.");if(!this.valueModel)throw new Error("Value model has not been created.");var r=this,n=r.eMiniFilter,o=r.gridOptionsWrapper,a=o.getLocaleTextFunc();n.setDisplayed(!this.setFilterParams.suppressMiniFilter),n.setValue(this.valueModel.getMiniFilter()),n.onValueChange(function(){return e.onMiniFilterInput()}),n.setInputAriaLabel(a("ariaSearchFilterValues","Search filter values")),this.addManagedListener(n.getInputElement(),"keypress",function(s){return e.onMiniFilterKeyPress(s)})},t.prototype.afterGuiAttached=function(e){if(!this.setFilterParams)throw new Error("Set filter params have not been provided.");i.prototype.afterGuiAttached.call(this,e),this.setFilterParams.excelMode&&(this.resetUiToActiveModel(),this.showOrHideResults()),this.refreshVirtualList();var r=this.eMiniFilter;r.setInputPlaceholder(this.translateForSetFilter("searchOoo")),(!e||!e.suppressFocus)&&r.getFocusableElement().focus();var n=!!(e&&e.container==="floatingFilter"),o;this.gridOptionsWrapper.isEnableRtl()?o={bottom:!0,bottomLeft:!0,left:!0}:o={bottom:!0,bottomRight:!0,right:!0},n?(this.positionableFeature.restoreLastSize(),this.positionableFeature.setResizable(o)):(this.positionableFeature.removeSizeFromEl(),this.positionableFeature.setResizable(!1))},t.prototype.applyModel=function(){var e=this;if(!this.setFilterParams)throw new Error("Set filter params have not been provided.");if(!this.valueModel)throw new Error("Value model has not been created.");this.setFilterParams.excelMode&&this.valueModel.isEverythingVisibleSelected()&&this.valueModel.selectAllMatchingMiniFilter();var r=i.prototype.applyModel.call(this),n=this.getModel();return n?this.appliedModelValues=n.values.reduce(function(o,a){return o[e.caseFormat(String(a))]=!0,o},{}):this.appliedModelValues=null,r},t.prototype.isModelValid=function(e){return this.setFilterParams&&this.setFilterParams.excelMode?e==null||e.values.length>0:!0},t.prototype.doesFilterPass=function(e){var r=this;if(!this.setFilterParams||!this.valueModel||!this.appliedModelValues)return!0;var n=e.node,o=e.data,a=this.setFilterParams,s=a.valueGetter,l=a.colDef.keyCreator,u=a.api,c=a.colDef,p=a.column,h=a.columnApi,d=a.context,f=s({api:u,colDef:c,column:p,columnApi:h,context:d,data:o,getValue:function(v){return o[v]},node:n});if(l){var g={value:f,colDef:c,column:p,node:n,data:o,api:u,columnApi:h,context:d};f=l(g)}return f=_.makeNull(f),Array.isArray(f)?f.some(function(v){return r.appliedModelValues[r.caseFormat(String(_.makeNull(v)))]===!0}):this.appliedModelValues[this.caseFormat(String(f))]===!0},t.prototype.onNewRowsLoaded=function(){if(!this.valueModel)throw new Error("Value model has not been created.");var e=this.valueModel.getValuesType();this.syncAfterDataChange(e===an.TAKEN_FROM_GRID_VALUES)},t.prototype.setFilterValues=function(e){var r=this;if(!this.valueModel)throw new Error("Value model has not been created.");this.valueModel.overrideValues(e).then(function(){r.refresh(),r.onUiChanged()})},t.prototype.resetFilterValues=function(){if(!this.valueModel)throw new Error("Value model has not been created.");this.valueModel.setValuesType(an.TAKEN_FROM_GRID_VALUES),this.syncAfterDataChange(!0)},t.prototype.refreshFilterValues=function(){var e=this;if(!this.valueModel)throw new Error("Value model has not been created.");!this.valueModel.isInitialised()||this.valueModel.refreshValues().then(function(){e.refresh(),e.onUiChanged()})},t.prototype.onAnyFilterChanged=function(){var e=this;setTimeout(function(){if(!!e.isAlive()){if(!e.valueModel)throw new Error("Value model has not been created.");e.valueModel.refreshAfterAnyFilterChanged().then(function(){return e.refresh()})}},0)},t.prototype.onMiniFilterInput=function(){if(!this.setFilterParams)throw new Error("Set filter params have not been provided.");if(!this.valueModel)throw new Error("Value model has not been created.");if(!!this.valueModel.setMiniFilter(this.eMiniFilter.getValue())){var e=this.setFilterParams||{},r=e.applyMiniFilterWhileTyping,n=e.readOnly;!n&&r?this.filterOnAllVisibleValues(!1):this.updateUiAfterMiniFilterChange()}},t.prototype.updateUiAfterMiniFilterChange=function(){if(!this.setFilterParams)throw new Error("Set filter params have not been provided.");if(!this.valueModel)throw new Error("Value model has not been created.");var e=this.setFilterParams||{},r=e.excelMode,n=e.readOnly;r==null||!!n?this.refresh():this.valueModel.getMiniFilter()==null?this.resetUiToActiveModel():(this.valueModel.selectAllMatchingMiniFilter(!0),this.refresh(),this.onUiChanged()),this.showOrHideResults()},t.prototype.showOrHideResults=function(){if(!this.valueModel)throw new Error("Value model has not been created.");var e=this.valueModel.getMiniFilter()!=null&&this.valueModel.getDisplayedValueCount()<1;_.setDisplayed(this.eNoMatches,e),_.setDisplayed(this.eSetFilterList,!e)},t.prototype.resetUiToActiveModel=function(){var e=this;if(!this.valueModel)throw new Error("Value model has not been created.");this.eMiniFilter.setValue(null,!0),this.valueModel.setMiniFilter(null),this.setModelIntoUi(this.getModel()).then(function(){return e.onUiChanged(!1,"prevent")})},t.prototype.onMiniFilterKeyPress=function(e){var r=this.setFilterParams||{},n=r.excelMode,o=r.readOnly;e.key===j.ENTER&&!n&&!o&&this.filterOnAllVisibleValues()},t.prototype.filterOnAllVisibleValues=function(e){e===void 0&&(e=!0);var r=(this.setFilterParams||{}).readOnly;if(!this.valueModel)throw new Error("Value model has not been created.");if(r)throw new Error("Unable to filter in readOnly mode.");this.valueModel.selectAllMatchingMiniFilter(!0),this.refresh(),this.onUiChanged(!1,e?"immediately":"debounce"),this.showOrHideResults()},t.prototype.focusRowIfAlive=function(e){var r=this;e!=null&&window.setTimeout(function(){if(!r.virtualList)throw new Error("Virtual list has not been created.");r.isAlive()&&r.virtualList.focusRow(e)},0)},t.prototype.onSelectAll=function(e){if(!this.valueModel)throw new Error("Value model has not been created.");if(!this.virtualList)throw new Error("Virtual list has not been created.");e?this.valueModel.selectAllMatchingMiniFilter():this.valueModel.deselectAllMatchingMiniFilter();var r=this.virtualList.getLastFocusedRow();this.refresh(),this.onUiChanged(),this.focusRowIfAlive(r)},t.prototype.onItemSelected=function(e,r){if(!this.valueModel)throw new Error("Value model has not been created.");if(!this.virtualList)throw new Error("Virtual list has not been created.");r?this.valueModel.selectValue(e):this.valueModel.deselectValue(e);var n=this.virtualList.getLastFocusedRow();this.refresh(),this.onUiChanged(),this.focusRowIfAlive(n)},t.prototype.setMiniFilter=function(e){this.eMiniFilter.setValue(e),this.onMiniFilterInput()},t.prototype.getMiniFilter=function(){return this.valueModel?this.valueModel.getMiniFilter():null},t.prototype.refresh=function(){if(!this.virtualList)throw new Error("Virtual list has not been created.");this.virtualList.refresh()},t.prototype.getValues=function(){return this.valueModel?this.valueModel.getValues():[]},t.prototype.refreshVirtualList=function(){this.setFilterParams&&this.setFilterParams.refreshValuesOnOpen?this.refreshFilterValues():this.refresh()},t.prototype.translateForSetFilter=function(e){var r=this.gridOptionsWrapper.getLocaleTextFunc();return r(e,hD[e])},t.prototype.isSelectAllSelected=function(){if(!this.setFilterParams||!this.valueModel)return!1;if(this.setFilterParams.defaultToNothingSelected){if(this.valueModel.hasSelections()&&this.valueModel.isEverythingVisibleSelected())return!0;if(this.valueModel.isNothingVisibleSelected())return!1}else{if(this.valueModel.hasSelections()&&this.valueModel.isNothingVisibleSelected())return!1;if(this.valueModel.isEverythingVisibleSelected())return!0}},t.prototype.destroy=function(){this.virtualList!=null&&(this.virtualList.destroy(),this.virtualList=null),i.prototype.destroy.call(this)},t.prototype.caseFormat=function(e){return e==null||typeof e!="string"||this.caseSensitive?e:e.toUpperCase()},t.SELECT_ALL_VALUE="__AG_SELECT_ALL__",Bu([I("eMiniFilter")],t.prototype,"eMiniFilter",void 0),Bu([I("eFilterLoading")],t.prototype,"eFilterLoading",void 0),Bu([I("eSetFilterList")],t.prototype,"eSetFilterList",void 0),Bu([I("eFilterNoMatches")],t.prototype,"eNoMatches",void 0),Bu([y("valueFormatterService")],t.prototype,"valueFormatterService",void 0),t}(us),rZ=function(){function i(t){this.model=t}return i.prototype.getRowCount=function(){return this.model.getDisplayedValueCount()},i.prototype.getRow=function(t){return this.model.getDisplayedValue(t)},i.prototype.isRowSelected=function(t){return this.model.isValueSelected(this.getRow(t))},i}(),nZ=function(){function i(t,e){this.model=t,this.isSelectAllSelected=e}return i.prototype.getRowCount=function(){return this.model.getDisplayedValueCount()+1},i.prototype.getRow=function(t){return t===0?hb.SELECT_ALL_VALUE:this.model.getDisplayedValue(t-1)},i.prototype.isRowSelected=function(t){return t===0?this.isSelectAllSelected():this.model.isValueSelected(this.getRow(t))},i}(),iZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),tm=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},oZ=function(i){iZ(t,i);function t(){var e=i.call(this,`
            <div class="ag-floating-filter-input" role="presentation">
                <ag-input-text-field ref="eFloatingFilterText"></ag-input-text-field>
            </div>`)||this;return e.availableValuesListenerAdded=!1,e}return t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.init=function(e){var r=this.columnModel.getDisplayNameForColumn(e.column,"header",!0),n=this.gridOptionsWrapper.getLocaleTextFunc();this.eFloatingFilterText.setDisabled(!0).setInputAriaLabel(r+" "+n("ariaFilterInput","Filter Input")).addGuiEventListener("click",function(){return e.showParentFilter()}),this.params=e},t.prototype.onParentModelChanged=function(e){this.updateFloatingFilterText(e)},t.prototype.parentSetFilterInstance=function(e){this.params.parentFilterInstance(function(r){if(!(r instanceof hb))throw new Error("AG Grid - SetFloatingFilter expects SetFilter as it's parent");e(r)})},t.prototype.addAvailableValuesListener=function(){var e=this;this.parentSetFilterInstance(function(r){var n=r.getValueModel();!n||e.addManagedListener(n,pD.EVENT_AVAILABLE_VALUES_CHANGED,function(){return e.updateFloatingFilterText()})}),this.availableValuesListenerAdded=!0},t.prototype.updateFloatingFilterText=function(e){var r=this;this.availableValuesListenerAdded||this.addAvailableValuesListener(),this.parentSetFilterInstance(function(n){var o=(e||n.getModel()||{}).values,a=n.getValueModel();if(o==null||a==null){r.eFloatingFilterText.setValue("");return}var s=r.gridOptionsWrapper.getLocaleTextFunc(),l=o.filter(function(h){return a.isValueAvailable(h)}),u=l.map(function(h){var d=r.params,f=d.column,g=d.filterParams,v=r.valueFormatterService.formatValue(f,null,h,g.valueFormatter,!1),m=v!=null?v:h;return m==null?s("blanks",hD.blanks):m}),c=u.length>10?u.slice(0,10).concat("..."):u,p="("+u.length+") "+c.join(",");r.eFloatingFilterText.setValue(p)})},tm([I("eFloatingFilterText")],t.prototype,"eFloatingFilterText",void 0),tm([y("valueFormatterService")],t.prototype,"valueFormatterService",void 0),tm([y("columnModel")],t.prototype,"columnModel",void 0),t}(ee),aZ={moduleName:oe.SetFilterModule,beans:[],userComponents:[{componentName:"agSetColumnFilter",componentClass:hb},{componentName:"agSetColumnFloatingFilter",componentClass:oZ}],dependantModules:[cr]},sZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),lZ=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},uZ=function(i){sZ(t,i);function t(){var e=i.call(this)||this;return e.allComponents={},e}return t.prototype.registerStatusPanel=function(e,r){this.allComponents[e]=r},t.prototype.getStatusPanel=function(e){return this.allComponents[e]},t=lZ([Q("statusBarService")],t),t}(H),cZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Na=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},pZ=function(i){cZ(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.postConstruct=function(){if(this.gridOptions.statusBar&&this.gridOptions.statusBar.statusPanels){var e=this.gridOptions.statusBar.statusPanels.filter(function(o){return o.align==="left"});this.createAndRenderComponents(e,this.eStatusBarLeft);var r=this.gridOptions.statusBar.statusPanels.filter(function(o){return o.align==="center"});this.createAndRenderComponents(r,this.eStatusBarCenter);var n=this.gridOptions.statusBar.statusPanels.filter(function(o){return!o.align||o.align==="right"});this.createAndRenderComponents(n,this.eStatusBarRight)}else this.setDisplayed(!1)},t.prototype.createAndRenderComponents=function(e,r){var n=this,o=[];e.forEach(function(a){var s={},l=n.userComponentFactory.getStatusPanelCompDetails(a,s),u=l.newAgStackInstance();!u||o.push({key:a.key||a.statusPanel,promise:u})}),ct.all(o.map(function(a){return a.promise})).then(function(){o.forEach(function(a){a.promise.then(function(s){var l=function(){n.getContext().destroyBean(s)};n.isAlive()?(n.statusBarService.registerStatusPanel(a.key,s),r.appendChild(s.getGui()),n.addDestroyFunc(l)):l()})})})},t.TEMPLATE=`<div class="ag-status-bar">
            <div ref="eStatusBarLeft" class="ag-status-bar-left" role="status"></div>
            <div ref="eStatusBarCenter" class="ag-status-bar-center" role="status"></div>
            <div ref="eStatusBarRight" class="ag-status-bar-right" role="status"></div>
        </div>`,Na([y("gridOptions")],t.prototype,"gridOptions",void 0),Na([y("userComponentFactory")],t.prototype,"userComponentFactory",void 0),Na([y("statusBarService")],t.prototype,"statusBarService",void 0),Na([I("eStatusBarLeft")],t.prototype,"eStatusBarLeft",void 0),Na([I("eStatusBarCenter")],t.prototype,"eStatusBarCenter",void 0),Na([I("eStatusBarRight")],t.prototype,"eStatusBarRight",void 0),Na([B],t.prototype,"postConstruct",null),t}(ee),hZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),S0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},lp=function(i){hZ(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.setLabel=function(e,r){this.setDisplayed(!1);var n=this.gridOptionsWrapper.getLocaleTextFunc();this.eLabel.innerHTML=n(e,r)},t.prototype.setValue=function(e){this.eValue.innerHTML=e},t.TEMPLATE=`<div class="ag-status-name-value">
            <span ref="eLabel"></span>:&nbsp;
            <span ref="eValue" class="ag-status-name-value-value"></span>
        </div>`,S0([I("eLabel")],t.prototype,"eLabel",void 0),S0([I("eValue")],t.prototype,"eValue",void 0),t}(ee),dZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),x0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},fZ=function(i){dZ(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){if(this.gridApi.getModel().getType()!=="clientSide"){console.warn("AG Grid: agTotalAndFilteredRowCountComponent should only be used with the client side row model.");return}this.setLabel("totalAndFilteredRows","Rows"),this.addCssClass("ag-status-panel"),this.addCssClass("ag-status-panel-total-and-filtered-row-count"),this.setDisplayed(!0),this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,this.onDataChanged.bind(this)),this.onDataChanged()},t.prototype.onDataChanged=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("thousandSeparator",","),n=e("decimalSeparator","."),o=_.formatNumberCommas(this.getFilteredRowCountValue(),r,n),a=_.formatNumberCommas(this.getTotalRowCount(),r,n);if(o===a)this.setValue(o);else{var s=this.gridOptionsWrapper.getLocaleTextFunc();this.setValue(o+" "+s("of","of")+" "+a)}},t.prototype.getFilteredRowCountValue=function(){var e=0;return this.gridApi.forEachNodeAfterFilter(function(r){r.group||e++}),e},t.prototype.getTotalRowCount=function(){var e=0;return this.gridApi.forEachNode(function(r){r.group||e++}),e},t.prototype.init=function(){},t.prototype.destroy=function(){i.prototype.destroy.call(this)},x0([y("gridApi")],t.prototype,"gridApi",void 0),x0([B],t.prototype,"postConstruct",null),t}(lp),gZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),T0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},vZ=function(i){gZ(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){if(this.setLabel("filteredRows","Filtered"),this.gridApi.getModel().getType()!=="clientSide"){console.warn("AG Grid: agFilteredRowCountComponent should only be used with the client side row model.");return}this.addCssClass("ag-status-panel"),this.addCssClass("ag-status-panel-filtered-row-count"),this.setDisplayed(!0);var e=this.onDataChanged.bind(this);this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,e),e()},t.prototype.onDataChanged=function(){var e=this.getTotalRowCountValue(),r=this.getFilteredRowCountValue(),n=this.gridOptionsWrapper.getLocaleTextFunc(),o=n("thousandSeparator",","),a=n("decimalSeparator",".");this.setValue(_.formatNumberCommas(r,o,a)),this.setDisplayed(e!==r)},t.prototype.getTotalRowCountValue=function(){var e=0;return this.gridApi.forEachNode(function(r){return e+=1}),e},t.prototype.getFilteredRowCountValue=function(){var e=0;return this.gridApi.forEachNodeAfterFilter(function(r){r.group||(e+=1)}),e},t.prototype.init=function(){},t.prototype.destroy=function(){i.prototype.destroy.call(this)},T0([y("gridApi")],t.prototype,"gridApi",void 0),T0([B],t.prototype,"postConstruct",null),t}(lp),mZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),O0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},yZ=function(i){mZ(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){if(this.setLabel("totalRows","Total Rows"),this.gridApi.getModel().getType()!=="clientSide"){console.warn("AG Grid: agTotalRowCountComponent should only be used with the client side row model.");return}this.addCssClass("ag-status-panel"),this.addCssClass("ag-status-panel-total-row-count"),this.setDisplayed(!0),this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,this.onDataChanged.bind(this)),this.onDataChanged()},t.prototype.onDataChanged=function(){var e=this.gridOptionsWrapper.getLocaleTextFunc(),r=e("thousandSeparator",","),n=e("decimalSeparator",".");this.setValue(_.formatNumberCommas(this.getRowCountValue(),r,n))},t.prototype.getRowCountValue=function(){var e=0;return this.gridApi.forEachLeafNode(function(r){return e+=1}),e},t.prototype.init=function(){},t.prototype.destroy=function(){i.prototype.destroy.call(this)},O0([y("gridApi")],t.prototype,"gridApi",void 0),O0([B],t.prototype,"postConstruct",null),t}(lp),CZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),E0=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},wZ=function(i){CZ(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){if(!this.isValidRowModel()){console.warn("AG Grid: agSelectedRowCountComponent should only be used with the client and server side row model.");return}this.setLabel("selectedRows","Selected"),this.addCssClass("ag-status-panel"),this.addCssClass("ag-status-panel-selected-row-count");var e=this.gridApi.getSelectedRows().length,r=this.gridOptionsWrapper.getLocaleTextFunc(),n=r("thousandSeparator",","),o=r("decimalSeparator",".");this.setValue(_.formatNumberCommas(e,n,o)),this.setDisplayed(e>0);var a=this.onRowSelectionChanged.bind(this);this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,a),this.addManagedListener(this.eventService,x.EVENT_SELECTION_CHANGED,a)},t.prototype.isValidRowModel=function(){var e=this.gridApi.getModel().getType();return e==="clientSide"||e==="serverSide"},t.prototype.onRowSelectionChanged=function(){var e=this.gridApi.getSelectedRows().length,r=this.gridOptionsWrapper.getLocaleTextFunc(),n=r("thousandSeparator",","),o=r("decimalSeparator",".");this.setValue(_.formatNumberCommas(e,n,o)),this.setDisplayed(e>0)},t.prototype.init=function(){},t.prototype.destroy=function(){i.prototype.destroy.call(this)},E0([y("gridApi")],t.prototype,"gridApi",void 0),E0([B],t.prototype,"postConstruct",null),t}(lp),bZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Gr=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},_Z=function(i){bZ(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.destroy=function(){i.prototype.destroy.call(this)},t.prototype.postConstruct=function(){if(!this.isValidRowModel()){console.warn("AG Grid: agAggregationComponent should only be used with the client and server side row model.");return}this.avgAggregationComp.setLabel("avg","Average"),this.countAggregationComp.setLabel("count","Count"),this.minAggregationComp.setLabel("min","Min"),this.maxAggregationComp.setLabel("max","Max"),this.sumAggregationComp.setLabel("sum","Sum"),this.addManagedListener(this.eventService,x.EVENT_RANGE_SELECTION_CHANGED,this.onRangeSelectionChanged.bind(this)),this.addManagedListener(this.eventService,x.EVENT_MODEL_UPDATED,this.onRangeSelectionChanged.bind(this))},t.prototype.isValidRowModel=function(){var e=this.gridApi.getModel().getType();return e==="clientSide"||e==="serverSide"},t.prototype.init=function(){},t.prototype.setAggregationComponentValue=function(e,r,n){var o=this.getAggregationValueComponent(e);if(_.exists(o)&&o){var a=this.gridOptionsWrapper.getLocaleTextFunc(),s=a("thousandSeparator",","),l=a("decimalSeparator",".");o.setValue(_.formatNumberTwoDecimalPlacesAndCommas(r,s,l)),o.setDisplayed(n)}},t.prototype.getAggregationValueComponent=function(e){var r=e+"AggregationComp",n=null,o=_.exists(this.gridOptions.statusBar)&&this.gridOptions.statusBar?this.gridOptions.statusBar.statusPanels.find(function(a){return a.statusPanel==="agAggregationComponent"}):null;return _.exists(o)&&o?(!_.exists(o.statusPanelParams)||_.exists(o.statusPanelParams)&&_.exists(o.statusPanelParams.aggFuncs)&&_.exists(o.statusPanelParams.aggFuncs.find(function(a){return a===e})))&&(n=this[r]):n=this[r],n},t.prototype.onRangeSelectionChanged=function(){var e=this,r=this.rangeService?this.rangeService.getCellRanges():void 0,n=0,o=0,a=0,s=null,l=null,u={};r&&!_.missingOrEmpty(r)&&r.forEach(function(h){for(var d=e.rangeService.getRangeStartRow(h),f=e.rangeService.getRangeEndRow(h);;){var g=_.missing(d)||!d||e.rowPositionUtils.before(f,d);if(g||!d||!h.columns)break;h.columns.forEach(function(v){if(d!==null){var m=e.cellPositionUtils.createId({rowPinned:d.rowPinned,column:v,rowIndex:d.rowIndex});if(!u[m]){u[m]=!0;var C=e.rowRenderer.getRowNode(d);if(!_.missing(C)){var w=e.valueService.getValue(v,C);_.missing(w)||w===""||(w.value&&(w=w.value),typeof w=="string"&&(w=Number(w)),typeof w=="number"&&!isNaN(w)&&(n+=w,(l===null||w>l)&&(l=w),(s===null||w<s)&&(s=w),a++),o++)}}}}),d=e.cellNavigationService.getRowBelow(d)}});var c=o>1,p=a>1;this.setAggregationComponentValue("count",o,c),this.setAggregationComponentValue("sum",n,p),this.setAggregationComponentValue("min",s,p),this.setAggregationComponentValue("max",l,p),this.setAggregationComponentValue("avg",n/a,p)},t.TEMPLATE=`<div class="ag-status-panel ag-status-panel-aggregations">
            <ag-name-value ref="avgAggregationComp"></ag-name-value>
            <ag-name-value ref="countAggregationComp"></ag-name-value>
            <ag-name-value ref="minAggregationComp"></ag-name-value>
            <ag-name-value ref="maxAggregationComp"></ag-name-value>
            <ag-name-value ref="sumAggregationComp"></ag-name-value>
        </div>`,Gr([De("rangeService")],t.prototype,"rangeService",void 0),Gr([y("valueService")],t.prototype,"valueService",void 0),Gr([y("cellNavigationService")],t.prototype,"cellNavigationService",void 0),Gr([y("rowRenderer")],t.prototype,"rowRenderer",void 0),Gr([y("gridOptions")],t.prototype,"gridOptions",void 0),Gr([y("gridApi")],t.prototype,"gridApi",void 0),Gr([y("cellPositionUtils")],t.prototype,"cellPositionUtils",void 0),Gr([y("rowPositionUtils")],t.prototype,"rowPositionUtils",void 0),Gr([I("sumAggregationComp")],t.prototype,"sumAggregationComp",void 0),Gr([I("countAggregationComp")],t.prototype,"countAggregationComp",void 0),Gr([I("minAggregationComp")],t.prototype,"minAggregationComp",void 0),Gr([I("maxAggregationComp")],t.prototype,"maxAggregationComp",void 0),Gr([I("avgAggregationComp")],t.prototype,"avgAggregationComp",void 0),Gr([B],t.prototype,"postConstruct",null),t}(ee),SZ={moduleName:oe.StatusBarModule,beans:[uZ],agStackComponents:[{componentName:"AgStatusBar",componentClass:pZ},{componentName:"AgNameValue",componentClass:lp}],userComponents:[{componentName:"agAggregationComponent",componentClass:_Z},{componentName:"agSelectedRowCountComponent",componentClass:wZ},{componentName:"agTotalRowCountComponent",componentClass:yZ},{componentName:"agFilteredRowCountComponent",componentClass:vZ},{componentName:"agTotalAndFilteredRowCountComponent",componentClass:fZ}],dependantModules:[cr]},xZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Ga=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},TZ=function(i){xZ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.firstRow=-1,e.lastRow=-1,e.rowCount=-1,e.rowNodesByIndex={},e}return t.prototype.ensureRowHeightsValid=function(e,r,n,o){return!1},t.prototype.init=function(){this.rowHeight=this.gridOptionsWrapper.getRowHeightAsNumber(),this.addManagedListener(this.eventService,x.EVENT_VIEWPORT_CHANGED,this.onViewportChanged.bind(this))},t.prototype.start=function(){this.gridOptionsWrapper.getViewportDatasource()&&this.setViewportDatasource(this.gridOptionsWrapper.getViewportDatasource())},t.prototype.isLastRowIndexKnown=function(){return!0},t.prototype.destroyDatasource=function(){!this.viewportDatasource||(this.viewportDatasource.destroy&&this.viewportDatasource.destroy(),this.rowRenderer.datasourceChanged(),this.firstRow=-1,this.lastRow=-1)},t.prototype.calculateFirstRow=function(e){var r=this.gridOptionsWrapper.getViewportRowModelBufferSize(),n=this.gridOptionsWrapper.getViewportRowModelPageSize(),o=e-r;return o<0?0:Math.floor(o/n)*n},t.prototype.calculateLastRow=function(e){if(e===-1)return e;var r=this.gridOptionsWrapper.getViewportRowModelBufferSize(),n=this.gridOptionsWrapper.getViewportRowModelPageSize(),o=e+r,a=Math.ceil(o/n)*n,s=this.rowCount-1;return Math.min(a,s)},t.prototype.onViewportChanged=function(e){var r=this.calculateFirstRow(e.firstRow),n=this.calculateLastRow(e.lastRow);(this.firstRow!==r||this.lastRow!==n)&&(this.firstRow=r,this.lastRow=n,this.purgeRowsNotInViewport(),this.viewportDatasource&&this.viewportDatasource.setViewportRange(this.firstRow,this.lastRow))},t.prototype.purgeRowsNotInViewport=function(){var e=this;Object.keys(this.rowNodesByIndex).forEach(function(r){var n=parseInt(r,10);(n<e.firstRow||n>e.lastRow)&&delete e.rowNodesByIndex[n]})},t.prototype.setViewportDatasource=function(e){this.destroyDatasource(),this.viewportDatasource=e,this.rowCount=0,e.init?e.init({setRowCount:this.setRowCount.bind(this),setRowData:this.setRowData.bind(this),getRow:this.getRow.bind(this)}):console.warn("AG Grid: viewport is missing init method.")},t.prototype.getType=function(){return L.ROW_MODEL_TYPE_VIEWPORT},t.prototype.getRow=function(e){return this.rowNodesByIndex[e]||(this.rowNodesByIndex[e]=this.createBlankRowNode(e)),this.rowNodesByIndex[e]},t.prototype.getRowNode=function(e){var r;return this.forEachNode(function(n){n.id===e&&(r=n)}),r},t.prototype.getRowCount=function(){return this.rowCount},t.prototype.getRowIndexAtPixel=function(e){return this.rowHeight!==0?Math.floor(e/this.rowHeight):0},t.prototype.getRowBounds=function(e){return{rowHeight:this.rowHeight,rowTop:this.rowHeight*e}},t.prototype.getTopLevelRowCount=function(){return this.getRowCount()},t.prototype.getTopLevelRowDisplayedIndex=function(e){return e},t.prototype.isEmpty=function(){return this.rowCount>0},t.prototype.isRowsToRender=function(){return this.rowCount>0},t.prototype.getNodesInRangeForSelection=function(e,r){var n=_.missing(e)?0:e.rowIndex,o=r.rowIndex,a=n<this.firstRow||n>this.lastRow,s=o<this.firstRow||o>this.lastRow;if(a||s)return[];for(var l=[],u=n<=o?n:o,c=n<=o?o:n,p=u;p<=c;p++)l.push(this.rowNodesByIndex[p]);return l},t.prototype.forEachNode=function(e){var r=this,n=0;Object.keys(this.rowNodesByIndex).forEach(function(o){var a=parseInt(o,10),s=r.rowNodesByIndex[a];e(s,n),n++})},t.prototype.setRowData=function(e){var r=this;_.iterateObject(e,function(n,o){var a=parseInt(n,10);if(a>=r.firstRow&&a<=r.lastRow){var s=r.rowNodesByIndex[a];_.missing(s)&&(s=r.createBlankRowNode(a),r.rowNodesByIndex[a]=s),s.setDataAndId(o,a.toString())}})},t.prototype.createBlankRowNode=function(e){var r=new Oe(this.beans);return r.setRowHeight(this.rowHeight),r.setRowTop(this.rowHeight*e),r.setRowIndex(e),r},t.prototype.setRowCount=function(e,r){if(r===void 0&&(r=!1),e!==this.rowCount){this.rowCount=e;var n={type:x.EVENT_MODEL_UPDATED,api:this.gridApi,columnApi:this.columnApi,newData:!1,newPage:!1,keepRenderedRows:r,animate:!1};this.eventService.dispatchEvent(n)}},t.prototype.isRowPresent=function(e){return!1},Ga([y("gridApi")],t.prototype,"gridApi",void 0),Ga([y("columnApi")],t.prototype,"columnApi",void 0),Ga([y("rowRenderer")],t.prototype,"rowRenderer",void 0),Ga([y("beans")],t.prototype,"beans",void 0),Ga([B],t.prototype,"init",null),Ga([nt],t.prototype,"destroyDatasource",null),t=Ga([Q("rowModel")],t),t}(H),OZ={moduleName:oe.ViewportRowModelModule,rowModels:{viewport:TZ},dependantModules:[cr]},gs=function(){function i(t,e,r,n){this.x=t,this.y=e,this.width=r,this.height=n}return i.prototype.isValid=function(){return isFinite(this.x)&&isFinite(this.y)&&isFinite(this.width)&&isFinite(this.height)},i.prototype.dilate=function(t){this.x-=t,this.y-=t,this.width+=t*2,this.height+=t*2},i.prototype.containsPoint=function(t,e){return t>=this.x&&t<=this.x+this.width&&e>=this.y&&e<=this.y+this.height},i.prototype.render=function(t,e){e===void 0&&(e=i.noParams),t.save(),e.resetTransform&&t.setTransform(1,0,0,1,0,0),t.strokeStyle=e.strokeStyle||"cyan",t.lineWidth=e.lineWidth||1,t.strokeRect(this.x,this.y,this.width,this.height),e.label&&(t.fillStyle=e.fillStyle||"black",t.textBaseline="bottom",t.fillText(e.label,this.x,this.y)),t.restore()},i.noParams={},i}(),Hu=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},ec=function(){function i(t){t===void 0&&(t=[1,0,0,1,0,0]),this.elements=t}return i.prototype.setElements=function(t){var e=this.elements;return e[0]=t[0],e[1]=t[1],e[2]=t[2],e[3]=t[3],e[4]=t[4],e[5]=t[5],this},i.prototype.setIdentityElements=function(){var t=this.elements;return t[0]=1,t[1]=0,t[2]=0,t[3]=1,t[4]=0,t[5]=0,this},Object.defineProperty(i.prototype,"identity",{get:function(){var t=this.elements;return t[0]===1&&t[1]===0&&t[2]===0&&t[3]===1&&t[4]===0&&t[5]===0},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"a",{get:function(){return this.elements[0]},set:function(t){this.elements[0]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"b",{get:function(){return this.elements[1]},set:function(t){this.elements[1]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"c",{get:function(){return this.elements[2]},set:function(t){this.elements[2]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"d",{get:function(){return this.elements[3]},set:function(t){this.elements[3]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"e",{get:function(){return this.elements[4]},set:function(t){this.elements[4]=t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"f",{get:function(){return this.elements[5]},set:function(t){this.elements[5]=t},enumerable:!0,configurable:!0}),i.prototype.AxB=function(t,e,r){var n=Hu(t,6),o=n[0],a=n[1],s=n[2],l=n[3],u=n[4],c=n[5],p=Hu(e,6),h=p[0],d=p[1],f=p[2],g=p[3],v=p[4],m=p[5];r=r||t,r[0]=o*h+s*d,r[1]=a*h+l*d,r[2]=o*f+s*g,r[3]=a*f+l*g,r[4]=o*v+s*m+u,r[5]=a*v+l*m+c},i.prototype.multiplySelf=function(t){return this.AxB(this.elements,t.elements),this},i.prototype.multiply=function(t){var e=new Array(6);return this.AxB(this.elements,t.elements,e),new i(e)},i.prototype.preMultiplySelf=function(t){return this.AxB(t.elements,this.elements,this.elements),this},i.prototype.inverse=function(){var t=Hu(this.elements,6),e=t[0],r=t[1],n=t[2],o=t[3],a=t[4],s=t[5],l=1/(e*o-r*n);return e*=l,r*=l,n*=l,o*=l,new i([o,-r,-n,e,n*s-o*a,r*a-e*s])},i.prototype.inverseTo=function(t){var e=Hu(this.elements,6),r=e[0],n=e[1],o=e[2],a=e[3],s=e[4],l=e[5],u=1/(r*a-n*o);return r*=u,n*=u,o*=u,a*=u,t.setElements([a,-n,-o,r,o*l-a*s,n*s-r*l]),this},i.prototype.invertSelf=function(){var t=this.elements,e=Hu(t,6),r=e[0],n=e[1],o=e[2],a=e[3],s=e[4],l=e[5],u=1/(r*a-n*o);return r*=u,n*=u,o*=u,a*=u,t[0]=a,t[1]=-n,t[2]=-o,t[3]=r,t[4]=o*l-a*s,t[5]=n*s-r*l,this},i.prototype.clone=function(){return new i(this.elements.slice())},i.prototype.transformPoint=function(t,e){var r=this.elements;return{x:t*r[0]+e*r[2]+r[4],y:t*r[1]+e*r[3]+r[5]}},i.prototype.transformBBox=function(t,e,r){e===void 0&&(e=0);var n=this.elements,o=n[0],a=n[1],s=n[2],l=n[3],u=t.width*.5,c=t.height*.5,p=t.x+u,h=t.y+c,d,f;if(e){u-=e,c-=e;var g=Math.sqrt(o*o+s*s),v=Math.sqrt(a*a+l*l);d=Math.abs(u*o)+Math.abs(c*s)+Math.abs(g*e),f=Math.abs(u*a)+Math.abs(c*l)+Math.abs(v*e)}else d=Math.abs(u*o)+Math.abs(c*s),f=Math.abs(u*a)+Math.abs(c*l);return r||(r=new gs(0,0,0,0)),r.x=p*o+h*s+n[4]-d,r.y=p*a+h*l+n[5]-f,r.width=d+d,r.height=f+f,r},i.prototype.toContext=function(t){if(!this.identity){var e=this.elements;t.transform(e[0],e[1],e[2],e[3],e[4],e[5])}},i.flyweight=function(t){return t?t instanceof i?i.matrix.setElements(t.elements):i.matrix.setElements(t):i.matrix.setIdentityElements(),i.matrix},i.matrix=new i,i}();function db(i){var t=i.constructor,e=t.className;if(!e)throw new Error("The "+t+" is missing the 'className' property.");return e+"-"+(t.id=(t.id||0)+1)}var EZ=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Wc;(function(i){i[i.All=0]="All",i[i.None=1]="None"})(Wc||(Wc={}));var Ho=function(){function i(){this.id=db(this),this.tag=NaN,this.isContainerNode=!1,this._children=[],this.childSet={},this.matrix=new ec,this.inverseMatrix=new ec,this._dirtyTransform=!1,this._scalingX=1,this._scalingY=1,this._scalingCenterX=null,this._scalingCenterY=null,this._rotationCenterX=null,this._rotationCenterY=null,this._rotation=0,this._translationX=0,this._translationY=0,this._dirty=!0,this._visible=!0,this.pointerEvents=Wc.All}return i.isNode=function(t){return t?t.matrix!==void 0:!1},i.prototype._setScene=function(t){this._scene=t;for(var e=this.children,r=e.length,n=0;n<r;n++)e[n]._setScene(t)},Object.defineProperty(i.prototype,"scene",{get:function(){return this._scene},enumerable:!0,configurable:!0}),i.prototype._setParent=function(t){this._parent=t},Object.defineProperty(i.prototype,"parent",{get:function(){return this._parent},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"children",{get:function(){return this._children},enumerable:!0,configurable:!0}),i.prototype.countChildren=function(t){if(t===void 0&&(t=i.MAX_SAFE_INTEGER),t<=0)return 0;for(var e=this.children,r=e.length,n=r,o=0;o<r;o++)n+=e[o].countChildren(t-1);return n},i.prototype.append=function(t){i.isNode(t)&&(t=[t]);for(var e=t.length,r=0;r<e;r++){var n=t[r];if(n.parent)throw new Error(n+" already belongs to another parent: "+n.parent+".");if(n.scene)throw new Error(n+" already belongs a scene: "+n.scene+".");if(this.childSet[n.id])throw new Error("Duplicate "+n.constructor.name+" node: "+n);this._children.push(n),this.childSet[n.id]=!0,n._setParent(this),n._setScene(this.scene)}this.dirty=!0},i.prototype.appendChild=function(t){if(t.parent)throw new Error(t+" already belongs to another parent: "+t.parent+".");if(t.scene)throw new Error(t+" already belongs to a scene: "+t.scene+".");if(this.childSet[t.id])throw new Error("Duplicate "+t.constructor.name+" node: "+t);return this._children.push(t),this.childSet[t.id]=!0,t._setParent(this),t._setScene(this.scene),this.dirty=!0,t},i.prototype.removeChild=function(t){if(t.parent===this){var e=this.children.indexOf(t);if(e>=0)return this._children.splice(e,1),delete this.childSet[t.id],t._setParent(),t._setScene(),this.dirty=!0,t}throw new Error("The node to be removed is not a child of this node.")},i.prototype.insertBefore=function(t,e){var r=t.parent;if(t.parent&&t.parent.removeChild(t),e&&e.parent===this){var n=this.children.indexOf(e);if(n>=0)this._children.splice(n,0,t),this.childSet[t.id]=!0,t._setParent(this),t._setScene(this.scene);else throw new Error(e+" has "+r+" as the parent, but is not in its list of children.");this.dirty=!0}else this.append(t);return t},Object.defineProperty(i.prototype,"nextSibling",{get:function(){var t=this.parent;if(t){var e=t.children,r=e.indexOf(this);if(r>=0&&r<=e.length-1)return e[r+1]}},enumerable:!0,configurable:!0}),i.prototype.transformPoint=function(t,e){for(var r=ec.flyweight(this.matrix),n=this.parent;n;)r.preMultiplySelf(n.matrix),n=n.parent;return r.invertSelf().transformPoint(t,e)},i.prototype.inverseTransformPoint=function(t,e){for(var r=ec.flyweight(this.matrix),n=this.parent;n;)r.preMultiplySelf(n.matrix),n=n.parent;return r.transformPoint(t,e)},Object.defineProperty(i.prototype,"dirtyTransform",{get:function(){return this._dirtyTransform},set:function(t){this._dirtyTransform=t,t&&(this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"scalingX",{get:function(){return this._scalingX},set:function(t){this._scalingX!==t&&(this._scalingX=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"scalingY",{get:function(){return this._scalingY},set:function(t){this._scalingY!==t&&(this._scalingY=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"scalingCenterX",{get:function(){return this._scalingCenterX},set:function(t){this._scalingCenterX!==t&&(this._scalingCenterX=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"scalingCenterY",{get:function(){return this._scalingCenterY},set:function(t){this._scalingCenterY!==t&&(this._scalingCenterY=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"rotationCenterX",{get:function(){return this._rotationCenterX},set:function(t){this._rotationCenterX!==t&&(this._rotationCenterX=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"rotationCenterY",{get:function(){return this._rotationCenterY},set:function(t){this._rotationCenterY!==t&&(this._rotationCenterY=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"rotation",{get:function(){return this._rotation},set:function(t){this._rotation!==t&&(this._rotation=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"rotationDeg",{get:function(){return this.rotation/Math.PI*180},set:function(t){this.rotation=t/180*Math.PI},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"translationX",{get:function(){return this._translationX},set:function(t){this._translationX!==t&&(this._translationX=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"translationY",{get:function(){return this._translationY},set:function(t){this._translationY!==t&&(this._translationY=t,this.dirtyTransform=!0)},enumerable:!0,configurable:!0}),i.prototype.containsPoint=function(t,e){return!1},i.prototype.pickNode=function(t,e){if(!(!this.visible||this.pointerEvents===Wc.None||!this.containsPoint(t,e))){var r=this.children;if(r.length)for(var n=r.length-1;n>=0;n--){var o=r[n].pickNode(t,e);if(o)return o}else if(!this.isContainerNode)return this}},i.prototype.computeBBox=function(){},i.prototype.computeBBoxCenter=function(){var t=this.computeBBox&&this.computeBBox();return t?[t.x+t.width*.5,t.y+t.height*.5]:[0,0]},i.prototype.computeTransformMatrix=function(){var t=EZ([0,0],2),e=t[0],r=t[1],n=this.scalingX,o=this.scalingY,a,s;n===1&&o===1?(a=0,s=0):(a=this.scalingCenterX===null?e:this.scalingCenterX,s=this.scalingCenterY===null?r:this.scalingCenterY);var l=this.rotation,u=Math.cos(l),c=Math.sin(l),p,h;l===0?(p=0,h=0):(p=this.rotationCenterX===null?e:this.rotationCenterX,h=this.rotationCenterY===null?r:this.rotationCenterY);var d=this.translationX,f=this.translationY,g=a*(1-n)-p,v=s*(1-o)-h;this.dirtyTransform=!1,this.matrix.setElements([u*n,c*n,-c*o,u*o,u*g-c*v+p+d,c*g+u*v+h+f]).inverseTo(this.inverseMatrix)},Object.defineProperty(i.prototype,"dirty",{get:function(){return this._dirty},set:function(t){this._dirty=t,t&&(this.parent?this.parent.dirty=!0:this.scene&&(this.scene.dirty=!0))},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"visible",{get:function(){return this._visible},set:function(t){this._visible!==t&&(this._visible=t,this.dirty=!0)},enumerable:!0,configurable:!0}),i.MAX_SAFE_INTEGER=Math.pow(2,53)-1,i}(),RZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),oa=function(i){RZ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.isContainerNode=!0,e._opacity=1,e}return Object.defineProperty(t.prototype,"opacity",{get:function(){return this._opacity},set:function(e){e=Math.min(1,Math.max(0,e)),this._opacity!==e&&(this._opacity=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.containsPoint=function(e,r){return!0},t.prototype.computeBBox=function(){var e=1/0,r=-1/0,n=1/0,o=-1/0;return this.dirtyTransform&&this.computeTransformMatrix(),this.children.forEach(function(a){if(!!a.visible){var s=a.computeBBox();if(!!s){if(!(a instanceof t)){a.dirtyTransform&&a.computeTransformMatrix();for(var l=ec.flyweight(a.matrix),u=a.parent;u;)l.preMultiplySelf(u.matrix),u=u.parent;l.transformBBox(s,0,s)}var c=s.x,p=s.y;c<e&&(e=c),p<n&&(n=p),c+s.width>r&&(r=c+s.width),p+s.height>o&&(o=p+s.height)}}}),new gs(e,n,r-e,o-n)},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this.children,n=r.length;e.globalAlpha*=this.opacity;for(var o=0;o<n;o++){var a=r[o];a.visible&&(e.save(),a.render(e),e.restore())}},t.className="Group",t}(Ho);function Df(i,t){var e=Object.create(i);for(var r in t)t.hasOwnProperty(r)&&(e[r]=t[r]);return e}var PZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),go=function(i){PZ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.lastInstanceId=0,e._fillOpacity=1,e._strokeOpacity=1,e._fill=t.defaultStyles.fill,e._stroke=t.defaultStyles.stroke,e._strokeWidth=t.defaultStyles.strokeWidth,e._lineDash=t.defaultStyles.lineDash,e._lineDashOffset=t.defaultStyles.lineDashOffset,e._lineCap=t.defaultStyles.lineCap,e._lineJoin=t.defaultStyles.lineJoin,e._opacity=t.defaultStyles.opacity,e.onShadowChange=function(){e.dirty=!0},e._fillShadow=t.defaultStyles.fillShadow,e._strokeShadow=t.defaultStyles.strokeShadow,e}return t.createInstance=function(e){var r=Object.create(e);return r._setParent(void 0),r.id=e.id+"-Instance-"+String(++e.lastInstanceId),r},t.prototype.restoreOwnStyles=function(){for(var e=this.constructor.defaultStyles,r=Object.getOwnPropertyNames(e),n=0,o=r.length;n<o;n++){var a=r[n];this[a]=e[a]}},t.prototype.restoreAllStyles=function(){var e=this.constructor.defaultStyles;for(var r in e)this[r]=e[r]},t.prototype.restoreOverriddenStyles=function(){var e=this.constructor.defaultStyles,r=Object.getPrototypeOf(e);for(var n in e)e.hasOwnProperty(n)&&r.hasOwnProperty(n)&&(this[n]=e[n])},Object.defineProperty(t.prototype,"fillOpacity",{get:function(){return this._fillOpacity},set:function(e){this._fillOpacity!==e&&(this._fillOpacity=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeOpacity",{get:function(){return this._strokeOpacity},set:function(e){this._strokeOpacity!==e&&(this._strokeOpacity=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fill",{get:function(){return this._fill},set:function(e){this._fill!==e&&(this._fill=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"stroke",{get:function(){return this._stroke},set:function(e){this._stroke!==e&&(this._stroke=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeWidth",{get:function(){return this._strokeWidth},set:function(e){this._strokeWidth!==e&&(this._strokeWidth=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"alignment",{get:function(){return Math.floor(this.strokeWidth)%2/2},enumerable:!0,configurable:!0}),t.prototype.align=function(e,r,n){return n!=null?Math.floor(n)+Math.floor(r%1+n%1):Math.floor(r)+e},Object.defineProperty(t.prototype,"lineDash",{get:function(){return this._lineDash},set:function(e){var r=this._lineDash;if(r!==e){if(r&&e&&r.length===e.length){for(var n=!0,o=e.length,a=0;a<o;a++)if(r[a]!==e[a]){n=!1;break}if(n)return}this._lineDash=e,this.dirty=!0}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineDashOffset",{get:function(){return this._lineDashOffset},set:function(e){this._lineDashOffset!==e&&(this._lineDashOffset=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineCap",{get:function(){return this._lineCap},set:function(e){this._lineCap!==e&&(this._lineCap=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineJoin",{get:function(){return this._lineJoin},set:function(e){this._lineJoin!==e&&(this._lineJoin=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"opacity",{get:function(){return this._opacity},set:function(e){e=Math.min(1,Math.max(0,e)),this._opacity!==e&&(this._opacity=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fillShadow",{get:function(){return this._fillShadow},set:function(e){var r=this._fillShadow;r!==e&&(r&&r.removeEventListener("change",this.onShadowChange),e&&e.addEventListener("change",this.onShadowChange),this._fillShadow=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"strokeShadow",{get:function(){return this._strokeShadow},set:function(e){var r=this._strokeShadow;r!==e&&(r&&r.removeEventListener("change",this.onShadowChange),e&&e.addEventListener("change",this.onShadowChange),this._strokeShadow=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.fillStroke=function(e){if(!!this.scene){var r=this.scene.canvas.pixelRatio||1,n=e.globalAlpha;if(this.fill){e.fillStyle=this.fill,e.globalAlpha=n*this.opacity*this.fillOpacity;var o=this.fillShadow;o&&o.enabled&&(e.shadowColor=o.color,e.shadowOffsetX=o.xOffset*r,e.shadowOffsetY=o.yOffset*r,e.shadowBlur=o.blur*r),e.fill()}if(e.shadowColor="rgba(0, 0, 0, 0)",this.stroke&&this.strokeWidth){e.strokeStyle=this.stroke,e.globalAlpha=n*this.opacity*this.strokeOpacity,e.lineWidth=this.strokeWidth,this.lineDash&&e.setLineDash(this.lineDash),this.lineDashOffset&&(e.lineDashOffset=this.lineDashOffset),this.lineCap&&(e.lineCap=this.lineCap),this.lineJoin&&(e.lineJoin=this.lineJoin);var a=this.strokeShadow;a&&a.enabled&&(e.shadowColor=a.color,e.shadowOffsetX=a.xOffset*r,e.shadowOffsetY=a.yOffset*r,e.shadowBlur=a.blur*r),e.stroke()}}},t.prototype.containsPoint=function(e,r){return this.isPointInPath(e,r)},t.defaultStyles=Df({},{fill:"black",stroke:void 0,strokeWidth:0,lineDash:void 0,lineDashOffset:0,lineCap:void 0,lineJoin:void 0,opacity:1,fillShadow:void 0,strokeShadow:void 0}),t}(Ho),R0=Math.sign?Math.sign:function(i){return i=+i,i===0||isNaN(i)?i:i>0?1:-1};function AZ(i,t){var e=-t/i;return i!==0&&e>=0&&e<=1?[e]:[]}function DZ(i,t,e){if(i===0)return AZ(t,e);var r=t*t-4*i*e,n=[];if(r===0){var o=-t/(2*i);o>=0&&o<=1&&n.push(o)}else if(r>0){var a=Math.sqrt(r),s=(-t-a)/(2*i),l=(-t+a)/(2*i);s>=0&&s<=1&&n.push(s),l>=0&&l<=1&&n.push(l)}return n}function MZ(i,t,e,r){if(i===0)return DZ(t,e,r);var n=t/i,o=e/i,a=r/i,s=(3*o-n*n)/9,l=(9*n*o-27*a-2*n*n*n)/54,u=s*s*s+l*l,c=1/3,p=[];if(u>=0){var h=Math.sqrt(u),d=R0(l+h)*Math.pow(Math.abs(l+h),c),f=R0(l-h)*Math.pow(Math.abs(l-h),c),g=Math.abs(Math.sqrt(3)*(d-f)/2),v=-c*n+(d+f);if(v>=0&&v<=1&&p.push(v),g===0){var m=-c*n-(d+f)/2;m>=0&&m<=1&&p.push(m)}}else{var C=Math.acos(l/Math.sqrt(-s*s*s)),w=c*n,b=2*Math.sqrt(-s),S=b*Math.cos(c*C)-w,T=b*Math.cos(c*(C+2*Math.PI))-w,E=b*Math.cos(c*(C+4*Math.PI))-w;S>=0&&S<=1&&p.push(S),T>=0&&T<=1&&p.push(T),E>=0&&E<=1&&p.push(E)}return p}function rm(i,t,e,r,n,o,a,s){var l=(e-i)*(s-o)-(r-t)*(a-n);if(l===0)return null;var u=((a-n)*(t-o)-(i-n)*(s-o))/l,c=((e-i)*(t-o)-(r-t)*(i-n))/l;return u>=0&&u<=1&&c>=0&&c<=1?{x:i+u*(e-i),y:t+u*(r-t)}:null}function FZ(i,t,e,r,n,o,a,s,l,u,c,p){for(var h=[],d=u-p,f=c-l,g=l*(p-u)-u*(c-l),v=P0(i,e,n,a),m=P0(t,r,o,s),C=d*v[0]+f*m[0],w=d*v[1]+f*m[1],b=d*v[2]+f*m[2],S=d*v[3]+f*m[3]+g,T=MZ(C,w,b,S),E=0;E<T.length;E++){var P=T[E],R=P*P,M=P*R,N=v[0]*M+v[1]*R+v[2]*P+v[3],W=m[0]*M+m[1]*R+m[2]*P+m[3],G=void 0;l!==c?G=(N-l)/(c-l):G=(W-u)/(p-u),G>=0&&G<=1&&h.push({x:N,y:W})}return h}function P0(i,t,e,r){return[-i+3*t-3*e+r,3*i-6*t+3*e,-3*i+3*t,i]}var IZ=function(){function i(){this.commands=[],this.params=[],this._closedPath=!1}return i.prototype.moveTo=function(t,e){this.xy?(this.xy[0]=t,this.xy[1]=e):this.xy=[t,e],this.commands.push("M"),this.params.push(t,e)},i.prototype.lineTo=function(t,e){this.xy?(this.commands.push("L"),this.params.push(t,e),this.xy[0]=t,this.xy[1]=e):this.moveTo(t,e)},i.prototype.rect=function(t,e,r,n){this.moveTo(t,e),this.lineTo(t+r,e),this.lineTo(t+r,e+n),this.lineTo(t,e+n),this.closePath()},i.prototype.arcTo=function(t,e,r,n,o,a,s){var l=this.xy;if(!!l){t<0&&(t=-t),e<0&&(e=-e);var u=l[0],c=l[1],p=(u-a)/2,h=(c-s)/2,d=Math.sin(r),f=Math.cos(r),g=f*p+d*h,v=-d*p+f*h,m=g/t,C=v/e,w=m*m+C*C,b=(u+a)/2,S=(c+s)/2,T=0,E=0;w>=1?(w=Math.sqrt(w),t*=w,e*=w):(w=Math.sqrt(1/w-1),n===o&&(w=-w),T=w*t*C,E=-w*e*m,b+=f*T-d*E,S+=d*T+f*E);var P=Math.atan2((v-E)/e,(g-T)/t),R=Math.atan2((-v-E)/e,(-g-T)/t)-P;this.cubicArc(b,S,t,e,r,P,P+R,1-o)}},i.cubicArc=function(t,e,r,n,o,a,s,l,u,c){if(c){var p=l;l=u,u=p}var h=e.length,d=.5522847498307935,f=Math.sin(l),g=Math.cos(l),v=Math.sin(s),m=Math.cos(s),C=Math.PI/2,w=m*g*o-v*f*a,b=v*g*o+m*f*a,S=-m*f*o-v*g*a,T=-v*f*o+m*g*a;for(u-=l,u<0&&(u+=Math.PI*2),t.push("M"),e.push(w+r,b+n);u>=C;){u-=C,t.push("C");var E=S+r;e.push(w+S*d+r,b+T*d+n,w*d+S+r,b*d+T+n,Math.abs(E)<1e-8?0:E,T+n);var p=w;w=S,S=-p,p=b,b=T,T=-p}if(u){var P=1.3333333333333333*Math.tan(u/4),R=Math.sin(u),M=Math.cos(u),N=M+P*R,W=R-P*M;t.push("C");var E=w*M+S*R+r;e.push(w+S*P+r,b+T*P+n,w*N+S*W+r,b*N+T*W+n,Math.abs(E)<1e-8?0:E,b*M+T*R+n)}if(c)for(var G=h,k=e.length-2;G<k;G+=2,k-=2){var p=e[G];e[G]=e[k],e[k]=p,p=e[G+1],e[G+1]=e[k+1],e[k+1]=p}},i.prototype.cubicArc=function(t,e,r,n,o,a,s,l){var u=this.commands,c=this.params,p=u.length;i.cubicArc(u,c,t,e,r,n,o,a,s,l);var h=c[c.length-2],d=c[c.length-1];this.xy?(u[p]="L",this.xy[0]=h,this.xy[1]=d):this.xy=[h,d]},i.prototype.deCasteljau=function(t,e){var r=t.length;if(r<2||r%2===1)throw new Error("Fewer than two points or not an even count.");if(r===2||e===0)return t.slice(0,2);if(e===1)return t.slice(-2);for(var n=[],o=r-2,a=0;a<o;a+=2)n.push((1-e)*t[a]+e*t[a+2],(1-e)*t[a+1]+e*t[a+3]);return this.deCasteljau(n,e)},i.prototype.approximateCurve=function(t,e){var r=this.deCasteljau(t,0);this.moveTo(r[0],r[1]);for(var n=1/e,o=n;o<=1;o+=n){var a=this.deCasteljau(t,o);this.lineTo(a[0],a[1])}},i.prototype.quadraticCurveTo=function(t,e,r,n){this.xy||this.moveTo(t,e),this.cubicCurveTo((this.xy[0]+2*t)/3,(this.xy[1]+2*e)/3,(2*t+r)/3,(2*e+n)/3,r,n)},i.prototype.cubicCurveTo=function(t,e,r,n,o,a){this.xy||this.moveTo(t,e),this.commands.push("C"),this.params.push(t,e,r,n,o,a),this.xy[0]=o,this.xy[1]=a},Object.defineProperty(i.prototype,"closedPath",{get:function(){return this._closedPath},enumerable:!0,configurable:!0}),i.prototype.closePath=function(){this.xy&&(this.xy=void 0,this.commands.push("Z"),this._closedPath=!0)},i.prototype.clear=function(){this.commands.length=0,this.params.length=0,this.xy=void 0,this._closedPath=!1},i.prototype.isPointInPath=function(t,e){for(var r=this.commands,n=this.params,o=r.length,a=-1e4,s=-1e4,l=NaN,u=NaN,c=0,p=0,h=0,d=0,f=0;d<o;d++)switch(r[d]){case"M":isNaN(l)||rm(l,u,c,p,a,s,t,e)&&h++,l=c=n[f++],u=p=n[f++];break;case"L":rm(c,p,c=n[f++],p=n[f++],a,s,t,e)&&h++;break;case"C":h+=FZ(c,p,n[f++],n[f++],n[f++],n[f++],c=n[f++],p=n[f++],a,s,t,e).length;break;case"Z":isNaN(l)||rm(l,u,c,p,a,s,t,e)&&h++;break}return h%2===1},i.fromString=function(t){var e=new i;return e.setFromString(t),e},i.parseSvgPath=function(t){return t.trim().split(i.splitCommandsRe).map(function(e){var r=e.match(i.matchParamsRe);return{command:e.substr(0,1),params:r?r.map(parseFloat):[]}})},i.prettifySvgPath=function(t){return i.parseSvgPath(t).map(function(e){return e.command+e.params.join(",")}).join(`
`)},i.prototype.setFromString=function(t){var e=this;this.clear();var r=i.parseSvgPath(t),n,o,a,s,l;function u(){l.match(i.quadraticCommandRe)||(a=n,s=o)}function c(){l.match(i.cubicCommandRe)||(a=n,s=o)}r.forEach(function(p){var h=p.params,d=h.length,f=0;switch(p.command){case"M":for(e.moveTo(n=h[f++],o=h[f++]);f<d;)e.lineTo(n=h[f++],o=h[f++]);break;case"m":for(e.moveTo(n+=h[f++],o+=h[f++]);f<d;)e.lineTo(n+=h[f++],o+=h[f++]);break;case"L":for(;f<d;)e.lineTo(n=h[f++],o=h[f++]);break;case"l":for(;f<d;)e.lineTo(n+=h[f++],o+=h[f++]);break;case"C":for(;f<d;)e.cubicCurveTo(h[f++],h[f++],a=h[f++],s=h[f++],n=h[f++],o=h[f++]);break;case"c":for(;f<d;)e.cubicCurveTo(n+h[f++],o+h[f++],a=n+h[f++],s=o+h[f++],n+=h[f++],o+=h[f++]);break;case"S":for(c();f<d;)e.cubicCurveTo(n+n-a,o+o-s,a=h[f++],s=h[f++],n=h[f++],o=h[f++]);break;case"s":for(c();f<d;)e.cubicCurveTo(n+n-a,o+o-s,a=n+h[f++],s=o+h[f++],n+=h[f++],o+=h[f++]);break;case"Q":for(;f<d;)e.quadraticCurveTo(a=h[f++],s=h[f++],n=h[f++],o=h[f++]);break;case"q":for(;f<d;)e.quadraticCurveTo(a=n+h[f++],s=o+h[f++],n+=h[f++],o+=h[f++]);break;case"T":for(u();f<d;)e.quadraticCurveTo(a=n+n-a,s=o+o-s,n=h[f++],o=h[f++]);break;case"t":for(u();f<d;)e.quadraticCurveTo(a=n+n-a,s=o+o-s,n+=h[f++],o+=h[f++]);break;case"A":for(;f<d;)e.arcTo(h[f++],h[f++],h[f++]*Math.PI/180,h[f++],h[f++],n=h[f++],o=h[f++]);break;case"a":for(;f<d;)e.arcTo(h[f++],h[f++],h[f++]*Math.PI/180,h[f++],h[f++],n+=h[f++],o+=h[f++]);break;case"Z":case"z":e.closePath();break;case"H":for(;f<d;)e.lineTo(n=h[f++],o);break;case"h":for(;f<d;)e.lineTo(n+=h[f++],o);break;case"V":for(;f<d;)e.lineTo(n,o=h[f++]);break;case"v":for(;f<d;)e.lineTo(n,o+=h[f++]);break}l=p.command})},i.prototype.toString=function(){for(var t=this.commands,e=this.params,r=t.length,n=[],o=0,a=0;o<r;o++)switch(t[o]){case"M":n.push("M"+e[a++]+","+e[a++]);break;case"L":n.push("L"+e[a++]+","+e[a++]);break;case"C":n.push("C"+e[a++]+","+e[a++]+" "+e[a++]+","+e[a++]+" "+e[a++]+","+e[a++]);break;case"Z":n.push("Z");break}return n.join("")},i.prototype.toPrettyString=function(){return i.prettifySvgPath(this.toString())},i.prototype.toSvg=function(){return i.xmlDeclaration+`
<svg width="100%" height="100%" viewBox="0 0 50 50" version="1.1" xmlns="`+i.xmlns+`">
    <path d="`+this.toString()+`" style="fill:none;stroke:#000;stroke-width:0.5;"/>
</svg>`},i.prototype.toDebugSvg=function(){var t=i.prettifySvgPath(this.toString());return i.xmlDeclaration+`
<svg width="100%" height="100%" viewBox="0 0 100 100" version="1.1" xmlns="`+i.xmlns+`">
    <path d="`+t+`" style="fill:none;stroke:#000;stroke-width:0.5;"/>
</svg>`},i.prototype.toCubicPaths=function(){var t=[],e=this.params,r,n,o,a,s,l=0;return this.commands.forEach(function(u){switch(u){case"M":r=[n=a=e[l++],o=s=e[l++]],t.push(r);break;case"L":var c=e[l++],p=e[l++];r.push((a+a+c)/3,(s+s+p)/3,(a+c+c)/3,(s+p+p)/3,a=c,s=p);break;case"C":r.push(e[l++],e[l++],e[l++],e[l++],a=e[l++],s=e[l++]);break;case"Z":r.push((a+a+n)/3,(s+s+o)/3,(a+n+n)/3,(s+o+o)/3,a=n,s=o);break}}),t},i.cubicPathToString=function(t){var e=t.length;if(!(e%2===0&&(e/2-1)/2>=1))throw new Error("Invalid path.");return"M"+t.slice(0,2).join(",")+"C"+t.slice(2).join(",")},i.splitCommandsRe=/(?=[AaCcHhLlMmQqSsTtVvZz])/g,i.matchParamsRe=/-?[0-9]*\.?\d+/g,i.quadraticCommandRe=/[QqTt]/,i.cubicCommandRe=/[CcSs]/,i.xmlDeclaration='<?xml version="1.0" encoding="UTF-8"?>',i.xmlns="http://www.w3.org/2000/svg",i}(),LZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),By=function(i){LZ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.path=new IZ,e._dirtyPath=!0,e._svgPath="",e}return Object.defineProperty(t.prototype,"dirtyPath",{get:function(){return this._dirtyPath},set:function(e){this._dirtyPath!==e&&(this._dirtyPath=e,e&&(this.dirty=!0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"svgPath",{get:function(){return this._svgPath},set:function(e){this._svgPath!==e&&(this._svgPath=e,this.path.setFromString(e),this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.isPointInPath=function(e,r){var n=this.transformPoint(e,r);return this.path.closedPath&&this.path.isPointInPath(n.x,n.y)},t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.updatePath=function(){},t.prototype.render=function(e){var r=this.scene;this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e),this.dirtyPath&&(this.updatePath(),this.dirtyPath=!1),r.appendPath(this.path),this.fillStroke(e),this.dirty=!1},t.className="Path",t}(go),NZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),El=function(i){NZ(t,i);function t(){var e=i.call(this)||this;return e._x1=0,e._y1=0,e._x2=0,e._y2=0,e.restoreOwnStyles(),e}return Object.defineProperty(t.prototype,"x1",{get:function(){return this._x1},set:function(e){this._x1!==e&&(this._x1=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y1",{get:function(){return this._y1},set:function(e){this._y1!==e&&(this._y1=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"x2",{get:function(){return this._x2},set:function(e){this._x2!==e&&(this._x2=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y2",{get:function(){return this._y2},set:function(e){this._y2!==e&&(this._y2=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.computeBBox=function(){return new gs(this.x1,this.y1,this.x2-this.x1,this.y2-this.y1)},t.prototype.isPointInPath=function(e,r){return!1},t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this.x1,n=this.y1,o=this.x2,a=this.y2;if(r===o){var s=Math.round(r)+Math.floor(this.strokeWidth)%2/2;r=s,o=s}else if(n===a){var l=Math.round(n)+Math.floor(this.strokeWidth)%2/2;n=l,a=l}e.beginPath(),e.moveTo(r,n),e.lineTo(o,a),this.fillStroke(e),this.dirty=!1},t.className="Line",t.defaultStyles=Df(go.defaultStyles,{fill:void 0,strokeWidth:1}),t}(go),A0=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},pa=function(){function i(){this.type="band",this.index=new Map,this.ordinalRange=[],this._domain=[],this._range=[0,1],this._bandwidth=1,this._paddingInner=0,this._paddingOuter=0,this._round=!1,this._align=.5}return Object.defineProperty(i.prototype,"domain",{get:function(){return this._domain},set:function(t){var e=this._domain;e.length=0,this.index=new Map;var r=this.index;t.forEach(function(n){r.get(n)===void 0&&r.set(n,e.push(n)-1)}),this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"range",{get:function(){return this._range},set:function(t){this._range[0]=t[0],this._range[1]=t[1],this.rescale()},enumerable:!0,configurable:!0}),i.prototype.ticks=function(){return this._domain},i.prototype.convert=function(t){var e=this.index.get(t);if(e===void 0)return NaN;var r=this.ordinalRange[e];return r===void 0?NaN:r},Object.defineProperty(i.prototype,"bandwidth",{get:function(){return this._bandwidth},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"padding",{get:function(){return this._paddingInner},set:function(t){t=Math.max(0,Math.min(1,t)),this._paddingInner=t,this._paddingOuter=t,this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"paddingInner",{get:function(){return this._paddingInner},set:function(t){this._paddingInner=Math.max(0,Math.min(1,t)),this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"paddingOuter",{get:function(){return this._paddingOuter},set:function(t){this._paddingOuter=Math.max(0,Math.min(1,t)),this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"round",{get:function(){return this._round},set:function(t){this._round=t,this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"align",{get:function(){return this._align},set:function(t){this._align=Math.max(0,Math.min(1,t)),this.rescale()},enumerable:!0,configurable:!0}),i.prototype.rescale=function(){var t,e=this._domain.length;if(!!e){var r=A0(this._range,2),n=r[0],o=r[1],a=o<n;a&&(t=A0([o,n],2),n=t[0],o=t[1]);var s=(o-n)/Math.max(1,e-this._paddingInner+this._paddingOuter*2);this._round&&(s=Math.floor(s)),n+=(o-n-s*(e-this._paddingInner))*this._align,this._bandwidth=s*(1-this._paddingInner),this._round&&(n=Math.round(n),this._bandwidth=Math.round(this._bandwidth));for(var l=[],u=0;u<e;u++)l.push(n+s*u);this.ordinalRange=a?l.reverse():l}},i}(),D0=function(){function i(t,e){this.next=null,this.scene=t.scene,this.parent=t,this.datum=e}return i.prototype.appendChild=function(t){if(this.next===null)return this.parent.insertBefore(t,null);if(!Ho.isNode(this.next))throw new Error(this.next+" is not a Node.");return this.parent.insertBefore(t,this.next)},i.prototype.insertBefore=function(t,e){return this.parent.insertBefore(t,e)},i}(),Vd=function(){function i(t,e){this.groups=t,this.parents=e}return i.select=function(t){return new i([[typeof t=="function"?t():t]],[void 0])},i.selectAll=function(t){return new i([t==null?[]:t],[void 0])},i.prototype.append=function(t){return this.select(function(e){return e.appendChild(new t)})},i.prototype.appendFn=function(t){return this.select(function(e,r,n,o){return e.appendChild(t(e,r,n,o))})},i.prototype.select=function(t){for(var e=this.groups,r=e.length,n=[],o=0;o<r;o++)for(var a=e[o],s=a.length,l=n[o]=new Array(s),u=0;u<s;u++){var c=a[u];if(c){var p=t(c,c.datum,u,a);p&&(p.datum=c.datum),l[u]=p}}return new i(n,this.parents)},i.prototype.selectByClass=function(t){return this.select(function(e){if(Ho.isNode(e))for(var r=e.children,n=r.length,o=0;o<n;o++){var a=r[o];if(a instanceof t)return a}})},i.prototype.selectByTag=function(t){return this.select(function(e){if(Ho.isNode(e))for(var r=e.children,n=r.length,o=0;o<n;o++){var a=r[o];if(a.tag===t)return a}})},i.prototype.selectAllByClass=function(t){return this.selectAll(function(e){var r=[];if(Ho.isNode(e))for(var n=e.children,o=n.length,a=0;a<o;a++){var s=n[a];s instanceof t&&r.push(s)}return r})},i.prototype.selectAllByTag=function(t){return this.selectAll(function(e){var r=[];if(Ho.isNode(e))for(var n=e.children,o=n.length,a=0;a<o;a++){var s=n[a];s.tag===t&&r.push(s)}return r})},i.prototype.selectNone=function(){return[]},i.prototype.selectAll=function(t){t||(t=this.selectNone);for(var e=[],r=[],n=this.groups,o=n.length,a=0;a<o;a++)for(var s=n[a],l=s.length,u=0;u<l;u++){var c=s[u];c&&(e.push(t(c,c.datum,u,s)),r.push(c))}return new i(e,r)},i.prototype.each=function(t){for(var e=this.groups,r=e.length,n=0;n<r;n++)for(var o=e[n],a=o.length,s=0;s<a;s++){var l=o[s];l&&t(l,l.datum,s,o)}return this},i.prototype.remove=function(){return this.each(function(t){if(Ho.isNode(t)){var e=t.parent;e&&e.removeChild(t)}})},i.prototype.merge=function(t){for(var e=this.groups,r=t.groups,n=e.length,o=r.length,a=Math.min(n,o),s=new Array(n),l=0;l<a;l++)for(var u=e[l],c=r[l],p=u.length,h=s[l]=new Array(p),d=0;d<p;d++){var f=u[d]||c[d];h[d]=f||void 0}for(;l<n;l++)s[l]=e[l];return new i(s,this.parents)},i.prototype.node=function(){for(var t=this.groups,e=t.length,r=0;r<e;r++)for(var n=t[r],o=n.length,a=0;a<o;a++){var s=n[a];if(s)return s}return null},i.prototype.attr=function(t,e){return this.each(function(r){r[t]=e}),this},i.prototype.attrFn=function(t,e){return this.each(function(r,n,o,a){r[t]=e(r,n,o,a)}),this},i.prototype.call=function(t){return t(this),this},Object.defineProperty(i.prototype,"size",{get:function(){var t=0;return this.each(function(){return t++}),t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"data",{get:function(){var t=[];return this.each(function(e,r){return t.push(r)}),t},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"enter",{get:function(){return new i(this.enterGroups?this.enterGroups:[[]],this.parents)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"exit",{get:function(){return new i(this.exitGroups?this.exitGroups:[[]],this.parents)},enumerable:!0,configurable:!0}),i.prototype.setDatum=function(t){return this.each(function(e){e.datum=t})},Object.defineProperty(i.prototype,"datum",{get:function(){var t=this.node();return t?t.datum:null},enumerable:!0,configurable:!0}),i.prototype.setData=function(t,e){if(typeof t!="function"){var r=t;t=function(){return r}}for(var n=this.groups,o=this.parents,a=n.length,s=new Array(a),l=new Array(a),u=new Array(a),c=0;c<a;c++){var p=n[c],h=o[c];if(!h)throw new Error("Group #"+c+" has no parent: "+p);var d=p.length,f=t(h,h.datum,c,o),g=f.length,v=l[c]=new Array(g),m=s[c]=new Array(g),C=u[c]=new Array(d);e?this.bindKey(h,p,v,m,C,f,e):this.bindIndex(h,p,v,m,C,f);for(var w=0,b=0;w<g;w++){var S=v[w];if(S){w>=b&&(b=w+1);for(var T=void 0;!(T=m[b])&&b<g;)b++;S.next=T||null}}}var E=new i(s,o);return E.enterGroups=l,E.exitGroups=u,E},i.prototype.bindIndex=function(t,e,r,n,o,a){for(var s=e.length,l=a.length,u=0;u<l;u++){var c=e[u];c?(c.datum=a[u],n[u]=c):r[u]=new D0(t,a[u])}for(;u<s;u++){var c=e[u];c&&(o[u]=c)}},i.prototype.bindKey=function(t,e,r,n,o,a,s){for(var l=e.length,u=a.length,c=new Array(l),p={},h=0;h<l;h++){var d=e[h];if(d){var f=c[h]=i.keyPrefix+s(d,d.datum,h,e);f in p?o[h]=d:p[f]=d}}for(var h=0;h<u;h++){var f=i.keyPrefix+s(t,a[h],h,a),d=p[f];d?(n[h]=d,d.datum=a[h],p[f]=void 0):r[h]=new D0(t,a[h])}for(var h=0;h<l;h++){var d=e[h];d&&p[c[h]]===d&&(o[h]=d)}},i.keyPrefix="$",i}(),ro=function(){function i(t,e,r){t===void 0&&(t=window.document),e===void 0&&(e=600),r===void 0&&(r=300),this._container=void 0,this._pixelRatio=NaN,this._width=100,this._height=100,this.document=t,this.element=t.createElement("canvas"),this.context=this.element.getContext("2d"),this.element.style.userSelect="none",this.element.style.display="block",this.setPixelRatio(),this.resize(e,r)}return Object.defineProperty(i.prototype,"container",{get:function(){return this._container},set:function(t){this._container!==t&&(this.remove(),t&&t.appendChild(this.element),this._container=t)},enumerable:!0,configurable:!0}),i.prototype.remove=function(){var t=this.element.parentNode;t!=null&&t.removeChild(this.element)},i.prototype.destroy=function(){this.element.remove(),this._canvas=void 0,Object.freeze(this)},i.prototype.toImage=function(){var t=this.document.createElement("img");return t.src=this.getDataURL(),t},i.prototype.getDataURL=function(t){return this.element.toDataURL(t)},i.prototype.download=function(t){t=((t||"").trim()||"image")+".png";var e="image/png",r=this.getDataURL(e),n=this.document,o=n.createElement("a");o.href=r,o.download=t,o.style.display="none",n.body.appendChild(o),o.click(),n.body.removeChild(o)},Object.defineProperty(i.prototype,"pixelRatio",{get:function(){return this._pixelRatio},enumerable:!0,configurable:!0}),i.prototype.setPixelRatio=function(t){var e=t||window.devicePixelRatio;e!==this.pixelRatio&&(i.overrideScale(this.context,e),this._pixelRatio=e,this.resize(this.width,this.height))},Object.defineProperty(i.prototype,"pixelated",{get:function(){return this.element.style.imageRendering==="pixelated"},set:function(t){this.element.style.imageRendering=t?"pixelated":"auto"},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"width",{get:function(){return this._width},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"height",{get:function(){return this._height},enumerable:!0,configurable:!0}),i.prototype.resize=function(t,e){if(t>0&&e>0){var r=this,n=r.element,o=r.context,a=r.pixelRatio;n.width=Math.round(t*a),n.height=Math.round(e*a),n.style.width=t+"px",n.style.height=e+"px",o.resetTransform(),this._width=t,this._height=e}},Object.defineProperty(i,"textMeasuringContext",{get:function(){if(this._textMeasuringContext)return this._textMeasuringContext;var t=document.createElement("canvas");return this._textMeasuringContext=t.getContext("2d")},enumerable:!0,configurable:!0}),Object.defineProperty(i,"svgText",{get:function(){if(this._svgText)return this._svgText;var t="http://www.w3.org/2000/svg",e=document.createElementNS(t,"svg");e.setAttribute("width","100"),e.setAttribute("height","100"),e.classList?e.classList.add("text-measuring-svg"):e.setAttribute("class","text-measuring-svg"),e.style.position="absolute",e.style.top="-1000px",e.style.visibility="hidden";var r=document.createElementNS(t,"text");return r.setAttribute("x","0"),r.setAttribute("y","30"),r.setAttribute("text","black"),e.appendChild(r),document.body.appendChild(e),this._svgText=r,r},enumerable:!0,configurable:!0}),Object.defineProperty(i,"has",{get:function(){if(this._has)return this._has;var t=navigator.userAgent.indexOf("Chrome")>-1,e=navigator.userAgent.indexOf("Firefox")>-1,r=!t&&navigator.userAgent.indexOf("Safari")>-1;return this._has=Object.freeze({textMetrics:this.textMeasuringContext.measureText("test").actualBoundingBoxDescent!==void 0&&!e&&!r,getTransform:this.textMeasuringContext.getTransform!==void 0})},enumerable:!0,configurable:!0}),i.measureText=function(t,e,r,n){var o=this.textMeasuringContext;return o.font=e,o.textBaseline=r,o.textAlign=n,o.measureText(t)},i.getTextSize=function(t,e){if(this.has.textMetrics){var r=this.textMeasuringContext;r.font=e;var n=r.measureText(t);return{width:n.width,height:n.actualBoundingBoxAscent+n.actualBoundingBoxDescent}}else return this.measureSvgText(t,e)},i.measureSvgText=function(t,e){var r=this.textSizeCache,n=r[e];if(n){var o=n[t];if(o)return o}else r[e]={};var a=this.svgText;a.style.font=e,a.textContent=t;var s=a.getBBox(),l={width:s.width,height:s.height};return r[e][t]=l,l},i.overrideScale=function(t,e){var r=0,n={save:function(){this.$save(),r++},restore:function(){r>0&&(this.$restore(),r--)},setTransform:function(a,s,l,u,c,p){this.$setTransform(a*e,s*e,l*e,u*e,c*e,p*e)},resetTransform:function(){this.$setTransform(e,0,0,e,0,0),this.save(),r=0}};for(var o in n)n.hasOwnProperty(o)&&(t["$"+o]||(t["$"+o]=t[o]),t[o]=n[o])},i.textSizeCache={},i}(),GZ=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},kZ=globalThis&&globalThis.__spread||function(){for(var i=[],t=0;t<arguments.length;t++)i=i.concat(GZ(arguments[t]));return i},WZ=function(){function i(t,e,r){var n=this;t===void 0&&(t=window.document),this.id=db(this),this._dirty=!1,this.animationFrameId=0,this._root=null,this.debug={renderFrameIndex:!1,renderBoundingBoxes:!1},this._frameIndex=0,this.render=function(){var o,a=n,s=a.ctx,l=a.root,u=a.pendingSize;if(n.animationFrameId=0,u&&((o=n.canvas).resize.apply(o,kZ(u)),n.pendingSize=void 0),l&&!l.visible){n.dirty=!1;return}s.clearRect(0,0,n.width,n.height),l&&(s.save(),l.visible&&l.render(s),s.restore()),n._frameIndex++,n.debug.renderFrameIndex&&(s.fillStyle="white",s.fillRect(0,0,40,15),s.fillStyle="black",s.fillText(n.frameIndex.toString(),2,10)),n.dirty=!1},this.canvas=new ro(t,e,r),this.ctx=this.canvas.context}return Object.defineProperty(i.prototype,"container",{get:function(){return this.canvas.container},set:function(t){this.canvas.container=t},enumerable:!0,configurable:!0}),i.prototype.download=function(t){this.canvas.download(t)},i.prototype.getDataURL=function(t){return this.canvas.getDataURL(t)},Object.defineProperty(i.prototype,"width",{get:function(){return this.pendingSize?this.pendingSize[0]:this.canvas.width},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"height",{get:function(){return this.pendingSize?this.pendingSize[1]:this.canvas.height},enumerable:!0,configurable:!0}),i.prototype.resize=function(t,e){t=Math.round(t),e=Math.round(e),!(t===this.width&&e===this.height)&&(this.pendingSize=[t,e],this.dirty=!0)},Object.defineProperty(i.prototype,"dirty",{get:function(){return this._dirty},set:function(t){t&&!this._dirty&&(this.animationFrameId=requestAnimationFrame(this.render)),this._dirty=t},enumerable:!0,configurable:!0}),i.prototype.cancelRender=function(){this.animationFrameId&&(cancelAnimationFrame(this.animationFrameId),this.animationFrameId=0,this._dirty=!1)},Object.defineProperty(i.prototype,"root",{get:function(){return this._root},set:function(t){t!==this._root&&(this._root&&this._root._setScene(),this._root=t,t&&(t.parent===null&&t.scene&&t.scene!==this&&(t.scene.root=null),t._setScene(this)),this.dirty=!0)},enumerable:!0,configurable:!0}),i.prototype.appendPath=function(t){var e=this.ctx,r=t.commands,n=t.params,o=r.length,a=0;e.beginPath();for(var s=0;s<o;s++)switch(r[s]){case"M":e.moveTo(n[a++],n[a++]);break;case"L":e.lineTo(n[a++],n[a++]);break;case"C":e.bezierCurveTo(n[a++],n[a++],n[a++],n[a++],n[a++],n[a++]);break;case"Z":e.closePath();break}},Object.defineProperty(i.prototype,"frameIndex",{get:function(){return this._frameIndex},enumerable:!0,configurable:!0}),i.className="Scene",i}(),Bd=globalThis&&globalThis.__assign||function(){return Bd=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},Bd.apply(this,arguments)},up=function(){function i(){this.allPropertyListeners=new Map,this.allEventListeners=new Map}return i.prototype.addPropertyListener=function(t,e,r){r===void 0&&(r=this);var n=this.allPropertyListeners,o=n.get(t);if(o||(o=new Map,n.set(t,o)),!o.has(e)){var a=new Set;o.set(e,a)}var s=o.get(e);s&&s.add(r)},i.prototype.removePropertyListener=function(t,e,r){r===void 0&&(r=this);var n=this.allPropertyListeners,o=n.get(t);if(o)if(e){var a=o.get(e);a&&(a.delete(r),a.size||o.delete(e))}else o.clear()},i.prototype.notifyPropertyListeners=function(t,e,r){var n=this,o=this.allPropertyListeners,a=o.get(t);a&&a.forEach(function(s,l){s.forEach(function(u){return l.call(u,{type:t,source:n,value:r,oldValue:e})})})},i.prototype.addEventListener=function(t,e,r){r===void 0&&(r=this);var n=this.allEventListeners,o=n.get(t);if(o||(o=new Map,n.set(t,o)),!o.has(e)){var a=new Set;o.set(e,a)}var s=o.get(e);s&&s.add(r)},i.prototype.removeEventListener=function(t,e,r){r===void 0&&(r=this);var n=this.allEventListeners,o=n.get(t);if(o)if(e){var a=o.get(e);a&&(a.delete(r),a.size||o.delete(e))}else o.clear()},i.prototype.notifyEventListeners=function(t){var e=this,r=this.allEventListeners;t.forEach(function(n){var o=r.get(n);o&&o.forEach(function(a,s){a.forEach(function(l){return s.call(l,{type:n,source:e})})})})},i.prototype.fireEvent=function(t){var e=this,r=this.allEventListeners.get(t.type);r&&r.forEach(function(n,o){n.forEach(function(a){return o.call(a,Bd(Bd({},t),{source:e}))})})},i.privateKeyPrefix="_",i}();function ol(){for(var i=[],t=0;t<arguments.length;t++)i[t]=arguments[t];var e=i.indexOf("debugger")>=0;return function(r,n){var o=up.privateKeyPrefix+n,a=o+"Events";r[n]||(i&&(r[a]=i),Object.defineProperty(r,n,{set:function(s){var l=this[o];if(e)debugger;if(s!==l||typeof s=="object"&&s!==null){this[o]=s,this.notifyPropertyListeners(n,l,s);var u=this[a];u&&this.notifyEventListeners(u)}},get:function(){return this[o]},enumerable:!0,configurable:!0}))}}var VZ=function(){function i(t,e,r,n){t===void 0&&(t=0),e===void 0&&(e=t),r===void 0&&(r=t),n===void 0&&(n=e),this.top=t,this.right=e,this.bottom=r,this.left=n}return i.prototype.clear=function(){this.top=this.right=this.bottom=this.left=0},i}(),BZ=`
.ag-sparkline-tooltip-wrapper {
    position: absolute;
    user-select: none;
    pointer-events: none;
}

.ag-sparkline-tooltip {
    position: relative;
    font: 12px arial,sans-serif;
    border-radius: 2px;
    box-shadow: 0 1px 3px rgb(0 0 0 / 20%), 0 1px 1px rgb(0 0 0 / 14%);
    line-height: 1.7em;
    overflow: hidden;
    white-space: nowrap;
    z-index: 99999;
    background-color: rgb(255, 255, 255);
    color: rgba(0,0,0, 0.67);
}

.ag-sparkline-tooltip-content {
    padding: 0 7px;
}

.ag-sparkline-tooltip-title {
    padding-left: 7px;
}

.ag-sparkline-tooltip-wrapper-hidden {
    top: -10000px !important;
}

.ag-sparkline-wrapper {
    box-sizing: border-box;
    overflow: hidden;
}
`;function Ur(i){return typeof i!="number"?!1:Number.isFinite(i)}function HZ(i){return!!i&&i.hasOwnProperty("valueOf")&&Ur(i.valueOf())}function jZ(i){return Ur(i)||HZ(i)}function Hy(i){return i instanceof Date&&!isNaN(+i)}function dD(i){return typeof i=="string"}function UZ(i){return!!i&&i.hasOwnProperty("toString")&&dD(i.toString())}function zZ(i){return jZ(i)||Hy(i)}var $Z=function(i){return function(){return i}};function Qa(i,t){return i=+i,t=+t,function(e){return i*(1-e)+t*e}}function KZ(i,t){var e=new Date,r=+i,n=+t;return function(o){return e.setTime(r*(1-o)+n*o),e}}function YZ(i,t){var e=t?t.length:0,r=i?Math.min(e,i.length):0,n=new Array(r),o=new Array(e),a;for(a=0;a<r;++a)n[a]=fb(i[a],t[a]);for(;a<e;++a)o[a]=t[a];return function(s){for(a=0;a<r;++a)o[a]=n[a](s);return o}}function XZ(i,t){var e={},r={},n;(i===null||typeof i!="object")&&(i={}),(t===null||typeof t!="object")&&(t={});for(n in t)n in i?e[n]=fb(i[n],t[n]):r[n]=t[n];return function(o){for(n in e)r[n]=e[n](o);return r}}var M0=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},Zi=function(){function i(t,e,r,n){n===void 0&&(n=1),this.r=Math.min(1,Math.max(0,t||0)),this.g=Math.min(1,Math.max(0,e||0)),this.b=Math.min(1,Math.max(0,r||0)),this.a=Math.min(1,Math.max(0,n||0))}return i.fromString=function(t){if(t.indexOf("#")>=0)return i.fromHexString(t);var e=i.nameToHex[t];if(e)return i.fromHexString(e);if(t.indexOf("rgb")>=0)return i.fromRgbaString(t);throw new Error("Invalid color string: '"+t+"'")},i.parseHex=function(t){t=t.replace(/ /g,"").slice(1);var e;switch(t.length){case 6:case 8:e=[];for(var r=0;r<t.length;r+=2)e.push(parseInt(""+t[r]+t[r+1],16));break;case 3:case 4:e=t.split("").map(function(n){return parseInt(n,16)}).map(function(n){return n+n*16});break}if(e.length>=3&&e.every(function(n){return n>=0}))return e.length===3&&e.push(255),e},i.fromHexString=function(t){var e=i.parseHex(t);if(e){var r=M0(e,4),n=r[0],o=r[1],a=r[2],s=r[3];return new i(n/255,o/255,a/255,s/255)}throw new Error("Malformed hexadecimal color string: '"+t+"'")},i.stringToRgba=function(t){for(var e=M0([NaN,NaN],2),r=e[0],n=e[1],o=0;o<t.length;o++){var a=t[o];if(!r&&a==="(")r=o;else if(a===")"){n=o;break}}var s=r&&n&&t.substring(r+1,n);if(!!s){for(var l=s.split(","),u=[],o=0;o<l.length;o++){var c=l[o],p=parseFloat(c);if(isNaN(p))return;c.indexOf("%")>=0?(p=Math.max(0,Math.min(100,p)),p/=100):o===3?p=Math.max(0,Math.min(1,p)):(p=Math.max(0,Math.min(255,p)),p/=255),u.push(p)}return u}},i.fromRgbaString=function(t){var e=i.stringToRgba(t);if(e){if(e.length===3)return new i(e[0],e[1],e[2]);if(e.length===4)return new i(e[0],e[1],e[2],e[3])}throw new Error("Malformed rgb/rgba color string: '"+t+"'")},i.fromArray=function(t){if(t.length===4)return new i(t[0],t[1],t[2],t[3]);if(t.length===3)return new i(t[0],t[1],t[2]);throw new Error("The given array should contain 3 or 4 color components (numbers).")},i.fromHSB=function(t,e,r,n){n===void 0&&(n=1);var o=i.HSBtoRGB(t,e,r);return new i(o[0],o[1],o[2],n)},i.padHex=function(t){return t.length===1?"0"+t:t},i.prototype.toHexString=function(){var t="#"+i.padHex(Math.round(this.r*255).toString(16))+i.padHex(Math.round(this.g*255).toString(16))+i.padHex(Math.round(this.b*255).toString(16));return this.a<1&&(t+=i.padHex(Math.round(this.a*255).toString(16))),t},i.prototype.toRgbaString=function(t){t===void 0&&(t=3);var e=[Math.round(this.r*255),Math.round(this.g*255),Math.round(this.b*255)],r=Math.pow(10,t);return this.a!==1?(e.push(Math.round(this.a*r)/r),"rgba("+e.join(", ")+")"):"rgb("+e.join(", ")+")"},i.prototype.toString=function(){return this.a===1?this.toHexString():this.toRgbaString()},i.prototype.toHSB=function(){return i.RGBtoHSB(this.r,this.g,this.b)},i.RGBtoHSB=function(t,e,r){var n=Math.min(t,e,r),o=Math.max(t,e,r),a=o!==0?(o-n)/o:0,s=NaN;if(n!==o){var l=o-n,u=(o-t)/l,c=(o-e)/l,p=(o-r)/l;t===o?s=p-c:e===o?s=2+u-p:s=4+c-u,s/=6,s<0&&(s=s+1)}return[s*360,a,o]},i.HSBtoRGB=function(t,e,r){isNaN(t)&&(t=0),t=(t%360+360)%360/360;var n=0,o=0,a=0;if(e===0)n=o=a=r;else{var s=(t-Math.floor(t))*6,l=s-Math.floor(s),u=r*(1-e),c=r*(1-e*l),p=r*(1-e*(1-l));switch(s>>0){case 0:n=r,o=p,a=u;break;case 1:n=c,o=r,a=u;break;case 2:n=u,o=r,a=p;break;case 3:n=u,o=c,a=r;break;case 4:n=p,o=u,a=r;break;case 5:n=r,o=u,a=c;break}}return[n,o,a]},i.prototype.derive=function(t,e,r,n){var o=i.RGBtoHSB(this.r,this.g,this.b),a=o[2];a==0&&r>1&&(a=.05);var s=((o[0]+t)%360+360)%360,l=Math.max(Math.min(o[1]*e,1),0);a=Math.max(Math.min(a*r,1),0);var u=Math.max(Math.min(this.a*n,1),0),c=i.HSBtoRGB(s,l,a);return c.push(u),i.fromArray(c)},i.prototype.brighter=function(){return this.derive(0,1,1/.7,1)},i.prototype.darker=function(){return this.derive(0,1,.7,1)},i.nameToHex=Object.freeze({aliceblue:"#F0F8FF",antiquewhite:"#FAEBD7",aqua:"#00FFFF",aquamarine:"#7FFFD4",azure:"#F0FFFF",beige:"#F5F5DC",bisque:"#FFE4C4",black:"#000000",blanchedalmond:"#FFEBCD",blue:"#0000FF",blueviolet:"#8A2BE2",brown:"#A52A2A",burlywood:"#DEB887",cadetblue:"#5F9EA0",chartreuse:"#7FFF00",chocolate:"#D2691E",coral:"#FF7F50",cornflowerblue:"#6495ED",cornsilk:"#FFF8DC",crimson:"#DC143C",cyan:"#00FFFF",darkblue:"#00008B",darkcyan:"#008B8B",darkgoldenrod:"#B8860B",darkgray:"#A9A9A9",darkgreen:"#006400",darkgrey:"#A9A9A9",darkkhaki:"#BDB76B",darkmagenta:"#8B008B",darkolivegreen:"#556B2F",darkorange:"#FF8C00",darkorchid:"#9932CC",darkred:"#8B0000",darksalmon:"#E9967A",darkseagreen:"#8FBC8F",darkslateblue:"#483D8B",darkslategray:"#2F4F4F",darkslategrey:"#2F4F4F",darkturquoise:"#00CED1",darkviolet:"#9400D3",deeppink:"#FF1493",deepskyblue:"#00BFFF",dimgray:"#696969",dimgrey:"#696969",dodgerblue:"#1E90FF",firebrick:"#B22222",floralwhite:"#FFFAF0",forestgreen:"#228B22",fuchsia:"#FF00FF",gainsboro:"#DCDCDC",ghostwhite:"#F8F8FF",gold:"#FFD700",goldenrod:"#DAA520",gray:"#808080",green:"#008000",greenyellow:"#ADFF2F",grey:"#808080",honeydew:"#F0FFF0",hotpink:"#FF69B4",indianred:"#CD5C5C",indigo:"#4B0082",ivory:"#FFFFF0",khaki:"#F0E68C",lavender:"#E6E6FA",lavenderblush:"#FFF0F5",lawngreen:"#7CFC00",lemonchiffon:"#FFFACD",lightblue:"#ADD8E6",lightcoral:"#F08080",lightcyan:"#E0FFFF",lightgoldenrodyellow:"#FAFAD2",lightgray:"#D3D3D3",lightgreen:"#90EE90",lightgrey:"#D3D3D3",lightpink:"#FFB6C1",lightsalmon:"#FFA07A",lightseagreen:"#20B2AA",lightskyblue:"#87CEFA",lightslategray:"#778899",lightslategrey:"#778899",lightsteelblue:"#B0C4DE",lightyellow:"#FFFFE0",lime:"#00FF00",limegreen:"#32CD32",linen:"#FAF0E6",magenta:"#FF00FF",maroon:"#800000",mediumaquamarine:"#66CDAA",mediumblue:"#0000CD",mediumorchid:"#BA55D3",mediumpurple:"#9370DB",mediumseagreen:"#3CB371",mediumslateblue:"#7B68EE",mediumspringgreen:"#00FA9A",mediumturquoise:"#48D1CC",mediumvioletred:"#C71585",midnightblue:"#191970",mintcream:"#F5FFFA",mistyrose:"#FFE4E1",moccasin:"#FFE4B5",navajowhite:"#FFDEAD",navy:"#000080",oldlace:"#FDF5E6",olive:"#808000",olivedrab:"#6B8E23",orange:"#FFA500",orangered:"#FF4500",orchid:"#DA70D6",palegoldenrod:"#EEE8AA",palegreen:"#98FB98",paleturquoise:"#AFEEEE",palevioletred:"#DB7093",papayawhip:"#FFEFD5",peachpuff:"#FFDAB9",peru:"#CD853F",pink:"#FFC0CB",plum:"#DDA0DD",powderblue:"#B0E0E6",purple:"#800080",rebeccapurple:"#663399",red:"#FF0000",rosybrown:"#BC8F8F",royalblue:"#4169E1",saddlebrown:"#8B4513",salmon:"#FA8072",sandybrown:"#F4A460",seagreen:"#2E8B57",seashell:"#FFF5EE",sienna:"#A0522D",silver:"#C0C0C0",skyblue:"#87CEEB",slateblue:"#6A5ACD",slategray:"#708090",slategrey:"#708090",snow:"#FFFAFA",springgreen:"#00FF7F",steelblue:"#4682B4",tan:"#D2B48C",teal:"#008080",thistle:"#D8BFD8",tomato:"#FF6347",turquoise:"#40E0D0",violet:"#EE82EE",wheat:"#F5DEB3",white:"#FFFFFF",whitesmoke:"#F5F5F5",yellow:"#FFFF00",yellowgreen:"#9ACD32"}),i}();function F0(i,t){if(typeof i=="string")try{i=Zi.fromString(i)}catch{i=Zi.fromArray([0,0,0])}if(typeof t=="string")try{t=Zi.fromString(t)}catch{t=Zi.fromArray([0,0,0])}var e=Qa(i.r,t.r),r=Qa(i.g,t.g),n=Qa(i.b,t.b),o=Qa(i.a,t.a);return function(a){return Zi.fromArray([e(a),r(a),n(a),o(a)]).toRgbaString()}}function fb(i,t){var e=typeof t,r;if(t==null||e==="boolean")return $Z(t);if(e==="number")return Qa(i,t);if(e==="string")try{return r=Zi.fromString(t),t=r,F0(i,t)}catch{}return t instanceof Zi?F0(i,t):t instanceof Date?KZ(i,t):Array.isArray(t)?YZ(i,t):typeof t.valueOf!="function"&&typeof t.toString!="function"||isNaN(t)?XZ(i,t):Qa(i,t)}var qZ=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},ZZ=function(i){return function(){return i}},al=function(i){return i};function I0(i){var t,e=i[0],r=i[i.length-1];return e>r&&(t=qZ([r,e],2),e=t[0],r=t[1]),function(n){return Math.max(e,Math.min(r,n))}}var fD=function(){function i(){this.unknown=void 0,this._clamp=al,this._domain=[0,1],this._range=[0,1],this.transform=al,this.untransform=al,this._interpolate=fb,this.rescale()}return Object.defineProperty(i.prototype,"clamp",{get:function(){return this._clamp!==al},set:function(t){this._clamp=t?I0(this.domain):al},enumerable:!0,configurable:!0}),i.prototype.setDomain=function(t){this._domain=Array.prototype.map.call(t,function(e){return+e}),this._clamp!==al&&(this._clamp=I0(this.domain)),this.rescale()},i.prototype.getDomain=function(){return this._domain.slice()},Object.defineProperty(i.prototype,"domain",{get:function(){return this.getDomain()},set:function(t){this.setDomain(t)},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"range",{get:function(){return this._range.slice()},set:function(t){this._range=Array.prototype.slice.call(t),this.rescale()},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"interpolate",{get:function(){return this._interpolate},set:function(t){this._interpolate=t,this.rescale()},enumerable:!0,configurable:!0}),i.prototype.rescale=function(){this.piecewise=this.bimap,this.output=void 0,this.input=void 0},i.prototype.normalize=function(t,e){return(e-=t=+t)?function(r){return(r-t)/e}:ZZ(isNaN(e)?NaN:.5)},i.prototype.bimap=function(t,e,r){var n=t[0],o=t[1],a=e[0],s=e[1],l,u;return o<n?(l=this.normalize(o,n),u=r(s,a)):(l=this.normalize(n,o),u=r(a,s)),function(c){return u(l(c))}},i.prototype.convert=function(t){return t=+t,isNaN(t)?this.unknown:(this.output||(this.output=this.piecewise(this.domain.map(this.transform),this.range,this.interpolate)),this.output(this.transform(this._clamp(t))))},i.prototype.invert=function(t){return this.input||(this.input=this.piecewise(this.range,this.domain.map(this.transform),Qa)),this._clamp(this.untransform(this.input(t)))},i}(),QZ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}();function JZ(i,t,e){var r=Hd(i,t,e);return i=Math.ceil(i/r)*r,t=Math.floor(t/r)*r+r/2,tQ(i,t,r)}var jy=Math.sqrt(50),Uy=Math.sqrt(10),zy=Math.sqrt(2);function Hd(i,t,e){var r=Math.abs(t-i)/Math.max(0,e),n=Math.pow(10,Math.floor(Math.log(r)/Math.LN10)),o=r/n;return o>=jy?n*=10:o>=Uy?n*=5:o>=zy&&(n*=2),t<i?-n:n}function nm(i,t,e){var r=(t-i)/Math.max(0,e),n=Math.floor(Math.log(r)/Math.LN10),o=r/Math.pow(10,n);return n>=0?(o>=jy?10:o>=Uy?5:o>=zy?2:1)*Math.pow(10,n):-Math.pow(10,-n)/(o>=jy?10:o>=Uy?5:o>=zy?2:1)}var eQ=function(i){QZ(t,i);function t(e,r){var n=i.call(this)||this;if(r)for(var o=0,a=r.length;o<a;o++)n[o]=r[o];return n.fractionDigits=e,n}return t}(Array);function tQ(i,t,e){e===void 0&&(e=1);for(var r=Math.abs(e),n=r>0&&r<1?Math.abs(Math.floor(Math.log(r)/Math.LN10)):0,o=Math.pow(10,n),a=Math.max(0,Math.ceil((t-i)/e))||0,s=new eQ(n),l=0;l<a;l++){var u=i+e*l;s[l]=Math.round(u*o)/o}return s}function rQ(i,t){var e=i.toPrecision(t);e:for(var r=e.length,n=1,o=-1,a=0;n<r;++n)switch(e[n]){case".":o=a=n;break;case"0":o===0&&(o=n),a=n;break;case"e":break e;default:o>0&&(o=0);break}return o>0?e.slice(0,o)+e.slice(a+1):e}var L0={"":rQ,"%":function(i,t){return(i*100).toFixed(t)},b:function(i){return Math.round(i).toString(2)},c:function(i){return String(i)},d:aQ,e:function(i,t){return i.toExponential(t)},f:function(i,t){return i.toFixed(t)},g:function(i,t){return i.toPrecision(t)},o:function(i){return Math.round(i).toString(8)},p:function(i,t){return G0(i*100,t)},r:G0,s:oQ,X:function(i){return Math.round(i).toString(16).toUpperCase()},x:function(i){return Math.round(i).toString(16)}},N0=["y","z","a","f","p","n","\xB5","m","","k","M","G","T","P","E","Z","Y"],im=function(){function i(t){t instanceof i?(this.fill=t.fill,this.align=t.align,this.sign=t.sign,this.symbol=t.symbol,this.zero=t.zero,this.width=t.width,this.comma=t.comma,this.precision=t.precision,this.trim=t.trim,this.type=t.type,this.string=t.string):(this.fill=t.fill===void 0?" ":String(t.fill),this.align=t.align===void 0?">":String(t.align),this.sign=t.sign===void 0?"-":String(t.sign),this.symbol=t.symbol===void 0?"":String(t.symbol),this.zero=!!t.zero,this.width=t.width===void 0?void 0:+t.width,this.comma=!!t.comma,this.precision=t.precision===void 0?void 0:+t.precision,this.trim=!!t.trim,this.type=t.type===void 0?"":String(t.type),this.string=t.string)}return i}(),nQ=/^(?:(.)?([<>=^]))?([+\-( ])?([$#])?(0)?(\d+)?(,)?(\.\d+)?(~)?([a-z%])?$/i,gD=/(#\{(.*?)\})/g;function $y(i){if(i instanceof im)return new im(i);var t=!1,e=i.replace(gD,function(){return t||(i=arguments[2],t=!0),"#{}"}),r=nQ.exec(i);if(!r)throw new Error("Invalid format: "+i);return new im({fill:r[1],align:r[2],sign:r[3],symbol:r[4],zero:r[5],width:r[6],comma:r[7],precision:r[8]&&r[8].slice(1),trim:r[9],type:r[10],string:t?e:void 0})}function iQ(i,t,e,r){var n=Hd(i,t,e),o=$y(r==null?",f":r),a;switch(o.type){case"s":{var s=Math.max(Math.abs(i),Math.abs(t));return o.precision==null&&!isNaN(a=hQ(n,s))&&(o.precision=a),mD(o,s)}case"":case"e":case"g":case"p":case"r":{o.precision==null&&!isNaN(a=dQ(n,Math.max(Math.abs(i),Math.abs(t))))&&(o.precision=a-+(o.type==="e"));break}case"f":case"%":{o.precision==null&&!isNaN(a=pQ(n))&&(o.precision=a-+(o.type==="%")*2);break}}return vD(o)}var Ky;function oQ(i,t){t===void 0&&(t=0);var e=jd(i,t);if(!e)return String(i);var r=e[0],n=e[1];Ky=Math.max(-8,Math.min(8,Math.floor(n/3)))*3;var o=n-Ky+1,a=r.length;if(o===a)return r;if(o>a)return r+new Array(o-a+1).join("0");if(o>0)return r.slice(0,o)+"."+r.slice(o);var s=jd(i,Math.max(0,t+o-1));return"0."+new Array(1-o).join("0")+s[0]}function aQ(i){return Math.abs(i=Math.round(i))>=1e21?i.toLocaleString("en").replace(/,/g,""):i.toString(10)}function sQ(i,t){return function(e,r){for(var n=[],o=e.length,a=0,s=i[0],l=0;o>0&&s>0&&(l+s+1>r&&(s=Math.max(1,r-l)),n.push(e.substring(o-=s,o+s)),!((l+=s+1)>r));)s=i[a=(a+1)%i.length];return n.reverse().join(t)}}function lQ(i){return function(t){return t.replace(/[0-9]/g,function(e){return i[+e]})}}function uQ(i){e:for(var t=i.length,e=1,r=-1,n=0;e<t;++e)switch(i[e]){case".":r=n=e;break;case"0":r===0&&(r=e),n=e;break;default:if(!+i[e])break e;r>0&&(r=0);break}return r>0?i.slice(0,r)+i.slice(n+1):i}function G0(i,t){var e=jd(i,t);if(!e)return String(i);var r=e[0],n=e[1];return n<0?"0."+new Array(-n).join("0")+r:r.length>n+1?r.slice(0,n+1)+"."+r.slice(n+1):r+new Array(n-r.length+2).join("0")}function jd(i,t){var e=t?i.toExponential(t-1):i.toExponential(),r=e.indexOf("e");if(!(r<0)){var n=e.slice(0,r);return[n.length>1?n[0]+n.slice(2):n,+e.slice(r+1)]}}function k0(i){return i}var om,vD,mD;cQ({thousands:",",grouping:[3],currency:["$",""]});function cQ(i){om=fQ(i),vD=om.format,mD=om.formatPrefix}function kl(i){var t=jd(Math.abs(i));return t?t[1]:NaN}function pQ(i){return Math.max(0,-kl(Math.abs(i)))}function hQ(i,t){var e=Math.floor(kl(t)/3);return e=Math.min(8,e),e=Math.max(-8,e),Math.max(0,e*3-kl(Math.abs(i)))}function dQ(i,t){return i=Math.abs(i),t=Math.abs(t)-i,Math.max(0,kl(t)-kl(i))+1}function fQ(i){var t=i.grouping===void 0||i.thousands===void 0?k0:sQ(Array.prototype.map.call(i.grouping,Number),String(i.thousands)),e=i.currency===void 0?"":String(i.currency[0]),r=i.currency===void 0?"":String(i.currency[1]),n=i.decimal===void 0?".":String(i.decimal),o=i.numerals===void 0?k0:lQ(Array.prototype.map.call(i.numerals,String)),a=i.percent===void 0?"%":String(i.percent),s=i.minus===void 0?"\u2212":String(i.minus),l=i.nan===void 0?"NaN":String(i.nan);function u(p){var h=$y(p),d=h.fill,f=h.align,g=h.sign,v=h.symbol,m=h.zero,C=h.width,w=h.comma,b=h.precision,S=h.trim,T=h.type;T==="n"?(w=!0,T="g"):L0[T]||(b===void 0&&(b=12),S=!0,T="g"),(m||d==="0"&&f==="=")&&(m=!0,d="0",f="=");var E=v==="$"?e:v==="#"&&/[boxX]/.test(T)?"0"+T.toLowerCase():"",P=v==="$"?r:/[%p]/.test(T)?a:"",R=L0[T],M=/[defgprs%]/.test(T);b===void 0?b=6:/[gprs]/.test(T)?b=Math.max(1,Math.min(21,b)):b=Math.max(0,Math.min(20,b));function N(W){var G=E,k=P,V;if(T==="c")k=R(+W)+k,V="";else{var U=+W,J=W<0||1/U<0;V=isNaN(U)?l:R(Math.abs(U),b),S&&(V=uQ(V)),J&&+V==0&&g!=="+"&&(J=!1);var Z=J?g==="("?g:s:g==="-"||g==="("?"":g,le=J&&g==="("?")":"";if(G=Z+G,k=(T==="s"?N0[8+Ky/3]:"")+k+le,M)for(var Y=0,ae=V.length;Y<ae;Y++){var z=V.charCodeAt(Y);if(48>z||z>57){k=(z===46?n+V.slice(Y+1):V.slice(Y))+k,V=V.slice(0,Y);break}}}w&&!m&&(V=t(V,1/0));var X=G.length+V.length+k.length,ue=X<C?new Array(C-X+1).join(d):"";switch(w&&m&&(V=t(ue+V,ue.length?C-k.length:1/0),ue=""),f){case"<":V=G+V+k+ue;break;case"=":V=G+ue+V+k;break;case"^":V=ue.slice(0,X=ue.length>>1)+G+V+k+ue.slice(X);break;default:V=ue+G+V+k;break}var pe=h.string;return pe?pe.replace(gD,function(){return o(V)}):o(V)}return N}function c(p,h){var d=$y(p);d.type="f";var f=u(d),g=Math.max(-8,Math.min(8,Math.floor(kl(h)/3)))*3,v=Math.pow(10,-g),m=N0[8+g/3];return function(C){return f(v*+C)+m}}return{format:u,formatPrefix:c}}var gQ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),am=function(i){gQ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.type="linear",e}return t.prototype.ticks=function(e){e===void 0&&(e=10);var r=this._domain;return JZ(r[0],r[r.length-1],e)},t.prototype.nice=function(e){e===void 0&&(e=10);var r=this.domain,n=0,o=r.length-1,a=r[n],s=r[o],l;s<a&&(l=a,a=s,s=l,l=n,n=o,o=l),l=nm(a,s,e),l>0?(a=Math.floor(a/l)*l,s=Math.ceil(s/l)*l,l=nm(a,s,e)):l<0&&(a=Math.ceil(a*l)/l,s=Math.floor(s*l)/l,l=nm(a,s,e)),l>0?(r[n]=Math.floor(a/l)*l,r[o]=Math.ceil(s/l)*l,this.domain=r):l<0&&(r[n]=Math.ceil(a*l)/l,r[o]=Math.floor(s*l)/l,this.domain=r)},t.prototype.tickFormat=function(e,r){var n=this.domain;return iQ(n[0],n[n.length-1],e==null?10:e,r)},t}(fD),vQ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),sm=new Date,lm=new Date,mQ=function(){function i(t,e){this._floor=t,this._offset=e}return i.prototype.floor=function(t){return t=new Date(+t),this._floor(t),t},i.prototype.ceil=function(t){return t=new Date(+t-1),this._floor(t),this._offset(t,1),this._floor(t),t},i.prototype.round=function(t){var e=this.floor(t),r=this.ceil(t),n=+t;return n-e.getTime()<r.getTime()-n?e:r},i.prototype.offset=function(t,e){return e===void 0&&(e=1),t=new Date(+t),this._offset(t,Math.floor(e)),t},i.prototype.range=function(t,e,r){r===void 0&&(r=1);var n=[];if(t=this.ceil(t),r=Math.floor(r),t>e||r<=0)return n;var o;do o=new Date(+t),n.push(o),this._offset(t,r),this._floor(t);while(o<t&&t<e);return n},i.prototype.filter=function(t){var e=this,r=function(o){if(o instanceof Date)for(e._floor(o);!t(o);)o.setTime(o.getTime()-1),e._floor(o);return o},n=function(o,a){if(o instanceof Date)if(a<0)for(;++a<=0;)do e._offset(o,-1);while(!t(o));else for(;--a>=0;)do e._offset(o,1);while(!t(o));return o};return new i(r,n)},i}(),si=function(i){vQ(t,i);function t(e,r,n,o){var a=i.call(this,e,r)||this;return a._count=n,a._field=o,a}return t.prototype.count=function(e,r){return sm.setTime(+e),lm.setTime(+r),this._floor(sm),this._floor(lm),Math.floor(this._count(sm,lm))},t.prototype.every=function(e){var r=this,n;if(e=Math.floor(e),isFinite(e)&&e>0)if(e>1){var o=this._field;o?n=this.filter(function(a){return o(a)%e===0}):n=this.filter(function(a){return r.count(0,a)%e===0})}else n=this;return n},t}(mQ);function yQ(i){return i}function CQ(i,t){i.setTime(i.getTime()+t)}function wQ(i,t){return t.getTime()-i.getTime()}var bQ=new si(yQ,CQ,wQ),eo=1e3,Pi=eo*60,Ja=Pi*60,vs=Ja*24,gb=vs*7,W0=vs*30,um=vs*365;function _Q(i){i.setTime(i.getTime()-i.getMilliseconds())}function SQ(i,t){i.setTime(i.getTime()+t*eo)}function xQ(i,t){return(t.getTime()-i.getTime())/eo}var TQ=new si(_Q,SQ,xQ);function OQ(i){i.setTime(i.getTime()-i.getMilliseconds()-i.getSeconds()*eo)}function EQ(i,t){i.setTime(i.getTime()+t*Pi)}function RQ(i,t){return(t.getTime()-i.getTime())/Pi}function PQ(i){return i.getMinutes()}var AQ=new si(OQ,EQ,RQ,PQ);function DQ(i){i.setTime(i.getTime()-i.getMilliseconds()-i.getSeconds()*eo-i.getMinutes()*Pi)}function MQ(i,t){i.setTime(i.getTime()+t*Ja)}function FQ(i,t){return(t.getTime()-i.getTime())/Ja}function IQ(i){return i.getHours()}var LQ=new si(DQ,MQ,FQ,IQ);function NQ(i){i.setHours(0,0,0,0)}function GQ(i,t){i.setDate(i.getDate()+t)}function kQ(i,t){var e=t.getTimezoneOffset()-i.getTimezoneOffset();return(t.getTime()-i.getTime()-e*Pi)/vs}function WQ(i){return i.getDate()-1}var Yy=new si(NQ,GQ,kQ,WQ);function Rs(i){function t(n){n.setDate(n.getDate()-(n.getDay()+7-i)%7),n.setHours(0,0,0,0)}function e(n,o){n.setDate(n.getDate()+o*7)}function r(n,o){var a=o.getTime()-n.getTime(),s=o.getTimezoneOffset()-n.getTimezoneOffset();return(a-s*Pi)/gb}return new si(t,e,r)}var yD=Rs(0),cm=Rs(1);Rs(2);Rs(3);var pm=Rs(4);Rs(5);Rs(6);function VQ(i){i.setDate(1),i.setHours(0,0,0,0)}function BQ(i,t){i.setMonth(i.getMonth()+t)}function HQ(i,t){return t.getMonth()-i.getMonth()+(t.getFullYear()-i.getFullYear())*12}function jQ(i){return i.getMonth()}var UQ=new si(VQ,BQ,HQ,jQ);function zQ(i){i.setMonth(0,1),i.setHours(0,0,0,0)}function $Q(i,t){i.setFullYear(i.getFullYear()+t)}function KQ(i,t){return t.getFullYear()-i.getFullYear()}function YQ(i){return i.getFullYear()}var tc=new si(zQ,$Q,KQ,YQ);function XQ(i,t){return i<t?-1:i>t?1:i>=t?0:NaN}function qQ(i,t,e,r,n){r===void 0&&(r=0),n===void 0&&(n=i.length);for(var o=ZQ(e);r<n;){var a=r+n>>>1;o(i[a],t)<0?r=a+1:n=a}return r}function ZQ(i){return function(t,e){return XQ(i(t),e)}}function QQ(i){i.setUTCHours(0,0,0,0)}function JQ(i,t){i.setUTCDate(i.getUTCDate()+t)}function eJ(i,t){return(t.getTime()-i.getTime())/vs}function tJ(i){return i.getUTCDate()-1}var V0=new si(QQ,JQ,eJ,tJ);function rJ(i){i.setUTCMonth(0,1),i.setUTCHours(0,0,0,0)}function nJ(i,t){i.setUTCFullYear(i.getUTCFullYear()+t)}function iJ(i,t){return t.getUTCFullYear()-i.getUTCFullYear()}function oJ(i){return i.getUTCFullYear()}var gh=new si(rJ,nJ,iJ,oJ);function Ps(i){function t(n){n.setUTCDate(n.getUTCDate()-(n.getUTCDay()+7-i)%7),n.setHours(0,0,0,0)}function e(n,o){n.setUTCDate(n.getUTCDate()+o*7)}function r(n,o){return(o.getTime()-n.getTime())/gb}return new si(t,e,r)}var aJ=Ps(0),hm=Ps(1);Ps(2);Ps(3);var dm=Ps(4);Ps(5);Ps(6);function sJ(i){if(i.y>=0&&i.y<100){var t=new Date(-1,i.m,i.d,i.H,i.M,i.S,i.L);return t.setFullYear(i.y),t}return new Date(i.y,i.m,i.d,i.H,i.M,i.S,i.L)}function vh(i){if(i.y>=0&&i.y<100){var t=new Date(Date.UTC(-1,i.m,i.d,i.H,i.M,i.S,i.L));return t.setUTCFullYear(i.y),t}return new Date(Date.UTC(i.y,i.m,i.d,i.H,i.M,i.S,i.L))}function ju(i){for(var t={},e=0,r=i.length;e<r;e++)t[i[e].toLowerCase()]=e;return t}function Uu(i){return{y:i,m:0,d:1,H:0,M:0,S:0,L:0}}var B0=37,tr=/^\s*\d+/,lJ=/^%/,uJ=/[\\^$*+?|[\]().{}]/g,cJ=function(i){return i.replace(uJ,"\\$&")},zu=function(i){return new RegExp("^(?:"+i.map(cJ).join("|")+")","i")},H0={"-":"",_:" ","0":"0"};function at(i,t,e){var r=i<0?"-":"",n=String(r?-i:i),o=n.length;return r+(o<e?new Array(e-o+1).join(t)+n:n)}function pJ(i){var t=i.dateTime,e=i.date,r=i.time,n=i.periods,o=i.days,a=i.shortDays,s=i.months,l=i.shortMonths,u=zu(n),c=ju(n),p=zu(o),h=ju(o),d=zu(a),f=ju(a),g=zu(s),v=ju(s),m=zu(l),C=ju(l),w={a:U,A:J,b:pe,B:be,c:void 0,d:X,e:X,f:R,H:k,I:G,j:ue,L:M,m:_e,M:W,p:V,Q:bo,s:Qe,S:N,u:Z,U:le,V:Y,w:ae,W:z,x:void 0,X:void 0,y:Te,Y:Pe,Z:Ee,"%":ga},b={a:mn,A:Yr,b:qe,B:it,c:void 0,d:Me,e:Me,f:Le,H:ie,I:Xe,j:ne,L:Ze,m:ke,M:Ge,p:He,Q:bo,s:Qe,S:Re,u:xr,U:Xr,V:Bn,w:yo,W:Co,x:void 0,X:void 0,y:Gi,Y:wo,Z:fa,"%":ga},S={a:If,A:Lf,b:Bf,B:Hf,c:Uf,d:ru,e:ru,f:yt,H:va,I:va,j:Ff,L:We,m:jf,M:pr,p:Mf,Q:Kf,s:Yf,S:St,u:Nf,U:Gf,V:kf,w:Vf,W:Wf,x:zf,X:$f,y:Xf,Y:qf,Z:Zf,"%":Qf};w.x=E(e,w),w.X=E(r,w),w.c=E(t,w),b.x=E(e,b),b.X=E(r,b),b.c=E(t,b);function T(O,D){return function(F){var A=Uu(1900),Zt=P(A,O,F+="",0);if(Zt==F.length){if("Q"in A)return new Date(A.Q);if("p"in A&&(A.H=A.H%12+A.p*12),"V"in A){if(A.V<1||A.V>53)return;if("w"in A||(A.w=1),"Z"in A){var ye=vh(Uu(A.y)),Je=ye.getUTCDay();ye=Je>4||Je===0?hm.ceil(ye):hm.floor(ye),ye=V0.offset(ye,(A.V-1)*7),A.y=ye.getUTCFullYear(),A.m=ye.getUTCMonth(),A.d=ye.getUTCDate()+(A.w+6)%7}else{var ye=D(Uu(A.y)),Je=ye.getDay();ye=Je>4||Je===0?cm.ceil(ye):cm.floor(ye),ye=Yy.offset(ye,(A.V-1)*7),A.y=ye.getFullYear(),A.m=ye.getMonth(),A.d=ye.getDate()+(A.w+6)%7}}else if("W"in A||"U"in A){"w"in A||(A.w="u"in A?A.u%7:"W"in A?1:0);var Je="Z"in A?vh(Uu(A.y)).getUTCDay():D(Uu(A.y)).getDay();A.m=0,A.d="W"in A?(A.w+6)%7+A.W*7-(Je+5)%7:A.w+A.U*7-(Je+6)%7}return"Z"in A?(A.H+=A.Z/100|0,A.M+=A.Z%100,vh(A)):D(A)}}}function E(O,D){return function(F){var A=[],Zt=O.length,ye=-1,Je=0;for(F instanceof Date||(F=new Date(+F));++ye<Zt;)if(O.charCodeAt(ye)===B0){A.push(O.slice(Je,ye));var Gt=O.charAt(++ye),qr=H0[Gt];qr!=null?Gt=O.charAt(++ye):qr=Gt==="e"?" ":"0";var Hn=D[Gt];Hn&&(Gt=Hn(F,qr)),A.push(Gt),Je=ye+1}return A.push(O.slice(Je,ye)),A.join("")}}function P(O,D,F,A){for(var Zt=0,ye=D.length,Je=F.length;Zt<ye;){if(A>=Je)return-1;var Gt=D.charCodeAt(Zt++);if(Gt===B0){var qr=D.charAt(Zt++),Hn=S[qr in H0?D.charAt(Zt++):qr];if(!Hn||(A=Hn(O,F,A))<0)return-1}else if(Gt!=F.charCodeAt(A++))return-1}return A}function R(O,D){return M(O,D)+"000"}function M(O,D){return at(O.getMilliseconds(),D,3)}function N(O,D){return at(O.getSeconds(),D,2)}function W(O,D){return at(O.getMinutes(),D,2)}function G(O,D){return at(O.getHours()%12||12,D,2)}function k(O,D){return at(O.getHours(),D,2)}function V(O){return n[O.getHours()>=12?1:0]}function U(O){return a[O.getDay()]}function J(O){return o[O.getDay()]}function Z(O){var D=O.getDay();return D===0?7:D}function le(O,D){return at(yD.count(tc.floor(O),O),D,2)}function Y(O,D){var F=O.getDay();O=F>=4||F===0?pm.floor(O):pm.ceil(O);var A=tc.floor(O);return at(pm.count(A,O)+(A.getDay()===4?1:0),D,2)}function ae(O){return O.getDay()}function z(O,D){return at(cm.count(tc.floor(O),O),D,2)}function X(O,D){return at(O.getDate(),D,2)}function ue(O,D){return at(1+Yy.count(tc.floor(O),O),D,3)}function pe(O){return l[O.getMonth()]}function be(O){return s[O.getMonth()]}function _e(O,D){return at(O.getMonth()+1,D,2)}function Te(O,D){return at(O.getFullYear()%100,D,2)}function Pe(O,D){return at(O.getFullYear()%1e4,D,4)}function Ee(O){var D=O.getTimezoneOffset();return(D>0?"-":(D*=-1,"+"))+at(Math.floor(D/60),"0",2)+at(D%60,"0",2)}function Le(O,D){return Ze(O,D)+"000"}function Ze(O,D){return at(O.getUTCMilliseconds(),D,3)}function Re(O,D){return at(O.getUTCSeconds(),D,2)}function Ge(O,D){return at(O.getUTCMinutes(),D,2)}function Xe(O,D){return at(O.getUTCHours()%12||12,D,2)}function ie(O,D){return at(O.getUTCHours(),D,2)}function He(O){return n[O.getUTCHours()>=12?1:0]}function Me(O,D){return at(O.getUTCDate(),D,2)}function ne(O,D){return at(1+V0.count(gh.floor(O),O),D,3)}function ke(O,D){return at(O.getUTCMonth()+1,D,2)}function qe(O){return l[O.getUTCMonth()]}function it(O){return s[O.getUTCMonth()]}function mn(O){return a[O.getUTCDay()]}function Yr(O){return o[O.getUTCDay()]}function xr(O){var D=O.getUTCDay();return D===0?7:D}function Xr(O,D){return at(aJ.count(gh.floor(O),O),D,2)}function Bn(O,D){var F=O.getUTCDay();O=F>=4||F===0?dm.floor(O):dm.ceil(O);var A=gh.floor(O);return at(dm.count(A,O)+(A.getUTCDay()===4?1:0),D,4)}function yo(O){return O.getUTCDay()}function Co(O,D){return at(hm.count(gh.floor(O),O),D,2)}function Gi(O,D){return at(O.getUTCFullYear()%100,D,2)}function wo(O,D){return at(O.getUTCFullYear()%1e4,D,4)}function fa(){return"+0000"}function ga(O){return"%"}function bo(O){return O.getTime()}function Qe(O){return Math.floor(O.getTime()/1e3)}function yt(O,D,F){var A=tr.exec(D.slice(F,F+6));return A?(O.L=Math.floor(parseFloat(A[0])/1e3),F+A[0].length):-1}function We(O,D,F){var A=tr.exec(D.slice(F,F+3));return A?(O.L=+A[0],F+A[0].length):-1}function St(O,D,F){var A=tr.exec(D.slice(F,F+2));return A?(O.S=+A[0],F+A[0].length):-1}function pr(O,D,F){var A=tr.exec(D.slice(F,F+2));return A?(O.M=+A[0],F+A[0].length):-1}function va(O,D,F){var A=tr.exec(D.slice(F,F+2));return A?(O.H=+A[0],F+A[0].length):-1}function Mf(O,D,F){var A=u.exec(D.slice(F));return A?(O.p=c[A[0].toLowerCase()],F+A[0].length):-1}function ru(O,D,F){var A=tr.exec(D.slice(F,F+2));return A?(O.d=+A[0],F+A[0].length):-1}function Ff(O,D,F){var A=tr.exec(D.slice(F,F+3));return A?(O.m=0,O.d=+A[0],F+A[0].length):-1}function If(O,D,F){var A=d.exec(D.slice(F));return A?(O.w=f[A[0].toLowerCase()],F+A[0].length):-1}function Lf(O,D,F){var A=p.exec(D.slice(F));return A?(O.w=h[A[0].toLowerCase()],F+A[0].length):-1}function Nf(O,D,F){var A=tr.exec(D.slice(F,F+1));return A?(O.u=+A[0],F+A[0].length):-1}function Gf(O,D,F){var A=tr.exec(D.slice(F,F+2));return A?(O.U=+A[0],F+A[0].length):-1}function kf(O,D,F){var A=tr.exec(D.slice(F,F+2));return A?(O.V=+A[0],F+A[0].length):-1}function Wf(O,D,F){var A=tr.exec(D.slice(F,F+2));return A?(O.W=+A[0],F+A[0].length):-1}function Vf(O,D,F){var A=tr.exec(D.slice(F,F+1));return A?(O.w=+A[0],F+A[0].length):-1}function Bf(O,D,F){var A=m.exec(D.slice(F));return A?(O.m=C[A[0].toLowerCase()],F+A[0].length):-1}function Hf(O,D,F){var A=g.exec(D.slice(F));return A?(O.m=v[A[0].toLowerCase()],F+A[0].length):-1}function jf(O,D,F){var A=tr.exec(D.slice(F,F+2));return A?(O.m=parseFloat(A[0])-1,F+A[0].length):-1}function Uf(O,D,F){return P(O,t,D,F)}function zf(O,D,F){return P(O,e,D,F)}function $f(O,D,F){return P(O,r,D,F)}function Kf(O,D,F){var A=tr.exec(D.slice(F));return A?(O.Q=+A[0],F+A[0].length):-1}function Yf(O,D,F){var A=tr.exec(D.slice(F));return A?(O.Q=+A[0]*1e3,F+A[0].length):-1}function Xf(O,D,F){var A=tr.exec(D.slice(F,F+2));return A?(O.y=+A[0]+(+A[0]>68?1900:2e3),F+A[0].length):-1}function qf(O,D,F){var A=tr.exec(D.slice(F,F+4));return A?(O.y=+A[0],F+A[0].length):-1}function Zf(O,D,F){var A=/^(Z)|^([+-]\d\d)(?::?(\d\d))?/.exec(D.slice(F,F+6));return A?(O.Z=A[1]?0:-(A[2]+(A[3]||"00")),F+A[0].length):-1}function Qf(O,D,F){var A=lJ.exec(D.slice(F,F+1));return A?F+A[0].length:-1}return{format:function(O){var D=E(O,w);return D.toString=function(){return O},D},parse:function(O){var D=T(O,sJ);return D.toString=function(){return O},D},utcFormat:function(O){var D=E(O,b);return D.toString=function(){return O},D},utcParse:function(O){var D=T(O,vh);return D.toString=function(){return O},D}}}var vb;hJ({dateTime:"%x, %X",date:"%-m/%-d/%Y",time:"%-I:%M:%S %p",periods:["AM","PM"],days:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],shortDays:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],months:["January","February","March","April","May","June","July","August","September","October","November","December"],shortMonths:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]});function hJ(i){return vb=pJ(i)}var dJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),fm=globalThis&&globalThis.__read||function(i,t){var e=typeof Symbol=="function"&&i[Symbol.iterator];if(!e)return i;var r=e.call(i),n,o=[],a;try{for(;(t===void 0||t-- >0)&&!(n=r.next()).done;)o.push(n.value)}catch(s){a={error:s}}finally{try{n&&!n.done&&(e=r.return)&&e.call(r)}finally{if(a)throw a.error}}return o},j0=function(i){dJ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.type="time",e.year=tc,e.month=UQ,e.week=yD,e.day=Yy,e.hour=LQ,e.minute=AQ,e.second=TQ,e.millisecond=bQ,e.format=vb.format,e.tickIntervals=[[e.second,1,eo],[e.second,5,5*eo],[e.second,15,15*eo],[e.second,30,30*eo],[e.minute,1,Pi],[e.minute,5,5*Pi],[e.minute,15,15*Pi],[e.minute,30,30*Pi],[e.hour,1,Ja],[e.hour,3,3*Ja],[e.hour,6,6*Ja],[e.hour,12,12*Ja],[e.day,1,vs],[e.day,2,2*vs],[e.week,1,gb],[e.month,1,W0],[e.month,3,3*W0],[e.year,1,um]],e.formatMillisecond=e.format(".%L"),e.formatSecond=e.format(":%S"),e.formatMinute=e.format("%I:%M"),e.formatHour=e.format("%I %p"),e.formatDay=e.format("%a %d"),e.formatWeek=e.format("%b %d"),e.formatMonth=e.format("%B"),e.formatYear=e.format("%Y"),e._domain=[new Date(2e3,0,1),new Date(2e3,0,2)],e}return t.prototype.defaultTickFormat=function(e){return(this.second.floor(e)<e?this.formatMillisecond:this.minute.floor(e)<e?this.formatSecond:this.hour.floor(e)<e?this.formatMinute:this.day.floor(e)<e?this.formatHour:this.month.floor(e)<e?this.week.floor(e)<e?this.formatDay:this.formatWeek:this.year.floor(e)<e?this.formatMonth:this.formatYear)(e)},t.prototype.tickInterval=function(e,r,n,o){var a;if(typeof e=="number"){var s=e,l=this.tickIntervals,u=Math.abs(n-r)/s,c=qQ(l,u,function(p){return p[2]});c===l.length?(o=Hd(r/um,n/um,s),e=this.year):c?(a=fm(l[u/l[c-1][2]<l[c][2]/u?c-1:c],2),e=a[0],o=a[1]):(o=Math.max(Hd(r,n,e),1),e=this.millisecond)}return o==null?e:e.every(o)},Object.defineProperty(t.prototype,"domain",{get:function(){return i.prototype.getDomain.call(this).map(function(e){return new Date(e)})},set:function(e){i.prototype.setDomain.call(this,Array.prototype.map.call(e,function(r){return r instanceof Date?+r:+new Date(+r)}))},enumerable:!0,configurable:!0}),t.prototype.invert=function(e){return new Date(i.prototype.invert.call(this,e))},t.prototype.ticks=function(e){e===void 0&&(e=10);var r=i.prototype.getDomain.call(this),n=r[0],o=r[r.length-1],a=o<n;if(a){var s=n;n=o,o=s}var l=this.tickInterval(e,n,o),u=l?l.range(n,o+1):[];return a?u.reverse():u},t.prototype.tickFormat=function(e,r){return r==null?this.defaultTickFormat.bind(this):this.format(r)},t.prototype.nice=function(e){e===void 0&&(e=10);var r=i.prototype.getDomain.call(this),n=this.tickInterval(e,r[0],r[r.length-1]);n&&(this.domain=this._nice(r,n))},t.prototype._nice=function(e,r){var n,o;e=e.slice();var a=0,s=e.length-1,l=e[a],u=e[s];return u<l&&(n=fm([s,a],2),a=n[0],s=n[1],o=fm([u,l],2),l=o[0],u=o[1]),e[a]=r.floor(l),e[s]=r.ceil(u),e},t}(fD);globalThis&&globalThis.__values;function fJ(i){return i}function Vc(i,t,e){for(var r=e||fJ,n=i.length,o=-1,a,s,l;++o<n;)if(a=i[o],t(a))for(s=l=a;++o<n;)a=i[o],t(a)&&(s>a&&(s=a),l<a&&(l=a));return s===void 0||l===void 0?void 0:[r(s),r(l)]}var CD=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),gJ=globalThis&&globalThis.__values||function(i){var t=typeof Symbol=="function"&&Symbol.iterator,e=t&&i[t],r=0;if(e)return e.call(i);if(i&&typeof i.length=="number")return{next:function(){return i&&r>=i.length&&(i=void 0),{value:i&&i[r++],done:!i}}};throw new TypeError(t?"Object is not iterable.":"Symbol.iterator is not defined.")},vJ=function(i){CD(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.type="category",e.stroke="rgb(204, 214, 235)",e.strokeWidth=1,e}return t}(up),mb=function(i){CD(t,i);function t(){var e=i.call(this)||this;e.id=db(e),e.seriesRect={x:0,y:0,width:0,height:0},e._context=void 0,e._container=void 0,e._data=void 0,e.padding=new VZ(3),e.xKey="x",e.yKey="y",e.dataType=void 0,e.xData=[],e.yData=[],e.min=void 0,e.max=void 0,e.yScale=new am,e.axis=new vJ,e.highlightStyle={size:6,fill:"yellow",stroke:"silver",strokeWidth:1},e._width=100,e._height=100,e.smallestInterval=void 0,e.layoutId=0,e.defaultDateFormatter=vb.format("%m/%d/%y, %H:%M:%S"),e._onMouseMove=e.onMouseMove.bind(e),e._onMouseOut=e.onMouseOut.bind(e);var r=new oa;e.rootGroup=r;var n=document.createElement("div");n.setAttribute("class","ag-sparkline-wrapper");var o=new WZ(document);if(e.scene=o,e.canvasElement=o.canvas.element,o.root=r,o.container=n,o.resize(e.width,e.height),e.seriesRect.width=e.width,e.seriesRect.height=e.height,t.tooltipDocuments.indexOf(document)===-1){var a=document.createElement("style");a.innerHTML=BZ,document.head.insertBefore(a,document.head.querySelector("style")),t.tooltipDocuments.push(document)}return e.setupDomEventListeners(e.scene.canvas.element),e}return Object.defineProperty(t.prototype,"context",{get:function(){return this._context},set:function(e){this._context!==e&&(this._context=e)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"container",{get:function(){return this._container},set:function(e){if(this._container!==e){var r=this.canvasElement.parentNode;r!=null&&r.removeChild(this.canvasElement),e&&e.appendChild(this.canvasElement),this._container=e}},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"data",{get:function(){return this._data},set:function(e){this._data!==e&&(this._data=e,this.processData())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this._width},set:function(e){this._width!==e&&(this._width=e,this.scene.resize(e,this.height),this.scheduleLayout())},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return this._height},set:function(e){this._height!==e&&(this._height=e,this.scene.resize(this.width,e),this.scheduleLayout())},enumerable:!0,configurable:!0}),t.prototype.update=function(){},t.prototype.updateYScale=function(){this.updateYScaleRange(),this.updateYScaleDomain()},t.prototype.updateYScaleDomain=function(){},t.prototype.updateYScaleRange=function(){var e=this,r=e.yScale,n=e.seriesRect;r.range=[n.height,0]},t.prototype.updateXScale=function(){var e=this.axis.type;this.xScale=this.getXScale(e),this.updateXScaleRange(),this.updateXScaleDomain()},t.prototype.updateXScaleRange=function(){this.xScale.range=[0,this.seriesRect.width]},t.prototype.updateXScaleDomain=function(){var e=this,r=e.xData,n=e.xScale,o;n instanceof am?o=Vc(r,Ur):n instanceof j0&&(o=Vc(r,zZ)),this.xScale.domain=o?o.slice():r},t.prototype.getXScale=function(e){switch(e===void 0&&(e="category"),e){case"number":return new am;case"time":return new j0;case"category":default:return new pa}},t.prototype.updateAxisLine=function(){},t.prototype.updateAxes=function(){this.updateYScale(),this.updateXScale(),this.updateAxisLine()},t.prototype.updateCrosshairs=function(){this.updateXCrosshairLine(),this.updateYCrosshairLine()},t.prototype.generateNodeData=function(){return[]},t.prototype.getNodeData=function(){return[]},t.prototype.updateNodes=function(){},t.prototype.updateXCrosshairLine=function(){},t.prototype.updateYCrosshairLine=function(){},t.prototype.highlightDatum=function(e){this.updateNodes()},t.prototype.dehighlightDatum=function(){this.highlightedDatum=void 0,this.updateNodes(),this.updateCrosshairs()},t.prototype.onMouseMove=function(e){var r=this.pickClosestSeriesNodeDatum(e.offsetX,e.offsetY);if(!!r){var n=this.highlightedDatum;this.highlightedDatum=r,(this.highlightedDatum&&!n||this.highlightedDatum&&n&&this.highlightedDatum!==n)&&(this.highlightDatum(r),this.updateCrosshairs()),this.tooltip.enabled&&this.handleTooltip(e,r)}},t.prototype.onMouseOut=function(e){this.dehighlightDatum(),this.tooltip.toggle(!1)},t.prototype.processData=function(){var e=this,r=this,n=r.data,o=r.yData,a=r.xData;if(!(!n||this.invalidData(this.data))){o.length=0,a.length=0;var s=n.length,l=this.getDataType(n);this.dataType=l;var u=this.axis.type,c=u!=="number"&&u!=="time"?"category":u,p=c==="number"||c==="time",h=function(E,P){e.smallestInterval==null&&(e.smallestInterval={x:1/0,y:1/0});var R=e.smallestInterval.x,M=Math.abs(E-P);M>0&&M<R&&(e.smallestInterval.x=M)},d;if(l==="number")for(var f=0;f<s;f++){var g=f,v=n[f],m=this.getDatum(g,c),C=this.getDatum(v,"number");p&&h(m,d),a.push(m),o.push(C),d=m}else if(l==="array")for(var f=0;f<s;f++){var w=n[f];if(Array.isArray(w)){var g=w[0],v=w[1],m=this.getDatum(g,c),C=this.getDatum(v,"number");if(m==null)continue;p&&h(m,d),a.push(m),o.push(C),d=m}}else if(l==="object")for(var b=this,S=b.yKey,T=b.xKey,f=0;f<s;f++){var w=n[f];if(typeof w=="object"&&!Array.isArray(w)){var g=w[T],v=w[S],m=this.getDatum(g,c),C=this.getDatum(v,"number");if(m==null)continue;p&&h(m,d),a.push(m),o.push(C),d=m}}this.updateAxes(),this.update()}},t.prototype.getDataType=function(e){var r,n;try{for(var o=gJ(e),a=o.next();!a.done;a=o.next()){var s=a.value;if(s!=null){if(Ur(s))return"number";if(Array.isArray(s))return"array";if(typeof s=="object")return"object"}}}catch(l){r={error:l}}finally{try{a&&!a.done&&(n=o.return)&&n.call(o)}finally{if(r)throw r.error}}},t.prototype.getDatum=function(e,r){if(r==="number"&&Ur(e)||r==="time"&&(Ur(e)||Hy(e)))return e;if(r==="category"){if(dD(e)||Hy(e)||Ur(e))return{toString:function(){return String(e)}};if(UZ(e))return e}},Object.defineProperty(t.prototype,"layoutScheduled",{get:function(){return!!this.layoutId},enumerable:!0,configurable:!0}),t.prototype.scheduleLayout=function(){var e=this;this.layoutId&&cancelAnimationFrame(this.layoutId),this.layoutId=requestAnimationFrame(function(){e.setSparklineDimensions(),!e.invalidData(e.data)&&(e.updateXScaleRange(),e.updateYScaleRange(),e.updateAxisLine(),e.update(),e.layoutId=0)})},t.prototype.setSparklineDimensions=function(){var e=this,r=e.width,n=e.height,o=e.padding,a=e.seriesRect,s=e.rootGroup,l=r-o.left-o.right,u=n-o.top-o.bottom;a.width=l,a.height=u,a.x=o.left,a.y=o.top,s.translationX=a.x,s.translationY=a.y},t.prototype.pickClosestSeriesNodeDatum=function(e,r){for(var n=1/0,o,a=this.rootGroup.transformPoint(e,r),s=this.getNodeData(),l=0;l<s.length;l++){var u=s[l];if(!u.point)return;var c=this.getDistance(a,u.point);if(c<=n)n=c,o=u;else return o}return o},t.prototype.getDistance=function(e,r){return Math.abs(e.x-r.x)},t.prototype.handleTooltip=function(e,r){var n=r.seriesDatum,o=this.canvasElement,a=e.clientX,s=e.clientY;this.tooltip.container==null&&(this.tooltip.container=o);var l={pageX:a,pageY:s},u=n.y,c=n.x,p=this.tooltip.enabled;if(this.tooltip.renderer){var h=this.tooltip.renderer({context:this.context,datum:n,yValue:u,xValue:c});p=typeof h!="string"&&h.enabled!==void 0?h.enabled:p}var d=p&&n.y!==void 0&&this.getTooltipHtml(r);d&&this.tooltip.show(l,d)},t.prototype.formatNumericDatum=function(e){return String(Math.round(e*10)/10)},t.prototype.formatDatum=function(e){var r=this.axis.type||"category";return r==="number"&&typeof e=="number"?this.formatNumericDatum(e):r==="time"&&(e instanceof Date||Ur(e))?this.defaultDateFormatter(e):String(e)},t.prototype.setupDomEventListeners=function(e){e.addEventListener("mousemove",this._onMouseMove),e.addEventListener("mouseout",this._onMouseOut)},t.prototype.cleanupDomEventListeners=function(e){e.removeEventListener("mousemove",this._onMouseMove),e.removeEventListener("mouseout",this._onMouseOut)},t.prototype.invalidData=function(e){return!e||!Array.isArray(e)||e.length===0},t.prototype.destroy=function(){this.scene.container=void 0,this.container=void 0,this.cleanupDomEventListeners(this.scene.canvas.element)},t.tooltipDocuments=[],t}(up),mJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}();function Wl(i,t){if(typeof i=="string")return i;t=t||{};var e=i.content,r=e===void 0?t.content||"":e,n=i.title,o=n===void 0?t.title||void 0:n,a=i.color,s=a===void 0?t.color:a,l=i.backgroundColor,u=l===void 0?t.backgroundColor:l,c=i.opacity,p=c===void 0?t.opacity||1:c,h,d;if(s?(h=o?'<span class="'+Wa.class+'-title"; style="color: '+s+'">'+o+"</span>":"",d='<span class="'+Wa.class+'-content" style="color: '+s+'">'+r+"</span>"):(h=o?'<span class="'+Wa.class+'-title">'+o+"</span>":"",d='<span class="'+Wa.class+'-content">'+r+"</span>"),u){var f=Zi.fromString(u.toLowerCase()),g=f.r,v=f.g,m=f.b;f.a;var C=p,w=Zi.fromArray([g,v,m,C]),b=w.toRgbaString();return'<div class="'+Wa.class+'" style="background-color: '+b+`">
                    `+h+`
                    `+d+`
                </div>`}else return'<div class="'+Wa.class+`">
                    `+h+`
                    `+d+`
                </div>`}var Wa=function(i){mJ(t,i);function t(){var e=i.call(this)||this;e.element=document.createElement("div"),e.enabled=!0,e.container=void 0,e.xOffset=10,e.yOffset=0,e.renderer=void 0;var r=document.body;return r.appendChild(e.element),e}return t.prototype.isVisible=function(){var e=this.element;if(e.classList)return!e.classList.contains(t.class+"-wrapper-hidden");var r=e.getAttribute("class");return r?r.split(" ").indexOf(t.class+"-wrapper-hidden")<0:!1},t.prototype.updateClass=function(e){var r=[t.class+"-wrapper"];e!==!0&&r.push(t.class+"-wrapper-hidden"),this.element.setAttribute("class",r.join(" "))},t.prototype.show=function(e,r){this.toggle(!1);var n=this.element;if(r!==void 0)n.innerHTML=r;else if(!n.innerHTML)return;var o=e.pageX+this.xOffset,a=e.pageY+this.yOffset,s=n.getBoundingClientRect(),l=window.innerWidth-s.width;if(this.container){var u=this.container.getBoundingClientRect();l=u.left+(u.width-s.width)}o>l&&(o=e.pageX-n.clientWidth-this.xOffset),n.style.left=Math.round(o)+"px",n.style.top=Math.round(a)+"px",this.toggle(!0)},t.prototype.toggle=function(e){this.updateClass(e)},t.prototype.destroy=function(){var e=this.element.parentNode;e&&e.removeChild(this.element)},t.class="ag-sparkline-tooltip",t}(up),yJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),yb=function(i){yJ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._x=0,e._y=0,e._size=3,e}return Object.defineProperty(t.prototype,"x",{get:function(){return this._x},set:function(e){this._x!==e&&(this._x=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this._y},set:function(e){this._y!==e&&(this._y=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"size",{get:function(){return this._size},set:function(e){this._size!==e&&(this._size=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t}(go),CJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),U0=function(i){CJ(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.isPointInPath=function(e,r){return!1},t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this,n=r.x,o=r.y,a=r.size,s=a/2;e.beginPath(),e.arc(n,o,s,0,2*Math.PI,!1),this.fillStroke(e),this.dirty=!1},t.className="Circle",t}(yb),wJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),bJ=function(i){wJ(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.isPointInPath=function(e,r){return!1},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this,n=r.x,o=r.y,a=r.size,s=a/2;e.beginPath(),e.moveTo(n,o-=s),e.lineTo(n+=s,o+=s),e.lineTo(n-=s,o+=s),e.lineTo(n-=s,o-=s),e.lineTo(n+=s,o-=s),e.closePath(),this.fillStroke(e),this.dirty=!1},t.className="Diamond",t}(yb),_J=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),SJ=function(i){_J(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.isPointInPath=function(e,r){return!1},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this,n=r.x,o=r.y,a=r.size,s=a/2;e.beginPath(),e.moveTo(n-=s,o-=s),e.lineTo(n+=a,o),e.lineTo(n,o+=a),e.lineTo(n-=a,o),e.lineTo(n,o-=a),e.closePath(),this.fillStroke(e),this.dirty=!1},t.className="Square",t}(yb);function wD(i){switch(i){case"circle":return U0;case"square":return SJ;case"diamond":return bJ;default:return U0}}function Ud(i,t){t===void 0&&(t="solid");var e={solid:[],dash:[4,3],dot:[1,3],dashDot:[4,3,1,3],dashDotDot:[4,3,1,3,1,3],shortDot:[1,1],shortDash:[3,1],shortDashDot:[3,1,1,1],shortDashDotDot:[3,1,1,1,1,1],longDash:[8,3],longDashDot:[8,3,1,3],longDashDotDot:[8,3,1,3,1,3]},r={solid:[],dash:[3,3],dot:[0,3],dashDot:[3,3,0,3],dashDotDot:[3,3,0,3,0,3],shortDot:[0,2],shortDash:[2,2],shortDashDot:[2,2,0,2],shortDashDotDot:[2,2,0,2,0,2],longDash:[7,3],longDashDot:[7,3,0,3],longDashDotDot:[7,3,0,3,0,3]};return i==="round"||i==="square"?r[t]==null?(console.warn("'"+t+"' is not a valid 'lineDash' option."),r.solid):r[t]:e[t]==null?(console.warn("'"+t+"' is not a valid 'lineDash' option."),e.solid):e[t]}var xJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),TJ=function(){function i(){this.enabled=!0,this.shape="circle",this.size=0,this.fill="rgb(124, 181, 236)",this.stroke="rgb(124, 181, 236)",this.strokeWidth=1,this.formatter=void 0}return i}(),OJ=function(){function i(){this.stroke="rgb(124, 181, 236)",this.strokeWidth=1}return i}(),EJ=function(){function i(){this.xLine={enabled:!0,stroke:"rgba(0,0,0, 0.54)",strokeWidth:1,lineDash:"solid",lineCap:void 0},this.yLine={enabled:!1,stroke:"rgba(0,0,0, 0.54)",strokeWidth:1,lineDash:"solid",lineCap:void 0}}return i}(),RJ=function(i){xJ(t,i);function t(){var e=i.call(this)||this;return e.fill="rgba(124, 181, 236, 0.25)",e.strokePath=new By,e.fillPath=new By,e.xCrosshairLine=new El,e.yCrosshairLine=new El,e.areaSparklineGroup=new oa,e.fillPathData=[],e.strokePathData=[],e.xAxisLine=new El,e.markers=new oa,e.markerSelection=Vd.select(e.markers).selectAll(),e.markerSelectionData=[],e.marker=new TJ,e.line=new OJ,e.crosshairs=new EJ,e.rootGroup.append(e.areaSparklineGroup),e.areaSparklineGroup.append([e.fillPath,e.xAxisLine,e.strokePath,e.xCrosshairLine,e.yCrosshairLine,e.markers]),e}return t.prototype.getNodeData=function(){return this.markerSelectionData},t.prototype.onMarkerShapeChange=function(){this.markerSelection=this.markerSelection.setData([]),this.markerSelection.exit.remove(),this.scheduleLayout()},t.prototype.update=function(){var e=this.generateNodeData();if(!!e){var r=e.nodeData,n=e.fillData,o=e.strokeData;this.markerSelectionData=r,this.fillPathData=n,this.strokePathData=o,this.updateSelection(r),this.updateNodes(),this.updateStroke(o),this.updateFill(n)}},t.prototype.updateYScaleDomain=function(){var e=this,r=e.yData,n=e.yScale,o=Vc(r,Ur),a=0,s=1;o!==void 0&&(a=this.min=o[0],s=this.max=o[1]),a=a<0?a:0,s=s<0?0:s,n.domain=[a,s]},t.prototype.generateNodeData=function(){var e=this,r=e.data,n=e.yData,o=e.xData,a=e.xScale,s=e.yScale;if(!!r){for(var l=a instanceof pa?a.bandwidth/2:0,u=n.length,c=[],p=[],h=[],d,f,g,v,m=s.convert(0),C=0;C<u;C++){var w=n[C],b=o[C],S=a.convert(b)+l,T=s.convert(w);C+1<u&&(v=a.convert(o[C+1])+l),h.push({seriesDatum:{x:b,y:w},point:{x:S,y:T}}),w===void 0&&g!==void 0?(p.push({seriesDatum:void 0,point:{x:g,y:m}}),v!==void 0&&p.push({seriesDatum:void 0,point:{x:v,y:m}})):w!==void 0&&(p.push({seriesDatum:{x:b,y:w},point:{x:S,y:T}}),c.push({seriesDatum:{x:b,y:w},point:{x:S,y:T}}),d=d!==void 0?d:S,f=S),g=S}return p.push({seriesDatum:void 0,point:{x:f,y:m}},{seriesDatum:void 0,point:{x:d,y:m}}),{nodeData:c,fillData:p,strokeData:h}}},t.prototype.updateAxisLine=function(){var e=this,r=e.xScale,n=e.yScale,o=e.axis,a=e.xAxisLine;a.x1=r.range[0],a.x2=r.range[1],a.y1=a.y2=0,a.stroke=o.stroke,a.strokeWidth=o.strokeWidth;var s=n.convert(0);a.translationY=s},t.prototype.updateSelection=function(e){var r=this.marker,n=wD(r.shape),o=this.markerSelection.setData(e),a=o.enter.append(n);o.exit.remove(),this.markerSelection=o.merge(a)},t.prototype.updateNodes=function(){var e=this,r=this,n=r.highlightedDatum,o=r.highlightStyle,a=r.marker,s=o.size,l=o.fill,u=o.stroke,c=o.strokeWidth,p=a.formatter;this.markerSelection.each(function(h,d,f){var g=d.point,v=d.seriesDatum;if(!!g){var m=d===n,C=m&&l!==void 0?l:a.fill,w=m&&u!==void 0?u:a.stroke,b=m&&c!==void 0?c:a.strokeWidth,S=m&&s!==void 0?s:a.size,T;if(p){var E=f===0,P=f===e.markerSelectionData.length-1,R=v.y===e.min,M=v.y===e.max;T=p({datum:d,xValue:v.x,yValue:v.y,min:R,max:M,first:E,last:P,fill:C,stroke:w,strokeWidth:b,size:S,highlighted:m})}h.size=T&&T.size!=null?T.size:S,h.fill=T&&T.fill!=null?T.fill:C,h.stroke=T&&T.stroke!=null?T.stroke:w,h.strokeWidth=T&&T.strokeWidth!=null?T.strokeWidth:b,h.translationX=g.x,h.translationY=g.y,h.visible=T&&T.enabled!=null?T.enabled:a.enabled&&h.size>0}})},t.prototype.updateStroke=function(e){var r=this,n=r.strokePath,o=r.yData,a=r.line;if(!(o.length<2)){var s=n.path,l=e.length,u=!0;s.clear();for(var c=0;c<l;c++){var p=e[c],h=p.point,d=p.seriesDatum,f=h.x,g=h.y;d.y==null?u=!0:u?(s.moveTo(f,g),u=!1):s.lineTo(f,g)}n.lineJoin=n.lineCap="round",n.fill=void 0,n.stroke=a.stroke,n.strokeWidth=a.strokeWidth}},t.prototype.updateFill=function(e){var r=this,n=r.fillPath,o=r.yData,a=r.fill,s=n.path,l=e.length;if(s.clear(),!(o.length<2)){for(var u=0;u<l;u++){var c=e[u].point,p=c.x,h=c.y;u>0?s.lineTo(p,h):s.moveTo(p,h)}s.closePath(),n.lineJoin="round",n.stroke=void 0,n.fill=a}},t.prototype.updateXCrosshairLine=function(){var e=this,r=e.yScale,n=e.xCrosshairLine,o=e.highlightedDatum,a=e.crosshairs.xLine;if(!a.enabled||o==null){n.strokeWidth=0;return}n.y1=r.range[0],n.y2=r.range[1],n.x1=n.x2=0,n.stroke=a.stroke,n.strokeWidth=a.strokeWidth||1,n.lineCap=a.lineCap==="round"||a.lineCap==="square"?a.lineCap:void 0;var s=a.lineDash;n.lineDash=Array.isArray(s)?s:Ud(n.lineCap,a.lineDash),n.translationX=o.point.x},t.prototype.updateYCrosshairLine=function(){var e=this,r=e.xScale,n=e.yCrosshairLine,o=e.highlightedDatum,a=e.crosshairs.yLine;if(!a.enabled||o==null){n.strokeWidth=0;return}n.x1=r.range[0],n.x2=r.range[1],n.y1=n.y2=0,n.stroke=a.stroke,n.strokeWidth=a.strokeWidth||1,n.lineCap=a.lineCap==="round"||a.lineCap==="square"?a.lineCap:void 0;var s=a.lineDash;n.lineDash=Array.isArray(s)?s:Ud(n.lineCap,a.lineDash),n.translationY=o.point.y},t.prototype.getTooltipHtml=function(e){var r=this.dataType,n=e.seriesDatum,o=n.y,a=n.x,s=this.formatNumericDatum(o),l=r==="array"||r==="object"?this.formatDatum(a):void 0,u={content:s,title:l};return this.tooltip.renderer?Wl(this.tooltip.renderer({context:this.context,datum:n,yValue:o,xValue:a}),u):Wl(u)},t.className="AreaSparkline",t}(mb),PJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),AJ=function(){function i(){this.enabled=!0,this.shape="circle",this.size=0,this.fill="rgb(124, 181, 236)",this.stroke="rgb(124, 181, 236)",this.strokeWidth=1,this.formatter=void 0}return i}(),DJ=function(){function i(){this.stroke="rgb(124, 181, 236)",this.strokeWidth=1}return i}(),MJ=function(){function i(){this.xLine={enabled:!0,stroke:"rgba(0,0,0, 0.54)",strokeWidth:1,lineDash:"solid",lineCap:void 0},this.yLine={enabled:!1,stroke:"rgba(0,0,0, 0.54)",strokeWidth:1,lineDash:"solid",lineCap:void 0}}return i}(),FJ=function(i){PJ(t,i);function t(){var e=i.call(this)||this;return e.linePath=new By,e.xCrosshairLine=new El,e.yCrosshairLine=new El,e.lineSparklineGroup=new oa,e.markers=new oa,e.markerSelection=Vd.select(e.markers).selectAll(),e.markerSelectionData=[],e.marker=new AJ,e.line=new DJ,e.crosshairs=new MJ,e.rootGroup.append(e.lineSparklineGroup),e.lineSparklineGroup.append([e.linePath,e.xCrosshairLine,e.yCrosshairLine,e.markers]),e}return t.prototype.getNodeData=function(){return this.markerSelectionData},t.prototype.onMarkerShapeChange=function(){this.markerSelection=this.markerSelection.setData([]),this.markerSelection.exit.remove(),this.scheduleLayout()},t.prototype.update=function(){var e=this.generateNodeData();!e||(this.markerSelectionData=e,this.updateSelection(e),this.updateNodes(),this.updateLine())},t.prototype.updateYScaleDomain=function(){var e=this,r=e.yData,n=e.yScale,o=Vc(r,Ur),a=0,s=1;if(o!==void 0&&(a=this.min=o[0],s=this.max=o[1]),a===s){var l=Math.abs(a*.01);a-=l,s+=l}n.domain=[a,s]},t.prototype.generateNodeData=function(){var e=this,r=e.data,n=e.yData,o=e.xData,a=e.xScale,s=e.yScale;if(!!r){for(var l=a instanceof pa?a.bandwidth/2:0,u=[],c=0;c<n.length;c++){var p=n[c],h=o[c];if(p!=null){var d=a.convert(h)+l,f=s.convert(p);u.push({seriesDatum:{x:h,y:p},point:{x:d,y:f}})}}return u}},t.prototype.updateSelection=function(e){var r=this.marker,n=wD(r.shape),o=this.markerSelection.setData(e),a=o.enter.append(n);o.exit.remove(),this.markerSelection=o.merge(a)},t.prototype.updateNodes=function(){var e=this,r=this,n=r.highlightedDatum,o=r.highlightStyle,a=r.marker,s=o.size,l=o.fill,u=o.stroke,c=o.strokeWidth,p=a.formatter;this.markerSelection.each(function(h,d,f){var g=d===n,v=g&&l!==void 0?l:a.fill,m=g&&u!==void 0?u:a.stroke,C=g&&c!==void 0?c:a.strokeWidth,w=g&&s!==void 0?s:a.size,b,S=d.seriesDatum,T=d.point;if(p){var E=f===0,P=f===e.markerSelectionData.length-1,R=S.y===e.min,M=S.y===e.max;b=p({datum:d,xValue:S.x,yValue:S.y,min:R,max:M,first:E,last:P,fill:v,stroke:m,strokeWidth:C,size:w,highlighted:g})}h.size=b&&b.size!=null?b.size:w,h.fill=b&&b.fill!=null?b.fill:v,h.stroke=b&&b.stroke!=null?b.stroke:m,h.strokeWidth=b&&b.strokeWidth!=null?b.strokeWidth:C,h.translationX=T.x,h.translationY=T.y,h.visible=b&&b.enabled!=null?b.enabled:a.enabled&&h.size>0})},t.prototype.updateLine=function(){var e=this,r=e.linePath,n=e.yData,o=e.xData,a=e.xScale,s=e.yScale,l=e.line;if(!(n.length<2)){var u=r.path,c=n.length,p=a instanceof pa?a.bandwidth/2:0,h=!0;u.clear();for(var d=0;d<c;d++){var f=o[d],g=n[d],v=a.convert(f)+p,m=s.convert(g);g==null?h=!0:h?(u.moveTo(v,m),h=!1):u.lineTo(v,m)}r.fill=void 0,r.stroke=l.stroke,r.strokeWidth=l.strokeWidth}},t.prototype.updateXCrosshairLine=function(){var e=this,r=e.yScale,n=e.xCrosshairLine,o=e.highlightedDatum,a=e.crosshairs.xLine;if(!a.enabled||o==null){n.strokeWidth=0;return}n.y1=r.range[0],n.y2=r.range[1],n.x1=n.x2=0,n.stroke=a.stroke,n.strokeWidth=a.strokeWidth||1,n.lineCap=a.lineCap==="round"||a.lineCap==="square"?a.lineCap:void 0;var s=a.lineDash;n.lineDash=Array.isArray(s)?s:Ud(n.lineCap,a.lineDash),n.translationX=o.point.x},t.prototype.updateYCrosshairLine=function(){var e=this,r=e.xScale,n=e.yCrosshairLine,o=e.highlightedDatum,a=e.crosshairs.yLine;if(!a.enabled||o==null){n.strokeWidth=0;return}n.x1=r.range[0],n.x2=r.range[1],n.y1=n.y2=0,n.stroke=a.stroke,n.strokeWidth=a.strokeWidth||1,n.lineCap=a.lineCap==="round"||a.lineCap==="square"?a.lineCap:void 0;var s=a.lineDash;n.lineDash=Array.isArray(s)?s:Ud(n.lineCap,a.lineDash),n.translationY=o.point.y},t.prototype.getTooltipHtml=function(e){var r=this.dataType,n=e.seriesDatum,o=n.y,a=n.x,s=this.formatNumericDatum(o),l=r==="array"||r==="object"?this.formatDatum(a):void 0,u={content:s,title:l};return this.tooltip.renderer?Wl(this.tooltip.renderer({context:this.context,datum:n,yValue:o,xValue:a}),u):Wl(u)},t.className="LineSparkline",t}(mb),IJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),LJ=function(i){IJ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._x=0,e._y=0,e._width=0,e._height=0,e._crisp=!1,e}return Object.defineProperty(t.prototype,"x",{get:function(){return this._x},set:function(e){this._x!==e&&(this._x=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this._y},set:function(e){this._y!==e&&(this._y=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"width",{get:function(){return this._width},set:function(e){this._width!==e&&(this._width=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"height",{get:function(){return this._height},set:function(e){this._height!==e&&(this._height=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"crisp",{get:function(){return this._crisp},set:function(e){this._crisp!==e&&(this._crisp=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.isPointInPath=function(e,r){return!1},t.prototype.render=function(e){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this,n=r.x,o=r.y,a=r.width,s=r.height,l=r.crisp;if(e.beginPath(),l){var u=this,c=u.alignment,p=u.align;e.rect(p(c,n),p(c,o),p(c,n,a),p(c,o,s))}else e.rect(n,o,a,s);this.fillStroke(e),this.dirty=!1},t.className="Column",t}(go),NJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}();(function(i){NJ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._x=0,e._y=0,e.lineBreakRegex=/\r?\n/g,e.lines=[],e._text="",e._dirtyFont=!0,e._fontSize=10,e._fontFamily="sans-serif",e._textAlign=t.defaultStyles.textAlign,e._textBaseline=t.defaultStyles.textBaseline,e._lineHeight=14,e}return Object.defineProperty(t.prototype,"x",{get:function(){return this._x},set:function(e){this._x!==e&&(this._x=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this._y},set:function(e){this._y!==e&&(this._y=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.splitText=function(){this.lines=this._text.split(this.lineBreakRegex)},Object.defineProperty(t.prototype,"text",{get:function(){return this._text},set:function(e){var r=String(e);this._text!==r&&(this._text=r,this.splitText(),this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"font",{get:function(){return this.dirtyFont&&(this.dirtyFont=!1,this._font=bD(this.fontSize,this.fontFamily,this.fontStyle,this.fontWeight)),this._font},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dirtyFont",{get:function(){return this._dirtyFont},set:function(e){this._dirtyFont!==e&&(this._dirtyFont=e,e&&(this.dirty=!0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontStyle",{get:function(){return this._fontStyle},set:function(e){this._fontStyle!==e&&(this._fontStyle=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontWeight",{get:function(){return this._fontWeight},set:function(e){this._fontWeight!==e&&(this._fontWeight=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontSize",{get:function(){return this._fontSize},set:function(e){isFinite(e)||(e=10),this._fontSize!==e&&(this._fontSize=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontFamily",{get:function(){return this._fontFamily},set:function(e){this._fontFamily!==e&&(this._fontFamily=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"textAlign",{get:function(){return this._textAlign},set:function(e){this._textAlign!==e&&(this._textAlign=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"textBaseline",{get:function(){return this._textBaseline},set:function(e){this._textBaseline!==e&&(this._textBaseline=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineHeight",{get:function(){return this._lineHeight},set:function(e){this._lineHeight!==e&&(this._lineHeight=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.computeBBox=function(){return ro.has.textMetrics?this.getPreciseBBox():this.getApproximateBBox()},t.prototype.getPreciseBBox=function(){var e=ro.measureText(this.text,this.font,this.textBaseline,this.textAlign);return new gs(this.x-e.actualBoundingBoxLeft,this.y-e.actualBoundingBoxAscent,e.width,e.actualBoundingBoxAscent+e.actualBoundingBoxDescent)},t.prototype.getApproximateBBox=function(){var e=ro.getTextSize(this.text,this.font),r=this,n=r.x,o=r.y;switch(this.textAlign){case"end":case"right":n-=e.width;break;case"center":n-=e.width/2}switch(this.textBaseline){case"alphabetic":o-=e.height*.7;break;case"middle":o-=e.height*.45;break;case"ideographic":o-=e.height;break;case"hanging":o-=e.height*.2;break;case"bottom":o-=e.height;break}return new gs(n,o,e.width,e.height)},t.prototype.isPointInPath=function(e,r){var n=this.transformPoint(e,r),o=this.computeBBox();return o?o.containsPoint(n.x,n.y):!1},t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.render=function(e){if(!(!this.lines.length||!this.scene)){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this,n=r.fill,o=r.stroke,a=r.strokeWidth;e.font=this.font,e.textAlign=this.textAlign,e.textBaseline=this.textBaseline;var s=this.scene.canvas.pixelRatio||1,l=e.globalAlpha;if(n){e.fillStyle=n,e.globalAlpha=l*this.opacity*this.fillOpacity;var u=this,c=u.fillShadow,p=u.text,h=u.x,d=u.y;c&&c.enabled&&(e.shadowColor=c.color,e.shadowOffsetX=c.xOffset*s,e.shadowOffsetY=c.yOffset*s,e.shadowBlur=c.blur*s),e.fillText(p,h,d)}if(o&&a){e.strokeStyle=o,e.lineWidth=a,e.globalAlpha=l*this.opacity*this.strokeOpacity;var f=this,g=f.lineDash,v=f.lineDashOffset,m=f.lineCap,C=f.lineJoin,w=f.strokeShadow,p=f.text,h=f.x,d=f.y;g&&e.setLineDash(g),v&&(e.lineDashOffset=v),m&&(e.lineCap=m),C&&(e.lineJoin=C),w&&w.enabled&&(e.shadowColor=w.color,e.shadowOffsetX=w.xOffset*s,e.shadowOffsetY=w.yOffset*s,e.shadowBlur=w.blur*s),e.strokeText(p,h,d)}this.dirty=!1}},t.className="Text",t.defaultStyles=Df(go.defaultStyles,{textAlign:"start",fontStyle:void 0,fontWeight:void 0,fontSize:10,fontFamily:"sans-serif",textBaseline:"alphabetic"}),t})(go);function bD(i,t,e,r){return[e||"",r||"",i+"px",t].join(" ").trim()}var GJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),sl=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},kJ=function(i){GJ(t,i);function t(){var e=i.call(this)||this;return e.enabled=!0,e.fontSize=8,e.fontFamily="Verdana, sans-serif",e.color="rgba(70, 70, 70, 1)",e}return t.prototype.getFont=function(){return bD(this.fontSize,this.fontFamily,this.fontStyle,this.fontWeight)},sl([ol("change","dataChange")],t.prototype,"enabled",void 0),sl([ol("change")],t.prototype,"fontSize",void 0),sl([ol("change")],t.prototype,"fontFamily",void 0),sl([ol("change")],t.prototype,"fontStyle",void 0),sl([ol("change")],t.prototype,"fontWeight",void 0),sl([ol("change")],t.prototype,"color",void 0),t}(up),WJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),VJ=function(i){WJ(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e._x=0,e._y=0,e.lineBreakRegex=/\r?\n/g,e.lines=[],e._text="",e._dirtyFont=!0,e._fontSize=10,e._fontFamily="sans-serif",e._textAlign=t.defaultStyles.textAlign,e._textBaseline=t.defaultStyles.textBaseline,e._lineHeight=14,e}return Object.defineProperty(t.prototype,"x",{get:function(){return this._x},set:function(e){this._x!==e&&(this._x=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"y",{get:function(){return this._y},set:function(e){this._y!==e&&(this._y=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.splitText=function(){this.lines=this._text.split(this.lineBreakRegex)},Object.defineProperty(t.prototype,"text",{get:function(){return this._text},set:function(e){var r=String(e);this._text!==r&&(this._text=r,this.splitText(),this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"font",{get:function(){return this.dirtyFont&&(this.dirtyFont=!1,this._font=BJ(this.fontSize,this.fontFamily,this.fontStyle,this.fontWeight)),this._font},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"dirtyFont",{get:function(){return this._dirtyFont},set:function(e){this._dirtyFont!==e&&(this._dirtyFont=e,e&&(this.dirty=!0))},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontStyle",{get:function(){return this._fontStyle},set:function(e){this._fontStyle!==e&&(this._fontStyle=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontWeight",{get:function(){return this._fontWeight},set:function(e){this._fontWeight!==e&&(this._fontWeight=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontSize",{get:function(){return this._fontSize},set:function(e){isFinite(e)||(e=10),this._fontSize!==e&&(this._fontSize=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"fontFamily",{get:function(){return this._fontFamily},set:function(e){this._fontFamily!==e&&(this._fontFamily=e,this.dirtyFont=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"textAlign",{get:function(){return this._textAlign},set:function(e){this._textAlign!==e&&(this._textAlign=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"textBaseline",{get:function(){return this._textBaseline},set:function(e){this._textBaseline!==e&&(this._textBaseline=e,this.dirty=!0)},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"lineHeight",{get:function(){return this._lineHeight},set:function(e){this._lineHeight!==e&&(this._lineHeight=e,this.dirty=!0)},enumerable:!0,configurable:!0}),t.prototype.computeBBox=function(){return ro.has.textMetrics?this.getPreciseBBox():this.getApproximateBBox()},t.prototype.getPreciseBBox=function(){var e=ro.measureText(this.text,this.font,this.textBaseline,this.textAlign);return new gs(this.x-e.actualBoundingBoxLeft,this.y-e.actualBoundingBoxAscent,e.width,e.actualBoundingBoxAscent+e.actualBoundingBoxDescent)},t.prototype.getApproximateBBox=function(){var e=ro.getTextSize(this.text,this.font),r=this,n=r.x,o=r.y;switch(this.textAlign){case"end":case"right":n-=e.width;break;case"center":n-=e.width/2}switch(this.textBaseline){case"alphabetic":o-=e.height*.7;break;case"middle":o-=e.height*.45;break;case"ideographic":o-=e.height;break;case"hanging":o-=e.height*.2;break;case"bottom":o-=e.height;break}return new gs(n,o,e.width,e.height)},t.prototype.isPointInPath=function(e,r){var n=this.transformPoint(e,r),o=this.computeBBox();return o?o.containsPoint(n.x,n.y):!1},t.prototype.isPointInStroke=function(e,r){return!1},t.prototype.render=function(e){if(!(!this.lines.length||!this.scene)){this.dirtyTransform&&this.computeTransformMatrix(),this.matrix.toContext(e);var r=this,n=r.fill,o=r.stroke,a=r.strokeWidth;e.font=this.font,e.textAlign=this.textAlign,e.textBaseline=this.textBaseline;var s=this.scene.canvas.pixelRatio||1,l=e.globalAlpha;if(n){e.fillStyle=n,e.globalAlpha=l*this.opacity*this.fillOpacity;var u=this,c=u.fillShadow,p=u.text,h=u.x,d=u.y;c&&c.enabled&&(e.shadowColor=c.color,e.shadowOffsetX=c.xOffset*s,e.shadowOffsetY=c.yOffset*s,e.shadowBlur=c.blur*s),e.fillText(p,h,d)}if(o&&a){e.strokeStyle=o,e.lineWidth=a,e.globalAlpha=l*this.opacity*this.strokeOpacity;var f=this,g=f.lineDash,v=f.lineDashOffset,m=f.lineCap,C=f.lineJoin,w=f.strokeShadow,p=f.text,h=f.x,d=f.y;g&&e.setLineDash(g),v&&(e.lineDashOffset=v),m&&(e.lineCap=m),C&&(e.lineJoin=C),w&&w.enabled&&(e.shadowColor=w.color,e.shadowOffsetX=w.xOffset*s,e.shadowOffsetY=w.yOffset*s,e.shadowBlur=w.blur*s),e.strokeText(p,h,d)}this.dirty=!1}},t.className="Text",t.defaultStyles=Df(go.defaultStyles,{textAlign:"start",fontStyle:void 0,fontWeight:void 0,fontSize:10,fontFamily:"sans-serif",textBaseline:"alphabetic"}),t}(go);function BJ(i,t,e,r){return[e||"",r||"",i+"px",t].join(" ").trim()}var _D=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),Xy;(function(i){i[i.Rect=0]="Rect",i[i.Label=1]="Label"})(Xy||(Xy={}));var no;(function(i){i.InsideBase="insideBase",i.InsideEnd="insideEnd",i.Center="center",i.OutsideEnd="outsideEnd"})(no||(no={}));var HJ=function(i){_D(t,i);function t(){var e=i!==null&&i.apply(this,arguments)||this;return e.formatter=void 0,e.placement=no.InsideEnd,e}return t}(kJ),SD=function(i){_D(t,i);function t(){var e=i.call(this)||this;return e.fill="rgb(124, 181, 236)",e.stroke="silver",e.strokeWidth=0,e.paddingInner=.1,e.paddingOuter=.2,e.valueAxisDomain=void 0,e.formatter=void 0,e.axisLine=new El,e.bandWidth=0,e.sparklineGroup=new oa,e.rectGroup=new oa,e.labelGroup=new oa,e.rectSelection=Vd.select(e.rectGroup).selectAll(),e.labelSelection=Vd.select(e.labelGroup).selectAll(),e.nodeSelectionData=[],e.label=new HJ,e.rootGroup.append(e.sparklineGroup),e.sparklineGroup.append([e.rectGroup,e.axisLine,e.labelGroup]),e.axisLine.lineCap="round",e.label.enabled=!1,e}return t.prototype.getNodeData=function(){return this.nodeSelectionData},t.prototype.update=function(){this.updateSelections(),this.updateNodes()},t.prototype.updateSelections=function(){var e=this.generateNodeData();!e||(this.nodeSelectionData=e,this.updateRectSelection(e),this.updateLabelSelection(e))},t.prototype.updateNodes=function(){this.updateRectNodes(),this.updateLabelNodes()},t.prototype.calculateStep=function(e){var r,n,o=this,a=o.xScale,s=o.paddingInner,l=o.paddingOuter,u=o.smallestInterval,c=a.domain[1]-a.domain[0],p=c/(n=(r=u)===null||r===void 0?void 0:r.x,n!=null?n:1)+1,h=50,d=Math.min(p,h),f=d-1,g=e/Math.max(1,2*l+f*s+d);return g},t.prototype.updateYScaleDomain=function(){var e=this,r=e.yScale,n=e.yData,o=e.valueAxisDomain,a=Vc(n,Ur),s=0,l=1;a!==void 0&&(s=this.min=a[0],l=this.max=a[1]),s=s<0?s:0,l=l<0?0:l,o&&(o[1]<l&&(o[1]=l),o[0]>s&&(o[0]=s)),r.domain=o||[s,l]},t.prototype.updateRectSelection=function(e){var r=this.rectSelection.setData(e),n=r.enter.append(LJ);r.exit.remove(),this.rectSelection=r.merge(n)},t.prototype.updateRectNodes=function(){var e=this,r=this,n=r.highlightedDatum,o=r.formatter,a=r.fill,s=r.stroke,l=r.strokeWidth,u=this.highlightStyle,c=u.fill,p=u.stroke,h=u.strokeWidth;this.rectSelection.each(function(d,f,g){var v=f===n,m=v&&c!==void 0?c:a,C=v&&p!==void 0?p:s,w=v&&h!==void 0?h:l,b,S=f.x,T=f.y,E=f.width,P=f.height,R=f.seriesDatum;if(o){var M=g===0,N=g===e.nodeSelectionData.length-1,W=R.y===e.min,G=R.y===e.max;b=o({datum:f,xValue:R.x,yValue:R.y,width:E,height:P,min:W,max:G,first:M,last:N,fill:m,stroke:C,strokeWidth:w,highlighted:v})}d.fill=b&&b.fill||m,d.stroke=b&&b.stroke||C,d.strokeWidth=b&&b.strokeWidth||w,d.x=d.y=0,d.width=E,d.height=P,d.visible=d.height>0,d.translationX=S,d.translationY=T})},t.prototype.updateLabelSelection=function(e){var r=this.labelSelection.setData(e),n=r.enter.append(VJ).each(function(o){o.tag=Xy.Label,o.pointerEvents=Wc.None});r.exit.remove(),this.labelSelection=r.merge(n)},t.prototype.updateLabelNodes=function(){var e=this.label,r=e.enabled,n=e.fontStyle,o=e.fontWeight,a=e.fontSize,s=e.fontFamily,l=e.color;this.labelSelection.each(function(u,c){var p=c.label;p&&r?(u.fontStyle=n,u.fontWeight=o,u.fontSize=a,u.fontFamily=s,u.textAlign=p.textAlign,u.textBaseline=p.textBaseline,u.text=p.text,u.x=p.x,u.y=p.y,u.fill=l,u.visible=!0):u.visible=!1})},t.prototype.getTooltipHtml=function(e){var r=this.dataType,n=e.seriesDatum,o=n.y,a=n.x,s=this.formatNumericDatum(o),l=r==="array"||r==="object"?this.formatDatum(a):void 0,u={content:s,title:l};return this.tooltip.renderer?Wl(this.tooltip.renderer({context:this.context,datum:n,yValue:o,xValue:a}),u):Wl(u)},t.prototype.formatLabelValue=function(e){return e%1!==0?e.toFixed(1):e.toFixed(0)},t}(mb),jJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),UJ=function(i){jJ(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.updateYScaleRange=function(){var e=this,r=e.seriesRect,n=e.yScale;n.range=[0,r.width]},t.prototype.updateXScaleRange=function(){var e=this,r=e.xScale,n=e.seriesRect,o=e.paddingOuter,a=e.paddingInner;if(r instanceof pa)r.range=[0,n.height],r.paddingInner=a,r.paddingOuter=o;else{var s=this.calculateStep(n.height),l=s*o;this.bandWidth=s*(1-a),r.range=[l,n.height-l-this.bandWidth]}},t.prototype.updateAxisLine=function(){var e=this,r=e.yScale,n=e.axis,o=e.axisLine,a=e.seriesRect,s=n.strokeWidth;o.x1=0,o.x2=0,o.y1=0,o.y2=a.height,o.stroke=n.stroke,o.strokeWidth=s+(s%2===1?1:0);var l=r.convert(0);o.translationX=l},t.prototype.generateNodeData=function(){var e=this,r=e.data,n=e.yData,o=e.xData,a=e.xScale,s=e.yScale,l=e.fill,u=e.stroke,c=e.strokeWidth,p=e.label;if(!!r){for(var h=p.fontStyle,d=p.fontWeight,f=p.fontSize,g=p.fontFamily,v=p.color,m=p.formatter,C=p.placement,w=[],b=s.convert(0),S=0,T=n.length;S<T;S++){var E=n[S],P=o[S],R=E===void 0;R&&(E=0);var M=a.convert(P),N=Math.min(s.convert(E),b),W=Math.max(s.convert(E),b),G=a instanceof pa?a.bandwidth:this.bandWidth,k=W-N,V={x:b,y:M},U=void 0;m?U=m({value:E}):U=E!==void 0&&Ur(E)?this.formatLabelValue(E):"";var J=M+G/2,Z=void 0,le="middle",Y=void 0,ae=E!==void 0&&E>=0,z=4;if(C===no.Center)Z=N+k/2,Y="center";else if(C===no.OutsideEnd)Z=N+(ae?k+z:-z),Y=ae?"start":"end";else if(C===no.InsideEnd){Z=N+(ae?k-z:z),Y=ae?"end":"start";var X=ro.getTextSize(U,g),ue=X.width||20,pe=b+ue,be=b-ue,_e=ae&&Z<pe||!ae&&Z>be;_e&&(Z=b+z*(ae?1:-1),Y=ae?"start":"end")}else Z=b+z*(ae?1:-1),Y=ae?"start":"end";w.push({x:N,y:M,width:k,height:G,fill:l,stroke:u,strokeWidth:c,seriesDatum:{x:P,y:R?void 0:E},point:V,label:{x:Z,y:J,text:U,fontStyle:h,fontWeight:d,fontSize:f,fontFamily:g,textAlign:Y,textBaseline:le,fill:v}})}return w}},t.prototype.getDistance=function(e,r){return Math.abs(e.y-r.y)},t.className="BarSparkline",t}(SD),zJ=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),$J=function(i){zJ(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.updateYScaleRange=function(){var e=this,r=e.seriesRect,n=e.yScale;n.range=[r.height,0]},t.prototype.updateXScaleRange=function(){var e=this,r=e.xScale,n=e.seriesRect,o=e.paddingOuter,a=e.paddingInner;if(r instanceof pa)r.range=[0,n.width],r.paddingInner=a,r.paddingOuter=o;else{var s=this.calculateStep(n.width),l=s*o;this.bandWidth=s*(1-a),r.range=[l,n.width-l-this.bandWidth]}},t.prototype.updateAxisLine=function(){var e=this,r=e.yScale,n=e.axis,o=e.axisLine,a=e.seriesRect,s=n.strokeWidth;o.x1=0,o.x2=a.width,o.y1=0,o.y2=0,o.stroke=n.stroke,o.strokeWidth=s+(s%2===1?1:0);var l=r.convert(0);o.translationY=l},t.prototype.generateNodeData=function(){var e=this,r=e.data,n=e.yData,o=e.xData,a=e.xScale,s=e.yScale,l=e.fill,u=e.stroke,c=e.strokeWidth,p=e.label;if(!!r){for(var h=p.fontStyle,d=p.fontWeight,f=p.fontSize,g=p.fontFamily,v=p.color,m=p.formatter,C=p.placement,w=[],b=s.convert(0),S=0,T=n.length;S<T;S++){var E=n[S],P=o[S],R=E===void 0;R&&(E=0);var M=Math.min(s.convert(E),b),N=a.convert(P),W=Math.max(s.convert(E),b),G=a instanceof pa?a.bandwidth:this.bandWidth,k=W-M,V={x:N+G/2,y:b},U=void 0;m?U=m({value:E}):U=E!==void 0&&Ur(E)?this.formatLabelValue(E):"";var J=N+G/2,Z=void 0,le="center",Y=void 0,ae=E!==void 0&&E>=0,z=2;if(C===no.Center)Z=M+k/2,Y="middle";else if(C===no.OutsideEnd)Z=M+(ae?-z:k+z),Y=ae?"bottom":"top";else if(C===no.InsideEnd){Z=M+(ae?z:k-z),Y=ae?"top":"bottom";var X=ro.getTextSize(U,g),ue=X.height||10,pe=b-ue,be=b+ue,_e=ae&&Z>pe||!ae&&Z<be;_e&&(Z=b+z*(ae?-1:1),Y=ae?"bottom":"top")}else Z=b+z*(ae?-1:1),Y=ae?"bottom":"top";w.push({x:N,y:M,width:G,height:k,fill:l,stroke:u,strokeWidth:c,seriesDatum:{x:P,y:R?void 0:E},point:V,label:{x:J,y:Z,text:U,fontStyle:h,fontWeight:d,fontSize:f,fontFamily:g,textAlign:le,textBaseline:Y,fill:v}})}return w}},t.className="ColumnSparkline",t}(SD),KJ=function(){function i(){}return i.create=function(t,e){t=Object.create(t);var r=YJ(t.type);return e&&(r.tooltip=e),qJ(r,t),XJ(r,t),t.data&&(r.data=t.data),r},i}();function YJ(i){switch(i===void 0&&(i="line"),i){case"column":return new $J;case"bar":return new UJ;case"area":return new RJ;case"line":default:return new FJ}}function XJ(i,t){switch(t.type){case"bar":z0(i,t);break;case"column":z0(i,t);break;case"area":QJ(i,t);break;case"line":default:ZJ(i,t);break}}function qJ(i,t){Ce(i,"context",t.context,t),Ce(i,"width",t.width,t),Ce(i,"height",t.height,t),Ce(i,"container",t.container,t),Ce(i,"xKey",t.xKey,t),Ce(i,"yKey",t.yKey,t),t.padding&&JJ(i.padding,t.padding),t.axis&&tee(i.axis,t.axis),t.highlightStyle&&ree(i.highlightStyle,t.highlightStyle),t.tooltip&&i.tooltip&&nee(i.tooltip,t.tooltip)}function ZJ(i,t){t.marker&&xD(i.marker,t.marker),t.line&&TD(i.line,t.line),t.crosshairs&&OD(i.crosshairs,t.crosshairs)}function QJ(i,t){Ce(i,"fill",t.fill,t),t.marker&&xD(i.marker,t.marker),t.line&&TD(i.line,t.line),t.crosshairs&&OD(i.crosshairs,t.crosshairs)}function z0(i,t){Ce(i,"valueAxisDomain",t.valueAxisDomain,t),Ce(i,"fill",t.fill,t),Ce(i,"stroke",t.stroke,t),Ce(i,"strokeWidth",t.strokeWidth,t),Ce(i,"paddingInner",t.paddingInner,t),Ce(i,"paddingOuter",t.paddingOuter,t),Ce(i,"formatter",t.formatter,t),t.label&&eee(i.label,t.label)}function JJ(i,t){Ce(i,"top",t.top,t),Ce(i,"right",t.right,t),Ce(i,"bottom",t.bottom,t),Ce(i,"left",t.left,t)}function xD(i,t){Ce(i,"enabled",t.enabled,t),Ce(i,"size",t.size,t),Ce(i,"shape",t.shape,t),Ce(i,"fill",t.fill,t),Ce(i,"stroke",t.stroke,t),Ce(i,"strokeWidth",t.strokeWidth,t),Ce(i,"formatter",t.formatter,t)}function eee(i,t){Ce(i,"enabled",t.enabled,t),Ce(i,"fontStyle",t.fontStyle,t),Ce(i,"fontWeight",t.fontWeight,t),Ce(i,"fontSize",t.fontSize,t),Ce(i,"fontFamily",t.fontFamily,t),Ce(i,"textAlign",t.textAlign,t),Ce(i,"textBaseline",t.textBaseline,t),Ce(i,"color",t.color,t),Ce(i,"formatter",t.formatter,t),Ce(i,"placement",t.placement,t)}function TD(i,t){Ce(i,"stroke",t.stroke,t),Ce(i,"strokeWidth",t.strokeWidth,t)}function tee(i,t){Ce(i,"type",t.type,t),Ce(i,"stroke",t.stroke,t),Ce(i,"strokeWidth",t.strokeWidth,t)}function ree(i,t){Ce(i,"fill",t.fill,t),Ce(i,"size",t.size,t),Ce(i,"stroke",t.stroke,t),Ce(i,"strokeWidth",t.strokeWidth,t)}function nee(i,t){Ce(i,"enabled",t.enabled,t),Ce(i,"container",t.container,t),Ce(i,"xOffset",t.xOffset,t),Ce(i,"yOffset",t.yOffset,t),Ce(i,"renderer",t.renderer,t)}function OD(i,t){i.xLine&&t.xLine&&$0(i.xLine,t.xLine),i.yLine&&t.yLine&&$0(i.yLine,t.yLine)}function $0(i,t){Ce(i,"enabled",t.enabled,t),Ce(i,"stroke",t.stroke,t),Ce(i,"strokeWidth",t.strokeWidth,t),Ce(i,"lineDash",t.lineDash,t),Ce(i,"lineCap",t.lineCap,t)}var K0={};function iee(i,t){K0[t]||(i(),K0[t]=!0)}var Y0=function(i,t,e){if(Ur(t))return!0;var r="AG Charts: "+i+" must be a number, the value you provided is not a valid number. Using the default of "+e+"px.";return iee(function(){return console.warn(r)},i+" not a number"),!1},oee={xOffset:Y0,yOffset:Y0};function Ce(i,t,e,r){if(t in r)if(t in i){var n=oee[t],o=n?n(t,e,i[t]):!0;o&&i[t]!==e&&(i[t]=e)}else console.warn("Property "+t+" does not exist on the target object.")}var aee=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),qy=globalThis&&globalThis.__assign||function(){return qy=Object.assign||function(i){for(var t,e=1,r=arguments.length;e<r;e++){t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(i[n]=t[n])}return i},qy.apply(this,arguments)},gm=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},see=function(i){aee(t,i);function t(){return i.call(this,t.TEMPLATE)||this}return t.prototype.init=function(e){var r=this,n=!0,o=function(){var s=r.getGui(),l=s.clientWidth,u=s.clientHeight;if(!(l===0||u===0))if(n){var c=qy({data:e.value,width:l,height:u,context:{data:e.data}},e.sparklineOptions);r.sparkline=KJ.create(c,r.sparklineTooltipSingleton.getSparklineTooltip()),r.eSparkline.appendChild(r.sparkline.canvasElement),n=!1}else r.sparkline.width=l,r.sparkline.height=u},a=this.resizeObserverService.observeResize(this.getGui(),o);this.addDestroyFunc(function(){return a()})},t.prototype.refresh=function(e){return this.sparkline?(this.sparkline.data=e.value,!0):!1},t.prototype.destroy=function(){this.sparkline&&this.sparkline.destroy(),i.prototype.destroy.call(this)},t.TEMPLATE=`<div class="ag-sparkline-wrapper">
            <span ref="eSparkline"></span>
        </div>`,gm([I("eSparkline")],t.prototype,"eSparkline",void 0),gm([y("resizeObserverService")],t.prototype,"resizeObserverService",void 0),gm([y("sparklineTooltipSingleton")],t.prototype,"sparklineTooltipSingleton",void 0),t}(ee),lee=globalThis&&globalThis.__extends||function(){var i=function(t,e){return i=Object.setPrototypeOf||{__proto__:[]}instanceof Array&&function(r,n){r.__proto__=n}||function(r,n){for(var o in n)n.hasOwnProperty(o)&&(r[o]=n[o])},i(t,e)};return function(t,e){i(t,e);function r(){this.constructor=t}t.prototype=e===null?Object.create(e):(r.prototype=e.prototype,new r)}}(),vm=globalThis&&globalThis.__decorate||function(i,t,e,r){var n=arguments.length,o=n<3?t:r===null?r=Object.getOwnPropertyDescriptor(t,e):r,a;if(typeof Reflect=="object"&&typeof Reflect.decorate=="function")o=Reflect.decorate(i,t,e,r);else for(var s=i.length-1;s>=0;s--)(a=i[s])&&(o=(n<3?a(o):n>3?a(t,e,o):a(t,e))||o);return n>3&&o&&Object.defineProperty(t,e,o),o},uee=function(i){lee(t,i);function t(){return i!==null&&i.apply(this,arguments)||this}return t.prototype.postConstruct=function(){this.tooltip=new Wa},t.prototype.getSparklineTooltip=function(){return this.tooltip},t.prototype.destroyTooltip=function(){this.tooltip&&this.tooltip.destroy()},vm([B],t.prototype,"postConstruct",null),vm([nt],t.prototype,"destroyTooltip",null),t=vm([Q("sparklineTooltipSingleton")],t),t}(H),cee={moduleName:oe.SparklinesModule,beans:[uee],userComponents:[{componentName:"agSparklineCellRenderer",componentClass:see}],dependantModules:[cr]},pee=[x4,AU,j5,Q5,nq,lq,gq,wq,sD,Tq,XP,qq,aZ,Xw,SZ,OZ,cee],hee=d4.concat(pee);zP.setLicenseKey("For_Trialing_ag-Grid_Only-Not_For_Real_Development_Or_Production_Projects-Valid_Until-13_July_2022_[v2]_MTY1NzY2NjgwMDAwMA==437484d1cc7859a47178692b53c012a0");const dee=({rows:i,columns:t})=>se("div",{className:"ag-theme-alpine",style:{width:"100%",height:"100%"},children:se(Fj,{sideBar:!0,enableRangeSelection:!0,modules:hee,rowModelType:"clientSide",enableCharts:!0,rowData:i,defaultColDef:{sortable:!0,enableRowGroup:!0,enablePivot:!0,enableValue:!0,menuTabs:["filterMenuTab"],resizable:!0,floatingFilter:!0},statusBar:{statusPanels:[{statusPanel:"agTotalAndFilteredRowCountComponent",align:"left"},{statusPanel:"agFilteredRowCountComponent"},{statusPanel:"agSelectedRowCountComponent"},{statusPanel:"agAggregationComponent"}]},children:t.map(e=>se(Ij,{field:e,headerName:e.startsWith("_unnamedColumn")?"":e,filter:"agMultiColumnFilter",filterParams:{filters:[{filter:"agSetColumnFilter"}]}},e))})}),X0=Et("div"),fee=({projectUrl:i,dataframe:t,onClose:e,getIdToken:r})=>{const{data:n,error:o,mutate:a}=gT(`${i}/dbtn/dataframes/${t.key}`,async l=>fetch(`${l}?limit=${t.numberOfRows}`,{headers:r?{authorization:`Bearer ${await r()}`}:{}}).then(u=>u.json())),s=()=>o?se(Qb,{intent:"danger",children:"Loading dataframe failed."}):n?se(dee,{rows:n.rows,columns:n.columns}):dt(Qb,{intent:"info",css:{display:"flex",gap:"$1"},children:[se(Fi,{spin:!0,icon:nM}),"Loading dataframe. For large datasets this might take a moment."]});return dt(io,{children:[dt(Zy,{css:{justifyContent:"space-between"},children:[se(X0,{css:{display:"flex",alignItems:"center",gap:"$1"},children:se(oc,{children:t.key})}),dt(X0,{css:{display:"flex",gap:"$1",alignItems:"flex-start"},children:[se(J0,{disabled:!n,onClick:()=>{a(void 0,{revalidate:!0})},children:"Refresh"}),se(QT,{onClick:e,children:se(Fi,{icon:vT})})]})]}),s()]})},gee=({schedule:i})=>se(mC,{type:"job",title:i.name,metadata:dt(io,{children:[se(Fi,{icon:iM}),dt(Fn,{color:"white",margin:"none",children:["Runs every ",se(bc,{children:i.seconds})," seconds"]})]})}),vee=()=>{const{width:i,height:t}=we.exports.useContext(eT);return{width:i,height:t}},mm=Et("div",{display:"flex",flexWrap:"wrap",gap:"$4"}),mee=Et("div",{display:"grid",gridTemplateColumns:"320px auto",gap:"$3"}),yee=Et("div",{$$height:"85vh",height:"$$height",width:"100%","&>iframe":{border:0,height:"$$height",width:"100%"}}),Cee=Et("div",{width:"650px"}),wee=i=>{const t=i.length>0?i[0]:null;return t?{type:"streamlit",component:t}:null},bee=({projectUrl:i,apps:t,jobs:e,dataframes:r,headerContent:n,shouldAutoFocusFirstComponent:o,getIdToken:a,mode:s})=>{const[l,u]=we.exports.useState(null),{width:c}=vee();we.exports.useEffect(()=>{o&&u(wee(t))},[o,t]);const p=g=>{window.location.replace(`${i}${g.route}`)},h=c>=tT.BP2,d=()=>(l==null?void 0:l.type)==="streamlit"?se(uF,{projectUrl:i,app:l.component,onClose:()=>u(null)}):(l==null?void 0:l.type)==="dataframe"?se(fee,{projectUrl:i,dataframe:l.component,getIdToken:a,onClose:()=>u(null)}):n?se(Cee,{children:n}):null,f=()=>s==="local"?se(vC,{width:{"@initial":"mobile","@bp1":"tablet","@bp2":"desktop"},children:se(ZT,{color:"gray",children:dt(Fn,{margin:"none",children:["To explore your dataframes, deploy your project and visit"," ",se("a",{href:"https://next.databutton.com",target:"_blank",rel:"noreferrer",children:"databutton.com"}),"."]})})}):r.length>0?se(mm,{children:r.map(g=>{const v=(l==null?void 0:l.type)==="dataframe"&&l.component.key===g.key;return se(lF,{isSelected:v,dataframe:g,onClick:m=>{u(v?null:{type:"dataframe",component:m})}},g.key)})}):se(ag,{message:"No data"});return dt(mee,{children:[dt("div",{children:[!h&&dt(io,{children:[n,se(Vo,{})]}),se(oc,{children:"Apps"}),t.length>0?se(mm,{children:t.map(g=>{const v=(l==null?void 0:l.type)==="streamlit"&&l.component.uid===g.uid;return se(sF,{isSelected:v,onClick:()=>{h?u(v?null:{type:"streamlit",component:g}):p(g)},app:g},g.uid)})}):se(ag,{message:s==="cloud"?"No apps deployed yet":"No apps"}),se(Vo,{}),se(oc,{children:"Data"}),f(),se(Vo,{}),se(oc,{children:"Jobs"}),e.length>0?se(mm,{children:e.map(g=>se(gee,{schedule:g},g.uid))}):se(ag,{message:s==="cloud"?"No jobs deployed yet":"No jobs"})]}),h&&se(yee,{children:d()})]})},q0=Et(Qy,{display:"flex",justifyContent:"center",alignItems:"center",gap:"$3"}),_ee=Et(Qy,{display:"flex",paddingX:"$3",height:"60px",backgroundColor:"$primaryDark",flexDirection:"row",alignItems:"center",justifyContent:"space-between",transition:"0.2s ease-in",width:"100%"}),See=({projectName:i})=>dt(_ee,{children:[dt(q0,{children:[se("img",{src:"/logo.png",height:"30px",width:"30px",alt:"Databutton"}),i&&se(Fn,{color:"white",margin:"none",children:i})]}),dt(q0,{children:[se(Gh,{href:"https://docs.databutton.com",target:"_blank",rel:"noreferrer",color:"white",weight:"semiBold",children:"Documentation"}),se(Gh,{href:"https://next.databutton.com",target:"_blank",rel:"noreferrer",children:se(J0,{children:"databutton.com"})})]})]}),Z0="http://localhost:8000";function xee(){const{data:i}=gT(`${Z0}/static/artifacts.json`,r=>fetch(r).then(n=>n.json()),{refreshInterval:500}),t=we.exports.useMemo(()=>i?i.streamlit_apps:[],[i]),e=(i==null?void 0:i.streamlit_apps.length)===0&&i.schedules.length===0;return dt(io,{children:[se(See,{}),se(ZD,{children:e?se(tM,{mode:"local"}):se(bee,{mode:"local",projectUrl:Z0,shouldAutoFocusFirstComponent:t.length===1,apps:t,dataframes:[],jobs:i?i.schedules:[]})})]})}ED.createRoot(document.getElementById("root")).render(se(Q0.StrictMode,{children:se(PD,{children:se(xee,{})})}));
